/*-
 * Copyright (c) 2010 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>

#include <ctype.h>
#include <inttypes.h>
#include <regex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "sharedkey.h"

/* get a number from a substring */
static int
re2num(const char *s, int64_t from, int64_t to)
{
	int64_t	i;
	size_t	ret;
	int	sign;

	if ((i = from) < 0) {
		return 0;
	}
	sign = (s[(int)i] == '-') ? -1 : 1;
	for (ret = 0 ; i < to ; i++) {
		if (isdigit((uint8_t)s[(int)i])) {
			ret = (ret * 10) + (s[(int)i] - '0');
		}
	}
	return sign * ret;
}


int
main(int argc, char **argv)
{
	regmatch_t	 matches[10];
	sharedkey_t		 sharedkey;
	regex_t		 re;
	int		 sharing;
	int		 ok;
	int		 i;

	(void) memset(&sharedkey, 0x0, sizeof(sharedkey));
	sharedkey.anon = 0;
	sharedkey.numbits = 2048;
	sharedkey.splitdir = NULL;
	sharedkey.out = "recover.out";
	sharedkey.threshold = sharedkey.shares = 0;
	sharing = 1;
	(void) regcomp(&re, "([0-9]+)/([0-9]+)", REG_EXTENDED);
	while ((i = getopt(argc, argv, "Vab:d:o:rs:t:")) != -1) {
		switch(i) {
		case 'V':
			(void) fprintf(stderr, "%s: %s\n", *argv, OTK_VERSION);
			exit(EXIT_SUCCESS);
			break;
		case 'a':
			sharedkey.anon = 1;
			break;
		case 'b':
			sharedkey.numbits = atoi(optarg);
			break;
		case 'd':
			sharedkey.splitdir = optarg;
			break;
		case 'o':
			sharedkey.out = optarg;
			break;
		case 'r':
			sharing = 0;
			break;
		case 's':
		case 't':
			if (regexec(&re, optarg, 10, matches, 0) == 0) {
				sharedkey.threshold = re2num(optarg, matches[1].rm_so, matches[1].rm_eo);
				sharedkey.shares = re2num(optarg, matches[2].rm_so, matches[2].rm_eo);
			} else if (i == 's') {
				sharedkey.shares = atoi(optarg);
			} else {
				sharedkey.threshold = atoi(optarg);
			}
			break;
		default:
			break;
		}
	}
	if (sharing) {
		if (optind >= argc) {
			(void) fprintf(stderr,
	"Usage: %s [-ar] [-b bits] [-d splitdir] [-o output] [-s shares] [-t threshold] file\n", *argv);
			return 0;
		}
		ok = sharedkey_share(&sharedkey, *argv, argv[optind]);
	} else {
		ok = sharedkey_recover(&sharedkey, *argv, argc, argv);
	}
	exit((ok) ? EXIT_SUCCESS : EXIT_FAILURE);
}
