/*-
 * Copyright (c) 2010,2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef SSSS_H_
#define SSSS_H_	20120220

#include <sys/types.h>
#include <sys/uio.h>

#include <inttypes.h>
#include <stdio.h>

#ifndef __BEGIN_DECLS
#  if defined(__cplusplus)
#  define __BEGIN_DECLS           extern "C" {
#  define __END_DECLS             }
#  else
#  define __BEGIN_DECLS
#  define __END_DECLS
#  endif
#endif

__BEGIN_DECLS

enum {
	SSSS_MAX_SHARES		= 256,

	SSSS_SPLIT_SOURCE	= SSSS_MAX_SHARES,
	SSSS_JOIN_DEST		= SSSS_MAX_SHARES
};

/* io vector - look familiar? */
typedef struct ssss_iovec_t {
	size_t		 size;		/* size of vector */
	uint8_t		*base;		/* its base */
} ssss_iovec_t;

/* a threshold string */
typedef struct ssss_str_t {
	unsigned	 iotype;	/* type of input mechanism */
	size_t		 c;		/* # of chars */
	ssss_iovec_t	 io;		/* iovec */
	FILE		*fp;		/* file descriptor */
} ssss_str_t;

/* structure to hold threshold information */
typedef struct ssss_t {
	void		*handle;	/* implementation internals */
	unsigned	 sharesc;	/* total # of shares */
	unsigned	 threshold;	/* # of shares in quorum */
	unsigned	 strsize;	/* size of share */
	unsigned	 availc;	/* # of shares we have added */
	ssss_str_t	 shares[SSSS_MAX_SHARES + 1];	/* the shares themselves */
} ssss_t;

/* initialisation function */
int ssss_init(ssss_t */*ssss*/, unsigned, unsigned);

/* split functions */
int ssss_split(ssss_t */*ssss*/);
int ssss_split_vec(ssss_t */*ssss*/ssss, const char */*buf*/, size_t /*size*/, struct iovec */*iov*/, int /*iovc*/);

/* reconstruction functions */
int ssss_combine(ssss_t */*ssss*/);
int ssss_combine_vec(ssss_t */*ssss*/, struct iovec */*iov*/, int /*iovc*/, char */*buf*/, size_t /*size*/);

/* functions to retrieve a share or the result of a join */
int ssss_add_share(ssss_t */*ssss*/, unsigned, const void *, ssize_t);
int ssss_get_share(ssss_t */*ssss*/, unsigned, void **, size_t *);
int ssss_write_share(ssss_t */*ssss*/, unsigned, const char *);

/* finalisation function */
int ssss_end(ssss_t */*ssss*/);

__END_DECLS

#endif
