/*-
 * Copyright (c) 2010,2011,2012 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/param.h>

#include <err.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "ssss.h"

#define MB(x)	(1024 * 1024 * (x))

/* read from stdin (up to a max size) */
static char *
readstdin(int sz, int *cc)
{
	char	*buf;
	int	 i;

	if ((buf = calloc(1, (unsigned)sz)) == NULL) {
		warn("can't allocate %d bytes", sz);
		return NULL;
	}
	for (*cc = 0 ; (i = read(STDIN_FILENO, &buf[*cc], (unsigned)(sz - *cc))) > 0 && *cc < sz ; *cc += i) {
	}
	return buf;
}

int
main(int argc, char **argv)
{
	const char	*out;
	unsigned	 shares;
	unsigned	 thresh;
	unsigned	 maxsz;
	ssss_t 	 	 ssss;
	char		 f[MAXPATHLEN];
	char		*s;
	int	 	 interactive;
	int	 	 split;
	int	 	 cc;
	int		 i;

	(void) memset(&ssss, 0x0, sizeof(ssss));
	split = 1;
	out = NULL;
	shares = thresh = 0;
	interactive = 0;
	maxsz = 4;
	while ((i = getopt(argc, argv, "ijm:o:st:")) != -1) {
		switch(i) {
		case 'i':
			interactive = 1;
			break;
		case 'j':
			split = 0;
			break;
		case 'm':
			maxsz = atoi(optarg);
			break;
		case 'o':
			out = optarg;
			break;
		case 's':
			split = 1;
			break;
		case 't':
			for (s = optarg, thresh = 0 ; *s && *s != '/' ; s++) {
				thresh = (thresh * 10) + (*s - '0');
			}
			if (*s == '/') {
				for (s++, shares = 0 ; *s ; s++) {
					shares = (shares * 10) + (*s - '0');
				}
			}
			break;
		default:
			break;
		}
	}
	if (split) {
		if (shares == 0 || thresh == 0) {
			errx(EXIT_FAILURE, "Need to specify threshold and shares (-t t/s)");
		}
		ssss_init(&ssss, thresh, shares);
		if (argc == optind) {
			if (interactive) {
				s = getpass("Data to share: ");
				cc = snprintf(f, sizeof(f), "%s\n", s);
				s = f;
			} else if ((s = readstdin(MB(maxsz), &cc)) == NULL) {
				err(EXIT_FAILURE, "can't allocate %d bytes", MB(maxsz));
			}
			ssss_add_share(&ssss, (unsigned)SSSS_SPLIT_SOURCE, s, strlen(s));
			if (out == NULL) {
				out = "ssss";
			}
		} else {
			if (!ssss_add_share(&ssss, (unsigned)SSSS_SPLIT_SOURCE, out = argv[optind], -1)) {
				err(EXIT_FAILURE, "can't read file '%s'", argv[optind]);
			}
		}
		ssss_split(&ssss);
		for (i = 0 ; i < (int)shares ; i++) {
			(void) snprintf(f, sizeof(f), "%s.%03d", out, i);
			ssss_write_share(&ssss, (unsigned)i, f);
		}
	} else {
		for (i = optind ; i < argc ; i++) {
			if (!ssss_add_share(&ssss, 0, argv[i], -1)) {
				warn("can't read file '%s'", argv[optind]);
			}
		}
		ssss_combine(&ssss);
		ssss_write_share(&ssss, SSSS_JOIN_DEST, out);
	}
	ssss_end(&ssss);
	exit(EXIT_SUCCESS);
}
