/* $NetBSD: bufgap.h,v 1.1.1.1 2011/06/12 07:48:28 agc Exp $ */

/*-
 * Copyright (c) 1996-2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef BUFGAP_H_
#define BUFGAP_H_ 20091023

#include <sys/types.h>

#include <inttypes.h>
#include <stdio.h>

#ifndef __BEGIN_DECLS
#  if defined(__cplusplus)
#  define __BEGIN_DECLS           extern "C" {
#  define __END_DECLS             }
#  else
#  define __BEGIN_DECLS
#  define __END_DECLS
#  endif
#endif

__BEGIN_DECLS

#ifndef BUFGAP_VERSION_STRING
#define BUFGAP_VERSION_STRING	"20091022"
#endif

#ifndef BUFGAP_AUTHOR_STRING
#define BUFGAP_AUTHOR_STRING	"Alistair Crooks (agc@netbsd.org)"
#endif

/* Constants for Buffer Gap routines */
enum {
	BGByte,
	BGChar,
	BGLine,

	BGFromBOF,
	BGFromHere,
	BGFromEOF
};

/* this struct describes a file in memory */
typedef struct bufgap_t {
	uint64_t	 size;		/* size of file */
	uint64_t	 abc;		/* # of bytes after the gap */
	uint64_t	 bbc;		/* # of bytes before the gap */
	uint64_t	 acc;		/* # of utf chars after the gap */
	uint64_t	 bcc;		/* # of utf chars before the gap */
	uint64_t	 alc;		/* # of records after the gap */
	uint64_t	 blc;		/* # of records before the gap */
	char		*name;		/* file name - perhaps null */
	char		*buf;		/* buffer-gap buffer */
	char		 modified;	/* file has been modified */
} bufgap_t;

int bufgap_open(bufgap_t */*bp*/, const char */*filename*/);
void bufgap_close(bufgap_t */*bp*/);
int bufgap_forwards(bufgap_t */*bp*/, uint64_t /*off*/, int /*type*/);
int bufgap_backwards(bufgap_t */*bp*/, uint64_t /*off*/, int /*type*/);
int bufgap_seek(bufgap_t */*bp*/, int64_t /*off*/, int, int /*type*/);
char *bufgap_getstr(bufgap_t */*bp*/);
int bufgap_getbin(bufgap_t */*bp*/, void */*dest*/, size_t /*len*/);
int64_t bufgap_tell(bufgap_t */*bp*/, int /*whence*/, int /*type*/);
int64_t bufgap_size(bufgap_t */*bp*/, int /*type*/);
int bufgap_insert(bufgap_t */*bp*/, const char */*text*/, int /*len*/);
int bufgap_delete(bufgap_t */*bp*/, uint64_t /*len*/);
int bufgap_peek(bufgap_t */*bp*/, int64_t /* delta */);
char *bufgap_gettext(bufgap_t */*bp*/, int64_t /*from*/, int64_t /*to*/);
char *bufgap_getline(bufgap_t */*bp*/, int64_t /*line*/);
int bufgap_write(bufgap_t */*bp*/, FILE */*fp*/);
int bufgap_dirty(bufgap_t */*bp*/);

__END_DECLS

#endif /* !BUFGAP_H_ */
