/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef CIRCA_H_
#define CIRCA_H_	20110511

#include <sys/types.h>

#include <inttypes.h>

#include <rs.h>

#ifndef __BEGIN_DECLS
#  if defined(__cplusplus)
#  define __BEGIN_DECLS           extern "C" {
#  define __END_DECLS             }
#  else
#  define __BEGIN_DECLS
#  define __END_DECLS
#  endif
#endif

__BEGIN_DECLS

/* magic numbers */
enum {
	CIRCA_DATA_SIZE		= 2348,	/* 2352 - overhead */
	CIRCA_SECTOR_SIZE	= 3136	/* 98 frames of 32 bytes */
};

typedef struct res_header_t {
	uint32_t	magic;		/* magic number */
	uint32_t	sectorsize;	/* encoded size of original data */
} res_header_t;

typedef struct circa_t {
	uint64_t	 c2errc;	/* # of C2 errors corrected */
	uint64_t	 c1errc;	/* # of C1 errors corrected */
	rs_t		 c2;		/* Q parity */
	rs_t		 c1;		/* P parity */
	uint8_t		 init;		/* initialised */
	res_header_t	 header;	/* header */
	uint32_t	 maxframes;	/* number of frames in sector */
	uint32_t	 datasize;	/* data size (for input) */
	uint8_t		*c1data;	/* work array for c1 data */
	uint8_t		*c3data;	/* work space for c3 data */
} circa_t;

/* initialise resiliency vectors etc */
int circa_init(circa_t */*circa*/, size_t /*sectorsize*/);
int circa_end(circa_t */*circa*/);

/* encode and decode in a resilient way */
ssize_t circa_encode(circa_t */*circa*/, const void */*in*/, size_t /*insize*/, uint8_t */*out*/, size_t /*outsize*/);
ssize_t circa_decode(circa_t */*circa*/, const void */*in*/, size_t /*insize*/, uint8_t */*out*/, size_t /*outsize*/);

int circa_file(circa_t */*circa*/, size_t /*sectorsize*/, int /*encoding*/, int /*header*/, char */*in*/, char */*fout*/);

/* resilient header routines */
ssize_t circa_get_header(circa_t */*circa*/, const void */*in*/, size_t /*size*/);
ssize_t circa_put_header(circa_t */*circa*/, void */*out*/, size_t /*size*/);

__END_DECLS

#endif
