/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/param.h>

#include <inttypes.h>
#include <rs.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "circa.h"

#define PKG_VERSION	"20120215"
#define PKG_AUTHOR	"Alistair Crooks (agc@NetBSD.org)"

int
main(int argc, char **argv)
{
	circa_t	 circa;
	size_t	 sectorsize;
	char	*out;
	int	 encoding;
	int	 header;
	int	 i;

	(void) memset(&circa, 0x0, sizeof(circa));
	header = 1;
	encoding = 1;
	sectorsize = CIRCA_SECTOR_SIZE;
	out = NULL;
	while ((i = getopt(argc, argv, "dno:s:v")) != -1) {
		switch(i) {
		case 'd':
			encoding = 0;
			break;
		case 'n':
			header = 0;
			break;
		case 'o':
			out = optarg;
			break;
		case 's':
			sectorsize = atoi(optarg);
			break;
		case 'v':
			printf("Version: %s\n", PKG_VERSION);
			printf("All praise, bug reports and chocolate to %s, please\n", PKG_AUTHOR);
			exit(EXIT_SUCCESS);
		default:
			break;
		}
	}
	/* Initialization of the ECC library */
	if (optind == argc) {
		circa_file(&circa, sectorsize, encoding, header, NULL, out);
	} else {
		for (i = optind ; i < argc ; i++) {
			circa_file(&circa, sectorsize, encoding, header, argv[i], out);
		}
	}
	circa_end(&circa);
	exit(EXIT_SUCCESS);
}
