/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "libcrc.h"

static unsigned
crcfile(int fd, unsigned poly, int width, int reverse)
{
	unsigned	crc;
	CRC_CTX		model;
	uint8_t	 	buf[BUFSIZ * 10];
	int	 	cc;
	int	 	i;

	(void) memset(&model, 0x0, sizeof(model));
	CRC_Init(&model);
	CRC_Args(&model, width, poly, reverse);
	while ((i = read(fd, buf, sizeof(buf))) > 0) {
		CRC_Update(&model, buf, (size_t)i);
		cc += i;
	}
	CRC_Final((unsigned *)(void *)&crc, &model);
	return crc;
}

int
main(int argc, char **argv)
{
	unsigned	 crc;
	unsigned	 poly;
	FILE		*fp;
	int		 reverse;
	int		 width;
	int		 ex;
	int		 i;

	poly = CRC_DEF_POLY;
	width = CRC_DEF_WIDTH;
	reverse = CRC_DEF_REVER;
	while ((i = getopt(argc, argv, "r:p:w:")) != -1) {
		switch(i) {
		case 'r':
			reverse = (strcasecmp(optarg, "true") == 0);
			break;
		case 'p':
			poly = (unsigned)strtol(optarg, NULL, 16);
			break;
		case 'w':
			width = atoi(optarg);
			break;
		default:
			break;
		}
	}
	ex = EXIT_SUCCESS;
	if (optind == argc) {
		crc = crcfile(STDIN_FILENO, poly, width, reverse);
		printf("%x\n", crc);
	} else {
		for (i = optind ; i < argc ; i++) {
			if ((fp = fopen(argv[i], "r")) == NULL) {
				(void) fprintf(stderr, "unable to open '%s'\n", argv[i]);
				ex = EXIT_FAILURE;
				continue;
			}
			crc = crcfile(fileno(fp), poly, width, reverse);
			printf("CRC (%s) = %x\n", argv[i], crc);
			(void) fclose(fp);
		}
	}
	exit(ex);
}
