/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/******************************************************************************/
/*                                                                            */
/* Author : Ross Williams (ross@guest.adelaide.edu.au.).                      */
/* Date   : 3 June 1993.                                                      */
/* Status : Public domain.                                                    */
/*                                                                            */
/* Description : This is the header (.h) file for the reference               */
/* implementation of the Rocksoft^tm Model CRC Algorithm. For more            */
/* information on the Rocksoft^tm Model CRC Algorithm, see the document       */
/* titled "A Painless Guide to CRC Error Detection Algorithms" by Ross        */
/* Williams (ross@guest.adelaide.edu.au.). This document is likely to be in   */
/* "ftp.adelaide.edu.au/pub/rocksoft".                                        */
/*                                                                            */
/* Note: Rocksoft is a trademark of Rocksoft Pty Ltd, Adelaide, Australia.    */
/*                                                                            */
/******************************************************************************/
#ifndef CRCMODEL_H_
#define CRCMODEL_H_	1

#include <sys/types.h>

#include <inttypes.h>

#ifndef __BEGIN_DECLS
#  if defined(__cplusplus)
#  define __BEGIN_DECLS           extern "C" {
#  define __END_DECLS             }
#  else
#  define __BEGIN_DECLS
#  define __END_DECLS
#  endif
#endif

__BEGIN_DECLS

/******************************************************************************/

#define CRC_DEF_WIDTH  		4
#define CRC_POLY_IEEE		0x04c11db7	/* IEEE, V.42, SATA, Ethernet, MPEG2 802.3 */
#define CRC_POLY_ISCSI		0x1edc6f41	/* iSCSI, SCTP */
#define CRC_POLY_KOOPMAN	0X741b8cd7
#define CRC_POLY_CRC32Q		0X814141ab	/* aviation */
#define CRC_DEF_REVER  		1

#define CRC_DEF_POLY   		CRC_POLY_IEEE

/* The following type stores the context of an executing instance of the  */
/* model algorithm. */
typedef struct {
	int		width;   /* Parameter: Width in bits [8,32].       */
	unsigned	poly;    /* Parameter: The algorithm's polynomial. */
	unsigned	init;    /* Parameter: Initial register value.     */
	unsigned	refin;   /* Parameter: Reflect input bytes?        */
	unsigned	refot;   /* Parameter: Reflect output CRC?         */
	unsigned	xorot;   /* Parameter: XOR this to output CRC.     */
	unsigned	ctxt;    /* Context: Context during execution.     */
} CRC_CTX;

void CRC_Init(CRC_CTX */*ctx*/);
void CRC_Args(CRC_CTX */*ctx*/, int /*width*/, unsigned /*poly*/, int /*reflect*/);
void CRC_Update(CRC_CTX */*ctx*/, uint8_t */*blk_adr*/, size_t /*blk_len*/);
void CRC_Final(unsigned */*crc*/, CRC_CTX */*ctx*/);
char *CRC_Data(uint8_t */*data*/, size_t /*len*/, char */*crc*/);
char *CRC_File(const char */*filename*/, char */*buf*/);
char *CRC_FileChunk(const char */*filename*/, char */*buf*/, off_t /*off*/, off_t /*len*/);

unsigned CRC_tabentry(CRC_CTX */*ctx*/, int /*indx*/);

__END_DECLS

#endif
