/* $NetBSD: find2.h,v 1.1.1.1 2011/03/19 04:44:51 agc Exp $ */

/*-
 * Copyright (c) 2005-2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef FIND2_H_
#define FIND2_H_	20110318

#include <sys/types.h>

#include <inttypes.h>
#include <stdio.h>
#include <time.h>

#ifndef __BEGIN_DECLS
#  if defined(__cplusplus)
#  define __BEGIN_DECLS           extern "C" {
#  define __END_DECLS             }
#  else
#  define __BEGIN_DECLS
#  define __END_DECLS
#  endif
#endif

__BEGIN_DECLS

/* a phrase in an expression */
typedef struct f2phrase_t {
	int		name;		/* token for name of var */
	int		op;		/* operation to perform */
	char	       *value;		/* value to check against */
	int64_t		num;		/* numeric value, if constant */
	char		init;		/* value was initialised */
} f2phrase_t;

/* a clause in a sentence */
typedef struct f2clause_t {
	f2phrase_t	*lhs;		/* left phrase */
	int		 op;		/* logical operator */
	f2phrase_t	*rhs;		/* right phrase */
} f2clause_t;

/* the paragraph to be matched */
typedef struct f2sentence_t {
	f2clause_t	*lhs;		/* left clause */
	int		 op;		/* logical operator */
	f2clause_t	*rhs;		/* right clause */
} f2sentence_t;

/* this structure holds the information on the find command */
typedef struct find_t {
	int		verbose;	/* be verbose */
	int		longlist;	/* any listings are long */
	int		ignorecase;	/* ignore case */
	int		nulterm;	/* 0x0-terminate file names */
	int		usestat;	/* use stat(2), not lstat(2) */
	int		samedev;	/* don't cross devices */
	int		nofollow;	/* don't follow sym links */
	dev_t		dev;		/* device we started from */
	time_t		started;	/* time the command was run */
	f2sentence_t	sent;		/* sentence to match */
} find_t;

int find2_parse(find_t *, const char *);
int find2(find_t *, const char *, FILE *);

__END_DECLS

#endif /* FIND2_H_ */
