/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "getopt2.h"

int
getopt2_init(getopt2_t *options)
{
	(void) memset(options, 0x0, sizeof(*options));
	options->optind = 1;
	return 1;
}

int
getopt2(getopt2_t *options, int argc, char **argv, const char *optstring)
{
	const char	*opt;
	char		 ch;
	int		 subind;

	if (options->optind == 0) {
		/* allow for straight getopt2() invocation without init */
		getopt2_init(options);
	}
	/* on entry, if optsubind > 0, then check for it */
	subind = (options->optsubind > 0) ? options->optsubind : 1;
	if (options->optind >= argc) {
		return options->optopt = -1;
	}
	ch = argv[options->optind][subind++];
	if (options->optsubind == 0 && *argv[options->optind] != '-') {
		return options->optopt = -1;
	}
	if ((opt = strchr(optstring, ch)) == NULL) {
		(void) fprintf(stderr, "getopt2: '%c' unknown option\n", ch);
		return options->optopt = '?';
	}
	switch(*(opt + 1)) {
	case ':':
		if (argv[options->optind][subind] == 0x0) {
			options->optarg = argv[options->optind + 1];
			options->optind += 2;
		} else {
			options->optarg = &argv[options->optind][subind];
			options->optind += 1;
		}
		options->optsubind = 0;
		break;
	case ';':
		if (argv[options->optind][subind] == 0x0) {
			if (argv[options->optind + 1][0] == '-') {
				options->optarg = NULL;
				options->optind += 1;
			} else {
				options->optarg = argv[options->optind + 1];
				options->optind += 2;
			}
		} else {
			options->optarg = &argv[options->optind][subind];
			options->optind += 1;
		}
		options->optsubind = 0;
		break;
	default:
		if (argv[options->optind][subind] == 0x0) {
			options->optsubind = 0;
			options->optind += 1;
		} else {
			options->optsubind = subind;
		}
		options->optarg = NULL;
		break;
	}
	return options->optopt = *opt;
}
