/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef HASH64_H_
#define HASH64_H_	20110602

#include <sys/types.h>

#include <inttypes.h>

typedef struct HASH64_CTX {
	uint32_t	algs[2];
	uint32_t	hashes[2];
} HASH64_CTX;

int HASH64_Init(HASH64_CTX */*ctx*/, const char */*algs*/);
int HASH64_Update(HASH64_CTX */*ctx*/, const uint8_t */*str*/, size_t /*len*/);
int HASH64_Final(uint64_t */*hash*/, HASH64_CTX */*ctx*/);

uint64_t HASH64_End(HASH64_CTX */*ctx*/, uint64_t */*hash*/);
uint64_t HASH64_File(char */*filename*/, uint64_t */*hash*/,
			const char */*algs*/);
uint64_t HASH64_Data(const uint8_t */*data*/, size_t /*len*/,
			uint64_t */*hash*/, const char */*algs*/);

int HASH64_ListAlgs(char */*buf*/, size_t /*size*/);

int HASH32_Init(HASH64_CTX */*ctx*/, const char */*algs*/);
int HASH32_Update(HASH64_CTX */*ctx*/, const uint8_t */*str*/, size_t /*len*/);
int HASH32_Final(uint32_t */*hash*/, HASH64_CTX */*ctx*/);

uint32_t HASH32_End(HASH64_CTX */*ctx*/, uint32_t */*hash*/);
uint32_t HASH32_File(char */*filename*/, uint32_t */*hash*/,
			const char */*algs*/);
uint32_t HASH32_Data(const uint8_t */*data*/, size_t /*len*/,
			uint32_t */*hash*/, const char */*algs*/);

#endif
