/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "hash64.h"

#ifndef PRIx64
#define PRIx64	"lx"
#endif

int
main(int argc, char **argv)
{
	const char	*algs;
	HASH64_CTX	 ctx;
	uint64_t	 hash;
	ssize_t		 cc;
	char		 buf[BUFSIZ * 10];
	int	 	 ex;
	int	 	 i;

	algs = NULL;
	while ((i = getopt(argc, argv, "a:l")) != -1) {
		switch(i) {
		case 'a':
			algs = optarg;
			break;
		case 'l':
			cc = HASH64_ListAlgs(buf, sizeof(buf));
			printf("%.*s", (int)cc, buf);
			exit(EXIT_SUCCESS);
		default:
			break;
		}
	}
	ex = EXIT_SUCCESS;
	if (optind == argc) {
		HASH64_Init(&ctx, algs);
		for (cc = 0 ;
		     (i = read(STDIN_FILENO, buf, sizeof(buf))) > 0 ;
		     cc += i) {
			HASH64_Update(&ctx, (uint8_t *)buf, i);
		}
		HASH64_Final(&hash, &ctx);
		printf("%016" PRIx64 "\n", hash);
	} else {
		for (i = optind ; i < argc ; i++) {
			if (HASH64_File(argv[i], &hash, algs)) {
				printf("HASH64 (%s) = %016" PRIx64 "\n", argv[i],
					hash);
			} else {
				ex = EXIT_FAILURE;
			}
		}
	}
	exit(ex);
}
