/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _HMAC_SHA1_H
#define _HMAC_SHA1_H

#include "rmd160.h"
#include "md5.h"
#include "sha1.h"
#include "sha2.h"

#ifdef __cplusplus
extern "C" {
#endif

#define MD5_DIGEST_SIZE		(128 / 8)
#define RMD160_DIGEST_SIZE	(160 / 8)
#define SHA1_DIGEST_SIZE	(160 / 8)
#define SHA224_DIGEST_SIZE	(224 / 8)
#define SHA256_DIGEST_SIZE	(256 / 8)
#define SHA384_DIGEST_SIZE	(384 / 8)
#define SHA512_DIGEST_SIZE	(512 / 8)

#define MD5_BLOCK_SIZE		(512 / 8)
#define RMD160_BLOCK_SIZE	(512 / 8)
#define SHA1_BLOCK_SIZE		(512 / 8)
#define SHA224_BLOCK_SIZE	(512 / 8)
#define SHA256_BLOCK_SIZE	(512 / 8)
#define SHA384_BLOCK_SIZE	(1024 / 8)
#define SHA512_BLOCK_SIZE	(1024 / 8)

typedef struct hmac_md5_ctx_t {
	MD5_CTX		ctx_inside;
	MD5_CTX		ctx_outside;

	/* for hmac_reinit */
	MD5_CTX		ctx_inside_reinit;
	MD5_CTX		ctx_outside_reinit;

	uint8_t		block_ipad[MD5_BLOCK_SIZE];
	uint8_t		block_opad[MD5_BLOCK_SIZE];
} hmac_md5_ctx_t;

typedef struct hmac_rmd160_ctx_t {
	RMD160_CTX	ctx_inside;
	RMD160_CTX	ctx_outside;

	/* for hmac_reinit */
	RMD160_CTX	ctx_inside_reinit;
	RMD160_CTX	ctx_outside_reinit;

	uint8_t		block_ipad[RMD160_BLOCK_SIZE];
	uint8_t		block_opad[RMD160_BLOCK_SIZE];
} hmac_rmd160_ctx_t;

typedef struct hmac_sha1_ctx_t {
	SHA1_CTX	ctx_inside;
	SHA1_CTX	ctx_outside;

	/* for hmac_reinit */
	SHA1_CTX	ctx_inside_reinit;
	SHA1_CTX	ctx_outside_reinit;

	uint8_t		block_ipad[SHA1_BLOCK_SIZE];
	uint8_t		block_opad[SHA1_BLOCK_SIZE];
} hmac_sha1_ctx_t;

typedef struct hmac_sha224_ctx_t {
	SHA224_CTX	ctx_inside;
	SHA224_CTX	ctx_outside;

	/* for hmac_reinit */
	SHA224_CTX	ctx_inside_reinit;
	SHA224_CTX	ctx_outside_reinit;

	uint8_t		block_ipad[SHA224_BLOCK_SIZE];
	uint8_t		block_opad[SHA224_BLOCK_SIZE];
} hmac_sha224_ctx_t;

typedef struct hmac_sha256_ctx_t {
	SHA256_CTX	ctx_inside;
	SHA256_CTX	ctx_outside;

	/* for hmac_reinit */
	SHA256_CTX	ctx_inside_reinit;
	SHA256_CTX	ctx_outside_reinit;

	uint8_t		block_ipad[SHA256_BLOCK_SIZE];
	uint8_t		block_opad[SHA256_BLOCK_SIZE];
} hmac_sha256_ctx_t;

typedef struct hmac_sha384_ctx_t {
	SHA384_CTX	ctx_inside;
	SHA384_CTX	ctx_outside;

	/* for hmac_reinit */
	SHA384_CTX	ctx_inside_reinit;
	SHA384_CTX	ctx_outside_reinit;

	uint8_t		block_ipad[SHA384_BLOCK_SIZE];
	uint8_t		block_opad[SHA384_BLOCK_SIZE];
} hmac_sha384_ctx_t;

typedef struct hmac_sha512_ctx_t {
	SHA512_CTX	ctx_inside;
	SHA512_CTX	ctx_outside;

	/* for hmac_reinit */
	SHA512_CTX	ctx_inside_reinit;
	SHA512_CTX	ctx_outside_reinit;

	uint8_t		block_ipad[SHA512_BLOCK_SIZE];
	uint8_t		block_opad[SHA512_BLOCK_SIZE];
} hmac_sha512_ctx_t;

void hmac_md5_init(hmac_md5_ctx_t *, const uint8_t *, size_t);
void hmac_md5_reinit(hmac_md5_ctx_t *);
void hmac_md5_update(hmac_md5_ctx_t *, const uint8_t *, size_t);
void hmac_md5_final(hmac_md5_ctx_t *, uint8_t *, size_t);
void hmac_md5(const uint8_t *, size_t, const uint8_t *, size_t, uint8_t *, size_t);

void hmac_rmd160_init(hmac_rmd160_ctx_t *, const uint8_t *, size_t);
void hmac_rmd160_reinit(hmac_rmd160_ctx_t *);
void hmac_rmd160_update(hmac_rmd160_ctx_t *, const uint8_t *, size_t);
void hmac_rmd160_final(hmac_rmd160_ctx_t *, uint8_t *, size_t);
void hmac_rmd160(const uint8_t *, size_t, const uint8_t *, size_t, uint8_t *, size_t);

void hmac_sha1_init(hmac_sha1_ctx_t *, const uint8_t *, size_t);
void hmac_sha1_reinit(hmac_sha1_ctx_t *);
void hmac_sha1_update(hmac_sha1_ctx_t *, const uint8_t *, size_t);
void hmac_sha1_final(hmac_sha1_ctx_t *, uint8_t *, size_t);
void hmac_sha1(const uint8_t *, size_t, const uint8_t *, size_t, uint8_t *, size_t);

void hmac_sha224_init(hmac_sha224_ctx_t *, const uint8_t *, size_t);
void hmac_sha224_reinit(hmac_sha224_ctx_t *);
void hmac_sha224_update(hmac_sha224_ctx_t *, const uint8_t *, size_t);
void hmac_sha224_final(hmac_sha224_ctx_t *, uint8_t *, size_t);
void hmac_sha224(const uint8_t *, size_t, const uint8_t *, size_t, uint8_t *, size_t);

void hmac_sha256_init(hmac_sha256_ctx_t *, const uint8_t *, size_t);
void hmac_sha256_reinit(hmac_sha256_ctx_t *);
void hmac_sha256_update(hmac_sha256_ctx_t *, const uint8_t *, size_t);
void hmac_sha256_final(hmac_sha256_ctx_t *, uint8_t *, size_t);
void hmac_sha256(const uint8_t *, size_t, const uint8_t *, size_t, uint8_t *, size_t);

void hmac_sha384_init(hmac_sha384_ctx_t *, const uint8_t *, size_t);
void hmac_sha384_reinit(hmac_sha384_ctx_t *);
void hmac_sha384_update(hmac_sha384_ctx_t *, const uint8_t *, size_t);
void hmac_sha384_final(hmac_sha384_ctx_t *, uint8_t *, size_t);
void hmac_sha384(const uint8_t *, size_t, const uint8_t *, size_t, uint8_t *, size_t);

void hmac_sha512_init(hmac_sha512_ctx_t *, const uint8_t *, size_t);
void hmac_sha512_reinit(hmac_sha512_ctx_t *);
void hmac_sha512_update(hmac_sha512_ctx_t *, const uint8_t *, size_t);
void hmac_sha512_final(hmac_sha512_ctx_t *, uint8_t *, size_t);
void hmac_sha512(const uint8_t *, size_t, const uint8_t *, size_t, uint8_t *, size_t);

#ifdef __cplusplus
}
#endif

#endif /* ! _HMAC_SHA2_H */

