/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*-
 * HMAC-SHA-224/256/384/512 implementation
 * Last update: 06/15/2005
 * Issue date:  06/15/2005
 *
 * Copyright (C) 2005 Olivier Gay <olivier.gay@@a3.epfl.ch>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* IETF Validation tests */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "hmacwrap.h"
#include "hmac.h"

static const char *expected[] = {
	/* HMAC-SHA-224 */
	"896fb1128abbdf196832107cd49df33f47b4b1169912ba4f53684b22",
	"a30e01098bc6dbbf45690f3a7e9e6d0f8bbea2a39e6148008fd05e44",
	"7fb3cb3588c6c1f6ffa9694d7d6ad2649365b0c1f65d69d1ec8333ea",
	"6c11506874013cac6a2abc1bb382627cec6a90d86efc012de7afec5a",
	"0e2aea68a90c8d37c988bcdb9fca6fa8",
	"95e9a0db962095adaebe9b2d6f0dbce2d499f112f2d2b7273fa6870e",
	"3a854166ac5d9f023f54d517d0b39dbd946770db9c2b95c9f6f565d1",
	/* HMAC-SHA-256 */
	"b0344c61d8db38535ca8afceaf0bf12b881dc200c9833da726e9376c2e32cff7",
	"5bdcc146bf60754e6a042426089575c75a003f089d2739839dec58b964ec3843",
	"773ea91e36800e46854db8ebd09181a72959098b3ef8c122d9635514ced565fe",
	"82558a389a443c0ea4cc819899f2083a85f0faa3e578f8077a2e3ff46729665b",
	"a3b6167473100ee06e0c796c2955552b",
	"60e431591ee0b67f0d8a26aacbf5b77f8e0bc6213728c5140546040f0ee37f54",
	"9b09ffa71b942fcb27635fbcd5b0e944bfdc63644f0713938a7f51535c3a35e2",
	/* HMAC-SHA-384 */
	"afd03944d84895626b0825f4ab46907f15f9dadbe4101ec682aa034c7cebc59c"
	"faea9ea9076ede7f4af152e8b2fa9cb6",
	"af45d2e376484031617f78d2b58a6b1b9c7ef464f5a01b47e42ec3736322445e"
	"8e2240ca5e69e2c78b3239ecfab21649",
	"88062608d3e6ad8a0aa2ace014c8a86f0aa635d947ac9febe83ef4e55966144b"
	"2a5ab39dc13814b94e3ab6e101a34f27",
	"3e8a69b7783c25851933ab6290af6ca77a9981480850009cc5577c6e1f573b4e"
	"6801dd23c4a7d679ccf8a386c674cffb",
	"3abf34c3503b2a23a46efc619baef897",
	"4ece084485813e9088d2c63a041bc5b44f9ef1012a2b588f3cd11f05033ac4c6"
	"0c2ef6ab4030fe8296248df163f44952",
	"6617178e941f020d351e2f254e8fd32c602420feb0b8fb9adccebb82461e99c5"
	"a678cc31e799176d3860e6110c46523e",
	/* HMAC-SHA-512 */
	"87aa7cdea5ef619d4ff0b4241a1d6cb02379f4e2ce4ec2787ad0b30545e17cde"
	"daa833b7d6b8a702038b274eaea3f4e4be9d914eeb61f1702e696c203a126854",
	"164b7a7bfcf819e2e395fbe73b56e0a387bd64222e831fd610270cd7ea250554"
	"9758bf75c05a994a6d034f65f8f0e6fdcaeab1a34d4a6b4b636e070a38bce737",
	"fa73b0089d56a284efb0f0756c890be9b1b5dbdd8ee81a3655f83e33b2279d39"
	"bf3e848279a722c806b485a47e67c807b946a337bee8942674278859e13292fb",
	"b0ba465637458c6990e5a8c5f61d4af7e576d97ff94b872de76f8050361ee3db"
	"a91ca5c11aa25eb4d679275cc5788063a5f19741120c4f2de2adebeb10a298dd",
	"415fad6271580a531d4179bc891d87a6",
	"80b24263c7c1a3ebb71493c1dd7be8b49b46d1f41b4aeec1121b013783f8f352"
	"6b56d037e05f2598bd0fd2215d6a1e5295e64f73f63f0aec8b915a985d786598",
	"e37b6a775dc87dbaa4dfa9f96e5e3ffddebd71f8867289865df5a32d20cdc944"
	"b6022cac3c4982b10d5eeb55c3e4de15134676fb6de0446065c97440fa8c6a58"
};
static const char *msgs[] = {
	"Hi There",
	"what do ya want for nothing?",
	NULL,
	NULL,
	"Test With Truncation",
	"Test Using Larger Than Block-Size Key - Hash Key First",
	"This is a test using a larger than block-size key "
	"and a larger than block-size data. The key needs"
	" to be hashed before being used by the HMAC algorithm."
};
static uint8_t *keys[7];
static size_t keys_len[7] = {20, 4, 20, 25, 20, 131, 131};

/* a struct to define the algorithm for the test */
typedef struct alg_t {
	const char	*alg;	/* HMAC algorithm */
	size_t		 size;	/* size of the digest */
	unsigned	 base;	/* expected value base */
} alg_t;

static alg_t 	algs[] = {
	{	"sha224",	SHA224_DIGEST_SIZE,	0	},
	{	"sha256",	SHA256_DIGEST_SIZE,	7	},
	{	"sha384",	SHA384_DIGEST_SIZE,	14	},
	{	"sha512",	SHA512_DIGEST_SIZE,	21	},
	{	NULL,		0,			0	}
};

/* check that the hmac() invocation produced what we expected */
static int
test(const char *expect, uint8_t *digest, size_t digest_size)
{
	size_t	i;
	char	output[2 * SHA512_DIGEST_SIZE + 1];

	output[2 * digest_size] = '\0';
	for (i = 0; i < digest_size ; i++) {
		(void) snprintf(&output[2 * i], 3, "%02x", digest[i]);
	}
	if (strcmp(expect, output)) {
		(void) fprintf(stderr, "Test failed. expected '%s', output '%s'\n", expect, output);
		return 0;
	}
	return 1;
}

int
main(int argc, char **argv)
{
	uint8_t	 mac[SHA512_DIGEST_SIZE];
	size_t	 messages2and3_len = 50;
	size_t	 size;
	size_t	 t;
	alg_t	*alg;
	char	*message2;
	char	*message3;
	int	 passed;
	int	 taken;

	for (t = 0; t < 7; t++) {
		keys[t] = malloc(keys_len[t]);
		if (keys[t] == NULL) {
			(void) fprintf(stderr, "Can't allocate memory\n");
			exit(EXIT_FAILURE);
		}
	}

	memset(keys[0], 0x0b, keys_len[0]);
	strcpy((char *)keys[1], "Jefe");
	memset(keys[2], 0xaa, keys_len[2]);
	for (t = 0; t < keys_len[3]; t++) {
		keys[3][t] = (uint8_t) t + 1;
	}
	memset(keys[4], 0x0c, keys_len[4]);
	memset(keys[5], 0xaa, keys_len[5]);
	memset(keys[6], 0xaa, keys_len[6]);

	message2 = malloc(messages2and3_len + 1);
	message3 = malloc(messages2and3_len + 1);

	if (message2 == NULL || message3 == NULL) {
		(void) fprintf(stderr, "Can't allocate memory\n");
		exit(EXIT_FAILURE);
	}

	message2[messages2and3_len] = '\0';
	message3[messages2and3_len] = '\0';

	memset(message2, 0xdd, messages2and3_len);
	memset(message3, 0xcd, messages2and3_len);

	msgs[2] = message2;
	msgs[3] = message3;

	printf("HMAC-SHA-2 IETF Validation tests\n");

	for (taken = 0, passed = 0, t = 0; t < 7; t++) {
		printf("Test %zu: ", t + 1);
		for (alg = algs ; alg->alg ; alg++) {
			size = (t == 4) ? 128 / 8 : alg->size;
			hmac(alg->alg, keys[t], keys_len[t], (const uint8_t *)msgs[t],
				strlen(msgs[t]), mac, size);
			printf("%s ", alg->alg);
			taken += 1;
			if (test(expected[alg->base + t], mac, size)) {
				passed += 1;
			}
		}
		printf("\n");
	}
	printf("%d/%d tests passed.\n", passed, taken);
	exit((passed == taken) ? EXIT_SUCCESS : EXIT_FAILURE);
}
