/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef LAMPORTSIG_H_
#define LAMPORTSIG_H_	20110621

#include <sys/types.h>

#include <inttypes.h>

/*
see:
	http://en.wikipedia.org/wiki/Lamport_signature

256 pairs of hashes of 32 bytes each
*/

enum {
	LAMPORT_DIGEST_SIZE	= 32,
	LAMPORT_MAX_PAIRS	= 256
};

/* a lamport key - strictly just a large area of storage */
typedef struct lamport_key_t {
	uint8_t		data[LAMPORT_MAX_PAIRS * 2 * LAMPORT_DIGEST_SIZE];
} lamport_key_t;

#define LAMPORT_OFFSET(h, p, d)	\
	((((h) * (LAMPORT_DIGEST_SIZE * 2)) + ((p) * LAMPORT_DIGEST_SIZE)) + (d))

/* prototypes 'r us */

int lamport_init(void);

void lamport_mk_sec(lamport_key_t */*sec*/);
void lamport_mk_semipub(lamport_key_t */*sec*/, /*uint8_t *key, size_t keysize, */ lamport_key_t */*pub*/);
void lamport_mk_pub(lamport_key_t */*sec*/, lamport_key_t */*pub*/);

int lamport_sign_data(lamport_key_t */*sec*/, const void */*msg*/,
	size_t /*size*/, uint8_t */*sig*/, size_t /*sigsize*/);
int lamport_verify_data(lamport_key_t */*pub*/, const void */*msg*/,
	size_t /*size*/, uint8_t */*sig*/, size_t /*sigsize*/);

int lamport_sign_file(lamport_key_t */*sec*/, const char */*f*/,
	uint8_t */*sig*/, size_t /*sigsize*/);
int lamport_verify_file(lamport_key_t */*pub*/, const char */*f*/,
	uint8_t */*sig*/, size_t /*sigsize*/);


#endif
