/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "mat.h"

/* struct to map string to mat frontend function */
typedef struct map_t {
	const char	*s;
	int		(*func)(int, char **);
} map_t;

static map_t	mappings[] = {
	{ "tar",	mat_tar		},
	{ "mtar",	mat_tar		},
	{ "mattar",	mat_tar		},
	{ "mat",	mat_tar		},
	{ "pax",	mat_pax		},
	{ "mpax",	mat_pax		},
	{ "matpax",	mat_pax		},
	{ NULL,		NULL		}
};

int
main(int argc, char **argv)
{
	map_t	*mp;
	char	*s;

	if ((s = strrchr(*argv, '/')) == NULL) {
		s = *argv;
	} else {
		s += 1;
	}
	for (mp = mappings ; mp->s ; mp++) {
		if (strcasecmp(s, mp->s) == 0) {
			exit((*mp->func)(argc, argv) ? EXIT_SUCCESS : EXIT_FAILURE);
		}
	}
	(void) fprintf(stderr, "command '%s' not found\n", s);
	exit(EXIT_FAILURE);
}
