/*-
 * Copyright (c) 2010,2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef THRESHOLD_H_
#define THRESHOLD_H_	20100130

#include <sys/types.h>

#include <inttypes.h>

#ifndef __BEGIN_DECLS
#  if defined(__cplusplus)
#  define __BEGIN_DECLS           extern "C" {
#  define __END_DECLS             }
#  else
#  define __BEGIN_DECLS
#  define __END_DECLS
#  endif
#endif

__BEGIN_DECLS

enum {
	THRESH_MAX_SHARES	= 256,

	THRESH_MAGIC_LENGTH	= 4,

	THRESH_SPLIT_SOURCE	= THRESH_MAX_SHARES,
	THRESH_JOIN_DEST	= THRESH_MAX_SHARES
};

/* threshold header written to output file when splitting */
typedef struct thresh_head_t {
	char		magic[THRESH_MAGIC_LENGTH];	/* magic string */
	uint8_t		sharesc;	/* # of shares */
	uint8_t		thresh;		/* threshold value */
	uint8_t		coeff;		/* coefficient of this share */
	uint8_t		pad;		/* padding character */
	uint64_t	size;		/* size of original file */
} thresh_head_t;

/* io vector - look familiar? */
typedef struct thresh_iovec_t {
	size_t		 size;		/* size of vector */
	uint8_t		*base;		/* its base */
} thresh_iovec_t;

/* a threshold string */
typedef struct thresh_str_t {
	size_t		 c;		/* # of chars */
	thresh_iovec_t	 io;		/* iovec */
} thresh_str_t;

/* structure to hold threshold information */
typedef struct threshold_t {
	int		 type;		/* type of implementation */
	void		*handle;	/* implementation internals */
	unsigned	 sharesc;	/* total # of shares */
	unsigned	 threshold;	/* # of shares in quorum */
	unsigned	 strsize;	/* size of share */
	unsigned	 availc;	/* # of shares we have added */
	thresh_str_t	 shares[THRESH_MAX_SHARES + 1];	/* the shares themselves */
	uint8_t		 mapped[THRESH_MAX_SHARES + 1];	/* share was mmapped */
} threshold_t;

/* initialisation function */
int thresh_init(threshold_t *, const char *, unsigned, unsigned);

/* split functions */
int thresh_split(threshold_t *);

/* reconstruction functions */
int thresh_combine(threshold_t *);

/* join function */
int thresh_join(threshold_t *);

/* functions to retrieve a share or the result of a join */
int thresh_add_share(threshold_t *, unsigned, const void *, ssize_t);
int thresh_get_share(threshold_t *, unsigned, void **, size_t *);
int thresh_write_share(threshold_t *, unsigned, const char *);

/* header functions */
int thresh_sane_header(thresh_head_t *, thresh_head_t *);
int thresh_get_header(thresh_head_t *, const uint8_t *);
int thresh_fmt_header(thresh_head_t *, const char *, unsigned, unsigned, uint64_t);

/* finalisation function */
int thresh_end(threshold_t *);

__END_DECLS

#endif
