/*-
 * Copyright (c) 2010,2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef INTERNAL_H_
#define INTERNAL_H_	20100415

#include <inttypes.h>

#include "threshold.h"

/* This file is mainly for the internal function prototypes */

enum {
	THRESH_UNKNOWN		= 0,
	THRESH_IDA		= 1,
	THRESH_RAID4		= 2,
	THRESH_RAID5		= 3
};

/* initialisations */
int raid_init(threshold_t *, unsigned, unsigned, unsigned);
int ida_init(threshold_t *, unsigned, unsigned);

/* various raid-specific functions - old interface */
int raid_split_memory(threshold_t *, const void *, unsigned, unsigned, unsigned);
int raid_split_file(threshold_t *, const char *, unsigned, unsigned);
int raid_join_file(threshold_t *, char **, unsigned, thresh_str_t *);
int raid_join_memory(threshold_t *, thresh_str_t *, unsigned, unsigned, thresh_str_t *);
int raid_read_header(const char *, uint8_t *, uint8_t *, uint8_t *);

/* split functions */
int ida_split(threshold_t *);
int raid_split(threshold_t *);

/* combine functions */
int ida_combine(threshold_t *);

/* finalisations */
int ida_end(threshold_t *);
int raid_end(threshold_t *);

#endif
