/*-
 * Copyright (c) 2010,2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/param.h>

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "threshold.h"
#define MB(x)	(1024 * 1024 * (x))

/* read from stdin (up to a max size) */
static char *
readstdin(int sz, int *cc)
{
	char	*buf;
	int	 i;

	if ((buf = calloc(1, (unsigned)sz)) == NULL) {
		(void) fprintf(stderr, "can't allocate %d bytes\n", sz);
		return NULL;
	}
	for (*cc = 0 ; (i = read(STDIN_FILENO, &buf[*cc], (unsigned)(sz - *cc))) > 0 && *cc < sz ; *cc += i) {
	}
	return buf;
}

int
main(int argc, char **argv)
{
	threshold_t 	 threshold;
	const char	*out;
	const char	*alg;
	unsigned	 shares;
	unsigned	 thresh;
	unsigned	 maxsz;
	char		 f[MAXPATHLEN];
	char		*s;
	int	 	 interactive;
	int	 	 split;
	int	 	 cc;
	int		 i;

	(void) memset(&threshold, 0x0, sizeof(threshold));
	split = 1;
	out = NULL;
	shares = thresh = 0;
	alg = "ida";
	interactive = 0;
	maxsz = 4;
	while ((i = getopt(argc, argv, "a:ijm:o:st:")) != -1) {
		switch(i) {
		case 'a':
			alg = optarg;
			break;
		case 'i':
			interactive = 1;
			break;
		case 'j':
			split = 0;
			break;
		case 'm':
			maxsz = atoi(optarg);
			break;
		case 'o':
			out = optarg;
			break;
		case 's':
			split = 1;
			break;
		case 't':
			for (s = optarg, thresh = 0 ; *s && *s != '/' ; s++) {
				thresh = (thresh * 10) + (*s - '0');
			}
			if (*s == '/') {
				for (s++, shares = 0 ; *s ; s++) {
					shares = (shares * 10) + (*s - '0');
				}
			}
			break;
		default:
			break;
		}
	}
	if (split) {
		if (shares == 0 || thresh == 0) {
			(void) fprintf(stderr, "Need to specify threshold and shares (-t t/s)\n");
			exit(EXIT_FAILURE);
		}
		thresh_init(&threshold, alg, thresh, shares);
		if (argc == optind) {
			if (interactive) {
				s = getpass("Data to share: ");
				cc = snprintf(f, sizeof(f), "%s\n", s);
				s = f;
			} else if ((s = readstdin(MB(maxsz), &cc)) == NULL) {
				(void) fprintf(stderr, "can't allocate %d bytes\n", MB(maxsz));
				exit(EXIT_FAILURE);
			}
			thresh_add_share(&threshold, (unsigned)THRESH_SPLIT_SOURCE, s, strlen(s));
			if (out == NULL) {
				out = "threshold";
			}
		} else {
			thresh_add_share(&threshold, (unsigned)THRESH_SPLIT_SOURCE, out = argv[optind], -1);
		}
		thresh_split(&threshold);
		for (i = 0 ; i < (int)shares ; i++) {
			(void) snprintf(f, sizeof(f), "%s.split%i", out, i);
			thresh_write_share(&threshold, (unsigned)i, f);
		}
	} else {
		for (i = optind ; i < argc ; i++) {
			thresh_add_share(&threshold, (unsigned)i - optind, argv[i], -1);
		}
		thresh_combine(&threshold);
		thresh_write_share(&threshold, THRESH_JOIN_DEST, out);
	}
	thresh_end(&threshold);
	exit(EXIT_SUCCESS);
}
