/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "tiger.h"

#define MB(x)	(1024 * 1024 * (x))

/* read from stdin (up to a max size) */
static int
dostdin(int sz, int version)
{
	uint8_t	*buf;
	char	 sum[TIGER_DIGEST_STRING_LENGTH];
	int	 ret;
	int	 cc;
	int	 i;

	if ((buf = calloc(1, (unsigned)sz)) == NULL) {
		(void) fprintf(stderr, "can't allocate %d bytes\n", sz);
		return 0;
	}
	for (cc = 0 ; (i = read(STDIN_FILENO, &buf[cc], (unsigned)(sz - cc))) > 0 && cc < sz ; cc += i) {
	}
	ret = 1;
	switch(version) {
	case 1:
	case 2:
		printf("%s\n", TIGER_Data(buf, cc, sum, version));
		break;
	default:
		(void) fprintf(stderr, "Unknown tiger version %d\n", version);
		ret = 0;
		break;
	}
	free(buf);
	return ret;
}

int
main(int argc, char **argv)
{
	char	*slash;
	char	 sum[TIGER_DIGEST_STRING_LENGTH];
	int	 sz;
	int	 ex;
	int	 version;
	int	 i;

	if ((slash = strrchr(*argv, '/')) == NULL) {
		slash = *argv;
	} else {
		slash += 1;
	}
	version = (strcmp(slash, "tiger2") == 0) ? 2 : 1;
	sz = MB(4);
	while ((i = getopt(argc, argv, "s:v:")) != -1) {
		switch(i) {
		case 's':
			sz = atoi(optarg);
			break;
		case 'v':
			version = atoi(optarg);
			break;
		default:
			break;
		}
	}
	ex = EXIT_SUCCESS;
	if (optind == argc) {
		if (!dostdin(sz, version)) {
			ex = EXIT_FAILURE;
		}
	} else {
		for (i = optind ; i < argc ; i++) {
			if (TIGER_File(argv[i], sum, version)) {
				printf("%s (%s) = %s\n",
					(version == 2) ? "TIGER2" : "TIGER",
					argv[i], sum);
			} else {
				ex = EXIT_FAILURE;
			}
			break;
		}
	}
	exit(ex);
}
