/*-
 * Copyright (c) 2012 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>

#include <ctype.h>
#include <err.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "vmdk.h"

int
main(int argc, char **argv)
{
	const char	*f;
	uint64_t	 multiplier;
	uint64_t	 slicesize;
	uint64_t	 bytes;
	uint64_t	 cc;
	char		 extname[MAXPATHLEN];
	int		 ls;
	int		 i;

	bytes = 0;
	f = NULL;
	ls = 0;
	while ((i = getopt(argc, argv, "f:ls:")) != -1) {
		switch(i) {
		case 'f':
			f = optarg;
			break;
		case 'l':
			ls = 1;
			break;
		case 's':
			switch(tolower((uint8_t)optarg[strlen(optarg) - 1])) {
			case 'k':
				multiplier = 1024;
				break;
			case 'm':
				multiplier = VMDK_MB(1);
				break;
			case 'g':
				multiplier = VMDK_GB(1);
				break;
			default:
				multiplier = 1;
				break;
			}
			bytes = vmdk_nearest_size(strtoull(optarg, NULL, 10) * multiplier);
			break;
		default:
			break;
		}
	}
	if (ls) {
		vmdk_list_extent(argv[optind]);
		exit(EXIT_SUCCESS);
	}
	/* just need one name */
	if (argc - optind < 1) {
		errx(EXIT_FAILURE, "usage: %s [-f image] [-s bytes] vmname", *argv);
	}
	if (f == NULL) {
		for (i = 0, cc = 0 ; cc < bytes ; i++, cc += slicesize) {
			(void) snprintf(extname, sizeof(extname), "%s-s%03d.vmdk", argv[optind], i + 1);
			slicesize = MIN(VMDK_SLICE_SIZE, bytes - cc);
			vmdk_make_extent(extname, slicesize);
		}
		vmdk_make_vmdk(argv[optind], i, bytes);
	} else {
		(void) snprintf(extname, sizeof(extname), "%s-s001.vmdk", f);
		vmdk_file_to_vmdk(argv[optind], f);
	}
	exit(EXIT_SUCCESS);
}
