/****************************************************************
Copyright (C) Lucent Technologies 1997
All Rights Reserved

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name Lucent Technologies or any of
its entities not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.

LUCENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
IN NO EVENT SHALL LUCENT OR ANY OF ITS ENTITIES BE LIABLE FOR ANY
SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
****************************************************************/

#ifndef LIBAWK_H_
#define LIBAWK_H_	20070501

#include <sys/types.h>

#include <inttypes.h>
#include <setjmp.h>

typedef double	awknum_t;

/* awkcell_t:  all information about a variable or constant */

typedef struct awkcell_t {
	uint8_t		  ctype;	/* OCELL, OBOOL, OJUMP, etc. */
	uint8_t		  csub;		/* CCON, CTEMP, CFLD, etc. */
	char		 *name;		/* name, for variables only */
	char		 *str;		/* string value */
	awknum_t	  num;		/* value as number */
	int		  type;		/* type info: STR|NUM|ARR|FCN|FLD|CON|DONTFREE */
	struct awkcell_t *cnext;	/* ptr to next if chained */
} awkcell_t;

typedef struct awkarray_t {		/* symbol table array */
	int		  nelem;	/* elements in table right now */
	int		  size;		/* size of tab */
	awkcell_t	**tab;		/* hash table pointers */
} awkarray_t;

/* awknode_t:  parse tree is made of nodes, with awkcell_t's at bottom */
typedef struct awknode_t {
	int		  ntype;
	struct awknode_t *nnext;
	int		  lineno;
	int		  nobj;
	struct awknode_t *narg[1];	/* variable: actual size set by calling malloc */
} awknode_t;

typedef struct awkfile_t {
	FILE		*fp;
	const char	*fname;
	int		 mode;		/* '|', 'a', 'w' => LE/LT, GT */
} awkfile_t;

typedef struct awkframe_t {		/* stack frame for awk function calls */
	int		  nargs;	/* number of arguments in this call */
	awkcell_t	 *fcncell;	/* pointer to awkcell_t for function */
	awkcell_t	**args;		/* pointer to array of arguments after execute */
	awkcell_t	 *retval;	/* return value */
} awkframe_t;

#define AWKLIB_PA2NUM	50		/* max number of pat,pat patterns allowed */

#define	AWKLIB_NFA	128		/* cache this many dynamic fa_t's */

typedef struct eawk_t {
	int		  safe;		/* 0 => unsafe, 1 => safe */
	char		**FS;		/* input file separators */
	char		**RS;		/* record separator */
	char		**OFS;		/* output field separator */
	char		**ORS;		/* output record separator */
	char		**OFMT;		/* output format for numbers */
	awknum_t	 *NR;		/* number of current record */
	awknum_t	 *FNR;		/* number of current record in current file */
	awknum_t	 *NF;		/* number of fields in current record */
	int		  recsize;	/* size of current record, orig RECSIZE */
	char		**FILENAME;	/* current filename argument */
	char		**SUBSEP;	/* subscript separator for a[i,j,k]; default \034 */
	awknum_t	 *RSTART;	/* start of re matched with ~; origin 1 (!) */
	awknum_t	 *RLENGTH;	/* length of same */
	uint8_t		 *record;	/* points to $0 */
	int		  lineno;	/* line number in awk program */
	int		  errorflag;	/* 1 if error has occurred */
	int		  donefld;	/* 1 if record broken into fields */
	int		  donerec;	/* 1 if record is valid (no fld has changed */
	int		  dbg;
	uint8_t		 *patbeg;	/* beginning of pattern matched */
	int		  patlen;	/* length of pattern matched.  set in b.c */
	awkarray_t	 *symtab;
	awkcell_t	 *nrloc;	/* NR */
	awkcell_t	 *fnrloc;	/* FNR */
	awkcell_t	 *nfloc;	/* NF */
	awkcell_t	 *rstartloc;	/* RSTART */
	awkcell_t	 *rlengthloc;	/* RLENGTH */
	awknode_t	 *winner;
	awknode_t	 *nullnode;
	int		  pairstack[AWKLIB_PA2NUM];/* state of each pat,pat */
	int		  paircnt;	/* number of them in use */
	char		 *lexprog;	/* points to program argument if it exists */
	int		  nfields;	/* last allocated slot for $i */
	awknum_t 	 *ARGC;		/* number of arguments from command line */
	awkarray_t	 *ARGVtab;	/* symbol table containing ARGV[...] */
	awkcell_t	 *fsloc;	/* FS */
	char		**CONVFMT;	/* format for conversions in eawk_getsval */
	awkarray_t	 *ENVtab;	/* symbol table containing ENVIRON[...] */
	awkcell_t	 *symtabloc;	/* SYMTAB */
	awkcell_t	**fldtab;	/* pointers to awkcell_t */
	awkcell_t	 *literal0;
	char		 *curfname;	/* current function name */
	int		  infunc;	/* = 1 if in arglist or body of func */
	int		  bracecnt;
	int		  brackcnt;
	int		  parencnt;
	awknode_t	 *curnode;	/* the node being executed, for debugging */
	awknode_t	 *arglist;	/* list of args for current function */
	char		  ebuf[300];
	char		 *ep;
	char		  yysbuf[100];	/* pushback buffer */
	char		 *yysptr;
	char		**scripts;	/* program filenames from -f's */
	int		  scriptc;	/* number of filenames */
	int		  scriptsize;	/* size of scripts array */
	int		  curscript;	/* current filename */
	FILE		 *yyin;		/* lex input file */
	int		  bracechecked;	/* if we've already checked braces */
	awkfile_t	 *files;
	unsigned	  nfiles;
	awkframe_t	 *frame;	/* base of stack frames; dynamically allocated */
	int		  nframe;	/* number of frames allocated */
	awkframe_t	 *fp;		/* frame pointer. bottom level unused */
	int		  init_getrec;
	FILE		 *infile;
	char		 *file;
	char		 *fields;
	int		  fieldssize;
	/* regexp vars */
	uint8_t		 *prestr;	/* current position in current re */
	uint8_t		 *lastre;	/* origin of last re */
	uint8_t		 *rlxstr;
	int		  maxsetvec;
	int		 *setvec;
	int		 *tmpset;
	int		  lastfld;	/* last used field */
	int		  rtok;		/* next token in current re */
	int		  rlxval;
	int		  setcnt;
	int		  poscnt;
	void		 *fatab[AWKLIB_NFA];
	int		 nfatab;	/* entries in fatab */
	int		  sc;		/* 1 => return a } right now */
	int		  reg;		/* 1 => return a REGEXPR now */
	int		  lexbufsize; 	/* BUG: setting this small causes core dump! */
	char		 *lexbuf;
	int		  stringbufsz;
	uint8_t		 *stringbuf;
	uint8_t		 *regexprbuf;
	int	 	  regexprbufsz;
	int		  make_dfa_now;
	uint8_t		 *cclenterbuf;
	int		  cclenterbufsz;
	uint8_t		 *relexbuf;
	int		  relexbufsz;
	/* misc */
	int		  argno;	/* current input argument number */
	/* FS stuff */
	char		  static_inputFS[16];
	size_t		  len_inputFS;
	char		 *inputFS;
	/* visited various functions */
	int		  syntaxc;	/* number of times in eawk_syntax() */
	int		  eprintc;	/* number of times in eprint() */
	/* control info */
	jmp_buf		  env;
	awkcell_t	 *tmps;		/* free temporary cells for execution */
	/* lex/parse */
	int		  inloop;	/* =1 if in while, for, do */
	awknode_t	 *beginloc;
	awknode_t	 *endloc;
	/* control structure */
	awkarray_t	 *control;	/* array for control structures */
	awkcell_t	  dollar0;
	awkcell_t	  dollar1;
} eawk_t;

/* external prototypes */
int	 eawk_setdefaults(eawk_t */*awk*/, const char */*progname*/);
int	 eawk_setup(eawk_t */*awk*/, int /*argc*/, char **/*argv*/);

int	 eawk_parse(void */*awk*/);
int	 eawk_run(eawk_t */*awk*/);
void	 eawk_err_display(eawk_t */*awk*/);

int	 eawk_ctlvar_set(eawk_t */*awk*/, const char *, int, const void *);
int	 eawk_ctlvar_get_int(eawk_t */*awk*/, const char */*varname*/);
void	*eawk_ctlvar_get_ptr(eawk_t */*awk*/, const char */*varname*/);
int	 eawk_setfs(eawk_t */*awk*/, const char */*fs*/);
int	 eawk_keyval(eawk_t */*awk*/, const char */*s*/);

awkcell_t *eawk_var_set(eawk_t */*eawk*/, const char */*n*/, const char */*s*/, awknum_t /*f*/, unsigned /*t*/, awkarray_t */*arr*/);

#endif /* LIBAWK_H_ */
