/****************************************************************
Copyright (C) Lucent Technologies 1997
All Rights Reserved

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name Lucent Technologies or any of
its entities not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.

LUCENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
IN NO EVENT SHALL LUCENT OR ANY OF ITS ENTITIES BE LIABLE FOR ANY
SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
****************************************************************/

static const char	*version = "version 20070501";
static const char	*embedversion = "embedded version 20081120";

#include <getopt.h>
#include <locale.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "eawk.h"

/* options descriptor */
static struct option longopts[] = {
	{ "FS",		required_argument,	NULL,		'F' },
	{ "version",	no_argument,		NULL,		'V' },
	{ "debug",	no_argument,		NULL,		'd' },
	{ "file",	required_argument,	NULL,		'f' },
	{ "safe",	no_argument,		NULL,		's' },
	{ "value",	required_argument,	NULL,		'v' },
	{ NULL,         0,                      NULL,           0 }
};

static void
fpecatch(int n)
{
	(void) fprintf(stderr, "floating point exception %d", n);
}

int
main(int argc, char **argv)
{
	const char	 *fs;
	eawk_t		  awk;
	int		  i;

	fs = NULL;
	(void) memset(&awk, 0x0, sizeof(awk));
	/* set the embedded awk defaults */
	eawk_setdefaults(&awk, *argv);

	if (argc == 1) {
		(void) fprintf(stderr, 
		  "usage: %s [-F fs] [-v var=value] [-f progfile | 'prog'] [file ...]\n", 
		  *argv);
		exit(EXIT_FAILURE);
	}
	(void) setlocale(LC_ALL, "");
	signal(SIGFPE, fpecatch);

	while ((i = getopt_long(argc, argv, "F:Vdf:sv:", longopts, NULL)) != -1) {
		switch(i) {
		case 'F':
			/* set field separator */
			fs = optarg;
			break;
		case 'V':
			printf("awk %s\n", version);
			printf("embedded awk: %s\n", embedversion);
			exit(EXIT_SUCCESS);
		case 'd':
			eawk_ctlvar_set(&awk, "debug", 1, NULL);
			printf("awk %s\n", version);
			printf("embedded awk: %s\n", embedversion);
			break;
		case 'f':
			/* handle multiple script files */
			eawk_ctlvar_set(&awk, "script", 0, optarg);
			break;
		case 's':
			eawk_ctlvar_set(&awk, "safe", 1, NULL);
			break;
		case 'v':
			/* -v a=1 to be done NOW.  one -v for each */
			eawk_keyval(&awk, optarg);
			break;
		default:
			(void) fprintf(stderr, "unknown option %c ignored", i);
			break;
		}
	}

	/* now set up the scripts, args and environment for embedded awk */
	eawk_setup(&awk, argc - optind + 1, &argv[optind - 1]);

	/* parse the awk scripts */
	eawk_parse(&awk);

	/* set locale back to whatever it is locally */
	setlocale(LC_NUMERIC, "");
	eawk_setfs(&awk, fs);

	if (eawk_ctlvar_get_int(&awk, "error flag")) {
		eawk_err_display(&awk);
	} else {
		eawk_run(&awk);
	}
	return eawk_ctlvar_get_int(&awk, "error flag");
}
