/****************************************************************
Copyright (C) Lucent Technologies 1997
All Rights Reserved

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name Lucent Technologies or any of
its entities not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.

LUCENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
IN NO EVENT SHALL LUCENT OR ANY OF ITS ENTITIES BE LIABLE FOR ANY
SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
****************************************************************/

#define DEBUG
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "eawk.h"
#include "awkgram.h"
#include "proto.h"


static awknode_t *
nodealloc(eawk_t *eawk, int n)
{
	awknode_t *x;

	x = (awknode_t *) calloc(1, sizeof(*x) + ((n-1)*sizeof(x)));
	if (x == NULL) {
		eawk_warning(eawk, "out of space in nodealloc");
	} else {
		x->nnext = NULL;
		x->lineno = eawk->lineno;
	}
	return x;
}

awknode_t *
eawk_exptostat(awknode_t *a)
{
	a->ntype = NSTAT;
	return(a);
}

static awknode_t *
node1(eawk_t *eawk, int a, awknode_t *b)
{
	awknode_t *x;

	x = nodealloc(eawk, 1);
	x->nobj = a;
	x->narg[0]=b;
	return(x);
}

static awknode_t *
node2(eawk_t *eawk, int a, awknode_t *b, awknode_t *c)
{
	awknode_t *x;

	x = nodealloc(eawk, 2);
	x->nobj = a;
	x->narg[0] = b;
	x->narg[1] = c;
	return(x);
}

static awknode_t *
node3(eawk_t *eawk, int a, awknode_t *b, awknode_t *c, awknode_t *d)
{
	awknode_t *x;

	x = nodealloc(eawk, 3);
	x->nobj = a;
	x->narg[0] = b;
	x->narg[1] = c;
	x->narg[2] = d;
	return(x);
}

static awknode_t *
node4(eawk_t *eawk, int a, awknode_t *b, awknode_t *c, awknode_t *d, awknode_t *e)
{
	awknode_t *x;

	x = nodealloc(eawk, 4);
	x->nobj = a;
	x->narg[0] = b;
	x->narg[1] = c;
	x->narg[2] = d;
	x->narg[3] = e;
	return(x);
}

static awknode_t *
node5(eawk_t *eawk, int a, awknode_t *b, awknode_t *c, awknode_t *d, awknode_t *e, awknode_t *f)
{
	awknode_t *x;

	x = nodealloc(eawk, 5);
	x->nobj = a;
	x->narg[0] = b;
	x->narg[1] = c;
	x->narg[2] = d;
	x->narg[3] = e;
	x->narg[4] = f;
	return(x);
}

awknode_t *
stat1(eawk_t *eawk, int a, awknode_t *b)
{
	awknode_t *x;

	x = node1(eawk, a,b);
	x->ntype = NSTAT;
	return(x);
}

awknode_t *
eawk_stat2(eawk_t *eawk, int a, awknode_t *b, awknode_t *c)
{
	awknode_t *x;

	x = node2(eawk, a,b,c);
	x->ntype = NSTAT;
	return(x);
}

awknode_t *
eawk_stat3(eawk_t *eawk, int a, awknode_t *b, awknode_t *c, awknode_t *d)
{
	awknode_t *x;

	x = node3(eawk, a,b,c,d);
	x->ntype = NSTAT;
	return(x);
}

awknode_t *
eawk_stat4(eawk_t *eawk, int a, awknode_t *b, awknode_t *c, awknode_t *d, awknode_t *e)
{
	awknode_t *x;

	x = node4(eawk, a,b,c,d,e);
	x->ntype = NSTAT;
	return(x);
}

awknode_t *
eawk_op1(eawk_t *eawk, int a, awknode_t *b)
{
	awknode_t *x;

	x = node1(eawk, a,b);
	x->ntype = NEXPR;
	return(x);
}

awknode_t *
eawk_op2(eawk_t *eawk, int a, awknode_t *b, awknode_t *c)
{
	awknode_t *x;

	x = node2(eawk, a,b,c);
	x->ntype = NEXPR;
	return(x);
}

awknode_t *
eawk_op3(eawk_t *eawk, int a, awknode_t *b, awknode_t *c, awknode_t *d)
{
	awknode_t *x;

	x = node3(eawk, a,b,c,d);
	x->ntype = NEXPR;
	return(x);
}

awknode_t *
eawk_op4(eawk_t *eawk, int a, awknode_t *b, awknode_t *c, awknode_t *d, awknode_t *e)
{
	awknode_t *x;

	x = node4(eawk, a,b,c,d,e);
	x->ntype = NEXPR;
	return(x);
}

awknode_t *
eawk_op5(eawk_t *eawk, int a, awknode_t *b, awknode_t *c, awknode_t *d, awknode_t *e, awknode_t *f)
{
	awknode_t *x;

	x = node5(eawk, a,b,c,d,e, f);
	x->ntype = NEXPR;
	return(x);
}

awknode_t *
eawk_celltonode(eawk_t *eawk, awkcell_t *a, int b)
{
	awknode_t *x;

	a->ctype = OCELL;
	a->csub = b;
	x = node1(eawk, 0, (awknode_t *)(void *)a);
	x->ntype = NVALUE;
	return(x);
}

/* make $0 into a awknode_t */
awknode_t *
eawk_rectonode(eawk_t *eawk)
{
	return eawk_op1(eawk, INDIRECT, eawk_celltonode(eawk, eawk->literal0, CUNK));
}

awknode_t *
eawk_makearr(eawk_t *eawk, awknode_t *p)
{
	awkcell_t *cp;

	if (ISVALUE(p)) {
		cp = (awkcell_t *)(void *)(p->narg[0]);
		if (ISFCN(cp))
			eawk_syntax(eawk,  "%s is a function, not an array", cp->name );
		else if (!ISARR(cp)) {
			XFREE(cp->str);
			cp->str = (char *)(void *)eawk_makesymtab(eawk, NSYMTAB);
			cp->type = ARR;
		}
	}
	return p;
}

/* pat, pat {...} */
awknode_t *
eawk_pa2stat(eawk_t *eawk, awknode_t *a, awknode_t *b, awknode_t *c)
{
	awknode_t *x;

	x = node4(eawk, PASTAT2, a, b, c, eawk_itonp(eawk->paircnt));
	if (eawk->paircnt++ >= AWKLIB_PA2NUM)
		eawk_syntax(eawk,  "limited to %d pat,pat statements", AWKLIB_PA2NUM );
	x->ntype = NSTAT;
	return(x);
}

awknode_t *
eawk_linkum(eawk_t *eawk, awknode_t *a, awknode_t *b)
{
	awknode_t *c;

	if (eawk->errorflag)	/* don't link things that are wrong */
		return a;
	if (a == NULL)
		return(b);
	else if (b == NULL)
		return(a);
	for (c = a; c->nnext != NULL; c = c->nnext)
		;
	c->nnext = b;
	return(a);
}

/* turn on FCN bit in definition, */
/*  body of function, arglist */
void
eawk_defn(eawk_t *eawk, awkcell_t *v, awknode_t *vl, awknode_t *st)
{
	awknode_t *p;
	int n;

	if (ISARR(v)) {
		eawk_syntax(eawk,  "`%s' is an array name and a function name", v->name );
		return;
	}
	if (eawk_isarg(eawk, v->name) != -1) {
		eawk_syntax(eawk,  "`%s' is both function name and argument name", v->name );
		return;
	}

	v->type = FCN;
	v->str = (char *)(void *)st;
	n = 0;	/* count arguments */
	for (p = vl; p; p = p->nnext)
		n++;
	v->num = n;
	DPRINTF(eawk, ("defining func %s (%d args)\n", v->name, n) );
}

/* is s in argument list for current function? */
int
eawk_isarg(eawk_t *eawk, const char *s)
{
	/* return -1 if not, otherwise arg # */
	awknode_t *p = eawk->arglist;
	int n;

	for (n = 0; p != 0; p = p->nnext, n++)
		if (strcmp(((awkcell_t *)(void *)(p->narg[0]))->name, s) == 0)
			return n;
	return -1;
}

int
eawk_ptoi(void *p)	/* convert pointer to integer */
{
	return (int) (long) p;	/* swearing that p fits, of course */
}

awknode_t *
eawk_itonp(int i)	/* and vice versa */
{
	return (awknode_t *) (long) i;
}
