/****************************************************************
Copyright (C) Lucent Technologies 1997
All Rights Reserved

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name Lucent Technologies or any of
its entities not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.

LUCENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
IN NO EVENT SHALL LUCENT OR ANY OF ITS ENTITIES BE LIABLE FOR ANY
SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
****************************************************************/

#ifndef PROTO_H_
#define PROTO_H_	20081006

/* awkcell_t subtypes: csub */
#define	CFREE	7
#define CCOPY	6
#define CCON	5
#define CTEMP	4
#define CNAME	3 
#define CVAR	2
#define CFLD	1
#define	CUNK	0

/* ctypes */
#define OCELL	1
#define OBOOL	2
#define OJUMP	3

/* bool subtypes */
#define BTRUE	11
#define BFALSE	12

/* jump subtypes */
#define JEXIT	21
#define JNEXT	22
#define	JBREAK	23
#define	JCONT	24
#define	JRET	25
#define	JNEXTFILE	26

/* node types */
#define NVALUE	1
#define NSTAT	2
#define NEXPR	3



#define NOTLEGAL(n)	(n <= FIRSTTOKEN || n >= LASTTOKEN || eawk_proctab[n-FIRSTTOKEN] == eawk_nullproc)
#define ISVALUE(n)	((n)->ntype == NVALUE)
#define ISEXPR(n)	((n)->ntype == NEXPR)
#define ISJUMP(n)	((n)->ctype == OJUMP)
#define ISEXIT(n)	((n)->csub == JEXIT)
#define	ISBREAK(n)	((n)->csub == JBREAK)
#define	ISCONT(n)	((n)->csub == JCONT)
#define	ISNEXT(n)	((n)->csub == JNEXT || (n)->csub == JNEXTFILE)
#define	ISRET(n)	((n)->csub == JRET)
#define ISREC(n)	((n)->type & REC)
#define ISFLD(n)	((n)->type & FLD)
#define ISSTR(n)	((n)->type & STR)
#define ISNUM(n)	((n)->type & NUM)
#define ISARR(n)	((n)->type & ARR)
#define ISFCN(n)	((n)->type & FCN)
#define ISTRUE(n)	((n)->csub == BTRUE)
#define ISTEMP(n)	((n)->csub == CTEMP)
#define	ISARGUMENT(n)	((n)->nobj == ARG)
/* #define freeable(p)	(!((p)->type & DONTFREE)) */
#define FREEABLE(p)	( ((p)->type & (STR|DONTFREE)) == STR )


/* awkcell_t.type values: */
#define	NUM	01	/* number value is valid */
#define	STR	02	/* string value is valid */
#define DONTFREE 04	/* string space is not freeable */
#define	CON	010	/* this is a constant */
#define	ARR	020	/* this is an array */
#define	FCN	040	/* this is a function name */
#define FLD	0100	/* this is a field $1, $2, ... */
#define	REC	0200	/* this is $0 */


/* BLTIN function types */
#define	FLENGTH	1
#define	FSQRT	2
#define	FEXP	3
#define	FLOG	4
#define	FINT	5
#define	FSYSTEM	6
#define	FRAND	7
#define	FSRAND	8
#define	FSIN	9
#define	FCOS	10
#define	FATAN	11
#define	FTOUPPER 12
#define	FTOLOWER 13
#define	FFLUSH	14
#define FSYSTIME	15
#define FSTRFTIME	16

#define	RECSIZE	(8 * 1024)	/* sets limit on records, fields, etc., etc. */

#define	NIL	((awknode_t *) NULL)

#define	NSYMTAB	50	/* initial size of a symbol table */

int	 eawk_lex(void *, void *);

extern	int	yywrap(void);
extern	void	setfname(eawk_t *, awkcell_t *);
extern	int	constnode(awknode_t *);
extern	char	*strnode(awknode_t *);
extern	awknode_t	*notnull(eawk_t *, awknode_t *);

extern	void	startreg(eawk_t *);
extern	int	input(eawk_t *);
extern	int	unput(eawk_t *, int);
extern	int	unputstr(eawk_t *, const char *);
extern	int	yylook(void);
extern	int	yyback(int *, int);
extern	int	yyinput(void);

void	*eawk_regcomp(eawk_t *, const char *, int);
int	eawk_match(eawk_t *, void *, const char *);
int	eawk_pmatch(eawk_t *, void *, const char *);
int	eawk_nematch(eawk_t *, void *, const char *);

extern	awknode_t	*eawk_exptostat(awknode_t *);
awknode_t	*eawk_stat3(eawk_t *, int, awknode_t *, awknode_t *, awknode_t *);
awknode_t	*eawk_op2(eawk_t *, int, awknode_t *, awknode_t *);
awknode_t	*eawk_op1(eawk_t *, int, awknode_t *);
awknode_t	*stat1(eawk_t *, int, awknode_t *);
awknode_t	*eawk_op3(eawk_t *, int, awknode_t *, awknode_t *, awknode_t *);
awknode_t	*eawk_op4(eawk_t *, int, awknode_t *, awknode_t *, awknode_t *, awknode_t *);
awknode_t	*eawk_op5(eawk_t *, int, awknode_t *, awknode_t *, awknode_t *, awknode_t *, awknode_t *);
awknode_t	*eawk_stat2(eawk_t *, int, awknode_t *, awknode_t *);
awknode_t	*eawk_stat4(eawk_t *, int, awknode_t *, awknode_t *, awknode_t *, awknode_t *);
awknode_t	*eawk_celltonode(eawk_t *, awkcell_t *, int);
awknode_t	*eawk_rectonode(eawk_t *);
awknode_t	*eawk_makearr(eawk_t *, awknode_t *);
awknode_t	*eawk_pa2stat(eawk_t *, awknode_t *, awknode_t *, awknode_t *);
awknode_t	*eawk_linkum(eawk_t *, awknode_t *, awknode_t *);
void	eawk_defn(eawk_t *, awkcell_t *, awknode_t *, awknode_t *);
int	eawk_isarg(eawk_t *, const char *);
extern	const char	*eawk_tokname(int, char *, size_t);
extern awkcell_t	*(*eawk_proctab[])(eawk_t *, awknode_t **, int);
extern	int	eawk_ptoi(void *);
extern	awknode_t	*eawk_itonp(int);

awkarray_t	*eawk_makesymtab(eawk_t *, int);
void	eawk_freesymtab(eawk_t *, awkcell_t *);
extern	void	eawk_freeelem(awkcell_t *, const char *);
extern	int	eawk_hash(const char *, int);
extern	awkcell_t	*eawk_lookup(const char *, awkarray_t *);
extern	awknum_t	eawk_setfval(eawk_t *, awkcell_t *, awknum_t);
extern	int	eawk_funnyvar(eawk_t *, awkcell_t *, const char *);
extern	char	*eawk_setsval(eawk_t *, awkcell_t *, const char *);
extern	awknum_t	eawk_getfval(eawk_t *, awkcell_t *);
extern	char	*eawk_getsval(eawk_t *, awkcell_t *);
extern	char	*eawk_getpssval(eawk_t *, awkcell_t *);     /* for print */
char	*eawk_tostring(eawk_t *, const char *);
char	*eawk_tostringN(eawk_t *, const char *, size_t);
char	*eawk_qstring(eawk_t *, const char *, int);

extern	int	eawk_getrec(eawk_t *, uint8_t **, int *, int);
extern	void	eawk_nextfile(eawk_t *);
extern	int	eawk_readrec(eawk_t *, uint8_t **, int *, FILE *);
extern	void	eawk_setclvar(eawk_t *, char *);
extern	int	eawk_fldbld(eawk_t *);
extern	void	eawk_newfld(eawk_t *, int);
extern	int	eawk_recbld(eawk_t *);
extern	awkcell_t	*eawk_fieldadr(eawk_t *, int);
extern	void	eawk_syntax(eawk_t *, const char *, ...);
extern	void	eawk_warning(eawk_t *, const char *, ...);
extern	awknum_t	errcheck(eawk_t *, awknum_t, const char *);
extern	int	eawk_isclvar(const char *);
extern	int	eawk_is_number(const char *);

int	eawk_adjbuf(eawk_t *, uint8_t **pb, int *sz, int min, int q, uint8_t **pbp, const char *what);
awkcell_t	*eawk_execute(eawk_t *, awknode_t *);
awkcell_t	*eawk_program(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_call(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_copycell(eawk_t *, awkcell_t *);
awkcell_t	*eawk_arg(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_jump(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_getline(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_getnf(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_array(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_delete(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_intest(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_matchop(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_boolop(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_relop(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_gettemp(eawk_t *);
awkcell_t	*eawk_indirect(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_substr(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_index(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_sprintf(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_printf(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_arith(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_bitarith(eawk_t *, awknode_t **, int);
awknum_t	eawk_ipow(awknum_t, int);
awkcell_t	*eawk_incrdecr(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_assign(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_cat(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_pastat(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_dopa2(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_split(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_condexpr(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_ifstat(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_whilestat(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_dostat(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_forstat(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_instat(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_bltin(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_print(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_nullproc(eawk_t *, awknode_t **, int);
FILE	*eawk_redirect(eawk_t *, int, awknode_t *);
awkcell_t	*eawk_closefile(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_sub(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_gsub(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_gensub(eawk_t *, awknode_t **, int);

awkcell_t	*eawk_dlopen(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_dlcall(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_dlclose(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_dlproto(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_dlalloc(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_dlfree(eawk_t *, awknode_t **, int);
awkcell_t	*eawk_dlfield(eawk_t *, awknode_t **, int);

void	 	eawk_syminit(eawk_t *);
void	 	eawk_arginit(eawk_t *, int, char **);
void	 	eawk_envinit(eawk_t *, char **);
int	 	eawk_recinit(eawk_t *, unsigned);

void	 	eawk_error(const char *);


#define	XFREE(a)	do {						\
	if ((a) != NULL) {						\
		free(__UNCONST(a));					\
		(a) = NULL;						\
	}								\
} while (/* CONSTCOND */ 0)

#define	NN(p)	((p) ? (p) : "(null)")	/* guaranteed non-null for DPRINTF */

#ifdef	DEBUG
/* uses have to be doubly parenthesized */
#	define	DPRINTF(eawk, x)	if ((eawk)->dbg) printf x
#else
#	define	DPRINTF(eawk, x)
#endif
 
#ifndef USE_ARG
#define USE_ARG(x)  /*LINTED*/(void)&x
#endif

#endif
