/****************************************************************
Copyright (C) Lucent Technologies 1997
All Rights Reserved

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name Lucent Technologies or any of
its entities not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.

LUCENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
IN NO EVENT SHALL LUCENT OR ANY OF ITS ENTITIES BE LIABLE FOR ANY
SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
****************************************************************/

#define DEBUG
#include <stdio.h>
#include <ctype.h>
#include <dlfcn.h>
#include <limits.h>
#include <locale.h>
#include <math.h>
#include <regex.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <wchar.h>
#include <wctype.h>

#include "awkre.h"
#include "eawk.h"
#include "awkgram.h"
#include "proto.h"

#ifndef MIN
#define	MIN(a,b)	(((a) < (b)) ? (a) : (b))
#endif

#ifndef ABS
#define ABS(x) 		(((x) >= 0) ? (x) : -(x))
#endif

#define TEMPFREE(eawk, x)	do {					\
	if (ISTEMP(x)) {						\
		tfree(eawk, x);						\
	}								\
} while (/* CONSTCOND */0)

#define	NARGS	50	/* max args in a call */

#define MAXFLD	2

/*
#undef tempfree

void tempfree(awkcell_t *p) {
	if (p->ctype == OCELL && (p->csub < CUNK || p->csub > CFREE)) {
		eawk_warning("bad csub %d in awkcell_t %d %s",
			p->csub, p->ctype, p->str);
	}
	if (ISTEMP(p))
		tfree(p);
}
*/



static awkcell_t	truecell	={ OBOOL, BTRUE, 0, 0, 1.0, NUM, NULL };
awkcell_t	*True	= &truecell;
static awkcell_t	falsecell	={ OBOOL, BFALSE, 0, 0, 0.0, NUM, NULL };
awkcell_t	*False	= &falsecell;
static awkcell_t	breakcell	={ OJUMP, JBREAK, 0, 0, 0.0, NUM, NULL };
awkcell_t	*jbreak	= &breakcell;
static awkcell_t	contcell	={ OJUMP, JCONT, 0, 0, 0.0, NUM, NULL };
awkcell_t	*jcont	= &contcell;
static awkcell_t	nextcell	={ OJUMP, JNEXT, 0, 0, 0.0, NUM, NULL };
awkcell_t	*jnext	= &nextcell;
static awkcell_t	nextfilecell	={ OJUMP, JNEXTFILE, 0, 0, 0.0, NUM, NULL };
awkcell_t	*jnextfile	= &nextfilecell;
static awkcell_t	exitcell	={ OJUMP, JEXIT, 0, 0, 0.0, NUM, NULL };
awkcell_t	*jexit	= &exitcell;
static awkcell_t	retcell		={ OJUMP, JRET, 0, 0, 0.0, NUM, NULL };
awkcell_t	*jret	= &retcell;
static awkcell_t	tempcell	={ OCELL, CTEMP, 0, 0, 0.0, NUM|STR|DONTFREE, NULL };

/* free a tempcell */
static void
tfree(eawk_t *eawk, awkcell_t *a)
{
	if (FREEABLE(a)) {
		DPRINTF(eawk, ("freeing %s %s %o\n", NN(a->name), NN(a->str), a->type) );
		XFREE(a->str);
	}
	if (a == eawk->tmps) {
		eawk_warning(eawk, "tempcell list is curdled");
	} else {
		a->cnext = eawk->tmps;
		eawk->tmps = a;
	}
}

/* buffer memory management */
int eawk_adjbuf(eawk_t *eawk, uint8_t **pbuf, int *psiz, int minlen, int quantum, uint8_t **pbptr,
	const char *whatrtn)
/* pbuf:    address of pointer to buffer being managed
 * psiz:    address of buffer size variable
 * minlen:  minimum length of buffer needed
 * quantum: buffer size quantum
 * pbptr:   address of movable pointer into buffer, or 0 if none
 * whatrtn: name of the calling routine if failure should cause fatal error
 *
 * return   0 for realloc failure, !=0 for success
 */
{
	if (minlen > *psiz) {
		char *tbuf;
		int rminlen = quantum ? minlen % quantum : 0;
		int boff = (pbptr) ? (int)(*pbptr - *pbuf) : 0;
		/* round up to next multiple of quantum */
		if (rminlen)
			minlen += quantum - rminlen;
		tbuf = realloc(*pbuf, minlen);
		DPRINTF(eawk, ("eawk_adjbuf %s: %d %d (pbuf=%p, tbuf=%p)\n", whatrtn, *psiz, minlen, *pbuf, tbuf) );
		if (tbuf == NULL) {
			if (whatrtn) {
				eawk_warning(eawk, "out of memory in %s", whatrtn);
			}
			return 0;
		}
		*pbuf = (uint8_t *)tbuf;
		*psiz = minlen;
		if (pbptr)
			*pbptr = (uint8_t *)tbuf + boff;
	}
	return 1;
}

/* in case stdin, etc., are not constants */
static int
stdinit(eawk_t *eawk)
{
	eawk->nfiles = FOPEN_MAX;
	eawk->files = calloc(eawk->nfiles, sizeof(awkfile_t));
	if (eawk->files == NULL) {
		eawk_warning(eawk, "can't allocate file memory for %zu files", eawk->nfiles);
		return 0;
	}
	eawk->files[0].fp = stdin;
	eawk->files[0].fname = "/dev/stdin";
	eawk->files[0].mode = LT;
	eawk->files[1].fp = stdout;
	eawk->files[1].fname = "/dev/stdout";
	eawk->files[1].mode = GT;
	eawk->files[2].fp = stderr;
	eawk->files[2].fname = "/dev/stderr";
	eawk->files[2].mode = GT;
	return 1;
}

static void
closeall(eawk_t *eawk)
{
	size_t i;
	int stat;

	for (i = 0; i < eawk->nfiles; i++) {
		if (eawk->files[i].fp) {
			if (ferror(eawk->files[i].fp))
				eawk_warning(eawk,  "i/o error occurred on %s", eawk->files[i].fname );
			if (eawk->files[i].mode == '|' || eawk->files[i].mode == LE)
				stat = pclose(eawk->files[i].fp);
			else
				stat = fclose(eawk->files[i].fp);
			if (stat == EOF)
				eawk_warning(eawk,  "i/o error occurred while closing %s", eawk->files[i].fname );
		}
	}
}

static FILE *
openfile(eawk_t *eawk, int a, const char *us)
{
	const char *s = us;
	size_t i;
	int m;
	FILE *fp = 0;

	if (*s == '\0') {
		eawk_warning(eawk, "null file name in print or getline");
		return NULL;
	}
	for (i = 0; i < eawk->nfiles; i++)
		if (eawk->files[i].fname && strcmp(s, eawk->files[i].fname) == 0) {
			if (a == eawk->files[i].mode || (a==APPEND && eawk->files[i].mode==GT))
				return eawk->files[i].fp;
			if (a == FFLUSH)
				return eawk->files[i].fp;
		}
	if (a == FFLUSH)	/* didn't find it, so don't create it! */
		return NULL;

	for (i = 0; i < eawk->nfiles; i++)
		if (eawk->files[i].fp == NULL)
			break;
	if (i >= eawk->nfiles) {
		awkfile_t *nf;
		size_t nnf = eawk->nfiles + FOPEN_MAX;

		nf = realloc(eawk->files, nnf * sizeof(*nf));
		if (nf == NULL) {
			eawk_warning(eawk, "cannot grow files for %s and %zu files", s, nnf);
			return NULL;
		}
		(void)memset(&nf[eawk->nfiles], 0, FOPEN_MAX * sizeof(*nf));
		eawk->nfiles = (unsigned)nnf;
		eawk->files = nf;
	}
	fflush(stdout);	/* force a semblance of order */
	m = a;
	if (a == GT) {
		fp = fopen(s, "w");
	} else if (a == APPEND) {
		fp = fopen(s, "a");
		m = GT;	/* so can mix > and >> */
	} else if (a == '|') {	/* output pipe */
		fp = popen(s, "w");
	} else if (a == LE) {	/* input pipe */
		fp = popen(s, "r");
	} else if (a == LT) {	/* getline <file */
		fp = strcmp(s, "-") == 0 ? stdin : fopen(s, "r");	/* "-" is stdin */
	} else {
		/* can't happen */
		eawk_warning(eawk, "illegal redirection %d", a);
		return NULL;
	}
	if (fp != NULL) {
		eawk->files[i].fname = eawk_tostring(eawk, s);
		eawk->files[i].fp = fp;
		eawk->files[i].mode = m;
	}
	return fp;
}

static const char *
filename(eawk_t *eawk, FILE *fp)
{
	size_t i;

	for (i = 0; i < eawk->nfiles; i++)
		if (fp == eawk->files[i].fp)
			return eawk->files[i].fname;
	return "???";
}

static void
backsub(uint8_t **pb_ptr, const uint8_t **sptr_ptr)/* handle \\& variations */
{						/* sptr[0] == '\\' */
	uint8_t *pb = *pb_ptr;
	const uint8_t *sptr = *sptr_ptr;

	if (sptr[1] == '\\') {
		if (sptr[2] == '\\' && sptr[3] == '&') { /* \\\& -> \& */
			*pb++ = '\\';
			*pb++ = '&';
			sptr += 4;
		} else if (sptr[2] == '&') {	/* \\& -> \ + matched */
			*pb++ = '\\';
			sptr += 2;
		} else {			/* \\x -> \\x */
			*pb++ = *sptr++;
			*pb++ = *sptr++;
		}
	} else if (sptr[1] == '&') {	/* literal & */
		sptr++;
		*pb++ = *sptr++;
	} else				/* literal \ */
		*pb++ = *sptr++;

	*pb_ptr = pb;
	*sptr_ptr = sptr;
}

/* execution of parse tree starts here */
int
eawk_run(eawk_t *eawk)
{
	eawk_var_set(eawk, "stage", "running", 0.0, STR, eawk->control);
	if (!stdinit(eawk)) {
		return 0;
	}
	eawk_execute(eawk, eawk->winner);
	closeall(eawk);
	return 1;
}

/* execute a node of the parse tree */
awkcell_t *
eawk_execute(eawk_t *eawk, awknode_t *u)
{
	awkcell_t *(*proc)(eawk_t *, awknode_t **, int);
	awkcell_t *x;
	awknode_t *a;

	if (u == NULL)
		return(True);
	for (a = u; ; a = a->nnext) {
		eawk->curnode = a;
		if (ISVALUE(a)) {
			x = (awkcell_t *)(void *)(a->narg[0]);
			if (ISFLD(x) && !eawk->donefld)
				eawk_fldbld(eawk);
			else if (ISREC(x) && !eawk->donerec)
				eawk_recbld(eawk);
			return(x);
		}
		if (NOTLEGAL(a->nobj)) {
			/* probably a awkcell_t* but too risky to print */
			eawk_warning(eawk, "illegal statement");
			return NULL;
		}
		proc = eawk_proctab[a->nobj-FIRSTTOKEN];
		x = (*proc)(eawk, a->narg, a->nobj);
		if (ISFLD(x) && !eawk->donefld) {
			eawk_fldbld(eawk);
		} else if (ISREC(x) && !eawk->donerec) {
			eawk_recbld(eawk);
		}
		if (ISEXPR(a)) {
			return(x);
		}
		if (ISJUMP(x)) {
			return(x);
		}
		if (a->nnext == NULL) {
			return(x);
		}
		TEMPFREE(eawk, x);
	}
}


/* execute an awk program */
awkcell_t *
eawk_program(eawk_t *eawk, awknode_t **a, int n)
{				/* a[0] = BEGIN, a[1] = body, a[2] = END */
	awkcell_t *x;

	USE_ARG(n);
	if (setjmp(eawk->env) != 0)
		goto ex;
	if (a[0]) {		/* BEGIN */
		x = eawk_execute(eawk, a[0]);
		if (ISEXIT(x))
			return(True);
		if (ISJUMP(x)) {
			eawk_warning(eawk, "illegal break, continue, next or nextfile from BEGIN");
			return NULL;
		}
		TEMPFREE(eawk, x);
	}
	if (a[1] || a[2])
		while (eawk_getrec(eawk, &eawk->record, &eawk->recsize, 1) > 0) {
			x = eawk_execute(eawk, a[1]);
			if (ISEXIT(x))
				break;
			TEMPFREE(eawk, x);
		}
  ex:
	if (setjmp(eawk->env) != 0)	/* handles exit within END */
		goto ex1;
	if (a[2]) {		/* END */
		x = eawk_execute(eawk, a[2]);
		if (ISBREAK(x) || ISNEXT(x) || ISCONT(x)) {
			eawk_warning(eawk, "illegal break, continue, next or nextfile from END");
			return NULL;
		}
		TEMPFREE(eawk, x);
	}
  ex1:
	return(True);
}

awkcell_t *
eawk_call(eawk_t *eawk, awknode_t **a, int n)	/* function call.  very kludgy and fragile */
{
	/* agc - static, but OK - for new args to be zero */
	static const awkcell_t newcopycell = { OCELL, CCOPY, 0, 0, 0.0, NUM|STR|DONTFREE, NULL };
	int i, ncall, ndef;
	int freed = 0; /* handles potential double freeing when fcn & param share a tempcell */
	awknode_t *x;
	awkcell_t *args[NARGS], *oargs[NARGS];	/* BUG: fixed size arrays */
	awkcell_t *y, *z, *fcn;
	char *s;

	USE_ARG(n);
	fcn = eawk_execute(eawk, a[0]);	/* the function itself */
	s = fcn->name;
	if (!ISFCN(fcn)) {
		eawk_warning(eawk, "calling undefined function %s", s);
		return NULL;
	}
	if (eawk->frame == NULL) {
		eawk->fp = eawk->frame = calloc(eawk->nframe += 100, sizeof(awkframe_t));
		if (eawk->frame == NULL) {
			eawk_warning(eawk, "out of space for stack frames calling %s", s);
			return NULL;
		}
	}
	for (ncall = 0, x = a[1]; x != NULL; x = x->nnext)	/* args in call */
		ncall++;
	ndef = (int) fcn->num;			/* args in eawk_defn */
	DPRINTF(eawk, ("calling %s, %d args (%d in defn), fp=%d\n", s, ncall, ndef, (int) (eawk->fp-eawk->frame)) );
	if (ncall > ndef)
		eawk_warning(eawk, "function %s called with %d args, uses only %d",
			s, ncall, ndef);
	if (ncall + ndef > NARGS) {
		eawk_warning(eawk, "function %s has %d arguments, limit %d", s, ncall+ndef, NARGS);
		return NULL;
	}
	for (i = 0, x = a[1]; x != NULL; i++, x = x->nnext) {	/* get call args */
		DPRINTF(eawk, ("evaluate args[%d], fp=%d:\n", i, (int) (eawk->fp-eawk->frame)) );
		y = eawk_execute(eawk, x);
		oargs[i] = y;
		DPRINTF(eawk, ("args[%d]: %s %f <%s>, t=%o\n",
			   i, NN(y->name), y->num, ISARR(y) ? "(array)" : NN(y->str), y->type) );
		if (ISFCN(y)) {
			eawk_warning(eawk, "can't use function %s as argument in %s", y->name, s);
			return NULL;
		}
		if (ISARR(y))
			args[i] = y;	/* arrays by ref */
		else
			args[i] = eawk_copycell(eawk, y);
		TEMPFREE(eawk, y);
	}
	for ( ; i < ndef; i++) {	/* add null args for ones not provided */
		args[i] = eawk_gettemp(eawk);
		*args[i] = newcopycell;
	}
	eawk->fp++;	/* now ok to up frame */
	if (eawk->fp >= eawk->frame + eawk->nframe) {
		int dfp = (int)(eawk->fp - eawk->frame);	/* old index */
		eawk->frame = realloc(eawk->frame, (eawk->nframe += 100) * sizeof(awkframe_t));
		if (eawk->frame == NULL) {
			eawk_warning(eawk, "out of space for stack frames in %s", s);
			return NULL;
		}
		eawk->fp = eawk->frame + dfp;
	}
	eawk->fp->fcncell = fcn;
	eawk->fp->args = args;
	eawk->fp->nargs = ndef;	/* number defined with (excess are locals) */
	eawk->fp->retval = eawk_gettemp(eawk);

	DPRINTF(eawk, ("start exec of %s, fp=%d\n", s, (int) (eawk->fp-eawk->frame)) );
	y = eawk_execute(eawk, (awknode_t *)(void *)(fcn->str));	/* execute body */
	DPRINTF(eawk, ("finished exec of %s, fp=%d\n", s, (int) (eawk->fp-eawk->frame)) );

	for (i = 0; i < ndef; i++) {
		awkcell_t *t = eawk->fp->args[i];
		if (ISARR(t)) {
			if (t->csub == CCOPY) {
				if (i >= ncall) {
					eawk_freesymtab(eawk, t);
					t->csub = CTEMP;
					TEMPFREE(eawk, t);
				} else {
					oargs[i]->type = t->type;
					oargs[i]->type &= ~(STR|NUM|DONTFREE);
					oargs[i]->str = t->str;
					TEMPFREE(eawk, t);
				}
			}
		} else if (t != y) {	/* kludge to prevent freeing twice */
			t->csub = CTEMP;
			TEMPFREE(eawk, t);
		} else if (t == y && t->csub == CCOPY) {
			t->csub = CTEMP;
			TEMPFREE(eawk, t);
			freed = 1;
		}
	}
	TEMPFREE(eawk, fcn);
	if (ISEXIT(y) || ISNEXT(y))
		return y;
	if (freed == 0) {
		TEMPFREE(eawk, y);	/* don't free twice! */
	}
	z = eawk->fp->retval;			/* return value */
	DPRINTF(eawk, ("%s returns %g |%s| %o\n", s, eawk_getfval(eawk, z), eawk_getsval(eawk, z), z->type) );
	eawk->fp--;
	return(z);
}

/* make a copy of a cell in a temp */
awkcell_t *
eawk_copycell(eawk_t *eawk, awkcell_t *x)
{
	awkcell_t *y;

	/* copy is not constant or field */

	y = eawk_gettemp(eawk);
	y->type = x->type & ~(CON|FLD|REC);
	y->csub = CCOPY;	/* prevents freeing until call is over */
	y->name = x->name;	/* BUG? */
	if (ISSTR(x) /* || x->ctype == OCELL */) {
		y->str = eawk_tostring(eawk, x->str);
		y->type &= ~DONTFREE;
	} else
		y->type |= DONTFREE;
	y->num = x->num;
	return y;
}

/* nth argument of a function */
awkcell_t *
eawk_arg(eawk_t *eawk, awknode_t **a, int n)
{

	n = eawk_ptoi(a[0]);	/* argument number, counting from 0 */
	DPRINTF(eawk, ("arg(%d), fp->nargs=%d\n", n, eawk->fp->nargs) );
	if (n+1 > eawk->fp->nargs) {
		eawk_warning(eawk, "argument #%d of function %s was not supplied",
			n+1, eawk->fp->fcncell->name);
		return NULL;
	}
	return eawk->fp->args[n];
}

/* break, continue, next, nextfile, return */
awkcell_t *
eawk_jump(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *y;

	switch (n) {
	case EXIT:
		if (a[0] != NULL) {
			y = eawk_execute(eawk, a[0]);
			eawk->errorflag = (int) eawk_getfval(eawk, y);
			TEMPFREE(eawk, y);
		}
		longjmp(eawk->env, 1);
		/*FALLTHROUGH*/
	case RETURN:
		if (a[0] != NULL) {
			y = eawk_execute(eawk, a[0]);
			if ((y->type & (STR|NUM)) == (STR|NUM)) {
				eawk_setsval(eawk, eawk->fp->retval, eawk_getsval(eawk, y));
				eawk->fp->retval->num = eawk_getfval(eawk, y);
				eawk->fp->retval->type |= NUM;
			}
			else if (y->type & STR)
				eawk_setsval(eawk, eawk->fp->retval, eawk_getsval(eawk, y));
			else if (y->type & NUM)
				eawk_setfval(eawk, eawk->fp->retval, eawk_getfval(eawk, y));
			else {
				/* can't happen */
				eawk_warning(eawk, "bad type variable %d", y->type);
				return 0;
			}
			TEMPFREE(eawk, y);
		}
		return(jret);
	case NEXT:
		return(jnext);
	case NEXTFILE:
		eawk_nextfile(eawk);
		return(jnextfile);
	case BREAK:
		return(jbreak);
	case CONTINUE:
		return(jcont);
	default:	/* can't happen */
		eawk_warning(eawk, "illegal jump type %d", n);
	}
	return 0;
}

/* get next line from specific input */
/* a[0] is variable, a[1] is operator, a[2] is filename */
awkcell_t *
eawk_getline(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *r, *x;
	FILE *fp;
	uint8_t *buf;
	int bufsize = eawk->recsize;
	int mode;

	if ((buf = calloc(1, bufsize)) == NULL) {
		eawk_warning(eawk, "out of memory in getline");
		return NULL;
	}

	fflush(stdout);	/* in case someone is waiting for a prompt */
	r = eawk_gettemp(eawk);
	if (a[1] != NULL) {		/* getline < file */
		x = eawk_execute(eawk, a[2]);		/* filename */
		mode = eawk_ptoi(a[1]);
		if (mode == '|')		/* input pipe */
			mode = LE;	/* arbitrary flag */
		fp = openfile(eawk, mode, eawk_getsval(eawk, x));
		TEMPFREE(eawk, x);
		if (fp == NULL)
			n = -1;
		else
			n = eawk_readrec(eawk, &buf, &bufsize, fp);
		if (n <= 0) {
			;
		} else if (a[0] != NULL) {	/* getline var <file */
			x = eawk_execute(eawk, a[0]);
			eawk_setsval(eawk, x, (char *)buf);
			TEMPFREE(eawk, x);
		} else {			/* getline <file */
			eawk_setsval(eawk, eawk->fldtab[0], (char *)buf);
			if (eawk_is_number(eawk->fldtab[0]->str)) {
				eawk->fldtab[0]->num = atof(eawk->fldtab[0]->str);
				eawk->fldtab[0]->type |= NUM;
			}
		}
	} else {			/* bare getline; use current input */
		if (a[0] == NULL)	/* getline */
			n = eawk_getrec(eawk, &eawk->record, &eawk->recsize, 1);
		else {			/* getline var */
			n = eawk_getrec(eawk, &buf, &bufsize, 0);
			x = eawk_execute(eawk, a[0]);
			eawk_setsval(eawk, x, (char *)buf);
			TEMPFREE(eawk, x);
		}
	}
	eawk_setfval(eawk, r, (awknum_t) n);
	free(buf);
	return r;
}

/* get NF */
awkcell_t *
eawk_getnf(eawk_t *eawk, awknode_t **a, int n)
{
	USE_ARG(n);
	if (eawk->donefld == 0)
		eawk_fldbld(eawk);
	return (awkcell_t *)(void *)a[0];
}

/* a[0] is symtab, a[1] is list of subscripts */
awkcell_t *
eawk_array(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x, *y, *z;
	char *s;
	awknode_t *np;
	uint8_t *buf;
	int bufsz = (int)eawk->recsize;
	int nsub = (int)strlen(*eawk->SUBSEP);

	USE_ARG(n);
	if ((buf = calloc(1, bufsz)) == NULL) {
		eawk_warning(eawk, "out of memory in array");
		return NULL;
	}

	x = eawk_execute(eawk, a[0]);	/* awkcell_t* for symbol table */
	buf[0] = 0;
	for (np = a[1]; np; np = np->nnext) {
		y = eawk_execute(eawk, np);	/* subscript */
		s = eawk_getsval(eawk, y);
		if (!eawk_adjbuf(eawk, &buf, &bufsz, (int)(strlen((char *)buf)+strlen(s)+nsub+1), eawk->recsize, 0, "array")) {
			eawk_warning(eawk, "out of memory for %s[%s...]", x->name, buf);
			return NULL;
		}
		strlcat((char *)buf, s, bufsz);
		if (np->nnext)
			strlcat((char *)buf, *eawk->SUBSEP, bufsz);
		TEMPFREE(eawk, y);
	}
	if (!ISARR(x)) {
		DPRINTF(eawk, ("making %s into an array\n", NN(x->name)) );
		if (FREEABLE(x))
			XFREE(x->str);
		x->type &= ~(STR|NUM|DONTFREE);
		x->type |= ARR;
		x->str = (char *)(void *)eawk_makesymtab(eawk, NSYMTAB);
	}
	z = eawk_var_set(eawk, (char *)(void *)buf, "", 0.0, STR|NUM, (awkarray_t *)(void *)x->str);
	z->ctype = OCELL;
	z->csub = CVAR;
	TEMPFREE(eawk, x);
	free(buf);
	return(z);
}

awkcell_t *
eawk_delete(eawk_t *eawk, awknode_t **a, int n)	/* a[0] is symtab, a[1] is list of subscripts */
{
	awkcell_t *x, *y;
	awknode_t *np;
	uint8_t *s;
	int nsub = (int)strlen(*eawk->SUBSEP);

	USE_ARG(n);
	x = eawk_execute(eawk, a[0]);	/* awkcell_t* for symbol table */
	if (!ISARR(x))
		return True;
	if (a[1] == 0) {	/* delete the elements, not the table */
		eawk_freesymtab(eawk, x);
		x->type &= ~STR;
		x->type |= ARR;
		x->str = (char *)(void *)eawk_makesymtab(eawk, NSYMTAB);
	} else {
		int bufsz = eawk->recsize;
		uint8_t *buf;
		if ((buf = calloc(1, bufsz)) == NULL) {
			eawk_warning(eawk, "out of memory in adelete");
			return NULL;
		}
		buf[0] = 0;
		for (np = a[1]; np; np = np->nnext) {
			y = eawk_execute(eawk, np);	/* subscript */
			s = (uint8_t *)eawk_getsval(eawk, y);
			if (!eawk_adjbuf(eawk, &buf, &bufsz, (int)(strlen((char *)buf)+strlen((char *)s)+nsub+1), eawk->recsize, 0, "eawk_delete")) {
				eawk_warning(eawk, "out of memory deleting %s[%s...]", x->name, buf);
				return NULL;
			}
			strlcat((char *)buf, (char *)s, bufsz);
			if (np->nnext)
				strlcat((char *)buf, *eawk->SUBSEP, bufsz);
			TEMPFREE(eawk, y);
		}
		eawk_freeelem(x, (char *)buf);
		free(buf);
	}
	TEMPFREE(eawk, x);
	return True;
}

/* a[0] is index (list), a[1] is symtab */
awkcell_t *
eawk_intest(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x, *ap, *k;
	awknode_t *p;
	uint8_t *buf;
	char *s;
	int bufsz = (int)eawk->recsize;
	int nsub = (int)strlen(*eawk->SUBSEP);

	USE_ARG(n);
	ap = eawk_execute(eawk, a[1]);	/* array name */
	if (!ISARR(ap)) {
		DPRINTF(eawk, ("making %s into an array\n", ap->name) );
		if (FREEABLE(ap))
			XFREE(ap->str);
		ap->type &= ~(STR|NUM|DONTFREE);
		ap->type |= ARR;
		ap->str = (char *)(void *)eawk_makesymtab(eawk, NSYMTAB);
	}
	if ((buf = malloc(bufsz)) == NULL) {
		eawk_warning(eawk, "out of memory in intest");
		return NULL;
	}
	buf[0] = 0;
	for (p = a[0]; p; p = p->nnext) {
		x = eawk_execute(eawk, p);	/* expr */
		s = eawk_getsval(eawk, x);
		if (!eawk_adjbuf(eawk, &buf, &bufsz, (int)(strlen((char *)buf)+strlen(s)+nsub+1), eawk->recsize, 0, "intest")) {
			eawk_warning(eawk, "out of memory deleting %s[%s...]", x->name, buf);
			return NULL;
		}
		strcat((char *)buf, s);
		TEMPFREE(eawk, x);
		if (p->nnext)
			strcat((char *)buf, *eawk->SUBSEP);
	}
	k = eawk_lookup((char *)(void *)buf, (awkarray_t *)(void *)ap->str);
	TEMPFREE(eawk, ap);
	free(buf);
	if (k == NULL)
		return(False);
	else
		return(True);
}


/* ~ and match() */
awkcell_t *
eawk_matchop(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x, *y;
	uint8_t *s;
	char *t;
	int i;
	fa_t *pfa;
	int (*mf)(eawk_t *, void *, const char *) = eawk_match, mode = 0;

	if (n == MATCHFCN) {
		mf = eawk_pmatch;
		mode = 1;
	}
	x = eawk_execute(eawk, a[1]);	/* a[1] = target text */
	s = (uint8_t *)eawk_getsval(eawk, x);
	if (a[0] == 0)		/* a[1] == 0: already-compiled reg expr */
		i = (*mf)(eawk, (fa_t *)(void *)a[2], (char *)s);
	else {
		y = eawk_execute(eawk, a[2]);	/* a[2] = regular expr */
		t = eawk_getsval(eawk, y);
		pfa = eawk_regcomp(eawk, t, mode);
		i = (*mf)(eawk, pfa, (char *)s);
		TEMPFREE(eawk, y);
	}
	TEMPFREE(eawk, x);
	if (n == MATCHFCN) {
		int start = (int)(eawk->patbeg - s) + 1;
		if (eawk->patlen < 0)
			start = 0;
		eawk_setfval(eawk, eawk->rstartloc, (awknum_t) start);
		eawk_setfval(eawk, eawk->rlengthloc, (awknum_t) eawk->patlen);
		x = eawk_gettemp(eawk);
		x->type = NUM;
		x->num = start;
		return x;
	} else if ((n == MATCH && i == 1) || (n == NOTMATCH && i == 0))
		return(True);
	else
		return(False);
}


/* a[0] || a[1], a[0] && a[1], !a[0] */
awkcell_t *
eawk_boolop(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x, *y;
	int i;

	x = eawk_execute(eawk, a[0]);
	i = ISTRUE(x);
	TEMPFREE(eawk, x);
	switch (n) {
	case BOR:
		if (i) return(True);
		y = eawk_execute(eawk, a[1]);
		i = ISTRUE(y);
		TEMPFREE(eawk, y);
		if (i) return(True);
		else return(False);
	case AND:
		if ( !i ) return(False);
		y = eawk_execute(eawk, a[1]);
		i = ISTRUE(y);
		TEMPFREE(eawk, y);
		if (i) return(True);
		else return(False);
	case NOT:
		if (i) return(False);
		else return(True);
	default:	/* can't happen */
		eawk_warning(eawk, "unknown boolean operator %d", n);
	}
	return 0;
}

/* a[0 < a[1], etc. */
awkcell_t *
eawk_relop(eawk_t *eawk, awknode_t **a, int n)
{
	int i;
	awkcell_t *x, *y;
	awknum_t j;

	x = eawk_execute(eawk, a[0]);
	y = eawk_execute(eawk, a[1]);
	if (x->type&NUM && y->type&NUM) {
		j = x->num - y->num;
		i = j<0? -1: (j>0? 1: 0);
	} else {
		i = strcmp(eawk_getsval(eawk, x), eawk_getsval(eawk, y));
	}
	TEMPFREE(eawk, x);
	TEMPFREE(eawk, y);
	switch (n) {
	case LT:	if (i<0) return(True);
			else return(False);
	case LE:	if (i<=0) return(True);
			else return(False);
	case NE:	if (i!=0) return(True);
			else return(False);
	case EQ:	if (i == 0) return(True);
			else return(False);
	case GE:	if (i>=0) return(True);
			else return(False);
	case GT:	if (i>0) return(True);
			else return(False);
	default:	/* can't happen */
		eawk_warning(eawk, "unknown relational operator %d", n);
	}
	return NULL;
}

/* get a tempcell */
awkcell_t *
eawk_gettemp(eawk_t *eawk)
{
	int i;
	awkcell_t *x;

	if (!eawk->tmps) {
		eawk->tmps = calloc(100, sizeof(awkcell_t));
		if (!eawk->tmps) {
			eawk_warning(eawk, "out of space for temporaries");
			return NULL;
		}
		for(i = 1; i < 100; i++)
			eawk->tmps[i-1].cnext = &eawk->tmps[i];
		eawk->tmps[i-1].cnext = 0;
	}
	x = eawk->tmps;
	eawk->tmps = x->cnext;
	*x = tempcell;
	return(x);
}

/* $( a[0] ) */
awkcell_t *
eawk_indirect(eawk_t *eawk, awknode_t **a, int n)
{
	awknum_t val;
	awkcell_t *x;
	int m;
	char *s;

	USE_ARG(n);
	x = eawk_execute(eawk, a[0]);
	val = eawk_getfval(eawk, x);	/* freebsd: defend against super large field numbers */
	if ((awknum_t)INT_MAX < val) {
		eawk_warning(eawk, "trying to access out of range field %s", x->name);
		return NULL;
	}
	m = (int) val;
	if (m == 0 && !eawk_is_number(s = eawk_getsval(eawk, x))) {
		/* suspicion! */
		eawk_warning(eawk, "illegal field $(%s), name \"%s\"", s, x->name);
		/* BUG: can x->name ever be null??? */
		return NULL;
	}
	TEMPFREE(eawk, x);
	x = eawk_fieldadr(eawk, m);
	x->ctype = OCELL;	/* BUG?  why are these needed? */
	x->csub = CFLD;
	return(x);
}

/* substr(a[0], a[1], a[2]) */
awkcell_t *
eawk_substr(eawk_t *eawk, awknode_t **a, int nnn)
{
	int k, m, n;
	char *s;
	int temp;
	awkcell_t *x, *y, *z = 0;

	USE_ARG(nnn);
	x = eawk_execute(eawk, a[0]);
	y = eawk_execute(eawk, a[1]);
	if (a[2] != 0)
		z = eawk_execute(eawk, a[2]);
	s = eawk_getsval(eawk, x);
	k = (int)(strlen(s) + 1);
	if (k <= 1) {
		TEMPFREE(eawk, x);
		TEMPFREE(eawk, y);
		if (a[2] != 0) {
			TEMPFREE(eawk, z);
		}
		x = eawk_gettemp(eawk);
		eawk_setsval(eawk, x, "");
		return(x);
	}
	m = (int) eawk_getfval(eawk, y);
	if (m <= 0)
		m = 1;
	else if (m > k)
		m = k;
	TEMPFREE(eawk, y);
	if (a[2] != 0) {
		n = (int) eawk_getfval(eawk, z);
		TEMPFREE(eawk, z);
	} else
		n = k - 1;
	if (n < 0)
		n = 0;
	else if (n > k - m)
		n = k - m;
	DPRINTF(eawk, ("substr: m=%d, n=%d, s=%s\n", m, n, s) );
	y = eawk_gettemp(eawk);
	temp = s[n+m-1];	/* with thanks to John Linderman */
	s[n+m-1] = '\0';
	eawk_setsval(eawk, y, s + m - 1);
	s[n+m-1] = temp;
	TEMPFREE(eawk, x);
	return(y);
}

awkcell_t *
eawk_index(eawk_t *eawk, awknode_t **a, int nnn)		/* index(a[0], a[1]) */
{
	awkcell_t *x, *y, *z;
	char *s1, *s2, *p1, *p2, *q;
	awknum_t v = 0.0;
 
	USE_ARG(nnn);
	x = eawk_execute(eawk, a[0]);
	s1 = eawk_getsval(eawk, x);
	y = eawk_execute(eawk, a[1]);
	s2 = eawk_getsval(eawk, y);

	z = eawk_gettemp(eawk);
	for (p1 = s1; *p1 != '\0'; p1++) {
		for (q=p1, p2=s2; *p2 != '\0' && *q == *p2; q++, p2++)
			;
		if (*p2 == '\0') {
			v = (awknum_t) (p1 - s1 + 1);	/* origin 1 */
			break;
		}
	}
	TEMPFREE(eawk, x);
	TEMPFREE(eawk, y);
	eawk_setfval(eawk, z, v);
	return(z);
}

#define	MAXNUMSIZE	50

/* printf-like conversions */
static int
format(eawk_t *eawk, char **pbuf, int *pbufsize, const char *s, awknode_t *a)
{
	uint8_t *fmt, *p, *t;
	const char *os;
	awkcell_t *x;
	int flag = 0, n;
	int fmtwd; /* format width */
	int fmtsz = eawk->recsize;
	uint8_t *buf = (uint8_t *)*pbuf;
	int bufsize = *pbufsize;
#define FMTSZ(a)   (fmtsz - ((a) - fmt))
#define BUFSZ(a)   (bufsize - ((a) - buf))

	os = s;
	p = buf;
	if ((fmt = calloc(1, fmtsz)) == NULL) {
		eawk_warning(eawk, "out of memory in format()");
		return 0;
	}
	while (*s) {
		eawk_adjbuf(eawk, &buf, &bufsize, (int)(MAXNUMSIZE+1+p-buf), eawk->recsize, &p, "format1");
		if (*s != '%') {
			*p++ = *s++;
			continue;
		}
		if (*(s+1) == '%') {
			*p++ = '%';
			s += 2;
			continue;
		}
		/* have to be real careful in case this is a huge number, eg, %100000d */
		fmtwd = atoi(s+1);
		if (fmtwd < 0)
			fmtwd = -fmtwd;
		eawk_adjbuf(eawk, &buf, &bufsize, (int)(fmtwd+1+p-buf), eawk->recsize, &p, "format2");
		for (t = fmt; (*t++ = *s) != '\0'; s++) {
			if (!eawk_adjbuf(eawk, &fmt, &fmtsz, (int)(MAXNUMSIZE+1+t-fmt), eawk->recsize, &t, "format3")) {
				eawk_warning(eawk, "format item %.30s... ran format() out of memory", os);
				return 0;
			}
			if (*s == 'l' || *s == 'h' || *s == 'L' || *s == 'q')
				goto weird;
			if (isalpha((uint8_t)*s))
				break;	/* the ansi panoply */
			if (*s == '*') {
				if (a == NULL) {
					eawk_warning(eawk, "not enough args in printf("
					    "\"%.30s\")", os);
					return 0;
				}
				x = eawk_execute(eawk, a);
				a = a->nnext;
				snprintf((char *)t - 1, (size_t)(FMTSZ(t - 1)), 
				    "%d", fmtwd=(int) eawk_getfval(eawk, x));
				if (fmtwd < 0)
					fmtwd = -fmtwd;
				eawk_adjbuf(eawk, &buf, &bufsize, (int)(fmtwd+1+p-buf), eawk->recsize, &p, "format");
				t = fmt + strlen((char *)fmt);
				TEMPFREE(eawk, x);
			}
		}
		*t = '\0';
		if (fmtwd < 0)
			fmtwd = -fmtwd;
		eawk_adjbuf(eawk, &buf, &bufsize, (int)(fmtwd+1+p-buf), eawk->recsize, &p, "format4");

		switch (*s) {
		case 'f': case 'e': case 'g': case 'E': case 'G':
			flag = 'f';
			break;
		case 'o': case 'u':
		case 'x': case 'X':
		case 'd': case 'i':
			flag = 'd';
			if(*(s-1) == 'l') break;
			*(t-1) = 'l';
			*t = *s;
			*++t = '\0';
			break;
		case 's':
			flag = 's';
			break;
		case 'c':
			flag = 'c';
			break;
		default:
weird:
			eawk_warning(eawk, "weird printf conversion %s", fmt);
			flag = '?';
			break;
		}
		if (a == NULL) {
			eawk_warning(eawk, "not enough args in printf(%s)", os);
			return 0;
		}
		x = eawk_execute(eawk, a);
		a = a->nnext;
		n = MAXNUMSIZE;
		if (fmtwd > n)
			n = fmtwd;
		eawk_adjbuf(eawk, &buf, &bufsize, (int)(1+n+p-buf), eawk->recsize, &p, "format5");
		switch (flag) {
		case '?':
			snprintf((char *)p, (size_t)BUFSZ(p), "%s", fmt);	/* unknown, so dump it too */
			t = (uint8_t *)eawk_getsval(eawk, x);
			n = (int)strlen((char *)t);
			if (fmtwd > n)
				n = fmtwd;
			eawk_adjbuf(eawk, &buf, &bufsize, (int)(1+strlen((char *)p)+n+p-buf), eawk->recsize, &p, "format6");
			p += strlen((char *)p);
			snprintf((char *)p, (size_t)BUFSZ(p), "%s", t);
			break;
		case 'f':
			snprintf((char *)p, (size_t)BUFSZ(p), (const char *)fmt, eawk_getfval(eawk, x));
			break;
		case 'd':
			snprintf((char *)p, (size_t)BUFSZ(p), (const char *)fmt, (long) eawk_getfval(eawk, x));
			break;
		case 'u':
			snprintf((char *)p, (size_t)BUFSZ(p), (const char *)fmt, (int) eawk_getfval(eawk, x));
			break;
		case 's':
			t = (uint8_t *)eawk_getsval(eawk, x);
			n = (int)strlen((char *)t);
			if (fmtwd > n)
				n = fmtwd;
			if (!eawk_adjbuf(eawk, &buf, &bufsize, (int)(1+n+p-buf), eawk->recsize, &p, "format7")) {
				eawk_warning(eawk, "huge string/format (%d chars) in printf %.30s... ran format() out of memory", n, t);
				return 0;
			}
			snprintf((char *)p, (size_t)BUFSZ(p), (char *)fmt, t);
			break;
		case 'c':
			if (ISNUM(x)) {
				if (eawk_getfval(eawk, x))
					snprintf((char *)p, (size_t)BUFSZ(p), (char *)fmt, (int) eawk_getfval(eawk, x));
				else {
					*p++ = '\0'; /* explicit null byte */
					*p = '\0';   /* next output will start here */
				}
			} else
				snprintf((char *)p, (size_t)BUFSZ(p), (char *)fmt, eawk_getsval(eawk, x)[0]);
			break;
		default:
			eawk_warning(eawk, "can't happen: bad conversion %c in format()", flag);
			return 0;
		}
		TEMPFREE(eawk, x);
		p += strlen((char *)p);
		s++;
	}
	*p = '\0';
	free(fmt);
	for ( ; a; a = a->nnext)		/* evaluate any remaining args */
		eawk_execute(eawk, a);
	*pbuf = (char *)buf;
	*pbufsize = bufsize;
	return (int)(p - buf);
}

/* sprintf(a[0]) */
awkcell_t *
eawk_sprintf(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;
	awknode_t *y;
	char *buf;
	int bufsz=3*eawk->recsize;

	USE_ARG(n);
	if ((buf = calloc(1, bufsz)) == NULL) {
		eawk_warning(eawk, "out of memory in eawk_sprintf");
		return NULL;
	}
	y = a[0]->nnext;
	x = eawk_execute(eawk, a[0]);
	if (format(eawk, &buf, &bufsz, eawk_getsval(eawk, x), y) == -1) {
		eawk_warning(eawk, "sprintf string %.30s... too long.  can't happen.", buf);
		return NULL;
	}
	TEMPFREE(eawk, x);
	x = eawk_gettemp(eawk);
	x->str = buf;
	x->type = STR;
	return(x);
}

/* printf */
awkcell_t *
eawk_printf(eawk_t *eawk, awknode_t **a, int n)
{	/* a[0] is list of args, starting with format string */
	/* a[1] is redirection operator, a[2] is redirection file */
	FILE *fp;
	awkcell_t *x;
	awknode_t *y;
	char *buf;
	int len;
	int bufsz=3*eawk->recsize;

	USE_ARG(n);
	if ((buf = calloc(1, bufsz)) == NULL) {
		eawk_warning(eawk, "out of memory in eawk_printf");
		return NULL;
	}
	y = a[0]->nnext;
	x = eawk_execute(eawk, a[0]);
	if ((len = format(eawk, &buf, &bufsz, eawk_getsval(eawk, x), y)) == -1) {
		eawk_warning(eawk, "printf string %.30s... too long.  can't happen.", buf);
		return NULL;
	}
	TEMPFREE(eawk, x);
	if (a[1] == NULL) {
		/* fputs(buf, stdout); */
		fwrite(buf, len, 1, stdout);
		if (ferror(stdout)) {
			eawk_warning(eawk, "write error on stdout");
			return NULL;
		}
	} else {
		fp = eawk_redirect(eawk, eawk_ptoi(a[1]), a[2]);
		/* fputs(buf, fp); */
		fwrite(buf, len, 1, fp);
		fflush(fp);
		if (ferror(fp)) {
			eawk_warning(eawk, "write error on %s", filename(eawk, fp));
			return NULL;
		}
	}
	free(buf);
	return(True);
}

/* a[0] + a[1], etc.  also -a[0] */
awkcell_t *
eawk_arith(eawk_t *eawk, awknode_t **a, int n)
{
	awknum_t i, j = 0;
	awknum_t v;
	awkcell_t *x, *y, *z;

	x = eawk_execute(eawk, a[0]);
	i = eawk_getfval(eawk, x);
	TEMPFREE(eawk, x);
	if (n != UMINUS) {
		y = eawk_execute(eawk, a[1]);
		j = eawk_getfval(eawk, y);
		TEMPFREE(eawk, y);
	}
	z = eawk_gettemp(eawk);
	switch (n) {
	case ADD:
		i += j;
		break;
	case MINUS:
		i -= j;
		break;
	case MULT:
		i *= j;
		break;
	case DIVIDE:
		if (j == 0) {
			eawk_warning(eawk, "division by zero");
			return NULL;
		}
		i /= j;
		break;
	case MOD:
		if (j == 0) {
			eawk_warning(eawk, "division by zero in mod");
			return NULL;
		}
		modf(i/j, &v);
		i = i - j * v;
		break;
	case UMINUS:
		i = -i;
		break;
	case POWER:
		if (j >= 0 && modf(j, &v) == 0.0)	/* pos integer exponent */
			i = eawk_ipow(i, (int) j);
		else
			i = errcheck(eawk, pow(i, j), "pow");
		break;
	default:	/* can't happen */
		eawk_warning(eawk, "illegal arithmetic operator %d", n);
		return NULL;
	}
	eawk_setfval(eawk, z, i);
	return(z);
}

/* x**n.  ought to be done by pow, but isn't always */
awknum_t
eawk_ipow(awknum_t x, int n)
{
	awknum_t v;

	if (n <= 0)
		return 1;
	v = eawk_ipow(x, n/2);
	if (n % 2 == 0)
		return v * v;
	else
		return x * v * v;
}

/* bitwise operations */
awkcell_t *
eawk_bitarith(eawk_t *eawk, awknode_t **a, int op)
{
	awkcell_t	*x, *y, *res;
	uint32_t	 i;
	uint32_t	 j = 0;

	x = eawk_execute(eawk, a[0]);
	i = (uint32_t)eawk_getfval(eawk, x);
	TEMPFREE(eawk, x);
	if (op != BITCOMPL) {
		y = eawk_execute(eawk, a[1]);
		j = (uint32_t)eawk_getfval(eawk, y);
		TEMPFREE(eawk, y);
	}
	res = eawk_gettemp(eawk);
	switch (op) {
	case BITAND:
		i &= j;
		break;
	case BITOR:
		i |= j;
		break;
	case BITXOR:
		i ^= j;
		break;
	case BITCOMPL:
		i = ~i;
		break;
	case BITLSHIFT:
		i <<= j;
		break;
	case BITRSHIFT:
		i >>= j;
		break;
	default:
		(void) fprintf(stderr, "bitarith: not reached\n");
		break;
	}
	eawk_setfval(eawk, res, (awknum_t)i);
	return res;
}

/* a[0]++, etc. */
awkcell_t *
eawk_incrdecr(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x, *z;
	int k;
	awknum_t xf;

	x = eawk_execute(eawk, a[0]);
	xf = eawk_getfval(eawk, x);
	k = (n == PREINCR || n == POSTINCR) ? 1 : -1;
	if (n == PREINCR || n == PREDECR) {
		eawk_setfval(eawk, x, xf + k);
		return(x);
	}
	z = eawk_gettemp(eawk);
	eawk_setfval(eawk, z, xf);
	eawk_setfval(eawk, x, xf + k);
	TEMPFREE(eawk, x);
	return(z);
}

/* a[0] = a[1], a[0] += a[1], etc. */
awkcell_t *
eawk_assign(eawk_t *eawk, awknode_t **a, int n)
{
	/* this is subtle; don't muck with it. */
	awkcell_t *x, *y;
	awknum_t xf, yf;
	awknum_t v;

	y = eawk_execute(eawk, a[1]);
	x = eawk_execute(eawk, a[0]);
	if (n == ASSIGN) {	/* ordinary assignment */
		if (x == y && !(x->type & (FLD|REC)))	/* self-assignment: */
			;		/* leave alone unless it's a field */
		else if ((y->type & (STR|NUM)) == (STR|NUM)) {
			eawk_setsval(eawk, x, eawk_getsval(eawk, y));
			x->num = eawk_getfval(eawk, y);
			x->type |= NUM;
		}
		else if (ISSTR(y))
			eawk_setsval(eawk, x, eawk_getsval(eawk, y));
		else if (ISNUM(y))
			eawk_setfval(eawk, x, eawk_getfval(eawk, y));
		else
			eawk_funnyvar(eawk, y, "read value of");
		TEMPFREE(eawk, y);
		return(x);
	}
	xf = eawk_getfval(eawk, x);
	yf = eawk_getfval(eawk, y);
	switch (n) {
	case ADDEQ:
		xf += yf;
		break;
	case SUBEQ:
		xf -= yf;
		break;
	case MULTEQ:
		xf *= yf;
		break;
	case DIVEQ:
		if (yf == 0) {
			eawk_warning(eawk, "division by zero in /=");
			return NULL;
		}
		xf /= yf;
		break;
	case MODEQ:
		if (yf == 0) {
			eawk_warning(eawk, "division by zero in %%=");
			return NULL;
		}
		modf(xf/yf, &v);
		xf = xf - yf * v;
		break;
	case POWEQ:
		if (yf >= 0 && modf(yf, &v) == 0.0)	/* pos integer exponent */
			xf = eawk_ipow(xf, (int) yf);
		else
			xf = errcheck(eawk, pow(xf, yf), "pow");
		break;
	default:
		eawk_warning(eawk, "illegal assignment operator %d", n);
		return NULL;
	}
	TEMPFREE(eawk, y);
	eawk_setfval(eawk, x, xf);
	return(x);
}

/* a[0] cat a[1] */
awkcell_t *
eawk_cat(eawk_t *eawk, awknode_t **a, int q)
{
	awkcell_t *x, *y, *z;
	size_t n1, n2;
	char *s;

	USE_ARG(q);
	x = eawk_execute(eawk, a[0]);
	y = eawk_execute(eawk, a[1]);
	eawk_getsval(eawk, x);
	eawk_getsval(eawk, y);
	n1 = strlen(x->str);
	n2 = strlen(y->str);
	s = calloc(1, n1 + n2 + 1);
	if (s == NULL) {
		eawk_warning(eawk, "out of space concatenating %.15s... and %.15s...",
			x->str, y->str);
		return NULL;
	}
	strcpy(s, x->str);
	strcpy(s+n1, y->str);
	TEMPFREE(eawk, y);
	z = eawk_gettemp(eawk);
	z->str = s;
	z->type = STR;
	TEMPFREE(eawk, x);
	return(z);
}

/* a[0] { a[1] } */
awkcell_t *
eawk_pastat(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;

	USE_ARG(n);
	if (a[0] == 0)
		x = eawk_execute(eawk, a[1]);
	else {
		x = eawk_execute(eawk, a[0]);
		if (ISTRUE(x)) {
			TEMPFREE(eawk, x);
			x = eawk_execute(eawk, a[1]);
		}
	}
	return x;
}

/* a[0], a[1] { a[2] } */
awkcell_t *
eawk_dopa2(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;
	int pair;

	USE_ARG(n);
	pair = eawk_ptoi(a[3]);
	if (eawk->pairstack[pair] == 0) {
		x = eawk_execute(eawk, a[0]);
		if (ISTRUE(x))
			eawk->pairstack[pair] = 1;
		TEMPFREE(eawk, x);
	}
	if (eawk->pairstack[pair] == 1) {
		x = eawk_execute(eawk, a[1]);
		if (ISTRUE(x))
			eawk->pairstack[pair] = 0;
		TEMPFREE(eawk, x);
		x = eawk_execute(eawk, a[2]);
		return(x);
	}
	return(False);
}

/* split(a[0], a[1], a[2]); a[3] is type */
awkcell_t *
eawk_split(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t *x = 0, *y, *ap;
	char *s;
	int sep;
	char *t, temp, num[50], *fs = 0;
	int n, tempstat, arg3type;

	USE_ARG(nnn);
	y = eawk_execute(eawk, a[0]);	/* source string */
	s = eawk_getsval(eawk, y);
	arg3type = eawk_ptoi(a[3]);
	if (a[2] == NULL)		/* fs string */
		fs = *eawk->FS;
	else if (arg3type == STRING) {	/* split(str,arr,"string") */
		x = eawk_execute(eawk, a[2]);
		fs = eawk_getsval(eawk, x);
	} else if (arg3type == REGEXPR)
		fs = __UNCONST("(regexpr)");	/* split(str,arr,/regexpr/) */
	else {
		eawk_warning(eawk, "illegal type of split");
		return NULL;
	}
	sep = *fs;
	ap = eawk_execute(eawk, a[1]);	/* array name */
	eawk_freesymtab(eawk, ap);
	DPRINTF(eawk, ("split: s=|%s|, a=%s, sep=|%s|\n", s, NN(ap->name), fs) );
	ap->type &= ~STR;
	ap->type |= ARR;
	ap->str = (char *)(void *)eawk_makesymtab(eawk, NSYMTAB);

	n = 0;
	if (*s != '\0' && (strlen(fs) > 1 || arg3type == REGEXPR)) {	/* reg expr */
		fa_t *pfa;
		if (arg3type == REGEXPR) {	/* it's ready already */
			pfa = (fa_t *)(void *)a[2];
		} else {
			pfa = eawk_regcomp(eawk, fs, 1);
		}
		if (eawk_nematch(eawk, pfa,s)) {
			tempstat = pfa->initstat;
			pfa->initstat = 2;
			do {
				n++;
				snprintf(num, sizeof(num), "%d", n);
				temp = *eawk->patbeg;
				*eawk->patbeg = '\0';
				if (eawk_is_number(s))
					eawk_var_set(eawk, num, s, atof(s), STR|NUM, (awkarray_t *)(void *)ap->str);
				else
					eawk_var_set(eawk, num, s, 0.0, STR, (awkarray_t *)(void *)ap->str);
				*eawk->patbeg = temp;
				s = (char *)eawk->patbeg + eawk->patlen;
				if (*(eawk->patbeg+eawk->patlen-1) == 0 || *s == 0) {
					n++;
					snprintf(num, sizeof(num), "%d", n);
					eawk_var_set(eawk, num, "", 0.0, STR, (awkarray_t *)(void *)ap->str);
					pfa->initstat = tempstat;
					goto spdone;
				}
			} while (eawk_nematch(eawk, pfa,s));
			pfa->initstat = tempstat; 	/* bwk: has to be here to reset */
							/* cf gsub and refldbld */
		}
		n++;
		snprintf(num, sizeof(num), "%d", n);
		if (eawk_is_number(s))
			eawk_var_set(eawk, num, s, atof(s), STR|NUM, (awkarray_t *)(void *)ap->str);
		else
			eawk_var_set(eawk, num, s, 0.0, STR, (awkarray_t *)(void *)ap->str);
spdone:
		pfa = NULL;
	} else if (sep == ' ') {
		for (n = 0; ; ) {
			while (*s == ' ' || *s == '\t' || *s == '\n')
				s++;
			if (*s == 0)
				break;
			n++;
			t = s;
			do
				s++;
			while (*s!=' ' && *s!='\t' && *s!='\n' && *s!='\0');
			temp = *s;
			*s = '\0';
			snprintf(num, sizeof(num), "%d", n);
			if (eawk_is_number(t))
				eawk_var_set(eawk, num, t, atof(t), STR|NUM, (awkarray_t *)(void *)ap->str);
			else
				eawk_var_set(eawk, num, t, 0.0, STR, (awkarray_t *)(void *)ap->str);
			*s = temp;
			if (*s != 0)
				s++;
		}
	} else if (sep == 0) {	/* new: split(s, a, "") => 1 char/elem */
		for (n = 0; *s != 0; s++) {
			char buf[2];
			n++;
			snprintf(num, sizeof(num), "%d", n);
			buf[0] = *s;
			buf[1] = 0;
			if (isdigit((uint8_t)buf[0]))
				eawk_var_set(eawk, num, buf, atof(buf), STR|NUM, (awkarray_t *)(void *)ap->str);
			else
				eawk_var_set(eawk, num, buf, 0.0, STR, (awkarray_t *)(void *)ap->str);
		}
	} else if (*s != 0) {
		for (;;) {
			n++;
			t = s;
			while (*s != sep && *s != '\n' && *s != '\0')
				s++;
			temp = *s;
			*s = '\0';
			snprintf(num, sizeof(num), "%d", n);
			if (eawk_is_number(t))
				eawk_var_set(eawk, num, t, atof(t), STR|NUM, (awkarray_t *)(void *)ap->str);
			else
				eawk_var_set(eawk, num, t, 0.0, STR, (awkarray_t *)(void *)ap->str);
			*s = temp;
			if (*s++ == 0)
				break;
		}
	}
	TEMPFREE(eawk, ap);
	TEMPFREE(eawk, y);
	if (a[2] != 0 && arg3type == STRING) {
		TEMPFREE(eawk, x);
	}
	x = eawk_gettemp(eawk);
	x->type = NUM;
	x->num = n;
	return(x);
}

/* a[0] ? a[1] : a[2] */
awkcell_t *
eawk_condexpr(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;

	USE_ARG(n);
	x = eawk_execute(eawk, a[0]);
	if (ISTRUE(x)) {
		TEMPFREE(eawk, x);
		x = eawk_execute(eawk, a[1]);
	} else {
		TEMPFREE(eawk, x);
		x = eawk_execute(eawk, a[2]);
	}
	return(x);
}

/* if (a[0]) a[1]; else a[2] */
awkcell_t *
eawk_ifstat(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;

	USE_ARG(n);
	x = eawk_execute(eawk, a[0]);
	if (ISTRUE(x)) {
		TEMPFREE(eawk, x);
		x = eawk_execute(eawk, a[1]);
	} else if (a[2] != 0) {
		TEMPFREE(eawk, x);
		x = eawk_execute(eawk, a[2]);
	}
	return(x);
}

/* while (a[0]) a[1] */
awkcell_t *
eawk_whilestat(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;

	USE_ARG(n);
	for (;;) {
		x = eawk_execute(eawk, a[0]);
		if (!ISTRUE(x))
			return(x);
		TEMPFREE(eawk, x);
		x = eawk_execute(eawk, a[1]);
		if (ISBREAK(x)) {
			x = True;
			return(x);
		}
		if (ISNEXT(x) || ISEXIT(x) || ISRET(x))
			return(x);
		TEMPFREE(eawk, x);
	}
}

/* do a[0]; while(a[1]) */
awkcell_t *
eawk_dostat(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;

	USE_ARG(n);
	for (;;) {
		x = eawk_execute(eawk, a[0]);
		if (ISBREAK(x))
			return True;
		if (ISNEXT(x) || ISEXIT(x) || ISRET(x))
			return(x);
		TEMPFREE(eawk, x);
		x = eawk_execute(eawk, a[1]);
		if (!ISTRUE(x))
			return(x);
		TEMPFREE(eawk, x);
	}
}

/* for (a[0]; a[1]; a[2]) a[3] */
awkcell_t *
eawk_forstat(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;

	USE_ARG(n);
	x = eawk_execute(eawk, a[0]);
	TEMPFREE(eawk, x);
	for (;;) {
		if (a[1]!=0) {
			x = eawk_execute(eawk, a[1]);
			if (!ISTRUE(x)) return(x);
			else TEMPFREE(eawk, x);
		}
		x = eawk_execute(eawk, a[3]);
		if (ISBREAK(x))		/* turn off break */
			return True;
		if (ISNEXT(x) || ISEXIT(x) || ISRET(x))
			return(x);
		TEMPFREE(eawk, x);
		x = eawk_execute(eawk, a[2]);
		TEMPFREE(eawk, x);
	}
}

/* for (a[0] in a[1]) a[2] */
awkcell_t *
eawk_instat(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x, *vp, *arrayp, *cp, *ncp;
	awkarray_t *tp;
	int i;

	USE_ARG(n);
	vp = eawk_execute(eawk, a[0]);
	arrayp = eawk_execute(eawk, a[1]);
	if (!ISARR(arrayp)) {
		return True;
	}
	tp = (awkarray_t *)(void *)arrayp->str;
	TEMPFREE(eawk, arrayp);
	for (i = 0; i < tp->size; i++) {	/* this routine knows too much */
		for (cp = tp->tab[i]; cp != NULL; cp = ncp) {
			eawk_setsval(eawk, vp, cp->name);
			ncp = cp->cnext;
			x = eawk_execute(eawk, a[2]);
			if (ISBREAK(x)) {
				TEMPFREE(eawk, vp);
				return True;
			}
			if (ISNEXT(x) || ISEXIT(x) || ISRET(x)) {
				TEMPFREE(eawk, vp);
				return(x);
			}
			TEMPFREE(eawk, x);
		}
	}
	return True;
}

static char *
nawk_toXXX(eawk_t *eawk, const char *s, int (*fun_c)(int), wint_t (*fun_wc)(wint_t))
{
	char *buf      = NULL;
	char *pbuf     = NULL;
	const char *ps = NULL;
	size_t n       = 0;
	mbstate_t mbs, mbs2;
	wchar_t wc;
	size_t sz = MB_CUR_MAX;

	if (sz == 1) {
		buf = eawk_tostring(eawk, s);

		for (pbuf = buf; *pbuf; pbuf++)
			*pbuf = fun_c((uint8_t)*pbuf);

		return buf;
	} else {
		/* upper/lower character may be shorter/longer */
		buf = eawk_tostringN(eawk, s, strlen(s) * sz + 1);

		memset(&mbs,  0, sizeof(mbs));
		memset(&mbs2, 0, sizeof(mbs2));

		ps   = s;
		pbuf = buf;
		while (n = mbrtowc(&wc, ps, sz, &mbs),
		       n > 0 && n != (size_t)-1 && n != (size_t)-2)
		{
			ps += n;

			n = wcrtomb(pbuf, fun_wc(wc), &mbs2);
			if (n == (size_t)-1) {
				eawk_warning(eawk, "illegal wide character %s", s);
				return NULL;
			}

			pbuf += n;
		}

		*pbuf = 0;

		if (n) {
			eawk_warning(eawk, "illegal byte sequence %s", s);
			return NULL;
		}

		return buf;
	}
}

static char *
nawk_toupper(eawk_t *eawk, const char *s)
{
	return nawk_toXXX(eawk, s, toupper, towupper);
}

static char *
nawk_tolower(eawk_t *eawk, const char *s)
{
	return nawk_toXXX(eawk, s, tolower, towlower);
}

static void
flush_all(eawk_t *eawk)
{
	size_t i;

	for (i = 0; i < eawk->nfiles; i++)
		if (eawk->files[i].fp)
			fflush(eawk->files[i].fp);
}

/* builtin functions. a[0] is type, a[1] is arg list */
awkcell_t *
eawk_bltin(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x, *y;
	awknum_t u;
	int t, sz;
	char *buf, *fmt;
	awknode_t *nextarg;
	FILE *fp;
	time_t tv;
	struct tm *tm;

	USE_ARG(n);
	t = eawk_ptoi(a[0]);
	x = eawk_execute(eawk, a[1]);
	nextarg = a[1]->nnext;
	switch (t) {
	case FLENGTH:
		if (ISARR(x))
			u = ((awkarray_t *)(void *)x->str)->nelem;	/* GROT.  should be function*/
		else
			u = strlen(eawk_getsval(eawk, x));
		break;
	case FLOG:
		u = errcheck(eawk, log(eawk_getfval(eawk, x)), "log"); break;
	case FINT:
		modf(eawk_getfval(eawk, x), &u); break;
	case FEXP:
		u = errcheck(eawk, exp(eawk_getfval(eawk, x)), "exp"); break;
	case FSQRT:
		u = errcheck(eawk, sqrt(eawk_getfval(eawk, x)), "sqrt"); break;
	case FSIN:
		u = sin(eawk_getfval(eawk, x)); break;
	case FCOS:
		u = cos(eawk_getfval(eawk, x)); break;
	case FATAN:
		if (nextarg == 0) {
			eawk_warning(eawk, "atan2 requires two arguments; returning 1.0");
			u = 1.0;
		} else {
			y = eawk_execute(eawk, a[1]->nnext);
			u = atan2(eawk_getfval(eawk, x), eawk_getfval(eawk, y));
			TEMPFREE(eawk, y);
			nextarg = nextarg->nnext;
		}
		break;
	case FSYSTEM:
		fflush(stdout);		/* in case something is buffered already */
		u = (awknum_t) system(eawk_getsval(eawk, x)) / 256;   /* 256 is unix-dep */
		break;
	case FRAND:
		/* in principle, rand() returns something in 0..RAND_MAX */
		u = (awknum_t) (rand() % RAND_MAX) / RAND_MAX;
		break;
	case FSRAND:
		if (ISREC(x))	/* no argument provided */
			u = time((time_t *)0);
		else
			u = eawk_getfval(eawk, x);
		srand((unsigned int) u);
		break;
	case FTOUPPER:
	case FTOLOWER:
		if (t == FTOUPPER)
			buf = nawk_toupper(eawk, eawk_getsval(eawk, x));
		else
			buf = nawk_tolower(eawk, eawk_getsval(eawk, x));
		TEMPFREE(eawk, x);
		x = eawk_gettemp(eawk);
		eawk_setsval(eawk, x, buf);
		free(buf);
		return x;
	case FFLUSH:
		if (ISREC(x) || strlen(eawk_getsval(eawk, x)) == 0) {
			flush_all(eawk);	/* fflush() or fflush("") -> all */
			u = 0;
		} else if ((fp = openfile(eawk, FFLUSH, eawk_getsval(eawk, x))) == NULL)
			u = EOF;
		else
			u = fflush(fp);
		break;
	case FSYSTIME:
		u = time(NULL);
		break;
	case FSTRFTIME:
		/* strftime([format [,timestamp]]) */
		if (nextarg) {
			y = eawk_execute(eawk, nextarg), nextarg = nextarg->nnext;
			tv = (time_t) eawk_getfval(eawk, y);
			TEMPFREE(eawk, y);
		} else
			tv = time(NULL);
		tm = localtime(&tv);

		if (ISREC(x)) {
			/* format argument not provided, use default */
			fmt = eawk_tostring(eawk, "%a %b %d %H:%M:%S %Z %Y");
		} else
			fmt = eawk_tostring(eawk, eawk_getsval(eawk, x));

		sz = 32, buf = NULL;
		do {
			if ((buf = realloc(buf, (sz *= 2))) == NULL) {
				eawk_warning(eawk, "out of memory in strftime");
				return NULL;
			}
		} while(strftime(buf, sz, fmt, tm) == 0);

		y = eawk_gettemp(eawk);
		eawk_setsval(eawk, y, buf);
		free(fmt);
		free(buf);

		return y;
	default:	/* can't happen */
		eawk_warning(eawk, "illegal function type %d", t);
		return NULL;
	}
	TEMPFREE(eawk, x);
	x = eawk_gettemp(eawk);
	eawk_setfval(eawk, x, u);
	if (nextarg != 0) {
		eawk_warning(eawk, "warning: function has too many arguments");
		for ( ; nextarg; nextarg = nextarg->nnext)
			eawk_execute(eawk, nextarg);
	}
	return(x);
}

/* print a[0] */
awkcell_t *
eawk_print(eawk_t *eawk, awknode_t **a, int n)
{
	awknode_t *x;
	awkcell_t *y;
	FILE *fp;

	USE_ARG(n);
	if (a[1] == 0)	/* a[1] is redirection operator, a[2] is file */
		fp = stdout;
	else
		fp = eawk_redirect(eawk, eawk_ptoi(a[1]), a[2]);
	for (x = a[0]; x != NULL; x = x->nnext) {
		y = eawk_execute(eawk, x);
		fputs(eawk_getpssval(eawk, y), fp);
		TEMPFREE(eawk, y);
		if (x->nnext == NULL)
			fputs(*eawk->ORS, fp);
		else
			fputs(*eawk->OFS, fp);
	}
	if (a[1] != 0)
		fflush(fp);
	if (ferror(fp)) {
		eawk_warning(eawk, "write error on %s", filename(eawk, fp));
		return NULL;
	}
	return(True);
}

awkcell_t *
eawk_nullproc(eawk_t *eawk, awknode_t **a, int n)
{
	eawk = eawk;
	n = n;
	a = a;
	return 0;
}


/* set up all i/o redirections */
FILE *
eawk_redirect(eawk_t *eawk, int a, awknode_t *b)
{
	FILE *fp;
	awkcell_t *x;
	char *fname;

	x = eawk_execute(eawk, b);
	fname = eawk_getsval(eawk, x);
	fp = openfile(eawk, a, fname);
	if (fp == NULL) {
		eawk_warning(eawk, "can't open file %s", fname);
		return NULL;
	}
	TEMPFREE(eawk, x);
	return fp;
}

awkcell_t *
eawk_closefile(eawk_t *eawk, awknode_t **a, int n)
{
	awkcell_t *x;
	size_t i;
	int stat;

	n = n;
	x = eawk_execute(eawk, a[0]);
	eawk_getsval(eawk, x);
	stat = -1;
	for (i = 0; i < eawk->nfiles; i++) {
		if (eawk->files[i].fname && strcmp(x->str, eawk->files[i].fname) == 0) {
			if (ferror(eawk->files[i].fp))
				eawk_warning(eawk,  "i/o error occurred on %s", eawk->files[i].fname );
			if (eawk->files[i].mode == '|' || eawk->files[i].mode == LE)
				stat = pclose(eawk->files[i].fp);
			else
				stat = fclose(eawk->files[i].fp);
			if (stat == EOF)
				eawk_warning(eawk,  "i/o error occurred closing %s", eawk->files[i].fname );
			if (i > 2)	/* don't do /dev/std... */
				XFREE(eawk->files[i].fname);
			eawk->files[i].fname = NULL;	/* watch out for ref thru this */
			eawk->files[i].fp = NULL;
		}
	}
	TEMPFREE(eawk, x);
	x = eawk_gettemp(eawk);
	eawk_setfval(eawk, x, (awknum_t) stat);
	return(x);
}

/* substitute command */
awkcell_t *
eawk_sub(eawk_t *eawk, awknode_t **a, int nnn)
{
	const uint8_t *sptr;
	uint8_t *q;
	awkcell_t *x, *y, *result;
	uint8_t *t, *buf, *pb;
	fa_t *pfa;
	int bufsz = eawk->recsize;

	USE_ARG(nnn);
	if ((buf = malloc(bufsz)) == NULL) {
		eawk_warning(eawk, "out of memory in eawk_sub");
		return NULL;
	}
	x = eawk_execute(eawk, a[3]);	/* target string */
	t = (uint8_t *)eawk_getsval(eawk, x);
	if (a[0] == 0)		/* 0 => a[1] is already-compiled regexpr */
		pfa = (fa_t *)(void *)a[1];	/* regular expression */
	else {
		y = eawk_execute(eawk, a[1]);
		pfa = eawk_regcomp(eawk, eawk_getsval(eawk, y), 1);
		TEMPFREE(eawk, y);
	}
	y = eawk_execute(eawk, a[2]);	/* replacement string */
	result = False;
	if (eawk_pmatch(eawk, pfa, (char *)t)) {
		sptr = t;
		eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+eawk->patbeg-sptr), eawk->recsize, 0, "sub");
		pb = buf;
		while (sptr < eawk->patbeg)
			*pb++ = *sptr++;
		sptr = (uint8_t *)eawk_getsval(eawk, y);
		while (*sptr != 0) {
			eawk_adjbuf(eawk, &buf, &bufsz, (int)(5+pb-buf), eawk->recsize, &pb, "sub");
			if (*sptr == '\\') {
				backsub(&pb, &sptr);
			} else if (*sptr == '&') {
				sptr++;
				eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+eawk->patlen+pb-buf), eawk->recsize, &pb, "sub");
				for (q = eawk->patbeg; q < eawk->patbeg+eawk->patlen; )
					*pb++ = *q++;
			} else
				*pb++ = *sptr++;
		}
		*pb = '\0';
		if (pb > buf + bufsz) {
			eawk_warning(eawk, "sub result1 %.30s too big; can't happen", buf);
			return NULL;
		}
		sptr = eawk->patbeg + eawk->patlen;
		if ((eawk->patlen == 0 && *eawk->patbeg) || (eawk->patlen && *(sptr-1))) {
			eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+strlen((const char *)sptr)+pb-buf), 0, &pb, "sub");
			while ((*pb++ = *sptr++) != 0) {
			}
		}
		if (pb > buf + bufsz) {
			eawk_warning(eawk, "sub result2 %.30s too big; can't happen", buf);
			return NULL;
		}
		eawk_setsval(eawk, x, (char *)buf);	/* BUG: should be able to avoid copy */
		result = True;;
	}
	TEMPFREE(eawk, x);
	TEMPFREE(eawk, y);
	free(buf);
	return result;
}

/* global substitute */
awkcell_t *
eawk_gsub(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t *x, *y;
	const char *rptr;
	const uint8_t *sptr;
	uint8_t *t, *q;
	uint8_t *pb, *buf;
	fa_t *pfa;
	int mflag, tempstat, num;
	int bufsz = eawk->recsize;

	USE_ARG(nnn);
	if ((buf = calloc(1, bufsz)) == NULL) {
		eawk_warning(eawk, "out of memory in eawk_gsub");
		return NULL;
	}
	mflag = 0;	/* if mflag == 0, can replace empty string */
	num = 0;
	x = eawk_execute(eawk, a[3]);	/* target string */
	t = (uint8_t *)eawk_getsval(eawk, x);
	if (a[0] == 0)		/* 0 => a[1] is already-compiled regexpr */
		pfa = (fa_t *)(void *)a[1];	/* regular expression */
	else {
		y = eawk_execute(eawk, a[1]);
		pfa = eawk_regcomp(eawk, eawk_getsval(eawk, y), 1);
		TEMPFREE(eawk, y);
	}
	y = eawk_execute(eawk, a[2]);	/* replacement string */
	if (eawk_pmatch(eawk, pfa, (char *)t)) {
		tempstat = pfa->initstat;
		pfa->initstat = 2;
		pb = buf;
		rptr = eawk_getsval(eawk, y);
		do {
			if (eawk->patlen == 0 && *eawk->patbeg != 0) {	/* matched empty string */
				if (mflag == 0) {	/* can replace empty */
					num++;
					sptr = (const uint8_t *)rptr;
					while (*sptr != 0) {
						eawk_adjbuf(eawk, &buf, &bufsz, (int)(5+pb-buf), eawk->recsize, &pb, "gsub");
						if (*sptr == '\\') {
							backsub(&pb, &sptr);
						} else if (*sptr == '&') {
							sptr++;
							eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+eawk->patlen+pb-buf), eawk->recsize, &pb, "gsub");
							for (q = eawk->patbeg; q < eawk->patbeg+eawk->patlen; )
								*pb++ = *q++;
						} else
							*pb++ = *sptr++;
					}
				}
				if (*t == 0)	/* at end */
					goto done;
				eawk_adjbuf(eawk, &buf, &bufsz, (int)(2+pb-buf), eawk->recsize, &pb, "gsub");
				*pb++ = *t++;
				if (pb > buf + bufsz) {
					/* BUG: not sure of this test */
					eawk_warning(eawk, "gsub result0 %.30s too big; can't happen", buf);
					return NULL;
				}
				mflag = 0;
			}
			else {	/* matched nonempty string */
				num++;
				sptr = t;
				eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+(eawk->patbeg-sptr)+pb-buf), eawk->recsize, &pb, "gsub");
				while (sptr < eawk->patbeg)
					*pb++ = *sptr++;
				sptr = (const uint8_t *)rptr;
				while (*sptr != 0) {
					eawk_adjbuf(eawk, &buf, &bufsz, (int)(5+pb-buf), eawk->recsize, &pb, "gsub");
					if (*sptr == '\\') {
						backsub(&pb, &sptr);
					} else if (*sptr == '&') {
						sptr++;
						eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+eawk->patlen+pb-buf), eawk->recsize, &pb, "gsub");
						for (q = eawk->patbeg; q < eawk->patbeg+eawk->patlen; )
							*pb++ = *q++;
					} else
						*pb++ = *sptr++;
				}
				t = eawk->patbeg + eawk->patlen;
				if (eawk->patlen == 0 || *t == 0 || *(t-1) == 0)
					goto done;
				if (pb > buf + bufsz) {
					eawk_warning(eawk, "gsub result1 %.30s too big; can't happen", buf);
					return NULL;
				}
				mflag = 1;
			}
		} while (eawk_pmatch(eawk, pfa, (char *)t));
		sptr = t;
		eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+strlen((const char *)sptr)+pb-buf), 0, &pb, "gsub");
		while ((*pb++ = *sptr++) != 0) {
		}
done:
		if (pb > buf + bufsz) {
			eawk_warning(eawk, "gsub result2 %.30s too big; can't happen", buf);
			return NULL;
		}
		*pb = '\0';
		eawk_setsval(eawk, x, (char *)buf);	/* BUG: should be able to avoid copy + free */
		pfa->initstat = tempstat;
	}
	TEMPFREE(eawk, x);
	TEMPFREE(eawk, y);
	x = eawk_gettemp(eawk);
	x->type = NUM;
	x->num = num;
	free(buf);
	return(x);
}

/* global selective substitute */
/* XXX incomplete - doesn't support backreferences \0 ... \9 */
awkcell_t *
eawk_gensub(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t *x, *y, *res, *h;
	char *rptr;
	const uint8_t *sptr;
	uint8_t *q, *pb, *t, *buf;
	fa_t *pfa;
	int mflag, tempstat, num, whichm;
	int bufsz = eawk->recsize;

	USE_ARG(nnn);
	if ((buf = malloc(bufsz)) == NULL) {
		eawk_warning(eawk, "out of memory in eawk_gensub");
		return NULL;
	}
	mflag = 0;	/* if mflag == 0, can replace empty string */
	num = 0;
	x = eawk_execute(eawk, a[4]);	/* source string */
	t = (uint8_t *)eawk_getsval(eawk, x);
	res = eawk_copycell(eawk, x);	/* target string - initially copy of source */
	if (a[0] == 0)		/* 0 => a[1] is already-compiled regexpr */
		pfa = (fa_t *)(void *)a[1];	/* regular expression */
	else {
		y = eawk_execute(eawk, a[1]);
		pfa = eawk_regcomp(eawk, eawk_getsval(eawk, y), 1);
		TEMPFREE(eawk, y);
	}
	y = eawk_execute(eawk, a[2]);	/* replacement string */
	h = eawk_execute(eawk, a[3]);	/* which matches should be replaced */
	sptr = (uint8_t *)eawk_getsval(eawk, h);
	if (sptr[0] == 'g' || sptr[0] == 'G')
		whichm = -1;
	else {
		/*
		 * The specified number is index of replacement, starting
		 * from 1. GNU awk treats index lower than 0 same as
		 * 1, we do same for compatibility.
		 */
		whichm = (int) eawk_getfval(eawk, h) - 1;
		if (whichm < 0)
			whichm = 0;
	}
	TEMPFREE(eawk, h);

	if (eawk_pmatch(eawk, pfa, (char *)t)) {
		char *sl;

		tempstat = pfa->initstat;
		pfa->initstat = 2;
		pb = buf;
		rptr = eawk_getsval(eawk, y);
		/*
		 * XXX if there are any backreferences in subst string,
		 * complain now.
		 */
		for(sl=rptr; (sl = strchr(sl, '\\')) && sl[1]; sl++) {
			if (strchr("0123456789", sl[1])) {
				eawk_warning(eawk, "gensub doesn't support backreferences (subst \"%s\")", rptr);
				return NULL;
			}
		}
		
		do {
			if (whichm >= 0 && whichm != num) {
				num++;
				eawk_adjbuf(eawk, &buf, &bufsz, (int)((pb - buf) + (eawk->patbeg - t) + eawk->patlen), eawk->recsize, &pb, "gensub");

				/* copy the part of string up to and including
				 * match to output buffer */
				while (t < eawk->patbeg + eawk->patlen)
					*pb++ = *t++;
				continue;
			}

			if (eawk->patlen == 0 && *eawk->patbeg != 0) {	/* matched empty string */
				if (mflag == 0) {	/* can replace empty */
					num++;
					sptr = (uint8_t *)rptr;
					while (*sptr != 0) {
						eawk_adjbuf(eawk, &buf, &bufsz, (int)(5+pb-buf), eawk->recsize, &pb, "gensub");
						if (*sptr == '\\') {
							backsub(&pb, &sptr);
						} else if (*sptr == '&') {
							sptr++;
							eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+eawk->patlen+pb-buf), eawk->recsize, &pb, "gensub");
							for (q = eawk->patbeg; q < eawk->patbeg+eawk->patlen; )
								*pb++ = *q++;
						} else
							*pb++ = *sptr++;
					}
				}
				if (*t == 0)	/* at end */
					goto done;
				eawk_adjbuf(eawk, &buf, &bufsz, (int)(2+pb-buf), eawk->recsize, &pb, "gensub");
				*pb++ = *t++;
				if (pb > buf + bufsz) {
					/* BUG: not sure of this test */
					eawk_warning(eawk, "gensub result0 %.30s too big; can't happen", buf);
					return NULL;
				}
				mflag = 0;
			}
			else {	/* matched nonempty string */
				num++;
				sptr = t;
				eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+(eawk->patbeg-sptr)+pb-buf), eawk->recsize, &pb, "gensub");
				while (sptr < eawk->patbeg)
					*pb++ = *sptr++;
				sptr = (const uint8_t *)rptr;
				while (*sptr != 0) {
					eawk_adjbuf(eawk, &buf, &bufsz, (int)(5+pb-buf), eawk->recsize, &pb, "gensub");
					if (*sptr == '\\') {
						backsub(&pb, &sptr);
					} else if (*sptr == '&') {
						sptr++;
						eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+eawk->patlen+pb-buf), eawk->recsize, &pb, "gensub");
						for (q = eawk->patbeg; q < eawk->patbeg+eawk->patlen; )
							*pb++ = *q++;
					} else
						*pb++ = *sptr++;
				}
				t = eawk->patbeg + eawk->patlen;
				if (eawk->patlen == 0 || *t == 0 || *(t-1) == 0)
					goto done;
				if (pb > buf + bufsz) {
					eawk_warning(eawk, "gensub result1 %.30s too big; can't happen", buf);
					return NULL;
				}
				mflag = 1;
			}
		} while (eawk_pmatch(eawk, pfa, (char *)t));
		sptr = t;
		eawk_adjbuf(eawk, &buf, &bufsz, (int)(1+strlen((const char *)sptr)+pb-buf), 0, &pb, "gensub");
		while ((*pb++ = *sptr++) != 0)
			;
done:		if (pb > buf + bufsz) {
			eawk_warning(eawk, "gensub result2 %.30s too big; can't happen", buf);
			return NULL;
		}
		*pb = '\0';
		eawk_setsval(eawk, res, (char *)buf);
		pfa->initstat = tempstat;
	}
	TEMPFREE(eawk, x);
	TEMPFREE(eawk, y);
	free(buf);
	return(res);
}

/* set default values for everything */
int
eawk_setdefaults(eawk_t *eawk, const char *prog)
{
	eawk->control = eawk_makesymtab(eawk, NSYMTAB);

	eawk_var_set(eawk, "stage", "error display", 0.0, STR, eawk->control);

	eawk->safe = 0;		/* 1 => "safe" mode */
	eawk->recsize = RECSIZE;
	eawk->lineno = 1;
	eawk->nfields = MAXFLD;	/* last allocated slot for $i */

	eawk->ep = eawk->ebuf;
	eawk->yysptr = eawk->yysbuf;

	eawk->file = __UNCONST("");
	eawk->fieldssize = RECSIZE;
	eawk->argno = 1;	/* current input argument number */

	eawk->lexbufsize = 5; /* BUG: setting this small causes core dump! */
	eawk->stringbufsz = 500;
	eawk->regexprbufsz = 500;
	eawk->make_dfa_now = 1;
	eawk->cclenterbufsz = 100;
	eawk->relexbufsz = 100;

	eawk->static_inputFS[0] = ' ';
	eawk->static_inputFS[1] = 0x0;
	eawk->len_inputFS = sizeof(eawk->static_inputFS) - 1;
	eawk->inputFS = eawk->static_inputFS;

	eawk_ctlvar_set(eawk, "symtab", 0, eawk_makesymtab(eawk, NSYMTAB/NSYMTAB));

	setlocale(LC_CTYPE, "");
	setlocale(LC_NUMERIC, "C"); /* for parsing cmdline & prog */
	eawk_ctlvar_set(eawk, "cmdname", 0, prog);

	return 1;
}

/*
 * Call a C function that returns nothing
 */
static void 
voidcall(void *f, long *a, double *d)
{
	void            (*func) (
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			double, double, double, double, double, double, double, double, double, double,
			double, double, double, double, double, double, double, double, double, double
		) = f;

	(*func) (
		 a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9],
	a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19],
	a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28], a[29],
	a[30], a[31], a[32], a[33], a[34], a[35], a[36], a[37], a[38], a[39],
		 d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], d[8], d[9],
	d[10], d[11], d[12], d[13], d[14], d[15], d[16], d[17], d[18], d[19]
		);
}

/*
 * Call a C function that returns an int
 */
static int 
intcall(void *f, long *a, double *d)
{
	int             (*func) (
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			double, double, double, double, double, double, double, double, double, double,
			double, double, double, double, double, double, double, double, double, double
		) = f;
	int             res = 0;

	res = (*func) (
		 a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9],
	a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19],
	a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28], a[29],
	a[30], a[31], a[32], a[33], a[34], a[35], a[36], a[37], a[38], a[39],
		 d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], d[8], d[9],
	d[10], d[11], d[12], d[13], d[14], d[15], d[16], d[17], d[18], d[19]
		);
	return res;
}

/*
 * Call a C function that returns an int64
 */
static int64_t 
int64call(void *f, long *a, double *d)
{
	int64_t             (*func) (
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			double, double, double, double, double, double, double, double, double, double,
			double, double, double, double, double, double, double, double, double, double
		) = f;
	int64_t             res = 0;

	res = (*func) (
		 a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9],
	a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19],
	a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28], a[29],
	a[30], a[31], a[32], a[33], a[34], a[35], a[36], a[37], a[38], a[39],
		 d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], d[8], d[9],
	d[10], d[11], d[12], d[13], d[14], d[15], d[16], d[17], d[18], d[19]
		);
	return res;
}

/*
 * Call a C function that returns a pointer
 */
static void    *
ptrcall(void *f, long *a, double *d)
{
	void           *(*func) (
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			double, double, double, double, double, double, double, double, double, double,
			double, double, double, double, double, double, double, double, double, double
		) = f;
	void           *res = NULL;

	res = (*func) (
		 a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9],
	a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19],
	a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28], a[29],
	a[30], a[31], a[32], a[33], a[34], a[35], a[36], a[37], a[38], a[39],
		 d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], d[8], d[9],
	d[10], d[11], d[12], d[13], d[14], d[15], d[16], d[17], d[18], d[19]
		);
	return res;
}

/*
 * Call a C function that returns a double
 */
static double 
floatcall(void *f, long *a, double *d)
{
	double          (*func) (
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			long, long, long, long, long, long, long, long, long, long,
			double, double, double, double, double, double, double, double, double, double,
			double, double, double, double, double, double, double, double, double, double
		) = f;
	double          res = 0.0;

	res = (*func) (
		 a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9],
	a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19],
	a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28], a[29],
	a[30], a[31], a[32], a[33], a[34], a[35], a[36], a[37], a[38], a[39],
		 d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], d[8], d[9],
	d[10], d[11], d[12], d[13], d[14], d[15], d[16], d[17], d[18], d[19]
		);
	return res;
}

/* mapping table used to map awk strings to popular C values */
typedef struct argmap_t {
	const char	*name;	/* awk string */
	size_t		 len;	/* its length */
	void		*arg;	/* appropriate value */
} argmap_t;

static argmap_t	namemaps[] = {
	{	"stdin",	6,	stdin	},
	{	"stdout",	6,	stdout	},
	{	"stderr",	6,	stderr	},
	{	NULL,		0,	NULL	}
};

/* find a mapping from an awk string to a C value */
static void *
findmap(eawk_t *eawk, const char *name)
{ 
	argmap_t	*map;

	USE_ARG(eawk);
	for (map = namemaps ; map->name ; map++) {
		if (strncmp(name, map->name, map->len) == 0) {
			return map->arg;
		}
	}
	(void) fprintf(stderr, "map arg '%s' not found\n", name);
	return NULL;
}

/* save 'n' chars of 's' in malloc'd memory */
/* bound the allocation with 'size' chars */
static char *
strnsave(const char *s, int n, int maxstr)
{
	char	*cp;

	if (n < 0) {
		n = (int)strlen(s);
	}
	if (n > maxstr) {
		n = maxstr;
	}
	if ((cp = calloc(1, n + 1)) != NULL) {
		(void) memcpy(cp, s, n);
		cp[n] = 0x0;
	}
	return cp;
}

/* structure used to get a reference to a C variable, for call by ref */
typedef struct cref_t {
	awkcell_t	*cell;	/* cell pointer */
	long		 addr;	/* its address */
} cref_t;

/* set up the C arg stack using values that were passed to us, and prototype arg types */
static int
makeargs(eawk_t *eawk, awknode_t *a, awkarray_t *types, char *funcname, long *largs, double *dargs, cref_t *crefs)
{
	awkcell_t	 *typecell;
	awkcell_t	 *argcell;
	awknode_t	 *node;
	uint32_t	  lsize;
	uint32_t	  dsize;
	uint32_t	  lpos;
	uint32_t	  dpos;
	uint32_t	  cpos;
	char		**sput;
	char		 *argtype;
	char		 *argument;
	char		  buf[1024];
	int		  i;

	lsize = dsize = cpos = dpos = lpos = 0;
	for (i = 0, node = a ; node != NULL ; i++, node = node->nnext) {
		if (lsize > 40 * sizeof(*largs) || dsize > 20 * sizeof(*dargs)) {
			return 0;
		}
		(void) snprintf(buf, sizeof(buf), "%s:%d", funcname, i);
		typecell = eawk_lookup(buf, types);
		argtype = eawk_getsval(eawk, typecell);
		if (strcmp(argtype, "void") == 0) {
			lsize += sizeof(int);
			largs[lpos++] = 0;
		} else if (strcmp(argtype, "bool") == 0) {
			lsize += sizeof(int);
			argcell = eawk_execute(eawk, node);
			argument = eawk_getsval(eawk, argcell);
			largs[lpos++] = (*argument != '0');
		} else if (strcmp(argtype, "int") == 0 ||
			   (strcmp(argtype, "long") == 0 && /*CONSTCOND*/sizeof(long) == 4)) {
			lsize += sizeof(int);
			argcell = eawk_execute(eawk, node);
			largs[lpos++] = (long) eawk_getfval(eawk, argcell);
		} else if (strcmp(argtype, "int64") == 0 ||
			   (strcmp(argtype, "long") == 0 && /*CONSTCOND*/sizeof(long) == 8)) {
			lsize += sizeof(int64_t);
			argcell = eawk_execute(eawk, node);
			largs[lpos++] = (long)eawk_getfval(eawk, argcell);
		} else if (strcmp(argtype, "cptr") == 0) {
			lsize += sizeof(void *);
			argcell = eawk_execute(eawk, node);
			largs[lpos++] = (long)eawk_getfval(eawk, argcell);
		} else if (strcmp(argtype, "cvar") == 0) {
			lsize += sizeof(void *);
			argcell = eawk_execute(eawk, node);
			sput = (char **) (largs + lpos);
			*sput = findmap(eawk, argcell->name);
			lpos += 1;
		} else if (strcmp(argtype, "cref") == 0) {
			lsize += sizeof(void *);
			crefs[cpos].cell = argcell = eawk_execute(eawk, node);
			crefs[cpos].addr = (long) eawk_getfval(eawk, argcell);
			sput = (char **) (largs + lpos);
			*sput = (char *)(void *)&crefs[cpos].addr;
			lpos += 1;
		} else if (strcmp(argtype, "float") == 0 ||
		           strcmp(argtype, "double") == 0) {
			dsize += sizeof(double);
			argcell = eawk_execute(eawk, node);
			dargs[dpos++] = (double) eawk_getfval(eawk, argcell);
		} else if (strcmp(argtype, "awkptr") == 0) {
			lsize += sizeof(void *);
			argcell = eawk_execute(eawk, node);
			/* this gets freed in unmakeargs() */
			asprintf(&argument, "%s", eawk_getsval(eawk, argcell));
			sput = (char **) (largs + lpos);
			*sput = argument;
			lpos += 1;
		} else {
			return 0;
		}
	}
	return 1;
}

/* set up the C arg stack using values that were passed to us, and prototype arg types */
static int
unmakeargs(eawk_t *eawk, awknode_t *a, awkarray_t *types, char *funcname, long *largs, double *dargs, cref_t *crefs)
{
	awkcell_t	 *typecell;
	awknode_t	 *node;
	uint32_t	  lsize;
	uint32_t	  dsize;
	uint32_t	  lpos;
	uint32_t	  dpos;
	uint32_t	  cpos;
	char		**sput;
	char		 *argtype;
	char		  buf[1024];
	int		  i;

	lsize = dsize = cpos = dpos = lpos = 0;
	for (i = 0, node = a ; node != NULL ; i++, node = node->nnext) {
		if (lsize > 40 * sizeof(*largs) || dsize > 20 * sizeof(*dargs)) {
			return 0;
		}
		(void) snprintf(buf, sizeof(buf), "%s:%d", funcname, i);
		typecell = eawk_lookup(buf, types);
		argtype = eawk_getsval(eawk, typecell);
		if (strcmp(argtype, "void") == 0) {
			lsize += sizeof(int);
			lpos += 1;
		} else if (strcmp(argtype, "bool") == 0) {
			lsize += sizeof(int);
			lpos += 1;
		} else if (strcmp(argtype, "int") == 0 ||
			   (strcmp(argtype, "long") == 0 && /*CONSTCOND*/sizeof(long) == 4)) {
			lsize += sizeof(int);
			lpos += 1;
		} else if (strcmp(argtype, "int64") == 0 ||
			   (strcmp(argtype, "long") == 0 && /*CONSTCOND*/sizeof(long) == 8)) {
			lsize += sizeof(int64_t);
			lpos += 1;
		} else if (strcmp(argtype, "cptr") == 0) {
			lsize += sizeof(void *);
			lpos += 1;
		} else if (strcmp(argtype, "cvar") == 0) {
			lsize += sizeof(void *);
			lpos += 1;
		} else if (strcmp(argtype, "cref") == 0) {
			lsize += sizeof(void *);
			lpos += 1;
			eawk_setfval(eawk, crefs[cpos].cell, (awknum_t)crefs[cpos].addr);
			cpos += 1;
		} else if (strcmp(argtype, "float") == 0 ||
		           strcmp(argtype, "double") == 0) {
			dsize += sizeof(double);
			dpos += 1;
		} else if (strcmp(argtype, "awkptr") == 0) {
			lsize += sizeof(void *);
			sput = (char **) (largs + lpos);
			(void) free(*sput);
			lpos += 1;
		} else {
			return 0;
		}
	}
	return 1;
}

/* parse a C prototype */
static int
parse_prototype(eawk_t *eawk, const char *proto, awkcell_t *arr, char *emerging_funcname, int esize)
{
	regmatch_t	 matchv[10];
	regex_t		 ret_regex;
	regex_t		 arg_regex;
	size_t		 protolen;
	char		 funcname[1024];
	char		 varname[1024];
	char		 value[1024];
	int		 cc;
	int		 i;

	/* XXX - agc make this use pfa_t */
	(void) regcomp(&ret_regex, "([^ \t]*)[ \t]*([^ \t(]*)[ \t]*\\(", REG_EXTENDED);
	regexec(&ret_regex, proto, 10, matchv, 0);
	/* use a temp funcname for convenience */
	(void) snprintf(funcname, sizeof(funcname), "%.*s",
		(int)(matchv[2].rm_eo - matchv[2].rm_so),
		&proto[(int)matchv[2].rm_so]);
	if (emerging_funcname != NULL) {
		(void) snprintf(emerging_funcname, esize, "%s", funcname);
	}
	/* func:return is return type */
	(void) snprintf(varname, sizeof(varname), "%s:return", funcname);
	(void) snprintf(value, sizeof(value), "%.*s", (int)(matchv[1].rm_eo - matchv[1].rm_so), proto);
	eawk_var_set(eawk, varname, value, 0.0, STR|NUM, (awkarray_t *)(void *)arr->str);
	regfree(&ret_regex);
	/* func:0, func:1 etc are the arg types */
	regcomp(&arg_regex, "^([ \t]*)([^ \t,)]*)[ \t]*([,)])", REG_EXTENDED);
	protolen = strlen(proto);
	for (cc = (int)matchv[0].rm_eo, i = 0 ; cc < (int)protolen ; i++, cc += (int)(matchv[3].rm_eo) + 1) {
		if (regexec(&arg_regex, &proto[cc], 10, matchv, 0) != 0) {
			break;
		}
		(void) snprintf(varname, sizeof(varname), "%s:%d", funcname, i);
		(void) snprintf(value, sizeof(value), "%.*s", (int)(matchv[2].rm_eo - matchv[2].rm_so), &proto[cc]);
		if (i == 0 && strcmp(value, "void") == 0) {
			break;
		}
		eawk_var_set(eawk, varname, value, 0.0, STR|NUM, (awkarray_t *)(void *)arr->str);
		if (proto[cc + (int)(matchv[3].rm_eo)] == ')') {
			break;
		}
	}
	/* func:argc is the arg count */
	(void) snprintf(value, sizeof(value), "%d", i);
	(void) snprintf(varname, sizeof(varname), "%s:argc", funcname);
	eawk_var_set(eawk, varname, value, 0.0, STR|NUM, (awkarray_t *)(void *)arr->str);
	return 1;
}

/* we use this for the numeric representation of the handle */
typedef union un_t {
	void		*vp;
	uint32_t	 i32;
	uint64_t	 i64;
} un_t;

#if !defined(SO_SUFFIX)
#  ifdef __APPLE__
#    define SO_SUFFIX	"dylib"
#  else
#    define SO_SUFFIX	"so"
#  endif
#endif

#ifndef MAXSTRSIZE
#define MAXSTRSIZE	(2048 * 1024)
#endif

#ifndef MAXFIELDS
#define MAXFIELDS	512
#endif

/* dlopen(a[0], a[1]); */
awkcell_t *
eawk_dlopen(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t	*pathcell;
	awkcell_t	*arr;
	awkcell_t	*res;
	char		 fullpath[1024];
	char		 handle[32];
	char		*path;
	un_t		 un;

	USE_ARG(nnn);
	pathcell = eawk_execute(eawk, a[0]);	/* path string */
	path = eawk_getsval(eawk, pathcell);
	arr = eawk_execute(eawk, a[1]);	/* array name */
	eawk_freesymtab(eawk, arr);
	arr->type &= ~STR;
	arr->type |= ARR;
	arr->str = (char *)(void *)eawk_makesymtab(eawk, NSYMTAB);
	/* fill in handle and path elements */
	if ((un.vp = dlopen(path, RTLD_LAZY)) == NULL) {
		(void) snprintf(fullpath, sizeof(fullpath), "%s.%s", path, SO_SUFFIX);
		path = fullpath;
	}
	if ((un.vp = dlopen(path, RTLD_LAZY)) == NULL) {
		(void) fprintf(stderr, "Unable to dlopen \"%s\"\n", path);
		res = eawk_gettemp(eawk);
		res->type = NUM;
		res->num = 0;
		return res;
	}
	if (/* CONSTCOND */sizeof(void *) == 8) {
		(void) snprintf(handle, sizeof(handle), "%" PRIu64, un.i64);
	} else {
		(void) snprintf(handle, sizeof(handle), "%d", un.i32);
	}
	eawk_var_set(eawk, "handle", handle, 0.0, STR|NUM, (awkarray_t *)(void *)arr->str);
	eawk_var_set(eawk, "path", path, 0.0, STR, (awkarray_t *)(void *)arr->str);
	TEMPFREE(eawk, arr);
	TEMPFREE(eawk, pathcell);
	res = eawk_gettemp(eawk);
	res->type = NUM;
	res->num = 2;
	return res;
}

/* dlproto(a[0], a[1]); */
awkcell_t *
eawk_dlproto(eawk_t *eawk, awknode_t **a, int nnn)
{
	regex_t		 ret_regex;
	regex_t		 arg_regex;
	awkcell_t	*arr;
	awkcell_t	*res;
	awkcell_t	*protocell;
	char		*proto;

	USE_ARG(nnn);
	arr = eawk_execute(eawk, a[0]);	/* array name */
	if (!(arr->type & ARR)) {
		res = eawk_gettemp(eawk);
		res->type = NUM;
		res->num = 0;
		return res;
	}
	/* XXX - agc make this use fa_t */
	(void) regcomp(&ret_regex, "([^ \t]*)[ \t]*([^ \t(]*)[ \t]*\\(", REG_EXTENDED);
	protocell = eawk_execute(eawk, a[1]);	/* C prototype */
	proto = eawk_getsval(eawk, protocell);
	parse_prototype(eawk, proto, arr, NULL, 0);
	TEMPFREE(eawk, arr);
	TEMPFREE(eawk, protocell);
	res = eawk_gettemp(eawk);
	res->type = NUM;
	res->num = 1;
	regfree(&arg_regex);
	return res;
}

/* call function */
awkcell_t *
eawk_dlcall(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t	*arr;
	awkcell_t	*handlecell;
	awkcell_t	*funccell;
	awkcell_t	*lretcell;
	awkcell_t	*res;
	void		(*vfunc)(void);
	int64_t		(*i64func)(void);
	int		(*ifunc)(void);
	double		(*dfunc)(void);
	void		*(*pfunc)(void);
	char		 efunc[1024];
	char		 buf[1024];
	char		*funcname;
	char		*handle;
	char		*rettype;
	long		 largs[40];
	double		 dargs[20];
	cref_t		 crefs[40];
	un_t		 vpret;
	un_t		 un;

	USE_ARG(nnn);
	arr = eawk_execute(eawk, a[0]); /* array */
	if ((handlecell = eawk_lookup("handle", (awkarray_t *)(void *)arr->str)) == NULL) {
		res = eawk_gettemp(eawk);
		res->type = NUM;
		res->num = -1;
		return res;
	}
	handle = eawk_getsval(eawk, handlecell);
	funccell = eawk_execute(eawk, a[1]);	/* function name string */
	funcname = eawk_getsval(eawk, funccell);
	if (strchr(funcname, '(') != NULL) {
		/* we have a function prototype */
		parse_prototype(eawk, funcname, arr, efunc, (int)sizeof(efunc));
		funcname = efunc;
	}
	if (/* CONSTCOND */sizeof(void *) == 8) {
		un.i64 = (uint64_t)strtoll(handle, NULL, 10);
	} else {
		un.i32 = atoi(handle);
	}
	res = eawk_gettemp(eawk);
	res->type = NUM;
	(void) snprintf(buf, sizeof(buf), "%s:return", funcname);
	lretcell = eawk_lookup(buf, (awkarray_t *)(void *)arr->str);
	rettype = eawk_getsval(eawk, lretcell);
	if (strcmp(rettype, "void") == 0) {
		if ((vfunc = dlsym(un.vp, funcname)) != NULL) {
			if (makeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs)) {
				voidcall(vfunc, largs, dargs);
				res->num = 0;
				unmakeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs);
			} else {
				(void) fprintf(stderr, "makeargs failed\n");
				res->num = -1;
			}
			return res;
		}
	} else if (strcmp(rettype, "awkptr") == 0 ||
		   strcmp(rettype, "cptr") == 0) {
		if ((pfunc = dlsym(un.vp, funcname)) != NULL) {
			if (makeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs)) {
				vpret.vp = ptrcall(pfunc, largs, dargs);
				if (/* CONSTCOND */sizeof(void *) == 8) {
					res->num = (awknum_t)vpret.i64;
				} else {
					res->num = vpret.i32;
				}
				unmakeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs);
			} else {
				(void) fprintf(stderr, "makeargs failed\n");
				res->num = -1;
			}
			return res;
		}
	} else if (strcmp(rettype, "int") == 0 ||
		   (strcmp(rettype, "long") == 0 && /*CONSTCOND*/sizeof(long) == 4)) {
		if ((ifunc = dlsym(un.vp, funcname)) != NULL) {
			if (makeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs)) {
				res->num = intcall(ifunc, largs, dargs);
				unmakeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs);
			} else {
				(void) fprintf(stderr, "makeargs failed\n");
				res->num = -1;
			}
			return res;
		}
	} else if (strcmp(rettype, "int64") == 0 ||
		   (strcmp(rettype, "long") == 0 && /*CONSTCOND*/sizeof(long) == 8)) {
		if ((i64func = dlsym(un.vp, funcname)) != NULL) {
			if (makeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs)) {
				res->num = int64call(i64func, largs, dargs);
				unmakeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs);
			} else {
				(void) fprintf(stderr, "makeargs failed\n");
				res->num = -1;
			}
			return res;
		}
	} else if (strcmp(rettype, "float") == 0) {
		if ((dfunc = dlsym(un.vp, funcname)) != NULL) {
			if (makeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs)) {
				res->num = floatcall(dfunc, largs, dargs);
				unmakeargs(eawk, a[1]->nnext, (awkarray_t *)(void *)arr->str, funcname, largs, dargs, crefs);
			} else {
				(void) fprintf(stderr, "makeargs failed\n");
				res->num = -1;
			}
			return res;
		}
	}
	(void) fprintf(stderr, "Symbol \"%s\" not found\n", funcname);
	res->num = -1;
	return res;
}

/* dlclose(lib) */
awkcell_t *
eawk_dlclose(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t	*handlecell;
	awkcell_t	*res;
	awkcell_t	*arr;
	char		*handle;
	un_t		 un;
	int		 ret;

	USE_ARG(nnn);
	arr = eawk_execute(eawk, a[0]); /* array */
	if ((handlecell = eawk_lookup("handle", (awkarray_t *)(void *)arr->str)) == NULL) {
		res = eawk_gettemp(eawk);
		res->type = NUM;
		res->num = -1;
		return res;
	}
	handle = eawk_getsval(eawk, handlecell);
	if (/* CONSTCOND */sizeof(void *) == 8) {
		un.i64 = (uint64_t)strtoll(handle, NULL, 10);
	} else {
		un.i32 = atoi(handle);
	}
	ret = dlclose(un.vp);
	res = eawk_gettemp(eawk);
	res->type = NUM;
	res->num = ret;
	return res;
}

/* dlalloc(size) */
awkcell_t *
eawk_dlalloc(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t	*sizecell;
	awkcell_t	*res;
	uint32_t	 size;
	un_t		 un;

	USE_ARG(nnn);
	sizecell = eawk_execute(eawk, a[0]); /* size */
	size = (uint32_t) eawk_getfval(eawk, sizecell);
	if ((un.vp = calloc(1, size)) == NULL) {
		(void) fprintf(stderr, "Unable to calloc \"%d\" bytes\n", size);
		res = eawk_gettemp(eawk);
		res->type = NUM;
		res->num = 0;
		return res;
	}
	TEMPFREE(eawk, sizecell);
	res = eawk_gettemp(eawk);
	res->type = NUM;
	if (/*CONSTCOND*/sizeof(void *) == 8) {
		res->num = (uint64_t) un.i64;
	} else {
		res->num = (uint32_t) un.i32;
	}
	return res;
}

/* dlfree(storage) */
awkcell_t *
eawk_dlfree(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t	*handlecell;
	awkcell_t	*res;
	un_t		 un;

	USE_ARG(nnn);
	handlecell = eawk_execute(eawk, a[0]); /* handle */
	if (/*CONSTCOND*/sizeof(void *) == 8) {
		un.i64 = (uint64_t) eawk_getfval(eawk, handlecell);
	} else {
		un.i32 = (uint32_t) eawk_getfval(eawk, handlecell);
	}
	(void) free(un.vp);
	TEMPFREE(eawk, handlecell);
	res = eawk_gettemp(eawk);
	res->type = NUM;
	res->num = 1;
	return res;
}

/* dlfield(storage, offset, type) */
awkcell_t *
eawk_dlfield(eawk_t *eawk, awknode_t **a, int nnn)
{
	awkcell_t	*handlecell;
	awkcell_t	*typecell;
	awkcell_t	*offcell;
	awkcell_t	*res;
	int64_t		 i64;
	int32_t		 i32;
	char		*storage;
	char		*comma;
	char		*type;
	un_t		 un;
	int		 size;
	int		 off;

	USE_ARG(nnn);
	handlecell = eawk_execute(eawk, a[0]); /* storage */
	un.i32 = (uint32_t) eawk_getfval(eawk, handlecell);
	storage = (char *) un.vp;
	offcell = eawk_execute(eawk, a[1]); /* offset */
	off = (uint32_t) eawk_getfval(eawk, offcell);
	typecell = eawk_execute(eawk, a[2]); /* type */
	type = eawk_getsval(eawk, typecell);
	res = eawk_gettemp(eawk);
	if (strncmp(type, "int32", 5) == 0 ||
	    (strncmp(type, "long", 4) == 0 && /*CONSTCOND*/sizeof(long) == 4)) {
		(void) memcpy(&i32, &storage[off], sizeof(i32));
		if ((comma = strchr(type, ',')) != NULL) {
			if (strcmp(comma + 1, "htonl") == 0) {
				i32 = htonl(i32);
			} else if (strcmp(comma + 1, "ntohl") == 0) {
				i32 = ntohl(i32);
			}
		}
		res->type = NUM;
		res->num = i32;
	} else if (strncmp(type, "cptr", 4) == 0 ||
		   strncmp(type, "awkptr", 6) == 0) {
		if (/*CONSTCOND*/sizeof(void *) == 8) {
			(void) memcpy(&i64, &storage[off], sizeof(i64));
			res->type = NUM;
			res->num = i64;
		} else {
			(void) memcpy(&i32, &storage[off], sizeof(i32));
			res->type = NUM;
			res->num = i32;
		}
	} else if (strncmp(type, "int16", 5) == 0) {
		i32 = 0;
		(void) memcpy(&i32, &storage[off], sizeof(i32));
		if ((comma = strchr(type, ',')) != NULL) {
			if (strcmp(comma + 1, "htons") == 0) {
				i32 = htons(i32 & 0xffff);
			} else if (strcmp(comma + 1, "ntohs") == 0) {
				i32 = ntohs(i32 & 0xffff);
			}
		}
		res->type = NUM;
		res->num = (i32 & 0xffff);
	} else if (strncmp(type, "int64", 5) == 0 ||
		   (strncmp(type, "long", 4) == 0 && /*CONSTCOND*/sizeof(long) == 8)) {
		(void) memcpy(&i64, &storage[off], 8);
		res->type = NUM;
		res->num = i64;
	} else if (strncmp(type, "int8", 4) == 0 ||
	           strncmp(type, "bool", 4) == 0) {
		res->type = NUM;
		res->num = storage[off];
	} else if (strncmp(type, "string", 6) == 0) {
		size = MAXSTRSIZE;
		if ((comma = strchr(type, ',')) != NULL) {
			size = (int)atol(comma + 1);
		}
		res->type = STR;
		res->str = strnsave(&storage[off], -1, MIN(MAXSTRSIZE, size));
	} else {
		res->type = NUM;
		res->num = -1;
	}
	TEMPFREE(eawk, handlecell);
	TEMPFREE(eawk, typecell);
	TEMPFREE(eawk, offcell);
	return res;
}
