/****************************************************************
Copyright (C) Lucent Technologies 1997
All Rights Reserved

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name Lucent Technologies or any of
its entities not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.

LUCENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
IN NO EVENT SHALL LUCENT OR ANY OF ITS ENTITIES BE LIABLE FOR ANY
SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
****************************************************************/

#define	DEBUG
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include "eawk.h"
#include "awkgram.h"
#include "proto.h"

#define	FULLTAB	2	/* rehash when table gets this x full */
#define	GROWTAB 4	/* grow table by this factor */

static void	rehash(awkarray_t *);

static awkcell_t	*nullloc;	/* a guaranteed empty cell */

/* initialize symbol table with builtin vars */
void
eawk_syminit(eawk_t *eawk)
{
	eawk->literal0 = eawk_var_set(eawk, "0", "0", 0.0, NUM|STR|CON|DONTFREE, eawk->symtab);
	/* this is used for if(x)... tests: */
	nullloc = eawk_var_set(eawk, "$zero&null", "", 0.0, NUM|STR|CON|DONTFREE, eawk->symtab);
	eawk->nullnode = eawk_celltonode(eawk, nullloc, CCON);

	eawk->fsloc = eawk_var_set(eawk, "FS", " ", 0.0, STR|DONTFREE, eawk->symtab);
	eawk->FS = &eawk->fsloc->str;
	eawk->RS = &eawk_var_set(eawk, "RS", "\n", 0.0, STR|DONTFREE, eawk->symtab)->str;
	eawk->OFS = &eawk_var_set(eawk, "OFS", " ", 0.0, STR|DONTFREE, eawk->symtab)->str;
	eawk->ORS = &eawk_var_set(eawk, "ORS", "\n", 0.0, STR|DONTFREE, eawk->symtab)->str;
	eawk->OFMT = &eawk_var_set(eawk, "OFMT", "%.6g", 0.0, STR|DONTFREE, eawk->symtab)->str;
	eawk->CONVFMT = &eawk_var_set(eawk, "CONVFMT", "%.6g", 0.0, STR|DONTFREE, eawk->symtab)->str;
	eawk->FILENAME = &eawk_var_set(eawk, "FILENAME", "", 0.0, STR|DONTFREE, eawk->symtab)->str;
	eawk->nfloc = eawk_var_set(eawk, "NF", "", 0.0, NUM, eawk->symtab);
	eawk->NF = &eawk->nfloc->num;
	eawk->nrloc = eawk_var_set(eawk, "NR", "", 0.0, NUM, eawk->symtab);
	eawk->NR = &eawk->nrloc->num;
	eawk->fnrloc = eawk_var_set(eawk, "FNR", "", 0.0, NUM, eawk->symtab);
	eawk->FNR = &eawk->fnrloc->num;
	eawk->SUBSEP = &eawk_var_set(eawk, "SUBSEP", "\034", 0.0, STR|DONTFREE, eawk->symtab)->str;
	eawk->rstartloc = eawk_var_set(eawk, "RSTART", "", 0.0, NUM, eawk->symtab);
	eawk->RSTART = &eawk->rstartloc->num;
	eawk->rlengthloc = eawk_var_set(eawk, "RLENGTH", "", 0.0, NUM, eawk->symtab);
	eawk->RLENGTH = &eawk->rlengthloc->num;
	eawk->symtabloc = eawk_var_set(eawk, "SYMTAB", "", 0.0, ARR, eawk->symtab);
	eawk->symtabloc->str = (char *)(void *)eawk->symtab;
}

/* set up ARGV and ARGC */
void
eawk_arginit(eawk_t *eawk, int argc, char **argv)
{
	awkcell_t *cp;
	int i;
	char temp[50];

	eawk->ARGC = &eawk_var_set(eawk, "ARGC", "", (awknum_t) argc, NUM, eawk->symtab)->num;
	cp = eawk_var_set(eawk, "ARGV", "", 0.0, ARR, eawk->symtab);
	eawk->ARGVtab = eawk_makesymtab(eawk, NSYMTAB);	/* could be (int) ARGC as well */
	cp->str = (char *)(void *)eawk->ARGVtab;
	for (i = 0; i < argc; i++) {
		(void) snprintf(temp, sizeof(temp), "%d", i);
		if (eawk_is_number(*argv)) {
			eawk_var_set(eawk, temp, *argv, atof(*argv), STR|NUM, eawk->ARGVtab);
		} else {
			eawk_var_set(eawk, temp, *argv, 0.0, STR, eawk->ARGVtab);
		}
		argv++;
	}
}

extern	char	**environ;

/* set up ENVIRON variable */
void
eawk_envinit(eawk_t *eawk, char **envp)
{
	awkcell_t 	 *cp;
	char 		 *p;

	if (!eawk->safe) {
		cp = eawk_var_set(eawk, "ENVIRON", "", 0.0, ARR, eawk->symtab);
		eawk->ENVtab = eawk_makesymtab(eawk, NSYMTAB);
		cp->str = (char *)(void *)eawk->ENVtab;
		if (envp == NULL) {
			envp = environ;
		}
		for ( ; *envp; envp++) {
			if ((p = strchr(*envp, '=')) == NULL)
				continue;
			if (p == *envp) /* no left hand side name in env string */
				continue;
			*p++ = 0;	/* split into two strings at = */
			if (eawk_is_number(p))
				eawk_var_set(eawk, *envp, p, atof(p), STR|NUM, eawk->ENVtab);
			else
				eawk_var_set(eawk, *envp, p, 0.0, STR, eawk->ENVtab);
			p[-1] = '=';	/* restore in case env is passed down to a shell */
		}
	}
}

/* set a variable in the awk struct */
int
eawk_ctlvar_set(eawk_t *eawk, const char *varname, int ivalue, const void *ptr)
{
	char	**newarray;
	char	 *newptr;
	int	  newsize;

	if (strcmp(varname, "FS") == 0) {
		asprintf(&newptr, "%s", (const char *)ptr);
		if (*eawk->FS) {
			(void) free(*eawk->FS);
		}
		*eawk->FS = newptr;
	} else if (strcmp(varname, "stage") == 0) {
		eawk_var_set(eawk, "stage", (const char *)ptr, 0.0, STR, eawk->control);
	} else if (strcmp(varname, "safe") == 0) {
		eawk->safe = ivalue;
	} else if (strcmp(varname, "cmdname") == 0) {
		eawk_var_set(eawk, "cmdname", (const char *)ptr, 0.0, STR, eawk->control);
	} else if (strcmp(varname, "lexprog") == 0) {
		asprintf(&newptr, "%s", (const char *)ptr);
		if (eawk->lexprog) {
			(void) free(eawk->lexprog);
		}
		eawk->lexprog = newptr;
	} else if (strcmp(varname, "script") == 0) {
		asprintf(&newptr, "%s", (const char *)ptr);
		if (eawk->scriptsize == 0) {
			newarray = calloc(20, sizeof(char *));
			if (newarray == NULL) {
				(void) fprintf(stderr, "can't calloc space for scripts array\n");
				return 0;
			}
			eawk->scripts = newarray;
			eawk->scriptsize = 20;
		} else if (eawk->scriptc == eawk->scriptsize) {
			newsize = eawk->scriptsize * 2;
			newarray = realloc(eawk->scripts, newsize * sizeof(char *));
			if (newarray == NULL) {
				(void) fprintf(stderr, "can't realloc space for scripts array\n");
				return 0;
			}
			eawk->scripts = newarray;
			eawk->scriptsize = newsize;
		}
		eawk->scripts[eawk->scriptc++] = newptr;
	} else if (strcmp(varname, "debug") == 0) {
		eawk->dbg = ivalue;
	} else if (strcmp(varname, "symtab") == 0) {
		eawk->symtab = __UNCONST(ptr);
	}
	return 1;
}

/* get the value as an integer */
int
eawk_ctlvar_get_int(eawk_t *eawk, const char *varname)
{
	if (strcmp(varname, "debug") == 0) {
		return eawk->dbg;
	}
	if (strcmp(varname, "current record size") == 0) {
		return eawk->recsize;
	}
	if (strcmp(varname, "error flag") == 0) {
		return eawk->errorflag;
	}
	return 0;
}

/* get the value as a pointer */
void *
eawk_ctlvar_get_ptr(eawk_t *eawk, const char *varname)
{
	if (strcmp(varname, "cmdname") == 0) {
		return eawk_getsval(eawk, eawk_lookup("cmdname", eawk->control));
	}
	if (strcmp(varname, "winner") == 0) {
		return eawk->winner;
	}
	if (strcmp(varname, "stage") == 0) {
		return eawk_getsval(eawk, eawk_lookup("stage", eawk->control));
	}
	return NULL;
}

/* make a new symbol table */
awkarray_t *
eawk_makesymtab(eawk_t *eawk, int n)
{
	awkarray_t *ap;
	awkcell_t **arr;

	ap = malloc(sizeof(awkarray_t));
	arr = calloc(n, sizeof(awkcell_t *));
	if (ap == NULL || arr == NULL) {
		eawk_warning(eawk, "out of space in eawk_makesymtab");
		return NULL;
	}
	ap->nelem = 0;
	ap->size = n;
	ap->tab = arr;
	return ap;
}

/* free a symbol table */
void
eawk_freesymtab(eawk_t *eawk, awkcell_t *ap)
{
	awkcell_t *cp, *temp;
	awkarray_t *arr;
	int i;

	if (!ISARR(ap))
		return;
	arr = (awkarray_t *)(void *)ap->str;
	if (arr == NULL)
		return;
	for (i = 0; i < arr->size; i++) {
		for (cp = arr->tab[i]; cp != NULL; cp = temp) {
			XFREE(cp->name);
			if (FREEABLE(cp))
				XFREE(cp->str);
			temp = cp->cnext;	/* avoids freeing then using */
			free(cp); 
			arr->nelem--;
		}
		arr->tab[i] = 0;
	}
	if (arr->nelem != 0)
		eawk_warning(eawk, "can't happen: inconsistent element count freeing %s", ap->name);
	free(arr->tab);
	free(arr);
}

/* free elem s from ap (i.e., ap["s"] */
void
eawk_freeelem(awkcell_t *ap, const char *s)
{
	awkarray_t *arr;
	awkcell_t *p, *prev = NULL;
	int h;
	
	arr = (awkarray_t *)(void *)ap->str;
	h = eawk_hash(s, arr->size);
	for (p = arr->tab[h]; p != NULL; prev = p, p = p->cnext)
		if (strcmp(s, p->name) == 0) {
			if (prev == NULL)	/* 1st one */
				arr->tab[h] = p->cnext;
			else			/* middle somewhere */
				prev->cnext = p->cnext;
			if (FREEABLE(p))
				XFREE(p->str);
			free(p->name);
			free(p);
			arr->nelem--;
			return;
		}
}

/* set a variable */
awkcell_t *
eawk_var_set(eawk_t *eawk, const char *name, const char *s, awknum_t f, unsigned t, awkarray_t *arr)
{
	int h;
	awkcell_t *p;

	if (name == NULL)
		name = "";

	if ((p = eawk_lookup(name, arr)) != NULL) {
		DPRINTF(eawk, ("eawk_var_set found %p: name=%s s=\"%s\" f=%g t=%o\n",
			p, NN(p->name), NN(p->str), p->num, p->type) );
		return p;
	}
	p = calloc(1, sizeof(*p));
	if (p == NULL) {
		eawk_warning(eawk, "out of space for symbol table at %s", name);
		return NULL;
	}
	p->name = eawk_tostring(eawk, name);
	p->str = eawk_tostring(eawk, (s) ? s : "");
	p->num = f;
	p->type = t;
	p->csub = CUNK;
	p->ctype = OCELL;
	arr->nelem++;
	if (arr->nelem > FULLTAB * arr->size)
		rehash(arr);
	h = eawk_hash(name, arr->size);
	p->cnext = arr->tab[h];
	arr->tab[h] = p;
	DPRINTF(eawk, ("eawk_var_set set %p: name=%s s=\"%s\" f=%g t=%o\n",
		p, p->name, p->str, p->num, p->type) );
	return p;
}

/* form hash value for string s */
int
eawk_hash(const char *s, int n)
{
	unsigned hashval;

	for (hashval = 0; *s != '\0'; s++)
		hashval = (*s + 31 * hashval);
	return hashval % n;
}

/* rehash items in small table into big one */
static void
rehash(awkarray_t *arr)
{
	int i, nh, nsz;
	awkcell_t *cp, *op, **np;

	nsz = GROWTAB * arr->size;
	np = calloc(nsz, sizeof(*np));
	if (np == NULL)		/* can't do it, but can keep running. */
		return;		/* someone else will run out later. */
	for (i = 0; i < arr->size; i++) {
		for (cp = arr->tab[i]; cp; cp = op) {
			op = cp->cnext;
			nh = eawk_hash(cp->name, nsz);
			cp->cnext = np[nh];
			np[nh] = cp;
		}
	}
	free(arr->tab);
	arr->tab = np;
	arr->size = nsz;
}

/* look for s in arr */
awkcell_t *
eawk_lookup(const char *s, awkarray_t *arr)
{
	awkcell_t *p;
	int h;

	h = eawk_hash(s, arr->size);
	for (p = arr->tab[h]; p != NULL; p = p->cnext)
		if (strcmp(s, p->name) == 0)
			return(p);	/* found it */
	return(NULL);			/* not found */
}

/* set float val of a awkcell_t */
awknum_t
eawk_setfval(eawk_t *eawk, awkcell_t *cell, awknum_t f)
{
	int fldno;

	f += 0.0;		/* normalise negative zero to positive zero */
	if ((cell->type & (NUM | STR)) == 0) 
		eawk_funnyvar(eawk, cell, "assign to");
	if (ISFLD(cell)) {
		eawk->donerec = 0;	/* mark $0 invalid */
		fldno = atoi(cell->name);
		if (fldno > *eawk->NF)
			eawk_newfld(eawk, fldno);
		DPRINTF(eawk, ("setting field %d to %g\n", fldno, f) );
	} else if (ISREC(cell)) {
		eawk->donefld = 0;	/* mark $1... invalid */
		eawk->donerec = 1;
	}
	if (FREEABLE(cell))
		XFREE(cell->str); /* free any previous string */
	cell->type &= ~STR;	/* mark string invalid */
	cell->type |= NUM;	/* mark number ok */
	DPRINTF(eawk, ("eawk_setfval %p: %s = %g, t=%o\n", cell, NN(cell->name), f, cell->type) );
	return cell->num = f;
}

int
eawk_funnyvar(eawk_t *eawk, awkcell_t *cell, const char *rw)
{
	if (ISARR(cell)) {
		eawk_warning(eawk, "can't %s %s; it's an array name.", rw, cell->name);
		return 1;
	}
	if (cell->type & FCN) {
		eawk_warning(eawk, "can't %s %s; it's a function.", rw, cell->name);
		return 1;
	}
	eawk_warning(eawk, "funny variable %p: n=%s s=\"%s\" f=%g t=%o",
		cell, cell->name, cell->str, cell->num, cell->type);
	return 0;
}

char *
eawk_setsval(eawk_t *eawk, awkcell_t *cell, const char *s)	/* set string val of a awkcell_t */
{
	char *t;
	int fldno;

	DPRINTF(eawk, ("starting eawk_setsval %p: %s = \"%s\", t=%o, r,f=%d,%d\n", 
		cell, NN(cell->name), s, cell->type, eawk->donerec, eawk->donefld) );
	if ((cell->type & (NUM | STR)) == 0 && eawk_funnyvar(eawk, cell, "assign to")) {
		return NULL;
	}
	if (ISFLD(cell)) {
		eawk->donerec = 0;	/* mark $0 invalid */
		fldno = atoi(cell->name);
		if (fldno > *eawk->NF)
			eawk_newfld(eawk, fldno);
		DPRINTF(eawk, ("setting field %d to %s (%p)\n", fldno, s, s) );
	} else if (ISREC(cell)) {
		eawk->donefld = 0;	/* mark $1... invalid */
		eawk->donerec = 1;
	}
	t = eawk_tostring(eawk, s);	/* in case it's self-assign */
	cell->type &= ~NUM;
	cell->type |= STR;
	if (FREEABLE(cell))
		XFREE(cell->str);
	cell->type &= ~DONTFREE;
	DPRINTF(eawk, ("eawk_setsval %p: %s = \"%s (%p) \", t=%o r,f=%d,%d\n", 
		cell, NN(cell->name), t,t, cell->type, eawk->donerec, eawk->donefld) );
	return(cell->str = t);
}

awknum_t
eawk_getfval(eawk_t *eawk, awkcell_t *cell)	/* get float val of a awkcell_t */
{
	if ((cell->type & (NUM | STR)) == 0)
		eawk_funnyvar(eawk, cell, "read value of");
	if (ISFLD(cell) && eawk->donefld == 0)
		eawk_fldbld(eawk);
	else if (ISREC(cell) && eawk->donerec == 0)
		eawk_recbld(eawk);
	if (!ISNUM(cell)) {	/* not a number */
		cell->num = atof(cell->str);	/* best guess */
		if (eawk_is_number(cell->str) && !(cell->type&CON))
			cell->type |= NUM;	/* make NUM only sparingly */
	}
	DPRINTF(eawk, ("eawk_getfval %p: %s = %g, t=%o\n", cell, NN(cell->name), cell->num, cell->type) );
	return(cell->num);
}

/* get string val of a awkcell_t */
static char *
get_str_val(eawk_t *eawk, awkcell_t *cell, char **fmt)
{
	char s[100];
	double dtemp;

	if ((cell->type & (NUM | STR)) == 0)
		eawk_funnyvar(eawk, cell, "read value of");
	if (ISFLD(cell) && eawk->donefld == 0)
		eawk_fldbld(eawk);
	else if (ISREC(cell) && eawk->donerec == 0)
		eawk_recbld(eawk);
	if (ISSTR(cell) == 0) {
		if (FREEABLE(cell))
			XFREE(cell->str);
		if (modf(cell->num, &dtemp) == 0)	/* it's integral */
			snprintf(s, sizeof(s), "%.30g", cell->num);
		else
			snprintf(s, sizeof(s), *fmt, cell->num);
		cell->str = eawk_tostring(eawk, s);
		cell->type &= ~DONTFREE;
		cell->type |= STR;
	}
	DPRINTF(eawk, ("eawk_getsval %p: %s = \"%s (%p)\", t=%o\n", cell, NN(cell->name), cell->str, cell->str, cell->type) );
	return(cell->str);
}

/* get string val of a awkcell_t */
char *
eawk_getsval(eawk_t *eawk, awkcell_t *cell)
{
      return get_str_val(eawk, cell, eawk->CONVFMT);
}

/* get string val of a awkcell_t for print */
char *
eawk_getpssval(eawk_t *eawk, awkcell_t *cell)
{
      return get_str_val(eawk, cell, eawk->OFMT);
}


/* make a copy of string s */
char *
eawk_tostring(eawk_t *eawk, const char *s)
{
	char *p;

	asprintf(&p, "%s", s);
	if (p == NULL) {
		eawk_warning(eawk, "out of space in eawk_tostring on %s", s);
	}
	return p;
}

/* make a copy of string s */
char *
eawk_tostringN(eawk_t *eawk, const char *s, size_t n)
{
	char *p;

	p = malloc(n + 1);
	if (p == NULL) {
		eawk_warning(eawk, "out of space in eawk_tostring on %s", s);
	} else {
		(void) memcpy(p, s, n);
		p[n] = 0x0;
	}
	return(p);
}

/* collect string up to next delim */
char *
eawk_qstring(eawk_t *eawk, const char *is, int delim)
{
	const char *os = is;
	int c, n;
	const uint8_t *s = (const uint8_t *) is;
	uint8_t *buf, *bp;

	if ((buf = malloc(strlen(is)+3)) == NULL) {
		eawk_warning(eawk,  "out of space in eawk_qstring(%s)", s);
		return NULL;
	}
	for (bp = buf; (c = *s) != delim; s++) {
		if (c == '\n')
			eawk_syntax(eawk,  "newline in string %.20s...", os );
		else if (c != '\\')
			*bp++ = c;
		else {	/* \something */
			c = *++s;
			if (c == 0) {	/* \ at end */
				*bp++ = '\\';
				break;	/* for loop */
			}	
			switch (c) {
			case '\\':	*bp++ = '\\'; break;
			case 'n':	*bp++ = '\n'; break;
			case 't':	*bp++ = '\t'; break;
			case 'b':	*bp++ = '\b'; break;
			case 'f':	*bp++ = '\f'; break;
			case 'r':	*bp++ = '\r'; break;
			default:
				if (!isdigit((uint8_t)c)) {
					*bp++ = c;
					break;
				}
				n = c - '0';
				if (isdigit((uint8_t)s[1])) {
					n = 8 * n + *++s - '0';
					if (isdigit((uint8_t)s[1]))
						n = 8 * n + *++s - '0';
				}
				*bp++ = n;
				break;
			}
		}
	}
	*bp++ = 0;
	return (char *) buf;
}

int
eawk_setfs(eawk_t *awk, const char *fs)
{
	char	*s;

	if (fs) {
		eawk_ctlvar_set(awk, "FS", 0, s = eawk_qstring(awk, fs, '\0'));
		free(s);
		return 1;
	}
	return 0;
}

int
eawk_keyval(eawk_t *awk, const char *s)
{
	if (eawk_isclvar(s)) {
		eawk_setclvar(awk, __UNCONST(s));
		return 1;
	}
	return 0;
}

/* set up the scripts, args and environment for embedded awk */
int
eawk_setup(eawk_t *awk, int argc, char **argv)
{
	extern char	**environ;

	awk->dollar0.ctype = OCELL;
	awk->dollar0.csub = CFLD;
	awk->dollar0.type = (REC | STR | DONTFREE);

	awk->dollar1.ctype = OCELL;
	awk->dollar1.csub = CFLD;
	awk->dollar1.type = (FLD | STR | DONTFREE);

	/* argv[optind] is now the first argument */
	if (awk->scriptc == 0) {
		/* no -f; first argument is program */
		if (argc <= 1) {
			if (awk->dbg) {
				return 0;
			}
			(void) fprintf(stderr, "no awk program\n");
			return 0;
		}
		if (awk->dbg) {
			(void) fprintf(stderr, "program = |%s|\n", argv[1]);
		}
		eawk_ctlvar_set(awk, "lexprog", 0, argv[1]);
	}
	if (!eawk_recinit(awk, (unsigned)eawk_ctlvar_get_int(awk, "current record size"))) {
		return 0;
	}
	eawk_syminit(awk);
	eawk_ctlvar_set(awk, "stage", 0, "compiling");
	/* put prog name at front of arglist */
	argv[1] = eawk_ctlvar_get_ptr(awk, "cmdname");
	if (awk->dbg) {
		(void) fprintf(stderr, "argc=%d, argv[0]=%s\n", argc - 1, argv[1]);
	}
	/* copy and initialise arguments */
	eawk_arginit(awk, argc - 1, argv + 1);
	eawk_envinit(awk, environ);
	return 1;
}
