/*
 *  vv_db.h - format of ascii table file
 */

/*
 *  Records are rounded up in size to a divider of disk block size to
 *  ensure that a record write will never cross a block boundary, for
 *  all-or-nothing success for a write interrupted by a machine crash.
 *
 *  Note: spacing characters are blanks for readability..
 */

/*
 *  'vv' table record - padded to 512 bytes
 */

typedef struct vbuf {
	char	vb_version[INTLEN];	char	b00;
	char	vb_class[VV_CLASSLEN];	char	b01;
	char	vb_serial[VV_SERIALLEN];char	b02;
	char	vb_cluster[VV_CLUSTERLEN];char	b03;
	char	vb_id[VV_IDLEN];	char	b04;
	char	vb_status;		char	b05;
	char	vb_use;			char	b06;
	char	vb_data[HEXLEN];	char	b07;
	char	vb_maxdata[HEXLEN];	char	b08;
	char	vb_creat[DATELEN];	char	b09;
	char	vb_write[DATELEN];	char	b10;
	char	vb_nwrite[INTLEN];	char	b11;
	char	vb_finish[DATELEN];	char	b12;
	char	vb_save[DATELEN];	char	b13;
	char	vb_read[DATELEN];	char	b14;
	char	vb_nread[INTLEN];	char	b15;
	char	vb_freed[DATELEN];	char	b16;
	char	vb_restore[DATELEN];	char	b17;
	char	vb_nrestore[INTLEN];	char	b18;
	char	vb_mntfail[INTLEN];	char	b19;
	char	vb_delete[DATELEN];	char	b20;
	char	vb_nvsns;		char	b21;

	/*  these are the class 'slots' to save vv to; see next struct */

	char	vb_vsn0[VSNLEN];	char	b22;
	char	vb_nrec0[INTLEN];	char	b23;
	char	vb_nblk0[INTLEN];	char	b24;
	char	vb_vdate0[DATELEN];	char	b25;
	char	vb_vflags0[2];		char	b26;

	char	vb_vsn1[VSNLEN];	char	b27;
	char	vb_nrec1[INTLEN];	char	b28;
	char	vb_nblk1[INTLEN];	char	b29;
	char	vb_vdate1[DATELEN];	char	b30;
	char	vb_vflags1[2];		char	b31;

	char	vb_vsn2[VSNLEN];	char	b32;
	char	vb_nrec2[INTLEN];	char	b33;
	char	vb_nblk2[INTLEN];	char	b34;
	char	vb_vdate2[DATELEN];	char	b35;
	char	vb_vflags2[2];		char	b36;

	char	vb_vsn3[VSNLEN];	char	b37;
	char	vb_nrec3[INTLEN];	char	b38;
	char	vb_nblk3[INTLEN];	char	b39;
	char	vb_vdate3[DATELEN];	char	b40;
	char	vb_vflags3[2];		char	b41;

	char	vb_buf[16];		char	end;
} vbuf_t;

typedef struct {
	char	vb_vsn[VSNLEN];		char	b1;
	char	vb_nrec[INTLEN];	char	b2;
	char	vb_nblk[INTLEN];	char	b3;
	char	vb_vdate[DATELEN];	char	b4;
	char	vb_vflags[2];		char	b5;
} ClassSlot_t;

/*  flags for vb_vflagsX[0] */

#define VSN_CHECK	'c'

/*  flags for vb_vflagsX[1] */

#define VSN_OK		'o'
#define VSN_NOTFOUND	'n'
#define VSN_BAD		'b'
#define VSN_RECYCLING	'r'
#define VSN_RECYCLED	'd'

/*
 *  vsn table record - 256 bytes
 */
#define VSN_ERRLEN	125

typedef struct vsbuf {
	char	vb_vsn[VSNLEN];			char	b01;
	char	vb_class[VV_CLASSLEN];		char	b02;
	char	vb_classI;			char	b03;	
	char	vb_status;			char	b04;
	char	vb_busy;			char	b05;
	char	vb_lastclass[VV_CLASSLEN];	char	b06;
	char	vb_lastserial[VV_SERIALLEN];	char	b07;
	char	vb_lastrec[INTLEN];		char	b08;
	char	vb_lastblk[INTLEN];		char	b09;
	char	vb_errsinarow[INTLEN];		char	b10;
	char	vb_toterrs[INTLEN];		char	b11;
	char	vb_livevv[INTLEN];		char	b12;
	char	vb_delvv[INTLEN];		char	b13;
	char	vb_errmsg[VSN_ERRLEN];		char	end;
} vsbuf_t; 

