#include	<stdio.h>
#include	<stdlib.h>
#include	<stdarg.h>

#include	"out.h"
#include	"cl_space.h"

FILE	*fsp = NULL;

char	*
vsnout(vsn_t	*vsn)
{
	static	char	vsnhold[sizeof(vsn_t)+1];	/* init to zero's */

	memcpy(vsnhold, vsn, sizeof(vsn_t));
	return vsnhold;
}

/*
**	Output metadata describing the removable media being exchanged.
*/
int
cart(char *vsn)
{
	VDR_t		*viP;
	int		vtype;
	VolType_t	*vtP;
	char		*compression;
	char		value[32];
	static	char	unknown[] = "UNKNOWN";

	viP = vm_vinfo(vsn);	/* lookup vsn */
	if (viP == NULL)
		return 1;
	vtype = viP->vdr_info.vi_voltype;
	vtP = vVolType(vtype);
	if (vtP == NULL)
		return 1;

	/* decide it compression is possible */
	compression = (vtP->vt_capacity1 < vtP->vt_capacity2) ? "YES" : "NO";

	out(fsp, "CART_ID", vsnout(&viP->vdr_info.vi_vsn), NULL);
	out(fsp, "CART_INFO",
		"MEDIA_TYPE", vtP->vt_id,
		"PHYSICAL_RECORDING_FORMAT", unknown, NULL);
	out(fsp, "CART_DRIVE_TYPE",
		"MFR", unknown,
		"MODEL", unknown,
		"TYPE", vtP->vt_name,
		"DRIVE_COMPRESSION", compression, NULL);

	sprintf(value, "%d", vtP->vt_capacity2);
	out(fsp, "CART_LAYOUT",
		"NO_SIDES", "1",
		"NO_PARTITIONS", "1",
		"SIDE_NO", "0",
		"PARTITION_NO", "0",
		"SIZE_MB", value,
		"FREE_SPACE_MB", unknown,
		"LABEL_TYPE", "ANSI X3.27-1987",
		"INTERNAL_LABEL", vsnout(&viP->vdr_info.vi_label), NULL);

	out(fsp, "CART_NO_BYTES_AFTER_TM",
		"NO_BLOCKS", "0",
		"NO_BYTES", "0", NULL);

	sprintf(value, "%d", vtP->vt_bufsize);
	out(fsp, "CART_BLOCK_SIZE",
		"MODE", "VARIABLE", NULL);
	out(fsp, "EXT_CART_BLOCK_SIZE",
		"MODE", "VARIABLE",
		"MAX_SIZE", value, NULL);

	sprintf(value, "%d", viP->vdr_mntwrite + viP->vdr_mntread);
	outx(fsp, "CART_STATS",
		"NO_MOUNTS", value,
		"CREATION", dateout(viP->vdr_creat), NULL);
	outx(fsp, "MODIFIED", dateout(viP->vdr_mod), NULL);
	out(fsp, "LAST_MOUNT", dateout(viP->vdr_acc),
		"NO_ERRORS", unknown, NULL);

	out(fsp, "CART_BYTE_ORDER", "ABCD", NULL);
	out(fsp, "CART_RECORDING_FSMS", "NAStore3", NULL);

	return 0;
}

int
main(int argc, char *argv[])
{
	char		*cart_name = CART_FILE;
	char		*vsn_in = VSN_FILE;
	int		bail = FALSE;
	int		ret, prob;
	int		c, i;
	int		verbose = FALSE;
	int		loglev = EL_ERROR1;
	FILE		*fp;
	char		line[100];
	int		errflg = FALSE;
	extern char	*optarg;
	extern int	optind, opterr;

	while ((c = getopt(argc, argv, "f:vdb")) != EOF) {
		switch (c) {

		case 'f':
			vsn_in = optarg;
			break;

		case 'v':
			verbose = TRUE;
			break;

		case 'd':
			loglev = EL_LEVEL;
			break;

		case 'a':
			bail = TRUE;
			break;

		case '?':
			errflg = TRUE;
			break;
		}
	}

	if (errflg == TRUE) {
		fprintf(stderr, "usage: %s [-f cartin][-vda]\n", argv[0]);
		return 2;
	}

	if ((fp = fopen(vsn_in, "r")) == NULL) {
		perror(vsn_in);
		return 1;
	}
	if ((fsp = fopen(cart_name, "w")) == NULL) {
		perror(cart_name);
		return 1;
	}

	nLogInitStderr(loglev);
	if (vm_init(argv[0]) != 0) {
		vm_prerr(stderr);
		return 1;
	}

	prob = 0;
	for (i=1;; i++) {
		if (fgets(line, sizeof(line), fp) == NULL)
			break;
		if (verbose)
			printf("%d:\t%s", i, line);

		if (cart(line) != 0) {
			prob = 1;
			if (bail)
				break;
		}
	}
	if ((ret = ferror(fp)) != 0)
		perror(vsn_in);

	vm_term();
	fclose(fsp);
	fclose(fp);

	if (prob) {
		fprintf(stderr,
			"Problems looking up the cartridge info.\n");
	}
	return prob;
}
