#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<fcntl.h>
#include	<dirent.h>
#include	<limits.h>
#include	<db.h>

#include	<sys/utsname.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<sys/param.h>
#include	<sys/sysctl.h>

#include	<miscfs/dmfs/dmfs.h>
#include	<miscfs/dmfs/dmfs_fcntl.h>
#include	<miscfs/dmfs/rash.h>
#include	"out.h"
#include	"nasutil.h"

char	*root_dir;
FILE	*fp = NULL;
DB	*db = NULL;
int	verbose = FALSE;

/*
**	Should be in rash.
*/
#define	BFIDLEN	32

/*
**	This routine is duplicated from dblib.c in rash.
*/
char  *
bfidout(bfid_t *bfidP)
{
	static	char	out[BFIDLEN+1];

	Bin2Hex(out, BFIDLEN, bfidP, sizeof(bfid_t));
	out[BFIDLEN] = '\0';
	return out;
}

#define	FMT(x)	(int)(sizeof(x)*2), (int)(sizeof(x)*2), x

/*
**	Create a file UID for a non-archived file.
**	Use the device number and inode number concatenated together
**	as hex numbers.
*/
char	*
fileuid(struct stat *sp)
{
	static	char	buf[40];

	sprintf(buf, "%0*.*X%0*.*X",
		FMT(sp->st_dev), FMT(sp->st_ino));
	return buf;
}

typedef struct  DirList {
	char		*name;
	struct	stat	sbuf;
	rashstat_t	rstat;
	struct	DirList	*next;
} DirList_t;

/*
**	Output metadata describing the directory and file structure
**	being exchanged.  This function is called recursively
**	so the stack variable space is minimized.
*/
int
traverse(const char *path, struct stat *sP, rashstat_t *rP)
{
	struct	stat	sbuf;
	char		filepath[PATH_MAX+1];
	int		filelen;
	static	char	*file_type;
	static	char	*file_bfid;
	static	char	file_mode[8];
	static	char	file_link[PATH_MAX+1];
	static	int	first;
	static	rashstat_t	rstat;
	static	DBT	key;
	static	DBT	data = { NULL, 0};
	struct	dirent	*dp;
	DIR		*dirp;
	DirList_t	*listp; 
	DirList_t	*lp;
	int		rc;

	file_bfid = NULL;
	if (S_ISREG(sP->st_mode)) {
		char	keystr[BFIDLEN+10];

		file_type = "FILE";
		if (rP->st_rashflg & DM_ARCH) {
			/*
			**	It is archived so put the dbuid and bfid
			**	into the index for later retrieval by
			**	the file segment program.
			*/
			file_bfid = bfidout(&rP->st_bfid);
			sprintf(keystr, "%s %s",
				uidname(rP->st_dbuid), file_bfid);
			key.data = keystr;
			key.size = strlen(keystr);
			if (db->put(db, &key, &data, 0) == -1) {
				perror("dbput");
				return 1;
			}
		}
	}
	else if (S_ISDIR(sP->st_mode))
		file_type = "DIR";
	else if (S_ISLNK(sP->st_mode)) {
		int	len;

		file_type = "SYM";
		len = readlink(path, file_link, PATH_MAX);
		if (len == -1) {
			perror(path);
			return 1;
		}
		file_link[len] = '\0';
	}
	else
		return 0;		/* not interesting */

	if (verbose)
		printf("%s:\t%s\n", file_type, path);

	sprintf(file_mode, "%4.4o", sP->st_mode & 06777);
	outx(fp, "D_OBJ_ATTR_LIST",
		"UID", (file_bfid != NULL) ? file_bfid : fileuid(sP),
		"NAME", path,
		"OBJ_TYPE", file_type,
		"PERMISSIONS", file_mode,
		"OWNER_ID", uidname(sP->st_uid),
		"GROUP_ID", gidname(sP->st_gid),
		NULL);

	if (!S_ISDIR(sP->st_mode))
		outx(fp, "LENGTH", offout(sP->st_size), NULL);
	if (S_ISLNK(sP->st_mode))
		outx(fp, "SYM_CONTENTS", file_link, NULL);

	outx(fp, "CTIME", dateout(sP->st_ctime), NULL);
	outx(fp, "ATIME", dateout(sP->st_atime), NULL);
	out(fp, "MTIME", dateout(sP->st_mtime), NULL);

	if (!S_ISDIR(sP->st_mode))	/* nothing else to do if not a dir */
		return 0;

	/*
	**	If the file is a directory, we need to open it and
	**	create a list of entries, then decend the tree.
	*/
	if ((dirp = opendir(path)) == NULL) {
		perror(path);
		return 1;
	}

	outx(fp, "D_CONTENTS_LIST", "DIR", fileuid(sP), NULL);
	first = 1;
	strcpy(filepath, path);
	filelen = strlen(filepath);
	filepath[filelen++] = '/';
	listp = NULL;

	/*
	**	Do the content list.
	*/
	while ((dp = readdir(dirp)) != NULL) {
		if (dp->d_name[0] == '.') {
			if ( dp->d_name[1] == '\0' ||
			    (dp->d_name[1] == '.' && dp->d_name[2] == '\0'))
				continue;
		}
		if (first) {
			outx(fp, "LIST", NULL);
			first = 0;
		}
		strcpy(&filepath[filelen], dp->d_name);
		if (lstat(filepath, &sbuf) == -1) {
			perror(filepath);
			continue;
		}

		/*
		**	See if it is a regular file with a bfid.
		*/
		file_bfid = NULL;
		if (S_ISREG(sbuf.st_mode)) {
			int	fd;

			/*
			**	It's a regular file so we have to open it
			**	to get the bfid.
			*/
			if ((fd = open(filepath, O_RDONLY)) == -1) {
				perror(filepath);
				continue;
			}
			/* get bfid */
			if (fcntl(fd, DMFS_GETRINFO, &rstat) == -1) {
				perror(filepath);
				close(fd);
				continue;
			}
			close(fd);
			if (rstat.st_rashflg & DM_ARCH)
				file_bfid = bfidout(&rstat.st_bfid);
		}
		else
			memset(&rstat, 0, sizeof(rstat));

		strcpy(&filepath[filelen], dp->d_name);
		lp = malloc(sizeof(*lp));
		lp->name = strdup(filepath);
		memcpy(&lp->sbuf, &sbuf, sizeof(sbuf));
		memcpy(&lp->rstat, &rstat, sizeof(rstat));
		lp->next = listp;
		listp = lp;

		outx(fp, dp->d_name,
			file_bfid != NULL ? file_bfid : fileuid(&sbuf),
			NULL);
	}
	closedir(dirp);
	outnl(fp);	/* finish record */

	/*
	**	Now, recursively call traverse() to
	**	handle each entry in this directory.
	*/
	rc = 0;
	for (lp = listp; lp; lp = listp) {
		if (traverse(lp->name, &lp->sbuf, &lp->rstat))
			rc = 1;
		free(lp->name);
		listp = lp->next;
		free(lp);
	}
	return rc;
}

int
main(int argc, char *argv[])
{
	char		*fsms_name = FSMS_FILE;
	char		*fileinfo_name = FILEINFO_FILE;
	char		*bfid_out = UID_BFID_FILE;
	char		*vendor = NULL;
	int		ret, c, i;
	int		loglev = EL_ERROR1;
	DBT		key;
	struct	stat	sbuf;
	int		errflg = FALSE;
	static	char	unknown[] = "UNKNOWN";
	struct	utsname	uts;
	char		*version = unknown;
	char		value[256];
	int		mib[2];
	size_t		len;
	extern char	*optarg;
	extern int	optind, opterr;

	while ((c = getopt(argc, argv, "b:vdV:")) != EOF) {
		switch (c) {

		case 'b':
			bfid_out = optarg;
			break;

		case 'v':
			verbose = TRUE;
			break;

		case 'd':
			loglev = EL_LEVEL;
			break;

		case 'V':
			vendor = optarg;
			break;

		case '?':
			errflg = TRUE;
			break;
		}
	}

	if (errflg == TRUE) {
		fprintf(stderr,
		"usage: %s [-b bfidout][-V vendor][-v][-d] rootdir ...\n",
			argv[0]);
		return 2;
	}

	for (i=optind; i<argc; i++) {
		root_dir = argv[i];
		if (root_dir[0] != '/') {
			fprintf(stderr, "%s: must use a full path\n", root_dir);
			return 2;
		}
	}

	/*
	**	Output metadata describing the originating FSMS and
	**	its environment.
	*/
	if ((fp = fopen(fsms_name, "w")) == NULL) {
		perror(fsms_name);
		return 1;
	}
	if (uname(&uts) == -1) {
		perror("uname");
		return 1;
	}
	mib[0] = CTL_HW;
	mib[1] = HW_MODEL;
	len = sizeof(value);
	if (sysctl(mib, 2, &value, &len, NULL, 0) != -1) {
		value[len] = '\0';
		version = value;
	}

	out(fp, "EXPORT_VERSION", "AIIM MS55-1999", NULL);
	out(fp, "EXPORT_HARDWARE_SYSTEM",
		"VENDOR", vendor ? vendor : unknown,
		"NAME", uts.machine,
		"VERSION", version, NULL);
	out(fp, "EXPORT_OS",
		"VENDOR", uts.sysname,
		"NAME", uts.nodename,
		"VERSION", uts.release, NULL);
	out(fp, "EXPORT_TIME", dateout(time(NULL)), NULL);
	out(fp, "EXPORT_FSMS",
		"VENDOR", FSMS_VENDOR,
		"FSMS", FSMS_NAME,
		"VERSION", FSMS_VERSION, NULL);
	outx(fp, "EXPORT_ROOT", NULL);
	for (i=optind; i<argc; i++)
		outx(fp, argv[i], NULL);
	outnl(fp);
	fclose(fp);

	/*
	**	Now start on output of file info.
	*/
	nLogInitStderr(loglev);
	nas_readpwd();
	nas_readgrp();

	/*
	**	Open the file describing the directory and file structure.
	*/
	if ((fp = fopen(fileinfo_name, "w")) == NULL) {
		perror(fileinfo_name);
		return 1;
	}
	/*
	**	Open a btree index to store uid/bfid keys.  This
	**	is an easy way to remove duplicates (from hard links)
	**	for generating file segments later.
	*/
	if ((db = dbopen(NULL, O_RDWR, 0600, DB_BTREE, NULL)) == NULL) {
		perror("dbopen");
		return 1;
	}

	ret = 0;
	for (i=optind; i<argc; i++) {
		root_dir = argv[i];
		if (lstat(root_dir, &sbuf) == -1) {
			perror(root_dir);
			return 1;
		}
		if (!S_ISDIR(sbuf.st_mode)) {
			fprintf(stderr, "%s: not a directory\n", root_dir);
			return 2;
		}
		if (traverse(root_dir, &sbuf, NULL)) {
			fprintf(stderr,
				"Problem with directory traversal of %s\n",
				root_dir);
			ret = 1;
		}
	}
	fclose(fp);

	if (ret) {
		fprintf(stderr,
			"Info in %s will not be complete/correct\n",
			fileinfo_name);
	}

	/* open file to list uid:bfid pairs */
	if ((fp = fopen(bfid_out, "w")) == NULL) {
		perror(bfid_out);
		db->close(db);
		return 1;
	}

	/* output bfid's from db index */
	key.data = NULL;
	key.size = 0;
	for (;;) {
		if ((ret = db->seq(db, &key, NULL, R_NEXT)) != 0)
			break;
		/* output key */
		if (fwrite(key.data, 1, key.size, fp) != key.size) {
			perror("fwrite");
			break;
		}
		fputc('\n', fp);
	}
	if (ret == -1) {
		perror("db_seq");
		ret = 1;
	}
	else
		ret = 0;
	db->close(db);		/* tmp db ... no leftover files */
	return ret;
}
