#include	<stdio.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<assert.h>
#include	<db.h>
#include	<sys/param.h>
#include	<miscfs/dmfs/rash.h>

#include	"out.h"
#include	"nasutil.h"
#include	"vv_cl.h"
#include	"dblib.h"
#include	"rashcom.h"
#include	"labels.h"

FILE		*fsp = NULL;
DB		*db = NULL;
int		bfid_offset = (int) &((file_hdr_lbl_t *)0)->bfid;
char		bfid_offout[10];
int		verbose = FALSE;

/*
**	Output metadata describing the file segments on the removable media
**	being exchanged.
**	Return number of vsn's processed.
*/
int
file_segs(char *uname, char *file_bfid)
{
	bfid_t		bfid;
	VVInfo_t	*vi;
	USR_FILEINFO	*fP;
	int		len, i, err;
	int		nvsns = 0;
	int		vv_offset;
	int		vv_numblocks;
	int		file_hdr_size;
	int		file_trail_size;
	static	char	num[32];
	static	DBT	key;
	static	DBT	data = { NULL, 0};
	static	char	lastu[40];
	static	char	vsn[VSNLEN+1];

	/*
	**	Open the RASH database for this user.
	*/
	if (strcmp(lastu, uname) != 0) {
		rashdb_close();
		if (rashdb_open(uname, O_DB_READ) != RASHDB_OK)
			return 0;
		strcpy(lastu, uname);
	}

	len = strlen(file_bfid);
	file_bfid[--len] = '\0';
	Hex2Bin(&bfid, sizeof(bfid), file_bfid, len);

	/*
	**	Find the active record for this file.
	*/
	if ((fP = rashdb_get_bfid(&err, &bfid)) == NULL) {
		fprintf(stderr, "bfid %s not found in rash database\n", 
			file_bfid);
		return 0;
	}
	len = strlen(fP->usr_fname);

	out(fsp, "FS_FILE_UID", file_bfid, NULL);
	out(fsp, "FS_FILE_NAME", fP->usr_fname, NULL);

	/*
	**	Get the information for each VV which contains
	**	a segment of this file.
	*/
	for(; fP != NULL; fP = rashdb_next_bfidrec(&err, &bfid)) {
		vi = vv_info(fP->usr_vvname);
		if (vi == NULL) {
			fprintf(stderr, "vv %s has no info\n",
					fP->usr_vvname);
			continue;
		}

		if (vi->v_status != VVS_MIGRATED) {
			fprintf(stderr, "vv %s status '%c' (not migrated)\n",
				fP->usr_vvname, vi->v_status);
			continue;
		}

		/*
		**	This VV has been migrated to tapes.  Get
		**	the information for each vsn.
		*/
		if (verbose) {
			printf("\tvv %s", fP->usr_vvname);
			for (i=0; i<vi->v_nvsns; i++) {
				strncpy(vsn,  vi->v_vsns[i].v_vsn, VSNLEN);
				printf("\t%s(%d)", vsn, i);
			}
			printf("\n");
		}
		for (i=0; i<vi->v_nvsns; i++) {		/* for each vsn */
			VVsn_t	*vv = &vi->v_vsns[i];
			Vsn_t	*vp;

			strncpy(vsn,  vv->v_vsn, VSNLEN);
			vp = vv_vsninfo(vsn);
			if (vp == NULL) {
				fprintf(stderr, "no info for vsn %s\n", vsn);
				continue;
			}
			nvsns++;

			/*	Insert vsn into db index with no dups. */
			key.data = vsn;
			key.size = VSNLEN;
			if (db->put(db, &key, &data, 0) == -1) {
				perror("dbput");
				continue;
			}

			/*
			**	Calculate offsets into the tape.
			*/
			vv_offset = DiskHdrSize() + fP->usr_bof;
			vv_numblocks =  vv_offset / vp->vs_blksize;
			vv_offset = vv_offset - (vv_numblocks * vp->vs_blksize);
			vv_numblocks = vv_numblocks + 1;
			file_hdr_size = sizeof(file_hdr_lbl_t) +
					len + ENDMRK_LEN;
			file_trail_size = sizeof(file_hdr_lbl_t) +
					ENDMRK_LEN;

			sprintf(num, "%d", fP->usr_vvno);
			out(fsp, "FS_NO", num, NULL);

			sprintf(num, "%d", i);
			out(fsp, "FS_COPY", num, NULL);

			/*
			**	Position using tape marks.
			*/
			sprintf(num, "%d", vv->v_nrec);
			outx(fsp, "FS_INFO",
				"OFFSET", offout(fP->usr_lseek), NULL);
			outx(fsp, "LENGTH", offout(fP->usr_vvdata),
				"CART_ID", vsn,
				"TAPE_MARK", num, NULL);
			sprintf(num, "%d", vv_numblocks+1);
			outx(fsp, "BLOCK", num, NULL);
			sprintf(num, "%d", vv_offset);
			out(fsp, "BYTE", num, NULL);

			/*
			**	Position using locate block.
			*/
			sprintf(num, "%u", vv->v_block);
			outx(fsp, "EXT_FS_INFO",
				"OFFSET", offout(fP->usr_lseek), NULL);
			outx(fsp, "LENGTH", offout(fP->usr_vvdata),
				"CART_ID", vsn,
				"COOKIE", num, NULL);
			sprintf(num, "%d", vv_numblocks);
			outx(fsp, "BLOCK", num, NULL);
			sprintf(num, "%d", vv_offset);
			out(fsp, "BYTE", num, NULL);

			sprintf(num, "%d", file_hdr_size);
			out(fsp, "FS_HEADER",  num, NULL);

			out(fsp, "EXT_FS_HEADER_MATCH",
				"OFFSET", "0",
				"KEY", "FILEHDR", NULL);

			out(fsp, "EXT_FS_HEADER_MATCH",
				"OFFSET", bfid_offout,
				"KEY", file_bfid, NULL);

			sprintf(num, "%d", file_trail_size);
			out(fsp, "FS_TRAILER", num, NULL);

			out(fsp, "EXT_FS_TRAILER_MATCH",
				"OFFSET", "0",
				"KEY", "FILE", NULL);

			out(fsp, "EXT_FS_TRAILER_MATCH",
				"OFFSET", bfid_offout,
				"KEY", file_bfid, NULL);
		}
	}
	if (nvsns == 0)
		fprintf(stderr, "no vsn's found for bfid %s\n", file_bfid);
	return nvsns;
}

int
main(int argc, char *argv[])
{
	char		*fileseg_name = FILESEG_FILE;
	char		*bfid_in = UID_BFID_FILE;
	char		*vsn_out = VSN_FILE;
	int		bail = 0;
	int		ret, prob;
	int		c, i;
	int		loglev = EL_ERROR1;
	FILE		*fp;
	char		line[100];
	char		*sep;
	DBT		key;
	int		errflg = FALSE;
	extern char	*optarg;
	extern int	optind, opterr;

	while ((c = getopt(argc, argv, "b:c:vda")) != EOF) {
		switch (c) {

		case 'b':
			bfid_in = optarg;
			break;

		case 'c':
			vsn_out = optarg;
			break;

		case 'v':
			verbose = TRUE;
			break;

		case 'd':
			loglev = EL_LEVEL;
			break;

		case 'a':
			bail = TRUE;
			break;

		case '?':
			errflg = TRUE;
			break;
		}
	}

	if (errflg == TRUE) {
		fprintf(stderr,
			"usage: %s [-b bfidin][-c cartout][-vda]\n",
			argv[0]);
		return 2;
	}

	if ((fp = fopen(bfid_in, "r")) == NULL) {
		perror(bfid_in);
		return 1;
	}
	if ((fsp = fopen(fileseg_name, "w")) == NULL) {
		perror(fileseg_name);
		return 1;
	}
	if ((db = dbopen(NULL, O_RDWR, 0600, DB_BTREE, NULL)) == NULL) {
		perror("dbopen");
		return 1;
	}
		
	sprintf(bfid_offout, "%d", bfid_offset);
	nLogInitStderr(loglev);
	if (vv_init(vvHost, 0) != VVE_NOERROR)
		vvErrorMsg(EL_FATAL, argv[0], NULL);

	prob = 0;
	for (i=1;; i++) {
		if (fgets(line, sizeof(line), fp) == NULL)
			break;
		if (verbose)
			printf("%d:\t%s", i, line);

		sep = strchr(line, ' ');
		if (sep == NULL) {
			fprintf(stderr,
			"line %d has no space separating the uid and bfid\n",
				i);
			break;
		}
		*sep++ = '\0';
		if (file_segs(line, sep) == 0) {
			prob = 1;
			if (bail)
				break;
		}
	}
	if ((ret = ferror(fp)) != 0)
		perror(bfid_in);

	rashdb_close();
	vv_term();
	fclose(fsp);
	fclose(fp);

	if (prob) {
		fprintf(stderr,
			"Problems looking up the file segment info.\n");
		fprintf(stderr, "Cartridge list will be incomplete.\n");
	}

	/* open file to list vsns */
	if ((fp = fopen(vsn_out, "w")) == NULL) {
		perror(vsn_out);
		return 1;
	}
	/* output vsn's from db index */
	key.data = NULL;
	key.size = 0;
	for (;;) {
		if ((ret = db->seq(db, &key, NULL, R_NEXT)) != 0)
			break;
		/* output key */
		if (fwrite(key.data, 1, key.size, fp) != key.size) {
			perror("fwrite");
			break;
		}
		fputc('\n', fp);
	}
	if (ret == -1) {
		perror("db_seq");
		ret = 1;
	}
	else
		ret = 0;
	db->close(db);		/* tmp db ... no leftover files */
	fclose(fp);
	return ret;
}
