#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<time.h>

#define	KEYDEF	"cart_keys.h"
#include	"com.h"

typedef struct {
	int	side_no;
	int	part_no;
	char	*name;
	char	*size_mb;
	char	*free_mb;
	char	*label_type;
	char	*int_label;
} part_t;

field_t	part_tab[] = {
    { CART_LAYOUT,	SIDE_NO,	OFF(part_t, side_no),	T_INTEGER },
    { CART_LAYOUT,	PARTITION_NO,	OFF(part_t, part_no),	T_INTEGER },
    { CART_LAYOUT,	PARTITION_NAME,	OFF(part_t, name),	T_STRING },
    { CART_LAYOUT,	SIZE_MB,	OFF(part_t, size_mb),	T_STRINTEGER },
    { CART_LAYOUT,	FREE_SPACE_MB,	OFF(part_t, free_mb),	T_STRINTEGER },
    { CART_LAYOUT,	LABEL_TYPE,	OFF(part_t, label_type),T_STRING },
    { CART_LAYOUT,	INTERNAL_LABEL,	OFF(part_t, int_label),	T_STRING }, 
    { NOKEY,		NOKEY,		0,			T_NOTYPE }
};

typedef struct {
	char	*ident;
	char	*media_type;
	char	*rec_format;
	char	*mfr;
	char	*model;
	char	*type;
	char	*drive_comp;
	int	no_partitions;
	int	no_sides;
	part_t	*partitions;
	char	*nbbtm;
	char	*nblockatm;
	char	*nbytesatm;
	char	*blockmode;
	char	*blocksize;
	char	*family;
	char	*billing;
	char	*volgrpname;
	char	*volgrpblk;
	char	*volgrpcomp;
	char	*lib;
	char	*place;
	char	*lot;
	char	*nomounts;
	char	*creation;
	char	*modified;
	char	*lastmount;
	char	*noerrors;
	char	*byteorder;
	char	*blocklen;
	char	*dblklen;
	char	*hdrlen;
	char	*trllen;
	char	*fsms;
} cart_t;

field_t	cart_tab[] = {
    { CART_ID,		NOKEY,		OFF(cart_t, ident),	T_STRING },
    { CART_INFO,	MEDIA_TYPE,	OFF(cart_t, media_type),T_STRING },
    { CART_INFO,PHYSICAL_RECORDING_FORMAT,OFF(cart_t, rec_format),T_STRING },
    { CART_DRIVE_TYPE,	MFR,		OFF(cart_t, mfr),	T_STRING },
    { CART_DRIVE_TYPE,	MODEL,		OFF(cart_t, model),	T_STRING },
    { CART_DRIVE_TYPE,	TYPE,		OFF(cart_t, type),	T_STRING },
    { CART_DRIVE_TYPE,	DRIVE_COMPRESSION,OFF(cart_t, drive_comp),T_BOOLEAN },
    { CART_LAYOUT,	NO_PARTITIONS,	OFF(cart_t, no_partitions),T_INTEGER },
    { CART_LAYOUT,	NO_SIDES,	OFF(cart_t, no_sides),	T_INTEGER },
    { CART_NO_BLOCKS_BETWEEN_TM,NOKEY,	OFF(cart_t, nbbtm),	T_STRINTEGER },
    { CART_NO_BYTES_AFTER_TM,NO_BLOCKS,	OFF(cart_t, nblockatm),	T_STRINTEGER },
    { CART_NO_BYTES_AFTER_TM,NO_BYTES,	OFF(cart_t, nbytesatm),	T_STRINTEGER },
    { CART_BLOCK_SIZE,	MODE,		OFF(cart_t, blockmode),	T_STRING },
    { CART_FAMILY,	NOKEY,		OFF(cart_t, family),	T_STRING },
    { CART_BILLING_ID,	NOKEY,		OFF(cart_t, billing),	T_STRING },
    { CART_VOL_GROUP,	NAME,		OFF(cart_t, volgrpname),T_STRING },
    { CART_VOL_GROUP,	BLOCKSIZE,	OFF(cart_t, volgrpblk),	T_STRINTEGER },
    { CART_VOL_GROUP,	COMPRESSION_ALLOWED,OFF(cart_t, volgrpcomp),T_BOOLEAN },
    { CART_LIB,		NOKEY,		OFF(cart_t, lib),	T_STRING },
    { CART_PLACE,	NOKEY,		OFF(cart_t, place),	T_STRING },
    { CART_LOT,		NOKEY,		OFF(cart_t, lot),	T_STRING },
    { CART_STATS,	NO_MOUNTS,	OFF(cart_t, nomounts),	T_STRINTEGER },
    { CART_STATS,	CREATION,	OFF(cart_t, creation),	T_DATE },
    { CART_STATS,	MODIFIED,	OFF(cart_t, modified),	T_DATE },
    { CART_STATS,	LAST_MOUNT,	OFF(cart_t, lastmount),	T_DATE },
    { CART_STATS,	NO_ERRORS,	OFF(cart_t, noerrors),	T_STRINTEGER },
    { CART_BYTE_ORDER,	NOKEY,		OFF(cart_t, byteorder),	T_STRING },
    { CART_BLOCK_STRUCT,BLOCK_LENGTH,	OFF(cart_t, blocklen),	T_STRINTEGER },
    { CART_BLOCK_STRUCT,DATA_BLOCK_LENGTH,OFF(cart_t, dblklen),	T_STRINTEGER },
    { CART_BLOCK_STRUCT,HDR_LENGTH,	OFF(cart_t, hdrlen),	T_STRINTEGER },
    { CART_BLOCK_STRUCT,TRL_LENGTH,	OFF(cart_t, trllen),	T_STRINTEGER },
    { CART_RECORDING_FSMS,NOKEY,	OFF(cart_t, fsms),	T_STRING },
    { NOKEY,		NOKEY,		0,			T_NOTYPE }
};

/*
**	Save/print cart info.
*/
void
cart_save(cart_t *cp)
{
	field_t	*xx;
	int	i, num, side, part;
	part_t	*pp;
	int	volgrp = 0;
	int	loc = 0;
	int	stats = 0;
	int	blkstr = 0;
	char	num1[48], num2[48];
	static	char	notset[] = "%d: %s %s side %d part %d %s not set\n";

	namval_t	cart[] = {
		{ "cart_id",	cp->ident },
		{ "media",	cp->media_type },
		{ "format",	cp->rec_format },
		{ "drive_mfr",	cp->mfr },
		{ "drive_model",	cp->model },
		{ "drive_type",	cp->type },
		{ "drive_comp",	cp->drive_comp },
		{ "partitions",	stringnum(cp->no_partitions, num1) },
		{ "sides",	stringnum(cp->no_sides, num2) },
		{ "blocksbetwntm",	cp->nbbtm },
		{ "blocksaftertm",	cp->nblockatm },
		{ "bytesaftertm",	cp->nbytesatm },
		{ "blocksize",	cp->blocksize },
		{ "family",	cp->family },
		{ "billingid",	cp->billing },
		{ "volgr_name",	cp->volgrpname },
		{ "volgr_blksize",	cp->volgrpblk },
		{ "volgr_comp",	cp->volgrpcomp },
		{ "library",	cp->lib },
		{ "place",	cp->place },
		{ "lot",	cp->lot },
		{ "mounts",	cp->nomounts },
		{ "creation",	cp->creation },
		{ "modified",	cp->modified },
		{ "lastmount",	cp->lastmount },
		{ "noerrors",	cp->noerrors },
		{ "byteorder",	cp->byteorder },
		{ "blocklen",	cp->blocklen },
		{ "data_len",	cp->dblklen },
		{ "hdr_len",	cp->hdrlen },
		{ "trl_len",	cp->trllen },
		{ "recording_fsms",	cp->fsms }
	};

	/* check for required records */
	for (xx = cart_tab; xx->record != NOKEY; xx++) {
		switch (xx->record) {

		/* required */
		case CART_INFO:
		case CART_DRIVE_TYPE:
		case CART_LAYOUT:
		case CART_NO_BYTES_AFTER_TM:
		case CART_BLOCK_SIZE:
		case CART_BYTE_ORDER:
		case CART_RECORDING_FSMS:
			bad_field(cp, xx);
			break;

		/* selecable */
		case CART_VOL_GROUP:
			if (check_field(cp, xx))
				volgrp = 1;
			else if (volgrp)
				print_field(xx);
			break;

		case CART_LOC:
			if (check_field(cp, xx))
				loc = 1;
			else if (loc)
				print_field(xx);
			break;

		case CART_STATS:
			if (check_field(cp, xx))
				stats = 1;
			else if (stats)
				print_field(xx);
			break;

		case CART_BLOCK_STRUCT:
			if (check_field(cp, xx))
				blkstr = 1;
			else if (blkstr)
				print_field(xx);
			break;

		default:
			break;
		}
	}
	if (cp->blockmode) {
		if (strcasecmp(cp->blockmode, "\'FIXED\'") == 0) {
			if (cp->blocksize == NULL) {
				fprintf(stderr,
					"%d: %s %s is %s but no size given\n",
					recordnum-1, key_tab[CART_BLOCK_SIZE],
					key_tab[MODE], cp->blockmode);
			}
		} else if (strcasecmp(cp->blockmode, "\'VARIABLE\'") != 0) {
			fprintf(stderr,
				"%d: %s has unknown MODE '%s'\n",
				recordnum-1, key_tab[CART_BLOCK_SIZE],
				cp->blockmode);
		}
	}

	putrec("cart", cart, sizeof(cart));

	pp = cp->partitions;
	num = cp->no_sides * cp->no_partitions;
	for (side = 0; side < cp->no_sides; side++) {
		for (part = 0; part < cp->no_partitions; part++, pp++) {
			namval_t	part_tab[] = {
				{ "cart_id",	cp->ident },
				{ "side",	stringnum(pp->side_no, num1) },
				{ "partition",	stringnum(pp->part_no, num2) },
				{ "part_name",	pp->name },
				{ "size",	pp->size_mb },
				{ "freespace",	pp->free_mb },
				{ "labeltype",	pp->label_type },
				{ "label",	pp->int_label }
			};

			if (pp->side_no != side || pp->part_no != part) {
				fprintf(stderr,
					"%d: %s %s side %d partition %d %s\n",
					recordnum-1,
					key_tab[CART_LAYOUT],
					cp->ident, side, part,
					"not initialized");
				continue;
			}
			if (pp->size_mb == NULL) {
				fprintf(stderr, notset,
					recordnum-1, key_tab[CART_LAYOUT],
					cp->ident, side, part,
					key_tab[SIZE_MB]);
			}
			if (pp->free_mb == NULL) {
				fprintf(stderr, notset,
					recordnum-1, key_tab[CART_LAYOUT],
					cp->ident, side, part,
					key_tab[FREE_SPACE_MB]);
			}

			putrec("cartpart", part_tab, sizeof(part_tab));
		}
	}

	if (verbose) {
		printf("ident\t\t%s\n", cp->ident);
		printf("media_type\t%s\n", cp->media_type);
		printf("rec_format\t%s\n", cp->rec_format);
		printf("mfr\t\t%s\n", cp->mfr);
		printf("model\t\t%s\n", cp->model);
		printf("type\t\t%s\n", cp->type);
		printf("drive_comp\t%s\n", cp->drive_comp);
		printf("no_partitions\t%d\n", cp->no_partitions);
		printf("no_sides\t%d\n", cp->no_sides);
		printf("nbbtm\t\t%s\n", cp->nbbtm);
		printf("nblockatm\t%s\n", cp->nblockatm);
		printf("nbytesatm\t%s\n", cp->nbytesatm);
		printf("blockmode\t%s\n", cp->blockmode);
		printf("blocksize\t%s\n", cp->blocksize);
		printf("family\t\t%s\n", cp->family);
		printf("billing\t\t%s\n", cp->billing);
		printf("volgrpname\t%s\n", cp->volgrpname);
		printf("volgrpblk\t%s\n", cp->volgrpblk);
		printf("volgrpcomp\t%s\n", cp->volgrpcomp);
		printf("lib\t\t%s\n", cp->lib);
		printf("place\t\t%s\n", cp->place);
		printf("lot\t\t%s\n", cp->lot);
		printf("nomounts\t%s\n", cp->nomounts);
		printf("creation\t%s\n", cp->creation);
		printf("modified\t%s\n", cp->modified);
		printf("lastmount\t%s\n", cp->lastmount);
		printf("noerrors\t%s\n", cp->noerrors);
		printf("byteorder\t%s\n", cp->byteorder);
		printf("blocklen\t%s\n", cp->blocklen);
		printf("dblklen\t\t%s\n", cp->dblklen);
		printf("hdrlen\t\t%s\n", cp->hdrlen);
		printf("trllen\t\t%s\n", cp->trllen);
		printf("fsms\t\t%s\n\n", cp->fsms);

		for (i=0, pp=cp->partitions; i<num; i++, pp++) {
			printf("PARTITION\t%d\n", i);
			printf("side_no\t\t%d\n", pp->side_no);
			printf("part_no\t\t%d\n", pp->part_no);
			printf("label_type\t%s\n", pp->label_type);
			printf("int_label\t%s\n", pp->int_label);
			printf("name\t\t%s\n",
				pp->name == NULL ? "NONE" : pp->name);
			printf("size_mb\t\t%s\n", pp->size_mb);
			printf("free_mb\t\t%s\n\n", pp->free_mb);
		}
	}

	for (i=0, pp=cp->partitions; i<num; i++, pp++) {
		FREE(pp->name);
		FREE(pp->size_mb);
		FREE(pp->free_mb);
		FREE(pp->label_type);
		FREE(pp->int_label);
	}
	FREE(cp->partitions);
	free_all(cp, cart_tab);

	return;
}

/*
**	Process a cartridge description.
**	Return 0 on failure.
*/
int
cart_description()
{
	reckey_t	cmd;
	int		i, tot_parts, partno, sideno, x;
	int		a, b, c, d, err;
	cart_t		*cart;
	part_t		*part;

	key_tab = localtab;

	if (match_token(CART_ID) == 0)
		return 0;

	cart = (cart_t *)malloc(sizeof(cart_t));
	memset(cart, '\0', sizeof(cart_t));
	if (get_token(CART_ID, NOKEY) == 0)
		return 0;

	init_field(cart, cart_tab);
	cart->ident = quotestr();

	while (start_token()) {
		cmd = find_key();
		switch (cmd) {

		case CART_INFO:
		case CART_DRIVE_TYPE:
		case CART_NO_BYTES_AFTER_TM:
		case CART_BLOCK_SIZE:
		case CART_VOL_GROUP:
		case CART_LOC:
		case CART_STATS:
		case CART_BLOCK_STRUCT:
			store_info(cart, cmd, cart_tab);
			break;

		case CART_LAYOUT:
			store_info(cart, cmd, cart_tab);
			tot_parts = cart->no_partitions * cart->no_sides;
			part = (part_t *)malloc(tot_parts * sizeof(part_t));
			memset(part, '\0', tot_parts * sizeof(part_t));
			for (i=0; i<tot_parts; i++)
				init_field(&part[i], part_tab);

			for (i=0; i<tot_parts; i++) {
				sideno = initint;
				partno = initint;

				if (check_key(SIDE_NO))
					break;
				if (get_token(cmd, SIDE_NO) == 0)
					break;
				if (storeint(&sideno, cmd, SIDE_NO))
					break;
				if (sideno < 0 || cart->no_sides < sideno) {
					fprintf(stderr,
					"%d: %s %s %d but NO_SIDES %d\n",
						recordnum, key_tab[cmd],
						key_tab[SIDE_NO],
						sideno, cart->no_sides);
					break;
				}
				if (get_token(cmd, PARTITION_NO) == 0)
					break;
				if (check_key(PARTITION_NO))
					break;
				if (get_token(cmd, PARTITION_NO) == 0)
					break;
				if (storeint(&partno, cmd, PARTITION_NO))
					break;
				if (partno < 0 ||
						cart->no_partitions < partno) {
					fprintf(stderr,
						"%d: %s %s %d but %s %d\n",
						recordnum, key_tab[cmd],
						key_tab[PARTITION_NO],
						partno, key_tab[NO_PARTITIONS],
						cart->no_partitions);
					break;
				}
				x = (sideno+1)*(partno+1) - 1;
				part[x].side_no = sideno;
				part[x].part_no = partno;

				store_info(&part[x], cmd, part_tab);
			}
			cart->partitions = part;
			break;

		case CART_NO_BLOCKS_BETWEEN_TM:
			if (get_token(cmd, NOKEY))
				storestrint(&cart->nbbtm, cmd, NOKEY);
			break;

		case CART_FAMILY:
			if (get_token(cmd, NOKEY))
				storestr(&cart->family, cmd, NOKEY);
			break;

		case CART_BILLING_ID:
			if (get_token(cmd, NOKEY))
				storestr(&cart->billing, cmd, NOKEY);
			break;

		case CART_LOT:
			if (get_token(cmd, NOKEY))
				storestr(&cart->lot, cmd, NOKEY);
			break;

		case CART_BYTE_ORDER:
			if (get_token(cmd, NOKEY) == 0)
				break;

			if ((x = strlen(token)) != 4) {
				fprintf(stderr,
					"%d: %s <- %s incorrect length %d\n",
					recordnum, key_tab[cmd], token, x);
				break;
			}

			a = b = c = d = err = 0;
			for (i=0; i<x; i++) {
				switch (token[i]) {
				case 'A':
				case 'a':
					if (a)
						err = 1;
					a = 1;
					break;

				case 'B':
				case 'b':
					if (b)
						err = 1;
					b = 1;
					break;

				case 'C':
				case 'c':
					if (c)
						err = 1;
					c = 1;
					break;

				case 'D':
				case 'd':
					if (d)
						err = 1;
					d = 1;
					break;

				default:
					err = 1;
					break;

				}
			}
			if (err) {
				fprintf(stderr,
					"%d: %s <- %s bad format\n",
					recordnum, key_tab[cmd], token);
				break;
			}

			storestr(&cart->byteorder, cmd, NOKEY);
			break;

		case CART_RECORDING_FSMS:
			if (get_token(cmd, NOKEY))
				storestr(&cart->fsms, cmd, NOKEY);
			break;

		default:
			if (ext_record())
				break;
			goto done;
		}
	}
 done:
	cart_save(cart);
	return 1;
}
