#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<time.h>
#include	<string.h>
#include	"com.h"

char	**key_tab;
char	*token = NULL;
int	tokenlen = 0;
int	tokenmax = 0;
int	recordnum = 1;
int	tokeneol = 0;

static	char	assigned[] = "%d: %s already assigned field: %s\n";
static	char	badfield[] = "(none)";

/*
**	Create a new string with single quotes surronding it.
**	Any single quote in the string will be escaped.
*/
char *
quotestr()
{
	char	*cc, *tp;
	int	len = tokenlen-1;
	int	new = len+3;		/* room for 2 quotes and nul */
	int	i;
	int	j = 0;

	cc = malloc(new);
	tp = token;
	cc[j++] = '\'';

	for (i=0; i<len; i++) {
		if (*tp == '\'') {
			new++;
			cc = realloc(cc, new);
			cc[j++] = '\\';
		}
		cc[j++] = *tp++;
	}
	cc[j++] = '\'';
	cc[j] = '\0';

	return cc;
}

char *
stringnum(int num, char *buf)
{
	(void)sprintf(buf, "%d", num);
	return buf;
}

char	*initstr = NULL;

int
storestr(void *to, reckey_t rec, reckey_t field)
{
	char	*cc;

	/* check if to is already filled in */
	if (memcmp(to, &initstr, sizeof(initstr)) != 0) {
		fprintf(stderr, assigned, recordnum, key_tab[rec],
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}

	cc = quotestr();
	memcpy(to, &cc, sizeof(cc));
	return 0;
}

int	initint = -1;

int
storeint(void *to, reckey_t rec, reckey_t field)
{
	int	num;
	char	*last;

	/* check if to is already filled in */
	if (memcmp(to, &initint, sizeof(initint)) != 0) {
		fprintf(stderr, assigned, recordnum, key_tab[rec],
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}

	num = (int)strtol(token, &last, 0);
	if (*last != '\0') {
		fprintf(stderr, "%d: %s bad integer '%s', field %s\n",
			recordnum, key_tab[rec], token,
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}
	memcpy(to, &num, sizeof(num));
	return 0;
}

/*
**	Store an integer without the check to see if it is
**	set to the initial value.
*/
int
savestrint(void *to, reckey_t rec, reckey_t field)
{
	int	num;
	char	*last;
	char	*cc;

	num = (int)strtol(token, &last, 0);
	if (*last != '\0') {
		fprintf(stderr, "%d: %s bad integer '%s', field %s\n",
			recordnum, key_tab[rec], token,
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}
	cc = strdup(token);
	memcpy(to, &cc, sizeof(cc));
	return 0;
}

int
storestrint(void *to, reckey_t rec, reckey_t field)
{
	/* check if to is already filled in */
	if (memcmp(to, &initstr, sizeof(initstr)) != 0) {
		fprintf(stderr, assigned, recordnum, key_tab[rec],
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}

	return (savestrint(to, rec, field));
}

char	*initoff = NULL;

int
storeoff(void *to, reckey_t rec, reckey_t field)
{
	off_t	num;
	char	*last, *cc;

	/* check if to is already filled in */
	if (memcmp(to, &initoff, sizeof(initoff)) != 0) {
		fprintf(stderr, assigned, recordnum, key_tab[rec],
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}

	num = (int)strtoq(token, &last, 10);
	if (*last != '\0') {
		fprintf(stderr, "%d: %s bad offset '%s', field %s\n",
			recordnum, key_tab[rec], token,
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}
	cc = strdup(token);
	memcpy(to, &cc, sizeof(cc));
	return 0;
}

char	*initbool = NULL;

int
storebool(void *to, reckey_t rec, reckey_t field)
{
	int		num;
	char		*ans, *cc;

	/* check if to is already filled in */
	if (memcmp(to, &initbool, sizeof(initbool)) != 0) {
		fprintf(stderr, assigned, recordnum, key_tab[rec],
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}

	if (strcasecmp(token, "yes") == 0)
		num = 1;
	else if (strcasecmp(token, "no") == 0)
		num = 0;
	else {
		fprintf(stderr, "%d: %s bad boolean value '%s', field %s\n",
			recordnum, key_tab[rec], token,
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}

	ans = num ? "\'yes\'" : "\'no\'";
	cc = strdup(ans);
	memcpy(to, &cc, sizeof(cc));
	return 0;
}

char	*initdate = NULL;

int
storedate(void *to, reckey_t rec, reckey_t field)
{
	static	int	where[5] = {  4,   7,  10,  13,  16  };
	static	char	what[5] =  { '/', '/', '/', ':', ':' };
	struct	tm	tm;
	int		i, j;
	int		num;
	char		*last, dstr[64], *cc;

	/* check if to is already filled in */
	if (memcmp(to, &initdate, sizeof(initdate)) != 0) {
		fprintf(stderr, assigned, recordnum, key_tab[rec],
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}

	if (strlen(token) != 19) {
		fprintf(stderr,
			"%d: %s date '%s' has bad length %d, field %s\n",
			recordnum, key_tab[rec], token, strlen(token),
			field == NOKEY ? badfield : key_tab[field]);
		return 1;
	}

	for (i=0; i<5; i++) {
		j = where[i];

		if (token[j] != what[i]) {
			fprintf(stderr,
				"%d: %s date '%s' char at %d bad, field %s\n",
				recordnum, key_tab[rec], token, j,
				field == NOKEY ? badfield : key_tab[field]);
			return 1;
		}
	}
	for (i=0; i<5; i++) {
		j = where[i];

		token[j] = '\0';
	}
	memset(&tm, '\0', sizeof(tm));

	num = (int)strtol(token, &last, 10);
	if (*last != '\0')
		goto bad;
	tm.tm_year = num - 1900;

	num = (int)strtol(&token[5], &last, 10);
	if (*last != '\0')
		goto bad;
	tm.tm_mon  = num - 1;
	if (tm.tm_mon < 0 || 11 < tm.tm_mon)
		goto bad;

	num = (int)strtol(&token[8], &last, 10);
	if (*last != '\0')
		goto bad;
	tm.tm_mday = num;
	if (tm.tm_mday < 1 || 31 < tm.tm_mday)
		goto bad;

	num = (int)strtol(&token[11], &last, 10);
	if (*last != '\0')
		goto bad;
	tm.tm_hour = num;
	if (tm.tm_hour < 0 || 23 < tm.tm_hour)
		goto bad;

	num = (int)strtol(&token[14], &last, 10);
	if (*last != '\0')
		goto bad;
	tm.tm_min  = num;
	if (tm.tm_min < 0 || 59 < tm.tm_min)
		goto bad;

	num = (int)strtol(&token[17], &last, 10);
	if (*last != '\0')
		goto bad;
	tm.tm_sec  = num;
	if (tm.tm_sec < 0 || 61 < tm.tm_sec)
		goto bad;

	if (strftime(dstr, sizeof(dstr),
			"\'%m/%d/%Y %H:%M:%S.00 UTC\'", &tm) == 0)
		goto bad;

	cc = strdup(dstr);
	memcpy(to, &cc, sizeof(cc));

	return 0;

 bad:
	/* put back the separators */
	for (i=0; i<5; i++) {
		j = where[i];

		token[j] = what[i];
	}
	fprintf(stderr, "%d: %s bad date '%s', field %s\n",
		recordnum, key_tab[rec], token,
		field == NOKEY ? badfield : key_tab[field]);
	return 1;
}

store_t	typetab[] = {
	{storestr,	&initstr,	sizeof(initstr)},	/* STRING */
	{storeint,	&initint,	sizeof(initint)},	/* INTEGER */
	{storestrint,	&initstr,	sizeof(initstr)},	/* STRINTEGER */
	{storeoff,	&initoff,	sizeof(initoff)},	/* OFFSET */
	{storebool,	&initbool,	sizeof(initbool)},	/* BOOLEAN */
	{storedate,	&initdate,	sizeof(initdate)}	/* DATE */
};

/*
**	Read a token which is not at the start of a record.
**	If key is not NOKEY, print a message if no token can be read.
**	Return 0 on failure.
*/
int
get_token(reckey_t rec, reckey_t field)
{
	int	ch;

	tokenlen = 0;
	if (tokeneol)
		goto eol;

	for (;;) {
		ch = fgetc(stdin);
		if (ch == EOF || ch == '\n') {
			if (ch == '\n')
				tokeneol = 1;
			goto eol;
		}
		if (tokenlen >= tokenmax) {
			if (token == NULL) {
				token = malloc(INITTOK);
				tokenmax = INITTOK;
			}
			else {
				tokenmax *= 2;
				token = realloc(token, tokenmax);
			}
		}
		token[tokenlen++] = ch;
		if (ch == '\0')
			break;
	}
	return 1;

 eol:
	if (rec != NOKEY) {
		fprintf(stderr, "%d: %s token missing",
			recordnum, key_tab[rec]);
		if (field != NOKEY)
			fprintf(stderr, " following %s", key_tab[field]);
		fprintf(stderr, "\n");
	}
	return 0;
}

/*
**	Read a token which must be the start of a record.
**	Return 0 on failure.
*/
int
start_token()
{
	int	first = 1;
	int	ch;

	/*
	**	Skip any leftover junk in old record.
	*/
	if (!tokeneol) {
		for (;;) {
			if ((ch = fgetc(stdin)) == EOF)
				return 0;
			if (ch == '\n')
				break;
			if (first) {
				fprintf(stderr, "%d: skipping", recordnum);
				if (verbose)
					fprintf(stderr, ": ");
				first = 0;
			}
			if (verbose)
				fputc(ch == '\0' ? '\t' : ch, stderr);
		}
		if (!first)
			fputc('\n', stderr);
	}
	tokeneol = 0;
	recordnum++;

	return (get_token(NOKEY, NOKEY));
}

/*
**	Check to see if the current token matches the given key.
**	Return TRUE for a good match, FALSE for no match.
*/
int
match_token(reckey_t key)
{
	if (tokenlen == 0)
		return FALSE;
	else
		return(strcasecmp(token, key_tab[key]) == 0);
}

/*
**	Return the key value for the current token.
**	Something fancy could be used here.  For now, just do a
**	linear search.
*/
reckey_t
find_key()
{
	int	i;

	if (tokenlen == 0)
		return NOKEY;
	for (i=0; key_tab[i]; i++) {
		if (strcasecmp(token, key_tab[i]) == 0)
			break;
	}
	return i;
}

/*
**	Make sure the current token matches the key x.
*/
int
check_key(reckey_t x)
{
	reckey_t	field;

	field = find_key();
	if (field != x) {
		fprintf(stderr,
			"%d: found %s expected %s\n", recordnum,
			token, key_tab[x]);
		return TRUE;
	}
	else
		return FALSE;
}

/*
**	Store information from a record according to the field table.
*/
void
store_info(void *obj, reckey_t rec, field_t *tab)
{
	reckey_t	key;
	field_t		*fp;
	store_t		*sp;

	while (get_token(NOKEY, NOKEY)) {
		key = find_key();
		if (key == NOKEY) {
			printf("%d: %s unrecognized keyword '%s'\n",
				recordnum, key_tab[rec], token);
			break;
		}
		for (fp=tab; fp->record != NOKEY; fp++) {
			if (fp->record == rec && fp->field == key)
				break;
		}
		if (fp->record == NOKEY)
			break;

		if (get_token(rec, key) == 0)
			break;

		sp = &typetab[fp->type];
		(void)sp->store((char *)obj+(fp->offset), rec, key);
	}
	return;
}

/*
**	Initialize all the fields defined in tab.
*/
void
init_field(void *obj, field_t *tab)
{
	store_t	*sp;

	for (; tab->record != NOKEY; tab++) {
		sp = &typetab[tab->type];
		memcpy((char *)obj+(tab->offset), sp->initval, sp->size);
	}
}

/*
**	Check to see if a field has been left unset.
*/
int
check_field(void *obj, field_t *tab)  
{
	store_t	*sp = &typetab[tab->type];

	return(memcmp((char *)obj+(tab->offset), sp->initval, sp->size));
}

void
print_field(field_t *tab)
{
	fprintf(stderr,
		"%d: %s field %s not set\n",
		recordnum-1, key_tab[tab->record],
		tab->field == NOKEY ? badfield : key_tab[tab->field]);
	return;
}

/*
**	Print message for a bad field.
*/
void
bad_field(void *obj, field_t *tab)
{
	int	ret;

	ret = check_field(obj, tab);
	if (ret == 0)
		print_field(tab);
	return;
}

/*
**	Free an object and all string fields.
*/
void
free_all(void *obj, field_t *tab)
{
	char	*str;

	for (; tab->record != NOKEY; tab++) {
		if (tab->type == T_INTEGER)	/* int is non-string */
			continue;

		memcpy(&str, (char *)obj+(tab->offset), sizeof(str));
		if (str != NULL)
			free(str);
	}
	free(obj);
}

int	ext_num = 0;
int	ext_out = FALSE;

/*
**	Process an EXT record.
**	Return FALSE on failure.
*/
int
ext_record()
{
	if (strncasecmp(token, "ext_", 4) != 0)
		return FALSE;

	if (ext_out)
		printf("%s:", token);

	while (get_token(NOKEY, NOKEY)) {
		if (ext_out)
			printf("\t%s", token);
	}
	if (ext_out)
		printf("\n");
	ext_num++;
	return TRUE;
}

/*
**	Create insert statement for sql database.
*/
void
putrec(char *table, namval_t *tab, size_t tabsize)
{
	int		i, len;
	char		names[5000], *nP;
	char		values[5000], *vP;

	if (verbose)
		return;

	nP = names;
	vP = values;
	tabsize /= sizeof(namval_t);

	for (i = 0; i<tabsize; i++) {
		namval_t	*vn = &tab[i];

		if (vn->value == NULL)
			continue;

		len = sprintf(nP, "%s, ", vn->name);
		nP += len;

		len = sprintf(vP, "%s, ", vn->value);
		vP += len;
	}
	nP -= 2;
	*nP = '\0';
	vP -= 2;
	*vP = '\0';

	printf("INSERT INTO %s (%s) VALUES (%s);\n", table, names, values);

	return;
}
