#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<time.h>

#define	KEYDEF	"finfo_keys.h"
#include	"com.h"

typedef	struct {
	char	*name;
	char	*uid;
} flist_t;

typedef	struct {
	char	*uid;
	char	*name;
	char	*type;
	char	*perms;
	char	*owner;
	char	*group;
	char	*length;
	char	*ctime;
	char	*atime;
	char	*mtime;
	char	*billing;
	char	*family;
	char	*container;
	char	*symlink;
	int	fnum;
	flist_t	*flist;
} finfo_t;

field_t	finfo_tab[] = {
    { D_OBJ_ATTR_LIST,	UID,		OFF(finfo_t, uid),	T_STRING },
    { D_OBJ_ATTR_LIST,	NAME,		OFF(finfo_t, name),	T_STRING },
    { D_OBJ_ATTR_LIST,	OBJ_TYPE,	OFF(finfo_t, type),	T_STRING },
    { D_OBJ_ATTR_LIST,	PERMISSIONS,	OFF(finfo_t, perms),	T_STRING },
    { D_OBJ_ATTR_LIST,	OWNER_ID,	OFF(finfo_t, owner),	T_STRING },
    { D_OBJ_ATTR_LIST,	GROUP_ID,	OFF(finfo_t, group),	T_STRING },
    { D_OBJ_ATTR_LIST,	LENGTH,		OFF(finfo_t, length),	T_OFFSET },
    { D_OBJ_ATTR_LIST,	CTIME,		OFF(finfo_t, ctime),	T_DATE },
    { D_OBJ_ATTR_LIST,	ATIME,		OFF(finfo_t, atime),	T_DATE },
    { D_OBJ_ATTR_LIST,	MTIME,		OFF(finfo_t, mtime),	T_DATE },
    { D_OBJ_ATTR_LIST,	BILLING_ID,	OFF(finfo_t, billing),	T_STRING },
    { D_OBJ_ATTR_LIST,	FAMILY_ID,	OFF(finfo_t, family),	T_STRING },
    { D_OBJ_ATTR_LIST,	CONTAINER_TYPE,	OFF(finfo_t, container),T_STRING },
    { D_OBJ_ATTR_LIST,	SYM_CONTENTS,	OFF(finfo_t, symlink),	T_STRING },
    { NOKEY,		NOKEY,		0,			T_NOTYPE }
};

void
finfo_save(finfo_t *fp)
{
	field_t	*xx;
	int	i;
	namval_t	finfo[] = {
		{ "uid", fp->uid },
		{ "name", fp->name },
		{ "type", fp->type },
		{ "permissions", fp->perms },
		{ "ownerid", fp->owner },
		{ "groupid", fp->group },
		{ "length", fp->length },
		{ "changed", fp->ctime },
		{ "accessed", fp->atime },
		{ "modified", fp->mtime },
		{ "billing", fp->billing },
		{ "family", fp->family },
		{ "container", fp->container },
		{ "symlink", fp->symlink }
	};

	/* check for required records */
	for (xx = finfo_tab; xx->record != NOKEY; xx++) {
		switch (xx->field) {

		case UID:
		case NAME:
		case OBJ_TYPE:
		case PERMISSIONS:
		case OWNER_ID:
		case GROUP_ID:
		case CTIME:
		case ATIME:
		case MTIME:
			bad_field(fp, xx);
			break;

		default:
			break;
		}
	}

	if (strcasecmp(fp->type, "\'DIR\'") == 0) {		/* directory */
		if (fp->length != initoff) {
			fprintf(stderr, "%d: length given for dir %s\n",
				recordnum-1, fp->name);
		}
	}
	else if (strcasecmp(fp->type, "\'FILE\'") == 0) {	/* regular */
		if (fp->length == initoff) {
			fprintf(stderr, "%d: length not given for %s\n",
				recordnum-1, fp->name);
		}
	}
	else if (strcasecmp(fp->type, "\'SYM\'") == 0) {	/* symlink */
		if (fp->symlink == NULL) {
			fprintf(stderr, "%d: no symlink info for %s\n",
				recordnum-1, fp->name);
		}
	}
	else if (strcasecmp(fp->type, "\'CONTAINER\'") == 0) {
		if (fp->container == NULL) {
			fprintf(stderr, "%d: no container info for %s\n",
				recordnum-1, fp->name);
		}
	}
	else {						/* bad */
		fprintf(stderr, "%d: unknown type %s for %s\n",
			recordnum-1, fp->type, fp->name);
	}

	putrec("file", finfo, sizeof(finfo));
	for (i=0; i<fp->fnum; i++) {
		flist_t		*fl = &fp->flist[i];
		namval_t	cont[] = {
			{ "container_uid", fp->uid },
			{ "name", fl->name },
			{ "uid", fl->uid }
		};

		putrec("content", cont, sizeof(cont));
	}

	if (verbose) {
		printf("uid\t\t%s\n", fp->uid);
		printf("name\t\t%s\n", fp->name);
		printf("type\t\t%s\n", fp->type);
		printf("perms\t\t%s\n", fp->perms);
		printf("owner\t\t%s\n", fp->owner);
		printf("group\t\t%s\n", fp->group);
		if (fp->length != initoff)
			printf("length\t\t%s\n", fp->length);
		printf("ctime\t\t%s\n", fp->ctime);
		printf("atime\t\t%s\n", fp->atime);
		printf("mtime\t\t%s\n", fp->mtime);
		printf("billing\t\t%s\n", fp->billing);
		printf("family\t\t%s\n", fp->family);
		printf("container\t%s\n", fp->container);
		printf("symlink\t\t%s\n", fp->symlink);
		for (i=0; i<fp->fnum; i++) {
			printf("\t%s\t\t%s\n",
				fp->flist[i].name, fp->flist[i].uid);
		}
		printf("\n");
	}

	for (i=0; i<fp->fnum; i++) {
		FREE(fp->flist[i].name);
		FREE(fp->flist[i].uid);
	}
	FREE(fp->flist);
	free_all(fp, finfo_tab);
}

/*
**	Process a fsms description.
**	Return 0 on failure.
*/
int
finfo_description()
{
	reckey_t	cmd;
	finfo_t		*finfo;
	flist_t		*flist;
	int		fnum;
	char		*content, *uid;
	static	char	container[] = "CONTAINER";
	static	char	dir[] = "DIR";

	key_tab = localtab;

	if (match_token(D_OBJ_ATTR_LIST) == 0)

		return 0;

	finfo = (finfo_t *)malloc(sizeof(finfo_t));
	memset(finfo, '\0', sizeof(finfo_t));
	init_field(finfo, finfo_tab);
	store_info(finfo, D_OBJ_ATTR_LIST, finfo_tab);

	if (!start_token())
		goto done;

	cmd = find_key();
	if (cmd == D_CONTENTS_LIST) {
		if ( !get_token(cmd, NOKEY))
			goto error;
		if (strcasecmp(token, dir) == 0)
			content = dir;
		else if (strcasecmp(token, container) == 0)
			content = container;
		else {
			fprintf(stderr,
				"%d: %s neither %s or %s present\n",
				recordnum, key_tab[cmd], dir, container);
			goto error;
		}
		if (!get_token(NOKEY, NOKEY)) {
			fprintf(stderr, "%d: %s %s UID missing\n",
				recordnum, key_tab[cmd], content);
			goto error;
		}
		uid = quotestr();
		if (strcmp(uid, finfo->uid) != 0) {
			fprintf(stderr, "%d: %s %s UID mismatch %s %s\n",
				recordnum, key_tab[cmd],
				content, finfo->uid, uid);
		}
		free(uid);
		flist = NULL;
		fnum = 0;
		if (get_token(NOKEY, NOKEY)) {
			int	maxnum = 16;

			if (strcasecmp(token, "LIST") != 0)  {
				fprintf(stderr, "%d: %s no content LIST\n",
					recordnum, key_tab[cmd]);
				goto error;
			}
			flist = (flist_t *)calloc(maxnum, sizeof(flist_t));
			while (get_token(NOKEY, NOKEY)) {
				if (fnum == maxnum) {
					maxnum *= 2;
					flist = (flist_t *)realloc(flist,
						maxnum * sizeof(flist_t));
				}
				flist[fnum].name = quotestr();
				if (!get_token(NOKEY, NOKEY)) {
					fprintf(stderr,
						"%d: %s no UID after %s\n",
						recordnum, key_tab[cmd],
						flist[fnum].name);
					goto error;
				}
				flist[fnum++].uid = quotestr();
			}
			flist = (flist_t *)realloc(flist,
					fnum * sizeof(flist_t));
		}
		finfo->fnum = fnum;
		finfo->flist = flist;

		if (!start_token())
			goto done;
	}
	else {
		finfo->fnum = 0;
		finfo->flist = NULL;
	}

	while (ext_record()) {
		if (!start_token())
			break;
	}
 done:
	finfo_save(finfo);
	return 1;

 error:
	return 0;
}
