#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<time.h>

#define	KEYDEF	"fsegs_keys.h"
#include	"com.h"

typedef struct {
	char		*cart_id;
	char		*part_num;
	char		*side_num;
	char		*tape_mark;
	char		*block;
	char		*record;
	char		*byte;
	char		*cookie;
} stripe_t;

typedef struct {
	char		*segment;
	char		*copynum;
	int		width;
	char		*block_len;
	char		*offset;
	char		*length;
	char		*header;
	char		*trailer;
	stripe_t	*stripes;
} seg_t;

field_t	seg_info_tab[] = {
    { FS_INFO,		STRIPE_WIDTH,	OFF(seg_t, width),	T_INTEGER },
    { FS_INFO,	STRIPE_BLOCK_LENGTH,	OFF(seg_t, block_len),	T_STRINTEGER },
    { FS_INFO,		OFFSET,		OFF(seg_t, offset),	T_OFFSET },
    { FS_INFO,		LENGTH,		OFF(seg_t, length),	T_OFFSET },
    { FS_HEADER,	NOKEY,		OFF(seg_t, header),	T_STRINTEGER },
    { FS_TRAILER,	NOKEY,		OFF(seg_t, trailer),	T_STRINTEGER },
    { NOKEY,		NOKEY,		0,			T_NOTYPE }
};

typedef	struct {
	char	*uid;
	char	*name;
	int	segnum;
	seg_t	*segs;
} fseg_t;

void
fseg_save(fseg_t *fp)
{
	int	i, j;
	char	num1[48];

	for (i=0; i<=fp->segnum; i++) {
		seg_t	*seg = &fp->segs[i];
		namval_t	fileseg[] = {
			{ "uid",	fp->uid },
			{ "name",	fp->name },
			{ "segment",	seg->segment },
			{ "copynum",	seg->copynum },
			{ "width",	stringnum(seg->width, num1) },
			{ "block_len",	seg->block_len },
			{ "foffset",	seg->offset },
			{ "length",	seg->length },
			{ "header",	seg->header },
			{ "trailer",	seg->trailer }
		};

		if (seg->segment == NULL) {
			fprintf(stderr,
				"%d: %s segment index %d not initialized\n",
				recordnum-1, fp->uid, i);
			continue;
		}
		if (seg->copynum == NULL) {
			fprintf(stderr,
				"%d: %s FS_NO %s FS_COPY not set\n",
				recordnum-1, fp->uid, seg->segment);
			continue;
		}

		if (seg->offset == initoff) {
			fprintf(stderr,
				"%d: %s FS_NO %s FS_COPY %s OFFSET not set\n",
				recordnum-1, fp->uid,
				seg->segment, seg->copynum);
			continue;
		}
		if (seg->length == initoff) {
			fprintf(stderr,
				"%d: %s FS_NO %s FS_COPY %s LENGTH not set\n",
				recordnum-1, fp->uid,
				seg->segment, seg->copynum);
			continue;
		}

		putrec("fileseg", fileseg, sizeof(fileseg));

		for (j=0; j<seg->width; j++) {
			stripe_t *sp = &seg->stripes[j];
			namval_t	stripe[] = {
				{ "uid",	fp->uid },
				{ "segment",	seg->segment },
				{ "copynum",	seg->copynum },
				{ "stripe",	stringnum(j, num1) },
				{ "cart_id",	sp->cart_id },
				{ "partition",	sp->part_num },
				{ "side",	sp->side_num },
				{ "tapemark",	sp->tape_mark },
				{ "block",	sp->block },
				{ "record",	sp->record },
				{ "byte",	sp->byte },
				{ "cookie",	sp->cookie }
			};

			putrec("stripe", stripe, sizeof(stripe));
		}
	}

	if (verbose) {
		printf("uid\t\t%s\n", fp->uid);
		printf("name\t\t%s\n", fp->name);
		for (i=0; i<=fp->segnum; i++) {
			seg_t	*seg = &fp->segs[i];

			printf("\tsegment\t\t%s\n", seg->segment);
			printf("\tcopynum\t\t%s\n", seg->copynum);
			printf("\twidth\t\t%d\n", seg->width);
			printf("\tblock_len\t%s\n", seg->block_len);
			printf("\toffset\t\t%s\n", seg->offset);
			printf("\tlength\t\t%s\n", seg->length);
			printf("\theader\t\t%s\n", seg->header);
			printf("\ttrailer\t\t%s\n", seg->trailer);

			for (j=0; j<seg->width; j++) {
				stripe_t *sp = &seg->stripes[j];

				printf("\t**stripe\t%d\n", j);
				printf("\t\tcart_id\t\t%s\n", sp->cart_id);
				printf("\t\tpart_num\t%s\n", sp->part_num);
				printf("\t\tside_num\t%s\n", sp->side_num);
				printf("\t\ttape_mark\t%s\n", sp->tape_mark);
				printf("\t\tblock\t\t%s\n", sp->block);
				printf("\t\trecord\t\t%s\n", sp->record);
				printf("\t\tbyte\t\t%s\n", sp->byte);
				printf("\t\tcookie\t\t%s\n", sp->cookie);
			}
		}
		printf("\n");
	}

	FREE(fp->uid);
	FREE(fp->name);
	for (i=0; i<=fp->segnum; i++) {
		seg_t	*seg = &fp->segs[i];

		FREE(seg->segment);
		FREE(seg->copynum);
		FREE(seg->block_len);
		FREE(seg->offset);
		FREE(seg->length);
		FREE(seg->header);
		FREE(seg->trailer);

		for (j=0; j<seg->width; j++) {
			stripe_t *sp = &seg->stripes[j];

			FREE(sp->cart_id);
			FREE(sp->part_num);
			FREE(sp->side_num);
			FREE(sp->tape_mark);
			FREE(sp->block);
			FREE(sp->record);
			FREE(sp->byte);
			FREE(sp->cookie);
		}
		FREE(seg->stripes);
	}
	FREE(fp->segs);
	free(fp);
}

/*
**	Process a fsms description.
**	Return 0 on failure.
*/
int
fseg_description()
{
	int		i;
	reckey_t	cmd;
	fseg_t		*fseg;
	seg_t		*seg;
	int		segmax;
	char		**nump, *num;
	char		*segnum;
	char		*copynum;
	char		*header;
	char		*trailer;

	key_tab = localtab;

	if (match_token(FS_FILE_UID) == 0)
		return 0;

	fseg = (fseg_t *)malloc(sizeof(fseg_t));
	memset(fseg, '\0', sizeof(fseg_t));

	if (!get_token(FS_FILE_UID, NOKEY))
		goto error;
	fseg->uid = quotestr();

	if (!start_token())
		goto done;

	if (match_token(FS_FILE_NAME)) {
		if (!get_token(FS_FILE_NAME, NOKEY))
			goto error;
		fseg->name = quotestr();

		if (!start_token())
			goto done;
	}

	segmax = 8;
	fseg->segs = (seg_t *)calloc(segmax, sizeof(seg_t));
	memset(fseg->segs, '\0', segmax*sizeof(seg_t));
	fseg->segnum = 0;

	segnum = NULL;
	copynum = NULL;
	header = NULL;
	trailer = NULL;

	for (;;) {
		num = NULL;

		cmd = find_key();
		switch (cmd) {

		case FS_NO:
			if (!get_token(cmd, NOKEY))
				goto error;
			if (storestrint(&num, cmd, NOKEY))
				goto error;
			nump = &segnum;
			break;

		case FS_COPY:
			if (!get_token(cmd, NOKEY))
				goto error;
			if (storestrint(&num, cmd, NOKEY))
				goto error;
			nump = &copynum;
			break;

		case FS_INFO:
			/*
			**	This is an oddball record with possibly
			**	repeated fields for multiple stripes.
			*/
			seg = &fseg->segs[fseg->segnum];
			init_field(seg, seg_info_tab);
			store_info(seg, cmd, seg_info_tab);
			if (seg->width == initint)
				seg->width = 1;		/* default value */

			seg->stripes = (stripe_t *) calloc(seg->width,
					sizeof(stripe_t));

			/*
			**	These fields must be in order.
			*/
			for (i=0; i<seg->width; i++) {
				stripe_t	*sp = &seg->stripes[i];

				if (!match_token(CART_ID))
					break;
				if (!get_token(cmd, CART_ID))
					goto error;
				sp->cart_id = quotestr();

				if (!get_token(NOKEY, NOKEY))
					continue;
				if (match_token(PARTITION)) {
					if (!get_token(cmd, PARTITION))
						goto error;
					if (savestrint(&sp->part_num, cmd,
							PARTITION))
						goto error;
					if (!get_token(NOKEY, NOKEY))
						break;
				}

				if (match_token(SIDE)) {
					if (!get_token(cmd, SIDE))
						goto error;
					if (savestrint(&sp->side_num, cmd,
							SIDE))
						goto error;
					if (!get_token(NOKEY, NOKEY))
						break;
				}

				if (match_token(TAPE_MARK)) {
					if (!get_token(cmd, TAPE_MARK))
						goto error;
					if (savestrint(&sp->tape_mark, cmd,
							TAPE_MARK))
						goto error;
					if (!get_token(NOKEY, NOKEY))
						break;
				}

				if (match_token(BLOCK)) {
					if (!get_token(cmd, BLOCK))
						goto error;
					if (savestrint(&sp->block, cmd, BLOCK))
						goto error;
					if (!get_token(NOKEY, NOKEY))
						break;
				}

				if (match_token(RECORD)) {
					if (!get_token(cmd, RECORD))
						goto error;
					if (savestrint(&sp->record, cmd,
							RECORD))
						goto error;
					if (!get_token(NOKEY, NOKEY))
						break;
				}

				if (match_token(BYTE)) {
					if (!get_token(cmd, BYTE))
						goto error;
					if (savestrint(&sp->byte, cmd, BYTE))
						goto error;
					if (!get_token(NOKEY, NOKEY))
						break;
				}

				if (match_token(COOKIE)) {
					if (!get_token(cmd, COOKIE))
						goto error;
					sp->cookie = quotestr();
					if (!get_token(NOKEY, NOKEY))
						break;
				}
			}
			if (tokenlen)
				store_info(seg, cmd, seg_info_tab);
			break;

		case FS_HEADER:
			if (!get_token(cmd, NOKEY))
				goto error;
			if (storestrint(&num, cmd, NOKEY))
				goto error;
			nump = &header;
			break;

		case FS_TRAILER:
			if (!get_token(cmd, NOKEY))
				goto error;
			if (storestrint(&num, cmd, NOKEY))
				goto error;
			nump = &trailer;
			break;

		default:
			if (ext_record())
				break;
			goto done;
		}

		if (num != NULL) {
			if (*nump != NULL) {	/* need a new segment */
				if (fseg->segnum == segmax) {
					segmax *= 2;
					fseg->segs = realloc(fseg->segs,
							segmax*sizeof(seg_t));
				}
				fseg->segs[fseg->segnum].segment = segnum;
				fseg->segs[fseg->segnum].copynum = copynum;
				fseg->segs[fseg->segnum].header = header;
				fseg->segs[fseg->segnum].trailer = trailer;

				fseg->segnum++;
				segnum = NULL;
				copynum = NULL;
				header = NULL;
				trailer = NULL;
			}
			*nump = num;
		}
		if (!start_token())
			break;
	}

 done:
	fseg->segs[fseg->segnum].segment = segnum;
	fseg->segs[fseg->segnum].copynum = copynum;
	fseg->segs[fseg->segnum].header = header;
	fseg->segs[fseg->segnum].trailer = trailer;

	fseg->segs = realloc(fseg->segs, (fseg->segnum+1)*sizeof(seg_t));
	fseg_save(fseg);
	return 1;

 error:
	return 0;
}
