#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<time.h>

#define	KEYDEF	"fsms_keys.h"
#include	"com.h"

typedef	struct {
	char	*version;
	char	*hw_vendor;
	char	*hw_name;
	char	*hw_version;
	char	*os_vendor;
	char	*os_name;
	char	*os_version;
	char	*export_time;
	char	*fsms_vendor;
	char	*fsms_name;
	char	*fsms_version;
	int	num_root;
	char	**export_root;
} fsms_t;

field_t	fsms_tab[] = {
    { EXPORT_VERSION,		NOKEY,	OFF(fsms_t, version),	T_STRING },
    { EXPORT_HARDWARE_SYSTEM,	VENDOR,	OFF(fsms_t, hw_vendor),	T_STRING },
    { EXPORT_HARDWARE_SYSTEM,	NAME,	OFF(fsms_t, hw_name),	T_STRING },
    { EXPORT_HARDWARE_SYSTEM,	VERSION,OFF(fsms_t, hw_version),T_STRING },
    { EXPORT_OS,		VENDOR,	OFF(fsms_t, os_vendor),	T_STRING },
    { EXPORT_OS,		NAME,	OFF(fsms_t, os_name),	T_STRING },
    { EXPORT_OS,		VERSION,OFF(fsms_t, os_version),T_STRING },
    { EXPORT_TIME,		NOKEY,	OFF(fsms_t, export_time),T_DATE },
    { EXPORT_FSMS,		VENDOR,	OFF(fsms_t, fsms_vendor),T_STRING },
    { EXPORT_FSMS,		FSMS,	OFF(fsms_t, fsms_name),	T_STRING },
    { EXPORT_FSMS,		VERSION,OFF(fsms_t, fsms_version),T_STRING },
    { NOKEY,			NOKEY,	0,			T_NOTYPE }
};

void
fsms_save(fsms_t *fp)
{
	field_t		*xx;
	int		i;

	/* all fields are required */
	for (xx = fsms_tab; xx->record != NOKEY; xx++)
		bad_field(fp, xx);
	if (fp->export_root[0] == NULL ) {
		fprintf(stderr, "%d: %s not set\n", recordnum,
			key_tab[EXPORT_ROOT]);
	}

	for (i=0; i<fp->num_root; i++) {
		namval_t	fsms[] = {
			{ "version", 		fp->version },
			{ "hw_vendor",		fp->hw_vendor },
			{ "hw_name",		fp->hw_name },
			{ "hw_version",		fp->hw_version },
			{ "os_vendor",		fp->os_vendor },
			{ "os_name",		fp->os_name },
			{ "os_version",		fp->os_version },
			{ "export_time",	fp->export_time },
			{ "fsms_vendor",	fp->fsms_vendor },
			{ "fsms_name",		fp->fsms_name },
			{ "fsms_version",	fp->fsms_version },
			{ "export_root",	fp->export_root[i] }
		};
		putrec("fsms", fsms, sizeof(fsms));
	}

	if (verbose) {
		printf("version\t\t%s\n", fp->version);
		printf("hw_vendor\t%s\n", fp->hw_vendor);
		printf("hw_name\t\t%s\n", fp->hw_name);
		printf("hw_version\t%s\n", fp->hw_version);
		printf("os_vendor\t%s\n", fp->os_vendor);
		printf("os_name\t\t%s\n", fp->os_name);
		printf("os_version\t%s\n", fp->os_version);
		printf("export_time\t%s\n", fp->export_time);
		printf("fsms_vendor\t%s\n", fp->fsms_vendor);
		printf("fsms_name\t%s\n", fp->fsms_name);
		printf("fsms_version\t%s\n", fp->fsms_version);
		printf("export_root\n");
		for (i=0; i<fp->num_root; i++)
			printf("\t%s\n", fp->export_root[i]);
		printf("\n");
	}

	for (i=0; i<fp->num_root; i++)
		free(fp->export_root[i]);
	FREE(fp->export_root);
	free_all(fp, fsms_tab);
}

/*
**	Process a fsms description.
**	Return 0 on failure.
*/
int
fsms_description()
{
	reckey_t	cmd;
	fsms_t		*fsms;
	int		i, maxroot;

	key_tab = localtab;

	if (match_token(EXPORT_VERSION) == 0)
		return 0;

	if (get_token(EXPORT_VERSION, NOKEY) == 0)
		return 0;

	fsms = (fsms_t *)malloc(sizeof(fsms_t));
	memset(fsms, 0, sizeof(fsms_t));
	init_field(fsms, fsms_tab);

	fsms->num_root = maxroot = 1;
	fsms->export_root = (char **)calloc(maxroot, sizeof(char **));

	storestr(&fsms->version, EXPORT_VERSION, NOKEY);

	while (start_token()) {
		cmd = find_key();
		switch (cmd) {

		case EXPORT_HARDWARE_SYSTEM:
		case EXPORT_OS:
		case EXPORT_FSMS:
			store_info(fsms, cmd, fsms_tab);
			break;

		case EXPORT_TIME:
			if (get_token(cmd, NOKEY))
				storedate(&fsms->export_time, cmd, NOKEY);
			break;

		case EXPORT_ROOT:
			if (fsms->export_root[0] != NULL ) {
				fprintf(stderr,
					"%d: EXPORT_ROOT already assigned\n",
					recordnum);
				break;
			}

			for (i=0; get_token(NOKEY, NOKEY); i++) {
				if (tokeneol)
					break;
				if (i == maxroot) {
					maxroot *= 2;
					fsms->export_root =
						realloc(fsms->export_root,
						maxroot*sizeof(char *));
				}
				fsms->export_root[i] = quotestr();
			}

			if (i == 0) {
				fprintf(stderr,
					"%d: no fields for EXPORT_ROOT\n",
					recordnum);
				break;
			}

			fsms->num_root = i;
			fsms->export_root = realloc(fsms->export_root,
				fsms->num_root * sizeof(char *));
			break;

		default:
			if (ext_record())
				break;
			goto done;
		}
	}
 done:
	fsms_save(fsms);
	return 1;
}
