/*	$NetBSD: dmfs.h,v 1.9 1998/12/29 09:32:31 waddell Exp $	*/

/*
 * Copyright (c) 1992, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software donated to Berkeley by
 * Jan-Simon Pendry.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	from: Id: lofs.h,v 1.8 1992/05/30 10:05:43 jsp Exp
 *	@(#)dmfs.h	8.2 (Berkeley) 1/21/94
 */

#ifndef	_DMFS_H_
#define	_DMFS_H_

#include <miscfs/dmfs/rash.h>
#include <miscfs/genfs/layer.h>

struct dm_args {
	struct	layer_args	la;	/* generic layerfs args */
};
#define	dmfsa_target	la.target
#define	dmfsa_export	la.export

#define	DM_RFINISHING	0x00000002	/* Did FINISHRESTORE, awaiting close */
#define	DM_UNLINKED	0x00000004	/* unlinked, RASH_OBSOLITE on inactive*/
#define DM_MODIFIED	0x00000008
#define DM_IOWANT	0x00000010
#define DM_RESTOREFAIL	0x00000020
#define	DM_COMPACT	0x00000080
#define	DM_ARCH		0x00000100
#define DM_NONR		0x00000200
#define	DM_AIP		0x00000400
#define	DM_RIP		0x00000800
#define	DM_BIP		0x00001000	/* We are the backup control channel */
#define	DM_HASMETA	0x00008000

#define DM_FLAGMASK	(DM_ARCH | DM_NONR | DM_RESTOREFAIL | DM_COMPACT | \
			DM_HASMETA)

/* The following flags are needed due to how VOP_UPDATE works... */
#define	DM_UPDATE_MTIME	0x20000000
#define	DM_UPDATE_ATIME	0x40000000
#define	DM_UPDATE_CTIME	0x80000000

/*
 * DM_MODIFIED	- dm_node's info changed
 * DM_IOWANT	- a read or write is blocked trying to access data
 *		    beyond block boundry. Or a process is sleeping for
 *		    for the end of a restore.
 *
 * Following flags saved to disk:
 *
 * DM_ARCH	- Archived to tape since last modification
 * DM_NONR	- Not fully resident. Enables blocking of access beyond
 *		    dm_bbarrier. Note dm_bbarrier is the first byte NOT
 *		    resident. So a value of 0, with this bit set,
 *		    indicates that NONE of the underlying file is resident.
 * DM_RESTOREFAIL - A restore attemp failed. Any sleeping io will return
 *		    EIO.
 *
 * Other flags:
 *
 * DM_AIP	- Archive in progress. Any writes will clear this bit, which
 *		    will inform the archiver that the archive is invalid.
 * DM_RIP	- Restore in progress. Accesses which block do not need to
 *		    generate a new request to userland.
 *
 * Flags infered from opaque data magic value:
 * DM_HASMETA, DM_MOUNTMETA, and DM_OTHERMETA indicate the results of
 * getting the meta data from the underlying layer. DM_HASMETA indicates
 * our node metadata was found in the lower node, DM_MOUNTMETA indicates
 * mount-point meta data was found in the lower node, and DM_OTHERMETA
 * indicates that some other type of metadata was found.
 */

struct	dmm_inode_locks {
	long	start, end;
};

struct	dmfs_md_ext {
	ino_t	start;
	int	count;
	void	*buf;
};

#ifdef _KERNEL

struct dmfs_mount_data;			/* see dmfs_mdata for def. */

struct dm_mount {
	struct	layer_mount	lm;	/* generic layer mountpoint stuff */
	struct vnode		*dmm_mnode;	/* vnode for mfile */
	struct ucred		*dmm_ucred;	/* our credentials */
	struct dmfs_mount_data	*dmm_mdata;	/* mount meta data */
	pid_t			dmm_bkpid;	/* pid of proc backing up fs */
	long			dmm_bk_start;	/* When backing up, start */
	long			dmm_bk_end;	/* and end of locked inode
						 * range. */
};
#define	dmm_vfs			lm.layerm_vfs
#define	dmm_rootvp		lm.layerm_rootvp
#define	dmm_export		lm.layerm_export
#define	dmm_flags		lm.layerm_flags
#define	dmm_size		lm.layerm_size
#define	dmm_tag			lm.layerm_tag
#define	dmm_bypass		lm.layerm_bypass
#define	dmm_alloc		lm.layerm_alloc
#define	dmm_vnodeop_p		lm.layerm_vnodeop_p
#define	dmm_node_hashtbl	lm.layerm_node_hashtbl
#define	dmm_node_hash		lm.layerm_node_hash
#define	dmm_hashlock		lm.layerm_hashlock

/* we have flags 0xfffff000 available */
#define	DMM_SWAPMD		0x00001000	/* byteswap metadata entries */
#define	DMM_METARO		0x00002000	/* metadata is not writable */
#define	DMM_SYNCMDATA		0x00004000	/* write mdata with IO_SYNC */
#define	DMM_BACKUP		0x00008000	/* backup in progress */
#define	DMM_NODES_LOCKED	0x00010000	/* dmm_bk_{start,end} refer
						 * to locked nodes */
#define	DMM_NODES_WANTED	0x00020000	/* sleeper waiting because of
						 * node locks */
#define	DMM_METALOCKED		0x00100000	/* mfile is locked */

/*
 * A cache of vnode references
 */
struct dm_node {
	struct	layer_node	ln;		/* generic layer stuff */
	u_int32_t		dm_flags;	/* locking, etc. */
	u_int32_t		dm_dbuid;	/* Database ID # */
			/* Following two values only valid iff DM_NONR set */
	off_t			dm_archsize;	/* Archived file size */
	off_t			dm_bbarrier;	/* blocking barrier */
				/* byte number of first non-res byte */
				/* == size in bytes of underlying file */
	bfid_t			dm_bfid;	/* File's BFID */
			/* if DM_MOUNTMETA, atime fields hold the mount fsid */
	int32_t			dm_atime;	/* apparent atime */
	int32_t			dm_atimensec;	/* apparent atime */
	int32_t			dm_mtime;	/* apparent mtime */
	int32_t			dm_mtimensec;	/* apparent mtime */
	int32_t			dm_rtime;	/* time of last restore */
	int32_t			dm_artime;	/* time of last archive */
	pid_t			dm_restorepid;	/* pid of restore process */
	pid_t			dm_archpid;	/* pid of archive process */
	u_int32_t		dm_gen;		/* generation # of file */
	long			dm_filenum;	/* inode # of file */
};
#define	dm_hash		ln.layer_hash
#define	dm_lowervp	ln.layer_lowervp
#define	dm_vnode	ln.layer_vnode
#define	dm_layerflags	ln.layer_flags

/* we can't use layer_flags for much as we need too many bits */


#define	MOUNTTODMMOUNT(mp) ((struct dm_mount *)((mp)->mnt_data))
#define	VTODM(vp) ((struct dm_node *)(vp)->v_data)
#define	DMTOV(xp) ((xp)->dm_vnode)
#ifdef DMFS_DIAGNOSTIC
extern struct vnode *dm_checkvp __P((struct vnode *vp, char *fil, int lno));
#define	DMVPTOLOWERVP(vp) dm_checkvp((vp), __FILE__, __LINE__)
#else
#define	DMVPTOLOWERVP(vp) (VTODM(vp)->dm_lowervp)
#endif

extern int (**dm_vnodeop_p) __P((void *));
extern struct vfsops dmfs_vfsops;

#define	SET(a,b)	((a) |= (b))
#define CLR(a,b)	((a) &= ~(b))
#define	ISSET(a,b)	((a) & (b))

#endif /* _KERNEL */
#endif /* _DMFS_H_ */
