/*	$NetBSD: $ */


#include "miscfs/dmfs/dmfs_mdata.h"

#ifdef DMFS_DIAGNOSTIC
extern struct vnode *dm_checkvp __P((struct vnode *vp, char *fil, int lno));
#define	DMVPTOLOWERVP(vp) dm_checkvp((vp), __FILE__, __LINE__)
#else
#define	DMVPTOLOWERVP(vp) (VTODM(vp)->dm_lowervp)
#endif

/* dmfs_io.c */

int  dm_read		__P((void *));
int  dm_write		__P((void *));
int  dm_bmap		__P((void *));
int  dmfs_callrash	__P((struct vnode *, uid_t));
int  dmfs_callwait	__P((struct vnode *, uid_t));
int  dmfs_ioblock	__P((struct vnode *, struct uio *, int, int, uid_t));
int  dmfs_rashdelete	__P((struct vnode *, uid_t, struct dmfs_mdata *, int));
int  dmfs_sendmessage	__P((struct vnode *, u_int32_t, dev_t, uid_t));
int  dmfs_wait		__P((struct vnode *));
void dmfs_wakeup	__P((struct vnode *));

/* dmfs_subr.c */

int dm_node_alloc __P((struct mount *mp, struct vnode *target, struct vnode **vpp));

int  dmfs_mount_mfile __P((struct vnode *vp, struct mount *mp));
		/*
		 * Takes vp and validates it as the mfile for mount
		 * point dmp.
		 *
		 * Will validate mfile contents, size, and version numbers.
		 * Also will reload meta data for all vnodes mounted on dmp.
		 */

int  dmfs_read_mdata __P((struct vnode *, int islocked));
		/*
		 * Read the metadata for the given inode from the mfile.
		 *
		 * The mfile vnode is known to be locked iff islocked is set.
		 *
		 * If the mfile is not yet available, just null the mdata.
		 * if the mfile is available, make sure that the node
		 * generation # matches that in the mdata. If not, generate
		 * an obsolecence error, and clear the mdata.
		 */
int  dmfs_write_mdata __P((struct vnode *, int islocked));
		/*
		 * Write the metadata for the given node to the mfile.
		 *
		 * The mfile vnode is known to be locked with LK_EXCLUSIVE
		 * iff islocked is set.
		 *
		 * If the mfile is not available or the fs is mounted r/o,
		 * return EROFS.
		 */

void dmfs_nullmeta __P((struct dm_node *));	/* null out internal data */
void dmfs_byteswap_meta __P((struct dmfs_mdata *));
void dmfs_expmeta __P((struct vnode *, struct dmfs_mdata *));
		/* Export metadata of vnode to metadata buffer. */

void dmfs_blankmeta __P((struct dmfs_mdata *));
		/* Clear out things not set in dmfs_expmeta */

void dmfs_impmeta __P((struct vnode *, struct dmfs_mdata *));
		/* import metadata for vnode from metadata buffer. The
		 * inode generation number does not need to match. */

/* dmfs_fcntl.c */

int	dm_fcntl	__P((void *));
int	dmfs_restorefail __P((struct vnode *, struct ucred *, struct proc *));

void	dmfs_dosync __P((struct mount *));
		/* does meat of sync, with mfile node locked */

/* dmfs_vnode.c */

#define dm_bypass	layer_bypass

int	dm_getattr	__P((void *));
int	dm_setattr	__P((void *));
int	dm_update	__P((void *));
int	dm_inactive	__P((void *));
int	dm_reclaim	__P((void *));
int	dm_print	__P((void *));
int	dm_strategy	__P((void *));
int	dm_bwrite	__P((void *));
int	dm_lock		__P((void *));
int	dm_unlock	__P((void *));
int	dm_islocked	__P((void *));
int	dm_lookup	__P((void *));
int	dm_access	__P((void *));
int	dm_truncate	__P((void *));
int	dm_update	__P((void *));

int	dm_open		__P((void *));
int	dm_close	__P((void *));
int	dm_revoke	__P((void *));

int	dm_create	__P((void *));
int	dm_link		__P((void *));
int	dm_mkdir	__P((void *));
int	dm_mknod	__P((void *));
int	dm_remove	__P((void *));
int	dm_rename	__P((void *));
int	dm_rmdir	__P((void *));
int	dm_symlink	__P((void *));

int	dmfs_backup	__P((struct vnode *, int, int, struct proc *));
int	dmfs_node_wait	__P((struct vnode *));
