#include <sys/param.h>
#include <sys/systm.h>
#include <sys/proc.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/kernel.h>
#include <sys/vnode.h>
#include <sys/fcntl.h>
#include <miscfs/dmfs/dmfs.h>
#include <miscfs/genfs/layer_extern.h>
#include <miscfs/dmfs/dmfs_extern.h>
#include <miscfs/dmfs/dmfs_fcntl.h>

extern	struct timespec	dmfs_nulltime;

int
dm_fcntl(v)
	void *v;
{
	struct vop_fcntl_args /* {
		struct vnode *a_vp;
		u_int a_command;
		caddr_t a_data;
		int a_fflag;
		struct ucred *a_cred;
		struct proc *a_p;
	} */ *ap = v;
	struct	vnode *vp = ap->a_vp;
	struct	dm_node *dp = VTODM(vp);
	void	*data 	= ap->a_data;
	struct	dmfs_mdata *dm = data;
	struct	rashstat *ri = data;
	struct	timespec ts;
	struct	vattr	va;
	int	error;

	error = 0;
	TIMEVAL_TO_TIMESPEC(&time, &ts);

	switch (ap->a_command) {
	case DMFS_GETMETA:
		dmfs_blankmeta(dm);
		dmfs_expmeta(vp, dm);
		break;

	case DMFS_SETMETA:
	case DMFS_CLRMETA:
		if (!ISSET(ap->a_fflag, FWRITE)) {
			error = EACCES;
			break;
		}
		if (ISSET(dp->dm_flags, DM_HASMETA) &&
				ISSET(dp->dm_flags, DM_AIP | DM_RIP)) {
			error = EBUSY;
			break;
		}
		if (ap->a_command == DMFS_SETMETA) {
			dmfs_impmeta(vp, dm);
		} else {
			int f,i,j;
			f = dp->dm_flags;
			i = dp->dm_gen;
			j = dp->dm_filenum;
			dmfs_nullmeta(dp);
			dp->dm_gen = i;
			dp->dm_filenum = j;
			SET(dp->dm_flags, f & DM_IOWANT);
		}
		if ISSET(dp->dm_flags, DM_IOWANT)
			dmfs_wakeup(vp);
		error = dmfs_write_mdata(vp, 0);
		break;

	case DMFS_SETARC:
		if (ISSET(dp->dm_flags, DM_ARCH)) {
			error = EINVAL;
		} else {
			SET(dp->dm_flags, DM_ARCH | DM_MODIFIED);
			error = VOP_UPDATE(vp, &ts, &ts, 0);
		}
		break;

	case DMFS_CLRARC:
		if (ISSET(dp->dm_flags, DM_ARCH)) {
			CLR(dp->dm_flags, DM_ARCH);
			SET(dp->dm_flags, DM_MODIFIED);
			error = VOP_UPDATE(vp, &ts, &ts, 0);
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_SETAIP:
		if (vp->v_type != VREG) {
			error = EINVAL;
			break;
		}
		if (ISSET(dp->dm_flags, DM_AIP)) {
			error = EINVAL;
		} else {
			dp->dm_archpid = ap->a_p->p_pid;
			SET(dp->dm_flags, DM_AIP | DM_MODIFIED);
			error = VOP_UPDATE(vp, &ts, &ts, 0);
		}
		break;

	case DMFS_CLRAIP:
		if (ISSET(dp->dm_flags, DM_AIP)) {
			CLR(dp->dm_flags, DM_AIP);
			error = 0;
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_SETCOMPACT:
		if (ISSET(dp->dm_flags, DM_COMPACT)) {
			error = EINVAL;
		} else {
			SET(dp->dm_flags, DM_COMPACT);
			error = 0;
		}
		break;

	case DMFS_CLRCOMPACT:
		if (ISSET(dp->dm_flags, DM_COMPACT)) {
			CLR(dp->dm_flags, DM_COMPACT);
			error = 0;
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_CLRNONR:
		if (ISSET(dp->dm_flags, DM_NONR)) {
			CLR(dp->dm_flags, DM_NONR);
			SET(dp->dm_flags, DM_MODIFIED);
			error = VOP_UPDATE(vp, &ts, &ts, 0);
			if (error == 0) {
				dmfs_wakeup(vp);
			}
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_SETRIP:
		if (dp->dm_restorepid != 0) {
			error = EINVAL;
			break;
		}
		dp->dm_restorepid = ap->a_p->p_pid;
		if (!ISSET(dp->dm_flags, DM_RIP)) {
			SET(dp->dm_flags, DM_RIP);
		}
		CLR(dp->dm_flags,DM_RESTOREFAIL);
		error = 0;
		break;

	case DMFS_CLRRIP:
		if (dp->dm_restorepid != ap->a_p->p_pid) {
			error = EINVAL;
			break;
		}
		if (ISSET(dp->dm_flags, DM_RIP)) {
			CLR(dp->dm_flags, DM_RIP);
			dp->dm_restorepid = 0;
			error = 0;
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_GETRINFO:
		ri->st_dbuid    = dp->dm_dbuid;
		ri->st_rashflg  = dp->dm_flags;
		ri->st_bfid     = dp->dm_bfid;
		ri->st_bbarrier = dp->dm_bbarrier;
		break;

	case DMFS_SETRINFO:
		CLR(dp->dm_flags, DM_FLAGMASK);
		dp->dm_flags    |= (ri->st_rashflg & DM_FLAGMASK);
		dp->dm_dbuid    = ri->st_dbuid;
		dp->dm_bfid     = ri->st_bfid;
		dp->dm_bbarrier = ri->st_bbarrier;
		SET(dp->dm_flags, DM_MODIFIED);
		dmfs_wakeup(vp);
		error = VOP_UPDATE(vp, &ts, &ts, 0);
		break;

	case DMFS_SETAPSIZE:
		if (ISSET(dp->dm_flags, DM_NONR)) {
			SET(dp->dm_flags, DM_MODIFIED);
			if ((*(off_t *)data) != dp->dm_archsize) {
				dp->dm_archsize = *(off_t *)data;
			}
			error = VOP_UPDATE(vp, &ts, &ts, 0);
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_SETNONR:
	case DMFS_MOVENONR:
		if (ap->a_command == DMFS_SETNONR) {
			if (ISSET(dp->dm_flags, DM_NONR) ||
					!ISSET(dp->dm_flags, DM_ARCH) ||
					ISSET(dp->dm_flags, DM_RIP)) {
				error = EINVAL;
				break;
			}
		} else {
			if (!ISSET(dp->dm_flags, DM_NONR) ||
					ISSET(dp->dm_flags, DM_RIP)) {
				error = EINVAL;
				break;
			}
		}
		error = VOP_GETATTR(dp->dm_lowervp, &va, ap->a_cred, ap->a_p);
		if (error)
			break;
		if (va.va_size <= *(off_t *)data)
			break;
		error = VOP_TRUNCATE(dp->dm_lowervp, *(off_t *)data, 0,
					ap->a_cred, ap->a_p);
		if (error)
			break;
		/* Only DMFS_SETNONR adjusts the reported file size */
		if (!ISSET(dp->dm_flags, DM_NONR))
			dp->dm_archsize = va.va_size;
		SET(dp->dm_flags, DM_NONR);
		/* FALLTHROUGH */

	case DMFS_SETBBOUND:
		if (!ISSET(dp->dm_flags, DM_NONR)) {
			error = EINVAL;
			break;
		}
		SET(dp->dm_flags, DM_MODIFIED);
		dp->dm_bbarrier = *(off_t *)data;
		dmfs_wakeup(vp);
		error = VOP_UPDATE(vp, &ts, &ts, 0);
		break;

	case DMFS_FINISHARCH:
		if (ISSET(dp->dm_flags, DM_AIP)) {
			CLR(dp->dm_flags, DM_AIP);
			SET(dp->dm_flags, DM_ARCH | DM_MODIFIED);
			dp->dm_artime = time.tv_sec;
			error = VOP_UPDATE(vp, &ts, &ts, 0);
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_SETRSTTIME:
		if (ISSET(dp->dm_flags, DM_ARCH)) {
			SET(dp->dm_flags, DM_MODIFIED);
			dp->dm_rtime = time.tv_sec;
			error = VOP_UPDATE(vp, &ts, &ts, 0);
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_FINISHRESTORE:
		if (ISSET(dp->dm_flags, DM_RESTOREFAIL)) {
			error = EINVAL;
		} else {
			if ((dp->dm_restorepid != ap->a_p->p_pid) ||
					(!ISSET(dp->dm_flags, DM_RIP))) {
				error = EINVAL;
				break;
			}
			/* Wake up any sleepers. */
			SET(dp->dm_flags, DM_MODIFIED | DM_RFINISHING);
			dp->dm_rtime = time.tv_sec;
			error = VOP_UPDATE(vp, &ts, &ts, 0);
		}
		break;

	case DMFS_SETRFAIL:
		if (ISSET(dp->dm_flags, DM_RESTOREFAIL)) {
			error = EINVAL;
		} else {
			if ((dp->dm_restorepid != ap->a_p->p_pid) ||
					(!ISSET(dp->dm_flags, DM_RIP))) {
				error = EINVAL;
				break;
			}
			error = dmfs_restorefail(vp, ap->a_cred, ap->a_p);
		}
		break;

	case DMFS_CLRRFAIL:
		if (ISSET(dp->dm_flags, DM_RESTOREFAIL)) {
			CLR(dp->dm_flags, DM_RESTOREFAIL);
			SET(dp->dm_flags, DM_MODIFIED);
			error = VOP_UPDATE(vp, &ts, &ts, 0);
		} else {
			error = EINVAL;
		}
		break;

	case DMFS_LOWERSTAT:
		error = VOP_GETATTR(dp->dm_lowervp, data,ap->a_cred, ap->a_p);
		break;

	case DMFS_SETINODELK:
		if ((error = suser(ap->a_cred, NULL)) == 0) {
		    struct dmm_inode_locks *dmlp = data;
		    error = dmfs_backup(vp, dmlp->start, dmlp->end, ap->a_p);
		}
		break;

	case DMFS_SETBIP:
		{
			struct mount	*mp = vp->v_mount;
			struct dm_mount	*dmp = MOUNTTODMMOUNT(mp);

			if ((error = vfs_busy(mp, 0, NULL)))
				break;
			if (ISSET(dmp->dmm_flags, DMM_BACKUP) ||
					ISSET(dp->dm_flags, DM_BIP)) {
				error = EINVAL;
			} else if ((error = suser(ap->a_cred, NULL)) == 0) {
				SET(dmp->dmm_flags, DMM_BACKUP);
				SET(dp->dm_flags, DM_BIP);
				CLR(dmp->dmm_flags,
					DMM_NODES_LOCKED | DMM_NODES_WANTED);
				dmp->dmm_bkpid = ap->a_p->p_pid;
			} else {
			}

			vfs_unbusy(mp);
		}
		break;

	case DMFS_CLRBIP:
		{
			struct mount	*mp = vp->v_mount;
			struct dm_mount	*dmp = MOUNTTODMMOUNT(mp);

			if ((error = vfs_busy(mp, 0, NULL)))
				break;
			if (!ISSET(dmp->dmm_flags, DMM_BACKUP) ||
					!ISSET(dp->dm_flags, DM_BIP)) {
				error = EINVAL;
			} else if (ap->a_p->p_pid == dmp->dmm_bkpid) {
				if (ISSET(dmp->dmm_flags, DMM_NODES_WANTED))
					wakeup(dmp);
				CLR(dmp->dmm_flags, DMM_BACKUP |
					DMM_NODES_LOCKED | DMM_NODES_WANTED);
				CLR(dp->dm_flags, DM_BIP);
			} else {
				error = EINVAL;
			}

			vfs_unbusy(mp);
		}
		break;

	default:
		error = dm_bypass(ap);
		break;
	}

	return (error);
}

/*
 * dmfs_restorefail: mark an in-progress restore as failed.
 *
 * vp must be locked.
 */
int
dmfs_restorefail(vp, ucred, p)
	struct vnode *vp;
	struct ucred *ucred;
	struct proc  *p;
{
	struct dm_node *dp = VTODM(vp);
	struct timespec ts;

	CLR(dp->dm_flags, DM_RIP);
	SET(dp->dm_flags, DM_RESTOREFAIL | DM_MODIFIED);

	VOP_TRUNCATE(dp->dm_lowervp, dp->dm_bbarrier, 0, ucred, p);

	dmfs_wakeup(vp);

	TIMEVAL_TO_TIMESPEC(&time, &ts);
	return (VOP_UPDATE(vp, &ts, &ts, 1));
}
