/*
 * Extended ops used for dmfs operations.
 */

#ifndef _DMFS_FCNTL_H_
#define _DMFS_FCNTL_H_

#define	_DMFS_O(c)	_FCNO_FSSPEC(c)
#define	_DMFS_R(c,t)	_FCNR_FSSPEC(c,t)
#define _DMFS_W(c,t)	_FCNW_FSSPEC(c,t)
#define	_DMFS_RW(c,t)	_FCNRW_FSSPEC(c,t)

#define	DMFS_GETRINFO	_DMFS_W(0, struct rashstat)
					/* Get bfid, dbuid, flags, etc. */
#define	DMFS_SETRINFO	_DMFS_R(1, struct rashstat)
					/* Set bfid, dbuid, flags, etc. */
#define	DMFS_SETAPSIZE	_DMFS_R(2, off_t)
					/* Set apparent size */
#define	DMFS_SETBBOUND	_DMFS_R(3, off_t)
					/* Set r/w-block boundry */
#define	DMFS_SETNONR	_DMFS_R(4, off_t)
			/* Set non-res flag, truncate to passed length,
			 * set apparent size to present size, and set byte
			 * boundry accordingly. */
#define DMFS_FINISHARCH	_DMFS_O(5)
			/* If AIP bit set, clear AIP bit, set archive bit,
			   and set archive time */
#define DMFS_SETRSTTIME _DMFS_O(6)
			/* If archive bit set, set restore time */
#define	DMFS_FINISHRESTORE _DMFS_O(7)
			/* If we are the restore process & restore successful,
			 * mark the file as resident & clear RIP */
#define	DMFS_LOWERSTAT	_DMFS_W(8, struct vattr) /* stat underlying file */
#define	DMFS_MOVENONR	_DMFS_R(9, off_t)
			/* Truncate already-non-resident file to
			 * specified size */
#define	DMFS_SETMETA	_DMFS_R(10, struct dmfs_mdata)	/* set meta info */
#define	DMFS_GETMETA	_DMFS_W(11, struct dmfs_mdata)	/* get meta info */
#define	DMFS_CLRMETA	_DMFS_O(12)			/* clear meta info */

#define	DMFS_SETINODELK	_DMFS_R(14, struct dmm_inode_locks)
			/* Set inode lock range */
/* Define start & end pair to represent null range */
#define	DMFS_INODELK_NULLSTART	1
#define	DMFS_INODELK_NULLEND	0

#define	DMFS_EXTRACT_MDATA0	_DMFS_R(13, struct dmfs_md_ext)

/*
 * Following commands will return an error if desired flag is already in
 * the requested state. Typically such errors indicate: for SET, that
 * another process has begun servicing the request, for CLR, that
 * other, user actions have cleared the flag already.
 *
 * AIP is cleared by a write to a file.
 */
#define	DMFS_SETARC	_DMFS_O(16)	/* Set archive bit */
#define	DMFS_CLRARC	_DMFS_O(17)	/* Clear archive bit */
#define	DMFS_SETRIP	_DMFS_O(18)	/* Set restore-in-progress bit */
#define	DMFS_CLRRIP	_DMFS_O(19)	/* Clear restore-in-progress bit */
#define	DMFS_SETAIP	_DMFS_O(20)	/* Set archive-in-progress bit */
#define	DMFS_CLRAIP	_DMFS_O(21)	/* Clear archive-in-progress bit */
#define	DMFS_SETRFAIL	_DMFS_O(22)	/* Set restore-failed bit */
#define	DMFS_CLRRFAIL	_DMFS_O(23)	/* Clear restore-failed bit */

#define	DMFS_CLRNONR	_DMFS_O(24)	/* Clear non-resident bit */

#define	DMFS_SETCOMPACT	_DMFS_O(25)	/* Set compact bit */
#define	DMFS_CLRCOMPACT	_DMFS_O(26)	/* Clear compact bit */

#define	DMFS_SETBIP	_DMFS_O(27)	/* Start a backup, and set this
					 * vnode as the backup control node */
#define	DMFS_CLRBIP	_DMFS_O(28)	/* End backup session */

#endif /* _DMFS_FCNTL_H_ */
