#ifndef DMFS_MDATA
#define DMFS_MDATA 1

/*
 * The dmfs metadata file consists of one struct dmfs_mount_mdata
 * followed by as many struct dmfs_mdata records as there are
 * inodes in the underlying filestore. The exact sizes for the mount and
 * node metadata are taken from entries in the mount metadata.
 *
 * 
 */

/*
 * Mount point metadata
 *
 * The magic number is used to determine the endianness of the entries in
 * the file and the validity of the file. The rest of the mount info
 * provides information about the layout of the mfile and about the
 * environment where the fs will mount.
 *
 * The mount info contains the size both of the mount info and of the
 * node info, and version numbers for them. The mount info also includes
 * the fsid which should be used for the mount and the number of files
 * (inodes) in the underlying filestore.
 *
 * The fsid is preserved so that userland processes can ensure that a given
 * dmfs layer mounts with a known fsid. 
 */

struct dmfs_mount_data {
	u_int32_t	dmmnt_magic;		/*  0: magic number */
	u_int16_t	dmmnt_mntsize;		/*  4: size of mount data */
	u_int16_t	dmmnt_mntvers;		/*  6: version of mount data */
	u_int16_t	dmmnt_nodesize;		/*  8: size of node data */
	u_int16_t	dmmnt_nodevers;		/* 10: version of node data */
	u_int32_t	dmmnt_flags;		/* 12: mount flags */
	fsid_t		dmmnt_fsid;		/* 16: fsid for mount */
	int32_t		dmmnt_files;		/* 24: file count underneath */
	u_int32_t	dmmnt_spare;		/* 28: spare */
};

#define	DMFS_MOUNT_MAGIC	0x011968
#define	DMFS_MOUNT_BIGGEROK	0x000001	/* larger than needed file ok */
#define	DMFS_MOUNT_ONDEV	0x000002	/* on raw device, ignore size */

/*
 * 
 */

struct dmfs_mdata {
	u_int32_t	dmfsm_flags;		/*  0: flags */
	int32_t		dmfsm_gen;		/*  4: gen number of node */
	off_t		dmfsm_arch_size;	/*  8: size user sees 
						 *     iff DM_NONR set */
	bfid_t		dmfsm_bfid;		/* 16: bfid for file */
	off_t		dmfsm_bbarrier;		/* 32: block beyond here
						 *     iff DM_NONR set */
	int32_t		dmfsm_atime;		/* 40: apparent atime */
	int32_t		dmfsm_atimensec;	/* 44: apparent atime */
	int32_t		dmfsm_mtime;		/* 48: apparent mtime */
	int32_t		dmfsm_mtimensec;	/* 52: apparent mtime */
	u_int32_t	dmfsm_dbuid;		/* 56: dbuid */
	int32_t		dmfsm_rtime;		/* 60: time of last restore */
	int32_t		dmfsm_artime;		/* 64: time of last archive */
	int32_t		dmfsm_spare[3];		/* 68: round to 80 bytes */
};

#endif /* define DMFS_MDATA */
