/*	$NetBSD: dmfs_vnops.c,v 1.13 1998/03/01 02:21:43 fvdl Exp $	*/

/*
 * Copyright (c) 1999 National Aeronautics & Space Administration
 * All rights reserved.
 *
 * This software was written by William Studenmund of the
 * Numerical Aerospace Similation Facility, NASA Ames Research Center.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the the name of the National Aeronautics & Space Administration
 *    nor the names of its contributors may be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NATIONAL AERONAUTICS & SPACE ADMINISTRATION
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ADMINISTRATION OR CONTRIB-
 * UTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * Copyright (c) 1992, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * John Heidemann of the UCLA Ficus project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)dmfs_vnops.c	8.6 (Berkeley) 12/29/98
 *
 * Ancestors:
 *	@(#)lofs_vnops.c	1.2 (Berkeley) 6/18/92
 *	$I d: dmfs_vnops.c,v 1.30 1999/12/02 01:55:53 wrstuden Exp $
 *	...and...
 *	@(#)null_vnodeops.c 1.20 92/07/07 UCLA Ficus project
 *      @(#)null_vnops.c            (Berkeley)
 */

/*
 * Dm Layer
 *
 * (See mount_dmfs(8) for more information.)
 *
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/proc.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/kernel.h>
#include <sys/vnode.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <sys/namei.h>
#include <sys/malloc.h>
#include <sys/buf.h>
#include <miscfs/dmfs/dmfs.h>
#include <miscfs/dmfs/dmfs_extern.h>
#include <miscfs/dmfs/dmfs_fcntl.h>
#include <miscfs/genfs/genfs.h>
#include <miscfs/genfs/layer_extern.h>

extern  struct timespec dmfs_nulltime;

/*
 * Setattr call. Disallow write attempts if the layer is mounted read-only.
 */
int
dm_setattr(v)
	void *v;
{
	struct vop_setattr_args /* {
		struct vnodeop_desc *a_desc;
		struct vnode *a_vp;
		struct vattr *a_vap;
		struct ucred *a_cred;
		struct proc *a_p;
	} */ *ap = v;
	struct vnode *vp = ap->a_vp;
	struct vattr *vap = ap->a_vap;
	struct dm_node *dp = VTODM(vp);
	int error;

  	if ((vap->va_flags != VNOVAL || vap->va_uid != (uid_t)VNOVAL ||
	    vap->va_gid != (gid_t)VNOVAL || vap->va_atime.tv_sec != VNOVAL ||
	    vap->va_mtime.tv_sec != VNOVAL || vap->va_mode != (mode_t)VNOVAL) &&
	    (vp->v_mount->mnt_flag & MNT_RDONLY))
		return (EROFS);
	/*
	 * What should we do if someone's changing the length?
	 *
	 * Also catch changes to archived times.
	 */
	if (vap->va_size != VNOVAL) {
 		switch (vp->v_type) {
 		case VDIR:
 			return (EISDIR);
 		case VCHR:
 		case VBLK:
 		case VSOCK:
 		case VFIFO:
			return (0);
		case VREG:
		case VLNK:
 		default:
			/*
			 * Disallow write attempts if the filesystem is
			 * mounted read-only.
			 */
			if (vp->v_mount->mnt_flag & MNT_RDONLY)
				return (EROFS);
			/*
			 * If archived, do the trunc on this level to
			 * set flags right.
			 */
			if ((ISSET(dp->dm_flags, DM_HASMETA)) &&
					(ISSET(dp->dm_flags, DM_ARCH))) {
				error = VOP_TRUNCATE(vp, vap->va_size, 0,
					ap->a_cred, ap->a_p);
				if (error)
					return (error);
				vap->va_size = VNOVAL;
			}
		}
	}
	error = dm_bypass(ap);
	if (error != 0)
		return (error);
	/* node was locked by caller */
	if (ISSET(dp->dm_flags, DM_HASMETA)) {
		/* now check times */
		if ((vap->va_atime.tv_sec != VNOVAL) || 
				(vap->va_mtime.tv_sec != VNOVAL)) {
			/*
			 * save off the times.
			 */
			VOP_UPDATE(vp, &vap->va_atime, &vap->va_mtime, 1);
		}
	}
	return (0);
}

int
dm_truncate(v)
	void *v;
{
	struct vop_truncate_args /* {
		struct vnode *a_vp;
		off_t a_length;
		int a_flags;
		struct ucred *a_cred;
		struct proc *a_p;
	} */ *ap = v;
	struct	vnode	*vp = ap->a_vp;
	struct	dm_node	*dp = VTODM(vp);
	int error;
	uid_t theuser;

	if (!ISSET(dp->dm_flags, DM_HASMETA))
		return (dm_bypass(v));

	if (ap->a_cred)
		theuser = ap->a_cred->cr_uid;
	else
		theuser = 0;

	if (ap->a_length <= dp->dm_bbarrier) {
		/*
		 * With new (smaller) size, file is now
		 * fully resident. Wait for any in-progress
		 * restores to finish.
		 */
		if (ISSET(dp->dm_flags, DM_RIP))
			if ((error = dmfs_wait(vp)))
				return (error);
	} else {
		/*
		 * Restore before proceeding.
		 */
		error = dmfs_callwait(vp, theuser);
		if (error != 0)
			return (error);
	}
	if (ISSET(dp->dm_flags, DM_ARCH)) {
		CLR(dp->dm_flags, DM_NONR | DM_ARCH);
		dmfs_rashdelete(vp, theuser, NULL, 0);
	}
	SET(dp->dm_flags, DM_MODIFIED);
	VOP_UPDATE(vp, NULL, NULL, 0);

	return (dm_bypass(v));
}

/*
 *  We handle getattr to change the fsid and to change the size.
 */
int
dm_getattr(v)
	void *v;
{
	struct vop_getattr_args /* {
		struct vnode *a_vp;
		struct vattr *a_vap;
		struct ucred *a_cred;
		struct proc *a_p;
	} */ *ap = v;
	struct vnode *vp = ap->a_vp;
	struct vattr *vap = ap->a_vap;
	struct dm_node *dp = VTODM(vp);
	int error, islocked;

	if ((error = dm_bypass(ap)) != 0)
		return (error);
	/* Requires that arguments be restored. */
	ap->a_vap->va_fsid = vp->v_mount->mnt_stat.f_fsid.val[0];
	if ((islocked = VOP_ISLOCKED(vp)) == 0)
		VOP_LOCK(vp, LK_EXCLUSIVE | LK_RETRY);
	if (ISSET(dp->dm_flags, DM_HASMETA)) {
		if (ISSET(dp->dm_flags, DM_NONR)) {
#if 0
			if (ap->a_vap->va_size > dp->dm_archsize) {
				/* Oops! Someone wrote the underlying file! */
				dp->dm_archsize = ap->a_vap->va_size;
				SET(dp->dm_flags, DM_MODIFIED);
				CLR(dp->dm_flags, DM_ARCH | DM_NONR);
				if (ISSET(dp->dm_flags, DM_IOWANT))
					dmfs_wakeup(vp);
			} else {
#endif
				ap->a_vap->va_size = dp->dm_archsize;
#if 0
			}
#endif
		}
		vap->va_atime.tv_sec = dp->dm_atime;
		vap->va_atime.tv_nsec = dp->dm_atimensec;
		vap->va_mtime.tv_sec = dp->dm_mtime;
		vap->va_mtime.tv_nsec = dp->dm_mtimensec;
		CLR(vap->va_mode, _S_ARCH1 | _S_ARCH2);
		if (ISSET(dp->dm_flags, DM_NONR)) {
			SET(vap->va_mode, _S_ARCH2);
		} else if (ISSET(dp->dm_flags, DM_ARCH)) {
			SET(vap->va_mode, _S_ARCH1);
		}
	}
	if (islocked == 0)
		VOP_UNLOCK(vp, 0);
	return (0);
}

/*
 * If we're checking VEXEC access on a file which is non-resident, before
 * returning success, make sure the file is successfully restored.
 */
int
dm_access(v)
	void *v;
{
	struct vop_access_args /* {
		struct vnode *a_vp;
		int  a_mode;
		struct ucred *a_cred;
		struct proc *a_p;
	} */ *ap = v;
	int	error;
	struct	vnode	*vp = ap->a_vp;
	struct	dm_node	*dp = VTODM(vp);

	if ((error = layer_access(v)))
		return (error);

	if ((vp->v_type == VREG) && (ap->a_mode == VEXEC) &&
		(ISSET(dp->dm_flags, DM_HASMETA | DM_ARCH | DM_NONR)
			== (DM_HASMETA | DM_ARCH | DM_NONR))) {
		/* Bring it back in */
		error = dmfs_callwait(vp, ap->a_cred->cr_uid);
	}
	return (error);
}

/*
 * If DM_UPDATE_X set, update respective times.
 *
 * If opaque data modified (possibly by above), save on lower layer.
 *
 * Assume that setting opaque data marks inode as modified so that
 * lower layer will D.T.R.T. about a_waitfor.
 */
int
dm_update(v)
	void *v;
{
	struct vop_update_args /* {
		struct vnode *a_vp;
		struct timespec *a_access;
		struct timespec *a_modify;
		int a_waitfor;
	} */ *ap = v;
	struct vnode *vp = ap->a_vp;
	struct dm_node *dp = VTODM(vp);
	struct vattr va;
	struct timespec ts;
	int error;

	if (~ISSET(dp->dm_flags, DM_HASMETA) &&
			(ISSET(dp->dm_flags, DM_MODIFIED))) {
		/*
		 * initialize cached atime & mtime
		 */
		if ((error = VOP_GETATTR(dp->dm_lowervp, &va, NULL, NULL))) {
			return (error);
		}
		dp->dm_atime     = va.va_atime.tv_sec;
		dp->dm_atimensec = va.va_atime.tv_nsec;
		dp->dm_mtime     = va.va_mtime.tv_sec;
		dp->dm_mtimensec = va.va_mtime.tv_nsec;
		dp->dm_gen	 = va.va_gen;
		dp->dm_filenum	 = va.va_fileid;
		
		/* Add metadata */
		SET(dp->dm_flags, DM_HASMETA);
		if ((error = dmfs_write_mdata(vp, 0))) {
			CLR(dp->dm_flags, DM_HASMETA);
			return (error);
		}
	}
	if (ISSET(dp->dm_flags, DM_HASMETA)) {
		if ((ISSET(dp->dm_flags, DM_UPDATE_ATIME) &&
					ap->a_access == NULL) ||
				(ap->a_modify == NULL &&
					ISSET(dp->dm_flags, DM_UPDATE_MTIME)))
			TIMEVAL_TO_TIMESPEC(&time, &ts);
		if (ISSET(dp->dm_flags, DM_UPDATE_ATIME)) {
			if (ap->a_access == NULL) {
				dp->dm_atime = ts.tv_sec;
				dp->dm_atimensec = ts.tv_nsec;
			} else if (ap->a_access->tv_sec != VNOVAL) {
				dp->dm_atime = ap->a_access->tv_sec;
				dp->dm_atimensec = ap->a_access->tv_nsec;
			}
			SET(dp->dm_flags, DM_MODIFIED);
			CLR(dp->dm_flags, DM_UPDATE_ATIME);
		}
		if (ISSET(dp->dm_flags, DM_UPDATE_MTIME)) {
			if (ap->a_modify == NULL) {
				dp->dm_mtime = ts.tv_sec;
				dp->dm_mtimensec = ts.tv_nsec;
			} else if (ap->a_modify->tv_sec != VNOVAL) {
				dp->dm_mtime = ap->a_access->tv_sec;
				dp->dm_mtimensec = ap->a_access->tv_nsec;
			}
			SET(dp->dm_flags, DM_MODIFIED);
			CLR(dp->dm_flags, DM_UPDATE_MTIME);
		}
		if (ISSET(dp->dm_flags, DM_MODIFIED)) {
			if ((error = dmfs_write_mdata(vp, 0))) {
				return (error);
			}
			CLR(dp->dm_flags, DM_MODIFIED);
		}
	}
	return (dm_bypass(ap));
}

int
dm_inactive(v)
	void *v;
{
	struct vop_inactive_args /* {
		struct vnode *a_vp;
		struct proc *a_p;
	} */ *ap = v;
	struct vnode *vp = ap->a_vp;
	struct dm_node *dp = VTODM(vp);

	/*
	 * Do nothing other than possibly update
	 * underlying metadata (and _don't_ bypass).
	 * Wait to vrele lowervp until reclaim,
	 * so that until then our dm_node is in the
	 * cache and reusable.
	 *
	 * NEEDSWORK: Someday, consider inactive'ing
	 * the lowervp and then trying to reactivate it
	 * with capabilities (v_id)
	 * like they do in the name lookup cache code.
	 * That's too much work for now.
	 */
	if (ISSET(dp->dm_flags, DM_HASMETA)) {
		if (ISSET(dp->dm_flags, DM_RIP)) {
			CLR(dp->dm_flags, DM_RIP);
			printf("inactivating node with RIP set.\n");
		}
		if (ISSET(dp->dm_flags, DM_IOWANT)) {
			dmfs_wakeup(vp);
			/*
			 * If we're vgone'ing, then the sleepers will block when
			 * trying to re-lock, and then will see vp->v_op ==
			 * dead_vnodeop_p.
			 */
		}
		if (ISSET(dp->dm_flags, DM_UNLINKED)) {
			/*
			 * The file was unlinked to 0 links. Now that it's
			 * going on the free list, no one's using it so we
			 * should blow away its mdata.
			 */
			dmfs_rashdelete(vp, 0, NULL, 0);
			dmfs_nullmeta(dp);
			SET(dp->dm_flags, DM_MODIFIED);
		}
		if (ISSET(dp->dm_flags, DM_MODIFIED)) {
			VOP_UPDATE(vp, NULL, NULL, 0);
		}
	}
	VOP_UNLOCK(vp, 0);
	return (0);
}

int
dm_close(v)
	void *v;
{
	struct vop_close_args /* {
		struct vnode *a_vp;
		int  a_fflag;
		struct ucred *a_cred;
		struct proc *a_p;
	} */ *ap = v;
	struct vnode *vp = ap->a_vp;
	struct dm_node *dp = VTODM(vp);
	struct dm_mount *dmp = MOUNTTODMMOUNT(vp->v_mount);
	struct proc *p = ap->a_p;
	int error;

	if ((error = dm_bypass(v)))
		return (error);
	/*
	 * Normally we get called with p pointing to the closing process. In
	 * the case of the vnode getting vclean'd, p is null. So if we are
	 * called from either vclen or the restore/archive process, we
	 * act accordingly.
	 */
	if (ISSET(dp->dm_flags, DM_BIP) &&
		    ((p == NULL) || (p->p_pid == dmp->dmm_bkpid))) {
		/*
		 * Oops! backup process died w/o ending backup!
		 * So unlock all locked vnodes & clear DM_BIP.
		 */
		dmfs_backup(vp, DMFS_INODELK_NULLSTART, DMFS_INODELK_NULLEND,
				p);
		CLR(dmp->dmm_flags, DMM_BACKUP);
		CLR(dp->dm_flags, DM_BIP);
	}
	if (ISSET(dp->dm_flags, DM_HASMETA)) {
		if ((dp->dm_restorepid != 0) && 
			     ((p == NULL) || (dp->dm_restorepid == p->p_pid))) {
			if (ISSET(dp->dm_flags, DM_RFINISHING) && (p)) {
			    /*
			     * We know this is the restore agent closing, and
			     * it's indicated a delayed finish restore. Do
			     * it now.
			     */
			    CLR(dp->dm_flags, DM_RIP | DM_RFINISHING | DM_NONR);
			    dmfs_wakeup(vp);
			} else {
			    /*
			     * Restore agent closing == restore failure
			     */
			    if (ISSET(dp->dm_flags, DM_RIP)) {
				error = dmfs_restorefail(vp, ap->a_cred, p);
			    }
			}
			dp->dm_restorepid = 0;
		} else if ((dp->dm_archpid != 0) &&
			    ((p == NULL) || (dp->dm_archpid == p->p_pid))) {
			/*
			 * Archiver closing == archive failure
			 */
			CLR(dp->dm_flags, DM_AIP);
			dp->dm_archpid = 0;
		}
	}
	return (error);
}

int
dm_create(v)
	void *v;
{
	struct vop_create_args /* {
		struct vnode *a_dvp;
		struct vnode **a_vpp;
		struct componentname *a_cnp;
		struct vattr *a_vap;
	} */ *ap = v;
	struct dm_node *dp;
	int error;

	if ((error = dm_bypass(v)))
		return (error);

	dp = VTODM(*ap->a_vpp);
	if (ISSET(dp->dm_flags, DM_HASMETA)) {
		CLR(dp->dm_flags, DM_IOWANT | DM_RESTOREFAIL | DM_ARCH | DM_NONR | DM_AIP | DM_RIP);
		SET(dp->dm_flags, DM_MODIFIED);
	}

	return (0);
}

int
dm_remove(v)
	void *v;
{
	struct vop_remove_args /* {
		struct vnode *a_dvp;
		struct vnode *a_vp;
		struct componentname *a_cnp;
	} */ *ap = v;
	struct	vattr	va;
	struct	vnode	*vp;
	struct	dm_node *dp;
	int	error;

	vp = ap->a_vp;
	dp = VTODM(vp);

	if (!ISSET(dp->dm_flags, DM_HASMETA))
		return (dm_bypass(v));

	if ((error = VOP_GETATTR(dp->dm_lowervp, &va, NULL, NULL)))
		return (error);

	if ((error = dm_bypass(v)))
		return (error);

	if (va.va_nlink == 1) {
		/*
		 * There was only one link before, now there are none!
		 *
		 * Bye bye file! Note: The file might still be in use by
		 * an active process, but it's no longer on the disk.
		 */
		SET(dp->dm_flags, DM_UNLINKED);
	}

	return (0);
}

/*
 * Change the range of inodes reserved for backup. Handles starting, changing,
 * and removing the range.
 *
 * For a valid range, first must be >= last, p must be a valid proc,
 * and first & last must be valid inode #'s.
 * The range 1, 0 is a special marker indicating a null range.
 *
 * If invalid range or BIP not set, return EINVAL.
 */
int
dmfs_backup(vp, first, last, p)
	struct vnode *vp;
	int first, last;
	struct proc *p;
{
	struct	dm_node		*dp = VTODM(vp);
	struct	mount		*mp = vp->v_mount;
	struct	dm_mount	*dmp = MOUNTTODMMOUNT(mp);
	int			error;

	if (!ISSET(dp->dm_flags, DM_BIP) ||
			!ISSET((volatile int)dmp->dmm_flags, DMM_BACKUP))
		return (EINVAL);

	/* If we are setting to a non-null inode set, validate it */
	if ((first != DMFS_INODELK_NULLSTART) ||
					(last != DMFS_INODELK_NULLEND)) {
		if ((p == NULL) || (p->p_pid != dmp->dmm_bkpid))
			return (EINVAL);
		if ((first < 0) || (first >= dmp->dmm_mdata->dmmnt_files) ||
		    (last < 0)  || (last >= dmp->dmm_mdata->dmmnt_files)  ||
		    (first > last))
			return (EINVAL);
	}

	if ((error = vfs_busy(mp, 0, NULL)))
		return (error);

	/*
	 * make sure backup didn't get cleared while waiting for lock.
	 * otherwise set new range.
	 */
	if (!ISSET((volatile int)dmp->dmm_flags, DMM_BACKUP)) {
		error = EINVAL;
	} else {
		error = 0;
		dmp->dmm_bk_start = first;
		dmp->dmm_bk_end = last;
		if ISSET(dmp->dmm_flags, DMM_NODES_WANTED) {
			wakeup(dmp);
			CLR(dmp->dmm_flags, DMM_NODES_WANTED);
		}
		if (first <= last) {
			SET(dmp->dmm_flags, DMM_NODES_LOCKED);
		} else {
			CLR(dmp->dmm_flags, DMM_NODES_LOCKED);
		}
	}

	vfs_unbusy(mp);

	return (error);
}

/*
 * Our mountpoint has (had) DMM_NODES_LOCKED set. Check this vnode to see if
 * it's ok to use. Return 0 if/when it is. If it's not currently ok, sleep
 * until it is.
 *
 * We only return an error iff the call was interrupted.
 */
int
dmfs_node_wait(vp)
	struct vnode *vp;
{
	struct	dm_node		*dp = VTODM(vp);
	struct	mount		*mp = vp->v_mount;
	struct	dm_mount	*dmp = MOUNTTODMMOUNT(mp);
	int			error, error1;

	if ((error = vfs_busy(mp, 0, NULL)))
		return (error);

	for (;;) {
		/* break if this node # isn't blocked. */
		if ((dp->dm_filenum < dmp->dmm_bk_start) &&
				(dp->dm_filenum > dmp->dmm_bk_end))
			break;

		SET(dmp->dmm_flags, DMM_NODES_WANTED);
		vfs_unbusy(mp);

		error = tsleep(dmp, PRIBIO | PCATCH, "dmbknd", 20 * 60 * hz);
		if ((error1 = vfs_busy(mp, 0, NULL)))
			return (error1);

		/*
		 * if error == either EINTR or ERESTART, exit. Also exit if
		 * we are no longer locking inodes/doing a backup.
		 */
		if ((error == EINTR) || (error == ERESTART))
			break;
		if (ISSET(~dmp->dmm_flags, DMM_NODES_LOCKED | DMM_BACKUP))
			break;
	}

	vfs_unbusy(mp);

	return (0);
}

int
dm_print(v)
	void *v;
{
	struct vop_print_args /* {
		struct vnode *a_vp;
	} */ *ap = v;
	register struct vnode *vp = ap->a_vp;
	printf ("\ttag VT_DMFS, vp=%p, lowervp=%p\n", vp, DMVPTOLOWERVP(vp));
	return (0);
}

/*
 * Global vfs data structures
 */
int (**dm_vnodeop_p) __P((void *));
struct vnodeopv_entry_desc dm_vnodeop_entries[] = {
	{ &vop_default_desc,	layer_bypass },

	{ &vop_lookup_desc,	layer_lookup },
	{ &vop_lock_desc,	layer_lock },
	{ &vop_unlock_desc,	layer_unlock },
	{ &vop_reclaim_desc,	layer_reclaim },

	{ &vop_access_desc,	dm_access },
	{ &vop_setattr_desc,	dm_setattr },
	{ &vop_getattr_desc,	dm_getattr },
	{ &vop_create_desc,	dm_create },
	{ &vop_fcntl_desc,	dm_fcntl},
	{ &vop_inactive_desc,	dm_inactive },
	{ &vop_print_desc,	dm_print },
	{ &vop_remove_desc,	dm_remove },
	{ &vop_truncate_desc,	dm_truncate },
	{ &vop_update_desc,	dm_update },

#if 0
	{ &vop_mknod_desc,	dm_mknod },
	{ &vop_remove_desc,	dm_remove },
	{ &vop_link_desc,	dm_link },
	{ &vop_rename_desc,	dm_rename },
	{ &vop_mkdir_desc,	dm_mkdir },
	{ &vop_rmdir_desc,	dm_rmdir },
	{ &vop_symlink_desc,	dm_symlink },
#endif

	{ &vop_close_desc,	dm_close },

	{ &vop_read_desc,	dm_read},
	{ &vop_write_desc,	dm_write},
	{ &vop_bmap_desc,	dm_bmap },

	{ (struct vnodeop_desc*)NULL, (int(*)__P((void *)))NULL }
};
struct vnodeopv_desc dm_vnodeop_opv_desc =
	{ &dm_vnodeop_p, dm_vnodeop_entries };
