/*
 * $Source: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/kern/dmfs/rash.c,v $
 */

#include        "h/param.h"
#include        "h/errno.h"
#include        "h/uio.h"
#include        "h/dmon.h"
#include        "h/buf.h"
#include        "h/vfs.h"
#include	"h/file.h"
#include	"h/stat.h"
#include        "ufs/inode.h"
#include        "ufs/fs.h"
#include        "ufs/mount.h"
#ifdef QUOTA
#include        "ufs/quota.h"
#endif
#ifdef  _ACL
#include        "h/fcntl.h"
#include        "h/acl.h"
#include        "sec/si_sec.h"
#endif
#if defined(SecureWare) && defined(B1)
#include        "h/security.h"
#endif
 
#include        "pm/si_prot.h"
#include        "vfs/si_vfs.h"
#include        "sync/si_sync.h"
#include        "sched/si_sched.h"
#include        "sched/si_clocks.h"
#include        "kernel/si_kernel.h"

#include        "ufs/mi_panic_ufs.h"
#include        "ufs/mi_ufs_bmap.h"
#include        "ufs/mi_ufs_subr.h"
#include        "ufs/mi_ufs_alloc.h"
#include        "ufs/mi_quota_ufs.h"
#include        "ufs/mi_quota_incore.h"
#include        "ufs/mi_ufs_vnodeops.h"

#include	"ufs/mi_ufs_inode.h"

#include "rash.h"


#define SINGLE  0       /* index of single indirect block */
#define DOUBLE  1       /* index of double indirect block */
#define TRIPLE  2       /* index of triple indirect block */

struct rashinfo rashinfo;	/* the rashinfo structure */

bsema_t		rashlock;


rashinit()
{
	BSEMA_INIT(&rashlock, BSTC_SELECT, &select_bclassinfo);
}

struct vnode *
rashfile(name)
char *name;
{
	struct file *fp;
	struct vnode *vp;
	struct inode *ip;

	t.t_error = lookupname(name, UIOSEG_USER, FOLLOW_LINK, 
		(struct vnode **) 0, &vp);
	if (t.t_error)
		return NULL;
	if (vp->v_type != VREG) {
		VN_RELE(vp);
		t.t_error = EINVAL;
		return NULL;
	}
	if (vp->v_op != &ufs_vnodeops) {
		VN_RELE(vp);
		t.t_error = EINVAL;
		return NULL;
	}
	ip = VTOI(vp);
	if (!FS_RASH(ip->i_fs)) {
		VN_RELE(vp);
		t.t_error = EINVAL;
		return NULL;
	}
	return vp;
}

struct vnode *
rashfdes(fdes, rashflag)
int fdes;
{
	struct file *fp;
	struct vnode *vp;
	struct inode *ip;

	fp = getf(fdes);
	if (fp == NULL) {
		return NULL;
	}
	if (!F_ISVNODE(fp)) {
		relf(fdes);
		t.t_error = EINVAL;
		return NULL;
	}
	vp = FPTOVP(fp);
	if (rashflag && vp->v_type != VREG) {
		relf(fdes);
		t.t_error = EINVAL;
		return NULL;
	}
	if (vp->v_op != &ufs_vnodeops) {
		relf(fdes);
		t.t_error = EINVAL;
		return NULL;
	}
	ip = VTOI(vp);
	if (rashflag && !FS_RASH(ip->i_fs)) {
		relf(fdes);
		t.t_error = EINVAL;
		return NULL;
	}
	VN_HOLD(vp);
	relf(fdes);
	return vp;
}


/*
 * the rash control system call.
 */
rashcntl()
{
	struct fs *fs;
	struct file *fp;
	struct vnode *vp;
	struct inode *ip;
	struct a {
		union p1 {
			int	fdes;
			char	*fname;
		} p1;
		int	cmd;
		int	arg;		/* used as (int) or (char *) */
	} *uap = (struct a *)t.t_ap;
	register	i;
	int		h;
	int		found;
	struct	rashstat	rstat;
	struct	rarch		rarch;
	struct	stat		xs;
	struct	mount		*mp;
	struct	inode		*rip;
	long long		bfid_seed, file_size;
	unsigned		*temp;
	dmonfh_t		fh;
	int			nsize;
	daddr_t			bpref, nb, lbn;
	struct buf		*bp;


	t.t_rval1 = 0;

	switch (uap->cmd) {

	case R_GETPST:
		if ((vp = rashfile(uap->p1.fname)) == NULL)
			break;
		ip = VTOI(vp);

		INODE_RACQ(ip);
		rstat.st_rashflg = ip->i_rashflag;
		rstat.st_resisize = ip->i_resisize;
		rstat.st_bfid = ip->i_bfid;
		rstat.st_dbuid = ip->i_dbuid;
		INODE_RREL(ip);

		VN_RELE(vp);
		if (copyout((caddr_t)&rstat, (caddr_t)uap->arg, sizeof(rstat)))
			t.t_error = EFAULT;
		break;

	case R_RESTINO:

		if (!suser())
			break;
		if (copyin((caddr_t)uap->arg, (caddr_t)&rstat, sizeof(rstat))) {
			t.t_error = EFAULT;
			break;
		}
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		ip->i_bfid = rstat.st_bfid;
		ip->i_size = rstat.st_resisize;
		ip->i_rashflag = rstat.st_rashflg;
		ip->i_dbuid = rstat.st_dbuid;

		imark(ip, ICHG);
		iupdat(ip, 1);
		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_SETSIZE :

		if (!suser())
			break;
		if (copyin((caddr_t)uap->arg, (caddr_t)&file_size, sizeof file_size)) {
			t.t_error = EFAULT;
			break;
		}
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		ip->i_size = file_size;

		imark(ip, ICHG);
		iupdat(ip, 1);
		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_SETARC:
		if (!suser())
			break;

		if (copyin((caddr_t)uap->arg, (caddr_t)&rarch, sizeof(rarch))) {
			t.t_error = EFAULT;
			break;
		}
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		if ( ip->i_mtime.tv_sec != rarch.mtime ) {
			t.t_error = EINVAL;
			IUNLOCK(ip);
			VN_RELE(vp);
			break;
		}
		if ( ip->i_bfid != rarch.bfid ) {
			t.t_error = EINVAL;
			IUNLOCK(ip);
			VN_RELE(vp);
			break;
		}

		ip->i_dbuid = rarch.dbuid;
		ip->i_rashflag |= RARCH;
		imark(ip, ICHG);

		iupdat(ip, 1);
		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_SETNONR:
		if (!suser())
			break;
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		if (((vp->v_count - ip->i_nlink) > 2) || (ip->i_rashflag & RRIP)) {
			t.t_error = EBUSY;
			IUNLOCK(ip);
			VN_RELE(vp);
			break;
		}
		if (ip->i_rashflag & RARCH) {
			rashtrunc(ip);
			ip->i_rashflag |= RNONR;
			ip->i_resisize = 0;
			imark(ip, ICHG);
			iupdat(ip, 1);
		} else {
			t.t_error = EINVAL;
		}
		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_SETRRIP:
		if (!suser())
			break;
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);
		fs = ip->i_fs;
		mp = VFSTOM(FSTOVFS(fs));

		/* check to see that this is a RASH process (called rashopen) */
		if (ip->i_pid != u.u_procp->p_pid) {
			t.t_error = EINVAL;
			IUNLOCK(ip);
			VN_RELE(vp);
			break;
		}

		if ((ip->i_rashflag & RNONR) != RNONR) {
			t.t_error = EINVAL;
			IUNLOCK(ip);
			VN_RELE(vp);
			break;
		}

		/* now set a few flags in the dependant inode */
		ip->i_resisize = 0;    	/* NOT from extracted code */
		ip->i_rashflag &= ~RRFAIL;	/* starting new restore */
		ip->i_rashflag |= RRIP;

		/*
		 *	fix for Berkeley Fast File System
		 */
		if (ip->i_size > 0 && ip->i_size <= NDADDR * fs->fs_bsize) {
			FS_LOCK(fs);
			mp->m_openunlink++;
			FS_UNLOCK(fs);
			lbn = lblkno(fs, (ip->i_size - 1));
			nsize = blkoff(fs, ip->i_size);
			nsize = nsize ? fragroundup(fs, nsize) : fs->fs_bsize;
			bpref = blkpref(ip, lbn, 0, (daddr_t *)0);
			bp = alloc(ip, bpref, nsize);
			if (bp == NULL) {
				FS_LOCK(fs);
				mp->m_openunlink--;
				FS_UNLOCK(fs);
				IUNLOCK(ip);
				VN_RELE(vp);
				break;
			}
			nb = dbtofsb(fs, bp->b_blkno);
			bdwrite(bp);
			ip->i_db[lbn] = nb;
			imark(ip, ICHG);
			FS_LOCK(fs);
			mp->m_openunlink--;
			FS_UNLOCK(fs);
		}

		imark(ip, ICHG);
		iupdat(ip, 1);
		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_SETRRFAIL:
		if (!suser())
			break;
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		/* check to see that this is a RASH deamon process (rashopen) */
		if (ip->i_pid != u.u_procp->p_pid) {
			t.t_error = EINVAL;
			IUNLOCK(ip);
			VN_RELE(vp);
			break;
		}

		if ((ip->i_rashflag & RRIP) != RRIP) {
			t.t_error = EINVAL;
			IUNLOCK(ip);
			VN_RELE(vp);
			break;
		}

		ip->i_rashflag |= RRFAIL;

		rash_wakeup(ip);

		imark(ip, ICHG);
		iupdat(ip, 1);

		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_CLRARC:
		if (!suser())
			break;
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		ip->i_resisize = 0;
		ip->i_rashflag &= ~(RARCH|RNONR);
		imark(ip, ICHG);

		iupdat(ip, 1);
		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_GETST:
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);

		INODE_RACQ(ip);
		rstat.st_rashflg = ip->i_rashflag;
		rstat.st_resisize = ip->i_resisize;
		rstat.st_bfid = ip->i_bfid;
		rstat.st_dbuid = ip->i_dbuid;
		INODE_RREL(ip);

		VN_RELE(vp);
		if (copyout((caddr_t)&rstat, (caddr_t)uap->arg, sizeof(rstat)))
			t.t_error = EFAULT;
		break;

	case R_SETRSIZE:
		if (!suser())
			break;
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		fp = getf(uap->p1.fdes);
		ip->i_resisize = fp->f_offset;	/* set resisize to current */
		relf(uap->p1.fdes);

		rash_wakeup(ip);

		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_SEEDBFID:
		if (!suser())
			break;
		if (copyin((caddr_t)uap->arg, (caddr_t)&bfid_seed, sizeof bfid_seed)) {
			t.t_error = EFAULT;
			break;
		}

		if ((vp = rashfdes(uap->p1.fdes, 0)) == NULL)
			break;
		ip = VTOI(vp);

		INODE_RACQ(ip);
		if ((mp = getmp(ip->i_dev)) == NULL) {
			INODE_RREL(ip);
			VN_RELE(vp);
			break;
		}
		if ((rip = iget(ip->i_dev, mp->m_bufp->b_un.b_fs, (ino_t)1)) == NULL) {
			INODE_RREL(ip);
			VN_RELE(vp);
			break;
		}
		if (rip->i_bfid < bfid_seed) {	/* need to switch-O-runy */
			temp = (unsigned *) &bfid_seed;
			rip->i_bfid = bfid_seed;
			rip->i_nlink = 1;
			rip->i_mode = IFREG;
			imark(rip, ICHG);
			iupdat(rip, 1);
		}
		else {
			t.t_error = EDOM;
		}
		iput(rip);
		INODE_RREL(ip);

		FS_LOCK(ip->i_fs);
		ip->i_fs->fs_flags |= FSF_RASH;
		FS_UNLOCK(ip->i_fs);
		syncsb(ip->i_fs, ip->i_devvp);

		VN_RELE(vp);
		break;

		/*
		 * This flag is used to indicate a file restore due to
		 * tape compaction (or other sysadmin activity), solely
		 * for the purpose of accurate metrics.
		 */
	case R_SETCOMPACT:
		if (!suser())
			break;

		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		ip->i_rashflag |= RCOMPACT;
		imark(ip, ICHG);

		iupdat(ip, 1);
		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_CLRCOMPACT:
		if (!suser())
			break;
		if ((vp = rashfdes(uap->p1.fdes, 1)) == NULL)
			break;
		ip = VTOI(vp);
		ILOCK(ip);

		ip->i_resisize = 0;
		ip->i_rashflag &= ~RCOMPACT;
		imark(ip, ICHG);

		iupdat(ip, 1);
		IUNLOCK(ip);
		VN_RELE(vp);
		break;

	case R_MAKEFH:
		if ((vp = rashfile(uap->p1.fname)) == NULL)
			break;

		t.t_error = VOP_MAKEDMONFH(vp,  &fh);
		VN_RELE(vp);
		if (copyout((caddr_t)&fh, (caddr_t)uap->arg, sizeof(fh)))
			t.t_error = EFAULT;
		break;
	default:
		t.t_error = EINVAL;
	}
	return 0;
}



/*
**	The rashopen call takes the mount point and inode number of an
**	inode, and returns an open file descriptor.  This is used
**	by the restore daemon because it doesn't have a path name
**	available to it.
*/
rashopen()
{
	struct a {
		dmonfh_t	*fh;
	} *uap = (struct a *) t.t_ap;
	struct vnode		*vp;
	dmonfh_t		fh;
	struct file		*fp;
	int			fdes;

	if (!suser())
		return;

	t.t_error = copyin(uap->fh, &fh, sizeof(dmonfh_t));
	if (t.t_error)
		return;

	t.t_error = copen((char *)&fh, FREAD|FWRITE|FRASH, 0, OPEN_FH);
}

/*
 * rashclose system call - like normal close except clear Restore-In-Progress
 * flag in inode
 */
rashclose()
{
	struct a {
		int	fd;
		int	final;
	} *uap = (struct a *) t.t_ap;
	register user_t		*up = &u;
	register struct ufd	*fdp = &u.u_fd[uap->fd];
	int			error;
	struct vnode		*vp;
	struct file		*fp;

	if (!suser())
		return;

	if ( (unsigned)uap->fd >= NOFILE ) {
		t.t_error = EBADF;
		return;
	}

	UFD_BACQ(up);
	if (fdp->flags & UF_ALLOCATED) {
		if (uap->final) {
			if ((fp = fdp->ofile) != NULL) {
				vp = (struct vnode *) fp->f_data;
				rashfinal(vp);
			}
		}

		error = close_one_ufd(uap->fd, UF_ALL);	
		if (error) {
			t.t_error = EBADF;	/* File in process of closing */
		}
		else {
			UFD_BACQ(up);
			while (up->u_lastfile >= 0 
				&& up->u_fd[up->u_lastfile].flags == NULL) {
				up->u_lastfile--;
			}
			UFD_BREL(up);
		}
	}
	else {
		UFD_BREL(up);
		t.t_error = EBADF;
	}
	return;
}

rashfinal(vp)
struct vnode *vp;
{
	struct inode	*ip;

	ip = VTOI(vp);
	if (!FS_RASH(ip->i_fs))
		return;
	ILOCK(ip);
	if (ip->i_pid == 0) {
		uprintf("RASH: rashclose - warning - non-registered close \n" );
		IUNLOCK(ip);
		return;
	}
	else if (ip->i_pid != u.u_procp->p_pid) {
		uprintf("RASH: rashclose - warning - non-registered process close\n" );
		IUNLOCK(ip);
		return;
	}
	ip->i_pid = 0;
	if ((ip->i_rashflag & RRIP) && (ip->i_rashflag & RRFAIL) == 0) {
		ip->i_rashflag &= ~RNONR;
		ip->i_resisize = ip->i_size;
	}
	ip->i_rashflag &= ~RRIP;
	imark(ip, ICHG);

	rash_wakeup(ip);

	if (ip->i_rashflag & RRFAIL) {
		rashtrunc(ip);
		ip->i_rashflag |= RNONR;
		ip->i_resisize = 0;
		imark(ip, ICHG);
	}

	/*
	 *	the close will update the inode,
	 *	so we do not need to do it here
	 */
	IUNLOCK(ip);
}

/* 
** Call the RASH Daemon to perform a function on an archived file's inode
*/
static int id_seq_num = 0;

call_rashd(cmd, ip, uid)
int cmd;
struct inode *ip;
uid_t	uid;
{
	struct rashcall rashcall;
	int rv;

	if (ip == NULL) {
		printf("W - os/rash.c(call_rashd) bad ip\n");
		return;
	}

	rashcall.cmd = cmd;
	rashcall.id = ++id_seq_num;
	rashcall.bfid  = ip->i_bfid;
	rashcall.dbuid = ip->i_dbuid;
	rashcall.uid = uid;

	t.t_error = VOP_MAKEDMONFH(ITOV(ip),  &rashcall.fh);
	if (t.t_error)
		return;

	rv = kdcput(RASHD, &rashcall);
	if (cmd == RASH_RESTORE) {
		ip->i_resisize = 0;
		if (rv == 0) {
			ip->i_rashflag |= RRIP;
			ip->i_rashflag &= ~RRFAIL;
		} else {
			ip->i_rashflag |= RRFAIL;
			if (rv > 0)
				t.t_error = rv;
		}
	}
	iupdat(ip, 1);
}

/*
** rash_wait - delay access if data being restored
*/
rash_wait(ip)
struct inode *ip;
{
	int	s;
	proc_t	*p = t.t_procp;

	s = splhigh();
	BSEMA_ACQ(&rashlock);

	/* restore is in progess */
	while (ip->i_rashflag & RRIP) {
		ip->i_rashflag |= RASHWANT;
		IUNLOCK(ip);

		pr_suspend(p);
		p->p_flag2 |= SMIGPND;
		pr_resume(p);

		pend((caddr_t)&rashlock, (caddr_t)&ip->i_rashflag,
			PZERO+1, bsema_release);

		pr_suspend(p);
		p->p_flag2 &= ~SMIGPND;
		pr_resume(p);

		ILOCK(ip);
		BSEMA_ACQ(&rashlock);
	}

	BSEMA_REL(&rashlock);
	splx(s);
	return 0;
}

/*
** rash_wait - delay access if data being restored
*/
rash_wait_exec(vp)
struct vnode *vp;
{
	struct inode *ip;
	int	s;
	proc_t	*p = t.t_procp;

	if (vp->v_op != &ufs_vnodeops)
		return;

	ip = VTOI(vp);
	if (!FS_RASH(ip->i_fs))
		return;

	ILOCK(ip);
	if ((ip->i_rashflag & RNONR) == RNONR
	&&  (ip->i_rashflag & RRIP) != RRIP) {
		call_rashd(RASH_RESTORE, ip);
	}

	s = splhigh();
	BSEMA_ACQ(&rashlock);

	/* restore is in progess */
	while (ip->i_rashflag & RRIP) {
		ip->i_rashflag |= RASHWANT;
		IUNLOCK(ip);

		pr_suspend(p);
		p->p_flag2 |= SMIGPND;
		pr_resume(p);

		pend((caddr_t)&rashlock, (caddr_t)&ip->i_rashflag,
			PZERO+1, bsema_release);

		pr_suspend(p);
		p->p_flag2 &= ~SMIGPND;
		pr_resume(p);

		ILOCK(ip);
		BSEMA_ACQ(&rashlock);
	}

	BSEMA_REL(&rashlock);
	splx(s);

	IUNLOCK(ip);
	return;
}

/*
** rash_wait - delay access if data being restored
*/
rash_wait_mmap(vp)
struct vnode *vp;
{
	struct inode *ip;
	int	s;
	proc_t	*p = t.t_procp;

	if (vp->v_op != &ufs_vnodeops)
		return;

	ip = VTOI(vp);
	if (!FS_RASH(ip->i_fs))
		return;

	ILOCK(ip);
	s = splhigh();
	BSEMA_ACQ(&rashlock);

	/* restore is in progess */
	while (ip->i_rashflag & RRIP) {
		ip->i_rashflag |= RASHWANT;
		IUNLOCK(ip);

		pr_suspend(p);
		p->p_flag2 |= SMIGPND;
		pr_resume(p);

		pend((caddr_t)&rashlock, (caddr_t)&ip->i_rashflag,
			PZERO+1, bsema_release);

		pr_suspend(p);
		p->p_flag2 &= ~SMIGPND;
		pr_resume(p);

		ILOCK(ip);
		BSEMA_ACQ(&rashlock);
	}

	BSEMA_REL(&rashlock);
	splx(s);
	IUNLOCK(ip);
	return;
}

/*
** rash_delay - delay access if data being restored
*/
rash_delay(ip,rw,uio)
struct inode *ip;
enum uio_rw rw;
struct uio *uio;
{
	int	s;
	proc_t	*p = t.t_procp;

	s = splhigh();
	BSEMA_ACQ(&rashlock);

	/* restore is in progess */
	while ((ip->i_rashflag&RRIP) &&
			(ip->i_resisize<(uio->uio_offset+uio->uio_resid))) {
		if (rw == UIO_READ)
			rashinfo.read_rip++;
		else
			rashinfo.write_rip++;

		ip->i_rashflag |= RASHWANT;
		IUNLOCK(ip);

		pr_suspend(p);
		p->p_flag2 |= SMIGPND;
		pr_resume(p);

		pend((caddr_t)&rashlock, (caddr_t)&ip->i_rashflag,
			PZERO+1, bsema_release);

		pr_suspend(p);
		p->p_flag2 &= ~SMIGPND;
		pr_resume(p);

		ILOCK(ip);
		BSEMA_ACQ(&rashlock);
	}

	BSEMA_REL(&rashlock);
	splx(s);
	return 0;
}

rash_wakeup(ip)
struct inode *ip;
{
	int	s;

	s = splhigh();
	BSEMA_ACQ(&rashlock);

	if (ip->i_rashflag & RASHWANT) {
		ip->i_rashflag &= ~RASHWANT;
		wakeup((caddr_t)&ip->i_rashflag);
	}

	BSEMA_REL(&rashlock);
	splx(s);
}


rashtrunc(oip)
struct inode	*oip;
{
	register		i;
	register daddr_t	lastblock;
	daddr_t 		bn, lastiblock[NIADDR];
	register struct fs 	*fs;
	register struct inode 	*ip;
	struct inode 		tip;
	long 			blocksreleased = 0, nblocks;
	long 			indirtrunc();
	int 			level;
	struct buf 		*bp;
	int			offset;		/* offset within a block */
	off64_t			osize;		/* old size of file */
	int			size;		/* size of a block */

	struct vnode 		*devvp;
	u_int			dmon;

	off64_t		length = 0ll;
	int		dmonlock = DMON_LOCK;

	fs = oip->i_fs;
	if (oip->i_size <= length) {
		/*
		 * rwright 1/90
		 * used to just return here, but we need to
		 * update the file times for POSIX even if
		 * this was a noop.
		 */
		goto do_imark;
	}

	if(VN_ISVMOBJECT(ITOV(oip))) {	/* if mapped */
		t.t_error = ETXTBSY;
		return;
	}

	DMONTRUNC(ITOV(oip), (long long)length,
                  (long long)(oip->i_size - length), dmonlock, dmon, t.t_error);

        /*
         * If the dmon faults the truncate, proceed with freeing
         * the blocks if this is an internal callout.
         */
        if (t.t_error && (dmonlock != DMON_LOCK)) {
                return;
        } else {
                t.t_error = 0;
        }

	/*
	 * Calculate index into inode's block list of
	 * last direct and indirect blocks (if any)
	 * which we want to keep.  Lastblock is -1 when
	 * the file is truncated to 0.
	 */
	lastblock = lblkno(fs, length + fs->fs_bsize - 1) - 1;
	lastiblock[SINGLE] = lastblock - NDADDR;
	lastiblock[DOUBLE] = lastiblock[SINGLE] - NINDIR(fs);
	lastiblock[TRIPLE] = lastiblock[DOUBLE] - NINDIR(fs) * NINDIR(fs);
	nblocks = btodb(fs->fs_bsize);


	/*
	 * Update the size of the file. If the file is not being
	 * truncated to a block boundry, the contents of the
	 * partial block following the end of the file must be
	 * zero'ed in case it ever become accessable again because
	 * of subsequent file growth.
	 */
	osize  = oip->i_size;
	offset = blkoff(fs, length);
#if 0
	if (offset == 0) {
		oip->i_size = length;
	} else {
		t.t_error = 0;
		bn  = fsbtodb(fs, bmap(oip, lastblock, B_WRITE, offset,
					(int *)0, (struct ra_block *)0));
		if (t.t_error || (long)bn < 0)
			return;
		oip->i_size = length;
		size        = blksize(fs, oip, lastblock);
		devvp       = oip->i_devvp;

		bp = bread(devvp, bn, size);
		if (bp->b_flags & B_ERROR) {
			t.t_error   = EIO;
			oip->i_size = osize;
			brelse(bp);
			return;
		}
		bzero(bp->b_un.b_addr + offset, (u_int)(size - offset));
		bdwrite(bp);
	}
#endif
	/*
	 * check the new EOF block to see if it is migrated.  Set IEOFMIG
	 * appropriately.
	 */
	if (lastblock != -1) {
		t.t_error = 0;
		bn = bmap(oip, lastblock, (B_READ|B_LOCAL), 0, (int *)0,
				(struct ra_block *)0);
		if (t.t_error == EISMIGRATED)
			oip->i_sysflags |= IEOFMIG;
		else
			oip->i_sysflags &= ~IEOFMIG;
		t.t_error = 0;
	} else {
		oip->i_sysflags &= ~IEOFMIG;
	}

	/*
	 * Update file and block pointers
	 * on disk before we start freeing blocks.
	 * If we crash before free'ing blocks below,
	 * the blocks will be returned to the free list.
	 * lastiblock values are also normalized to -1
	 * for calls to indirtrunc below.
	 * (? fsck doesn't check validity of pointers in indirect blocks)
	 */
	tip        = *oip;
	tip.i_size = osize;
	for (level = TRIPLE; level >= SINGLE; level--) {
		if (lastiblock[level] < 0) {
			oip->i_ib[level]  = 0;
			lastiblock[level] = -1;
		}
	}
	for (i = NDADDR - 1; i > lastblock; i--)
		oip->i_db[i] = 0;

#if 0
	oip->i_size = length;
#endif
	oip->i_rashflag |= RNONR;
	oip->i_resisize = 0;
	imark(oip, ICHG);
	/*
	 * at this point, the inode no longer references some data
	 * blocks, yet the blocks are still allocated.  here we use
	 * the openunlink file to hold the filesystem dirty until
	 * the blocks have been added to the free list.
	 */
	FS_LOCK(fs);
	VFSTOM(FSTOVFS(fs))->m_openunlink++;
	FS_UNLOCK(fs);
	syncip(oip);
	ip = &tip;

	/*
	 * Indirect blocks first.
	 */
	for (level = TRIPLE; level >= SINGLE; level--) {
		bn = ip->i_ib[level];
		if (bn > 0) {
			blocksreleased += indirtrunc(ip, bn, lastiblock[level],
							level);
			if (lastiblock[level] < 0) {
				ip->i_ib[level] = 0;
				blkfree(ip, bn, (int)fs->fs_bsize);
				blocksreleased += nblocks;
			}
		}
		if (lastiblock[level] >= 0)
			goto done;
	}

	/*
	 * All whole direct blocks or frags.
	 */
	for (i = NDADDR - 1; i > lastblock; i--) {
		register int size_1;

		bn = ip->i_db[i];
		if (bn == 0 || bn == IBLK_MIG)
			continue;

		ip->i_db[i]    = 0;
		size_1         = blksize(fs, ip, i);
		blkfree(ip, bn, size_1);
		blocksreleased += btodb(size_1);
	}
	if (lastblock < 0)
		goto done;

	/*
	 * Finally, look for a change in size of the
	 * last direct block; release any frags.
	 */
	bn = ip->i_db[lastblock];
	if (bn > 0) {
		int oldspace, newspace;

		/*
		 * Calculate amount of space we're giving
		 * back as old block size minus new block size.
		 */
		oldspace   = blksize(fs, ip, lastblock);
		ip->i_size = length;
		newspace   = blksize(fs, ip, lastblock);
		if (newspace == 0)
			Panic(NULL,"ufs",P_9914,"rashtrunc: newspace");

		if (oldspace - newspace > 0) {
			/*
			 * Block number of space to be free'd is
			 * the old block # plus the number of frags
			 * required for the storage we're keeping.
			 */
			bn += numfrags(fs, newspace);
			blkfree(ip, bn, oldspace - newspace);
			blocksreleased += btodb(oldspace - newspace);
		}
	}
done:

	oip->i_blocks        -= blocksreleased;
	if (oip->i_blocks < 0)			/* sanity */
		oip->i_blocks = 0;

        if ((length == 0) && (dmonlock == DMON_LOCK) && (oip->i_blocks != 0)) {
	    printf("rashtrunc: inode: %d, fs: '%s'\n",oip->i_number,fs->fs_fsmnt);
	    printf("rashtrunc: blocks still allocated\n");
	    /*
	     * we bump up the openunlink count once extra because we know
	     * the filesystem is corrupt, but we cannot fix it.
	     */
	    FS_LOCK(fs);
	    VFSTOM(FSTOVFS(fs))->m_openunlink++;
	    FS_UNLOCK(fs);
	}

	/*
	 * at this point there should be no unreferenced blocks
	 * so we can decrement the openunlink count.
	 */
	FS_LOCK(fs);
	VFSTOM(FSTOVFS(fs))->m_openunlink--;
	FS_UNLOCK(fs);

	/*
	 * rwright 1/90
	 * for POSIX conformance, we need to update ctime and mtime...
	 * we used to only update ctime.
	 */
do_imark:
	imark(oip, ICHG);

#ifdef QUOTA
	if ( blocksreleased )
		(void) chkdq(oip, -blocksreleased, 0);
#endif
}

rash_write_open(vp, uid)
struct vnode *vp;
uid_t	uid;
{
	struct inode *ip;

	if (vp->v_op != &ufs_vnodeops)
		return 1;

	ip = VTOI(vp);
	if (!FS_RASH(ip->i_fs))
		return 1;

	ILOCK(ip);
	if ((ip->i_rashflag & RNONR) != RNONR) {
		if ((ip->i_rashflag & RARCH) == RARCH) {
			ip->i_rashflag &= ~RARCH;
			imark(ip, IUPD|ICHG);
			iupdat(ip, 1);
		}
		IUNLOCK(ip);
		return 1;
	}
	
	if ((ip->i_rashflag & RRIP) != RRIP) {
		call_rashd(RASH_RESTORE, ip, uid);
	}

	IUNLOCK(ip);
	return 0;
}

rash_read_open(vp, offset, count, uid)
struct vnode *vp;
u_long offset;
u_long count;
uid_t	uid;
{
	struct inode *ip;

	if (vp->v_op != &ufs_vnodeops)
		return 1;

	ip = VTOI(vp);
	if (!FS_RASH(ip->i_fs))
		return 1;

	ILOCK(ip);
	if ((ip->i_rashflag & RNONR) != RNONR) {
		IUNLOCK(ip);
		return 1;
	}
	
	if ((ip->i_rashflag & RRIP) != RRIP) {
		call_rashd(RASH_RESTORE, ip, uid);
		IUNLOCK(ip);
		return 0;
	}
	else if (ip->i_resisize<(offset+count)) {
		IUNLOCK(ip);
		return 0;
	}
	else {
		IUNLOCK(ip);
		return 1; 
	}
}

