/*	$NetBSD: dmfs_subr.c,v 1.13 1998/12/29 06:50:10 waddell Exp $	*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/param.h>
#include <machine/bswap.h>
#include <miscfs/dmfs/dmfs.h>
#include <miscfs/dmfs/dmfs_fcntl.h>
#include <miscfs/dmfs/dmfs_mdata.h>

#include "dmfs_mfile.h"

#define	SET(a,b)	((a) |= (b))
#define	ISSET(a,b)	((a) & (b))

int
dmfs_open_mfile(path, dmmpp)
	char			*path;
	struct	dmfs_usermount	**dmmpp;
{
	struct	stat		sb;
	struct	dmfs_usermount	*dmmp;
	struct	dmfs_mount_data	*dmpmd;
	int			fd;
	long			filecount;
	u_quad_t		expsize;

	dmpmd = (struct dmfs_mount_data *)
				malloc(sizeof(struct dmfs_mount_data));
	if (dmpmd == NULL)
		return (-1);

	dmmp = (struct dmfs_usermount *)malloc(sizeof(struct dmfs_usermount));
	if (dmmp == NULL)
		goto bad;

	dmmp->flags = 0;

	if ((fd = open(path, O_RDONLY, 0)) < 0)
		goto bad1;

	if (pread(fd, dmpmd, sizeof(*dmpmd), 0) != sizeof(*dmpmd))
		goto bad1;

	/* now check things out. */
	if (dmpmd->dmmnt_magic != DMFS_MOUNT_MAGIC) {
		if (bswap32(dmpmd->dmmnt_magic) != DMFS_MOUNT_MAGIC) {
			errno = EINVAL;
			goto bad1;
		} else {
			SET(dmmp->flags, DMM_SWAPMD);
			dmpmd->dmmnt_mntsize = bswap16(dmpmd->dmmnt_mntsize);
			dmpmd->dmmnt_mntvers = bswap16(dmpmd->dmmnt_mntvers);
			dmpmd->dmmnt_nodesize = bswap16(dmpmd->dmmnt_nodesize);
			dmpmd->dmmnt_nodevers = bswap16(dmpmd->dmmnt_nodevers);
		}
	}

	/*
	 * Now the size & versions for the mount and the node data are in
	 * host byte order. Do they make sense?
	 */
	switch (dmpmd->dmmnt_mntvers) {
	case 0:
		if ((dmpmd->dmmnt_mntsize != sizeof(*dmpmd)) ||
			(dmpmd->dmmnt_nodevers != 0) ||
			(dmpmd->dmmnt_nodesize != sizeof(struct dmfs_mdata))) {
		    errno = EINVAL;
		    goto bad1;
		}
		if (ISSET(dmmp->flags, DMM_SWAPMD)) {
			dmpmd->dmmnt_flags = bswap32(dmpmd->dmmnt_flags);
			dmpmd->dmmnt_files = bswap32(dmpmd->dmmnt_files);
			dmpmd->dmmnt_fsid.val[0] =
					bswap32(dmpmd->dmmnt_fsid.val[0]);
			dmpmd->dmmnt_fsid.val[1] =
					bswap32(dmpmd->dmmnt_fsid.val[1]);
		}
		break;	

	default:
		errno = EINVAL;
		goto bad1;
	}

#if 0
	fsstat = &uarea.ustatfs;
	if ((error = VFS_STATFS(dmp->dmm_vfs, fsstat, curproc)))
		goto bad1;
	filecount = fsstat->f_files;
	if (dmpmd->dmmnt_files != filecount) {
		errno = EINVAL;
		goto bad1;
	}
#else
	filecount = dmpmd->dmmnt_files;
#endif

	expsize = (u_quad_t)filecount * dmpmd->dmmnt_nodesize +
			dmpmd->dmmnt_mntsize;

	if (fstat(fd, &sb))
		goto bad1;

	if (ISSET(dmpmd->dmmnt_flags, DMFS_MOUNT_ONDEV)) {
		if (!(S_ISCHR(sb.st_mode))) {
			errno = ENODEV;
			goto bad1;
		}
	} else {
		/*
		 * The file must be at least as big as we expect.
		 * bigger than we expect is only ok iff so flagged.
		 */
		if ((sb.st_size < expsize) || ((sb.st_size > expsize) &&
			    !ISSET(dmpmd->dmmnt_flags, DMFS_MOUNT_BIGGEROK))) {
			errno = EINVAL;
			goto bad1;
		}
	}

	dmmp->mdata	= dmpmd;
	dmmp->fd	= fd;
	*dmmpp		= dmmp;

	return (fd);

bad1:
	free(dmmp);
bad:
	free(dmpmd);
	return (-1);
}

int
dmfs_close_mfile(dmmp)
        struct  dmfs_usermount  *dmmp;
{
	int	fd;

	fd = dmmp->fd;

	free(dmmp->mdata);
	free(dmmp);

	return (close(fd));
}

/*
 * Read metadata from mfile for count inodes starting with start.
 * will be read into buffer pointed to by buf, and if needed,
 * byte-swapped in place.
 */
int
dmfs_read_mdata(dmmp, start, count, buf)
	struct	dmfs_usermount	*dmmp;
	int			start, count;
	void			*buf;
{
	struct	dmfs_mount_data	*dmpmd = dmmp->mdata;
	int			error, i, j;
	off_t			offset;

	if ((start < 0) || (count < 1) ||
				((start + count) > dmpmd->dmmnt_files)) {
		errno = EINVAL;
		return (-1);
	}

	offset	= (off_t)dmpmd->dmmnt_mntsize +
			((off_t)dmpmd->dmmnt_nodesize) * start;

	error = pread(dmmp->fd, buf, dmpmd->dmmnt_nodesize * count, offset);
	if (error < 0)
		return (error);

	if (dmmp->flags & DMM_SWAPMD) {
		j = error / dmpmd->dmmnt_nodesize; /* count of full entries */
		for (i = 0; i < j; i++, ((struct dmfs_mdata *)buf)++)
			dmfs_byteswap_meta((struct dmfs_mdata *)buf);
	}

	return (error);
}

void
dmfs_byteswap_meta(meta)
	struct dmfs_mdata *meta;
{
	meta->dmfsm_flags	= bswap32(meta->dmfsm_flags);
	meta->dmfsm_gen		= bswap32(meta->dmfsm_gen);
	meta->dmfsm_arch_size	= bswap64(meta->dmfsm_arch_size);
	meta->dmfsm_bbarrier	= bswap64(meta->dmfsm_bbarrier);
	meta->dmfsm_dbuid	= bswap32(meta->dmfsm_dbuid);
	/* dmfsm_bfid is endian-independent */
	meta->dmfsm_atime	= bswap32(meta->dmfsm_atime);
	meta->dmfsm_atimensec	= bswap32(meta->dmfsm_atimensec);
	meta->dmfsm_mtime	= bswap32(meta->dmfsm_mtime);
	meta->dmfsm_mtimensec	= bswap32(meta->dmfsm_mtimensec);
	meta->dmfsm_rtime	= bswap32(meta->dmfsm_rtime);
	meta->dmfsm_artime	= bswap32(meta->dmfsm_artime);
}

void
dmfs_bufswap_meta(to, from)
	struct dmfs_mdata *to, *from;
{
	to->dmfsm_flags		= bswap32(from->dmfsm_flags);
	to->dmfsm_gen		= bswap32(from->dmfsm_gen);
	to->dmfsm_arch_size	= bswap64(from->dmfsm_arch_size);
	to->dmfsm_bbarrier	= bswap64(from->dmfsm_bbarrier);
	to->dmfsm_dbuid		= bswap32(from->dmfsm_dbuid);
	/* dmfsm_bfid is endian-independent */
	to->dmfsm_atime		= bswap32(from->dmfsm_atime);
	to->dmfsm_atimensec	= bswap32(from->dmfsm_atimensec);
	to->dmfsm_mtime		= bswap32(from->dmfsm_mtime);
	to->dmfsm_mtimensec	= bswap32(from->dmfsm_mtimensec);
	to->dmfsm_rtime		= bswap32(from->dmfsm_rtime);
	to->dmfsm_artime	= bswap32(from->dmfsm_artime);
}

int
dmfs_open_control(name)
	char			*name;
{
	int	fd, error;

printf("About to open %s\n", name);
	if ((fd = open(name, O_RDONLY, 0)) < 0)
		return (fd);

printf("About to SETBIP\n");
	if (fcntl(fd, DMFS_SETBIP, NULL) < 0) {
		error = errno;
		close(fd);
		errno = error;
		return (-1);
	}

	return (fd);
}

int
dmfs_close_control(fd)
	int			fd;
{
	fcntl(fd, DMFS_CLRBIP, NULL);
	return (close(fd));
}

int
dmfs_lock_nodes(fd, start, end)
	int	fd;
	long	start, end;
{
	struct	dmm_inode_locks	lb;

	lb.start = start;
	lb.end   = end;

	return(fcntl(fd, DMFS_SETINODELK, &lb));
}
