/*
 * Copyright (c) 1999 National Aeronautics & Space Administration
 * All rights reserved.
 *
 * This software was written by William Studenmund of the
 * Numerical Aerospace Similation Facility, NASA Ames Research Center.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the the name of the National Aeronautics & Space Administration
 *    nor the names of its contributors may be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NATIONAL AERONAUTICS & SPACE ADMINISTRATION
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ADMINISTRATION OR CONTRIB-
 * UTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <err.h>

#include <sys/param.h>
#include <sys/types.h>
#include <sys/mount.h>
#include <miscfs/dmfs/dmfs.h>
#include <miscfs/dmfs/dmfs_mdata.h>

int	makefstype	__P(( char *type));

int
makefstype(type)
	char *type;
{
	int rv;

	for (rv = 0; *type; type++) {
		rv <<= 2;
		rv ^= *type;
	}
	return (rv);
}

int
main(int argc, char **argv)
{
	int			ch, fd, extrafiles;
	char			*myname = argv[0];
	struct	statfs		thestats;
	struct	stat		mdata_stat;
	struct	dmfs_mount_data	mount_data;
	off_t			filesize;
	size_t			writesize;
	char			*buffer;

	while ((ch = getopt(argc, argv, "F:")) != -1)
		switch (ch) {
/*
		case 'o':
			offset = atol(optarg);
			break;
*/
		case 'F':
			extrafiles = atoi(optarg);
			if ((extrafiles < 0) || (extrafiles > 10)) {
			}
			break;

		}
	argc -= optind;
	argv += optind;

	if( argc != 2 ) {
		fprintf(stderr,"%s [-F extra-files] mfile-path mountpoint\n", myname);
		exit(1);
	}

	if ((fd = open(argv[0], O_RDWR | O_CREAT, 0600)) < 0)
		err(1, "%s: opening mdata file", argv[0]);

	if (fstat(fd, &mdata_stat))
		err(1, "%s: fstat'ing file", argv[0]);

	if (statfs(argv[1], &thestats))
		err(1, "%s: trying to stat filesystem", argv[1]);

	if ((buffer = (char *)calloc(64*1024, sizeof(char))) == NULL)
		err(1, "Couldn't allocate buffer");

	mount_data.dmmnt_magic		= 0x011968;
	mount_data.dmmnt_mntsize	= sizeof(struct dmfs_mount_data);
	mount_data.dmmnt_mntvers	= 0;
	mount_data.dmmnt_nodesize	= sizeof(struct dmfs_mdata);
	mount_data.dmmnt_nodevers	= 0;
	mount_data.dmmnt_flags		= 0;
	mount_data.dmmnt_files		= thestats.f_files;
	mount_data.dmmnt_spare		= 0;

	ch = makefstype(MOUNT_DMFS);
	mount_data.dmmnt_fsid.val[0]	= makedev(254, mdata_stat.st_ino);
	mount_data.dmmnt_fsid.val[1]	= ch;

	filesize = sizeof(struct dmfs_mdata) * (off_t)thestats.f_files +
				sizeof(struct dmfs_mount_data);
printf("Making file length %lld for %ld files\n", (long long)filesize, thestats.f_files);
	if (ftruncate(fd, filesize))
		err(1, "truncating file");
	for(; filesize; filesize -= writesize) {
		if (filesize > 64*1024) 
			writesize = 64 * 1024;
		else
			writesize = filesize;
		if (write(fd, buffer, writesize) != writesize)
		    err(1, "error zeroing file with %ld remaining",
					(long)filesize);
	}

	if (pwrite(fd, &mount_data, sizeof(struct dmfs_mount_data), 0)
			!= sizeof(struct dmfs_mount_data))
		err(1, "Writing mount data");

	return (0);
}
