#include <dba.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>


#define MAXDATE 80


int main()
{
  DbA* db;
  DbA_Info in_a;
  DbA_Info in_b;
  DbA_Info out_a;
  DbA_Info out_b;

  /* opening and creating table */
  if((db= newDbAbstractor("tmp_info_functions", 'd', 'i')) == NULL){
    fprintf(stderr, "Error opening tmp_info_function table (integer value)\n");
    exit(1);
  }

  sprintf(in_a.description, "testing of storing table description");
  sprintf(in_a.data_units, "sample_units");

  /* storing info in descriptions table */
  if(dba_set_info(db, &in_a) != DBA_SUCCESS){
    dba_destroy(db);
    fprintf(stderr, "Error setting table description\n");
    exit(1);
  }

  /* retrieving info from descriptions table */
  if(dba_get_info(db, &out_a) != DBA_SUCCESS){
    dba_destroy(db);
    fprintf(stderr, "Error getting table description\n");
    exit(1);
  }

  /* comparing stored to retrieved info */
  if(strcmp(in_a.description, out_a.description) != 0){
    dba_destroy(db);
    fprintf(stderr, "Error: in and out descriptions do not match!\n");
    fprintf(stderr, "%s != %s\n", in_a.description, out_a.description);
    exit(1);
  }

  if(strcmp(in_a.data_units, out_a.data_units) != 0){
    dba_destroy(db);
    fprintf(stderr, "Error: in and out data_units do not match!\n");
    fprintf(stderr, "%s != %s\n", in_a.data_units, out_a.data_units);
    exit(1);
  }


  /* test replacing info in descriptions table */

  sprintf(in_b.description, "testing update of stored table description");
  sprintf(in_b.data_units, "2_sample_units");

  /* storing info in descriptions table */
  if(dba_set_info(db, &in_b) != DBA_SUCCESS){
    dba_destroy(db);
    fprintf(stderr, "Error setting table description with replacement info\n");
    exit(1);
  }

  /* retrieving info from descriptions table */
  if(dba_get_info(db, &out_b) != DBA_SUCCESS){
    dba_destroy(db);
    fprintf(stderr, "Error getting table description of replacement info\n");
    exit(1);
  }

  /* comparing stored to retrieved info */
  if(strcmp(in_b.description, out_b.description) != 0){
    dba_destroy(db);
    fprintf(stderr, 
	    "Error: in and out replacement descriptions do not match!\n");
    fprintf(stderr, "%s != %s\n", in_b.description, out_b.description);
    exit(1);
  }

  if(strcmp(in_b.data_units, out_b.data_units) != 0){
    dba_destroy(db);
    fprintf(stderr, 
	    "Error: in and out replacement data_units do not match!\n");
    fprintf(stderr, "%s != %s\n", in_b.data_units, out_b.data_units);
    exit(1);
  }
  

  /* not in the api, don't use*/
  if(dba_remove(db) != DBA_SUCCESS)
    fprintf(stderr, "error removing table from database");


  dba_destroy(db);

  return(0);
}
