#include <dba.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>


int main(int argc, char** argv)
{
  struct timeval start, end;
  struct timezone tzp;

  DbA* db;
  DBAREC val;

  int recnumber = 1000, i; 
  double conn_create_time, insert_time; 
  
  char datebuf[] = "1997-09-23 10:13:01";
  long int value;

  if(argc < 2){
    fprintf(stderr, "usage: pgbench dbname [records]\n");
    exit(1);
  }
  if(argc > 2)
    recnumber = atoi(argv[2]);

  gettimeofday(&start, &tzp);

  if((db = newDbAbstractor("tmp_dbabench", 'd', 'i')) == NULL){
    fprintf(stderr, "Error opening tmp_dbabench database\n");
    exit(1);
  }
  gettimeofday(&end, &tzp);

  conn_create_time = ((double)end.tv_sec + (end.tv_usec / 1000000.0)) - ((double)start.tv_sec + (start.tv_usec / 1000000.0));

  gettimeofday(&start, &tzp);

  dba_begin(db);
  val.date = &datebuf; 
  val.tag = "boo";
 
  for(i = 0; i < recnumber; ++i){
    value = 100 * i;
    val.value = &value;
    if((dba_store(db, &val)) != 0){
      fprintf(stderr, "Error storing date = %s, value = %ld\n", 
	      datebuf, value);
      exit(1);
    }
  }

  dba_end(db);

  gettimeofday(&end, &tzp);

  /* not in the api, don't use*/
  if(dba_remove(db)!= DBA_SUCCESS)
    fprintf(stderr, "error removing table from database\n");

  dba_destroy(db);

  insert_time = ((double)end.tv_sec + (end.tv_usec / 1000000.0)) - ((double)start.tv_sec + (start.tv_usec / 1000000.0));

  printf("connection/create latency: %f, inserted %d records at %f recs/sec\n", 
	 conn_create_time, recnumber, (recnumber / insert_time));
  
  return(0);
}

