#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>

#include <libpq-fe.h> 



int main(int argc, char** argv)
{
  char* backend_host = "gnome.nas.nasa.gov";
  char buffer[256];

  PGconn* db; 
  PGresult* res; 
  int recnumber = 1000, i; 
  
  struct timeval start, end;
  struct timezone tzp;

  double connect_time, insert_time; 

  char dtime[] = "1997-09-23 10:13";
  time_t utime = 8916924;
  long int value;
  char subset[] = "foo";
  char tag[] = "boo";

  if(argc < 2){
    fprintf(stderr, "usage: pgbench dbname [records]\n");
    exit(1);
  }
  if(argc > 2)
    recnumber = atoi(argv[2]);

  gettimeofday(&start, &tzp);

  db = PQsetdb(backend_host, NULL, NULL, NULL, argv[1]);

  gettimeofday(&end, &tzp);
  connect_time = ((double)end.tv_sec + (end.tv_usec / 1000000.0)) - ((double)start.tv_sec + (start.tv_usec / 1000000.0));

  if (PQstatus(db) == CONNECTION_BAD) { 
    fprintf(stderr,"Connection to database '%s' failed.\n", argv[1]); 
    fprintf(stderr,"%s", PQerrorMessage(db)); 
    PQfinish(db);
    return(1); 
  }


  res = PQexec(db, "CREATE TABLE tmp_pgbench (dtime datetime, utime int4, value int4, ctime datetime, subset varchar(32), tag varchar(32))");
  if (PQresultStatus(res) != PGRES_COMMAND_OK) { 
    fprintf(stderr,"CREATE command failed\n"); 
    PQclear(res); 
    PQfinish(db);
    exit(1);
  }
  PQclear(res);

  gettimeofday(&start, &tzp);

 
  for(i = 0; i < recnumber; ++i){
    value = 100 * i;
    sprintf(buffer, "INSERT INTO tmp_pgbench VALUES ('%s', %ld, %ld, 'now', '%s', '%s')",
	    dtime, utime, value, subset, tag);
    res = PQexec(db, buffer);
    if (PQresultStatus(res) != PGRES_COMMAND_OK) { 
      fprintf(stderr,"INSERT command failed\n"); 
      PQclear(res); 
      res = PQexec(db, "drop table tmp_pgbench");
      PQclear(res); 
      PQfinish(db);
      exit(1);
    }
    PQclear(res);
  }

  gettimeofday(&end, &tzp);
  
  insert_time = ((double)end.tv_sec + (end.tv_usec / 1000000.0)) - ((double)start.tv_sec + (start.tv_usec / 1000000.0));

  res = PQexec(db, "DROP TABLE tmp_pgbench");
  if (PQresultStatus(res) != PGRES_COMMAND_OK) { 
    fprintf(stderr,"DROP TABLE command failed\n"); 
    PQclear(res); 
    PQfinish(db);
    exit(1);
  }
  PQclear(res);
  PQfinish(db);

  /* generate the timing stats */

  printf("connection latency: %f, inserted %d records at %f recs/sec\n", 
	 connect_time, recnumber, (recnumber / insert_time));
  
  return(0);
}
