#include <stdio.h>
#include <dba.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>


#ifdef sgi
#include <getopt.h>
#else
extern int optind;
#endif

void usage()
{
  fprintf(stderr,"batch_insert_dba (-d | -t) (-i | -f | -c) tablename\n");
  fprintf(stderr, 
	  "-d\tascii date format {Year-Month-Day[Hour:Minute:Second]}\n");
  fprintf(stderr, "-t\tUNIX time in integer seconds\n");
  fprintf(stderr, "-i\tvalue is an 4 byte integer\n");
  fprintf(stderr, "-f\tvalue is an 8 byte floating point\n");
  fprintf(stderr, "-c\tvalue is null-terminated character string\n");
  fprintf(stderr, 
	  "\t\t(see DBA_MAX_CHARARRAY in dba.h for maximum string length)\n");
}


int main(int argc, char** argv)
{
  DbA* db;
  int dtype, vtype, c;
  char d_date[80], c_val[DBA_MAX_CHARARRAY];
  long int t_date, i_val; 
  double f_val;
  char* p;
  char linebuf[1024];
  
  while((c = getopt (argc, argv, "hdtifc")) != EOF){
    switch((char)c){
    case 'h':
      usage();
      exit(0);
    case 'd':
      dtype = 'd';
      break;
    case 't':
      dtype = 't';
      break;
    case 'i':
      vtype = 'i';
      break;
    case 'f':
      vtype = 'f';
      break;
    case 'c':
      vtype = 'c';
      break;
    }
  }
  if(optind != (argc - 1)){
    usage();
    exit(1);
  }
  if(dtype != 'd' && dtype != 't'){
    fprintf(stderr, "Error: must specify date type\ndtype = %d\n", dtype);
    usage();
    exit(1);
  }
  if(vtype != 'i' && vtype != 'f' && vtype != 'c'){
    fprintf(stderr, "Error: must specify value type\nvtype = %d\n", vtype);
    usage();
    exit(1);
  }
  
  if((db = newDbAbstractor(argv[optind], dtype, vtype)) == NULL){
    fprintf(stderr, "%s table failed to open\n", argv[optind]);
    exit(1);
  }
  
  
  while(fgets(linebuf, 1024, stdin) != NULL){
    if((p = strtok(linebuf, " \t")) == NULL || p[0] == '\0')
      continue;
    if(dtype == 't')
      t_date = strtol(p, (char **)NULL, 10);
    else
      strcpy(d_date, p);

    if((p = strtok(NULL, " \t")) == NULL || p[0] == '\0')
      continue;

    if((p = strtok(NULL, "\n")) == NULL || p[0] == '\0')
      continue;
    if(vtype == 'i')
      i_val = strtol(p, (char **)NULL, 10);
    else if(vtype == 'f')
      f_val = strtod(p, (char **)NULL);
    else
      strcpy(c_val, p);
    
    if(dtype == 'd'){
      if(vtype == 'i'){
	if((dba_store(db, d_date, &i_val, NULL, NULL)) != 0){
	  fprintf(stderr, "error storing date = %s, value = %ld\n", 
		  d_date, i_val);
	  dba_destroy(db);
	  exit(1);
	}
      }
      else if(vtype == 'f'){
	if((dba_store(db, d_date, &f_val, NULL, NULL)) != 0){
          fprintf(stderr, "error storing date = %s, value = %f\n",
                  d_date, f_val);
          dba_destroy(db);
          exit(1);
        }
      }
      else{
	if((dba_store(db, d_date, c_val, NULL, NULL)) != 0){
          fprintf(stderr, "error storing date = %s, value = %s\n",
                  d_date, c_val);
          dba_destroy(db);
          exit(1);
        }
      }
    }
    else{
      if(vtype == 'i'){
        if((dba_store(db, &t_date, &i_val, NULL, NULL)) != 0){
          fprintf(stderr, "error storing date = %ld, value = %ld\n",
		  t_date, i_val);
          dba_destroy(db);
          exit(1);
        }
      }
      else if(vtype == 'f'){
	if((dba_store(db, &t_date, &f_val, NULL, NULL)) != 0){
	  fprintf(stderr, "error storing date = %ld, value = %f\n",
		  t_date, f_val);
	  dba_destroy(db);
          exit(1);
        }
      }
      else{
        if((dba_store(db, &t_date, c_val, NULL, NULL)) != 0){
          fprintf(stderr, "error storing date = %ld, value = %s\n",
		  t_date, c_val);
          dba_destroy(db);
          exit(1);
        }
      }
    }
  }
  dba_destroy(db);
  return(0);
}
  

