#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>

#include <string.h>
#include <ndbm.h>

#define SYSNAMEMAX 20
#define DATELEN 7
#define LONGLEN 16
#define LLONGLEN 32

#ifndef min
#define        min(a,b) (((a)<(b))?(a):(b))
#endif

typedef long long int64_t;

typedef struct{
  char sysname[SYSNAMEMAX];
  char date[DATELEN];
  char dirs[LONGLEN];
  char files[LONGLEN];
  char arcfiles[LONGLEN];
  char nonresfiles[LONGLEN];
  char nonarcfiles[LONGLEN];
  char arcdata[LLONGLEN];
  char nonresdata[LLONGLEN];
  char nonarcdata[LLONGLEN];
  char data[LLONGLEN];
}DATAREC;

static char* monnames[] = { "Jan", "Feb", "Mar", "Apr", "May", 
			      "Jun", "Jul", "Aug", "Sep", "Oct",
		      "Nov", "Dec" };


#define DBDIR "/usr/mss/lib/metrics/DB"

                                               /* RES = TOT - NONRES */
DBM *DIRS, *FILES, *ARCFILES, *NRFILES, *NAFILES, *ARCH, *RES, *NARCH, *TOTAL; 
static char* dbname[] = {"DIRS", "FILES", "ARCFILES", "NRFILES", "NAFILES",
			   "ARCH", "RES", "NARCH", "TOTAL"};
#define DBNUM 9
DBM **dbarray[DBNUM];


int init_dbs(char* hostname)
{
  int i;
  char pathname[1024];

  dbarray[0] = &DIRS;
  dbarray[1] =  &FILES;
  dbarray[2] =  &ARCFILES;
  dbarray[3] = &NRFILES;
  dbarray[4] =  &NAFILES;
  dbarray[5] =  &ARCH;
  dbarray[6] = &RES;
  dbarray[7] =  &NARCH;
  dbarray[8]= &TOTAL;
  
  for(i=0 ; i < DBNUM; ++i){
    sprintf(pathname, "%s/%s/%s", DBDIR, hostname, dbname[i]);
    if((*(dbarray[i]) = dbm_open(pathname, O_RDWR|O_CREAT, 0644)) == NULL){
      fprintf(stderr, "An error occurred opening db %s\n", pathname);
      exit(1);
    }
  }
  return(0);

}


void close_dbs()
{
  int i;
  for(i=0 ; i < DBNUM; ++i)
    dbm_close(*(dbarray[i]));

}

 
void str_by_1000(char* str)
{
  int len; 

  len = strlen(str);
  if(len <= 3){
    str[0] = '\0';
    return;
  }

  str[len - 3] = '\0';
}
 
int parse_datarec(char* buffer, DATAREC* data, char* hostname)
{
  char* p;
  char  mon[4];
  int day, year;
  int i;
  long long temp;

  memset(data, '\0', sizeof(DATAREC));
  if(buffer[0] == '\n' || buffer[0] == '#' )
    return(1);
  
  sscanf(buffer, "%s %d %s %d %s %s %s %s %s %s Mb %s Mb %s Mb %s Mb", 
	 data->sysname, &day, mon, &year, data->dirs, data->files, 
	 data->arcfiles, data->nonresfiles, data->nonarcfiles, 
	 data->arcdata, data->nonresdata, data->nonarcdata, data->data);


  /* quick way to change Mb to Gb */
  
  str_by_1000(data->arcdata);
  str_by_1000(data->nonresdata);
  str_by_1000(data->nonarcdata);
  str_by_1000(data->data);


  if(hostname != NULL && strcmp(data->sysname, hostname) != 0)
    return(1);

  for(i = 0; i< 12; ++i){
    if(strcmp(monnames[i], mon) == 0){
      ++i;
      break;
    }
  }
  
  sprintf(data->date, "%02d%02d%02d", year % 100, i, day);
  return(0);
}
  
void update(DATAREC* rec)
{
  datum key;
  datum content;
  char resdata_str[LLONGLEN];
  int ret;
    
  key.dptr = rec->date;
  key.dsize = strlen(rec->date);

  content.dptr = rec->dirs;
  content.dsize = strlen(rec->dirs);
  if((ret = dbm_store(DIRS, key, content, DBM_REPLACE)) == 1) /* already in db */
    return;
  else if(ret == -1){
    fprintf(stderr, "error updating DIRS db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }

  content.dptr = rec->files;
  content.dsize = strlen(rec->files);
  if(dbm_store(FILES, key, content, DBM_REPLACE) < 0){
    fprintf(stderr, "error updating FILES db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }
   
  content.dptr = rec->arcfiles;
  content.dsize = strlen(rec->arcfiles);
  if(dbm_store(ARCFILES, key, content, DBM_INSERT) < 0){
    fprintf(stderr, "error updating ARCFILES db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }
   
  content.dptr = rec->nonresfiles;
  content.dsize = strlen(rec->nonresfiles);
  if(dbm_store(NRFILES, key, content, DBM_REPLACE) < 0){
    fprintf(stderr, "error updating NRFILES db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }
  
  content.dptr = rec->nonarcfiles;
  content.dsize = strlen(rec->nonarcfiles);
  if(dbm_store(NAFILES, key, content, DBM_REPLACE) < 0){
    fprintf(stderr, "error updating NAFILES db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }

  content.dptr = rec->arcdata;
  content.dsize = strlen(rec->arcdata);
  if(dbm_store(ARCFILES, key, content, DBM_REPLACE) < 0){
    fprintf(stderr, "error updating ARCFILES db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }
  
  sprintf(resdata_str, "%lld", rec->data - rec->nonresdata);
  content.dptr = resdata_str;;
  content.dsize = strlen(resdata_str);
  if(dbm_store(RES, key, content, DBM_REPLACE) < 0){
    fprintf(stderr, "error updating RES db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }

  content.dptr = rec->nonarcdata;
  content.dsize = strlen(rec->nonarcdata);
  if(dbm_store(NARCH, key, content, DBM_REPLACE) < 0){
    fprintf(stderr, "error updating NARCH db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }

  content.dptr = rec->data;
  content.dsize = strlen(rec->data);
  if(dbm_store(TOTAL, key, content, DBM_REPLACE) < 0){
    fprintf(stderr, "error updating TOTAL db, key = %s\n", rec->date);
    close_dbs();
    exit(1);
  }

}
  
  


main(int argc, char** argv)
{
  char buffer[1024];
  DATAREC rec;
  
  if(argc != 2){
    fprintf(stderr, "usage: update_dbm hostname\n");
    exit(1);
  }
  else if(init_dbs(argv[1]))
    exit(1);
  
  
  while(fgets(buffer, 1023, stdin)){
    if(parse_datarec(buffer, &rec, argv[1]) == 0)
      update(&rec);
  }

  close_dbs();
  exit(0);
}
  
  
