#include <stdio.h>
#include <dba.h>
#include <msscompat.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>

#ifdef sgi
#include <getopt.h>
#else
extern char *optarg;
extern int optind;
#endif


void usage()
{
  fprintf(stderr, 
	  "dump_dba (-i | -f | -c) [ -d | -t ] [-s subset] [-a tag] [-M | -G | -T] tablename\n");
  fprintf(stderr, "-i\t\tvalue is an 4 byte integer\n");
  fprintf(stderr, "-f\t\tvalue is an 8 byte floating point\n");
  fprintf(stderr, "-c\t\tvalue is null-terminated character string\n");
  fprintf(stderr,
          "\t\t(see DBA_MAX_CHARARRAY in dba.h for maximum string length)\n");
  fprintf(stderr, "-d\t\tascii date format\n");
  fprintf(stderr, "\t\t\t{YYYY-MM-DD [Hour:Minute:Second]}\n");
  fprintf(stderr, "-t\t\tUNIX time in integer seconds\n");
  fprintf(stderr, "-s\t\tif not specified subset = NULL\n");
  fprintf(stderr, "-a\t\tif not specified tag = NULL\n");
  fprintf(stderr, "-M, -G, -T\tif value is a number, divide by 1024 to the\n");
  fprintf(stderr, "\t\t\t2nd, 3rd, or 4th power, respectively\n");
}


int main(int argc, char* argv[])
{
  DbA* db;
  DBAREC val;
  int len, row, currentrow, dtype, vtype;
  char datebuf[80];
  long int ivalue;
  long int date; 
  double fvalue;
  unsigned long divisor = (unsigned long)0;
  double tdivisor;
  char* valbuf;
  int teramode = 0, c;
  char sub_set[32];
  char ta_g[32];
  
  DBARECINIT(val);

  while((c = getopt (argc, argv, "dtifcs:a:MGT")) != EOF){
    switch((char)c){
    case 'd':
      dtype = 'd';
      break;
    case 't':
      dtype = 't';
      break;
    case 'i':
      vtype = 'i';
      break;
    case 'f':
      vtype = 'f';
      break;
    case 'c':
      vtype = 'c';
      break;
    case 's':
      strncpy(sub_set, optarg, 32);
      val.subset = &sub_set;
      break;
    case 'a':
      strncpy(ta_g, optarg, 32);
      val.tag = &ta_g;
      break;
    case 'M':
      divisor = (1024 * 1024);
      break;
    case 'G':
      divisor = (1024 * 1024 * 1024);
      break;
    case 'T':
      tdivisor = (1024.0  * 1024.0  * 1024.0  * 1024.0 );
      teramode = 1;
      break;
    }
  }
  if(optind != (argc - 1)){
    usage();
    exit(1);
  }
  if(vtype != 'i' && vtype != 'f' && vtype != 'c'){
    fprintf(stderr, "Error: must specify value type\nvtype = %d\n", vtype);
    usage();
    exit(1);
  }
  if(dtype != 'd' && dtype != 't')
    dtype = 'd';
  if((db= newDbAbstractor(argv[optind], dtype, vtype)) == NULL){
    fprintf(stderr, "%s table failed to open\n", argv[optind]);
    dba_destroy(db);
    exit(1);
  }

  if(dtype == 't'){
    val.date = &date;
    val.date_size = 4;
  }
  else{
    val.date = &datebuf;
    val.date_size = 80;
  }
  if(vtype == 'i'){
    val.value_size = 4;
    val.value = &ivalue;
  }
  else if(vtype == 'f'){
    val.value_size = 8;
    val.value = &fvalue;
  }
  else{
    val.value_size = DBA_MAX_CHARARRAY;
    val.value = &valbuf;
  }
  if((dba_first(db, &val, &row)) != 0){
    fprintf(stderr, "an error occured getting date/value pair\n");
    dba_destroy(db);
    exit(1);
  }
  if(dtype == 't')
    date = *((int*)val.date);
  else
    strcpy((char*)datebuf, val.date);
  if(vtype == 'i')
    ivalue = *((int*)val.value);
  else if(vtype == 'f')
    fvalue = *((double *)val.value);
  else
    strcpy((char*)valbuf, val.value);

  for(currentrow = 0; currentrow < row; currentrow++){
    len = strlen(datebuf);
    datebuf[len] = '\0';

    if(vtype == 'c'){
      if(dtype == 'd')
	printf("%s \t%s\n", datebuf, valbuf);
      else{
	printf("%ld \t%s\n", date, valbuf);
      }
    }
    else if(vtype == 'i'){
      if(dtype == 'd'){
	if(divisor == 0LL && !teramode){
	  printf("%s \t%ld\n", datebuf, ivalue);
	}
	else if(teramode){ 
	  fvalue = ivalue/tdivisor;
	  printf("%s \t%.3e\n", datebuf, fvalue);
	}
	else{ 
	  ivalue /= divisor;
	  printf("%s \t%ld\n", datebuf, ivalue);
	}
      }
      else{
	if(divisor == 0LL && !teramode){
          printf("%ld \t%ld\n", date, ivalue);
        }
        else if(teramode){
	  fvalue = ivalue/tdivisor;
          printf("%ld \t%.3e\n", date, fvalue);
        }
        else{
          ivalue /= divisor;
          printf("%ld \t%ld\n", date, ivalue);
        }
      }
    }
    else{
      if(dtype == 'd'){
        if(divisor == 0LL && !teramode)
	  printf("%s \t%.3e\n", datebuf, fvalue);
	else if(teramode){
	  fvalue /= tdivisor;
          printf("%s \t%.3e\n", datebuf, fvalue);
        }
	else{
	  fvalue /= divisor;
          printf("%s \t%.3e\n", datebuf, fvalue);
        }
      }
      else{
	if(divisor == 0LL && !teramode)
	  printf("%ld \t%.3e\n", date, fvalue);
	else if(teramode){
	  fvalue /= tdivisor;
          printf("%ld \t%.3e\n", date, fvalue);
        }
	else{
	  fvalue /= divisor;
          printf("%ld \t%.3e\n", date, fvalue);
        }
      }
    }
    if(currentrow < (row-1)){
      if((dba_next(db, &val)) != 0){
	fprintf(stderr, "an error occured getting next date/value pair\n");
	dba_destroy(db);
	exit(1);
      }
      if(dtype == 't')
	date = *((int*)val.date);
      else
	strcpy((char*)datebuf, val.date);
      if(vtype == 'i')
	ivalue = *((int*)val.value);
      else if(vtype == 'f')
	fvalue = *((double *)val.value);
      else
	strcpy((char*)valbuf, val.value);
    }
  }
  dba_destroy(db);
  
  return(0);
}
  







