#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <inode_walk.h>
#include <stdio.h>
#include "globals.h"
#include <histogram.h>

#ifdef RASH
#include <sys/rash.h>
#endif

/* 
   routine that is passed to inode_walk and processes each inode 
*/

void check_inodes(DINODE* ino, ino_t inum)
{
#ifdef RASH
  time_t age; 
#endif

  if((ino->di_mode & S_IFMT) == S_IFREG){
    if(ino->di_size == 0)
      return;

#ifdef FIXISIZE      /* a convex'ism */
    ino->di_size = FIXISIZE(ino->di_size);
#endif

    if(ino->di_size < 0LL){
      fprintf(stderr, "neg file size, inum %ld size %lld uid %d\n",  
	      inum, ino->di_size, ino->di_uid);
      return;
    }

    total_bytes += ino->di_size;
    ++files;
    if(ino->di_mtime > controls.newer_than && 
       ino->di_mtime < controls.older_than){
      bytes_created += ino->di_size;
      files_created++;
      if(controls.histogram)
	if(histogram(xvals, xtotal, points, intervals, ino->di_size) != 0)
	  fprintf(stderr, "error computing file size histogram, size = %lld\n",
		  ino->di_size);
    }
    if((ino->di_atime > controls.newer_than) &&
       (ino->di_atime < controls.older_than)){
#ifdef RASH
      /* hack to get around atime reset when RASH archives a file */
      if(ino->di_atime > ino->di_mtime && ino->di_atime > ino->di_ctime ){
	bytes_accessed += ino->di_size;
	files_accessed++;
	if(controls.histogram){
	  if(histogram(xvals, axtotal, apoints, intervals, 
			    ino->di_size) != 0)
	    fprintf(stderr, "error computing file size histogram, size = %lld\n", 
		    ino->di_size);
	}
      }
#else
      bytes_accessed += ino->di_size;  
      files_accessed++;
      if(controls.histogram){
	if(histogram(xvals, axtotal, apoints, intervals, ino->di_size) != 0)
	fprintf(stderr, "error computing file size histogram, size = %lld\n", 
		ino->di_size);
      }
#endif

    }

	
#ifdef RASH

    /* resident files and bytes */
    if((ino->di_rashflag & RNONR) != RNONR){
  
      resident_bytes += ino->di_size;
      ++resident_files;

      /* for resident files we also compute the average residency size by 
	 file size */

      if(ino->di_mtime < controls.older_than){
	age = controls.older_than - ino->di_mtime;
	if( age < 0){
	  fprintf(stderr, "ino = %ld, mtime = %ld, age = %ld?\n", 
		  inum, ino->di_mtime, age);
	  exit(1);
	}
	files_time += age;
	if(controls.histogram){
	  if(histogramTNG(xvals, timetotal, tipoints, intervals, 
			    ino->di_size, (long long)(age)) != 0){
	    fprintf(stderr, 
		    "error computing file size histogram, size = %lld\n", 
		    ino->di_size);
	  }
	}
      }

    }

    /* archived files stats */

    if((ino->di_rashflag & RARCH) == RARCH){
      archived_files++;
      archived_bytes += ino->di_size;
    }
#endif

    histogram(xvals, txtotal, tpoints, intervals, ino->di_size);

    
  }
}
  




