  /* $Id: fsmetrics.c,v 1.1.1.1 2000-02-28 02:18:11 wrstuden Exp $ */

/*************************** includes  *****************************/
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>

#include <inode_walk.h>
#include <histogram.h>

#include "globals.h"

void check_inodes(DINODE* , INO_T);

void update_dbs();


#include <dba.h>

int dbcount = 8;

static char* dbs[] = { "fs_tot_file", "fs_tot_byte", 
			 "arch_new_file", "arch_new_byte", 
			 "fs_new_file", "fs_new_byte",
			 "fs_acc_file", "fs_acc_byte"};

static OFF_T* vals[] = { &files, &total_bytes, &archived_files, 
			   &archived_bytes, &files_created, &bytes_created,
			   &files_accessed, &bytes_accessed };





/* handle getopt includes */
#ifdef sgi
#include <getopt.h>
#else
extern char *optarg;
extern int optind;

#endif
  

/* the obvious */
void usage()
{
  fprintf(stderr, "usage: fsmetrics [flags] device [device2 ... devicen]\n");
  fprintf(stderr, "\t-v be verbose\n");
  fprintf(stderr, "\t-h produce file histograms\n");
  fprintf(stderr, "\t-y produce NAStore metrics table for proceeding day\n");


  fprintf(stderr, "\t-m update NAStore metrics table\n");
  fprintf(stderr, "\t-d specify date other current date in the form YYYY-MM-DD\n");

}

void  print_stats()
{
  int i;

  /* print statistics */
  printf("%-25s = %10lld\n%-25s = %10lld\n", 
	 "Total files", files, "Total megabytes", total_bytes / MB);
    
#ifdef RASH   
  printf("%-25s = %10lld\n%-25s = %10lld\n", 
	 "Resident files", resident_files, 
	 "Resident megabytes", resident_bytes / MB);
  printf("%-25s = %10lld\n%-25s = %10lld\n", 
	 "Archived files", archived_files, 
	 "Archived megabytes", archived_bytes / MB);
  printf("%-25s = %10lld\n%-25s = %10lld\n", 
	 "Unarchived files", files - archived_files, 
	 "Unarchived megabytes", (total_bytes - archived_bytes) / MB);
#endif

  /* new/modified files */
  printf("%-25s = %10lld\n%-25s = %10lld\n", 
	 "New/Modified files", files_created, 
	 "New/Modified megabytes", bytes_created /MB);
  /* data that was accessed */
#ifdef RASH
  printf("\nNote: access stats are approximate\n\n");
#endif
  printf("%-25s = %10lld\n%-25s = %10lld\n", 
	 "Accessed files", files_accessed, 
	 "Accessed megabytes", bytes_accessed/MB);

  /* optional histogram */
  if(controls.histogram && (total_bytes != 0)){

    /* new and modified files */
    printf("\n\nfile size histogram, new data\n%-20s %12s %12s\n",
	   "[min:max]", "files", "megabytes");
    for(i = 0; i < intervals; ++i){
      printf("[%.1e:%.1e]   %12lld %12lld\n", 
	     1.0 * xvals[i], 1.0 * xvals[i+1], 
	     points[i], xtotal[i]/(1024 * 1000));
    }
    /* files that have been accessed */
    printf("\n\nfile size histogram, accessed data\n%-20s %12s %12s\n",
	   "[min:max]", "files", "megabytes"); 
    for(i = 0; i < intervals; ++i){
      printf("[%.1e:%.1e]   %12lld %12lld\n",
	     1.0 * xvals[i], 1.0 * xvals[i+1], 
	     apoints[i], axtotal[i]/(1024 * 1000));
    }
    /* all files histogram */
    printf("\n\nfile size histogram, all data\n%-20s %12s %12s\n",
           "[min:max]", "files", "megabytes");
    for(i = 0; i < intervals; ++i){
      printf("[%.1e:%.1e]   %12lld %12lld\n",
             1.0 * xvals[i], 1.0 * xvals[i+1],
             tpoints[i], txtotal[i]/(1024 * 1000));
    }
    /* average file residency histogram */
    printf("\n\naverage file residency histogram, all data\n%-20s %12s\n",
           "[min:max]", "day(s)");
    for(i = 0; i < intervals; ++i){
      if(tipoints[i] == 0){
	printf("[%.1e:%.1e]   %12s \n",
             1.0 * xvals[i], 1.0 * xvals[i+1], "0.0");
      }
      else
	printf("[%.1e:%.1e]   %12.1f \n",
             1.0 * xvals[i], 1.0 * xvals[i+1],
             ((1.0 * timetotal[i])/(1.0 * tipoints[i]*86400)));
    }
  }
}

void init_vars()
{
  struct tm *now;
  time_t foo;

  time(&foo);
  now = localtime(&foo);

  if(!controls.yesterday)
    controls.older_than = foo;
  else{
    foo -= (now->tm_hour * 3600);
    foo -= (now->tm_min * 60);
    /* just to push it over the edge into yesterday */
    foo -= (now->tm_sec + 1); 
    controls.older_than = foo;
    now = localtime(&foo);
  }
  controls.newer_than = controls.older_than - (24 * 3600);
  
  if(controls.verbose){
    fprintf(stderr, "scanning inodes for activity between %s and", 
	    ctime(&(controls.newer_than)));
    fprintf(stderr, " %s\n", ctime(&(controls.older_than)));
  }

/* 
  memset(xtotal, 0, sizeof(OFF_T) * intervals);
  memset(points, 0, sizeof(OFF_T) * intervals);
  memset(axtotal, 0, sizeof(OFF_T) * intervals);
  memset(apoints, 0, sizeof(OFF_T) * intervals);
*/
  total_bytes=0;
  files = 0;
  
  if(controls.metricdb && (controls.date == NULL)){
    controls.date = malloc(10);
    if(now->tm_year < 30)
      sprintf(controls.date, "%04d-%02d-%02d", now->tm_year + 2000 , 
	      now->tm_mon + 1, now->tm_mday);
    else
      sprintf(controls.date, "%04d-%02d-%02d", now->tm_year + 1900 , 
	      now->tm_mon + 1, now->tm_mday);
    
  }
}


int main(int argc, char** argv)
{
  int i, c, row;
  double buffer = NULL;
  char date[80];

  DbA* db;
  DBAREC val;
  memset((void *)(&controls), 0, sizeof(controls));
  controls.date = NULL;
  DBARECINIT(val);
  while((c = getopt (argc, argv, "hvmd:yo")) != EOF){
    switch((char)c){
    case 'h':
      if(intervals < 1)
	fprintf(stderr, 
		"cant't calculate histograms, no interval definitions compiled into code\n");
      else
	controls.histogram = 1;
      break;
    case 'y':
      controls.yesterday = 1;
      break;
    case 'd':
      controls.date = optarg;
      break;
    case 'v':
      controls.verbose = 1;
      break;
    case 'm':
      controls.metricdb = 1;
      break;
    case '?':
      usage();
      exit(1);
    }; 
  } 

  if(optind == argc){
    usage(); 
    exit(1);
  }

  init_vars();

  if(controls.metricdb){
    if((db = newDbAbstractor("fs_tot_file", 'd', 'f')) == NULL){
      fprintf(stderr, "FATAL: error opening fs_tot_file table\n");
      exit(1);
    }
    strcpy((char*)date, controls.date);
    val.date = &date;
    val.value = &buffer;
    val.value_size = 8;
    if((dba_fetch(db, &val, &row))!= 0){
      fprintf(stderr, "dba_fetch failed\n");
      dba_destroy(db);
      exit(0);
    }
    buffer = *((double *)val.value);
    if(buffer != '\0'){
      fprintf(stderr, "table already updated...exiting\n");
      dba_destroy(db);
      exit(0);
    }
    else{
      dba_destroy(db);
    }
  }

  /* walk the inodes */
  if(controls.verbose)
    fprintf(stderr, "processing");

  for(i = optind; i < argc; ++i){
    if(controls.verbose)
      fprintf(stderr, " %s", argv[i]);
#ifdef convex
    (void) inode_walk(argv[i], check_inodes);
#else
    (void) glop_walk(argv[i], check_inodes);
#endif
  }

  if(controls.verbose)
    fprintf(stderr, "\n");

  print_stats();

  if(controls.metricdb)
     update_dbs();

  return(0);
}


/* 
   tables are 
   
   fs_tot_file, fs_tot_byte,
   arch_new_file, arch_new_byte,
   fs_new_file, fs_new_byte,
   fs_acc_file, fs_acc_byte

   calculated separatly:
   fs_new_histbyte, fs_new_histfile,
   fs_acc_histbyte, fs_acc_histfile,
   fs_tot_histbyte, fs_tot_histfile

   fs_tot_time, fs_tot_histtime
   see static array above....

*/



void update_dbs()
{
  DbA* db;
  DBAREC val;
  char buffer[1024];
  char buffer2[1024];
  char date[80];
  int i;
  double zero = 0.0;
  double value;

  DBARECINIT(val);
  
  strcpy((char*)date, controls.date);
  val.date = &date;
  
  /* do the easy stuff */
  for(i = 0; i < dbcount; ++i){
    if((db = newDbAbstractor(dbs[i], 'd', 'f')) == NULL){
      fprintf(stderr, "Error opening %s table\n", dbs[i]);
      exit(1);
    }
    value = (*(vals[i]) * 1.0 );
    val.value = &value;
    /* notice that we are not storing the '\0' at the end of the string */
    if(dba_store(db, &val) !=0){
      fprintf(stderr, "Error storing to %s table\n", dbs[i]);
      exit(1);
    }
    dba_destroy(db);
  }
  if((db = newDbAbstractor("fs_tot_time", 'd', 'f')) == NULL){
    fprintf(stderr, "Error opening fs_tot_time table\n");
    exit(1);
  }
  if(files==0){
    value = zero;
  }
  else{
    value = ((1.0 * files_time)/files);
  }
  val.subset = "avg"
  val.value = &value;
  /* notice that we are not storing the '\0' at the end of the string */
  if(dba_store(db, &val)!=0){
    fprintf(stderr, "Error storing to fs_tot_time table\n");
    exit(1);
  }
  dba_destroy(db);

  /* permanent tables for New, Accessed and Total files and bytes*/
  
  if(controls.histogram){
    if((db = newDbAbstractor("fs_new_histbyte", 'd', 'c')) == NULL){
      fprintf(stderr, "Error opening fs_new_histbyte table\n");
      exit(1);
    }
    sprintf(buffer, "%d", intervals); 
    for(i = 0; i < intervals ; ++i){
      sprintf(buffer2, ":[%lld,%lld]%lld",xvals[i],xvals[i+1],xtotal[i]);
      strcat(buffer,buffer2);
    }
    val.value = &buffer;
    if(dba_store(db, &val)!=0){
      fprintf(stderr, "Error storing to fs_new_histbyte table\n");
      exit(1);
    }
    dba_destroy(db);

    if((db = newDbAbstractor("fs_new_histfile", 'd', 'c')) == NULL){
      fprintf(stderr, "Error opening fs_new_histfile table\n");
      exit(1);
    }
    sprintf(buffer, "%d", intervals);
    for(i = 0; i < intervals ; ++i){
      sprintf(buffer2, ":[%lld,%lld]%lld",xvals[i],xvals[i+1],points[i]);
      strcat(buffer,buffer2);
    }
    val.value = &buffer;
    if(dba_store(db, &val)!=0){
      fprintf(stderr, "Error storing to fs_new_histfile table\n");
      exit(1);
    }
    dba_destroy(db);
    
    if((db = newDbAbstractor("fs_acc_histbyte", 'd', 'c')) == NULL){
      fprintf(stderr, "Error opening fs_acc_histbyte table\n");
      exit(1);
    }
    sprintf(buffer, "%d", intervals);
    for(i = 0; i < intervals ; ++i){
      sprintf(buffer2, ":[%lld,%lld]%lld",xvals[i],xvals[i+1],axtotal[i]);
      strcat(buffer,buffer2);
    }
    val.value = &buffer;
    if(dba_store(db, &val)!=0){
      fprintf(stderr, "Error storing to fs_acc_histbyte table\n");
      exit(1);
    }
    dba_destroy(db);

    if((db = newDbAbstractor("fs_acc_histfile", 'd', 'c')) == NULL){
      fprintf(stderr, "Error opening fs_acc_histfile table\n");
      exit(1);
    }
    sprintf(buffer, "%d", intervals);
    for(i = 0; i < intervals ; ++i){
      sprintf(buffer2, ":[%lld,%lld]%lld",xvals[i],xvals[i+1],apoints[i]);
      strcat(buffer,buffer2);
    }
    val.value = &buffer;
    if(dba_store(db, &val)!=0){
      fprintf(stderr, "Error storing to fs_acc_histfile table\n");
      exit(1);
    }
    dba_destroy(db);

    if((db = newDbAbstractor("fs_tot_histbyte", 'd', 'c')) == NULL){
      fprintf(stderr, "Error opening fs_tot_histbyte table\n");
      exit(1);
    }
    sprintf(buffer, "%d", intervals);
    for(i = 0; i < intervals ; ++i){
      sprintf(buffer2, ":[%lld,%lld]%lld",xvals[i],xvals[i+1],txtotal[i]);
      strcat(buffer,buffer2);
    }
    val.value = &buffer;
    if(dba_store(db, &val)!=0){
      fprintf(stderr, "Error storing to fs_tot_histbyte table\n");
      exit(1);
    }
    dba_destroy(db);
    
    if((db = newDbAbstractor("fs_tot_histfile", 'd', 'c')) == NULL){
      fprintf(stderr, "Error opening fs_tot_histfile table\n");
      exit(1);
    }
    sprintf(buffer, "%d", intervals);
    for(i = 0; i < intervals ; ++i){
      sprintf(buffer2, ":[%lld,%lld]%lld",xvals[i],xvals[i+1],tpoints[i]);
      strcat(buffer,buffer2);
    }
    val.value = &buffer;
    if(dba_store(db, &val)!=0){
      fprintf(stderr, "Error storing to fs_tot_histfile table\n");
      exit(1);
    }
    dba_destroy(db);
    
    if((db = newDbAbstractor("fs_tot_histtime", 'd', 'c')) == NULL){
      fprintf(stderr,"Error opening fs_tot_histtime table\n");
      exit(1);
    }
    sprintf(buffer, "%d", intervals);
    for(i = 0; i < intervals ; ++i){
      if(tipoints[i] == 0){
	sprintf(buffer2, ":[%lld,%lld]0",xvals[i],xvals[i+1]);
      }
      else{ 
	sprintf(buffer2, ":[%lld,%lld]%lld",xvals[i],xvals[i+1],
		(timetotal[i]/tipoints[i]));
      }
      strcat(buffer,buffer2);
    }
    val.subset = "avg";
    val.value = &buffer;
    if(dba_store(db, &val)!=0){
      fprintf(stderr, "Error storing to fs_tot_histtime table\n");
      exit(1);
    }
    dba_destroy(db);
  }
}




