#ifndef DBA_IMPL_H
#define DBA_IMPL_H

#include <libpq-fe.h>

#define DBA_CONFFILE "/usr/local/etc/metriclib.cf"

typedef struct _DBAREC {
  void* date;
  int date_size;
  void* value;
  int value_size;
  void* start_date;
  void* subset;
  void* tag;
} DBAREC;

struct _DBA_Info{
  char description[80];
  char data_units[20];
};

typedef struct _DBA_Info DbA_Info;

struct _DBA_IMPL{
/***
  general stuff 
***/

  char* tablename;
/*   char* process_id; */
  void *_impl_ref;
  void *_dba_ref;
  int db_errno;
  int date_type, value_type;

/*****
  query state info 
*****/

  PGresult* result;
  int rows;
  int current_row;

/***
   backend db dependent stuff 
***/

  char* pg_db;
  char* backend_host;
  char* backend_port;
  PGconn* db;
  int pg_status, transaction;
  char  *q_buffer;

/***
  pointers to functions 
***/ 

  /* returns DBA_SUCCESS on success, else failure */

  int (*store)(struct _DBA_IMPL* impl, DBAREC* rec);
  int (*fetch)(struct _DBA_IMPL* impl, DBAREC* rec, int* rows);

  int (*first)(struct _DBA_IMPL* impl, DBAREC* rec, int* rows);
  int (*last)(struct _DBA_IMPL* impl, DBAREC* rec, int* rows);
  int (*next)(struct _DBA_IMPL* impl, DBAREC* rec);
  int (*update)(struct _DBA_IMPL* impl, DBAREC* rec, void* new_value);

  int (*remove)(struct _DBA_IMPL* impl);

  int (*get_info)(struct _DBA_IMPL* impl, DbA_Info* info );
  int (*set_info)(struct _DBA_IMPL* impl, DbA_Info* info );
  
  int (*begin)(struct _DBA_IMPL* impl);
  int (*end)(struct _DBA_IMPL* impl);

  int (*open)(struct _DBA_IMPL* impl);

  /* void */
  void (*destroy)(struct _DBA_IMPL* impl);

};

typedef struct  _DBA_IMPL DbA_impl;

void _impl_destroy( DbA_impl* impl);
DbA_impl* _impl_create( char *name, int date_type, 
		       int value_type); 


#endif





