#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>


void usage()
{
  fprintf(stderr, "usage: join_metrics output_file_1 output_file_2\n");
}


struct line_info{
  int month, day, year;
  int yday;
  long files;
  long bytes;
  int printed;
};


void print_info(struct line_info* info)
{
  if(!info->printed){
    printf("%02d|%02d|%d|%d|%ld|%ld\n", info->month,
	   info->day, info->year, info->yday, 
	   info->files, info->bytes);
    info->printed = 1;
  }
}


void parse_info(char* line, struct line_info* info)
{
  char buffer[1024], *p;
  int *foo, i;

  info->printed = 0;

  /* I know that this sucks, makes assumptions about padding 
     and alignment in structures */
  foo=(int *)info;
  strcpy(buffer, line);
  p = strtok(buffer, "|\n");

  for(i = 0; i < 4; ++i){
    if(p == NULL){
      fprintf(stderr, "join_metrics: bad input to parse_info, field %d, line: %s\n", i + 1, line);
      exit(1);
    }
    foo[i] = atoi(p);
    p = strtok(NULL, "|\n");
  }

  if((p) == NULL){
    fprintf(stderr, "bad input to parse_info, field 5. line: %s\n", line);
    exit(1);
  }
  else
    info->files = atol(p);

  if((p = strtok(NULL, "|\n")) == NULL){
    fprintf(stderr, "bad input to parse_info, field 6, line: %s\n", line);
    exit(1);
  }
  else
    info->bytes = atol(p);
}


void add_info(struct line_info* a, struct line_info* b)
{
  if((a->yday != b->yday) || (a->year != b->year) ){
    fprintf(stderr, "add_info: ydays don't match\n");
    print_info(a);
    print_info(b);
    exit(1);
  }
  
  a->files += b->files;
  a->bytes += b->bytes;
  a->printed = 0;
}


int main( int argc, char** argv)
{
  
  char linebuf[2][1024];
  FILE* fp[2], *last_fp;
  int ateof[2];
  int i;
  struct line_info info[2];
  
  if(argc != 3){
    usage();
    exit(1);
  }

  /* open files */
  for(i = 0; i < 2; ++i){
    if((fp[i] = fopen(argv[i + 1], "r")) == NULL){
      perror(argv[i+1]);
      exit(1);
    }
    ateof[i]=0;
  }
  
  while( ateof[0] == 0 && ateof[1] == 0){
    /* read  a line from each file, must begin with a number, i.e.
       that's how we see if it looks like it is a good line with a date at 
       the beginning */
    for(i = 0; i<2; ++i){
      if(!ateof[i]){
	do{
	  ateof[i] = (fgets(linebuf[i], 1024, fp[i]) == NULL);
	}while(!isdigit(linebuf[i][0]) && !ateof[i]);
      }
    }
    for(i = 0; i<2; ++i){
      if(!ateof[i])
	parse_info(linebuf[i], &(info[i]));
    }
    
    /* if files are out of sync, force the file with the older entry 
       to catch up */
  resync:
    if(ateof[0] || ateof[1])
      break;

    if( (info[0].year < info[1].year) ||
       ((info[0].year == info[1].year)  && (info[0].yday <  info[1].yday))){
      print_info(&(info[0]));
      do{
	if((ateof[0] = (fgets(linebuf[0], 1024, fp[0]) == NULL)) == 1){
	  print_info(&(info[1]));
	  break;
	}
      }while(!isdigit(linebuf[0][0]));
      if(!ateof[0])
	parse_info(linebuf[0], &(info[0]));
      goto resync;
    }
    else if((info[0].year > info[1].year) || 
	    ((info[0].year == info[1].year) && (info[0].yday > info[1].yday))){
      print_info(&(info[1]));
      do{
	if((ateof[1] = (fgets(linebuf[1], 1024, fp[1]) == NULL)) == 1){
	  print_info(&(info[0]));
	  break;
	}
      }while(!isdigit(linebuf[1][0]));
      if(!ateof[0])
	parse_info(linebuf[1], &(info[1]));
      goto resync;
    }
    else if((info[0].year == info[1].year) && (info[0].yday ==  info[1].yday)){
      /* the above should have us in sync now */
      add_info(&(info[0]), &(info[1]));
      print_info(&(info[0]));
    }
    else{
      fprintf(stderr, "missed a case, should get here\n");
      exit(1);
    }

  }

  /* if one file has remaining lines, just output them verbatium */
  if(ateof[0] && ateof[1]){
  }
  else {
    fputs(linebuf[ateof[0]], stdout); /* print out last line we read */
    last_fp=fp[ateof[0]]; /* trickery to get remaining file pointer */
    while(fgets(linebuf[0], 1024, last_fp) != NULL)
      fputs(linebuf[0], stdout);
  }

  for(i = 0; i<2; ++i)
    fclose(fp[i]);

  exit(0);
}  
    

