#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <datelib.h>



/* ranges as per struct tm */
void yday2MMDD(int yday, int year, int* mon, int* day)
{
  int m=1 , d;
/*
  struct tm foo;
  int trash;
*/
  while(yday > last_day(m, year)){
    yday -= last_day(m++, year);
  }
  d = yday;
/*
  memset(&foo, 0, sizeof(foo));
  foo.tm_yday = yday;
  if(year > 200)
    year -= 1900;
  foo.tm_year = year;
  foo.tm_isdst = -1;
  trash = mktime(&foo);
  if(trash == -1){
    perror("mktime");
    exit(1);
  }
  *mon = foo.tm_mon;
  *day = foo.tm_mday;
*/
  *mon = m;
  *day = d;
}

  
/* convoluted crap. take month and day, find period */
int periodofyear(int mon, int day, int year, int modulus)
{
  static int lastday, lastmon = -1;
  static int last_period = -1;
  int i, yday = 0;

  /* cache the last value looked up */
  if(last_period != -1 && lastmon == mon && day == lastday )
    return(last_period);

  for(i = 1; i < mon; ++i)
    yday += last_day(i, year);
  yday += day;
  lastday = day; 
  lastmon = mon;
  last_period = (yday / modulus) + (modulus != 1); 
  return(last_period);

}

/* returns last day of any month */
int last_day(int month, int year)
{
  static result, last_month = -1, last_year;
  
  if(month == last_month && year == last_year)
    return(result);
  
  last_month = month;
  last_year = year;

  if(month == 1 || month == 3 || month == 5 || month == 7 || month == 8 || month == 10 || month == 12){
    result = 31;
    return 31;
  }
  if(month == 2){
    if(year%4==0){
      if(year%2000==0){
	result = 28;
	return 28;
      }
      else{
	result = 29;
	return 29;
      }
    }
    else{
      result = 28;
      return 28;
    }
  }
  else{
    result = 30;
    return 30;
  }
}

double periodtojulian(int period, int year, int modulus)
{
  double julian;

  julian = julday(1, 1, year, 0, 0, 0.0);
  julian += ((period - 1) * (1.0 * modulus));
  return(julian);
}

    
/*
** Takes a date, and returns a Julian day. A Julian day is the number of
** days since some base date  (in the very distant past).
** Handy for getting date of x number of days after a given Julian date
** (use jdate to get that from the Gregorian date).
** Author: Robert G. Tantzen, translator: Nat Howard
** Translated from the algol original in Collected Algorithms of CACM
** (This and jdate are algorithm 199).
*/
double julday(int mon, int day, int year, int h, int mi, double se)
{
    long m = mon, d = day, y = year;
    long c, ya, j;
    double seconds = h * 3600.0 + mi * 60 + se;

    if (m > 2)
        m -= 3;
    else {
        m += 9;
        --y;
    }
    c = y / 100L;
    ya = y - (100L * c);
    j = (146097L * c) / 4L + (1461L * ya) / 4L + (153L * m + 2L) / 5L + d + 1721119L;
    if (seconds < 12 * 3600.0) {
        j--;
        seconds += 12.0 * 3600.0;
    }
    else {
        seconds = seconds - 12.0 * 3600.0;
    }
    return (j + (seconds / 3600.0) / 24.0);
}


/* Julian date converter. Takes a julian date (the number of days since
** some distant epoch or other), and returns an int pointer to static space.
** ip[0] = month;
** ip[1] = day of month;
** ip[2] = year (actual year, like 1977, not 77 unless it was  77 a.d.);
** ip[3] = day of week (0->Sunday to 6->Saturday)
** These are Gregorian.
** Copied from Algorithm 199 in Collected algorithms of the CACM
** Author: Robert G. Tantzen, Translator: Nat Howard
*/

GREGINFO* calcdate(double jd)
{
    static GREGINFO ret;
    long j = jd;
    double  frac = jd - j;

    ret.julian = jd;
    if (frac >= 0.5) {
        frac = frac - 0.5;
        j++;
    }
    else {
        frac = frac + 0.5;
    }

    ret.wday = (j + 1L) % 7L;
    j -= 1721119L;
    ret.year = (4L * j - 1L) / 146097L;
    j = 4L * j - 1L - 146097L * ret.year;
    ret.day = j / 4L;
    j = (4L * ret.day + 3L) / 1461L;
    ret.day = 4L * ret.day + 3L - 1461L * j;
    ret.day = (ret.day + 4L) / 4L;
    ret.month = (5L * ret.day - 3L) / 153L;
    ret.day = 5L * ret.day - 3 - 153L * ret.month;
    ret.day = (ret.day + 5L) / 5L;
    ret.year = 100L * ret.year + j;
    if (ret.month < 10)
        ret.month += 3;
    else {
        ret.month -= 9;
        ret.year++;
    }

    return(&ret);
}




