/*
 * inode_walk -- walk through the inodes for a given filesystem
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>

#include <sys/dir.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/fcntl.h>
#include <pwd.h>
#include <errno.h>

#include <msscompat.h>



#ifdef RASH
#include <sys/rash.h>
#endif


/**** defines *********************************************************/


#define	MAXNINDIR	(MAXBSIZE / sizeof (daddr_t))

#ifdef __convex__
#define OPENFLAGS (O_RDONLY | O_LARGEFILE | O_NONBLOCK)
#else 
#define OPENFLAGS (O_RDONLY | O_NONBLOCK)
#endif

#ifdef sgi 
struct efs sblock;
#ifndef EFS_NEWMAGIC
#define MAGIC EFS_MAGIC
#else
#define MAGIC EFS_NEWMAGIC
#endif
#ifndef DINODE
#define DINODE struct efs_dinode
#endif

#else   /* all other normal sane os'es */
struct fs       sblock;
#define MAGIC FS_MAGIC
#ifndef DINODE
#define DINODE struct dinode
#endif
#endif


static void
bread(int fd, daddr_t bno, char *buf, unsigned cnt)
{
  register        i;

  (void) LSEEK(fd, (OFF_T) bno * DEV_BSIZE, SEEK_SET);
  if (read(fd, buf, cnt) != cnt) {
    (void) fprintf(stderr, "inode_walk: read error %ld\n", bno);
    for (i = 0; i < cnt; i++)
      buf[i] = 0;
  }
}

/* 
inode_walk(): returns number of inodes processed or, on error, -1 
*/

int inode_walk(char* device, void (*f)(DINODE*, ino_t))
{
  register int    i, j, c;
  INO_T ino = 0;
  int fi;
  DINODE   *itab;
  long inodes_per_group;

  fi = open(device, OPENFLAGS);
  if (fi < 0) {
    perror("open");
    (void) fprintf(stderr, "inode_walk(): cannot open %s\n", device);
    return(-1);
  }
  
  sync();
  bread(fi, SBLOCK, (char *) &sblock, sizeof(sblock));
  if (sblock.fs_magic != MAGIC) {
    (void) printf("%s: not a file system\n", device);
    close(fi);
    return(-1);
  }

#ifdef __convex__
  inodes_per_group = sblock.fs_ipg;
#endif
#ifdef __NetBSD__
  inodes_per_group = sblock.fs_ipg;
#endif
#ifdef sgi
  inodes_per_group = sblock.fs_ipcg;
#endif

  if((itab = (DINODE*)malloc(inodes_per_group * sizeof(DINODE))) 
     == NULL){
    perror("malloc:could not allocate inode array");
    exit(1);
  }
#ifdef BROKEN
  return(0);
#else
  
  for (c = 0; c < sblock.fs_ncg; c++) {
    bread(fi, fsbtodb(&sblock, cgimin(&sblock, c)), (char *) itab,
	  inodes_per_group * sizeof(DINODE));
    for (j = 0; j < inodes_per_group; j++) {
      if (itab[j].di_mode != 0){
	f(&itab[j], ino);
      }
      ino++;
    }
  }
  close(fi);
  free(itab);
  return(ino);
#endif
}

