#include <dba.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

#include "util.h"

#ifndef PATH_MAX
#define PATH_MAX 1024
#endif


#define MAXDATE 80

/*********************************************

impl member functions 

**********************************************/

int _impl_begin(DbA_impl* impl)
{
  sprintf(impl->q_buffer, "BEGIN");

  impl->db_errno = DBA_ERROR;
  if(sql_exec(impl) != 0){
    fprintf(stderr, "libdba: Error BEGINing the transaction\n");
    return(impl->db_errno);
  }
  impl->transaction =  1;
  impl->db_errno = DBA_SUCCESS;
  return(impl->db_errno);
}


int _impl_end(DbA_impl* impl)
{
  impl->db_errno = DBA_ERROR;
  if(impl->transaction != 1){
    fprintf(stderr, "libdba: BEGIN must be used before END\n");
    return(impl->db_errno);
  }
  sprintf(impl->q_buffer, "END");

  if(sql_exec(impl) != 0){
    fprintf(stderr, "libdba: Error ENDing the transaction\n");
    return(impl->db_errno);
  }
  impl->transaction = 0;
  PQfinish(impl->db );
  impl->db = NULL;
  impl->db_errno = DBA_SUCCESS;
  return(impl->db_errno);

}


int _impl_fetch(DbA_impl* impl, DBAREC* rec, int *rows)
{
  char* p;
  
  /* query table using date_type specified */
  if(impl->date_type == 'd'){
    sprintf(impl->q_buffer, "SELECT dtime, value FROM %s WHERE dtime = '%s' ",
	    impl->tablename, (char*)(rec->date));
  }
  else{
    sprintf(impl->q_buffer, "SELECT utime, value FROM %s WHERE utime = %ld ",
	    impl->tablename, *((time_t*)(rec->date)));
  }
  if(rec->subset != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and subset = '%s' ", (char *)rec->subset);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and subset is NULL");
    
  if(rec->tag != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and tag = '%s' ", (char *)rec->tag);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and tag is NULL");

  impl->db_errno = DBA_ERROR;
  if(sql_exec(impl) != 0)
    return(impl->db_errno);

  *rows = impl->rows;

  if(impl->rows == 0){
    if(!impl->transaction)
      cleanup_query(impl);
    return(impl->db_errno);
  }

  /* retrieve data from query result */
  p =  PQgetvalue(impl->result, 0, 1);
  impl->current_row += 1;

  return_value(p, impl, rec->value, rec->value_size);

  if((impl->rows == impl->current_row) && !impl->transaction)
    cleanup_query(impl);

  return(impl->db_errno);
}


int _impl_first(DbA_impl* impl, DBAREC* rec, int *rows)
{
  char* p;
  char* q;
  char *dstring;
  
  if(impl->date_type == 'd')
    dstring = "dtime";
  else
    dstring = "utime";
  
  /* query table using date_type specified */
  sprintf(impl->q_buffer, "SELECT %s, value FROM %s",
	  dstring, impl->tablename);

  if(rec->subset != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " where subset = '%s' ", (char *)rec->subset);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " where  subset is NULL");
    
  if(rec->tag != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and tag = '%s' ", (char *)rec->tag);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and tag is NULL");

  sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	  " ORDER BY %s", dstring);

  if(sql_exec(impl) != 0)
    return(impl->db_errno);
  
  *rows = impl->rows;
  
  if(impl->rows == 0){
    if(!impl->transaction)
      cleanup_query(impl);
    return(impl->db_errno);
  }

  /* retrieve date from query result */
  p = PQgetvalue(impl->result, 0, 0);

  if(impl->date_type == 'd'){
    if((rec->date_size) < sizeof(time_t)){
      impl->db_errno = DBA_EBUFFER;
      if(!impl->transaction)
	cleanup_query(impl);

      fprintf(stderr, "libdba: date buffer too small for result\n");
      return(impl->db_errno);
    }
    strcpy((char*)(rec->date), p);
  }
  else{
    if((rec->date_size) < 4){
      impl->db_errno = DBA_EBUFFER;
      if(!impl->transaction)
	cleanup_query(impl);

      fprintf(stderr, "libdba: date buffer too small for result\n");
      return(impl->db_errno);
    }
    else
      *((int*)(rec->date)) = atoi(p);
  }

  /* retrieve value from query result */
  q =  PQgetvalue(impl->result, 0, 1);

  impl->current_row += 1;

  return_value(q, impl, rec->value, rec->value_size);

  if((impl->rows == impl->current_row) && !impl->transaction)
    cleanup_query(impl);

  return(impl->db_errno);
}


int _impl_last(DbA_impl* impl, DBAREC* rec, int *rows)
{
  char* p;
  char* dstring;

  if(impl->date_type == 'd')
    dstring = "dtime";
  else
    dstring = "utime";

  /* find last date entered into the database table*/
  sprintf(impl->q_buffer, "SELECT max(%s) FROM %s",
	  dstring, impl->tablename);

  if(rec->subset != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " where subset = '%s' ", (char *)rec->subset);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " where subset is NULL");
    
  if(rec->tag != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and tag = '%s' ", (char *)rec->tag);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and tag is NULL");
 
  if(sql_exec(impl) != 0)
    return(impl->db_errno);

  /* retrieve date from query result */
  p = PQgetvalue(impl->result, 0, 0);

  if(impl->date_type == 'd'){
    if(rec->date_size < sizeof(time_t)){
      impl->db_errno = DBA_EBUFFER;
      if(!impl->transaction)
	cleanup_query(impl);

      fprintf(stderr, "libdba: date buffer too small for result\n");
      return(impl->db_errno);
    }
    strcpy((char*)(rec->date), p);
  }
  else{
    if(rec->date_size < (4)){
      impl->db_errno = DBA_EBUFFER;
      if(!impl->transaction)
	cleanup_query(impl);

      fprintf(stderr, "libdba: date buffer too small for result\n");
      return(impl->db_errno);
    }
    else
      *((int*)(rec->date)) = atoi(p);
  }

  _impl_fetch(impl, rec, rows);

  return(impl->db_errno);

}


int _impl_next(DbA_impl* impl, DBAREC* rec)
{
  char* p;
  char* q;
  
  if(impl->current_row < 1){
    impl->db_errno = DBA_INVARG;
    cleanup_query(impl);
    return(impl->db_errno);
  }
  else if(impl->current_row >= impl->rows){
    impl->db_errno = DBA_INVARG;
    cleanup_query(impl);
    return(impl->db_errno);
  }
  
  p = PQgetvalue(impl->result, impl->current_row, 0);

  if(impl->date_type == 'd'){
    if(rec->date_size < sizeof(time_t)){
      impl->db_errno = DBA_EBUFFER;
      if(!impl->transaction)
	cleanup_query(impl);

      fprintf(stderr, "libdba: date buffer too small for result\n");
      return(impl->db_errno);
    }
    strcpy((char*)(rec->date), p);
  }
  else{
    if(rec->date_size < (4)){
      impl->db_errno = DBA_EBUFFER;
      if(!impl->transaction)
	cleanup_query(impl);

      fprintf(stderr, "libdba: date buffer too small for result\n");
      return(impl->db_errno);
    }
    else
      *((int*)(rec->date)) = atoi(p);
  }

   
  /* retrieve value in next row from query result */
  q = PQgetvalue(impl->result, impl->current_row, 1);
  impl->current_row += 1;
  
  return_value(q, impl, rec->value, rec->value_size);

  if((impl->rows == impl->current_row) && !impl->transaction)
    cleanup_query(impl);
 
  return(impl->db_errno);

}


int _impl_store(DbA_impl* impl, DBAREC* rec)
{
/*  char pro_id[24]; */
  char dtime[MAXDATE];
  char st_dtime[MAXDATE];
  time_t utime;
  time_t st_utime;
  
  if(impl->date_type == 'd'){
    if(strlen((char *)(rec->date)) > (MAXDATE - 1)){
      fprintf(stderr, "libdba: date string passed is too large\n");
      if(impl->transaction != 1)
	cleanup_query(impl);
      impl->db_errno = DBA_EBUFFER;
      return(impl->db_errno);
    }
    strcpy(dtime, (char *)(rec->date));
    utime = dtime_to_utime((char *)(rec->date));

    if(rec->start_date != NULL){
      if(strlen((char *)(rec->start_date)) > (MAXDATE - 1)){
	if(impl->transaction != 1)
	  cleanup_query(impl);
	impl->db_errno = DBA_EBUFFER;
	return(impl->db_errno);
      }
      strcpy(st_dtime, (char *)(rec->start_date));
      st_utime = dtime_to_utime((char *)(rec->start_date));
    }
  }
  else{
    utime = *((time_t*)rec->date);
    utime_to_dtime(*((time_t*)rec->date), dtime, MAXDATE);

    if(rec->start_date != NULL){
      st_utime = *((time_t*)rec->start_date);
      utime_to_dtime(*((time_t*)rec->start_date), st_dtime, MAXDATE);
    }
  }

  sprintf(impl->q_buffer, 
	  "INSERT into %s (dtime, utime, ctime, value, start_dtime, start_utime, subset, tag) VALUES ('%s', %ld, 'now',", 
	  impl->tablename, dtime, utime);

/*  if(rec->start_date != NULL)
    strcat(impl->q_buffer, ", start_dtime, start_utime");*/
/*if(process_id != NULL){
    strcpy(pro_id, (char *)process_id);
  if(strlen((char *)pro_id) > 24){
    fprintf(stderr, "libdba: process_id passed is too large\n");
  if(impl->transaction != 1)
    cleanup_query(impl);
    impl->db_errno = DBA_EBUFFER;
    return(impl->db_errno);
    }
    strcat(impl->q_buffer, ", process_id");
  }
*/
/*  if(rec->subset != NULL)
    strcat(impl->q_buffer, ", subset");
  if(rec->tag != NULL)
    strcat(impl->q_buffer, ", tag");

  sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	  ") VALUES ('%s', %ld, 'now',", dtime, utime);
*/
  if(impl->value_type == 'i')
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]),"%ld", 
	    *((long *)(rec->value)));
  else if(impl->value_type == 'f')
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]),"%e",
	    *((double *)(rec->value)));
  else{
    if(strlen((char *)(rec->value)) > DBA_MAX_CHARARRAY){
      fprintf(stderr, "libdba: value too large for storage\n");
      impl->db_errno = DBA_EBUFFER;
      if(impl->transaction != 1)
	cleanup_query(impl);
      return(impl->db_errno);
    }
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]),"'%s'", 
	    (char *)(rec->value));
  }

  if(rec->start_date != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), ",'%s', %ld", 
	    st_dtime, st_utime);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), ", NULL, NULL");

/*  if(process_id != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]),",'%s'", pro_id);
*/
  if(rec->subset != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), ",'%s'", 
	    (char *)(rec->subset));
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), ", NULL");

  if(rec->tag != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), ",'%s')", 
	    (char *)(rec->tag));
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), ", NULL)");
      
  if(sql_exec(impl) != 0)
    return(impl->db_errno);
  
  if(impl->transaction != 1){
    PQfinish(impl->db );
    impl->db = NULL;
  }

  return(impl->db_errno);

}


int _impl_update(DbA_impl* impl, DBAREC* rec, void* new_value)
{
  char* oid;
  char dstring[128];
    
  if(impl->date_type == 'd'){
    sprintf(dstring, "dtime = '%s'", (char *)rec->date);
  }else {
    sprintf(dstring, "utime = '%ld'", *((time_t *)rec->date));
  }
  
  if(impl->value_type == 'i'){
    sprintf(impl->q_buffer, 
	    "SELECT oid FROM %s WHERE %s and value = %ld", 
	    impl->tablename, dstring,  *((long *)(rec->value)));
  }
  else if(impl->value_type == 'f'){
    sprintf(impl->q_buffer, 
	    "SELECT oid FROM %s WHERE %s and value = %.16e", 
	    impl->tablename, dstring,  *((double *)(rec->value)));
  } 
  else if(impl->value_type == 'c'){
    if(strlen((char *)new_value) > DBA_MAX_CHARARRAY){
      fprintf(stderr, "libdba: value too large for storage\n");
      impl->db_errno = DBA_EBUFFER;
      return(impl->db_errno);
    }
    sprintf(impl->q_buffer, 
	    "SELECT oid FROM %s WHERE %s and value = '%s'", 
	    impl->tablename, dstring, (char *)(rec->value));
  } 

  if(rec->subset != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and subset = '%s' ", (char *)rec->subset);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and subset is NULL");
    
  if(rec->tag != NULL)
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and tag = '%s' ", (char *)rec->tag);
  else
    sprintf(&(impl->q_buffer[strlen(impl->q_buffer)]), 
	    " and tag is NULL");
  
  if(sql_exec(impl) != 0)
    return(impl->db_errno);

  if((oid = PQgetvalue(impl->result, 0, 0)) == NULL){
    impl->db_errno = DBA_INVARG;
    fprintf(stderr, "libdba: value/date pair not found\n");
    return(impl->db_errno);
  }

  if(impl->value_type == 'i'){
    sprintf(impl->q_buffer, "UPDATE %s SET value = %ld WHERE oid = %s;",
	    impl->tablename, *((long int*)new_value), oid);
  }
  if(impl->value_type == 'f'){
    sprintf(impl->q_buffer, "UPDATE %s SET value = %.16e WHERE oid = %s;",
	    impl->tablename, *((double *)new_value), oid);
  }
  if(impl->value_type == 'c'){
    sprintf(impl->q_buffer, "UPDATE %s SET value = '%s' WHERE oid = %s;",
	    impl->tablename, (char *)new_value, oid);
  }
  if(sql_exec(impl) != 0){
    impl->db_errno = DBA_ERROR;
    return(impl->db_errno);
  }
  impl->db_errno = DBA_SUCCESS;
  return(impl->db_errno);
}


int _impl_open(DbA_impl* impl)
{
  impl->db = PQsetdb(impl->backend_host, impl->backend_port, NULL, NULL, impl->pg_db);

  if(PQstatus(impl->db) == CONNECTION_BAD) {
    fprintf(stderr,"Connection to database failed.\n");
    PQfinish(impl->db);
    impl->db = NULL;
    cleanup_impl(impl);
    fprintf(stderr, "An error occurred opening db %s\n", impl->tablename);
    return(NULL);
  }
  else{
    impl->db_errno = DBA_SUCCESS;
    return(impl->db_errno);
  }
}

void _impl_destroy(DbA_impl* impl)
{
  cleanup_impl(impl);
}
  

int _impl_remove(DbA_impl* impl)
{
  remove_table(impl);
  impl->db_errno = DBA_SUCCESS;
  return(impl->db_errno);

}



int _impl_get_info(DbA_impl* impl, DbA_Info* info)
{
  char* p;
  char* q;

  sprintf(impl->q_buffer, "SELECT description, units FROM descriptions WHERE table_name = '%s';", 
	  impl->tablename);

  impl->db_errno = DBA_ERROR;
  if(sql_exec(impl) != 0)
    return(impl->db_errno);

  p = PQgetvalue(impl->result, 0, 0);
  strncpy(info->description, p, 79);
  (info->description)[79] = '\0';

  q = PQgetvalue(impl->result, 0, 1);
  strncpy(info->data_units, q, 19);
  (info->data_units)[19] = '\0';

  impl->db_errno = DBA_SUCCESS;
  return(impl->db_errno);

}


int _impl_set_info(DbA_impl* impl, DbA_Info* info)
{
  if((strlen(info->description) > 79) || (strlen(info->data_units) > 19)){
    fprintf(stderr, "libdba: info strings passed in too large\n");
    impl->db_errno = DBA_EBUFFER;
    return(impl->db_errno);
  }

  sprintf(impl->q_buffer, "UPDATE descriptions SET description = '%s', units = '%s' WHERE table_name = '%s';", info->description, info->data_units, impl->tablename);

  if(sql_exec(impl) != 0){
    impl->db_errno = DBA_ERROR;
    return(impl->db_errno);
  }
  impl->db_errno = DBA_SUCCESS;
  return(impl->db_errno);

}


DbA_impl* _impl_create( char *tablename, int date_type, int value_type)
{
  DbA_impl* impl;

  char buffer[255];
  char* p;

  impl = (DbA_impl *)(malloc(sizeof(DbA_impl)));

  /* initialize the members of the impl object */
  impl->tablename = NULL;
  impl->_dba_ref = NULL;
  impl->db_errno = DBA_SUCCESS;
  impl->pg_db = NULL; 
  impl->db = NULL;
  impl->backend_host = NULL;
  impl->backend_port = NULL;
  impl->result = NULL;
  impl->transaction = 0;
  impl->q_buffer = NULL;
  impl->rows = 0;
  impl->current_row = 0;

  
  if(! (date_type == 'd' || date_type == 't')){
    fprintf(stderr, "libdba: bad date_type specified");
    free(impl);
    return(NULL);
  }

  impl->date_type = date_type;

  if(strlen(tablename) > 31){
    fprintf(stderr, "libdba: tablename too long\n");
    free(impl);
    return(NULL);
  }
    
  switch(value_type){
  case 'i':
    impl->q_buffer = (char *)malloc(128);
    break;
  case 'f':
    impl->q_buffer = (char *)malloc(128);
    break;
  case'c':
    impl->q_buffer = (char *)malloc(128 + DBA_MAX_CHARARRAY);
    break;
  default:
    fprintf(stderr, "libdba: bad value_type specified\n");
    free(impl);
    return(NULL);
  };

  if(impl->q_buffer == NULL){
    perror("libdba:malloc");
    cleanup_impl(impl);
    return(NULL);
  }

  impl->value_type = value_type;

  if((impl->tablename = (char *)(malloc(strlen(tablename) + 1))) == NULL){
    perror("malloc");
    return(NULL);
  }
  strcpy(impl->tablename, tablename);
  impl->_impl_ref = impl;
    

  if((gethostname(buffer, 254)) != 0){
    perror("libdba: gethostname");
    cleanup_impl(impl);
    return(NULL);
  }

  if((p = strchr(buffer, '.')) != NULL)
    p[0] = '\0';

  if((impl->pg_db = (char *)(malloc(strlen(buffer) + 1))) == NULL){
    perror("libdba: malloc");
    cleanup_impl(impl);
    return (NULL);
  }

  strcpy(impl->pg_db, buffer);


  if(_read_dba_conf(impl) != 0){
    cleanup_impl(impl);
    return(NULL);
  }

  impl->db = PQsetdb(impl->backend_host, impl->backend_port, NULL, NULL, impl->pg_db);

  if (PQstatus(impl->db) == CONNECTION_BAD) { 
    fprintf(stderr,"Connection to database failed.\n"); 
    cleanup_impl(impl);
    fprintf(stderr, "An error occurred opening db %s\n", tablename);
    return(NULL);
  }
  sprintf(impl->q_buffer, "set DateStyle to 'ISO';");
  if(sql_exec(impl) != 0){
    cleanup_impl(impl);
    fprintf(stderr, "An error occurred setting DateStyle\n");
    return(NULL);
  }

  switch(check_table_exists(impl, tablename)){
  case -1:
    cleanup_impl(impl);
    return(NULL);
  case 0:
    if(create_table(impl ) != 0){
      cleanup_impl(impl);
      return(NULL);
    }
    break;
  default:
    break;
  };
       

  impl->fetch = _impl_fetch;
  impl->store = _impl_store;
  impl->first = _impl_first;
  impl->next = _impl_next;
  impl->update = _impl_update;
  impl->open = _impl_open;
  impl->destroy = _impl_destroy;
  impl->remove = _impl_remove;
  impl->get_info = _impl_get_info;
  impl->set_info = _impl_set_info;
  impl->begin = _impl_begin; 
  impl->last = _impl_last;
  impl->end = _impl_end; 


  return(impl);
}

