
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <limits.h>

#include <inode_walk.h>
#include <msscompat.h>
#include <histogram.h>
#include <sys/rash.h>

#ifdef sgi
#include <getopt.h>

#else
extern char *optarg;
extern int optind;

#endif

//should not need to do this, but include files for these C routines are a bit 
//"thin". :-)
extern "C"{
#include <rash/rash.h>
#include <rash/dblib.h>

#ifdef _CONVEX_SOURCE
char* syserr(int);
#endif

}

/***************************** globals *****************************/

/* histogram stuff */
static OFF_T xvals[] = {0, 8 * KB, 64 * KB, 256 * KB, \
			1 * MB, 16 * MB, 32 * MB, 64 * MB,\
			128 * MB, 256 * MB, \
			512 * MB, 1LL* GB, 4LL * GB, 8LL * GB, \
			LONG_LONG_MAX };
int intervals = 14;
static OFF_T xtotaldata[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static OFF_T points[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static OFF_T xdiskhitsdata[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static OFF_T points2[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

time_t newer_than = -1;
RASH_DB *rashdbp;

//is rash database for user newer than the update time. used for incremental
//update of vsn database
int rash_db_newer(char* username, time_t last_check)
{
  char path[PATH_MAX + 1];
  char buffer[PATH_MAX + 1];
  struct stat fileinfo;
  
  /* warning - db_namedir is internal to rash dblib.a*/
#ifdef OLDMSS
  db_namedir(username, path);
#else
  db_namedir(DBPATH, username, path);
#endif
  strcat(path, "/");
  strcat(path, username);
  strcpy(buffer, path);
  strcat(path, ".dat");
  if(stat(path, &fileinfo) >= 0){
    if(fileinfo.st_mtime >  last_check)
      return 1; //db is newer
  }
  else
    return 0; //no file or file is older

  strcat(buffer, ".hot");
  if(stat(buffer, &fileinfo) >= 0){
    if(fileinfo.st_mtime >  last_check)
      return 1; //hotfile has been changed
  }
  return 0; //no file or file is older
}


time_t arctime(DINODE* ino)
{
  struct pwd userinfo;
  USR_FILEINFO fileinfo;

  if(



void restorestats(DINODE* ino, ino_t inum)
{
  if((ino->di_mode & S_IFMT) == S_IFREG){
    if(ino->di_size == 0)
      return;
#ifdef FIXISIZE
    ino->di_size = FIXISIZE(ino->di_size);
#endif
    if(ino->di_size < 0LL){
      fprintf(stderr, "neg file size, inum %ld size %lld uid %d\n",  inum, ino->di_size, ino->di_uid);
      return;
    }


    if(ino->di_atime 



    if((ino->di_rashflag & RARCH) != RARCH){
       resident_bytes += ino->di_size;
       ++resident_files;
    }
    if(histogram(xvals, xtotal, points, intervals, ino->di_size) != 0)
      fprintf(stderr, "error computing file size histogram, size = %lld\n", ino->di_size);
  }
}

void usage()
{
  fprintf(stderr, "usage: fsmetrics device [device2 ... devicen]\n");
}


main(int argc, char** argv)
{
  int i;
  
  if(argc == 1){
    usage();
    exit(1);
  }

  for(i = 1; i < argc; ++i){
    memset(xtotal, 0, sizeof(OFF_T) * intervals);
    memset(points, 0, sizeof(OFF_T) * intervals);
    total_bytes=0;
    files = 0;
    resident_bytes = 0;
    resident_files = 0;
    
    inode_walk(argv[i], check_inodes);
    printf("%lld files totalling %lld bytes\n", files, total_bytes);
    printf("%lld resident bytes in %lld resident files\n", 
	   resident_bytes, resident_files);

    if(total_bytes != 0){
      printf("\t\tHistogram data\nx[i]\tx[i+1]avg_size\tpercent_data\n");
      for(i = 0; i < intervals; ++i){
	if(points[i] == 0)
	  printf("%lld\t%lld\t%lld\t.000\n", xvals[i], xvals[i+1] ,
		 xvals[i] );
	else
	  printf("%lld\t%lld\t%lld\t%.3f\n", xvals[i], xvals[i+1], 
		 xtotal[i] / points[i], (1.0 * xtotal[i])/total_bytes);
      }
    }

  }

}

