
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <limits.h>

#include <inode_walk.h>
#include <msscompat.h>
#include <histogram.h>
#include <sys/rash.h>

#ifdef sgi
#include <getopt.h>

#else
extern char *optarg;
extern int optind;

#endif

enum {notfile=-1, nonarc = 0, arcres = 1, nonresident =2};
#define MAXTYPE  2
typedef int index;

/***************************** globals *****************************/
OFF_T bytes[MAXTYPE + 1];
OFF_T files[MAXTYPE + 1];
OFF_T accessbytes[MAXTYPE + 1];
OFF_T accessfiles[MAXTYPE + 1];
OFF_T modifybytes[MAXTYPE + 1];
OFF_T modifyfiles[MAXTYPE + 1];

#ifdef sgi
#include <getopt.h>

#else
extern char *optarg;
extern int optind;

#endif

time_t lower_bound = -1;
time_t upper_bound = ~((time_t)(0))

void usage()
{
  fprintf(stderr, "usage: totalfiles [-t seconds] [-s]\n");
  fprintf(stderr, "\t-t seconds \tany file newer than seconds is new\n");
  fprintf(stderr, "\t-s\t\tbe silent, avoid unneeded output to stdout\n");
}


/* 
   determine which type of file we are dealing with as per types 
   defined in enum above
*/
index fileclass(DINODE* ino)
{
  if((ino->di_mode & S_IFMT) != S_IFREG)
    return(notfile);
  else if((ino->di_rashflag & RARCH) == RARCH){
    if((ino->di_rashflag & RNONR) == RNONR)
      return(nonresident);
    else
      return(arcres);
  }
  else
    return(nonarc);
}


/* 
   routine that inode walk calls on each inode read. 
   this is where we get all of our file data, e.g. size, atime, mtime, etc.
*/
void check_inodes(DINODE* ino, ino_t inum)
{
  index i;

  if((ino->di_mode & S_IFMT) == S_IFREG){
/*     if(ino->di_size == 0)
      return; */
#ifdef FIXISIZE
    ino->di_size = FIXISIZE(ino->di_size);
#endif
    if(ino->di_size < 0LL){
      fprintf(stderr, "neg file size, inum %ld size %lld uid %d\n",  inum, ino->di_size, ino->di_uid);
      return;
    }

    /* find out what king of file we are dealing with */

    i = fileclass(ino);
    bytes[i] += ino->di_size;
    ++(files[i]);
    
    /* find files that were created or changed/overwritten */
    if(ino->di_mtime > lower_bound){
     modifybytes[i] += ino->di_size;
      ++(modifyfiles[i]);
    }
    /* find files that were read in */
    if(ino->di_atime > ino->di_mtime && ino->di_atime > lower_bound){
      accessbytes[i] += ino->di_size;
      ++(accessfiles[i]);
    } 
  }
}


main(int argc, char** argv)
{
  int i;
  OFF_T total_modfiles = 0, total_modbytes = 0, total_files = 0, 
        total_bytes =0, total_accfiles=0, total_accbytes = 0;

  while((c = getopt (argc, argv, "t:s")) != EOF){
    switch((char)c){
    case 't':
      lower_bound = time((time_t*)0) - atoi(optarg);
      break;
    case 'u':
      url = optarg; 
      break;
    }
  }
  if(optind != (argc - 2)){
    usage();
    exit(1);
  }



  /* initialize counters */
  for(i = 0; i <= MAXTYPE; ++i){
    bytes[i] = 0;
    files[i] = 0;
    accessbytes[i] = 0;
    accessfiles[i] = 0;
    modifybytes[i] = 0;
    modifyfiles[i] = 0;
  }

  /* read and process all inodes */
  inode_walk(argv[1], check_inodes);  

  /* total some types of data across all classes */
  for(i = 0; i <= MAXTYPE ; ++i){
    total_files += files[i];
    total_bytes += bytes[i];
    total_modfiles +=  modifyfiles[i];
    total_modbytes += modifybytes[i];
    total_accfiles +=  accessfiles[i];
    total_accbytes += accessbytes[i];
  }

  /* output stats */
  fprintf(stderr, 
	 "nonarcb\tarcresb\tnonresb\tfiles\tbytes\tinf\tinb\toutf\toutb\n");
  printf("%lld\t%lld\t%lld\t%lld\t%lld\t%lld\t%lld\t%lld\t%lld\n", 
	 bytes[nonarc], bytes[arcres],
	 bytes[nonresident], total_files, total_bytes, 
	 total_modfiles, total_modbytes, 
	 total_accfiles, total_accbytes);
}


