#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <stdio.h> 
#include <limits.h>
#include <dba.h>   
/* #include <nastorelib.h>*/
#include <histogram.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#define MAXDATE 80 

#if !defined(INSTROOT)

you need to add  INSTROOT as a define to cc 

#endif


#define INPUT1 "histo_input.gnuplot"
#define INPUT2 "/tmp/plotit.gnuplot"

char tmp_path[PATH_MAX + 1];
FILE* tmpfd;
int intervals;
OFF_T  *data, *files, *bounds, trash;


/* handle getopt includes */
#ifdef sgi
#include <getopt.h>
#else
extern char *optarg;
extern int optind;

#endif
  
void usage()
{
  fprintf(stderr, "usage: plothisto [-p plot_type] [flag] outfile\n");
  fprintf(stderr, "\t-t produce plot using total data/files\n");
  fprintf(stderr, "\t-n produce plot using new data/files\n");
  fprintf(stderr, "\t-a produce plot using accessed data/files\n");
}

int main(int argc, char* argv[])
{
  struct tm *now;
  time_t foo;
  int i;
  double t_bytes;
  float bigpercent;
  char data_table[32], files_table[32];
  char plot_title_1[128], plot_title_2[128];
  char date[MAXDATE];
  char *p;
  char buffer[1024];
  int row;
  char host[1024];
  
  /* start parsing options */

  char outfile[PATH_MAX + 1];
  char type[128];
  int c;
  DbA* db;
  DBAREC val;
  buffer[0] = '\0';
  type[0] = '\0';
  data_table[0] = '\0';
  files_table[0] = '\0';
  plot_title_1[0] = '\0';
  plot_title_2[0] = '\0';

  DBARECINIT(val);

  while((c = getopt (argc, argv, "p:tna")) != EOF){
    switch((char)c){
    case 'p':
      strncpy(type, optarg, 128);
      break;
  /*
    set table pair and title for graph according to chosen flag
  */
    case 't':
      strcpy(data_table,"fs_tot_histbyte");
      strcpy(files_table,"fs_tot_histfile");
      strcpy(plot_title_1, "Total");
      break;
    case 'n':
      strcpy(data_table,"fs_new_histbyte");
      strcpy(files_table,"fs_new_histfile");
      strcpy(plot_title_1, "New");
      strcpy(plot_title_2, "(based on file modification times)");
      break;
    case 'a':
      strcpy(data_table,"fs_acc_histbyte");
      strcpy(files_table,"fs_acc_histfile");
      strcpy(plot_title_1, "Accessed");
      strcpy(plot_title_2, "(based on file access times)");
      break;
    case '?':
      usage();
      exit(1);
    } 
  } 

  if(strlen(type) == 0)
    strcpy(type, "postscript color");
  
  if(strlen(data_table)==0){
    strcpy(data_table,"fs_tot_histbyte");
    strcpy(files_table,"fs_tot_histfile");
    strcpy(plot_title_1, "Total");
  }
  if(gethostname(host, 1023) != 0){
    perror("gethostname");
    exit(1);
  }
  
  if(optind != (argc - 1)){
    usage();
    exit(1);
  }

  strncpy(outfile, argv[optind], PATH_MAX);

  /* for security reasons, don't allow this program to be setuid since
     we rely on an external PATH */

  setuid(getuid());


  /* start doing table stuff */

  t_bytes = 0;
    
  /*
    open data_table table (specified data within each file size range)
  */

  if((db = newDbAbstractor(data_table, 'd', 'c')) == NULL){
    fprintf(stderr, "Error opening %s table\n", data_table);
    exit(1);
  }

  /****
     make date for today 
  *****/

  time(&foo);
  now = localtime(&foo);

  foo -= (now->tm_hour * 3600);
  foo -= (now->tm_min * 60);

  /* just to push it over the edge into yesterday */
  foo -= (now->tm_sec + 1);
  now = localtime(&foo);

  if(now->tm_year < 30)
    sprintf(date, "%04d-%02d-%02d", now->tm_year + 2000,
	    now->tm_mon + 1, now->tm_mday);
  
  else
    sprintf(date, "%04d-%02d-%02d", now->tm_year + 1900,
	    now->tm_mon + 1, now->tm_mday);

  val.date = &date;
  val.value = &buffer;
  val.value_size = 1024;

  /* fetch value associated with date */
  if((dba_fetch(db, &val, &row)) != 0){
    fprintf(stderr, "Error fetching value associated with date\n");
    exit(1);
  }
    /* or return value associated with largest date */
  if(buffer[0] == '\0'){
    date[0] = '\0';
    val.date = &date;
    val.date_size = MAXDATE;
    if(dba_last(db, &val, &row) !=0){
      fprintf(stderr, 
	      "error retrieving last value associated with largest date");
      dba_destroy(db);
      exit(1);
    }
    strcpy((char*)date, val.date);
  }

  /* parse values associated with date   */
  strcpy((char*)buffer, val.value);
  if((p = strtok(buffer, ":")) == NULL){
    fprintf(stderr, 
	    "Could not parse value string from %s for number of intervals\n", 
	    data_table);
    dba_destroy(db);
    exit(1);
  }

  intervals = atoi(p);

  /* 
    now that we have the number of intervals, allocate storage for 
    the data arrays 
    */

  data = (OFF_T *)(malloc(intervals * sizeof(OFF_T)));
  files = (OFF_T *)(malloc(intervals * sizeof(OFF_T)));
  bounds = (OFF_T *)(malloc((1 + intervals) * sizeof(OFF_T)));

  /*
    scan and store number of bytes within each file size range 
    */ 
  
  for(i = 0; i< intervals; ++i){
    if((p = strtok(NULL, ":")) == NULL){
      fprintf(stderr, 
	      "Could not parse value string from %s table\n", data_table);
      dba_destroy(db);
      exit(1);
    }
    sscanf(p, "[%lld,%lld]%lld", &(bounds[i]), &(bounds[i+1]), &(data[i]));
    t_bytes+=data[i];
  }
  dba_destroy(db);


  /* 
     open files_table table (number of specified files in each range)
  */
  buffer[0] = '\0';
  
  if((db = newDbAbstractor(files_table, 'd', 'c')) == NULL){
    fprintf(stderr, "Error opening %s table\n", files_table);
    exit(1);
  }
  val.value = &buffer;
  if((dba_fetch(db, &val, &row)) != 0){
    fprintf(stderr,
         "Error getting value from %s table for date %s\n", files_table, date);
    exit(1);
  }

  /* 
     parse values associated with date 
  */
  strcpy((char*)buffer, val.value);
  if( (p = strtok(buffer, ":")) == NULL){
    fprintf(stderr, 
	    "Could not parse value string from %s for number of intervals\n", files_table);
    dba_destroy(db);
    exit(1);
  }

  /*
    scan and store number of specified files within each file size range 
  */
  
  for(i = 0; i< intervals; ++i){
    if( (p = strtok(NULL, ":")) == NULL){
      fprintf(stderr,
              "Could not parse value string from %s table\n", files_table);
      dba_destroy(db);
      exit(1);
    }
    sscanf(p, "[%lld,%lld]%lld", &trash, &trash, &(files[i]));
  }
  dba_destroy(db);

  /* 
     done doing table reads, now we make a temp data file for gnuplot
  */

  tmpnam(tmp_path);
  if((tmpfd = fopen( tmp_path, "w+")) == NULL){
    perror("fopen");
    exit(1);
  }
  bigpercent=0.0;

  /* 
    place values in a temp file 
  */
  
  for(i = 0; i< intervals; ++i){
    if(t_bytes == 0){
      fprintf(stderr,
              "Can not divide by zero (Total specified bytes = %e\n", t_bytes);
      exit(1);
    }

    if((100.0*(data[i]/t_bytes))> bigpercent){
      bigpercent = (100.0*(data[i]/t_bytes));
    }
    
    /*
      place data in temp file only if files[i] and data[i] are not zero
      */

    if(files[i] != 0 && data[i] != 0 && (data[i]/files[i]) >= bounds[i]){
      fprintf(tmpfd, "%lld\t%e\t%lld\n", bounds[i],
	      (100.0*(data[i]/t_bytes)), (data[i]/files[i]));
    } 
  }
  
  /* 
    find last nonzero files[i] or data[i], 
    and determne that average file size is within bounds 
    */
  
  i=i-1;
  for(;files[i]==0 || data[i]==0 || (data[i]/files[i])< bounds[i];--i);
  
  
  /* 
    repeat last nonzero dataset with increased bound for understandable plot
    */
  
  if (i==(intervals-1)){
    if((data[i]/files[i])<(bounds[i]*10)){
      fprintf(tmpfd,"%lld\t%e\t%lld\n", (bounds[i]*10),
	      (100.0*(data[i]/t_bytes)), (data[i]/files[i]));
    }
    else{
      fprintf(tmpfd,"%lld\t%e\t%lld\n",
	      ((data[i]/files[i])+ (data[i]/files[i])-(bounds[i])),
	      (100.0*(data[i]/t_bytes)), (data[i]/files[i]));
    }
  }
  
  else{
    fprintf(tmpfd,"%lld\t%e\t%lld\n", (bounds[i+1]),
	    (100.0*(data[i]/t_bytes)), (data[i]/files[i]));
  }
  
  fclose(tmpfd);
  
  /* done building temp data file */
  
  
  /* 
    now build the final command line input file for gnuplot. 
    This sets the output file and file type.
    */
  
  if((tmpfd = fopen(INPUT2, "w+")) == NULL){
    perror("fopen");
    exit(1);
  }
  
  
  fprintf(tmpfd, 
	  "set title \"Percentage of %s Data by File Size  - %s \\n %s\"\n", 
	  plot_title_1, host, plot_title_2);
  fprintf(tmpfd, "set term %s\n", type);
  fprintf(tmpfd, "set output \"%s\"\n", outfile);
  fprintf(tmpfd, "set yrange [0:%f]\n", (bigpercent + 5.0));
  fprintf(tmpfd, "plot '%s' using 1:2 title'' with steps, '%s' using 3:2 title'Average File Size in Each Range' with linespoints, '%s' using 1:2 title'' with impulses\n", tmp_path, tmp_path, tmp_path);
  
  fclose(tmpfd);
  
  sprintf(buffer, "/usr/local/bin/gnuplot %s/lib/%s %s", INSTROOT, INPUT1, INPUT2);

  if((tmpfd = popen(buffer, "r")) == NULL){
    perror("popen");
    exit(1);
  }

  while(fgets(buffer, 1023, tmpfd) != NULL)
    fputs(buffer, stdout);

  pclose(tmpfd);

  unlink(tmp_path);
  exit(0);
}






