#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#define OFF_T long long
#include "datelib.h"

int year;
#define MAXDAYS 366

/* megabytes */
#define UNITS (1024 * 1024)

/* caution, built for speed, not robustness. error checking is off */


long long str2int8(char* str)
{
  long long foo;
#ifdef __NetBSD__
  foo = strtoq(str, (char **)NULL, 0);
  return(foo);
#else
  return(atoll(str));
#endif
}


int main(int argc, char** argv)
{
  FILE* fp = stdin;
  char *p, linebuffer[1024], buffer[1024 * 64];
  int month, day,  i, last_yday = -1, yday, ydays;
  OFF_T filesize, xtotal[MAXDAYS + 1], points[MAXDAYS + 1];

  /* parse args, check for stdin vs files on cmd line usage, 
     initialize arrays */
  if(argc  == 2){
    year = atoi(argv[1]);
    if(year < 100){
      if(year > 70)
	year += 1900;
      else
	year += 2000;
      fprintf(stderr, "%s was a long time ago, assuming %d\n", argv[1], year);
    }
     
  }
  else{
    fprintf(stderr, "usage: rashlogstat starting_year\n");
    exit(1);
  }

  setvbuf(fp, buffer, _IOFBF, 1024 * 64);

  /* read loop for each file */
  while(fgets(linebuffer, 1024, fp) != NULL){

    /* get line, parse fields */
    if((p = strtok(linebuffer, " /\t\n")) == NULL)
      continue;

    month = atoi(p);
    if((p = strtok(NULL, " /\t\n")) == NULL)
      continue;
    day = atoi(p);
    if((p = strtok(NULL, " /\t\n")) == NULL)
      continue;
    filesize = str2int8(p);
    
    /* get yday number */
    if((yday = periodofyear(month, day, year, 1)) == -1)
      continue;

    /* see if we just rolled over into a new year.
       if so, print out stats for the previous year, and
       reinitialize the arrays */
    /* some log files have been screwed up in the past. 
       The month == 1 part minimizes the problem */

    if(yday < last_yday && month == 1){	
      ydays = 365;
      if(((year % 4) == 0) && ((year % 2000) != 0))
	++ydays;

      for(i = 1; i <= ydays; ++i){
	yday2MMDD((i) , year, &month, &day);
	printf("%02d|%02d|%d|%d|%lld|%lld\n", 
	       month , day, year, i,
	       points[i], xtotal[i]/UNITS);
      }
      memset(&xtotal, 0, ((MAXDAYS + 1) * sizeof(OFF_T)));
      memset(&points, 0, ((MAXDAYS + 1) * sizeof(OFF_T)));
      ++year;
    }
      
    /* count stats from last line read */
    ++(points[yday]);
    xtotal[yday] += filesize;
    last_yday = yday;
  }
  /* end of read loop */

  
  /* print out partial results for the last year being processed */
  ydays = 365;
  if(((year % 4) == 0) && ((year % 2000) != 0))
    ++ydays;
  for(i = 1; i <= ydays; ++i){
    yday2MMDD((i), year, &month, &day);
    printf("%02d|%02d|%d|%d|%lld|%lld\n", 
	   month, day, year, i,
	   points[i], xtotal[i]/UNITS);
  }
  exit(0);
}













