#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <datelib.h>
#include <string.h>
#ifdef sgi
#include <getopt.h>

#else
extern char *optarg;
extern int optind;

#endif

typedef struct {
  int month;
  int day;
  int year;
  int yday;
  size_t files;
  size_t data; 
} DBRECORD;

double julian_offset;

#define USE_JULIAN 0x1

int isEqual(DBRECORD* a, DBRECORD* b)
{
  return (a->month == b->month && a->day == b->day && a->year == b->year);
}

int isBefore(DBRECORD* a, DBRECORD *b)
{
  if(a->year < b->year)
    return(1);
  else if(a->year > b->year)
    return(0);
  else if(a->month < b->month)
    return(1);
  else if(a->month == b->month && a->day < b->day)
    return(1);
  else
    return(0);
}

int isAfter(DBRECORD* a, DBRECORD *b)
{
  return( !isEqual(a,b) && !isBefore(a, b));
}


/* usage is 

   argv[1] = db path
   argv[2] = start_date
   argv[3] = end_date
   argv[4] = type {(D,d)aily , (W,w)eekly, (M,m)onthly}

*/

void fatal(char* a)
{
  fprintf(stderr, "%s\n", a);
  exit(1);
}

int get_search_type(char* a){
  if(strlen(a) > 1){
    if(strcasecmp(a, "daily") == 0)
      return('D');
    else if(strcasecmp(a, "weekly") == 0)
      return('W');
    else if(strcasecmp(a, "monthly") == 0)
      return('M');
    else
      return('U');
  }
  else if(strlen(a) == 0)
    return('U');
  
  switch(a[0]){
  case 'M':
    return('M');
  case 'm':
    return('M');
  case 'D':
    return('D');
  case 'd':
    return('D');
  case 'W':
    return('W');
  case 'w':
    return('W');
  default:
    return('U');
  };
}


int newRecord(char* str, DBRECORD* a)
{
  char* p;
  char buffer[1024];

  if(str[0] == '#' || str[0] == '\0')
    return(1);

  a->yday = a->files = a->data = 0;

  strcpy(buffer, str);

  /* process required, date only info */
  p = strtok(str, "/|\n");
  if(p == NULL)
    return(1);
  else
    a->month = atoi(p);

  p = strtok(NULL,"/|\n"); 
  if(p == NULL)
    return(1);
  else
    a->day = atoi(p);

  p = strtok(NULL,"/|\n");   
  if(p == NULL){
    return(1);
  }
  else
    a->year = atoi(p);

  /* optional info */

  p = strtok(NULL,"/|\n");   
  if(p == NULL){
    return(0);
  }
  else
    a->yday = atoi(p);

  p = strtok(NULL,"/|\n");   
  if(p == NULL){
    return(0);
  }
  else
    a->files = atol(p);

  p = strtok(NULL,"/|\n");   
  if(p == NULL){
    return(0);
  }
  else
    a->data = atol(p);  

  return(0);
}

/* get first dbrecord that is new enough */
int first_dbent(FILE* fp, DBRECORD* record, DBRECORD* start)
{
  char buffer[1024];
  while(fgets(buffer, 1024, fp) != NULL){
    if(start == NULL)
      return(0);
    else if(newRecord(buffer, record) != 0)
      continue;
    else if(isBefore(record, start)){
      continue;
    }
    else{
      return(0);
    }
  }
  return(1);
}


/*  get another database record */
int next_dbent(FILE* fp, DBRECORD* record, DBRECORD* end)
{
  char buffer[1024];
  while(fgets(buffer, 1024, fp) != NULL){
    if(end == NULL)
      return(0);
    else if(newRecord(buffer, record) != 0)
      continue;
    else if(isAfter(record, end))
      return(1);
    else
      return(0);
  }

  return(1);
}

/* returns NULL if not new period, else returns date string to 
   use as midpoint of previous period */

char* isNewPeriod(DBRECORD* current, int period_type, int flags)
{
  static _init_status = -1;
  int jweek;
  int ret=0;
  static DBRECORD last_bound;
  static int last_jweek;
  double foo_julian;
  GREGINFO* foo_gregorian;
  static char datestr[32];

  if(_init_status++ == -1){
    if(period_type == 'W'){
      julian_offset = julday(current->month, current->day, current->year, 1, 1, 1.0);
      last_jweek = 0;
    }
    else{
      memcpy(&last_bound, current, sizeof(last_bound));
      return(0);
    }
  }
  
  switch(period_type){
  case 'D':
    if(!isEqual(&last_bound, current))
      ++ret;
    break;
  case 'M':
    if(!(current->year == last_bound.year && 
	 current->month == last_bound.month))
      ++ret;
    break;
  case 'W':
    jweek = (int)(julday(current->month, current->day, current->year, 1, 1, 1.0) - julian_offset ) / 7;
    if(jweek != last_jweek)
      ++ret;
    break;
  default:
    fatal("period type unknown\n sorry, bailing\n");
  };

  if(ret){
    switch(period_type){
    case 'W':
      foo_julian = (last_jweek * 7.0) + 3.5 + julian_offset ;
      foo_gregorian = calcdate(foo_julian);
      if(flags & USE_JULIAN)
	sprintf(datestr, "%f", foo_gregorian->julian
);
      else{
	sprintf(datestr, "%02d/%02d/%d", foo_gregorian->month, 
		foo_gregorian->day, foo_gregorian->year);
      }
      last_jweek = jweek;
      break;
    case 'D':
      if(flags & USE_JULIAN)
	sprintf(datestr,"%.2f", julday(
			last_bound.month, last_bound.day, last_bound.year,
			0, 0, 0.0 ));
      else
	sprintf(datestr, "%02d/%02d/%d", last_bound.month, last_bound.day, 
		last_bound.year);
      break;
    case 'M':
      if(flags & USE_JULIAN)
	sprintf(datestr,"%.2f", julday(
			last_bound.month, 
			last_day(last_bound.month, last_bound.year) / 2,
			last_bound.year, 0, 0, 0.0 ));
      else
	sprintf(datestr, "%02d/%02d/%d", last_bound.month, 
		last_day(last_bound.month, last_bound.year) / 2,
		last_bound.year);
      break;
    default:
      fatal("isNewPeriod:: reached point in case statement where we shouldn';t be, check for covered cases\n");
    };
    memcpy(&last_bound, current, sizeof(last_bound));
  }

  if(ret)
    return(datestr);
  else
    return(NULL);
}
    

int main(int argc, char** argv)
{
  DBRECORD start, end;
  DBRECORD current;
  size_t files=0, data=0;
  FILE* fp;
  int flags=0, c;
  char* datestr;
  int type;
  int ret;

  while((c = getopt (argc, argv, "j")) != EOF){
    switch((char)c){
    case 'j':
      flags = USE_JULIAN;
      break;
    }
  }

  if(optind  != argc - 4)
    fatal("wrong number of args\nusage:[-j] db_path start_date end_date daily|monthly|weekly");
  
  type = get_search_type(argv[argc - 1]);
  if(type == 'U')
    fatal("bad search type\nusage: [-j] db_path start_date end_date daily|monthly|weekly");
  
  if(newRecord(argv[argc - 3], &start) != 0)
    fatal("bad start date\nusage: db_path start_date end_date daily|monthly|weekly, bad start date");

  else if(newRecord(argv[argc - 2], &end) != 0)
    fatal("usage: [-j] db_path start_date end_date daily|monthly|weekly, bad end date");

  else if((fp = fopen(argv[argc - 4], "r")) == NULL){
    perror(argv[1]);
    fatal("usage: [-j] db_path start_date end_date daily|montly|weekly, could not open db_path");
  }

  
  for(ret = first_dbent(fp, &current, &start); ret == 0; 
      ret = next_dbent(fp, &current, &end)){
    if((datestr = isNewPeriod(&current, type, flags)) == NULL){
      files+=current.files;
      data += current.data;
    }
    else{
      printf("%s\t%d\t%d\n", datestr, files, data);
      files = data = 0L;
      files+=current.files;
      data += current.data;
    }
  }
  return(0);
}      
      
    
  
