#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <inode_walk.h>
#include <stdio.h>
#include <histogram.h>
#include <unistd.h>
#include <errno.h>
#include <dba.h>
#include <time.h>

#ifdef RASH
#include <sys/rash.h>
#endif

void check_inodes(DINODE*, INO_T);

/* handle getopt includes */
#ifdef sgi
#include <getopt.h>
#else
extern char *optarg;
extern int optind;

#endif
  
OFF_T total_bytes = 0;
OFF_T files = 0;
OFF_T archived_files = 0;
OFF_T archived_bytes = 0;

/* used to determine total files, bytes and archived files, bytes */
void check_inodes(DINODE* ino, INO_T inum)
{
  if((ino->di_mode & S_IFMT) == S_IFREG){
    if(ino->di_size == 0)
      return;

#ifdef FIXISIZE      /* a convex'ism */
    ino->di_size = FIXISIZE(ino->di_size);
#endif
    
    if(ino->di_size < 0LL){
      fprintf(stderr, "neg file size, inum %lld size %lld uid %d\n",  
	      inum, ino->di_size, ino->di_uid);
      return;
    }
    
    total_bytes += ino->di_size;
    ++files;
#ifdef RASH

    if((ino->di_rashflag & RARCH) == RARCH){
      archived_files++;
      archived_bytes += ino->di_size;
    }
    
#endif

  }
}

int main(int argc, char** argv)
{
  DbA* db;
  DBAREC rec;
  time_t date;
  double value;
  int i;

  for(i = optind; i < argc; ++i){
#ifdef convex
    (void) inode_walk(argv[i], check_inodes);
#else
    (void) glop_walk(argv[i], check_inodes);
#endif
  }
  time(&date);
  DBARECINIT(rec);
  /* calculates unarchived from archived and total */
  
  value = ((files - archived_files) * 1.0 );
  if((db = newDbAbstractor("unarch_tot_file_snap", 't', 'f')) == NULL){
    fprintf(stderr, "Error opening unarch_tot_file_snap table\n");
    exit(1);
  }
  rec.value = &value;
  rec.date = &date;
  if(dba_store(db, &rec)!=0){
    fprintf(stderr, "Error opening unarch_tot_file_snap table\n");
    exit(1);
  }
  dba_destroy(db);
  
  value = ((total_bytes - archived_bytes) * 1.0 );
  if((db = newDbAbstractor("unarch_tot_byte_snap", 't', 'f')) == NULL){
    fprintf(stderr, "Error opening unarch_tot_byte_snap table\n");
    exit(1);
  }  
  rec.value = &value;
  if(dba_store(db, &rec)!=0){
    fprintf(stderr, "Error opening unarch_tot_byte_snap table\n");
    exit(1);
  }
  dba_destroy(db);


  return(0);
}











