#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <pwd.h>
#include <string.h>
#include <limits.h>

#include <inode_walk.h>
#include <msscompat.h>
#include <histogram.h>
#define ID_MAX 1000000

/* get RASH related stuff where applicable */
#if defined(__convex__) && !defined(RASH)
#define RASH
#endif


#ifdef RASH
#include <sys/rash.h>
#endif


/* handle getopt includes */
#ifdef sgi
#include <getopt.h>
#else
extern char *optarg;
extern int optind;

/*
 *      Variables used by getopt2.c
 */
char            *optarg;
int             optind2=0;
int             opterr2=0;

#endif

/***************************** globals *****************************/
char  type[3];
OFF_T *bytes;
OFF_T *files;
uid_t id_upper;
OFF_T size_limit;

void check_inodes(DINODE* ino, INO_T inum)
{
  if ( ( ino->di_mode & S_IFMT ) != S_IFREG )
    return;
  
  if( ino->di_size == 0 )
    return;

#ifdef FIXISIZE
  ino->di_size = FIXISIZE(ino->di_size);
#endif

  if ( ino->di_size < 0LL ) {
    if(strlen(type) == 0)
      fprintf(stderr, "neg file size, inum %lld size %lld uid %d\n", 
	      inum, ino->di_size, ino->di_uid);
    else
      fprintf(stderr, "neg file size, inum %lld size %lld gid %d\n", 
	      inum, ino->di_size, ino->di_gid);
    return;
  }
  
  if ( ino->di_size < size_limit )  
    return;
  
  /* place user data in their buckets, neg user nums in root bucket,
     uid(s) large than max in the overflow bucket */
  if(strlen(type) == 0){
    if( ( ino->di_uid > 0 ) && ( ino->di_uid < id_upper ) ){
      bytes[ino->di_uid] += ino->di_size;
      ++(files[ino->di_uid]);
    }
    else {
      bytes[0] += ino->di_size;
      ++(files[0]);
    }
  }
  else{
    if( ( ino->di_gid > 0 ) && ( ino->di_gid < id_upper ) ){
      bytes[ino->di_gid] += ino->di_size;
      ++(files[ino->di_gid]);
    }
    else {
      bytes[0] += ino->di_size;
      ++(files[0]);
    }
  }
}

	
void usage()
{
  fprintf(stderr, "usage: userstats [-g] [-s size] device [device2 ... devicen]\n");
  fprintf(stderr, "\t-g produces userstats using group id (gid)\n");
  fprintf(stderr, "\t   rather than the normally used user id (uid)\n");
  fprintf(stderr, "\t-s size indicates minimum file size to search for\n");
  fprintf(stderr, "\t  and must be in the form: number followed by m or k.\n");
}

void init_buckets()
{
  struct passwd* p;
  
  id_upper=0;
  
  fprintf(stderr, "reading passwd file....");
  fflush(stderr);
  
  while ( ( p = getpwent() ) != NULL ) {
    if(strlen(type) == 0){
      if ( (p->pw_uid > id_upper) && ( p->pw_uid < ID_MAX ) )
	id_upper = p->pw_uid;
    }
    else{
      if ( (p->pw_gid > id_upper) && ( p->pw_gid < ID_MAX ) )
	id_upper = p->pw_gid;
    }
  }
  fprintf(stderr, " done\n");
  if ( ( bytes = ((OFF_T *)malloc(sizeof(OFF_T) * (id_upper + 1))) ) 
      == NULL ) {
    
    perror("malloc");
    exit(1);
  }
  
  if ( ( files = ((OFF_T *)malloc(sizeof(OFF_T) * (id_upper + 1))) )
      == NULL ) {
    
    perror("malloc");
    exit(1);
  }
  
  memset(files, 0, (sizeof(OFF_T) * (id_upper + 1)));
  memset(bytes, 0, (sizeof(OFF_T) * (id_upper + 1)));
}


void print_stats()
{
  
  int i;
  
  struct passwd *user_entry;
/*  char	      *user_name;*/
  
  if(strlen(type) == 0)
  printf("%20s\tfiles\tTotal bytes\n------------------------------------------------------\n\n", "Username" );
  else
    printf("%20s\tfiles\tTotal bytes\n------------------------------------------------------\n\n", type);
  for ( i = 0; i <= id_upper ; ++i ) {
    
    if(files[i] > 0) {
      if(strlen(type) == 0){
	user_entry = getpwuid( i );	
      
	if ( user_entry != NULL ) 
	  printf("%20s\t%lld\t%lld\n",
		 user_entry->pw_name,files[i],bytes[i]);
	else
	  printf("%20d\t%lld\t%lld\n",
		 i,files[i],bytes[i]);
      }
      else
	printf("%20d\t%lld\t%lld\n",
	       i,files[i],bytes[i]);
      
    }
  }
}
int us_isdigit( char *numP)
{
  char            *fieldP = numP;
  
  while( *numP != '\0'){
    if ( (isdigit(*numP++)) == 0 ){
      fprintf(stderr,
              "%s is an invalid number", fieldP);
      return(EINVAL);
    }
  }
  
  return(0);
}


int us_getopts( int argc, char **argv)
{
  long long	mult;
  int		i;
  int		rc = 0;
  char		last;
  char		c;
/*   char		user[128];
  char		tmpstr[2048];
  */

  /* Process arguments */
  
  optind2 = 1;
  opterr2 = 0;
  
  /* set defaults */
  
  size_limit = 0LL;
  
  while((c = getopt(argc, argv, "gs:")) != EOF){
    switch(c){
    case 'g':
      strcpy(type, "gid");
      break;
    case 's':
      mult = 1;
      i = strlen(optarg) - 1;
      last = optarg[i];
      mult = 0;
      if ( (last == 'K') || (last == 'k') ) {
	mult = 1024;
	optarg[i] = '\0';
      }
      if ( (last == 'M') || (last == 'm') ) {
	mult = 1024 * 1024;
	optarg[i] = '\0';
      }
      if ( (mult == 0) ) {
	fprintf(stderr,
		"Illegal %s", optarg );
	rc = EINVAL;
	break;
      }
      if( rc = us_isdigit(optarg) ) 
	break;
      size_limit =  atol(optarg) * mult;
    break;
    case '?':
      fprintf(stderr,	
	      "Illegal option %s", optarg);
      usage();
      exit(-1);
    }
  }
  if ( rc ) 
    usage();
  return(rc);
}

int main(int argc, char** argv)
     
{
  int i;
  type[0] = '\0';
  if ( argc <= 1 ) {
    usage();
    exit(-1);
  }
  
  init_buckets();
  
  us_getopts( argc, argv );
  
  if ( optind2 >= argc ) {
    usage();
    exit(-1);
  }
  
  fprintf(stderr, "Counting files larger than or equal to %lld\n", 
	  size_limit);
  
  fprintf(stderr, "reading inodes for ");
  
  for ( i = optind; i < argc; ++i ) {
    fprintf(stderr, "%s ", argv[i]); 
    fflush(stderr);
    (void) inode_walk(argv[i], check_inodes);
  }
  
  fprintf(stderr, "\n");
  print_stats();
  return(0);
}


