/*
 * Test of kdcom via loopback.
 *
 * No arguments needed. Assumes a /dev/kdcom and a /dev/kdcom1 as
 * source and output sinks (relative).
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/ioctl.h>
#include "/usr/src/sys/dev/kdcom.h"

#define	NPASS	10
#define	MSGOFF	23
#define	BYEBYE	0x15

int main(void);
static int xorsum(char *, int);
static void reader (void);
static void writer (void);


int
main(void)
{
    pid_t pid = fork();

    if (pid == 0) {
	reader();
    } else if (pid < 0) {
	perror("fork");
	return (1);
    } else {
	sleep(2);
	writer();
	sleep(1);
    }
    return (0);
}

static int
xorsum(char *buf, int size)
{
    register int i, k = 0;
    k = buf[0];
    for (i = 1; i < size; i++)
	k ^= buf[i];
    return (k);
}

static void
reader(void)
{
    char buf[KDCOM_MESSAGE_SIZE];
    int fd, r;

    fd = open("/dev/kdcom", O_RDONLY);
    if (fd < 0) {
	perror("[reader] open");
	exit (1);
    }
    while ((r = read(fd, buf, KDCOM_MESSAGE_SIZE)) == KDCOM_MESSAGE_SIZE) {
	printf("[reader] chksum 0x%x\n", xorsum(buf, KDCOM_MESSAGE_SIZE));
	if (buf[MSGOFF] == BYEBYE)
		break;
    }
    if (r == 0) {
	printf("[reader] EOF\n");
    } else if (r < 0) {
	perror("[reader] read");
    }
    (void) close (fd);
}

static void
writer(void)
{
    char buf[KDCOM_MESSAGE_SIZE];
    int fd, i, pass;

    fd = open("/dev/kdcom1", O_WRONLY);
    if (fd < 0) {
	perror("[writer] open");
	exit (1);
    }
    for (pass = 0; pass < NPASS; pass++) {
	static const dev_t d = 0;
	for (i = 0; i < KDCOM_MESSAGE_SIZE; i++) {
	    buf[i] = rand();
	    if (buf[i] == BYEBYE && i == MSGOFF)
		i--;
	}
	if (pass == NPASS-1)
	    buf[MSGOFF] = BYEBYE;
	printf("[writer] chksum 0x%x\n", xorsum(buf, KDCOM_MESSAGE_SIZE));
	if (write(fd, buf, KDCOM_MESSAGE_SIZE) != KDCOM_MESSAGE_SIZE) {
	    perror("[writer] write");
	    break;
	}
	if (ioctl(fd, KDCIOCTL_LOOP_OUT2IN, (caddr_t) &d) < 0) {
	    perror("[writer] ioctl");
	    break;
	}
    }
    (void) close (fd);
}

/*
 * mode: c
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * End:
 */

