
/*
 *  For ease of maintenance and manipulation, it is suggested to use this
 *	construction for all queues in nastore.
 *
 *	A queue is headed by a QHead_t struct which has pointers
 *	to the first and last entry in the queue.  The items in a queue
 *	are linked into the queue via QLink_t structs, which contain
 *	forward and backward pointers that point to the previous and
 *	next *item*, not the item's QLink_t struct.
 *
 *  The macros for ENQUEUE_[HT] and DEQUEUE allow various structs to have
 *	multiple doubly-linked lists constructed of them, all of which
 *	are manipulated by the same macros.
 *
 *	ENQUEUE_H() enqueues an item at the head of the list
 *	ENQUEUE_T() enqueues an item at the tail of the list
 *	structp is a pointer to the item to be enqueued.
 *	Qnam is the name of the QLink_t structure
 *		(e.g. "c_cq" in Client_t), not a pointer or value.
 *	head is a pointer to the queue head.
 *	structnam is the item's struct type (for castes)
 */

/*
 *  QLink_t: general purpose doubly-linked queue structure.
 *	q_next and q_prev point to beginning of next struct,
 *	not necessarily to a QLink_t struct
 */
typedef struct	QueueLink {
	char	*q_next;		/* ptr to next item in queue	*/
	char	*q_prev;		/* ptr to previous item in Q	*/
} QLink_t;

/*
 *  QHead_t: general purpose head and tail of a queue
 */
typedef struct	QueueHead {
	char	*q_head;		/* ptr to first entry in queue	*/
	char	*q_tail;		/* ptr to last entry in queue	*/
} QHead_t;

/*
 *  the queue-manipulation macros
 */

#define	ENQUEUE(a, b, c, d)	ENQUEUE_T(a, b, c, d)
	/* enqueue at tail if not specified */
#define	ENQUEUE_T(structp, headp, Qnam, structnam) {		\
	char		*c_sp = (char *)(structp);		\
	structnam	*s_sp = (structnam *)(structp);		\
\
	(structp)->Qnam.q_next = s_sp->Qnam.q_prev = NULL;	\
	if ((headp)->q_tail) {					\
		((structnam *) ((headp)->q_tail))->Qnam.q_next = c_sp; \
		s_sp->Qnam.q_prev = (char *) ((headp)->q_tail);	\
	}							\
	(headp)->q_tail = c_sp;					\
	if ((headp)->q_head == NULL)				\
		(headp)->q_head = c_sp;				\
}

#define	ENQUEUE_H(structp, headp, Qnam, structnam) {		\
	char		*c_sp = (char *)(structp);		\
	structnam	*s_sp = (structnam *)(structp);		\
\
	(structp)->Qnam.q_next = s_sp->Qnam.q_prev = NULL;	\
	if ((headp)->q_head) {					\
		((structnam *) ((headp)->q_head))->Qnam.q_prev = c_sp; \
		s_sp->Qnam.q_next = (char *) ((headp)->q_head);	\
	}							\
	(headp)->q_head = c_sp;					\
	if ((headp)->q_tail == NULL)				\
		(headp)->q_tail = c_sp;				\
}

#define	DEQUEUE(structp, headp, Qnam, structnam) {		\
	char		*c_sp = (char *)(structp);		\
	structnam	*s_sp = (structnam *)(structp);		\
\
	if ((headp)->q_head == c_sp)				\
		(headp)->q_head = (char *) (s_sp->Qnam.q_next);	\
	if ((headp)->q_tail == c_sp)				\
		(headp)->q_tail = (char *) (s_sp->Qnam.q_prev);	\
	if (s_sp->Qnam.q_next)					\
		((structnam *) (s_sp->Qnam.q_next))->Qnam.q_prev = \
		  s_sp->Qnam.q_prev;				\
	if (s_sp->Qnam.q_prev)					\
		((structnam *) (s_sp->Qnam.q_prev))->Qnam.q_next = \
		  s_sp->Qnam.q_next;				\
}
