/*
 *  nas_tape.c - machine-specific tape config & driver stuff.
 *	Uses nLogMsg() pkg fron nas_logmsg.c to	log any errors.
 */

#include <stdlib.h>
#include "nasutil.h"
#include "nas_tape.h"

/*
 *  some generic redefinitions
 */
#ifndef MTIOCASTAT     
# define MTIOCASTAT     MTIOCGET
#endif
#ifndef MTUNLOAD
# define MTUNLOAD	MTOFFL
#endif
#ifndef MTBLANK
# define MTBLANK        MTERASE
#endif

#ifdef __sgi
# define MT_FPT		MT_WPROT
# define MT_RDY		MT_ONL
#endif

#if (defined (sun) || defined(_LINUX_SOURCE) || defined(__NetBSD__))
/*
 *  mt_dsreg - try these..
 */
# define MT_ONL  0x0001          /* Online Status                        */
# define MT_RDY  0x0002          /* Ready Status                         */
# define MT_REW  0x0004          /* Rewinding Status                     */
# define MT_TM   0x0008          /* Tape Mark Status                     */
# define MT_FPT  0x0010          /* Write Protect Status                 */
# define MT_OVR  0x0020          /* Block Long on Read                   */
# define MT_BOT  0x0040          /* Beginning of Tape                    */
# define MT_EOT  0x0080          /* End of Tape                          */


#endif


#include <sys/ioctl.h>
#include <sys/mtio.h>

#ifdef __NetBSD__
# define SCSI_MOUNTED	(MT_DS_RDONLY|MT_DS_MOUNTED)
# define GEN_SCSI_DRIVE	0x7	/* hard-coded thus in /usr/src/bin/mt/mt.c */
#endif

static struct mtget	mget;	/* struct for getting tape status */
static struct mtop	mop;	/* struct for issuing driver cmds */

int
ntp_open(char *path, int write, int blocksize, int compress, int logflg)
{
	char	*id = "ntp_open";
	int	fd;

#if	defined(lint)
	blocksize = blocksize;
	compress = compress;
#endif
	fd = open(path, (write ? O_RDWR : O_RDONLY));
	if (fd == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, id, "open(%s)", path);
		return(-1);
	}
#ifdef TP_SET_CMP
	mop.mt_op = MTCMPRESS;
	if (compress)
		mop.mt_count = 1;
	else
		mop.mt_count = 0;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, id,
					"ioctl/compress (%s)", path);
		close(fd);
		return(-1);
	}
#endif
#ifdef TP_SET_BLK
	mop.mt_op = MTSETBSIZ;
	mop.mt_count = blocksize;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, id,
					"ioctl/blksz (%s)", path);
		close(fd);
		return(-1);
	}
#endif
	return(fd);
}

/*
 *  ntp_getstatus() - just get status; cmds following interpret it
 */	
static int	statok = 0;
int
ntp_getstatus(int fd, char *vsn, int logflg)
{
	/* MTIOCASTAT returns status on _previous_ call ..?? */
	ioctl(fd, MTIOCASTAT, &mget);	/* query status */
	if (ioctl(fd, MTIOCASTAT, &mget) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_getstatus", 
					"ioctl/getstatus (%s)", vsnstr(vsn));
		return(-1);
	}
	statok = 1;
	return(0);
}
/*

#define TABLE(a) {a, #a}

#define STATUS_STATES(FUNC) \
    FUNC(MT_EOT),   \
    FUNC(MT_BOT),  \
    FUNC(MT_WPROT),  \
    FUNC(MT_EW),  \
    FUNC(MT_ONL),  \
    FUNC(MT_EOD),  \
    FUNC(MT_FMK),  \
    FUNC(MT_POSMSK), \
    FUNC(MT_QIC24), \
    FUNC(MT_QIC120)

struct  stat_states {
    int     value;
    char    *string;
} stat_states[] = {
    STATUS_STATES(TABLE)
};

#define STAT_MAX (sizeof(stat_states)/sizeof(stat_states[0]))

void
ntp_logstatus(char *vsn, int logflg) {
	char	c[160], *id = "ntp_logstatus";
	int 	i, j, k;

	if (!statok) {
		nLogMsg(EL_FATAL, id,
				"ntp_getstatus must be called 1st (%s)",
				vsnstr(vsn));
		return;
	}
	nLogMsg(EL_ALWAYS|logflg, id, "Tape type = %x", mget.mt_type);
	nLogMsg(EL_ALWAYS|logflg, id, "Position  = %x", mget.mt_dposn);
	nLogMsg(EL_ALWAYS|logflg, id, "Status Register = %x", mget.mt_dsreg);
	strcpy(c, "     ");
	k = mget.mt_dsreg;
	for(j=5, i=0; i<STAT_MAX; i++) {
		if ((stat_states[i].value & k) == stat_states[i].value) {
			if ((j += strlen(stat_states[i].string)+3) > 80) {
				j = 5+strlen(stat_states[i].string);
				nLogMsg(EL_ALWAYS, id, "%s", c);
				strcpy(c, "     ");
			}
			strcat(c, stat_states[i].string);
			strcat(c, "|");
			k = k & ~stat_states[i].value;
		}
	}
	if (strlen(c) > 5)
		nLogMsg(EL_ALWAYS, id, "%s", c);
	nLogMsg(EL_ALWAYS|logflg, id, "Error register = %x", mget.mt_erreg);
	nLogMsg(EL_ALWAYS|logflg, id, "Residual = %d", mget.mt_resid);
	nLogMsg(EL_ALWAYS|logflg, id, "File #   = %d", mget.mt_fileno);
	nLogMsg(EL_ALWAYS|logflg, id, "Block #  = %d", mget.mt_blkno);
}
*/

int
ntp_ismounted(char *vsn)
{
	if (!statok)
		nLogMsg(EL_FATAL, "ntp_ismounted", 
				"ntp_getstatus must be called 1st (%s)",
				vsnstr(vsn));

#ifdef __NetBSD__
	if (mget.mt_type == GEN_SCSI_DRIVE)
		return(mget.mt_dsreg & SCSI_MOUNTED);
#endif

	return ((mget.mt_dsreg & (MT_ONL | MT_RDY)) == (MT_ONL | MT_RDY));
}

int
ntp_write_protected(char *vsn)
{
	if (!statok)
		nLogMsg(EL_FATAL, "ntp_write_protected", 
				"ntp_getstatus must be called 1st (%s)",
				vsnstr(vsn));
#ifdef __NetBSD__
	if (mget.mt_type == GEN_SCSI_DRIVE)
		return (mget.mt_dsreg & MT_DS_RDONLY);
#endif
	return (mget.mt_dsreg & MT_FPT);
}

int
ntp_ateot(char *vsn)
{
#ifdef __NetBSD__
	errno = ENOSYS;
	nLogMsg(EL_ERROR1, "ntp_ateot", "not supported by netbsd");
#endif
	if (!statok)
		nLogMsg(EL_FATAL, "ntp_ateot", 
				"ntp_getstatus must be called 1st (%s)",
				vsnstr(vsn));
	return (mget.mt_dsreg & MT_EOT);
}

struct tposition_hdr {
	unsigned long   format;
	unsigned long   revision;
	unsigned long   total_size;
	unsigned long   flags;
	char            file_no[8];
	char            blk_no[8];
	long            chan_loc;
	long            dev_loc;
};

/*
**	ntp_locblk()
**	fd is the file descriptor on which to issue the ioctl
**	loc is the location to goto
**	a 0 is returned if all is well, -1 otherwise and errno set
*/
int
ntp_locblk(int fd, TBLKID loc, char *vsn, int logflg)
{
#ifdef _CONVEX_SOURCE
	struct dev_buf_desc	mtbuf;
	struct tposition_hdr	sendloc;

	mtbuf.buffer = (char *) &sendloc;
	mtbuf.size = sizeof(sendloc);

	sendloc.flags = 1;
	sendloc.format = 2;
	sendloc.revision = 0;
	sendloc.total_size = sizeof(sendloc);
	sendloc.chan_loc = loc;
	sendloc.dev_loc = loc;

	if (ioctl(fd, MTIOLOCATEPOS, &mtbuf) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_locblk", 
				"ioctl/locatepos (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
#elif defined (__sgi)
	mop.mt_op = MTSEEK;
	mop.mt_count = loc;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_locblk", 
				"ioctl/locatepos (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
#elif defined (__NetBSD__)
	if (ioctl(fd, MTIOCHLOCATE, (caddr_t) &loc)) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_locblk", 
				"ioctl/MTIOCHLOCATE (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
#else
	nLogMsg(EL_FATAL, "ntp_locblk", "not implemented");
#endif
}

/*
**	ntp_getblk
**	fd is the file descriptor on which to issue the ioctl
**	loc is a pointer to a two integer array
**	a 0 is returned if all is well, -1 otherwise and errno set
*/
int
ntp_getblk(int fd, TBLKID *loc, char *vsn, int logflg)
{
#ifdef _CONVEX_SOURCE
	struct dev_buf_desc	mtbuf;
	struct tposition_hdr	getloc;

	mtbuf.buffer = (char *) &getloc;
	mtbuf.size = sizeof(getloc);

	if (ioctl(fd, MTIOREPORTPOS, &mtbuf) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_getblk", 
					"ioctl/getblk (%s)", vsnstr(vsn));
		return(-1);
	}
	*loc = getloc.chan_loc;
	return (0);
#elif defined (__NetBSD__)
	/* get hdware blkno */
	if (ioctl(fd, MTIOCRDHPOS, loc) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_getblk", 
					"ioctl/getblk (%s)", vsnstr(vsn));
		return(-1);
	}
	return (0);
#else
	if (ioctl(fd, MTIOCGET, &mget) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_getblk", 
					"ioctl/getblk (%s)", vsnstr(vsn));
		return(-1);
	}
# if (defined(sun) || defined(__NetBSD__)) || defined(_LINUX_SOURCE)
	*loc = mget.mt_blkno; 
# else
	*loc = mget.mt_dposn; /* 'posn' = position; mt_blkno =? */
# endif
	return (0);
#endif
}



int
ntp_taprew(int fd, char *vsn, int logflg)
{
	mop.mt_count = 1;
	mop.mt_op = MTREW;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_taprew", 
					"ioctl/rewind (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
}

int
ntp_tapfsb(int fd, int count, char *vsn, int logflg)
{
	mop.mt_count = count;
	mop.mt_op = MTFSR;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_tapfsb", 
					"ioctl/fsb (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
}

int
ntp_tapbsb(int fd, int count, char *vsn, int logflg)
{
	mop.mt_count = count;
	mop.mt_op = MTBSR;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_tapbsb", 
					"ioctl/bsb (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
}

int
ntp_tapfsf(int fd, int count, char *vsn, int logflg)
{
	mop.mt_count = count;
	mop.mt_op = MTFSF;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_tapfsf", 
					"ioctl/fsf (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
}

int
ntp_tapbsf(int fd, int count, char *vsn, int logflg)
{
	mop.mt_count = count;
	mop.mt_op = MTBSF;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_tapbsf", 
					"ioctl/bsf (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
}

int
ntp_tapwtm(int fd, char *vsn, int logflg)
{
	mop.mt_count = 1;
	mop.mt_op = MTWEOF;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_tapwtm", 
					"ioctl/wtm (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
}

int
ntp_rewunload(int fd, char *vsn, int logflg)
{
	mop.mt_count = 1;
	mop.mt_op = MTUNLOAD;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_rewunload", 
					"ioctl/rewunld (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
}

int
ntp_tapdse(int fd, char *vsn, int logflg)	/* data security erase     */
{
	mop.mt_count = 1;
	mop.mt_op = MTBLANK;
	if (ioctl(fd, MTIOCTOP, &mop) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|logflg, "ntp_tapdse", 
					"ioctl/dse (%s)", vsnstr(vsn));
		return(-1);
	}
	return(0);
}

/*
 *  display message on 3480 drive display
 *	Must be called with file descriptor to 3480
 */

#ifdef _CONVEX_SOURCE
int
ntp_tapled(int fd, struct mtdisplay *display)
{
	int             ret = 0;
	nLogMsg(EL_DEBUG3, 0, "tapled-ing");
	ret = ioctl(fd, MTIODISPLAY, display);
	nLogMsg(EL_DEBUG3, 0, "tapled-ed");
	return (ret);
}
#endif

int
LED34X0(int fd, char flag, char *line1, char *line2)
{
#if	defined(lint)
	fd = fd;
	flag = flag;
	line1 = line1;
	line2 = line2;
#endif
#ifdef _CONVEX_SOURCE
	char           *id = "LED34X0";
	int             i;
	struct mtdisplay led;

	led.function_code = flag;

	for (i = 0; i < 8; i++)
		led.message1[i] = ' ';	/* blank fill 		 */
	if (line1) {
		for (i = 0; i < 8; i++) {
			if (*(line1 + i) == '\0')
				break;
			else
				led.message1[i] = *(line1 + i);
		}
	}
	for (i = 0; i < 8; i++)
		led.message2[i] = ' ';
	if (line2) {
		for (i = 0; i < 8; i++) {
			if (*(line2 + i) == '\0')
				break;
			else
				led.message2[i] = *(line2 + i);
		}
	}
	return (ntp_tapled(fd, &led));
#else
	return(0);
#endif
}

/*
 *  this stuff may be useful someday for tape drives w/out
 *  a 'data security erase' function
 */

#define BUFSIZE 60000

/*
 *  random number generator for tape data security erase;
 *	there is probably a more deterministic pattern
 *	that works better for this.
 */
static unsigned keyseed = 0;

static int
garble()
{
	unsigned	cnt, seed;

	/*
	 *  first time through, seed system's random # generator
	 */
	if (!keyseed)
		srand(71277);

	/*
	 *  use system's random # to pick a number 0..64
	 */
	cnt = ((unsigned)rand() >> 6) % 64;

	/*
	 *  make new seed for 2nd random # generator and seed it
	 */
	keyseed += 71277;
	seed = keyseed;

	/*
	 *  index into 2nd random sequence by number 0..64 from
	 *	system random sequence
	 */
	while (cnt--)
		seed = seed * 78125 + 59049;

	return (seed);
}

static char *
dseblock(size_t size)
{
	unsigned       *bp;
	size_t		i, count = size / sizeof(unsigned);

	/*
	 *  make block
	 */
	bp = (unsigned *) nAlloc((size_t)BUFSIZE);

	for (i = 0; i < count; i++)
		bp[i] = garble();
	return ((char *) bp);
}

/*
 *  ntp_tapdse2() - non-driver version of ntp_tapdse().
 *	'enough' parameter == min Mbytes to write
 */
#ifndef MB
# define MB	(1024 * 1024)
#endif
#define FMB	(float) MB

static char	*dsebuf = NULL;

int
ntp_tapdse2(int fd, int enough, char *vsn, int logflg)
{
	char		*id = "ntp_tapdse2";
	TBLKID		last, now;
	int		i, ret;

	/*
	 *  make sure tape is rewound fully
	 *	TODO - some tape drivers may need extra work to
	 *	back up over initial label
	 */
	if (ntp_taprew(fd, vsn, logflg) == -1)
		return (-1);

	if (ntp_getblk(fd, &last, vsn, logflg) == -1)
		return (-1);

	if (dsebuf == NULL) {
		/*
		 *  make block
		 */
		dsebuf = dseblock((size_t)BUFSIZE);
	}

	/*
	 *  write away, checking for backspacing
	 */
	for (i = 1;; i++) {
		ret = (int)write(fd, dsebuf, (size_t)BUFSIZE);
		if (ret == -1) {
			/*
			 *  don't complain if 'enough' megabytes written
			 */
			if (errno == EIO && i * BUFSIZE > enough * MB) {
				ret = 0;
				break;
			}
			nLogMsg(EL_ERROR1|EL_SYS, id, 
					"iteration %d writing %d bytes (%s)",
					i, BUFSIZE, vsnstr(vsn));
			break;
		} else if (ret != BUFSIZE) {
			nLogMsg(EL_ERROR1, id,
			    "iteration %d writing %d bytes - only %d (%s)",
			    i, BUFSIZE, ret, vsnstr(vsn));
			i--;
			break;
		}
		if (ntp_getblk(fd, &now, vsn, logflg) == -1) {
			i--;
			ret = -1;
			break;
		}
		/*
		 *  TODO - this assumes 3480/3490 location format
		 */
		if ((now & 0xfffffL) < (last & 0xfffffL)) {
			nLogMsg(EL_ERROR1, id,
			    "backed up from %lx to %lx after block %d (%s)",
			    last, now, i, vsnstr(vsn));
			i--;
			ret = 1;
			break;
		}
		last = now;
	}
	if (ret)
		nLogMsg(EL_ERROR1, id, "wrote %f Mbytes (%s)",
			(float) (i * BUFSIZE) / FMB, vsnstr(vsn));
	return (ret);
}

/*
 *  seektry() - Call this routine after a tape read error to see if
 *	there is data beyond EOT (to see if it's an overwritten tape
 *	w/ new EOT before the old one). Returns block id if successful.
 *
 *	This used to be useful with 3480-type tapes, but
 *	hasn't worked since 1995 or so, presumably because
 *	of firmware improvements.
 */

TBLKID
seektry(int fd, TBLKID lastblk, int tries, char *vsn, int logflg)
{
	char		*id = "seektry";
	int		i, ret;
	time_t		tm, tm2;

	tries++;
	for (i = 1; i < tries; i++) {
		time(&tm);
		ret = ntp_locblk(fd, (TBLKID) i + lastblk, vsn, logflg);
		time(&tm2);
		nLogMsg(EL_STATUS2, id, "seek %d time %ld", i, tm2 - tm);
		if (ret != -1)
			break;
	}
	if (i != tries) {
		nLogMsg(EL_STATUS1, id, 
				"got beyond read err after %d tries", i);
		return (lastblk + (TBLKID) i);
	}
	return (-1);
}
