#ifndef NASUTIL_H
#define NASUTIL_H

#include <portutil.h>
#include <nas_logmsg.h>
#include <nas_queue.h>
#include <bsdtime.h>

/*
**	WARNING WARNIMG WARNING y'all
**
**		LINELEN, VSNLEN, and DATELEN define sizes in
**		the volman, vvm, and possibly other databases
**		which are flat text files). Changing them here
**		will break existing databases, possibly with
**		loss of data.
**
*/
#define LINELEN		80
#define VSNLEN		6       /* ANSI standard */
#define	DATELEN		16	/* size of hex field for 64-bit date */


/*
 *  CodeString_t: used to look up strings associated with code numbers
 */
typedef struct	{
	int	cs_code;		/* #define code for string */
	char	*cs_string;		/* string */
} CodeString_t;


/*
 *  struct for pkt protocol table
 */
typedef struct	ProtocolTable {
	char	*m_nam;			/* message name			*/
	int	m_num;			/* message number		*/
	ssize_t	m_siz;			/* size of associated packet	*/
} ProTab_t;

/*
 *  exported routines from files w/out exported .h
 */

/* from nas_time.c: */
char		*nas_time(time_t p);
time_t		TimeStr(char *str);

/* from nas_net.c: */

char		*nHostErr();
char		*nIPCFamily(int af_code);
struct hostent	*nGetHostInfo(char *hostname);
int		nStartListening(u_long port);
void		nStopConn(int sd);
int		nCloseConn(int sd);

/*  from nas_ascii.c  */

int		Bin2Hex(char *buf, size_t bufsize, void *nump, size_t numsize);
int		Hex2Bin(void *nump, size_t numsize, char *buf, size_t bufsize);
int		HexUshort(unsigned short *nump, char *buf, size_t size);
int		SetHexUshort(char *buf, size_t size, unsigned short num);
int		HexInt(uint *nump, char *buf, size_t size);
int		SetHexInt(char *buf, size_t size, uint num);
int		HexOFF(OFF_T *nump, char *buf, size_t size);
int		SetHexOFF(char *buf, size_t size, OFF_T num);
int		IncHex(char *buf, size_t size);
int		DecInt(int *nump, char *str, size_t size);
int		SetDecInt(char *str, size_t size, int num);
int		IncDecInt(char *str, size_t size);
int		DecUint(unsigned int *nump, char *str, size_t size);
int		SetDecUint(char *str, size_t size, unsigned int num);
int		DecUlong(unsigned long *nump, char *str, size_t size);
int		SetDecUlong(char *str, size_t size, unsigned long num);
void		SetTime(char *buf, time_t time);
time_t		GetTime(char *buf);


#define	C2BIN(func, dest, source)	func(dest, source, sizeof(source))
#define C2ASC(func, dest, source)	func(dest, sizeof(dest), source)

/*  from nas_pwd_grp.c  */

void		nas_readpwd();
void		nas_readgrp();
char		*nas_uidname(uid_t uid);
char		*nas_gidname(uid_t gid);

/*  from nas_sig.c  */

char		*nSigName(int sig);
int		sig_ign(int sig);

/*  from misc.c  */

char		getYorN();
int		nSymSize(char *str, OFF_T *size);
void		nCreatePath(char *path, uint perm);
int		nCheckMount(char *path);
void		*nAlloc(size_t bytes);
void		*nReAlloc(void *ptr, size_t bytes);
void		nFree(void *ptr);
int		isnum(const char *str);
int		nMember(uid_t quid, gid_t qgid);
int		nAccess(int write, uid_t quid, uid_t uid, gid_t gid,short mode);
char		*nGetUIDName(uid_t uid);
char		*nas_basename(char *path);
char		*vsnstr(char *vsnp);
int		checkvsn(char *vsn);
int		countintbits(int bits);
void		nGetLock(char *lock_fname, char *program);

char		*nCodeToStr(int code, CodeString_t *tp);
int		nStrToCode(char *str, CodeString_t *tp);
void		nListStrings(uint logflags, char *id, CodeString_t *tp);

size_t		PadSize(size_t size);
ProTab_t	*nMsgInfo(int msgno, ProTab_t *ptp);
char		*nMsgType(int type, ProTab_t *tab);
ssize_t		nPktSiz(int msgno, ProTab_t *tab);

#ifdef _CONVEX_SOURCE
utmp *getutent();
#endif


/*
	Because SGI conditionally defines it but unconditionally uses it
 */
#if !defined(_NO_POSIX) || !defined(_NO_XOPEN4)
#ifdef __sgi
typedef enum { B_FALSE, B_TRUE } boolean_t;
#endif
#endif

#endif /* NASUTIL_H */
