#ifndef _PORTUTIL_H
#define _PORTUTIL_H

/* 
 *  portutil.h - portability for NAStore; basic & 'normal' fs stuff
 */
#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/*
 *  define a safe size to pad structs to - so msg components can
 *	begin on word boundaries. 8 means machines <= 64bit covered.
 */
#define NAS_MAXWORD	8


/* basic stuff */

#ifndef TRUE
# define TRUE		1
#endif
#ifndef FALSE
# define FALSE		0
#endif

#ifndef MIN
# define MIN(a, b)	((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
# define MAX(a, b)	((a) > (b) ? (a) : (b))
#endif

#ifndef O_FORCEOPEN
# define O_FORCEOPEN	0
#endif

#ifndef O_SYNC
# define O_SYNC		O_FSYNC		/* NETBSD */
#endif

#if defined(__sgi) || defined(_CONVEX_SOURCE)
# define SIGNAL(_SIG, _ROUT)	(int)signal((_SIG),(void(*)(int))(_ROUT))==-1
#else
# define SIGNAL(_SIG, _ROUT)	signal((_SIG),(void(*)(int))(_ROUT))==(void *)-1
#endif

#ifdef __sgi

  struct dev_buf_desc {
	int size;
	char *buffer;
  };

typedef struct stat	stat_t;
typedef struct stat64	stat64_t;
#endif /* __sgi */

#if ( !defined(BSD4_4) && !defined(sun))
/*
 * then we need to use foo64 defines.  Used to be _BIG_FS
 */

# define OFF_T		off64_t
# define LSEEK		lseek64
# define FSEEK		fseek64
# define FTELL		ftell64
# define FSTAT 		fstat64
# define STAT_T		stat64_t
# define STAT		stat64
# define LSTAT		lstat64
# define TRUNCATE	truncate64
# define FTRUNCATE	ftruncate64
# define INO_T 		ino64_t

#else /* ! _BIG_FS */

# define OFF_T		off_t
# define LSEEK		lseek
# define FSEEK		fseek
# define FTELL		ftell
# define TRUNCATE	truncate
# define FTRUNCATE	ftruncate
# define INO_T		ino_t
# define STAT		stat
# define FSTAT		fstat
# define STAT_T		struct stat
# define LSTAT		lstat

#endif		/*  _BIG_FS */


#endif /* end _PORTUTIL_H */
