#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<ctype.h>
#include	<string.h>
#include	<fcntl.h>
#include	<limits.h>

#include	<sys/types.h>
#include	<sys/param.h>
#include	<sys/mount.h>
#include	<sys/stat.h>
#include	<sys/sysctl.h>

#include	<miscfs/dmfs/dmfs.h>
#include	<miscfs/dmfs/dmfs_fcntl.h>
#include	<miscfs/dmfs/rash.h>

#include	"nasutil.h"
#include	"dmfsd.h"

#define	ARCLIST		0
#define	FREELIST	1

int	prog;

off_t	min_size = 8*1024;
int	verbose = FALSE;

ulong	empty = 0;
ulong	needarc = 0;
ulong	regfile = 0;
ulong	totfile = 0;

void
printcan(register char *s)
{
	while (*s) {
		if (isascii(*s) && isprint(*s) &&
				*s != ' ' && *s != '\t' &&
				*s != '\\' && *s != '\n')
			(void) putchar(*s++);
		else
			(void) printf("\\%03o", *s++ & 0xFF);
	}
	putchar('\n');
}

/*
**	Function to process file.
*/
int
fileout(char *path, struct stat *sP)
{
	static	char		errline[256];
	static	rashstat_t	rstat;
	int			fd;

	totfile++;
	/*
	**	If it's zero length, no other processing
	**	needs to be done.
	*/
	if (sP->st_size == 0) {
		empty++;
		return 0;
	}

	if (prog == FREELIST && sP->st_size <= min_size)
		return 0;

	/*
	**	Get the bfid and flags to see if it is archived.
	*/
	if ((fd = open(path, O_RDONLY)) == -1) {
		sprintf(errline, "open: %s", path);
		perror(errline);
		return 1;
	}
	/* get bfid */
	if (fcntl(fd, DMFS_GETRINFO, &rstat) == -1) {
		sprintf(errline, "GETRINFO: %s", path);
		perror(errline);
		close(fd);
		return 1;
	}
	close(fd);
	regfile++;

	if (rstat.st_rashflg & DM_ARCH) {	/* already archived */
		if (prog == ARCLIST) {
			if (verbose) {
				fprintf(stderr,
					"%s: already archived\n", path);
			}
			return 0;
		}

		/*
		**	If non-resident and resident size is less
		**	than our minimum skip it.
		*/
		if (rstat.st_rashflg & DM_NONR &&
				rstat.st_bbarrier <= min_size) {
			if (verbose) {
				fprintf(stderr,
					"%s: barrier %lld under limit %lld\n",
					path, (long long)rstat.st_bbarrier,
					(long long)min_size);
			}
			return 0;
		}
	}
	else if (prog == FREELIST) {		/* not archived */
		if (verbose)
			fprintf(stderr, "%s: not archived\n", path);
		return 0;
	}

	needarc++;
	printf("%8.1ld %8.1ld %20.1lld ",
		(long)sP->st_uid, (long)sP->st_gid, (long long)sP->st_size);
	printcan(path);
	return 0;
}

void
usage(char *prog)
{
	fprintf(stderr, "usage: %s [-v] rootdir\n", prog);
}

int
main(int argc, char *argv[])
{
	int		ret, c;
	int		errflg = FALSE;
	char		*root_dir;
	char		progname[40];
	struct	stat	sbuf;
	extern	char	*optarg;
	extern	int	optind, opterr;
	static	char	*buildtype[] = {
		"arc",
		"free",
		NULL
	};

	strcpy(progname, nas_basename(argv[0]));
	for (prog = 0; buildtype[prog]; prog++) {
		if (strncmp(progname, buildtype[prog],
				strlen(buildtype[prog])) == 0)
			break;
	}
	while ((c = getopt(argc, argv, "vm:")) != EOF) {
		switch (c) {

		case 'm':
			min_size = (off_t)atol(optarg);
			if (min_size < 0)
				errflg = TRUE;
			else
				min_size *= 1024;	/* in Kb */
			break;

		case 'v':
			verbose = TRUE;
			break;

		case '?':
			errflg = TRUE;
			break;
		}
	}

	if (optind != argc-1)
		errflg = TRUE;
	if (errflg == TRUE) {
		usage(progname);
		return 2;
	}

	root_dir = argv[optind];
	if (root_dir[0] != '/') {
		fprintf(stderr, "must use a full path\n");
		return 2;
	}

	/*
	**	If stdout is on disk, lock it.
	*/
	if (fstat(fileno(stdout), &sbuf) == -1)
		perror("fstat stdout");
	else if (S_ISREG(sbuf.st_mode)) {
		if (flock(fileno(stdout), LOCK_EX) == -1)
			perror("flock stdout");
	}

	ret = traverse(root_dir, fileout);

	if (verbose) {
		fprintf(stderr,
			"files\t\t%lu\nreg\t%lu\nempty\t\t%lu\nnotarc\t%lu\n",
			totfile, regfile, empty, needarc);
	}
	return ret;
}
