#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<fcntl.h>
#include	<limits.h>

#include	"db3/db.h"

#include	<sys/utsname.h>
#include	<sys/types.h>
#include	<sys/param.h>
#include	<sys/mount.h>
#include	<sys/stat.h>
#include	<sys/sysctl.h>

#include	<miscfs/dmfs/dmfs.h>
#include	<miscfs/dmfs/dmfs_fcntl.h>
#include	<miscfs/dmfs/rash.h>

#include	"dmfsd.h"

DB	*db = NULL;
int	verbose = FALSE;

ulong	empty = 0;
ulong	arcfile = 0;
ulong	regfile = 0;
ulong	totfile = 0;

/*
**	Function to process file.
*/
int
savefh(char *path, struct stat *sP)
{
	static	char	errline[256];
	static	rashstat_t	rstat;
	static	DBT	key;
	static	DBT	data;
	static	bfid_t	zerobfid;	/* zero fill */
	int		fd;

	totfile++;
	/*
	**	If it's zero length, no other processing
	**	needs to be done.
	*/
	if (sP->st_size == 0) {
		empty++;
		return 0;
	}

	/*
	**	It's a regular file so we have to open it
	**	to get the bfid and flags to see if it is
	**	archived.
	*/
	if ((fd = open(path, O_RDONLY)) == -1) {
		sprintf(errline, "open: %s", path);
		perror(errline);
		return 1;
	}
	/* get bfid */
	if (fcntl(fd, DMFS_GETRINFO, &rstat) == -1) {
		sprintf(errline, "GETRINFO: %s", path);
		perror(errline);
		close(fd);
		return 1;
	}
	close(fd);
	regfile++;

	if (memcmp(&rstat.st_bfid, &zerobfid, sizeof(zerobfid))) {
		fhandle_t	fh;

		/*
		**	It is archived so put the bfid
		**	into the index for later retrieval.
		*/
		key.data = &rstat.st_bfid;
		key.size = sizeof(rstat.st_bfid);

		if (getfh(path, &fh) == -1) {
			sprintf(errline, "getfh: %s", path);
			perror(errline);
			return 1;
		}
		data.data = &fh;
		data.size = sizeof(fh);

		if (db->put(db, NULL, &key, &data, 0) == -1) {
			perror("dbput");
			return 1;
		}
		arcfile++;
		if (verbose)
			printf("%s\n", path);
	}
	return 0;
}

int
main(int argc, char *argv[])
{
	int		ret, c;
	int		errflg = FALSE;
	char		*root_dir;
	char		*dbfile = "/tmp/bfid_fh";
	extern char	*optarg;
	extern int	optind, opterr;

	while ((c = getopt(argc, argv, "b:v")) != EOF) {
		switch (c) {

		case 'b':	/* bfid index file */
			dbfile = optarg;
			break;

		case 'v':
			verbose = TRUE;
			break;

		case '?':
			errflg = TRUE;
			break;
		}
	}

	if (optind != argc-1)
		errflg = TRUE;
	if (errflg == TRUE) {
		fprintf(stderr,
			"usage: %s [-b index_file][-v] path\n",
			argv[0]);
		return 2;
	}

	root_dir = argv[optind];
	if (root_dir[0] != '/') {
		fprintf(stderr, "must use a full path\n");
		return 2;
	}

	/*
	**	Open a btree index to store bfid keys.
	*/
	if (db_create(&db, NULL, 0) != 0) {
		perror("db_create");
		return 1;
	}
	if (db->open(db, dbfile, NULL, DB_BTREE,
			DB_CREATE|DB_TRUNCATE, 0600) != 0) {
		perror("dbopen");
		return 1;
	}

	ret = traverse(root_dir, savefh);
	db->close(db, 0);
	printf("files\t\t%lu\nregular\t%lu\nempty\t\t%lu\narchived\t%lu\n",
		totfile, regfile, empty, arcfile);
	return ret;
}
