#ifndef __DBLIB_H
#define __DBLIB_H

/*
** Definitions required to support the btree data base interface for the 
** rash data base
*/

#define	RASH_VERSION			300

/*
**  The next definition describe the location of the data bases.
**  The "?" in DBPATH is a wild character that is subtituted with the
**  the first letter of the user name.
*/

#define DBPATH  "/usr/mss/lib/rash/dbase/r?"

/*
**  The next definition specifies the directory where the 
**  tcompact associated files will be allocated. The directory must
**  exist prior to invoking tcompact.
*/
#define DBDIR   "/usr/mss/lib/rash/dbase" /* database directory for tcompact */

/*
** Miscellaneous defines
*/

#define 	RASH_NULL_DBUID (uid_t) -3 /* null dbuid value		*/
#define		RASH_NULL_FD	-1	/* for non-rashopen fd's only	*/
#define		RASH_MAX_DBNAME	32	/* longest possible db name	*/

/*
** RASHREL	Vol1 hdr implementation identifier (max 13 characters)
** TAPEGROUP 	RASH tape group owner definition 
** TAPEOWNER 	tape label implementation identifier (Max of 8 characters)
** MAXTAPEFILES maximun number of files allowed on a tape  
**		Note: this file should not exceed 9999 since that's the 
**	 	maximum value allowed in the hdr1 of a tape. I.e. ANSI standard.
*/
#define		RASHREL		"2.1.1"
#define		TAPEGROUP   	"rash"		
#define		TAPEOWNER	"NAS RASH"
#define		MAXTAPEFILES    9999

/*
**  The ASCII Data base definition describes how the data is stored on 
**  disk. 
*/
#define OFFLEN		16	/* for off_t */
#define BFIDLEN		32	/* 128 bits in hex */
#define	VVNAMELEN	33
#ifndef	INTLEN
#define	INTLEN		10
#endif

typedef struct db_fileinfo {
    char	
	db_nrec[3], 		b01,	/* each is '1', '2', or '3'	*/
	db_version[INTLEN],	b02,	/* rash version			*/
	db_rectm[DATELEN],	b03,	/* record time stamp in hex	*/
	db_deltm[DATELEN],	b04,	/* record delete time in hex	*/
	db_bfid[BFIDLEN],	b05,	/* bfid in ascii hex		*/
	db_vvno[4],		b06,	/* vv sequence number		*/
	db_rflg[4],		b07,	/* Record flags           	*/
	db_mode[4],		b08,	/* file permissions		*/
	db_fsize[OFFLEN],	b09,	/* file size in ascii decimal	*/
	db_uid[INTLEN],		b10,	/* uid in ascii dec		*/
	db_gid[INTLEN],		b11,	/* gid in ascii dec		*/
	db_ctime[DATELEN],	b12,	/* ctime in hex			*/
	db_mtime[DATELEN],	b13,	/* mtime in hex			*/
	db_arctm[DATELEN],	b14,	/* arctm in hex			*/
	db_group_no[2],		b15,   	/* rash group number in dec 	*/
	db_fname[232],		b16,	/* first part of name		*/

	db_vvname[VVNAMELEN],	b17,	/* vv name        		*/
	db_fno[4],		b18,	/* file # in vv in dec		*/
	db_bof[8],		b19,	/* bof number in hex  		*/
	db_vflg[4],		b20,	/* vv flag			*/
	db_lseek[OFFLEN],	b21,	/* lseek disk data location     */
	db_vvdata[OFFLEN],	b22,	/* amnt of data on vv for fno	*/

	end1,			end2;
} DB_FILEINFO;

typedef struct {
    char	
	db_nrec[3], 		b01,	/* each is '1', '2', or '3'	*/
	db_aux[506],		end1, end2;
} DB_AUXREC;


/*
 *  The user data base definition describes how a user program sees the  
 *  the data via the various rash data base library routines  
 */
typedef struct usr_fileinfo {
	int		usr_version;	/* rash version			*/
	bfid_t		usr_bfid;	/* bfid              		*/
	int		usr_vvno;    	/* multiple volume sequence #  	*/
	unsigned short	usr_rflg;	/* record flags			*/
	unsigned short	usr_vflg;	/* vv flags			*/

	char		usr_vvname[VVNAMELEN+1];	/* vv name  	*/
	int  		usr_fno;	/* file number in multifile vv	*/
	unsigned int	usr_bof;	/* byte offset into the vv 	*/
	off_t		usr_lseek;	/* lseek write to disk location */
	off_t		usr_vvdata;	/* amnt of data on vv for fno	*/
	off_t		usr_fsize;	/* file size                 	*/
	uid_t 		usr_uid;	/* uid              		*/
	gid_t 		usr_gid;	/* gid              		*/
	time_t		usr_ctime;	/* ctime             		*/
	time_t		usr_mtime;	/* mtime           	   	*/
	time_t		usr_arctm;	/* arctm             		*/
	mode_t		usr_mode;
	time_t		usr_rectm;	/* record timestamp		*/
	time_t		usr_deltm;	/* record delete time		*/
	int             usr_group_no;	/* rash archive group number	*/
	char		*usr_fname;	/* file name pointer 		*/
} USR_FILEINFO;

/*
 *  Possible usr_rflg values
 */
#define RFLG_INODB	0x8000		/* inodb has examined this      */
#define RFLG_NOIDX	0x4000		/* Ignore this rec in index file*/
#define RFLG_DELETED 	0x2000		/* Record flagged as deleted	*/

/*
 *  Possible usr_vflg values
 */
#define VFLG_TAPEDB	      0x001f	/* mask of bits for tapedb	*/
#define VFLG_OK               0x0001	/* file checked ok by tapedb	*/
#define VFLG_BAD              0x0002	/* file on VV but damaged	*/
#define VFLG_NONEXISTENT      0x0004	/* file not on VV		*/
#define VFLG_RECYCLE	      0x8000	/* VV flagged as recycled	*/

/*
 * The following defines are the possible flags that can be passed to 
 * db_open.  Stored in the status field.
 */
#define	O_DB_READ	0x00000001 	/* for read only		*/
#define	O_DB_RDWR 	0x00000002	/* for read and write		*/
#define	O_DB_COPY 	0x00000003	/* read old, write copy		*/

/*
 * The following flags are used by the db_rread to limit the information 
 * read from the ascii data file.                 
 */
#define USR_ALL		0xffffffff
#define	USR_OFFSET	0x80000000
#define	USR_BFID  	0x40000000
#define USR_VVNO    	0x20000000
#define	USR_FLG   	0x10000000
#define USR_VV   	0x08000000
#define	USR_FNO   	0x04000000
#define	USR_BOF   	0x02000000
#define	USR_VFLG  	0x01000000
#define	USR_FSIZE 	0x00800000
#define	USR_UID   	0x00400000
#define	USR_GID   	0x00200000
#define	USR_CTIME 	0x00100000
#define	USR_MTIME 	0x00080000
#define	USR_ARCTM 	0x00040000
#define	USR_FNAME 	0x00010000
#define	USR_LSEEK 	0x00008000
#define	USR_TDATA 	0x00004000
#define	USR_GROUP 	0x00002000
#define	USR_MODE 	0x00001000
#define	USR_RECTM 	0x00000800
#define	USR_DELTM 	0x00000400


/*
 * The following are two additional errors codes that can be expected
 * from the dblib routines in addtion to the standard Unix errors found
 * in errno.h
 */
#define	RASHDB_OK	0		/* All is well			*/
#define	RASHDB_EOF	99999		/* End of data base reached	*/
#define RASHDB_ERROR  	99997		/* general error		*/

/* 
 * dblib.c  external functions
 */

int		rashdb_open(char *username, uint access);
void 		rashdb_close(void);
int 		rashdb_add(USR_FILEINFO *u);
void		rashdb_firstrec(void);
USR_FILEINFO	*rashdb_nextrec(int *err, uint mask);
USR_FILEINFO	*rashdb_get_bfid(int *err, bfid_t *bfidP);
USR_FILEINFO	*rashdb_next_bfidrec(int *err, bfid_t *bfidP);
USR_FILEINFO	*rashdb_get_vv(int *err, char *vvname);
USR_FILEINFO	*rashdb_next_vvrec(int *err, char *vvname);
int		rashdb_delete_bfid(bfid_t *bfidP);
int		rashdb_set_flgs(unsigned short rflg, unsigned short vflg);

char		*bfidout(bfid_t *bfidP);
#endif
