#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<fcntl.h>

#include	<sys/types.h>
#include	<sys/param.h>

#include	"dmfsd.h"

/*
 *  Routine to construct a full path name for a filename.
 */
char	*
fullname(char *name, const char *cwd)
{
	static char	result[PATH_MAX];
	char		*ch1, *ch2;
	struct  path {
		char	ele[PATH_MAX+2];
		struct	path	*forw;
		struct	path	*back;
	} *nhead, *ntail, *nlt, *nlt2;

	if (name == NULL)		/* trivial case */
		return(name);
	else if (*name == '/')
		strcpy(result, name);
	else
		sprintf(result, "%s/%s", cwd, name);

	ch1 = &result[1];
	nhead = NULL;
	ntail = NULL;

	for (;;) {	/* generate linked list of path elements */
		ch2 = strchr(ch1, '/');
		if (ch2 != NULL)
			*ch2 = '\0';
		if (strlen(ch1) > 0) {
			nlt = (struct path *)malloc(sizeof(struct path));
			nlt->forw = NULL;
			nlt->back = ntail;
			strcpy(nlt->ele, ch1);

			if (ntail == NULL) {
				ntail = nlt;
				nhead = nlt;
			} else {
				ntail->forw = nlt;
				ntail = nlt;
			}
		}
		if (ch2 == NULL)
			break;
		ch1 = ch2 + 1;
	}

	nlt = nhead;
	while (nlt != NULL) {

		if (strcmp(nlt->ele, ".") == 0) {	/* remove "." entries */

			if (nhead == nlt) {
				nhead = nlt->forw;
				if (nhead != NULL)
					nhead->back = NULL;
			} else
				nlt->back->forw = nlt->forw;

			if (nlt == ntail) {
				ntail = nlt->back;
				if (ntail != NULL)
					ntail->forw = NULL;
			} else
				nlt->forw->back = nlt->back;

			nlt2 = nlt->forw;
			free(nlt);
			nlt = nlt2;
		}
		else if (strcmp(nlt->ele, "..") == 0) {/* remove ".." */

			if (nhead == nlt) {		/* first element */
				nhead = nlt->forw;
				if (nhead != NULL)
					nhead->back = NULL;
			} else {
				nlt->back->forw = nlt->forw;
			}

			if (nlt == ntail) {
				ntail = nlt->back;
				if (ntail != NULL)
					ntail->forw = NULL;
			} else {
				nlt->forw->back = nlt->back;
			}

			if (nlt->back == NULL) {	/* nowhere to backup */
				nlt2 = nlt->forw;	/* just go forward */
			} else {			/* go back one and */
				nlt2 = nlt->back;	/* copy in "." so it */
				strcpy(nlt2->ele, ".");	/* will be deleted */
			}				/* on the next pass */
			free(nlt);
			nlt = nlt2;
		}
		else {				/* normal element */
			nlt = nlt->forw;
		}
	}

	result[0]='\0';				/* start null */
	nlt = nhead;
	while (nlt != NULL) {
		struct  path	*hold = nlt->forw;

		strcat(result, "/");
		strcat(result, nlt->ele);
		free(nlt);
		nlt = hold;
	}
	return(*result == '\0' ? "/" : result);
}
