#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "nas_logmsg.h"
#include "dmfsd.h"

/*
**  newproc
**  Start a child process running "func" with "parm" for a parameter.
**  When "func" returns, exit with it's return code.  Return the
**  child pid to calling routine.
*/
int
newproc(ifunc func, void *parm)
{
	static	char	id[] = "newproc";
	int	 	proc_stat;
	pid_t		whoami;
	extern	int	procs;

#if 1
	while ((whoami=fork()) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS, id, "fork");
		sleep(10);		/* so try again */
	}
	if (whoami == 0) {		/* child */
		proc_stat=(*func)(parm);
		exit(proc_stat);
	}
	procs++;
	return(whoami);
#else
	proc_stat=(*func)(parm);
	return(0);
#endif
}

/*
**	Wait for a process return.  Set stat to return status of
**	proc or 1 if it received a signal.  If waitpid() fails, set it
**	to -1.  Return the pid of the child or zero.
*/
int
waitproc(int *proc_stat)
{
	static	char	id[] = "waitproc";
	pid_t		pid;
	int		err;
	extern	int	procs;

	if ((pid = wait(&err)) == -1) {
		if (errno == ECHILD)
			return 0;
		nLogMsg(EL_ERROR1|EL_SYS, id, "wait");
		*proc_stat = -1;
		return 0;
	}
	procs--;
	if (WIFSIGNALED(err)) {		/* signal */
		nLogMsg(EL_STATUS2, id, "child %d terminated on signal %d",
			pid, WTERMSIG(err));
		*proc_stat = 1;
	}
	else
		*proc_stat = WEXITSTATUS(err);
	return pid;
}

/*
**	Got a SIGCHLD for a process return.  Set stat to return status of
**	proc or 1 if it received a signal.  If waitpid() fails, set it
**	to -1.  Return the pid of the child or zero.
*/
int
getproc(int *proc_stat)
{
	static	char	id[] = "getproc";
	int		status, pid;
	extern int	procs;

	if ((pid = waitpid(-1, &status, WNOHANG)) == -1) {
		if (errno == ECHILD)
			return 0;

		nLogMsg(EL_ERROR1|EL_SYS, id, "waitpid");
		*proc_stat = -1;
		return 0;
	}
	if (pid == 0)
		return 0;

	procs--;
	if (WIFSIGNALED(status))	/* signal */
		*proc_stat = 1;
	else
		*proc_stat = WTERMSIG(status);
	return pid;
}
