/*
*   This code is based on the code from the book
*   "Advanced Programming in the Unix Environment" by Richard Stevens.
*
*   It has been modified by me [waddell@nas.nasa.gov] to handle interrupts.
*   I have also added a function to read '\n' terminated lines of text.
*
*   Because of this code origins, this code can be freely used and distributed 
*   provided that this block of comments crediting Richard Stevens and 
*   Harry Waddell is left intact at the top of the file.
*
*/

#include <unistd.h>
#include <errno.h>

/*
 * Write "n" bytes to a descriptor.
 * Use in place of write() when fd is a stream socket.
 */

ssize_t
writen(fd, ptr, nbytes)
 int	fd;
 void	*ptr;
 size_t	nbytes;
{
  size_t	nleft;
  ssize_t	nwritten;
  char* 	_ptr;
  
  _ptr = (char *)ptr;
  nleft = nbytes;
  while (nleft > 0) {
    nwritten = write(fd, _ptr, nleft);
    if (nwritten <= 0 && !(errno == EINTR)){
      if (errno != EINTR)
	return(nwritten);		/* error */
      else
	continue;                       /* interrupted, try again */
    }
    else {				/* we wrote something */
      nleft -= nwritten;
      _ptr   += nwritten;
    }
  }
  return(nbytes - nleft);
}


/*
 * Read "n" bytes from a descriptor.
 * Use in place of read() when fd is a stream socket.
 */

ssize_t
readn(fd, ptr, nbytes)
     int	fd;
     void	*ptr;
     size_t	nbytes;
{
  
  size_t	nleft;
  ssize_t	nread;

  nleft = nbytes;
  while (nleft > 0) {
    nread = read(fd, (char *)ptr + nbytes - nleft, nleft);
   
    if (nread < 0 ){
      if(errno != EINTR)
	return(nread);		/* error, return < 0 */
      else 
	continue;               /* read was interrupted, try again */
    }
    else if (nread == 0)
      break;			/* EOF - return number of chars before EOF*/
    else
      nleft -= nread;
  }
  return(nbytes - nleft);		/* return >= 0 */
}


/*
 * read single char using readn to get a \n terminated line
 * 
 */

ssize_t
readlinen(fd, ptr, nbytes)
     int	fd;
     void	*ptr;
     size_t	nbytes;
{
  int i;
  
  for(i=0; i< nbytes; ){
    switch(readn(fd, (char *)ptr + i, (size_t)1)){
    case -1:
      /* got bad read */
      return(-1);
    case 0:
      /* got eof */
      return(0);
    case 1:
      /* got another char */
      if( ((char *)(ptr))[i] == '\n'){
	((char *)(ptr))[i] = '\0';
	return(i + 1);
      }
      else
	++i;
    }
  }
  ((char *)(ptr))[nbytes - 1] = '\0';
  return(nbytes);
}
