#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/param.h>		/* all this just to get bfid_t */
#include <sys/types.h>
#include <miscfs/dmfs/rash.h>

#include "nasutil.h"
#include "portutil.h"
#include "dblib.h"

typedef	struct vv_entry {
	char			vvname[VVNAMELEN+1];
	u_int			bad;
	u_int			recycled;
	u_int			seqno;
	u_int			tfiles;
	u_int			dfiles;
	off_t			tdata;
	off_t			ddata;
	struct	vv_entry	*link;
} vv_entry_t;

#define	VVHASH	0x1f
vv_entry_t	*vvarray[VVHASH+1];

int
vvhash(char *vv)
{
	int	i;
	int	hash = 0;

	for (i=0; i<VVNAMELEN; i++) {
		if (vv[i] == '\0')
			break;
		hash += vv[i];
	}
	hash &= VVHASH;
	return hash;
}

/*
**	Find (or create) the vv entry that represents a given vv.
*/
vv_entry_t	*
vvfind(char *vv)
{
	static	vv_entry_t	*oldvv;
	vv_entry_t		*vP;
	int			hval;

	if (oldvv != NULL && strcmp(vv, oldvv->vvname) == 0)
		return oldvv;

	hval = vvhash(vv);
	for (vP = vvarray[hval]; vP; vP = vP->link) {
		if (strcmp(vv, vP->vvname) == 0)
			break;
	}
	if (vP == NULL) {
		vP = malloc(sizeof(*vP));
		memset(vP, '\0', sizeof(*vP));
		strcpy(vP->vvname, vv);
		vP->link = vvarray[hval];
		vvarray[hval] = vP;
	}
	oldvv = vP;
	return vP;
}

void
douser(char *name)
{
	USR_FILEINFO	*uP;
	int		ret, i;
	vv_entry_t	*vP;
	static	int	uflg = USR_VVNO|USR_FLG|USR_VV|USR_VFLG|USR_TDATA;

	if (rashdb_open(name, O_DB_READ) != RASHDB_OK ) {
		fprintf(stderr, "open failed for user %s\n", name);
		return;
	}

	printf("\tVV's for %s\n", name);
	rashdb_firstrec();
	for (;;) {
		uP = rashdb_nextrec(&ret, uflg);
		if (ret != RASHDB_OK) {
			if (ret != RASHDB_EOF)
				fprintf(stderr, "Dbase error for %s\n", name);
			break;
		}
		vP = vvfind(uP->usr_vvname);

		vP->tfiles++;
		vP->tdata += uP->usr_vvdata;
		vP->seqno = uP->usr_vvno;
		if (uP->usr_rflg & RFLG_DELETED) {
			vP->dfiles++;
			vP->ddata += uP->usr_vvdata;
		}
		if (uP->usr_vflg & (VFLG_BAD|VFLG_NONEXISTENT))
			vP->bad++;
		if (uP->usr_vflg & VFLG_RECYCLE)
			vP->recycled++;
	}
	rashdb_close();

	printf("vvname            seq files/deleted      data/deleted(Kb)");
	printf("  recycled\n");
	for (i=0; i<=VVHASH; i++) {
		vP = vvarray[i];
		while (vP != NULL) {
			printf("%-16.16s %4u %4u / %-4u ",
				vP->vvname, vP->seqno,
				vP->tfiles, vP->dfiles);
			printf("%10lld / %-10lld      %4u\n",
				(long long)(vP->tdata/1024),
				(long long)(vP->ddata/1024),
				vP->recycled);
			vP = vP->link;
		}
	}
	return;
}

int
main(int argc, char *argv[])
{
	int		i, j;
	vv_entry_t	*vP, *vN;

	for (i=1; i<argc; i++) {
		douser(argv[i]);

		for (j=0; j<=VVHASH; j++) {
			for (vP = vvarray[j]; vP; vP = vN) {
				vN = vP->link;
				free(vP);
			}
			vvarray[j] = NULL;
		}
	}
	return 0;
}
