/*
**	vvrecycle
**	Go through a list of VV's and restore any active files.
**	Then call vv_recycle() for each VV which no longer has
**	active files.
**
**	usage: vvrecycle username vv vv ...
*/
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <assert.h>

#include "db3/db.h"

#include <sys/types.h>
#include <sys/param.h>
#include <miscfs/dmfs/dmfs.h>
#include <miscfs/dmfs/rash.h>
#include <miscfs/dmfs/dmfs_fcntl.h>

#include "nas_logmsg.h"
#include "vv_cl.h"
#include "dblib.h"
#include "nasutil.h"
#include "rashcom.h"
#include "dmfsd.h"

typedef	struct	bfident {
	USR_FILEINFO	*fP;
	fhandle_t	fh;
	vvlist_t	*vvlist;
	int		nvv;
	int		bad;
	struct	bfident	*next;
} bfident_t;

#define	BFID_HASH	0xfff
bfident_t		*barray[BFID_HASH+1];

int	resetflg = TRUE;	/* cause archived status to be turned off */

int	verbose = FALSE;
int	dorecycle = FALSE;

int
bfid_hash(bfid_t *bfid)
{
	int	i;
	int	hash = 0;
	char	*charbfid = (char *)bfid;

	for (i=0; i<sizeof(bfid_t); i++)
		hash += charbfid[i];
	hash &= BFID_HASH;
	return hash;
}

int	hval;		/* remember hash bin */
int	problem;

/*
**	Find the bfid entry that represents a given bfid.
*/
bfident_t	*
bfid_find(bfid_t *bfid)
{
	bfident_t	*bP;

	hval = bfid_hash(bfid);
	for (bP = barray[hval]; bP; bP = bP->next) {
		if (memcmp(&bP->fP->usr_bfid, bfid, sizeof(bfid_t)) == 0)
			break;
	}
	return bP;
}

#define	FULLPRT(x)	(int)sizeof(x), (int)sizeof(x), x

char *
fhinfo(fhandle_t *fhp)
{
	static	char	info[200];
	char		*cp;
	int		i, len;

	len = fhp->fh_fid.fid_len;
	sprintf(info, "fsid %*.*x:%*.*x fid(%d)",
		FULLPRT(fhp->fh_fsid.val[0]),
		FULLPRT(fhp->fh_fsid.val[1]), len);
	cp = info + strlen(info);
	for (i=0; i<len; i++) {
		sprintf(cp, " %d", fhp->fh_fid.fid_data[i]);
		cp += strlen(cp);
	}
	*cp = '\0';

	return info;
}

DB	*db = NULL;

void
bfid_insert(USR_FILEINFO *rP)
{
	static	char	id[] = "bfid_insert";
	bfident_t	*bP;
	vvlist_t	*vvlist;
	time_t		arctm;
	off_t		fsize;
	int		listsize, nvv, i;
	int		ret, err;
	USR_FILEINFO	*uhold, *ufP;
	DBT		key, data;
	bfid_t		bfid;

	/* check if entry is deleted */
	if (rP->usr_rflg & RFLG_DELETED)
		return;

	/* no need to continue if bfid is already there */
	if (bfid_find(&rP->usr_bfid))
		return;

	memset(&key, 0, sizeof(key));
	key.data = &rP->usr_bfid;
	key.size = sizeof(bfid_t);
	memset(&data, 0, sizeof(data));
	ret = db->get(db, NULL, &key, &data, 0);
	if (ret != 0) {
		if (ret == -1)
			nLogMsg(EL_ERROR1|EL_SYS, id, "dbget");
		return;
	}
	assert(data.size = sizeof(fhandle_t));

	bfid = rP->usr_bfid;
	ufP = rashdb_get_bfid(&err, &bfid);
	if (ufP == NULL) {
		nLogMsg(EL_ERROR1, id, "%s: no record, err %d",
			bfidout(&bfid));
		return;
	}
	if (verbose) {
		nLogMsg(EL_STATUS1, id, "saving %s %s",
			bfidout(&rP->usr_bfid), ufP->usr_fname);
	}
	uhold = nAlloc(sizeof(*uhold));
	memcpy(uhold, ufP, sizeof(*uhold));
	uhold->usr_fname = NULL;

	/*
	**	initialize vvlist
	*/
	nvv = 0;
	arctm = ufP->usr_arctm;
	fsize = ufP->usr_fsize;
	listsize = 5;
	vvlist = (vvlist_t *) nAlloc(listsize * sizeof(vvlist_t));

	for (i=0; ; i++) {

		/*
		**	perform sanity checks
		*/
		if (ufP->usr_arctm != arctm) {
			nLogMsg(EL_DEBUG1, id,
				"skipping %s: archive time mismatch",
				bfidout(&bfid));
			nFree(vvlist);
			return;
		}
		if (ufP->usr_fsize != fsize) {
			nLogMsg(EL_ERROR1, id,
				"%s vvno %d fsize(%lld), ino(%lld)",
				bfidout(&bfid), ufP->usr_vvno,
				ufP->usr_fsize, fsize);
			nFree(vvlist);
			return;
		}

		/*
		**	expand VV list if necc
		*/
		if (i == listsize) {
			listsize *= 2;
			vvlist = (vvlist_t *) nReAlloc((void *)vvlist,
						listsize * sizeof(vvlist_t));
		}

		/*
		**	save db info to VV list
		*/
		strcpy(vvlist[i].vvname, ufP->usr_vvname);
		vvlist[i].vvlseek = ufP->usr_bof;
		vvlist[i].flseek  = ufP->usr_lseek;
		vvlist[i].vvdata  = ufP->usr_vvdata;
		vvlist[i].fno	  = ufP->usr_fno;

		nvv += 1;
		/*
		**	get next bfid.VV record
		*/
		if ((ufP = rashdb_next_bfidrec(&err, &bfid)) == NULL) {
			if (err == RASHDB_ERROR) {
				nLogMsg(EL_ERROR1, id,
					"rashdb_next_bfidrec %s failed %d",
					bfidout(&bfid), err);
			}
			break;	/* no more VV records */
		}
	} /* end for loop */
	vvlist = (vvlist_t *)nReAlloc((void *)vvlist, (i+1)*sizeof(vvlist_t));

	bP = nAlloc(sizeof(*bP));
	memset(bP, '\0', sizeof(*bP));
	bP->fP = uhold;
	memcpy(&bP->fh, data.data, sizeof(fhandle_t));
	bP->vvlist = vvlist;
	bP->nvv = nvv;
	bP->bad = TRUE;
	bP->next = barray[hval];
	barray[hval] = bP;

	return;
}

void
bfid_check(USR_FILEINFO *rP)
{
	bfident_t	*bP;

	if (rP->usr_rflg & RFLG_DELETED)
		return;

	/* lookup bfid in list */
	if ((bP = bfid_find(&rP->usr_bfid)) == NULL)
		return;

	if (bP->bad)
		problem = TRUE;
	return;
}

int
main(int argc, char *argv[])
{
	static	char	id[] = "main";
	char		eline[512], *eprt;
	int		err, i, c;
	char		*com;
	char		*dbfile = "/tmp/bfid_fh";
	char		*username;
	USR_FILEINFO	*rP;
	bfident_t	*bP;
	struct	stat	sbuf;
	rashstat_t	rstat;
	int		errflg;
	extern	int	rash_fd;
	extern	char	*optarg;
	extern	int	optind, opterr;

	com = nas_basename(argv[0]);
	errflg = FALSE;
	while ((c = getopt(argc, argv, "rvd:")) != EOF) {
		switch (c) {
		case 'r':
			dorecycle = TRUE;
			break;
		case 'v':
			verbose = TRUE;
			break;
		case 'd':
			dbfile = optarg;
			break;
		case '?':
			errflg = TRUE;
			break;
		}
	}
	if ((optind + 2) > argc)		/* not enough parameters */
		errflg = TRUE;

	if (errflg) {
		fprintf(stderr,
			"usage: %s [-rv][-d dbase] user VV [VV ...]\n", com);
		exit(1);
	}

	if (db_create(&db, NULL, 0) != 0) {
		perror("db_create");
		exit(1);
	}
	if ((err = db->open(db, dbfile, NULL,
			DB_BTREE, DB_RDONLY, 0600)) != 0) {
		sprintf(eline, "%s err %d", dbfile, err);
		perror(eline);
		exit(1);
	}

	nLogInit(com, NULL);
	nLogInitStderr(EL_STATUS1|EL_STATUS2|EL_ERROR1|EL_ERROR2);
	username = argv[optind++];

	if (rashdb_open(username, O_DB_READ) != RASHDB_OK)
		exit(1);
	/*
	**	For each VV on the command line, lookup every file on
	**	the VV and add an entry to the list of files to process.
	*/
	for (i=optind; i<argc; i++) {
		char	*vvname = argv[i];

		rP = rashdb_get_vv(&err, vvname);
		if (rP == NULL) {
			if (err != RASHDB_OK)
				nLogMsg(EL_ERROR1, id, "db_get_vv %d", err);
			continue;
		}
		bfid_insert(rP);
		while ((rP = rashdb_next_vvrec(&err, vvname)) != NULL)
			bfid_insert(rP);
		if (err != RASHDB_OK)
			nLogMsg(EL_ERROR1, id, "db_next_vvrec %d", err);
	}
	db->close(db, 0);

	vv_init(vvHost, RETRY_FOREVER);
	for (i=0; i<=BFID_HASH; i++) {
		for (bP = barray[i]; bP; bP = bP->next) {

			rash_fd = fhopen(&bP->fh, O_WRONLY|O_ALT_IO);
			if (rash_fd == -1) {
				nLogMsg(EL_ERROR1|EL_SYS, "fhopen",
					"bfid %s fh %s",
					bfidout(&bP->fP->usr_bfid),
					fhinfo(&bP->fh));
				continue;
			}
			if (fstat(rash_fd, &sbuf) == -1) {
				nLogMsg(EL_ERROR1|EL_SYS, "fstat",
					"bfid %s fh %s",
					bfidout(&bP->fP->usr_bfid),
					fhinfo(&bP->fh));
				close(rash_fd);
				continue;
			}
			if (fcntl(rash_fd, DMFS_GETRINFO, &rstat) == -1) {
				nLogMsg(EL_ERROR1|EL_SYS,
					"fcntl(DMFS_GETRINFO)",
					"bfid %s fh %s",
					bfidout(&bP->fP->usr_bfid),
					fhinfo(&bP->fh));
				close(rash_fd);
				continue;
			}
			if (memcmp(&rstat.st_bfid, &bP->fP->usr_bfid,
					sizeof(bfid_t))) {
				nLogMsg(EL_ERROR1|EL_SYS, "memcmp",
					"bfid mismatch %s fh %s",
						bfidout(&bP->fP->usr_bfid),
						fhinfo(&bP->fh));
				close(rash_fd);
				continue;
			}

			if (verbose) {
				eprt = eline;
				for (c=0; c<bP->nvv; c++) {
					sprintf(eprt, " %d:%s", c,
						bP->vvlist[c].vvname);
					eprt += strlen(eprt);
				}
				nLogMsg(EL_STATUS1, id,
					"restoreing %s from%s",
					bfidout(&rstat.st_bfid), eline);
			}
			bP->bad = restfile(bP->fP, &sbuf, &rstat,
				bP->vvlist, bP->nvv);
		}
	}

	if (dorecycle) {
		rashdb_close();		/* was open for read */

		if (rashdb_open(username, O_DB_RDWR) != RASHDB_OK)
			exit(1);
	}

	for (i=optind; i<argc; i++) {
		char	*vvname = argv[i];

		rP = rashdb_get_vv(&err, vvname);
		if (rP == NULL)
			continue;

		problem = FALSE;
		bfid_check(rP);
		while ((rP = rashdb_next_vvrec(&err, vvname)) != NULL)
			bfid_check(rP);
		if (err != RASHDB_OK || problem)
			continue;

		if (dorecycle == FALSE) {
			nLogMsg(EL_STATUS1, id, "skipping recycle for %s",
				vvname);
			continue;
		}
		if (verbose)
			nLogMsg(EL_STATUS1, id, "recycling %s", vvname);
		if (vv_recycle(vvname)) {
			vvErrorMsg(EL_ERROR1, id, vvname);
			continue;
		}

		/*
		**	The VV has been recycled.  Now, go back
		**	over the records in the database and
		**	mark them to show the new status.
		*/
		rP = rashdb_get_vv(&err, vvname);
		if (err != RASHDB_OK)
			break;
		rP->usr_rflg |= RFLG_DELETED;
		rP->usr_vflg |= VFLG_RECYCLE;
		if (rashdb_set_flgs(rP->usr_rflg, rP->usr_vflg) != RASHDB_OK)
			continue;
		while ((rP = rashdb_next_vvrec(&err, vvname)) != NULL) {
			rP->usr_rflg |= RFLG_DELETED;
			rP->usr_vflg |= VFLG_RECYCLE;
			if (rashdb_set_flgs(rP->usr_rflg, rP->usr_vflg)
					!= RASHDB_OK)
				break;
		}
		if (err != RASHDB_OK)
			break;
	}
	rashdb_close();
	vv_term();
	return 0;
}
