/*
 *  vchown & vchgrp commands -- user interface to change 
 *	uid or group id of volumes in the volman(8) database.
 *
 *	Bill Ross, NASA Ames, 3/92
 */

#include <stdio.h>
#include <ctype.h>

#include "cl_space.h"

extern int errno;

static char	*arg0, *cmdarg;
static int	vchown = 1;

/*
 * Print out usage message and exit.
 */
 void
usage()
{
	fprintf(stderr,
	  "usage: %s %s volsers \n", arg0, cmdarg);
	exit(1);
}

main(int argc, char *argv[])
{
	int		id, rval = 0;
	char		pbuf[PKTSIZ(vmpVsn_t)];
	vmpHdr_t	*shp;
	vmpVsn_t	*vp;

	arg0 = nas_basename(argv[0]);
	if (!strcmp(arg0, "vchown")) {
		cmdarg = "user";
	} else if (!strcmp(arg0, "vchgrp")) {
		cmdarg = "group";
		vchown = 0;
	} else {
		fprintf(stderr, "unknown name or link to vch[own|grp]: %s\n",
							arg0);
		exit(1);
	}

	if (argc < 3)
		usage();

	/*
	 *  get uid (vchown) or gid (vchgrp)
	 */

	argv++;
	if (vchown) {
		if (vm_userid(*argv, &id) == -1) {
			if (errno == ENOENT)
				fprintf(stderr, 
					"%s: not defined in system\n", *argv);
			else
				perror(*argv);
			usage();
		}
	} else {
		if (vm_groupid(*argv, &id) == -1) {
			if (errno == ENOENT)
				fprintf(stderr, 
					"%s: not defined in system\n", *argv);
			else
				perror(*argv);
			usage();
		}
	}

	/*
	 *  connect to volcd & build pkt
	 */

	if (vm_init(arg0)) {
		vm_prerr(stderr);
		exit(VMError);
	}
	if (vchown) 
		shp = vBuildHdr(M_VCHOWN, vGetSeq(), (vmpHdr_t *) pbuf);
	else
		shp = vBuildHdr(M_VCHGRP, vGetSeq(), (vmpHdr_t *) pbuf);

	vp = MSGPTR(shp, vmpVsn_t);
	vp->v_rc = id;

	/*
	 *  last arg(s) = volume serial number(s) (volser): try to do them
	 */

	for (++argv; *argv != NULL; argv++) {
		if (checkvsn(*argv)) {
			printf("\n%s\n", nErrStr);
			rval++;
			continue;
		}
		printf("sending %s\n", *argv);
		memcpy(&vp->v_vsn, *argv, VSNLEN);
		if (lSnd2RP(shp)) {
			vm_prerr(stderr);
			exit(VMError);
		}
		rval += GetResponse(*argv, shp->vh_seq);
		shp->vh_seq++;
	}
	vm_term();
	exit(rval);
}

int
GetResponse(char *vol, Seq_t seq)
{
	vmpHdr_t	*rhp;
	vmpVsn_t	*vsp;
	vmpError_t	*ep;

	for (;;) {
		if ((rhp = lRcvFromRP(IPC_DELAY)) == NULL) {
			vm_prerr(stderr);
			exit(VMError);
		}
		if (seq != rhp->vh_seq  &&  rhp->vh_type != M_SHUTDOWN) {
			fprintf(stderr, "bad packet sequence (skipping): %s", 
								vHdrStr(rhp));
			continue;
		}
		switch (rhp->vh_type) {
		    case M_VCHOWN:
		    case M_VCHGRP:
			vsp = MSGPTR(rhp, vmpVsn_t);
			if (strncmp((char *)&vsp->v_vsn, vol, VSNLEN)) {
				printf("%.6s: wrong vsn returned\n",
								&vsp->v_vsn);
				exit(1);
			}
			/* printf("%.6s ok\n"); */
			nFree(rhp);
			return(0);
		    case M_VERROR: 
			ep = MSGPTR(rhp, vmpError_t);
			printf("error from vold: %s\n",ep->vn_msg);
			if (ep->vn_error != E_USER)
				exit(1);
			nFree(rhp);
			return(1);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			exit(1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			printf("Unexpected packet: %s\n", vHdrStr(rhp));
			exit(1);
		}
	}
}
