/*
 *  vconfig command -- user interface to get info on volman(8)
 *
 *
 *	Bill Ross, NASA Ames, 1995
 */

/* #define DEBUG */


#include <stdio.h>
#include <ctype.h> 
#include <pwd.h>

#include "cl_space.h"

extern int 	errno;

extern RC_t		*RCArray;
extern int		RCTotal;
extern HostInfo_t	*Hosts;
extern int		HostTotal;

/*
 * Print out usage message and exit.
 */
  void
usage(char *arg0)
{
	fprintf(stderr, "usage: %s [-r RC] [-h]\n", arg0);
	exit(1);
}

 void
PrintRC(RC_t *rcp)
{
	int	i;
	for (i=0; i<rcp->rcd_nmedia; i++) {
		printf("ID %s NUMBER %d LOGIN %s",
			rcp->rcd_id,
			rcp->rcd_number,
			rcp->rcd_login);
		printf(" MEDIUM %s DRIVES %d VARIED_ON %d MOUNTS %d\n",
			vMediumName(rcp->rcd_devs[i].d_medium),
			rcp->rcd_devs[i].d_nconfigdev,
			rcp->rcd_devs[i].d_nvaryondev,
			rcp->rcd_devs[i].d_mountdev);
	}
}

 void
PrintHosts()
{
	HostInfo_t	*hop = Hosts;
	int		i;

	for (i=0; i<HostTotal; i++, hop++) {
		printf("HOST %.*s VOLCD %c CLIENTS %d VOLND %c\n",
			HOSTNAMELEN, hop->h_name,
			(hop->h_flag & H_HASVOLCD  ? 'y' : 'n'),
			hop->h_nclients,
			(hop->h_flag & H_HASVOLND  ? 'y' : 'n'));
	}
}


main(int argc, char *argv[])
{
	char		*arg0 = argv[0];
	extern  int     optind;
	extern  char    *optarg;
	RC_t		*rcp;
	int		cc, didopt = 0;

	/*
	 *  connect to voldaemon & get rc config 
	 */
	if (vm_init(arg0)) {
		vm_prerr(stderr);
		exit(VMError);
	}
	if (vm_configRC()) {
		vm_prerr(stderr);
		exit(VMError);
	}
	if (vm_hostinfo()) {
		vm_prerr(stderr);
		exit(VMError);
	}

	/*
	 *  process args
	 */
	while ((cc = getopt(argc, argv, "hr:")) != EOF) {
		switch (cc) {
		    case 'r':
			rcp = cGetRcp(optarg);	
			if (rcp == NULL) {
				fprintf(stderr, "%s: bad RC\n", optarg);
				usage(arg0);;
			}
			PrintRC(rcp);
			didopt++;
			break;
		    case 'h':
			PrintHosts();
			didopt++;
			break;
		    case '?':
		    default:
			fprintf(stderr, "unknown option: %c\n", cc);
			usage(arg0);
		}
	}
	if (!didopt) {
		PrintHosts();
		rcp = RCArray;
		for (; didopt<RCTotal; didopt++,rcp++)
			PrintRC(rcp);
	}

	vm_term();
	exit(0);
}

