#ifndef lint
static char	rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/clnt/vol.c,v 1.1.1.1 2000-02-28 02:18:24 wrstuden Exp $";
#endif

/*
 *  vol command -- user interface to allocate and [un]mount volumes.
 *
 */

#include "cl_space.h"

#include <stdio.h>
#include <signal.h>
#include <ctype.h>

extern int	VMError;

static char	id[20];
static char	*arg0;

/*
 *  usage() - Print out usage message and exit.
 *	-a is an undocumented synonym for -q
 * 	-t is an undocumented option that will turn full debugging on
 */
 void
usage(char *opts)
{
	arg0 = nas_basename(arg0);
	if (opts)
		fprintf(stderr, "You must choose only one of the %s options.\n",
		  opts);
	fprintf(stderr, "Usage:\n");
	fprintf(stderr,
"  mount:    %s [-m] [-q] [-i rid] [-t tag] [-{R|w}] [-b] volser\n",
	  arg0);
	fprintf(stderr,
"            %s -p  [-t tag] [-{R|w}] [-b] volser(s)\n",
	  arg0);
	fprintf(stderr,
"            %s -1  [-t tag] [-{R|w}] [-b] volser(s)\n",
	  arg0);
	fprintf(stderr,
"  unmount:  %s -u [-q] volser\n", arg0);
	fprintf(stderr,
"  reattach: %s -s volser\n", arg0);
	exit(E_USER);
}

done()
{
	int	rval = VMError;
	vm_term();
	exit(rval);
	return(rval);		/* for compiler warning */
}


main(int argc, char *argv[])
{
	int		cc, sig,
			flag, zflag = 0; /* zflag catches bad arg combos */
	char		volser[VSNLEN+1];
	int		reslen = 0;
	char		*ridp = NULL, *tag = NULL, deftag[TAGLEN+1];
	extern  int     optind;
	extern  char    *optarg;
	extern	char	*vSpoolDir;
		char	*log; 
	int		tflg = 0, cmd, cmdset, prob;

	arg0 = argv[0];
	sprintf(id,"vol(%d)",getpid());
	
	/*
	 *  setup defaults:
	 *	Although VO_WAIT is set, as soon as status is received,
	 *	we send an M_DETACH packet because the next thing we do
	 *	is exit.
	 */
	
	cmd = M_USRMOUNT;
	cmdset = 0;
	flag = VO_WAIT;
	
	while ((cc = getopt(argc, argv, "Rrwbmpus1aqnTi:d:t:")) != EOF) {
		switch (cc) {
		    case 'R':				/* ignore ring */
			if (zflag & VO_WRITE)
				usage("w, R");
			flag |= VO_IGNORE_RING;
			zflag |= VO_IGNORE_RING;
			break;
		    case 'w':				/* mount for write */
			if (zflag & VO_IGNORE_RING)
				usage("w, R");
			flag |= VO_WRITE;
			zflag |= VO_WRITE;
			break;
		    case 'b':			/* bypass label checking */
			flag |= VO_BYPASS;
			zflag |= VO_BYPASS;
			break;
		    case 'm':				/* mount */
			if (cmdset)
				usage("m, p, 1, u, r, s");
			cmdset = 1;
			cmd = M_USRMOUNT;
			break;
		    case 'p':
			if (cmdset)
				usage("m, p, 1, u, r, s");
			cmdset = 1;
			cmd = M_USRMOUNT_PLL;
			break;
		    case '1':
			if (cmdset)
				usage("m, p, 1, u, r, s");
			cmdset = 1;
			cmd = M_USRMOUNT_SEL;
			break;
		    case 'i':			/* use a previous reservation */
			reslen = strlen(optarg);
			if (!reslen) {
				fprintf(stderr, 
				  "reservation id required with -i option\n");
				usage("m, i");
			}
			ridp = optarg;
			break;
		    case 'u':				/* unmount */
			if (cmdset)
				usage("m, p, 1, u, r, s");
			cmdset = 1;
			cmd = M_UNMOUNT;
			break;
		    case 'r':			/* requeue an active mount */
			if (cmdset)
				usage("m, p, 1, u, r, s");
			cmdset = 1;
			cmd = M_REQUEUE;
			break;
		    case 's':
			if (cmdset)
				usage("m, p, 1, u, r, s");
			cmdset = 1;
			cmd = M_ATTACH;
			flag &= ~(VO_WAIT);
			break;
		    case 'q':			/* don't notify */
			flag &= ~VO_RESPONSE;
			flag |= VO_ASYNC;		/* async request */
			zflag |= VO_ASYNC;		/* async request */
			break;
		    case 't':
			tag = optarg;
			if (tag == NULL)
				usage(NULL);
			break;
		    case 'T':
			tflg++;
			break;
		    case '?':
		    default:
			usage(NULL);
		}
	} /* while */

	/*
	 *  check consistency
	 */
	if ( cmd != M_USRMOUNT  &&  cmd != M_USRMOUNT_PLL  &&  
	     cmd != M_USRMOUNT_SEL  &&
	     (reslen  ||  tag != NULL  ||
	      flag & (VO_IGNORE_RING|VO_WRITE|VO_BYPASS)) ) {

		/*
		 *  mount-type flags used w/ non-mount
		 */
		fprintf(stderr, "Inappropriate option(s) for non-mount.\n");
		usage(NULL);
	}
	argc -= optind;
	argv += optind;

	if (cmd == M_USRMOUNT_PLL  ||  cmd == M_USRMOUNT_SEL) {
		int	i;

		/*
		 *  parallel mount, options limited
		 */
		if (ridp != NULL) {
			fprintf(stderr, "-i incompatible w/ -p\n");
			usage(NULL);
		}
		if (flag & VO_ASYNC) {
			fprintf(stderr, "-q incompatible w/ -p\n");
			usage(NULL);
		}
		for (i=0; i<argc; i++) {
			if (checkvsn(argv[i])) {
				printf("\n%s\n", nErrStr);
				exit(E_USER);
			}
		}
	} else {
		if ( cmd == M_ATTACH  &&  (flag & (VO_ASYNC))) {
			fprintf(stderr, 
				"Inappropriate option(s) for reattach.\n");
			usage(NULL);
		}
		if (argc != 1) {
			fprintf(stderr, "No vsn\n");
			usage(0);
		}
		/*
		 *  last (and only) arg is the volume serial number (volser)
		 */
		if (checkvsn(*argv)) {
			printf("\n%s\n", nErrStr);
			exit(E_USER);
		}
		memcpy(volser, *argv, VSNLEN);
	}


	for (sig = 1; sig < NSIG; sig++) {
#ifdef DEBUG
		if (sig == SIGQUIT || sig == SIGSEGV || sig == SIGBUS)
			continue;
#endif
		if (sig == SIGKILL ||  sig == SIGSTOP)
			continue;
		if (SIGNAL(sig, done))
			fprintf(stderr,"signal %d\n", sig);
	}

	/*
	 *  if the process was started in the background then ignore 
	 *  the keyboard interrupt signal by disconnecting from the terminal
	 */

	if (vSetPgrp()) {
		perror("setpgrp failed");
		exit(E_SYS);
	}

	nLogInit(id, done); 
	nLogInitStderr(EL_LEVEL);

	/*
	 *  Turn full tracing on if trace flag was specified
	 *  Note: trace information is recorded in the voldaemon log, thus
 	 *  root priviledge must be established.
	 */

	if (tflg) {
		sprintf(id,"vol(%d)",getpid());
		vInitDirNames();
		nLogSetLevel(EL_DEBUG4);
		nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
	}

	if (_vm_init(id, 0)) {
		vm_prerr(stderr);
		exit(VMError);
	}
	if (vm_hostinfo()) {
		vm_prerr(stderr);
		exit(VMError);
	}
		

	/*
	 *  send request
	 */

	switch (cmd) {
	    case M_USRMOUNT:
		if (tag == NULL) {
			sprintf(deftag, "V.%.*s", TAGLEN-2, 
						nGetUIDName(getuid()));
			tag = deftag;
		}
		prob = vm_mount((vsn_t *)volser, flag, ridp, tag, 
							!(flag & VO_ASYNC),
							!(flag & VO_WAIT));
		break;
	    case M_USRMOUNT_PLL:
		if (tag == NULL) {
			sprintf(deftag, "VP.%.*s", TAGLEN-2,
						nGetUIDName(getuid()));
			tag = deftag;
		}
		prob = vm_mount_pll(argc, argv, flag, tag);
		break;
	    case M_USRMOUNT_SEL:
		if (tag == NULL) {
			sprintf(deftag, "VS.%.*s", TAGLEN-2,
						nGetUIDName(getuid()));
			tag = deftag;
		}
		prob = vm_mount_sel(argc, argv, flag, tag);
		flag &= ~VO_WAIT;
		break;
	    case M_ATTACH:
		prob = vm_attach((vsn_t *)volser, flag, !(flag & VO_WAIT));
		break;
	    case M_UNMOUNT:
		prob = vm_unmount((vsn_t *)volser, flag, !(flag & VO_ASYNC),
							!(flag & VO_WAIT));
		break;
	    default:
		exit(E_INTERNAL);
	}
	if (prob) {
		vm_prerr(stderr);
		done();
	}
	if (flag & VO_WAIT) {
		if (vm_prvolstat((vsn_t *)volser, stdout)) { /* print status */
			vm_prerr(stderr);
			done();
		}
	}

	/*
	 *  Must detach from request; e.g. otherwise death of this process
	 *	will cause mounted vol to be unmounted.  Unmount requests
	 *	are deleted on detach unless they are explicitly reserved.
	 */
	
	if (!(flag & VO_ASYNC)  &&  cmd != M_UNMOUNT)
		if (vm_detach()) {
			vm_prerr(stderr);
			done();
		}
	done();
#ifdef lint
	return 0;
#endif
}
