/*
 *  voldeldb.c - delete volsers from the Volume Manager dbase.
 *	Only works if volume is in vault (not robot).
 *
 *	B. Ross, NASA Ames Research Center, 11/4/88, 2/28/92
 */

#include        "cl_space.h"

#include	<stdio.h>
#include	<time.h>
#include	<pwd.h>
#include	<ctype.h>

extern int	errno;

 void
usage(char *str)
{
	fprintf(stderr, "usage: %s [-f] volsers\n", str);
	fprintf(stderr, "usage: %s [-f] < volsers\n", str);
	exit(1);
}

main(int argc, char *argv[])
{
	int 		j, force = 0, rval = 0, fromstdin = 0;
	char		pbuf[PKTSIZ(vmpVsn_t)];
	vmpHdr_t	*shp;
	vmpVsn_t	*vp;
	
	/*
	 *  check arg
	 */
	if (argc > 1) {
		if (argv[1][0] == '-') {
			if (strcmp(argv[1], "-f"))
				usage(argv[0]);
			force = 1;
		}
	}
	if (isatty(fileno(stdin))) {
		/*
		 *  want to see explicit vsns
		 */
		if (argc - force < 2)
			usage(argv[0]);
	} else {
		/*
		 *  don't want to see explicit vsns
		 */
		if (argc - force > 1)
			usage(argv[0]);

		fromstdin++;
	}

	/*
	 *  connect to vold & set up packet
	 */

	if (vm_init(argv[0])) {
		vm_prerr(stderr);
		exit(VMError);
	}
	shp = vBuildHdr(M_DELVSN, vGetSeq(), (vmpHdr_t *) pbuf);
	vp = MSGPTR(shp, vmpVsn_t);
	if (force)
		vp->v_flag = VQ_FORCE;
	else
		vp->v_flag = VQ_VSN;	/* anything but VQ_FORCE */

	for (j=force+1; ; j++) {
		if (fromstdin) {
			char	stdinvsn[100];
			int	ret;

			ret = scanf("%s", stdinvsn);
			if (ret != 1) {
				if (ret == EOF)
					break;
				perror("stdin");
				vm_term();
				exit(1);
			}
			if (checkvsn(stdinvsn)) {
				printf("\n%s\n", nErrStr);
				rval++;
				continue;
			}
			memcpy(&vp->v_vsn, stdinvsn, VSNLEN);	
		} else {
			if (j >= argc)
				break;
			if (checkvsn(argv[j])) {
				printf("\n%s\n", nErrStr);
				rval++;
				continue;
			}
			memcpy(&vp->v_vsn, argv[j], VSNLEN);
		}

		printf("sending %.*s\n", VSNLEN, &vp->v_vsn);
		if (lSnd2RP(shp)) {
			vm_prerr(stderr);
			exit(VMError);
		}
		rval += GetResponse(&vp->v_vsn, shp->vh_seq);
		shp->vh_seq++;
	}
	vm_term();
	exit(rval);
}

int
GetResponse(vol, seq)
char	*vol;
Seq_t	seq;
{
	vmpHdr_t	*rhp;
	vmpVsn_t	*vp;
	vmpError_t	*ep;

	for (;;) {
		if ((rhp = lRcvFromRP(IPC_DELAY)) == NULL) {
			vm_prerr(stderr);
			vm_term();
			exit(VMError);
		}
		if (seq != rhp->vh_seq  &&  rhp->vh_type != M_SHUTDOWN) {
			fprintf(stderr, "bad packet sequence (skipping): %s\n", 
								vHdrStr(rhp));
			continue;
		}
		switch (rhp->vh_type) {
		    case M_DELVSN:
			vp = MSGPTR(rhp, vmpVsn_t);
			if (strncmp((char *)&vp->v_vsn, vol, VSNLEN)) {
				printf("%.*s: wrong vsn returned\n", VSNLEN,
								&vp->v_vsn);
				vm_term();
				exit(1);
			}
			printf("deleted %.*s owner %d\n", VSNLEN,vol, vp->v_rc);
			nFree(rhp);
			return(0);
		    case M_VERROR: 
			ep = MSGPTR(rhp, vmpError_t);
			printf("error from vold: %s\n",ep->vn_msg);
			if (ep->vn_error != E_USER) {
				vm_term();
				exit(1);
			}
			nFree(rhp);
			return(1);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			exit(1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			printf("Unexpected packet: %s\n", vHdrStr(rhp));
			vm_term();
			exit(1);
		}
	}
}
