/*
 *  volentdb.c - add a list of volumes to the Volume Descriptor Database. 
 *
 *	For volsers, accepts either the name of file containing a list 
 *	of volsers, a list, or a numerical range of volsers (first character
 *	must be a letter) 
 *
 *	Volsers can be added as virgin, scratch or alloc to round
 *	or square vault or square robot - assumes NAS rc ids.
 *
 *	B. Ross, NASA Ames Research Center, 11/4/88
 */

#include	"cl_space.h"

#include	<stdio.h>
#include	<time.h>
#include	<ctype.h>
#include	<pwd.h>

extern RC_t		*RCArray;
extern int		RCTotal;
extern VolType_t	VolTypeTab[];

int		rc;
short		mode;
char		status;
int 		uid, poolgid, voltype;

FILE		*log;
int		logg = 0;

extern int	errno;


char 		*volslist;
int 		noerror = 1;
int		entries = 0;
int		ilabel = 0;
int		voln;
int		totvol = 0;
char		instring[80];
FILE 		*fp;

#define SCANSIZE	200
char		*scanbuf, inbuf[SCANSIZE];

void
getline()
{
	char	*cp;

start:
	while (fgets(inbuf, SCANSIZE, stdin) == NULL) {
		if (feof(stdin))
			exit(1);
		printf("Nothing read; try again: ");
	}
	cp = strchr(inbuf, '\0');
	cp--;
	if (cp == inbuf) {
		printf("Nothing read; try again: ");
		goto start;
	}
	*cp = '\0';

	scanbuf = inbuf;
	while (isspace(*scanbuf))
		scanbuf++;
	if (strlen(scanbuf) == 0) {
		printf("Nothing read; try again: ");
		goto start;
	}
}

/*
 *  getsequence - get a sequence of volsers, calculating total # of volsers
 */
 void
getsequence()
{
	char	c;
	int	entry, i, okvols = 0;

	printf("\nEntering Series - First character (alphanumeric): ");
	while (1) {
		getline();
		i = strlen(scanbuf);
		if (i > 1) {
			printf("Type ONE alphanumeric character: ");
			continue;
		}
		if (!i) {
			printf("Type an alphanumeric character: ");
			continue;
		}
	    	c = toupper(scanbuf[0]);
	    	if(isalnum(c))
			break;

		printf( "\nIllegal character: %c %s",
				c, "(alphanumeric only) try again: ");
	}
	while (1) {
		printf("\n\nEnter First Volser (up to 5 digits): ");
		getline();
		if (!isnum(scanbuf)) {
			printf("\nmust be a decimal number");
			continue;
		}
		if (strlen(scanbuf) > 5) {
			printf("\nmax digits is 5");
			continue;
		}
		if (sscanf(scanbuf, "%d", &i) != 1) {
			printf("\nbad number (%s) - try again\n", scanbuf);
			continue;
		}
		if (i < 0) 
			printf("\nERROR: non-negative number required\n");
		else
			break;
	}
	while (1) {

		printf("\nEnter Number of Volsers: ");
		getline();
		if (!isnum(scanbuf)) {
			printf("\nERROR: number required\n");
			continue;
		}
		if (sscanf(scanbuf, " %d", &totvol) != 1)
			continue;
		if (totvol < 1) 
			printf("\nERROR: 0 volumes or backward sequence\n");
		else
			break;
	}
	if (i + totvol > 99999) {
		printf(
		    "\nERROR: Can only add up to volume 99999 sequentially\n");
		vm_term();
		exit(1);
	}
	volslist = (char *) nAlloc(7 * totvol);
	voln = 0;
	for (entry=0; entry<totvol; entry++) {
		sprintf(volslist+voln, "%c%05d", c, i++);
		if (checkvsn(volslist+voln)) {
			printf("\n%s\n", nErrStr);
			exit(1);
		}
		okvols++;
		voln += 6;
		volslist[voln] = ' ';
		voln += 1;
	}
	if (okvols < 1) {
		printf("\nNo valid volumes - exiting\n");
		vm_term();
		exit(1);
	}

}

/*
 *  printvols - print volser list
 */
 void
printvols()
{
	int i;

	voln = 0;
	for(i=0;i<totvol;i++) {
		printf("%.*s ", VSNLEN, volslist+voln);
		voln += 7;
		if (!((1+i)%10))
			printf("\n");
	}
}



main(int argc, char *argv[])
{
	char		*id = argv[0];
	char 		c, str[20], path[80];
	int  		i, j, volmanuid;
	VolType_t	*vtp;
	RC_t		*rcp;
	struct passwd	*pwp;
	int		err = 0;

	uid = getuid();
	if (pwp = getpwnam(VolManOperAcct)) {
		volmanuid = pwp->pw_uid;
	} else {
		perror("getpwuid for volman");
		exit(1);
	}
	if (uid != 0  &&  uid != volmanuid) {
		fprintf(stderr, "Must be root or %s to run\n", VolManOperAcct);
		exit(1);
	}

	/*
	 *  start vold connection
	 */

	printf("Connecting to vold..\n");
	fflush(stdout);
	if (vm_init(id)) {
		vm_prerr(stderr);
		exit(VMError);
	}

	if (vm_configRC()) {
		vm_prerr(stderr);
		vm_term();
		exit(1);
	}
	
	/*
	 * get vsns
	 */

	if (argc > 1) {

		/*
		 * vsns in arglist 
		 */
		volslist = (char *) nAlloc(7 * (argc - 1));
        	voln = 0;
		for (i=1; i<argc; i++) {
			if (checkvsn(argv[i])) {
				printf("\n%s\n", nErrStr);
			} else {
				sprintf(&volslist[voln], "%s ", argv[i]);
				voln = voln + 7;
				totvol++;
			}
		}
	} else {
		
		/*
		 * interactive specing of vsns
		 */

		printf("\n        ** Volume Manager Volume Entry **\n");
		printf("\nChoose mode of VSN entry:");
		printf("\n   (S) Sequential expression mode (specify a range)");
		printf("\n   (F) File");
		while (1) {
		    printf("\n\nChoose (S/F): ");
		    getline();
		    if (strlen(scanbuf) != 1)
			continue;
		    switch(scanbuf[0]) {
			case 'F':
			case 'f':
				while (1) {
					printf("\nFile name: ");
					getline();
					sscanf(scanbuf, "%s", path);
					fp = fopen(path, "r");
					if (fp == NULL) 
						perror("\nOpen failed");
					else
						break;
				}
				for (totvol=0;j!=EOF;totvol++) /* count vsns */
					j = fscanf(fp, "%s", str);
				fclose(fp);
				totvol--;
				printf("\n%d item(s) in %s",totvol, path);
				fp = fopen(path, "r");
				if (fp == NULL) {
					perror("2nd Open failed");
					vm_term();
					exit(1);
				}
				volslist = (char *) malloc(7 * totvol);
        			voln = 0;
        			i = 0;
				str[0] = ' ';
				while (1) {
					j = fscanf(fp, "%s", &volslist[voln]);
					if (j == EOF  || i == totvol)
						break;
					if (checkvsn(&volslist[voln])) {
						printf("\n%s\n", nErrStr);
					} else {
						voln = voln + 7;
						volslist[voln-1] = ' ';
						i++;
					}
				}
				fclose(fp);
				totvol = i;
				printf("\n%d volume(s) ok in %s\n",totvol,path);
				break;
			case 'S':
			case 's': 
				getsequence();
				break;
			default:
				printf("\n bad selection: %c: try again",c);
				continue;
		    }
		    break;
		}
	}
	if (totvol == 0) {
		printf("\nNo volumes defined - exiting\n");
		vm_term();
		exit(0);
	}
	printf("\n\nView List? (y/n) ");
	if (getYorN() == 'y')
		printvols();
 	printf("\nOK to add? (y/n) ");
	if (getYorN() == 'n')
		exit(1);

	/*
	 *  volser list ok'd by user, so get other info for adding
	 */

	/*
	 *  get voltype info 
	 */
	while (1) {
		printf("\nTape type: pick a number: ");
		printf("\n\tnumber\tdescription\n");
		for (i=1,vtp=VolTypeTab; vtp->vt_name; vtp++,i++)
			printf("\t%d\t%s\n", i, vtp->vt_name);
		printf("number: ");
		getline();
		if (!isnum(scanbuf)) {
			printf("not a number\n");
			continue;
		}
		if (sscanf(scanbuf, "%d", &voltype) != 1) {
			printf("\nBad number (%s) - try again ", scanbuf);
			continue;
		}
		if (voltype < 1 || voltype >= i) {
			printf("\nBad number (%d) - try again ", voltype);
			continue;
		}
		voltype--;
		vtp = &VolTypeTab[voltype];
		/* set 'real' voltype global */
		voltype = vtp->vt_map;
		break;
	}

	/*
	 *  figure RC considering voltype
	 */
	for (j=0,i=0; i<RCTotal; i++) {
		if (RCArray[i].rcd_media & vtp->vt_medium) {
			rc = i;
			j++;
		}
	}
	if (j == 0) {
		fprintf(stderr, "\nNo RC is configured to hold type: %s\n", 
							vtp->vt_name);
		exit(1);
	}
	if (j == 1) {
		printf("\nOnly RC that can hold %s is %s. Ok? (y/n): ",
			vtp->vt_name, RCArray[rc].rcd_name);
		if (getYorN() == 'n')
			exit(0);
		rc = RCArray[rc].rcd_number;
	} else {
		while (1) {

			printf("\nChoose from RCs that can hold %s:\n", 
								vtp->vt_name);
			for (i=0; i<RCTotal; i++) {
				if (RCArray[i].rcd_media & vtp->vt_medium) 
					printf("\t%d\t%s\n", 
						RCArray[i].rcd_number, 
						RCArray[i].rcd_name);
			}
			printf("number: ");
			getline();
			if (sscanf(scanbuf, "%d", &rc) != 1) {
				printf("bad number (%s) - try again\n",scanbuf);
				continue;
			}
			rcp = cRCInfo(rc);
			if (rcp == NULL) {
				printf("bad number (%d)\n", rc);
				continue;
			}
			if ( rcp->rcd_media & vtp->vt_medium )
				break;
			printf("bad number (%d)\n", rc);
		}
	}

	/*
	 *  get status
	 */

	while (1) {
		printf("\nScratch (labelled), Virgin or Alloc? (S/V/A): ");
		getline();
		if (strlen(scanbuf) != 1) {
			printf("1 character expected");
			continue;
		}
		switch (scanbuf[0]) {
		    case 's':
		    case 'S':
			status = S_SCRATCH;
			ilabel = 1;
			mode = 0400;		/* no write till allocated */
			break;
		    case 'v':
		    case 'V':
			status = S_VIRGIN;
			mode = 0600;		/* ready to write label */
			break;
		    case 'a':
		    case 'A':
			/* old tapes, ownership not known */
			status = S_ALLOC;
			mode = 0640;		
			break;
		    default:
			printf("\nBad Initialization (%c) - try again ", 
								scanbuf[0]);
			continue;
		}
		break;
	}
	uid = volmanuid;
	if (status == S_ALLOC) {

		while (1) {
			printf("\nAllocated volumes - give user id: ");
			getline();
			if (vm_userid(scanbuf, &uid) == 0)
				break;
			printf("uid not in /etc/passwd\n");
		}
		printf("\nAssume internal label == external? (y/n) ");
		if (getYorN() == 'y')
			ilabel = 1;
	}

	/*
	 *  get 'pool group'
	 */

	while (1) {
		printf("\ngive group id for pool (0 is for unprotected access");
		printf("\n  with no tracking of batches): ");
		getline();
		if (vm_groupid(scanbuf, &poolgid) == 0)
			break;
		if (errno)
			printf("gid not in /etc/group (%s)\n", strerror(errno));
		else
			printf("gid not in /etc/group\n");
	}
	printf("\nFilename to output list of entries (^D for no list): ");
	while(1) {
		if (scanf("%s", scanbuf) != EOF) {
			if ((log = fopen(scanbuf, "a")) == NULL) {
				perror("open failed");
				printf("\nFilename (<return> for no list): ");
				continue;	/* bad name */
			}
			logg = 1;
			break;	/* got name */
		} else {
			logg = 0;
			break;	/* not logging */
		}
	}
	err += addtovdd();
	if (logg)
		fclose(log);
	printf("\n\n");
	return(err);
}

	
/*
 *  addtovdd - insert Volume Descriptor Records in database
 */
 
addtovdd()
{
	vmpHdr_t	*shp;
	vmpAdd_t	*ap;
	char		pbuf[PKTSIZ(vmpAdd_t)];
	char		*vol;
	int 		i, err = 0;

/* printf("\nstatus %c  uid %d  mode %o  rc %d\n", status, uid, mode, rc); */
	/*
	 *  set up send msg buf pointers & constant values
	 */

	shp = vBuildHdr(M_ADDVSN, vGetSeq(), (vmpHdr_t *) pbuf);
	ap = MSGPTR(shp, vmpAdd_t);

	ap->vad_vinfo.vi_voltype = voltype;
	ap->vad_status = status;
	ap->vad_vinfo.vi_uid = uid;
	ap->vad_vinfo.vi_gid = 0;
	ap->vad_vinfo.vi_poolgid = poolgid;
	ap->vad_vinfo.vi_mode = mode;
	ap->vad_vinfo.vi_srcrc = rc;
	ap->vad_vinfo.vi_dstrc = 0;

	printf("\n\n..sending requests to vold..\n");
	voln = 0;	
	for (i=0; i<totvol; i++) { 

		vol = volslist+voln;
		printf("Sending: (%.*s)\n", VSNLEN, vol);
		fflush(stdout);
	
		memcpy(&ap->vad_vinfo.vi_vsn, vol, VSNLEN);
		if (ilabel)
			memcpy(&ap->vad_vinfo.vi_label, vol, VSNLEN);
		else
			memset(&ap->vad_vinfo.vi_label, ' ', VSNLEN);
		if (lSnd2RP(shp)) {
			exit(1);
		}
		if (GetResponse(vol, shp->vh_seq)) {
			if (logg)
				fprintf(log, "%.6s\n", vol);
		} else
			err++;
		voln = voln + 7;
	}
	vm_term();
	return(err);
}

int
GetResponse(vol, seq)
char	*vol;
Seq_t	seq;
{
	char		*id = "GetResponse";
	vmpHdr_t	*rhp;
	vmpVsn_t	*vsp;
	vmpError_t	*ep;

	while (1) {
		if ((rhp = lRcvFromRP(IPC_DELAY)) == NULL) {
			fprintf(stderr, "null packet received\n");
			vm_term();
			exit(1);
		}
		if (seq != rhp->vh_seq) {
			fprintf(stderr, "bad sequence packet received\n");
			vm_term();
			exit(1);
		}
		switch (rhp->vh_type) {
		    case M_VSN:
			vsp = MSGPTR(rhp, vmpVsn_t);
			if (strncmp((char *) &vsp->v_vsn, vol, VSNLEN)){
				printf("%.6s: wrong vsn returned\n",
								&vsp->v_vsn);
				vm_term();
				exit(1);
			}
			nFree(rhp);
			return(1);
		    case M_VERROR: 
			ep = MSGPTR(rhp, vmpError_t);
			printf("error from vold: %s\n",ep->vn_msg);
			if (ep->vn_error != E_USER) {
				vm_term();
				exit(1);
			}
			nFree(rhp);
			return(0);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			exit(1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			printf("Unexpected packet: %s\n", vHdrStr(rhp));
			vm_term();
			exit(1);
		}
	}
}
