/*
 *  volmoddb command -- admin interface to set attributes of volumes
 *	in volman(8) database. 
 *
 *
 *	Bill Ross, NASA Ames, 4/92
 */

#include "cl_space.h"

#include <stdio.h>
#include <ctype.h> 
#include <time.h> 

extern int errno;

extern	CodeString_t	vStatTab[];

static	vmpHdr_t	*rhp = NULL;
static	VDR_t		*vdr;
static	VInfo_t		*vin;

/*
 * Print out usage message and exit.
 */

char	*arg0;

 void
usage()
{
	vmpAttr_t	*ap;

	fprintf(stderr, "usage: %s {flags} volsers...\n", arg0);
	fprintf(stderr, "usage: %s {flags} < volsers..\n", arg0);
	fprintf(stderr, "\tflags: at least one:\n");
	fprintf(stderr, "\t\t-r RC -s status -u user -g group\n");
	fprintf(stderr, "\t\t-p perms -t type -P poolgid\n");
	fprintf(stderr, "\t\t-c \"comment, max = %d chars\"", 
						sizeof(ap->a_desc));
	fprintf(stderr, "\t\t-L  (set internal label=external)\n");
	vm_prRC(stderr, "", NULL, 0);
	fprintf(stderr, "Volume types:\n");
	vPrintVolTypes(stderr);
    
	exit(1);
}



getvsn(char *vsn)
{
	int		i, cc;

	while (1) {	/* vsns */
next:
		/*
		 *  skip whitespace
		 */
		for (;;) {	
			cc = getc(stdin);
			if (isspace(cc))
				continue;
			if (cc == EOF) {
				return(1);
			}
			break;
		}
		/*
		 *  read vsn
		 */
		for (i=0;;i++) {	
			if (isspace(cc)) {
				if (i < 5) {
					fprintf(stderr, "Vsn too short\n");
					vsn[i] = '\0';
					fprintf(stderr, "%s\n", vsn);
					goto next;
				}
				break;
			}
			if (!isalnum(cc)) {
				fprintf(stderr, "Illegal vsn character (%d)\n",
								cc);
				vsn[i] = '_';
				vsn[i+1] = '\0';
				fprintf(stderr, "skipping %s\n", vsn);
				goto next;
			}
			if (i > 5) {
				fprintf(stderr, "Vsn too long\n");
				vsn[i] = cc;
				vsn[i+1] = '\0';
				fprintf(stderr, "skipping %s\n", vsn);
				goto next;
			}
			if (islower(cc))
				vsn[i] = toupper(cc);
			else
				vsn[i] = cc;
			cc = getc(stdin);
		}
		return(0);
	}
}

/*
 *  option-getting routines
 */
 void
getvrc(char *name, int *rcnump)
{
	RC_t	*rcp;

	if ((rcp = cGetRcp(name)) == NULL) {
		fprintf(stderr,"RC %s not found\n", name);
		usage();
	}
	*rcnump = rcp->rcd_number;
}

 void
getvstatus(char *name, char *status)
{
	int	i;

	if (strlen(name) == 1) {
		for (i=0; vStatTab[i].cs_string; i++) {
			if (name[0] == vStatTab[i].cs_code) {
				*status = name[0];
				return;
			}
		}
	} else {
		for (i=0; vStatTab[i].cs_string; i++) {
			if (!strcmp(name, vStatTab[i].cs_string)) {
				*status = vStatTab[i].cs_code;
				return;
			}
		}
	}
	fprintf(stderr, "%s: not a status\n", name);
	vPrintStatTab(stderr);
	usage();
}
	
 void
getvsntype(char *str, int *nump)
{
	VolType_t	*vtp;

	vtp = vVolTypeByID(str);
	if (vtp == NULL) {
		fprintf(stderr, "%s: not a volume type\n", str);
		usage();
	}
	*nump = vtp->vt_map;
}

main(int argc, char *argv[])
{
	extern  int     optind;
	extern  char    *optarg;
	int		cc, i, fromstdin = 0, rval = 0;
	char		pbuf[PKTSIZ(vmpAttr_t)];
	vmpHdr_t	*shp;
	vmpAttr_t	*ap;

	arg0 = argv[0];

	/*
	 *  connect to vold & build send pkt from arguments
	 */

	if (vm_init(arg0)) {
		vm_prerr(stderr);
		exit(VMError);
	}
	if (vm_configRC()) {
		vm_prerr(stderr);
		exit(VMError);
	}

	shp = vBuildHdr(M_VATTR, vGetSeq(), (vmpHdr_t *) pbuf);
	ap = MSGPTR(shp, vmpAttr_t);

	/*
	 * process args
	 */

	ap->a_flags = 0;
	while ((cc = getopt(argc, argv, "c:r:s:t:u:g:p:L")) != EOF) {
		switch (cc) {
		    case 'c':
			ap->a_flags |= AT_COMMENT;
			i = strlen(optarg);
			if (i > sizeof(ap->a_desc)) {
				fprintf(stderr, "comment > %d\n", 
							sizeof(ap->a_desc));
				usage();
			}
			memset(ap->a_desc, ' ', sizeof(ap->a_desc));
			memcpy(ap->a_desc, optarg, i);
			break;
		    case 'r':
			ap->a_flags |= AT_RC;
			getvrc(optarg, &ap->a_rc);
			break;
		    case 's':
			ap->a_flags |= AT_STATUS;
			getvstatus(optarg, &ap->a_status);
			if (ap->a_flags & AT_LABEL  &&  
			    ap->a_status == S_VIRGIN) {
		    	 	fprintf(stderr, 
					" virgins cannot have labels\n");
				usage();
			}
			break;
		    case 'g':
			ap->a_flags |= AT_VGID;
			if (vm_groupid(optarg, &ap->a_gid) == -1) {
				perror(optarg);
				usage();
			}
			break;
		    case 'u':
			ap->a_flags |= AT_VUID;
			if (vm_userid(optarg, &ap->a_uid) == -1) {
				perror(optarg);
				usage();
			}
			break;
		    case 'p':
			ap->a_flags |= AT_PERM;
			if (vm_octalperm(optarg, &ap->a_perm) == -1) {
		    	 	fprintf(stderr, " must be octal %s\n",
						"read/write (0, 2, 4, or 6)");
		    	 	usage();
			}
			break;
		    case 't':
			ap->a_flags |= AT_VTYPE;
			getvsntype(optarg, &ap->a_type);
			break;
		    case 'P':
			ap->a_flags |= AT_POOLGID;
			if (vm_groupid(optarg, &ap->a_poolgid) == -1) {
				perror(optarg);
				usage();
			}
			break;
		    case 'L':
			ap->a_flags |= AT_LABEL;
			if (ap->a_flags & AT_STATUS  &&  
			    ap->a_status == S_VIRGIN) {
		    	 	fprintf(stderr, 
					" virgins cannot have labels\n");
				usage();
			}
			break;
		    case '?':
		    default:
			usage();
		}
	}
	if (!ap->a_flags) {
		fprintf(stderr, "nothing set\n");
		usage();
	}
	if (optind == argc) {
		fromstdin++;
	}

	/*
	 *  send the msg for each vsn
	 */

	for (;optind<argc || fromstdin;) {

		if (fromstdin) {
			char	stdinvsn[VSNLEN+1];

			if (getvsn(stdinvsn))
				break;
			memcpy(&ap->a_vsn, stdinvsn, VSNLEN);
		} else {
			if (checkvsn(argv[optind])) {
				printf("\n%s\n", nErrStr);
				rval++;
				optind++;
				continue;
			}
			memcpy(&ap->a_vsn, argv[optind], VSNLEN);
			optind++;
		}
		printf("sending %.6s\n", &ap->a_vsn);
		shp->vh_seq++;
		if (lSnd2RP(shp)) {
			vm_prerr(stderr);
			exit(VMError);
		}

		/*
		 *  receive the response
		 */

		rval += GetResponse(&ap->a_vsn, shp->vh_seq);

	}
	vm_term();
	exit(rval);
}

int
GetResponse(vol, seq)
char	*vol;
Seq_t	seq;
{
	char		*id = "GetResponse";
	vmpAttr_t	*ap;
	vmpError_t	*ep;

	/*
	 *  free any pkt from previous msg rcvd
	 */

	if (rhp != NULL)
		nFree(rhp);

	for (;;) {
		if ((rhp = lRcvFromRP(IPC_DELAY)) == NULL) {
			vm_prerr(stderr);
			exit(VMError);
		}
		if (seq != rhp->vh_seq  &&  rhp->vh_type != M_SHUTDOWN) {
			fprintf(stderr, "bad packet sequence (skipping): %s", 
								vHdrStr(rhp));
			exit(1);
		}
		switch (rhp->vh_type) {
		    case M_VATTR:
			ap = MSGPTR(rhp, vmpAttr_t);
			if (strncmp((char *)&ap->a_vsn, vol, VSNLEN)) {
				printf("%.*s: wrong vsn returned\n",
							VSNLEN, &ap->a_vsn);
				exit(1);
			}
			return(0);
		    case M_VERROR: 

			/*
			 *  E_USER = vsn does not exist
			 */

			ep = MSGPTR(rhp, vmpError_t);
			printf("error from vold: %s\n",ep->vn_msg);
			return(1);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			exit(1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			printf("Unexpected packet: %s\n", vHdrStr(rhp));
			exit(1);
		}
	}
}
