#ifndef lint
static char	rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/clnt/volmv.c,v 1.1.1.1 2000-02-28 02:18:24 wrstuden Exp $";
#endif

/*
 * volmv command -- user interface to move volumes.
 *	usage: volmv VSN-spec RC
 */
	
#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include "cl_space.h"

static char	*cmdname;
static vsnseq_t	*vsnarray;
static int	vsncount = 0;
static char	*msgbuf = NULL;

 void
usage()
{
	fprintf(stderr, 
	    "usage: %s [-e] [-i rid] {VSN|VSN_spec|-f file} RC\n", cmdname);
	fprintf(stderr, 
	    "       %s [-e] [-i rid] RC < vsn_list\n", cmdname);
	fprintf(stderr, 
	    "       \t-e\texclusive use: do not queue on another request\n");
	vm_prRC(stderr, "", NULL, 0);
	exit(E_USER);
}

done()
{
	int	rval = VMError;

	vm_term();
	exit(rval);
	return(rval);	/* for compiler warning */
}


main(argc, argv)
int     argc;
char    *argv[];
{
	char		*ridp = NULL;
	int		ridlen = 0;
	char		*fname = NULL;
	FILE		*fp = NULL;
	char		*rcname;
	RC_t		*rcp;
	vmpHdr_t	*hp;
	vmpMove_t	*mp;
	int		cc, excl = 0, moverr = 0, i;
	extern  int     optind;
	extern  char    *optarg;

	cmdname = argv[0];

	/*
	 *  see if stdin used for vsns
	 */
	if (!isatty(fileno(stdin))) {
		fname = "stdin";
		fp = stdin;
	}

	/*
	 *  prepare response to interrupt, start vold connection, get config
	 */
	if (SIGNAL(SIGINT, done))
		perror("signal");
	nLogInit(cmdname, done);
	nLogInitStderr(EL_LEVEL);
	if (_vm_init(argv[0], 0)) {
		vm_prerr(stderr);
		return (VMError);
	}
	if (vm_configRC()) {
		vm_prerr(stderr);
		done();
	}

	/*
	 *  parse args
	 */
	if (argc < 2)
		usage();
	while ((cc = getopt(argc, argv, "ei:f:")) != EOF) {
		switch (cc) {
		    case 'e':
			if (ridp) {
				fprintf(stderr, 
					"-i and -e cannot be combined\n");
				exit(1);
			}
			excl = 1;
			break;
		    case 'i':
			if (excl) {
				fprintf(stderr, 
					"-i and -e cannot be combined\n");
				exit(1);
			}
			ridp = optarg;
			ridlen = strlen(ridp);
			if (!ridlen) {
				fprintf(stderr, 
				    "reservation id required with -i option\n");
				usage();
			}
			if (ridlen > RIDLEN) {
				fprintf(stderr, 
				    "reservation id too long, max is %d\n",
							RIDLEN);
				usage();
			}
			break;
		    case 'f':
			if (fp) {
				fprintf(stderr, 
				    "only one source of input accepted\n");
				usage();
			}
			fname = optarg;
			fp = fopen(fname, "r");
			if (fp == NULL) {
				perror(fname);
				usage();
			}
			break;
		    default:
			usage();
		}
	}
	/*
	 *  need at least 1 more arg for RC
	 */
	if (argc - optind < 1)
		usage();
	rcname = argv[argc-1];

	/*
	 *  get list of vsns & rc name -> in msg buffer
	 *
	 *  	init count of space req'd in msg buffer for
	 * 	packet header
	 */
	vm_initlist(cmdname, PadSize(sizeof(vmpHdr_t)) 
			   + PadSize(sizeof(vmpMove_t)));

	/*
	 *  vsns can come from file/stdin or single-string cmdline spec
	 */
	if (fp) {
		/*
		 *  from file/stdin
		 *	only non-getopt arg would be RC
		 */
		if (argc - optind != 1)
			usage();
		msgbuf = vm_getvsnfilelist(fp, &vsncount);
	} else {
		/*
		 *  from vsn_spec on cmd line
		 *	only non-getopt args would be vsn_specs & RC
		 */
		if (argc - optind < 1)
			usage();
		msgbuf = vm_getvsnarglist(argv+optind, argc-optind-1,
							&vsncount, &moverr);
	}

	/*
	 *  check vsn spec  TODO- consider sorting list & rmvng dups
	 */
	if (vsncount == 0) {
		fprintf(stderr, "%s: no valid vsns\n", cmdname);
		exit(E_USER);
	}

	/*
	 *  check RC
	 */
	if ((rcp = cGetRcp(rcname)) == NULL) {
		fprintf(stderr, "%s: unknown RC\n", rcname);
		usage();
	}

	/*
	 *  if the process was started in the background then ignore 
	 *	the keyboard interrupt signal by disconnecting from 
	 *	the terminal
	 */
	if (vSetPgrp()) {
		perror("setpgrp failed");
		exit(E_SYS);
	}

	/*
	 *  fill in header of the custom-built pkt (which already
	 *	has the vsns filled in above)
	 */
	hp = (vmpHdr_t *) msgbuf;
	hp->vh_magic = VMP_MAGIC;
	hp->vh_size = PadSize(sizeof(vmpMove_t)) + vsncount * sizeof(vsnseq_t);
	hp->vh_type = M_MOVE;
	hp->vh_seq = 0;

	hp->vh_count = vsncount;

	/*
	 *  fill in move cmd part of pkt
	 */
	mp = MSGPTR(hp, vmpMove_t);
	mp->vmv_dstrc = rcp->rcd_number;
	memset(mp->vmv_rid, '\0', RIDLEN);
	if (ridp != NULL)
		memcpy(mp->vmv_rid, ridp, ridlen);
	mp->vmv_flags = VO_WAIT;
	if (excl)
		 mp->vmv_flags |= VO_EXCLUSIVE;

	/*
	 *  note where vsn part (filled in above) starts
	 */
	vsnarray = (vsnseq_t *) ((char *)mp + PadSize(sizeof(*mp)));

	/*
	 *  Built request, now send it 
	 */
	if (lSnd2RP(hp) != E_NOERROR) {
		vm_prerr(stderr);
		done();
	}

	/*
	 *  get responses
	 */
	for (i=0; i<vsncount; i++) 
		moverr += get_move_stat();

	VMError = (moverr ? -1 : 0);			/* set return code */
	done();
#ifdef lint
	return 0;
#endif
}


get_move_stat()
{
	int		i, errors = 0;
	vmpHdr_t	*hp;
	vmpError_t	*ep;

	while (1) {
		hp = lRcvFromRP(IPC_DELAY);
		if (hp == NULL)
			exit(1);
		if (hp->vh_seq == 0  &&  hp->vh_type == M_VERROR) {
			ep = MSGPTR(hp, vmpError_t);
			nLogMsg(EL_ERROR2, 0, "Request failed: %s", ep->vn_msg);
			exit(1);
		}
		if (hp->vh_type == M_SHUTDOWN) {
			nLogMsg(EL_ERROR1, 0, "Voldaemon shutdown");
			exit(1);
		}
		if (hp->vh_seq < 1 || hp->vh_seq > vsncount) {
			nLogMsg(EL_ERROR1, 0, 
				"unexpected sequence # %d in reply: %s", 
				hp->vh_seq, vHdrStr(hp));
			exit(1);
		}
		switch(hp->vh_type) {
		    case M_ACK:
			nLogMsg(EL_STATUS1, 0, "VSN %.*s moved",
					VSNLEN, &vsnarray[hp->vh_seq-1].r_vsn);
			return(0);
		    case M_VERROR:
			ep = MSGPTR(hp, vmpError_t);
			nLogMsg(EL_ERROR2, 0, "VSN %.*s not moved: %s",
					VSNLEN, &vsnarray[hp->vh_seq-1].r_vsn,
					ep->vn_msg);
			return(1);
		    case M_NOP:
			continue;
		    default:
			nLogMsg(EL_ERROR1, 0, 
					"Unexpected packet: %s", vHdrStr(hp));
		}
	}
}
