#ifndef lint
static char rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/clnt/volstat.c,v 1.1.1.1 2000-02-28 02:18:24 wrstuden Exp $";
#endif

/*
 *  volstat command -- user interface to list status of volumes
 *
 *	Bill Ross, NASA Ames 1992, 1995
 */
	
#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include "cl_space.h"


char	*cmdname;
extern QHead_t	CVolReqQ;	/* queue of all requests sent out by this AC */

 void
usage()
{
	fprintf(stderr, "usage: %s [-adnpuD] [-v VSN] [-rRC]\n", cmdname);
	fprintf(stderr, "\t-a\tall requests (default: UID)\n");
	fprintf(stderr, "\t-d\tprint device idle times if mounted\n");
	fprintf(stderr, "\t\t(prints requests in order of delay)\n");
	fprintf(stderr, "\t-p\tprint client PID instead of reservation id\n");
	fprintf(stderr, "\t-u\tprint user instead of user tag\n");
	fprintf(stderr, "\t-R\tprint destination RC (move) instead of host\n");
	fprintf(stderr, "\t-D\tprint drive instead of time in current state\n");
	exit(1);
}

done()
{
	vm_term();
	exit(0);
	return(0);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	int		cc, request, rcno, dest = 0;
	int		ret, sort = 0, tag = 1, ptime = 1, ppid = 0, count;
	int		devtimes = 0;
	extern  int     optind;
	extern  char    *optarg;
	vsn_t		*vsnp = NULL;
	Seq_t		seq, send_volstat();
	vmpVolStat_t	*vsp;

	cmdname = argv[0];

	if (SIGNAL(SIGINT, done))
		perror("signal");

	if (getuid() == 0)
		ret = vm_initp(cmdname);
	else
		ret = vm_init(cmdname);
	if (ret) {
		vm_prerr(stderr);
		done();
	}
	
	request = VQ_NONE;

	while ((cc = getopt(argc, argv, "adptTuDRr:v:")) != EOF) {
		switch (cc) {
		    case 'a':
			if (request != VQ_NONE)
				usage();
			request = VQ_ALL;
			break;
		    case 'p':
			ppid = 1;
			break;
		    case 'd':
			if (sort == 2) {
				fprintf(stderr, "-d and -n are incompatible\n");
				usage();
			}
			devtimes = 1;
			sort = 1;
			break;
		    case 'D':
			ptime = 0;
			break;
		    case 'R':
			dest = 1;
			break;
		    case 'r':
			if (request != VQ_NONE)
				usage();
			rcno = getRCnum(optarg);
			request = VQ_RC;
			break;
		    case 'v':
			if (request != VQ_NONE)
				usage();
			if (checkvsn(optarg)) {
				printf("\n%s\n", nErrStr);
				usage();
			}
			vsnp = (vsn_t *) optarg;
			request = VQ_VSN;
			break;
		    case 'T':
		    case 't':
			break;
		    default:
			fprintf(stderr, "%c: unknown flag\n", cc);
			usage();
		}
	} /* while */

	if (optind != argc)
		usage();

	/*
	 *  get host info if the host/dest field will be used for host
	 */
	if (!dest) {
		if (vm_hostinfo()) {
			vm_prerr(stderr);
			exit(1);
		}
	}

	/*
	 *  set default if necc: get active vsns under requestor's uid
	 */
	if (request == VQ_NONE)
		request = VQ_UID;

	/*
	 *  if the process was started in the background then ignore 
	 *  the keyboard interrupt signal by disconnecting from the terminal
	 */

	if (vSetPgrp()) {
		perror("setpgrp failed");
		exit(E_SYS);
	}

	if ((seq = send_volstat(request, rcno, vsnp, devtimes)) == 0){
		vm_prerr(stderr);
		done();
	}
	if (vsp = vm_getstat(NULL, seq, 1, 0, &count, 1)) {
		/*
		 *  print header, then vsns
		 */
		if (vm_prvolstatarr(NULL, 0, stdout, sort, ppid, 
							tag, ptime, dest)) {
			vm_prerr(stderr);
			done();
		}
		if (vm_prvolstatarr(vsp, count, stdout, sort, ppid, 
							tag, ptime, dest)) {
			vm_prerr(stderr);
			done();
		}
	} else if (VMError) {
		vm_prerr(stderr);
		done();
	} else
		puts("no requests found");


	done();
#ifdef lint
	return 0;
#endif
}


  Seq_t
send_volstat(int flag, int rc, vsn_t *vsnp, int timeflag)
{
	char		*id = "send_volstat";
	char		pbuf[PKTSIZ(vmpVsn_t)];
	vmpHdr_t	*hp;
	CVolReq_t	*cvrp;
	vmpVsn_t	*vsp;

	vClearError();			/* clear global errors */
	hp = vBuildHdr(M_VOLQRY, vGetSeq(), (vmpHdr_t *) pbuf);
	vsp = MSGPTR(hp, vmpVsn_t);
	vsp->v_flag = flag;
	vsp->v_flag2 = timeflag;
	vsp->v_rc = rc;
	if (vsnp)
		vsp->v_vsn = *vsnp;
	/* Built request, now send it and enqueue it...  */
	if (lSnd2RP(hp))
		return (0);
	/*
	 * Create a CVRQ entry
	 */
	cvrp = (CVolReq_t *) nAlloc(sizeof (CVolReq_t));
	cvrp->cvr_seq = hp->vh_seq;
	if (vsnp)
		cvrp->cvr_vsn = *vsnp;
	cvrp->cvr_lastpkt = NULL;			/* no status in yet */
	EnQCVR(cvrp, &CVolReqQ);
	
	return (cvrp->cvr_seq);
}


getRCnum(char *rcname)
{
	RC_t	*rcp;

	if (vm_configRC()) {
		vm_prerr(stderr);
		done();
	}
	if ((rcp = cGetRcp(rcname)) == NULL) {
		fprintf(stderr, "%s: unknown RC\n", rcname);
		vm_prRC(stderr, "", NULL, 0);
		exit(1);
	}
	return (rcp->rcd_number);
}
