#ifndef lint
static char	rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/clnt/volvary.c,v 1.1.1.1 2000-02-28 02:18:24 wrstuden Exp $";
#endif

/*
 *  volvary command -- user interface to vary on and off devices
 *	if no args, print vary status for all devs.
 *	Can only -f devices which are already off line (enforced in vold).
 */
	
#include <stdio.h>
#include <signal.h>
#include "cl_space.h"

#define HOSTLEN		7
#define RCLEN		6

/*
 *  since only vary uses the drive info, client drive stuff is all here;
 *	should be moved to cl_device.c if other progs need it
 */
extern DevEnt_t	*DriveArray;
extern int	DriveTotal;

static vmpHdr_t	*arraypkt = NULL;

static char	*cmdname;

 void
usage()
{
	fprintf(stderr, 
		"usage: %s [on | off [-e err] | reset] [DEVICE...|-rRC|all]\n",
	  cmdname);
	done(1);
}

done(int val)
{
	vm_term();
	exit(val);
	return(val);	/* for compiler warning */
}

bomb()
{
	exit(1);
	return(1);	/* for compiler warning */
}


vLookupDevNum(char *devnam)
{
	DevEnt_t	*dep;
	int		devno;

	if (strlen(devnam) > MAXDEVLEN-1) {
		fprintf(stderr, "%s: name too long\n", devnam);
		return(-1);
	}
	for (dep=DriveArray, devno=0; devno<DriveTotal; dep++, devno++) {
		char	*strrchr();

		if (*devnam == '/') {
			if (strncmp(devnam, "/dev/", 5)) 
				return(-1);
			devnam = strrchr(devnam, '/');
		}
		if (strcmp(devnam, dep->de_name) == MATCH)
			return(devno);
	}
	return(-1);
}

/*
 *  vm_prvary(): print vary status of the devices in the DriveArray 
 *	selected by the mask bitmap
 */
void
vm_prvary(FILE *stream, DevMap_t *maskp)
{
	int		devno;
	DevEnt_t	*dep;
	char		*errp;

	if (!DriveTotal) {
		fprintf(stream, "No drives defined\n");
		return;
	}
	fprintf(stream, "DRIVE   HOST...volnd  RC...status  VARIED   ERROR\n");

	dep = DriveArray;
	errp = (char *)(DriveArray + DriveTotal);
	for (devno = 0; devno < DriveTotal; devno++, dep++) {
		HostInfo_t	*hip;
		char		*status, *host, *hoststatus, *rcnam, *rcstatus;
		RC_t		*rcp;

		/*
		 *  keep error pointers up-to-date
		 */
		if (dep->de_errstr) {
			dep->de_errstr = (unsigned long)errp;
			errp += ERRSTRSIZE;
		}

		/*
		 *  skip drive if user didn't specify
		 */
		if (maskp && !DM_ISSET(devno, *maskp))
			continue;


		/*
		**	host/volnd info
		*/
		hip = HostInfo(dep->de_host);
		if (hip == NULL) {
			fprintf(stderr, "unknown host %d\n", dep->de_host);
			exit(1);
		}
		host = hip->h_name;
		if (hip->h_flag & H_HASVOLND)
			hoststatus = "(ok)";
		else
			hoststatus = "DOWN";

		/*
		**	rc info
		*/
		rcp = cRCInfo(dep->de_rc);
		if (rcp == NULL) {
			fprintf(stderr, "unknown rc %d\n", dep->de_rc);
			exit(1);
		}
		rcnam = rcp->rcd_id;
		if (rcp->rcd_flags & RCF_READY)
			rcstatus = "(ok)";
		else
			rcstatus = "DOWN";

		/*
		**	vary status
		*/
		if (dep->de_status & DS_VARY)
			status = "ON ";
		else
			status = "OFF"; 

		fprintf(stream, "%-*.*s %-*.*s %-*.*s %-*.*s %-*.*s   %s",
			MAXDEVLEN-1, MAXDEVLEN-1, dep->de_name,
			HOSTLEN, HOSTLEN, host,
			MAXCHANLEN-3, MAXCHANLEN-3, hoststatus,
			RCLEN, RCLEN, rcnam,
			MAXCHANLEN-4, MAXCHANLEN-4, rcstatus,
			status);
		if (dep->de_errstr)
			fprintf(stream, " %.*s\n", ERRSTRSIZE, 
							(char *)dep->de_errstr);
		else
			fprintf(stream, "\n");
	}
}

main(int argc, char *argv[])
{
	int		flag, i, ret = 0, drives = 0;
	DevMap_t	maskmap;
	char		*errstr = NULL;

	cmdname = argv[0];
	nLogInit(cmdname, bomb);
	nLogInitStderr(EL_LEVEL);

	/*
	 * if the process was started in the background then ignore 
	 * the keyboard interrupt signal by disconnecting from the terminal
	 */
	if (vSetPgrp()) {
		perror("setpgrp failed");
		exit(E_SYS);
	}
	if (SIGNAL(SIGINT, done))
		perror("signal");


	/*
	 *  start priority connection to vold
	 */
	if (getuid() == 0)
		ret = vm_initp(cmdname);
	else
		ret = vm_init(cmdname);
	if (ret) {
		vm_prerr(stderr);
		done(VMError);
	}
	/*
	 *  get RC, host & drive config
	 */
	if (vm_configRC()) {
		vm_prerr(stderr);
		done(VMError);
	}
	if (vm_hostinfo()) {
		vm_prerr(stderr);
		done(VMError);
	}
	if (vm_vary(NULL, VVA_DISPREQ, NULL))  {
		vm_prerr(stderr);
		done(VMError);
	}

	/*
	 *  the drive mask will be used if the flag doesn't
	 *	have VVA_ALL set
	 */
	DM_ZERO(maskmap);
	flag = VVA_DISPREQ;

	if (argc == 1) {
		flag |= VVA_ALL;
	} else for (i=1; i<argc; i++) {
		int		 devno;

		if (i == 1 && !strcmp(argv[i], "off")) {
			flag &= ~VVA_DISPREQ;
			flag |= VVA_VARYOFF;

			/*
			 *  get "-e errmsg" option 
			 */
			if (i+1<argc && !strncmp(argv[i+1], "-e", 2)) {
				i++;
				if (strlen(argv[i]) > 2)
					errstr = argv[i] + 2;
				else {
					if ( !(++i<argc) ) 
						usage();
					errstr = argv[i];
				}
			}
			continue;
		}
		if (i == 1 && strcmp(argv[i], "on") == MATCH) {
			flag &= ~VVA_DISPREQ;
			flag |= VVA_VARYON;
			continue;
		}
		if (strcmp(argv[i], "all") == MATCH) {
			flag |= VVA_ALL;
			break;
		}

		/* 
		 *  device or RC name - fill in bitmap specification
		 */

		if (argv[i][0] == '-' && argv[i][1] == 'r') {	
			char		*rcnam = &argv[i][2];
			RC_t		*rcp;
			int		rcnum;

			/* 
			 *  RC name or #
			 */

			if ((rcp = cGetRcp(rcnam)) == NULL) {
				fprintf(stderr, "%s: not an RC\n", rcnam);
				ret++;
				continue;
			}
			rcnum = rcp->rcd_number;
			for (devno = 0; devno < MAXDEVS; devno++) {
				register DevEnt_t	*dep;
					
				if (dep = vGetDevByNum(devno))
					if (dep->de_rc == rcnum) {
						DM_SET(devno, maskmap);
						drives++;
					}
			}
			continue;
		}

		/*
		 *  drive name
		 */
		if ((devno = vLookupDevNum(argv[i])) != -1) {
			DM_SET(devno, maskmap);
			drives++;
		} else {
			fprintf(stderr, "%s: %s: no such device\n", cmdname,
			  argv[i]);
			ret++;
		}
	}

	/*
	 * make sure at least one device was selected or that there were
	 *	 no args (for a display)
	 */
	if (drives == 0  &&  !(flag & VVA_ALL))
		usage();

	/*
	 *  if just after status & it has already been gotten as
	 *	a side effect of looking up drive names, display
	 *	& quit
	 */
	if (flag & VVA_DISPREQ  &&  DriveArray != NULL) {
		vm_prvary(stdout, flag & VVA_ALL ? NULL : &maskmap);
		done(0);
	}

	/*
	 *  send the vary
	 */
	if (vm_vary(&maskmap, flag, errstr)) {
		ret++;
		if (VMError == E_PERM) {
			fprintf(stderr, 
				"Permission denied: must be root or volman\n");
			done(1);
		}
		if (!(flag & VVA_RESET))
			vm_vary(NULL, VVA_DISPREQ, NULL); /* get status */
	}
	if (!(flag & VVA_RESET))	/* don't prvary when resetting */
		vm_prvary(stdout, flag & VVA_ALL ? NULL : &maskmap);
	done(ret);
#ifdef lint
	return 0;
#endif
}

