/*
 *  vquota command -- user interface to get info on quotas in the
 *	volman(8) database. Operator interface to set quotas.
 *
 *
 *	Bill Ross, NASA Ames, 1989, 1991
 */

/* #define DEBUG */


#include <stdio.h>
#include <ctype.h> 
#include <pwd.h>

#include "cl_space.h"

extern int errno;

/*
 * Print out usage message and exit.
 */
 void
usage(char *arg0)
{
	fprintf(stderr,
	  "usage: %s RC [-u user] [-s quota]\n", arg0);
	vm_prRC(stderr, "", NULL, 0);
	exit(1);
}


main(int argc, char *argv[])
{
	char		*arg0 = argv[0];
	int		ownuid = -1;
	int		volmanuid, vuid, cc, newquota = -1;
	extern  int     optind;
	extern  char    *optarg;
	RC_t		*rcp;
	char		pbuf[PKTSIZ(vmpQuota_t)];
	vmpHdr_t	*hp;
	vmpQuota_t	*qp;
	vmpError_t	*ep;
	Seq_t		seq;

	/*
	 *  connect to voldaemon & get rc config & rc
	 */
	if (vm_init(arg0)) {
		vm_prerr(stderr);
		exit(VMError);
	}
	if (vm_configRC()) {
		vm_prerr(stderr);
		exit(VMError);
	}
	if (argc < 2  ||  argc > 6)
		usage(arg0);

	/* 
	 *  get default uid 
	 */

	vuid = getuid();
	ownuid = vuid;

	rcp = cGetRcp(argv[1]);	
	if (rcp == NULL) {
		fprintf(stderr, "%s: bad RC\n", argv[1]);
		usage(arg0);
	}
	argv++; argc--;

	/*
	 *  process args
	 */

	while ((cc = getopt(argc, argv, "u:s:")) != EOF) {
		switch (cc) {
			case 'u':
				if (vm_userid(optarg, &vuid)) {
					perror(optarg);
					usage(arg0);
				}
				break;
			case 's':
				if (vm_userid(VolManOperAcct, &volmanuid)) {
					perror("looking up volman uid");
					exit(E_SYS);
				}

				if (ownuid != 0  &&  ownuid != volmanuid) {
				    fprintf(stderr,
				      "Must be root or %s to set quota\n", 
				      VolManOperAcct);
				    exit(1);
				}
				if (!isnum(optarg))  {
				    fprintf(stderr, "expected number\n");
				    usage(arg0);
				}
				newquota = atoi(optarg);
				break;
			case '?':
			default:
				fprintf(stderr, "unknown option: %c\n", cc);
				usage(arg0);
				break;
		}
	}

	/*
	 *  send msg
	 */
	seq = vGetSeq();
	hp = vBuildHdr(M_QUOTQRY, seq, (vmpHdr_t *) pbuf);
	qp = MSGPTR(hp, vmpQuota_t);
	qp->vqq_uid = vuid;
	qp->vqq_rc = rcp->rcd_number;
	if (newquota > -1) {
		hp->vh_type = M_SETQUOTA;
		qp->vqq_quota = newquota;
	}

	if (lSnd2RP(hp)) {
		exit(1);
	}

	/*
	 *  get response
	 */

	for (;;) {
		if ((hp = lRcvFromRP(IPC_DELAY)) == NULL) {
			exit(1);
		}
		if (seq != hp->vh_seq) {
			printf("Unexpected seq #: %s\n", vHdrStr(hp));
			continue;
		}
		switch (hp->vh_type) {
		    case M_QUOTRPLY:
			qp = MSGPTR(hp, vmpQuota_t);
			goto ok;
		    case M_VERROR:
			ep = MSGPTR(hp, vmpError_t);
			printf("error from voldaemon: %s\n",ep->vn_msg);
			vm_term();
			exit(1);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			exit(1);
		    case M_NOP:
			nFree(hp);
			continue;
		    default:
			printf("Unexpected packet: %s\n", vHdrStr(hp));
			vm_term();
			exit(1);
		}
	}
ok:
	vm_term();

	/*
	 *  print result
	 */

	if (newquota > -1)
		printf("quota set: %d (%d used)\n", 
						qp->vqq_quota, qp->vqq_used);
	else
		printf("quota %d  used %d\n", qp->vqq_quota, qp->vqq_used);

	vm_term();
	exit(0);
}
