#ifndef lint
static char	rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/clnt/vreserve.c,v 1.1.1.1 2000-02-28 02:18:25 wrstuden Exp $";
#endif

/*
 * vreserve command -- user interface to manipulate reservations.
 *
 */
	
#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include "cl_space.h"

extern	int	VMError;

static	char	id[20];
static	char	*arg0;

/*
 *  usage() - print out usage message and exit.
 */
 void
usage()
{
	fprintf(stderr, "usage:\n  %s [-a]\n", arg0);
	fprintf(stderr,
	  "  %s -{c|u|d} name [-r RC [-m medium] [-s num][-h num]] [vsn(s)]\n",
		arg0);
	fprintf(stderr,
		"\t(no arg)\tlist reservations for own uid\n");
	fprintf(stderr,
		"\t-a\tlist all reservations\n");
	fprintf(stderr,
		"\t-c\tcreate reservation id\n");
	fprintf(stderr,
		"\t-u\tuse existing reservation id (adding to it)\n");
	fprintf(stderr,
		"\t-d\tdelete reservation id or delete items from it\n");
	fprintf(stderr,
		"\t-r\tRC (id or number; used with -h or -s):\n");
	vm_prRC(stderr, "\t\t", NULL, 1);
	fprintf(stderr,
		"\t-m\tmedium (required if >1 in RC)\n");
	fprintf(stderr,
		"\t-s\tsoft drives\n\t-h\thard drives\n");
	
	exit(E_USER);
}

struct list {
	QLink_t	lst_q;
	char	*lst_p;
};

done()
{
	int	rval = VMError;
	vm_term();
	exit(rval);
	return(rval);	/* for compiler warning */
}


#define USER	1
#define ALL	2

main(int argc, char *argv[])
{
	int		i, cc, sig, num, psize, recvd;
	char		*rid_name=NULL;
	extern  int     optind;
	extern  char    *optarg;
	extern	char	*vSpoolDir;
		char	*log; 
	int     	lflg =0, tflg =0, cflg =0, dflg =0, uflg =0;
	vmpHdr_t	*hp;
	Seq_t		seq;
	RC_t		*rcp = NULL;
	int		rc_nmedia, rc_medium=0, rc_num=0, n_rcs=0, n_vsns=0;
	char		*pbuf =NULL;
        vmpError_t      *ep;
	vmpReserveRCMed_t	*rrcp =NULL;
	QHead_t		vsn_lst;
	QHead_t		rc_lst;
	struct	list	*listp;
	vmpReserveVsn_t	*rvsnp;
	vmpReserveHdr_t	*rp;

	arg0 = argv[0];

	if (_vm_init(id, 0)) {
		vm_prerr(stderr);
		exit(VMError);
	}
	if (vm_configRC()) {
		vm_prerr(stderr);
		exit(VMError);
	}

	sprintf(id, "vreserve(%d)", getpid());
	vsn_lst.q_head = NULL;
	vsn_lst.q_tail = NULL;
	rc_lst.q_head = NULL;
	rc_lst.q_tail = NULL;
	
	while ((cc = getopt(argc, argv, "ar:m:c:u:d:s:h:t")) != EOF) {
		switch (cc) {

		    case 'a':
			if ( tflg || cflg || dflg || uflg || rcp ) {
				usage();
			}
			lflg = ALL;
			break;
		    case 'c':	/* create reservation id */
		    case 'u':	/* use reservation id */
		    case 'd':	/* delete reservation id */
			if (lflg)
				usage();
			if (rid_name)
				usage();
			if (strlen(optarg) > RIDLEN) {
				fprintf(stderr, "id too long, max is %d\n",
						RIDLEN);
				usage();
			}
			rid_name = nAlloc(strlen(optarg)+1);
			strcpy(rid_name, optarg);
			if (cc == 'c') {
				printf("create rid %s\n", rid_name);
				cflg = 1;
			} else if (cc == 'd') {
				printf("delete [from] rid %s\n", rid_name);
				dflg = 1;
			} else {
				printf("reference rid %s\n", rid_name);
				uflg = 1;
			}
			break;

		    case 'r':	/* specify RC */
			if (lflg)
				usage();
			if ((rcp = cGetRcp(optarg)) == NULL) {
				fprintf(stderr, "%s: bad RC\n", optarg);
				usage();
			}
			printf("RC: %s\n", rcp->rcd_id);
			rc_nmedia = countintbits(rcp->rcd_media);
			break;

		    case 'm':
			if (rcp == NULL) {
				fprintf(stderr,"medium: must specify RC 1st\n");
				usage();
			}
			rc_medium = vMediumCode(optarg);
			if (rc_medium == -1) {
				fprintf(stderr,"unknown medium: %s\n",
								optarg);
				usage();
			}
			if (!(rc_medium & rcp->rcd_media)) {
				fprintf(stderr, "medium %s not in %s\n",
							optarg, rcp->rcd_id);
				usage();
			}
			break;
		    case 's':	/* soft reservation */
		    case 'h':	/* hard reservation */
			if (lflg)
				usage();
			num = atoi(optarg);
			if (num <= 0) {
				fprintf(stderr,
					"-%c: expected # of drives: %s\n",
							cc, optarg);
				usage();
			}
			if (rcp == NULL) {
				fprintf(stderr, "no RC specified for drive%c\n",
						((num > 1) ? 's' : ' '));
				usage();
			}

			n_rcs++;
			listp=(struct list *)nAlloc(sizeof(struct list));
			listp->lst_q.q_next = NULL;
			listp->lst_q.q_prev = NULL;
			ENQUEUE(listp, &rc_lst, lst_q, struct list)
			rrcp=(vmpReserveRCMed_t *)
					nAlloc(sizeof(vmpReserveRCMed_t));
			rrcp->re_rc = rcp->rcd_number;
			rrcp->re_drives = dflg ? -num : num;
			rrcp->re_type = cc;
			if (rc_nmedia > 1  &&  !rc_medium) {
				fprintf(stderr, 
					"%s is multimedia: need medium spec\n",
					rcp->rcd_id);
				usage();
			}
			if (rc_medium)
				rrcp->re_medium = rc_medium;
			else
				rrcp->re_medium = rcp->rcd_media;
			listp->lst_p = (char *)rrcp;
			break;

		    case 't':	/* trace on */
			tflg = 1;
			break;

		    case '?':
		    default:
			usage();
		}
	} /* while */

	if (rcp != NULL  &&  n_rcs == 0) {
		fprintf(stderr, "RC specified but no drives\n");
		usage();
	}
	if ( tflg + cflg + dflg + uflg + lflg == 0 ) {
		lflg = USER;
	} else if (lflg != ALL) {
		if (rid_name == NULL) {
			fprintf(stderr, "No reservation name given\n");
			usage();
		}
	}

	/*
	 * last args are the volume serial numbers (volser(s))
	 */
	for (; optind < argc; optind++) {

		if (lflg)
			usage();

		if (checkvsn(argv[optind])) {
			printf("\n%s\n", nErrStr);
			exit(E_USER);
		}
		n_vsns++;
		listp=(struct list *)nAlloc(sizeof(struct list));
		listp->lst_q.q_next = NULL;
		listp->lst_q.q_prev = NULL;
		ENQUEUE(listp, &vsn_lst, lst_q, struct list);
		rvsnp = (vmpReserveVsn_t *)nAlloc(sizeof(vmpReserveVsn_t));
		strncpy((char *)&rvsnp->re_vsn, argv[optind], VSNLEN);
		rvsnp->re_type = dflg ? RE_DELVSN : RE_ADD;
		listp->lst_p = (char *)rvsnp;
	}

	/*
	 *  if not doing a list operation check what action is specified
	 */
	if (!lflg) {
		if (uflg &&	/* use rid */
		    rc_lst.q_head == NULL && vsn_lst.q_head == NULL) {
			fprintf(stderr,
				"No action requested on reservation\n");
			usage();
		}

		if (dflg) {	/* delete rid */
			if (rc_lst.q_head == NULL && vsn_lst.q_head == NULL) {
				printf("deleting reservation\n");
			} else {
				printf("deleting from reservation\n");
			}
		}
	}

	for (sig = 1; sig < NSIG; sig++) {
#ifdef DEBUG
		if (sig == SIGQUIT || sig == SIGSEGV || sig == SIGBUS)
			continue;
#endif
		if (SIGNAL(sig, done))
			perror("signal");
	}
	
	/*
	 * if the process was started in the background then ignore 
	 * the keyboard interrupt signal by disconnecting from the terminal
	 */
	
	if (vSetPgrp()) {
		perror("setpgrp failed");
		exit(E_SYS);
	}

	nLogInit(id, done);
	nLogInitStderr(EL_LEVEL);

	/*
	 *  Turn full tracing on if trace flag was specified
	 *  Note: trace information is recorded in the vold log, thus
 	 *  root privilege must be established.
	 */
	if (tflg) {
		vInitDirNames();
		nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
		nLogSetLevel(EL_DEBUG4);
	}

	/*
	 *  since the reserve packet is variable size, it must be
	 *	constructed explicitly
	 */
	seq = vGetSeq();
	psize = PadSize(sizeof(vmpHdr_t)) +
		PadSize(sizeof(vmpReserveHdr_t)) +
		PadSize(n_rcs * sizeof(vmpReserveRCMed_t)) +
		n_vsns * sizeof(vmpReserveVsn_t);

	hp = (vmpHdr_t *) nAlloc(psize);
        hp->vh_magic = VMP_MAGIC;
        hp->vh_type = M_RESERVE;
        hp->vh_seq = seq;
        hp->vh_size = PadSize(sizeof(vmpReserveHdr_t)) +
			PadSize(n_rcs * sizeof(vmpReserveRCMed_t)) +
			n_vsns * sizeof(vmpReserveVsn_t);

	rp = MSGPTR(hp, vmpReserveHdr_t);
	memset(rp->re_id, '\0', sizeof(rp->re_id));
	if (rid_name)
		memcpy(rp->re_id, rid_name, strlen(rid_name));
	if (dflg) {
		if (n_rcs==0 && n_vsns==0)
			rp->re_flag = RE_DELETE;	/* delete reservation */
		else
			rp->re_flag = RE_USE;		/* delete from rsrvtn */
	} else if (cflg)
		rp->re_flag = RE_CREATE|RE_ASYNC; /* create reservation */
	else if (uflg)
		rp->re_flag = RE_USE|RE_ASYNC;	/* use reservation */
	else if (lflg == USER)
		rp->re_flag = RE_LUSER;		/* list user's reservations */
	else if (lflg == ALL)
		rp->re_flag = RE_LALL;		/* list all reservations */
	else if (!dflg)
		fprintf(stderr, "programming error: abandon the building\n");

	rp->re_rcmedcount = n_rcs;
	rp->re_vsncount = n_vsns;

	rrcp = (vmpReserveRCMed_t *) ((char *)rp + PadSize(sizeof(*rp)));

	for (listp=(struct list *)rc_lst.q_head; listp; 
				listp=(struct list *)rc_lst.q_head) {
		memcpy(rrcp, listp->lst_p, sizeof(vmpReserveRCMed_t));
		rrcp++;

		DEQUEUE(listp, &rc_lst, lst_q, struct list);
		nFree(listp->lst_p);
		nFree(listp);
	}
	rrcp--;
	rvsnp = (vmpReserveVsn_t *) ((char *)rrcp + PadSize(sizeof(*rrcp)));

	for (listp=(struct list *)vsn_lst.q_head; listp; 
				listp=(struct list *)vsn_lst.q_head) {
		memcpy(rvsnp, listp->lst_p, sizeof(vmpReserveVsn_t));
		rvsnp++;

		DEQUEUE(listp, &vsn_lst, lst_q, struct list);
		nFree(listp->lst_p);
		nFree(listp);
	}
	if (lSnd2RP(hp)) {
		fprintf(stderr, "send failed\n");
		exit(1);
	}

	/*
	 *  get response
	 */

	while (1) {
		if ((hp = lRcvFromRP(IPC_DELAY)) == NULL)
			exit(1);
		if (seq != hp->vh_seq) {
			printf("Unexpected seq #: %s\n", vHdrStr(hp));
			continue;
		}
		switch (hp->vh_type) {
		    case M_ACK:
			if (lflg  &&  recvd == 0)
				printf("no reservations\n");
			goto ok;

		    case M_RESERVE:
			recvd++;
			rp = MSGPTR(hp, vmpReserveHdr_t);
			if (rp->re_id[0])
				printf("ID=``%.*s''\n", RIDLEN, rp->re_id);
			else
			 	printf("ID: (none; implicit)\n");
			rrcp = (vmpReserveRCMed_t *) 
					((char *)rp + PadSize(sizeof(*rp)));
			printf("\t%s  uid=%d\n", 
				 	(rp->re_flag ? "ACTIVE" : "INACTIVE"),
					rp->re_uid);
			for (i=0; i<rp->re_rcmedcount; i++, rrcp++) {
				rcp = cRCInfo(rrcp->re_rc);
				if (rcp) {
				    printf(
					"\t%s/%s: %s rsrvd %d used %d\n",
					rcp->rcd_id, 
					vMediumName(rrcp->re_medium),
					(rrcp->re_type == RE_HARD ?
							"hard" : "soft"),
					rrcp->re_drives,
					rrcp->re_used);
				} else
					printf("unknown RC %d\n", rrcp->re_rc);
			}
			if (rp->re_rcmedcount == 0)
				printf("\t(no drives reserved)\n");
			rrcp--;
			rvsnp = (vmpReserveVsn_t *) 
					((char *)rrcp + PadSize(sizeof(*rrcp)));
			for (i=0; i<rp->re_vsncount; i++, rvsnp++)
				printf("\t    %.*s\n", VSNLEN, &rvsnp->re_vsn);
			if (rp->re_vsncount == 0)
				printf("\t(no vsns reserved)\n");
			break;

		    case M_VERROR:
			ep = MSGPTR(hp, vmpError_t);
			printf("error from vold: %s\n", ep->vn_msg);
			vm_term();
			exit(1);

		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			exit(1);

		    case M_NOP:
			nFree(hp);
			continue;

		    default:
			printf("Unexpected packet: %s\n", vHdrStr(hp));
			vm_term();
			exit(1);
		}
	}
ok:

	done();
#ifdef lint
	return 0;
#endif
}
