
/*
 *  vscratch command -- admin utility to scratch virgin & recycle tapes
 *	(assumes 3480/3490 tapes)
 *
 *	Bill Ross, Computer Sciences Corporation for Nasa Ames, 1994.
 */

#include "cl_space.h"

#include <stdio.h>
#include <signal.h>
#include <ctype.h>

extern int	VMError;

char	*arg0;

/*
 *  usage() - Print out usage message and exit.
 */
 void
usage()
{
	fprintf(stderr, "usage: %s [-b] vsnlist..\n", arg0);
	fprintf(stderr, "usage: %s [-b] -f file\n", arg0);
	fprintf(stderr, "usage: %s [-b] < file\n", arg0);
	fprintf(stderr, "\t-b\tbypass label check on mount\n");
	exit(E_USER);
}

done(int rval)
{
	vm_term();
	exit(rval);
	return(rval);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	int		cc, sig, bypass = 0, total, i, fd, err = 0;
	vsnseq_t	*vsnarr = NULL;
	extern  int     optind;
	extern  char    *optarg;
	FILE		*fp = NULL;

	arg0 = argv[0];

	nLogInitStderr(EL_ERROR1);

	/*
	 *  get vsns from stdin, file, or cmd line
	 */
	vm_initlist(arg0, 0);
	while ((cc = getopt(argc, argv, "bf:")) != EOF) {
		switch (cc) {
		    case 'b':
			bypass = VO_BYPASS;
			break;
		    case 'f':
			if ((fp = fopen(optarg, "r")) == NULL) {
				perror(optarg);
				usage();
			}
			vsnarr = (vsnseq_t *) vm_getvsnfilelist(fp, &total);
			break;
		    default:
			usage();
		}
	}
	if (optind == argc) {
		if (vsnarr == NULL) {
			if (isatty(fileno(stdin)))
				fprintf(stderr, "enter vsns followed by ^D:\n");
			vsnarr = (vsnseq_t *) vm_getvsnfilelist(stdin, &total);
		}
	} else {
		if (vsnarr != NULL) {
			fprintf(stderr, "-f file plus extra args\n");
			usage();
		}
		vsnarr = (vsnseq_t *) vm_getvsnarglist(argv+optind, argc-optind,
							&total, &err);
	}
	if ( total == 0) {
		fprintf(stderr, "no vsns defined\n");
		usage();
	}

	for (sig = 1; sig < NSIG; sig++) {
		if (sig == SIGBUS)
			continue;
#ifdef DEBUG
		if (sig == SIGQUIT || sig == SIGSEGV)
			continue;
#endif
		signal(sig, (void (*) (int)) done);
	}

	/*
	 *  connect to vold
	 */
	if (_vm_init(arg0, 0)) {
		vm_prerr(stderr);
		exit(VMError);
	}

	/*
	 *  if the process was started in the background then ignore 
	 *  the keyboard interrupt signal by disconnecting from the terminal
	 */
	if (vSetPgrp())
		exit(E_SYS);

	/*
	 *  do vsns
	 */

	for (i=0; i<total; i++, vsnarr++) {
		char		*vsnp = (char *) &vsnarr->r_vsn;
		char		pbuf[PKTSIZ(vmpNode_t)];
		int		failflag;
		vmpHdr_t	*hp;
		vmpNode_t	*np;

		/*
		 *  build pkt
		 */
		hp = vBuildHdr(M_SCRATCH, vGetSeq(), (vmpHdr_t *) pbuf);
		np = MSGPTR(hp, vmpNode_t);
		memcpy(&np->vs_vsn, vsnp, VSNLEN);
		memcpy(&np->vs_label, vsnp, VSNLEN);
		np->vs_flag = bypass;

		/*
		 *  send pkt, get response
		 */
		if (lSnd2RP(hp)) {
			err++;
			vm_prerr(stderr);
			continue;
		}
		if (GetResponse(vsnp, hp->vh_seq)) 
			err++;
	}
	done(err);
#ifdef lint
	return 0;
#endif
}

/*
 *  GetResponse() - same as vlabel
 */
  int
GetResponse(char *vol, Seq_t seq)
{
	char		*id = "GetResponse";
	vmpHdr_t	*rhp;
	vmpError_t	*ep;

	while (1) {
		if ((rhp = lRcvFromRP(IPC_DELAY)) == NULL) {
			done(1);
		}
		switch (rhp->vh_type) {
		    case M_ACK:
			if (seq != rhp->vh_seq) {
				fprintf(stderr, "wrong seq\n");
				return(1);
			}
			return(0);
		    case M_VERROR: 
			if (seq != rhp->vh_seq)
				fprintf(stderr, "(wrong seq)\n");
			ep = MSGPTR(rhp, vmpError_t);
			fprintf(stderr, "error from vold: %s\n",ep->vn_msg);
			return(1);
		    case M_SHUTDOWN:
			fprintf(stderr, "shutdown received\n");
			vm_term();
			return(1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			if (seq != rhp->vh_seq)
				fprintf(stderr, "(wrong seq)\n");
			fprintf(stderr, "Unexpected packet: %s\n", 
							vHdrStr(rhp));
			return(1);
		}
	}
}
