/*
 *  cl_device.c - get drive config, do vary
 *
 *
 *  Bill Ross, NASA Ames, June 1995
 */

#include "cl_space.h"

#include <setjmp.h>
jmp_buf	jmpbuf;

DevEnt_t	*DriveArray = NULL;
int		DriveTotal = 0;

static vmpHdr_t	*drivepkt = NULL;

/*
 *  vm_vary()  - get status, if dev(s) specified, vary on or off;
 *	wait for status back. 
 */
 int
vm_vary(DevMap_t *dmp, int flags, char *errstr)
{
	char		*id = "vm_vary";
	vmpHdr_t	*hp;
	vmpVary_t	*vp;
	char		vbuf[PKTSIZ(vmpVary_t)];
	Seq_t		seq = vGetSeq();
	
	vClearError();			/* clear global errors */
	if (setjmp(jmpbuf)) {		/* Shutdown escape hatch */
		nLogMsg(EL_DEBUG1, id, "setjmp escape");
		return (1);
	}

	/*
	 *  build and send vary msg
	 */
	hp = vBuildHdr(M_VARY, seq, (vmpHdr_t *) vbuf);
	vp = MSGPTR(hp, vmpVary_t);

	if (dmp)
		vp->vva_dev = *dmp;
	else
		DM_ZERO(vp->vva_dev);
	vp->vva_flags = flags;
	if (errstr) {
		size_t	len = strlen(errstr);

		if (len > ERRSTRSIZE)
			len = ERRSTRSIZE;
		memcpy(vp->vva_err, errstr, len);
		if (len < ERRSTRSIZE)
			vp->vva_err[len] = '\0';
	} else {
		vp->vva_err[0] = '\0';
	}
	if (lSnd2RP(hp))
		return (1);

	/*
	 *  process replies
	 */
	for (;;) {
		if ((hp = lRcvFromRP(IPC_DELAY)) == NULL) {
			vm_prerr(stderr);
			exit(VMError);
		}
		if (seq != hp->vh_seq  &&  hp->vh_type != M_SHUTDOWN) {
			fprintf(stderr, 
				"%s: bad packet sequence (skipping): %s\n", 
				id, vMsgType(hp->vh_type));
			continue;
		}
		switch (hp->vh_type) {
		    case M_DRIVEINFO:
			if (drivepkt != NULL)
				nFree(drivepkt);
			drivepkt = hp;
			DriveTotal = hp->vh_count;
			DriveArray = MSGPTR(hp, DevEnt_t); 
			return(0);
		    case M_VERROR: 
			vInstallError(MSGPTR(hp, vmpError_t));
			nFree(hp);
			return(1);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			exit(1);
			break;		/* for lint */
		    case M_NOP:
			nFree(hp);
			break;
		    default:
			printf("Unexpected packet: %s\n",vMsgType(hp->vh_type));
			exit(1);
		}
	}
}

