/*
 *  cl_host.c - get info on allowed hosts
 *
 *	Bill Ross, NASA Ames, October 1995
 */

#include "cl_space.h"

HostInfo_t	*Hosts	= NULL;
int		HostTotal = 0;

static vmpHdr_t	*hostpkt = NULL;

/*
 *  vm_hostinfo()  
 */
int
vm_hostinfo()
{
	char		*id = "vm_hostinfo";
	vmpHdr_t	hdr, *rhp;
	int		seq = vGetSeq();
	
	vClearError();			/* clear global errors */

	/*
	**	build and send hostinfo msg
	*/
	memset(&hdr, 0, sizeof(hdr));	/* for purify */
	hdr.vh_magic = VMP_MAGIC;
	hdr.vh_type = M_HOSTINFO;
	hdr.vh_seq = seq;
	hdr.vh_size = 0;

	if (lSnd2RP(&hdr))
		return(1);

	/*
	**	process replies
	*/
	for (;;) {
		if ((rhp = lRcvFromRP(IPC_DELAY)) == NULL)
			return(1);
		if (seq != rhp->vh_seq  &&  rhp->vh_type != M_SHUTDOWN)
			continue;
		switch (rhp->vh_type) {
		    case M_HOSTINFO:
			if (hostpkt != NULL)
				nFree(hostpkt);
			hostpkt = rhp;
			HostTotal = rhp->vh_count;
			Hosts = MSGPTR(rhp, HostInfo_t);
			return(0);
		    case M_VERROR: 
			vInstallError(MSGPTR(rhp, vmpError_t));
			nFree(rhp);
			return(1);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			return(1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			VMError = E_INTERNAL;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
					"Unexpected packet: %s",
					vMsgType(rhp->vh_type));
			return(1);
		}
	}
}

  HostInfo_t *
HostInfo(int i)
{
	char	*id = "HostInfo";

	if (Hosts == NULL)
		nLogMsg(EL_FATAL, id, "Host info not initialized");
	if (i < 0  ||  i > HostTotal)
		nLogMsg(EL_FATAL, id, "Host %d not defined", i);
	return(&Hosts[i]);
}
