/*
 *  vm_rc.c - general RC stuff
 */

#include <ctype.h>
#include "cl_space.h"

RC_t			*RCArray = NULL;
int			RCTotal = 0;

extern VolType_t	VolTypeTab[];

static vmpHdr_t		*rcpkt = NULL;

/*
 *  vm_config() - get RC config from RP
 */
  int
vm_configRC()
{
	char		*id = "vm_configRC";
	vmpHdr_t	pkt;
	int		seq = vGetSeq();

	if (rcpkt != NULL) {
		nFree(rcpkt);
		rcpkt = NULL;
		RCArray = NULL;
		RCTotal = 0;
	}

	/*
	 *  send request to RP
	 */
	memset(&pkt, 0, sizeof(pkt));	/* for purify */
	pkt.vh_type = M_RCINFO;
	pkt.vh_magic = VMP_MAGIC;
	pkt.vh_seq = seq;
	pkt.vh_size = 0;
	if (lSnd2RP(&pkt))
		return (1);

	/*
	 *  get response
	 */
	for (;;) {
		vmpHdr_t	*hp;
		vmpError_t	*ep;

		hp = lRcvFromRP(IPC_DELAY);
		if (hp == NULL)
			nLogMsg(EL_FATAL, id, "rcv from RP");

		if (seq != hp->vh_seq  &&  hp->vh_type != M_SHUTDOWN) {
			nLogMsg(EL_ERROR2, id, 
					"bad packet sequence (skipping): %s", 
					vMsgType(hp->vh_type));
			continue;
		}
		switch (hp->vh_type) {
		    case M_RCINFO:
			/*
			 *  Use the packet itself for the
			 *	config, ignoring the header.
			 *	If the config ever became
			 *	dynamic, then we'd want to
			 *	copy & then free the pkt.
			 */
			RCTotal = hp->vh_count;
			RCArray = MSGPTR(hp, RC_t);
			rcpkt = hp;
			return(0);
		    case M_VERROR: 
			ep = MSGPTR(hp, vmpError_t);
			vInstallError(ep);
			nFree(hp);
			return(1);
		    case M_SHUTDOWN:
			nLogMsg(EL_ERROR1, 0, "volman shutdown received");
			VMError = E_DOWN;
			nFree(hp);
			return(1);
		    case M_NOP:
			nFree(hp);
			continue;
		    default:
			nLogMsg(EL_FATAL, id, "Unexpected packet: %s", 
						vMsgType(hp->vh_type));
		}
	}
}

  RC_t *
cRCInfo(int number)
{
	int	i;
	RC_t	*rcp = RCArray;

	if (rcp == NULL)
		nLogMsg(EL_FATAL, 0, "RCArray not initted");

	for (i=0; i<RCTotal; i++, rcp++) {
#ifdef _INTRINSA_PREFIX_
		if (rcp == NULL)
			exit(1);
#endif
		if (rcp->rcd_number == number)
			return (rcp);
	}
	return (NULL);
}

  RC_t *
cGetRcp(char *rcname)		/* get the rcp based on RC name or number */
{
	RC_t	*rcp = RCArray;
	char	*cp;
	int	i;

	if (rcp == NULL)
		nLogMsg(EL_FATAL, 0, "RCArray not initted");

	/*
	 *  see if rcname is all digits, if so, consider it the rc number
	 *	else it is an RC ID string
	 */

	for (cp = rcname; *cp; cp++)
		if (!isdigit(*cp))
			break;
	if (*cp == '\0')
		return(cRCInfo(atoi(rcname)));

	/*
	 *  look up RC ID
	 */

	for (i=0; i<RCTotal; i++, rcp++)
		if (strcmp(rcp->rcd_id, rcname) == MATCH)
			return(rcp);
	return (NULL);
}

  char *
cRCId(int number)
{
	RC_t		*rcp;
	static char	returnstr[20];
	
	rcp = cRCInfo(number);
	if (rcp)
		return (rcp->rcd_id);
	sprintf(returnstr, "[RC %d]", number);
	return (returnstr);
}


/* 
 *  vm_prRC() - print a table of RCs for various usage() cmds
 */
  void
vm_prRC(FILE *stream, char *offset, char *type, int media)
{
	RC_t	*rcp = RCArray;
	int	rcno;

	if (rcp == NULL)
		nLogMsg(EL_FATAL, 0, "RCArray not initted");

	if (media)
		fprintf(stream, "%sRCs:\tID\tnumber\tmedia\n", offset);
	else
		fprintf(stream, "%sRCs:\tID\tnumber\n", offset);
	for (rcno=0; rcno<RCTotal; rcno++,rcp++) {
#ifdef _INTRINSA_PREFIX_
		if (rcp == NULL)
			exit(1);
#endif
		if (type  &&  strcmp(type, rcp->rcd_type))
			continue;
		if (media) {
			int	i;
			fprintf(stream, "%s\t%s\t%2d\t", 
				offset, rcp->rcd_id, rcp->rcd_number);
			for (i=0; i<8*sizeof(int); i++) {
				int	med = 1<<i;
				if (med & rcp->rcd_media)
					fprintf(stream, "%s ",vMediumName(med));
			}
			printf("\n");
		} else 
			fprintf(stream, "%s\t%s\t%2d\n", 
				offset, rcp->rcd_id, rcp->rcd_number);
	}
}

/* 
 *  vm_prRCTypes() - print a table of RCs w/ vsn types for various usage() cmds
 */
  void
vm_prRCTypes(FILE *stream)
{
	RC_t		*rcp = RCArray;
	VolType_t	*vtp;
	int		rcno, i;

	if (rcp == NULL)
		nLogMsg(EL_FATAL, 0, "RCArray not initted");

	fprintf(stream, "RCs:\tID\tnumber\tVOL_TYPES\n");
	for (rcno=0; rcno<RCTotal; rcno++,rcp++) {
#ifdef _INTRINSA_PREFIX_
		if (rcp == NULL)
			exit(1);
#endif
		fprintf(stream, "\t%s\t%2d\n", rcp->rcd_id, rcp->rcd_number);
		for (i=0,vtp=VolTypeTab; vtp->vt_name; vtp++, i++)
			if (vtp->vt_medium & rcp->rcd_media)
				fprintf(stream, "\t\t\t%s\n", vtp->vt_id);
	}
}

