/*
 *  Volume Manager
 *	Client IPC: socket version
 *
 */

#define CLNTSOCKSLEEP 5

#include "cl_space.h"

extern char	*volcd_service;
extern ushort	volcd_port;

static IPC_t	Clientsock;

int
ClientIPCRcvfd()
{
	return(Clientsock.sock_fd);
}

  void
ClientTermIPC(int saverror)
{
	char		*id = "ClientTerm/sock";
	int		savVMError = VMError;
	vmpHdr_t 	*hp;
	char		buf[PKTSIZ(M_GOODBYE)];

	vClearError();

	switch (Clientsock.sock_state) {
	      case SS_NOSOCK:
		break;
	      case SS_TALKING:
	      case SS_HELLOD:
		/*
		 *  Send M_GOODBYE to RP, then shut down connection.
		 */
		hp = vBuildHdr(M_GOODBYE, NOSEQ, (vmpHdr_t *)buf);
		lSnd2RP(hp);
		Clientsock.sock_state = SS_GOODBYE;
		CloseIPC(&Clientsock);
		break;
	      default:
		nLogMsg(EL_ERROR1, id, "invalid Client socket state %d",
			Clientsock.sock_state);
		break;
	}
	if (saverror == SAVERROR) {		/* restore error vars */
		VMError = savVMError;
	}
}
/*
 *  FUNCTION:	ClientInitIPC()
 *		Initialize a client connection to the RP server.
 *		Returns socket descriptor if success, -1 if failure.
 */
  int
ClientInitIPC(int flag, unsigned int logflags)
{
	char			*id = "ClientInitIPC";
	struct sockaddr_in	ip_addr;
	struct hostent 		*hostent;
	struct servent		*servent;
	int			sd, fd;
	char 			myhost[MAXHOSTNAMELEN];
	int			retry;
	socklen_t		len;
	char			hbuf[PKTSIZ(vmpHello_t)];
	vmpHdr_t		*hp;
	vmpHello_t		*hellop;

	Clientsock.sock_state = SS_NOSOCK;

	/*
	 *  set keys =; new sndkey comes w/ volcd response
	 */
	Clientsock.sock_rcvkey = Clientsock.sock_sndkey = vGetID();

	if ((servent = getservbyname(volcd_service, "tcp")) == NULL) {
		VMError = E_SYS;
		nLogMsg(logflags|EL_SYS, id, "getservbyname");
		return(VMError);
	}
	volcd_port = ntohs(servent->s_port);

        /*
	 *  NOGIVEUP:
         *  At request of RNS... replace 'for' loop condition with
	 *  this one to have client procs wait indefinitely for connection
	 *  (i.e. never give up)
	 *
         *  for (retry = 0; ; retry++) {
	 *  }
	 *	<put in IORETRY>
         */

	len = sizeof( struct sockaddr_in );

        for (retry=0; ; retry++) {

		(void) sleep((unsigned int)(retry > 1 ? CLNTSOCKSLEEP : 1));

		if (gethostname(myhost, (size_t)MAXHOSTNAMELEN) == -1) {
			VMError = E_SYS;
			nLogMsg(logflags|EL_SYS, id, "gethostname");
			continue;
		}

		hostent = gethostbyname(myhost);
		if (hostent == NULL) {
			VMError = E_SYS;
			nLogMsg(logflags|EL_SYS, id, "gethostbyname (%s)", 
								myhost);
			continue;
		}

		sd = socket(AF_INET, SOCK_STREAM, 0);
		if (sd == -1) {
			VMError = E_SYS;
			nLogMsg(logflags|EL_SYS, id, "socket");
			return(VMError);
		}
		memcpy(&ip_addr.sin_addr, hostent->h_addr,
			(size_t)hostent->h_length);
	
		ip_addr.sin_family = hostent->h_addrtype;
		ip_addr.sin_port = htons(volcd_port);

		if (connect(sd, (struct sockaddr *)&ip_addr, len) == 0)
			break;

		/*  connect failed */
		switch (errno) {
		    case ETIMEDOUT:
			if (retry < IORETRY) {
			    nLogMsg(logflags, 0,
			      "timeout on connect to vold; retrying...");
			}
			break;
		    case EADDRINUSE:
			/* socket synchronization error? 
			   occurs infrequently.. retry */
			break;
		    default:
			VMError = E_COMM;
			nLogMsg(logflags|EL_SYS, 0, 
			  "volman %s probably down: \n\t%s",
			  "client daemon (volcd) or vold",
			  "connect system err");
			close(sd);
			return(VMError);
		}
		close(sd);
		if (retry > IORETRY) {
			VMError = E_COMM;
			nLogMsg(logflags|EL_ERROR1, id, "giving up on volcd");
			return(VMError);
		}
	}
	Clientsock.sock_fd = sd;
	
	/*
	 *  build and send M_HELLO message
	 */
	hp = vBuildHdr(M_HELLO, NOSEQ, (vmpHdr_t *)hbuf);
	hellop = MSGPTR(hp, vmpHello_t);	/* point to message */
	hellop->vhe_uid = geteuid();		/* tell who we are */
	hellop->vhe_version = VM_VERSION;	/* version number */
	hellop->vhe_pid = lPid;			/* our pid */
	hellop->vhe_flag = flag;

	if (lSnd2RP(hp)) {
		ClientTermIPC(SAVERROR);
		return (VMError);
	}
	nLogMsg(EL_DEBUG2, id, "sent M_HELLO");

	/*
	 *  get reply, should be M_HELLO packet
	 */

	if ((hp = lRcvFromRP(IPC_DELAY)) == NULL) {
		ClientTermIPC(SAVERROR);
		return (VMError);
	}
	if (hp->vh_type != M_HELLO) {
		VMError = E_INTERNAL;
		nLogMsg(logflags|EL_ERROR1, id, 
				"expected HELLO, got %s", vHdrStr(hp));
		ClientTermIPC(SAVERROR);
		return (VMError);
	}
	nLogMsg(EL_DEBUG2, id, "rcvd M_HELLO");
	hellop = MSGPTR(hp, vmpHello_t);	/* point to message */

	/*
	 *  open/read file containing sndkey 
	 */
	fd = open(vIDFile(hellop->vhe_id), O_RDONLY);
	if (fd == -1) {
		VMError = E_INTERNAL;
		nLogMsg(logflags|EL_SYS, id, "opening key file %s", 
					vIDFile(hellop->vhe_id));
		ClientTermIPC(SAVERROR);
		return (VMError);
	}
	switch (read(fd, &Clientsock.sock_sndkey, sizeof(Id_t))) {
		case sizeof(Id_t):
			break;
		case -1:
			logflags |= EL_SYS;
			/* fall thru */
		default:
			VMError = E_INTERNAL;
			nLogMsg(logflags, id, "reading key file %s",
					vIDFile(hellop->vhe_id));
			ClientTermIPC(SAVERROR);
			close(fd);
			return (VMError);
	}
	nLogMsg(EL_DEBUG2, id, "read key");
	close(fd);

	Clientsock.sock_fd = sd;
	/* these other things are pretty much meaningless for the client: */
	Clientsock.sock_state = SS_TALKING;
	Clientsock.sock_id = lPid;
	return(0);
}


/*
 *  lSnd2RP(): call vSend() and check for dead RP by vDeadReader
 *	(set by vSigPipe()).  If RP is dead, init shutdown.
 */
  int
lSnd2RP(vmpHdr_t *hp)
{
	char	*id = "lSnd2RP";

	vClearError();
	/*
	 * M_HELLO messages are sent to the RP before SS_TALKING, but
	 *	all others require it
	 */
	if (hp->vh_type != M_HELLO && Clientsock.sock_state != SS_TALKING) {
		VMError = E_DOWN;
		nLogMsg(EL_CODE|EL_ERROR1, NULL,
		  "Not connected to volcd!  Call vm_init() first. (%s)",
				vHdrStr(hp));
		return (VMError);
	}

	if (vSndPkt(hp, &Clientsock) == E_COMM) {
		Clientsock.sock_state = SS_NOSOCK;
		VMError = E_DOWN;
		nLogMsg(EL_STATUS1|EL_CODE, id, "volcd terminated");
		ClientTermIPC(SAVERROR);
	}

	return (VMError);
}

/*
 *  lRcvFromRP() - receive packet on client sock
 */
 vmpHdr_t *
lRcvFromRP(int delay)
{
	return(vRcvPkt(&Clientsock, delay));
}

