/*
 * Volume Manager
 *
 * vmerror.h: Volume Manager errors
 *
 * $Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/lib/vm_error.h,v 1.1.1.1 2000-02-28 02:18:22 wrstuden Exp $
 */

/*
 * Error reporting philosophy:
 *	All functions use LogMsg() to log errors.  LogMsg() supports
 *	both global variable-based (for the AC) and file-based (for
 *	RP/RC/VDM) error reporting.
 *
 * Global variable errors:
 *	VMError contains the error code for the last error.
 *	All functions can set VMError, some set nErrStr to a comment
 *	about what happened.
 *	VMErrno is set to errno when a system error (E_SYS) occurs.
 *	VMErrorToString converts a VMError to a string (not the same
 *	as VMErrStr).
 *
 *	Functions return 0 for success, or the value of VMError if an
 *	error, except functions which return an address which return
 *	the address if success, or NULL (and set VMError) if error.
 */

#ifndef _VMERROR_
#define _VMERROR_


/*
 *  Error codes for VMError
 *	These are the (only) error codes the user sees,
 *	so they should be simple and high-level in nature.
 *	They can be supplemented with a descriptive string in VMErrStr.
 *	Any changes to these errors should be reflected in vm_p.c.
 */
#define	E_NOERROR	0		/* indicates no error;must be 0 */
#define	E_NOMEM		1		/* couldn't allocate memory	*/
#define	E_SYS		2		/* system error (check errno)	*/
#define	E_DOWN		3		/* RP not processing requests	*/
#define	E_INTERNAL	4		/* internal error		*/
#define	E_BADPACKET	5		/* packet check failed		*/
#define	E_IO		6		/* I/O error			*/
#define E_USER		7		/* user gave bad parameter	*/
#define E_DBASE		8		/* Informix error		*/
#define E_ADMIN		9		/* data base inconsistency	*/
#define E_PERM		10		/* permission denied		*/
#define	E_NOTFOUND	11		/* volume can't be located	*/
#define	E_UNAVAIL	12		/* vsn/RC unavailable/in use	*/
#define E_OPS		13		/* ops problem - not logged in	*/
#define E_CANCELED	14		/* request was canceled		*/
#define E_COMM		15		/* communications problem 	*/
#define E_HARDWARE	16		/* problem in the real world	*/
#define E_STUCK		17		/* stuck in drive 		*/

#endif	/* _VMERROR_ */
