/*
 *  Volume Manager
 *
 *  vm_ipc.h: IPC definitions, type of IPC selected by Makefile -Dxxx.
 *
 *	Each type of ipc defines an IPC_t which among other things
 *	is part of Client_t on the RP side. 
 *
 *	Choices are:
 *		sock
 *		fifo <way superseded code>
 *
 * $Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/lib/vm_ipc.h,v 1.1.1.1 2000-02-28 02:18:22 wrstuden Exp $
 */

#ifndef	_VM_IPC_
#define	_VM_IPC_

/*
 *  general flags used for send/rcv ipc; 
 */
#define IPC_DELAY	0x0001	/* set for DELAY read/write */
#define IPC_NDELAY	0x0002	/* set for NDELAY read/write */

#ifdef vmipc_sock

/*
 *  socket stuff
 */

#define VM_SOCKTIMEOUT	5

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

/*
 *  SockState_t:
 *	The states in this struct are mainly used to keep track
 *	of the security handshake between the RP and its client.
 *
 *  NOTE: All changes in SockState_t must also be made to ShutdownSock()!!!!
 */
typedef enum SockState {
	SS_NOSOCK,		/* unconnected socket 			*/
	SS_CONNECTED,		/* connection established, nothing rcvd	*/
	SS_HELLOD,		/* next msg w/ correct key accepted	*/
	SS_TALKING,		/* Client connection complete		*/
	SS_GOODBYE		/* RP rcvd M_GOODBYE from Client	*/
} SockState_t;

/*
 *  IPC_t: included in Client_t
 */
typedef struct {
	SockState_t	sock_state;		/* socket state		    */
	int		sock_flags;		/* socket flags		    */
	int		sock_fd;		/* socket fd                */
	int		sock_id;		/* socket id (pid for RP)   */
	Id_t		sock_sndkey;
	Id_t		sock_rcvkey;
	/* 'process' info used by servers: */
	int		sock_ptype;		/* type of process	    */
	void		*sock_proc;		/* process info		    */
	void		*sock_host;		/* host info		    */
} IPC_t;

/* sock_ptype */
#define PT_NONE		0
#define PT_VOLCD	1
#define PT_VOLNODE	2
#define PT_RC		3
#define PT_TIMER	4
#define PT_CONSOLE	5

#endif /* sock */

#ifdef fifo
/*
 *  fifo stuff - totally out of date..
 */

/*
 *  NOTE: any changes here must be also made to FifoShutdown()!!!!
 */
typedef enum FifoState {
	FS_IDLE,			/* Idle; not connected		*/
	FS_HELLO,			/* received M_HELLO		*/
	FS_CREATED,			/* fifo created			*/
	FS_OPENED,			/* fifo opened			*/
	FS_HIPVRL,			/* sent M_HELLO to PVRL		*/
	FS_ACKPVRL,			/* rcvd M_ACK from PVRL		*/
	FS_TALKING,			/* connection established	*/
	FS_GOODBYE			/* rcvd M_GOODBYE from client	*/
} FifoState_t;

typedef enum FifoType {
	FT_RPFIFO,			/* RP Master Input Fifo		*/
	FT_CLIENTFIFO			/* Client Master Input Fifo	*/
} FifoType_t;

/*
 *  IPC_t: one of these is created in the RP for each fifo
 *
 *	Reads on fifos are done through stdio, writes are not.
 *	The RP needs to open fifos twice for rd/wr (for M_HELLO)
 *	and wronly (for all other writes).  fifo_rwfd is closed
 *	after the M_HELLO packet is written out (see InitClient()).
 */
typedef struct Fifo {
	char		fifo_nam[FIFONAMLEN+1];	/* null-term'ed file name */
	int		fifo_rwfd;	/* fifo file descriptor (rd/wr) */
	int		fifo_wofd;	/* fifo fd (write only)		*/
	int		fifo_fd;	/* fifo fd used by vSend()	*/
	FILE		*fifo_strp;	/* fifo stream ptr (readers only) */
	FifoState_t	fifo_state;	/* fifo state			*/
	FifoType_t	fifo_type;	/* fifo type			*/
	int		fifo_flags;	/* fifo flags  			*/
}  IPC_t;

/* #defines for fifo_flags */
#define	FI_PRIOR	0x0001		/* Use a reserved fifo if needed */
#define FI_RESERV	0x0002		/* this fifo used a reserved fifo*/

#endif /* fifo */

#endif	/* _VM_IPC_ */
