#ifndef	_VM_PKT_H
#define	_VM_PKT_H
%
%/* 
% *  rpcgen source: vm_pkt.x - .c & .h files containing this
% *	message should never be checked in.
% */
%
%
/*
 *  Volume Manager
 *
 *  vm_pkt.x: Volume Manager Packet Protocol
 *
 *	rpcgen is used to generate vm_pkt.h and vm_pkt_xdr.c from 
 *	this file. 
 *
 *	All structures that are included in packets must be defined in
 *	this file, in order to generate the necessary XDR routines to
 *	send them between hosts. Thus this file is in the lib dir,
 *	even though all specific XDR use is confined to the srvr dir
 *	(clients need to see the same structures).
 *
 *	Contents:
 *			volume definition
 *			device definition
 *			packet structures
 *
 *  Bill Ross, NASA Ames, June 1995. 
 */
/*
 *  All lines beginning with '%' are passed through rpcgen unchanged.
 *	Otherwise, comments are stripped and structures modified.
 *
 *	vm_pkt.h is used for all communications, vm_pkt_xdr.c is
 *	only used for inter-host XDR-encoded packets. Note: in
 *	vm_pkt.h, for all "struct xx" rpcgen adds "typedef xx").
 *
 *	The XDR protocol guarantees that the data types in the packet 
 *	structures aren't affected by architectural differences
 *	between machines.
 */

/*
 *  HACK - time_t defined as an XDR-known type
 */
#define time_t int

/*************************************************************************
			VOLUME DEFINITION
**************************************************************************/

#ifdef RPC_HDR
%/*
% *  note that changing VSNLEN or DESCLEN would invalidate any 
% *	existing dbase, so a translation prog would have to be
% *	written (see dbtrans.c for an earlier generation of
% *	translation). NOTENOTE: VSNLEN moved to nasutil.h; it's
% *	a standard by the way (ansi?)
% */
%#include "nasutil.h"
%#define	DESCLEN		40	/* length of dbase description field */
#endif

struct vsn_t {
	char    v_vsn[VSNLEN];
};

typedef unsigned int	Id_t;		/* message key type		*/
typedef unsigned int	Seq_t;		/* sequence number type		*/

/*
 *  Volume Descriptor Record (VDR), used for dbase & packets
 */

/*
 *  The VInfo subset of the VDR elements
 */
struct VInfo_t {
	vsn_t		vi_vsn;		/* volume serial number		*/
	vsn_t		vi_label;	/* internal label		*/
	int		vi_voltype;	/* key to volume attributes	*/
	int		vi_poolgid;	/* volume's pool group ID	*/
	int		vi_srcrc;	/* repository controller number	*/
	int		vi_dstrc;	/* repository controller number	*/
	int		vi_uid;		/* volume's user ID		*/
	int		vi_gid;		/* volume's group ID		*/
	short		vi_mode;	/* volume's permissions		*/
};


/*
 *  the Volume Descriptor Record (VDR) is the struct which 
 *	is stored in the database
 */
struct	VDR_t {
	VInfo_t	vdr_info;		/* vol info (passed to RP)	*/
	char	vdr_status;		/* volume status		*/
	char	vdr_use;		/* volume use status		*/
	time_t	vdr_creat;		/* creation date		*/
	time_t	vdr_sttim;		/* status change date		*/
	time_t	vdr_mod;		/* last modification date	*/
	time_t	vdr_acc;		/* last access date		*/
	time_t	vdr_lastmov;		/* last move			*/
	int	vdr_mntwrite;		/* times mnt writable tried	*/
	int	vdr_mntread;		/* times mnt rdonly tried	*/
	int	vdr_mntfail;		/* times in a row mnt failed	*/
	int	vdr_recycle;		/* times recycled		*/
	char	vdr_desc[DESCLEN];	/* volume description		*/
};

#ifdef RPC_HDR
%/* #defines for ease of access to VInfo struct in VDR_t struct */
%#define	vdr_vsn		vdr_info.vi_vsn
%#define	vdr_label	vdr_info.vi_label
%#define	vdr_voltype	vdr_info.vi_voltype
%#define	vdr_poolgid	vdr_info.vi_poolgid
%#define	vdr_uid		vdr_info.vi_uid
%#define	vdr_gid		vdr_info.vi_gid
%#define	vdr_mode	vdr_info.vi_mode
%#define	vdr_rc		vdr_info.vi_srcrc

%/* vdr_use: 	NOTE: these values are used in the volman dbase & for volstat.
% */
%#define	U_IDLE		'I'	/* not being moved or mounted	*/
%#define	U_MOVE		'M'	/* volume in transit		*/
%#define	U_MOUNT		'm'	/* mounted or mount in progress	*/
%#define	U_UNMOUNT	'u'	/* only used for volstat now	*/
%#define	U_SCRATCH	'S'	/* only used for volstat now	*/

%/* vdr_stat: 	WARNING: these are the values used in the volman dbase:
% *		changing them here could lead to serious inconsistencies
% *		unless the dbase were updated (IN THE PROPER ORDER) at
% *		the same time.
% *		NOTE - update string tables in vm_p.c and vls.c if
% *		adding new states.
% */
%#define	S_VIRGIN	'V'	/* new tape waiting for mag label */
%#define	S_SCRATCH	'S'	/* scratch			*/
%#define	S_ALLOC		'A'	/* user-owned volume		*/
%#define	S_RECYCLE	'R'	/* waiting to be scratched	*/
%#define	S_RECYCLEBAD	'B'	/* waiting to be scratched	*/
%#define	S_TRASH		'T'	/* waiting to be discarded	*/
%#define	S_OBSOLETE	'O'	/* waiting to be sold off	*/
%#define	S_DELETED	'D'	/* deleted from dbase		*/

% /* stat bits: used to squash multiple states together in vmpVls_t -
%    update StatusToBit in vm_p.c if these change */
%#define	ST_VIRGIN	0x01
%#define	ST_SCRATCH	0x02
%#define	ST_ALLOC	0x04
%#define	ST_RECYCLE	0x08
%#define	ST_RECYCLEBAD	0x10
%#define	ST_TRASH	0x20
%#define	ST_OBSOLETE	0x40
%#define	ST_DELETED	0x80

#endif /* RPC_HDR */

#ifdef __cplusplus
extern "C"{
char* vm_vinfo(char* volser);
}
#endif

/*************************************************************************
			HOST DEFINITION
**************************************************************************/

/*
 *  The RP reads a list of allowed hosts from its config file.
 *	Clients can get the list via M_HOSTINFO in order to
 *	map which host a drive is being accessed from. 
 */
#ifdef RPC_HDR
%#define HOSTNAMELEN	12
%/* h_flag */
%#define H_HASVOLCD	0x1
%#define H_HASVOLND	0x2
#endif

struct HostInfo_t {
	char	h_name[HOSTNAMELEN];
	int	h_flag;
	int	h_nclients;
};

/*************************************************************************
			DEVICE DEFINITION
**************************************************************************/

/*
 *  The RP keeps the master device config, and clients (including RCs) 
 *	download.
 *
 *  Note: if full distributed volume mgt is implemented, the device
 *	host and path prefix (/dev, /dev/rmt, etc) can be found
 *	from the RC indicated in the device structure.
 *
 *  Original bitmapping & structs by Jon Hahn.
 */
#ifdef RPC_HDR

%#define	MAXCHANLEN	8	/* max length / dev data path tag   */
%#define	MAXDEVLEN	8	/* max length / dev name for humans */

%#define	MAXDEVS		50	/* max number of devices	   */
					/* we can know about	           */
%#define	DM_SETSIZE	MAXDEVS	
%#ifndef NBBY
%# define	NBBY		8		/* number of bits in a byte */
%#endif
%#define	NDMBITS		(sizeof(unsigned int) * NBBY)  	/* bits per mask element    */


%#ifdef howmany
%# undef howmany
%#endif
%#define	howmany(x, y)	(((x)+((y)-1))/(y))

%#define 	DMS_BITSIZE	howmany(DM_SETSIZE, NDMBITS)

#endif /* RPC_HDR */

struct DevMap_t {
	unsigned int	dms_bits[DMS_BITSIZE];
};

/*
 *  AcsDevLoc: ACS device location:
 *	additional description for each device in an ACS
 */
struct AcsDevLoc_t {
	int	ad_lsm;			/* library storage module	*/
	int	ad_panel;		/* panel			*/
	int	ad_drv;			/* drive number in panel	*/
};				/* specifies where a drive is	*/

/*
 *  DevEnt: device entry: one of these for each device in the system
 *
 *
 */
struct DevEnt_t {
	int		de_rc;			/* repository controller */
	int		de_medium;		/* medium (VM_XXX)       */
	int		de_density;		/* densities supported	 */
						/* (not relevant/1999)   */
	int		de_rcflags;		/* RC-dependant issues   */
	char		de_name[MAXDEVLEN];	/* name for humans	 */
	char		de_defvary;		/* vary on or not	 */
	/* the next 3 or 4 items would become an array if drives are
		mounted on multiple hosts */
	int		de_host;		/* host drive is on      */
	int		de_controller;		/* dev controller number */
	int		de_devno;		/* device node number    */
	char		de_chan[MAXCHANLEN];	/* data path		 */
	AcsDevLoc_t	de_acsdevloc;		/* for ACS RC            */
	int		de_status;		/* DS_VARY, etc.         */
	unsigned long	de_errstr;		/* last error if down    */
						/* used as char*, int for xdr */
};

#ifdef RPC_HDR
/* de_errstr size: */
%#define ERRSTRSIZE	40

/* de_status: device status: */
%#define	DS_VARY		0x01	
%#define	DS_CREATENODE	0x02		
%#define	DS_DELETENODE	0x04

/* de_density: density capability: */
%#define	DEN_800		0x01
%#define	DEN_1600	0x02
%#define	DEN_6250	0x04
%#define	DEN_38K		0x08
#endif /* RPC_HDR */

/*************************************************************************
			    RC DEFINITION
**************************************************************************/
/*
 *	This is the subset of the RC info which is sent to clients
 *	(as the body of M_RCINFO response packets.) 
 *
 *	The info is maintained by the RP along with other RC info 
 *	defined in rp_rc.h.
 *
 *	The rcd_number is the info most central to the definition 
 *	of an RC: it is on every vsn in the database that is 
 *	in or moving to or from that RC. It is recommended that 
 *	at a given site these numbers never be recycled. 
 *
 */

#ifdef RPC_HDR
%#define RC_STRLEN	40
%#define RC_MEDIA_TYPES	2
#endif


struct Media_t {
	int		d_medium;	/* DT_ codes */
	int		d_nconfigdev;	/* total in d_devmap */
	int		d_nvaryondev;	/* ones varied on */
	int		d_mountdev;	/* total w/ mounts sent &/or done */
	DevMap_t	d_devmap;	/* all drives of type d_medium in RC */
};

struct RC_t {
	int	rcd_number;		/* the RC_ number #defined below   */
	int	rcd_index;		/* index in the RCArray		   */
	char	rcd_name[RC_STRLEN];	/* a string describing the RC	   */
	char	rcd_id[RC_STRLEN];	/* condensed name for RC	   */
	char	rcd_type[RC_STRLEN];	/* for checking w/in RC program	   */
	char	rcd_login[RC_STRLEN];	/* login this rc should run as	   */
	int	rcd_uid;		/* rcd_login uid, filled in by RP! */
	int	rcd_syslog;		/* syslog facility, 0 if none	   */
	int	rcd_flags;		/* see below			   */
	int	rcd_deftype;		/* default voltype for alloc	   */
	int	rcd_density;		/* densities this RC supports	   */
	int	rcd_defden;		/* default density for this RC	   */
	int	rcd_media;		/* vol media RC can hold	   */
	int	rcd_nmedia;		/* number of different media drives */
	Media_t	rcd_devs[RC_MEDIA_TYPES]; /* per-medium info		   */
	DevMap_t rcd_devmap;		/* devices in this RC		*/
	int	rcd_nconfigdev;		/* count of devmap		   */
	int	rcd_mounttimeout;	/* optional; triggers RP cons msgs */
	int     rcd_capacity;		/* max number vsns; -1==plenty	   */
        int	rcd_used;		/* #/vsns in RC plus moves to it   */
        int	rcd_freeslots;		/* # slots free (from RC); -1==plenty */
	int	rcd_mounttime;		/* mount timeout, < 0 if none	*/
	char	rcd_acsid;		/* for ACS's: which lib farm	*/
};

#ifdef RPC_HDR
/* rcd_flags: */

%#define	RCF_READY	0x0001		/* RC ready for requests	*/
%#define	RCF_TIMER	0x0002		/* enable RC M_TIMER messages	*/
%#define	RCF_ASYNCUNMNT	0x0004		/* enable async unmount msgs	*/
%#define	RCF_SETLED	0x0008		/* flag for setting drive LED   */
%#define	RCF_MOUNTOK	0x0010		/* has drives, i.e. can mount	*/
%#define	RCF_ALLOCOK	0x0020		/* ok to allocate scratch	*/
%#define	RCF_HASQUOTA	0x0040		/* user needs quota to alloc	*/
%#define	RCF_RECYCLEOK	0x0080		/* ok to recycle (deallocate)	*/
%#define	RCF_NODEMOUNT	0x0100		/* check node for mount		*/
%#define	RCF_NONODE	0x0200		/* RC won't do any node stuff	*/
%#define	RCF_CACHEMOUNT	0x0400		/* set drive LCD on node delete */
%#define	RCF_CLEARFIRST	0x0800		/* rew/unload b4 node delete	*/
%#define	RCF_REWIND	0x1000		/* rewind b4 dismount		*/
%#define	RCF_REWUNLOAD	0x2000		/* rew/unload b4 dismount	*/

#endif

/*************************************************************************
				PACKETS
**************************************************************************/

/* 
 *  VMP message types - vh_type = int in vmpHdr_t
 *	don't change types or numbers without also changing VmpTab[]!
 */

#ifdef RPC_HDR
%#define	M_MINTYPE	1	/* for range checking */
%#define	M_ACK		1	/* acknowledge			*/
%#define	M_DEBUG		2	/* for debug messages		*/
%#define	M_DETACH	3	/* make a request async		*/
%#define	M_DEVQRY	4	/* query device status		*/
%#define	M_DEVSTAT	5	/* contains device status	*/
%#define	M_VERROR	6	/* error code			*/
%#define	M_GOODBYE	7	/* shutdown a connection	*/
%#define	M_HELLO		8	/* initiate a connection	*/
%#define	M_LBLOOK	9	/* VDR lookup by label & UID	*/
%#define	M_RCMOUNT	10	/* mount a tape			*/
%#define	M_MOVE		11	/* change the location of a vol	*/
%#define	M_NOP		12	/* no-op			*/
%#define	M_QUOTQRY	13	/* user wants quota info	*/
%#define	M_QUOTRPLY	14	/* user quota rc/medium		*/
%#define	M_RC		15	/* this client is an RC		*/
%#define	M_RCREADY	16	/* RC is ready for business	*/
%#define	M_RCMSG		17	/* send message to an RC	*/
%#define	M_REQUEUE	18	/* unmount and try another drive*/
%#define	M_SHUTDOWN	19	/* shut down daemon/client	*/
%#define	M_TIMER		20	/* timer packet			*/
%#define	M_UNMOUNT	21	/* unmount it			*/
%#define	M_VALLOC	22	/* allocate a scratch volume	*/
%#define	M_VARY		23	/* dis/allow use of a device	*/
%#define	M_VRECYCLE	24	/* deallocate a volume		*/
%#define	M_VDR		25	/* contains a copy of a VDR	*/
%#define	M_VOLSTAT	26	/* contains status of volume(2)	*/
%#define	M_VOLSTATS	27	/* contains 2+ status of vol	*/
%#define	M_VOLQRY	28	/* query volume mount status	*/
%#define	M_VLS		29	/* vsn info lookup		*/
%#define	M_VSN		30	/* VSN in response to M_VALLOC	*/
%#define	M_ADDVSN	31	/* add vsn to dbase		*/
%#define	M_DELVSN	32	/* del vsn from dbase		*/
%#define	M_SETQUOTA	33	/* set or add quota to dbase	*/
%#define	M_LBLQRY  	34	/* look up uid/label / set label*/
%#define	M_VCHMOD	35	/* change permissions		*/
%#define	M_VCHGRP	36	/* change permissions		*/
%#define	M_VCHOWN	37	/* change permissions		*/
%#define	M_AUDIT		38	/* look up vsns by rc #		*/
%#define	M_VATTR		39	/* admin set of vsn attributes	*/
%#define	M_RESERVE	40	/* reserve drives/vsns		*/
%#define	M_USRMOUNT	41	/* usr mount w/ rid		*/
%#define	M_USRMOUNT_PLL	42	/* usr mount parallel N vsns	*/
%#define	M_USRMOUNT_SEL	43	/* usr mount any one of N vsns	*/
%#define	M_NEWLOG	44	/* restart log file(s)		*/
%#define	M_ATTACH	45	/* attach async req		*/
%#define	M_RCINFO	46	/* request/receive RC config	*/
%#define	M_CREATENODE	47	/* lbl chk / node setup		*/
%#define	M_DELETENODE	48	/* rmv /dev/vol/xxxxxx		*/
%#define	M_NODED		49	/* register as volnode		*/
%#define	M_VOLCD		50	/* register as volcd		*/
%#define	M_TIMERD	51	/* register as timer		*/
%#define	M_NEWCLIENT	52	/* volcd->vold notification	*/
%#define	M_HOSTINFO	53	/* table of valid hosts		*/
%#define	M_SIGNAL	54	/* tell volcd to signal client  */
%#define	M_CONSOLE	55	/* direct conn for console cmd	*/
%#define	M_DRIVEINFO	56	/* response to M_VARY		*/
%#define	M_IDLETIME	57	/* time /dev/vol/xxxxxx unused	*/
%#define	M_SCRATCH	58	/* relabel/ return to pool	*/
%#define	M_PRIORITY	59	/* relabel/ return to pool	*/

%#define M_MAXTYPE	M_PRIORITY
#endif  /* RPC_HDR */

/*
 *  the VMP message header
 */

struct vmpHdr_t {
	char		vh_magic;	/* magic number (must be first)	*/
	int		vh_type;	/* message type			*/
	int		vh_slot;	/* remote host client slot (fd)	*/
	Seq_t		vh_seq;		/* sequence number		*/
	unsigned int	vh_size;	/* msg size (excl. of header)	*/
	unsigned int	vh_count;	/* count of subpkts if needed	*/
	Id_t		vh_key;		/* identifies sender		*/
};

#ifdef RPC_HDR
%#define	VMP_MAGIC	'%'	/* (unusual) magic for header	*/
%#define	HdrNULL			((vmpHdr_t *) 0)
#endif /* RPC_HDR */

/*
 *	Each message type may have an associated data struct.
 *	The following message types have no struct associated
 *	with them (i.e. vh_size = 0):
 *
 *		M_ACK, M_DETACH, M_GOODBYE, M_NOP, M_NEWLOG
 *
 *		M_RCINFO (request; response has vh_size != 0)
 *
 *	Some message types share a common structure, others use
 *	an exclusive structure.  The vmpVsn_t struct is used by the
 *	following message types:
 *
 *	type		v_vsn			v_rc
 *	----		-----			-----
 *	M_DELVSN	vsn			owner uid in response
 *	M_ATTACH	vsn to be reconnected	N/A
 *	M_REQUEUE	vsn to be requeued	N/A
 *	M_UNMOUNT	vsn to unmount		N/A
 *	M_VRECYCLE	vsn of recycle vol	reply: former owner
 *	M_VOLQRY	optional vsn to query	[optional]
 *	M_VSN		vsn of allocated vol	N/A
 *	M_VCHMOD	vsn			N/A
 *	M_VCHOWN	vsn			uid
 *	M_VCHGRP	vsn			gid
 *	M_AUDIT		N/A			RC identification
 *	M_SIGNAL	N/A			signal
 *
 *	If there's a data struct associated with a message type, normally
 *	only *	one such struct will accompany it; see the VmpTab in vm_p.c -
 *	variably-composed packet types are indicated by a -1 in the size field.
 *
 *      Message types using vmpHello_t:
 *
 *	M_HELLO
 *	M_VOLCD
 *	M_VOLNODE	
 *	M_RC
 *	M_TIMERD
 *	M_CONSOLE
 *	M_NEWCLIENT
 */

struct vmpHello_t {		/* for M_HELLO packets		*/
	int		vhe_version;	/* program/protocol version	*/
	Id_t		vhe_id;		/* rp->srvr key			*/
	int		vhe_uid;	/* process uid			*/
	int		vhe_pid;	/* process id			*/
	int		vhe_flag;	/* another int to play w/	*/
};


struct vmpTimer_t {		/* for M_TIMER			*/
	time_t		vt_time;	/* time vold begins sending	*/
};


struct vmpVAlloc_t {		/* for M_VALLOC			*/
	int		va_rc;		/* location/medium		*/
	int		va_poolgid;	/* id of allocation group 	*/
	int		va_type;	/* type				*/
	short		va_mode;	/* permissions			*/
	int		va_gid;		/* group ID			*/
	char		va_desc[DESCLEN]; /* optional description	*/
};

struct vmpVls_t {
	vsn_t		l_vsn;
	int		l_uid;
	int		l_rc1;
	int		l_rc2;
	int		l_rc3;
	int		l_media;
	int		l_status;	/* bit vector for ST_ codes below */
};

struct vmpAttr_t {		/* for M_VATTR			*/
	vsn_t		a_vsn;
	int		a_type;		/* vsn type			*/
	int		a_flags;	/* whether to set things	*/
	int		a_rc;		/* rc to set			*/
	char		a_status;	/* status to set		*/
	int		a_uid;		/* uid to set			*/
	int		a_gid;		/* gid to set			*/
	int		a_poolgid;	/* pool gid to set		*/
	short		a_perm;		/* permission to set		*/
	char		a_desc[40];	/* user-settable description	*/
};

#ifdef RPC_HDR
/* a_flags */
%#define	AT_RC		0x001
%#define	AT_STATUS	0x002
%#define	AT_VUID		0x004
%#define	AT_VGID		0x008
%#define	AT_PERM   	0x010
%#define	AT_VTYPE   	0x020
%#define	AT_POOLGID	0x040
%#define	AT_LABEL	0x080
%#define	AT_COMMENT	0x100
#endif /* RPC_HDR */

struct vmpVsn_t {			/* see comment above for M_ list*/
	vsn_t		v_vsn;		/* vsn of volume		*/
	int		v_rc;		/* repository controller	*/
	int		v_flag;		/* flags			*/
	int		v_flag2;	/* used by M_VCHMOD 		*/
};

#ifdef RPC_HDR
/* v_flag: (different flags for different message types) (VQ=VOLQRY) */
%#define	VQ_NONE		-1	/* dummy 			*/
%#define	VQ_VSN       	0	/* return stat on vsn		*/
%#define	VQ_UID		1	/* " on all vsn's for my uid	*/
%#define	VQ_CLIENT	2	/* " "  on all vsn's for client	*/
%#define	VQ_ALL		3	/* " on all vsn's (RPUID only)	*/
%#define	VQ_RC		4	/* " " on RC (v_rc)		*/
%#define	VQ_FORCE 	5	/* force delete dbase (M_DELVSN)*/
/* for M_UNMOUNT, v_flag gets the VO_RESPONSE flags */
#endif /* RPC_HDR */

struct vmpLblLook_t {		/* for M_LBLOOK packets		*/
	int		vl_uid;		/* uid to search for		*/
	vsn_t		vl_label;	/* internal label to search for	*/
};

struct vmpLblQry_t {		/* for M_LBLQRY packets		    */
	vsn_t		vq_vsn;		/* vsn to look up		    */
	vsn_t		vq_label;	/* internal label to check usr	    */
	int		vq_obsolete;	/* if recycling, obsolete from pool */
};

#ifdef RPC_HDR
/* NOTE - TAGLEN is set also manually as the length of
          the 2nd-to-last field in the 'fmt' string in cl_lib.c */
%#define TAGLEN 12
%#define RIDLEN	6	/* max length of rid that fits in volstat display */
#endif /* RPC_HDR */

struct vmpUsrMount_t {		/* for M_USRMOUNT packets	*/
	vsn_t		vu_vsn;		/* vsn of vol to be mounted	*/
	int		vu_flags;	/* mount parameters (see volop.h) */
	char		vu_tag[TAGLEN];	/* user can hang a string on req */
	char		vu_rid[RIDLEN];	/* length of reservation id	*/
};

struct vmpUsrMountN_t {		/* for M_USRMOUNT_PLL & M_USRMOUNT_SEL pkts */
	int		vx_flags;	/* mount parameters (see volop.h) */
	char		vx_tag[TAGLEN];	/* user can hang a string on req */
};

struct vmpRCMount_t {		/* for M_RCMOUNT packets	*/
	vsn_t		vm_vsn;		/* vsn of vol to be mounted	*/
	vsn_t		vm_label;	/* internal label 		*/
	int		vm_flag;	/* VO_ flags from volop.h	*/
	int		vm_uid;		/* uid for /dev/vol/xxxxxx node	*/
	int		vm_host;	/* hostarray index for client	*/
	DevMap_t	vm_devmap;	/* device map for RC		*/
};

struct vmpNode_t {		/* for M_NODE packets		*/
	vsn_t		vs_vsn;		/* external label for /dev/vol	*/
	vsn_t		vs_label;	/* internal label to check	*/
	int		vs_drive;	/* -1 = rmv /dev/vol node 	*/
	int		vs_flag;	/* VO_ flags relevant to node	*/
	int		vs_uid;		/* node owner			*/
	int		vs_host;	/* node owner's host		*/
	int		vs_req;		/* rp will expect volreq or not	*/
	int		vs_rc;		/* for hooking up req-less w/ rc */
};

struct vmpMove_t {		/* for M_MOVE packets		*/
	int		vmv_srcrc;	/* source RC (not used by AC)	*/
	int		vmv_dstrc;	/* destination RC		*/
	int		vmv_flags;
	char		vmv_rid[RIDLEN];/* reservation id 		*/
};

/*  vmv_nvsns vsnseq structs follow the vmpMove */

struct vsnseq_t {
	vsn_t		r_vsn;
	Seq_t		r_seq;
};

struct vmpQuota_t {
	int		vqq_uid;
	int		vqq_rc;
	int		vqq_quota;
	int		vqq_used;
};

struct vmpReserveHdr_t {
	char		re_id[RIDLEN];	/* id string			*/
	int		re_flag;	/* type of reservation		*/
	int		re_uid;		/* for status reporting 	*/
	int		re_rcmedcount;	/* number of RC/medium requests	*/
	int		re_vsncount;	/* number of vsn requests	*/
};

#ifdef RPC_HDR
/* values for re_flag */
%#define	RE_CREATE	0x001	/* create reservation id 	*/
%#define	RE_DELETE	0x002	/* delete reservation		*/
%#define	RE_USE		0x004	/* use existing reservation	*/
%#define	RE_LUSER	0x008	/* list user's reservations	*/
%#define	RE_LALL		0x010	/* list all reservations	*/
%#define	RE_ASYNC	0x020	/* process won't stay connected */
#endif /* RPC_HDR */

struct vmpReserveRCMed_t {
	int		re_rc;
	int		re_medium;
	int		re_drives;
	int		re_used;	/* for reporting status		*/
	char		re_type;	/* hard or soft reservation	*/
};

#ifdef RPC_HDR
/* values for RC re_type */
%#define	RE_HARD		'h'
%#define	RE_SOFT		's'

/* values for vsn re_type */
%#define	RE_ADD		'a'
%#define	RE_DELVSN	'd'
#endif /* RPC_HDR */

struct vmpReserveVsn_t {
	vsn_t		re_vsn;
	char		re_type;
};


struct vmpVary_t {		/* for M_VARY packets		*/
	DevMap_t	vva_dev;	/* device(s) to vary		*/
	char		vva_flags;	/* vary on or off		*/
	char		vva_err[ERRSTRSIZE];	/* error if vary off	*/
};

#ifdef RPC_HDR
/* vva_flags: */
%#define	VVA_VARYON	0x01	/* vary cmd			*/
%#define	VVA_VARYOFF	0x02	/* vary cmd			*/
%#define	VVA_RESET	0x08	/* reset devs, overrides VVA_VARY*/
%#define	VVA_DISPREQ	0x10	/* passive display request	*/
%#define	VVA_DISPREP	0x20	/* passive display reply	*/
%#define	VVA_ALL		0x40	/* 1 => apply _VARY to all devs	*/
#endif /* RPC_HDR */

struct vmpVdr_t {			/* for M_VDR packets		*/
	VDR_t		vv_vdr;		/* copy of requested VDR	*/
};

struct vmpAdd_t {			/* for M_ADD packets		*/
	VInfo_t		vad_vinfo;	/* vinfo to add			*/
	char		vad_status;	/* ownership status		*/
};

/*
 * vmpVolStat_t structs can come more than one to a packet;
 *	vv_last is non-zero for the last one
 */

struct vmpVolStat_t {		/* for M_VOLSTAT struct		*/
	vsn_t		vv_vsn;		/* vsn this is in response to	*/
	int		vv_flags;	/* see below			*/
	int		vv_rpord;	/* order of req in RP 		*/
	int		vv_pid;		/* client pid if active		*/
	char		vv_use;		/* type of operation		*/
	int		vv_state;	/* state of volume operation	*/
	time_t		vv_time;	/* time of last major state chng*/
	int		vv_dev;		/* device if mounted		*/
	int		vv_host;	/* host if mounted		*/
	int		vv_voltype;	/* info on medium/size		*/
	int		vv_uid;		/* uid of user controlling vol	*/
	int		vv_rc;		/* rc volume is in		*/
	int		vv_dstrc;	/* rc moving to if any		*/
	char		vv_rid[RIDLEN];	/* reservation id		*/
	char		vv_tag[TAGLEN];	/* user tag			*/
};

#ifdef RPC_HDR
%#define	VolStatNULL		((vmpVolStat_t *) 0)
/* vv_flags: */
%#define	VV_LAST		0x01	/* set if last struct in list	*/
%#define	VV_UID		0x02	/* set if vv_uid valid		*/
%#define	VV_RC		0x04	/* set if vv_rc valid		*/
%#define	VV_RING		0x08	/* RC sets if write ring set	*/

%#define	RCMSGSIZ	250		/* size of fixed-length message	*/
#endif

struct vmpRcMsg_t {		/* for M_RCMSG			*/
	int	rm_rc;			/* RC to send message to	*/
	char	rm_flag;		/* flags */
	vsn_t	rm_vsn;			/* optional RC specifier	*/
	int	rm_cmd;			/* optional cmd code		*/
	int	rm_arg1;		/* optional cmd arg code	*/
	int	rm_arg2;		/* optional cmd arg code	*/
	int	rm_arg3;		/* optional cmd arg code	*/
	char	rm_msg[RCMSGSIZ];	/* fixed-length message		*/
};

#ifdef RPC_HDR
/* rm_flag bits */
%#define	RCM_RC		0x01	/* use rm_rc */
%#define	RCM_VSN		0x02	/* use rm_vsn */
%#define	RCM_SRC		0x04	/* rm_vsn: send to src RC if moving */
%#define	RCM_DST		0x08	/* rm_vsn: send to dst RC if moving */
%#define	RCM_NORPFORWARD	0x10	/* don't forward contents to RP */

/* rm_cmd values - these are rc-specific & technically belong in per-rc .h */
%#define	RMC_ENTER	1	/* put CAP rm_arg in enter mode	*/
%#define	RMC_CANCEL 	2	/* cancel enter/eject on rm_arg */
%#define	RMC_REPORT	3	/* report on status		*/
%#define	RMC_SHUTDOWN	4	/* shut down rc gracefully      */
%#define	RMC_CLEAN	5	/* check cleaning media status	*/

%#define	RcMsgNULL	((vmpRcMsg_t *) 0)
#endif

struct vmpError_t {		/* for M_ERROR, M_SHUTDOWN	*/
	vsn_t		vn_vsn;		/* vsn this is in response to	*/
	int		vn_state;	/* state of volume request	*/
	int		vn_flag;	/* another integer		*/
	DevMap_t	vn_dev;		/* device			*/
	int		vn_error;	/* error code			*/
	char		vn_retry;	/* retry advisory (attempt if != 0) */
	char		vn_msg[LINELEN]; /* error string		*/
};

#ifdef RPC_HDR
/* flags for shutdown used in vn_state */
%#define	SH_WAIT		0x001
%#define	SH_RESTART	0x002
#endif

struct vmpDebug_t {		/* for M_DEBUG			*/
	int		vd_cmd;
	int		vd_level;
};

struct vmpNodeTime_t {		/* for M_IDLETIME		*/
	vsn_t		t_vsn;
	time_t		t_idle;
};


#endif	/* _VM_PKT_H */
