#ifndef VM_SPACE_H
#define VM_SPACE_H

/*
 * Volume Manager
 *
 *  vm_space.h: generic includes, space and type declarations for all
 *	volman components; space is allocated in vm_p.c
 *
 * $Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/lib/vm_space.h,v 1.1.1.1 2000-02-28 02:18:23 wrstuden Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>

#ifdef AMDAHL_UTS
# include <memory.h>
#endif

#include <string.h>
#include <nasutil.h>

#include "vm_p.h"
#include "vm_pkt.h"
#include "vm_pkt2.h"
#include "vm_ipc.h"
#include "vm_volop.h"
#include "vm_device.h"
#include "vm_host.h"
#include "vm.h"
#include "vm_voltype.h"
#include "vm_volstat.h"
#include "vm_error.h"

extern char	*vHostType;		/* set from Make */
extern char	*VolManOperAcct;	/* set from Make */
extern int	VMError;

#define VOL1LEN		80		/* ANSI label length */

/* from vm_p.c: */
extern char		*vSpoolDir, *vLibDir, *vEtcDir, *vBinDir;
extern char		*vNodeDir, *vClientDir, *vDBDir;
extern char		*vLogFile;
extern char		*volcd_service, *vold_service;
extern ushort		volcd_port, vold_port;
extern int		rp_syslog;
extern char		*vVoldHost;
extern char		*vVoldConfig;

int		CheckVO_Flags(int flags);
void		copytag(char *to, char *from);
void		vInstallError(vmpError_t *ep);
void		vInitDirNames(void);
char		*vVsnNode(char *vsn);
char		*vMsgType(int type);
void		vClearError(void);
Seq_t		vGetSeq(void);
char		*vVolStateStr(int state), *vReqStateStr(int state);
char		*vUseStr(int state), *vPortTypeStr(int type);
char		*vHdrStr(vmpHdr_t *hp), *vDevMapStr(DevMap_t *dmp);
DevMap_t	*vDMand(DevMap_t *map1p, DevMap_t *map2p);
char		*vErrStr(int err);
Id_t		vGetID(void);
char		*vIDFile(Id_t anum);
void		SaveErrs(void);
void		RestoreErrs(void);

/* from vm_{ipc}.c */
vmpHdr_t	*vRecv(IPC_t *sockp, int delay);
vmpHdr_t	*vRcvPkt(IPC_t *sockp, int delay);
vmpHdr_t	*vBuildHdr(int msg, Seq_t seq, vmpHdr_t *chp);
int		vCheckPacket(vmpHdr_t *hp);
void		vSigPipe(int sig);

void		CloseIPC(IPC_t *sp);
int		vSndPkt(vmpHdr_t *hp, IPC_t *sockp);
int		vSend(vmpHdr_t *hp, IPC_t *sockp);
int		vDMffs(DevMap_t *dmp);

/* from vm_voltype.c */
char		*vMediumName(int medium);
void		vListMedia(int logflags, char *id);
void		vPrintVolTypes(FILE *fptr);
int		vMediumCode(char *str);
int		vVolTypeMedium(int number);
char		*vVolTypeID(int number);
char		*vVolTypeName(int number);
VolType_t	*vVolType(int number);
VolType_t	*vVolTypeByID(char *str);

/* from vm_device.c */
char		*vDriveName(int host_type, int controller, 
				int devno, int medium);
DevEnt_t	*vGetDev(DevMap_t *dmp);
DevEnt_t	*vGetDevByNum(int devno);
int		vCountVaryErrors(void);
void		PrintDrives(char *id, char *str);

/* from vm_host.c */
char		*vHostStr(int type);
int		vHostCode(char *str);
void		vPrintHostTypes(int loglev, char *id);
int		vHostTypeConfig(int type);

/* from cl_rc.c */

int		vm_configRC(void);
RC_t		*cRCInfo(int number);
RC_t		*cGetRcp(char *rcname);
char		*cRCId(int number);
void		vm_prRC(FILE *stream, char *offset, 
				char *type, int media);
void		vm_prRCTypes(FILE *stream);

#ifdef lint
extern void		exit();
#endif

#endif /* VM_SPACE_H */
