/*
 * Volume Manager
 *
 * vdr.h: Volume Descriptor Record
 *
 * $Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/lib/vm_vdr.h,v 1.1.1.1 2000-02-28 02:18:23 wrstuden Exp $
 */

#ifndef	_VDR_
#define	_VDR_

/*
 * the VInfo struct is passed up to the RP by the VDM; it's a subset of the VDR
 */
typedef struct VInfo {
	vsn_t		vi_vsn;		/* volume serial number		*/
	int		vi_voltype;	/* key to volume attributes	*/
	int		vi_srcrc;	/* repository controller number	*/
	int		vi_dstrc;	/* repository controller number	*/
	int		vi_uid;		/* volume's user ID		*/
	int		vi_gid;		/* volume's group ID		*/
	int		vi_poolgid;	/* volume's pool group ID	*/
	short		vi_mode;	/* volume's permissions		*/
	vsn_t		vi_label;	/* internal label		*/
} VInfo_t;

#define	DESCLEN		40		/* length of description field	*/

/*
 * the VolDescRec is the struct which is stored in the VDD
 */
typedef struct	VolDescRec {
	VInfo_t	vdr_info;		/* vol info (passed to RP)	*/
	char	vdr_stat;		/* volume status		*/
	char	vdr_use;		/* volume use status		*/
	time_t	vdr_creat;		/* creation date		*/
	time_t	vdr_mod;		/* last modification date	*/
	time_t	vdr_acc;		/* last access date		*/
	time_t	vdr_lastmov;		/* last move			*/
	int	vdr_mntwrite;		/* times mnt writable tried	*/
	int	vdr_mntread;		/* times mnt rdonly tried	*/
	int	vdr_recycle;		/* times recycled		*/
	char	vdr_desc[DESCLEN];	/* volume description		*/
} VDR_t;

/* #defines for ease of access to VInfo struct in VDR_t struct */
#define	vdr_vsn		vdr_info.vi_vsn
#define	vdr_uid		vdr_info.vi_uid
#define	vdr_gid		vdr_info.vi_gid
#define	vdr_mode	vdr_info.vi_mode
#define	vdr_rc		vdr_info.vi_rc
#define	vdr_label	vdr_info.vi_label

/* vdr_use: 	NOTE: these values are used in the volman dbase & for volstat.
 */
#define U_IDLE		'I'		/* not being moved or mounted	*/
#define	U_MOVE		'M'		/* volume in transit		*/
#define	U_MOUNT		'm'		/* mounted or mount in progress	*/
#define U_UNMOUNT	'u'		/* only used for volstat now	*/

/* vdr_stat: 	WARNING: these are the values used in the volman dbase:
 *		changing them here could lead to serious inconsistencies
 *		unless the dbase were updated (IN THE PROPER ORDER) at
 *		the same time.
 *		NOTE - update string tables in vm_p.c and vls.c if
 *		adding new states.
 */
#define S_VIRGIN	'V'		/* new tape waiting for mag label */
#define S_SCRATCH	'S'		/* scratch			*/
#define	S_ALLOC		'A'		/* user-owned volume		*/
#define S_RECYCLE	'R'		/* waiting to be scratched	*/
#define S_RECYCLEBAD	'B'		/* waiting to be scratched	*/
#define S_TRASH		'T'		/* waiting to be discarded	*/
#define S_OBSOLETE	'O'		/* waiting to be sold off	*/
#define S_DELETED	'D'		/* deleted from dbase		*/

#ifdef __cplusplus
extern "C"{
char* vm_vinfo(char* volser);
}
#endif

#endif	/* _VDR_ */
