/*
 * volop.h: #defines for vm_volop(): C interface to Volume Manager
 *	note: "-x" (for various x) below refers to 'x' option in tape(1)
 */

/*
 *	MASK
 *		BITS - within each mask set either singly or as count
 */

#define	VO_COMMAND		0x0000000f	/* action requested          */
#define		VO_MOUNT	0x00000001	/* used for volnode reqs     */
#define		VO_UNMOUNT	0x00000002	/* used for volnode reqs     */
#define		VO_CANCEL	0x00000004	/* used for volnode reqs     */
#define		VO_REQUEUE	0x00000008	/* -r: requeue (new drive)   */

#define VO_WRITE		0x00000010	/* -w: mount for writing     */
#define VO_IGNORE_RING		0x00000020	/* -R: ignore write ring     */
#define	VO_BYPASS		0x00000040	/* -b: skip label processing */
#define VO_HANDOFF		0x00000080	/* delay dismount, & handoff */

#define VO_RESPONSE		0x00000f00
#define 	VO_ASYNC	0x00000100	/* detach proc from req	     */
#define		VO_WAIT		0x00000200	/* proc expects status       */
#define		VO_SIGNAL	0x00000400	/* send a signal when ready  */

#define	VO_SIGMASK		0xff000000	/* signal to snd if VO_SIGNAL */
#define VO_SIGSHFT		24		/* VO_SIGNAL shift            */
/* following used to embed and extract VO_SIGNAL signal from flag word        */
#define	VO_SIGSET(sig)		(((sig) << VO_SIGSHFT) & VO_SIGMASK)
#define	VO_SIGGET(flag)		(((flag) & VO_SIGMASK) >> VO_SIGSHFT)

#define VO_VOLNDMASK		(VO_WRITE|VO_IGNORE_RING|VO_BYPASS)

#define	VO_SCRATCH		0x00001000	/* mount/scratch/dismount    */
#define	VO_EXCLUSIVE		0x00002000	/* make this sole req for vsn*/

#define VO_RECYCLE		0x00004000	/* recycle after unmount     */
#define	VO_RECYCLEBAD		0x00008000	/* bump to recyclebad state  */
#define	VO_TRASH		0x00010000	/* bump to trash state       */

#define	VO_CLEARFIRST		0x00020000	/* unmount drive b4 mount    */
#define	VO_CACHEMOUNT		0x00040000	/* set LCD to '* vsn'	     */

#define	VO_PRIORITY		0x00080000

#define VO_MYHOST		0x00100000	/* mount only on clnt host   */

/* things that need to be equal for simple handoff of mount to another client */
#define VO_SETUP  (VO_WRITE|VO_BYPASS|VO_IGNORE_RING)


/* 
 *  vr_state in RP's VolReq_t and RC->RP msgs 
 *	*** the order is important ***
 *
 *  also used in status msgs from RP->clients
 */
#define VS_NULL			0	/* merely allocated space */
#define VS_IDLE 		1	/* reserved/idle	*/
#define VS_MOUNTWAIT		2	/* mount requested */
#define	VS_DRIVEWAIT		4	/* waiting for drive */
#define VS_RESERVEMOUNTWAIT	5	/* waiting for volume availability */
#define	VS_RCWAIT		6	/* waiting for RC to mount */
#define VS_CREATENODE		7	/* waiting for lbl chck / node setup */
#define	VS_MOUNTED		8	/* volume mounted  & node ready */
#define VS_UNMOUNTWAIT		9	/* about to unmount */
#define VS_DELETENODE		10	/* waiting for node removal */
#define	VS_RCWAIT2		11	/* waiting for RC to unmount */
#define VS_UNMOUNTED		12	/* flag from RC	*/

#define MOUNT_STATES(state) (state >= VS_MOUNTWAIT && state <= VS_MOUNTED)
#define NODE_STATES(state) (state >= VS_CREATENODE && state <= VS_UNMOUNTWAIT)

#define	VS_MOVEWAIT		13	/* waiting to move */
#define	VS_RESERVEMOVEWAIT	14	/* waiting to move */
#define	VS_MOVING		15	/* in a move, no status returned yet  */

#define MOVE_STATES(state) ( (state) >= VS_MOVEWAIT && (state) <= VS_MOVING )

#define BUSY_STATES(state) ( (state) == VS_RCWAIT \
			  || (state) == VS_MOUNTED \
			  || (state) == VS_CREATENODE \
			  || (state) == VS_DELETENODE \
			  || (state) == VS_RCWAIT2 \
			  || (state) == VS_MOVING )

/* following states are used only in RC to RP packets (not in vr_state) */
#define	VS_EJECTED		16	/* vol has been ejected from RC */
#define	VS_ACCEPTED		17	/* vol has been accepted by RC */
#define	VS_1MSGMOVE		18	/* move completed with 1st message */

#ifdef __cplusplus
#include <stdio.h>
extern "C"{
int vm_init(char* id);
int vm_term(void);
int vm_prerr(FILE* str);
int vm_volop(char* volser, int flag, char* label);
int vm_prvolstat(char* volser, FILE* str);
int vm_detach(char* volser);
int vm_move(char* volser, int rcno);
vsn_t *vm_alloc(int rcno, int voltype, char* desc, int mode, int gid);
}
#endif

