/* #define TESTCONFIG */

/*
 *  Volume Manager
 *
 *  vm_voltype.c: volume/device types
 */

#include "vm_space.h"

/*
 *  obsoletetypes is used for marking vsns obsolete on recycle;
 *	hard-coded for now - always needs a 0 for term
 */
int obsoletetypes[] = {
	VT_RND_400,
	0,
};

/*
 *  volume type = medium, i/o buffer size, capacity, 
 *	"wear time" to leave mounted/idle, id/name
 */
VolType_t	VolTypeTab[] = {
  /* map	medium   buf/capacities/flags/wear	id		name	*/
  { VT_RND_400,	VM_RND,	4096,300,400,0,3600,  "RND400","large_rnd_tape 400M", },
  { VT_34X0_600,VM_34X0,4096,400,600,0,1800,  "SQR600",	"3480_600M", },
  { VT_34X0_800,VM_34X0,4096,600,800,0,1800,  "SQR800",	"3480_800M", },
  { VT_D3, 	VM_D3,  4096,30000,50000,0,300,		"D3",	"D3", },
  { VT_MAGSTAR,	VM_MAGSTAR,4096,0,0,0,300,    "MAGSTAR",	"MAGSTAR", },
  { VT_AIT,     VM_AIT, 4096,0,0,0,300,		"AIT",  "AIT", },
  { VT_DLT,     VM_DLT, 4096,0,0,0,300,		"DLT",  "DLT", },
  { -1, 0, 0,0,0,0,0, 0, 0},
};

CodeString_t	vMediaTab[] = {
	{	VM_RND,		"Round_tape"	},
	{	VM_34X0,	"34X0"	},
	{	VM_D2,		"D2"		},
	{	VM_D3,		"D3"		},
	{	VM_MAGSTAR,	"MAGSTAR",	},
	{	VM_AIT,		"AIT",		},
	{	VM_DLT,		"DLT",		},
	{	0,		0			}
};

  char *
vMediumName(int medium)
{
	return(nCodeToStr(medium, vMediaTab));
}

  int
vMediumCode(char *str)
{
	return(nStrToCode(str, vMediaTab));
}

  void
vListMedia(int logflags, char *id)
{
	nListStrings(logflags, id, vMediaTab);
}

  void
vPrintVolTypes(FILE *fptr)
{
	VolType_t	*vtp;

	fprintf(fptr, "\tID\tDescription\n");
	for (vtp=VolTypeTab; vtp->vt_name; vtp++)
		fprintf(fptr, "\t%s\t%s\n", vtp->vt_id, vtp->vt_name);
}

  VolType_t *
vVolType(int number)
{
	VolType_t	*vtp = VolTypeTab;

	for (vtp=VolTypeTab; vtp->vt_name; vtp++)
		if (vtp->vt_map == number)
			return(vtp);
	return(NULL);
}

  int
vVolTypeMedium(int number)
{
	VolType_t	*vtp = vVolType(number);

	if (vtp == NULL)
		return(-1);
	return(vtp->vt_medium);
}

  char	*
vVolTypeID(int number)
{
	char		*unk = "[unknown voltype]";
	VolType_t	*vtp = vVolType(number);

	if (vtp == NULL)
		return(unk);
	return(vtp->vt_id);
}

  char	*
vVolTypeName(int number)
{
	char		*unk = "[unknown voltype]";
	VolType_t	*vtp = vVolType(number);

	if (vtp == NULL)
		return(unk);
	return(vtp->vt_name);
}

  VolType_t *
vVolTypeByID(char *str)
{
	VolType_t	*vtp = VolTypeTab;

	for (vtp=VolTypeTab; vtp->vt_name; vtp++)
		if (!strcmp(str, vtp->vt_id))
			return(vtp);
	return(NULL);
}
