#ifndef lint
static char rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/admin/rcerr.c,v 1.1.1.1 2000-02-28 02:18:27 wrstuden Exp $";
#endif
/*
 *  Volume Manager
 *
 *  rcerr.c - operator error interface to Repository Controllers.
 *	
 *
 *  Bill Ross, Sterling Software for NAS/NASA Ames, 8/22/89
 */


#ifdef EXTERN
# undef EXTERN
#endif
#define EXTERN
#include "srv_space.h"

#include <pwd.h>
#include <ctype.h>

static char	*arg0;

usage(str)
{
	fprintf(stderr, "usage: %s VSN ERROR\n", arg0);
	fprintf(stderr, "	ERROR	reason\n");
	fprintf(stderr, "	  1	VSN not found\n");
	fprintf(stderr, "	  2	problem with drive\n");
	fprintf(stderr, "%s\n", str);
	RPCLGoodbye();
	exit(1);
}

char *ErrMsg[] = {
	/* 0 */ "",
	/* 1 */ "operator can't find volume in repository",
	/* 2 */ "drive problem, try again",
	""
};


main(argc, argv)
  char	*argv[];
{
	int		operr, err, uid, flag;

	arg0 = argv[0];
	CheckUID(arg0);

	/*
	 *  set up connection with vold
	 */
	flag = 0;
	RPCLInit(M_CONSOLE, &flag);

	/*
	 *  check arguments
	 */

	if (argc != 3)
		usage("wrong # of arguments");

	if (checkvsn(argv[1]))
		usage("bad vsn");

	operr = atoi(argv[2]);
	if (operr<1 || operr>2)
		usage("unknown error field");

	if (err = RcMsg(argv[1], operr))
		vm_prerr(stderr);
	else 
		err = GetResponse();
	RPCLGoodbye();
	exit(err);
}

/*
 *  RcMsg():
 */
RcMsg(vsn, operr)
 char	*vsn;
{
	vmpHdr_t        *hp, *ihp;
	vmpRcMsg_t	*mp;
	vmpError_t	*ep;

	/* 
  	 *  the rcmsg is a message within a message:
	 *
	 *	rcmsg:		
	 *	  header	  (hp)
	 *	  body:		  (mp)
	 *	    rc #
	 *	    error pkt:
	 *	      header	  (ihp)
	 *	      body	  (ep)
	 */

	hp = vBuildHdr(M_RCMSG, NOSEQ, NULL);
	mp = MSGPTR(hp, vmpRcMsg_t);
	mp->rm_rc = -1;
	mp->rm_flag = RCM_VSN;
	strncpy((char *)&mp->rm_vsn, vsn, VSNLEN);
	ihp = (vmpHdr_t *) mp->rm_msg;
	ihp->vh_magic = VMP_MAGIC;
	ihp->vh_type = M_VERROR;
	ihp->vh_size = sizeof(vmpError_t);
	ihp->vh_seq = NOSEQ;
	ihp->vh_key = 0;
	ep = MSGPTR(ihp, vmpError_t);
	strncpy((char *)&ep->vn_vsn, vsn, VSNLEN);
	sprintf(ep->vn_msg, "%.*s: %s", VSNLEN, vsn, ErrMsg[operr]);
	if (operr == 1)
		ep->vn_error = E_CANCELED;
	if (operr == 2)
		ep->vn_error = E_UNAVAIL;
	RPCLSend(hp, 1);
	return(VMError);
}

GetResponse()
{
	vmpHdr_t	*rhp;
	vmpVsn_t	*vsp;
	vmpError_t	*ep;

	for (;;) {
		/*
		 *  since no select on rcv, can ignore RPCLNextRecord
		 *	return value
		 */
		rhp = RPCLRecv("getting response");
		RPCLNextRecord();

		switch (rhp->vh_type) {
		    case M_ACK:
			return(0);
		    case M_VERROR: 
			ep = MSGPTR(rhp, vmpError_t);
			printf("error from voldaemon: %s\n",ep->vn_msg);
			return(1);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			exit(1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			printf("Unexpected packet type: %s\n", 
							vHdrStr(rhp));
			exit(1);
		}
	}
}
