/*
 *  vls command -- user interface to get info on volumes in the
 *	volman(8) database. It is a console cmd because the vold
 *	forks to answer long requests, so a unique connection is
 *	desirable so that volcd doesn't get msgs from both copies
 *	of vold, resulting in msgs back to vold waiting on wrong
 *	one to read. Also, direct conn speeds up long responses..
 *
 *
 *	Bill Ross, NASA Ames, 3/92
 *	Converted to volman console util, 10/96	-BR
 */

#include <stdio.h>
#include <ctype.h> 
#include <time.h> 
#include <pwd.h>
#include <grp.h>
#include <bsdtime.h>

#ifdef EXTERN
# undef EXTERN
#endif
#define EXTERN
#include "srv_space.h"

extern int errno;

static	vmpHdr_t	*rhp = NULL;
static	VDR_t		*vdr;
static	VInfo_t		*vin;
static	char		*arg0;

/*
 * Print out usage message and exit.
 */

usage(char *str)
{
	fprintf(stderr, "\n*** %s\n\n", str);
	fprintf(stderr,
	"usage: %s [-{l|L|C}] [-{i|I}] [-u user] [-r rc] [-m medium] \\\n", 
									arg0);
	fprintf(stderr, "\t\t\t\t[-s status] [-s status2] \\\n");
	fprintf(stderr, "\t\t\t\t[volsers.. | -f file | -f -]\n");
	vm_prRC(stderr, "", NULL, 1);
	fprintf(stderr, "\n-s:\tStatus\tDescription\n");
	vPrintStatTab(stderr);
	exit(1);
}

bomb()
{
	RPCLGoodbye();
	exit(1);
}

domode(short mode, char *name)
{
	int i;
	char *ch = "xwr";

	for (i=0; i<9; i++)
		if (1<<i & mode)
			name[8-i] = ch[i%3];
		else 
			name[8-i] = '-';
	name[9] = '\0';
}

int
username(int uid, char *name)
{
	struct passwd *pwd;

	pwd = getpwuid(uid);
	if (pwd == NULL) {
		sprintf(name, "%d", uid);
		return(0);
	} else {
		sprintf(name, "%s", pwd->pw_name);
		return(1);
	}
}

dogrp(int gid, char *name)
{
	struct group *grp;

	grp = getgrgid(gid);
	if (grp == NULL)
		sprintf(name, "%d", gid);
	else
		sprintf(name, "%s", grp->gr_name);
}


main(int argc, char *argv[])
{
	extern  int     optind;
	extern  char    *optarg;
	FILE		*fp = NULL;
	int		med, vuid, cc, status, flag;
	int		col = 0, columns = 0;
	int		info = 0, allinfo = 0;
	int		volspec = 0, found = 0;
	int		uidopt = 0, infostring = 0;
	int		u_rcnum;
	int		maxstat = 30;	/* hacking */
	char		uname[15], gname[15], mod[25], today[25], mode[10];
	char		*rcname, *rcshort;
	RC_t		*rcp;
	char		pbuf[PKTSIZ(vmpVls_t)];
	vmpHdr_t	*shp;
	vmpVls_t	*vp;

	arg0 = argv[0];
	vuid = getuid();

	nLogInit(arg0, bomb);
	nLogInitStderr(EL_ERROR1|EL_STATUS1);

	/*
	 *  connect to vold as console & get RC config
	 */
	flag = 0;
	RPCLInit(M_CONSOLE, &flag);
	RPCLRCConfig();

	/*
	 *  build msg w/ default search (uid)
	 */
	shp = vBuildHdr(M_VLS, vGetSeq(), (vmpHdr_t *) pbuf);
	vp = MSGPTR(shp, vmpVls_t);
	memset(&vp->l_vsn, 0, sizeof(vp->l_vsn));
	vp->l_uid = vuid;
	vp->l_rc1 = vp->l_rc2 = vp->l_rc3 = 0;
	vp->l_media = 0;
	vp->l_status = 0;

	/*
	 *  process args
	 */
	while ((cc = getopt(argc, argv, "ClLiIr:m:s:u:f:")) != EOF) {
		switch (cc) {

		    /*
		     *  output format
		     */
		    case 'C':
			columns++;
			break;
		    case 'l':
			info++;
			break;
		    case 'L':
			info++;
			allinfo++;
			break;
		    case 'i':
			infostring = 1;
			break;
		    case 'I':
			infostring = 2;
			break;

		    /*
		     *  selection criteria
		     */
		    case 'r':
			rcp = cGetRcp(optarg);
			if (rcp == NULL)
				usage("unknown RC");
			if (vp->l_rc1 == 0)
				vp->l_rc1 = rcp->rcd_number;
			else if (vp->l_rc2 == 0)
				vp->l_rc2 = rcp->rcd_number;
			else if (vp->l_rc3 == 0)
				vp->l_rc3 = rcp->rcd_number;
			else {
				fprintf(stderr, "limit of 3 -r RC args\n");
				exit(1);
			}
			break;
		    case 'm':
			med = vMediumCode(optarg);
			if (med == -1) {
				fprintf(stderr,"unknown medium: %s\n", optarg);
				usage("");
			}
			vp->l_media |= med;
			break;
		    case 's':
			status = vStatusBit(optarg);
			if (!status) {
				fprintf(stderr,"unknown status: %s\n", optarg);
				usage("");
			}
			vp->l_status |= status;
			break;
		    case 'u':
			if (vm_userid(optarg, &vp->l_uid)) {
				fprintf(stderr, "%s: user unknown\n", optarg);
				exit(1);
			}
			break;
		    case 'f':
			if (fp != NULL) 
				usage("only one form of input allowed");
			if (!strcmp(optarg, "-"))
				fp = stdin;
			else 
				fp = fopen(optarg, "r");
			if ( !fp ) {
				perror(optarg);
				exit(1);
			}
			volspec = 1;
			break;
		    case '?':
		    default:
			usage("");
		}
	}
	if (info  ||  infostring)
		columns = 0;

	if (optind < argc) {

		/*
		 *  remaining args should be vsns
		 */
		if (fp)
			usage("only one form of input allowed");
		volspec = 1;
	}

	if (!volspec) {

		/*
		 *  send 1 msg w/ the uid
		 */

		RPCLSend(shp, 1);
	}
	for(;;) {
		if (volspec) {

			/*
			 *  get next vsn
			 */
			if ( fp ) {
				char	vsn[VSNLEN+1];
				if (vm_getvsnfromfile(fp, vsn) == EOF)
					break;
				memcpy(&vp->l_vsn, vsn, VSNLEN);
			} else {
				if (optind >= argc)
					break;
				if (checkvsn(argv[optind])) {
					optind++;
					continue;
				} 
				memcpy(&vp->l_vsn, argv[optind], VSNLEN);
			}

			/*
			 *  send msg w/ the vsn
			 */
			shp->vh_seq++;
			RPCLSend(shp, 1);
			optind++;
		}

		/*
		 *  receive the next msg (vsn info or end)
		 */
		if (GetResponse(&vp->l_vsn, shp->vh_seq)) {
			if (VMError != E_USER  || VMError == E_NOERROR)
				break;
			if (volspec)
				printf("%.*s: not found\n", VSNLEN, &vp->l_vsn);
		}
		if (vdr == NULL)	/* not found */
			continue;

		/*
		 *  print vsn info
		 */
		found++;

		if (!info) {
			/*
			 * minimal info
			 */
			printf("%.*s ", VSNLEN, &vin->vi_vsn);
			if (infostring)
				printf("%.*s", DESCLEN, vdr->vdr_desc);
			if (!columns 
			    || (columns  &&  (col = (col + 1)%10)  == 0))
				printf("\n");
			continue;
		}

		/*
		 *  format basic stuff for -l or -L
		 */
		domode(vin->vi_mode, mode);
		username(vin->vi_uid, uname);
		dogrp(vin->vi_gid, gname);

		rcp = cRCInfo(vin->vi_srcrc);
		if (rcp != NULL) {
			rcname = rcp->rcd_name;
			rcshort = rcp->rcd_id;
		} else {
			rcname = "?RC?";
			rcshort = "?RC?";
		}

		if (allinfo) {
			time_t	thetime;

			/*
			 *  total info: -L
			 */

			VolType_t	*vtp = vVolType(vin->vi_voltype);

			printf("%.*s:\n", VSNLEN, &vin->vi_vsn);
			printf("\tuid %d  (%s)   gid %d  (%s)\n",
				vin->vi_uid, uname, vin->vi_gid, gname);
			printf("\tmode %o  label (%.*s)  rc %s\n",
					vin->vi_mode, VSNLEN, &vin->vi_label, 
					rcname);
			printf("\ttype %s\n", (vtp ? vtp->vt_name : "Unknown"));
			thetime = vdr->vdr_creat;
			printf("\tcreat \t%s", vdr->vdr_creat > -1 ?
				    bsd_ctime(&thetime) : "not set\n");
			thetime = vdr->vdr_sttim;
			printf("\tstat  \t%s", vdr->vdr_sttim> -1 ?
				    bsd_ctime(&thetime) : "not set\n");
			thetime = vdr->vdr_mod;
			printf("\tmod \t%s", vdr->vdr_mod > -1 ?
					bsd_ctime(&thetime) : "not set\n");
			thetime = vdr->vdr_acc;
			printf("\taccess \t%s", vdr->vdr_acc > -1 ?
					bsd_ctime(&thetime) : "not set\n");
			thetime = vdr->vdr_lastmov;
			printf("\tmove \t%s", vdr->vdr_lastmov > -1 ?
				bsd_ctime(&thetime) : "not set\n");
			printf("\tmounted: read %-6d  write %-6d",
					vdr->vdr_mntread, vdr->vdr_mntwrite);
			printf("\tmount failures in a row: %d\n", 
					vdr->vdr_mntfail);
			printf("\tuse:  ");
			switch(vdr->vdr_use) {
			    case U_IDLE:
				printf("not being moved or mounted\n");
				break;
			    case U_MOVE:
				printf("volume in transit to ");
				rcp = cRCInfo(vin->vi_dstrc); 
				if (rcp != NULL)
					printf(" %s\n", rcp->rcd_name);
				else
					printf("undefined (??) rc %d\n", 
							vin->vi_dstrc);
				break;
			    case U_MOUNT:
				printf("mounted or mount in progress\n");
				break;
			    case U_SCRATCH:
				printf("mount to convert->scratch\n");
				break;
			    default:
				printf("undefined %d (?)\n",vdr->vdr_use);
			}
			printf("\tstate:");
			switch(vdr->vdr_status) {
			    case S_VIRGIN:
				printf("\tnew unlabeled tape\n");
				break;
			    case S_SCRATCH:
				printf("\tscratch\n");
				break;
			    case S_ALLOC:
				printf("\tuser-owned volume\n");
				break;
			    case S_RECYCLE:
				printf("\twaiting to be recycled\n");
				break;
			    case S_TRASH:
				printf("\twaiting to be discarded\n");
				break;
			    case S_OBSOLETE:
				printf("\twaiting to be sold off\n");
			    default:
				printf("\tundefined %d (?)\n", vdr->vdr_status);
			}
			printf("\tuser description: [%.*s]\n", 
					DESCLEN, vdr->vdr_desc);
			continue;
		}

		if (infostring == 1) {
			/*
			 *  replacing date field w/ equiv len from desc
			 */
			strncpy(mod, vdr->vdr_desc, 20);
		} else {
			/*
			 *  get ls -l convention date string: if year 
			 *	(mod[20..23]) is not same as today, 
			 *	write it on time field (mod[11])
			 */
			time_t thetime;
			if (vdr->vdr_mod != -1)
				thetime = vdr->vdr_mod;
			else	/* never touched - use create time */
				thetime = vdr->vdr_creat;
			strncpy(mod, bsd_ctime(&thetime), 25);
			thetime = time((time_t *) 0);
			vdr->vdr_mod = thetime;
			strncpy(today, bsd_ctime(&thetime), 25);
			if (strncmp(&mod[20], &today[20], 4) != 0) 
				sprintf(&mod[11], "%.4s    ", &mod[20]);
		}
		mod[19] = '\0';

		/*
		 *  'ls -l' style info
		 */
		printf("%s  %-10.8s%-7.6s%-21.20s%-9.10s%-12.10s%.6s\n", 
		    mode, uname, gname, mod, rcshort,
		    vVolStateStr(vdr->vdr_status), &vin->vi_vsn);
		if (infostring == 2)
			printf("\tuser description: [%.*s]\n", 
					DESCLEN, vdr->vdr_desc);
	}
	if (columns  &&  (col = (col + 1)%10)  == 0)
		printf("\n");

	RPCLGoodbye();
	exit(!found);
}

 int
GetResponse(char *vol, Seq_t seq)
{
	char		*id = "GetResponse";
	vmpVdr_t	*vsp;
	vmpError_t	*ep;

	/*
	 *  free any pkt from previous msg rcvd
	 */

	if (rhp != NULL)
		nFree(rhp);

	while (1) {
		rhp = RPCLRecv("get resp");
		RPCLNextRecord();
		if (seq != rhp->vh_seq  &&  rhp->vh_type != M_SHUTDOWN) {
			fprintf(stderr, "bad packet sequence (skipping): %s", 
								vHdrStr(rhp));
			continue;
		}
		switch (rhp->vh_type) {
		    case M_VDR:
			vsp = MSGPTR(rhp, vmpVdr_t);
			vdr = &vsp->vv_vdr;
			vin = &vdr->vdr_info;
			if (vol[0] &&
			  strncmp((char *) &vin->vi_vsn, vol, VSNLEN)) {
				printf("%.*s: wrong vsn returned\n",
							VSNLEN,	&vin->vi_vsn);
				exit(1);
			}
			return(0);
		    case M_VERROR: 

			/*
			 *  E_USER = vsn does not exist
			 */

			ep = MSGPTR(rhp, vmpError_t);
			if (ep->vn_error != E_USER) {
				printf("error from vold: %s\n", ep->vn_msg);
				exit(1);
			}
			VMError = E_USER;
			vdr = NULL;
			return(1);
		    case M_ACK: 

			/*
			 *  last vsn sent
			 */
			return(1);
		    case M_SHUTDOWN:
			printf("shutdown received\n");
			vm_term();
			exit(1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			printf("Unexpected packet: %s\n", vHdrStr(rhp));
			exit(1);
		}
	}
}
