
/*
 *  Volume Manager
 *
 *  rl_sock.c - server socket routines for vold, volcd, volnode
 */

#include "rl_space.h"

#include <time.h>
#include <sys/stat.h>

extern int 		errno;

/*
 *  array of sockets to index by file descriptor on select()
 */
IPC_t			SockArray[FD_SETSIZE];

static socklen_t	socksize = sizeof( struct sockaddr_in );


/*
 *  IPCStartListening() - Open a 'well-known' socket to accept connections
 */
 int
IPCStartListening(u_long port)
{
	char			*id = "IPCStartListening";
	struct sockaddr_in 	ip_addr;
	int 			listen_sd, i, on;

	/*
	 *  create master socket
	 */
	listen_sd = socket(AF_INET, SOCK_STREAM, 0);
	if (listen_sd == -1)
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "socket() failed");
	nLogMsg(EL_DEBUG1, id, "listen_sd %d", listen_sd);

	/*
	 *  set socket to recycle quickly .. SO_LINGER too?
	 */
	on = 1;
	if (setsockopt(listen_sd, SOL_SOCKET, SO_REUSEADDR, 
					(char *)&on, sizeof(int)) == -1) {
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "setsockopt() failed");
	}

	ip_addr.sin_addr.s_addr = htonl( INADDR_ANY );
	ip_addr.sin_family = AF_INET;
	ip_addr.sin_port = htons(port);

	for (i=1; i<16; i++) {
	    if ((bind(listen_sd, (struct sockaddr *)&ip_addr, socksize)) ==-1) {
		if (errno == EADDRINUSE) {
		    nLogMsg(EL_TOSTDERR|EL_STATUS2, id,
			"Main port address already in use - retrying");
		    sleep(5*i);
		    continue;
		}
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "bind() failed");
	    }
	    break;
	}
	if (i == 16)
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, 
					"giving up - wait & retry later");
	if (i != 1)
		nLogMsg(EL_TOSTDERR|EL_STATUS2, id, "Main port address OK");

	if (listen(listen_sd, 5) == -1)
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "listen() failed");
	return(listen_sd);
}

/*
 *  IPCStopConn() - stop accepting new connections
 */
  void
IPCStopConn(int sd)
{
	char	*id = "IPCStopConn(socket)";

	if (close(sd) == -1)
		nLogMsg(EL_SYS|EL_ERROR1, id, "closing sock");
}

  int
IPCClose(int fd)
{
	char	*id = "IPCClose";

	if (shutdown(fd, 2) == -1) 
		nLogMsg(EL_SYS|EL_CONSOLE0, id, "socket shutdown");
	if (close(fd) == -1) {
		nLogMsg(EL_SYS|EL_CONSOLE0, id, "socket close");
		return(1);
	}
	return(0);
}
