
/*
 *  Volume Manager
 *
 *  rpcl_xdr.c - comm for rp 'root clients' (volcd & volnode)
 *
 *	Note: XDR comms use the SendKey, ReceiveKey instead of the
 *	sock keys.
 *
 */

#include "srv_space.h"

#include <netdb.h>
#include <time.h>
#include <setjmp.h>
#include <rpc/xdr.h>

#define RETRYSLEEP	30

extern int		errno;

extern DevEnt_t		*DriveArray;
extern int		DriveTotal;

extern RC_t		*RCArray;
extern int		RCTotal;

/*
 *  immortality in case of vold termination is available
 *	to users of this communications lib
 */
int			immortal = 0;
jmp_buf			jmpbuf_i;

static int		vold_sd = -1;
static XDR		RPxdr;
static IPC_t		RPipc;	

static Id_t		SendKey, ReceiveKey;


static socklen_t	len = sizeof( struct sockaddr_in );

 void
RPCLConnect()
{
	char			*id = "RPCLConnect";
	struct hostent 		*htp;
	struct sockaddr_in	ip_addr;
	struct servent		*servent;
	int			retry;

	nLogMsg(EL_STATUS1, 0, "(vold on host %s)", vVoldHost);

	if ((servent = getservbyname(vold_service, "tcp")) == NULL)
		nLogMsg(EL_FATAL|EL_SYS, id, "getservbyname");
	vold_port = ntohs(servent->s_port);

        for (retry=0; ; retry++) {

		(void) sleep((unsigned int)(retry ? RETRYSLEEP : 1));

		htp = nGetHostInfo(vVoldHost);

		vold_sd = socket(AF_INET, SOCK_STREAM, 0);
		if (vold_sd == -1)
			nLogMsg(EL_FATAL|EL_SYS, id, "socket");

		memcpy(&ip_addr.sin_addr, htp->h_addr, (size_t)htp->h_length);
		nLogMsg(EL_DEBUG1, id, "vold host family %s",
						nIPCFamily(htp->h_addrtype));
		ip_addr.sin_family = htp->h_addrtype;
		ip_addr.sin_port = htons(vold_port);

		if (connect(vold_sd, (struct sockaddr *)&ip_addr, len) == 0)
			break;

		/*  connect failed */
		switch (errno) {
		    case ETIMEDOUT:
			nLogMsg(EL_STATUS1, 0, "timeout on connect to vold");
			break;
		    case EADDRINUSE:
			/* 
			 *  socket synchronization error? 
			 *	occurs infrequently.. retry
			 */
			break;
		    default:
			nLogMsg(EL_SYS, 0, 
				"vold probably down: connect system err");
			break;
		}
		close(vold_sd);
		vold_sd = -1;
		nLogMsg(EL_STATUS1, id, "retrying...");
	}
	RPipc.sock_fd = vold_sd;
	XDRSetup(&RPipc, &RPxdr);
}

 void
RPCLSend(vmpHdr_t *hp, int now)
{
	char	*id = "send to vold";

	hp->vh_key = SendKey;
	if (XDRSend(hp, &RPxdr, now)) {
		if (immortal) {
			nLogMsg(EL_STATUS1, id, "longjmp");
			longjmp(jmpbuf_i, 1);
			nLogMsg(EL_FATAL|EL_SYS, id, "longjmp");
		}
		nLogMsg(EL_FATAL, id, "(%s)", 
					vMsgType(hp->vh_type));
	}
}

/*
 *  FUNCTION:	RPCLInit()
 *		Initialize a client connection to the RP server.
 */
RPCLInit(int rpcl_type, int *flag)
{
	char			*id = "RPCLInit";
	char			hbuf[PKTSIZ(vmpHello_t)];
	vmpHdr_t		*hp, *rhp;
	vmpHello_t		*hellop;

	RPCLConnect();

	/*
	 *  build and send M_{RPCLIENT} message
	 */
	SendKey = ReceiveKey = vGetID();
	hp = vBuildHdr(rpcl_type, NOSEQ, (vmpHdr_t *)hbuf);
	hp->vh_slot = -1;
	hellop = MSGPTR(hp, vmpHello_t);
	hellop->vhe_version = VM_VERSION;
	hellop->vhe_flag = *flag;
	hellop->vhe_pid = getpid();

	RPCLSend(hp, 1);

	nLogMsg(EL_DEBUG2, id, "sent %s", vMsgType(rpcl_type));

	/*
	 *  get reply, should be same packet
	 */
	rhp = RPCLRecv(vMsgType(rpcl_type));
	if (rhp->vh_slot != -1)
		nLogMsg(EL_FATAL, id, "bad slot (%d) / %s", 
						rhp->vh_slot, vHdrStr(rhp));
	if (rhp->vh_type != rpcl_type)
		nLogMsg(EL_FATAL, id, "got wrong response %s", vHdrStr(rhp));

	hellop = MSGPTR(rhp, vmpHello_t);
	SendKey = hellop->vhe_id;
	*flag = hellop->vhe_flag;
	nLogMsg(EL_STATUS2, id, "vold connection completed");
	return(RPCLNextRecord());
}

RPCLfd()
{
	return(vold_sd);
}

 void
RPCLGoodbye()
{
	vmpHdr_t	hdr;

	if (vold_sd == -1)
		return;

	(void) vBuildHdr(M_GOODBYE, NOSEQ, &hdr);
	RPCLSend(&hdr, 1);
}

/*
 *  RPCLDriveConfig() - get drive info from vold
 */
RPCLDriveConfig()
{
	char		*id = "RPCLDriveConfig";
	vmpHdr_t	*hp, hdr;

	vBuildHdr(M_DEVQRY, NOSEQ, &hdr);

	RPCLSend(&hdr, 1);

	hp = RPCLRecv("getting M_DEVQRY");

	DriveTotal = hp->vh_count;
	if (DriveTotal < 0  ||  DriveTotal > MAXDEVS)
		nLogMsg(EL_FATAL, id, "DriveTotal (%d) must be in 0..%d",
			DriveTotal, MAXDEVS);

	DriveArray = MSGPTR(hp, DevEnt_t);

	nLogMsg(EL_DEBUG1, id, "done");
	return(RPCLNextRecord());
}

 void
RPCLSendNop()
{
	char		*id = "send no-op to vold";
	vmpHdr_t	hdr;

	hdr.vh_type = M_NOP;
	if (XDRSend(&hdr, &RPxdr, 1)) {
		if (immortal) {
			nLogMsg(EL_STATUS1, id, "longjmp");
			longjmp(jmpbuf_i, 1);
			nLogMsg(EL_FATAL|EL_SYS, id, "longjmp");
		}
		nLogMsg(EL_FATAL, id, "failed");
	}
}

 void
RPCLErr2RP(vsn_t *vsnp, Seq_t seq, int state, int flag, char retry)
{
	vmpHdr_t	*hp;
	vmpError_t	*ep;
	char		ebuf[PKTSIZ(vmpError_t)];

	hp = vBuildHdr(M_VERROR, seq, (vmpHdr_t *)ebuf);
	ep = MSGPTR(hp, vmpError_t);
	if (vsnp != NULL)
		ep->vn_vsn = *vsnp;
	ep->vn_error = VMError;
	ep->vn_state = state;
	ep->vn_flag = flag;
	ep->vn_retry = retry;
	emsgcpy(ep->vn_msg, nErrStr);
	RPCLSend(hp, 1);
	nLogMsg(EL_DEBUG3, 0, "srv_ sent err");
}

  vmpHdr_t  *
RPCLRecv(char *str)
{
	char		*id = "RPCLRecv";
	vmpHdr_t	*hp;

	hp = XDRRecv(&RPxdr);
	if (hp == NULL) {
		if (errno == EINTR) {	/* volnd gets sigcld */
			nLogMsg(EL_DEBUG1, id, "--recursive call on EINTR");
			return(RPCLRecv(str));
		}
		if (immortal) {
			nLogMsg(EL_STATUS1, id, "longjmp on vold closing conn");
			longjmp(jmpbuf_i, 1);
			nLogMsg(EL_FATAL|EL_SYS, id, "longjmp");
		}
		nLogMsg(EL_SYS|EL_FATAL, id, 
			"%s: vold died or closed connection", str);
	}

	if (hp->vh_key != ReceiveKey)
		nLogMsg(EL_FATAL, id, 
			"%s: rcvd vold packet w/ bad key (%s)",
			str, vHdrStr(hp));
	return(hp);
}

RPCLNextRecord()
{
	return(XDRNextRecord(&RPxdr));
}

 void
RPCLVaryOff(int drive, char *msg)
{
	char		mbuf[PKTSIZ(vmpVary_t)];
	vmpHdr_t        *hp;
	vmpVary_t	*vvp;

	nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, "%s", msg);
	hp = (vmpHdr_t *) mbuf;
	hp->vh_size = sizeof(vmpVary_t);
	hp->vh_magic = VMP_MAGIC;
	hp->vh_type = M_VARY;
	hp->vh_seq = NOSEQ;

	vvp = MSGPTR(hp, vmpVary_t);
	DM_ZERO(vvp->vva_dev);
	DM_SET(drive, vvp->vva_dev);
	vvp->vva_flags = VVA_VARYOFF;
	if (msg == NULL)
		strncpy(vvp->vva_err, "[no msg!]", sizeof(vvp->vva_err));
	else
		strncpy(vvp->vva_err, msg, sizeof(vvp->vva_err));
	RPCLSend(hp, 1);
}

RPCLRCConfig()
{
	char		*id = "RPCLRCConfig";
	vmpHdr_t	hdr;
	vmpHdr_t	*hp;
	vmpError_t	*ep;
	int		ret;

	hdr.vh_type = M_RCINFO;
	hdr.vh_magic = VMP_MAGIC;
	hdr.vh_seq = NOSEQ;
	hdr.vh_size = 0;
	hdr.vh_count = 0;
	hdr.vh_slot = -1;

	RPCLSend(&hdr, 1);

	for (;;) {

		/*
		 *  get response
		 */

		hp = RPCLRecv(id);
		ret = RPCLNextRecord();

		if (hp->vh_seq != hdr.vh_seq  &&  hp->vh_type != M_SHUTDOWN) {
			nLogMsg(EL_ERROR2, id, 
					"bad packet sequence (skipping): %s", 
					vMsgType(hp->vh_type));
		}
		switch (hp->vh_type) {
		    case M_RCINFO:
			/*
			 *  Use the packet itself for the
			 *	config, ignoring the header.
			 *	If the config ever became
			 *	dynamic, then we'd want to
			 *	copy & then free the pkt.
			 */
			RCTotal = hp->vh_count;
			RCArray = MSGPTR(hp, RC_t);
			return(ret);
			break;		/* for lint */
		    case M_TIMER:
		    case M_NOP:
			break;
		    case M_VERROR: 
			ep = MSGPTR(hp, vmpError_t);
			nLogMsg(EL_FATAL, id, "%s", ep->vn_msg);
			break;		/* for lint */
		    case M_SHUTDOWN:
			if (immortal) {
				nLogMsg(EL_STATUS1, id, "longjmp on shutdown");
				longjmp(jmpbuf_i, 1);
				nLogMsg(EL_FATAL|EL_SYS, id, "longjmp");
			}
			nLogMsg(EL_FATAL, id, "vold shutdown");
			break;		/* for lint */
		    default:
			nLogMsg(EL_FATAL, id, "Unexpected packet: %s", 
						vMsgType(hp->vh_type));
		}
	}
}

 void
MMDebug(vmpHdr_t *hp)
{
	char		*id = "MMDebug";
	vmpDebug_t	*dp;

	dp = MSGPTR(hp, vmpDebug_t);

	if (dp->vd_cmd != 0) {
		nLogMsg(EL_ERROR1, id, "0x%x: unknown debug cmd", dp->vd_cmd);
		return;
	}
	if (dp->vd_level < 0  ||  dp->vd_level > EL_DEBUG4) {
		nLogMsg(EL_ERROR1, id, "0x%x: unknown log level", dp->vd_level);
		return;
	}
	if (dp->vd_level & ~EL_LEVEL) {
		nLogMsg(EL_ERROR1, id, "0x%x: non-loglevel bit(s)", 
								dp->vd_level);
		return;
	}
	nLogSetLevel(dp->vd_level);
	nLogMsg(EL_CONSOLE1|EL_CONSOLE0|EL_STATUS1, id, 
				"setting log level to 0x%x", dp->vd_level);
}
