/*
 *  dbtrans.c - translate older ascii vsntable to newer one
 *	with fields for consec mount errs and converting
 *	dbase times from ctime() strings to hex 64-bit
 *	Unix time (forseeing expansion of time_t to 64 bits).
 *
 *
 *  A 1-time program. Usage:  dbtrans < old_tbl > new_tbl
 *
 */

/*****************************************************************
		NEW DBASE VSN TABLE
 *****************************************************************/
#include "rp_space.h"
#include "rp_db.h"


/*****************************************************************
		OLD DBASE VSN TABLE
 *****************************************************************/
#include <time.h>


#define DATELEN		16	/* 64 bits in hex - good for millenia */
#define INTLEN		5
#define BIGINTLEN	10

typedef struct oldvsnrec {
	char	v_vsn[VSNLEN];		char	b01;
	char	v_label[VSNLEN]; 	char	b02;
	char	v_status; 		char	b03;
	char	v_use; 			char	b04;
	char	v_srcrc[INTLEN];	char	b05;
	char	v_dstrc[INTLEN]; 	char	b06;
	char	v_voltype[INTLEN]; 	char	b07;
	char	v_mntwrite[INTLEN]; 	char	b08;
	char	v_mntread[INTLEN]; 	char	b09;
	char	v_mntfail[INTLEN]; 	char	b10;
	char	v_recycle[INTLEN]; 	char	b11;
	char	v_uid[BIGINTLEN]; 	char	b12;
	char	v_gid[BIGINTLEN]; 	char	b13;
	char	v_poolgid[BIGINTLEN]; 	char	b14;
	char	v_mode[3]; 		char	b15;
	char	v_creat[DATELEN]; 	char	b16;
	char	v_atime[DATELEN]; 	char	b17;
	char	v_mtime[DATELEN]; 	char	b18;
	char	v_movtm[DATELEN]; 	char	b19;
	char	v_desc[40];		char	b20;
	char	v_buf[49];		char	end;	/* fill to 256 */
} oldvbuf;

oldvbuf	vbuf1;
vbuf	vbuf2;


static int 	dmsize[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

static char 	months[12][4] = {
			"Jan","Feb","Mar","Apr", "May","Jun",
			"Jul","Aug", "Sep","Oct","Nov","Dec" };



 static int
ToInt(nump, str, size)
 int	*nump;
 char	*str;
 int	size;
{
	char	*id = "db to integer";
	int     i;

	/*
	 *  boundary char should be space
	 */
	if (str[size] != ' ') {
		fprintf(stderr, "int field discrepancy (%.*s)\n",
							size+1, str);
		return(1);
	}

	/*
	 *  skip leading space
	 */

	for (i=0; i<size && str[i] == ' '; i++);

	/*
	 *  if all spaces, number is 0
	 */

	if (i == size) {
		*nump = 0;
		return(0);
	}

	/*
	 *  leading '-' ok
	 */

	if (str[i] == '-') {
		if (i == size-1) {
			fprintf(stderr, "leading '-' w/ no # (%.*s)\n",
							size+1, str);
			return(1);
		}
		i++;
	}

	/*
	 *  rest should be numerals
	 */
	for (; i<size; i++)
		if (!isdigit(str[i])) {
			fprintf(stderr,
				"int field (%.*s) has non-numeral\n",size, str);
			return(1);
		}
	*nump = atoi(str);
	return(0);
}

 int
SetInt(str, num, size)
 char   *str;
 int    num;
 int    size;
{
        char    arr[32];
        int     len;

        sprintf(arr, "%d", num);
        len = strlen(arr);

        if (len > size) {
                fprintf(stderr, 
			"setting integer %d: int len %d > reserved %d\n",
                                                num, len, size);
                return(1);
        }
        sprintf(str, "%*d", size, num);
        str[size] = ' ';
        return(0);
}

 static int
modeto(num)
 short  *num;
{
        char    *str = &vbuf1.v_mode[0];
        int     i;

        *num = 0;
        for (i=0; i<3; i++) {
                if (str[i] == '0' ||  str[i] == '2' ||  str[i] ==  '4'
                        || str[i] == '6') {
                        *num *= 8;
                        *num += str[i] - 48;
                }
                else {
                        fprintf(stderr, "permissions: bad octal digit: (%c)\n",
                                                        str[i]);
                        return(1);
                }
        }
        return(0);
}


main()
{
	int	fd1, fd2, n, rec, i, prob;
	int	uid, gid, poolgid, srcrc, dstrc, voltype, deleted = 0;
	int	mntread, mntwrite, recycle;
	short	mode;

	/*
	 *  a dbase designer self-check
	 */
	if (sizeof(vbuf) != 256) {
		fprintf(stderr, "vsn record size %d, not 256!\n", sizeof(vbuf));
		exit(1);
	}
	fd1 = fileno(stdin);
	fd2 = fileno(stdout);

	memset(&vbuf2, ' ', sizeof(vbuf2));
	vbuf2.end = '\n';

	for (rec=1;;rec++) {
		/*
		 *  read a record
		 */
		n = read(fd1, &vbuf1, sizeof(oldvbuf));
		if (n == -1) {
			perror("input");
			exit(1);
		}
		if (n == 0)
			break;
		if (n < sizeof(oldvbuf)) {
			char	*cp = (char *)&vbuf1;
			cp[n] = '\0';
			fprintf(stderr, 
				"record %d: read only %d, expected %d\n",
				rec, n, sizeof(oldvbuf));
			fprintf(stderr, "partial record:\n[%s]\n", &vbuf1);
			exit(1);
		}

		/*
		 *  check record - spacers, alphanumerics, numbers, times
		 */

		if ( vbuf1.b01 != ' ' || vbuf1.b02 != ' ' || vbuf1.b03 != ' '
		  || vbuf1.b04 != ' ' || vbuf1.b05 != ' ' || vbuf1.b06 != ' '
		  || vbuf1.b07 != ' ' || vbuf1.b08 != ' ' || vbuf1.b09 != ' '
		  || vbuf1.b10 != ' ' || vbuf1.b11 != ' ' || vbuf1.b12 != ' '
		  || vbuf1.b13 != ' ' || vbuf1.b14 != ' ' || vbuf1.b15 != ' '
		  || vbuf1.b16 != ' ' || vbuf1.b17 != ' ' || vbuf1.b18 != ' '
		  || vbuf1.b19 != ' ' || vbuf1.end != '\n'){
			fprintf(stderr, 
				"delimiter(s) bad in record %d:\n%.*s\n", 
				rec, n, vbuf1);
			continue;
		}

		prob = 0;
		for (i=0; i<VSNLEN; i++) {
			if (!isalnum(vbuf1.v_vsn[i])) {
				fprintf(stderr, "bad char in vsn, record %d\n", 
					rec);
				prob++;
				break;
			}
		}
		if (prob)
			continue;

		if (vbuf1.v_label[0] != ' ') {
			for (i=0; i<VSNLEN; i++) {
				if (!isalnum(vbuf1.v_label[i])) {
					fprintf(stderr, 
					    "bad char in label, record %d\n", 
						rec);
					prob++;
					break;
				}
			}
		}
		if (prob)
			continue;

		if (   ToInt(&uid,   vbuf1.v_uid, BIGINTLEN)
                    || ToInt(&gid,   vbuf1.v_gid, BIGINTLEN)
                    || ToInt(&poolgid,   vbuf1.v_poolgid, BIGINTLEN)
                    || modeto(&mode, vbuf1.v_mode)
                    || ToInt(&voltype, vbuf1.v_voltype, INTLEN)
                    || ToInt(&mntread, vbuf1.v_mntread, INTLEN)
                    || ToInt(&mntwrite, vbuf1.v_mntwrite, INTLEN)
                    || ToInt(&recycle, vbuf1.v_recycle, INTLEN)
                    || ToInt(&srcrc, vbuf1.v_srcrc, INTLEN)
                    || ToInt(&dstrc, vbuf1.v_dstrc, INTLEN)) {
			fprintf(stderr, 
				"integer translation problem, record %d\n",rec);
			continue;
		}

		if (vbuf1.v_status == S_DELETED) {
			deleted++;
			continue;
		}
		if (vVolType(voltype) == NULL) {
			fprintf(stderr, "voltype %d undefined\n", voltype);
		}

		/*
		 *  'older' fields
		 */

		if (  SetInt( vbuf2.v_uid, uid, sizeof(vbuf2.v_uid))
		   || SetInt( vbuf2.v_gid, gid, sizeof(vbuf2.v_gid))
		   || SetInt( vbuf2.v_srcrc, srcrc, sizeof(vbuf2.v_srcrc))
		   || SetInt( vbuf2.v_dstrc, dstrc, sizeof(vbuf2.v_dstrc)))
			continue;
		sprintf(vbuf2.v_mode, "%.3o", mode);
		vbuf2.v_mode[3] = ' ';
		vbuf2.v_use = vbuf1.v_use;
		vbuf2.v_status = vbuf1.v_status;

		memcpy(vbuf2.v_vsn, vbuf1.v_vsn, VSNLEN);
		memcpy(vbuf2.v_label, vbuf1.v_label, VSNLEN);
        	memcpy(vbuf2.v_desc, vbuf1.v_desc, DESCLEN);

		memcpy(vbuf2.v_creat, vbuf1.v_creat, DATELEN);
		memcpy(vbuf2.v_mtime, vbuf1.v_mtime, DATELEN);
		memcpy(vbuf2.v_atime, vbuf1.v_atime, DATELEN);
		memcpy(vbuf2.v_movtm, vbuf1.v_movtm, DATELEN);

		/*
		 *  more setint fields
		 */

		if (  SetInt( vbuf2.v_voltype, voltype, sizeof(vbuf2.v_voltype))
		   || SetInt( vbuf2.v_poolgid, poolgid, sizeof(vbuf2.v_poolgid))
		   || SetInt( vbuf2.v_mntfail, 0, sizeof(vbuf2.v_mntfail))
		   || SetInt( vbuf2.v_mntwrite, mntwrite, 
						sizeof(vbuf2.v_mntwrite))
		   || SetInt( vbuf2.v_mntread, mntread, sizeof(vbuf2.v_mntread))
		   || SetInt( vbuf2.v_recycle, recycle, sizeof(vbuf2.v_recycle)) ) 
			exit(1);
		/*
		 *  write new record
		 */
		n = write(fd2, &vbuf2, sizeof(vbuf2));
		if (n == -1) 
			perror("output");
		if (n != sizeof(vbuf)) {
			fprintf(stderr, "problem writing table\n");
			exit(1);
		}
	}
	fprintf(stderr, "omitted %d deleted vsns\n", deleted);
	exit(0);
}
