#ifndef lint
static char rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/master/rp_1.c,v 1.1.1.1 2000-02-28 02:18:28 wrstuden Exp $";
#endif
/*
 *  rp_1.c: Volume Manager Request Processor - main() routine
 *
 *	rp_1.c	main, packet proc switch to routines, housekeeping routines
 *	rp_2.c	packet proc: mounting & moving volumes, errors & reporting 
 *			status
 *	rp_3.c	packet proc: reporting/changing dbase info
 *	rp_4.c	packet proc: varying drives
 *	rp_5.c	packet proc: reserving drives & volumes
 *	rp_q.c	active packet proc
 *	rp_rid.c reservation of drives & vsns
 *	rp_vol.c setup & manipulate volume requests
 *
 */

#include "rp_space.h"
#include <bsdtime.h>

 static void
usage(char *cmdnam)
{
	fprintf(stderr, "usage: %s [-a] [-c] [-r] [-t time]\n", cmdnam);
	fprintf(stderr, "\t-a\tvary on all drives\n");
	fprintf(stderr, "\t-c\tcheck RC quota counts in database\n");
	fprintf(stderr, "\t-r\tdon't log startup volume use in dbase\n");
	fprintf(stderr, "\t\t\t(faster startup)\n");
	fprintf(stderr, "\t-t\ttime to delay dismount on reserved vsns\n");
	exit(0);
}

/*  debugging options:
 *
 *	-d: don't auto-detach (for debug, undocumented)
 *	-l: install (hexadecimal) debug level (undocumented)
 */

extern int	timerpid, idleshutdown, shutdownrestart;
extern int	totalHosts;
extern time_t	Now;

time_t		Start;

char		shutdownmsg[LINELEN];
char		**argvIn;
char		StartDir[MAXPATHLEN+1];

int	UnmountWaitTime = UNMOUNT_WAITTIME;

int	IsChild = 0;	/* children are used for long vls lookups */
int	DBLogRcs = 1;	/* for -r option */

int	prioritize = 1;	/* allow admin to bump up requests */

main(int argc, char *argv[])
{
	char		*id = "main";
	int		cc, aflg = 0, cflg = 0, dflg = 0;
	char		*cmdnam;
	extern  int     optind;
	extern  char    *optarg;

	time(&Start);

	cmdnam = argv[0];
	CheckUID(cmdnam);

	/*
	 *  save args for vshutdown -r(estart)
	 */
	argvIn = argv;
	if (getcwd(StartDir, sizeof(StartDir)) == NULL)
		fprintf(stderr, "getwd() error: %s\n", StartDir);

	/*
	 *  parse args
	 */
	while ((cc = getopt(argc, argv, "acdr")) != EOF) {
		switch (cc) {
		    case 'a':			/* vary all devs on */
			aflg = 1;
			break;
		    case 'c':			/* init counts in vdmlib */
			cflg = 1;
			break;
		    case 'd':			/* don't fork */
			dflg = 1;
			break;
/*
		    case 'l':
			if (sscanf(optarg, "%x", &nLogLevel) != 1)
				usage(cmdnam);
			fprintf(stderr, "LogLevel = 0x%x\n", nLogLevel);
			break;
*/
		    case 'r':			/* don't check db vsn status */
			DBLogRcs = 0;
			break;
		    case 't':
			UnmountWaitTime = atoi(optarg);
			if (UnmountWaitTime < 1)
				fprintf(stderr, "(no delay on dismounts)\n");
			break;
		    default:
			usage(cmdnam);
		}
	}
	if (optind < argc) 
		usage(cmdnam);

	/*
	 *  go into the background
	 */
#ifndef PURIFY
	if (!dflg) {
		switch (fork()) {
		  case -1:				/* fork failed */
			fprintf(stderr, "%s: can't fork... exiting\n", argv[0]);
			exit(1);
			break;
		  case 0:				/* child */
			if (setpgid(0,0))
				perror("setpgid");
			break;
		  default:				/* parent */
			exit(0);	/* detach */
		}
	}
#endif
	/*
	 *  start up & vary drives if reqd
	 */

	RPInit(cflg);
	if (aflg)
		VaryAll(1, VolManOperUID);
	else
		VaryDefaults();

	/*
	 *  read & process all msgs using configured IPC
	 */

	IPCServer();
}


/*
 *  MRcMsg() - receive and process M_RCMSG messages.  These are messages
 *	from clients (including other RCs) which are forwarded to an RC.  
 *	Permission is granted based on the client's UID and RC selected.
 */
 static void
MRcMsg(vmpHdr_t *hp, SrvProc_t *sp)
{
	char		*id = "MRcMsg";
	vmpRcMsg_t	*rcmp;
	RPRC_t		*rcp;

	rcmp = MSGPTR(hp, vmpRcMsg_t);

	/*
	 *  the RC to receive the message can be specified directly
	 *	or indirectly by reference to a vsn active in it
	 */
	rcp = NULL;
	if (rcmp->rm_flag & RCM_RC) {
		/*
		 *  explicit spec of RC
		 */
		rcp = RCInfo(rcmp->rm_rc);
		if (rcp == NULL) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_STATUS1, id, "bad RC (%d)",
			  rcmp->rm_rc);
			Err2Srv(hp->vh_seq, sp);
			return;
		}
	} else if (rcmp->rm_flag & RCM_VSN) {
		VolReq_t	*vrp;

		/*
		 *  indirect spec of RC by vsn
		 */
		if (vrp = FindActiveVolReq((char *)&rcmp->rm_vsn)) {
			vmpHdr_t	*hp2 = (vmpHdr_t *) rcmp->rm_msg;

			hp2->vh_seq = vrp->vr_rpseq;

			switch (vrp->vr_state) {
			    case VS_MOVING:
				if (rcmp->rm_flag & RCM_SRC)	/* get dst RC */
					rcp = vrp->vr_rcp;	/* dst RC */
				else		/* else get src RC by default */
					rcp = RCInfo(vrp->vr_rc); /* src RC */
				break;
			    default:
				rcp = vrp->vr_rcp;		/* dst RC */
			}
		}
		if (rcp == NULL) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_STATUS1, id,
				"no request for VSN %.*s active in an RC",
				VSNLEN, &rcmp->rm_vsn);
			Err2Srv(hp->vh_seq, sp);
			return;
		}
	} else {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR1, id, "no flags set to determine RC");
		Err2Srv(hp->vh_seq, sp);
		return;
	}

	/*
	 *  the RC's OK
	 */
	if (Send2RC(rcp, VolReqNULL, M_RCMSG, rcmp) != E_NOERROR)
		Err2Srv(hp->vh_seq, sp);
	else {
		hp->vh_type = M_ACK;
		hp->vh_size = 0;
		Snd2Srv(hp, sp, 1);
	}
}

/*
 *  MDebug() - process M_DEBUG packets received by the RP
 *	print status, change logging level
 */
 static void
MDebug(vmpHdr_t *hp)
{
	char		*id = "MDebug";
	vmpDebug_t	*dp;
	RPRC_t		*rcp;
  	vmpHdr_t	*rhp;
	vmpDebug_t	*rdp;
	char		buf[PKTSIZ(vmpDebug_t)];
	int		rcno;

	dp = MSGPTR(hp, vmpDebug_t);

	switch (dp->vd_cmd) {
	    case 0:
		if (dp->vd_level & ~EL_LEVEL) {
			nLogMsg(EL_ERROR1, id, "0x%x: unknown debug level",
			  				dp->vd_level);
			break;
		}

		nLogSetLevel(dp->vd_level);
		nLogMsg(EL_STATUS1, id, "debug level set to 0x%x",
			  				dp->vd_level);
		rhp = vBuildHdr(M_DEBUG, NOSEQ, (vmpHdr_t *) buf);
		rdp = MSGPTR(rhp, vmpDebug_t);
		rdp->vd_cmd = dp->vd_cmd;
		rdp->vd_level = dp->vd_level;
		for (rcp=RCArray,rcno=0; rcno<RCTotal; rcno++,rcp++) {
	    		if (rcp->rc_srv.s_sd != -1)
				XDRSendI(rhp, rcp->rc_srv.s_sd, 1);
		}
		/* TODO - send to volnd, volcd */
		break;
	    case 1:				/* print ClientQ */
		vLogClients();
		break;

	    case 2:				/* send each client a NOP */
		nLogMsg(EL_STATUS1, id, "Check clients not implemented");
		break;

	    case 3:
		vLogClients();
		PrintVolReqQ(EL_ALWAYS, id);		/* print VolReqQ */
		PrintReservations();
		PrintRCArray(EL_ALWAYS, id);		/* print RC table */
		break;
		
	    case 4: 			/* print a message on the RP Console */
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, "Test Console Message");
		break;

	    default:
		nLogMsg(EL_ERROR1, id, "%d: unknown debug cmd", dp->vd_cmd);
		break;
	}
}

/*
 *  MNewLog() - restart log file (gets unwieldy if vold runs a long time)
 */
 static void
MNewLog(vmpHdr_t *hp, SrvProc_t *sp)
{
	char		*id = "MNewLog";
	RPRC_t		*rcp;
	extern int	RPShutdown();
	Host_t		*hop;
	int		i;

	/*
	 *  restart log and tell volcd's, volnd's & RC's
	 */
	AckSrv(hp->vh_seq, sp);
	nLogMsg(EL_STATUS1, id, "=========== restarting log ==============");
	nLogInitFile(vLogFile, EL_LEVEL, 1, NULL);
	nLogMsg(EL_STATUS1, id, "---------- RESTARTED LOG -------------------");
	nLogMsg(EL_STATUS1, id, "process started %s", bsd_ctime(&Start));

	/*
	 *  tell volcd/volnd on each host: first the volcd's
	 *	since it's their responsibility to rename the
	 *	file when on non-vold hosts, then the volnd's 
	 *	which will reopen the restarted logfiles.
	 */
	hp->vh_slot = -1;
	for (i=0; hop=HostIndex(i); i++) {
		if (hop->h_volcd.s_sd != -1)
			Snd2Srv(hp, &hop->h_volcd, 1);
	}
	sleep(3);
	for (i=0; hop=HostIndex(i); i++) {
		if (hop->h_volnd.s_sd != -1)
			Snd2Srv(hp, &hop->h_volnd, 1);
	}

	/*
	 *  tell RCs TODO: they should know to rename log if there
	 *	isn't a volcd on their host
	 */
	for (rcp=RCArray,i=0; i<RCTotal; i++,rcp++) { 
		if (rcp->rc_srv.s_sd != -1) {
			XDRSendI(hp, rcp->rc_srv.s_sd, 1);
		}
	}

}

 void
MPriority(vmpHdr_t *hp, SrvProc_t *sp)
{
	char		*id = "MPriority";
	VolReq_t	*vrp;
	int		i;

	if (hp->vh_count == 0) {
		if (prioritize) {
			nLogMsg(EL_STATUS1, id, 
					"Turning off request prioritization");
			prioritize = 0;
		} else {
			nLogMsg(EL_STATUS1, id, 
					"Turning on request prioritization");
			prioritize = 1;
		}
		AckSrv(hp->vh_seq, sp);
		return;
	}
	for (i=0,vrp=VRHEAD(VolReqQ); vrp; i++,vrp=VRNEXT(vrp->vr_vrq)) {
		if (vrp->vr_rpord == hp->vh_count)
			break;
	}
	if (vrp == NULL) {
		VMError = E_USER;
		nLogMsg(EL_ERROR1|EL_CODE, id, "request %d not found", 
							hp->vh_count);
		Err2Srv(hp->vh_seq, sp);
		return;
	}
	nLogMsg(EL_STATUS1, id, "moving req %d back %d places to head of queue",
				vrp->vr_rpord, i);
	DEQUEUE(vrp, &VolReqQ, vr_vrq, VolReq_t);
	ENQUEUE_H(vrp, &VolReqQ, vr_vrq, VolReq_t);
	AckSrv(hp->vh_seq, sp);
}

/*
 *  MShutdown() - process M_SHUTDOWN packets received by the RP
 *	only allow M_SHUTDOWNs from volman or root uid
 */
 static void
MShutdown(vmpHdr_t *hp, IPC_t *ip)
{
	char		*id = "MShutdown";
	vmpError_t	*ep;

	/*
	 *  tip off client if prev idle shutdown requested
	 */
	ep = MSGPTR(hp, vmpError_t);
	ep->vn_error = idleshutdown;

	/*
	 *  Permission ok, ack client
	 */
	XDRSendI(hp, ip->sock_fd, 1);

	/*
	 *  check flags
	 */
	if (ep->vn_state & SH_WAIT) {
		nLogMsg(EL_STATUS1, id, "Idle down");
		idleshutdown++;
	} else {
		if (idleshutdown)
			nLogMsg(EL_STATUS1, id, "Overriding idle down");
		idleshutdown = 0;	/* override any prev */
	}
	if (ep->vn_state & SH_RESTART) {
		shutdownrestart = 1;
	} else {
		if (shutdownrestart)
			nLogMsg(EL_STATUS1, id, "Overriding shutdown/restart");
		shutdownrestart = 0;
	}
	if (!idleshutdown)
		RPShutdown(id, ep->vn_msg[0] ? ep->vn_msg :
	  		"Shutdown command from console");
	memcpy(shutdownmsg, ep->vn_msg, LINELEN);
}



/*
 *  *MsgSwitch() - called from IPC - Pass incoming packets with client to
 *	appropriate functions for processing. Note if there is a need to
 *	process the request queue (procQ) as a result.. this is as much
 *	to keep the debug Q logging restricted to state changes as to 
 *	save CPU.
 */
 int
ConsoleMsgSwitch(vmpHdr_t *hp, IPC_t *ip, int slot)
{
	char		*id = "ConsoleMsgSwitch";
	SrvProc_t	srv;
	int		procQ = 0;

	srv.s_sd = slot;
	srv.s_pid = ip->sock_id;

	switch (hp->vh_type) {
	    case M_GOODBYE:
		CloseIPC(ip); break;
	    case M_RCINFO:
		RCInfo2Srv(hp, &srv); break;
	    case M_VLS:		/* look up vsn(s)		*/
		MVslook(hp, &srv); break;
	    case M_RCMSG:
		MRcMsg(hp, &srv); break;
	    case M_SHUTDOWN:
		MShutdown(hp, ip); break;
	    case M_NEWLOG:
		MNewLog(hp, &srv); break;
	    case M_PRIORITY:
		MPriority(hp, &srv); break;
	    case M_DEBUG:
		MDebug(hp); break;
	    default:
		nLogMsg(EL_FATAL, id, "unexpected msg %s", vHdrStr(hp));
	}
	return(procQ);
}

VolCDMsgSwitch(vmpHdr_t *hp, Host_t *hop)
{
	char	*id = "VolCDMsgSwitch";
	int	procQ = 0;

	hop->h_volcd.s_lastmsg = Now;

	switch (hp->vh_type) {
	    case M_NEWCLIENT:
		MNewClient(hp, hop);
		break;
	    case M_GOODBYE:
		procQ = CleanupVolCD(hop);
		break;
	    default:
		nLogMsg(EL_FATAL, id, "unexpected msg %s", vHdrStr(hp));
	}

	return(procQ);
}

CleanupVolCD(Host_t *hop)
{
	hop->h_volcd.s_sd = -1;
	hop->h_volcd.s_pid = -1;
	ShutdownHostClients(hop);
	return(1);
}

/*
 *  MUnexpected() - process packets received by the RP which
 *	it wasn't expecting (lots you can do, right?)
 */
 static void
MUnexpected()
{
	nLogMsg(EL_ERROR1|EL_CONSOLE0, 0, "rcvd UNEXPECTED packet");
}

/*
 *  Munimplemented() - process packets received by the RP for which
 *	there is no handler yet (i.e. work in progress)
 */
 static void
Munimplemented(vmpHdr_t *hp, Client_t *cp)
{
	char		*id = "Munimplemented";

	VMError = E_USER;
	nLogMsg(EL_CODE|EL_ERROR2, id, 
				"%s: not implemented", vMsgType(hp->vh_type));
	Err2Client(hp->vh_seq, cp);
}


/*
 *  ClientMsgSwitch() - msgs from volcd client
 */
 int
ClientMsgSwitch(vmpHdr_t *hp, Client_t *cp)
{
	char		*id = "ClientMsgSwitch";
	int		procQ = 0;

	/* 
	 * TODO - consider indexing into an array of calls 
	 * 	for now, ordered approx by frequency
	 */
	switch (hp->vh_type) {
	    case M_RCINFO:	/* send RC config		*/
		RCInfo2Client(hp, cp); break;
	    case M_HOSTINFO:	/* send host config		*/
		HostInfo2Client(hp, cp); break;
	    case M_GOODBYE:	/* shutdown a connection	*/
		MClientGoodbye(cp); break;
	    case M_VOLQRY:	/* query volume mount status	*/
		MVolQry(hp, cp); break;
	    case M_USRMOUNT:	/* mount a volume	*/
		procQ = MMount(hp, cp); break;
	    case M_USRMOUNT_SEL:/* mount 1 of N volumes		*/
		procQ = MMountSel(hp, cp); break;
	    case M_USRMOUNT_PLL:/* mount N volumes at once	*/
		procQ = MMountPll(hp, cp); break;
	    case M_ATTACH:	/* connect w/ async request	*/
		MAttach(hp, cp); break;
	    case M_DETACH:	/* make a request async		*/
		MDetach(hp, cp); break;
	    case M_UNMOUNT:	/* unmount a volume		*/
		procQ = MUnmount(hp, cp); break;
	    case M_VLS:		/* look up 1 vsn		*/
		MVinfo(hp, cp); break;
	    case M_VARY:		/* dis/allow use of a device	*/
		procQ = MVary(hp, cp, NULL); break;
	    case M_MOVE:		/* change the location of a vol	*/
		procQ = MMove(hp, cp); break;
	    case M_RESERVE:	/* set up / modify a reservation*/
		procQ = MReserve(hp, cp); break;
	    case M_VALLOC:	/* allocate a scratch volume	*/
		MVAlloc(hp, cp); break;
	    case M_VRECYCLE:	/* deallocate a volume		*/
		MRecycle(hp, cp); break;
	    case M_LBLQRY:	/* look up / update a label	*/
		MLabelQuery(hp, cp); break;
	    case M_VCHMOD:	/* change volume's permissions 	*/
		MChmod(hp, cp); break;
	    case M_VCHOWN:	/* change volume's ownership	*/
		MChown(hp, cp); break;
	    case M_VCHGRP:	/* change volume's group	*/
		MChgrp(hp, cp); break;
	    case M_VATTR:		/* set volume attributes	*/
		MVattr(hp, cp); break;
	    case M_ADDVSN:	/* add vsn to dbase		*/
		MAddVsn(hp, cp); break;
	    case M_SCRATCH:	/* have vsn mounted/erased/relabeled	*/
		procQ = MScratch(hp, cp); break;
	    case M_DELVSN:	/* delete vsn from dbase	*/
		MDeleteVsn(hp, cp); break;
	    case M_SETQUOTA:	/* admin create/set quota	*/
		MSetQuota(hp, cp); break;
	    case M_QUOTQRY:	/* get quota info		*/
		MGetQuota(hp, cp); break;
	    case M_LBLOOK:	/* VDR lookup by internal label/UID*/
		MLook(hp, cp); break;
	    case M_ACK:		/* acknowledge			*/
		MUnexpected(); break;
	    case M_REQUEUE:	/* unmount and try another drive*/
		Munimplemented(hp, cp); break;
	    case M_AUDIT:		/* dump contents of an rc	*/
		Munimplemented(hp, cp); break;
	    case M_DEVQRY:	/* query device status		*/
	    case M_DEVSTAT:	/* contains device status	*/
		Munimplemented(hp, cp); break;
/*
TODO
add procQ++ when done debugging
*/
	    case M_VDR:		/* contains a copy of a VDR	*/
		Munimplemented(hp, cp); break;
	    default:
		VMError = E_USER;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
				"%s: unexpected message type (0x%x)",
		          	vHdrStr(hp), hp->vh_type);
		Err2Client(hp->vh_seq, cp);
		break;
	}
	return(procQ);
}	

/*
 *  RCUnReadyMsgSwitch() - RC hasn't identified itself yet -
 *	in this state, only info requests & 'ready' make sense
 */
 int
RCUnReadyMsgSwitch(vmpHdr_t *hp, int slot, IPC_t *ip, Host_t *hop)
{
	char		*id = "RCUnReadyMsgSwitch";
	vmpHello_t	*hep;
	RPRC_t		*rcp;
	SrvProc_t	srv;
	int		i;

	/*
	 *  init a generic SrvProc_t for sending msg
	 */
	srv.s_sd = slot;

	switch (hp->vh_type) {
	    case M_RCINFO:
		RCInfo2Srv(hp, &srv);
		return(0);
	    case M_DEVQRY:
		Dev2Srv(hp->vh_seq, &srv);
		return(0);
	    case M_VARY:
		return(MVary(hp, NULL, &srv));
	    case M_RCREADY:
		/*
		 *  RC is identifying self & registering 'ready-to-serve'
		 */
		hep = MSGPTR(hp, vmpHello_t);
		rcp = RCInfo(hep->vhe_id);
		if (rcp == NULL) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_STATUS1, id,
					"unknown rc %d from %s - refusing",
					hep->vhe_id, hop->h_name);
			Err2Srv(hp->vh_seq, &srv);
			CloseIPC(ip);
			return(0);
		}
		if (rcp->rc_srv.s_sd != -1) {
			/*
			**	RC already registered - check if
			**	still in communication
			*/
			(void)Nop2Srv(&rcp->rc_srv);
		}
		if (rcp->rc_srv.s_sd != -1) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_STATUS1, id, 
					"duplicate %s on %s - refusing",
					rcp->rc_id, hop->h_name);
			Err2Srv(hp->vh_seq, &srv);
			CloseIPC(ip);
			return(0);
		}
		/*
		 *  connect RC & sock to each other
		 */
		ip->sock_proc = (void *) rcp;
		rcp->rc_srv.s_sd = slot;
		/*
		 *  set up drives, slots, & mark ready
		 */
		for (i=0; i<RC_MEDIA_TYPES; i++) {
			rcp->rc_devs[i].d_mountdev = 0;
        		DM_ZERO(rcp->rc_inuse[i]);
			rcp->rc_avail[i] = rcp->rc_devs[i].d_devmap;
		}
		rcp->rc_freeslots = hep->vhe_flag;
		rcp->rc_flags |= RCF_READY;
		hp->vh_slot = -1;
		Snd2Srv(hp, &rcp->rc_srv, 1);
		nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, "==== registered %s on %s",
						rcp->rc_id, hop->h_name);
		return(1);

	    case M_GOODBYE:
		nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
					"RC on %s canceling initialization",
					hop->h_name);
for (rcp=RCArray,i=0; i<RCTotal; i++,rcp++)
if (rcp->rc_number == -1)
nLogMsg(EL_FATAL, id, "pre-close rc number %d is -1", i);
		CloseIPC(ip);
for (rcp=RCArray,i=0; i<RCTotal; i++,rcp++)
if (rcp->rc_number == -1)
nLogMsg(EL_FATAL, id, "post-close rc number %d is -1", i);
		return(0);
					
	    default:
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_STATUS1, id, 
				"unknown msg type %s from RC on %s - refusing",
				vHdrStr(hp), hop->h_name);
		Err2Srv(hp->vh_seq, &srv);
		CloseIPC(ip);
		return(0);
	}
}

 int
RCMsgSwitch(vmpHdr_t *hp, RPRC_t *rcp)
{
	char		*id = "RCMsgSwitch";

	switch (hp->vh_type) {
	    case M_VOLSTAT:	/* RC response to a request 	*/
		return(MVolStat(hp, rcp)); 
	    case M_CREATENODE:
	    case M_DELETENODE:
	    case M_SCRATCH:
		return(MNodeRC(hp, rcp));
	    case M_VARY:
		return(MVary(hp, NULL, &rcp->rc_srv));
	    case M_VERROR:
		return(MErrorRC(hp, rcp));
	    case M_GOODBYE:
		return(CleanupRC(rcp));
	    case M_RCREADY:
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_STATUS1, id, 
					"%s already ready", rcp->rc_id);
		Err2Srv(hp->vh_seq, &rcp->rc_srv);
		return(0);
	    default:
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_STATUS1, id, 
					"unexpected msg %s from %s", 
					vHdrStr(hp), rcp->rc_id);
		Err2Srv(hp->vh_seq, &rcp->rc_srv);
		return(0);
	}
}

/*
 *  CleanupRC() - perform housekeeping for an RC which has gone away:
 *
 *	1) Close comms
 *	2) Remove all requests for these drives
 *	3) reset rc_mountdev & bitmaps
 *
 *  Don't bother to issue unmounts to the RC since it is down.
 *	Database is cleaned up next time this RC comes up.
 */
 int
CleanupRC(RPRC_t *rcp)
{
	char		*id = "CleanupRC";
	VolReq_t	*vrp, *vrpnext;
	int		rcindex = rcp->rc_index;
	int		i, procQ = 0;

	rcp->rc_srv.s_sd = -1;

	nLogMsg(EL_STATUS1, id, "Dequeing all %s requests:", rcp->rc_id);
	for (vrp=VRHEAD(VolReqQ); vrp; vrp=vrpnext) {
		int	ismove;

		vrpnext = VRNEXT(vrp->vr_vrq); /* since vrp might be DQ'd */

		/*
		 *  check the two RC indicators (vr_rcp and vr_dstrcp) 
		 *	for this RC
		 */

		if (vrp->vr_rcp != rcp  &&  vrp->vr_dstrcp != rcp)
			continue;

		ismove = (vrp->vr_state == VS_MOVING ? 1 : 0);

		if (ismove || vrp->vr_state == VS_MOUNTED)
			VMError = E_CANCELED;
		else
			VMError = E_UNAVAIL;
		nLogMsg(EL_STATUS1|EL_CODE, id,
			  "%.*s_%d: %s canceled because %s is down", 
			  VSNLEN, &vrp->vr_vsn, vrp->vr_rpseq,
			  ismove ? "move" : "request", rcp->rc_id);
		procQ++;
		if (vrp->vr_clientp)
			Err2Client(vrp->vr_clseq, vrp->vr_clientp);
		if (ismove) {

			/*
			 *  if move in progress, notify the other RC
			 *	that the move is canceled because this
			 *	RC has gone down
			 */
			CancelMove(vrp, rcp); /* also sends M_VERROR */
		}

		/*
		 *  delete or inactivate request
		 */
		if (BUSY_STATES(vrp->vr_state))
			vrp->vr_ridp->rid_active--;
		if (MOUNT_STATES(vrp->vr_state)) {
			int	medium, index;
			medium= vVolTypeMedium(vrp->vr_info.vi_voltype);
			index = rcindex * RC_MEDIA_TYPES +
					RCMedium(rcp, medium);
			FreeDriveUsed(&vrp->vr_ridp->rid_RCMed[index]);
			if (NODE_STATES(vrp->vr_state))
				DeleteNode(vrp, 0);
		}

		if (vrp->vr_flag & VF_IMPLICITVSN) {
			FreeVolReq(vrp, id);	
		} else {
			vrp->vr_state = VS_IDLE;
			DEQUEUE(vrp, &VolReqQ, vr_vrq, VolReq_t);
		}
	}
	for (i=0; i<RC_MEDIA_TYPES; i++) {
		rcp->rc_devs[i].d_mountdev = 0;
        	DM_ZERO(rcp->rc_inuse[i]);
	}
	return(procQ);
}

/*
 *  VolNodeMsgSwitch() - msgs from volnd on some host
 */
 int
VolNodeMsgSwitch(vmpHdr_t *hp, Host_t *hop)
{
	char		*id = "VolNodeMsgSwitch";
	vmpTimer_t	*tp;

	hop->h_volnd.s_lastmsg = Now;

	switch (hp->vh_type) {
	    case M_TIMER:
		tp = MSGPTR(hp, vmpTimer_t);
		if (Now - (time_t) tp->vt_time > 10)
			nLogMsg(EL_STATUS1, id, "roundtrip time to %s = %d sec",
						hop->h_name,
						Now - (time_t) tp->vt_time);
		break;
	    case M_IDLETIME:
		MIdleTimes(hp);
		return(0);
	    case M_CREATENODE:
	    case M_DELETENODE:
	    case M_SCRATCH:
		return(MNodeV(hp, hop));
	    case M_VERROR:
		return(MErrorV(hp, hop));
	    case M_VARY:
		return(MVary(hp, NULL, &hop->h_volnd));
	    case M_DEVQRY:
		Dev2Srv(hp->vh_seq, &hop->h_volnd);
		return(0);
	    case M_GOODBYE:
		return(CleanupVolNodeD(hop));
	    default:
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_STATUS1, id, 
				"unexpected type %s", vHdrStr(hp));
		Err2Srv(hp->vh_seq, &hop->h_volnd);
		return(0);
	}
}

/*
 *  CleanupVolNodeD() - close all active requests on volnd host
 */
 int
CleanupVolNodeD(Host_t *hop)
{
	char		*id = "CleanupVolNodeD";
	VolReq_t	*vrp, *vrpnext;
	int		procQ = 0;

	hop->h_volnd.s_pid = -1;
	hop->h_volnd.s_sd = -1;

	SetVolndMapOFF(hop->h_id);

	nLogMsg(EL_STATUS1, id, 
		"Dequeueing all active volnd requests on host %s:",
		hop->h_name);
	for (vrp=VRHEAD(VolReqQ); vrp; vrp=vrpnext) {
		vrpnext = VRNEXT(vrp->vr_vrq); /* since vrp might be DQ'd */

nLogMsg(EL_STATUS1, id, "%.*s: host %s, %s",
VSNLEN, &vrp->vr_vsn, 
(vrp->vr_ndhostp ? vrp->vr_ndhostp->h_name : "(null)"),
vReqStateStr(vrp->vr_state));

		if (vrp->vr_ndhostp != hop)
			continue;

		switch (vrp->vr_state) {
		    case VS_RCWAIT:
			if (vrp->vr_rcp->rc_flags & RCF_NONODE) {
				/*
				 *  RC relies on vold to make node -
				 *	for now allow request to
				 *	continue in RC in case new
				 *	volnd comes up..
				 *	NOT TESTED
				 */
				continue;
			}
			if (vrp->vr_rcp->rc_flags & RCF_NODEMOUNT) {
				/*
				 *  assume that volnd was waiting
				 *	on mount on behalf of RC
				 */
				Unmount(vrp);
				procQ++;
			}
			break;
		    case VS_DELETENODE: 
			/*
			 *  since new volnd will clean up, act as if
			 *	delete happened ok
			 */
			DeleteNodeDone(vrp, vDMffs(&vrp->vr_devinuse));
			procQ++;
			break;
		    case VS_CREATENODE: 
			/*
			 *  fail req
			 */
			/* NOT TESTED */
			Unmount(vrp);
			procQ++;
			break;
		    default:
			/* leave it alone for now.. */
			nLogMsg(EL_STATUS1, id, "%.*s_%d - %s (leaving alone)",
					VSNLEN, &vrp->vr_vsn, vrp->vr_rpseq,
					vReqStateStr(vrp->vr_state));
			break;
		}
	}
	return(procQ);
}


/*
 *  MTimer() - received M_TIMER message from voltimer - forward to
 *	any RCs that are configured to get it, and to all volnd's.
 *	Use the occasion to see if any requests are waiting for
 *	drives that could be varied on.
 */
 void
MTimer(vmpHdr_t *hp)
{
	char		*id = "MTimer";
	RPRC_t		*rcp;
	Host_t		*hop;
	vmpTimer_t	*tp;
	int		i;

	if (hp->vh_type != M_TIMER) {
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, "got %s", vHdrStr(hp));
		return;
	}

	/*
	 *  see how long packet took and rejuvenate it
	 *	for retransmission
	 */
	tp = MSGPTR(hp, vmpTimer_t);
	if (Now - tp->vt_time > 2)
		nLogMsg(EL_STATUS2, id, "time from voltimer to vold %d",
						Now - tp->vt_time);
	tp->vt_time = Now;

	/*
	 *  send to RCs and volnd's
	 */
	for (i=0,rcp=RCArray; i<RCTotal; i++,rcp++) {
if (rcp->rc_number == -1)
nLogMsg(EL_FATAL, id, "rc number %d is -1", i);
		if (rcp->rc_srv.s_sd != -1  &&
		    rcp->rc_flags & RCF_TIMER) {
			nLogMsg(EL_DEBUG2, id, "to %s", rcp->rc_id);
			XDRSendI(hp, rcp->rc_srv.s_sd, 1);
		}
	}
	for (i=0; i<totalHosts; i++) {
		hop = HostIndex(i);
		if (hop->h_volnd.s_sd != -1) {
			XDRSendI(hp, hop->h_volnd.s_sd, 1);
		}
	}
	
	/*
	 *  check for requests waiting on drives that
	 *	might be able to be varied on
	 */		
	DoVaryMsg();
}



