
/*
TODO - replace OFF_T addresses in index w/ line #s

 *  rp_db.c - database-using routines for the Request Processor
 * 	module of the NAStore Volume Manager
 *
 *	 rp_db.c uses Sleepycat db3 btree indexing shareware.
 *
 *	Originally written using Informix ESQL/C library by
 *		Bill Ross, Sterling Software 
 *		NASA Ames Research Center, October 1988
 *
 *	Converted to bplus by
 *		Bill Ross, Computer Sciences Corporation
 *		NASA Ames Research Center, December, 1991
 *
 *	Converted to db3 by
 *		Bill Ross, MRJ
 *		NASA Ames Research Center, January, 2000
 */
/*
 *	For administrative configuration of the database (file names),
 *	see vconfig.h
 */
/*
 *	Btree errors should cause an immediate shutdown. Here is how they
 *	are handled/reported. If the error is reported, the caller should
 *	perform the shutdown, including calling CloseDb().
 *
 *	shuts down within routine (EL_FATAL) if error:
 *	
 *	        InitDb()
 *	        DBRcVols()
 * TODO: update these:	
 *	returns E_DBASE
 *	
 *	        UnmountDone()
 *	        UpdateLoc()
 *	        Recycle()
 *	
 *	returns NULL with VMError == E_DBASE
 *	
 *	        MountQuery()
 *	        MoveQuery()
 *	        AsyncUpdateLoc()
 *	        AllocScratch()
 *	        QuotaQuery()
 *	        VsnQuery()
 */

/*
 *  administrative parameters: name of dbase dir and base names of tables
 *	are in vconfig.h
 */

/****************************************************************************/

#include "rp_space.h"
#include "rp_db.h"

#include <ctype.h>
#include <time.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>


extern int      	VolManOperUID;
extern int		DBLogRcs;
extern CodeString_t	vStatTab[];
extern time_t		Now;

static int		dbopen = 0;
static struct stat	sbuf;
static vbuf		vsnbuf;
static qbuf		quotabuf;

static char		dbf_lock[MAXPATHLEN];

static char		dbf_vsn_tbl[MAXPATHLEN];
static char		dbf_vsn_vsn_idx[MAXPATHLEN];
static char		dbf_vsn_uid_idx[MAXPATHLEN];
static char		dbf_vsn_lab_idx[MAXPATHLEN];
static char		dbf_vsn_sta_idx[MAXPATHLEN];

static char		dbf_quo_tbl[MAXPATHLEN];
static char		dbf_quo_idx[MAXPATHLEN];


/*****************************************************************
 			vars used to return values
 *****************************************************************/

VDR_t		vdr;		/* ALL info on vsn */
#define	vinfo	vdr.vdr_info 	/* some info on vsn */
vmpQuota_t	quota;		/* info on quota */


/*****************************************************************
		vars used for accessing dbase
 *****************************************************************/

/***************************************
	table file descriptors 
 ***************************************/

int	db_vsn_fd = -1, db_quota_fd = -1;

/***************************************
	db index structures	
 ***************************************/

/* unique key vsn indexes */
DB	*db_vsn_vsn_idx = NULL;
DB	*db_vsn_lab_idx = NULL;

/* dup key vsn indexes    */
DB	*db_vsn_uid_idx = NULL; 
DB	*db_vsn_sta_idx = NULL;

/* unique key quota index */
DB	*db_quota_idx = NULL;

DBT	key, data;
char	quotakey[128], vsnkey[128];

int	dberr;

OFF_T	vsnaddr, quotaddr;

/* useful db macros */

#define ZERO_OBJ(obj)	bzero(&obj, sizeof(obj))

#define FILL_DBT(dbt, item) \
	ZERO_OBJ(dbt); \
	dbt.data = &(item); \
	dbt.size = sizeof(item)

#define PTR_DBT(dbt, item) \
	ZERO_OBJ(dbt); \
	dbt.data = item; \
	dbt.size = sizeof(*item)

#define STR_DBT(dbt, item) \
	ZERO_OBJ(dbt); \
	dbt.data = item; \
	dbt.size = strlen(item)

#define	COPY_DBT(obj, dbt) \
	if (sizeof(obj) != dbt.size) \
		nLogMsg(EL_ERROR1|EL_FATAL, id, "db data size mismatch"); \
	memcpy(&obj, dbt.data, sizeof(obj))

/***********************************************************************
   			basic dbase table routines
 ***********************************************************************/

/***********************************************************
  					    	UNLINK()
 ***********************************************************/

/*
 *  Unlink() - ignore ENOENT
 */

 static void
Unlink(char *name)
{
	char	*id = "Unlink";

	if (unlink(name) == -1) {
		if (errno != ENOENT)
			nLogMsg(EL_SYS|EL_FATAL, id, "%s: unlink failed", name);
	} else
		nLogMsg(EL_DEBUG1, id, "unlinked %s", name);
}


/***********************************************************
						TOINT()
 ***********************************************************/

/*
 *  ToInt() - check & translate char -> int
 */

 static int
ToInt(int *nump, char *str, int size)
{
	char	*id = "db to integer";
	int     i;

	/*
	 *  boundary char should be space
	 */
	if (str[size] != ' ') {
		nLogMsg(EL_ERROR1, id, "int field discrepancy (%.*s)",
							size+1, str);
		return(1);
	}

	/*
	 *  skip leading space
	 */

	for (i=0; i<size && str[i] == ' '; i++);

	/*
	 *  if all spaces, number is 0
	 */

	if (i == size) {
		*nump = 0;
		return(0);
	}

	/*
	 *  leading '-' ok
	 */

	if (str[i] == '-') {
		if (i>size-1) {
			nLogMsg(EL_ERROR1, id, "leading '-' w/ no # (%.*s)",
							size, str);
			return(1);
		}
		i++;
	}

	/*
	 *  rest should be numerals
	 */
	for (; i<size; i++)
		if (!isdigit(str[i])) {
			nLogMsg(EL_ERROR1, id, 
				"int field (%.*s) has non-numeral '%c' at %d", 
				size, str, str[i], i);
			return(1);
		}
	*nump = atoi(str);
	return(0);
}

/*
 *  to ensure that each type (size) of integer is read properly, 
 *	define a macro for each field
 */
#define ToIntSrcRC( nump ) ToInt( nump, vsnbuf.v_srcrc, sizeof(vsnbuf.v_srcrc))
#define ToIntDstRC( nump ) ToInt( nump, vsnbuf.v_dstrc, sizeof(vsnbuf.v_dstrc))
#define ToIntVolType( nump ) ToInt( nump, vsnbuf.v_voltype, \
						sizeof(vsnbuf.v_voltype))
#define ToIntMntWrite( nump ) ToInt( nump, vsnbuf.v_mntwrite, \
						sizeof(vsnbuf.v_mntwrite))
#define ToIntMntRead( nump ) ToInt( nump, vsnbuf.v_mntread, \
						sizeof(vsnbuf.v_mntread))
#define ToIntMntFail( nump ) ToInt( nump, vsnbuf.v_mntfail, \
						sizeof(vsnbuf.v_mntfail))
#define ToIntUidVsn( nump ) ToInt( nump, vsnbuf.v_uid, \
						sizeof(vsnbuf.v_uid))
#define ToIntGid( nump ) ToInt( nump, vsnbuf.v_gid, \
						sizeof(vsnbuf.v_gid))
#define ToIntPoolGid( nump ) ToInt( nump, vsnbuf.v_poolgid, \
						sizeof(vsnbuf.v_poolgid))
#define ToIntRecycle( nump ) ToInt( nump, vsnbuf.v_recycle, \
						sizeof(vsnbuf.v_recycle))

#define ToIntUidQuo( nump ) ToInt( nump, quotabuf.q_uid, \
						sizeof(quotabuf.q_uid))
#define ToIntRCQuo( nump ) ToInt( nump, quotabuf.q_rc, \
						sizeof(quotabuf.q_rc))
#define ToIntQuota( nump ) ToInt( nump, quotabuf.q_quota, \
						sizeof(quotabuf.q_quota))
#define ToIntUsed( nump ) ToInt( nump, quotabuf.q_used, \
						sizeof(quotabuf.q_used))
/*
 *  modeto() - translate permissiopns
 */
 static int
modeto(short *num)
{
	char	*id = "mode translation";
	char	*str = &vsnbuf.v_mode[0];
	int	i;

	*num = 0;
 	for (i=0; i<3; i++) {
                if (str[i] == '0' ||  str[i] == '2' ||  str[i] ==  '4'
                        || str[i] == '6') {
                        *num *= 8;
                        *num += str[i] - 48;
                }
                else {
                        VMError = E_DBASE;
                        nLogMsg(EL_ERROR1, id, "bad octal digit: (%c)",
                                                        str[i]);
                        return(1);
                }
        }
        return(0);
}

/***********************************************************
						SETINT()
 ***********************************************************/

/*
 *  SetInt() - translate int into char field
 *	return: 0 if ok, otherwise 1 & set VMerror & EL_CODE
 */

 static int
SetInt(char *str, int num, int size)
{
	char	*id = "SetInt";
	char	arr[32];
	int	len;

	sprintf(arr, "%d", num);
	len = strlen(arr);

	if (len > size) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%d: int len %d > reserved %d", 
						num, len, size);
		return(1);
	}
	sprintf(str, "%*d", size, num);
	str[size] = ' ';
	return(0);
}

#define SetIntUidVsn( uid ) SetInt( vsnbuf.v_uid, uid, sizeof(vsnbuf.v_uid))
#define SetIntVolType( type ) SetInt( vsnbuf.v_voltype, type, \
						sizeof(vsnbuf.v_voltype))
#define SetIntGid( gid ) SetInt( vsnbuf.v_gid, gid, sizeof(vsnbuf.v_gid))
#define SetIntSrcRC( rc ) SetInt( vsnbuf.v_srcrc, rc, sizeof(vsnbuf.v_srcrc))
#define SetIntDstRC( rc ) SetInt( vsnbuf.v_dstrc, rc, sizeof(vsnbuf.v_dstrc))
#define SetIntMntWrite( w ) SetInt( vsnbuf.v_mntwrite, w, \
						sizeof(vsnbuf.v_mntwrite))
#define SetIntMntRead( r ) SetInt( vsnbuf.v_mntread, r, \
						sizeof(vsnbuf.v_mntread))
#define SetIntMntFail( w ) SetInt( vsnbuf.v_mntfail, w, \
						sizeof(vsnbuf.v_mntfail))
#define SetIntPoolGid( g ) SetInt( vsnbuf.v_poolgid, g, \
						sizeof(vsnbuf.v_poolgid))
#define SetIntRecycle( r ) SetInt( vsnbuf.v_recycle, r, \
						sizeof(vsnbuf.v_recycle))

#define SetIntUidQuo( uid ) SetInt( quotabuf.q_uid, uid, sizeof(quotabuf.q_uid))
#define SetIntRCQuo( rc ) SetInt( quotabuf.q_rc, rc, sizeof(quotabuf.q_rc))
#define SetIntQuota( q ) SetInt( quotabuf.q_quota, q, sizeof(quotabuf.q_quota))
#define SetIntUsed( u )  SetInt( quotabuf.q_used, u, sizeof(quotabuf.q_used))

/***********************************************************
					   INCINT()
 ***********************************************************/

/*
 *  IncInt() - increment counter
 */

 static void
IncInt(char *str, int size)
{
	char	*id = "IncInt";
	int	n;

	if (ToInt(&n, str, size))
		nLogMsg(EL_FATAL, id, "vsn dbase corruption");

	if (n < 0)
		n--;
	else
		n++;

	if (SetInt(str, n, size))
		nLogMsg(EL_FATAL, id, "vsn dbase corruption");
}

#define IncMountWrite()	IncInt(vsnbuf.v_mntwrite, sizeof(vsnbuf.v_mntwrite))
#define IncMountRead()	IncInt(vsnbuf.v_mntread, sizeof(vsnbuf.v_mntread))
#define IncMountFail()	IncInt(vsnbuf.v_mntfail, sizeof(vsnbuf.v_mntfail))



/***********************************************************
						SETVSNMODE()
 ***********************************************************/

/*
 *  setvsnmode() - set mode in buffer in octal
 */

 static int
setvsnmode(short mode)
{
	char	*id = "setvsnmode";

	if (mode != mode & 0666) {
		nLogMsg(EL_ERROR2, id, "bad permissions: %o", mode);
		return(1);
	}
	sprintf(vsnbuf.v_mode, "%.3o", mode);
	vsnbuf.b15 = ' ';
	return(0);
}

/***********************************************************
						TBL_OP()
 ***********************************************************/

/*
 *  tbl_op() - seek and read or write an indexed table file
 *		if error, logs error & returns 1 
 */

#define OP_READ		111
#define OP_WRITE	222
#define TBL_EOF		333
#define TBL_ERR		999

 static int
tbl_op(int fd, int op)
{
	char	*id = "dbop";
	OFF_T	addr;
	char	*buf;
	int	size, ret;

	/*
	 *  it is assumed that the file offset has been set by a 
	 *	table-specific means (e.g index operation) and that
	 *	(for write) the buffer has been set up.
	 */

	if (fd == db_vsn_fd) {
		addr = vsnaddr;
		buf = (char *) &vsnbuf;
		size = sizeof(vsnbuf);
	} else if (fd == db_quota_fd) {
		addr = quotaddr;
		buf = (char *) &quotabuf;
		size = sizeof(quotabuf);
	} else
		nLogMsg(EL_FATAL, id, "programming error fd %d", fd);
	if (LSEEK(fd, addr, SEEK_SET) != addr) {
		VMError = E_DBASE;
		nLogMsg(EL_ERROR1|EL_CONSOLE0|EL_SYS|EL_CODE, id,
				"table LSEEK error");
		return(TBL_ERR);
	}
	switch (op) {
	    case OP_READ:
		ret = read(fd, buf, size);
		break;
	    case OP_WRITE:
		ret = write(fd, buf, size);
		break;
	    default:
			nLogMsg(EL_FATAL, id, "programming error op %d", op);
	}
	if (!ret  &&  op == OP_READ) {

		/* 
		 *  this should only happen when reading a whole table 
		 *	into memory
		 */

		nLogMsg(EL_STATUS2, id, "end of file");
		return(TBL_EOF);
	}
	if (ret != size) {
		if (ret == -1)
			 nLogMsg(EL_ERROR1|EL_SYS, id, "db table %s ",
				(op == OP_READ) ? "read" : "write");
		else
			 nLogMsg(EL_ERROR1, id, 
				"db table %s: %d bytes, attempted %d", 
				(op == OP_READ) ? "read" : "write", size);
		VMError = E_DBASE;
		return(TBL_ERR);
	}
	return(0);
}

/***********************************************************
						STARTINDEX()
 ***********************************************************/

/*
 *  StartIndex() - open it if it exists, otherwise make it
 */

 static int
StartIndex(char *name, DB **ix, int flags)
{
	char	*id = "StartIndex";
	int	ret, err;

	/*
	**	keep track of whether index is being created
	*/
	if (stat(name, &sbuf) == -1) {
		if (errno != ENOENT)
			nLogMsg(EL_FATAL|EL_SYS, id, 
					"index (%s) stat failed:", name);
		nLogMsg(EL_STATUS2, id, "creating %sindex (%s)", 
				(flags & DB_DUP ? "" : "exclusive "), name);
		ret = 1;
	} else
		ret = 0;
nLogMsg(EL_STATUS1, id, "db_create..");
	if (dberr = db_create(ix, NULL, 0))
		nLogMsg(EL_FATAL, id, "db_create(): %s", 
						db_strerror(dberr));

	if (flags & DB_DUP) {
nLogMsg(EL_STATUS1, id, "set_flags ..");
		if (dberr = (*ix)->set_flags(*ix, DB_DUP))
			nLogMsg(EL_FATAL, id, "db->set_flags(%s): %s", name,
						db_strerror(dberr));
	}
nLogMsg(EL_STATUS1, id, "open %s ..", name);
	if (dberr = (*ix)->open(*ix, name, NULL, DB_BTREE, 
					DB_CREATE|(flags & DB_RDONLY), 0600))
		nLogMsg(EL_FATAL, id, "db->open(%s): %s", name,
						db_strerror(dberr));

	return(ret);
}

/***********************************************************
						CHECKVSN()
 ***********************************************************/

/*
 *  CheckVsn() - make sure vsnbuf.v_vsn ok
 */

 static int
CheckVsn()
{
	char	*id = "CheckVsn";
	int	i;

	if (vsnbuf.b01 != ' ') {
		nLogMsg(EL_CODE|EL_ERROR1, id, 
					"%.7s: no space seperator after vsn", 
					vsnbuf.v_vsn);
		return(1);
	}
	for (i=0; i<VSNLEN; i++) {
		if (vsnbuf.v_vsn[i] == ' ') {
			nLogMsg(EL_CODE|EL_ERROR1, id, "vsn %.*s: too short", 
							VSNLEN, vsnbuf.v_vsn);
			return(1);
		}
		if (!isalnum(vsnbuf.v_vsn[i])) {
			nLogMsg(EL_CODE|EL_ERROR1, id, 
					"vsn %.*s: Illegal character (%c)",
					VSNLEN, vsnbuf.v_vsn, vsnbuf.v_vsn[i]);
			return(1);
		}
	}
	return(0);
}

/***********************************************************
 						PRINTVSN()
 ***********************************************************/

/*
 *  printvsn() - print contents of vsnbuf
 */

 static void
printvsn()
{
        char    buf[sizeof(vsnbuf)];

        memcpy(buf, &vsnbuf, sizeof(vsnbuf));

	nLogMsg(EL_STATUS2, 0, "VSN RECORD:\n%.*s", sizeof(vsnbuf), buf);
}	
	
/***********************************************************
						ADDQUOTA()
 ***********************************************************/

/*
 *  AddQuota() - add user rc quota to table
 */

 static int
AddQuota(int uid, int rc, int quot, int used)
{
	char		*id = "AddQuota";
	OFF_T		addr;
	int		ret;

	/*
	 *  get addr & check it
	 */

	if ((addr = LSEEK(db_quota_fd, 0, SEEK_END)) == -1) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_SYS, id, 
						"dbase quota table seek");
		return(1);
	}
	if (addr % sizeof(quotabuf)) {	/* consistency check */
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0, id, "dbase quota table bad size");
		return(1);
	}

	/*
	 *  fill in the record & write it
	 */

	memset((char *) &quotabuf, ' ', sizeof(quotabuf));
	if (SetIntUidQuo(uid)
	    || SetIntRCQuo(rc)
	    || SetIntQuota(quot)
	    || SetIntUsed(used))
		return(1);

	quotabuf.end = '\n';
	nLogMsg(EL_STATUS1, id, "adding user quota \n\t%.*s", 
					sizeof(quotabuf), (char *) &quotabuf);
	if ((ret = write(db_quota_fd, (char *) &quotabuf, sizeof(quotabuf))) 
			!= sizeof(quotabuf)) {
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"quota table wrote %d instead of %d", 
				ret, sizeof(quotabuf));
		return(1);
	}

	/*
	 *  add to quota index
	 */

	FILL_DBT(data, addr);
	sprintf(quotakey, "%d_%d", uid, rc);
	STR_DBT(key, quotakey);
	if (dberr = db_quota_idx->put(db_quota_idx, NULL, &key, &data, 
							DB_NOOVERWRITE)) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"quota put(%s): %s", 
						quotakey, db_strerror(dberr));
		return(1);
	}
	return(0);
}
/***********************************************************
 					UPDATEQUOTAUSED()
 ***********************************************************/

/*
 *  UpdateQuotaUsed() - count uid_rc entries in temporary
 *	index built from reading vsntable, and check/update
 *	quota used
 */

 static void
UpdateQuotaUsed(AVL_IX_DESC *avl_idx)
{
	char		*id = "UpdateQuotaUsed";
	int		ret, uid, rc;
	char		buf[sizeof(AVL_IX_REC) + 100];
	AVL_IX_REC	*avl_entry = (AVL_IX_REC *)buf;

	/*
	**	start at head of index
	*/
	avl_first_key(avl_idx);

	/*
	**	process entries
	*/
	while ((ret=avl_next_key(avl_entry, avl_idx)) == AVL_IX_OK) {
		nLogMsg(EL_DEBUG1, id, "uid_rc %s  used %d",
					avl_entry->key, avl_entry->count);

		STR_DBT(key, avl_entry->key);
		ZERO_OBJ(data);
		dberr = db_quota_idx->get(db_quota_idx, NULL, &key, &data, 0);
		if (dberr == 0) {
				
		    	/*
		    	**	uid_rc quota exists; read, check,
		    	**		update if necc
		    	*/
			COPY_DBT(quotaddr, data);

		    	if (tbl_op(db_quota_fd, OP_READ)) 
				nLogMsg(EL_FATAL, id, "quitting");
		    	if (ToIntUidQuo(&quota.vqq_uid) ||
			    	    ToIntRCQuo(&quota.vqq_rc)   ||
			    	    ToIntQuota(&quota.vqq_quota)||
			    	    ToIntUsed(&quota.vqq_used))
					nLogMsg(EL_FATAL, id, "quitting");
			if (quota.vqq_used != avl_entry->count) {
				nLogMsg(EL_CONSOLE0|EL_STATUS2, id,
					  "updating quota used %d_%d: %d->%d",
					  quota.vqq_uid, quota.vqq_rc,
					  quota.vqq_used, avl_entry->count);
				if (SetIntUsed(avl_entry->count))
					nLogMsg(EL_FATAL, id, "quitting");
				if (tbl_op(db_quota_fd, OP_WRITE))
					nLogMsg(EL_FATAL, id, "quitting");
			}

		} else if (dberr == DB_NOTFOUND)  {

			/*
			**	no entry - extract uid, rc & make one
			*/
			if ((ret= sscanf(avl_entry->key, "%d_%d", &uid, &rc)) 
									!= 2)
				nLogMsg(EL_FATAL, id, "sscan %d", ret);
			nLogMsg(EL_CONSOLE0|EL_STATUS2, id, 
					"adding entry for uid %d rc %d",
					uid, rc);
			if (AddQuota(uid, rc, 0, avl_entry->count))
				nLogMsg(EL_FATAL, id, "quitting");
		} else
			nLogMsg(EL_FATAL, id, "get(%s): %s", avl_entry->key,
						db_strerror(dberr));
	}
	if (ret != AVL_EOIX)
		nLogMsg(EL_FATAL, id, "avl_next_key() failure");
}

/***********************************************************
 					REBUILDVSNVSNINDEX()
 ***********************************************************/

/*
 *  RebuildVsnVsnIndex() - read vsn table & build vsn index 
 */

 static void
RebuildVsnVsnIndex()
{
	char		*id = "RebuildVsnVsnIndex";
	int		i, ret;

	nLogMsg(EL_DEBUG1, id, "pid %d", getpid());

	/*
	**	open vsn table and index
	*/

	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
						"vsn table (%s) open failed", 
						dbf_vsn_tbl);
		exit(2);
	}

	if (StartIndex(dbf_vsn_vsn_idx, &db_vsn_vsn_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "what th");
		exit(2);
	}
	
	/*
	**	read the whole table
	*/
	vsnaddr = 0;
	for (i=1;;i++) {

		ret = read(db_vsn_fd, &vsnbuf, sizeof(vsnbuf));
		if (ret != sizeof(vsnbuf)) {
			if (ret == 0)
				break;
			if (ret == -1)
				nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
						"%s: failure at record %d",
						dbf_vsn_tbl, i);
			else 
				nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
						"%s: incomplete record %d",
						dbf_vsn_tbl, i);
			exit(2);
		}

		/*
		**	see if record is really there
		*/

		if (vsnbuf.v_status == S_DELETED) {
			vsnaddr += sizeof(vsnbuf);
			continue;
		}

		/*
		**	check vsn part of record & end of record
		*/

		if (CheckVsn()) {
			printvsn();
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"%s: bad record %d", dbf_vsn_tbl, i);
			exit(2);
		}
		if (vsnbuf.end != '\n') {
			printvsn();
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"%s: bad record %d terminator", 
					dbf_vsn_tbl, i);
			exit(2);
		}

		/*
		**	add to vsn index
		*/
		FILL_DBT(data, vsnaddr);
		FILL_DBT(key, vsnbuf.v_vsn);
		if (dberr = db_vsn_vsn_idx->put(db_vsn_vsn_idx, NULL, 
							&key, &data, 
							DB_NOOVERWRITE)) {
			VMError = E_DBASE;
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
			  "%s record %d: put() - probable duplicate (%s)",
				vsnstr(vsnbuf.v_vsn), vsnaddr/sizeof(vsnbuf),
				db_strerror(dberr));
		}
		vsnaddr += sizeof(vsnbuf);
		if (!(i%25000))
			nLogMsg(EL_DEBUG1, id, "record %d..", i);
	}

	/*
	**	close/flush index
	*/
	if (dberr = db_vsn_vsn_idx->close(db_vsn_vsn_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_vsn_vsn_idx,
						db_strerror(dberr));
		exit(2);
	}
	nLogMsg(EL_STATUS1, id, "vsn_vsn index build complete");
	exit(0);
}

/***********************************************************
 					REBUILDVSNUIDINDEX()
 ***********************************************************/

/*
 *  RebuildVsnUidIndex() - read vsn table & build vsn_uid index &
 *	initialize counts as requested. This routine does more record
 *	checking than the others.
 */

 static void
RebuildVsnUidIndex(int index, int counts)
{
	char		*id = "RebuildVsnUidIndex";
	int		i, ret, deleted = 0;
	RPRC_t		*rcp;
	AVL_IX_DESC	avl_idx;
	char		buf[sizeof(AVL_IX_REC) + 100];
	AVL_IX_REC	*avl_entry = (AVL_IX_REC *) buf; 

	nLogMsg(EL_DEBUG1, id, "pid %d", getpid());

	/*
	 *  open vsn table and uid index
	 */

	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
						"vsn table (%s) open failed", 
						dbf_vsn_tbl);
		exit(2);
	}
	if (StartIndex(dbf_vsn_uid_idx, &db_vsn_uid_idx, DB_DUP)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "what th");
		exit(2);
	}
	if (counts) {

		/*
		 *  open quota index too
		 */

		if (StartIndex(dbf_quo_idx,  &db_quota_idx, 0)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "what th");
			exit(2);
		}
		/*
		**  	create in-memory 'counting index' for tracking 
		**		quota used 
		*/
		avl_create_index(&avl_idx, AVL_COUNT_DUPS, 0);
	}

	/*
	**	now read the whole vsn table
	*/
	vsnaddr = 0;
	for (i=0;;i++) {

		ret = read(db_vsn_fd, &vsnbuf, sizeof(vsnbuf));
		if (ret != sizeof(vsnbuf)) {
			if (ret == 0)
				break;
			if (ret == -1)
				nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
						"%s: failure at record %d",
						dbf_vsn_tbl, i);
			else 
				nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
						"%s: incomplete record %d",
						dbf_vsn_tbl, i);
			exit(2);
		}

		/*
		**	translate record parts that aren't looked at elsewhere
		*/
		if ( ToIntUidVsn(&vinfo.vi_uid) ||
		     ToIntGid(&vinfo.vi_gid)    ||
		     modeto(&vinfo.vi_mode)     ||
		     ToIntSrcRC(&vinfo.vi_srcrc)||
		     ToIntDstRC(&vinfo.vi_dstrc)) {
			printvsn();
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"record %d: bad field", i);
			exit(2);
		}

		/*
		**	see if it's really there
		*/
		if (vsnbuf.v_status == S_DELETED) {
			nLogMsg(EL_STATUS1, id, "skipped %s (deleted)", 
							vsnstr(vsnbuf.v_vsn));
			deleted++;
			vsnaddr += sizeof(vsnbuf);
			continue;
		}
		if (vsnbuf.v_use != U_IDLE)
			nLogMsg(EL_DEBUG1, id, "%s: use %c (status %c rc %d)",
					vsnstr(vsnbuf.v_vsn), vsnbuf.v_use, 
					vsnbuf.v_status, vinfo.vi_srcrc);

		if (index) {
			/*
			**	add to index 
			*/
			sprintf(vsnkey, 
#ifdef _BIG_FS
						"%04x_%llx", 
#else
						"%04x_%lx", 
#endif
						vinfo.vi_uid, vsnaddr);
			STR_DBT(key, vsnkey);
			FILL_DBT(data, vsnaddr);
			if (dberr = db_vsn_uid_idx->put(db_vsn_uid_idx, NULL,
						&key, &data, DB_NOOVERWRITE)) {
				nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
					"put(uid %s): %s", vsnkey, 
					db_strerror(dberr));
				exit(2);
			}
		}
		if (counts) {
			/*
			**  	keep track of counts
			*/
			rcp = RCInfo(vinfo.vi_srcrc);
			if (rcp != NULL) {
			    if (rcp->rc_flags & RCF_HASQUOTA) {
				avl_entry->recptr = 0;
				sprintf(avl_entry->key, "%d_%d", 
					vinfo.vi_uid, vinfo.vi_srcrc);
				if (avl_add_key(avl_entry, &avl_idx) 
							== AVL_IX_FAIL) {
					nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"avl_add_key(uid_rc %s) failed",
						avl_entry->key);
					exit(2);
				}
			    }
			}
		}
		vsnaddr += sizeof(vsnbuf);
		if (!(i%25000))
			nLogMsg(EL_DEBUG1, id, "record %d..", i);
	}
	if (deleted)
		nLogMsg(EL_STATUS1, id, 
			"total %d vsn records, skipped %d deleted record%c", 
			i, deleted, deleted>1 ? 's':' ');
	else 
		nLogMsg(EL_STATUS1, id, "total %d vsn records", i);

	if (counts) {

		/*
		**	update quota used table & throw away temp index
		*/
		UpdateQuotaUsed(&avl_idx);
		avl_destroy_index(&avl_idx);

		nLogMsg(EL_STATUS1, id, "-i option completed");
	}

	/*
	**	close/flush index
	*/
	if (dberr = db_vsn_uid_idx->close(db_vsn_uid_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_vsn_uid_idx,
						db_strerror(dberr));
		exit(2);
	}
	nLogMsg(EL_STATUS1, id, "vsn table examination complete");
	exit(0);
}

/***********************************************************
 					REBUILDVSNLABELINDEX()
 ***********************************************************/

/*
 *  RebuildVsnLabelIndex() - read vsn table & build label_uid index
 */

 static void
RebuildVsnLabelIndex()
{
	char		*id = "RebuildVsnLabelIndex";
	int		i, ret;

	nLogMsg(EL_DEBUG1, id, "pid %d", getpid());

	/*
	 *  open vsn table and label index
	 */

	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
						"vsn table (%s) open failed", 
						dbf_vsn_tbl);
		exit(2);
	}
	if (StartIndex(dbf_vsn_lab_idx, &db_vsn_lab_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "what th");
		exit(2);
	}
	/*
	 *  read the whole table
	 */
	vsnaddr = 0;
	for (i=1;;i++) {

		ret = read(db_vsn_fd, &vsnbuf, sizeof(vsnbuf));
		if (ret != sizeof(vsnbuf)) {
			if (ret == 0)
				break;
			if (ret == -1)
				nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
						"%s: failure at record %d",
						dbf_vsn_tbl, i);
			else 
				nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
						"%s: incomplete record %d",
						dbf_vsn_tbl, i);
			exit(2);
		}

		/*
		 *  see if it's really there
		 */

		if (vsnbuf.v_status == S_DELETED)  {
			vsnaddr += sizeof(vsnbuf);
			continue;
		}

		/*
		 *  get uid
		 */

		if (ToIntUidVsn(&vinfo.vi_uid) ) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"bad record field %d", i);
			exit(2);
		}

		if (vsnbuf.v_label[0] != ' ') {
			sprintf(vsnkey, "%s_%d", 
					vsnstr(vsnbuf.v_label), vinfo.vi_uid);
			STR_DBT(key, vsnkey);
			FILL_DBT(data, vsnaddr);
			if (dberr = db_vsn_lab_idx->put(db_vsn_lab_idx, NULL,
						&key, &data, DB_NOOVERWRITE)) {
				nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
					"put(ilbl_uid %s): %s", vsnkey, 
					db_strerror(dberr));
			   	nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"--probably a duplicate label for uid");
			}
		}
		vsnaddr += sizeof(vsnbuf);
		if (!(i%25000))
			nLogMsg(EL_DEBUG1, id, "record %d..", i);
	}

	/*
	**	close/flush index
	*/
	if (dberr = db_vsn_lab_idx->close(db_vsn_lab_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_vsn_lab_idx,
						db_strerror(dberr));
		exit(2);
	}
	nLogMsg(EL_STATUS1, id, "vsn table examination complete");
	exit(0);
}

/***********************************************************
 				    REBUILDVSNSTATUSINDEX()
 ***********************************************************/

/*
 *  RebuildVsnStatusIndex() - read vsn table & build status index
 */

 static void
RebuildVsnStatusIndex()
{
	char		*id = "RebuildVsnStatusIndex";
	int		i, ret;

	nLogMsg(EL_DEBUG1, id, "pid %d", getpid());

	/*
	**	open vsn table and status index
	*/

	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
						"vsn table (%s) open failed", 
						dbf_vsn_tbl);
		exit(2);
	}
	if (StartIndex(dbf_vsn_sta_idx, &db_vsn_sta_idx, DB_DUP)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "what th");
		exit(2);
	}

	/*
	**	read the whole table
	*/

	vsnaddr = 0;
	for (i=1;;i++) {

		ret = read(db_vsn_fd, &vsnbuf, sizeof(vsnbuf));
		if (ret != sizeof(vsnbuf)) {
			if (ret == 0)
				break;
			if (ret == -1)
				nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
						"%s: failure at record %d",
						dbf_vsn_tbl, i);
			else 
				nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
						"%s: incomplete record %d",
						dbf_vsn_tbl, i);
			exit(2);
		}

		/*
		 *  see if it's really there
		 */

		if (vsnbuf.v_status == S_DELETED) {
			vsnaddr += sizeof(vsnbuf);
			continue;
		}
		sprintf(vsnkey, 
#ifdef _BIG_FS
						"%c_%llx", 
#else
						"%c_%lx", 
#endif
						vsnbuf.v_status, vsnaddr);
		STR_DBT(key, vsnkey);
		FILL_DBT(data, vsnaddr);
		if (dberr = db_vsn_sta_idx->put(db_vsn_sta_idx, NULL,
						&key, &data, DB_NOOVERWRITE)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"put(status %s): %s", vsnkey,
					db_strerror(dberr));
			exit(2);
		}

		vsnaddr += sizeof(vsnbuf);
		if (!(i%25000))
			nLogMsg(EL_DEBUG1, id, "record %d..", i);
	}

	/*
	**	close/flush index
	*/
	if (dberr = db_vsn_sta_idx->close(db_vsn_sta_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_vsn_sta_idx,
						db_strerror(dberr));
		exit(2);
	}
	nLogMsg(EL_STATUS1, id, "vsn table examination complete");
	exit(0);
}

/***********************************************************
   					     CLOSEINDEXES()
 ***********************************************************/

/*
**	CloseIndexes() - close indexes 
*/

 static int
CloseIndexes()
{
	char	*id = "CloseIndexes";

	/* vsn table */

	if (dberr = db_vsn_vsn_idx->close(db_vsn_vsn_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_vsn_vsn_idx,
						db_strerror(dberr));
		return(1);
	}
	if (dberr = db_vsn_uid_idx->close(db_vsn_uid_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_vsn_uid_idx,
						db_strerror(dberr));
		return(1);
	}
	if (dberr = db_vsn_lab_idx->close(db_vsn_lab_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_vsn_lab_idx,
						db_strerror(dberr));
		return(1);
	}
	if (dberr = db_vsn_sta_idx->close(db_vsn_sta_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_vsn_sta_idx,
						db_strerror(dberr));
		return(1);
	}

	/*  quota index  */

	if (dberr = db_quota_idx->close(db_quota_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db close(%s): %s",
						dbf_quo_idx,
						db_strerror(dberr));
		return(1);
	}
	return(0);
}

/***********************************************************
   					 REBUILDVSNINDEXES()
 ***********************************************************/

/*
 *  RebuildVsnIndexes() - close indexes, fork rebuilders, open indexes
 *	The rebuild is multi-threaded for speed. Each process has its
 *	own copy of the db3 index space, so all indexes are first
 *	closed and each process just opens the index(es) it needs and 
 *	closes it/them when done to flush the changes (this is done
 *	done in the RebuildVsnXXXIndex() routines). Then the indexes
 *	are all reopened to absorb the changes.
 */
 
 static void
RebuildVsnIndexes(int vsnvsn, int vsnusr, int initcts, int vsnlab, int vsnsta)
{
	char	*id = "RebuildVsnIndexes";
	int	i, nprocs = 0, pid, status, sig;

	nLogMsg(EL_DEBUG1, id, "vsn %d uid %d counts %d lab %d sta %d",
 			vsnvsn, vsnusr, initcts, vsnlab, vsnsta);

	/*
	**	scope the size of the vsn table
	*/
	if ((vsnaddr = LSEEK(db_vsn_fd, 0, SEEK_END)) == -1) 
		nLogMsg(EL_FATAL|EL_SYS, id, "table LSEEK (%s) failed", 
					dbf_vsn_tbl);
	if (vsnaddr%sizeof(vsnbuf))
		nLogMsg(EL_FATAL|EL_SYS, id, "table (%s) size %d not mod %d", 
					dbf_vsn_tbl, vsnaddr, sizeof(vsnbuf));
	nLogMsg(EL_STATUS1, id, "%d entries in %s", 
				(int)(vsnaddr/(OFF_T)sizeof(vsnbuf)),
				dbf_vsn_tbl);

	/*
	**	close it so each proc can open w/ unique file table entry
	*/
	if (close(db_vsn_fd) != 0) 
		nLogMsg(EL_FATAL|EL_SYS, id, "table LSEEK (%s) failed: %s", 
					dbf_vsn_tbl);
		
	/*
	**	close indexes to free buffer space
	*/
	if (CloseIndexes())
		nLogMsg(EL_FATAL, id, "quitting");

	/*
	**	fork appropriate index building processes
	*/
	if (vsnusr  ||  initcts) {
		nprocs++;
		if ((i=fork()) == 0)
			RebuildVsnUidIndex(vsnusr, initcts);
		nLogMsg(EL_STATUS1, id, "forked %d RebuildVsnUidIndex", i);
	}
	if (vsnlab) {
		nprocs++;
		if ((i=fork()) == 0)
			RebuildVsnLabelIndex();
		nLogMsg(EL_STATUS1, id, "forked %d RebuildVsnLabelIndex", i);
	}
	if (vsnvsn) {
		nprocs++;
		if ((i=fork()) == 0)
			RebuildVsnVsnIndex();
		nLogMsg(EL_STATUS1, id, "forked %d RebuildVsnVsnIndex", i);
	}
	if (vsnsta) {
		nprocs++;
		if ((i=fork()) == 0)
			RebuildVsnStatusIndex();
		nLogMsg(EL_STATUS1, id, "forked %d RebuildVsnStatusIndex", i);
	}

	/*
	**	wait for children & check status
	*/

	for (i=0; i<nprocs; i++) {
		int	k;
		for (k=0; k<5; k++) {
			if ((pid = wait(&status)) != -1) 
				break;
			nLogMsg(EL_SYS|EL_CONSOLE0|EL_ERROR1, id, 
								"wait error ");
		}
		if (pid == -1)
			kill(0,9);
		status &= 0177777;
		sig = status & 0177;
		status >>= 8;
		if (sig) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
				"child %d terminated on signal %d - shutdown",
				pid, sig);
			kill(0,9);
		}
		if (status) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
				"child %d exited abnormally (%d) - shutdown",
				pid, status);
			kill(0,9);
		}
	}

	/*
	**	reopen vsn table & indexes
	*/
	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDWR|O_SYNC, 0))== -1)
		nLogMsg(EL_FATAL|EL_SYS, id, 
				"vsn table (%s) open failed: %s", dbf_vsn_tbl);
	if (StartIndex(dbf_vsn_vsn_idx, &db_vsn_vsn_idx, 0)  ||
	    StartIndex(dbf_vsn_lab_idx, &db_vsn_lab_idx, 0)  ||
	    StartIndex(dbf_vsn_uid_idx, &db_vsn_uid_idx, DB_DUP)  ||
	    StartIndex(dbf_vsn_sta_idx, &db_vsn_sta_idx, DB_DUP)  ||
	    StartIndex(dbf_quo_idx,     &db_quota_idx, 0))
			nLogMsg(EL_FATAL, id, "index should exist; quitting");
				
				
	nLogMsg(EL_DEBUG1, id, "done");
}

/***********************************************************
  					REBUILDQUOTAINDEX()
 ***********************************************************/

/*
**	RebuildQuotaIndex() - read quota tbl & build uid_rc index
*/
 static void
RebuildQuotaIndex()
{
	char	*id = "RebuildQuotaIndex";
	int	ret;

	quotaddr = 0;
	for (;;) {
		ret = tbl_op(db_quota_fd, OP_READ);
		switch (ret) {
		    case 0:
			break;
		    case TBL_EOF:
			return;
		    default:
			nLogMsg(EL_FATAL, id, "%s: failure", dbf_quo_tbl);
		}
		if ( ToIntUidQuo(&quota.vqq_uid)  ||
		     ToIntRCQuo(&quota.vqq_rc)    ||
		     ToIntQuota(&quota.vqq_quota) || 
		     ToIntUsed(&quota.vqq_used))
			nLogMsg(EL_FATAL, id,
				"dbase table translation error on \n%.*s", 
				sizeof(quotabuf), (char *) &quotabuf);

		sprintf(quotakey, "%d_%d", quota.vqq_uid, quota.vqq_rc);
		STR_DBT(key, quotakey);
		FILL_DBT(data, quotaddr);
		if (dberr = db_quota_idx->put(db_quota_idx, NULL,
						&key, &data, DB_NOOVERWRITE)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1|EL_FATAL, id, 
					"put(status %s): %s", quotakey,
					db_strerror(dberr));
		}
		quotaddr += sizeof(quotabuf);
	}
}


/***********************************************************
					  VSNADDRTOVINFO()
 ***********************************************************/
/*
 *  VsnAddrToVinfo() - using global vsnaddr, read record &
 *	fill in vinfo.
 *	return 0 if no error, otherwise set VMError & EL_CODE 
 *	and return 1
 */

 static int
VsnAddrToVinfo(char *vsn)
{
	char	*id = "VsnAddrToVinfo";

	/*
	**	read record from table
	*/

	if (tbl_op(db_vsn_fd, OP_READ)) {
		VMError = E_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"dbase table error %s", vsnstr(vsn));
		return(1);
	}

	/*
	**	basic record sanity check & translation to vinfo
	*/

	if (vsnbuf.end != '\n') {
		VMError = E_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
			"dbase index error on (%*s) vsn field (%.*s): %s",
			VSNLEN, vsn,
			VSNLEN, vsnbuf.v_vsn,
			"bad record terminator");
		return(1);
	}
	memcpy((char *)&vinfo.vi_vsn, vsnbuf.v_vsn, VSNLEN);

	/*
	**	poetic differentiation for virgins
	*/
	if (vsnbuf.v_status == S_VIRGIN)
		memset((char *)&vinfo.vi_label,   0, VSNLEN);
	else if (vsnbuf.v_label[0] == ' ')
		memset((char *)&vinfo.vi_label, ' ', VSNLEN);
	else
		memcpy((char *)&vinfo.vi_label, vsnbuf.v_label, VSNLEN);

	if (ToIntUidVsn(&vinfo.vi_uid) 
	    || ToIntGid(&vinfo.vi_gid)   
	    ||  modeto(&vinfo.vi_mode)
	    ||  ToIntVolType(&vinfo.vi_voltype)
	    ||  ToIntPoolGid(&vinfo.vi_poolgid)
	    ||  ToIntSrcRC(&vinfo.vi_srcrc)
	    ||  ToIntDstRC(&vinfo.vi_dstrc)) {
		VMError = E_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
			"dbase index error on (%*s) vsn field (%.*s): %s",
			VSNLEN, vsn,
			VSNLEN, vsnbuf.v_vsn,
			"dbase translation error");
		printvsn();
		return(1);
	}

	return(0);
}

/***********************************************************
  						INITDB()
 ***********************************************************/

/*
 *  DBInit() - open database & initialize indexes. It is assumed that
 *	another voldaemon cannot be running via the locking in BecomeVoldaemon.
 */

 void
DBInit(int initcounts)
{
	char		*id = "DBInit";
	int		ret;
	OFF_T		addr;
	int		cflag = O_RDWR|O_SYNC|O_CREAT;
	int		vsnvsn, vsnusr, vsnlab, vsnsta;

	/*
	 *  make sure compiled to use records that won't overlap
	 *	page boundary
	 */
	if (sizeof(vsnbuf) != 256)
		nLogMsg(EL_FATAL, id, "vsn record size (%d) != 256", 
							sizeof(vsnbuf));
	/*
	**	set up buffers
	*/
	memset(&vsnbuf, ' ', sizeof(vsnbuf));
	vsnbuf.end = '\n';
	memset(&quotabuf, ' ', sizeof(quotabuf));
	quotabuf.end = '\n';

	/*
	**	check dbase files & create if necc
	*/
	nLogMsg(EL_STATUS1, id, "DB dir is %s", vDBDir);

	sprintf(dbf_lock, "%s/DB_LOCK", vDBDir);

	sprintf(dbf_vsn_tbl, "%s/DB_VSN_TBL", vDBDir);
	sprintf(dbf_vsn_vsn_idx, "%s/DB_VSN_VSN_IDX", vDBDir);
	sprintf(dbf_vsn_uid_idx, "%s/DB_VSN_USR_IDX", vDBDir);
	sprintf(dbf_vsn_lab_idx, "%s/DB_VSN_LAB_IDX", vDBDir);
	sprintf(dbf_vsn_sta_idx, "%s/DB_VSN_STA_IDX", vDBDir);

	sprintf(dbf_quo_tbl, "%s/DB_QUO_TBL", vDBDir);
	sprintf(dbf_quo_idx, "%s/DB_QUO_IDX", vDBDir);

	if (stat(dbf_lock, &sbuf) != -1) {

		/*
		**	dbase wasn't closed properly - 
		**		prepare to rebuild indexes
		*/
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
				"DBASE NOT CLOSED - REBUILDING INDEXES");
		Unlink(dbf_vsn_vsn_idx);
		Unlink(dbf_vsn_lab_idx);
		Unlink(dbf_vsn_uid_idx);
		Unlink(dbf_vsn_sta_idx);
		Unlink(dbf_quo_idx);
	} else {

		/*
		**	db about to be set up, so lock
		*/
		if ((ret=open(dbf_lock, O_CREAT, 0600)) == -1)
			nLogMsg(EL_FATAL|EL_SYS, id, "create lock failed");
		close(ret);
	}

	nLogMsg(EL_DEBUG3, id, "open tables & indexes:");

	/* vsn table and associated indexes */

	if (stat(dbf_vsn_tbl, &sbuf) != -1) {

		/*
		**	table exists - open, check size & check 1st record
		*/

		if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDWR|O_SYNC, 0))== -1)
			nLogMsg(EL_FATAL|EL_SYS, id, 
				"vsn table (%s) open failed: %s", dbf_vsn_tbl);
		if ((addr = LSEEK(db_vsn_fd, 0, SEEK_END)) == -1) 
			nLogMsg(EL_FATAL|EL_SYS, id, 
					"table LSEEK (%s) failed: %s", 
					dbf_vsn_tbl);
		if (addr % sizeof(vsnbuf))	/* consistency check */
		  nLogMsg(EL_FATAL, id, 
#ifdef _BIG_FS
		 "table (%s) size %lld not consistent with records of %d: %d",
#else
		 "table (%s) size %ld not consistent with records of %d: %d",
#endif
					dbf_vsn_tbl, addr, sizeof(vsnbuf),
					addr % sizeof(vsnbuf));
		vsnaddr = 0;
		switch(tbl_op(db_vsn_fd, OP_READ)) {
		    case TBL_EOF:
			break;
		    case TBL_ERR:
			nLogMsg(EL_FATAL, id, 
				    "vsn table %s:\n\n\t\t%s\n\n", dbf_vsn_tbl,
				    "1st record read problem");
			break;
		    case 0:
			if (VsnAddrToVinfo("1strec"))
				nLogMsg(EL_FATAL, id, 
				    "vsn table %s:\n\n\t\t%s\n\n", dbf_vsn_tbl,
				    "1st record bad or wrong format");
			break;
		}
	} else {

		/*
		**	table doesn't exist
		*/
		if (errno != ENOENT)
			nLogMsg(EL_FATAL|EL_SYS, id, "table stat problem: %s:",
							dbf_vsn_tbl);
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
					"CREATING VSN TABLE %s AND INDEXES", 
					dbf_vsn_tbl);
		if ((db_vsn_fd = open(dbf_vsn_tbl, cflag, 0644))== -1)
			nLogMsg(EL_FATAL|EL_SYS, id, 
				"vsn table (%s) open/create failed:", 
								dbf_vsn_tbl);
		Unlink(dbf_vsn_vsn_idx);
		Unlink(dbf_vsn_uid_idx);
		Unlink(dbf_vsn_lab_idx);
		Unlink(dbf_vsn_sta_idx);
	}
	vsnvsn = StartIndex(dbf_vsn_vsn_idx, &db_vsn_vsn_idx, 0);
	vsnlab = StartIndex(dbf_vsn_lab_idx, &db_vsn_lab_idx, 0);
	vsnusr = StartIndex(dbf_vsn_uid_idx, &db_vsn_uid_idx, DB_DUP);
	vsnsta = StartIndex(dbf_vsn_sta_idx, &db_vsn_sta_idx, DB_DUP);

	/* quota table & its single index */

	if (stat(dbf_quo_tbl, &sbuf) != -1) {
		if ((db_quota_fd = open(dbf_quo_tbl, O_RDWR|O_SYNC, 0))== -1)
			nLogMsg(EL_FATAL|EL_SYS, id, 
				"quota table (%s) open failed:", dbf_quo_tbl);
		if ((addr = LSEEK(db_quota_fd, 0, SEEK_END)) == -1) 
			nLogMsg(EL_FATAL|EL_SYS, id, 
					"quota table (%s) LSEEK failed:", 
					dbf_quo_tbl);
		if (addr % sizeof(quotabuf))	/* consistency check */
			nLogMsg(EL_FATAL, id, 
#ifdef _BIG_FS
				"quota table (%s): size %lld not consistent",
#else
				"quota table (%s): size %ld not consistent",
#endif
				dbf_quo_tbl, addr);
	} else {
		if (errno != ENOENT)
			nLogMsg(EL_FATAL|EL_SYS, id, 
					"quota table (%s) stat problem: ",
					dbf_quo_tbl);
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
					"CREATING QUOTA TABLE %s AND INDEX", 
					dbf_quo_tbl);
		if ((db_quota_fd = open(dbf_quo_tbl, cflag, 0644))== -1)
			nLogMsg(EL_FATAL|EL_SYS, id, 
				"quota table (%s) open/create failed: ", 
								dbf_quo_tbl);
		Unlink(dbf_quo_idx);
	}
	if (StartIndex(dbf_quo_idx,  &db_quota_idx, 0))
		RebuildQuotaIndex();
	
	nLogMsg(EL_STATUS2, id, "tables opened / size checked");

	/*
	**	rebuild any vsn indexes & init counts
	*/
	if (vsnvsn + vsnusr + initcounts +  vsnlab + vsnsta) {
		RebuildVsnIndexes(vsnvsn, vsnusr, initcounts,
				  vsnlab, vsnsta);
	}
	dbopen++;
	nLogMsg(EL_STATUS2, "DBASE", "database started");
}

/***********************************************************
   						CLOSEDB()
 ***********************************************************/

/*
**	CloseDb() - close indexes & tables
*/
 void
CloseDb()
{
	char	*id = "closedb";

	if (!dbopen)
		return;

	CloseIndexes();

	/*
 	**	close tables
 	*/

	if (db_vsn_fd > -1) {
		if (close(db_vsn_fd) == -1)
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"%s: close() error", dbf_vsn_tbl);
	}
	if (db_quota_fd > -1) {
		if (close(db_quota_fd) == -1)
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"%s: close() error", dbf_quo_tbl);
	}
	if (unlink(dbf_lock) == -1)
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
					"unlink lock file (%s)", dbf_lock);

	nLogMsg(EL_STATUS1, id, "Database shut down");
}

/***********************************************************
						FILLINVDR()
 ***********************************************************/

 static void
fillinvdr()
{
	/*
	**	fill in non-vinfo parts of vdr
	*/
	if (ToIntMntWrite(&vdr.vdr_mntwrite) ||
	    ToIntMntRead(&vdr.vdr_mntread) ||
	    ToIntMntFail(&vdr.vdr_mntfail))
		nLogMsg(EL_FATAL, 0, "vsn table corruption - mnt[r/w/f]");

	vdr.vdr_use = vsnbuf.v_use;
	vdr.vdr_status = vsnbuf.v_status;
	vdr.vdr_creat = GetTime(vsnbuf.v_creat);
	vdr.vdr_sttim = GetTime(vsnbuf.v_sttim);
	vdr.vdr_mod = GetTime(vsnbuf.v_mtime);
	vdr.vdr_acc = GetTime(vsnbuf.v_atime);
	vdr.vdr_lastmov = GetTime(vsnbuf.v_movtm);
	strncpy(vdr.vdr_desc, vsnbuf.v_desc, DESCLEN);
}


 int
DBFetchVsnKey(char *vsn, int complain)
{
	char	*id = "DBFetchVsnKey";

	strcpy(vsnkey, vsnstr(vsn));
	STR_DBT(key, vsnkey);
	ZERO_OBJ(data);
	if (dberr = db_vsn_vsn_idx->get(db_vsn_vsn_idx, NULL, &key, &data, 0)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "get(%s): %s", vsnkey,
							db_strerror(dberr));
		VMError = E_USER;
		if (complain) {
			nLogMsg(EL_CODE|EL_ERROR2, id, 
					"Volser (%s) not in database", vsnkey);
		} else
			nLogMsg(EL_DEBUG1, id, 
					"Volser (%s) not in database", vsnkey);
		return(1);
	}
	COPY_DBT(vsnaddr, data);
	return(0);
}

/***********************************************************
						GETVSN()
 ***********************************************************/
/*
**	GetVsn() - get vsn record into vsnbuf and translate it
**		return 0 if no error, otherwise set VMError & EL_CODE 
**		and return 1
*/
 static int
GetVsn(char *vsn, int complain)
{
	char	*id = "gvsn";

	/*
	**	look up vsn in index, load & translate
	*/
	if (DBFetchVsnKey(vsn, complain))
		return(1);

	if (VsnAddrToVinfo(vsn))
		return(1);

	/*
	**	check record
	*/
	if (strncmp(vsnbuf.v_vsn, vsn, VSNLEN)) {
		VMError = E_DBASE;
		nLogMsg(EL_ERROR1, id, "vsnbuf (%s)", vsnstr(vsnbuf.v_vsn));
		nLogMsg(EL_ERROR1, id, "key (%.*s)", key.size, key.data);
		nLogMsg(EL_ERROR1, id, "vsn (%s)", vsnstr(vsn));

		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
				"dbase index error on (%.*s): got (%s)", 
				key.size, key.data, vsnstr(vsnbuf.v_vsn));
		return(1);
	}
	return(0);
}

/***********************************************************
 						SETVSNTYPE()
 ***********************************************************/

 static int
SetVsnType(int type)
{
	char	*id = "SetVsnType";

	if (vVolType(type) == NULL) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR1, id, "vsn type %d unknown", type);
		return(1);
	}
	return(SetIntVolType(type));
}

/***********************************************************
 						SETVSNUSE()
 ***********************************************************/

/*
 *  SetVsnUse() - reset vsnbuf.v_use of the vsn fetched by GetVsn() -
 *	writes back any other changes to vsnbuf & sets dates
 *	if requested. E.g. used after a move or mount has 
 *	completed or err'd. Returns 0 if no error, otherwise
 *	sets VMError (in tbl_op) & EL_CODE & returns 1
 */

 static int
SetVsnUse(char use, char *time1, char *time2)
{
	char	*id = "SetVsnUse";

	vsnbuf.v_use = use;

	if (time1) {

		/*
		 *  update time(s)
		 */
		if (Now != -1) {
			SetTime(time1, Now);
			if (time2) 
				SetTime(time2, Now);
		} else
			nLogMsg(EL_ERROR2|EL_SYS, id, "time() error");
	}

	if (tbl_op(db_vsn_fd, OP_WRITE)) {
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
			"%s: dbase table update failed", vsnstr(vsnbuf.v_vsn));
		return(1);
	}

	return(0);
}

/***********************************************************
 					   RESETVSNSTATUS()
 ***********************************************************/

/*
 *  ResetVsnStatus() - mark vsn status, updating index and noting time; 
 *	assumes vsnaddr was set by GetVsn() first
 *	return: 0 if success, 1 if failure and set VMError & EL_CODE
 */

 static int
ResetVsnStatus(char status)
{
	char	*id = "ResetVsnStatus";
	char	oldstatus;
	int	i;

	/*
	 *  check legality
	 */

	 for (i=0; vStatTab[i].cs_string; i++)
		if (status == vStatTab[i].cs_code)
			break;
	if (!vStatTab[i].cs_string) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "Bad status: char (%c) int %d", 
						status, status);
		return(1);
	}

	oldstatus = vsnbuf.v_status;

	/*
	**	update the status index: delete then re-add
	*/
	sprintf(vsnkey, "%c_%llx", oldstatus, (long long)vsnaddr); 
	STR_DBT(key, vsnkey);
	if (dberr = db_vsn_sta_idx->del(db_vsn_sta_idx, NULL, &key, 0)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "del(%s): %s", vsnkey,
							db_strerror(dberr));
		VMError = E_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
		    "old status del(%s): %s", vsnkey, db_strerror(dberr));
		return(1);
	} 

	sprintf(vsnkey, "%c_%llx", status, (long long)vsnaddr); 
	STR_DBT(key, vsnkey);
	FILL_DBT(data, vsnaddr);
	if (dberr = db_vsn_sta_idx->put(db_vsn_sta_idx, NULL, &key, &data, 
							DB_NOOVERWRITE)) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, "put(status %s): %s",
					vsnkey, db_strerror(dberr));
		return(1);
	}
	vsnbuf.v_status = status;
	SetTime(vsnbuf.v_sttim, Now);

	/*
	**	virgins would not have labels (never written)
	*/
	if (status == S_VIRGIN) {
		if (vsnbuf.v_label[0] != ' ') {
			/*
			**	delete ilabel_uid index entry
			*/
			sprintf(vsnkey, "%s_%d", 
					vsnstr(vsnbuf.v_label), vinfo.vi_uid);
			STR_DBT(key, vsnkey);
			if (dberr = db_vsn_lab_idx->del(db_vsn_lab_idx, 
							NULL, &key, 0)) {
				nLogMsg(EL_CONSOLE0|EL_ERROR2, id,
						"->virgin del(%s): %s", 
						vsnkey,
						db_strerror(dberr));
			}
		}
		memset(vsnbuf.v_label, ' ', VSNLEN);
	}
	return(0);
}

/***********************************************************
 					   CHECKUIDLABEL()
 ***********************************************************/

/*
 *  CheckUidLabel() - do vsn label_uid index ops for changing uid
 */
 static int
CheckUidLabel(char *label, int uid)
{
	char	*id = "CheckUidLabel";

	sprintf(vsnkey, "%s_%d", vsnstr(label), uid);
	STR_DBT(key, vsnkey);
	if (dberr = db_vsn_lab_idx->get(db_vsn_lab_idx, NULL, &key, &data, 0)) {
		if (dberr == DB_NOTFOUND)
			return(0);

		/*
		**	unexpected error - deny check to be on safe side
		*/
		VMError = E_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
		    "get(lbl_uid %s): %s", vsnkey, db_strerror(dberr));
		return(1);
	}
	nLogMsg(EL_CODE|EL_ERROR2, id, 
				"internal label key (%s) would be duplicate",
				vsnkey);
	return(1);
}

/***********************************************************
 					   RESETUIDLABEL()
 ***********************************************************/

/*
 *  ResetUidLabel() - do vsn label_uid index ops for changing uid.
 *	Assumes vsnaddr set for this record in table.
 */

 static int
ResetUidLabel(int uid, int olduid)
{
	char	*id = "ResetUidLabel";

	if (vsnbuf.v_label[0] == ' ')
		return(0);

	if (uid != -1) {	/* uid == -1 is a recycle */

		/*
		**	see if new uid would violate label_uid uniqueness
		*/
		if (CheckUidLabel(vsnbuf.v_label, uid))
			return(1);
	}
	if (olduid != -1) {
		
		/*
		**	delete old entry
		*/
		sprintf(vsnkey, "%s_%d", vsnstr(vsnbuf.v_label), olduid);
		STR_DBT(key, vsnkey);
		if (dberr = db_vsn_lab_idx->del(db_vsn_lab_idx, 
							NULL, &key, 0)) {
			VMError = E_DBASE;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
			   "db del(vsn_uid %s): %s", vsnkey, db_strerror(dberr));
			return(1);
		}
	}
	if (uid != -1) {

		/*
		**	add label_uid for new user
		*/

		sprintf(vsnkey, "%s_%d", vsnstr(vsnbuf.v_label), uid);
		STR_DBT(key, vsnkey);
		FILL_DBT(data, vsnaddr);
		if (dberr = db_vsn_lab_idx->put(db_vsn_lab_idx, NULL, 
						&key, &data, DB_NOOVERWRITE)) {
			VMError = E_DBASE;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
					"db put(vsn_uid %s): %s", 
					vsnkey, db_strerror(dberr));
			return(1);
		}
	}
	return(0);
}

/***********************************************************
						GETQUOTA()
 ***********************************************************/

/*
 *  GetQuota() - get quota record into quotabuf and translate it
 *	return: 0 if ok, otherwise 1 & set VMError & EL_CODE
 */

 static int
GetQuota(int uid, int rc)
{
	char		*id = "GetQuota";
	RPRC_t		*rcp;

	if ((rcp = RCInfo(rc)) == NULL) {
		VMError = E_ADMIN;
		nLogMsg(EL_CODE|EL_ERROR1, id, "RC %d not configured", rc);
		return(1);
	}

	/*
	**	get index for user, rc quota entry
	*/
	sprintf(quotakey, "%d_%d", uid, rc);
	STR_DBT(key, quotakey);
	ZERO_OBJ(data);
	if (dberr = db_quota_idx->get(db_quota_idx, NULL, &key, &data, 0)) {
		if (dberr == DB_NOTFOUND) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
						"uid %d has no quota in %s", 
						uid, rcp->rc_id);
		} else {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR1|EL_CONSOLE0, id, 
					"db get(uid_rc %s): %s",
					vsnkey, db_strerror(dberr));
		}
		return(1);
	}

	/*
	**	read record and translate
	*/
	COPY_DBT(quotaddr, data);
	if (tbl_op(db_quota_fd, OP_READ)) {
		VMError = E_DBASE;
		nLogMsg(EL_ERROR1|EL_CONSOLE0|EL_CODE, id, 
				"dbase table error on %s", quotakey);
		return(1);
	}

	if (ToIntUidQuo(&quota.vqq_uid)    ||
	    ToIntRCQuo(&quota.vqq_rc)  ||
	    ToIntQuota(&quota.vqq_quota)   || 
	    ToIntUsed(&quota.vqq_used)) {
		VMError = E_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CONSOLE0|EL_CODE, id, 
				"dbase table translation error on %s", 
				quotakey);
		return(1);
	}
	if (quota.vqq_uid != uid) {
		VMError = E_DBASE;
		nLogMsg(EL_ERROR1|EL_CODE|EL_CONSOLE0, id, 
				"quota index corrupted");
		return(1);
	}

	return(0);
}




/***********************************************************
 					    INCQUOTAUSED()
 ***********************************************************/

/*
 *  IncQuotaUsed() - do it; return: 0 if ok, otherwise 1 and
 *	set VMError & EL_CODE
 */

 static int
IncQuotaUsed(int uid, RPRC_t *rcp, int change, int *force)
{
	char	*id = "IncQuotaUsed";
	int	rc = rcp->rc_number;

	nLogMsg(EL_STATUS1, id, "%s uid %d: %d", rcp->rc_id, uid, change);

	if (GetQuota(uid, rc)) {
		if (VMError == E_USER) {
			if (uid == VOLMANUID  &&  change == 1) {
				/*
				**	let volman have quota whenever needed
				*/
				nLogMsg(EL_STATUS1, id, "ADD volman quota =1");
				return(AddQuota(uid, rc, 1, 1));
			} else if (force) {

				/*
				**	user has no quota
				*/
				*force = 1;
				nLogMsg(EL_CONSOLE0|EL_STATUS2, id,
					"adding user quota %d of 0 in %s", 
					vinfo.vi_uid, rcp->rc_id);
				return(AddQuota(vinfo.vi_uid, rc, 0, 1));
			}
		}
		return(1);
	}
	quota.vqq_used += change;
	nLogMsg(EL_DEBUG2, id, "updated used count for %d, vqq_used = %d",
		quota.vqq_uid, quota.vqq_used);
	if (quota.vqq_used > quota.vqq_quota) {
		if (uid == VOLMANUID) {
			/*
			**	give volman whatever it needs
			*/
			nLogMsg(EL_STATUS1, id, "INC volman quota to %d",
							quota.vqq_used);
			quota.vqq_quota = quota.vqq_used;
		} else if (force) {
			*force = 1;
		} else {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
					"would exceed uid %d quota of %d", 
					uid, quota.vqq_quota);
			return(1);
		}
	}
	if (quota.vqq_used >= 0) {
		if (SetIntUsed(quota.vqq_used))
			return(1);
		if (tbl_op(db_quota_fd, OP_WRITE)) {
			VMError = E_DBASE;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"dbase table error on %s", quotakey);
			vsnbuf.v_use = U_IDLE;
			return(1);
		}
	}
	return(0);
}


/***********************************************************
 					   RESETVSNUID()
 ***********************************************************/

/*
 *  ResetVsnUid() - check quota & ilabel, then set vsn uid, updating index; 
 *	assumes vsnaddr was set by GetVsn() first.
 *	return: 0 if success, 1 if failure and set VMError & EL_CODE
 */

 static int
ResetVsnUid(RPRC_t *rcp, int uid, int olduid)
{
	char	*id = "ResetVsnUid";
	int	ret;

	if (uid == olduid)
		return(0);

	/*
	**	see if there would be a label_uid conflict
	*/
	if (CheckUidLabel(vsnbuf.v_label, uid))
		return(1);

	/*
	**	check/update quota
	*/
	if (rcp) {
		if (rcp->rc_flags & RCF_HASQUOTA) {

			/*
		 	**	see if adding vsn would blow user's 
			**		quota & update if ok
		 	*/
			if (IncQuotaUsed(uid, rcp, 1, (int *)NULL)) {
				if (uid != VolManOperUID)
					return(1);
				else	/* assume volman can cover itself */
					VMError = 0;
			}

			/*
			**	don't worry if old user e.g. has no quota
			*/
			IncQuotaUsed(olduid, rcp, -1, (int *)NULL);
			VMError = E_NOERROR;
		}
	}

	/*
	**	check/update internal label index
	*/
	if (ResetUidLabel(uid, olduid))
		return(1);

	if (SetIntUidVsn(uid))
		return(1);

	/*
	**	delete from uid index & re-add
	*/
	sprintf(vsnkey, "%04x_%llx", olduid, (long long)vsnaddr); 
	STR_DBT(key, vsnkey);
	if (dberr = db_vsn_uid_idx->del(db_vsn_uid_idx, NULL, &key, 0)) {
		VMError = E_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"db del(uid_addr %s): %s",
				vsnkey, db_strerror(dberr));
		return(1);
	}
	nLogMsg(EL_DEBUG1, id, "%s: deleted key %s", 
					vsnstr(vsnbuf.v_vsn), vsnkey);

	sprintf(vsnkey, "%04x_%llx", uid, (long long)vsnaddr); 
	STR_DBT(key, vsnkey);
	FILL_DBT(data, vsnaddr);
	if (dberr = db_vsn_uid_idx->put(db_vsn_uid_idx, NULL, &key, &data, 
							DB_NOOVERWRITE)) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
				"db put(uid_addr %s): %s",
				vsnkey, db_strerror(dberr));
		return(1);
	}
	return(0);
}

/***********************************************************
 						RESETVSNRC()
 ***********************************************************/

/*
 *  ResetVsnRc() - set vsn rc, updating index; 
 *	assumes vsnaddr was set by GetVsn() first.
 *	return: 0 if success, 1 if failure and set VMError & EL_CODE
 */

 static int
ResetVsnRc(int rc, int oldrc)
{
	char	*id = "ResetVsnRc";

	if (SetIntSrcRC(rc)  ||  SetIntDstRC(0))
		return(1);

	return(0);
}

/***********************************************************
 						ADDVSN()
 ***********************************************************/

/*
 *  DBAddVsn() - add vsn to dbase (table & indexes)
 *	return: 0 if ok, otherwise 1 & set VMError & EL_CODE
 */
 
 int
DBAddVsn(VInfo_t *info, char status)
{
	char		*id = "DBAddVsn";
	RPRC_t		*rcp;
	int		ret;
	char		*vsn = (char *) &info->vi_vsn;

	nLogMsg(EL_STATUS1, id, 
	    "%.*s lbl [%.*s]  uid %d  gid %d  poolgid %d mode %.3o  status %c",
		VSNLEN, vsn, VSNLEN, &info->vi_label, 
		info->vi_uid, info->vi_gid, info->vi_poolgid, 
		info->vi_mode, status);

	/*
	**	see if vsn exists already
	*/
	if (GetVsn(vsn, 0)) {
		if (VMError != E_USER) 
			return(1);
		VMError = E_NOERROR;
	} else {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
				"%.*s: vsn already exists", VSNLEN, vsn);
		return(1);
	}

	/*
	**	see if pool gid exists
	*/
	if (getgrgid(info->vi_poolgid) == NULL) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_SYS, id, "looking up pool gid %d",
							info->vi_poolgid);
		return(1);
	}

	/*
	**	see if internal label present
	*/
	memcpy(vsnbuf.v_label, &info->vi_label, VSNLEN);
	if (vsnbuf.v_label[0] != ' ') {

		/*
		**	has internal label: check for uid / ilabel uniqueness
		*/
		if (CheckUidLabel(vsnbuf.v_label, info->vi_uid)) {
			/*
			**	label_uid exists - fetch record & return error
			*/
			COPY_DBT(vsnaddr, data);
			if (tbl_op(db_vsn_fd, OP_READ))
				return(1);
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
			  "owner %d already has volume %.*s w/ internal (%.*s)",
			  info->vi_uid, VSNLEN, vsnbuf.v_vsn, 
			  VSNLEN, &info->vi_label);
			return(1);
		}
	}

	/*
	**	check rc 
	*/
	if ((rcp = RCInfo(info->vi_srcrc)) == NULL) {
		VMError = E_ADMIN;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"RC %d not configured", info->vi_srcrc);
		return(1);
	}
	if (rcp->rc_flags & RCF_HASQUOTA) {

		/*
		**	see if adding vsn would blow user's quota & 
		**		update if necc
		*/
		if (IncQuotaUsed(info->vi_uid, rcp, 1, (int *)NULL))
			return(1);
	}

	/*
	**	set up record & add vsn to table (unset integer fields
	**		are blank, which counts as 0)
	*/
	memset((char *) &vsnbuf, ' ', sizeof(vsnbuf));
	memcpy(vsnbuf.v_vsn, vsn, VSNLEN);
	memcpy(vsnbuf.v_label, &info->vi_label, VSNLEN);
	if (SetIntSrcRC(info->vi_srcrc) 
	    || SetIntDstRC(0)
	    || SetIntUidVsn(info->vi_uid)
	    || SetIntVolType(info->vi_voltype)
	    || SetIntGid(info->vi_gid)
	    || setvsnmode(info->vi_mode)
	    || SetIntPoolGid(info->vi_poolgid) 
	    || SetIntRecycle(0)) 
		return(1);

	vsnbuf.v_status = status;
	if (status == S_SCRATCH)
		memcpy(vsnbuf.v_label, vsn, VSNLEN);

	vsnbuf.v_use = U_IDLE;
	vsnbuf.end = '\n';
	if (Now != -1) {
		SetTime(vsnbuf.v_creat, Now);
		SetTime(vsnbuf.v_sttim, Now);
	} else
		nLogMsg(EL_ERROR2|EL_SYS, id, "time() error");
	if ((vsnaddr = LSEEK(db_vsn_fd, 0, SEEK_END)) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
							"dbase vsn table seek");
		return(1);
	}
	if (vsnaddr % sizeof(vsnbuf)) {	/* consistency check */
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
				"dbase vsn table bad size (%d)", vsnaddr);
		return(1);
	}
	if ((ret = write(db_vsn_fd, (char *) &vsnbuf, sizeof(vsnbuf)))
			!= sizeof(vsnbuf)) {
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"vsn table wrote %d", ret);
		return(1);
	}

	/*
	**	add vsn to indexes: vsn, user, label, rc, status
	*/

	/* VSN INDEX */
	sprintf(vsnkey, "%.*s", VSNLEN, vsn);
	STR_DBT(key, vsnkey);
	FILL_DBT(data, vsnaddr);
	if (dberr = db_vsn_vsn_idx->put(db_vsn_vsn_idx, NULL, &key, &data, 
							DB_NOOVERWRITE)) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, "db put(vsn %s): %s",
						vsnkey, db_strerror(dberr));
		return(1);
	}


	/* LABEL INDEX */

	if (vsnbuf.v_label[0] != ' ') {
		sprintf(vsnkey, "%.*s_%d",VSNLEN, vsnbuf.v_label, info->vi_uid);
		STR_DBT(key, vsnkey);
		FILL_DBT(data, vsnaddr);
		if (dberr = db_vsn_lab_idx->put(db_vsn_lab_idx, NULL, 
						&key, &data, DB_NOOVERWRITE)) {
			VMError = E_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"db put(ilbl_uid %s): %s", 
						vsnkey, db_strerror(dberr));
			return(1);
		}
	}
	
	/* UID INDEX */

	sprintf(vsnkey, "%04x_%llx", info->vi_uid, (long long)vsnaddr);
	STR_DBT(key, vsnkey);
	FILL_DBT(data, vsnaddr);
	if (dberr = db_vsn_uid_idx->put(db_vsn_uid_idx, NULL, &key, &data, 
							DB_NOOVERWRITE)) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"db put(uid_addr %s): %s",
						vsnkey, db_strerror(dberr));
		return(1);
	}
	nLogMsg(EL_DEBUG1, id, "%.*s: added key %s ", 
					VSNLEN, vsnbuf.v_vsn, vsnkey);
	
	/* STATUS INDEX */
	
	sprintf(vsnkey, "%c_%llx", status, (long long)vsnaddr);
	STR_DBT(key, vsnkey);
	FILL_DBT(data, vsnaddr);
	if (dberr = db_vsn_sta_idx->put(db_vsn_sta_idx, NULL, &key, &data, 
							DB_NOOVERWRITE)) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"db put(status_addr %s): %s",
						vsnkey, db_strerror(dberr));
		return(1);
	}
	
	nLogMsg(EL_STATUS1, id, "added");
	return(0);
}

/***********************************************************
 						DELETEVSN()
 ***********************************************************/

/*
 *  DBDeleteVsn() - change status & delete from unique indexes
 *	(other indexes will require checking if vsn deleted)
 *	return: 0 if ok, otherwise 1 or VMError & set EL_CODE
 */

 int
DBDeleteVsn(char *vsn, int *uid, int force)
{
	char		*id = "DeleteVsn";
	RPRC_t		*rcp;
	int		saverr;

	VMError = E_NOERROR;

	/*
	**	get record, check RC type 
	*/
	if (GetVsn(vsn, 1))
		return(1);

	if ((rcp = RCInfo(vinfo.vi_srcrc)) == NULL) {
		VMError = E_ADMIN;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"vsn RC %d not configured",vinfo.vi_srcrc);
		return(1);
	}
	if (rcp->rc_flags & RCF_HASQUOTA  &&  !force) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
		  "%s: RC %s hasquota: need to use force option to delete",
		  vsnstr(vsn), rcp->rc_id);
		return(1);
	}

	/*
	**	update status & write back record
	*/
	if (ResetVsnStatus(S_DELETED))
		 return(1);
	if (tbl_op(db_vsn_fd, OP_WRITE)) {
		VMError = E_DBASE;
		nLogMsg(EL_ERROR1|EL_CONSOLE0|EL_CODE, id, 
				"dbase table error on %s", vsnkey);
		return(1);
	}
	nLogMsg(EL_STATUS1, id, "DELETED %s owner %d", 
					vsnstr(vsnbuf.v_vsn), vinfo.vi_uid);

	/*
	**	inform the caller who the owner was
	*/
	*uid = vinfo.vi_uid;

	/*
	**	delete from vsn index 
	*/
	strcpy(vsnkey, vsnstr(vsn)); 
	STR_DBT(key, vsnkey);
	if (dberr = db_vsn_vsn_idx->del(db_vsn_vsn_idx, NULL, &key, 0)) {
		VMError = E_DBASE;
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "db del(vsn %s): %s",
						vsnkey, db_strerror(dberr));
	}

	/* 
	**	delete from label_uid index
	*/
	if (vsnbuf.v_label[0] != ' ') {
		sprintf(vsnkey, "%s_%d", vsnstr(vsnbuf.v_label), vinfo.vi_uid);
		STR_DBT(key, vsnkey);
		if (dberr = db_vsn_lab_idx->del(db_vsn_lab_idx, NULL, 
								&key, 0)) {
			VMError = E_DBASE;
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"db del(ilbl_uid %s): %s",
						vsnkey, db_strerror(dberr));
		}
	}

	if (VMError != E_NOERROR) 
		nLogMsg(EL_CODE|EL_ERROR1, id, "dbase index deletion error(s)");

	/*
	**	if quota RC, update user's quota 
	*/
	if (rcp->rc_flags & RCF_HASQUOTA) {
		/*
		**	throw away any quota error status since
		**		deleting the vsn sort of fixes
		**		whatever the problem is
		*/
		int	saverr = VMError;
		IncQuotaUsed(*uid, rcp, -1, (int *)NULL);
		VMError = saverr;
	}

	return(VMError);
}


/***********************************************************
 					     DBFAILMOUNT()
 ***********************************************************/

/*
 *  DBFailMount() - reset use to idle & increment mount failure
 */

 int
DBFailMount(char *vsn)
{
	int	failures;

	/*
	**	get vsn record & translate
	*/
	if (GetVsn(vsn, 1))
		return(1);

	/*
	**	update record and save
	*/
	vsnbuf.v_use = U_IDLE;
	IncMountFail();
	if (tbl_op(db_vsn_fd, OP_WRITE))
		return(1);

	/*
	**	check if volume is looking like a bad apple
	*/
	if (ToIntMntFail(&failures))
		return(1);
	if (failures > 4)
		nLogMsg(EL_CONSOLE0, 0, "%s: %d mount failures in a row", 
				vsnstr(vsn), failures);
	return(0);
}

/***********************************************************
 					DBMOUNTQUERY()
 ***********************************************************/

/*
 *  DBMountQuery() - if volume can be mounted in its present location.
 *	Return: pointer if ok, otherwise NULL & set VMError & EL_CODE
 */
 
 VInfo_t *
DBMountQuery(char *vsn, int uid, int rw, RPRC_t **rcpp)
{
	char		*id = "DBMountQuery";

	vClearError();
	nLogMsg(EL_DEBUG3, id, "vsn: (%s), uid: %d, rw %d", 
						vsnstr(vsn), uid, rw);

	/*
	**	get vsn record & translate, look up RC
	*/
	if (GetVsn(vsn, 1))
		return(NULL);
	if ((*rcpp = RCInfo(vinfo.vi_srcrc)) == NULL) {
		VMError = E_ADMIN;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%s: RC (%d) not defined", 
						vsnstr(vsn), vinfo.vi_srcrc);
		return(NULL);
	}

	nLogMsg(EL_DEBUG3, id, 
		" dbase: uid: %d, gid: %d, mode: %.3o, label %s", 
		vinfo.vi_uid, vinfo.vi_gid, vinfo.vi_mode, 
		vsnstr((char *)&vinfo.vi_label));
	
	/* 
	**	check perms 
	*/

	if (!nAccess(rw, uid, vinfo.vi_uid, vinfo.vi_gid, vinfo.vi_mode)) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
				"access to volser (%s) denied", vsnstr(vsn));
		return(NULL);
	}
	nLogMsg(EL_DEBUG3, id, "access ok");

	/*
	**	check 'mountability' of rc
	*/
	if (!((*rcpp)->rc_flags & RCF_MOUNTOK)) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
				"can't mount in %s", (*rcpp)->rc_id);
		return(NULL);
	}
	nLogMsg(EL_DEBUG3, id, "rc ok to mount");

	return(&vinfo);
}

 int
DBMountInit(char *vsn, int cmd, int rw)
{
	char	*id = "DBMountInit";
	int	ret;
	char	use;

	if (cmd == M_USRMOUNT)
		use = U_MOUNT;
	else if (cmd == M_SCRATCH)
		use = U_SCRATCH;
	else
		nLogMsg(EL_FATAL, id, "unexpected cmd %d", cmd);
	

	vClearError();
	nLogMsg(EL_DEBUG3, id, "vsn: (%s), rw %d", vsnstr(vsn), rw);

	/*
	**	get vsn record & translate
	*/
	if (GetVsn(vsn, 1))
		return(1);

	/*
	**	set use flag & time(s)
	*/

	if (rw) {
		IncMountWrite();
		ret = SetVsnUse(use, vsnbuf.v_atime, vsnbuf.v_mtime);
	} else {
		IncMountRead();
		ret = SetVsnUse(use, vsnbuf.v_atime, (char *)NULL);
	}

	return(ret);
}

/***********************************************************
 					DBUNMOUNTDONE()
 ***********************************************************/


/*
 *  DBUnmountDone() - update status of volume
 *	unmounting after successful mount
 */

 int
DBUnmountDone(char *vsn)
{
	if (GetVsn(vsn, 1))
		return(1);
	SetIntMntFail(0);
	return(SetVsnUse(U_IDLE, vsnbuf.v_atime, NULL));
}

/***********************************************************
 					DBMOVEQUERY()
 ***********************************************************/

/*
 *  DBMoveQuery() - evaluate a volume move request
 */

 VInfo_t *
DBMoveQuery(char *vsn, int uid, RPRC_t *destrcp)
{
	char		*id = "DBMoveQuery";
	RPRC_t		*srcrcp;
	int		destrc = destrcp->rc_number;

	/*
	 *  get & translate vsn record
	 */

	if (GetVsn(vsn, 1))
		return(NULL);

	/*
	**	check privilege
	*/
	if (vinfo.vi_uid != uid  &&  uid != 0  &&  uid != VolManOperUID) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id,
			"only owner, root or oper can move volume (%s)", 
								vsnstr(vsn));
		return(NULL);
	}
 
	/*
	**	check meaningfulness
	*/
	if ((srcrcp = RCInfo(vinfo.vi_srcrc)) == NULL) {
		VMError = E_ADMIN;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
			"source rc (%d) for vsn (%s) not configured",
			vinfo.vi_srcrc, vsnstr(vsn));
		return(NULL);
	}

	if (destrcp == srcrcp) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "volume (%s) already in %s", 
					vsnstr(vsn), destrcp->rc_id);
		return(NULL);
	}

	/*
	**	check if move is legal 
	*/
	if (MoveProblem(srcrcp, destrcp, vinfo.vi_voltype)) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id,
			"Illegal move: vsn type %s from rc %s to %s",
			vVolTypeName(vinfo.vi_voltype),
			srcrcp->rc_id, destrcp->rc_id);
		return(NULL);
	}
	
	/*
	**	set destination in buffer
	*/
	vinfo.vi_dstrc = destrcp->rc_number;

	/*
	**	check destination for quota
	*/
	if (!(destrcp->rc_flags & RCF_HASQUOTA)) {

		/* inform RP of source */
		
		return(&vinfo);
	}

	/*
	**	destination is quota controlled - need to check capacity
	**		& set rc's "used" to reserve slot
	*/
	if (destrcp->rc_capacity > -1) {
		if (!(destrcp->rc_used < destrcp->rc_capacity)) {
			VMError = E_ADMIN;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
					"NO ROOM IN %s", destrcp->rc_id);
			return(NULL);
		}
	}
 
	/*
	**	check the quota
	*/
	if (GetQuota(uid, destrc))
		return(NULL);

	if (quota.vqq_used >= quota.vqq_quota) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id,
					"denied: %s quota(%d) used up ", 
					destrcp->rc_id, quota.vqq_quota);
		return(NULL);
	}
	return(&vinfo);
}

  int
DBMoveInit(char *vsn, int uid, RPRC_t *destrcp)
{
	char	*id = "DBMoveInit";

	/*
	**	make sure buffers are correctly set up by DBMoveQuery
	*/
	if (DBMoveQuery(vsn, uid, destrcp) == NULL)
		return(1);
	vinfo.vi_dstrc = destrcp->rc_number;
	SetIntDstRC( destrcp->rc_number );

	/*
	**	set volume status & destination 1st 
	**		(date is set on completion of move)
	*/
	if (SetVsnUse(U_MOVE, (char *)NULL, (char *)NULL))
		return(1);

	if (!(destrcp->rc_flags & RCF_HASQUOTA))
		return(0);
	
	/*
	 *  would be nice to update quota & set volume status as a single 
	 *	action - if either fails, the move would not take place.
	 *	However, to avoid implementing transactions, the actions
	 *	are taken singly with customized error recovery.
	 */

	/*
	 *  update user quota
	 */

	quota.vqq_used++;
	if (!SetIntUsed(quota.vqq_used)) 
	   	tbl_op(db_quota_fd, OP_WRITE);
	if (VMError) {
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"dbase quota table error on %s", quotakey);
		vsnbuf.v_use = U_IDLE;
		tbl_op(db_vsn_fd, OP_WRITE);
		return(1);
	}

	/* 
	 *  everything ok - reserve the slot in dest rc
	 */

	destrcp->rc_used++;
	
	return(0);
}

  void
DBMoveFail(char *vsn, int uid, RPRC_t *destrcp)
{
	if (GetVsn(vsn, 1))
		return;
	if (SetVsnUse(U_IDLE, (char *)NULL, (char *)NULL))
		return;
	if (!(destrcp->rc_flags & RCF_HASQUOTA))
		return;
	IncQuotaUsed(uid, destrcp, -1, (int *)NULL);
}



/*
 *  UpdateLoc() - volume has been physically moved: if from quota location, 
 *	update user & rc 'used'; change 'location' of volser to reflect move. 	
 *	Any errors that prevent the vsn location from being updated are
 *	serious, since the vsn could be lost if the new location is
 *	lost.
 */
  int
DBUpdateLoc(char *vsn, RPRC_t *destrcp)
{
	char		*id = "DBUpdateLoc";
	RPRC_t		*srcrcp;
 	int		destrc = destrcp->rc_number;
	int		srcrc;

	nLogMsg(EL_DEBUG2,id,"(%s) dest %d", vsnstr(vsn), destrc);

	/*
	 *  get vsn record
	 */

	if (GetVsn(vsn, 1))
		return(1);

	srcrc = vinfo.vi_srcrc;

	if ((srcrcp = RCInfo(srcrc)) == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
			"vsn (%s) came from non-configured RC (%d)", 
			vsnstr(vsn), srcrc);
	}

	/*
	 *  the volume record is updated immediately, since vol is safely
	 *	in its new location - the longer we wait, the more the
	 *	chance of losing the location info
	 */
	
	nLogMsg(EL_DEBUG2, id, "%s: updating use=%c->%c, rc=%d->%d",
		vsnstr(vsn), vsnbuf.v_use, U_IDLE, vinfo.vi_srcrc, destrc);

	if (ResetVsnRc(destrc, srcrc))
		return(1);

	if (SetVsnUse(U_IDLE, vsnbuf.v_movtm, (char *)NULL)) {
		VMError = E_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1,id,"vsn update failed");
		return(1);	/* forget about quota update for now */
	}

	if (srcrc == destrc)		
		/* didn't move: no quota update needed */
		return(0);

	/*
	 *  update vsn count of source and dest
	 */

	if (srcrcp->rc_freeslots != -1)
		srcrcp->rc_freeslots++;
	if (destrcp->rc_freeslots != -1)
		destrcp->rc_freeslots--;

	/* 
	 *  see if source quota exists i.e. needs update now that volume
	 *	is definitely out of there
	 */

	if (!(srcrcp->rc_flags & RCF_HASQUOTA))
		/* no source quota update needed - ack now */
		return(0);
	
	/*
	 *  user source quota needs update
	 */

	if (IncQuotaUsed(vinfo.vi_uid, srcrcp, -1, (int *)NULL)) {
		nLogMsg(EL_DEBUG1, id, "srcrc quota rollback failed for uid %d",			vinfo.vi_uid);
		return(1);
	}

	return(0);
}

/*
 *  AsyncUpdateLoc() - volume has appeared in an rc w/out a previous
 *	moveqry in this voldaemon process: change 'location' of volser 
 *	to reflect move. 
 *	Update 'used' in source &/or dest quota if necc, creating
 *	dest quota record if none. If quota exceeded, return a vInfo
 *	pointer so that the RP can move it out of the illegal rc.
 *	Any errors that prevent the vsn location from being updated are
 *	serious, since the vsn could be lost if the new location is lost.
 *
 *	A NULL is returned for an error. A VInfo ptr is returned otherwise
 *	with the following meanings for the vi_rc field:
 *
 *		-1	vsn not in dbase or user has no quota in source
 *			or where it is now: get it out of system/fix quota
 *		src 	(!=dest) user exceeded quota: move back to src rc
 *	  	dest	update ok, nothing else to do
 */

 VInfo_t *
DBAsyncUpdateLoc(char *vsn, RPRC_t *destrcp)
{
	char		*id = "DBAsyncUpdateLoc";
	RPRC_t		*srcrcp;
	int		srcrc, quotaprob = 0;

	if (destrcp)
		nLogMsg(EL_DEBUG2, id, "(%s) dest %d", vsnstr(vsn), 
							destrcp->rc_number);

	if (GetVsn(vsn, 1)) {
		if (VMError == E_USER) {
			/*
			 *  an illegal volume has appeared that isn't in 
			 *	the dbase: this tells RP to reject it
			 */
			VMError = E_NOERROR;
			vinfo.vi_srcrc = -1;
			return(&vinfo);
		}
		return(NULL);
	}
	if (destrcp) {
		if (destrcp->rc_number == vinfo.vi_srcrc) {
			/* 
			 *  vol didn't actually move, so nothing needs updating:
			 *	this tells the RP to do nothing further
			 */
			vinfo.vi_srcrc = -2;
			return(&vinfo);
		}
	} else {
		/*
		 *  there should be a move that this is the completion of
		 */
		if (vsnbuf.v_use != U_MOVE) {
			nLogMsg(EL_ERROR1|EL_CONSOLE0, id,
				"(%s) moved without cmd - LOCATION UNKNOWN", 
				vsnstr(vsn));
			return(NULL);
		}
		/*
		 *  assume vsn completed move
		 */
		if ((destrcp = RCInfo(vinfo.vi_dstrc)) == NULL) {
			nLogMsg(EL_ERROR1|EL_CONSOLE0, id,
				"(%s) moved to nonconfigured RC %d %s",
				vsnstr(vsn), vinfo.vi_dstrc, 
				"- LOCATION UNKNOWN");
			return(NULL);
		}
	}
	if ((srcrcp = RCInfo(vinfo.vi_srcrc)) == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"source RC %d not configured", 
						vinfo.vi_srcrc);
	}

	/*
	 *  the volume record is updated immediately, since vol is safely
	 *	in its new location - the longer we wait, the more the
	 *	chance of losing the location info
	 */
	
	nLogMsg(EL_DEBUG2, id, "updating vsn record");
	if (ResetVsnRc(destrcp->rc_number, vinfo.vi_srcrc))
		return(NULL);

	if (SetVsnUse(U_IDLE, vsnbuf.v_movtm, (char *)NULL)) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"Error updating database");
                return(NULL);
	}

	/* TODO -  get this right: */
	destrcp->rc_used++;
	srcrcp->rc_used--;

	/*
	 *  a crucial question: does the user have any/sufficient
	 * 	quota in the rc in which this vsn asynchronously appeared?
	 */

	if (!destrcp)
		return(NULL);
	nLogMsg(EL_DEBUG2,id,"checked dest quota (%s)", 
		(!(destrcp->rc_flags & RCF_HASQUOTA)) ?  "yes" : "no");
	if (!(destrcp->rc_flags & RCF_HASQUOTA)) {

		/*
	 	 * async move to a quota-controlled rc: do bookkeeping/checking
	 	 */

		if (IncQuotaUsed(vinfo.vi_uid, destrcp, 1, &quotaprob))
			return(NULL);
	}
	/* save srcrc for possible quota rollback */
	srcrc = vinfo.vi_srcrc;	

	if (!quotaprob)
		vinfo.vi_srcrc = destrcp->rc_number;	/* accept the move */

	/* 
	 *  see if source quota exists i.e. needs update now that volume
	 *	is definitely out of there
	 */
	if (!srcrcp)
		return(&vinfo);

	nLogMsg(EL_DEBUG2, id, "source hasquota (%s)", 
		(!(srcrcp->rc_flags & RCF_HASQUOTA)) ? "yes" : "no");
	if (srcrcp->rc_flags & RCF_HASQUOTA) {

		/*
		 *  source quota needs update; whether it succeeds or fails,
		 *	return the vinfo struct
		 */
		if (IncQuotaUsed(vinfo.vi_uid, srcrcp, -1, (int *)NULL))
			nLogMsg(EL_ERROR1|EL_CONSOLE0,id,
				"srcrc quota rollback failed for uid %d",
				vinfo.vi_uid);
	}
	return(&vinfo);
}

/*
 *  DBAllocateScratch() - allocate a scratch volume
 */

 vsn_t *
DBAllocateScratch(int rc, int poolgid, int voltype, int uid, int gid, 
		  short mode, char *desc)
{
	char 		*id = "DBAllocateScratch";
	int		i, ret, xrc, dbtype, olduid, dbpool, inrc = 0;
	int		desclen, deletelast = 0, poolprob = 0;
	char		dupkey[20];
	RPRC_t		*rcp;

	nLogMsg(EL_DEBUG1, id, "rc %d, uid %d mode %.3o", rc, uid, gid, mode);

	/*
	 *  first, check desclen & that location is ok 
	 */

	desclen = strlen(desc);
	if (desclen > sizeof(vsnbuf.v_desc)) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR2, id,
				"description field too long (%d)", desclen);
		return(NULL);
	}

	if ((rcp = RCInfo(rc)) == NULL) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "RC %d not configured", rc);
		return(NULL);
	}
	if (!(rcp->rc_flags & RCF_ALLOCOK)) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id,
					"can't alloc in rc %s", rcp->rc_id);
		return(NULL);
	}
	if (voltype != -1) {
		if (RCVolTypeProblem(rcp, voltype))
			return(NULL);
	}
	if (poolgid != 0) {
		if (!nMember(uid, poolgid)) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id,"uid %d not in gid %d",
						uid, poolgid);
			return(NULL);
		}
	}

	/*
	 *  see if quota needs to be considered
	 */

	if (rcp->rc_flags & RCF_HASQUOTA) {

		/*
		 *  need to check quota
		 */
		if (IncQuotaUsed(uid, rcp, 1, (int *)NULL))
			return(NULL);
		/*
		 *  volman quota/used not vital
		 */
		IncQuotaUsed(VOLMANUID, rcp, -1, (int *)NULL);
		VMError = E_NOERROR;
	}

	/*
	 *  check vsn status index for scratch
	 */

	sprintf(dupkey, "%c", S_SCRATCH);

	for (i=0;;i++) {

		ret = dup_key_vol(dupkey, db_vsn_sta_idx, 
						1, i, deletelast, id);
		deletelast = 0;
		nLogMsg(EL_DEBUG1, id, "status index returned %d", ret);

		/*
		**	eval result of index
		*/
		if (ret) {
			char	*cp, buf[LINELEN];
			buf[0] = '\0';

			nLogMsg(EL_DEBUG1, id, "%d seen, %d in rc", i, inrc);
			sprintf(buf, "%s %s", rcp->rc_id, 
				(voltype == -1 ? 
					"(any vsn type)" : 
					vVolTypeID(voltype)));
			cp = strchr(buf, '\0');
			if (poolgid)
				sprintf(cp, "pool %d", poolgid);
			else if (poolprob)
				sprintf(cp, "(%d in restricted pool(s))",
							poolprob);

			VMError = E_UNAVAIL;
			nLogMsg(EL_CODE|EL_ERROR2, id, "No scratch: %s", buf);
			goto rollback;
		}

		/*
		**	got a vsn index; read the table
		*/
		COPY_DBT(vsnaddr, data);
		if (tbl_op(db_vsn_fd, OP_READ)) {
			VMError = E_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"Getting scratch record");
			goto rollback;
		}

		if (vsnbuf.v_status != S_SCRATCH) {
			if (vsnbuf.v_status == S_DELETED) {
				deletelast++;
				continue;
			}
			printvsn();
			VMError = E_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
			    "Status index corrupted (addr %d)", vsnaddr);
			goto rollback;
		}
		if (ToIntSrcRC(&xrc)) {
			VMError = E_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"Translating RC number");
			goto rollback;
		}
		if (ToIntUidVsn(&olduid)) {
			VMError = E_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"Translating uid");
			goto rollback;
		}
		if (ToIntPoolGid(&dbpool)) {
			VMError = E_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"Translating pool gid");
			goto rollback;
		}
		if (ToIntVolType(&dbtype)) {
			VMError = E_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
						"Translating vol type");
			goto rollback;
		}

		if (xrc == rc) {

			inrc++;

			/*
			 *  make sure type ok
			 */
			if (voltype != -1  &&  voltype != dbtype)
				continue;

			/*
			 *  make sure group ok
			 */
			if (dbpool != poolgid) {
				poolprob++;
				continue;
			}

			/*
			 *  make sure internal label, if any, ok for this user
			 */
			if (vsnbuf.v_label[0] != ' '  &&  uid != olduid) {
				if (CheckUidLabel(vsnbuf.v_label, uid))
					continue;
			}
			break;
		}
	}

	/*
	**	update vsn record & write it out
	*/

	if (SetIntGid(gid)  ||  setvsnmode(mode))
		goto rollback;

	/*
	**	set uid; rcp is null to avoid updating the quota again
	*/
	if (ResetVsnUid((RPRC_t *)NULL, uid, olduid))
		goto rollback;
	if (ResetVsnStatus(S_ALLOC))
		goto rollback;
	memset(vsnbuf.v_desc, ' ', sizeof(vsnbuf.v_desc));
	memcpy(vsnbuf.v_desc, desc, desclen);
	if (SetVsnUse(U_IDLE, (char *)NULL, (char *)NULL))
		goto rollback;

	return((vsn_t *) vsnbuf.v_vsn);

rollback:
	nLogMsg(EL_STATUS1, id, "rollback");
	if (rcp->rc_flags & RCF_HASQUOTA) {

		/*
		**	try to roll back user's quota
		*/

		if (IncQuotaUsed(uid, rcp, -1, (int *)NULL))
			nLogMsg(EL_STATUS1, id, "failed to reset user quota");
		if (IncQuotaUsed(VOLMANUID, rcp, 1, (int *)NULL))
			nLogMsg(EL_STATUS1, id, "failed to reset volman quota");
	}
	return(NULL);
}


/*
 *  DBRecycle - mark volume to be recycled / change ownership to volman
 */

 int
DBRecycle(char *vsnp, int uid, int *vsnuid, int bad)
{
	char		*id = "DBRecycle";
	int		recycle;
	RPRC_t		*rcp;

	nLogMsg(EL_DEBUG1, id, "RECYCLE %s/%d by uid %d", vsnstr(vsnp), 
						bad, uid);

	/*
	 * first, look up the volume & check ownership & status
	 */

	if (GetVsn(vsnp, 1)) 
		return(1);

	if (vinfo.vi_uid != uid) {
		VMError = E_PERM;
		nLogMsg(EL_CODE|EL_ERROR2, id,
				"denied: not owner of (%s)", vsnstr(vsnp));
		return(1);
	}
	*vsnuid = vinfo.vi_uid;
	if (vsnbuf.v_status != S_ALLOC) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id,
			"denied: state not ALLOCATED (%s)", vsnstr(vsnp));
		return(1);
	}
	if (vsnbuf.v_use != U_IDLE ) {
		nLogMsg(EL_STATUS2, id, "(%s) was in use", vsnstr(vsnp));
	}

	if ((rcp = RCInfo(vinfo.vi_srcrc)) == NULL) {
		VMError = E_ADMIN;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"in not configured RC %d", vinfo.vi_srcrc);
		return(1);
	}

	if (ResetVsnStatus( bad ? S_RECYCLEBAD : S_RECYCLE))
		return(1);

	if (ResetVsnUid(rcp, VolManOperUID, vinfo.vi_uid))
		return(1);

	if (SetIntGid(0) || setvsnmode(0))
		return(1);

	if (ToIntRecycle(&recycle))
		return(1);
	recycle++;
	SetIntRecycle(recycle);

	memset(vsnbuf.v_desc, ' ', DESCLEN);

	if (tbl_op(db_vsn_fd, OP_WRITE)) {
		nLogMsg(EL_CODE|EL_ERROR1, id, "writing back vsn record");
		return(1);
	}

	return(0);
}

/*
 *  Chmod() - change permissions on volume
 */

 int
DBChmod(char *vsnp, int uid, short addflag, short subflag)
{
	char	*id = "DBChmod";
	short	mode;

	nLogMsg(EL_STATUS1, id,"%s %d %.3o %.3o", 
				vsnstr(vsnp), uid, addflag, subflag);

	if (GetVsn(vsnp, 1))
		return(1);

	if (uid != vinfo.vi_uid  &&  uid != 0  &&  uid != VolManOperUID) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "not owner - permission denied");
		return(1);
	}

	mode = vinfo.vi_mode;
	mode &= subflag;
	mode |= addflag;
	if (mode == vinfo.vi_mode) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
				"null command: same permission as before");
		return(1);
	}

	if (setvsnmode(mode)) {
		return(1);
	}
	if (tbl_op(db_vsn_fd, OP_WRITE)) {
		nLogMsg(EL_CODE|EL_ERROR1, id, "writing back vsn record");
		return(1);
	}
	return(0);
}

/*
 *  DBChown() - change vsn owner
 */

 int
DBChown(char *vsnp, int newuid, int uid)
{
	char		*id = "DBChown";
	RPRC_t		*rcp;

	nLogMsg(EL_STATUS1, id, "uid %d: chown %s to %d", uid, 
						vsnstr(vsnp), newuid);
	if (GetVsn(vsnp, 1))
		return(1);

	if (uid != vinfo.vi_uid  &&  uid != 0  &&  uid != VolManOperUID) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "not owner - permission denied");
		return(1);
	}

	if (newuid == vinfo.vi_uid) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "no change - same uid");
		return(1);
	}
	if ((rcp = RCInfo(vinfo.vi_srcrc)) == NULL) {
		VMError = E_ADMIN;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
				"in not configured RC %d", vinfo.vi_srcrc);
		return(1);
	}

	if (ResetVsnUid(rcp, newuid, vinfo.vi_uid))
		return(1);

	if (tbl_op(db_vsn_fd, OP_WRITE)) {
		nLogMsg(EL_CODE|EL_ERROR1, id, "writing back vsn record");
		return(1);
	}
	return(0);
}

/*
 *  DBChgrp() - change vsn group
 */

 int
DBChgrp(char *vsnp, int newgid, int uid)
{
	char	*id = "DBChgrp";

	if (GetVsn(vsnp, 1))
		return(1);

	if (uid != vinfo.vi_uid  &&  uid != 0  &&  uid != VolManOperUID) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "Permission denied");
		return(1);
	}

	if (newgid == vinfo.vi_gid) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "no change - same gid");
		return(1);
	}

	if (SetIntGid(newgid))
		return(1);

	if (tbl_op(db_vsn_fd, OP_WRITE)) {
		nLogMsg(EL_CODE|EL_ERROR2, id, "writing back vsn record");
		return(1);
	}
	return(0);
}

/*
 *  Vattr() - admin set of vsn attributes for volmoddb. 
 */
 int
DBVattr(vmpAttr_t *ap)
{
	char		*id = "DBVattr";
	RPRC_t		*old_rcp = NULL, *new_rcp;
	int		set = 0, rollstat = 0, rolluser = 0, rollrc = 0;
	int		douid, dorc, rollstatus, rolluid;

	if (!ap->a_flags) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "no options given");
		return(1);
	}
	if (GetVsn((char *)&ap->a_vsn, 1))
		return(1);

	if (vsnbuf.v_use != U_IDLE) {
		nLogMsg(EL_STATUS2, id, "vsn %s was in use", 
					vsnstr((char *)&ap->a_vsn));
	}

	douid = (ap->a_flags & AT_VUID  &&  vinfo.vi_uid != ap->a_uid);
	dorc =  (ap->a_flags & AT_RC);

	if (dorc) {
		/*
		 *  pre-check
		 */
		int	medium = vVolTypeMedium(vinfo.vi_voltype);

		if ((new_rcp = RCInfo(ap->a_rc)) == NULL) {
			VMError = E_ADMIN;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
					"new RC %d not configured", ap->a_rc);
			return(1);
		}
		if (RCMedium(new_rcp, medium) == -1) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
					"new RC (%s) can't hold %s", 
					new_rcp->rc_id, vMediumName(medium));
			return(1);
		}
		if (vinfo.vi_srcrc == ap->a_rc)
			dorc = 0;
	}
	if (douid  ||  dorc) {
		if ((old_rcp = RCInfo(vinfo.vi_srcrc)) == NULL) {
			VMError = E_ADMIN;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
				"current RC %d not configured", vinfo.vi_srcrc);
			return(1);
		}
	}

	/*
	 *  see what settings are required, starting with status,
	 *	since it could be denied & we don't want to set
	 *	other indexes & then have to back out if uid failed.
	 */

	if (ap->a_flags & AT_STATUS  &&  vsnbuf.v_status != ap->a_status) {
		if (vsnbuf.v_status == S_ALLOC  &&  ap->a_status != S_RECYCLE) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
				"status is ALLOC - can only change to RECYCLE");
			return(1);
		}
		if (ap->a_status == S_VIRGIN  &&  ap->a_flags & AT_LABEL) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
				"virgin can have no label written");
			return(1);
		}
		nLogMsg(EL_STATUS1, id, "%s: changing status from %c to %c", 
					vsnstr((char *)&ap->a_vsn), 
					vsnbuf.v_status, ap->a_status);
		rollstatus = vsnbuf.v_status;
		if (ResetVsnStatus(ap->a_status))
			return(1);
		rollstat = 1;
		set++;
	}

	/*
	 *  uid & RC are interdependant because of quotas
	 */
	if (douid  &&  dorc) {
		int	force;

		/*
		 *  in doing uid, set quota in new RC
		 */
		rolluid = vinfo.vi_uid;
		if (ResetVsnUid(new_rcp, ap->a_uid, vinfo.vi_uid))
			goto rollback;
		rolluser = 1;
		vinfo.vi_uid = ap->a_uid;	/* for label, RC quota */
		set++;

		/*
		 *  update RC, setting quota in old RC
		 */
		if (ResetVsnRc(ap->a_rc, vinfo.vi_srcrc))
			goto rollback;
		rollrc = 1;
		if (Now != -1)
			SetTime(vsnbuf.v_movtm, Now);
		if (old_rcp == NULL)
			old_rcp = RCInfo(vinfo.vi_srcrc);
		if (old_rcp  &&  old_rcp->rc_flags & RCF_HASQUOTA) {
			if (IncQuotaUsed(rolluid, old_rcp, -1, &force))
				nLogMsg(EL_STATUS1, id,
					"%s: rc %s uid %d: %s",
					vsnstr((char *)&ap->a_vsn),
					old_rcp->rc_id, vinfo.vi_uid,
					" quota update (-1) failed");
		}
	} else if (douid) {
		nLogMsg(EL_STATUS1, id, "%s: changing uid from %d to %d", 
						vsnstr((char *)&ap->a_vsn), 
						vinfo.vi_uid, ap->a_uid);
		rolluid = vinfo.vi_uid;
		if (ResetVsnUid(old_rcp, ap->a_uid, vinfo.vi_uid))
			goto rollback;
		rolluser = 1;
		vinfo.vi_uid = ap->a_uid;	/* for label, RC quota */
		set++;
	} else if (dorc) {
		int	force;

		nLogMsg(EL_STATUS1, id, 
					"%s: changing rc from %d to %d", 
					vsnstr((char *)&ap->a_vsn), 
					vinfo.vi_srcrc, ap->a_rc);

		if (ResetVsnRc(ap->a_rc, vinfo.vi_srcrc))
			goto rollback;
		rollrc = 1;
		if (Now != -1)
			SetTime(vsnbuf.v_movtm, Now);

		/*
		 *  if 'old' rc not looked up, try it now
		 */
		if (old_rcp == NULL)
			old_rcp = RCInfo(vinfo.vi_srcrc);

		/*
		 *  update any quotas - not a showstopper 
		 *	if it fails, since knowing correct
		 *	location of volume is more important
		 */
		if (old_rcp  &&  old_rcp->rc_flags & RCF_HASQUOTA) {
			if (IncQuotaUsed(vinfo.vi_uid, old_rcp, -1, &force))
				nLogMsg(EL_STATUS1, id,
					"%s: rc %s uid %d: %s",
					vsnstr((char *)&ap->a_vsn),
					old_rcp->rc_id, vinfo.vi_uid,
					" quota update (-1) failed");
		}
		if (new_rcp->rc_flags & RCF_HASQUOTA) {
			if (IncQuotaUsed(vinfo.vi_uid, new_rcp, 1, &force))
				nLogMsg(EL_STATUS1, id,
					"%s: rc %s uid %d: %s",
					vsnstr((char *)&ap->a_vsn),
					old_rcp->rc_id, vinfo.vi_uid,
					" quota update (+1) failed");
		}
		set++;
	}
	if (ap->a_flags & AT_LABEL) {
		if (vsnbuf.v_status == S_VIRGIN) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
				"virgin can have no label written");
			return(1);
		}
		if (!memcmp(vsnbuf.v_label, vsnbuf.v_vsn, VSNLEN)) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
				"internal label already = external (%s)",
				vsnstr(vsnbuf.v_vsn));
			return(1);
		}
		if (CheckUidLabel(vsnbuf.v_vsn, vinfo.vi_uid)) {
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_ERROR2, id, 
				"uid %d already has ilabel %s",
				vinfo.vi_uid, vsnstr(vsnbuf.v_vsn));
			return(1);
		}
		memcpy(vsnbuf.v_label, vsnbuf.v_vsn, VSNLEN);
		set++;
	}
	if (ap->a_flags & AT_VGID  &&  vinfo.vi_gid != ap->a_gid) {
		nLogMsg(EL_STATUS1, id, "%s: changing gid from %d to %d", 
				vsnstr((char *)&ap->a_vsn), vinfo.vi_gid, ap->a_gid);
		if (SetIntGid(ap->a_gid))
			goto rollback;
		set++;
	}
	if (ap->a_flags & AT_POOLGID  &&  vinfo.vi_poolgid != ap->a_poolgid) {
		nLogMsg(EL_STATUS1, id, 
			"%s: changing pool gid from %d to %d", 
			vsnstr((char *)&ap->a_vsn), vinfo.vi_poolgid, ap->a_poolgid);
		if (SetIntPoolGid(ap->a_poolgid))
			goto rollback;
		set++;
	}
	if (ap->a_flags & AT_PERM  &&  vinfo.vi_mode != ap->a_perm) {
		nLogMsg(EL_STATUS1, id, "%s: changing mode from %.3o to %.3o",
				vsnstr((char *)&ap->a_vsn), vinfo.vi_mode, 
				ap->a_perm);
		if (setvsnmode(ap->a_perm))
			goto rollback;
		set++;
	}
	if (ap->a_flags & AT_VTYPE  &&  vinfo.vi_voltype != ap->a_type) {
		if (SetVsnType(ap->a_type))
			goto rollback;
		nLogMsg(EL_STATUS1, id, "%s: changing type from %s to %s",
			vsnstr((char *)&ap->a_vsn), 
			vVolTypeID(vinfo.vi_voltype), 
			vVolTypeID(ap->a_type));
		set++;
	}
	if (ap->a_flags & AT_COMMENT) {
		nLogMsg(EL_STATUS1, id,
			"%s: changing desc from [%.*s] to [%.*s]",
			vsnstr((char *)&ap->a_vsn),
			sizeof(vsnbuf.v_desc), vsnbuf.v_desc,
			MIN(sizeof(vsnbuf.v_desc), sizeof(ap->a_desc)), 
			ap->a_desc);
		memset(vsnbuf.v_desc, ' ', sizeof(vsnbuf.v_desc));
		memcpy(vsnbuf.v_desc, ap->a_desc,
			MIN(sizeof(vsnbuf.v_desc), sizeof(ap->a_desc)));
		set++;
	}

	if (!set) {
		nLogMsg(EL_STATUS1, id, "(nothing changed)");
		return(0);
	}

	/*
	 *  write out the vsn record & log as changed
	 */
	if (tbl_op(db_vsn_fd, OP_WRITE)) {
		nLogMsg(EL_CODE|EL_ERROR1, id, "writing back vsn record");
		goto rollback;
	}
	nLogMsg(EL_DEBUG1, id, "set ok");
	return(0);

rollback:
	/*
	 *  undo index changes in reverse order
	 */
	nLogMsg(EL_STATUS1, id, "rolling back index updates");
	if (rollrc)
		ResetVsnRc(old_rcp->rc_number, ap->a_rc);
	if (rolluser)
		ResetVsnUid(old_rcp, rolluid, ap->a_uid);
	if (rollstat)
		ResetVsnStatus(rollstatus);
	return(1);
}

  void
DBRecyclebad(char *vsnp)
{
	ResetVsnStatus(S_RECYCLEBAD);
}

  void
DBTrash(char *vsnp)
{
	ResetVsnStatus(S_TRASH);
}

/*
 *  DBQuotaQuery() - tell user quota in a restricted location
 */

 int
DBQuotaQuery(vmpQuota_t *quo)
{
	char		*id = "DBQuotaQuery";
	RPRC_t		*rcp;
	
	if ((rcp = RCInfo(quo->vqq_rc)) == NULL) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
					"RC %d not configured", quo->vqq_rc);
		return(1);
	}

	if (!(rcp->rc_flags & RCF_HASQUOTA)) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id,
					"RC %s is not a quota RC", rcp->rc_id);
		return(1);
	}

	if (GetQuota(quo->vqq_uid, quo->vqq_rc)) {
		if (VMError != E_USER)
			return(1);
		quo->vqq_quota = 0;
		quo->vqq_used = 0;
	} else {
		quo->vqq_quota = quota.vqq_quota;
		quo->vqq_used = quota.vqq_used;
	}
	return(0);
}

/***********************************************************
						DBSETQUOTA()
 ***********************************************************/

/*
 *  DBSetQuota() - get quota record into quotabuf and set to new val if ok
 *	return: 0 if ok, otherwise 1 & set VMError & EL_CODE
 */

 int
DBSetQuota(vmpQuota_t *quo)
{
	char		*id = "DBSetQuota";
	RPRC_t		*rcp;

	if (quo->vqq_quota < 0) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
				"quota must be >= 0 (%d)", quo->vqq_quota);
		return(1);
	}
	if ((rcp = RCInfo(quo->vqq_rc)) == NULL) {
		VMError = E_ADMIN;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"RC %d not configured", quo->vqq_rc);
		return(1);
	}
	if (!(rcp->rc_flags & RCF_HASQUOTA)) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, 
				"RC %s not a quota RC", rcp->rc_id);
		return(1);
	}
	if (GetQuota(quo->vqq_uid, quo->vqq_rc)) {
		if (VMError == E_USER) {

			/*
			 *  user has no quota, so add
			 */

			nLogMsg(EL_CONSOLE0|EL_STATUS2, id,
				"adding UID %d quota %d %s", 
				quo->vqq_uid, quo->vqq_quota, rcp->rc_id);
			quo->vqq_used = 0;
			return(AddQuota(quo->vqq_uid, quo->vqq_rc, 
				quo->vqq_quota, 0));
		}
		return(1);
	}
	if (quota.vqq_used > quo->vqq_quota) {
		VMError = E_USER;
		 nLogMsg(EL_CODE|EL_ERROR2, id, 
		    "requested quota would leave a quota violation: used=%d",
		    quota.vqq_used);
		return(1);
	}
	quo->vqq_used = quota.vqq_used;
	if (SetIntQuota(quo->vqq_quota))
		return(1);
	if (tbl_op(db_quota_fd, OP_WRITE))
		return(1);
	return(0);
}

 int
DBLabelQuery(char *vsnp, char *newlabel, int uid, int obsolete)
{
	char	*id = "DBLabelQuery";
	char	oldlabel[VSNLEN];
	int	labelchanged = 0;
	char	oldstatus = 0;

	VMError = E_NOERROR;

	if (GetVsn(vsnp, 1))
		return(1);

	memcpy(oldlabel, vsnbuf.v_label, VSNLEN);

	/*
	 *  see if it's just an inquiry
	 */
	if (newlabel[0] == ' ') {
		if (vsnbuf.v_label[0] == ' ')
			memset(newlabel, 0, VSNLEN);
		else
			memcpy(newlabel, vsnbuf.v_label, VSNLEN);
		return(0);
	}
	
	/*
	 *  this is a set label command - need to check permission
	 */
	if (!nAccess(1, uid, vinfo.vi_uid, vinfo.vi_gid, vinfo.vi_mode)) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR2, id, "Permission denied");
		return(1);
	}

	/*
	 *  if no label and consistent with the old one, done
	 *	(newlabel[0] == 0  iff the tape has no label)
	 */

	if (newlabel[0] == 0  &&  oldlabel[0] == ' ')
		return(0);

	/*
	 *  if the label has changed, check legality & update the label field
	 */

	if (memcmp(newlabel, vsnbuf.v_label, VSNLEN)) {
		labelchanged++;

		if (newlabel[0]) {

			/*
			 *  see if this label would violate the 
			 *	"no dups for a uid" rule
			 */

			if (CheckUidLabel(newlabel, vinfo.vi_uid))
				return(1);
		}

		nLogMsg(EL_STATUS2, id, "%.*s: ilabel %.*s -> %s",
				VSNLEN, vsnbuf.v_vsn, 
				VSNLEN, vsnbuf.v_label, 
				vsnstr(newlabel));

		/*
		 *  set up the new label for writing back to the table
		 */

		if (newlabel[0])
			memcpy(vsnbuf.v_label, newlabel, VSNLEN);
		else
			memset(vsnbuf.v_label, ' ', VSNLEN);
	}

	/*
	 *  consider whether to change VIRGIN/RECYCLE -> SCRATCH
	 */

	if ( (vsnbuf.v_status == S_VIRGIN  ||  
	      vsnbuf.v_status == S_RECYCLE ||
	      vsnbuf.v_status == S_RECYCLEBAD)
	    && !memcmp(vsnbuf.v_vsn, newlabel, VSNLEN)) {
		char	newstatus;

		oldstatus = vsnbuf.v_status;
		if (obsolete) {
			nLogMsg(EL_STATUS1, id, "OBSOLETEing %s", 
							vsnstr(vsnbuf.v_vsn));
			if (vsnbuf.v_status == S_VIRGIN)
				nLogMsg(EL_CONSOLE0|EL_ERROR2, id, 
					"OBSOLETING A VIRGIN!? (%s)",
					vsnstr(vsnbuf.v_vsn));
			newstatus = S_OBSOLETE;
		} else {
			nLogMsg(EL_STATUS1, id, "SCRATCHing %s", 
							vsnstr(vsnbuf.v_vsn));
			newstatus = S_SCRATCH;
			
			/*
			 *  scratching is the end of a process..
			 */
			vsnbuf.v_use = U_IDLE;
			SetTime(vsnbuf.v_atime, Now);
			SetTime(vsnbuf.v_mtime, Now);
			memset(vsnbuf.v_desc, ' ', sizeof(vsnbuf.v_desc));
		}
		if (ResetVsnStatus(newstatus))
			return(1);
	}
	/* 
	 *  write the vol entry
	 */

	if (tbl_op(db_vsn_fd, OP_WRITE)) {
		if (oldstatus)
			ResetVsnStatus(oldstatus);
		return(1);
	}

	if (labelchanged) {

		if (oldlabel[0] != ' ') {

			/*
			**	find & delete the old index entry 
			**		(key saved above)
			*/
			sprintf(vsnkey, "%s_%d", vsnstr(oldlabel),vinfo.vi_uid);
			STR_DBT(key, vsnkey);
			if (dberr = db_vsn_lab_idx->del(db_vsn_lab_idx, NULL, 
							&key, 0)) {
				VMError = E_DBASE;
				nLogMsg(EL_CODE|EL_ERROR1, id, 
					"db del(oldlbl_uid %s): %s",
					vsnkey, db_strerror(dberr));
				return(1);
			}
		}

		if (newlabel[0]) {

			/*
			**	add the new label to the index
			*/
			sprintf(vsnkey, "%s_%d",vsnstr(newlabel), vinfo.vi_uid);
			STR_DBT(key, vsnkey);
			FILL_DBT(data, vsnaddr);
			if (dberr = db_vsn_lab_idx->put(db_vsn_lab_idx, NULL, 
						&key, &data, DB_NOOVERWRITE)) {
				VMError = E_DBASE;
				nLogMsg(EL_CODE|EL_ERROR1, id, 
						"db put(ilbl_uid %s): %s",
						vsnkey, db_strerror(dberr));
				return(1);
			}
		}
	}
	return(0);
}

/*
 *  DBVsnQuery() - return info on vsn
 */
 VDR_t *
DBVsnQuery(char *vsn)
{
	char *id = "DBVsnQuery";

	VMError = E_NOERROR;

	if (GetVsn(vsn, 1))
		return(NULL);

	fillinvdr();
	return(&vdr);
}

/*
 *  DBVsnUidQuery() - get vsns by uid
 */

char	uid_ix_cpy_name[80];

 VDR_t *
DBVsnUidQuery(int uid, int started)
{
	char		*id = "DBVsnUidQuery";
	int		delete_last = 0;
	static char	dupkey[100];

	VMError = E_NOERROR;

	if (!started) {
		/*
		 *  main vold starting a query
		 */
		uid_ix_cpy_name[0] = 0;
		/*
		**	prepare for search of uid_addr by uid;
		**		make key for dup_key_vol 
		*/
		sprintf(dupkey, "%04x", uid);

	}
	for (;; started++) {
		if (dup_key_vol(dupkey, db_vsn_uid_idx, 4, 
				started, delete_last, id)) {
			if (uid_ix_cpy_name[0] != 0) {
				/*
				 *  forked for a big vls: throw away index copy
				 */
				unlink(uid_ix_cpy_name);
			}
			return(NULL);
		}

		/*
		 *  got vsn address - look up
		 */
		COPY_DBT(vsnaddr, data);
		if (VsnAddrToVinfo("(uid)"))
			return(NULL);

		/*
		**	if entry was deleted, skip it, and delete
		**		if this is the main vold
		*/
		delete_last = 0;
		if (vsnbuf.v_status == S_DELETED) {
			if (uid_ix_cpy_name[0] == 0)
				delete_last++;
			continue;
		}

		/*
		**	if uid not the same, assume it is due to
		**		update of table by parent of a forked 
		**		vold & just ignore the entry
		*/
		if (vinfo.vi_uid != uid)
			continue;

		/*
		**	entry is valid
		*/
		break;
	}
	fillinvdr();
	return(&vdr);
}

  void
DBCleanupVls()
{
	if (uid_ix_cpy_name[0] != 0) {
		/*
		 *  forked for a big vls: throw away index copy
		 */
		unlink(uid_ix_cpy_name);
	}
}

#define BUFSIZE	4096

  int
DBCloseForUidQuery(int pid)
{
	char	*id = "CloseForUidQuery";
	char	buf[BUFSIZE];
	int	ifd, ofd;
	OFF_T	rd, wr;

	/*
	 *  close the indexes to flush them and make a copy of the uid
	 *	index for the child to query without fear of rearrangement 
	 *	by parent
	 */

	if (CloseIndexes())
		nLogMsg(EL_CONSOLE0|EL_FATAL, id, "close indexes failed");
	sprintf(uid_ix_cpy_name, "/tmp/vdm.%d", pid);
	
	if ((ofd = open(uid_ix_cpy_name, O_RDWR|O_CREAT, 0600)) == -1) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_SYS, id, "opening index copy");
		return(1);
	}

	if ((ifd = open(dbf_vsn_uid_idx, O_RDONLY)) == -1) {
		VMError = E_DBASE;
		nLogMsg(EL_CODE|EL_SYS, id, "opening index file");
		close(ofd);
		return(1);
	}
	
	nLogMsg(EL_DEBUG2, id, "copying uid index");
	for (;;) {
		rd = read(ifd, buf, BUFSIZE);
		if (rd < 1) {
			if (!rd)
				break;
			VMError = E_DBASE;
			nLogMsg(EL_CODE|EL_SYS, id, "copying uid index");
			close(ifd);
			close(ofd);
			return(1);
		}
		if ((wr = write(ofd, buf, rd)) < rd) {
			int flg = EL_CODE|EL_ERROR1;
			if (wr == -1)
				flg |= EL_SYS;
			VMError = E_DBASE;
			nLogMsg(flg, id, "writing index copy");
			close(ifd);
			close(ofd);
			return(1);
		}
	}
	close(ifd);
	close(ofd);
	return(0);
}

DBReopenForUidQuery(int parent)
{
	char	*id = "DBReopenForUidQuery";
	int	i;

	/*
	 *  restart stuff index that was closed to flush it
	 */

	if (parent) {

		uid_ix_cpy_name[0] = 0;
		if (StartIndex(dbf_vsn_vsn_idx,     &db_vsn_vsn_idx, 0)  ||
	    		StartIndex(dbf_vsn_lab_idx, &db_vsn_lab_idx, 0)  ||
	    		StartIndex(dbf_vsn_uid_idx, &db_vsn_uid_idx, DB_DUP)  ||
	    		StartIndex(dbf_vsn_sta_idx, &db_vsn_sta_idx, DB_DUP)  ||
	    		StartIndex(dbf_quo_idx,     &db_quota_idx, 0))
				nLogMsg(EL_FATAL, id, 
					"indexes should start ok; quitting");
		return(0);
	}

	/*
	 *  child
	 */

	close(db_vsn_fd);
	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
				"vsn table (%s) reopen failed", dbf_vsn_tbl);
		exit(2);
	}
	if (StartIndex(uid_ix_cpy_name, &db_vsn_uid_idx, DB_DUP)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "restarting index");
		return(1);
	}

	/*
	 *  skip over index entries for uid till last previous vsn
	 *	- depends on vsnkey remaining the same
	 */
	for (i=0; ; i++) {
		if (dup_key_vol(vsnkey, db_vsn_uid_idx, 4, i, 0, id)) 
			return(-1);

		if (!memcmp(&vsnaddr, data.data, sizeof(vsnaddr)))
			break;
	}
	return(0);
}


static int	forward = -1;

 int
dup_key_vol(char *dupkey, DB *index, int size, int started, 
				int deletelast, char *caller)
{
	char		*id = "dup_key_vol";
	static DB	*ix;
	static DBC	*cursor = NULL;
	static char	savekey[100], delkey[100];
	static OFF_T	deladdr;
	static int	lastsize;
	int		ret;

	if (!started) {

		forward = -1;

		if (deletelast)
			nLogMsg(EL_ERROR1|EL_FATAL, id, 
						"deletelast on 1st call");

		/* 
		**	save info for checking future calls
		*/
		ix = index;
		lastsize = size;
		strcpy(savekey, dupkey);
		strcpy(vsnkey, dupkey);

		/*
		**	the old cursor went away on the index close,
		**		so get fresh cursor 
		*/
		if (dberr = ix->cursor(ix, NULL, &cursor, 0)) {
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
						db_strerror(dberr));
		}

		/* 
		**	get 1st instance, if any
		*/
		STR_DBT(key, savekey);
		ZERO_OBJ(data);
		if (dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE)) {
			if (dberr == DB_NOTFOUND) { 
				/* 
				**	nothing at all there 
				**		-is this possible?
				*/
				(void)cursor->c_close(cursor);
				cursor = NULL;
				return(1);
			}
			nLogMsg(EL_ERROR1|EL_FATAL, id, "db c_get(%s): %s",
						savekey, db_strerror(dberr));
		}

		/* 
		**	got something: see if it matches size
		*/
		if (strncmp(key.data, savekey, size)) {
			/*
			**	substring doesn't match
			*/
			(void)cursor->c_close(cursor);
			cursor = NULL;
			return(1);
		}

		/*
		**	match - save info in case of delete
		*/
		strcpy(delkey, key.data);
		COPY_DBT(deladdr, data);
		forward = 1;
		return(0);
	}
	if (forward < 0) 
		nLogMsg(EL_FATAL|EL_CONSOLE0, id, "discrepancy");

	/* not 1st time: make sure index & entry are unchanged */
	if (index != ix  ||  size != lastsize  || 
				strncmp(dupkey, savekey, size))
		nLogMsg(EL_FATAL|EL_CONSOLE0, id, "discrepancy 2");


	/*
	**	get the next key per this request
	*/
	if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
		if (dberr != DB_NOTFOUND) {
			nLogMsg(EL_ERROR1|EL_FATAL, id, "db c_get(next %s): %s",
						savekey, db_strerror(dberr));
		}
		forward = -1;
		ret = 1;
	} else {
		/* see if key still matches */
		if (memcmp(key.data, savekey, size)) {
			ret = 1;
			forward = -1;
		} else
			ret = 0;
	}

	/*
	**	delete prev key if req'd - probably could do it
	**		w/ cursor b4 search
	*/
	if (deletelast) {
		DBT	dkey;

		STR_DBT(dkey, delkey);
		if (dberr = ix->del(ix, NULL, &dkey, 0)) {
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db del(%s): %s",
					delkey, db_strerror(dberr));
		}
	}

	if (ret) {
		/*
		**	not found 
		*/
		(void)cursor->c_close(cursor);
		cursor = NULL;
	} else {
		/*
		**	found - save in case current will be deleted
		*/
		strcpy(delkey, key.data);
		COPY_DBT(deladdr, data);
	}
	return(ret);
}
