/*****************************************************************
		types used for accessing dbase
 *****************************************************************/

#include "db3/db.h"
#include "avltree.h"

/***************************************
	table layouts
 ***************************************/
/*
 *  Records are rounded up in size to a divider of disk block size to
 *  ensure that a record write will never cross a block boundary, for
 *  all-or-nothing success for a write interrupted by a machine crash.
 *
 *  Note: spacing characters are blanks for readability..
 */

/***************************************
	vsn table layout and buffer
 ***************************************/

#define DATELEN		16	/* 64 bits in hex - good for millenia */
#define INTLEN		5
#define BIGINTLEN	10

typedef struct vsnrec {
	char	v_vsn[VSNLEN];		char	b01;
	char	v_label[VSNLEN]; 	char	b02;
	char	v_status; 		char	b03;
	char	v_use; 			char	b04;
	char	v_srcrc[INTLEN];	char	b05;
	char	v_dstrc[INTLEN]; 	char	b06;
	char	v_voltype[INTLEN]; 	char	b07;
	char	v_mntwrite[INTLEN]; 	char	b08;
	char	v_mntread[INTLEN]; 	char	b09;
	char	v_mntfail[INTLEN]; 	char	b10;
	char	v_recycle[INTLEN]; 	char	b11;
	char	v_uid[BIGINTLEN]; 	char	b12;
	char	v_gid[BIGINTLEN]; 	char	b13;
	char	v_poolgid[BIGINTLEN]; 	char	b14;
	char	v_mode[3]; 		char	b15;
	char	v_creat[DATELEN]; 	char	b16;
	char	v_sttim[DATELEN];	char	b17;
	char	v_atime[DATELEN]; 	char	b18;
	char	v_mtime[DATELEN]; 	char	b19;
	char	v_movtm[DATELEN]; 	char	b20;
	char	v_desc[40];		char	b21;
	char	v_buf[32];		char	end;	/* fill to 256 */
} vbuf;

/***************************************
	quota table layout and buffer
 ***************************************/

typedef struct quotarec {
	char	q_uid[BIGINTLEN];	char	b01;
	char	q_rc[BIGINTLEN];	char	b02;
	char	q_quota[BIGINTLEN];	char	b03;
	char	q_used[BIGINTLEN];	char	b04;
	char	fill[19];		char	end;	/* fill to 64 */
} qbuf;


