/*
 *  rp_device.c - drive stuff
 *
 *  Bill Ross, NASA Ames, June 1995
 */

#include "rp_space.h"
#include <nas_tape.h>

 int
VaryStub(DevEnt_t *dep, int on, int reset)
{
	char	*id = "VaryStub";

	nLogMsg(EL_STATUS1, id, "dep=%d on=%d reset=%d", dep, on, reset);
	return(0);
}

 int
Vary34X0 (DevEnt_t *dep, int action, int setLED)
{
	char	*id = "Vary34X0";

	nLogMsg(EL_FATAL, id, "not implemented");
	return(0);
}

#ifdef CROLF

/*
 * Vary34X0() - for device checking/varying 3420 and 3480 tape drives
 *
 *  Bill Ross, Sterling Software for NAS/NASA Ames, 8/10/89,
 *  Bill Ross, Computer Sciences Corp for NAS/NASA Ames, 11/4/93.
 */

#ifdef AMDAHL_UTS
#include <tape.h>
#include <sys/tapio.h>
#include <sys/file.h>
#endif
 

#define NSENSE  32                      /* number of sense bytes        */
Vary34X0 (dep, action, setLED)
  DevEnt_t	*dep;
{
	char	*id = "Vary34X0";
	int 	fd, type, errlev, fdunreserved = 0;
	char    sense[NSENSE];
	char	devnam[MAXDEVPATH];

	sprintf(devnam, "/dev/%s", dep->de_devnam);
#ifdef AMDAHL_UTS
	/*
	 *  If it's a reset request handle it & return.
	 *	Caller should verify that device is off line.
	 */
	if (action & VVA_RESET) {
	/* HACK - O_NDELAY was TRESET in UTS1.2; find out if equiv in 2.1 */
		fd = Open(devnam, O_NDELAY);
		if (fd < 0) {
		    switch(errno) {
			case EMFILE:
			case ENFILE:
				/*
				 *  we are out of file descriptors, free one
				 *	temporarily and try open again
				 */
				UnreserveFD("em/en file");
				if ((fd = Open(devnam, O_NDELAY)) < 0) {
					VMError = E_SYS;
					nLogMsg(EL_CODE|EL_URGENT|EL_CONSOLE0, id,
				  		"%s: can't reset device", 
						devnam);
					ReserveFD("em/en file");
					return (-1);
				}
				close(fd);
				ReserveFD("em/en file ok");
				break;
		    	default:
				VMError = E_SYS;
				nLogMsg(EL_CODE|EL_URGENT|EL_CONSOLE0, id,
				  "%s: can't reset device", devnam);
				return (-1);
		    }
		} else
			close(fd);
		nLogMsg(EL_URGENT|EL_CONSOLE0, id, "%s: device reset", devnam);
		return (0);
	}

	/*
	 *  vary on/off
	 */

	fd = Open(devnam, O_RDWR|O_NDELAY);
	if (fd < 0  && (errno == EMFILE  ||  errno == ENFILE) ) {
		/*
		 *  we are out of file descriptors, free one
		 *	temporarily and try open again
		 */
		UnreserveFD("fd < 0");
		fdunreserved = 1;
		fd = Open(devnam, O_RDWR|O_NDELAY);
	}
	if (fd < 0) {
		switch(errno) {
		    case EAGAIN:
			nLogMsg(EL_CODE|EL_SYS, id,
				  "Device is in use by another domain");
			break;
		    case ENXIO:	/* indicates device is off-line */
			/*
			 * it's no big deal if requested to vary dev off
			 * and it's off line, but consider it an error
			 * if requested to vary on and it's off line
			 */
			errlev = EL_DEBUG1;
			if (on)
				errlev = EL_ERROR1;
			VMError = E_SYS;	/* override default */
			nLogMsg(errlev|EL_SYS|EL_CODE, id, "open(%s)", devnam);
			break;
		    case EBUSY:
			/*
			 * Device is busy, or a device or driver prob.
			 * In any any event, want to notify operator
			 * that this device is unusable.
			 */
			nLogMsg(EL_CONSOLE0, id, 
					"Problem with device %s (busy)",devnam);
			break;
		    default:		/* unexpected errno */
			VMError = E_INTERNAL;	/* override default */
			nLogMsg(EL_INTERNAL|EL_SYS|EL_CODE, id,
				  "unexpected error: open(%s)", devnam);
		}
		if (fdunreserved)
			ReserveFD("vary failed");
		return(0);		/* vary attempt failed */
	}
	if (ioctl(fd, TAPTYPE, &type) < 0) {
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, id, "ioctl(%s,TAPTYPE)",
		  devnam);
		close(fd);
		if (fdunreserved)
			ReserveFD("taptype");
		return(0);
	}
	if (type == 3480) {
		/*
		 * 3480 tapes need to be varied on before 
		 * other ioctls can be done
		 */
		if (ioctl(fd, (on ? TAPON : TAPOFF)) < 0) {
			/*
			 * We know we get EBADF when trying to vary off
			 *	a drive that is already varied off.
			 *	We get EIO when device is busy (I think).
			 *	Don't consider these to be internal errors.
			 */
			if (!(errno == EBADF && !on)) {
				VMError = E_SYS;
				nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, id,
				  "ioctl(%s, %s)", devnam,
				  (on ? "TAPON" : "TAPOFF"));
				close(fd);
				if (fdunreserved)
					ReserveFD("tapon");
				return(0);
			}
		}
		if (setLED) {
			/*
			 *  clear the score board
			 */
			(void) LED34X0(fd, 0, NULL, NULL, 1);
		}
	}
	close(fd);
	if (fdunreserved)
		ReserveFD("closed");
	nLogMsg(EL_DEBUG2, id, "device %s %s line", devnam, on ? "ON" : "OFF");
#else
	fd = ioctlopen(devnam);
	if (fd < 0  && (errno == EMFILE  ||  errno == ENFILE) ) {
		/*
		 *  we are out of file descriptors, free one
		 *	temporarily and try open again
		 */
		UnreserveFD("fd < 0");
		fdunreserved = 1;
		fd = ioctlopen(devnam);
	}
	if (fd < 0) {
		VMError = E_SYS;
		/*
		 *  first log a short string for volvary err
		 */
		nLogMsg(EL_SYS|EL_CODE, id, "open", devnam);
		/*
		 *  main console msg
		 */
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "%s %s", devnam, nErrStr);
		if (fdunreserved)
			ReserveFD("openioctl");
		return(1);
	} 
	if (setLED) {
		if (action & VVA_VARYON) {
		    if (LED34X0(fd, 0, NULL, NULL, 1) == -1)
			nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
					"ioctl tapled %s", devnam);
		} else if (action & VVA_VARYOFF) {
		    if (LED34X0(fd, TAPLED_FIRST, NULL, NULL, 0) == -1)
			nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
					"ioctl tapled %s", devnam);
		}
	}
	close(fd);
	if (fdunreserved)
		ReserveFD("close 2");
#endif
	return (0);
}
#endif /* CROLF */

VaryName_t	VaryNames[] = {
	"VaryStub",	VaryStub,
	"Vary34X0",	Vary34X0,
	"",
};

  int 
(*VaryName(str))()
  char	*str;
{
	VaryName_t	*vn = VaryNames;

	while (vn->v_name) {
		if (!strcmp(str, vn->v_name))
			return(vn->v_vary);
		vn++;
	}
	return(NULL);
}
