
#include "rp_space.h"

static Host_t		*HostArray = NULL;
int			totalHosts = 0;

/*
 *  VoldHostOK() - check that vold host is same one in Mkconfig.volman.
 *	The latter needs to be one of the aliases at least for the
 *	name gethostname() returns.me gethostname() returns.
 */
 void
VoldHostOK()
{
	char		myhost[MAXHOSTNAMELEN];
	struct hostent	*htp;

	if (gethostname(myhost, MAXHOSTNAMELEN) == -1) {
		perror("gethostname");
		exit(1);
	}

	/*
	 *  myhost is the 'local' name xxx as opposed to xxx.yyy.zzz
	 */
	if ((htp = nGetHostInfo(myhost)) == NULL) {
		fprintf(stderr, "Can't get full info on host (%s)\n", myhost);
		exit(1);
	}

	if (strcmp(htp->h_name, vVoldHost)  &&  strcmp(myhost, vVoldHost)) {
		char	**cpp;

		for (cpp=htp->h_aliases; *cpp; cpp++)
			if (!strcmp(*cpp, vVoldHost))
				break;
		if (*cpp == NULL) {
			fprintf(stderr, 
			    "vold started on %s but Mkconfig.volman says %s\n",
			    myhost, vVoldHost);
			exit(1);
		}
	}
}

/*
 *  InitSrv() - initialize volman server process tracking
 */
 static void
InitSrv(SrvProc_t *sp)
{
	sp->s_pid = -1;
	sp->s_sd = -1;
}

/*
 *  CheckHosts() - used once for debugging a situation where something
 *	tromped on mem used by gethostbyaddr()
 */
 void
CheckHosts(char *id)
{
	int		i;
	struct hostent	*htp;

	for (i=0; i<totalHosts; i++) {
		unsigned char	*cp = HostArray[i].h_addr0;

		htp = gethostbyaddr(cp, HOST_ADDRLEN, AF_INET);

		if (htp==NULL)
			nLogMsg(EL_FATAL, id, "failed: %s %d.%d.%d.%d",
				HostArray[i].h_name, 
				cp[0], cp[1], cp[2], cp[3]);
	}
}

/*
 *  NewHost() - config a host qualified to run volman servers
 */
NewHost(char *name)
{
	char		*id = "NewHost";
	struct hostent	*htp;
	int		i;
	Host_t		*hop;

	/*
	 *  it better be an interpretable host name
	 */
	if ((htp = nGetHostInfo(name)) == NULL) {
		nLogMsg(EL_ERROR1|EL_SYS, id, "CAN'T GET INFO ON HOST %s",name);
		return(1);
	}
	if (htp->h_length != HOST_ADDRLEN)
		nLogMsg(EL_FATAL, id, "%s h_length=%d != HOST_ADDRLEN=%d: %s",
				"struct hostent", htp->h_length, HOST_ADDRLEN,
				"change rp_host.h");
	if (HostArray != NULL) {

		/*
		 *  check for duplicate entry
		 */
		for (i=0; i<totalHosts; i++) {
			unsigned char	*cp = HostArray[i].h_addr0;
			char		**cpp;

			for (cpp=htp->h_addr_list; *cpp; cpp++) {
				if (!memcmp(cp, *cpp, HOST_ADDRLEN)) {
					nLogMsg(EL_ERROR1, id, 
						"%s: host already added as %s",
						name, HostArray[i].h_name);
					return(1);
				}
			}
		}
		/*
		 *  prepare space
		 */
		HostArray = (Host_t *) nReAlloc((void *)HostArray,
					(totalHosts+1) * sizeof(Host_t));
	} else
		HostArray = (Host_t *) nAlloc(sizeof(Host_t));

	hop = &HostArray[totalHosts];
	hop->h_id = totalHosts;
	strcpy(hop->h_name, name);
	strcpy(hop->h_ipname, htp->h_name);
	memcpy(hop->h_addr0, htp->h_addr_list[0], HOST_ADDRLEN);
	InitSrv(&hop->h_volcd);
	hop->h_clients = NULL;
	InitSrv(&hop->h_volnd);
	DM_ZERO(hop->h_drives);
	hop->h_ndrives = 0;
	totalHosts++;
	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "added  %s  addr %d.%d.%d.%d", name,
					(unsigned char)hop->h_addr0[0],
					(unsigned char)hop->h_addr0[1],
					(unsigned char)hop->h_addr0[2],
					(unsigned char)hop->h_addr0[3]);
	return(0);
}

  Host_t *
LookupHost(struct hostent *htp)
{
	int	i;

	/*
	 *  check names
	 */
	for (i=0; i<totalHosts; i++) {
		unsigned char	*cp = HostArray[i].h_addr0;
		char		**cpp;

		nLogMsg(EL_DEBUG1, 0, 
				"compare %s %d.%d.%d.%d  (%s %d.%d.%d.%d)",
				HostArray[i].h_name, cp[0], cp[1], cp[2], cp[3],
				htp->h_name, 
				(unsigned char)htp->h_addr[0], 
				(unsigned char)htp->h_addr[1], 
				(unsigned char)htp->h_addr[2],
				(unsigned char)htp->h_addr[3]);

		for (cpp=htp->h_addr_list; *cpp; cpp++) {
			if (!memcmp(cp, *cpp, HOST_ADDRLEN))
				return(&HostArray[i]);
		}
	}
	return(NULL);
}

/*
 *  HostByName() - look up under name in VCONF
 */
  Host_t *
HostByName(char *str)
{
	int	i;

	for (i=0; i<totalHosts; i++) {
		if (!strcmp(HostArray[i].h_name, str))
			return(&HostArray[i]);
	}
	return(NULL);
}

  Host_t *
HostIndex(int i)
{
	if (i < 0)
		nLogMsg(EL_FATAL, "HostIndex", "index %d", i);
	if (i >= totalHosts)
		return(NULL);

	return(&HostArray[i]);
}

 void
ShutdownHostClients(Host_t *hop)
{
	char		*id = "ShutdownHostClients";
	Client_t	*cp;
	int		i;

	cp = hop->h_clients;
	if (cp == NULL)
		nLogMsg(EL_FATAL, id, "no client array");
	for (i=0; i<FD_SETSIZE; i++, cp++)
		if (cp->c_flags & CL_EXIST)
			MClientGoodbye(cp);
}
