#ifndef lint
static char rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/master/rp_init.c,v 1.1.1.1 2000-02-28 02:18:28 wrstuden Exp $";
#endif

/*
 * Volume Manager
 *
 *  rp_init.c: Volume Manager Request Processor initialization/termination
 */

/*
 *  RESTARTTIME: time of delay for everything to clear 
 *	b4 restart on vshutdown -r 
 */
#define RESTARTTIME	10

/* 
 *  make globals non-extern in this file 
 */
#undef  EXTERN
#define EXTERN

#include "rp_space.h"

#include <stdlib.h>

#include <dirent.h>
#include <sys/stat.h>
#include <signal.h>		/* for NSIG */
#include <sys/time.h>		/* for gettimeofday() */
#include <pwd.h>
#include <sys/resource.h>	/* for setrlimit() */

int		shuttingdown = 0, idleshutdown = 0, shutdownrestart = 0;
int		timerpid;		/* tracked for log screening purposes */


extern int	IsChild;
extern char	**argvIn;
extern char	StartDir[];

extern char	NodeClientBusy[];
extern int	TotalNodeClients;
extern Client_t	Clients[];


/*
 *  lock for critical section of BecomeVoldaemon()
 *	(also used in RPShutdown())
 */

static char	*startup_lock = "/tmp/newvold_lock";

  void
ForkTimer()
{
	char	buf[MAXPATHLEN];

	sprintf(buf, "%s/%s", vLibDir, TIMER);
	timerpid = ForkExec(buf, TIMER, TIMERSECS);
}

/*
 *  SigChild(): called on SIGCHLD which is generated when one of the
 *	children (i.e. an RC) dies.
 */
static int	timerforks = 0;
#define MAXTIMERFORK	3

 void
SigChild(sig)
{
	char		*id = "SigChild";
	int		status, pid, csig;

	if ((pid = wait(&status)) < 0) {
		nLogMsg(EL_SYS, id, "wait");
		if (signal(sig, SigChild) == SIG_ERR)
			nLogMsg(EL_FATAL|EL_SYS, id, "signal SIGCHLD");
		return;
	}
	status &= 0177777;
	csig = status & 0177;
	status >>= 8;
	if (csig) 
		nLogMsg(EL_STATUS1, id, "child %d terminated on signal %d",
								pid, csig);
	
	if (!shuttingdown) {
		if (pid == timerpid) {
			if (status == 7)
				nLogMsg(EL_FATAL, id, 
					"TIMER (PID %d) DIED ON CONFIG PROBLEM",
					pid);
			if (++timerforks > MAXTIMERFORK)
				nLogMsg(EL_FATAL, id, 
					"TIMER (PID %d) DIED TOO MANY TIMES",
					pid);
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
				"TIMER (PID %d) DIED STATUS %x - restarting",
				pid, status);
			ForkTimer();
			if (signal(sig, SigChild) == SIG_ERR)
				nLogMsg(EL_FATAL|EL_SYS, id, "signal SIGCHLD");
			return;
		}
		nLogMsg(EL_STATUS2, id, "pid %d exited with status %x", 
					pid, status);
	}
	/* reinstate handler (must be after wait()) */
	if (signal(sig, SigChild) == SIG_ERR)
		nLogMsg(EL_FATAL|EL_SYS, id, "signal SIGCHLD");
}

/*
 *  BecomeVoldaemon(): Check for another RP running and get a lock on
 *	the critical "becoming an RP" section.
 */
 static void
BecomeVoldaemon()
{
	char		*id = "BecomeVoldaemon";

	/*
	 *  get exclusive lock
	 */
	nGetLock(startup_lock, "vold");

	/*
	 *  start log file as vold
	 */
	nLogInit("RP", RPShutdown);
	nLogSetLevel(EL_STATUS2);
	nLogInitFile(vLogFile, EL_LEVEL, 1, NULL);
	nLogInitConsole(rp_syslog, EL_CONSOLE0, EL_LEVEL);
	nLogMsg(EL_STATUS1, id, "--------------------------------------------");
#ifdef DEBUG
	{	char	*lognam;

		if (lognam = getlogin()) {
			if (strlen(VolManOperAcct) >= strlen(lognam))
				strcpy(VolManOperAcct, lognam);
		}
	}
#endif
}

/*
 *  RPInit(): initialize RP
 *
 *	initialize data structures, get lock on database, and start IPC
 */
 void
RPInit(int dbflg)
{
	char		*id = "RPInit", *getlogin();
	DIR		*dirp;
	RPRC_t		*rcp;
	time_t		time();
	int		rcno;
	char		pathdir[MAXPATHLEN];
	struct rlimit	limits;

#ifdef MALLOC_DEBUG
	mallopt(MALLOC_DEBUG, 1);
#endif
	/*
	 *  initialize EXTERN vars from rp_space.h
	 */
	VolReqQ.q_head = NULL;
	VolReqQ.q_tail = NULL;
	RPRidp = NULL;
	clientShutdown = 0;
	DM_ZERO(DMVary);
	DM_ZERO(DMVolnd);
	RCTotal = 0;
	RCArray = NULL;
	DriveTotal = 0;
	DriveArray = (DevEnt_t *) nAlloc(MAXDEVS * sizeof(DevEnt_t));

	vInitDirNames();

#ifndef PURIFY
	/*
	 *  max out the file descriptors for max connections
	 */
	if (getrlimit(RLIMIT_NOFILE, &limits) == -1) {
		perror("getrlimit(RLIMIT_NOFILE)");
		exit(1);
	}
	limits.rlim_cur = FD_SETSIZE;
	limits.rlim_max = FD_SETSIZE;
	if (setrlimit(RLIMIT_NOFILE, &limits) == -1) {
		fprintf(stderr, "setrlimit(RLIMIT_NOFILE..) %d\n: %s",
						FD_SETSIZE, strerror(errno));
		exit(1);
	}
# if 0
	if (getrlimit(RLIMIT_NOFILE, &limits) == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "getrlimit(RLIMIT_NOFILE)");
	nLogMsg(EL_STATUS1, id, "reset fd limits %d %d", limits.rlim_cur, 
						   limits.rlim_max);
# endif
#endif

	/*
	 *  make sure vSpoolDir exists and go to it
	 */
        if ((dirp = opendir(vSpoolDir)) == NULL) {
		fprintf(stderr, "NO SPOOLDIR (%s) - CREATING IT NOW\n", vSpoolDir);
		if (mkdir(vSpoolDir, 0755) < 0) {		/* create it */
			fprintf(stderr, "SpoolDir mkdir(%s) error: %s\n", vSpoolDir,
							strerror(errno));
			exit(1);
		}
        } else
		closedir(dirp);
	if (chdir(vSpoolDir) < 0) {
		perror(vSpoolDir);
		exit(1);
	}

	/*
	 *  make sure running on same host that local volcd would expect
	 */
	VoldHostOK();

	/*
	 *  Reserve an 'ace-in-the-hole' file descriptor for
	 *	getting out of tight situations. 
	 */
	ReserveFD("init");

	/*
	 *  negotiate any locks
	 */
	BecomeVoldaemon();
	nLogMsg(EL_CONSOLE0|EL_STATUS1, id, "vold starting, version %d",
								VM_VERSION);

	/*
	 *  read in the RC/drive config file and set up a
	 *	vary-per-RC array and hash space
	 */
	RPConfig();
	InitVaryCounts();
	InitRidSpace();

	/*
	 *  Start dbase - must be done before setting SIGCHLD
	 *	since InitDb() needs to wait() if rebuilding indexes,
	 *	and before starting other children since they might
	 *	also interfere w/ index process wait()s.
	 */
	nLogMsg(EL_DEBUG4, id, "calling initdb");
	DBInit(dbflg);
	nLogMsg(EL_DEBUG4, id, "initdb done");

	/*
	 *  catch all signals; SIGCHILD has a specific handler
	 */
	SigInit();
	if (signal(SIGCHLD, SigChild) == SIG_ERR)
		nLogMsg(EL_FATAL|EL_SYS, id, "signal SIGCHLD");

	/*
	 *  init socket/XDR structures, start listening for connections
	 */
	IPCInit();

	/*
	 *  start the timer & RCs
	 */
	ForkTimer();
	for (rcp=RCArray,rcno=0; rcno<RCTotal; rcno++,rcp++) {
		struct passwd 	*pwp;

		if (rcp->rc_number < 1) {
			nLogMsg(EL_CONSOLE0|EL_FATAL, id, "Invalid RC num %d", 
							rcp->rc_number);
			continue;
		}

		if (rcp->rc_login && (pwp = getpwnam(rcp->rc_login)))
			rcp->rc_uid = pwp->pw_uid;
		if (rcp->rc_path) {
			char	nbuf[LINELEN];
			
			sprintf(nbuf, "%d", rcp->rc_number);
			ForkExec(rcp->rc_path, rcp->rc_id, nbuf);
		}
	}
	/*
	 *  for now, start volcd & volnd on this machine
	sprintf(pathdir, "%s/volnd", vBinDir);
	ForkExec(pathdir, "volnd", NULL);
	 */
	sprintf(pathdir, "%s/volcd", vBinDir);
	ForkExec(pathdir, "volcd", NULL);

	/*
	 *  now that children are off, close stdin, stdout
	 */
	fclose(stdin);			/* don't need standard input */
	fclose(stdout);			/* don't need standard output */

	nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
				"vold initialized (pid %d)", getpid());
}


/*
 *  RPShutdown() - terminate RP
 */
 int
RPShutdown(char *caller, char *msg)
{
	char		*id = "RPShutdown";

	/*
	 *  if this is a child process (forked to write a long msg)
	 *	don't clean up - this would mess up the indexes etc.
	 */

	if (IsChild) {
		DBCleanupVls();	/* in case index was copied */
		exit(0);
	}

	if (caller == NULL)
		caller = "";
	if (shuttingdown) {
		nLogMsg(EL_STATUS2, id, "recursive call by %s", caller);
		return(0);
	}

	shuttingdown = 1;
	nLogMsg(EL_STATUS1, id, "called by %s", caller);
	if (msg && *msg)
		nLogMsg(EL_STATUS1|EL_TOSTDERR, id, msg);

	/*
	 *  close dbase 1st in case machine is going down
	 *	and vold might die soon - this preserves
	 *	indexes so that they don't need to be
	 *	rebuilt next time. NOTE: if any other
	 *	shutdown steps come to need dbase, this
	 *	would need to be rethought.
	 */
	nLogMsg(EL_STATUS2, id, "Closing dbase");
	CloseDb();

	nLogMsg(EL_STATUS2, id, "No longer accepting connections");
	IPCShutdown();

	nLogMsg(EL_STATUS2, id, "Shutting down client procs");
	ShutdownAllClients(msg);
	sleep(1);
	nLogMsg(EL_STATUS2, id, "Shutting down servers");
	ShutdownServers();

	kill(timerpid, SIGINT);
	nLogMsg(EL_STATUS2, id, "Varying off drives");
	VaryAll(0, VolManOperUID);

	/*
	 *  release startup lock in case death at startup critical section
	 */
	if (unlink(startup_lock)) {
		if (errno != ENOENT)
			nLogMsg(EL_SYS, id, 
				"unable to delete lock %s", startup_lock);
	}
	nLogMsg(EL_STATUS1|EL_CONSOLE0, id, "exiting");
	if (shutdownrestart) {
		char	name[MAXPATHLEN];

		/*
		 *  starting 'same' vold..
		 */
		if (!strncmp(argvIn[0], "..", 2)) {
			char	*p = argvIn[0];
			
			while (!strncmp(p, "..", 2)) {
				p += 3;
				*strrchr(StartDir, '/') = '\0';
			}
			sprintf(name, "%s/%s", StartDir, p);
		} else if (argvIn[0][0] == '.')
			sprintf(name, "%s%s", StartDir, &argvIn[0][1]);
		else if (argvIn[0][0] == '/')
			sprintf(name, "%s", argvIn[0]);
		else
			sprintf(name, "%s/vold", vBinDir);
		sleep(5);
		nLogMsg(EL_STATUS1|EL_CONSOLE0, id, 
	"****** starting new vold in %d seconds: %s for vshutdown -r ******",
			RESTARTTIME, name);
		if (fork())
			exit(0);
		sleep(RESTARTTIME);
		execv(name, argvIn);
	}
	exit(0);
	return(0);	/* for compiler warning */
}







