#ifndef lint
static char rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/master/rp_msg.c,v 1.1.1.1 2000-02-28 02:18:28 wrstuden Exp $";
#endif

/*
 *  rp_msg.c - Volume Manager Request Processor msg protocol support
 *
 *	Where convenient, stack is used for packet memory
 *	to avoid cost of malloc().
 */

#include "rp_space.h"
#include <signal.h>

extern time_t	Now;

extern int	totalHosts;
extern IPC_t	SockArray[];

static vmpHdr_t	*RCInfoPkt = NULL, *HostInfoPkt = NULL;
static int	RCInfoSize, HostInfoSize;

static char	DriveInfoBuf[sizeof(vmpHdr_t) + 8 +
				MAXDEVS * (sizeof(DevEnt_t) + ERRSTRSIZE)];

/*
 *  Send2RC() - compose and send off a request
 *	to the given Repository Controller 
 *	based on the info in the VolReq_t.
 *	Return error indication.
 *	TODO - consider using stack for packet rather than malloc
 */
  int
Send2RC(RPRC_t *rcp, VolReq_t *vrp, int type, vmpRcMsg_t *rcmp)
{
	char		*id = "Send2RC";
	vmpHdr_t	*hp;
	SrvProc_t	*sp;

	vClearError();
	sp = &rcp->rc_srv;
	if (sp->s_sd == -1) {
		VMError = E_DOWN;
		nLogMsg(EL_STATUS1|EL_CODE|EL_CONSOLE0, id, 
			"%s not running", rcp->rc_id);
		return (VMError);
	}

	/*
	 *  set up the appropriate packet to send
	 */
	switch (type) {
	  case M_RCMOUNT: {
		vmpRCMount_t	*mntp;

		hp = vBuildHdr(type, vrp->vr_rpseq, HdrNULL);
		mntp = MSGPTR(hp, vmpRCMount_t);
		mntp->vm_vsn = vrp->vr_vsn;
		mntp->vm_label = vrp->vr_label;
		mntp->vm_flag = vrp->vr_voflag;
		mntp->vm_uid = vrp->vr_uid;
		mntp->vm_host = vrp->vr_clhost;
		mntp->vm_devmap = *vDMand(&vrp->vr_devtotry, &DMVary);
		break;
	  }
	  case M_UNMOUNT: {
		vmpVsn_t	*vp;

		hp = vBuildHdr(type, vrp->vr_rpseq, HdrNULL);
		vp = MSGPTR(hp, vmpVsn_t);
		vp->v_vsn = vrp->vr_vsn;
		/*
		 *  send the drive number to the RC so test RC's don't
		 *	need to keep track of this themselves.
		 */
		vp->v_flag = vDMffs(&vrp->vr_devinuse);
		/*
		 *  flag2 instituted for VF_NORESP on cancel-mount-to-rc
		 *	when vsn turns up in another RC
		 */
		vp->v_flag2 = vrp->vr_flag;
		break;
	  }
	  case M_RCMSG: {
		vmpRcMsg_t	*orcmp;			/* output message */

		/*
		 *  (rcmsg is a msg within a msg)
		 */
		hp = vBuildHdr(type, NOSEQ, HdrNULL);
		orcmp = MSGPTR(hp, vmpRcMsg_t);
		*orcmp = *rcmp;
		break;
	  }
	  default:
		VMError = E_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, "%d: unexpected type", type);
		return (VMError);
	}
	/*
	 *  send to RC
	 */
	XDRSendI(hp, sp->s_sd, 1);
	nFree(hp);

	return (VMError);
}

/*
 *  SendNode() - send RP-originated node request to 
 *	volnd server on drive's host
 */
  int
SendNode(int cmd, VolReq_t *vrp, int drive, int expectvolreq)
{
	char		*id = "SendNode";
	Host_t		*hop;
	SrvProc_t	*sp;
	char		buf[PKTSIZ(vmpNode_t)];
	vmpHdr_t	*hp;
	vmpNode_t	*np;

	hop = HostIndex(DriveArray[drive].de_host);
	sp = &hop->h_volnd;
	if (sp->s_sd == -1) {
		/*
		 *  TODO - add back off/retry state & precheck
		 *	label client when mount req comes in
		 *	Also, allow unmount handoff between hosts
		 */
		VMError = E_UNAVAIL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
						"no volnd proc on host %s",
						hop->h_name);
		return(VMError);
	}

	hp = vBuildHdr(cmd, vrp->vr_rpseq, (vmpHdr_t *) buf);
	np = MSGPTR(hp, vmpNode_t);
	np->vs_vsn = vrp->vr_vsn;
	np->vs_drive = drive;
	np->vs_req = expectvolreq;
	np->vs_flag = 0;
	switch (cmd) {
	    case M_CREATENODE:
		/*
		 *  create node
		 */
		np->vs_label = vrp->vr_label;
		np->vs_flag = vrp->vr_voflag & VO_SETUP;
		np->vs_flag |= VO_MOUNT;
		if (vrp->vr_rcp->rc_flags & RCF_NODEMOUNT) {
			/*
			 *  RC doesn't know if it's mounted
			 */
			np->vs_flag |= VO_WAIT;
		}
		break;
	    case M_DELETENODE:
		/*
		 *  Note: RCs that handle their own node stuff
		 *	can decide to postpone the physical
		 *	unmount, e.g. if they can do an
		 *	instantaneous dismount.
		 */
		if (vrp->vr_rcp->rc_flags & RCF_NONODE) {
			/*
			 *  RCs needing all-vold-initiated node work
			 *	get complete dismount as well as the
			 *	basic removal of the /dev/vol/xxxxxx node.
			 */
			np->vs_flag = VO_UNMOUNT;
		}
		if (vrp->vr_rcp->rc_flags & RCF_CACHEMOUNT)
			np->vs_flag |= VO_CACHEMOUNT;
		if (vrp->vr_rcp->rc_flags & RCF_CLEARFIRST)
			np->vs_flag |= VO_CLEARFIRST;
		break;
	    default:
		nLogMsg(EL_FATAL, id, "prog err cmd %d", cmd);
	}
	return(XDRSendI(hp, sp->s_sd, 1));
}


/*
 *  Snd2Client() - figure out volcd & call XDR
 */
  int
Snd2Client(vmpHdr_t *hp, Client_t *cp, int flush)
{
	char		*id = "Snd2Client";
	SrvProc_t	*sp;
	int		err;			/* retry count */

	vClearError();
	if (cp->c_flags & CL_SHUTDOWN) {
		nLogMsg(EL_DEBUG1, id, "skipped write to dead client (%s)",
		  vHdrStr(hp));
		return(E_CANCELED);
	}
	sp = IPCClientServer(cp);
	if (sp == NULL) {
		cp->c_flags |= CL_SHUTDOWN;
		nLogMsg(EL_STATUS1, id, "Skipped - client server down, %s",
				vHdrStr(hp));
		return(E_DOWN);
	}
	hp->vh_slot = cp->c_id;
	err = XDRSendI(hp, sp->s_sd, flush);

	if (err != E_NOERROR) {
		if (VMError != E_BADPACKET) {
			nLogMsg(EL_STATUS1, id,
			  "Shutting down dead client (%s)", IPCClientDesc(cp));
			cp->c_flags |= CL_SHUTDOWN;
		}
	}
	return (VMError);
}

  int
Snd2Srv(vmpHdr_t *hp, SrvProc_t *sp, int flush)
{
	char	*id = "Snd2Srv";

	if (sp->s_sd == -1) {
		nLogMsg(EL_ERROR2, id, "server not connected");
		return(1);
	}
	if (sp->s_sd < 0  ||  sp->s_sd >= FD_SETSIZE) {
		nLogMsg(EL_ERROR2, id, "invalid server slot %d", sp->s_sd);
		return(1);
	}
	return(XDRSendI(hp, sp->s_sd, flush));
}

  static void
UpdateRCInfoPkt(vmpHdr_t *hp)
{
	RPRC_t		*rcp;
	RC_t		*rcpp;
	int		rcno;

	if (RCInfoPkt == NULL) {
		/*
		 *  allocate config info pkt for sending to clients
		 */
		RCInfoSize = RCTotal * sizeof(RC_t);
		RCInfoPkt = (vmpHdr_t *) 
			nAlloc(PadSize(sizeof(vmpHdr_t)) + RCInfoSize);
	}
	rcp = RCArray;
	rcpp = MSGPTR(RCInfoPkt, RC_t);
	for (rcno=0; rcno<RCTotal; rcno++, rcpp++, rcp++) 
		*rcpp = rcp->RCDef;
	memcpy(RCInfoPkt, hp, sizeof(vmpHdr_t));
	RCInfoPkt->vh_size = RCInfoSize;
	RCInfoPkt->vh_count = RCTotal;
}

  void
RCInfo2Client(vmpHdr_t *hp, Client_t *cp)
{
	UpdateRCInfoPkt(hp);
	(void) Snd2Client(RCInfoPkt, cp, 1);
}

  void
RCInfo2Srv(vmpHdr_t *hp, SrvProc_t *sp)
{
	UpdateRCInfoPkt(hp);
	XDRSendI(RCInfoPkt, sp->s_sd, 1);
}

  void
HostInfo2Client(vmpHdr_t *hp, Client_t *cp)
{
	char		*id = "HostInfo2Client";
	HostInfo_t	*hip;
	Host_t		*hop;
	int		i;

/* TODO - make sure HOSTNAMELEN <= MAXHOSTNAMELEN */

	if (HostInfoPkt == NULL) {

		/*
		**	allocate config info pkt for sending to clients
		*/
		HostInfoSize = totalHosts * sizeof(HostInfo_t);
		HostInfoPkt = (vmpHdr_t *) 
				nAlloc(PadSize(sizeof(vmpHdr_t)) 
					+ HostInfoSize);

		/*
		**	initialize basic info
		*/
		memset(HostInfoPkt, 0, PadSize(sizeof(vmpHdr_t)));
		hip = MSGPTR(HostInfoPkt, HostInfo_t);
		for (i=0; i<totalHosts; i++, hip++) {
			hop = HostIndex(i);
			memcpy(hip->h_name, hop->h_name, HOSTNAMELEN);
		}
	}

	/*
	**	set host pkt hdr for reply
	*/
	memcpy(HostInfoPkt, hp, sizeof(vmpHdr_t));
	HostInfoPkt->vh_size = HostInfoSize;
	HostInfoPkt->vh_count = totalHosts;

	/*
	**	update current hosts' status
	*/
	hip = MSGPTR(HostInfoPkt, HostInfo_t);
	hip->h_flag = 0;
	for (i=0; i<totalHosts; i++, hip++) {
		hop = HostIndex(i);

		hip->h_flag &= ~(H_HASVOLCD|H_HASVOLND);
		if (hop->h_volcd.s_sd != -1)
			hip->h_flag |= H_HASVOLCD;
		hip->h_nclients = hop->h_nclients;
		if (hop->h_volnd.s_sd != -1)
			hip->h_flag |= H_HASVOLND;
	}

	(void) Snd2Client(HostInfoPkt, cp, 1);
}

/*
 *  AckClient() - send a simple ack
 */
  void
AckClient(Seq_t seq, Client_t *cp)
{
	vmpHdr_t	hdr;

	(void) vBuildHdr(M_ACK, seq, &hdr);
	(void) Snd2Client(&hdr, cp, 1);
}


/*
 *  Err2Client() - build an M_VERROR message out of the VMError and
 *	nErrStr and send it to the client.
 */
  void
Err2Client(Seq_t seq, Client_t *cp)
{
	vmpHdr_t	*hp;
	vmpError_t	*ep;
	char		ebuf[PKTSIZ(vmpError_t)];

	hp = vBuildHdr(M_VERROR, seq, (vmpHdr_t *)ebuf);
	ep = MSGPTR(hp, vmpError_t);
	ep->vn_error = VMError;
	emsgcpy(ep->vn_msg, nErrStr);
	(void)Snd2Client(hp, cp, 1);
}

/*
 *  AckSrv() - send a simple ack
 */
  void
AckSrv(Seq_t seq, SrvProc_t *sp)
{
	vmpHdr_t	hdr;

	(void) vBuildHdr(M_ACK, seq, &hdr);
	XDRSendI(&hdr, sp->s_sd, 1);
}

  void
Err2Srv(Seq_t seq, SrvProc_t *sp)
{
	vmpHdr_t	*hp;
	vmpError_t	*ep;
	char		ebuf[PKTSIZ(vmpError_t)];

	hp = vBuildHdr(M_VERROR, seq, (vmpHdr_t *)ebuf);
	ep = MSGPTR(hp, vmpError_t);
	ep->vn_error = VMError;
	emsgcpy(ep->vn_msg, nErrStr);
	XDRSendI(hp, sp->s_sd, 1);
}


/*
 *  ErrAck2Client() - send error, then ack to say that request
 *	processing is complete. Used when requests that generate
 *	multiple responses (including errors) fail completely.
 */
  void
ErrAck2Client(Seq_t seq, Client_t *cp)
{
	vmpHdr_t	hdr;

	Err2Client(seq, cp);
	vBuildHdr(M_ACK, seq, &hdr);
	Snd2Client(&hdr, cp, 1);
}

/*
 *  Nop2Client() - send M_NOP to client to test if alive (fifo ipc)
 */
  void
Nop2Client(Client_t *cp)
{
	vmpHdr_t	hdr;

	vBuildHdr(M_NOP, NOSEQ, &hdr);
	Snd2Client(&hdr, cp, 1);
}

/*
 *  Nop2Srv()  - send M_NOP to server
 */
  int
Nop2Srv(SrvProc_t *sp)
{
	vmpHdr_t	hdr;

	vBuildHdr(M_NOP, NOSEQ, &hdr);
	return(XDRSendI(&hdr, sp->s_sd, 1));
}


/*
 *  Dev2Srv() - send device config to volnd
 */
  void
Dev2Srv(Id_t seq, SrvProc_t *sp)
{
	vmpHdr_t	*hp;
	char		*cp;

	hp = (vmpHdr_t *) DriveInfoBuf;
	hp->vh_size = DriveTotal * sizeof(DevEnt_t);
	hp->vh_magic = VMP_MAGIC;
	hp->vh_type = M_DEVQRY;
	hp->vh_seq = seq;
	hp->vh_slot = -1;

	hp->vh_count = DriveTotal;

	cp = MSGPTR(hp, char);

	memcpy(cp, (void *)DriveArray, DriveTotal * sizeof(DevEnt_t));

	XDRSendI(hp, sp->s_sd, 1);
}

/*
 *  BuildVaryMsg() - same as for Dev2Srv/DEVQRY, plus errors
 */
  static vmpHdr_t *
BuildVaryMsg(Seq_t seq)
{
	char		*id = "BuildVaryMsg";
	vmpHdr_t	*hp;
	char		*cp;
	int		i, errs;

	/*
	 *  count error msgs
	 */
	errs = vCountVaryErrors();
	if (errs)
		nLogMsg(EL_DEBUG2, id, "%d errs to send", errs);

	/*
	 *  pack header, drive count, drive array into buf
	 */
	hp = (vmpHdr_t *) DriveInfoBuf;
	hp->vh_magic = VMP_MAGIC;
	hp->vh_type = M_DRIVEINFO;
	hp->vh_seq = seq;
	hp->vh_size = DriveTotal * sizeof(DevEnt_t) + errs * ERRSTRSIZE;
	hp->vh_slot = -1;

	hp->vh_count = DriveTotal;

	cp = MSGPTR(hp, char);
	memcpy(cp, (void *)DriveArray, DriveTotal * sizeof(DevEnt_t));

	cp += DriveTotal * sizeof(DevEnt_t);

	/*
	 *  stuff any errs into packet
	 */
	if (errs) {
		DevEnt_t	*dep = DriveArray;

		for (i=0; i<DriveTotal; i++, dep++) {
			if (dep->de_errstr) {
				memcpy(cp, (void *)dep->de_errstr, ERRSTRSIZE);
				cp += ERRSTRSIZE;
			}
		}
	}
	return(hp);
}

/*
 *  Vary2Client() - send a vary msg (response to a vary rcvd). Includes
 *	errors (if any) that occurred on each drive.
 */
  void
Vary2Client(Seq_t seq, Client_t *cp)
{
	vmpHdr_t	*hp;

	hp = BuildVaryMsg(seq);
	(void) Snd2Client(hp, cp, 1);
}

/*
 *  Vary2Srv() - send a vary msg (response to a vary rcvd). Includes
 *	errors (if any) that occurred on each drive.
 */
  void
Vary2Srv(Seq_t seq, SrvProc_t *sp)
{
	vmpHdr_t	*hp;

	if (sp->s_sd < 0)
		return;
	if (SockArray[sp->s_sd].sock_ptype != PT_CONSOLE) {
		/*
		 *  volnd doesn't care about result of vary
		 */
		return;
	}
	hp = BuildVaryMsg(seq);
	XDRSendI(hp, sp->s_sd, 1);
}

/*
 *  Volstat2Client() - compose and send off a packet to the client
 *	that requested volume - basic response on a satisfied request.
 */
  void
Volstat2Client(VolReq_t *vrp)
{
	char		buf[PKTSIZ(vmpVolStat_t)];
	vmpHdr_t	*hp;
	vmpVolStat_t	*vsp;
	Client_t	*cp;
	vsn_t		*vsnp;
	int		dev;

	cp = vrp->vr_clientp;
	if (cp == NULL)
		return;			/* ASYNC request: no ack necessary */
	vsnp = &vrp->vr_vsn;

	hp = vBuildHdr(M_VOLSTAT, vrp->vr_clseq, (vmpHdr_t *) buf);
	vsp = MSGPTR(hp, vmpVolStat_t);
	vsp->vv_vsn = *vsnp;
	vsp->vv_voltype = vrp->vr_info.vi_voltype;
	vsp->vv_state = vrp->vr_state;
	vsp->vv_rpord = vrp->vr_rpord;

	/*
	 *  set 'use' based on state
	 */
	if (MOUNT_STATES(vrp->vr_state))
		vsp->vv_use = U_MOUNT;
	else if (vrp->vr_state == VS_RCWAIT2 || 
		 vrp->vr_state == VS_UNMOUNTED) 
		vsp->vv_use = U_UNMOUNT; 
	else if (MOVE_STATES(vrp->vr_state)) 
		vsp->vv_use = U_MOVE;
	else 
		vsp->vv_use = 'x';
	vsp->vv_uid = vrp->vr_uid;
	vsp->vv_rc = vrp->vr_rcp->rc_number;
	vsp->vv_flags = VV_LAST | VV_UID | VV_RC;
	vsp->vv_dev = vDMffs(&vrp->vr_devinuse);
	if (vsp->vv_dev >= 0)
		vsp->vv_host = DriveArray[vsp->vv_dev].de_host;
	else
		vsp->vv_host = -1;
	memcpy(vsp->vv_rid, vrp->vr_ridp->rid_id, sizeof(vsp->vv_rid));

	(void) Snd2Client(hp, cp, 1);

	/* TODO - maybe add signal to other 2Client routines */
	if (vrp->vr_voflag & VO_SIGNAL)
		SignalClient(VO_SIGGET(vrp->vr_voflag), cp);
}

/*
 *  SignalClient() - send client a signal - unused by clients at NAS..
 *			probably not tested since day 1; on top of
 *			that, added volcd layer
 */
  void
SignalClient(int sig, Client_t *cp)
{
	char		*id = "SignalClient";
	char		buf[PKTSIZ(vmpVsn_t)];
	vmpHdr_t	*hp;
	vmpVsn_t	*vp;
	SrvProc_t	*sp;

	sp = IPCClientServer(cp);
	if (sp == NULL) {
		nLogMsg(EL_STATUS1, id, "client server down");
		return;
	}
	if (sig <= 0 || sig >= NSIG) {
		nLogMsg(EL_ERROR1, id,
			  "user (%s) requested bad signal (%d); using SIGTERM",
			  IPCClientDesc(cp), sig);
		sig = SIGTERM;
	}
	nLogMsg(EL_STATUS1, id, "sending SIG %d to clnt %s", 
					sig, IPCClientDesc(cp));
	hp = vBuildHdr(M_SIGNAL, NOSEQ, (vmpHdr_t *)buf);
	vp = MSGPTR(hp, vmpVsn_t);
	vp->v_rc = sig;
	XDRSendI(hp, sp->s_sd, 1);
}


