
/*
 *  rp_rc.h 
 *
 *  Master info on RCs maintained by vold. 
 *
 *	Some info is read at startup time, the rest is current state.
 *
 *	The RCInfo section is sent to clients & is defined in vm_pkt.h
 *	(which is generated from vm_pkt.x).
 *
 *  Bill Ross, NASA Ames, June 1995.
 */

/*
 *  RCs: when the RP comes up, it forks off an RC for each entry in
 *	RCTab[].  The same binary can be invoked for different RCs.
 *	execl() calling sequence is:
 *		name:	rc_path
 *		arg0:	rc_name
 *		arg1:	rc_number (as a decimal string)
 *	from which the RC can determine his identity in case more
 *	than one instance of a given RC binary is being run.
 */

typedef struct RPRC {

    /* PUBLIC STUFF */

	RC_t		RCDef;		/* sent to clients 		*/

    /* RP INTERNAL INFO */

	char		*rc_path;	/* the path to the executable	*/
	int		(*rc_vary)();	/* vary on/off function		*/

        DevMap_t 	rc_avail[RC_MEDIA_TYPES];  
					/* varied-on devs / no current mounts */
        DevMap_t 	rc_inuse[RC_MEDIA_TYPES];	
					/* mounted/unmounting devs            */

        /*    filled in on RP side when RC makes a connection   */

        SrvProc_t	rc_srv;	
} RPRC_t;

/* shorthand for public stuff */
#define rc_number	RCDef.rcd_number
#define rc_index	RCDef.rcd_index
#define rc_name		RCDef.rcd_name
#define rc_id		RCDef.rcd_id
#define rc_type		RCDef.rcd_type
#define rc_login	RCDef.rcd_login
#define rc_uid		RCDef.rcd_uid
#define rc_syslog	RCDef.rcd_syslog
#define rc_flags	RCDef.rcd_flags
#define rc_density	RCDef.rcd_density
#define rc_defden	RCDef.rcd_defden
#define rc_media	RCDef.rcd_media
#define rc_nmedia	RCDef.rcd_nmedia
#define rc_devs		RCDef.rcd_devs
#define rc_deftype	RCDef.rcd_deftype
#define rc_mounttimeout	RCDef.rcd_mounttimeout
#define rc_capacity	RCDef.rcd_capacity
#define rc_used		RCDef.rcd_used
#define rc_freeslots	RCDef.rcd_freeslots
#define rc_nconfigdev	RCDef.rcd_nconfigdev
#define rc_mounttime	RCDef.rcd_mounttime
#define rc_devmap	RCDef.rcd_devmap
#define rc_acsid	RCDef.rcd_acsid

typedef struct mv {
	int	source;
	int	dest;
} MoveTable_t;

