/*
 *  Volume Manager
 *
 *  rp_space.h: space and type declarations;
 *	space is allocated in rp_init.c
 *
 * $Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/master/rp_space.h,v 1.1.1.1 2000-02-28 02:18:28 wrstuden Exp $
 */

#include "rl_space.h"

#include "rp_clnt.h"
#include "rp_host.h"
#include "rp_rc.h"
#include "rp_device.h"
#include "rp_rid.h"
#include "rp_volreq.h"

#define UNMOUNT_WAITTIME	6

/*
 *  the most common vars, initialized in rp_init()
 */
#ifndef EXTERN
#define EXTERN extern
#endif

EXTERN QHead_t	VolReqQ;		/* queue of active volume reqeusts */
EXTERN int	VolManOperUID;		/* UID of volume manager operator */
EXTERN Rid_t	*RPRidp;		/* reservation for RP */
EXTERN int	clientShutdown;		/* set by SigChild() */
EXTERN DevMap_t	DMVary;			/* vary on/off device map */
EXTERN DevMap_t	DMVolnd;		/* device map of volnd status */
EXTERN int	RCTotal;
EXTERN RPRC_t	*RCArray;
EXTERN int	DriveTotal;
EXTERN DevEnt_t	*DriveArray;

/* from rp_config.c: */
extern RPRC_t		*RCInfo(int);
extern void		SetVolndMapON(int host);
extern void		SetVolndMapOFF(int host);

/* from rp_1.c: */
extern int		ConsoleMsgSwitch(vmpHdr_t *hp, IPC_t *ip, int slot);
extern int		VolNodeMsgSwitch(vmpHdr_t *hp, Host_t *hop);
extern int		ClientMsgSwitch(vmpHdr_t *hp, Client_t *cp);
extern int		RCUnReadyMsgSwitch(vmpHdr_t *hp, int slot, 
						IPC_t *ip, Host_t *hop);
extern int		RCMsgSwitch(vmpHdr_t *hp, RPRC_t *rcp);
extern void		MTimer(vmpHdr_t *hp);
extern int		CleanupVolNodeD(Host_t *hop);
extern int		CleanupRC(RPRC_t *rcp);

/* from rp_2.c: */
extern void		MDetach(vmpHdr_t *hp, Client_t *cp);
extern void		MIdleTimes(vmpHdr_t *hp);
extern void		MVolQry(vmpHdr_t *ihp, Client_t *cp);
extern void		MAttach(vmpHdr_t *ihp, Client_t *cp);
extern void		CancelMove(VolReq_t *vrp, RPRC_t *badrcp);
extern void		DeleteNodeDone(VolReq_t *vrp, int drive);

/* from rp_3.c: */
extern void		MAddVsn(vmpHdr_t *hp, Client_t *cp);
extern void		MDeleteVsn(vmpHdr_t *hp, Client_t *cp);
extern void		MVattr(vmpHdr_t *hp, Client_t *cp);
extern void		MVAlloc(vmpHdr_t *hp, Client_t *cp);
extern void		MRecycle(vmpHdr_t *hp, Client_t *cp);
extern void		MVinfo(vmpHdr_t *hp, Client_t *cp);
extern void		MVslook(vmpHdr_t *hp, SrvProc_t *sp);
extern void		MChmod(vmpHdr_t *hp, Client_t *cp);
extern void		MChown(vmpHdr_t *hp, Client_t *cp);
extern void		MChgrp(vmpHdr_t *hp, Client_t *cp);
extern void		MLabelQuery(vmpHdr_t *hp, Client_t *cp);
extern void		MGetQuota(vmpHdr_t *hp, Client_t *cp);
extern void		MSetQuota(vmpHdr_t *hp, Client_t *cp);
extern void		MLook(vmpHdr_t *hp, Client_t *cp);

/* from rp_4.c: */
extern void		seterr(DevEnt_t *dep, char *errstr);
extern void		VaryDefaults();

/* from rp_config.c: */
extern void		RPConfig();
extern void		PrintRCArray(int lev, char *id);

/* from rp_sock.c: */
extern void  		IPCInit();
extern void		IPCShutdown();
extern void		ShutdownServers();
extern char		*IPCClientDesc(Client_t *);
extern SrvProc_t	*IPCClientServer(Client_t *);
extern Host_t		*IPCHost(Client_t *);

/* from rp_host.c: */
extern void		VoldHostOK();
extern Host_t		*LookupHost(struct hostent *);
extern Host_t		*HostByName(char *);
extern Host_t		*HostIndex(int);
extern void		ShutdownHostClients(Host_t *hop);

/* from rp_xdr.c: */
extern void		XDRInit();
extern vmpHdr_t		*XDRRecvI(int i);
extern int		XDRNextRecordI(int i);
extern int		XDRSendI(vmpHdr_t *hp, int i, int now);
extern void		XDRFlushI(int i);

/* from rp_q.c: */
extern void		ProcVolReqQ();
extern RPRC_t		*CheckRC(int, char *, char *);
extern void		InitVaryCounts();
extern void		DoVaryMsg();

/* from rp_init.c: */
extern void		RPInit(int dbflg);
extern int		RPShutdown(char *caller, char *msg);

/* from rp_vol.c */
extern VolReq_t		*NewVolReq();
extern VolReq_t		*FindVolReq(char *);
extern VolReq_t		*FindVolReqSeq(char *, Seq_t);
extern VolReq_t		*FindActiveVolReq(char *);
extern void		EnQVolReq(VolReq_t *vrp);
extern void		FreeVolReq(VolReq_t *vrp, char *cid);
extern void		Orphan(VolReq_t *vrp, Client_t *cp);
extern void		CancelRequest(VolReq_t *vrp, Client_t *cp, char *cid);
extern void		ErrOutVolReq(VolReq_t *vrp, char *id);
extern void		PrintVolReq(VolReq_t *vrp, int lev, char *id);
extern void		PrintClientVsns(Client_t *cp, char *id);
extern void		PrintReservationReqs(int lev, Rid_t *ridp, char *id);

/* rp_rid.c */
extern void		InitRidSpace();
extern void		PrintReservations();
extern void		UserRidReport(vmpHdr_t *ihp, Client_t *cp);
extern void		AllRidReport(vmpHdr_t *ihp, Client_t *cp);
extern int		BadRidName(char *);
extern Rid_t		*GetRid(char *, int);
extern Rid_t		*MakeRid(char *, int, Client_t *, Seq_t);
extern void		DeleteRid(Rid_t *, Client_t *);
extern VolReq_t		*AddVsnToRid(char *, Rid_t *, Client_t *, int, int);
extern int		DeleteVsnFromRid(char *, Rid_t *, Client_t *);
extern void		MoveVolReq2RPRid(VolReq_t *, Client_t *);
extern VolReq_t		*VolActiveInRid(VolReq_t *);
extern int		VolIntersect(Rid_t *);
extern int		HardReserve(Rid_t *, RPRC_t *, int,  int);
extern int		SoftReserve(Rid_t *, RPRC_t *, int,  int);
extern VolReq_t		*GetRidVsn(Rid_t *, char *, int, Client_t *);
extern void		FreeDriveUsed(RidRCMed_t *);

/* rp_clnt.c */
extern void		MNewClient(vmpHdr_t *, Host_t *);
extern void		MClientGoodbye(Client_t *cp);
extern void		ShutdownAllClients(char *msg);
extern void		vLogClients();

/* rp_db.c */
extern void		CloseDb();
extern int		DBFetchVsnKey(char *vsn, int complain);
extern int		DBAddVsn(VInfo_t *info, char status);
extern int		DBDeleteVsn(char *vsn, int *uid, int force);
extern int		DBFailMount(char *vsn);
extern VInfo_t		*DBMountQuery(char *vsn, int uid, int rw, RPRC_t **);
extern int		DBMountInit(char *vsn, int cmd, int rw);
extern int		DBUnmountDone(char *vsn);
extern VInfo_t		*DBMoveQuery(char *vsn, int uid, RPRC_t *destrcp);
extern int		DBMoveInit(char *vsn, int uid, RPRC_t *destrcp);
extern void		DBMoveFail(char *vsn, int uid, RPRC_t *destrcp);
extern int		DBUpdateLoc(char *vsn, RPRC_t *destrcp);
extern VInfo_t		*DBAsyncUpdateLoc(char *vsn, RPRC_t *destrcp);
extern vsn_t		*DBAllocateScratch(int rc, int poolgid, int voltype, 
				int uid, int gid, short mode, char *desc);
extern int		DBRecycle(char *vsnp, int uid, int *vsnuid, int bad);
extern int		DBChmod(char *vsnp, int uid, short addflag, 
				short subflag);
extern int		DBChown(char *vsnp, int newuid, int uid);
extern int		DBChgrp(char *vsnp, int newgid, int uid);
extern int		DBVattr(vmpAttr_t *ap);
extern void		DBRecyclebad(char *vsnp);
extern void		DBTrash(char *vsnp);
extern int		DBQuotaQuery(vmpQuota_t *quo);
extern int		DBSetQuota(vmpQuota_t *quo);
extern int		DBLabelQuery(char *vsnp, char *newlabel, int uid, 
				int obsolete);
extern VDR_t		*DBVsnQuery(char *vsn);
extern VDR_t		*DBVsnUidQuery(int uid, int started);
extern void		DBCleanupVls();
extern int		DBCloseForUidQuery(int pid);
extern int		DBReopenForUidQuery(int parent);

/*  rp_msg.c */
extern int		Send2RC(RPRC_t *rcp, VolReq_t *vrp, int type, 
				vmpRcMsg_t *rcmp);
extern int		SendNode(int cmd, VolReq_t *vrp, int drive, 
				int expectvolreq);
extern int		Snd2Client(vmpHdr_t *hp, Client_t *cp, int flush);
extern int		Snd2Srv(vmpHdr_t *hp, SrvProc_t *sp, int flush);
extern void		RCInfo2Client(vmpHdr_t *hp, Client_t *cp);
extern void		RCInfo2Srv(vmpHdr_t *hp, SrvProc_t *sp);
extern void		HostInfo2Client(vmpHdr_t *hp, Client_t *cp);
extern void		AckClient(Seq_t seq, Client_t *cp);
extern void		Err2Client(Seq_t seq, Client_t *cp);
extern void		AckSrv(Seq_t seq, SrvProc_t *sp);
extern void		Err2Srv(Seq_t seq, SrvProc_t *sp);
extern void		ErrAck2Client(Seq_t seq, Client_t *cp);
extern void		Nop2Client(Client_t *cp);
extern int		Nop2Srv(SrvProc_t *sp);
extern void		Dev2Srv(Id_t seq, SrvProc_t *sp);
extern void		Vary2Client(Seq_t seq, Client_t *cp);
extern void		Vary2Srv(Seq_t seq, SrvProc_t *sp);
extern void		Volstat2Client(VolReq_t *vrp);
extern void		SignalClient(int sig, Client_t *cp);
