
/*
 *  Volume Manager
 *
 *  rp_sock.c - vold connection mgt
 */

#include "rp_space.h"

#include <time.h>
#include <sys/stat.h>

extern int 		errno;

extern IPC_t		SockArray[];

extern int		timerpid;
extern int		UnmountWaitTime;
extern int		idleshutdown;
extern int		activerequests;
extern char		shutdownmsg[];

static int		timerfd;
int			Timeout = 0;
time_t			Now;	/* approximate time for non-critical use */

static fd_set		fds;
static int		Master_sd;		/* for receiving conns */

static int		socksize = sizeof( struct sockaddr_in );

  static char *
PortDesc(int i)
{
	if (SockArray[i].sock_ptype == PT_RC) {
		RPRC_t	*rcp = (RPRC_t *)SockArray[i].sock_proc;

		if (rcp != NULL)
			return(rcp->rc_id);
			
		return("initializing RC");
	}
	return(vPortTypeStr(SockArray[i].sock_ptype));
}

 void
IPCInit()
{
	static	char	id[] = "IPCInit";
	IPC_t		*ip = SockArray;
	struct servent	*servent;
	int		i;

	/*
	 *  initialize sock structs
	 */
	for (i=0; i<FD_SETSIZE; i++, ip++) {
		ip->sock_fd = -1;
        	ip->sock_id = -1;
        	ip->sock_state = SS_NOSOCK;
		ip->sock_ptype = PT_NONE;
		ip->sock_proc = NULL;
		ip->sock_host = NULL;
	}

	/*
	 *  tie XDR structs to sock structs
	 *	TODO - only on connect to save mem
	 */
	XDRInit();

	if ((servent = getservbyname(vold_service, "tcp")) == NULL)
		nLogMsg(EL_ERROR1|EL_SYS, id, "getservbyname");
	vold_port = ntohs(servent->s_port);
	Master_sd = IPCStartListening(vold_port);
}

 void
IPCShutdown()
{
	shutdown(Master_sd, 2);
	close(Master_sd);
}

 static void
IPCAcceptConn()
{
	char			*id = "AcceptConn";
	int			new_soc;
	struct sockaddr_in	new_conn;
	int			new_len;
	struct hostent		*htp;
	Host_t			*hop;
	IPC_t			*ip;
	unsigned char		*cp = (unsigned char *)&new_conn.sin_addr;

	/*
	 *  Create new socket with accept()
	 *	Note that resulting fd indexes Connections array
	 */
	socksize = sizeof(struct sockaddr_in);
	new_soc = accept(Master_sd, (struct sockaddr *)&new_conn, 
							&socksize);
	if (new_soc == -1) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1|EL_SYS, id, "accept failed");
		return;
	}
	nLogMsg(EL_STATUS1, id, "new client, addr %d.%d.%d.%d",
					cp[0], cp[1], cp[2], cp[3]);

	/*
	 *  See if authorized host
	 */
	htp = gethostbyaddr((char *)&new_conn.sin_addr, 
				sizeof(struct in_addr), new_conn.sin_family);
	if (htp == NULL) {
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
		  "attempted connection from untranslatable host (herror %s)",
			nHostErr());
		IPCClose(new_soc);
		return;
	}
	hop = LookupHost(htp);
	if (hop == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
			"attempted connection from unauthorized host (%s)",
								htp->h_name);
		IPCClose(new_soc);
		return;
	}
	nLogMsg(EL_STATUS2, id, "accepted slot %d from %s",
						new_soc, htp->h_name);

	/*
	 *  note the connection & wait for identifying 1st msg
	 */
	ip = &SockArray[new_soc];
	ip->sock_host = (void *) hop;
	ip->sock_ptype = PT_NONE;
	ip->sock_state = SS_CONNECTED;
	ip->sock_flags = 0;
	ip->sock_fd = new_soc;
	ip->sock_id = -2;
}

 static int
IPCProcPkt(vmpHdr_t *hp, int slot)
{
	char	*id = "IPCProcPkt";
	IPC_t	*ip = &SockArray[slot];
	Host_t	*hop = (Host_t *) ip->sock_host;

	/*
	 *  special case: handle 1st msg after connection
	 */
	if (ip->sock_state == SS_CONNECTED)
		return(IPCNewProcSwitch(hp, slot, ip, hop));

	switch (ip->sock_ptype) {
	    case PT_VOLCD:
		if (hp->vh_slot == -1) {
			/*
			 *  server->server msg
			 */
			return(VolCDMsgSwitch(hp, hop));
		} else {
			/*
			 *  from client
			 */
			Client_t	*cp;

			if (hp->vh_slot < 0  ||  hp->vh_slot>= FD_SETSIZE) {
				nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"bad slot %d from %s",
						hp->vh_slot, hop->h_name);
				return(0);
			}
			cp = &hop->h_clients[hp->vh_slot];
			if (!(cp->c_flags & CL_EXIST)) {
				VMError = E_INTERNAL;
				nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
					"host %s slot %d doesn't exist",
					hop->h_name, hp->vh_slot);
				return(0);
			}
			
			return(ClientMsgSwitch(hp, cp));
		}

	    case PT_RC:
		if (ip->sock_proc != NULL)
			return(RCMsgSwitch(hp, (RPRC_t *)ip->sock_proc));
		return(RCUnReadyMsgSwitch(hp, slot, ip, hop));

	    case PT_VOLNODE:
		return(VolNodeMsgSwitch(hp, hop));

	    case PT_TIMER:
		MTimer(hp);
		return(0);

	    case PT_CONSOLE:
		return(ConsoleMsgSwitch(hp, ip, slot));

	    default:
		nLogMsg(EL_FATAL, id, "unexpected sock_ptype %d", 
						ip->sock_ptype);
	}
}


/*
 *  FUNCTION:	IPCServer()
 *
 *		Main select loop for new connections and input from clients.
 */
IPCServer()
{
	char		*id = "IPCServer";
	int		i, nsoc, socseen, procQ;
	struct stat	st;
	struct timeval	tm, *timeoutp;
	vmpHdr_t	*hp;

	/*
	 *  initialize timeout
	 */
	tm.tv_sec = UnmountWaitTime;
	tm.tv_usec = 0;

	while(1) {

		/*
		 *  set up array for select() & do it
		 */
		FD_ZERO(&fds);
		FD_SET(Master_sd, &fds);

		for (i=0; i < FD_SETSIZE; i++) {
			if (SockArray[i].sock_state != SS_NOSOCK) {
				FD_SET(i, &fds);
			}
		}

		nLogMsg(EL_DEBUG4, id, "WAIT on select/rcv");
		if (Timeout)
			timeoutp = &tm;
		else
			timeoutp = NULL;
		while ((nsoc = select(FD_SETSIZE, &fds, (fd_set *) NULL, 
					(fd_set *) NULL, timeoutp)) == -1 
					&& errno == EINTR);
		time(&Now);
		if (nsoc == 0) {
			if (!Timeout)
				nLogMsg(EL_DEBUG2, id, "0 read.. no timeout-?");
			ProcVolReqQ();
			continue;
		}
		if (nsoc < 0)
			nLogMsg(EL_SYS|EL_FATAL, id, "select failed");

		/*
		 *  handle connect request if any
		 */
		socseen = 0;

		if (FD_ISSET(Master_sd, &fds)) {
			FD_CLR(Master_sd, &fds);
			IPCAcceptConn();
			socseen++;
		}

 		/*
		 *  process all others
		 */
		procQ = 0;
		for (i=0; i<FD_SETSIZE && socseen<nsoc; i++) {
			if (!FD_ISSET(i, &fds)) 
				continue;

			socseen++;

			if (SockArray[i].sock_state == SS_NOSOCK)
				nLogMsg(EL_STATUS1, id, "msg from former conn");

			if (i != timerfd)
				nLogMsg(EL_DEBUG3, id, "rcvd fd %d", i);

			/*
			 *  receive all (possibly buffered) packets
			 */
			do {
			    hp = XDRRecvI(i);
			    if (hp == NULL) {
				nLogMsg(EL_STATUS1, id, 
				    "null msg from %s on %s - %s",
				    PortDesc(i),
				    ((Host_t *)SockArray[i].sock_host)->h_name,
				    "shutting down connection");
				procQ += ShutdownServer(i);
				break;
			    }

			    /*
			     *  use the information
			     */
			    procQ += IPCProcPkt(hp, i);

			    nFree(hp);
			} while ( XDRNextRecordI(i));
			nLogMsg(EL_DEBUG2, id, "no more from %d", i);
		}
		if (procQ  ||  Timeout)
			ProcVolReqQ();
		if (idleshutdown  &&  !activerequests) {
			nLogMsg(EL_STATUS1|EL_CONSOLE0, id, 
				"Shutting down per idle-down request");
			RPShutdown(id, shutdownmsg[0] ? shutdownmsg :
	  			"Shutdown/idle command from console");
		}
	}
}

 void
ShutdownServers()
{
	int	i;

	for (i=0; i<FD_SETSIZE; i++)
		if (SockArray[i].sock_ptype != PT_NONE)
			ShutdownServer(i);
}

ShutdownServer(int slot)
{
	char	*id = "ShutdownServer";
	IPC_t	*ip = &SockArray[slot];
	Host_t	*hop = (Host_t *) ip->sock_host;
	RPRC_t	*rcp;
	int	procQ = 0;

	switch (ip->sock_ptype) {
	    case PT_VOLCD:
		procQ = CleanupVolCD(hop);
		break;
	    case PT_RC:
		if (rcp = (RPRC_t *)ip->sock_proc) {
			procQ = CleanupRC(rcp);
		}
		break;
	    case PT_VOLNODE:
		procQ = CleanupVolNodeD(hop);
		break;
	    case PT_TIMER:
		/* assume timer handled by catching child death */
		break;
	    case PT_CONSOLE:
		/* console connections are throwaways w/ no assoc status */
		nLogMsg(EL_DEBUG2, id, "console, slot %d", slot);
		break;
	    case PT_NONE:
		/* type not assigned yet */
		break;
	    default:
		nLogMsg(EL_FATAL, id, "unexpected sock_ptype %d", 
						ip->sock_ptype);
	}
	XDRFlushI(slot);
	CloseIPC(ip);

	return(procQ);
}


/*
 *  IPCNewProcSwitch() - check/setup new connection. 
 *
 *	Since the vold port is in the root-only range (<5000) and
 *	only trusted-host connections are accepted per the vold
 *	config file, the request is taken at face value.
 */
IPCNewProcSwitch(vmpHdr_t *hp, int slot, IPC_t *ip, Host_t *hop)
{
	char		*id = "IPCNewProcSwitch";
	vmpHello_t	*hep = MSGPTR(hp, vmpHello_t);
	RPRC_t		*rcp;
	SrvProc_t	srv;
	int		i;

	/*
	 *  set pkt for response
	 */
	hp->vh_slot = -1;
	hep->vhe_id = vGetID();		/* rcvkey for future msgs */

	/*
	 *  make a dummy SrvProc_t in case needed for Err2Srv()
	 */
	srv.s_sd = slot;

	/*
	 *  simple protocol check on this, the 1st pkt from process
	 */
	if (hep->vhe_version != VM_VERSION) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
			"%s version %d wrong on %s - refusing (version=%d)",
			vHdrStr(hp), hep->vhe_version, hop->h_name, VM_VERSION);
		Err2Srv(hp->vh_seq, &srv);
		CloseIPC(ip);
		return(0);
	}

	/*
	 *  set up the appropriate type of server
	 */
	switch (hp->vh_type) {
	    case M_VOLCD:
		if (hop->h_volcd.s_sd != -1) {
			nLogMsg(EL_STATUS1, id, 
				"apparent duplicate volcd on %s; %s %d %s",
				hop->h_name, "last msg from old volcd",
				Now - hop->h_volcd.s_lastmsg,
				"seconds ago; probing..");
			/*
			 *  see if old one still exists
			 */
			(void)Nop2Srv(&hop->h_volcd);
		}
		if (hop->h_volcd.s_sd != -1) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
					"duplicate volcd on %s - refusing",
					hop->h_name);
			Err2Srv(hp->vh_seq, &srv);
			CloseIPC(ip);
			return(0);
		}
		ip->sock_id = hep->vhe_pid;
		ip->sock_rcvkey = hep->vhe_id;
		ip->sock_sndkey = hp->vh_key;
		if (XDRSendI(hp, ip->sock_fd, 1)) {
			CloseIPC(ip);
			return(0);
		}
		ip->sock_state = SS_HELLOD;
		ip->sock_ptype = PT_VOLCD;
		ip->sock_proc = (void *) &hop->h_volcd;
		hop->h_volcd.s_sd = slot;
		hop->h_volcd.s_pid = hep->vhe_pid;
		hop->h_volcd.s_lastmsg = Now;
		hop->h_nclients = 0;
		if (hop->h_clients == NULL)
			hop->h_clients = (Client_t *) 
					nAlloc(FD_SETSIZE * sizeof(Client_t));
		memset(hop->h_clients, 0, FD_SETSIZE * sizeof(Client_t));
		for (i=0; i<FD_SETSIZE; i++) {
			hop->h_clients[i].c_ipcp = ip;
			hop->h_clients[i].c_host = hop->h_id;
		}
		nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
				"==== registered volcd pid %d on %s, slot %d",
				hop->h_volcd.s_pid, hop->h_name,
				hop->h_volcd.s_sd);
		return(0);

	    case M_NODED:
		if (hop->h_volnd.s_sd != -1) {
			/*
			 *  see if old one still exists
			 */
			nLogMsg(EL_STATUS1, id, 
				"apparent duplicate volnd on %s; %s %d %s",
				hop->h_name, "last msg from old volnd",
				Now - hop->h_volnd.s_lastmsg, 
				"seconds ago; probing..");
			(void)Nop2Srv(&hop->h_volnd);
		}
		if (hop->h_volnd.s_sd != -1) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
					"duplicate volnd on %s - refusing",
					hop->h_name);
			Err2Srv(hp->vh_seq, &srv);
			CloseIPC(ip);
			return(0);
		}
		/*
		 *  check host type
		 */
		if (!vHostTypeConfig(hep->vhe_flag)) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"volnd on %s claims unrecognized host type %d",
				hop->h_name, hep->vhe_flag);
			Err2Srv(hp->vh_seq, &srv);
			CloseIPC(ip);
			return(0);
		}
		ip->sock_id = hep->vhe_pid;
		ip->sock_rcvkey = hep->vhe_id;
		ip->sock_sndkey = hp->vh_key;

		/*
		 *  tell this volnd its host id so it
		 *	can  know which drives it's 
		 *	supposed to have
		 */
		hep->vhe_flag = hop->h_id;
		if (XDRSendI(hp, ip->sock_fd, 1)) {
			CloseIPC(ip);
			return(0);
		}
		ip->sock_state = SS_HELLOD;
		ip->sock_ptype = PT_VOLNODE;
		ip->sock_proc = (void *) &hop->h_volnd;
		hop->h_type = hep->vhe_flag;
		hop->h_volnd.s_sd = slot;
		hop->h_volnd.s_pid = hep->vhe_pid;
		hop->h_volnd.s_lastmsg = Now;
		SetVolndMapON(hop->h_id);
		nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
				"==== registered volnd pid %d on %s, slot %d",
				hop->h_volnd.s_pid, hop->h_name,
				hop->h_volnd.s_sd);

		/*
		 *  could be requests waiting for node check
		 */
		return(1);
	    case M_CONSOLE:
		ip->sock_id = hep->vhe_pid;
		ip->sock_rcvkey = hep->vhe_id;
		ip->sock_sndkey = hp->vh_key;
		if (XDRSendI(hp, ip->sock_fd, 1)) {
			CloseIPC(ip);
			return(0);
		} 
		ip->sock_state = SS_HELLOD;
		ip->sock_ptype = PT_CONSOLE;
		nLogMsg(EL_DEBUG1, id, "==== registered console pid %d on %s", 
					hep->vhe_pid, hop->h_name);
		return(0);
		
	    case M_RC:
		ip->sock_id = hep->vhe_pid;
		ip->sock_rcvkey = hep->vhe_id;
		ip->sock_sndkey = hp->vh_key;
		if (XDRSendI(hp, ip->sock_fd, 1)) {
			CloseIPC(ip);
			return(0);
		} 
		/*
		 *  (not tying it to a particular RC)
		 */
		ip->sock_state = SS_HELLOD;
		ip->sock_ptype = PT_RC;
		nLogMsg(EL_DEBUG1, id, 
				"==== registered RC pid %d on %s, slot %d", 
				hep->vhe_pid, hop->h_name, ip->sock_fd);
		return(0);
	    case M_TIMERD:		/* timer			*/
		if (hep->vhe_version != VM_VERSION) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
					"voltimer version %d on %s - refusing",
					hep->vhe_version, hop->h_name);
			Err2Srv(hp->vh_seq, &srv);
			CloseIPC(ip);
			return(0);
		} 
		if (hep->vhe_pid != timerpid) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"voltimer pid %d != %d - refusing",
					hep->vhe_pid, timerpid);
			Err2Srv(hp->vh_seq, &srv);
			CloseIPC(ip);
			return(0);
		}
		ip->sock_id = hep->vhe_pid;
		ip->sock_rcvkey = hep->vhe_id;
		ip->sock_sndkey = hp->vh_key;
		if (XDRSendI(hp, ip->sock_fd, 1)) {
			CloseIPC(ip);
			return(0);
		} 
		ip->sock_state = SS_HELLOD;
		ip->sock_ptype = PT_TIMER;
		ip->sock_proc = NULL;
		nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
				"==== registered timer pid %d on %s, slot %d",
				hep->vhe_pid, hop->h_name, ip->sock_fd);
		return(0);
	    default:
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"unexpected msg type from new conn on %s: %s",
				hop->h_name, vHdrStr(hp));
		Err2Srv(hp->vh_seq, &srv);
		CloseIPC(ip);
		return(0);
	}
}

  SrvProc_t *
IPCClientServer(Client_t *cp)
{
	char	*id = "IPCClientServer";
	IPC_t	*ip = cp->c_ipcp;

	if (ip->sock_ptype != PT_VOLCD) {
		nLogMsg(EL_ERROR1, id, "client server not volcd?? (%s)",
				vPortTypeStr(ip->sock_ptype));
	}
	return((SrvProc_t *) ip->sock_proc);
}



  char *
IPCClientDesc(Client_t *cp)
{
	static char	buf[128];
	Host_t		*hop = (Host_t *) cp->c_ipcp->sock_host;

	sprintf(buf, "uid %d pid %d, host %s", cp->c_uid, cp->c_pid,
		hop->h_name);
	return(buf);
}

  Host_t *
IPCHost(Client_t *cp)
{
	return((Host_t *) cp->c_ipcp->sock_host);
}
