/*
 *  rp_vol.c - volume request routines. See also rp_rid.c.
 *	volume request structures are sorted by vsn in
 *	btree-indexed hash queues, in addition to other
 *	queues they can be in
 */

#include "rp_space.h"
#include "avltree.h"

extern QHead_t	VolReqQ;

extern AVL_IX_DESC	rid_ix;
extern AVL_IX_REC	*rid_entry;

extern time_t	Now;

static int	order = 0;

  VolReq_t *
NewVolReq()
{
	VolReq_t	*vrp;

	vrp = (VolReq_t *) nAlloc(sizeof(VolReq_t));
	vrp->vr_state = VS_NULL;
	vrp->vr_time = 0;
	vrp->vr_idletime = 0;
	vrp->vr_qtime = 0;
	vrp->vr_rctime1 = 0;
	vrp->vr_usrtime = 0;
	vrp->vr_rctime2 = 0;
	memset(vrp->vr_tag, 0, TAGLEN);
	vrp->vr_flag = 0;
	vrp->vr_voflag = 0;
	vrp->vr_retrycnt = 0;
	vrp->vr_rpseq = vGetSeq();	/* give it a sequence number */
	vrp->vr_rpord = order++;
	vrp->vr_uid = -1;
	vrp->vr_clhost = -1;
	vrp->vr_clhostp = NULL;
	vrp->vr_ndhostp = NULL;
	vrp->vr_ridp = NULL;
	vrp->vr_rcp = NULL;
	vrp->vr_dstrcp = NULL;
	vrp->vr_headvrp = NULL;
	vrp->vr_vrq.q_prev = NULL;
	vrp->vr_vrq.q_next = NULL;
	vrp->vr_ridq.q_prev = NULL;
	vrp->vr_ridq.q_next = NULL;
	vrp->vr_cvrq.q_prev = NULL;
	vrp->vr_cvrq.q_next = NULL;
	vrp->vr_vsnq.q_prev = NULL;
	vrp->vr_vsnq.q_next = NULL;
	vrp->vr_scrq.q_prev = NULL;
	vrp->vr_scrq.q_next = NULL;
	return(vrp);
}

/*
 *  FindVolReq() - look up volreq by vsn - return 1st in 'hash' queue
 */

  VolReq_t *
FindVolReq(char *vsn)
{
	char	*id = "FindVolReq";

	strcpy(rid_entry->key, vsnstr(vsn));
	if (avl_find_key(rid_entry, &rid_ix) == AVL_IX_FAIL) 
		return(NULL);
	return((VolReq_t *) rid_entry->recptr);
}

/*
 *  FindVolReqSeq() - look up request by sequence
 */
  VolReq_t *
FindVolReqSeq(char *vsn, Seq_t seq)
{
	char		*id = "FindVolReqSeq";
	VolReq_t	*vrp;

	/*
	 *  for now, just search active q for seq; if traffic
	 *	increases, consider having RCs put vsn in err pkt
	 *	so vsn index can be used
	 */
	for (vrp=VRHEAD(VolReqQ); vrp; vrp=VRNEXT(vrp->vr_vrq)) {

		/*
		 *  see if sequence matches
		 */

		if (vrp->vr_rpseq == seq)
			break;
		nLogMsg(EL_DEBUG3, id, "skipped seq #%d", vrp->vr_rpseq);
	}
	return(vrp);
}
/*
 *  FindActiveVolReq() - find a request for vsn that is being worked on now
 */
  VolReq_t *
FindActiveVolReq(char *vsn)
{
	VolReq_t	*vrp;
	
	for (vrp=FindVolReq(vsn); vrp; vrp=VRNEXT(vrp->vr_vsnq)) {

		/*
		 *  see if state active TODO - consider broader criteria
		 *		for activity, including VS_MOUNTED.. no need
		 *		for any particular purpose now..
		 */

		if (vrp->vr_state == VS_RCWAIT  ||  vrp->vr_state == VS_MOVING)
			return(vrp);
	}
	return(vrp);
}

/*
 *  EnQVolReq() - put volreq in non-request queues: client, rid, vsn
 */
 void
EnQVolReq(VolReq_t *vrp)
{
	char		*id = "EnQVolReq";
	Client_t	*cp = vrp->vr_clientp;
	Rid_t		*ridp = vrp->vr_ridp;
	VolReq_t	*ixvrp;

	if (cp) {
		ENQUEUE(vrp, &cp->c_cvrq, vr_cvrq, VolReq_t)
	} else {
		vrp->vr_voflag &= ~VO_RESPONSE;
		vrp->vr_voflag |= VO_ASYNC;
	}
	if (ridp == NULL)
		nLogMsg(EL_FATAL, id, "ridp is null");
	ENQUEUE(vrp, &ridp->rid_vsnq, vr_ridq, VolReq_t)
	ridp->rid_vsns++;

	/*
	 *  vsn index hash queue - put volreq in index if vsn is unique, 
	 *	otherwise queue it after already indexed entry
	 */

	ixvrp = FindVolReq((char *)&vrp->vr_vsn);
	if (ixvrp == vrp) {
		nLogMsg(EL_ERROR1, id, "doubly queueing %s", 
					vsnstr((char *)&vrp->vr_vsn));
		kill(getpid(), 5);
	}
	if (ixvrp == NULL) {

		/*
		 *  add index entry
		 */

		rid_entry->recptr = vrp;
		/* strncpy was done in FindVolReq, but.. */
		strcpy(rid_entry->key, vsnstr((char *)&vrp->vr_vsn));
		if (avl_add_key(rid_entry, &rid_ix) == AVL_IX_FAIL) 
			nLogMsg(EL_FATAL|EL_SYS, id, "avl_add_key(%s) failed", rid_entry->key);
		vrp->vr_vsnq.q_next = NULL;
		vrp->vr_vsnq.q_prev = NULL;
		nLogMsg(EL_DEBUG3, id, "added %s_%d to vsn index", 
				rid_entry->key, vrp->vr_rpseq);
	} else {

		/*
		 *  put at tail of list
		 */

		vrp->vr_vsnq.q_next = NULL;

		while (VRNEXT(ixvrp->vr_vsnq) != NULL)
			ixvrp = VRNEXT(ixvrp->vr_vsnq);

		vrp->vr_vsnq.q_prev = (char *) ixvrp;
		ixvrp->vr_vsnq.q_next = (char *) vrp;
		
		nLogMsg(EL_DEBUG3, id, "added %.*s_%d to vsn q %.*s", 
			VSNLEN, &vrp->vr_vsn, vrp->vr_rpseq, 
			VSNLEN, &ixvrp->vr_vsn);
	}
}

/*
 *  FindWaitingMount() - find a mount request 
 *	in single-vsn implicit rsrv waiting for vsn
 *	vrp assumed to have VO_HANDOFF set
 */
 VolReq_t	*
FindWaitingMount(VolReq_t *vrp)
{
	char		*id = "FindWaitingMount";
	VolReq_t	*vrp2;

	nLogMsg(EL_DEBUG3, id, "%.*s_%d %lld", 
		VSNLEN, (char *)&vrp->vr_vsn, vrp->vr_rpseq, vrp->vr_ridp);

	/*
	 *  if there is another mount request for the vsn w/ same
	 *	uid & implicit, single-vsn rsrv, return it. 
	 *	The fairness isn't strict, but efficiency justifies it..
	 *	TODO - limit # of handoffs if a req is being starved.
	 */

	for (vrp2=VRNEXT(vrp->vr_vsnq); vrp2; vrp2=VRNEXT(vrp2->vr_vsnq)) {
		nLogMsg(EL_DEBUG3, id, "next %.*s %lld", 
				VSNLEN, (char *)&vrp2->vr_vsn, vrp2->vr_ridp);
		if (vrp2->vr_uid != vrp->vr_uid)
			continue;
		if (vrp2->vr_ridp->rid_id[0])
			continue;
		if (vrp2->vr_ridp->rid_vsns > 1)
			continue;
		if (vrp2->vr_state == VS_MOUNTWAIT  ||
		    vrp2->vr_state == VS_DRIVEWAIT  ||
		    vrp2->vr_state == VS_RESERVEMOUNTWAIT)
			return(vrp2);
	}
	for (vrp2=VRPREV(vrp->vr_vsnq); vrp2; vrp2=VRPREV(vrp2->vr_vsnq)) {
		nLogMsg(EL_DEBUG3, id, "prev %.*s", 
						VSNLEN, (char *)&vrp2->vr_vsn);
		if (vrp2->vr_uid != vrp->vr_uid)
			continue;
		if (vrp2->vr_ridp->rid_id[0])
			continue;
		if (vrp2->vr_ridp->rid_vsns > 1)
			continue;
		if (vrp2->vr_state == VS_MOUNTWAIT  ||
		    vrp2->vr_state == VS_DRIVEWAIT  ||
		    vrp2->vr_state == VS_RESERVEMOUNTWAIT)
			return(vrp2);

	}
	nLogMsg(EL_DEBUG3, id, "no match");
	return(NULL);
}

/*
 *  FindWaitingUnMount() - find an unmount request in grace waiting period
 *	(vrp must be single-vsn implicit rsrv; waiting vrp assumed to be)
 */
 VolReq_t	*
FindWaitingUnmount(VolReq_t *vrp)
{
	char		*id = "FindWaitingUnmount";
	VolReq_t	*vrp2;

	nLogMsg(EL_DEBUG3, id, "%.*s_%d %lld", 
		VSNLEN, (char *)&vrp->vr_vsn, vrp->vr_rpseq, vrp->vr_ridp);

	if (vrp->vr_ridp->rid_id[0])
		nLogMsg(EL_FATAL, id, "for explicit rsrv - not allowed");
	if (vrp->vr_ridp->rid_vsns > 1)
		nLogMsg(EL_FATAL, id, "for rsrv >1 vsn - not allowed");

	/*
	 *  if there is an unmount request for the vsn in this
	 *	reservation, return it. 
	 */

	for (vrp2=VRNEXT(vrp->vr_vsnq); vrp2; vrp2=VRNEXT(vrp2->vr_vsnq)) {
		nLogMsg(EL_DEBUG3, id, "next %.*s %lld", 
				VSNLEN, (char *)&vrp2->vr_vsn, vrp2->vr_ridp);
		if (vrp2->vr_state != VS_UNMOUNTWAIT)
			continue;
		if (vrp2->vr_uid != vrp->vr_uid)
			continue;
		return(vrp2);
	}
	for (vrp2=VRPREV(vrp->vr_vsnq); vrp2; vrp2=VRPREV(vrp2->vr_vsnq)) {
		nLogMsg(EL_DEBUG3, id, "prev %.*s", 
						VSNLEN, (char *)&vrp2->vr_vsn);

		if (vrp2->vr_state != VS_UNMOUNTWAIT)
			continue;
		if (vrp2->vr_uid != vrp->vr_uid)
			continue;
		return(vrp2);
	}
	nLogMsg(EL_DEBUG3, id, "no match");
	return(NULL);
}

 static void
PrintVsnQ(int lev, char *vsnp, char *id)
{
	VolReq_t	*vrp;

	if (!(lev & nLogLevel()))
		return;

	nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"Volume %s queue --------------------------------",
			vsnstr(vsnp));
	vrp = FindVolReq(vsnp);
	if (vrp == NULL) {
		nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"--------------------------------------------");
		return;
	}

	nLogMsg(EL_ALWAYS|EL_NOTIME, NULL,
	  "vsn    state   uid  RC1    RC2    flg vrp  clnt devinuse/totry");
	for ( ; vrp; vrp=VRNEXT(vrp->vr_vsnq)) 
		PrintVolReq(vrp, EL_ALWAYS, NULL);
	nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"--------------------------------------------");
}

/*
 *  FreeVolReq() - clean up queuing & free
 */
 void
FreeVolReq(VolReq_t *vrp, char *cid)
{
	char		*id = "FreeVolReq";
	Client_t	*cp = vrp->vr_clientp;
	Rid_t		*ridp = vrp->vr_ridp;
	VolReq_t	*vrpprev = VRPREV(vrp->vr_vsnq);
	VolReq_t	*vrpnext = VRNEXT(vrp->vr_vsnq);

	nLogMsg(EL_DEBUG3, id, "delete %.*s_%d/%d",
			VSNLEN, &vrp->vr_vsn, vrp->vr_rpseq, vrp->vr_rpord);
	PrintVsnQ(EL_DEBUG3, (char *)&vrp->vr_vsn, id);

	/*
	 *  DEQUEUE does no harm if not queued
	 */

	DEQUEUE(vrp, &VolReqQ, vr_vrq, VolReq_t)
	if (cp) 
		DEQUEUE(vrp, &cp->c_cvrq, vr_cvrq, VolReq_t)

	/*
	 *  dequeue from rid, then consider whether to
	 *	delete rid (other way around would recurse
	 *	since deleting rid deletes requests)
	 */
	DEQUEUE(vrp, &ridp->rid_vsnq, vr_ridq, VolReq_t)
	ridp->rid_vsns--;
	if (ridp->rid_vsns == 0  &&  !ridp->rid_id[0]) {

		/*
		 *  delete the rid too - it was created for this volreq
		 */

		DeleteRid(ridp, ridp->rid_cp);
	}

	if (vrpprev == NULL) {
		VolReq_t	*vrp2 = FindVolReq((char *)&vrp->vr_vsn);

		/*
		 *  this is the head of the index hash queue: get the index
		 */

		nLogMsg(EL_DEBUG3, id, "delete vsnq q head %.*s_%d",
					VSNLEN, &vrp->vr_vsn, vrp->vr_rpseq);
		if (vrp != vrp2) {
			PrintVolReq(vrp, EL_ERROR1, "seek");
			PrintVsnQ(EL_ERROR1, (char *)&vrp->vr_vsn, id);
			if (VRNEXT(vrp->vr_vsnq)) {
				nLogMsg(EL_ERROR1, 0, "next defined");
				PrintVolReq(VRNEXT(vrp->vr_vsnq), EL_ERROR1, 
								"next");
			}
			if (vrp2) {
				nLogMsg(EL_ERROR1, id, 
					"bad hash q defn, got %s_%d",
					vsnstr((char *)&vrp2->vr_vsn), 
					vrp2->vr_rpseq);
				PrintVolReq(vrp2, EL_ERROR1, "vrp2");
				PrintVsnQ(EL_ERROR1, (char *)&vrp2->vr_vsn, id);
			} else 
			  nLogMsg(EL_ERROR1, id, "vsn %s missing from index",
						vsnstr((char *)&vrp->vr_vsn));
			kill(getpid(), 5);
		}

		/*
		 *  delete index entry and index vrp->vr_vsnq.q_next, if any
		 */

		if (avl_delete_key(rid_entry, &rid_ix) == AVL_IX_FAIL)
			nLogMsg(EL_FATAL|EL_SYS, id, "avl_delete_key(%s)", rid_entry->key);
		if (vrpnext) {
			rid_entry->recptr = vrpnext;
			if (avl_add_key(rid_entry, &rid_ix) == AVL_IX_FAIL)
				nLogMsg(EL_FATAL|EL_SYS, id, "(%s) avl_add_key(%s)",
									cid, rid_entry->key);
			vrpnext->vr_vsnq.q_prev = NULL;
		}
	} else {

		if (vrpnext != NULL)
			nLogMsg(EL_DEBUG3, id, 
					"delete vsn q middle %.*s_%d",
					VSNLEN, &vrp->vr_vsn, vrp->vr_rpseq);
		else
			nLogMsg(EL_DEBUG3, id, 
					"delete vsn q tail %.*s_%d",
					VSNLEN, &vrp->vr_vsn, vrp->vr_rpseq);
		/*
		 *  dequeing from middle or end of queue
		 */

		vrpprev->vr_vsnq.q_next = (char *) vrpnext;
		if (vrpnext != NULL) {
			/* prev exists for all but index entry volreq */
			vrpnext->vr_vsnq.q_prev = (char *) vrpprev;
		}
	}
	PrintVsnQ(EL_DEBUG3, (char *)&vrp->vr_vsn, id);
	nFree(vrp);
	nLogMsg(EL_DEBUG3, id, "done");
}

/*
 *  Orphan() - take vsn away from its client, possibly for another client
 */
 void
Orphan(VolReq_t *vrp, Client_t *cp)
{
	char		*id = "Orphan";
	Client_t	*cp2 = vrp->vr_clientp;
	extern int	shuttingdown;

	if (cp2 == NULL) {
		nLogMsg(EL_DEBUG3, id, "(no client to orphan from)");
		return;
	}
	
	/*
	 *  remove from client queue 
	 */
	
	DEQUEUE(vrp, &cp2->c_cvrq, vr_cvrq, VolReq_t)

	if (cp2 != cp  &&  vrp->vr_voflag & VO_WAIT) {

		/*
		 *  A client cares about this request being chopped 
		 *	for another client: notify.
		 */

		if (shuttingdown)
			VMError = E_DOWN;
		else
			VMError = E_CANCELED;
		if (cp == NULL)
			nLogMsg(EL_CODE|EL_ERROR2, id, 
				"request (%s_%d) deleted", 
				vsnstr((char *)&vrp->vr_vsn), vrp->vr_rpseq);
		else
			nLogMsg(EL_CODE|EL_ERROR2, id, 
				"request (%s_%d) deleted by %s pid %d on %s", 
				vsnstr((char *)&vrp->vr_vsn), vrp->vr_rpseq,
				nGetUIDName(cp->c_uid), cp->c_pid,
				IPCHost(cp)->h_name);
		if (vrp->vr_clientp)
			Err2Client(vrp->vr_clseq, vrp->vr_clientp);
	}
	vrp->vr_clientp = NULL;
	vrp->vr_voflag &= ~VO_WAIT;
}

/*
 *  CancelRequest() - remove volreq from rid; put it in RP rid if
 *	it's waiting for a response
 */
 void
CancelRequest(VolReq_t *vrp, Client_t *cp, char *cid)
{
	char	*id = "CancelRequest";
	int	unmount = 0, delete = 0;

	switch (vrp->vr_state) {
	    case VS_MOUNTED:
	    case VS_RCWAIT:
	    case VS_UNMOUNTWAIT:
	    case VS_DELETENODE:
		unmount++;
		break;
	    case VS_RESERVEMOUNTWAIT:
	    case VS_RESERVEMOVEWAIT:
	    case VS_DRIVEWAIT:
	    case VS_MOUNTWAIT:
	    case VS_MOVEWAIT:
	    case VS_IDLE:
		delete++;
		break;
	    case VS_RCWAIT2:
	    case VS_MOVING:
	    case VS_EJECTED:
		RPRidp->rid_active++;
		vrp->vr_ridp->rid_active--;
		break;
	    default:
		nLogMsg(EL_CONSOLE0|EL_FATAL, id, 
					"%s_%d: unexpected state: %s",
					vsnstr((char *)&vrp->vr_vsn), 
					vrp->vr_rpseq,
					vReqStateStr(vrp->vr_state));
	}
	if (unmount) {
		int		index;
		RidRCMed_t	*ridrcmed;
		
		/*
		 *  a drive reservation has been used: transfer it to
		 *	RPRid
		 */
		index = vrp->vr_rcp->rc_index * RC_MEDIA_TYPES +
			RCMedium(vrp->vr_rcp, 
				 vVolTypeMedium(vrp->vr_info.vi_voltype));
		ridrcmed = &RPRidp->rid_RCMed[index];
		ridrcmed->rr_softused++;
		ridrcmed->rr_softalloc++;
		ridrcmed->rr_implicit++;
		RPRidp->rid_active++;
		vrp->vr_ridp->rid_active--;
		nLogMsg(EL_STATUS1, id, "(%s) unmount %s_%d",
				cid, vsnstr((char *)&vrp->vr_vsn), 
				vrp->vr_rpseq);
	}
	MoveVolReq2RPRid(vrp, cp);	/*  give to RPrid */
	if (unmount)
		Unmount(vrp);
	if (delete)
		FreeVolReq(vrp, cid);
}

 void
ErrOutVolReq(VolReq_t *vrp, char *id)
{
	Rid_t	*ridp = vrp->vr_ridp;

	if (ridp->rid_parallel == 1) {
		VolReq_t	*vrp2;
		/*
		 *  this is the 1st to go wrong in a
		 *	parallel mount (others may be
		 *	failing, too). Inform client
		 *	& detach/cancel other reqs
		 */
		ridp->rid_parallel = 0;
		Err2Client(ridp->rid_clseq, ridp->rid_cp);
		FreeVolReq(vrp, id);

		for (vrp=VRHEAD(ridp->rid_vsnq); vrp; vrp=vrp2) {
			vrp2 = VRNEXT(vrp->vr_ridq);	/* in case vrp dq'd */
			CancelRequest(vrp, NULL, "parallel cancel");
		}
		DeleteRid(ridp, NULL);

		return;
	}
	if (ridp->rid_parallel == 2) {
		Seq_t	seq = vrp->vr_clseq;
		/*
		 *  delete this req, & if no others in rid,
		 *	error to client
		 */
		FreeVolReq(vrp, id);
		if (VRHEAD(ridp->rid_vsnq) == NULL) {
			VMError = E_UNAVAIL;
			nLogMsg(EL_ERROR1, id, "all requests failed");
			Err2Client(seq, ridp->rid_cp);
			DeleteRid(ridp, NULL);
		}
		return;
	}
	if (vrp->vr_voflag & VO_WAIT)
		Err2Client(vrp->vr_clseq, vrp->vr_clientp);
	if (vrp->vr_flag & VF_IMPLICITVSN) {
		FreeVolReq(vrp, id);
	} else {
		DEQUEUE(vrp, &VolReqQ, vr_vrq, VolReq_t)
		vrp->vr_state = VS_IDLE;
		vrp->vr_time = Now;
		vrp->vr_qtime = 0;
		vrp->vr_rctime1 = 0;
		vrp->vr_usrtime = 0;
		vrp->vr_rctime2 = 0;
	}
}

 void
PrintVolReq(VolReq_t *vrp, int lev, char *id)
{
	char	dmstr[LINELEN * 2], *vReqStateStr();
	char	vsnrec[LINELEN];

	if (!(lev & nLogLevel()))
		return;

	if (vrp == NULL) {
		nLogMsg(EL_ERROR1, id, "INTERNAL ERROR: null pointer");
		return;
	}
	sprintf(vsnrec, "%s_%d/%d %s", 
			vsnstr((char *)&vrp->vr_vsn), vrp->vr_rpseq, 
			vrp->vr_rpord,
			vReqStateStr(vrp->vr_state));

	strcpy(dmstr, vDevMapStr(&vrp->vr_devinuse));
	strcat(dmstr, "/");
	strcat(dmstr, vDevMapStr(&vrp->vr_devtotry));
		
	nLogMsg(EL_ALWAYS|EL_NOTIME, id, " %s %4d %6.6s %6.6s %2x  %s", 
		  vsnrec, vrp->vr_uid,
		  (vrp->vr_rcp ? vrp->vr_rcp->rc_id : "undef"),
		  (vrp->vr_dstrcp ? vrp->vr_dstrcp->rc_id : "undef"),
		  vrp->vr_flag & 0xff,
		  dmstr);
}

 void
PrintVolReqQ(int lev, char *id)
{
	VolReq_t	*vrp;

	if (!(lev & nLogLevel()))
		return;

	nLogMsg(EL_ALWAYS|EL_NOTIME, id, "");
	nLogMsg(EL_ALWAYS|EL_NOTIME, 0, 
			"Volume Request queue ==============================");
	vrp = VRHEAD(VolReqQ);
	if (vrp == NULL) {
		nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"--------------------------------------------");
		return;
	}

	nLogMsg(EL_ALWAYS|EL_NOTIME, 0,
	  "vsn    state   uid  RC1    RC2    flg vrp  clnt devinuse/totry");
	for ( ; vrp; vrp = VRNEXT(vrp->vr_vrq))
		PrintVolReq(vrp, EL_ALWAYS, NULL);
	nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"--------------------------------------------");
}

 void
PrintClientVsns(Client_t *cp, char *id)
{

	VolReq_t	*vrp;

	vrp = VRHEAD(cp->c_cvrq);
        if (vrp == NULL)
		return;

	nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"client uid %d pid %d --------------------------------",
			cp->c_uid, cp->c_pid);
	nLogMsg(EL_ALWAYS|EL_NOTIME, NULL,
	  "vsn    state   uid  RC1    RC2    flg vrp  clnt devinuse/totry");
	for ( ; vrp; vrp=VRNEXT(vrp->vr_cvrq))
		PrintVolReq(vrp, EL_ALWAYS, NULL);
	nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"--------------------------------------------");
}

 void
PrintReservationReqs(int lev, Rid_t *ridp, char *id)
{

	VolReq_t	*vrp;

	if (!(lev & nLogLevel()))
		return;

	nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"rid %d vol q --------------------------------", ridp);
	vrp = VRHEAD(ridp->rid_vsnq);
	if (vrp == NULL)
		return;

	nLogMsg(EL_ALWAYS|EL_NOTIME, NULL,
	  "vsn    state   uid  RC1    RC2    flg vrp  clnt devinuse/totry");
	for ( ; vrp; vrp=VRNEXT(vrp->vr_ridq))
		PrintVolReq(vrp, EL_ALWAYS, NULL);
	nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
			"--------------------------------------------");
}

