/*
 * Volume Manager
 *
 *  volreq.h: per-volume-request data structure
 *
 */

#ifndef	_VOLREQ_
#define	_VOLREQ_

/*
 *  VolReq_t: one of these exists for each volume request received by the RP
 *
 *	Queue connections:
 *
 *	1) vr_vrq:  item on the work queue
 *	2) vr_cvrq: item on a queue headed by a Client_t struct
 *		(not used for VO_ASYNC requests)
 *	3) vr_ridq:  same reservation
 *	4) vr_vsnq:  same vsn (functions as a hash bucket for 
 *		the vsn work index)
 *	5) vr_scrq:  scratch queue for batching requests to RCs
 *
 * In the case of a move, vr_srcrc is the source RC number,
 *	and vr_rcp is a pointer to the dest RC struct.
 */
typedef struct	VolReq {
    QLink_t	vr_vrq;			/* VolReqQ item			*/
    QLink_t	vr_cvrq;		/* client work queue item	*/
    QLink_t	vr_ridq;		/* same rid q (for volintersect)*/
    QLink_t	vr_vsnq;		/* same vsn q (index bucket)	*/
    QLink_t	vr_scrq;		/* scratch q for batching to RCs*/
    struct VolReq *vr_headvrp;		/* head of scratch q		*/
    Client_t 	*vr_clientp;		/* Client_t struct or 0 if detached */
    Host_t	*vr_clhostp;		/* host that req came from	*/
    int		vr_uid;			/* uid of requestor client	*/
    int		vr_clhost;		/* host index /requestor client	*/
    int		vr_cmd;			/* request type from vm_p.h	*/
    char	vr_tag[TAGLEN];		/* user tag if mount		*/
    Rid_t	*vr_ridp;		/* reservation struct for bank	*/
    RPRC_t	*vr_rcp;		/* pointer to rc description	*/
    RPRC_t	*vr_dstrcp;		/* pointer to rc description	*/
    Seq_t	vr_clseq;		/* msg seq no of client req	*/
    Seq_t	vr_rpseq;		/* msg seq no of RP req		*/
    Seq_t	vr_rpord;		/* order of req in RP		*/
		/* vr_state & vr_flag values are defined in vm_volstat.h */
    int		vr_state;		/* request state VS_		*/
    int	 	vr_flag;		/* volume request flags VF_	*/
    int	 	vr_voflag;		/* volop (mount) flags VO_	*/
    VInfo_t	vr_info;		/* subset of VDR		*/
    DevMap_t 	vr_devtotry;		/* device to try bitmap		*/
    DevMap_t 	vr_devinuse;		/* device in use bitmap		*/
    Host_t	*vr_ndhostp;		/* host drive is on		*/
    int	 	vr_retrycnt;		/* retry count			*/
    time_t	vr_time;		/* timeouts			*/
    time_t	vr_idletime;		/* volnd report on mounted node */
    time_t	vr_qtime;		/* time spent queued		*/
    time_t	vr_rctime1;		/* rc time to mount		*/
    time_t	vr_usrtime;		/* time left mounted		*/
    time_t	vr_rctime2;		/* unmount time			*/
    /* vr_dep;	hard reservation to wait on	*/
} VolReq_t;

#define	VolReqNULL	((VolReq_t *) 0)

#define	vr_vsn		vr_info.vi_vsn
#define	vr_rc		vr_info.vi_srcrc
#define	vr_srcrc	vr_info.vi_srcrc
#define	vr_dstrc	vr_info.vi_dstrc
#define	vr_owneruid	vr_info.vi_uid
#define	vr_gid		vr_info.vi_gid
#define	vr_mode		vr_info.vi_mode
#define	vr_label	vr_info.vi_label
#define vr_voltype	vr_info.vi_voltype

/* #defines for accessing queues */
#define EnQVolCVR(volp, headp)	ENQUEUE(volp, headp, vr_cvrq, VolReq_t)
#define	DeQVolCVR(volp, headp)	DEQUEUE(volp, headp, vr_cvrq, VolReq_t)

#define VRHEAD(qhead)		((VolReq_t *) (qhead).q_head)
#define VRNEXT(qlink)		((VolReq_t *) (qlink).q_next)
#define VRPREV(qlink)		((VolReq_t *) (qlink).q_prev)

#endif	/* _VOLREQ_ */
