
/*
 *  rp_xdr.c: Volume Manager vold XDR routines for communicating 
 *	with its servers
 *
 *	All vold communication uses XDR over Internet sockets; 
 *	inet sockets are used so that connections can be made
 *	from remote hosts, and XDR is used to translate packets
 *	to and from a universal format for making sure that packet
 *	data structures are the same on different host architectures.
 *
 *	An array of XDR structures parallels the array of IPC_t
 *	socket management structures.
 *
 */

#include "rp_space.h"
#include <rpc/xdr.h>

extern IPC_t		SockArray[];
static XDR		XDRArray[FD_SETSIZE];

  void
XDRInit()
{
	IPC_t	*ip = SockArray;
	XDR	*xp = XDRArray;
	int	i;

	/*
	 *  associate socket array w/ XDR array
	 */
	for (i=0; i<FD_SETSIZE; i++, ip++, xp++)
		XDRSetup(ip, xp);
}

  vmpHdr_t *
XDRRecvI(int i)
{
	char		*id = "XDRRecvI";
	vmpHdr_t	*hp;

	if (i < 0  ||  i >= FD_SETSIZE)
		nLogMsg(EL_FATAL, id, "index i out of range: %d", i);

	hp = XDRRecv(&XDRArray[i]);
	if (hp == NULL)
		return(NULL);

	if (SockArray[i].sock_ptype != PT_NONE  &&
	    hp->vh_key != SockArray[i].sock_rcvkey) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
				"sock %d bad key %d expected %d, type %s",
				i, hp->vh_key, SockArray[i].sock_rcvkey,
				vPortTypeStr(SockArray[i].sock_ptype));
		return(NULL);
	}
	return(hp);

}

  int
XDRNextRecordI(int i)
{
	return(XDRNextRecord(&XDRArray[i]));
}

  int
XDRSendI(vmpHdr_t *hp, int i, int now)
{
	if (i < 0  ||  i >= FD_SETSIZE)
		nLogMsg(EL_FATAL, "XDRSendI", "index i out of range: %d", i);

	hp->vh_key = SockArray[i].sock_sndkey;
	return(XDRSend(hp, &XDRArray[i], now));
}

  void
XDRFlushI(int slot)
{
	XDRFlush(&XDRArray[slot]);
}
