/*
 *  vaudit: dump vsns according to Repository Controller, uid, gid;
 *	do counts by uid, RC, type..
 *
 *	Bill Ross, NASA Ames, 3/92
 */

#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <pwd.h>
#include <grp.h>
#include <sys/types.h>
#include <unistd.h>
#include <netdb.h>

#include "cl_space.h"
#include "rp_db.h"

extern int errno;

extern int		RCTotal;
extern RC_t		*RCArray;

int			VolManOperUID;	/* for linking w/ volserv lib */
static char		*arg0;

static AVL_IX_DESC	rc_ix, uid_ix;
static AVL_IX_REC	*ix_ent = NULL;

/*
 *  Print out usage message and exit.
 */
 void
usage()
{
	fprintf(stderr, "usage: %s -a\n", arg0);
	fprintf(stderr,
	  "       %s -r RC -m medium -s status -u user -g group -P poolgid\n",
	  arg0);
	fprintf(stderr, "       %s -e consecutive_mount_errors\n", arg0);
	fprintf(stderr, "       %s -R\n", arg0);
	vm_prRC(stderr, "", NULL, 1);
	fprintf(stderr, "\n-s:\tStatus\tDescription\n");
	vPrintStatTab(stderr);

	exit(1);
}

 static int
toInt(int *nump, char *str)
{
	int     i;

	if (str[INTLEN] != ' ') {
		fprintf(stderr, "db int field discrepancy (%.*s)\n",
							INTLEN+1, str);
		return(1);
	}
	for (i=0; i<INTLEN; i++) {
		if (str[i] != ' '  &&  !isdigit(str[i])  &&  str[i] != '-') {
			fprintf(stderr, 
				"int field (%.*s) has non-numeral",INTLEN, str);
			return(1);
		}
	}
	*nump = atoi(str);
	return(0);
}

int
useruid(char *name)
{
	struct passwd *pwd;
	extern struct passwd *getpwnam();

	pwd = getpwnam(name);
	if (pwd == NULL) {
		fprintf(stderr, "unknown user %s\n", name);
		usage();
		return(-1);		/* for compiler warning */
	} else
		return(pwd->pw_uid);
}

int
groupgid(char *name)
{
	struct group *grp;
	extern struct group *getgrnam();

	grp = getgrnam(name);
	if (grp == NULL) {
		fprintf(stderr, "unknown group %s\n", name);
		usage();
		return(-1);		/* for compiler warning */
	} else
		return(grp->gr_gid);
}


 void
printRC(int i)
{
	char	*cp2, *cp = ix_ent->key;
	char	rc[RC_STRLEN+1], voltype[100];
	RC_t	*rcp;

	if (!i)
		printf("RC      voltype     status    count\n");

	for (i=0; *cp != ' '; i++, cp++)
		rc[i] = *cp;
	rc[i] = '\0';
	rcp = cGetRcp(rc);
	if (rcp)
		strcpy(rc, rcp->rcd_id);

	cp++;
	for (i=0; *cp != ' '; i++, cp++)
		voltype[i] = *cp;
	voltype[i] = '\0';
	i = atoi(voltype);
	cp2 = vVolTypeName(i);
	if (cp2)
		strcpy(voltype, cp2);

	cp++;
	
	printf("%.*s %8.8s       %c       %d\n", 
				RC_STRLEN, rc, voltype, *cp, ix_ent->count);
}


main(int argc, char *argv[])
{
	extern  int     optind;
	extern  char    *optarg;
	char		myhost[MAXHOSTNAMELEN];
	int		cc, ret, fd, i;
	int		doall=0, dorc=0, domedium=0, douid=0; 
	int		dogid=0, dopool=0, doerr=0, dotypes, errcut;
	int		rcstats = 0, uidstats = 0, stats = 0;
	int		status = 0, media=0, uid, gid, poolgid;
	char		dbf_vsn_tbl[MAXPATHLEN];
	RC_t		*rcp;
	vbuf		vsnbuf;
	VInfo_t		vinfo;

	arg0 = nas_basename(argv[0]);

	if (gethostname(myhost, MAXHOSTNAMELEN) == -1) {
		perror("getting host name");
		exit(1);
	}
	if (strncmp(myhost, vVoldHost, strlen(myhost))) {
		fprintf(stderr, 
			"%s: must be run on vold host:\n\n\t\t%s\n\n%s\n\n",
			arg0, vVoldHost, "to directly open dbase.");
		exit(1);
	}

	/*
	 *  get RC config 
	 */
	if (_vm_init(arg0, 0)) {
		vm_prerr(stderr);
		exit(VMError);
	}
	vm_configRC();
	vm_term();

	if (argc == 1)
		usage();

	/*
	 *  set rcd_flags to 1 == skip
	 */
	for (i=0,rcp=RCArray; i<RCTotal; i++, rcp++)
		rcp->rcd_flags = 1;

	/*
	 *  get args
	 */

	while ((cc = getopt(argc, argv, "RUar:m:s:u:g:e:")) != EOF) {
		switch (cc) {
			
		    case 'a':
			doall = 1;
			break;
		    case 'e':
			if (!isnum(optarg)) {
				fprintf(stderr, " -e: need a number\n");
				usage();
			}
			doerr = 1;
			errcut = atoi(optarg);
			break;
		    case 'g':
			if (isnum(optarg))
				gid = atoi(optarg);
			else
				gid = groupgid(optarg);
			dogid++;
			break;
		    case 'm':
			i = vMediumCode(optarg);
			if (i == -1) {
				fprintf(stderr,"unknown medium: %s\n", optarg);
				usage();
			}
			media |= i;
			domedium++;
			break;
		    case 'r':
			if ((rcp = cGetRcp(optarg)) == NULL) {
				fprintf(stderr, "RC %s not defined\n", optarg);
				usage();
			}
			if (rcp->rcd_flags) {
				rcp->rcd_flags = 0;
				dorc++;
			}
			break;
		    case 's':
			i = vStatusBit(optarg);
			if (i == 0) {
				fprintf(stderr,"unknown status: %s\n", optarg);
				usage();
			}
			status |= i;
			break;
		    case 'u':
			if (isnum(optarg))
				uid = atoi(optarg);
			else
				uid = useruid(optarg);
			douid++;
			break;
		    case 'P':
			if (isnum(optarg))
				poolgid = atoi(optarg);
			else
				poolgid = groupgid(optarg);
			dopool++;
			break;
		    case 'R':
			rcstats = 1;
			break;
		    case 'U':
			uidstats = 1;
			break;
		    default:
			usage();
		}
	}
	stats = rcstats + uidstats;
	dotypes = dorc + domedium + douid + dogid + dopool;
	if (stats) {
		if (doall || doerr || dotypes) { 
			fprintf(stderr, 
			    "'stats' options incompatible w/ others\n");
			usage();
		}
		if (rcstats) {
			/*
			 *  make per-RC scratch index
			 */
			avl_create_index(&rc_ix, AVL_COUNT_DUPS, 0);
		}
		if (uidstats) {
			/*
			 *  make per-uid scratch index
			 */
			avl_create_index(&uid_ix, AVL_COUNT_DUPS, 0);
		}
		if (rcstats || uidstats) {
			/* index entry w/ plenty of space for string keys */
			ix_ent = (AVL_IX_REC *) nAlloc(sizeof(AVL_IX_REC) + 100);
		}
			
	} else if (doall  &&  (dotypes || doerr)) {
		fprintf(stderr, "`all' (-a) incompatible w/ selection\n");
		usage();
	} else if (doerr  &&  dotypes) {
		fprintf(stderr, "`errors' (-e) incompatible w/ selection\n");
		usage();
	}

	/*
	 *  open dbase vsn table
	 */
	sprintf(dbf_vsn_tbl, "%s/DB_VSN_TBL", vDBDir);
	if ((fd = open(dbf_vsn_tbl, O_RDONLY, 0))== -1) {
		perror(dbf_vsn_tbl);
		exit(1);
	}
	
	for (i=0; 1; i++) {
		int	mntfail;

		ret = read(fd, &vsnbuf, sizeof(vsnbuf));
		if (ret != sizeof(vsnbuf)) {
			if (!ret)
				break;
			if (ret == -1) {
				perror("reading vsn table");
				exit(1);
			}
			fprintf(stderr, "bad size record %d\n", ret);
			exit(1);
		}
		if (vsnbuf.v_status == S_DELETED)
			continue;
		if (   toInt(&vinfo.vi_uid,     vsnbuf.v_uid) 
		    || toInt(&vinfo.vi_gid,     vsnbuf.v_gid) 
		    || toInt(&vinfo.vi_poolgid, vsnbuf.v_poolgid) 
		    || toInt(&vinfo.vi_voltype, vsnbuf.v_voltype) 
		    || toInt(&vinfo.vi_srcrc,   vsnbuf.v_srcrc)
		    || toInt(&mntfail,          vsnbuf.v_mntfail)) {
			fprintf(stderr, "skipping %.*s\n", 
					VSNLEN, vsnbuf.v_vsn);
			continue;
		}
		if (stats) {
			if (rcstats) {
				sprintf(ix_ent->key, "%d %d %c",
						vinfo.vi_srcrc,
						vinfo.vi_voltype,
						vsnbuf.v_status);
				if (avl_add_key(ix_ent, &rc_ix) != AVL_IX_OK) {
					fprintf(stderr, "avl_add_key(%s) error\n",
								ix_ent->key);
					exit(1);
				}
			}
			if (uidstats) {
				if (vsnbuf.v_status != S_ALLOC)
					continue;
				sprintf(ix_ent->key, "%d %d %d",
						vinfo.vi_uid,
						vinfo.vi_srcrc,
						vinfo.vi_voltype);
				if (avl_add_key(ix_ent, &uid_ix) != AVL_IX_OK) {
					fprintf(stderr, "avl_add_key(%s) error\n",
								ix_ent->key);
					exit(1);
				}
			}
			continue;
		}
		if (doerr) {
			if (mntfail >= errcut)
				printf("%.*s  %d\n", VSNLEN, vsnbuf.v_vsn, 
							mntfail);
			continue;
		}
		if (douid)
			if (uid != vinfo.vi_uid)
				continue;
		if (dogid)
			if (gid != vinfo.vi_gid)
				continue;
		if (dopool)
			if (poolgid != vinfo.vi_poolgid)
				continue;
		if (status) {
			int	stat = vStatusToBit(vsnbuf.v_status);
			if (!stat) {
				fprintf(stderr, 
					"unknown status code %c (vsn %.*s)\n",
					vsnbuf.v_status, VSNLEN, vsnbuf.v_vsn);
			}
			if (!(stat & status))
				continue;
		}
		if (domedium) {
			int	med = vVolTypeMedium(vinfo.vi_voltype);
			if (!(med & media))
				continue;
		}
		if (dorc) {
			rcp = cRCInfo(vinfo.vi_srcrc);
			if (rcp == NULL)
				continue;
			if (rcp->rcd_flags)
				continue;
		}
		printf("%.*s\n", VSNLEN, vsnbuf.v_vsn);
	}

	if (rcstats) {
		/*
		 *  print contents of per-RC scratch index
		 */
		avl_first_key(&rc_ix);
		i = 0;
		while ((ret = avl_next_key(ix_ent, &rc_ix)) == AVL_IX_OK)
			printRC(i++);
		if (ret != AVL_EOIX) {
			fprintf(stderr, "rc stats: avl_next_key() err\n");
			exit(1);
		}
	}
	if (uidstats) {
		/*
		 *  print contents of per-uid scratch index
		 */
		avl_first_key(&uid_ix);
		i = 0;
		while ((ret = avl_next_key(ix_ent, &uid_ix)) == AVL_IX_OK) {
			printRC(i++);
		}
		if (ret != AVL_EOIX) {
			fprintf(stderr, "uid stats: avl_next_key() err\n");
			exit(1);
		}
	}
	exit(0);
}


