/*
**  "Media Changer" Repository Controller
**
**	ch_config.c:  Get & manage config/inventory
*/
#include "chrc.h"
#include "avltree.h"

extern RC_t		*rcp;
extern DevEnt_t		*DriveArray;	
extern DevMap_t		AvailDrives;
extern int		drive_uses[];
extern int		DriveTotal;
extern int		Pid;

static ch_stat_t	*Slots = NULL, *Drives = NULL;
static ch_stat_t	*Pickers = NULL, *Portals = NULL;
static ch_param_t	params;

static AVL_IX_DESC	VsnIx, EmptyIx;
static AVL_IX_REC	*recP = NULL;

/*
**	check_stat() - check status of generic changer element as
**		returned by querying the hardware for the element
**		type
*/
static int	fake_tag = 0;

static int
check_stat(ch_stat_t *elp, char *type, int i, char *id)
{
	if ((elp->stat.ces_flags & CESTATUS_STATUS_VALID) == 0) {
		nLogMsg(EL_STATUS1|EL_NOTIME, id, "%s %d no status", type, i);
		return(1);
	}
	if (elp->stat.ces_flags & CESTATUS_EXCEPT) {
		nLogMsg(EL_STATUS1|EL_NOTIME, id, "%s %d exception %d/%d", 
					type, i, elp->stat.ces_asc, 
					elp->stat.ces_ascq);
		return(1);
	}
	if ((elp->stat.ces_flags & CESTATUS_ACCESS) == 0) {
		nLogMsg(EL_STATUS1|EL_NOTIME, id, "%s %d inaccessible", 
								type, i);
		return(1);
	}
	if (elp->stat.ces_flags & CESTATUS_FULL) {
		/*
		**	some errors are fatal to bring the
		**		inconsistency to someone's
		**		attention
		*/
		if ((elp->stat.ces_flags & CESTATUS_PVOL_VALID) == 0)
			nLogMsg(EL_FATAL, id, "%s %d full but vsn not valid", 
								type, i);
		if (strlen(elp->stat.ces_pvoltag.cv_tag) == 0) {
			sprintf(elp->stat.ces_pvoltag.cv_tag, "?_%d",
							fake_tag++);
			nLogMsg(EL_ERROR1, id, 
					"%s %d: tag 0 len: generating tag %s",
					type, i, elp->stat.ces_pvoltag.cv_tag);
		} else if (strlen(elp->stat.ces_pvoltag.cv_tag) != VSNLEN)
			nLogMsg(EL_ERROR1, id, 
					"%s %d: tag len should be %d [%s]",
					type, i, VSNLEN,
					elp->stat.ces_pvoltag.cv_tag);
	}

	return(0);
}

void
ch_config(DevMap_t *drvmap)
{
	char				*id = "ch_config";
	ch_req_t			req;
	int				i;
	struct changer_element_status	*scratch;

	/*
	**	initialize avl tree indexing
	*/
	recP = (AVL_IX_REC *) nAlloc(sizeof(AVL_IX_REC) + VSNLEN + 1);
	avl_create_index(&VsnIx, AVL_NO_DUP_KEYS, 0);
	avl_create_index(&EmptyIx, AVL_NO_DUP_KEYS, 0);

	/*
	**	get params from changer
	*/
	bzero((void *)&params, sizeof(params));
	if (ioctl(changer_fd, CHIOGPARAMS, &params))
		nLogMsg(EL_FATAL|EL_SYS, id, "ioctl(CHIOGPARAMS)");

	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "-------------- configuration");
	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "slots:    %d", params.cp_nslots);
	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "drives:   %d", params.cp_ndrives);
	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "pickers:  %d", params.cp_npickers);
	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "portals:  %d", params.cp_nportals);
	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "current picker:  %d",
						params.cp_curpicker);

	/*
	**	fully cautious sizing of scratch array
	*/
	i = params.cp_nslots;
	if (params.cp_ndrives > i)
		i = params.cp_ndrives;
	if (params.cp_npickers > i)
		i = params.cp_npickers;
	if (params.cp_nportals > i)
		i = params.cp_nportals;

	scratch = (ch_elt_t *) nAlloc(i*sizeof(ch_elt_t));

	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "----------------------------");

	/*
	**	get full slot info 
	*/
	bzero((void *)&req, sizeof (req));
	req.cesr_data = scratch;
	req.cesr_type = CHET_ST;
	req.cesr_unit = 0;
	req.cesr_count = params.cp_nslots;
	req.cesr_flags = CESR_VOLTAGS;
	req.cesr_vendor_data = NULL;		/* ??? */
 
	if (ioctl(changer_fd, CHIOGSTATUS, &req))
		nLogMsg(EL_FATAL|EL_SYS, id, "ioctl(CHIOGSTATUS, slots)");

	/*
	**	examine/index slots
	*/
	Slots = (ch_stat_t *) nAlloc(params.cp_nslots * sizeof(ch_stat_t));
	for (i=0; i<params.cp_nslots; i++) {

		Slots[i].type = CHET_ST;
		Slots[i].index = i;
		Slots[i].stat = scratch[i];

		if (check_stat(&Slots[i], "slot", i, id))
			continue;

		/*
		**	add to Vsn or empty slot index
		*/
		recP->recptr = &Slots[i];
		if (Slots[i].stat.ces_flags & CESTATUS_FULL) {
			/*
			**	vsn index
			*/
			strcpy(recP->key, Slots[i].stat.ces_pvoltag.cv_tag);
			if (avl_add_key(recP, &VsnIx) != AVL_IX_OK)
				nLogMsg(EL_FATAL, id, 
						"slot %d vsn add_key(%s)", 
						i, recP->key);
		} else {
			/*
			**	empty slot index
			*/
nLogMsg(EL_STATUS1, id, "add emptyslot %d", i);
			sprintf(recP->key, "%d", i);
			if (avl_add_key(recP, &EmptyIx) != AVL_IX_OK)
				nLogMsg(EL_FATAL, id, 
						"slot %d 'empty' add_key(nil)",
						i);
		}
	}

	/*
	**	get full drive info 
	*/
	bzero((void *)&req, sizeof (req));
	req.cesr_data = scratch;
	req.cesr_type = CHET_DT;
	req.cesr_unit = 0;
	req.cesr_count = params.cp_ndrives;
	req.cesr_flags = CESR_VOLTAGS;
	req.cesr_vendor_data = NULL;		/* ??? */
 
	if (ioctl(changer_fd, CHIOGSTATUS, &req))
		nLogMsg(EL_FATAL|EL_SYS, id, "ioctl(CHIOGSTATUS, drives)");

	/*
	**	examine/index drives
	*/
	Drives = (ch_stat_t *) nAlloc(params.cp_ndrives * sizeof(ch_stat_t));
	for (i=0; i<params.cp_ndrives; i++) {
		int	bitdrv = ch_drives[i];

		Drives[i].type = CHET_DT;
		Drives[i].index = i;
		Drives[i].stat = scratch[i];

		if (check_stat(&Drives[i], "drive", i, id)) {
			DM_SET(bitdrv, *drvmap);
			continue;
		}

		/*
		**	deal with leftover mounts
		*/
		if (Drives[i].stat.ces_flags & CESTATUS_FULL) {
			ch_stat_t	*slotp;

			/*
			**	add to Vsn Index
			*/
			strcpy(recP->key, Drives[i].stat.ces_pvoltag.cv_tag);
			recP->recptr = &Drives[i];
			if (avl_add_key(recP, &VsnIx) != AVL_IX_OK)
				nLogMsg(EL_FATAL, id, 
						"drive %d vsn add_key(%s)", 
						i, recP->key);

			/*
			**	get slot
			*/
			slotp = ch_getemptyslot();
			if (slotp == NULL) {
				DM_SET(bitdrv, *drvmap);
				continue;
			}

			/*
			**	unmount
			*/
			nLogMsg(EL_STATUS1, id, 
					"dismounting %s drive %d to slot %d",
					Drives[i].stat.ces_pvoltag.cv_tag, i,
					slotp->index);
			if (ch_rawunmount(&Drives[i], slotp)) {
				/* TODO - varyoff msg */
				DM_SET(bitdrv, *drvmap);
				continue;
			}
			mark_move(&Drives[i], slotp);
		}
	}

	/*
	**	get full picker info 
	*/
	bzero((void *)&req, sizeof (req));
	req.cesr_data = scratch;
	req.cesr_type = CHET_MT;
	req.cesr_unit = 0;
	req.cesr_count = params.cp_npickers;
	req.cesr_flags = CESR_VOLTAGS;
	req.cesr_vendor_data = NULL;		/* ??? */
 
	if (ioctl(changer_fd, CHIOGSTATUS, &req))
		nLogMsg(EL_FATAL|EL_SYS, id, "ioctl(CHIOGSTATUS, pickers)");

	/*
	**	examine/index pickers
	*/
	Pickers = (ch_stat_t *) nAlloc(params.cp_npickers * sizeof(ch_stat_t));
	for (i=0; i<params.cp_npickers; i++) {

		Pickers[i].type = CHET_MT;
		Pickers[i].index = i;
		Pickers[i].stat = scratch[i];

		if (check_stat(&Pickers[i], "picker", i, id))
			continue;

		if (Pickers[i].stat.ces_flags & CESTATUS_FULL) {
			nLogMsg(EL_STATUS1|EL_CONSOLE0, id, "%s in picker",
					Pickers[i].stat.ces_pvoltag.cv_tag);
			strcpy(recP->key, Pickers[i].stat.ces_pvoltag.cv_tag);
			if (avl_add_key(recP, &VsnIx) != AVL_IX_OK)
				nLogMsg(EL_FATAL, id, 
						"picker %d vsn add_key(%s)", 
						i, recP->key);
		}
	}

	if (params.cp_nportals == 0) {
		/* worth keeping if a reason to reinit arises */
		nFree(scratch);
		return;
	}

	/*
	**	portals
	*/
	bzero((void *)&req, sizeof (req));
	req.cesr_data = scratch;
	req.cesr_type = CHET_IE;
	req.cesr_unit = 0;
	req.cesr_count = params.cp_nportals;
	req.cesr_flags = CESR_VOLTAGS;
	req.cesr_vendor_data = NULL;		/* ??? */
 
	if (ioctl(changer_fd, CHIOGSTATUS, &req))
		nLogMsg(EL_FATAL|EL_SYS, id, "ioctl(CHIOGSTATUS, portals)");

	/*
	**	examine/index pickers
	*/
	Portals = (ch_stat_t *)nAlloc(params.cp_nportals * sizeof(ch_stat_t));
	for (i=0; i<params.cp_nportals; i++) {

		Portals[i].type = CHET_IE;
		Portals[i].index = i;
		Portals[i].stat = scratch[i];

		if (check_stat(&Portals[i], "portal", i, id))
			continue;

		if (Portals[i].stat.ces_flags & CESTATUS_FULL) {
			/*
			**	vsn index
			*/
			strcpy(recP->key, Portals[i].stat.ces_pvoltag.cv_tag);
			if (avl_add_key(recP, &VsnIx) != AVL_IX_OK)
				nLogMsg(EL_FATAL, id, 
						"portal %d vsn add_key(%s)", 
						i, recP->key);
		}
	}
	nFree(scratch);
}


/*
**	ch_getvsn() - look up vsn, whatever kind of place
**		it is in
*/
ch_stat_t *
ch_getvsn(char *vsnp)
{
	char	*id = "ch_getvsn";

	strncpy(recP->key, vsnp, VSNLEN);
	if (avl_find_key(recP, &VsnIx) != AVL_IX_OK) {
		VMError = E_INTERNAL;
		nLogMsg(EL_ERROR1, id, "%s: not found", recP->key);
		return(NULL);
	}
	return((ch_stat_t *)recP->recptr);
}

/*
**	ch_getemptyslot() - get empty slot off list
**		(uses index mainly to be like the vsn routine;
**		LIFO queue would actually be faster :-)
*/
ch_stat_t *
ch_getemptyslot()
{
	char	*id = "ch_getemptyslot";

	avl_first_key(&EmptyIx);

	switch(avl_next_key(recP, &EmptyIx)) {
	    case AVL_IX_OK:
		return((ch_stat_t *)recP->recptr);
	    case AVL_EOIX:
		VMError = E_UNAVAIL;
		nLogMsg(EL_ERROR1|EL_CODE, id, "no empty slot");
		return(NULL);
	    default:
		nLogMsg(EL_FATAL, id, "unexpected");
	}
	return(NULL);	/* for checkers */
}

/*
**	get_elt_status() - get info on an element
*/
int
get_elt_status(ch_stat_t *ep, char *id, char *id2)
{
	ch_req_t	req;

	bzero((void *)&req, sizeof (req));
	req.cesr_data = &ep->stat;
	req.cesr_type = ep->type;
	req.cesr_unit = ep->index;
	req.cesr_count = 1;
	req.cesr_flags = CESR_VOLTAGS;
	req.cesr_vendor_data = NULL;		/* ??? */
 
nLogMsg(EL_STATUS1, id, "ioctl(CHIOGSTATUS) %s type %s unit %d count 1",
id2, EltTypeStr(ep->type), ep->index);

	if (ioctl(changer_fd, CHIOGSTATUS, &req))
		nLogMsg(EL_FATAL|EL_SYS, id, "ioctl(CHIOGSTATUS)");
nLogMsg(EL_STATUS1, id, "ioctl ok");

	if (check_stat(ep, id2, ep->index, id))
		return(1);

	nLogMsg(EL_STATUS2, id, "  (access etc ok)");
	return(0);
}

/*
**	mark_move() - register the fact that a vsn has moved
**		from one place to another
*/
void
mark_move(ch_stat_t *from, ch_stat_t *to)
{
	char		*id = "mark_move";

nLogMsg(EL_STATUS1, id, "from [%s] to [%s]",
from->stat.ces_pvoltag.cv_tag,
to->stat.ces_pvoltag.cv_tag);

	/*
	**	delete 'from' key before overwriting fields w/
	**		fresh inventory info, in case a temporary
	**		vsn was used to replace null tag from changer
	**		(seen on Spectralogic treefrog)
	*/
	strcpy(recP->key, from->stat.ces_pvoltag.cv_tag);
	if (avl_delete_key(recP, &VsnIx) != AVL_IX_OK)
		nLogMsg(EL_FATAL, id, "vsn avl_delete_key(%s)", recP->key);

	/*
	**	get/check destination
	*/
	(void)get_elt_status(to, id, "['to' elt]");
	if (from->stat.ces_pvoltag.cv_tag[0] != '?'  &&
	    strcmp(to->stat.ces_pvoltag.cv_tag, from->stat.ces_pvoltag.cv_tag))
		nLogMsg(EL_FATAL, id, "from [%s] should match to [%s]",
					from->stat.ces_pvoltag.cv_tag,
					to->stat.ces_pvoltag.cv_tag);
	if (!(to->stat.ces_flags & CESTATUS_FULL))
		nLogMsg(EL_FATAL, id, "to [%s] not full",
					to->stat.ces_pvoltag.cv_tag);
	if (from->stat.ces_pvoltag.cv_tag[0] == '?')
		nLogMsg(EL_STATUS1, id, "vsn now known: [%s]", 
					to->stat.ces_pvoltag.cv_tag);

	/*
	**	get/check source
	*/
	(void)get_elt_status(from, id, "['from' elt]");
	if (strlen(from->stat.ces_pvoltag.cv_tag))
		nLogMsg(EL_FATAL, id, "from [%s] should be empty",
					from->stat.ces_pvoltag.cv_tag);
	if (from->stat.ces_flags & CESTATUS_FULL)
		nLogMsg(EL_FATAL, id, "from [%s] full",
					from->stat.ces_pvoltag.cv_tag);

	/*
	**	update vsn index w/ destination tag
	*/
	strcpy(recP->key, to->stat.ces_pvoltag.cv_tag);
	recP->recptr = to;
	if (avl_add_key(recP, &VsnIx) != AVL_IX_OK)
		nLogMsg(EL_FATAL, id, "vsn avl_add_key(%s)", recP->key);

/*
nLogMsg(EL_STATUS1|EL_NOTIME, id, "----- empty slots:");
if (avl_first_key(&EmptyIx) != AVL_IX_OK)
nLogMsg(EL_FATAL, id, "avl_first_key");
while (avl_next_key(recP, &EmptyIx) != AVL_EOIX)
nLogMsg(EL_STATUS1, id, "    %s", recP->key);
nLogMsg(EL_STATUS1|EL_NOTIME, id, "-----------------");
*/
	/*
	**	update emptyslot index
	*/
	if (from->type == CHET_ST) {
		sprintf(recP->key, "%d", from->index);
		recP->recptr = from;
		if (avl_add_key(recP, &EmptyIx) != AVL_IX_OK)
			nLogMsg(EL_FATAL, id, "slot avl_delete_key(%s)", 
								recP->key);
	}
	if (to->type == CHET_ST) {
		sprintf(recP->key, "%d", to->index);
		recP->recptr = to;
		if (avl_delete_key(recP, &EmptyIx) != AVL_IX_OK)
			nLogMsg(EL_FATAL, id, "slot avl_add_key(%s)", 
								recP->key);
	}
}


/*
**  AllocateDrive() - Allocate drives in the most efficient manner possible.
**		Ensure equal use of all drives.
**
**		TODO - %%% this is biased toward the numerical order of
**			drives to the extent that there are few mounts 
**			per process start
**		TODO - %%% consider data paths & cart/drive locations for this
*/
int
AllocateDrive(RCReq_t *reqp)
{
	char		*id = "AllocateDrive";
	int		i, ch_drive, bitdrv;
	DevMap_t	okdrives;

	/*
	**	ok drives are ones set in the request pkt & not already used
	*/
	DM_AND(reqp->rr_devmap, AvailDrives, okdrives);
	nLogMsg(EL_DEBUG3, id, "req: %s", vDevMapStr(&reqp->rr_devmap));
	nLogMsg(EL_DEBUG3, id, "ch:  %s", vDevMapStr(&AvailDrives));
	nLogMsg(EL_DEBUG3, id, "and: %s", vDevMapStr(&okdrives));

	ch_drive = -1;
	for (i=0; i<params.cp_ndrives; i++) {
		bitdrv = ch_drives[i];

		if (bitdrv == -1)
			nLogMsg(EL_FATAL, id, "drive %d doesn't map", i);

nLogMsg(EL_STATUS1, id, "drive %d bitdrv %d", i, bitdrv);
		if (!DM_ISSET(bitdrv, okdrives))
			continue;

nLogMsg(EL_STATUS1, id, " -drive ok");
		if (ch_drive == -1) {
			ch_drive = i;
			continue;
		}
		if (drive_uses[i] < drive_uses[ch_drive])
			ch_drive = i;
	}
	if (ch_drive == -1) {
		VMError = E_UNAVAIL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "No acceptable drive");
		return(1);
	}
	drive_uses[ch_drive]++;
	reqp->rr_chdrvp = &Drives[ch_drive];
	reqp->rr_drive = bitdrv;
	return(0);
}

