/*
**  "Media Changer" Repository Controller
**
**	ch_init.c:  initialize connection with changer
**
TODO - uses hardwired /dev/chN - dev numbers (0 & 1 here) 
*/
#include "vm_space.h"		/* volman */
#include "chrc.h"	
#include <signal.h>

extern RC_t	*rcp;

#define DMOUNTTIMEOUT	200
#define RETRIES		3

Seq_t		initseq = 222;		/* seq # of initial msg */

char		changer_name[100];
int		changer_fd = -1;
int		drivestatus = 0;/* whether drive status known yet on startup */

static int	shuttingdown = 0;

/*
**  SigAlarm() - Handle alarm signal.
*/
static void
SigAlarm(sig)
{
	nLogMsg(EL_STATUS2, 0, "alarm");
	if (SIGNAL(SIGALRM, SigAlarm) == -1)
		nLogMsg(EL_FATAL, 0, "signal SIGALRM");
}

/*
**  ch_init() - Initialize connection with server and prepare for
**		regular synchronous processing.
*/
void
ch_init(DevMap_t *drvmap)
{
	char		*id = "ch_init";
	int		i;

	/*
	**	arrange to catch timeout alarms
	*/
	if (SIGNAL(SIGALRM, SigAlarm) == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "can't catch SIGALRM");

	/*
	**	open connection to changer
	*/
	if (rcp->rcd_number == 4)
		sprintf(changer_name, "/dev/ch0");
	else
		sprintf(changer_name, "/dev/ch1");

	nLogMsg(EL_STATUS1, id, "opening changer (%s)", changer_name);
	for (i=0; i<RETRIES; i++) {

		if ((changer_fd = open(changer_name, O_RDWR, 0600)) != -1)
			break;
		nLogMsg(EL_ERROR1|EL_SYS|EL_CONSOLE1|EL_CONSOLE0, id, 
				"open(%s)", changer_name);
		sleep(3);
	}

	if (i == RETRIES  ||  changer_fd == -1) {
		nLogMsg(EL_CONSOLE1|EL_CONSOLE0|EL_FATAL, id, 
				"can't init changer connection");
	}
	nLogMsg(EL_CONSOLE1|EL_STATUS1, id, "changer connection started");

	/*
	**	get config, inventory
	*/
	ch_config(drvmap);
}

/*
**  ch_terminate() - close conection to changer
*/
void
ch_terminate()
{
	if (changer_fd != -1) {
		close(changer_fd);
		changer_fd = -1;
	}
}

/*
**  done() - Exit routine - inform vold & die
*/
int
done()
{
	if (!shuttingdown) {
		shuttingdown = 1;
		RPCLGoodbye();
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"================= shutdown =================" );
		PrintQueues();
		ch_terminate();
		PrintStats();
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
					"================= shutdown complete");
	} else
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
					"------(recursive) shutdown complete");
	sleep(3);	/* allow vold to rcv goodbye */
	exit(VMError);
}
