
/*
**  "Changer" Repository Controller
**
**	ch_vm.c:  vold packet handling
*/

#include "chrc.h"
#include <setjmp.h>

extern int		errno;

extern int		immortal;
extern jmp_buf		jmpbuf_i;

extern DevEnt_t		*DriveArray;
extern RC_t		*rcp;                   /* points to config   */
						/* info for this RC   */
extern int		WaitingEjects;
extern QHead_t		RCQ;			/* mount q             */
extern QHead_t		EjectQ;			/* eject q             */

extern movnode_t	*FindWaitingMove();

extern int		CHmounted;
extern int		*DriveChannels, *ChannelUse;
extern DevMap_t		AvailDrives;

static Seq_t           lastseq = 0;  /* last seq # request from vold      */

static int		Mounts = 0;			/* reporting	     */

/************************************
	PACKET-HANDLING
*************************************/

static void
DismountErr(RCReq_t *reqp)
{
	if (reqp->rr_cmd == M_RCMOUNT)
		SendRecoverErr(reqp, "(unmount failed too)");
	else 
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, reqp->rr_cmd, 0);

	RPCLVaryOff(reqp->rr_drive, "unmount failed");
	DelReq(reqp);
}

static void
MMTimer(vmpHdr_t *hp)
{
	char		*id = "MMTimer";
	vmpTimer_t	*tp;
	time_t		diff;
	RCReq_t		*reqp, *reqp2;

	tp = MSGPTR(hp, vmpTimer_t);

	/*
	**  figure time between RP sending & receipt -
	**	long times could be due to communications
	**	or acsrc hung on read or ioctl
	*/
	diff = Now - tp->vt_time;
	if (diff > 10) {
		/*
		**  note the fact - may be useful for debugging
%%% for async
		nLogMsg(EL_STATUS2, 0, 
				"(msg time from vold = %d seconds)", diff);
		*/
	} 

	/*
	**	check q for retries
	*/
	for (reqp=QHEAD; reqp; reqp=reqp2) {
		reqp2 = QNEXT(reqp);
		if (reqp->rr_state == S_RETRY) {
nLogMsg(EL_STATUS1, id, "got retry");
			switch( reqp->rr_type) {
			    case T_UNMOUNT:
			    case T_UNMNTRECOV:
				if (ch_unmount(reqp))
					reqp->rr_retries++;
				if (reqp->rr_retries > 5)
					DismountErr(reqp);
				break;
			    default:
				nLogMsg(EL_FATAL, id, 
					"unexpected retry, type %d", 
					reqp->rr_state);
			}
		}
	}
}

/*
**  SendRecoverErr() -
**		An unmount was issued to recover a mount that had
**		label or device problems. The unmount has completed 
**		(or failed), so now send the original error that was 
**		saved at mount time,
**
*/
void
SendRecoverErr(RCReq_t *reqp, char *str)
{
	VMError = reqp->rr_vmerror;
	errno = reqp->rr_errno;

	/* vsn if any should be in reqp->rr_errstr */
	nLogMsg(EL_CODE|reqp->rr_errflags, reqp->rr_errid, "%s %s",
					reqp->rr_errstr, str);

	RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, reqp->rr_cmd, reqp->rr_erretry);
	vClearError();
	errno = 0;
}

/*
**	MountCheck() - things needed by mount & scratch cmds
*/
static int
MountPreCheck(char *id, vsn_t *vsnp, Seq_t seq, int cmd)
{
	RCReq_t		*reqp;

	/*
	**	got a mount request, simple check of configuration
	*/
	if (!rcp->rcd_nconfigdev) {
		/*
	 	**	this RC will never mount - no drives configured
	 	*/
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "no drives configured");
		RCErr2RP(vsnp, seq, cmd, 0);
		return(1);
	}

	/*
	**	only 1 mount per vsn
	**		%%% TODO - could use an index rather than
	**			seq search if lots of drives
	*/
	for (reqp=QHEAD; reqp; reqp=QNEXT(reqp)) {
		if (SAMEVSN(vsnp, &reqp->rr_vsn)) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
					"mount already requested");
			RCErr2RP(vsnp, seq, cmd, 0);
			return(1);
		}
	}
	return(0);
}

/*
**  FailMount() - Clean up and dismount a failed mount.
*/
static void
FailMount(RCReq_t *reqp)
{
	int	chan;

	reqp->rr_type = T_UNMNTRECOV;

nLogMsg(EL_STATUS1, "FailMount", "call ch_unmount.. 0x%x", reqp);
	if (ch_unmount(reqp)) {
		reqp->rr_state = S_RETRY;
		return;
	}
	/*
	**	update arrays & indexes
	*/
	mark_move(reqp->rr_chdrvp, reqp->rr_slotp);
	DM_SET(reqp->rr_drive, AvailDrives);

	chan = DriveChannels[reqp->rr_drive];
	if (ChannelUse[chan] > 0)
		ChannelUse[chan]--;

	SendRecoverErr(reqp, "(umounted)");
	DelReq(reqp);
nLogMsg(EL_STATUS1, "FailMount", "done..");
}


/*
**  MError() - Handle error from vold or volnd
*/
static void
MMError(vmpHdr_t *hp)
{
	char		*id = "MMError";
	vmpError_t      *vep;
	RCReq_t		*reqp;

	/*
	**	log msg & look up request
	*/
        vep = MSGPTR(hp, vmpError_t);

nLogMsg(EL_STATUS1, id, "entered..");

	if (!strncmp((char *)&vep->vn_vsn, vep->vn_msg, VSNLEN))
        	nLogMsg(EL_ERROR1, 0, vep->vn_msg);
	else
        	nLogMsg(EL_ERROR1, 0, "%.*s_%d: %s", 
					VSNLEN, (char *)&vep->vn_vsn, 
					hp->vh_seq, vep->vn_msg);
	reqp = FindReqRPSeq(hp->vh_seq);
nLogMsg(EL_STATUS1, id, "FindReqRPSeq done..");
	if (reqp == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "no req %.*s_%d", 
				VSNLEN, &vep->vn_vsn, hp->vh_seq);
		return;
	}
	if (reqp->rr_state == S_MOUNTED  ||  reqp->rr_state == S_UNMOUNTING) {
		nLogMsg(EL_STATUS2, id, "%.*s_%d: spurious error from ND",
				VSNLEN, &vep->vn_vsn, hp->vh_seq);
		return;
	}

	/*
	**	copy err info for later report to vold
	**		& start unmount
	*/
nLogMsg(EL_STATUS1, 0, "msg 0x%x  %*.*s %d", 
reqp,VSNLEN,VSNLEN, &reqp->rr_vsn,reqp->rr_drive);
	memcpy(reqp->rr_errstr, vep->vn_msg, sizeof(reqp->rr_errstr));
nLogMsg(EL_STATUS1, id, "memcpy done..  0x%x", reqp);
nLogMsg(EL_STATUS1, 0, "msg %*.*s %d", VSNLEN,VSNLEN, &reqp->rr_vsn,reqp->rr_drive);

	reqp->rr_vmerror = vep->vn_error;
	reqp->rr_erretry = vep->vn_retry;
	reqp->rr_errflags = EL_ERROR1;
	reqp->rr_errid = "volnd err";
	RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
	FailMount(reqp);
}

/*
**	MountReq() - finish mounted
*/
static int
MountReq(RCReq_t *reqp, char *id)
{
nLogMsg(EL_STATUS1, 0, "entered 0x%x  %*.*s %d", 
reqp,VSNLEN,VSNLEN, &reqp->rr_vsn,reqp->rr_drive);
	/*
	**	check state
	*/
	if (reqp->rr_volstate != 0) {
		VMError = E_UNAVAIL;
		nLogMsg(EL_ERROR1|EL_CODE|EL_CONSOLE1, id, 
				"%.*s not idle; volume %s",
				VSNLEN, &reqp->rr_vsn, ReqState(reqp));
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(1);
	}

	/*
	**	look up location
	*/
	if (ch_locate(reqp)) {
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(1);
	}
nLogMsg(EL_STATUS1, 0, "located  0x%x  %*.*s %d", 
reqp,VSNLEN,VSNLEN, &reqp->rr_vsn,reqp->rr_drive);

	/*
	**	choose drive
	*/
	if (AllocateDrive(reqp)) {
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(1);
	}
nLogMsg(EL_STATUS1, id, "got drive bitdrv %d", reqp->rr_drive);
nLogMsg(EL_STATUS1, 0, "msg 0x%x  %*.*s %d", 
reqp,VSNLEN,VSNLEN, &reqp->rr_vsn,reqp->rr_drive);
	/*
	**	request mount (synchronous)
	*/
	if (ch_mount(reqp)) {
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(1);
	}

	/*
	**	update arrays & indexes
	*/
	mark_move(reqp->rr_slotp, reqp->rr_chdrvp);

	return(0);
}

/*
**  MMMount() - Set up mount request, get list of drives, mount, start node
*/
static void
MMMount(vmpHdr_t *hp)
{
	char			*id = "MMMount";
	vmpRCMount_t		*mp;
	RCReq_t			*reqp;

	Mounts++;
	mp = MSGPTR(hp, vmpRCMount_t);

	if (MountPreCheck(id, &mp->vm_vsn, hp->vh_seq, M_RCMOUNT))
		return;

	/*
	**	make a request 
	*/
	reqp = NewReq(&mp->vm_vsn, hp->vh_seq, M_RCMOUNT, T_MOUNT);
	reqp->rr_label = mp->vm_label;
	reqp->rr_devmap = mp->vm_devmap;
	reqp->rr_flag = mp->vm_flag;
	reqp->rr_flag &= ~VO_WAIT;
	reqp->rr_uid = mp->vm_uid;
	reqp->rr_host = mp->vm_host;

nLogMsg(EL_STATUS1, 0, "newreqd   0x%x  %*.*s %d", 
reqp,VSNLEN,VSNLEN, &reqp->rr_vsn,reqp->rr_drive);

	/*
	**	pick drive and do the mount
	*/
	if (MountReq(reqp, id))
		return;

	/*
	**	send appropriate cmd to volnd
	*/
	if (reqp->rr_flag & VO_SCRATCH) {
		reqp->rr_state = S_NODESCRATCH;
		RCSendVolNode(M_SCRATCH, reqp->rr_seq, 
					&reqp->rr_vsn, &reqp->rr_label, 
					reqp->rr_flag, reqp->rr_drive, 
					reqp->rr_host, reqp->rr_uid);
	} else {
		/*
		**	check label / create node
		*/
		reqp->rr_state = S_NODECREATE;
		RCSendVolNode(M_CREATENODE, reqp->rr_seq,
					&reqp->rr_vsn, &reqp->rr_label,
					reqp->rr_flag, reqp->rr_drive,
					reqp->rr_host, reqp->rr_uid);
	}
}



/*
**	MMNodecreated() - Node created, mount finished.
*/
static void
MMNodecreated(vmpHdr_t *hp)
{
	static char	*id = "MMNode";
	vmpNode_t	*np;
	RCReq_t		*reqp;

	np = MSGPTR(hp, vmpNode_t);
	reqp = FindReqRPSeq(hp->vh_seq);
	if (reqp == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "no req %.*s_%d", 
				VSNLEN, &np->vs_vsn, hp->vh_seq);
		return;
	}
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"MOUNTED/lbl ok %.*s_%d  drive %d (%s)",
			VSNLEN, &reqp->rr_vsn, reqp->rr_seq,
			reqp->rr_chdrvp->index,
			DriveArray[reqp->rr_drive].de_name);
	RCSendMountDone(reqp->rr_seq, &reqp->rr_vsn, reqp->rr_drive, 
						np->vs_flag);
	reqp->rr_state = S_MOUNTED;
}


static int
UnmountReq(RCReq_t *reqp)
{
	int           	chan;

	/*
	**	get empty slot
	*/
	reqp->rr_slotp = ch_getemptyslot();
	if (reqp->rr_slotp == NULL) {
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, reqp->rr_cmd, 0);
		DelReq(reqp);
		return(1);
	}

	/*
	**	try unmount 
	*/
	if (ch_unmount(reqp)) {
		reqp->rr_state = S_RETRY;
		return(1);
	}
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
				"UNMOUNTED %*.*s  drive %d  (%s)",
				VSNLEN, VSNLEN, &reqp->rr_vsn,
				reqp->rr_chdrvp->index,
				DriveArray[reqp->rr_drive].de_name);

	/*
	**	update arrays & indexes
	*/
	mark_move(reqp->rr_chdrvp, reqp->rr_slotp);
	DM_SET(reqp->rr_drive, AvailDrives);

	chan = DriveChannels[reqp->rr_drive];
	if (ChannelUse[chan] > 0)
		ChannelUse[chan]--;

	return(0);
}

static void
FinishUnmount(RCReq_t *reqp)
{
	char	*id = "FinishUnmount";

	if (reqp->rr_errstr[0]) {
		/*
		**	dismount was a recovery from mount problem
		*/
		SendRecoverErr(reqp, "(unmounted)");
		DelReq(reqp);
		return;
	}
	if (reqp->rr_type == T_UNMOUNTFIRST) {
		/*
		**  Volume req'd for mount was stuck in drive
		**  and halleluja, now it is free. Hopefully
		**  someone will vary the drive back on if
		**  it was varied off, but we can't since
		**  it might have been varied off for other
		**  reasons. Resume the mount as if it were new.
%% async sort of thing, not tested for sync server
		*/
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
				"DRIVE %s DISMOUNTED, MAY WANT TO VARY ON",
				DriveArray[reqp->rr_drive].de_name);
		nLogMsg(EL_STATUS1, id, "%*.*s resuming mount",
						VSNLEN, VSNLEN, &reqp->rr_vsn);
		reqp->rr_type = T_MOUNT;
		reqp->rr_check = 0;
		/*
		**	start mount
		*/
		reqp->rr_time = Now;
		MountReq(reqp, id);
		return;
	}

	if (reqp->rr_flag2 & VF_NORESP)
		nLogMsg(EL_STATUS2, id, "(no response requested)");
	else
		RCSendUnmountDone(reqp->rr_seq, &reqp->rr_vsn);

	DelReq(reqp);
}

/*
**  MMUnmount() - Tell changer to unload vsn and put it away
*/
static void
MMUnmount(vmpHdr_t *hp)
{
	char		*id = "MMUnmount";
	vmpVsn_t      	*vp;
	RCReq_t		*reqp;

	vp = MSGPTR(hp, vmpVsn_t);

	/*
        **	should be dismount of completed mount on proper drive
        */
	reqp = FindReqVsn(&vp->v_vsn);
	if (reqp == NULL) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%.*s: no prev request", 
							VSNLEN, &vp->v_vsn);
		RCErr2RP(&vp->v_vsn, hp->vh_seq, M_UNMOUNT, 0);
		return;
	}
	if (reqp->rr_state != S_MOUNTED) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%.*s: not mounted, status %s", 
					VSNLEN, &vp->v_vsn, ReqState(reqp));
		RCErr2RP(&vp->v_vsn, hp->vh_seq, M_UNMOUNT, 0);
		return;
	}
	
        if (vp->v_flag != reqp->rr_drive) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"%.*s: mounted on other drive: %s", 
		    		VSNLEN, VSNLEN, &vp->v_vsn, 
				DriveArray[reqp->rr_drive].de_name);
		RCErr2RP(&vp->v_vsn, hp->vh_seq, M_UNMOUNT, 0);
		return;
	}

	/*
	**	request is reasonable: install new seq and execute it
	*/
	reqp->rr_seq = hp->vh_seq;
	reqp->rr_type = T_UNMOUNT;
	reqp->rr_flag2 = vp->v_flag2;

	if (UnmountReq(reqp))
		return;

	FinishUnmount(reqp);
}


/*
**	MMScratched() - volnd has scratched & unloaded
*/
static void
MMScratched(vmpHdr_t *hp)
{
	char		*id = "MMScratched";
	RCReq_t		*reqp;
	vmpNode_t	*np;

	np = MSGPTR(hp, vmpNode_t);

	reqp = FindReqRPSeq(hp->vh_seq);
nLogMsg(EL_STATUS1, id, "FindReqRPSeq done..");
	if (reqp == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "no req %.*s_%d", 
				VSNLEN, &np->vs_vsn, hp->vh_seq);
		return;
	}
	if (reqp->rr_state != S_NODESCRATCH) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%.*s: not scratch, status %s", 
					VSNLEN, &reqp->rr_vsn, ReqState(reqp));
		RCErr2RP(&reqp->rr_vsn, hp->vh_seq, M_UNMOUNT, 0);
		DelReq(reqp);
		return;
	}

	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"SCRATCHED %s_%d  drive %d (%s)",
			vsnstr((char *)&reqp->rr_vsn), reqp->rr_seq,
			reqp->rr_chdrvp->index,
			DriveArray[reqp->rr_drive].de_name);

	/*
	**	unmount 
	*/
	if (UnmountReq(reqp))
		return;


	/*
	**	tell vold
	*/
	RCSendUnmountDone(reqp->rr_seq, &reqp->rr_vsn);
	DelReq(reqp);
}


/*
**  ErrMoves() - Fail each move in pkt - e.g. 
**		internal problems w/ pkt header.
*/
static void
ErrMoves(int nvsns, vsnseq_t *vsnseqp)
{
	int		i, saverr = VMError;

	for (i=0; i<nvsns; i++, vsnseqp++) {
		VMError = saverr;
		RCErr2RP(&vsnseqp->r_vsn, vsnseqp->r_seq, M_MOVE, 0);
	}
}



/*
**  MMMove() - Set up move request ...
**		In practice, Eject is the only type of move handled.
*/
static void
MMMove(vmpHdr_t *hp)
{
	char		*id = "MMMove";
	vmpMove_t       *mvp;
	vsnseq_t	*vsnseqp;
	
	mvp = MSGPTR(hp, vmpMove_t);
	nLogMsg(EL_DEBUG2, 0, "Move from %d to %d",
		mvp->vmv_srcrc, mvp->vmv_dstrc);

	vsnseqp = (vsnseq_t *) ((char *)mvp + sizeof(vmpMove_t));

	/*
	**	is this a bad move, eject or accept for this RC?
	*/
	if (mvp->vmv_srcrc == mvp->vmv_dstrc) { 
		/* bad move */
		VMError = E_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
					"%s: move to same location (%d)",
					rcp->rcd_id, mvp->vmv_srcrc);
		ErrMoves(hp->vh_count, vsnseqp);
		return;
	}
	if (rcp->rcd_number != mvp->vmv_srcrc  
			&&  rcp->rcd_number != mvp->vmv_dstrc) {
		VMError = E_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
				"move doesn't involve %s: %d->%d",
				rcp->rcd_id, mvp->vmv_srcrc, mvp->vmv_dstrc);
		ErrMoves(hp->vh_count, vsnseqp);
		return;
	}
	if (mvp->vmv_dstrc == rcp->rcd_number) {
		/* 
		**  ACS doesn't need to do anything - operator will put CAP
		**	in Enter mode and put in vsns.
		*/
		return;
	}

	/*
	**  Looks good - try to do something with it
	**	Vsns are queued, and when an eject is sent,
	**	they are deq'd and put in the caparray (in BatchEjects()).
	*/

	nLogMsg(EL_FATAL, id, "EJECT: not impl");
}


void
PrintQueues()
{
	RCReq_t		*reqp;
	movnode_t	*ejp;

	if (QHEAD || EHEAD)
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"VSN     REQUEST   STATUS     SEQ              TIME");
	else {
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, "No volumes queued");
		return;
	}
	sleep(1);
	for (reqp=QHEAD; reqp; reqp=QNEXT(reqp))
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"%.*s   %s %s    %d/%d    %d",
			VSNLEN, &reqp->rr_vsn, 
			ReqType(reqp), ReqState(reqp), 
			reqp->rr_seq, reqp->rr_req_id,
			Now - reqp->rr_time);
	sleep(1);
	for (ejp=EHEAD; ejp; ejp=ENEXT(ejp))
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"%.*s: %s %s %d/%d    %d",
			VSNLEN, &ejp->vsnseq.r_vsn,
			"EJECT    ", "wait", 
			ejp->vsnseq.r_seq, 0,
			Now - ejp->time);
}

extern time_t		StartTime;	/* for reporting		     */
extern int		CHmounted;	/* vsns mounted			     */
extern int		Ejected;	/* vsns ejected from cap	     */
extern int		Entered;	/* vsns entered in cap		     */

void
PrintStats()
{
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
		"==================== STATISTICS =====================");
	sleep(1);
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
		"Mounts requested: %d  mounted: %d", Mounts, CHmounted);
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
		"Entered: %d  Ejected: %d", Entered, Ejected);
	/* 
	**  Note that since nas_time() sets a single char array, it
	**	can't be called more than once in a nLogMsg()
	*/
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
				"Time up: %s", nas_time(Now - StartTime));
	sleep(1);
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
		"=====================================================");

}

/*
**  MMRCMsg() - handle msg from operator
*/
static void
MMRCMsg(vmpHdr_t *hp)
{
	char		*id = "MMRCMsg";
	vmpRcMsg_t      *rm;
	vmpHdr_t        *hp2;
	
	rm = MSGPTR(hp, vmpRcMsg_t);
	hp2 = (vmpHdr_t *) rm->rm_msg;

	/*
	**  while the rcmsg type includes space for a regular packet 
	**	inside it, in this rc only the rm_cmd and rm_arg fields 
	**	of the primary level packet are used.
	*/

	switch (rm->rm_cmd) {
	    case RMC_ENTER:
		nLogMsg(EL_FATAL, id, "ENTER: not impl");
		break;
	    case RMC_CANCEL:
		nLogMsg(EL_FATAL, id, "CANCEL ENTER: not impl");
		break;
	    case RMC_CLEAN:
		nLogMsg(EL_FATAL, id, "CLEAN: not impl");
		break;
	    case RMC_REPORT:
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"================ report ===============");
		sleep(1);
		PrintQueues();
		PrintStats();
		break;
	    case RMC_SHUTDOWN:
		nLogMsg(EL_CONSOLE1|EL_CONSOLE0|EL_FATAL, id,
			"shutdown from chrc console");
		break;
	    default:
		nLogMsg(EL_CONSOLE0|EL_CONSOLE1|EL_ERROR1, id,
				"bad command type %d", rm->rm_cmd);
		break;
	}
}



/*
**  vold_pkt() - Get & handle a packet from the RP
*/
void
vold_pkt()
{
        char    	*id = "vold_pkt";
	vmpHdr_t        *hp;

	/*
	**	select() has indicated there are packet(s) to read from
	**		vold: read & keep reading while xdr pkts
	**		are in buffer
	*/
	while (1) {
		nLogMsg(EL_DEBUG4, id, "reading");
		hp = RPCLRecv(id);
        	if (hp == NULL)
                	nLogMsg(EL_FATAL|EL_CONSOLE0, 0, "vold died");

		lastseq = hp->vh_seq;

		/*
		**	process the pkt according to type
		*/
        	switch (hp->vh_type) {

		    /* (ordered by approximate frequency of reception) */

        	    case M_RCMOUNT: 
			MMMount(hp); break;
		    case M_CREATENODE:
			MMNodecreated(hp); break;
        	    case M_UNMOUNT: 
			MMUnmount(hp); break;
		    case M_MOVE: 
			MMMove(hp); break;
        	    case M_TIMER:
			MMTimer(hp); break;
        	    case M_NOP:
                	break;
		    case M_SCRATCH:
			MMScratched(hp); break;
		    case M_VARY:
		    case M_VOLSTAT:
	  		/* msgs that are just reflected by vold */
                	break;
        	    case M_RCMSG: 
			MMRCMsg(hp); break;
        	    case M_VERROR: 
                	MMError(hp); break;
        	    case M_SHUTDOWN:
			if (immortal) {
				nLogMsg(EL_STATUS1, 0, "shutdown - longjmp");
				longjmp(jmpbuf_i, 1);
				nLogMsg(EL_ERROR1, 0, "longjmp");
			}
                	done();
		    case M_NEWLOG:
			/* assumes sharing vold log, so doesn't rename */
			nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
			break;
		    case M_DEBUG:
			MMDebug(hp); break;
        	    default:
                	nLogMsg(EL_ERROR1, id, 
					"%s: unexpected packet", vHdrStr(hp));
        	}
		nFree(hp);
		if (!RPCLNextRecord())
			break;
	}
}
