/*
** $Header:
**
**	Volume Manager Repository Controller for generic 'media changer'
**	robots.
**
**	chrc.c
**
**	This process receives mount and move(eject) requests from 
**	the vold, translates them into request structures in 
**	memory, and transmits them over a wire to the media changer.
**	It blocks on a select() on vold ipc and, "for now," executes
**	requests synchronously.
**
**	Bill Ross, NASA Ames, 1999.
*/

#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>

#ifdef EXTERN
# undef EXTERN
#endif
#define EXTERN

#include "srv_space.h"
#include "chrc.h"

#include <setjmp.h>

extern int	errno;

/*  from rclib.c  */
extern int	*DriveChannels, *ChannelUse;

DevEnt_t	*DriveArray;
int		DriveTotal;
int		tot_drives = 0;
DevMap_t	AvailDrives;
int		ch_drives[CH_MAX_DRIVES];	/* for mapping */
int		drive_uses[CH_MAX_DRIVES];	/* for allocation */

QHead_t		RCQ;			/* mount q			     */
QHead_t		EjectQ;			/* eject q			     */
RC_t		*rcp = NULL;		/* points to config info for this RC */
time_t		Now;			/* time last pkt rcvd		     */
time_t		StartTime;		/* for reporting		     */
int		Mounted = 0;		/* vsns mounted			     */
int		Ejected = 0;		/* vsns ejected 		     */
int		Entered = 0;		/* vsns entered in cap		     */
int		Pid;

extern int	immortal;
extern jmp_buf	jmpbuf_i;
extern int	changer_fd;

char    	*DriveErrors[MAXDEVS];	/* TODO maintain beyond startup */

/*
**  (relatively) local globals
*/
static int	vm_fd;			/* copy of vold ipc fd for select()  */
static fd_set	readfds;		/* struct for select()		     */

static char	spooldir[100];
static char	rcno[10], *arg0, **args;

static void	Init();
static void	InitDrives();
static void	VaryOffMap();

static void
usage()
{
	fprintf(stderr, "usage: %s rc_num [-i]\n", arg0);
	exit(1);
}




/**************
   MAIN
***************/
int
main(int argc, char *argv[])
{
	int	nfound;

	args = argv;
	arg0 = argv[0];

	/*
	**	check arg & save rcno
	*/
	if (argc < 2  ||  argc > 3)
		usage();
	if (argc == 2) {
		strcpy(rcno, argv[1]);
        } else {
		if (!strcmp(argv[1], "-i")) {
			immortal = 1;
			strcpy(rcno, argv[2]);
		} else if (!strcmp(argv[2], "-i")) {
			immortal = 1;
			strcpy(rcno, argv[1]);
		} else {
			usage();
		}
	}
	time(&StartTime);
	Now = StartTime;

	/*
	**	set up
	*/
	Init();

	/*
	**	if anything is buffered from vold setup, get it
	if (RPCLNextRecord())
		vold_pkt();
	*/

	/*
	**	loop reading packets
	*/
	while (1) {

		/*
		**	make sure communications are intact
		*/
		if (vm_fd < 0)
			nLogMsg(EL_CONSOLE0|EL_CONSOLE1|EL_FATAL, 0, 
				"bad vold socket (%d): shutting down", vm_fd);
		if (changer_fd < 0)
			nLogMsg(EL_CONSOLE0|EL_CONSOLE1|EL_FATAL, 0, 
			    "bad changer fd (%d): shutting down", changer_fd);

		/* 
		**	set bit(s) of file descriptor(s) for select()
		**		to wait on
		*/
		FD_ZERO(&readfds);
		FD_SET(vm_fd, &readfds);
/*
%%% robot stuff is synchronous for now
		FD_SET(changer_fd, &readfds);
*/

		/* 
		**	block on select for input 
nLogMsg(EL_STATUS1, 0, "WAIT on select/rcv");
nLogMsg(EL_STATUS1, 0, "GOT select/rcv");
		*/
		nLogMsg(EL_DEBUG4, 0, "WAIT on select/rcv");
		nfound = select(FD_SETSIZE, &readfds, (fd_set *)NULL,
				(fd_set *)NULL, (struct timeval *) NULL);

		if (nfound == -1) {
			if (errno == EINTR) {
				nLogMsg(EL_DEBUG2, 0, "EINTR on select");
				continue;
			}
			nLogMsg(EL_SYS|EL_ERROR1, 0, "select failed");
			return(-1);
		}
		if (nfound != 1)
			nLogMsg(EL_FATAL, 0, "select!!");
		time(&Now);

		/*
		**	Find out where data is from 
		**		and process request(s) from vold 
%%% and response(s) from robot /when made async
		if (FD_ISSET(changer_fd, &readfds))	
			%%%ch_pkt();
		*/
		if (FD_ISSET(vm_fd, &readfds))  /* volman (client) request */
			vold_pkt();
		else
			nLogMsg(EL_FATAL, 0, "select2!!");
	}
}


/*****************
  INITIALIZATION
******************/

static void
Init()
{
	char		*id = "Init";
	DevMap_t	VaryDrives;
	int		i;
	char		lockfile[256];
	char		logtag[10];

	/*
	**	go to RC spool dir in case core is dumped
	*/
	vInitDirNames();
	sprintf(spooldir, "%s/ch%s", vSpoolDir, rcno);
	nCreatePath(spooldir, 0755);
/*
	if (chdir(spooldir) == -1) {
		fprintf(stderr, "can't chdir to %s: %s\n", 
						spooldir, nSysErr(errno));
		exit(1);
	}
*/
	/*
	**	make sure process isn't already running
	*/
	sprintf(lockfile, "%s/lock", spooldir);
	nGetLock(lockfile, arg0);

	/*
	**	start logging
	*/
	sprintf(logtag, "CH%s", rcno);
        nLogInit(logtag, done);
        nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);

	/*
	**	here is where to come back to 
	**		if immortal & vold goes away
	*/
	if (setjmp(jmpbuf_i)) {
		nLogMsg(EL_STATUS1, 0, "setjmp'd");
		ch_terminate();
		CleanupQ();
		sleep(30);
	}

	/*
	**	establish vold connection, getting config from vold;
	**		note its ipc rcv file descriptor for select()
        */
	nLogMsg(EL_STATUS1, 0, "starting vold connection");
	RCConnectRP();
	vm_fd = RPCLfd();

        /*
        **	look up invoked RC & check that it's consistent
	**		(vold could have been restarted w/ new config)
        */
        if ((rcp = cGetRcp(rcno)) == NULL)
                nLogMsg(EL_FATAL|EL_TOSTDERR|EL_CONSOLE0|EL_CONSOLE1, id, 
					"%s: can't get RC %s info - quitting", 
					arg0, rcno);
	if (strcmp(rcp->rcd_type, RCTYPE))
                nLogMsg(EL_FATAL|EL_TOSTDERR|EL_CONSOLE0|EL_CONSOLE1, id, 
			"%s: RC %s is type %s - this prog type %s - quitting",
			arg0, rcp->rcd_id, rcp->rcd_type, RCTYPE);
	if ((rcp->rcd_flags & RCF_TIMER) == 0)
                nLogMsg(EL_FATAL|EL_TOSTDERR|EL_CONSOLE0|EL_CONSOLE1, id, 
		    "%s (%s) needs RCF_TIMER not set in rc.volman - quitting",
		    arg0, rcp->rcd_id);

        /*
        **	make sure we are ok to run as root
	*/
        CheckId(getuid(), rcp);

	/*
	**	start console logging & announce presence
	**		(presence != readiness btw)
	*/
	nLogInitConsole(rp_syslog, EL_CONSOLE0, EL_LEVEL);
	if (rcp->rcd_syslog)
		nLogInitConsole(rcp->rcd_syslog, EL_CONSOLE1, EL_LEVEL);

	Pid = getpid();
        nLogMsg(EL_STATUS1, 0, "==== %s here: rc=%s/%d pid=%d ====",
          				arg0, rcp->rcd_id, rcp->rcd_number,Pid);
	nLogMsg(EL_STATUS1, id, "--------------------------------------------");

	/*
	**	init drive bitmap and robot->devmap index
	*/
	DM_ZERO(AvailDrives);
	for (i=0; i<CH_MAX_DRIVES; i++) {
		ch_drives[i] = -1;
		drive_uses[i] = 0;
	}

	for (i=0; i<DriveTotal; i++) {
		int	ch_drive = DriveArray[i].de_acsdevloc.ad_drv;

nLogMsg(EL_STATUS1, id, "%s rc %d  loc %d", 
DriveArray[i].de_name, DriveArray[i].de_rc, ch_drive);
		if (DriveArray[i].de_rc != rcp->rcd_number)
			continue;
		DM_SET(i, AvailDrives);
		tot_drives++;

		/*
		**	check/add the index entry
		*/
		if (ch_drive >= CH_MAX_DRIVES  ||  ch_drive < 0)
			nLogMsg(EL_FATAL|EL_CONSOLE0|EL_CONSOLE1, id,
				"%s: volman config: must be in 0..%d %s",
				DriveArray[i].de_name, CH_MAX_DRIVES-1,
				"or increase CH_MAX_DRIVES in chrc.h");
		    
		if (ch_drives[ch_drive] != -1)
			nLogMsg(EL_FATAL|EL_CONSOLE0|EL_CONSOLE1, id,
				"Drive number in robot multiply defined: %d %s",
				ch_drive, DriveArray[i].de_name);
		ch_drives[ch_drive] = i;
nLogMsg(EL_STATUS1, id, "ch_drives[%d] = %d", ch_drive, i);
	}

	/*
	**	init rotating drive allocation array & err counts
	*/
	InitDrives();

	/*
	**	init channel tracking for drive alloc
	*/
	RCInitChan();

	/*
	**	init EjectQ
	*/
	EjectQ.q_head = NULL;
	EjectQ.q_tail = NULL;

	memset(DriveErrors, 0, sizeof(DriveErrors));
	DM_ZERO(VaryDrives);
	ch_init(&VaryDrives);

	/*
	**	vary off any drives that e.g. had dismount 
	**		problems in ch_init()
	**	doing this before announcing Ready so vold has correct drive 
	**	counts and won't send more mounts than available drives.
	*/
	VaryOffMap(&VaryDrives);

	/*
	**	announce to vold that we are up & running as an RC
	*/
        RCReadyRP(rcp);

	/*
	**  update vold on status of eject requests that may have
	**	completed   %%% TODO eject/enter stuff
	InitCheckEjects();
	*/

	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, "------Initialization complete");
}


/*
**  InitDrives() - Initialize the local drive map.
*/
static void
InitDrives()
{
	int	i;

	/*
	**	set all drivearray statuses = 0 for err counting
	*/
	for (i=0; i<DriveTotal; i++)
		DriveArray[i].de_status = 0;

}


/*
**  VaryOffMap() - Vary off drives w/ hardware or dismount problems.
**  		Must be called after RCConnectRP().
*/
static void
VaryOffMap(DevMap_t *drvmap)
{
	int	i;

	for (i=0; i<DriveTotal; i++) {
		if (DM_ISSET(i, *drvmap)) {
			RPCLVaryOff(i, DriveErrors[i]);
		}
	}
}



static CodeString_t EltTypeTab[] = {
	{	CHET_ST,	"slot"		},
	{	CHET_DT,	"drive"		},
	{	CHET_MT,	"picker"	},
	{	CHET_IE,	"portal"	},
	{	0,		""		},
};

char *
EltTypeStr(int type)
{
	return(nCodeToStr(type, EltTypeTab));
}
