/*
**  'Changer' Repository Controller
**
**  chrc.h  -  Main structures and definitions
**
*/

#include "srv_space.h"
#include "rclib.h"
#include <sys/ioctl.h> 
#include <sys/chio.h>


#define RETRY_LIMIT 	3
#define TIMEOUT		300

#define RCTYPE		"CH"

#define CH_MAX_DRIVES	10


typedef struct changer_params			ch_param_t;
typedef struct changer_element_status_request	ch_req_t;
typedef struct changer_element_status		ch_elt_t;
typedef struct changer_move_request		ch_mov_t;

typedef struct {
	int		type;
	int		index;
	ch_elt_t	stat;
} ch_stat_t;



extern char	changer_name[];
extern int	changer_fd;
extern time_t	Now;

extern int	ch_drives[];

/************************************************************************
**  Definitions for request queue functions.
*************************************************************************/

typedef struct RCReq {
	QLink_t		rr_q;		/* work q			*/
	Seq_t		rr_seq;		/* seq # from vold		*/
	uint		rr_req_id;	/* seq # from changer		*/
	time_t		rr_time;	/* time of arrival 		*/
	vsn_t		rr_vsn;		/* vsn requested		*/
	vsn_t		rr_label;	/* internal label if any	*/
	int		rr_cmd;		/* request from vold		*/
	int		rr_type;	/* type/phase of request	*/
	int		rr_state;	/* where we are w/ this request	*/
	int		rr_flag;	/* client bit flags		*/
	int		rr_flag2;	/* client bit flags		*/
	int		rr_uid;		/* uid for mount/createnode	*/
	int		rr_host;	/* host for mount/createnode	*/
	DevMap_t	rr_devmap;	/* client acceptable drives	*/
	int		rr_drive;	/* bitmap # of drive allocated	*/
	ch_stat_t	*rr_slotp;	/* slot of vsn			*/
	ch_stat_t	*rr_chdrvp;	/* drive from robot's view	*/
	int		rr_retries;	/* when slots not avail		*/
	int		rr_check;	/* flag for request check	*/
	int		rr_count;	/* # vsns ejecting in this req	*/
	char		*rr_ejp;	/* points to movenode chain     */
	/* the next group is filled in by qry_vol			*/
	int		rr_volstate;	/* obvious..			*/
	/* the next group is for tracking/reinstalling errors */
	int		rr_errno;	/* errno at time of initial prob*/
	int		rr_vmerror;	/* VMError			*/
	int		rr_errflags;	/* flags for vLogMsg()		*/
	char		*rr_errid;	/* id of routine w/ error	*/
	char		rr_errstr[LINELEN]; /* description of error	*/
	int		rr_erretry;	/* whether to set retry flag	*/
} RCReq_t;


/* queueing macros */

#define QHEAD			((RCReq_t *) RCQ.q_head)
#define QNEXT(reqp)		((RCReq_t *) ((reqp)->rr_q.q_next))

#define ENQ(reqp)		ENQUEUE((reqp), &RCQ, rr_q, RCReq_t)
#define DQ(reqp)		DEQUEUE((reqp), &RCQ, rr_q, RCReq_t)

/* rr_type */

#define T_MOUNT		1
#define T_UNMOUNTFIRST	2
#define T_UNMOUNT	3
#define T_UNMNTRECOV	4
#define	T_EJECT		5

/* rr_state */

#define S_MOUNTING	2
#define S_NODECREATE	3
#define S_NODESCRATCH	4
#define S_MOUNTED	5
#define S_UNMOUNTING	6
#define S_RETRY		7
#define S_NEW		99



/*
**  Doubly linked list of movnodes is kept for tracking move requests
*/
typedef struct movnode {
	QLink_t		q;
	char		prio;
	vsnseq_t	vsnseq;
	time_t		time;
} movnode_t;

#define EHEAD			((movnode_t *) EjectQ.q_head)
#define ENEXT(movp)		((movnode_t *) ((movp)->q.q_next))

#define EDQ(movp)		DEQUEUE((movp), &EjectQ, q, movnode_t)

/*
**  routines
*/
/*  chrc.c	 */
extern char		*EltTypeStr(int type);

/*  ch_init.c    */
extern void		ch_init(DevMap_t *drvmap);
extern void		ch_terminate();
extern int		done();

/*  ch_config.c  */
extern int		get_elt_status(ch_stat_t *ep, char *id, char *id2);
extern void		ch_config(DevMap_t *drvmap);
extern ch_stat_t	*ch_getvsn(char *vsnp);
extern ch_stat_t	*ch_getemptyslot();
extern void		mark_move(ch_stat_t *from, ch_stat_t *to);
extern int		AllocateDrive(RCReq_t *reqp);

/*  ch_locate.c  */
extern int		ch_locate(RCReq_t *reqp);

/*  ch_mount.c  */
extern int		ch_mount(RCReq_t *reqp);

/*  ch_unmount.c */
extern int		ch_rawunmount(ch_stat_t *drivep, ch_stat_t *slotp);
extern int		ch_unmount(RCReq_t *reqp);

/*  ch_vm.c     */
extern void		vold_pkt();
extern void		SendRecoverErr(RCReq_t *reqp, char *str);
extern void		PrintStats();
extern void		PrintQueues();

/*  ch_q.c      */
extern void		CheckQ();
extern RCReq_t		*NewReq(vsn_t *vsn, Seq_t vseq, int cmd, int type);
extern char		*ReqType(RCReq_t *reqp);
extern char		*ReqState(RCReq_t *reqp);
extern RCReq_t		*FindReqRPSeq(Seq_t seq);
extern RCReq_t		*FindReqVsn(vsn_t *vsn);
extern char		*ReqType(RCReq_t *reqp);
extern void		DelReq(RCReq_t *reqp);
extern void		CleanupQ();
