/*
**  IBM 3494 Repository Controller
**
**	ibm_init.c:  initialize connection with controller daemon
*/
#include "vm_space.h"		/* volman */
#include "ibmrc.h"	
#include <signal.h>

#define DMOUNTTIMEOUT	200
#define RETRIES		3

extern RC_t	*rcp;
extern DevEnt_t	*DriveArray;	
extern int	DriveTotal;
extern int	ibm_drives[];
extern char	*DriveErrors[];

Seq_t		initseq = 222;		/* seq # of initial msg */

int		drivestatus = 0;/* whether drive status known yet on startup */

DevMap_t	HungDrives;

/*
**  FUNCTION:	SigAlarm()
**		Handle alarm signal.
*/
void
SigAlarm(sig)
{
	nLogMsg(EL_STATUS2, 0, "alarm");
	if (signal(SIGALRM, SigAlarm) == SIG_ERR)
		nLogMsg(EL_FATAL, 0, "signal SIGALRM");
}

/*
**  FUNCTION:	ibm_init()
**		Initialize connection with server and prepare for
**		regular synchronous processing.
*/
ibm_init(drvmap)
DevMap_t	*drvmap;
{
	char		*id = "ibm_init";
	char		*chkmsg = 
			"\n\t\t\t\tCHECK IBM LOG AND CONFIG FILE";
	int		i;

	/*
	**  arrange to catch timeout alarms
	*/
	if (signal(SIGALRM, SigAlarm) == SIG_ERR)
		nLogMsg(EL_FATAL|EL_SYS, id, "can't catch SIGALRM");

	nLogMsg(EL_STATUS1, id, "opening ibm lib daemon connection");

	/*
	**  open connection to IBM3494
	*/
	for (i=0; i<RETRIES; i++) {
		char	buf1[20], buf2[20];

		if ((ibm_fd = open_ibmatl(IBM_NAME)) != -1)
			break;
		nLogMsg(EL_ERROR1|EL_SYS|EL_CONSOLE1|EL_CONSOLE0, id, 
				"can't init ibm connection");
		sleep(3);
	}

	if (i == RETRIES  ||  ibm_fd == -1) {
		nLogMsg(EL_CONSOLE1|EL_CONSOLE0|EL_FATAL, id, 
				"can't init ibm connection");
	}
	nLogMsg(EL_CONSOLE1|EL_STATUS1, id, "ibm connection started");

	/*
	**   Check Server state
	*/
	if (ibm_srvstate(1)) {
		nLogMsg(EL_FATAL|EL_CONSOLE0|EL_CONSOLE1, id,
			"CAN'T GET SERVER STATE - PLEASE %s %s", chkmsg,
			"\n\t\t\t\t- SHUTTING DOWN");
	}

	/*
	**  Get status of drives
	*/
	if (ibm_drvinit(drvmap))
		 nLogMsg(EL_FATAL, id, "can't init drives, %s", chkmsg);

}


/*
**  FUNCTION:	ibm_srvstate()
**		Get state of lib & components
*/
ibm_srvstate(try)
int try;
{
	char		*id = "ibm_srvstate";
	struct mtlqarg	arg;
	int		state, in, out, seqno, num_cells, avail_cells;

	nLogMsg(EL_DEBUG4, id, "try %d", try);

	arg.versn = 0;
	arg.sub_cmd = MT_QLD;	/* Query Library Data */
	arg.hostid[0] = '\0';

	if (ioctl_ibmatl(ibm_fd, MTIOCLQ, &arg) == -1) {
		nLogMsg(EL_ERROR1|EL_SYS|EL_CONSOLE0|EL_CONSOLE1, id,
			"ioctl_ibmatl error (%s)", IBM_CC(arg.mtlqret.cc));
		return(-1);
	}
	if (arg.mtlqret.cc) {
		nLogMsg(EL_ERROR1, id, "return status err %s",
						IBM_CC(arg.mtlqret.cc));
		return(-1);
	}
	nLogMsg(EL_STATUS1, id, "rcvd %s", 
				IBM_InfoType(arg.mtlqret.info.info_type));

	state = arg.mtlqret.info.data.library_data.op_state[0];
	state |= arg.mtlqret.info.data.library_data.op_state[1]<<sizeof(char);
	nLogMsg(EL_STATUS1, id, "op_state %d == %s", 
					state, IBM_LibState(state));

	if (state  &  MT_LIB_POS)
		nLogMsg(EL_CONSOLE0|EL_CONSOLE1, id, 
				"Warning: operation in 'pause' mode");
	if (state  &  MT_LIB_IR)
		nLogMsg(EL_CONSOLE0|EL_CONSOLE1, id, 
				"Warning: operator intervention required");

	in = arg.mtlqret.info.data.library_data.inout_no & 0x0f;
	out = (arg.mtlqret.info.data.library_data.inout_no & 0xf0)>>4;
	nLogMsg(EL_STATUS1, id, "in %d out %d", in, out);

	if (arg.mtlqret.info.data.library_data.in_out_status & 0x01)
		nLogMsg(EL_STATUS1, id, "  all conventional input empty");
	if (arg.mtlqret.info.data.library_data.in_out_status & 0x04)
		nLogMsg(EL_STATUS1, id, "  all conventional output empty");
	if (arg.mtlqret.info.data.library_data.in_out_status & 0x08)
		nLogMsg(EL_STATUS1, id, "  all conventional output full");
	if (arg.mtlqret.info.data.library_data.in_out_status & 0x20)
		nLogMsg(EL_STATUS1, id, "  bulk input/output allowed");
	if (arg.mtlqret.info.data.library_data.in_out_status & 0x40)
		nLogMsg(EL_STATUS1, id, "  bulk output station full");

	seqno = arg.mtlqret.info.data.library_data.atl_seqno[0];
	seqno |=arg.mtlqret.info.data.library_data.atl_seqno[1]<<sizeof(char);
	seqno |=arg.mtlqret.info.data.library_data.atl_seqno[2]<<2*sizeof(char);
	nLogMsg(EL_STATUS1, id, "seqno %d", seqno);

	num_cells = arg.mtlqret.info.data.library_data.num_cells[0];
	num_cells |= arg.mtlqret.info.data.library_data.num_cells[1]
							<<sizeof(char);
	num_cells |= arg.mtlqret.info.data.library_data.num_cells[2]
							<<2*sizeof(char);
	num_cells |= arg.mtlqret.info.data.library_data.num_cells[3]
							<<3*sizeof(char);

	avail_cells = arg.mtlqret.info.data.library_data.avail_cells[0];
	avail_cells |= arg.mtlqret.info.data.library_data.avail_cells[1]
							<<sizeof(char);
	avail_cells |= arg.mtlqret.info.data.library_data.avail_cells[2]
							<<2*sizeof(char);
	avail_cells |= arg.mtlqret.info.data.library_data.avail_cells[3]
							<<3*sizeof(char);

	nLogMsg(EL_STATUS1, id, "num cells: %d  avail: %d", 
							num_cells, avail_cells);
	nLogMsg(EL_STATUS1, id, "num subsys [drives?] %d",
			arg.mtlqret.info.data.library_data.num_subsys);
	nLogMsg(EL_STATUS1, id, "num cart slots in each station %d",
			arg.mtlqret.info.data.library_data.conven_cap);

	rcp->rcd_freeslots = avail_cells;

	if (state != MT_LIB_AOS) {
		nLogMsg(EL_CONSOLE0|EL_CONSOLE1|EL_ERROR1,id,
				"bad state - add error handling?");
	}
	return(0);
}




/*
**  FUNCTION:	ibm_drvinit()
**		Query drive status and return bitmap.
*/
ibm_drvinit(drvmap)
DevMap_t	*drvmap;
{
	char		*id = "ibm_drvinit";
	struct mtlqarg	arg;
	int		bitdrv, i, ndmounts = 0, config, err;
	char		*cp;

	/*
	**  o  Check each drive's state and set appropriate bits
	**     in drive bitmap
	**  o  Also check and force dismounts on drives that
	**     have carts in them
	*/
	nLogMsg(EL_DEBUG3, id, "Getting status of drives");

	/*
	**  Set up pkt header
	*/
	arg.versn = 0;
	arg.sub_cmd = MT_QDD;	/* Query Drive Data */
	arg.hostid[0] = '\0';

	for (i=0; i<IBM_MAXDRIVES; i++) {
		if (ibm_drives[i] == -1)
			continue;
		config++;
		bitdrv = ibm_drives[i];

		/*
		**  spec drive & get info
		*/
		arg.device = i;
		if (ioctl_ibmatl(ibm_fd, MTIOCLQ, &arg) == -1) {
			nLogMsg(EL_ERROR1|EL_SYS|EL_CONSOLE0|EL_CONSOLE1, id,
				"ioctl_ibmatl error (%s)", 
				IBM_CC(arg.mtlqret.cc));
			DM_SET(bitdrv, *drvmap);
			err++;
			continue;
		}
		if (arg.mtlqret.cc) {
			nLogMsg(EL_ERROR1, id, "return status err %s",
						IBM_CC(arg.mtlqret.cc));
			DM_SET(bitdrv, *drvmap);
			err++;
			continue;
		}
		/*
		**  TODO - handle status; if mounted, dismount
		*/
	}

	if (err == config) {
		nLogMsg(EL_ERROR1|EL_CONSOLE0|EL_CONSOLE1, id,
				"status problems w/ all drives");
		return(-1);
	}
	return(0);
}


