/*
**  IBM 3494 Repository Controller
**
**	ibm_mount.c:  handle mounting of volumes on drives
*/
#include "vm_space.h"		/* volman */
#include "ibmconfig.h"
#include "ibmrc.h"

extern RC_t	*rcp;
extern DevEnt_t	*DriveArray;	
extern int	IBMmounted;
extern DevMap_t	AvailDrives;

/*
**  FUNCTION:	ibm_mount()
**		Construct & send a mount request (synchronous).
*/
int
ibm_mount(reqp)
RCReq_t	*reqp;
{
	static char		*id = "ibm_mount";
	struct mtlmarg		mtreq;

	mtreq.versn = 0;
	mtreq.device = reqp->rr_ibmdrv;
	mtreq.wait_flg = 1;	/* wait since in devel we've only got 1 hand */
	mtreq.target_cat = 0;
	mtreq.source_cat = 0;
	memset(mtreq.volser, 0, sizeof(mtreq.volser));
	memcpy(mtreq.volser, &reqp->rr_vsn, VSNLEN);

	nLogMsg(EL_STATUS2, 0, "MOUNT %.*s_%d drive %d (%s)", 
				VSNLEN, &reqp->rr_vsn, reqp->rr_seq,
				reqp->rr_ibmdrv, 
				DriveArray[reqp->rr_drive].de_name);
	if (ioctl_ibmatl(ibm_fd, MTIOCLM, &mtreq) < 0) {
		VMError = E_DOWN;
		nLogMsg(EL_ERROR1|EL_CODE|EL_SYS|EL_CONSOLE0|EL_CONSOLE1, id,
			"ioctl_ibmatl error (%s)", IBM_CC(mtreq.mtlmret.cc));
		return(-1);
	}
	if (mtreq.mtlmret.cc != MTCC_COMPLETE) {
		VMError = E_UNAVAIL;
		nLogMsg(EL_ERROR1|EL_CODE|EL_CONSOLE0|EL_CONSOLE1, id,
			"mount problem: %s", IBM_CC(mtreq.mtlmret.cc));
		return(-1);
	}
	reqp->rr_req_id = mtreq.mtlmret.req_id;
	return(0);
}


#ifdef WOW
/*
**  FUNCTION:	ibm_mount_rsp()
**		Handle response to mount request
**		--vestigial ACS code for adapting if/when
**		  this RC is made async
*/
ibm_mount_rsp(buf)
char	*buf;
{
	char			*id = "ibm_mount_rsp";
	RCReq_t			*reqp;
	int			seq, err = 0, retry = 0;
	char			*vsnp;

	reqp = FindReqSeq(seq);
	if (reqp == NULL) {

		/*
		**  presumably timed out earlier
		*/
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
			"request seq %d not found in queue", seq);

		return(-1);
	}

	/*
	**  Match response to request
	vsnp = 
	drive = 
	*/

	nLogMsg(EL_DEBUG3, id, "vsn (%s) seq %d", vsnp, seq);

	if (reqp->rr_state != S_MOUNTING) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"%.*s: bad state %d for request %d/%d",
				VSNLEN, vsnp, reqp->rr_state, reqp->rr_seq,seq);
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(-1);
	}

	/*
	**  (If problem with the drive, need to vary it off 1st.)
see acs_mount.c for example cases
	switch () {
	  case STATUS_SUCCESS:
		nLogMsg(EL_STATUS2, 0, 
			"MOUNTED %s_%d/%d on %s %s",
				vsnp, reqp->rr_seq, seq, 
				DriveArray[reqp->rr_drive].de_name, lsm,
				(reqp->rr_flag & VO_BYPASS ?
					"(no lblchk)" : "(check lbl)"));
		IBMmounted++;
		reqp->rr_state = S_NODECREATE;
		RCSendVolNode(M_CREATENODE, reqp->rr_seq, 
			&reqp->rr_vsn, &reqp->rr_label, 
			reqp->rr_flag, reqp->rr_drive, 
			reqp->rr_host, reqp->rr_uid);
		break;
	*/

	if (err) {
		/*
		**  For the error cases, need to make the drive available
		**	again by resetting the bit in AvailDrives.
		*/
		DM_SET(reqp->rr_drive, AvailDrives);

		/*
		**  Notify vold & clean up.
		*/
		nLogMsg(EL_ERROR1, id, 
			"mount failed for %.*s, drive %s",
			VSNLEN, &reqp->rr_vsn,
			DriveArray[reqp->rr_drive].de_name);
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, retry);
		DelReq(reqp);
		return(-1);
	}
	return(0);
}
#endif

/*
**  FUNCTION:	FailMount()
**		Clean up and dismount a failed mount.
*/
FailMount(reqp)
RCReq_t	*reqp;
{
	reqp->rr_type = T_UNMOUNT;

	if (ibm_dismount(reqp))
		reqp->rr_state = S_RETRY;
	else
		reqp->rr_state = S_UNMOUNTING;
}
