/*
**  ACS Repository Controller
**
**	ibm_pkt.c:  handling of response packets - not used in
**			initial synchronous version
*/
#include "vm_space.h"		/* volman */
#include <sys/stat.h>
#include <sys/errno.h>
#include <fcntl.h>
#include "defs.h"		/* ssi    */
#include "structs.h"		/* ssi    */
#include "lm_structs.h"		/* ssi    */
#include "acsconfig.h"
#include "acsrc.h"

static char	buf[MAX_MESSAGE_SIZE];

/*
**  FUNCTION:	acs_pkt()
**		Receives a msg, then passes it to the response
**		routine.  This routine had to be separated from
**		acs_response() because the latter could be called
**		directly, AFTER a pkt has already been received.
*/
acs_pkt()
{
	char			*id = "acs_pkt";
	REQUEST_HEADER		*rhp;
	COMMAND			cmd;

	if (acs_recv(buf) < 0) {
		VMError = E_SYS;
		nLogMsg(EL_ERROR1|EL_CONSOLE0|EL_SYS, id, "data recv failed");
		return(-1);
	} 
	
	if (acs_response(buf))
		nLogMsg(EL_ERROR1, id, "response failed");;
}


/*
**  FUNCTION:	acs_response()
**  		Provide some handling for orphaned requests canceled
**  		during acsrc initiation, and for error packets.
*/
acs_response(buf)
char	*buf;
{
	char		*id = "acs_response";
	REQUEST_HEADER	*rhp;
	int		err = 0;

	rhp = (REQUEST_HEADER *) buf;

	switch(rhp->message_header.command) {
          case COMMAND_QUERY:
                err = acs_query_rsp(buf);
                break;
          case COMMAND_MOUNT:
                err = acs_mount_rsp(buf);
                break;
          case COMMAND_DISMOUNT:
                err = acs_dismount_rsp(buf);
                break;
          case COMMAND_EJECT:
                err = acs_eject_rsp(buf);
                break;
          case COMMAND_ENTER:
                err = acs_enter_rsp(buf);
                break;
          case COMMAND_CANCEL:
                err = acs_cancel_rsp(buf);
                break;
          default:
                VMError = E_INTERNAL;
                nLogMsg(EL_CONSOLE1|EL_ERROR1, id, 
				"unrecognized response pkt, cmd: %s",
                                cl_command(rhp->message_header.command));
		err = -1;
                break;
	}
	return(err);
}
