/*
**  ACS Repository Controller
**
**  	ibm_q.c:  Request queue handling routines
*/
#include <stdio.h>
#include <sys/errno.h>
#include "cl_space.h"		/* volman */
#include "ibmrc.h"

extern int	errno;

extern RC_t		*rcp;		/* points to config info for this RC */
extern DevMap_t		AvailDrives;
extern int		WaitingEjects;
extern QHead_t		RCQ;		/* work q */
extern movnode_t	*movq;		/* request move q */

/*
**  forward declarations
*/
void		DelReq();

static int	activerequests = 0;	/* number of active requests         */


/*
**  FUNCTION:	CheckQ()
**		Check the request queue for timed out requests
**		either retry or delete.  Ejects and enters wait
**		indefinitely.  
**
**		There is a special case for eject retries when no
**		CAPs are available.
*/
CheckQ()
{
	char		*id = "CheckQ";
	int 		busycaps = 0, availcaps = 0;
	time_t		timeout;
	RCReq_t		*reqp, *nextreq;
	int		cix, capind;
	int		retry_now;

#ifdef WOW
this is for async operation..
	/*
	**  check the requests on the queue to see if anything has
	**	timed out or could be retried
	*/
	if (!QHEAD)
		return;

	/*
	**  timeout is anything stamped earlier than the timeout period
	*/

	timeout = Now - TIMEOUT;

	/*
	**  see if anything has timed out or is waiting for a retry
	*/
	for (reqp=QHEAD; reqp; reqp=nextreq) {
		int	type;
		nextreq = QNEXT(reqp);		/* in case req is DQ'd */
		retry_now = 0;

		/*
		**  don't time out already mounted carts since we aren't
		**	waiting for anything
		*/
		if (reqp->rr_state == S_MOUNTED)
			continue;

		if (reqp->rr_state == S_NODECREATE)
			continue;

		if (reqp->rr_type == T_MOUNT)
			type = M_RCMOUNT;
		else
			type = M_UNMOUNT;
		/*
		**  check for timeout
		*/
		if (reqp->rr_time < timeout) {

			/*
			**  timed out
			*/
			if (reqp->rr_state == S_MOUNTING ||
			    reqp->rr_state == S_UNMOUNTING) {
				if (!reqp->rr_check) {
					/*
					**  Check request to see if it is
					**  still active at the ACSLS.
					*/
					if (reqp->rr_req_id <= 0) {
						/*
						**  No ACK was recvd, so have
						**  no acs id to do a reqqry 
						**  on - can only retry req
						*/
						nLogMsg(EL_STATUS1, id,
						   "%.*s_%d missing ACK seq %d",
						   VSNLEN, &reqp->rr_vsn,
						   reqp->rr_seq,reqp->rr_req_id);
						reqp->rr_state = S_RETRY;
						retry_now = 1;
					} else {
						/*
						**  ACK was rcvd, so use acs
						**  id for req to query req
						**  status. If request lost
						**  in acs, handle further
						**  in the reqqry response.
						*/
						nLogMsg(EL_STATUS1, id,
						   "%.*s_%d/%d check %d",
						   VSNLEN, &reqp->rr_vsn,
							reqp->rr_seq,
							reqp->rr_req_id,
							reqp->rr_req_id);
						reqp->rr_check = 1;
						acs_reqqry(reqp->rr_req_id);
						reqp->rr_time = 
							time((time_t *) 0);
						continue;
					}
				} else {
					/*
					**  followup check timed out too
					*/
					if (reqp->rr_check++ > 3) {
					    nLogMsg(EL_CONSOLE0|EL_CONSOLE1|
						    EL_ERROR1, 0, 
						    "ACSLS TIMEOUT PROBLEM");
			    		    if (reqp->rr_state == S_MOUNTING ||
			    		        reqp->rr_state == S_UNMOUNTING){
						DM_SET(reqp->rr_drive, 
							AvailDrives);
					    }
					    /*
					    **  if request was unmounting,
					    **  don't bother to vary off
					    **  drive - unmount may have
					    **  succeeded, after all.
					    */
			    		    VMError = E_UNAVAIL;
			    		    nLogMsg(EL_CODE|EL_CONSOLE0|
						EL_CONSOLE1|EL_ERROR1,
					    	id,"%.*s_%d/%d: %s",
				 		    VSNLEN, &reqp->rr_vsn, 
						    reqp->rr_seq, reqp->rr_req_id,
						    "Request timed out");
			    		    RCErr2RP(&reqp->rr_vsn, 
							reqp->rr_seq, type, 0);
					    DelReq(reqp);
					}
					continue;
				}
			}

			/*
			**  if this is failed recovery from another error,
			**	send the other error, otherwise just timeout
			*/

			if (!retry_now) {
				if (reqp->rr_errstr[0]) {
					reqp->rr_erretry = 0;
					SendRecoverErr(reqp,
						"recover timed out, too");
				} else {
				    VMError = E_UNAVAIL;
				    nLogMsg(EL_CODE|EL_CONSOLE0|EL_CONSOLE1|
					EL_ERROR1, id, 
				        "%.*s seq %d/%d: Request timed out",
					VSNLEN, &reqp->rr_vsn, 
					reqp->rr_seq, reqp->rr_req_id);
				    RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, 
								type, 0);
				}
				DelReq(reqp);
				continue;
			}
		}
		if (reqp->rr_state != S_RETRY)
			continue;
		
		/*
		**  check retry limit
		*/
		if (reqp->rr_retries > RETRY_LIMIT) {

			/*
			**  retry limit exceeded; same error handling logic
			**	as timeout, above
			*/

			if (reqp->rr_type == T_UNMOUNT) {
				RPCLVaryOff(reqp->rr_drive, 
						"unmount retries exhausted");
				DM_SET(reqp->rr_drive, AvailDrives);
			}
			if (reqp->rr_errstr[0]) {
				SendRecoverErr(reqp, 
					"recover retries exhausted too");
			} else {
				VMError = E_UNAVAIL;
				nLogMsg(EL_CODE|EL_ERROR1, id, 
				  "%.*s: %d retries; possible %s problem",
				  VSNLEN, &reqp->rr_vsn, reqp->rr_retries, 
				  rcp->rcd_name);
				RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, type, 0);
			}
			DelReq(reqp);
			continue;
		}

		/*
		**  retry
		*/

		nLogMsg(EL_STATUS2, id, "retrying seq %d/%d", 
						reqp->rr_seq, reqp->rr_req_id);
		reqp->rr_retries++;
		switch (reqp->rr_type) {
		    case T_UNMOUNT:
			if (acs_dismount(reqp)) 
				continue;
			reqp->rr_state = S_UNMOUNTING;
			reqp->rr_time = time((time_t *) 0);
			break;
		    case T_MOUNT:
			nLogMsg(EL_FATAL, id, "TODO: implement mount retry");
			reqp->rr_time = time((time_t *) 0);
			break;
		    default:
			VMError = E_UNAVAIL;
			nLogMsg(EL_CODE|EL_ERROR1, id,
				"no retry support for type %d requests",
				reqp->rr_type);
			RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, type, 0);
			DelReq(reqp);
			break;
		}
	}
	vClearError();
#endif
}


/***************************
	Request Q Utilities
****************************/

/*
**  FUNCTION:	NewReq()
**		Initialize new request and enqueue on queue.
*/
RCReq_t *
NewReq(vsn, vseq, type)
vsn_t	*vsn;
Seq_t	vseq;
int	type;
{
	char		*id = "NewReq";
	RCReq_t		*reqp, *tqp;

	reqp = (RCReq_t *) nAlloc(sizeof (RCReq_t));
	reqp->rr_vsn = *vsn;
	reqp->rr_state = S_NEW;
        reqp->rr_seq = vseq; 
        reqp->rr_req_id = 0; 
        reqp->rr_type = type;
        reqp->rr_flag = 0;
	reqp->rr_count = 0;
	reqp->rr_ejp = NULL;
        DM_ZERO(reqp->rr_devmap);
        reqp->rr_drive = 0;
	reqp->rr_time = time((time_t *) 0);
	reqp->rr_check = 0;
	reqp->rr_retries = 0;
	reqp->rr_vmerror = 0;
	reqp->rr_errno = 0;
	reqp->rr_errflags = 0;
	reqp->rr_errid = NULL;
	memset(reqp->rr_errstr, 0, sizeof(reqp->rr_errstr));
	reqp->rr_erretry = 0;

	ENQ(reqp);

	if (nLogLevel() & EL_DEBUG4) {
		for (tqp=QHEAD; tqp != NULL; tqp=QNEXT(tqp)) {
			nLogMsg(EL_ALWAYS|EL_NOTIME, id, 
				"seq: %d/%d  type: %d  vsn: %.*s",
				tqp->rr_seq, tqp->rr_req_id, tqp->rr_type, 
				VSNLEN, &tqp->rr_vsn);
		}
	}

	activerequests++;
	return(reqp);
}



/*
**  FUNCTION:	FindReqSeq()
**		Find request on request queue using acsrc seq for key.
*/
RCReq_t *
FindReqSeq(seq)
unsigned short	seq;
{
	RCReq_t		*reqp;

	nLogMsg(EL_DEBUG3, 0, "find vsn seq %d", seq);
	for (reqp=QHEAD; reqp != NULL; reqp=QNEXT(reqp)) {
		nLogMsg(EL_DEBUG4, 0, "vsn seq %d", reqp->rr_req_id);
		if (reqp->rr_req_id == seq)
			break;
	}
	return(reqp);
}

/*
**  FUNCTION:	FindReqRPSeq()
**		Find request on request queue using vold seq for key.
*/
RCReq_t *
FindReqRPSeq(seq)
Seq_t	seq;
{
	RCReq_t		*reqp;

	nLogMsg(EL_DEBUG3, 0, "find vsn seq %d", seq);
	for (reqp=QHEAD; reqp != NULL; reqp=QNEXT(reqp)) {
		nLogMsg(EL_DEBUG4, 0, "vsn seq %d", reqp->rr_seq);
		if (reqp->rr_seq == seq)
			break;
	}
	return(reqp);
}

/*
**  FUNCTION:	FindReqSSISeq()
**		Find request on request queue using ACSLS req_id for key.
*/
RCReq_t *
FindReqSSISeq(seq)
{
	RCReq_t		*reqp;

	nLogMsg(EL_DEBUG3, 0, "find vsn acs seq %d", seq);
	for (reqp=QHEAD; reqp != NULL; reqp=QNEXT(reqp)) {
		if (reqp->rr_req_id == seq)
			break;
	}
	return(reqp);
}


/*
**  FUNCTION:	FindReqVsn()
**		Find request on request queue using vsn for key.
*/
RCReq_t *
FindReqVsn(vsn)
vsn_t	*vsn;
{
	char		*id = "FindReqVsn";
	RCReq_t		*reqp;

	nLogMsg(EL_DEBUG3, id, "find vsn: %.*s", VSNLEN, vsn);
	for (reqp=QHEAD; reqp != NULL; reqp=QNEXT(reqp)) {
		nLogMsg(EL_DEBUG4, id, "seq: %d/%d  seqvsn: %.*s  type: %d",
			reqp->rr_seq, reqp->rr_req_id, VSNLEN, &reqp->rr_vsn, 
			reqp->rr_type);
		if (SAMEVSN(&reqp->rr_vsn, vsn))
			break;
	}
	return(reqp);
}


/*
**  FUNCTION:	DelReq()
**		Remove request from request queue.
*/
void
DelReq(reqp)
RCReq_t	*reqp;
{
	char	*id = "DelReq";

	DQ(reqp);
	reqp->rr_state = S_DELETED;
	nFree(reqp);
	activerequests--;

	if (nLogLevel() & EL_DEBUG4) {
		for (reqp=QHEAD; reqp != NULL; reqp=QNEXT(reqp)) {
			nLogMsg(EL_ALWAYS, id, "seq: %d/%d  vsn: %.*s",
					reqp->rr_seq, reqp->rr_req_id, 
					VSNLEN, &reqp->rr_vsn);
		}
	}
}
