/*
**  ACS Repository Controller
**
**	acs_qry_req.c  -  query status of ACSLS requests
**	Not used in initial syncronous version
*/
#include "vm_space.h"		/* volman */
#include <sys/stat.h>
#include <sys/errno.h>
#include <fcntl.h>
#include "ibmrc.h"		/* acsrc  */


/*
**  FUNCTION:	ibm_reqqry()
**		Query request states
*/
acs_reqqry(req_id)
uint	req_id;
{
	char			*id = "ibm_reqqry";
	struct mtlqmidarg	req;

	req.versn = 0;
	req.req_id = req_id;

	if (ioctl_ibmatl(ibm_fd, MTIOCLQMID, &req)) < 0) {
		nLogMsg(EL_ERROR1|EL_SYS|EL_CONSOLE0|EL_CONSOLE1, id,
			"ioctl_ibmatl error (%s)", IBM_CC(req.mtlqmidret.cc));
		return(-1);
	}

	return(0);
}


acs_reqqry_rsp(qrp)
QUERY_RESPONSE	*qrp;
{
	char		*id = "acs_reqqry_rsp";
	unsigned short	seq;
	QU_REQ_STATUS	*rsp;
	RCReq_t         *reqp;
	LSM		lsm;
	PANEL		panel;
	DRIVE		drive;
	int		bitdrv;
	int		i, err = 0;
	int		foundreq = 0;

	seq = qrp->request_header.message_header.packet_id;

        /*
        **  Check for ACK
        */
        if (qrp->request_header.message_header.message_options & ACKNOWLEDGE) {
                nLogMsg(EL_STATUS1, id, "recvd ACK for seq %d", seq);
                return(0);
        }

	for (i = 0; i < qrp->count; i++) {
		/*
		**  Display current requests (or one specific request)
		*/
		rsp = &qrp->status_response.request_status[i];
		nLogMsg(EL_STATUS1, id, "%s (req %d): %s", 
			cl_command(rsp->command), 
			rsp->request,
			cl_status(rsp->status));

		/*
		**  Nothing else to do on CAP & Query requests
		*/
		if (rsp->command == COMMAND_ENTER 
		     || rsp->command == COMMAND_QUERY)
			continue;

		/*
		**  Mount/dismount request check
		*/
		reqp = FindReqSSISeq(rsp->request);
		if (reqp == NULL) {
			nLogMsg(EL_STATUS1, id, "(no vsn req seq %d)", 
							rsp->request);
			continue;
		}
		if (reqp->rr_check == 0) {
			nLogMsg(EL_STATUS1, id, "no check on seq %d", 
								rsp->request);
			continue;
		}

		/*
		**  it's a followup on a timeout
	 	*/

		reqp->rr_time = time((time_t *) 0);
		reqp->rr_check = 0;

		switch (rsp->status) {
		    case STATUS_CURRENT:
			nLogMsg(EL_STATUS1, id,"%.*s_%d status current",
							VSNLEN, &reqp->rr_vsn,
							reqp->rr_seq);
			break;
		    case STATUS_PENDING:
			nLogMsg(EL_STATUS1, id,"%.*s_%d status pending",
							VSNLEN, &reqp->rr_vsn,
							reqp->rr_seq);
			break;
		    case STATUS_MESSAGE_NOT_FOUND:
			if (reqp->rr_type == T_MOUNT  &&  
			    reqp->rr_state == S_MOUNTED) {
				/*
				**  request completed already
				*/
				continue;
			}
			/*
			**  The acs has forgotten about it
			**  completely. Get the vsn location.
			*/
			nLogMsg(EL_STATUS1, id,
				    "%.*s_%d status not found, querying loc",
							VSNLEN, &reqp->rr_vsn,
							reqp->rr_seq);
			if (acs_volqry(reqp)) {
				RPCLErr2RP(reqp->rr_seq, 1);
				DelReq(reqp);
				continue;
			}
			reqp->rr_check = 1;
			break;

		     default:
			nLogMsg(EL_ERROR1, id, "bad status %d %s", 
					rsp->status, cl_status(rsp->status));
			return(E_INTERNAL);
			break;
		}
	}
	return(0);
}
