/*
**  ACS Repository Controller
**
**	acs_qry_vol.c  -  query location of a volume for a mount request
*/
#include "vm_space.h"		/* volman */
#include <sys/stat.h>
#include <sys/errno.h>
#include <fcntl.h>
#include "ibmrc.h"

/*
**  FUNCTION:	ibm_qry_vol()
**		Get volume info; return pointer to static struct.
*/
ibm_qry_vol(reqp)
RCReq_t	*reqp;
{
	char			*id = "ibm_volqry";
	static struct mtlqarg	req;

	nLogMsg(EL_DEBUG2, id, "%.*s_%d (check %d)", 
			VSNLEN, (char *)&reqp->rr_vsn, 
			reqp->rr_seq, reqp->rr_check);

	/*
	**  get location info & list of drives nearby
	*/
	req.versn = 0;
	req.sub_cmd = MT_QVD;
	memset(req.volser, 0, sizeof(req.volser));
	memcpy(req.volser, (char *)&reqp->rr_vsn, VSNLEN);
	req.hostid[0] = 0;			/* this host's info -? */
	if (ioctl_ibmatl(ibm_fd, MTIOCLQ, (void *) &req) == -1) {
		VMError = E_UNAVAIL;
		nLogMsg(EL_ERROR1|EL_SYS|EL_CODE|EL_CONSOLE0|EL_CONSOLE1, id,
			"ioctl_ibmatl error (%s)", IBM_CC(req.mtlqret.cc));
		return(-1);
	}
	if (req.mtlqret.cc != MTCC_COMPLETE) {
		VMError = E_UNAVAIL;
		nLogMsg(EL_ERROR1|EL_CODE|EL_CONSOLE0|EL_CONSOLE1, id,
			"response error (%s)", IBM_CC(req.mtlqret.cc));
		return(-1);
	}
	if (req.sub_cmd != MT_QVD)
		nLogMsg(EL_FATAL, id, "response info not MT_QVD (%d)", 
				req.sub_cmd);
	if (memcmp(req.mtlqret.info.data.volume_data.volser, 
						&reqp->rr_vsn, VSNLEN))
		nLogMsg(EL_FATAL, id, "response vsn (%.*s) doesn't match %.*s", 
				VSNLEN, 
				req.mtlqret.info.data.volume_data.volser, 
				VSNLEN, &reqp->rr_vsn);

	reqp->rr_volstate = req.mtlqret.info.data.volume_data.vol_state;
	memcpy(reqp->rr_drivelist, 
			req.mtlqret.info.data.volume_data.affinity_lst, 
			NEARBY_DRIVES);
	return(0);
}

