/*
**  IBM 3494 Repository Controller
**
**	ibm_vm.c:  VOLDAEMON PACKET HANDLING
*/

void MMMove();

#include <stdio.h>
#include <sys/errno.h>
#include <setjmp.h>

#include "srv_space.h" 		/* volman */

#include "ibmrc.h"

extern int		errno;

extern int		immortal;
extern jmp_buf		jmpbuf_i;

extern DevEnt_t		*DriveArray;
extern RC_t		*rcp;                   /* points to config   */
						/* info for this RC   */
extern int		WaitingEjects;
extern QHead_t		RCQ;			/* mount q             */
extern QHead_t		EjectQ;			/* eject q             */

extern movnode_t	*FindWaitingMove();
extern char		*MountState();

extern int		IBMmounted;
extern int		*DriveChannels, *ChannelUse;
extern DevMap_t		AvailDrives;

static Seq_t           lastseq = 0;  /* last seq # request from vold      */

static int		Mounts = 0;			/* reporting	     */

/*
**  FUNCTION:	vold_pkt() 
**		Get & handle a packet from the RP
*/
vold_pkt()
{
        char    	*id = "vold_pkt";
	vmpHdr_t        *hp;

	/*
	**  select() has indicated there are packet(s) to read from
	**	the voldaemon: read & keep reading while xdr pkts
	**	are in buffer
	*/
	while (1) {
		nLogMsg(EL_DEBUG4, id, "reading");
		hp = RPCLRecv(id);
        	if (hp == NULL)
                	nLogMsg(EL_FATAL|EL_CONSOLE0, 0, "voldaemon died");

		lastseq = hp->vh_seq;

		/*
		**  process the pkt according to type
		*/
        	switch (hp->vh_type) {

		    /* (ordered by approximate frequency of reception) */

        	    case M_RCMOUNT: 
			MMMount(hp); break;
		    case M_CREATENODE:
			MMNodecreate(hp); break;
        	    case M_UNMOUNT: 
			MMUnmount(hp); break;
		    case M_MOVE: 
			MMMove(hp); break;
        	    case M_TIMER:
			MMTimer(hp); break;
        	    case M_NOP:
                	break;
		    case M_VARY:
		    case M_VOLSTAT:
	  		/* msgs that are just reflected by voldaemon */
                	break;
        	    case M_RCMSG: 
			MMRCMsg(hp); break;
        	    case M_VERROR: 
                	MMError(hp); break;
        	    case M_SHUTDOWN:
			if (immortal) {
				nLogMsg(EL_STATUS1, 0, "shutdown - longjmp");
				longjmp(jmpbuf_i, 1);
				nLogMsg(EL_ERROR1, 0, "longjmp");
			}
                	done();
		    case M_NEWLOG:
			/* assumes sharing voldaemon log, so doesn't rename */
			nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
			break;
		    case M_DEBUG:
			MMDebug(hp); break;
        	    default:
                	nLogMsg(EL_ERROR1, id, 
					"%s: unexpected packet", vHdrStr(hp));
        	}
		nFree(hp);
		if (!RPCLNextRecord())
			break;
	}
}

MMTimer(hp)
  vmpHdr_t	*hp;
{
	vmpTimer_t	*tp;
	time_t		diff;

	tp = MSGPTR(hp, vmpTimer_t);

	/*
	**  figure time between RP sending & receipt -
	**	long times could be due to communications
	**	or acsrc hung on read or ioctl
	*/
	diff = Now - tp->vt_time;
	if (diff > 10) {
		/*
		**  note the fact - may be useful for debugging
		*/
		nLogMsg(EL_STATUS2, 0, 
				"(msg time from vold = %d seconds)", diff);
	} 
}

/*
**  FUNCTION:	MError()
**		Handle error from vold or volnd
*/
MMError(hp)
  vmpHdr_t	*hp;
{
	char		*id = "MMError";
	vmpError_t      *vep;
	RCReq_t		*reqp;

	/*
	 *  log msg & look up request
	 */
        vep = MSGPTR(hp, vmpError_t);

	if (!strncmp((char *)&vep->vn_vsn, vep->vn_msg, VSNLEN))
        	nLogMsg(EL_ERROR1, 0, vep->vn_msg);
	else
        	nLogMsg(EL_ERROR1, 0, "%.*s_%d: %s", 
					VSNLEN, (char *)&vep->vn_vsn, 
					hp->vh_seq, vep->vn_msg);
	reqp = FindReqRPSeq(hp->vh_seq);
	if (reqp == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "no req %.*s_%d", 
				VSNLEN, &vep->vn_vsn, hp->vh_seq);
		return;
	}
	if (reqp->rr_state == S_MOUNTED  ||  reqp->rr_state == S_UNMOUNTING) {
		nLogMsg(EL_STATUS2, id, "%.*s_%d: spurious error from ND",
				VSNLEN, &vep->vn_vsn, hp->vh_seq);
		return;
	}

	/*
	 *  copy err info for later report to vold
	 *	& start unmount
	 */
	memcpy(reqp->rr_errstr, vep->vn_msg, sizeof(reqp->rr_errstr));
	reqp->rr_vmerror = vep->vn_error;
	reqp->rr_erretry = vep->vn_retry;
	reqp->rr_errflags = EL_ERROR1;
	reqp->rr_errid = "volnd err";
	FailMount(reqp);
}


/*
**  FUNCTION:	MMMount()
**		Set up mount request, get list of drives, mount, start node
*/
int
MMMount(hp)
  vmpHdr_t        *hp;
{
	char			*id = "MMMount";
	vmpRCMount_t		*mp;
	RCReq_t			*reqp;
	int			status = -1;
	struct lib_query_info	*info;

	Mounts++;
	mp = MSGPTR(hp, vmpRCMount_t);

	/*
	**  got a mount request, simple check of configuration
	*/
	if (!rcp->rcd_nconfigdev) {
		/*
	 	**  this RC will never mount - no drives configured
	 	*/
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "no drives configured");
		RCErr2RP(&mp->vm_vsn, hp->vh_seq, M_RCMOUNT, 0);
		return(FALSE);
	}

	/*
	**  only 1 mount per vsn
	*/
	for (reqp=QHEAD; reqp; reqp=QNEXT(reqp)) {
		if (SAMEVSN(&mp->vm_vsn, &reqp->rr_vsn)) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
					"mount already requested");
			RCErr2RP(&mp->vm_vsn, hp->vh_seq, M_RCMOUNT, 0);
			return(FALSE);
		}
	}

	/*
	**  make a request 
	*/
	reqp = NewReq(&mp->vm_vsn, hp->vh_seq, T_MOUNT);
	reqp->rr_label = mp->vm_label;
	reqp->rr_devmap = mp->vm_devmap;
	reqp->rr_flag = mp->vm_flag;
	reqp->rr_flag &= ~VO_WAIT;
	reqp->rr_uid = mp->vm_uid;
	reqp->rr_host = mp->vm_host;

	/*
	**  do it
	*/
	return(MountReq(reqp, id));
}

MountReq(reqp, id)
  RCReq_t	*reqp;
  char		*id;
{
	/*
	**  get info on volume (synchronous)
	*/
	if (ibm_qry_vol(reqp)) {
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(FALSE);
	}

	/*
	**  check state
	*/
	if (reqp->rr_volstate != 0) {
		VMError = E_UNAVAIL;
		nLogMsg(EL_ERROR1|EL_CODE|EL_CONSOLE1, id, 
				"%.*s not idle; volume %s",
				VSNLEN, &reqp->rr_vsn, 
				IBM_VolState(reqp->rr_volstate));
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(FALSE);
	}

	/*
	**  choose drive
	*/
	if (AllocateDrive(reqp)) {
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(FALSE);
	}

	/*
	**  request mount (synchronous)
	*/
	if (ibm_mount(reqp)) {
		RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, 0);
		DelReq(reqp);
		return(FALSE);
	}

	/*
	**  check label / create node  TODO %%%
	*/
	IBMmounted++;
	reqp->rr_state = S_NODECREATE;
	RCSendVolNode(M_CREATENODE, reqp->rr_seq,
		&reqp->rr_vsn, &reqp->rr_label,
		reqp->rr_flag, reqp->rr_drive,
		reqp->rr_host, reqp->rr_uid);

	return(TRUE);
}


/*
**  FUNCTION:	MMNode()
**		Node created, mount finished.
*/
MMNodecreate(hp)
  vmpHdr_t	*hp;
{
	static char	*id = "MMNode";
	vmpNode_t	*np;
	RCReq_t		*reqp;

	np = MSGPTR(hp, vmpNode_t);
	reqp = FindReqRPSeq(hp->vh_seq);
	if (reqp == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "no req %.*s_%d", 
				VSNLEN, &np->vs_vsn, hp->vh_seq);
		return;
	}
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"MOUNTED/lbl ok %.*s_%d  drive %d (%s)",
			VSNLEN, &reqp->rr_vsn, reqp->rr_seq,
			reqp->rr_ibmdrv,
			DriveArray[reqp->rr_drive].de_name);
	RCSendMountDone(reqp->rr_seq, &reqp->rr_vsn, reqp->rr_drive, 
						np->vs_flag);
	reqp->rr_state = S_MOUNTED;
}

/*
**  FUNCTION:	MMUnmount()
**		Tell RC to unload vsn and put it away
*/
MMUnmount(hp)
  vmpHdr_t        *hp;
{
	char		*id = "MMUnmount";
	vmpVsn_t      	*vp;
	int           	chan;
	RCReq_t		*reqp;

	vp = MSGPTR(hp, vmpVsn_t);

	/*
        **  should be dismount of completed mount on proper drive
        */
	reqp = FindReqVsn(&vp->v_vsn);
	if (reqp == NULL) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%.*s: no prev request", 
							VSNLEN, &vp->v_vsn);
		RCErr2RP(&vp->v_vsn, hp->vh_seq, M_UNMOUNT, 0);
		return;
	}
	if (reqp->rr_state != S_MOUNTED) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%.*s: not mounted, status %s", 
					VSNLEN, &vp->v_vsn, 
					MountState(reqp->rr_state));
		RCErr2RP(&vp->v_vsn, hp->vh_seq, M_UNMOUNT, 0);
		return;
	}
	
        if (vp->v_flag != reqp->rr_drive) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"%.*s: mounted on other drive: %s", 
		    		VSNLEN, VSNLEN, &vp->v_vsn, 
				DriveArray[reqp->rr_drive].de_name);
		RCErr2RP(&vp->v_vsn, hp->vh_seq, M_UNMOUNT, 0);
		return;
	}

	/*
	**  request is reasonable: install new seq and execute it
	*/
	reqp->rr_seq = hp->vh_seq;
	reqp->rr_type = T_UNMOUNT;
	reqp->rr_flag2 = vp->v_flag2;

	/*
	**  if dismounting, we assume data path is idle
	*/
	chan = DriveChannels[reqp->rr_drive];
	if (ChannelUse[chan] > 0)
		ChannelUse[chan]--;

	/*
	**  try unmount 
	*/
	nLogMsg(EL_STATUS2, 0, "UNMOUNT %*.*s_%d (%s)",
				VSNLEN, &reqp->rr_vsn, reqp->rr_seq,
				DriveArray[reqp->rr_drive].de_name);
	if (ibm_dismount(&reqp->rr_vsn, reqp->rr_ibmdrv, &reqp->rr_req_id)) {
		RCErr2RP(&vp->v_vsn, hp->vh_seq, M_UNMOUNT, 0);
		DelReq(reqp);
		return;
	}
	FinishUnmount(reqp);
}

FinishUnmount(reqp)
RCReq_t		*reqp;
{
	char	*id = "FinishUnmount";

	nLogMsg(EL_DEBUG1, id, 
			"%.*s_%d unmounted, drive %d",
			VSNLEN, &reqp->rr_vsn, reqp->rr_seq, 
			reqp->rr_ibmdrv);
	DM_SET(reqp->rr_drive, AvailDrives);

	if (reqp->rr_errstr[0]) {
		/*
		**  dismount was a recovery from mount problem
		*/
		SendRecoverErr(reqp, "(unmounted)");
		DelReq(reqp);
		return;
	}
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
				"UNMOUNTED %*.*s  drive %d  (%s)",
				VSNLEN, VSNLEN, &reqp->rr_vsn,
				reqp->rr_ibmdrv,
				DriveArray[reqp->rr_drive].de_name);
	if (reqp->rr_type == T_UNMOUNTFIRST) {
		/*
		**  Volume req'd for mount was stuck in drive
		**  and halleluja, now it is free. Hopefully
		**  someone will vary the drive back on if
		**  it was varied off, but we can't since
		**  it might have been varied off for other
		**  reasons. Resume the mount as if it were new.
		*/
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
				"DRIVE %s DISMOUNTED, MAY WANT TO VARY ON",
				DriveArray[reqp->rr_drive].de_name);
		nLogMsg(EL_STATUS1, id, "%*.*s resuming mount",
						VSNLEN, VSNLEN, &reqp->rr_vsn);
		reqp->rr_type = T_MOUNT;
		reqp->rr_check = 0;
		/*
		**  start mount
		*/
		reqp->rr_time = Now;
		MountReq(reqp, id);
		return;
	}
	if (reqp->rr_flag2 & VF_NORESP)
		nLogMsg(EL_STATUS2, id, "(no response requested)");
	else
		RCSendUnmountDone(reqp->rr_seq, &reqp->rr_vsn);

	DelReq(reqp);
}


/*
**  FUNCTION:	MMMove()
**		Set up move request ...
**		In practice, Eject is the only type of move handled.
*/
void
MMMove(hp)
  vmpHdr_t        *hp;
{
	char		*id = "MMMove";
	vmpMove_t       *mvp;
	vsnseq_t	*vsnseqp;
	
	mvp = MSGPTR(hp, vmpMove_t);
	nLogMsg(EL_DEBUG2, 0, "Move from %d to %d",
		mvp->vmv_srcrc, mvp->vmv_dstrc);

	vsnseqp = (vsnseq_t *) ((int)mvp + sizeof(vmpMove_t));

	/*
	**  is this a bad move, eject or accept for this RC?
	*/
	if (mvp->vmv_srcrc == mvp->vmv_dstrc) { 
		/* bad move */
		VMError = E_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
					"%s: move to same location (%d)",
					rcp->rcd_id, mvp->vmv_srcrc);
#if 0 /* XXX */
		ErrMoves(mvp->vmv_nvsns, vsnseqp);
#endif
		return;
	}
	if (rcp->rcd_number != mvp->vmv_srcrc  
			&&  rcp->rcd_number != mvp->vmv_dstrc) {
		VMError = E_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
				"move doesn't involve %s: %d->%d",
				rcp->rcd_id, mvp->vmv_srcrc, mvp->vmv_dstrc);
#if 0 /* XXX */
		ErrMoves(mvp->vmv_nvsns, vsnseqp);
#endif
		return;
	}
	if (mvp->vmv_dstrc == rcp->rcd_number) {
		/* 
		**  ACS doesn't need to do anything - operator will put CAP
		**	in Enter mode and put in vsns.
		*/
		return;
	}

	/*
	**  Looks good - try to do something with it
	**	Vsns are queued, and when an eject is sent,
	**	they are deq'd and put in the caparray (in BatchEjects()).
	*/

	nLogMsg(EL_FATAL, id, "EJECT: not impl");
}


/*
**  FUNCTION:	ErrMoves()
**		Fail each move in pkt - e.g. 
**		internal problems w/ pkt header.
*/
ErrMoves(nvsns, vsnseqp)
  vsnseq_t	*vsnseqp;
{
	int		i, saverr = VMError;

	for (i=0; i < nvsns; i++, vsnseqp++) {
		VMError = saverr;
		RCErr2RP(&vsnseqp->r_vsn, vsnseqp->r_seq, M_MOVE, 0);
	}
}

CodeString_t TypeTab[] = {
	T_MOUNT,		"MOUNT  ",
	T_UNMOUNTFIRST,		"UMNT1st",
	T_UNMOUNT,		"UNMOUNT",
	T_EJECT,		"EJECT  ",
	0,
};

CodeString_t StateTab[] = {
	S_MOUNTING,		"MOUNTING  ",
	S_NODECREATE,		"NODECREATE",
	S_MOUNTED,		"MOUNTED   ",
	S_UNMOUNTING,		"UNMOUNTING",
	S_RETRY,		"RETRY     ",
	S_NEW,			"NEW       ",
	0,
};

/*
**  FUNCTION:	MMRCMsg() 
**		handle msg from operator
*/
MMRCMsg(hp)
  vmpHdr_t        *hp;
{
	char		*id = "MMRCMsg";
	vmpRcMsg_t      *rm;
	vmpHdr_t        *hp2;
	int		capind, i;
	
	rm = MSGPTR(hp, vmpRcMsg_t);
	hp2 = (vmpHdr_t *) rm->rm_msg;

	/*
	**  while the rcmsg type includes space for a regular packet 
	**	inside it, in this rc only the rm_cmd and rm_arg fields 
	**	of the primary level packet are used.
	*/

	switch (rm->rm_cmd) {
	  case RMC_ENTER:
		nLogMsg(EL_FATAL, id, "ENTER: not impl");
		break;
	  case RMC_CANCEL:
		nLogMsg(EL_FATAL, id, "CANCEL ENTER: not impl");
		break;
	  case RMC_CLEAN:
		nLogMsg(EL_FATAL, id, "CLEAN: not impl");
		break;
	  case RMC_REPORT:
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"================ report ===============");
		sleep(1);
		PrintQueues();
		PrintStats();
		break;
	  case RMC_SHUTDOWN:
		nLogMsg(EL_CONSOLE1|EL_CONSOLE0|EL_FATAL, id,
			"shutdown from ibmrc console");
		break;
	  default:
		nLogMsg(EL_CONSOLE0|EL_CONSOLE1|EL_ERROR1, id,
				"bad command type %d", rm->rm_cmd);
		break;
	}
}

PrintQueues()
{
	RCReq_t		*reqp;
	movnode_t	*ejp;

	if (QHEAD || EHEAD)
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"VSN     REQUEST   STATUS     SEQ              TIME");
	else {
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, "No volumes queued");
		return;
	}
	sleep(1);
	for (reqp=QHEAD; reqp; reqp=QNEXT(reqp))
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"%.*s   %s %s    %d/%d    %d",
			VSNLEN, &reqp->rr_vsn, 
			nCodeToStr(reqp->rr_type, TypeTab), 
			nCodeToStr(reqp->rr_state, StateTab), 
			reqp->rr_seq, reqp->rr_req_id,
			Now - reqp->rr_time);
	sleep(1);
	for (ejp=EHEAD; ejp; ejp=ENEXT(ejp))
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"%.*s: %s %s %d/%d    %d",
			VSNLEN, &ejp->vsnseq.r_vsn,
			"EJECT    ", "wait", 
			ejp->vsnseq.r_seq, 0,
			Now - ejp->time);
}


extern time_t		StartTime;	/* for reporting		     */
extern int		IBMmounted;	/* vsns mounted			     */
extern int		Ejected;	/* vsns ejected from cap	     */
extern int		Entered;	/* vsns entered in cap		     */

PrintStats()
{
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
		"==================== STATISTICS =====================");
	sleep(1);
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
		"Mounts requested: %d  mounted: %d", Mounts, IBMmounted);
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
		"Entered: %d  Ejected: %d", Entered, Ejected);
	/* 
	**  Note that since nas_time() sets a single char array, it
	**	can't be called more than once in a nLogMsg()
	*/
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
				"Time up: %s", nas_time(Now - StartTime));
	sleep(1);
	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
		"=====================================================");

}
