/*
** ACS Repository Controller
**
**	acscon.c:  send a console cmd to ACS RC
*/
#include <pwd.h>
#include <ctype.h>

#ifdef EXTERN
# undef EXTERN
#endif
#define EXTERN
#include "srv_space.h"


char	*arg0;

usage()
{
	fprintf(stderr, "usage: %s rc_id cmd [arg]\n", arg0);
	vm_prRC(stderr, "\t", "ACS", 0);
	fprintf(stderr, "\tcmd:\n");
	fprintf(stderr,
	    "\t  enter  acs lsm cap\tput cap in enter mode\n");
	fprintf(stderr,
	    "\t  cancel acs lsm cap\tcancel enter or eject on cap\n");
	fprintf(stderr,
	    "\t\t\t\tcaps 0,1 are big, cap2 is the PCAP\n");
	fprintf(stderr,
	    "\t  report   \t\tlog status to console acct\n");
	fprintf(stderr,
	    "\t  clean    \t\tlog cleaning cartridge status to console acct\n");
	fprintf(stderr,"\t  shutdown \t\tshut down acsrc\n");
	exit(1);
}

RC_t		*rcp;

main(argc, argv)
char	*argv[];
{
	char		*rc_id, *consolecmd;
	vmpHdr_t	*hp;
	vmpRcMsg_t	*rcmp;
	int		uid, err, i;

	arg0 = argv[0];
	CheckUID(arg0);

	/*
	**  start vold connection & get RC config
	*/
	RPCLInit(M_CONSOLE);
	RPCLRCConfig();

	/*
	**  must have rc_id & cmd
	*/
	if (argc < 3)
		usage();

	/*
	**  check RC
	*/
	rc_id = argv[1];
	if ((rcp = cGetRcp(rc_id)) == NULL) {
		fprintf(stderr, "-- Can't get %s info\n", rc_id);
		usage();
	}
	if (strcmp(rcp->rcd_type, "ACS")) {
		fprintf(stderr, "-- %s is not an ACS\n", rc_id);
		usage();
	}
	
	/*
	**  start building packet, using key from voldaemon
	*/
	hp = vBuildHdr(M_RCMSG, vGetSeq(), HdrNULL);
	rcmp = MSGPTR(hp, vmpRcMsg_t);
	rcmp->rm_rc = rcp->rcd_number;
	rcmp->rm_flag = RCM_RC;
	
	consolecmd = argv[2];
	if (!strcmp(consolecmd, "enter")) {
		if (argc != 6) {
			fprintf(stderr, "-- Wrong number of arguments\n");
			goto bomb;
		}
		rcmp->rm_cmd = RMC_ENTER;
	} else if (!strcmp(consolecmd, "cancel")) {
		if (argc != 6) {
			fprintf(stderr, "-- Wrong number of arguments\n");
			goto bomb;
		}
		rcmp->rm_cmd = RMC_CANCEL;
	} else if (!strcmp(consolecmd, "report")) {
		if (argc != 3) {
			fprintf(stderr, "-- Wrong number of arguments\n");
			goto bomb;
		}
		rcmp->rm_cmd = RMC_REPORT;
	} else if (!strcmp(consolecmd, "clean")) {
		if (argc != 3) {
			fprintf(stderr, "-- Wrong number of arguments\n");
			goto bomb;
		}
		rcmp->rm_cmd = RMC_CLEAN;
	} else if (!strcmp(consolecmd, "shutdown")) {
		if (argc != 3) {
			fprintf(stderr, "-- Wrong number of arguments\n");
			goto bomb;
		}
		rcmp->rm_cmd = RMC_SHUTDOWN;
	} else {
		fprintf(stderr, "-- cmd not recognized (%s)\n", consolecmd);
		goto bomb;
	}
	if (argc == 6) {	/*  there is a CAP specification */
		/*
		**  check CAP arg
		*/
		for (i = 0; argv[3][i]; i++) {
			if (!isdigit(argv[3][i])) {
				fprintf(stderr, "-- ACS nondigit [%c]\n", 
							argv[3][i]);
				goto bomb;
			}
		}
		for (i = 0; argv[4][i]; i++) {
			if (!isdigit(argv[4][i])) {
				fprintf(stderr, "-- LSM nondigit [%c]\n", 
							argv[4][i]);
				goto bomb;
			}
		}
		for (i = 0; argv[5][i]; i++) {
			if (!isdigit(argv[5][i])) {
				fprintf(stderr, "-- CAP nondigit [%c]\n", 
							argv[5][i]);
				goto bomb;
			}
		}
		
		rcmp->rm_arg1 = atoi(argv[3]);
		if (rcmp->rm_arg1 < 0) {
			fprintf(stderr, "-- ACS must be >= 0\n");
			goto bomb;
		}
		rcmp->rm_arg2 = atoi(argv[4]);
		if (rcmp->rm_arg2 < 0) {
			fprintf(stderr, "-- LSM must be >= 0\n");
			goto bomb;
		}
		rcmp->rm_arg3 = atoi(argv[5]);
		if (rcmp->rm_arg3 < 0) {
			fprintf(stderr, "-- CAP must be >= 0\n");
			goto bomb;
		}
	} 
	/*
	**  send msg & quit (response gets broadcast to console)
	*/
	RPCLSend(hp, 1);
	RPCLGoodbye();

	return(err);
bomb:
	RPCLGoodbye();
	usage();
}

GetResponse()
{
	vmpHdr_t	*rhp;
	vmpVsn_t	*vsp;
	vmpError_t	*ep;

	for (;;) {
		rhp = RPCLRecv("GetResponse");
		switch (rhp->vh_type) {
			case M_ACK:
				return(0);
			case M_VERROR: 
				ep = MSGPTR(rhp, vmpError_t);
				printf("error from voldaemon: %s\n",ep->vn_msg);
				return(1);
			case M_SHUTDOWN:
				printf("shutdown received\n");
				vm_term();
				exit(1);
			case M_NOP:
				nFree(rhp);
				continue;
			default:
				printf("Unexpected packet: %s\n", 
							vMsgType(rhp));
				exit(1);
		}
	}
}
