/*
** $Header:
**
**	Volume Manager Repository Controller for IBM 3494.
**
**	ibmrc.c
**
**	This process receives mount and move(eject) requests from 
**	the vold, translates them into request structures in 
**	memory, and forwards them to the local host's daemon
**	which forwards them over a wire to the PC controller.
**	It blocks on a select() on vold ipc and for now, executes
**	requests synchronously since the async daemon interface
**	seems more problematical i.e. not msg driven so select()
**	won't work - better maybe to graft this RC onto the daemon
**	source for async purposes.
**
**	Adapted from the ACSRC (RC for StorageTek ACS) by Bill Ross.
**	ACSRC by Bill Ross, portions co-designed by Ruth Iverson.
**
**	NASA Ames, 1997.
*/

#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>

#ifdef EXTERN
# undef EXTERN
#endif
#define EXTERN
#include "srv_space.h"
#include "ibmrc.h"

#include <fcntl.h>
#include <utmp.h>
#include <pwd.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <stdlib.h>
#include <setjmp.h>

extern int	errno;

extern char		confnam[];

/*  from rclib.c  */
extern int	*DriveChannels, *ChannelUse;

DevEnt_t	*DriveArray;
int		DriveTotal;
int		ibm_drives[IBM_MAXDRIVES]; /* 2 bytes in drive count info pkt */
int		tot_drives = 0;
DevMap_t	AvailDrives;
QHead_t		RCQ;			/* mount q			     */
QHead_t		EjectQ;			/* eject q			     */
RC_t		*rcp = NULL;		/* points to config info for this RC */
int		ibm_fd = -1;		/* fd for IBM interface		     */
time_t		Now;			/* time last pkt rcvd		     */
time_t		StartTime;		/* for reporting		     */
int		IBMmounted = 0;		/* vsns mounted			     */
int		Ejected = 0;		/* vsns ejected from cap	     */
int		Entered = 0;		/* vsns entered in cap		     */
int		Pid;

extern int	immortal;
extern jmp_buf	jmpbuf_i;

char    	*DriveErrors[MAXDEVS];	/* TODO maintain beyond startup */

/*
**  (relatively) local globals
*/
static int	vm_fd;			/* copy of vold ipc fd for select()  */
static fd_set	readfds;		/* struct for select()		     */
static int	lock_fd;
static int	shuttingdown = 0;

static char	spooldir[100];
static char	rcno[10], *arg0, **args;

usage()
{
	fprintf(stderr, "usage: %s rc [-i]\n", arg0);
	exit(1);
}

/*
**  FUNCTION:	done()	
**		Exit routine - inform RP & die
*/
done()
{
	if (!shuttingdown) {
		shuttingdown = 1;
		RPCLGoodbye();
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
			"================= shutdown =================" );
		PrintQueues();
		ibm_terminate();
		PrintStats();
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
					"================= shutdown complete");
	} else
		nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, 
					"------(recursive) shutdown complete");
	sleep(3);	/* allow vold to rcv goodbye */
	exit(VMError);
}

Cleanup()
{
	RCReq_t		*reqp, *nextreq;

	ibm_terminate();

	for (reqp=QHEAD; reqp != NULL; reqp=nextreq) {
		nextreq = QNEXT(reqp);
		DelReq(reqp);
	}
}

/**************
   MAIN
***************/
main(argc, argv)
char	*argv[];
{
	int	nfound;

	args = argv;
	arg0 = argv[0];

	/*
	 *  check arg & save rcno
	 */
	if (argc < 2  ||  argc > 3)
		usage();
	if (argc == 2) {
		strcpy(rcno, argv[1]);
        } else {
		if (!strcmp(argv[1], "-i")) {
			immortal = 1;
			strcpy(rcno, argv[2]);
		} else if (!strcmp(argv[2], "-i")) {
			immortal = 1;
			strcpy(rcno, argv[1]);
		} else {
			usage();
		}
	}
	time(&StartTime);
	Now = StartTime;

	/*
	**  set up
	*/
	Init();

	/*
	**  if anything is buffered from vold setup, get it
	*/
	if (RPCLNextRecord())
		vold_pkt();

	/*
	**  loop reading packets
	*/
	while (1) {

		/*
		**  make sure communications are intact
		*/
		if (vm_fd < 0)
			nLogMsg(EL_CONSOLE0|EL_CONSOLE1|EL_FATAL, 0, 
				"bad vold socket (%d): shutting down", vm_fd);
		if (ibm_fd < 0)
			nLogMsg(EL_CONSOLE0|EL_CONSOLE1|EL_FATAL, 0, 
				"bad ibm socket (%d): shutting down", ibm_fd);

		/* 
		**  set bit(s) of file descriptor(s) for select()
		**	to wait on
		*/
		FD_ZERO(&readfds);
		FD_SET(vm_fd, &readfds);
/*
%%% robot stuff is synchronous 
		FD_SET(ibm_fd, &readfds);
*/

		/* 
		**  block on select for input 
		*/
		nLogMsg(EL_DEBUG4, 0, "WAIT on select/rcv");
		nfound = select(FD_SETSIZE, &readfds, (fd_set *)NULL,
				(fd_set *)NULL, (struct timeval *) NULL);

		if (nfound == -1) {
			if (errno == EINTR) {
				nLogMsg(EL_DEBUG2, 0, "EINTR on select");
				continue;
			}
			nLogMsg(EL_SYS|EL_ERROR1, 0, "select failed");
			return(-1);
		}
		if (nfound != 1)
			nLogMsg(EL_FATAL, 0, "select!!");
		time(&Now);

		/*
		**  Find out where data is from and process request(s)
		**  from vold %%% and response(s) from robot /when made async

		if (FD_ISSET(ibm_fd, &readfds))	
			%%%ibm_pkt();
		*/
		if (FD_ISSET(vm_fd, &readfds))  /* volman (client) request */
			vold_pkt();
		else
			nLogMsg(EL_FATAL, 0, "select2!!");
	}
}


/*****************
  INITIALIZATION
******************/


Init()
{
	char		*id = "Init";
	DevMap_t	VaryDrives;
	int		i, j, err;
	char		logtag[10];

	/*
	 *  go to ibm spool dir in case core is dumped
	 */
	vInitDirNames();
	sprintf(spooldir, "%s/ibm%s", vSpoolDir, rcno);
	if (chdir(spooldir) == -1) {
		fprintf(stderr, "can't chdir to %s: %s\n", 
						spooldir, nSysErr(errno));
		exit(1);
	}

	/*
	**  establish RP connection, getting config from RP;
	**	note its ipc rcv file descriptor for select()
        */
	sprintf(logtag, "IBM%s", rcno);
        nLogInit(logtag, done);
        nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);

	/*
	**  come here if we get lonely for vold
	*/
	if (setjmp(jmpbuf_i)) {
		nLogMsg(EL_STATUS1, 0, "setjmp'd");
		Cleanup();
		sleep(30);
	}
	nLogMsg(EL_STATUS1, 0, "starting");
	RCConnectRP();
	vm_fd = RPCLfd();

        /*
        **  look up invoked RC & check that it's consistent
        */
        if ((rcp = cGetRcp(rcno)) == NULL) {
                nLogMsg(EL_FATAL|EL_TOSTDERR|EL_CONSOLE0|EL_CONSOLE1, id, 
					"%s: can't get RC %s info - quitting", 
					arg0, rcno);
        }
	if (strcmp(rcp->rcd_type, RCTYPE)) {
                nLogMsg(EL_FATAL|EL_TOSTDERR|EL_CONSOLE0|EL_CONSOLE1, id, 
			"%s: RC %s is type %s - this prog type %s - quitting",
			arg0, rcp->rcd_id, rcp->rcd_type, RCTYPE);
	}
	if (rcp->rcd_flags & RCF_TIMER == 0) {
                nLogMsg(EL_FATAL|EL_TOSTDERR|EL_CONSOLE0|EL_CONSOLE1, id, 
			"%s (%s) needs RCF_TIMER not set in VCONF - quitting",
			arg0, rcp->rcd_id);
	}
        /*
        **  make sure we are ok to run as root
	*/
        CheckId(getuid(), rcp);

	/*
	**  see if an IBM 3494 RC is already running
	*/
	Pid = getpid();
	BecomeIBMRC(arg0);

	/*
	**  init drive bitmap and robot->devmap index
	*/
	for (i=0; i<IBM_MAXDRIVES; i++)
		ibm_drives[i] = -1;
	DM_ZERO(AvailDrives);
	for (i=0; i<DriveTotal; i++) {
		int	ibm_drive = DriveArray[i].de_acsdevloc.ad_drv;

		if (DriveArray[i].de_rc != rcp->rcd_number)
			continue;
		DM_SET(i, AvailDrives);
		tot_drives++;

		/*
		**  check/add the index entry
		*/
		if (ibm_drive >= IBM_MAXDRIVES  ||  ibm_drive < 0)
			nLogMsg(EL_FATAL|EL_CONSOLE0|EL_CONSOLE1, id,
				"%s: volman config: must be in 0..%d %s",
				DriveArray[i].de_name, IBM_MAXDRIVES-1,
				"or increase IBM_MAXDRIVES");
		    
		if (ibm_drives[ibm_drive] != -1)
			nLogMsg(EL_FATAL|EL_CONSOLE0|EL_CONSOLE1, id,
				"Drive number in robot multiply defined: %d %s",
				ibm_drive, DriveArray[i].de_name);
		ibm_drives[ibm_drive] = i;
	}

	/*
	**  init rotating drive allocation array & err counts
	*/
	InitDrives();
	/*
	**  init channel tracking for drive alloc
	*/
	RCInitChan();

	/*
	**  init RCQ (mounts) and EjectQ
	*/
	RCQ.q_head = NULL;
	RCQ.q_tail = NULL;
	EjectQ.q_head = NULL;
	EjectQ.q_tail = NULL;

	memset(DriveErrors, 0, sizeof(DriveErrors));
	DM_ZERO(VaryDrives);
	ibm_init(&VaryDrives);

	/*
	**  vary off any drives that e.g. had dismount problems in ibm_init()
	**	doing this before announcing Ready so vold has correct drive 
	**	counts and won't send more mounts than available drives.
	*/
	VaryOffMap(&VaryDrives);

	/*
	**  announce to vold that we are up & running as an RC
	*/
        RCReadyRP(rcp);

	/*
	**  update vold on status of eject requests that may have
	**	completed   %%% TODO eject/enter stuff
	InitCheckEjects();
	*/

	nLogMsg(EL_CONSOLE1|EL_STATUS1, 0, "------Initialization complete");
}


/*
**  FUNCTION:	InitDrives()
**		Initialize the local drive map.
*/
InitDrives()
{
	int	i;

	/*
	 *  set all drivearray statuses = 0 for err counting
	 */
	for (i=0; i<DriveTotal; i++)
		DriveArray[i].de_status = 0;

}


/*
**  FUNCTION:	VaryOffMap()
**		Vary off drives w/ hardware or dismount problems.
**  		Must be called after RCConnectRP().
*/
VaryOffMap(drvmap)
DevMap_t	*drvmap;
{
	int	i;

	for (i=0; i<DriveTotal; i++) {
		if (DM_ISSET(i, *drvmap)) {
			RPCLVaryOff(i, DriveErrors[i]);
		}
	}
}


/**********************
  GENERAL UTILITIES
***********************/


/*
**  FUNCTION:	SendRecoverErr()
**		An unmount was issued to recover a mount that had
**		label or device problems. The unmount has completed 
**		(or failed), so now send the original error that was 
**		saved at mount time,
**
*/
SendRecoverErr(reqp, str)
RCReq_t	*reqp;
char	*str;
{
	VMError = reqp->rr_vmerror;
	errno = reqp->rr_errno;

	/* vsn if any should be in reqp->rr_errstr */
	nLogMsg(EL_CODE|reqp->rr_errflags, reqp->rr_errid, "%s %s",
					reqp->rr_errstr, str);

	RCErr2RP(&reqp->rr_vsn, reqp->rr_seq, M_RCMOUNT, reqp->rr_erretry);
	vClearError();
	errno = 0;
}


/*
**  FUNCTION:	AllocateDrive()
**		Allocate drives in the most efficient manner possible.
**		Ensure equal use of all drives and use of least-active
**		data paths to maximize thruput.
**
**		TODO - %%% consider data paths for this
*/
int
AllocateDrive(reqp)
RCReq_t	*reqp;
{
	char		*id = "AllocateDrive";
	int		i, bitdrv;
	DevMap_t	okdrives;

	/*
	**  ok drives are ones set in the request pkt & not already used
	*/
	DM_AND(reqp->rr_devmap, AvailDrives, okdrives);
	nLogMsg(EL_DEBUG3, id, "req: %s", vDevMapStr(&reqp->rr_devmap));
	nLogMsg(EL_DEBUG3, id, "ibm: %s", vDevMapStr(&AvailDrives));
	nLogMsg(EL_DEBUG3, id, "and: %s", vDevMapStr(&okdrives));

	/*
	**  the vsn location query returns a list of up to
	**	NEARBY_DRIVES, ordered by proximity
	*/
	for (i=0; i<NEARBY_DRIVES; i++) {
		int	ibm_drive = reqp->rr_drivelist[i];

		/*
		**  there is conflicting info about whether
		**	ibm_drive == 0 is a drive or a list
		**	terminator
		*/
if (ibm_drive < 1)
nLogMsg(EL_STATUS1, id, "drive %d is %d", i, ibm_drive);
		bitdrv = ibm_drives[ibm_drive];
		if (DM_ISSET(bitdrv, okdrives)) {
			reqp->rr_ibmdrv = ibm_drive;
			reqp->rr_drive = bitdrv;
			return(0);
		}
	}
	VMError = E_UNAVAIL;
	nLogMsg(EL_CODE|EL_ERROR1, id, "No acceptable drive");
	return(1);
}



/*
**  FUNCTION:	BecomeIBMRC()
**		Ensure there is no other ibmrc already running.
*/
BecomeIBMRC(name)
char	*name;
{
	char	*id = "BecomeIBMRC";
	int	i, fd;
	char	lockfile[256];
	char	logtag[10];

	sprintf(lockfile, "%s/lock", spooldir);

	GetLock(lockfile, name);

	/*
	**  got lock - restart logging w/ syslog
	*/
	sprintf(logtag, "IBM%d", rcp->rcd_number);
	nLogInit(logtag, done);
	nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
	nLogInitConsole(rp_syslog, EL_CONSOLE0, EL_LEVEL);
	if (rcp->rcd_syslog)
		nLogInitConsole(rcp->rcd_syslog, EL_CONSOLE1, EL_LEVEL);
        nLogMsg(EL_STATUS1, 0, "==== %s here: rc=%s pid=%d ====",
          					name, rcp->rcd_id, Pid);
	nLogMsg(EL_STATUS1, id, "--------------------------------------------");
/*
#ifdef DEBUG
	{	char	*lognam;

		if (lognam = getlogin()) {
			if (strlen(VolManOperAcct) >= strlen(lognam))
				strcpy(VolManOperAcct, lognam);
		}
	}
#endif
*/
}

CodeString_t	MountStateTab[] = {
	S_MOUNTING,	"S_MOUNTING",
	S_NODECREATE,	"S_NODECREATE",
	S_MOUNTED,	"S_MOUNTED",
	S_UNMOUNTING,	"S_UNMOUNTING",
	S_RETRY,	"S_RETRY",
	S_NEW,		"S_NEW",
	0,		"",
};

char *
MountState(state)
{
	return(nCodeToStr(state, MountStateTab));
}

