/*
** $Header
**
**  IBM 3494 Repository Controller
**
**  ibmrc.h  -  Main structures and definitions
**
*/
#ifndef	_IBMRC_H_
#define	_IBMRC_H_

/*
**  symbolic name from etc/ibmatl.conf
*/
#define IBM_NAME	"3494a"



#include "mtlibio.h"


#define IBM_MAXDRIVES	256	/* 1 bytes in drive affinity list */

#define RETRY_LIMIT 	3
#define TIMEOUT		300

#define RCTYPE		"IBM"

extern int	ibm_fd;
extern time_t	Now;

/************************************************************************
**  Definitions for request queue functions.
*************************************************************************/

/*
**  max number of nearby drives returned by vsn loc query
*/
#define NEARBY_DRIVES	32		

typedef struct RCReq {
	QLink_t		rr_q;		/* work q			*/
	Seq_t		rr_seq;		/* seq # from voldaemon		*/
	uint		rr_req_id;	/* seq # from ACSLM		*/
	time_t		rr_time;	/* time of arrival 		*/
	vsn_t		rr_vsn;		/* vsn requested		*/
	vsn_t		rr_label;	/* internal label if any	*/
	int		rr_type;	/* type of request		*/
	int		rr_state;	/* where we are w/ this request	*/
	int		rr_flag;	/* client bit flags		*/
	int		rr_flag2;	/* client bit flags		*/
	int		rr_uid;		/* uid for mount/createnode	*/
	int		rr_host;	/* host for mount/createnode	*/
	DevMap_t	rr_devmap;	/* client acceptable drives	*/
	int		rr_drive;	/* bitmap # of drive allocated	*/
	int		rr_ibmdrv;	/* # of drive from robot's view */
	int		rr_retries;	/* when slots not avail		*/
	int		rr_check;	/* flag for request check	*/
	int		rr_count;	/* # vsns ejecting in this req	*/
	char		*rr_ejp;	/* points to movenode chain     */
	/* the next group is filled in by qry_vol			*/
	int		rr_volstate;	/* obvious..			*/
	char		rr_drivelist[NEARBY_DRIVES]; /* list of nearby drives*/
	/* the next group is for reinstalling errors */
	int		rr_errno;	/* errno at time of initial prob*/
	int		rr_vmerror;	/* VMError			*/
	int		rr_errflags;	/* flags for vLogMsg()		*/
	char		*rr_errid;	/* id of routine w/ error	*/
	char		rr_errstr[LINELEN]; /* description of error	*/
	int		rr_erretry;	/* whether to set retry flag	*/
} RCReq_t;

extern RCReq_t		*NewReq();
extern RCReq_t		*FindReq(), *FindReqVsn();
extern RCReq_t		*FindReqSeq(), *FindReqRPSeq();

/* queueing macros */

#define QHEAD			((RCReq_t *) RCQ.q_head)
#define QNEXT(reqp)		((RCReq_t *) ((reqp)->rr_q.q_next))

#define ENQ(reqp)		ENQUEUE((reqp), &RCQ, rr_q, RCReq_t)
#define DQ(reqp)		DEQUEUE((reqp), &RCQ, rr_q, RCReq_t)

/* rr_type */

#define T_MOUNT		1
#define T_UNMOUNTFIRST	2
#define T_UNMOUNT	3
#define	T_EJECT		4

/* rr_state & c_rstat */

#define S_MOUNTING	2
#define S_NODECREATE	3
#define S_MOUNTED	4
#define S_UNMOUNTING	5
#define S_RETRY		6
#define S_NEW		99


typedef enum {
        T_EMPTY, T_COMMENT, T_INT, T_STR
} Token_t;                      /* return value types from GetToken() */


/*
**  Doubly linked list of movnodes is kept for tracking move requests
*/
typedef struct movnode {
	QLink_t		q;
	char		prio;
	vsnseq_t	vsnseq;
	time_t		time;
} movnode_t;

#define EHEAD			((movnode_t *) EjectQ.q_head)
#define ENEXT(movp)		((movnode_t *) ((movp)->q.q_next))

#define EDQ(movp)		DEQUEUE((movp), &EjectQ, q, movnode_t)

/*
**  routines
*/
/*  ibm_qry_vol.c  */
extern int	ibm_qry_vol(RCReq_t *reqp);

/*  ibm_tables.c  */
extern char	*IBM_CC(int code);
extern char	*IBM_VolState(char code);
extern char	*IBM_InfoType(char code);
extern char	*IBM_LibState(int code);

#endif 	/* _IBMRC_H_ */
