
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>


char	*captest = "captest";

typedef char            ACS;
typedef char            LSM;
typedef char            CAP;

typedef struct {
    ACS             acs;
    LSM             lsm;
} LSMID;

typedef struct {
    LSMID           lsm_id;
    CAP             cap;
} CAPID;

typedef unsigned short  MESSAGE_ID;

#define MAX_LSM         	15
#define ECAP_SIZE           	40
#define MAXLSMS                     (MAX_LSM+1)
#define MAXCAPS                     (MAXLSMS*3)

typedef unsigned int Seq_t;
#define VSNLEN 6
typedef struct {
        char    v_vsn[VSNLEN];
} vsn_t;

typedef struct vsnseq {
        vsn_t           r_vsn;
        Seq_t           r_seq;
} vsnseq_t;

typedef struct cap {
	CAPID		c_cap;		/* CAP identifier                  */
	int		c_cmd;		/* CAP command in progress	   */
	int		c_type;		/* CAP type (SCAP, ECAP, or PCAP)  */
	int		c_state;	/* CAP state                       */
	Seq_t		c_seq;		/* seq # used for req              */
	MESSAGE_ID	c_req_id;	/* ACSLS request identifier        */
	unsigned short	c_cnclseq;	/* sequence # of cancel request    */
	int		c_prio;		/* cap priority for eject          */
	int		c_count;	/* # of vsns to be ejected         */
	vsnseq_t   c_vsnseq[ECAP_SIZE];	/* vsns/vseqs sent as eject	   */
	time_t		c_time;		/* time status change request made */
} cap_t;

/* c_cmd */

#define CMD_NONE	0
#define CMD_EJECT	1
#define CMD_ENTER	2
#define CMD_AUDIT	3

/* c_type */

#define	CT_PCAP		1	/* priority (1 vsn) CAP		  	*/
#define	CT_SCAP		2	/* standard (21 vsn) CAP		*/
#define	CT_ECAP		3	/* extended (40 vsn) CAP		*/
#define CT_UNKNOWN	99	/* CAP type unknown (init)		*/

/* c_state - order is used for checking status */

#define C_BROKEN	1	/* CAP or protocol problem                */
#define C_UNINIT	2	/* acsrc coming up (state unknown)        */
#define C_AVAIL 	3	/* CAP is available for requests (LOCKED) */
#define C_ENTERMSG	4	/* ENTER request issued                   */
#define C_ENTER		5	/* ACK received on ENTER  (ENTER active)  */
#define C_CANCEL	6	/* CANCEL ENTER issued                    */
#define C_CANCELED	7	/* CANCEL ENTER resp rcvd		  */
#define C_EJECT		8	/* EJECT issued on CAP    (EJECT active)  */
#define C_UNAVAIL	9	/* CAP is in use by another client or
				 * is ejecting cancelled ENTERs           */
#define C_AUDIT		10 	/* AUDIT is in progress (CAP reserved)    */
#define C_DONE		99	/* EJECT done, not recovered              */

cap_t   CapArray[MAXCAPS];
cap_t   CapTest[MAXCAPS];

/*
**  FUNCTION:	InitCapArray()
**		Initialize CapArray in memory
**		(it will be updated and written to disk later in init)
*/
void
InitCapArray()
{
	register int	i;

	fprintf(stderr,"init CapArray...\n");

	for (i = 0; i < MAXCAPS; i++) {
		CapArray[i].c_type = CT_UNKNOWN;
		CapArray[i].c_state = C_UNINIT;
		CapArray[i].c_cmd = CMD_NONE;
		CapArray[i].c_seq = 0;
		CapArray[i].c_req_id = 0;
		CapArray[i].c_cnclseq = 0;
		CapArray[i].c_count = 0;
		CapArray[i].c_prio = -1;
		memset(CapArray[i].c_vsnseq, 0, sizeof(CapArray[i].c_vsnseq));
	}
}
/*
**  FUNCTION:	LogCapState()
**		Create/update file associated with CapArray change.
*/
LogCapState()
{
	char	*id = "LogCapState";
	int	fd;
	int	casize;		/* size of configured CapArray */
	int	capstatesize, i;

	fd = open(captest, O_CREAT|O_RDWR|O_SYNC, 0600);
	if (fd == -1) {
		perror(captest);
		exit(1);
	}

	casize = MAXCAPS * sizeof(cap_t);
	printf("casize %d, cap_t %d\n", casize, sizeof(cap_t));

	if (write(fd, CapArray, casize) != casize) {
		perror("write");
		exit(1);
	}
	if (fsync(fd) == -1) {
		perror("fsync");
		exit(1);
	}
	close(fd);
	fd = open(captest, O_RDONLY, 0600);
	if (fd == -1) {
		perror("reopen");
		exit(1);
	}
	capstatesize = read(fd, CapTest, 2 * MAXCAPS * sizeof(cap_t));
	if (capstatesize == -1) {
		perror("read");
		exit(1);
	} else if (capstatesize == 0) {
		fprintf(stderr, "read 0\n");
		exit(1);
	} else if (capstatesize != casize) {
		fprintf(stderr, "read %d of %d", capstatesize, casize);
		exit(1);
	}
	for (i=0; i<MAXCAPS; i++) {
		if (memcmp(&CapTest[i], &CapArray[i], sizeof(cap_t))) {
			fprintf(stderr, "cap %d doesn't agree\n", i);
			fprintf(stderr,
		    		"%d    %d:%d:%d     %d\t%d   %d:%d:%d %d\n",
				CapArray[i].c_type,
				CapArray[i].c_cap.lsm_id.acs,
				CapArray[i].c_cap.lsm_id.lsm,
				CapArray[i].c_cap.cap,
				CapArray[i].c_prio,
				CapArray[i].c_cmd, 
				CapArray[i].c_seq,
				CapArray[i].c_req_id,
				CapArray[i].c_cnclseq,
				CapArray[i].c_state);
			fprintf(stderr,
		    		"%d    %d:%d:%d     %d\t%d   %d:%d:%d %d\n",
				CapTest[i].c_type,
				CapTest[i].c_cap.lsm_id.acs,
				CapTest[i].c_cap.lsm_id.lsm,
				CapTest[i].c_cap.cap,
				CapTest[i].c_prio,
				CapTest[i].c_cmd,
				CapTest[i].c_seq,
				CapTest[i].c_req_id,
				CapTest[i].c_cnclseq,
				CapTest[i].c_state);
		}
	}
	close(fd);
}

main()
{
	InitCapArray();
	LogCapState();
}
