
/*
 *  rclib.c - routines that RCs have in common
 */

#include <pwd.h>
#include "srv_space.h"
#include "rclib.h"

extern RC_t	*rcp;
extern DevEnt_t	*DriveArray;

int		*DriveChannels = NULL;
int		*ChannelUse = NULL;
int		TotalChannels;

/*
 *  CheckId() - check uid legality & setuid root if necessary
 */
  void
CheckId(int uid, RC_t *rcp)
{
	struct 	passwd *pwp, *getpwnam();
        
	/*
         *  Who is attempting to breathe life into us? if not root, must be
         *      volman acct or appropriate rc acct; must setuid root for
         *      tape driver to accept our special orders.
         */

        if (uid != 0) {

                /*
                 * get the volume manager operator's uid
                 */

                if (!(pwp = getpwnam(VolManOperAcct)))
                        nLogMsg(EL_FATAL, 0, "error getting oper %s info: %d",
                                                        VolManOperAcct, errno);
                if (uid != pwp->pw_uid) {
                        if (!(pwp = getpwnam(rcp->rcd_login)))
                                nLogMsg(EL_FATAL, 0,
                                        "error getting oper %s info: %d",
                                                        rcp->rcd_login, errno);
                        if (uid != pwp->pw_uid)
                                nLogMsg(EL_FATAL, 0,
                                        "need to be %s, %s or root",
                                        VolManOperAcct, rcp->rcd_login);
                }
                setuid(0);
        }
}

  void
RCInitChan()
{
	int	i, j;

	if (DriveChannels != NULL) {
		nLogMsg(EL_STATUS1, 0, "re-initting drive channels");
		nFree(DriveChannels);
		nFree(ChannelUse);
	}
	TotalChannels = 0;
	DriveChannels = (int *) nAlloc(MAXDEVS * sizeof(int));
	for (i=0; i<MAXDEVS; i++)
		DriveChannels[i] = -1;
	for (i=0; i<MAXDEVS; i++) {
		if (DriveArray[i].de_rc != rcp->rcd_number)
			continue;
		if (DriveChannels[i] > -1)	/* already 'seen' */
			continue;
		/*
		 *  1st instance of channel - mark all w/ channel #
		 */
		DriveChannels[i] = TotalChannels;
		for (j=i+1; j<MAXDEVS; j++) {
			if (DriveArray[j].de_rc != rcp->rcd_number)
				continue;
			if (DriveChannels[j] > -1)	/* already 'seen' */
				continue;
			if (!strcmp(DriveArray[i].de_chan, 
				    DriveArray[j].de_chan))
				DriveChannels[j] = TotalChannels;
		}
		TotalChannels++;
	}
	/*
	 *  alloc/init counts of drives-per-channel in use
	 */
	ChannelUse = (int *) nAlloc(TotalChannels * sizeof(int));
	for (i=0; i<TotalChannels; i++)
		ChannelUse[i] = 0;
}

  void
RCConnectRP()
{
	int	flag;

	/*
	 *  start connection
	 */
	flag = 0;
	RPCLInit(M_RC, &flag);
	/*
	 *  get RC config
	 */
	RPCLRCConfig();
	/*
	 *  get drive config
	 */
	RPCLDriveConfig();
}

/* 
 *  RPRCReadyRP - tell RP we're ready for business and get response
 *	pass on rcp->rcd_freeslots
 */
  void
RCReadyRP(RC_t *rcp)
{
	char			*id = "RCReadyRP";
	char			hbuf[PKTSIZ(vmpHello_t)];
	vmpHdr_t		*hp, *rhp;
	vmpHello_t		*hellop;
	int			nextrec;

	nLogMsg(EL_STATUS1, 0, "Registering ready..");

	/*
	 *  build and send M_RCREADY message
	 */
	hp = vBuildHdr(M_RCREADY, NOSEQ, (vmpHdr_t *)hbuf);
	hp->vh_slot = -1;
	hellop = MSGPTR(hp, vmpHello_t);
	hellop->vhe_version = VM_VERSION;
	hellop->vhe_id = rcp->rcd_number;
	hellop->vhe_flag = rcp->rcd_freeslots;

	RPCLSend(hp, 1);
	nLogMsg(EL_DEBUG2, id, "sent M_RCREADY");

	/*
	 *  get reply, should be same packet
	 */
	while (1) {
		rhp = RPCLRecv(id);
		nextrec = RPCLNextRecord();
		switch (rhp->vh_type) {
		    case M_RCREADY:
			if (rhp->vh_slot != -1)
				nLogMsg(EL_FATAL, id, "bad slot (%d) / %s", 
						rhp->vh_slot, vHdrStr(rhp));
			nLogMsg(EL_STATUS1, 0, "Registered ready..");
			nFree(rhp);
			return; /* could return(nextrec); */
		    case M_DRIVEINFO:	/* response to startup-vary-off */
		    case M_TIMER:
		    case M_NOP:
			break;
		    default:
			nLogMsg(EL_FATAL, id, 
				"got wrong response %s", vHdrStr(rhp));
		}
		nFree(rhp);
	}

}

  void
RCSendMountDone(Seq_t seq, vsn_t *vsn, int drivebitno, int write_protected)
{
	char		mbuf[PKTSIZ(vmpVolStat_t)];
	vmpHdr_t	*hp;
	vmpVolStat_t	*vp;

	hp = vBuildHdr(M_VOLSTAT, seq, (vmpHdr_t *)mbuf);
	vp = MSGPTR(hp, vmpVolStat_t);
	vp->vv_vsn = *vsn;
	vp->vv_dev = drivebitno;
	vp->vv_state = VS_MOUNTED;
	if (write_protected)
		vp->vv_flags = 0;		/* no write-enable 'ring' */
	else
		vp->vv_flags = VV_RING;
	RPCLSend(hp, 1);
}

  void
RCSendUnmountDone(Seq_t seq, vsn_t *vsn)
{
	char		mbuf[PKTSIZ(vmpVolStat_t)];
	vmpHdr_t	*hp;
	vmpVolStat_t	*vp;

	hp = vBuildHdr(M_VOLSTAT, seq, (vmpHdr_t *)mbuf);
	vp = MSGPTR(hp, vmpVolStat_t);
	vp->vv_vsn = *vsn;
        vp->vv_state = VS_UNMOUNTED;
        vp->vv_flags = VV_LAST;
	vp->vv_dev = -1;
        RPCLSend(hp, 1);
}

  void
RCSendAccepted(vsn_t *vsn, RC_t *rcp)
{
	char		mbuf[PKTSIZ(vmpVolStat_t)];
	vmpHdr_t	*hp;
	vmpVolStat_t	*vp;

	hp = vBuildHdr(M_VOLSTAT, NOSEQ, (vmpHdr_t *)mbuf);
	vp = MSGPTR(hp, vmpVolStat_t);
	vp->vv_vsn = *vsn;
        vp->vv_state = VS_ACCEPTED;
        vp->vv_flags = VV_RC;
	vp->vv_rc = rcp->rcd_number;
	vp->vv_dev = -1;
        RPCLSend(hp, 1);
}

/* 
 *  SendEjected() - only used by acs, so VS_1MSGMOVE rather than VS_EJECTED
 */
  void
RCSendEjected(vsn_t *vsn, Seq_t seq)
{
	char		mbuf[PKTSIZ(vmpVolStat_t)];
	vmpHdr_t	*hp;
	vmpVolStat_t	*vp;

	hp = vBuildHdr(M_VOLSTAT, seq, (vmpHdr_t *)mbuf);
	vp = MSGPTR(hp, vmpVolStat_t);
	vp->vv_vsn = *vsn;
        vp->vv_state = VS_1MSGMOVE;
        vp->vv_flags = 0;
        vp->vv_rc = -1;			/* let vold look up the dest in dbase */
	vp->vv_dev = -1;
        RPCLSend(hp, 1);
}

  void
RCSendVolNode(int cmd, Seq_t seq, vsn_t *vsnp, vsn_t *labelp, 
		int flag, int drive, int host, int uid)
{
	char		buf[PKTSIZ(vmpNode_t)];
	vmpHdr_t	*hp;
	vmpNode_t	*np;

	hp = vBuildHdr(cmd, seq, (vmpHdr_t *)buf);
	np = MSGPTR(hp, vmpNode_t);

	np->vs_req = -1;	/* pass-thru-vold req from RC */
	/* 
	 *  (host=-1: send to any host)
	 */
	np->vs_host = host;
	np->vs_uid = uid;	

	np->vs_rc = rcp->rcd_number;
	np->vs_vsn = *vsnp;
	if (labelp == NULL)
		np->vs_label.v_vsn[0] = '\0';
	else
		np->vs_label = *labelp;
	np->vs_flag = flag;

	/*
	 *  (drive=-1: just remove /dev/vol/xxxxxx node - 
	 *	for mount cacheing)
	 */
	np->vs_drive = drive;
	RPCLSend(hp, 1);
}

  void
RCErr2RP(vsn_t *vsnp, Seq_t seq, int cmd, char retry)
{
	RPCLErr2RP(vsnp, seq, cmd, 0, retry);
}
