#ifndef lint
static char rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/rc/proto/protorc.c,v 1.1.1.1 2000-02-28 02:18:32 wrstuden Exp $";
#endif

/*
 *  protorc.c
 *
 * 	NAStore Volume Manager Repository Controller (RC) for manual tape 
 *	(3420 & 3480)
 *
 * The RC connects with the Volume Manager's Request Processor (RP) 
 * via the ipc protocol linked from libpvrl.a. After establishing a
 * connection, the RC opens for business by identifying itself as 
 * an RC to the RP, which may then send it requests. The RC communicates
 * with operators by a blast-to-uid-logins protocol, also from libpvrl.a.
 * Operators & other RCs communicate with the RC by M_RCMSG packets which 
 * are sent e.g. by the rcerr utility via the RP.
 *
 * The RC blocks on reading input from the RP.  It receives timer 
 * messages from the RP which inspire it to examine its queues to see 
 * if anything has changed. E.g. if testing a drive reveals
 * that a mount hasn't occurred, it will broadcast another message to
 * the operator console, or if the mount has happened, the label can be
 * checked and the RP informed of the completion.
 *
 * Bill Ross, NAS/NASA Ames, 8/18/89
 * Ported to Convex 1/26/93 -BR
 *
 */


#include <stdio.h>
#include <signal.h>
#include "cl_space.h"
#include "rp_volstat.h"
#include <fcntl.h>
#include <utmp.h>
#include <pwd.h>
#include <sys/file.h>
#include "systape.h"

#define LABELLEN 80
/*
 * WARNTIME defines the number of seconds before the operator is notified
 * about a pending tape mount.
 */
#define WARNTIME 60

#define TRUE	1
#define FALSE	0

#define IDLE	1
#define MOUNT	2
#define MOUNTED 3
#define DISMNT	4
#define EJECT	5
#define ACCEPT	6
#define NOTALLOC 7
#define DOWN	8
#define RETRIES 3

extern int	errno;
int		serrno;
extern struct passwd *getpwnam();

/*
 *  libpvrl.a stuff
 */

extern  DevEnt_t vDevice[MAXDEVS];
extern int	 vLogLevel;

/*
 *  repository controller info ptr, usable drive counter & status of rc login
 */

RepCon_t	*rcp;		
int		logged;
int		activerequests = 0;

char		*name = NULL;

/*
 * a doubly linked list of movnodes is kept for tracking move requests
 */

typedef struct movnode {
	struct movnode	*next;
	struct movnode	*prev;
	int	seq;
	vsn_t	vsn;
	char	rcname[20];
	short	src;
	short	dst;
	long	time;
	int 	status;
} movnode_t;

movnode_t	*movq;
movnode_t 	*findmov();

/*
 * an array of DriveEntries is kept to track mount requests and drive
 *	availability - NOTE that this parallels the vDevice[] array
 *	from libvol.
 */

typedef struct DriveEntry {
	int	fd;
	vsn_t	vsn, label;
	short	flags;
	int	den;
	int	seq;
	int 	status;
	long	time;
} DriveEntry_t;

DriveEntry_t	DriveArray[MAXDEVS];

int		lastdrv = 0,	/* last drive considered for rotating alloc */
		mindrv = MAXDEVS, 
		maxdrv = -1; 	/* min, max for this rc	*/


/*
 * exit routine: inform RP & die
 */

done()
{
	vm_term();
	exit(VMError);
}

/**************
   MAIN
***************/
main(argc, argv)
  char	*argv[];
{
	char		*id = "main";
	vmpHdr_t	*ihp;

	Init(argc, argv);
vLogMsg(EL_STATUS1, id, "Init ok");
vLogLevel = EL_DEBUG4;
	/*
	 * loop reading packets
	 */
	while (1) {
		vLogMsg(EL_DEBUG3, 0, "WAIT on rcv");
		ihp = lRcvFromRP(IPC_DELAY);
		if (ihp == NULL)
			vLogMsg(EL_FATAL|EL_RPCON, 0, "voldaemon died");
		procpkt(ihp);
		vFree(ihp);
		ihp = NULL;
		if (activerequests) {
			ProcMountQ();
			ProcMoveQ();
		}
		if (activerequests)
			if (!loggedin())
			  vLogMsg(EL_RPCON, 0, 
			  "Operator acct (%s) not logged in: requests waiting",
				rcp->rc_login);
	}
}

/*****************
  INITIALIZATION
******************/

Init(argc, argv)
  char **argv;
{

        if (argc != 2) {
		fprintf(stderr,"usage: %s rc\n",*argv);
		exit(1);
	}
	name = argv[0];

        signal(SIGCHLD,SIG_IGN);

        /*
         * read RC config info in RCCONF file, get pointer to it, use
         *      it to set login name to broadcast to.
         */

        vConfigRC();
        if ((rcp = vGetRcp(argv[1])) == NULL) {
                fprintf(stderr, "%s: can't get RC %s info", argv[0], argv[1]);
                exit(1);
        }
        vLogInit(argv[0], done, EL_TOFILE, "ErrorLog", rcp->rc_syslog);
        vLogLevel = EL_DEBUG2;
        vLogMsg(EL_STATUS1, 0, "==== %s here: rc=%s pid=%d ====",
          argv[0], rcp->rc_id, lPid);

	/*
	 * see if rc login exists
	 */

	if (!(logged = loggedin())) 
		vLogMsg(EL_RPCON, 0, 
			"%s: console %s not logged in", 
			rcp->rc_name, rcp->rc_login);

        /*
         *  Make sure we are ok to run as root, establish RP connection,
         *      init drive array & tell RP we're ready for business
         */

        CheckId(getuid(), rcp);
        if (_vm_init(argv[0], 0)) {
                vm_prerr(stderr);
                return;
        }
        InitDriveArray();
        RegisterRC(rcp);

	/*
	 *  change name of process
	 */

   	ChangeName(argv, rcp->rc_name);
vLogMsg(EL_STATUS1, 0, "ChangeName ok");
}

InitDriveArray()
{
	char 	*id = "InitDriveArray";
	int 	i;

	
        if (!rcp->rc_nconfigdev)
                vLogMsg(EL_STATUS1|EL_RPCON, 0, "NO DEVICES CONFIGURED");

        /*
         * initialize drive array based on available ones in configuration.
         *      mindrv = 1st drive for rc, 
	 *	maxdrv = last drv for rc, +1 for loop checking
         */


        for (i=0; i<MAXDEVS; i++) {
                if (DM_ISSET(i, rcp->rc_avail))  {
                        vLogMsg(EL_DEBUG2, id, "drive %d configured", i);
                        DriveArray[i].status = IDLE;
                        if (i > maxdrv)
                                maxdrv = i;
                        if (i < mindrv)
                                mindrv = i;
                } else
                        DriveArray[i].status = NOTALLOC;
        }
        vLogMsg(EL_DEBUG2, id, "min/max configured %d %d", mindrv, maxdrv);
        maxdrv++;
        lastdrv = mindrv - 1;
}

/*******************
    PACKET HANDLING
********************/

/*
 * procpkt - handle a packet from the RP
 */

procpkt(ihp)
vmpHdr_t        *ihp;
{
        char    *id = "procpkt";

        switch (ihp->vh_type) {
          case M_NOP:
          case M_TIMER:          /* just an excuse to check the queues */
	  case M_VARY:
                break;
          case M_ERROR: 
                MMError(ihp);
                break;
          case M_RCMOUNT: 
		if (MMMount(ihp))
			activerequests++;
		break;
          case M_UNMOUNT: 
		MMUnmount(ihp);
		break;
	  case M_MOVE: 
		activerequests += MMMove(ihp);
		break;
          case M_RCMSG: 
		MMRCMsg(ihp);
		break;
	  case M_VOLSTAT:
		MMVolstat(ihp);
		break;
	  case M_NEWLOG:
		/* assumes sharing voldaemon log, so doesn't rename */
		vLogInit("acsrc", done, EL_TOFILE, NULL, NULL);
		break;
          case M_SHUTDOWN:
                done();

          default:
                vLogMsg(EL_ERROR1, id, "%s: unexpected packet", vHdrType(ihp));
        }
}

/*
 * MError - handle error; either becoming an RC or cancelling a move
 */

MMError(ihp)
  vmpHdr_t        *ihp;
{
	vmpError_t      *vep;

        vep = MSGPTR(ihp, vmpError_t);
        vLogMsg(EL_ERROR1, 0, vep->vn_msg);
	if (moverr(ihp))
		vLogMsg(EL_LCLCON, 0, "ACS Enter failed on %.6s",&vep->vn_vsn);
	else
		vLogMsg(EL_ERROR1, 0, "Unexpected error pkt");
}

/*
 * MMount - set up mount request
 */

int
MMMount(ihp)
  vmpHdr_t        *ihp;
{
	char		*id = "MMMount";
	vmpRCMount_t      *mp;
	int             drive, start;
	int             tried = 0;
        int     	type;
	int		retries;
	char		vsn[8];

	/*
	 * got a mount request, alloc drive
	 */

	if (!rcp->rc_nconfigdev) {
		/*
	 	 * this process will never mount - no drives configured
	 	 */

		VMError = E_INTERNAL;
		vLogMsg(EL_CODE|EL_ERROR1, id, "no drives configured");
		Err2RP(ihp->vh_seq, 0);
		return(FALSE);
	}

	mp = MSGPTR(ihp, vmpRCMount_t);

vLogMsg(EL_DEBUG2, id, "label %.6s", &mp->vm_label);
vLogMsg(EL_DEBUG2, id, "drive req: %s", vDevMapStr(&mp->vm_devmap));
vLogMsg(EL_DEBUG2, id, "drive avail: %s", vDevMapStr(&rcp->rc_avail));
vLogMsg(EL_DEBUG2, id, "drive and: %s", vDevMapStr(vDMand(&rcp->rc_avail, 
&mp->vm_devmap)));

	drive = -1;
	start = (lastdrv + 1);
	if (start == maxdrv)
	start = mindrv;
	for (;;) {

		lastdrv++;
		if (lastdrv == maxdrv)
		lastdrv = mindrv;
		if (lastdrv == start) {
			if (tried)
				break;
			else
				tried++;
		}
		if (DriveArray[lastdrv].status != IDLE
				||  !DM_ISSET(lastdrv, mp->vm_devmap)) {
			vLogMsg(EL_DEBUG2,0,"rej: drv %d status %d",
                                    lastdrv, DriveArray[lastdrv].status);
			continue;
		}
		vLogMsg(EL_DEBUG2,0,"drv %d legal",lastdrv);

		for(retries=0; retries < RETRIES; retries++){

			/*
			 *  open for driver-level
			 *	queries of status & setting of state
			 */

			DriveArray[lastdrv].fd =
				ioctlopen(vDevice[lastdrv].de_devnam);
			if (DriveArray[lastdrv].fd == -1) {

				/*
				 *  Amdahl: we want to open the drive &
				 *	then test it periodically
				 */

				vLogMsg(EL_RPCON|EL_SYS, 0, 
					"open of drive %s failed",
                                        vDevice[lastdrv].de_devnam);
				varyoff(lastdrv);
				retries = RETRIES;
				continue;
			}

			/*
			 *  see if something mounted
			 */

			if (getstatus(DriveArray[lastdrv].fd) == -1) {
				vLogMsg(EL_RPCON|EL_SYS, 0,
                                         "getstatus for dev %s failed",
                                         vDevice[lastdrv].de_devnam);
				close(DriveArray[lastdrv].fd);
				varyoff(lastdrv);
				retries = RETRIES;
				continue;
			}
			if (ismounted()) {
				/*
                                 *  ClearDev closes the drive fd
                                 */
				vLogMsg(EL_RPCON, 0,
			    "idle dev %s has something mounted (??) unmounting",
                                vDevice[lastdrv].de_devnam);
				if(ClearDev(lastdrv, DriveArray[lastdrv].fd)) {
					varyoff(lastdrv); 
                                	DriveArray[lastdrv].fd = -1;
					retries = RETRIES;
					continue;
				}
                                DriveArray[lastdrv].fd = -1;
				continue;
			}
			break;		/* drive seems to be ok */
		}
		/*
		 * If the number of RETRIES was reached, the drive selected was
		 * no good. Inform the operator, and continue searching for
		 * another drive. 
		 */

		if(retries >= RETRIES){
			vLogMsg(EL_RPCON, 0, 
			   "Unable to mount on %s. Selecting another drive",
		  	vDevice[lastdrv].de_devnam);
			continue;
		}
		drive = lastdrv;
		break;
	}

	if (drive < 0) {
		/*
	 	 * couldn't find a good one
	 	 */

		VMError = E_SYS;
		vLogMsg(EL_CODE|EL_ERROR1, id, "no drive available");
		Err2RP(ihp->vh_seq, 0);
		return(FALSE);
	}
	switch (mp->vm_flags & VO_DENSITY) {
                  case VO_38000:
                        DriveArray[drive].den = BPI38k;
                        break;
                  case VO_6250:
                        DriveArray[drive].den = BPI6250;
                        break;
                  case VO_1600:
                        DriveArray[drive].den = BPI1600;
                        break;
                  case VO_800:
                        DriveArray[drive].den = BPI800;
                        break;
                  default:
                        vLogMsg(EL_CODE|EL_SYS, id, "Bad density %x",
                                                        mp->vm_flags);
                        VMError = E_SYS;
                        Err2RP(ihp->vh_seq, 0);
	}

	strcpy(vsn,"  ");
	strncat(vsn, (char *)&mp->vm_vsn, 6);
	if (TapeMsg(DriveArray[drive].fd,  vDevice[drive].de_medium,
				TAPLED_ALTERNATE, "  MOUNT ", vsn, 0) == -1)
		vLogMsg(EL_SYS|EL_ERROR1, 0, "ioctl tapled");
	vLogMsg(EL_DEBUG2, id, "setting to mount: %.*s label %.6s on drive %d",
                                VSNLEN, &mp->vm_vsn, &mp->vm_label, drive);
	DriveArray[drive].status = MOUNT;
	DriveArray[drive].time = 0;
	DriveArray[drive].vsn = mp->vm_vsn;
	DriveArray[drive].label = mp->vm_label;
	DriveArray[drive].flags = mp->vm_flags;
	DriveArray[drive].seq = ihp->vh_seq;
	return(TRUE);
}

/*
 * MUnmount - unload volume from drive
 */

MMUnmount(ihp)
  vmpHdr_t        *ihp;
{
	char		*id = "MMUnmount";
	vmpVsn_t      	*vp;
	int           	cancel = FALSE;

	vp = MSGPTR(ihp, vmpVsn_t);

	if (vp->v_flag == -1) {

                /*
                 *  this is a cancellation rather than a dismount; 
		 *	(i.e. there's an open file descriptor for drive)
                 *      figure out the drive
                 */

                int i;

                for (i=mindrv; (!SAMEVSN(&DriveArray[i].vsn, &vp->v_vsn)
                            && (i < maxdrv)); i++);
                if (i == maxdrv) {
                        vLogMsg(EL_CODE|EL_ERROR1, id,
                                  "vsn %.6s not mounted here",&vp->v_vsn);
                        Err2RP(ihp->vh_seq, 0);
                        return;
                }
                vp->v_flag = i;
                if (DriveArray[i].status == MOUNT) {
			vLogMsg(EL_DEBUG2, 0, 
				"cancelling LED msg for %.6s", &vp->v_vsn);
			/* reset led to hostname */
			if (TapeMsg(DriveArray[i].fd, vDevice[i].de_medium,
							0, NULL, NULL, 1) == -1)
				vLogMsg(EL_SYS|EL_ERROR1, 0, "ioctl tapled");
                        close(DriveArray[i].fd);
                        vLogMsg(EL_LCLCON, 0, "CANCEL MOUNT: %.6s", &vp->v_vsn);
                } else if (DriveArray[i].status == MOUNTED) {
                        vLogMsg(EL_STATUS1, id, "unloading %.6s on drive %s",
                                        &vp->v_vsn,
                                        &vDevice[i].de_devnam[DEVNAM]);
                        if (ClearDev(i, DriveArray[i].fd)) {
                                varyoff(i);
                                Err2RP(ihp->vh_seq, 0);
        			DriveArray[vp->v_flag].status = IDLE;
        			DriveArray[vp->v_flag].vsn.v_vsn[0] = 0;
        			DriveArray[vp->v_flag].time = 0;
                                return;
                        }
                } else {
                        vLogMsg(EL_ERROR1, id, "unexpected drive state %d",
                                         DriveArray[i].status);
                        Err2RP(ihp->vh_seq, 0);
                        return;
                }
	} else {

                /*
                 * dismount of completed mount (no file descriptor open)
                 * make sure drive is mounted with named vsn
                 */

                if (DriveArray[vp->v_flag].status != MOUNTED) {
			VMError = E_INTERNAL;
                        vLogMsg(EL_CODE|EL_ERROR1, id,
                                  "nothing mounted in drive %d", vp->v_flag);
                        Err2RP(ihp->vh_seq, 0);
                        return;
                }
                if (!SAMEVSN(&vp->v_vsn, &DriveArray[vp->v_flag].vsn)) {
			VMError = E_INTERNAL;
                        vLogMsg(EL_CODE|EL_ERROR1, id,"another vsn in drive %s",
                                        &vDevice[vp->v_flag].de_devnam[DEVNAM]);
                        Err2RP(ihp->vh_seq, 0);
                        return;
                }
                vLogMsg(EL_STATUS1, id, "unloading %.6s on drive %s",
                            &vp->v_vsn, &vDevice[vp->v_flag].de_devnam[DEVNAM]);

                if (ClearDev(vp->v_flag, -1)) {
                        varyoff(vp->v_flag);
                        Err2RP(ihp->vh_seq, 0);
        		DriveArray[vp->v_flag].status = IDLE;
        		DriveArray[vp->v_flag].vsn.v_vsn[0] = 0;
        		DriveArray[vp->v_flag].time = 0;
			return;
                }
	}

	/*
	 * clean up drive record & ack to RP
	 */

        DriveArray[vp->v_flag].status = IDLE;
        DriveArray[vp->v_flag].vsn.v_vsn[0] = 0;
        DriveArray[vp->v_flag].time = 0;

	if (!(vp->v_flag2 & VF_NORESP))
		SendUnmountDone(ihp->vh_seq, &vp->v_vsn);
}

/*
 * MMove - set up move request 
 */

int
MMMove(ihp)
  vmpHdr_t        *ihp;
{
	char		*id = "MMMove";
	vmpMove_t       *mvp;
	vsnseq_t	*vsnseqp;
	movnode_t       *mov;
	RepCon_t	*lrcp;		
	int		i, reqcount = 0, headererr = 0;
	
	mvp = MSGPTR(ihp, vmpMove_t);
	vLogMsg(EL_DEBUG2, 0, "Move from %d to %d", 
						mvp->vmv_srcrc,mvp->vmv_dstrc);
	/*
	 * is this a bad move, eject or accept for this RC?
	 */

	if (mvp->vmv_srcrc == mvp->vmv_dstrc) { 
		/*
		 *  bad move
		 */
		VMError = E_INTERNAL;
		vLogMsg(EL_INTERNAL|EL_CODE, id, 
					"%s: move to same location (%d)",
					rcp->rc_id, mvp->vmv_srcrc);
		headererr++;
		goto vsnloop;
	}
	if (rcp->rc_number != mvp->vmv_srcrc  
			&&  rcp->rc_number != mvp->vmv_dstrc) {
		/*
		 *  bad move
		 */
		VMError = E_INTERNAL;
		vLogMsg(EL_INTERNAL|EL_CODE, id, 
				"move doesn't involve %s: %d->%d",
				rcp->rc_id, mvp->vmv_srcrc, mvp->vmv_dstrc);
		headererr++;
		goto vsnloop;
	}
	if (mvp->vmv_srcrc == RC_ACS_SQR) {
		/* 
		 *  ACS eject to CAP will complete move so no operator action
		 *  req'd
		 */
		return(0);		
	}

	/*
	 *  OK.. put node(s) at head of movq
	 */
vsnloop:
	vsnseqp = (vsnseq_t *) &mvp->vmv_buf;
	for (i=0; i<mvp->vmv_nvsns; i++, vsnseqp++) {

		if (headererr) {
			Err2RP(vsnseqp->r_seq, 0);
			continue;
		}
		mov = (movnode_t *) vAlloc(sizeof(movnode_t));
		if (mov == NULL) {
			VMError = E_NOMEM;
			vLogMsg(EL_ERROR1|EL_CODE, id, "vAlloc error");
			Err2RP(vsnseqp->r_seq, 0);
			continue;
		}

		mov->next = movq;
		mov->prev = NULL;
		if (movq != NULL)
			movq->prev = mov;
		movq = mov;
		mov->seq = vsnseqp->r_seq;
		mov->vsn = vsnseqp->r_vsn;
		mov->src = mvp->vmv_srcrc;
		mov->dst = mvp->vmv_dstrc;
		mov->time = 0;
		if (mvp->vmv_srcrc == rcp->rc_number) {   /* eject */
			vLogMsg(EL_STATUS1, id, 
				"%.*s eject status", VSNLEN, &vsnseqp->r_vsn);
        		if ((lrcp = vRCInfo(mov->dst)) == NULL) 
				sprintf(mov->rcname, "RC # %d", mov->dst);
			else
				strcpy(mov->rcname, lrcp->rc_name);
			mov->status = EJECT;
		} else {
			vLogMsg(EL_STATUS1, id, 
				"%.*s accept status", VSNLEN, &vsnseqp->r_vsn);
        		if ((lrcp = vRCInfo(mov->src)) == NULL) 
				sprintf(mov->rcname, "RC # %d", mov->src);
			else
				strcpy(mov->rcname, lrcp->rc_name);
			mov->status = ACCEPT;
		}
		reqcount++;
	}
	return(reqcount);
}

/*
 * MMRCMsg - handle msg from operator
 */

MMRCMsg(ihp)
  vmpHdr_t        *ihp;
{
	char		*id = "MMRCMsg";
	vmpRcMsg_t      *rm;
	vmpHdr_t        *hp;
	
	rm = MSGPTR(ihp, vmpRcMsg_t);
	hp = (vmpHdr_t *) rm->rm_msg;
	vLogMsg(EL_DEBUG2, 0, "subpkt magic %c type %d", 
						hp->vh_magic, hp->vh_type);
	switch (hp->vh_type)
	{
		/* case M_VOLSTAT: 
			MMVolstat(hp);
			break; */
		case M_VALLOC:		/* arbitrary type for msg from ACS */
			vLogMsg(EL_LCLCON,0,
				"CAP FULL in LSM %d: PLEASE EMPTY",hp->vh_seq);
			break;
		case M_ERROR: {
			vmpError_t      *ep;

			ep = MSGPTR(hp, vmpError_t);
			if(moverr(hp)) {
				/*
				 * messages originating from the RP shouldn't
				 *	be forwarded back to the RP.  Only
				 *	messages from the rcerr program should
				 *	be forwarded.
				 */
				if (!(rm->rm_flag & RCM_NORPFORWARD))
					lSnd2RP(hp);
			} else if(!mounterr(hp))
				vLogMsg(EL_LCLCON,0,
                                          "rcerr rcvd for %.6s but no request",
                                          &ep->vn_vsn);
			break;
		}
		default:
			vLogMsg(EL_LCLCON,0, "unsupported RCMSG type %d",
                                                        hp->vh_type);
	}
}

MMVolstat(hp)
  vmpHdr_t        *hp;
{
	vmpVolStat_t	*vp;
	movnode_t	*mov;
	char		*id = "MMVolstat";

	vp = MSGPTR(hp, vmpVolStat_t);
	switch (vp->vv_state) {
		case VS_ACCEPTED:		
			/*
			 * this is a copy of the pkt sent to voldaemon by ACS
			 * so cart has arrived there: just cancel the req
			 */
			
			mov = findmov(&vp->vv_vsn);
			if (mov == NULL) {
				vLogMsg(EL_ERROR2, id, 
				    "No record of move (%.6s)", &vp->vv_vsn);
				return;
			}
			if (mov->src != rcp->rc_number)  
				vLogMsg(EL_ERROR2, id, "not source rc %.6s",
								&vp->vv_vsn);
			vLogMsg(EL_DEBUG2, 0, "ACS accepted %.6s", &vp->vv_vsn);
			activerequests--;
			remove_mv(mov); 
			break;
		default:
			vLogMsg(EL_DEBUG1, id, "unexpected state %d", 
						vp->vv_state);
	}
}
			
int
moverr(hp)
vmpHdr_t        *hp;
{
	vmpError_t      *vep;
	movnode_t	*mov;
	char		*id = "moverr";

	vep = MSGPTR(hp, vmpError_t);
	mov = findmov(&vep->vn_vsn); 
	if (mov == NULL) {
		vLogMsg(EL_DEBUG2, id, "No record of move (%.6s)",&vep->vn_vsn);
		return(0);
	}

	/*
	 * set up pkt in case it was in an RCMSG & needs to be sent to RP
	 */

	hp->vh_seq = mov->seq;
	hp->vh_key = lClientkey;
	
	remove_mv(mov);
	activerequests--;
	return(1);
}

int
mounterr(hp)
vmpHdr_t        *hp;
{
        vmpError_t      *ep;
        int             drv;

        ep = MSGPTR(hp, vmpError_t);

        /*
         * match an operator error message with the appropriate vsn
         */

        for (drv=mindrv; drv<maxdrv; drv++)
                if(SAMEVSN(&ep->vn_vsn, &DriveArray[drv].vsn))
                        break;
        if (drv == maxdrv)
                return(0);
        if (DriveArray[drv].status == MOUNT) {
		vLogMsg(EL_DEBUG2, 0, 
			"cancelling LED msg for %.6s", &ep->vn_vsn);
		/* reset led to hostname */
		if (TapeMsg(DriveArray[drv].fd, vDevice[drv].de_medium,
							0, NULL, NULL, 1) == -1)
			vLogMsg(EL_SYS|EL_ERROR1, 0, "ioctl tapled");
                close(DriveArray[drv].fd);
	}
        DriveArray[drv].status = IDLE;
        DriveArray[drv].vsn.v_vsn[0] = 0;
        if (ep->vn_error == E_UNAVAIL) {
                /*
                 * drive is out: vary it off to RP before sending error
                 *      on mount
                 */
                varyoff(drv);
        }
        hp->vh_key = lClientkey;
        hp->vh_seq = DriveArray[drv].seq;
        DM_SET(drv,ep->vn_dev);
        lSnd2RP(hp);
	activerequests--;
        return(1);
}

/**************************
  FOLLOW-UP: QUEUE HANDLING
***************************/



ProcMountQ()
{
	char		*id = "ProcMountQ";
	int 		i;

	/*
	 * first check all drives for outstanding requests
	 */

	for (i=mindrv; i<maxdrv; i++) {
		switch (DriveArray[i].status) {
		  case IDLE:
			vLogMsg(EL_DEBUG3,id,"drv %d idle",i);
			break;
		  case NOTALLOC:
		  case MOUNTED:
			break;
		  case MOUNT:
			if (CheckDrive(i))
				activerequests--;
			break;
		  default:
			vLogMsg(EL_DEBUG1, id, "not implemented: %d",
						DriveArray[i].status);
			break;
		}
	}
}

/*
 * CheckDrive - test drive to see if mount has occurred: TRUE if 
 *	request finished = success or failure
 */

int
CheckDrive(i)
{
	int		fd;
	char		msg[11];
	char 		*id = "CheckDrive";
	int		lbl = 0;
	
	vLogMsg(EL_DEBUG2, 0, "getting tapsense");
	if (getstatus(DriveArray[i].fd) == -1) {
		if (errno == ENXIO) {
			MountMsg(i);   	/* inform interested parties of mount */
			return(FALSE);
		} else {
			DriveArray[i].status = IDLE;
        		DriveArray[i].vsn.v_vsn[0] = 0;
        		DriveArray[i].time = 0;
			serrno = errno;
			close(DriveArray[i].fd);
			varyoff(i);
			VMError = E_SYS;
			errno = serrno;
			vLogMsg(EL_RPCON|EL_SYS, "Mount Error",
					"tapsense on drive %s",
					&vDevice[i].de_devnam[DEVNAM]);
			Err2RP(DriveArray[i].seq, 1);
			return(TRUE);
		}
	} 
	if (!ismounted()) {
		MountMsg(i);   	/* inform interested parties of mount */
		return(FALSE);
	}

	/*
	 *  mount has occurred
	 */

#ifndef AMDAHL_UTS
	fd = DriveArray[i].fd;
#else
	/*
	 *  ifdef Amdahl: open non-NDELAY for serious operations
	 */
			
	close(DriveArray[i].fd);
	vLogMsg(EL_DEBUG2, 0, "mounted, opening again");
	if ((fd = open(vDevice[i].de_devnam, 0)) == -1) {
		serrno = errno;
		DriveArray[i].status = IDLE;
        	DriveArray[i].vsn.v_vsn[0] = 0;
        	DriveArray[i].time = 0;
		varyoff(i);
		VMError = E_SYS;
		errno = serrno;
		vLogMsg(EL_CODE|EL_RPCON|E_SYS, 0, "Open error on drive %s",
					vDevice[i].de_devnam);
		Err2RP(DriveArray[i].seq, 1);
		return(TRUE);
	}
			
	/*
	 *  ifdef Amdahl: clear tape motion flags
	 */
			
	vLogMsg(EL_DEBUG2, 0, "clearing tapmotion");
	if (ioctl(fd, TAPMOTION,0) == -1) {
		serrno = errno;
		DriveArray[i].status = IDLE;
        	DriveArray[i].vsn.v_vsn[0] = 0;
		close(fd);
		rewunload(fd);
		varyoff(i);
		VMError = E_SYS;
		errno = serrno;
		vLogMsg(EL_CODE|EL_RPCON|EL_SYS, 0, 
					"ioctl/tapmotion error on drive %s",
					vDevice[i].de_devnam);
		Err2RP(DriveArray[i].seq, 1);
		return(TRUE);
	}

	/*
	 *  ifdef Amdahl: set the density if not 3480	
	 */
			
	if (DriveArray[i].den != BPI38k)
	if (ioctl(fd, TAPMODE, DriveArray[i].den) == -1) {
		serrno = errno;
		DriveArray[i].status = IDLE;
        	DriveArray[i].vsn.v_vsn[0] = 0;
		close(fd);
		rewunload(fd);
		varyoff(i);
		VMError = E_SYS;
		errno = serrno;
		vLogMsg(EL_CODE|EL_RPCON|EL_SYS, 0, 
					"ioctl/tapmode error on drive %s",
					vDevice[i].de_devnam);
		Err2RP(DriveArray[i].seq, 1);
		return(TRUE);
	}
#endif /* AMDAHL_UTS */

	/*
	 *  check physical protection
	 */

	if ((DriveArray[i].flags & VO_WRITE) && write_protected()) {

		/*
		 *  mounted for write && no write ring
		 */

		DriveArray[i].status = IDLE;
        	DriveArray[i].vsn.v_vsn[0] = 0;
		VMError = E_USER;
		vLogMsg(EL_CODE|EL_RPCON,0,"%s %s", 
					"Volume not physically write enabled",
					"- contact operator");
		Err2RP(DriveArray[i].seq, 0);
		DriveArray[i].status = IDLE;
		if (rewunload(fd) == -1) {
			vLogMsg(EL_RPCON|EL_SYS, 0, "rewunload err");
			close(fd);
			varyoff(i);
			return(TRUE);
		}
		close(fd);
		return(TRUE);
	}
	if (!(DriveArray[i].flags & VO_WRITE) && !write_protected()
				&& !(DriveArray[i].flags & VO_IGNORE_RING)) {
		/*
		 * mounted for read && physically writeable & no ignore-ring
		 */

		DriveArray[i].status = IDLE;
        	DriveArray[i].vsn.v_vsn[0] = 0;
		vLogMsg(EL_CODE|EL_RPCON,0,"%s %s", 
				"Volume physically write enabled",
				"but mounted for read - contact operator");
		VMError = E_USER;
		Err2RP(DriveArray[i].seq, 0);
		if (rewunload(fd) == -1) {
			vLogMsg(EL_RPCON|EL_SYS, 0, "rewunload err");
			close(fd);
			varyoff(i);
			return(TRUE);
		}
		close(fd);
		return(TRUE);
	}

	/*
	 * if bypass flag not set, check label
	 */

	if (!(DriveArray[i].flags & VO_BYPASS)) {
		vLogMsg(EL_DEBUG2, id, "checking label: %.6s", 
							&DriveArray[i].label);
		if(!LabelOk(fd, &DriveArray[i].label)) {
			vLogMsg(EL_DEBUG1, 0, "label not ok, unmounting");
			DriveArray[i].status = IDLE;
        		DriveArray[i].vsn.v_vsn[0] = 0;
			VMError = E_USER;
			Err2RP(DriveArray[i].seq, (VMError==E_SYS));
			if (rewunload(fd) == -1) {
				vLogMsg(EL_RPCON|EL_SYS, 0,
					    	"rewunload err");
				close(fd);
				varyoff(i);
				return(TRUE);
			}
			close(fd);
			return(TRUE);
		} else {
			vLogMsg(EL_DEBUG2, 0, "label ok");
#ifdef AMDAHL_UTS
			/*
			 *  ifdef Amdahl: note if labeled
			 */
			if (((char *)(&DriveArray[i].label))[0])
				lbl = LABELED;
#endif
		}
		vLogMsg(EL_DEBUG2,0,"label still ok");
	}
#ifdef AMDAHL_UTS
	/*
	 *  ifdef Amdahl: set default motion flags:
	 *	rewind-on-open, & label if labelled
	 */

	vLogMsg(EL_DEBUG2, 0, "setting default motion");
	if (ioctl(fd, TAPMOTION, REWOPEN | lbl) == -1) {
		vLogMsg(EL_CODE,0,"can't set motion flags: %d", errno);
		DriveArray[i].status = IDLE;
        	DriveArray[i].vsn.v_vsn[0] = 0;
		VMError = E_SYS;
		Err2RP(DriveArray[i].seq, 1);
		if (rewunload(fd) == -1) {
			vLogMsg(EL_RPCON|EL_SYS, 0, "rewunload err");
			close(fd);
			varyoff(i);
			return(TRUE);
		}
		close(fd);
		return(TRUE);
	}
#endif /* AMDAHL_UTS */
	if (DriveArray[i].den == BPI38k) {
		/* put vsn on LED */
		sprintf(msg, "  %.6s  ", &DriveArray[i].vsn);
		if (TapeMsg(fd, vDevice[i].de_medium, TAPLED_FIRST, 
						msg,msg,0) == -1)
			vLogMsg(EL_SYS|EL_ERROR1, id, "tapled");
	}

	/*
	 *  mount was successful
	 */
		
	close(fd);
	DriveArray[i].status = MOUNTED;
	vLogMsg(EL_STATUS1, id, "mounted %.6s on %s",
			   &DriveArray[i].vsn, vDevice[i].de_devnam);
	SendMountDone(DriveArray[i].seq, &DriveArray[i].vsn, i);
	return(TRUE);
}

ProcMoveQ()
{
	movnode_t	*mov;
	char		putmsg[800], getmsg[800];
	int		get = 0, put = 0;

	putmsg[0] = 0;
	getmsg[0] = 0;
	for (mov=movq; mov!=NULL; mov=mov->next)
	    if (mov->src == rcp->rc_number) {
		if (put > 110) {
			vLogMsg(EL_LCLCON, 0, "PUT in ACS: %s", putmsg);
			put = 0;
			putmsg[0] = 0;
		}
		if (!(put++ % 10))
			strcat(putmsg, "\n");
		else
			strcat(putmsg, " ");
		strncat(putmsg, (char *) &mov->vsn, VSNLEN);
	    } else {
		if (get > 110) {
			vLogMsg(EL_LCLCON, 0, "GET from ACS: %s", getmsg);
			get = 0;
			getmsg[0] = 0;
		}
		if(!(get++ % 10))
			strcat(getmsg, "\n");
		else
			strcat(getmsg, " ");
		strncat(getmsg, (char *) &mov->vsn, VSNLEN);
	    }
	if (put > 0) 
		vLogMsg(EL_LCLCON, 0, "PUT in ACS: %s", putmsg);
	if (get > 0) 
		vLogMsg(EL_LCLCON, 0, "GET from ACS: %s", getmsg);
}

/**********************
  GENERAL UTILITIES
***********************/

  movnode_t *
findmov(vsn)
  char *vsn;
{
	movnode_t *mov;

	for (mov=movq; mov!=NULL; mov=mov->next)
		if (SAMEVSN(vsn, &mov->vsn))
			return(mov);
	return(NULL);
}

/*
 *  remove_mv() - remove a move node
 */

remove_mv(mov)
  movnode_t *mov;
{
	char	*id = "remove_mv";

	if (mov == movq) {			/* 1st on q */
		movq = mov->next;
		if (movq != NULL) {
			movq->prev = NULL;
		}
	} else if (mov->next == NULL) { 	/* last on q, q>1 */
		mov->prev->next = NULL; 
	} else {				/* between nodes */
		mov->prev->next = mov->next;
		mov->next->prev = mov->prev;
	}
	vFree(mov);
	mov = NULL;
}



/*
 * CheckLogin - if operator isn't logged in, log msg on system console
 */

int
CheckLogin()
{
	if (loggedin()) {
		vLogMsg(EL_DEBUG3,0,"RC acct logged in");
		logged = TRUE;
		return(TRUE);
	}
	vLogMsg(EL_RPCON, 0, "operator not logged in");
	if (logged)
		logged = FALSE;
	/* if (activerequests) {
		FailMounts(); 
		if (rcp->rc_number == RC_VLT_SQR)
			FailMoves();
		activerequests = 0;
	} */
	return(FALSE);
}

FailMounts()
{
	int 	i;

	vLogMsg(EL_STATUS1,0,"failing mounts");
	if (!rcp->rc_nconfigdev)
		return;
	for (i=mindrv; i<maxdrv; i++) 
		if (DriveArray[i].status == MOUNT) {
			close(DriveArray[i].fd);
			DriveArray[i].status = IDLE;
			DriveArray[i].vsn.v_vsn[0] = 0;
			VMError = E_OPS;
			vLogMsg(EL_CODE, 0, "%s: not logged in", rcp->rc_name);
			Err2RP(DriveArray[i].seq, 0);
		}
}

FailMoves()
{
	movnode_t	*mvp;

	vLogMsg(EL_STATUS1,0,"Operator not logged in: failing moves");
	mvp = movq;
	while (mvp != NULL) {
		VMError = E_OPS;
		vLogMsg(EL_CODE, 0, "%s: not logged in", rcp->rc_name);
		Err2RP(mvp->seq, 0);
		remove_mv(mvp);
	}
}

/*
 *  Inform the interested parties that a mount message is outstanding 
 * 	and refresh the billboard just in case the message was lost.
 */
MountMsg(i)
int	i;
{
        char    msg[9];

        /*
         * Issue TAPLED ioctl with request volid
         */

        strcpy(msg,"  ");
        strncat(msg, (char *)&DriveArray[i].vsn, 6);
	if (TapeMsg(DriveArray[i].fd,  vDevice[i].de_medium,
				TAPLED_ALTERNATE, "  MOUNT ", msg, 0) == -1)
		vLogMsg(EL_SYS|EL_ERROR1, 0, "ioctl tapled");

	if( DriveArray[i].time > (time ( (long *) 0)- WARNTIME) )
		return(0);
		
	vLogMsg(EL_LCLCON,0, "MOUNT %.6s on drive %s",
		&DriveArray[i].vsn, &vDevice[i].de_devnam[DEVNAM]);
	DriveArray[i].time = time ( (long *) 0);
}


/*
 * loggedin - is RC anyone logged in under console login?
 */

 int
loggedin()
{
	char		*id = "loggedin";
	struct utmp 	*utmp, *getutent();
	void		setutent();

	setutent();
	for (;;) {
		utmp = getutent();
		if (utmp == NULL) {
			if (errno != ENOTTY &&  errno != 0)  
				/* enotty if system coming up */
				vLogMsg(EL_ERROR1|E_SYS, id, "getutent error"); 
			return(0);
		}
#ifdef AMDAHL_UTS
		if (utmp->ut_type == USER_PROCESS) 
#endif
			if (!strncmp(utmp->ut_name, rcp->rc_login,
			  sizeof (utmp->ut_name)))
				return(1);
	}
}

