#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>

#include "vmerror.h"
#include "commline.h"

extern	int scannerfd;

int	scanner_child;

 void
HangupLine()
{
	char	*id = "HangupLine";
	int	modem = TIOCM_DTR;
	/*
	 * turn off dtr
	 */
	if (scannerfd != -1) {
		IOCTL(scannerfd, TIOCMBIC, &modem);
		IOCTL(scannerfd, TCFLSH, TCIOFLUSH);
		close(scannerfd);
		scannerfd = -1;
	}
}

 static
 void
hangup(sig)
 int	sig;
{
	vLogMsg(EL_ERROR1, NULL, "signal %d received by child", sig);
	HangupLine();
	exit(1);
}

 void
PrepLine()
{
	char	*id = "PrepLine";
	int	off = 0;
	int	modem;
	
	if (( scannerfd = open(port, O_RDWR|O_NDELAY)) == -1) {
		vLogMsg(EL_SYS|EL_ERROR1, id, "open %s", port);
		return;
	}
		
	if ((scanner_child = fork()) == 0) {	/* child process */
		signal(SIGINT, hangup);
		signal(SIGQUIT, hangup);
		signal(SIGTERM, hangup);
		signal(SIGHUP, hangup);

		/*
		 * turn off software carrier
		 */
		IOCTL(scannerfd, TIOCSSOFTCAR, &off);

		/*
		 * turn on dtr and rts
		 */
		modem = TIOCM_RTS|TIOCM_DTR;
		IOCTL(scannerfd, TIOCMBIS, &modem);

		/*
		 * close line to wait for real open
		 */
		close(scannerfd);

		vLogMsg(EL_DEBUG1, id, "awaiting open");
		scannerfd = open(port, O_RDWR);
		vLogMsg(EL_DEBUG1, id, "open returned");
		exit(0);
	}
}

 void
SetupComm()
{
	char	*id = "SetupComm";
	struct	termios		term;
	int	modem;

	/*
	 * set-up terminal line
	 */
	IOCTL(scannerfd, TCGETS, &term);
	term.c_iflag = IGNBRK;
	term.c_oflag = 0;
	term.c_cflag = B9600|CS8|CREAD;
	term.c_lflag = 0;
	term.c_cc[VMIN] = 0;		/* get this number of characters */
	term.c_cc[VTIME] = 1;		/* wait in 1/10 second units */
	IOCTL(scannerfd, TCSETS, &term);
	
	/*
	 * turn off rts
	 */
	modem = TIOCM_RTS;
	IOCTL(scannerfd, TIOCMBIC, &modem);
}


/*
 * Build a packet - return the size of the packet built
 */
 int
BuildPacket(out, mess, mlen)
 char	*out;		/* create packet here */
 char	*mess;		/* this is the message */
 int	mlen;		/* this is the length of the message */
{
	register ushort chksm;	/* working copy of checksum */
	register unsigned char c;
	unsigned int i, plen;
	char *p;
	
	chksm = 0;
	plen = mlen+HDR_SIZE+LEN_SIZE+CRC_SIZE;
	
	c = out[0] = SYN;
	c &= 0xff;
	chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
	
	c = out[1] = ((plen >>8) & 0xff);
	c &= 0xff;
	chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
	
	c = out[2] = (plen & 0xff);
	c &= 0xff;
	chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
	
	p = mess;
	for (i = 0; i < mlen; i++) {
		c = out[(MSG_OFFSET+i)] = *p++;
		c &= 0xff;
		chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
	}
	chksm &= 0xffff;
	
	out[(MSG_OFFSET+mlen)] = ((chksm >> 8) & 0xff);	/* MSB */
	out[(MSG_OFFSET+mlen+1)] = (chksm & 0xff);   	/* LSB */
	
	return(plen);
}

/*
**	SendPacket
*/
 void
SendPacket(mess, len)
 char	*mess;
 int	len;
{
	char	*id = "SendPacket";
	char	packet[MAXCOMM];
	int	i;
	int	ret = 1;

	if (scannerfd == -1) {
		vLogMsg(EL_ERROR1, id,
			"attempt to write when line is not open");
		PollLoop();
		return;
	}

	if (len > 1)
		len = BuildPacket(packet, mess, len);
	else
		packet[0] = mess[0];

	IOCTL(scannerfd, TCFLSH, TCIOFLUSH);
	if (( i = write(scannerfd, packet, len)) == -1)
		vLogMsg(EL_SYS|EL_ERROR1, id, "scanner write");
	else if (i < len)
		vLogMsg(EL_ERROR1, id, "short write %d", i);
	else
		ret = 0;

	if (ret)
		DeadScanner();
}

 int
ReadComm(buf, size)
 char	*buf;
 int	size;
{
	char	*id = "ReadComm";
	int	got = 0;
	int	i;

	if (scannerfd == -1)
		return(-1);

	while ((i = read(scannerfd, buf, size-got)) > 0) {
		buf += i;
		got += i;
		if (got >= size)
			break;
	}
	if (i == -1) {
		vLogMsg(EL_SYS|EL_ERROR1, id, "read error");
		return(i);
	}
	else if (got == 0)
		vLogMsg(EL_ERROR1, id, "zero length read");
	else
		vLogMsg(EL_DEBUG2, id, "read %d chars", got);
	return(got);
}

 int
GoodPacket(bufp, len)
 char	**bufp;
 int	*len;
{
	char	*id = "GoodPacket";
	char	*packet;
	u_char	c;
	int	ret = 0;
	int	i, hold, start = 0;
	ushort	plength, chksm;
	union	{
		u_short	i;
		u_char	c[2];
	} ctos;

	packet = *bufp;
	*bufp = NULL;

	for (start=0;;start=hold+1) {
		for (i=start; i < *len; i++)	/* look for start char */
			if (packet[i] == SYN)
				break;
		if ((*len - i) < ZERO_PACKET) {	/* not enough for a real pkt */
			vLogMsg(EL_ERROR1, id,
				"no SYN found or short pkt");
			break;
		}
		hold = i++;
		ctos.c[1] = packet[i++];
		ctos.c[0] = packet[i];
		plength = ctos.i;

		vLogMsg(EL_DEBUG5, id, "packet len %d", plength);
		if ((plength < ZERO_PACKET) ||
		    ((*len - start) < plength)) {
			vLogMsg(EL_ERROR1, id,
				"bad packet length %d data read %d",
			       plength, *len - start);
			continue;
		}

                /*
                 * compute CRC on local packet
                 */
                chksm = 0;
                for (i=hold; i < plength - CRC_SIZE; i++) {
			c = packet[i] & 0xff;
			chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
                }
                chksm &= 0xffff;
		ctos.c[1] = packet[i++];
		ctos.c[0] = packet[i];
		if (chksm != ctos.i) {
			vLogMsg(EL_ERROR1, id, "CRC bad %04.4x %04.4x",
				chksm, ctos.i);
			continue;
		}

		/*
		**  We made it!  A good packet is here.
		*/
		ret = 1;
		*len = plength - ZERO_PACKET;
		*bufp = *len ? &packet[hold+MSG_OFFSET] : NULL;
		vLogMsg(EL_DEBUG2, id, "ok: (%d) %.*s",
			*len, *len - 1, &packet[hold+MSG_OFFSET]);
		break;
	}
	return ret;
}		

/*
**	Check a comm character to see if it requires resend.
**	Return 1 if it does, 0 otherwise.
*/
 int
CheckChar(ch)
 char	ch;
{
	switch (ch) {	/* see what it is */
	case ACK:
	case EOT:
		return(0);
	case ETX:
		vLogMsg(EL_STATUS1, NULL,
			"battery low indication");
		PollLoop();
		return(1);
	default:
		return(1);
	}
}

 void
SendAck()
{
	char	ch = ACK;

	vLogMsg(EL_DEBUG2, NULL, "send ACK");
	SendPacket(&ch, 1);
}

 void
SendEnquire()
{
	char	ch = ENQ;

	vLogMsg(EL_DEBUG2, NULL, "send ENQ");
	SendPacket(&ch, 1);
}

/*
**	GetPacket
**	Read comm line and crack a packet.  Return 1 on error, 0 if ok.
**	The first argument is a pointer char pointer which will be set
**	to the packet read
*/
 int
GetPacket(bufp, len)
 char	**bufp;
 int	*len;
{
	char	*id = "GetPacket";
	int	i, retval = 1;
	static	char	inbuf[MAXCOMM];
	static	int	got;

	vLogMsg(EL_DEBUG3, id, "entry");

	*bufp = NULL;
	*len = 0;
	
	got = ReadComm(inbuf, MAXCOMM);
	switch (got) {

	case -1:		/* we were told we could read */
	case 0:			/* something by select -- ignore */
		return(retval);

	case 1:				/* single char read */
		switch (inbuf[0]) {	/* see what it is */
		case ACK:
		case ETX:
		case NAK:
		case ENQ:
		case EOT:
		case RVI:
			vLogMsg(EL_DEBUG2, id,
				"got single char 0x%x", inbuf[0]);
			retval = 0;
			*bufp = inbuf;
			*len = got;
			break;
		default:
			vLogMsg(EL_ERROR1,
				"bad single char %d", inbuf[0]);
			break;
		}
		break;

	default:
		*len = got;
		*bufp = inbuf;
		if (GoodPacket(bufp, len))
			retval = 0;
		break;
	}

	if (retval) {
		vLogMsg(EL_STATUS1, id, "send NAK");
		inbuf[0] = NAK;
		SendPacket(inbuf, 1);
	}
	
	return(retval);
}
