#include	<stdio.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<sys/time.h>
#include	<sys/param.h>

#include	"vmerror.h"
#include	"vconfig.h"
#include	"sysfile.h"
#include	"bplus.h"
#include	"vltlib.h"

int		db_vsn_fd = -1;
IX_DESC		db_vsn_idx, db_loc_idx;		/* unique key index */
ENTRY		db_vsn_entry, db_loc_entry;
OFF_T		vsnaddr;

/*
 *  toInt() - check & translate char -> int
 */

 static int
toInt(num, str)
 int	*num;
 char	*str;
{
	char	*id = "db to integer";
	int     i;

	if (str[INTLEN] != ' ') {
		vLogMsg(EL_ERROR1, id,
			"int field discrepancy (%.*s)",
			INTLEN+1, str);
		return(1);
	}
	for (i=0; i<INTLEN; i++)
		if (str[i] != ' '  &&  !isdigit(str[i])  &&  str[i] != '-') {
			vLogMsg(EL_ERROR1, id, 
				"int field (%.*s) has non-numeral",
				INTLEN, str);
			return(1);
		}
	*num = atoi(str);
	return(0);
}

CloseDB()
{
	/*
	 *  release lock
	 */

	if (unlink(DB_LOCK) == -1)
		vLogMsg(EL_ERROR1, NULL,
			"unlink lock file (%s)", DB_LOCK);

	vLogMsg(EL_STATUS1, NULL, "Database shut down");
}

die()
{
	CloseDB();
	exit(1);
}
	

 void
TruncStr(str, len)
 char	*str;
 int	len;
{
	int	i;

	for (i=len-1; i>=0; i--)
		if (str[i] != ' ')
			break;
	i++;
	if ((0 <= i) && (i < len) &&		/* in range */
	    str[i] == ' ')			/* and is last blank */
		str[i] = '\0';
}

main(argc, argv)
 int	argc;
 char	*argv[];
{
	struct	stat	sbuf;
	extern	int	vLogLevel;
	vsnrec_t	vsnbuf;
	locrec_t	*lrp;
	vaultrec_t	*vrp;
	int		i, lastloc, vrow, vslot;

	vLogInit(argv[0], die, EL_TOSTDERR, NULL, NULL);
	vLogLevel = EL_DEBUG4;
	if (stat(DB_LOCK, &sbuf) == 0) {
		vLogMsg(EL_ERROR1, NULL, "db in use");
		exit(1);
	}

	signal(SIGINT, die);
	signal(SIGQUIT, die);
	signal(SIGTERM, die);

	/*
	 *  db about to be set up, so lock
	 */
	if (open(DB_LOCK, O_CREAT|O_EXCL, 0600) == -1)
		vLogMsg(EL_FATAL|EL_SYS, NULL, "create lock failed");

	if ((db_vsn_fd = open(DB_VSN_TBL, O_RDONLY))== -1) {
		vLogMsg(EL_FATAL|EL_SYS, NULL,
			"vsn table (%s) open failed: %s", DB_VSN_TBL);
	}

	if (open_index(DB_VSN_IDX, &db_vsn_idx, 0) != IX_OK) {
		vLogMsg(EL_FATAL, NULL,
			"could not start index %s", DB_VSN_IDX);
	}
	if (open_index(DB_LOC_IDX, &db_loc_idx, 0) != IX_OK) {
		vLogMsg(EL_FATAL, NULL,
			"could not start index %s", DB_LOC_IDX);
	}
	for (vsnaddr=0;;vsnaddr+=sizeof(vsnrec_t)) {
		if (read(db_vsn_fd, &vsnbuf, sizeof(vsnrec_t)) !=
		    sizeof(vsnrec_t))
			break;

		/*
		 *  basic record sanity check
		 */
		if (vsnbuf.end != '\n') {
			vLogMsg(EL_ERROR1, NULL, 
				"error on (%.6s): bad record terminator", 
				vsnbuf.v_vsn);
			continue;
		}
		
		/*
		 *  see if record is really there
		 */
		if (vsnbuf.v_status == S_DELETED)
			continue;
		
		
		if (vsnbuf.v_vsn[VSNLEN] != ' ') {
			vLogMsg(EL_ERROR1, NULL,
				"%.7s: no space seperator after vsn", 
				vsnbuf.v_vsn);
			continue;
		}
		for (i=0; i<VSNLEN; i++) {
			if (vsnbuf.v_vsn[i] == ' ') {
				vLogMsg(EL_ERROR1, NULL,
					"vsn %.6s: too short", 
					vsnbuf.v_vsn);
				continue;
			}
			if (!isalnum(vsnbuf.v_vsn[i])) {
				vLogMsg(EL_ERROR1, NULL,
					"vsn %.6s: Illegal character (%c)",
					vsnbuf.v_vsn, vsnbuf.v_vsn[i]);
				continue;
			}
		}

		/*
		 *  check to make sure last known location is ok
		 */
		
		lastloc = vsnbuf.v_lastloc - '0';
		if (lastloc < 0 || HISTORY <= lastloc) {
			vLogMsg(EL_ERROR1, NULL, 
				"%.6s: bad location '%c' in history array",
				vsnbuf.v_vsn, vsnbuf.v_lastloc);
			continue;
		}

		clearkey(&db_vsn_entry);
		sprintf(db_vsn_entry.key, "%.6s", vsnbuf.v_vsn);
		if (find_key(&db_vsn_entry, &db_vsn_idx) != IX_OK) {
			vLogMsg(EL_STATUS1|EL_CODE, NULL,
				"volser (%.*s) not found",
				VSNLEN, vsnbuf.v_vsn);
			continue;
		}
		if (db_vsn_entry.recptr != vsnaddr) {
			vLogMsg(EL_ERROR1, NULL,
				"volser (%.*s) out of place",
				VSNLEN, vsnbuf.v_vsn);
			continue;
		}

		lrp = &vsnbuf.v_loc[lastloc];
		if (lrp->l_type != VAULT)
			continue;

		vrp = &lrp->l_field.vu_vault;
		clearkey(&db_loc_entry);
		sprintf(db_loc_entry.key, "%.*s",
			sizeof(vaultrec_t), (char *)vrp);

		TruncStr(vrp->vr_section, SECSIZE);
		if (toInt(&vrow, vrp->vr_row) ||
		    toInt(&vslot, vrp->vr_slot)) {
			vLogMsg(EL_ERROR1, NULL,
				"%.*s: dbase translation error",
				VSNLEN, vsnbuf.v_vsn);
			continue;
		}

		if (find_key(&db_loc_entry, &db_loc_idx) != IX_OK) {
			vLogMsg(EL_STATUS1|EL_CODE, NULL,
				"volser (%.*s) location %s %d %d not found",
				VSNLEN, vsnbuf.v_vsn,
				vrp->vr_section, vrow, vslot);
			continue;
		}
		if (db_loc_entry.recptr != vsnaddr) {
			vLogMsg(EL_ERROR1, NULL,
				"volser (%.*s) location %s %d %d out of place",
				VSNLEN, vsnbuf.v_vsn,
				vrp->vr_section, vrow, vslot);
			continue;
		}
		vLogMsg(EL_DEBUG5, NULL, "volser (%.*s) ok",
			VSNLEN, vsnbuf.v_vsn);
	}
	CloseDB();
	return 0;
}
