/*
 * COMM.C 
 * Version 1.1 Beta 7/16/93
 * J. Musch
 */

#include "tapelib.h"
#ifdef DOS
#include <telxon.h>
#else
#include "telxon.h"
#endif
#include <stdio.h>
#include <time.h>
#include <string.h>

typedef unsigned int ushort;

#define MAXPACKET 512

char read_que[MAXPACKET];

extern struct tapes tapes;

int comm_port = COM1;

comm() {
    int save_val;
    
    setpage(WIN_COMM);
    settimeout(TIMEOUT_COMM);
    
    save_val = getlowbatlen();
    setlowbatlen(10*20);
    
    printf("COMM\n");
    
    if (comm_init() == 0)
        proto();

    recvintrctrl(comm_port, TELX_OFF);	/* Disable receive interrupts */
    dtrctrl (comm_port, TELX_OFF);	/* Disable DTR */
    rtsctrl (comm_port, TELX_OFF);	/* Disable RTS */

    setlowbatlen(save_val);
    commpwroff();
}

int comm_init()
{
    int comm_mode, baud, type;

    type = INTR ;	   /* Initialize Communications for Interrupts */
    comm_mode = DATA_8 | STOP_1 | NO_PARITY ;
    baud = BAUD_9600;

    if (comminit(comm_port, comm_mode, baud, type) == TELX_ERR){
	printf("comminit error\n");
	return(-1);
    }

    if (inituserque(comm_port, RECV, read_que, MAXPACKET)  == TELX_ERR){
	printf("read_que init error\n");
	return(-1);
    }

    dtrctrl (comm_port, TELX_ON);	/* Enable DTR */
    recvintrctrl(comm_port, TELX_ON);	/* Enable receive interrupts */
    return(0);
}


comm_gets(ptr, cnt, timeout)
char *ptr;
int cnt;
int timeout;
{
    int i = 0;
    unsigned long time;
    char ch;

    if (cnt <= 0)
	return(0);
        
    for (time = 0; timeout == 0 || time < 10*timeout ; ) {

	switch ( deque(&ch)) {

	case RECV_OK:
	    *ptr++ = ch;
	    if (++i >= cnt)
		return(i);
	    continue;

	case RECV_QUE_EMPTY:
	    break;

	case LOST_DSR:
	    printf("HANGUP\n");
	    return(-1);

	case RECV_QUE_OVR:
	    printf("Rec Q Overflow\n");
	    return(-1);
		
	case RECV_ERR:
	    pstat((unsigned int)commstatus(comm_port));
	    return(-1);
	}
	if (timeout == 0)
		break;
	sysdelay(5);	/* 1/10 sec delay */
	time++;
    }
    printf("Timeout: characters received = %d\n", i);
    return(i);
}


comm_puts(ptr, cnt)
char *ptr;
int cnt;
{
    unsigned int status;
    int i = 0;
    char ch;

    if (cnt <= 0)
	return(0);

    /*
     * Before sending data, make sure the input queue is empty.
     * This helps preserve the handshaking.
     */
    while((status = deque(&ch)) != RECV_QUE_EMPTY) {
	switch (status) {

	case RECV_OK:
	    continue;

	case LOST_DSR:
	    printf("HANGUP\n");
	    return(-1);

	case RECV_QUE_OVR:
	    printf("Rec Q Overflow\n");
	    return(-1);
		
	case RECV_ERR:
	    pstat((unsigned int)commstatus(comm_port));
	    return(-1);
	}
    }

    /*
     * Send the data
     */
    for (i = 0; i < cnt; i++) {
	ch = *ptr++;

	switch(status = enque(comm_port, ch)) {

	case SEND_OK:
	    continue;

	case LOST_DSR:
	    printf("HANGUP\n");
	    return(-1);

	case TELX_ERR:
	    pstat((unsigned int)commstatus(comm_port));
	    return(-1);
	}
    }
    return(0);
}

pstat(stat)
unsigned int stat;
{
    if ((stat&PARITY_ERR) == PARITY_ERR)
    	printf("parity error ");		
    if ((stat&FRAMING_ERR) == FRAMING_ERR)		
    	printf("framing error ");
    if ((stat&OVRRUN_ERR) == OVRRUN_ERR)		
    	printf("overrun error ");
    printf("port errors = %04x\n", stat);
}



/*
 *   CRC-16 constants.  From Usenet contribution by Mark G. Mendel, 
 *   Network Systems Corp.  (ihnp4!umn-cs!hyper!mark)
 */
#define	P	0x1021	/* the CRC polynomial. */
#define W	16	/* number of bits in CRC */
#define B	8	/* the number of bits per char */


/*   CRC-16 constant array...
 *   from Usenet contribution by Mark G. Mendel, Network Systems Corp.
 *   (ihnp4!umn-cs!hyper!mark)
 */

/* crctab as calculated by initcrctab() */
ushort crctab[1<<B] = {
    0x0000,  0x1021,  0x2042,  0x3063,  0x4084,  0x50a5,  0x60c6,  0x70e7,
        0x8108,  0x9129,  0xa14a,  0xb16b,  0xc18c,  0xd1ad,  0xe1ce,  0xf1ef,
        0x1231,  0x0210,  0x3273,  0x2252,  0x52b5,  0x4294,  0x72f7,  0x62d6,
        0x9339,  0x8318,  0xb37b,  0xa35a,  0xd3bd,  0xc39c,  0xf3ff,  0xe3de,
        0x2462,  0x3443,  0x0420,  0x1401,  0x64e6,  0x74c7,  0x44a4,  0x5485,
        0xa56a,  0xb54b,  0x8528,  0x9509,  0xe5ee,  0xf5cf,  0xc5ac,  0xd58d,
        0x3653,  0x2672,  0x1611,  0x0630,  0x76d7,  0x66f6,  0x5695,  0x46b4,
        0xb75b,  0xa77a,  0x9719,  0x8738,  0xf7df,  0xe7fe,  0xd79d,  0xc7bc,
        0x48c4,  0x58e5,  0x6886,  0x78a7,  0x0840,  0x1861,  0x2802,  0x3823,
        0xc9cc,  0xd9ed,  0xe98e,  0xf9af,  0x8948,  0x9969,  0xa90a,  0xb92b,
        0x5af5,  0x4ad4,  0x7ab7,  0x6a96,  0x1a71,  0x0a50,  0x3a33,  0x2a12,
        0xdbfd,  0xcbdc,  0xfbbf,  0xeb9e,  0x9b79,  0x8b58,  0xbb3b,  0xab1a,
        0x6ca6,  0x7c87,  0x4ce4,  0x5cc5,  0x2c22,  0x3c03,  0x0c60,  0x1c41,
        0xedae,  0xfd8f,  0xcdec,  0xddcd,  0xad2a,  0xbd0b,  0x8d68,  0x9d49,
        0x7e97,  0x6eb6,  0x5ed5,  0x4ef4,  0x3e13,  0x2e32,  0x1e51,  0x0e70,
        0xff9f,  0xefbe,  0xdfdd,  0xcffc,  0xbf1b,  0xaf3a,  0x9f59,  0x8f78,
        0x9188,  0x81a9,  0xb1ca,  0xa1eb,  0xd10c,  0xc12d,  0xf14e,  0xe16f,
        0x1080,  0x00a1,  0x30c2,  0x20e3,  0x5004,  0x4025,  0x7046,  0x6067,
        0x83b9,  0x9398,  0xa3fb,  0xb3da,  0xc33d,  0xd31c,  0xe37f,  0xf35e,
        0x02b1,  0x1290,  0x22f3,  0x32d2,  0x4235,  0x5214,  0x6277,  0x7256,
        0xb5ea,  0xa5cb,  0x95a8,  0x8589,  0xf56e,  0xe54f,  0xd52c,  0xc50d,
        0x34e2,  0x24c3,  0x14a0,  0x0481,  0x7466,  0x6447,  0x5424,  0x4405,
        0xa7db,  0xb7fa,  0x8799,  0x97b8,  0xe75f,  0xf77e,  0xc71d,  0xd73c,
        0x26d3,  0x36f2,  0x0691,  0x16b0,  0x6657,  0x7676,  0x4615,  0x5634,
        0xd94c,  0xc96d,  0xf90e,  0xe92f,  0x99c8,  0x89e9,  0xb98a,  0xa9ab,
        0x5844,  0x4865,  0x7806,  0x6827,  0x18c0,  0x08e1,  0x3882,  0x28a3,
        0xcb7d,  0xdb5c,  0xeb3f,  0xfb1e,  0x8bf9,  0x9bd8,  0xabbb,  0xbb9a,
        0x4a75,  0x5a54,  0x6a37,  0x7a16,  0x0af1,  0x1ad0,  0x2ab3,  0x3a92,
        0xfd2e,  0xed0f,  0xdd6c,  0xcd4d,  0xbdaa,  0xad8b,  0x9de8,  0x8dc9,
        0x7c26,  0x6c07,  0x5c64,  0x4c45,  0x3ca2,  0x2c83,  0x1ce0,  0x0cc1,
        0xef1f,  0xff3e,  0xcf5d,  0xdf7c,  0xaf9b,  0xbfba,  0x8fd9,  0x9ff8,
        0x6e17,  0x7e36,  0x4e55,  0x5e74,  0x2e93,  0x3eb2,  0x0ed1,  0x1ef0
};


/*  ASCII Constants  */
#define SOH  	0x01 
#define STX	0x02
#define ETX	0x03
#define EOT     0x04
#define ENQ     0x05
#define ACK     0x06
#define RVI     0x11
#define NAK     0x15
#define SYN     0x16
#define CAN	0x18

#define RETRIES 10
#define TIMEOUT 60

#define HDR_OFFSET 0
#define HDR_SIZE   ( sizeof (unsigned char) )
#define LEN_OFFSET ( HDR_OFFSET + HDR_SIZE )
#define LEN_SIZE   ( sizeof ( ushort ) )
#define MSG_OFFSET ( LEN_OFFSET + LEN_SIZE )

#define CRC_SIZE (W/B)

char in_packet[MAXPACKET+1];
char out_packet[MAXPACKET+1] = { ACK };

comm_getpkt(mbuf, mlen)
char **mbuf;
int *mlen;
{

    unsigned char c;
    ushort chksm, pcrc;
    int i, j, msglen, plength;
    int ret;

    *mbuf = NULL;
    *mlen = 0;
    for (i = 0; i < RETRIES; i++) {

	if ((ret = comm_gets(&in_packet[HDR_OFFSET], 
		(int)HDR_SIZE, TIMEOUT)) == HDR_SIZE) {
	    switch (in_packet[HDR_OFFSET]) {
	    case ACK :
	    case NAK :
	    case ENQ :
		return((int)in_packet[HDR_OFFSET]);

	    case SYN :
		/* 
		 * get length of packet
		 */
		if (comm_gets (&in_packet[LEN_OFFSET], (int)LEN_SIZE,
			TIMEOUT) != LEN_SIZE)
		    break;

		plength = ((in_packet[LEN_OFFSET]<<8) & 0xff00);    /* MSB */
		plength |= ((in_packet[LEN_OFFSET+1]) & 0xff);      /* LSB */

		if (plength > MAXPACKET) {
		    printf("Packet too large %x\n", (unsigned) plength);
		    break;
		}
		/*
		 * get message plus CRC
		 */
		msglen = plength - ( HDR_SIZE + LEN_SIZE + CRC_SIZE );
		if (comm_gets(&in_packet[MSG_OFFSET], msglen + CRC_SIZE,
		    TIMEOUT) != msglen + CRC_SIZE)
		    break;

		/*
		 * compute CRC on local packet
		 */
		chksm = 0;
		for (j = 0; j < plength - CRC_SIZE; j++) {
		    c = ((in_packet[j]) & 0xff);

		    chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
		}
		chksm &= 0xffff;

		pcrc = ((in_packet[j]<<8) & 0xff00);    /* MSB */
		pcrc |= ((in_packet[j+1]) & 0xff);      /* LSB */

		if (chksm != pcrc) {
		    printf("CRC mismatch: %x != %x\n", chksm, pcrc);
		    break;
		}
		*mbuf = (char *)&in_packet[MSG_OFFSET];
		*mlen = msglen;
		return(SYN);

	    case -1: /* Hangup */
	    	return(-1);
	    	
	    default :
		comm_gets(&in_packet[0], MAXPACKET, 0);  /* purge buffer */
		out_packet[0] = NAK;
		out_packet[1] = '\0';
		comm_puts(out_packet, 1);
		continue;

	    }
	}
	else if (ret == 0) {
	    /*
	     * Must have been a timeout
	     */
	    out_packet[0] = ENQ;
	    out_packet[1] = '\0';
	    comm_puts(out_packet, 1);
	}
	else
	    break;
    }
    /* Too many retries */
    return(-1);
}


/*
 * Build a packet - return the size of the packet built
 */
build_packet(pp, mess, mlen)
char *pp;		/* creat packet here */
char *mess;		/* this is the message */
int mlen;		/* this is the length of the message */
{
    register ushort chksm;	/* working copy of checksum */
    register unsigned char c;
    unsigned int i, plen;
    char *p;

    chksm = 0;
    plen = mlen+HDR_SIZE+LEN_SIZE+CRC_SIZE;

    c = pp[HDR_OFFSET] = SYN;
    c &= 0xff;
    chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];

    c = pp[1] = (plen & 0xff);		/* LSB */
    c &= 0xff;
    chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];

    c = pp[2] = ((plen >>8) & 0xff);	/* MSB */
    c &= 0xff;
    chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];

    p = mess;
    for (i = 0; i < mlen; i++) {
	c = pp[(MSG_OFFSET+i)] = *p++;
	c &= 0xff;
	chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
    }
    chksm &= 0xffff;

    pp[(MSG_OFFSET+mlen)] = (chksm & 0xff);	        /* LSB */
    pp[(MSG_OFFSET+mlen+1)] = ((chksm >> 8) & 0xff);	/* MSB */
    pp[(MSG_OFFSET+mlen+2)] = '\0';
    
    return(plen);
}



#define  CK_LOBAT	5
unsigned loop_cnt;

proto()
{
    char *bp;
    int len, ret, stat;

    loop_cnt = 0;
    while ((ret = comm_getpkt(&bp, &len)) != -1) {
     	if ((loop_cnt++ % CK_LOBAT) == 0 && lowbat() == TELX_ON) {
     	    out_packet[0] = ETX;
    	    out_packet[1] = '\0';
    	    comm_puts(out_packet, 1);
    	    return;
    	}
	switch(ret) {

	case SYN:
	    bp[len] = '\0';
            if ((stat = parse(bp)) == -1)
		return;
	    else if (stat != 0)
	        printf("parse errors\n");
	    continue;

	case ENQ:
       	    comm_puts(out_packet, (int)strlen(out_packet));
	    /* Fall Through */

	case ACK:
	case NAK:
	    continue;

	default:
	    printf("Unallowed packet\n");
	    continue;
	}
    }
}


#define SET_DATE    '1'	/* Set the date */
#define UPLOAD      '2'
#define UPDATE_VSN  '3'
#define DELETE_VSN  '4'
#define ADD_REQUEST '5'
#define DEL_REQUEST '6'

#define CMD_SIZE 1    /* MAX size of message id */
#define DATE_SIZE 10  /* MAX size of date in ascii */

#define MSGSIZE 80    /* Max size of individual messages */

parse(buf)
char *buf;
{
  char ch, *bp, errbuf[MAXPACKET], errmsg[MSGSIZE+1], last_token;
  int errors, row, col;
  int size; 
  char vsn[VSN_SIZE+1], sec[SEC_SIZE+1];
  unsigned long date;
  static int first = 0;
  long index;
  static char date_map[MSGSIZE+1], vsn_loc_map[MSGSIZE+1];
  static char vsn_map[MSGSIZE+1];

  if (first == 0) {
    sprintf(date_map,   "%%*%dc %%%dld", CMD_SIZE, DATE_SIZE);
    sprintf(vsn_loc_map,  "%%*%dc %%%ds %%%ds %%d %%d", CMD_SIZE, VSN_SIZE,
	  SEC_SIZE); 
    sprintf(vsn_map, "%%*%dc %%%ds", CMD_SIZE, VSN_SIZE);
    first++;
  }

  *errbuf = '\0';
  errors = 0;
  bp = buf;

  while ((ch = *bp) != '\0') {

    switch (ch) {

    case SET_DATE :
      if (sscanf(bp, date_map, &date) != 1) {
	errors++;
	break;
      }
      if (stime((time_t *)&date) != 0)
	sprintf(&errbuf[strlen(errbuf)], "error return from setting date\n");
      break;

    case UPLOAD :
      if(upload_trx() == 0)
      	break;
      return(-1);

    case UPDATE_VSN :
      if (sscanf(bp, vsn_loc_map, vsn, sec, &row, &col) != 4) {
	errors++;
	break;
      }

      if((index = db_geom(sec, row, col)) == -1L)
	sprintf(&errbuf[strlen(errbuf)], "%c %s bad geom\n", UPDATE_VSN, vsn);

      else if (db_lookup(vsn, index) != 0)
	db_write(vsn, 0);

      break;

    case DELETE_VSN :
      if (sscanf(bp, vsn_loc_map, vsn, sec, &row, &col) != 4) {
	errors++;
	break;
      }

      if((index = db_geom(sec, row, col)) == -1L)
	sprintf(&errbuf[strlen(errbuf)], "%c %s bad geom\n", DELETE_VSN, vsn);

      else if (db_lookup(vsn, index) == 0)
	db_write(NULL, 0);
	
      break;

    case ADD_REQUEST :
      if (sscanf(bp, vsn_loc_map, vsn, sec, &row, &col) != 4) {
	errors++;
	break;
      }
     if (rl_que_length >= MAX_REQLOG) {     /* Check size of reqlog */
       	    out_packet[0] = ETX;
    	    out_packet[1] = '\0';
    	    comm_puts(out_packet, 1);
    	    return(0);
      }
      if((index = db_geom(sec, row, col)) == -1L) {
	sprintf(&errbuf[strlen(errbuf)], "%c %s bad geom\n", ADD_REQUEST, vsn);
	break;
      }
      if (db_lookup(vsn, index) == 0) {
        if (rl_add(vsn, sec, row, col, index) != 0)
          sprintf(&errbuf[strlen(errbuf)], "%c %s failed\n", ADD_REQUEST, vsn);
        else {
      	  tapes.flag |= REQUESTED;
      	  put_record();
      	}
      }	
      break;
      
    case DEL_REQUEST :
      if (sscanf(bp, vsn_map, vsn) != 1) {    
        errors++;
        break;
      }
      rl_del(vsn, 1);
      break;

    default:
      sscanf(bp, "%.80s\n", errmsg);
      printf("parse error: %s\n", errmsg);
      errors++;
      break;
    }

    last_token = ch;

    /* 
     * More index pointer to next message 
     */
    if ((bp = strchr(bp, '\n')) != NULL)
      bp++;
    else
      break;

  }
  if (last_token != UPLOAD) {
  	if (errors != 0) {
  	     out_packet[0] = NAK;
  	     out_packet[1] = '\0';
	     comm_puts(out_packet, 1);  
	}
	else if (strlen(errbuf) == 0) {
	     out_packet[0] = ACK;
	     out_packet[1] = '\0';
    	     comm_puts(out_packet, 1);  
    	}
    	else {
	    size = build_packet(out_packet, errbuf, (int)strlen(errbuf));
	    comm_puts(out_packet, size);
	}
  }
  return(errors > 0 ? 1 : 0);
}


#define MAX_COMPLEX_MESS 25

extern void trx_rewind();
extern unsigned long trx_report();

upload_trx()
{
    char *vsn, *sec, *bp;
    char mess[MSGSIZE], complex_mess[MAXPACKET+1];
    unsigned int cmd, row, col, stat, i;
    int size, len, complex_mess_size;
    long rec;
    unsigned long date, message_size;
    unsigned long record_index[MAX_COMPLEX_MESS], record_cnt;

    trx_rewind();
    record_cnt = 0;
    complex_mess_size = 0;
    for(;;) {
     	if ((loop_cnt++ % CK_LOBAT) == 0 && lowbat() == TELX_ON) {
    	    out_packet[0] = ETX;
    	    out_packet[1] = '\0';
    	    comm_puts(out_packet, 1);
    	    return(-1);
        }
       	if ((rec = trx_report(&cmd, &vsn, &sec, &row, &col, &date)) >= 0L) {
	    if (cmd == TRX_ADD)
	       message_size = 
	         sprintf(mess, "%s %s %u %u %lu\n", vsn, sec, row, col, date);
	    else if (cmd == TRX_DEL)
	       message_size =
	         sprintf(mess, "%s %s %d %d %lu\n", vsn, "?", 0, 0, date);
	    else
	      continue;	

	    if (message_size + complex_mess_size < 
	      (MAXPACKET - (HDR_SIZE + LEN_SIZE + CRC_SIZE +1)) && 
	      record_cnt < MAX_COMPLEX_MESS) {
	        strncpy(&complex_mess[complex_mess_size], mess, message_size);
	        complex_mess_size += message_size;
	        record_index[record_cnt++] = rec;
	        continue;
	    }
	}
	else if (complex_mess_size == 0)
	    break;

	/*
	 * Build and send the packet.
	 */
	size = build_packet(out_packet, complex_mess, complex_mess_size);
	comm_puts(out_packet, size);

	while (( stat = comm_getpkt(&bp, &len)) != ACK) {
	    if (stat == -1)
	    	return(-1);
	    comm_puts(out_packet, size);
	}
	/*
	 * Clear all the record indexes we've sent.
	 */
	for (i = 0; i < record_cnt; i++)
	  trx_clear(record_index[i]);

	if (rec >= 0L) {
	    record_cnt = 0;
	    record_index[record_cnt++] = rec;

            strncpy(complex_mess, mess, message_size);
	    complex_mess_size = message_size;
	}
	else
	    break;

    }
    out_packet[0] = EOT;
    out_packet[1] = '\0';
    comm_puts(out_packet , 1);
    return(0);
}
