/*
 * DB.C 
 * Version 1.1 Beta 7/16/93
 * J. Musch
 */

#include "tapelib.h"
#include "win.h"
#ifdef DOS
#include <telxon.h>
#else
#include "telxon.h"
#endif
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef DOS
#include <io.h>
#else
#define O_BINARY 0
#endif

static char db_name[] = "A:tapedb";
static int fd;

static long cur_index = -1;
static long last_read_index = -1;

int row;
int col;
int win = 0;
int redraw = 0;

struct tapes tapes;

#include "db.h"
long total_tapes = TAPES;

struct section *cur_section = NULL;

static int db_open();

void db_reset() {
    cur_section = NULL;
    row = -1;
    col = -1;
    win = 0;
    redraw++;
}

db_init() {
    struct section *sp;
    long index = 0;

    if (db_open() != 0 || trx_log_init() != 0)
	exit(-1);

    /*
     * Set-up links for structure
     */
    for (sp=section; *sp->section_name != NULL; sp++) {
	sp->start = index;
	index += (sp->num_rows * sp->num_cols);
    }
    if (index != total_tapes) {
	printf("invalid index = %ld\n", index);
	exit(-1);
    }
}

static int
db_open() {

    struct stat sbuf;
    struct win *wp;
    char *args[4];

    if ((fd = open(db_name, O_RDWR|O_BINARY)) != -1) {
        if ( fstat(fd, &sbuf) == -1) {
	    perror("db_open stat failed");
	    close(fd);
	    return(-1);
        }
        if (sbuf.st_size == sizeof(struct tapes) * total_tapes) {
            db_index(0L);
            return(0);
        }
        if ((wp = win_alloc()) == NULL) {
	    printf("db_open - wrong table size\n");
	    close(fd);
	    return(-1);
        }
        wp->win_row = POPUP_RBLD_R;
        wp->win_col = POPUP_RBLD_C;
        args[0] = "Database File";
        args[1] = (sbuf.st_size > sizeof(struct tapes) * total_tapes) ?
          "too large" : "too small";
        args[2] = "Resize? (y/n)";
        args[3] = NULL;
        win_popup(wp, args);

        if (query() != QUERY_YES) {
            win_free(wp);
	    close(fd);
	    return(-1);
        }
	win_free(wp);
	setcurpos(0, 7, 5);
	writestr(0, "Resizing DB ", 0, NORMAL_VID);
    }
    else {
        if ((fd = open(db_name, O_CREAT|O_RDWR|O_BINARY,
          S_IREAD|S_IWRITE)) == -1){
            perror("db_open failed");
            return(-1);
	}
	setcurpos(0, 7, 5);
	writestr(0, "Creating DB ", 0, NORMAL_VID);
    }
    chsize(fd, (long)sizeof(struct tapes) * total_tapes);
    db_index((long) total_tapes-1);
    get_record();
    put_record();
    close(fd);
    if ((fd = open(db_name, O_RDWR|O_BINARY)) == -1) {
	    perror("db_open re-open failed");
	    return(-1);
    }
    db_index(0L);
    return(0);
}

db_index(index)
long index;
{
    long offset;

    offset = index * sizeof(struct tapes);
    lseek(fd, offset, SEEK_SET);
    cur_index = index;
}

/*
 *
 *
 * RETURN: 0 if successful
 *         1 if failed
 */
int
db_write(vsn, flag)
char *vsn;
int flag;	/* if flag = 0, don't mark record as updated */
{

    struct win *wp;
    char *args[4];

    if (cur_index < 0 || cur_index >= total_tapes) {
	return(EOF);
    }

    /*
     * Before we write, we need to inspect the current VSN in this slot in
     * the database. If we change the VSN, we need to set the UPDATE flag
     * so we know to down-load this record later. If we aren't making any 
     * change, we don't really need to do the write.
     *
     * Check which index is currently in the buffer. If what we want
     * is there, we don't need to read it in again.
     */

    if (cur_index != last_read_index) {
	/* do the read */
	get_record();
    }
    if (vsn == NULL) {			/* This represents a delete */
    
	if ((tapes.flag&VALID) != VALID)
	    return(0);    /* No change necessary */

        if (flag == 0) {
	    tapes.flag = 0;
	    return(put_record());		
	}

	if (translog_size >= MAX_TRANSLOG) {
	    if ((wp = win_alloc()) == NULL)
	        return(-1);
	    wp->win_row = POPUP_TRXL_R;
	    wp->win_col = POPUP_TRXL_C;
	    args[0] = "Trans-action";
	    args[1] = "log overflow:";
	    args[2] = "Please upload!";
	    args[3] = NULL;

	    win_popup(wp, args);
	    beep(BEEP_ERROR);
	    delay(POPUP_TRXL_W);
	    win_free(wp);
	    return(-1);
	}
	if ((tapes.flag&REQUESTED) == REQUESTED)
	    rl_del(tapes.tapeid, 1);
	trx_del(tapes.tapeid, cur_index);  /* add del to trans log */
	tapes.flag = UPDATED;              /* and invalid */
	return(put_record());
    }

    if ((tapes.flag&VALID) == VALID) {
	if (strncmp(tapes.tapeid, vsn, sizeof(tapes.tapeid)) == 0) {
	    /* 
             * No change necessary
             */
	    return(0);
	}
	if (flag != 0) {
	    if (translog_size+1 >= MAX_TRANSLOG) {

		if ((wp = win_alloc()) == NULL)
		    return(-1);
		wp->win_row = POPUP_TRXL_R;
		wp->win_col = POPUP_TRXL_C;
		args[0] = "Trans-action";
		args[1] = "log overflow:";
		args[2] = "Please upload!";
		args[3] = NULL;

		win_popup(wp, args);
		beep(BEEP_ERROR);
		delay(POPUP_TRXL_W);
		win_free(wp);
		return(-1);
	    }
	    trx_del(tapes.tapeid, cur_index);
	}
    }
    if (flag == 0)
	tapes.flag = VALID;
    else {
	trx_add(vsn, cur_index);
	tapes.flag = UPDATED|VALID;
    }
    strncpy(tapes.tapeid, vsn, sizeof(tapes.tapeid));
    return(put_record());
}

put_record() {
    int bytes;

    if ((bytes = write(fd, (char *)&tapes, sizeof(struct tapes))) == -1){
	clearline(13,15);
	setcurpos(WIN_INV, 13, 0);
	perror("db_write failed");
	return(-1);
    }
    if (bytes != sizeof(struct tapes)) {
	clearline(13,15);
	setcurpos(WIN_INV, 13, 0);
	printf("db_write - short write\n");
	db_index(cur_index);
	return(-1);
    }
    db_index(cur_index);
    return(0);
}

/*
 *  This function does not advance the file pointer
 *
 * RETURN: 0 if successful 
 *         1 if failed
 */
int
get_record()
{
    int bytes;

    if (cur_index == last_read_index)
	return(0);

    if ((bytes = read(fd, (char *)&tapes, sizeof(struct tapes))) == -1) {
	clearline(13,15);
	setcurpos(WIN_INV, 13, 0);
	perror("get_record failed");
	return(-1);
    }
    if (bytes != sizeof(struct tapes)) {

	setcurpos(WIN_INV, 13, 0);
	printf("get_record - short read\n");
	db_index(cur_index);
	return(-1);
    }
    db_index(cur_index);
    last_read_index = cur_index;
    return(0);
}


int db_search(vsn)
char *vsn;
{

    if (cur_section != NULL && strncmp(vsn, SECTION_ID, SECTION_ID_LEN) != 0)
	if (db_local_search(vsn) == 0)
	    return(0);

    if (db_scan_section(vsn) == 0)
	return(0);

    return(-1);
}

/*
 * Given the section_id, find the section we are currently in.
 *
 * Success: Return 0
 *          Set cur_section
 *              cur_index = cur_section->start
 *
 * Failure: Return -1
 *
 */
int db_scan_section(section_id)
char *section_id;
{
    struct section *sp;

    for (sp=section; *sp->section_name != NULL; sp++)
	if (strcmp(section_id, sp->section_bcid ) == 0) {
	    cur_section = sp;
	    db_index(sp->start);
	    redraw++;
	    win = 0;
	    db_redraw();
	    return(0);
	}
    return(-1);
}

db_local_search(vsn)
char *vsn;
{
    long tmp_cur_index, search_end;
    long i;

    tmp_cur_index = cur_index;
    search_end = cur_section->num_rows * cur_section->num_cols
        + cur_section->start;

    for (i = cur_index; i< search_end ; db_index(i++)) {
	if (get_record() != 0)
	    continue;
	if (tapes.tapeid == NULL || (tapes.flag&VALID) != VALID)
	    continue;
	if (strncmp(tapes.tapeid, vsn, sizeof(tapes.tapeid)) == 0) {
	    if (cur_index < search_end)
		db_index(cur_index+1L);
	    redraw++;
	    win = 0;
	    db_redraw();
	    return(0);
	}
    }

    db_index(cur_section->start);
    for (i = cur_index; i < tmp_cur_index; db_index(i++)) {
	if (get_record() != 0)
	    continue;
	if (tapes.tapeid == NULL || (tapes.flag&VALID) != VALID)
	    continue;
	if (strncmp(tapes.tapeid, vsn, sizeof(tapes.tapeid)) == 0) {
	    db_index(cur_index+1L);
	    redraw++;
	    win = 0;
	    db_redraw();
	    return(0);
	}
    }
    db_index(tmp_cur_index);
    return(-1);
}

#define CK_LOBAT 500

db_full_search(vsn, sec, row, col, index)
char *vsn;
char **sec;
unsigned int *row;
unsigned int *col;
unsigned long *index;
{
    long offset, i;
    struct section *sp;
    unsigned char ch, scan_code;

    for (i = 0L; i < total_tapes; i++) {
    	db_index(i);
        if (( i % CK_LOBAT) == 0) {
            if (ckpower() != PWR_NODELTA)
	        return(-1);
            if (chkkeybrd(&ch, &scan_code) == KEY_READY) {
		ch = readkeybrd(&scan_code);
		if (scan_code == 0x01)
		    return(-1);
	    }
	}
	if (get_record() != 0)
	    continue;
	if (tapes.tapeid == NULL || (tapes.flag&VALID) != VALID)
	    continue;
	if (strncmp(tapes.tapeid, vsn, sizeof(tapes.tapeid)) == 0)
	    break;
    }
    if (i >= total_tapes)
        return(1);

    for (sp = section; *sp->section_name != NULL; sp++)
        if (i < sp->start + sp->num_rows * sp->num_cols)
	    break; 

    if (*sp->section_name == NULL || sp->num_cols <= 0)
        return(1);

    offset = i - sp->start;
    *sec = sp->section_name;
    *row = offset/sp->num_cols;
    *col = offset%sp->num_cols;
    *index = i;
    return(0);  
}

/*
 * Given the name of the section, and the row, and column
 * compute the index into the database.
 *
 * Return -1 if the geometry is bad
 */
long db_geom(sec, l_row, l_col)
char *sec;
int l_row, l_col;
{
    static struct section *cash = section;
    struct section *sp;
    long index;
    int success = 0;

    if (l_row >= 0 && l_col >= 0) {

        for (sp = cash; *sp->section_name != NULL; sp++)
	    if (strcmp(sec, sp->section_name) == 0) {
	        success++;
	        break;
	    }

        if (success == 0)
	    for (sp = section; sp != cash; sp++)
	        if (strcmp(sec, sp->section_name) == 0) {
		    success++;
		    break;
	        }

        if ( success > 0 && l_row < sp->num_rows && l_col < sp->num_cols) {
	    cash = sp;
	    index = sp->start + (l_row * sp->num_cols) + l_col;
	    return((long) index);
        }
    }
    printf("BAD GEOM: %s %d %d\n", sec, l_row, l_col);
    return(-1L);
}

/*
 * Check to see if the specified tape is at the specified location.
 * Return 0 if the tape is there and it is valid.
 * Return -1 if the tape isn't there, or it is invalid.
 */
db_lookup(vsn, index)
char *vsn;
long index;
{

    db_index(index);
    if (get_record() != 0)
	return(-1);

    if ((tapes.flag&VALID) != VALID)
	return(-1);

    if (strncmp(tapes.tapeid, vsn, sizeof(tapes.tapeid)) != 0)
	return(-1);

    return(0);
}


void db_left(count)
int count;
{

    if (cur_section == NULL)
	return;
    if ( col - count <= 0 )
	count = col;
    if (count == 0)
	return;
    col -= count;
    db_index(cur_index -= count);
    db_redraw();
}

void db_right(count)
int count;
{

    if (cur_section == NULL)
	return;
    if ( col + count >= cur_section->num_cols )
	count = (cur_section->num_cols -1) - col;
    if (count == 0)
	return;
    col += count;
    db_index(cur_index += count);
    db_redraw();
}

void db_up(count)
int count;
{

    if (cur_section == NULL)
	return;
    if ( row - count <= 0)
	count = row;
    if (count == 0)
	return;
    row -= count;
    db_index(cur_index -= cur_section->num_cols * count);
    redraw++;
    db_redraw();
}

void db_down(count)
int count;
{

    if (cur_section == NULL)
	return;
    if ( row + count >= cur_section->num_rows)
	count = (cur_section->num_rows - 1) - row;
    if (count == 0)
	return;
    row += count;
    db_index(cur_index += cur_section->num_cols * count);
    redraw++;
    db_redraw();
}

void db_return() {

    if (cur_section == NULL)
	return;
    if (col > 0) {
	db_index(cur_index -= col);
	col = 0;
	win = 0;
    }
    if ( row+1 < cur_section->num_rows) {
	db_index(cur_index += cur_section->num_cols);
	row++;
    }
    redraw++;
    db_redraw();
}

vid_init() {

    setpage(WIN_INV);

    ttystr("SEC =", 0, 1, 1);
    ttystr("ROW =", 0, 2, 1);
    ttystr("COL =", 0, 3, 1);
    db_redraw();
}

#define WIDTH MAXCOL-2

/*
 * Draw the contents of the window
 */
void db_redraw()
{
    struct section *sp;
    long offset, tmp_cur_index;
    int i, j, vid, tmp_win, index;
    char vsn[7], num[3];
    char *cp, marg[WIDTH+1];
    char buf[SEC_SIZE+1];

    if (( sp = cur_section) == NULL || ( cur_index < sp->start ||
        cur_index >= ( sp->start + sp->num_rows * sp->num_cols ))) {

	/*
	 * Invalid Section
	 */
	cur_section = NULL;
	row = -1;
	col = -1;
	win = 0;
	redraw++;
	clearline(6,11); 
	clearline(13, 14);
	vid = NORMAL_VID;
	setcurpos(WIN_INV, 8, 4);
	writestr(WIN_INV, "Please scan", 0, NORMAL_VID);
	setcurpos(WIN_INV, 9, 4);
	writestr(WIN_INV, "section label", 0, NORMAL_VID);
	index = -1;
	cp = "Invalid";
    }
    else {
    	offset = cur_index - sp->start;
        row = offset/sp->num_cols;
        col = offset%sp->num_cols;

        tmp_win = win;

        if ((col - win) < 2)		/* slide window left */
	    if (col - 7 < -1)
	        win = -1;
	    else
	        win = col - 7;

        if ((col - win) > 10 - 3)  	/* slide window right */
	    if (col - 2 > sp->num_cols - 9)
	        win = sp->num_cols - 9;
	    else
	        win = col - 2;

        if (tmp_win != win)
	    redraw++;

        if (redraw > 0) {
	    tmp_cur_index = cur_index;
	    db_index(cur_index - (col - win));

	    clearline(6,11);  
	    clearline(13,14);

	    for (j = 0; j < 10; j++) {	/* column */
	        if ((win == -1 && j == 0) ||
	          (win == sp->num_cols-9 && j == 9)) {
		    for (i = 0; i < 6; i++)
		        vsn[i] = 0x14;	/* Draw the wall */
		    sprintf(num, "  ");
		    vid = NORMAL_VID;
	        }
	        else {
		    get_record();

		    if ((tapes.flag&VALID) == VALID)
		        strncpy(vsn, tapes.tapeid, sizeof(tapes.tapeid));
		    else
		        strncpy(vsn, "      ", 6);

		    if ((tapes.flag&UPDATED) == UPDATED)
		        vid = INVERSE_VID;
		    else 
		        vid = NORMAL_VID;
		    sprintf(num, "%02d", win + j);
	        }
	        for (i = 0; i < 6; i++) {	/* row */
		    setcurpos(WIN_INV, i+6, j*2+1);
		    writechar(vsn[i], vid, WIN_INV, 1);
	        }
	        for (i = 0; i < 2; i++) {	/* row index */
		    setcurpos(WIN_INV, i+13, j*2+1);
		    writechar(num[i], NORMAL_VID, WIN_INV, 1);
	        }
	        db_index(cur_index += 1L);
	    }
	    db_index(tmp_cur_index);
	    redraw = 0;
        }
        index = col - win;
	cp = cur_section->section_name;
    }

    /*
     * Update the section, row, and column
     */
    strncpy(buf, cp, SEC_SIZE);
    strncat(buf, "        ", SEC_SIZE-strlen(buf));

    setcurpos(WIN_INV, 1, 7);
    writestr(WIN_INV, buf, 0, NORMAL_VID);

    setcurpos(WIN_INV, 1, 19);
    if (translog_size <= 0)
        writestr(WIN_INV, " ", 1, NORMAL_VID);
    else
        writestr(WIN_INV, "*", 1, NORMAL_VID);

    if (row >= 0)
	sprintf(buf, "%02d", row);
    else
	sprintf(buf, "??");
    setcurpos(WIN_INV, 2, 7);
    writestr(WIN_INV, buf, 0, NORMAL_VID);

    if (col >= 0)
	sprintf(buf, "%02d", col);
    else
	sprintf(buf, "??");
    setcurpos(WIN_INV, 3, 7);
    writestr(WIN_INV, buf, 0, NORMAL_VID);

    /* 
     * Draw the frame
     */
    if (cur_section != NULL && row == 0) {
	marg[0] = (win == -1) ? 0x10 : 0x11;
	marg[WIDTH-1] = (win == cur_section->num_cols-9) ? 0x12 : 0x11;
	for (i = 1; i < WIDTH-1; i++)
	    marg[i] = 0x11;
    }
    else
	for (i = 0; i < WIDTH; i++)
	    marg[i] = 0x01;

    if (index >= 0)
	marg[index*2] = 0x1F;

    setcurpos(WIN_INV, 5, 1);
    writestr(WIN_INV, marg, WIDTH, NORMAL_VID);

    if (cur_section != NULL && row == cur_section->num_rows -1) {
	marg[0] = (win == -1) ? 0x15 : 0x16;
	marg[WIDTH-1] = (win == cur_section->num_cols-9) ? 0x17 : 0x16;
	for (i = 1; i < WIDTH-1; i++)
	    marg[i] = 0x11;
    }
    else
	for (i = 0; i < WIDTH; i++)
	    marg[i] = 0x01;

    if (index >= 0)
	marg[index*2] = 0x1E;

    setcurpos(WIN_INV, 12, 1);
    writestr(WIN_INV, marg, WIDTH, NORMAL_VID);
}
