/*
 * REQLOG.C 
 * Version 1.1 Beta 7/16/93
 * J. Musch
 */

/*
 * These functions all have to do with the request log.
 */
#include "tapelib.h"
#include "win.h"
#ifdef DOS
#include <telxon.h>
#else
#include "telxon.h"
#endif
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef DOS
#include <io.h>
#else
#define O_BINARY 0
#endif

extern struct tapes tapes;

long rl_que_length = 0;	  	/* the length of the queue in records */

static long first_valid_entry = -1L;
static long last_valid_entry = -1L;
static long first_invalid_entry = -1L;

static int highlight = 2;
static long top_offset, bot_offset;

static char rl_name[] = "A:request.log";
static int  rl_fd;

static char rl_map[30];

static struct rls {
    unsigned char rls_flag;
    char rls_vsn[VSN_SIZE];
    char rls_sec[SEC_SIZE];
    int rls_row;
    int rls_col;
    unsigned long rls_index;
} rls;

#define RLS_VALID 0x01

#define START   (long)( (first_valid_entry >= 0L) ? first_valid_entry : 0L)

#define END     (long)( (last_valid_entry >= 0L) ? last_valid_entry \
 		        : MAX_REQLOG * sizeof(struct rls))

void rl_reset() {

    setpage(WIN_REQ);
    settimeout(TIMEOUT_REQ);
    highlight = 2;
    top_offset = 0L;
    rl_disp();
}

int rl_init() {
    struct stat sbuf;
    unsigned int ret;
    long offset;

    if ((rl_fd = open(rl_name, O_BINARY|O_CREAT|O_RDWR, 
        S_IREAD|S_IWRITE)) == -1) {
	perror("rl_name: open error");
	return(-1);
    }
    if ((ret = fstat(rl_fd, &sbuf)) == -1) {
	perror("stat failure");
	return(-1);
    }
    for (offset = 0; offset < sbuf.st_size; offset += sizeof(struct rls)) {
	lseek(rl_fd, offset, SEEK_SET);
	if ((ret = read(rl_fd, (char *)&rls, sizeof(struct rls))) == 0)
	    break;
	else if (ret == -1) {
	    perror("rl_init:");
	    return(-1);
	}
	if ((rls.rls_flag&RLS_VALID) == RLS_VALID) {
 	   rl_que_length++;
 	   if (first_valid_entry < 0)
	       first_valid_entry = offset;
	   last_valid_entry = offset;
	}
	else {
 	   if (first_invalid_entry < 0)
	       first_invalid_entry = offset;
	}
    }
    lseek(rl_fd, 0L, SEEK_SET);

    if (last_valid_entry >= 0)
        chsize(rl_fd, (long)(last_valid_entry + sizeof(struct rls)));
    else
        chsize(rl_fd, 0L);

    sprintf(rl_map, "%%-%d.%ds %%-%d.%ds %%0%dd %%0%dd", VSN_SIZE, VSN_SIZE,
        SEC_SIZE, SEC_SIZE, ROW_SIZE, COL_SIZE);

    cleardisp(WIN_REQ);
    writestr(WIN_REQ, "VSN    Section  RW CL", 0, NORMAL_VID);

    setcurpos(WIN_REQ, 1, 0);
    writechar(0x11, NORMAL_VID, WIN_REQ, VSN_SIZE);

    setcurpos(WIN_REQ, 1, 7);
    writechar(0x11, NORMAL_VID, WIN_REQ, SEC_SIZE);

    setcurpos(WIN_REQ, 1, 16);
    writechar(0x11, NORMAL_VID, WIN_REQ, ROW_SIZE);

    setcurpos(WIN_REQ, 1, 19);
    writechar(0x11, NORMAL_VID, WIN_REQ, COL_SIZE);

    setcurpos(WIN_REQ, 2, 0);

    return(0);
}


/*
 * 
 */
int rl_add(vsn, sec, row, col, index)
char *vsn;
char *sec;
int row;
int col;
long index;
{
    unsigned int ret;
    register unsigned long offset;

    for (offset = START; offset <= END; offset += sizeof(struct rls)) {
	lseek(rl_fd, offset, SEEK_SET);
	if ((ret = read(rl_fd, (char *)&rls, sizeof(struct rls)))==0) /* EOF */
	    break;
	else if (ret == -1) {
	    perror("rl_init:");
	    return(-1);
	}

	if ((rls.rls_flag&RLS_VALID) == RLS_VALID) {
	    if(strncmp(rls.rls_vsn, vsn, sizeof(rls.rls_vsn)) == 0)
		return(0);
	    if (first_valid_entry < 0 || offset < first_valid_entry)
	    	first_valid_entry = offset;
	    if (last_valid_entry < offset)
	    	last_valid_entry = offset;
	}
	else 
	    if (first_invalid_entry < 0 || offset < first_invalid_entry)
		first_invalid_entry = offset;
    }

    if (first_invalid_entry >= 0L) {
	offset = first_invalid_entry;
	if (first_valid_entry > 0)
	    first_invalid_entry = 0L;
	else
	    first_invalid_entry = -1L;
    }
    else if (offset > MAX_REQLOG * sizeof(struct rls))
    	return(-1);

    lseek(rl_fd, offset, SEEK_SET);

    rls.rls_flag = RLS_VALID;
    strncpy(rls.rls_vsn, vsn, sizeof(rls.rls_vsn));
    strncpy(rls.rls_sec, sec, sizeof(rls.rls_sec));
    rls.rls_row = row;
    rls.rls_col = col;
    rls.rls_index = index;
    if ((ret = write(rl_fd, (char *)&rls, sizeof(struct rls))) == -1) {
	perror("write failed");
	return(-1);
    }
    rl_que_length++;
    if (first_valid_entry < 0 || first_valid_entry > offset)
    	first_valid_entry = offset;
    if (last_valid_entry < 0 || last_valid_entry < offset)
    	last_valid_entry = offset;
    return(0);
}


/*
 * Delete vsn from request log queue.
 * If flag == 0, also delete from database.
 */
int rl_del(vsn, flag)
char *vsn;
int flag;
{
    register long offset;
    long index;
    unsigned int ret;

    for (offset = START; offset <= END; offset += sizeof(struct rls)) {
	lseek(rl_fd, offset, SEEK_SET);
	if ((ret = read(rl_fd, (char *)&rls, sizeof(struct rls))) == 0)
	    return(0);
	else if (ret == -1) {
	    perror("rl_del:");
	    return(-1);
	}
	if ((rls.rls_flag&RLS_VALID) == RLS_VALID) {
	    if (strncmp(vsn, rls.rls_vsn, sizeof (rls.rls_vsn)) == 0) {
 	        if (flag == 0) {
		    if ((index = db_geom(rls.rls_sec, rls.rls_row,
			 rls.rls_col)) >= 0L)
		        if (db_lookup(rls.rls_vsn, index) == 0) {
		            tapes.flag &= ~REQUESTED;
			    db_write(NULL, 1);
			}
	        }
	        rls.rls_flag = 0;
	        lseek(rl_fd, offset, SEEK_SET);
	        if (write(rl_fd, (char *)&rls, sizeof(struct rls)) == -1) {
		    perror("write failed");
		    return(-1);
	        }
                rl_que_length--;
	        if (first_invalid_entry < 0 || offset < first_invalid_entry)
		    first_invalid_entry = offset;
		return(0);
	    }
	    if (first_valid_entry < 0 || offset < first_valid_entry)
	    	first_valid_entry = offset;
	    if (last_valid_entry < offset)
	    	last_valid_entry = offset;
	}
	else {
	    if (first_invalid_entry < 0 || offset < first_invalid_entry)
		first_invalid_entry = offset;
	    continue;
	}
    }
    return(-1);
}


/*
 * Draw the request screen on the display
 */
void rl_disp()
{
    register unsigned long offset;
    unsigned int ret, line;
    char buffer[MAXCOL+1];

    offset = top_offset;
    for (line = 2; line < MAXLINE; line++) {
	for ( ; offset <= last_valid_entry; offset += sizeof(struct rls)) {
	    lseek(rl_fd, offset, SEEK_SET);
	    if ((ret = read(rl_fd, (char *)&rls, sizeof(struct rls))) == 0) {
		clearline(line, MAXLINE-1);
		if (line <= highlight) {
		    rl_up(highlight - line + 1);
		}
		return;
	    }
	    else if (ret == -1) {
		perror("rl_init:");
		return;
	    }
	    if ((rls.rls_flag&RLS_VALID) == RLS_VALID) {
		sprintf(buffer, rl_map,
		    rls.rls_vsn, rls.rls_sec, rls.rls_row, rls.rls_col);
		setcurpos(WIN_REQ, line, 0);
		writestr(WIN_REQ, buffer, 0,
		    (line == highlight) ? INVERSE_VID : NORMAL_VID);
		bot_offset = offset;
		if ((offset += sizeof(struct rls)) > last_valid_entry)
		    line++;
		break;
	    }
	}
	if (offset > last_valid_entry) {
	    if (line <= MAXLINE-1) {
		clearline(line, MAXLINE-1);
	        if (line <= highlight)
		    rl_up(highlight - line + 1);
	    }
	    return;
	}
    }
}

void rl_up(cnt)
int cnt;
{
    int i;
    register long offset;
    unsigned int ch;

    if (highlight - cnt < 2 &&
        (offset = top_offset - sizeof(struct rls)) >= 0) {

	cnt = cnt - highlight + 2;
	highlight = 2;

	for (i = 0; i < cnt; i++) {
	    for ( ; offset >= 0; offset -= sizeof(struct rls)) {
		lseek(rl_fd, offset, SEEK_SET);
		if (read(rl_fd, (char *)&rls, sizeof(struct rls)) != 
		    sizeof(struct rls)) {
		    printf("rl_up\n");
		    return;
		}
		if ((rls.rls_flag&RLS_VALID) == RLS_VALID)
		    break;
	    }
	    if (offset < 0)
		break;
	    top_offset = offset;
	    offset -= sizeof(struct rls);
	}
	rl_disp();
	return;

    }
    if (highlight == 2)
    	return;
    	
    if (cnt > highlight - 2 )
        cnt = highlight - 2;

    for (i = 0; i < MAXCOL; i++) {
	ch = readchar(WIN_REQ, highlight-cnt, i);

	setcurpos(WIN_REQ, highlight-cnt, i);
	writechar((char)ch, INVERSE_VID, WIN_REQ, 1);
    }
    for (i = 0; i < MAXCOL; i++) {
	ch = readchar(WIN_REQ, highlight, i);

	setcurpos(WIN_REQ, highlight, i);
	writechar((char)ch, NORMAL_VID, WIN_REQ, 1);
    }
    highlight -= cnt;

}

void rl_down(cnt)
int cnt;
{

    register long t_offset, b_offset;
    int i;
    unsigned int ch;


    if (highlight + cnt > MAXLINE-1 && bot_offset >= 0 && 
        (b_offset = bot_offset + sizeof(struct rls)) < last_valid_entry) {

	cnt -= MAXLINE-1 - highlight;
	highlight = MAXLINE-1;

	for (i = 0; i < cnt; i++) {
	    for ( ; b_offset <= last_valid_entry;
	        b_offset += sizeof(struct rls)) {
		lseek(rl_fd, b_offset, SEEK_SET);
		if (read(rl_fd, (char *)&rls, sizeof(struct rls)) != 
		    sizeof(struct rls)) {
		    printf("rl_down\n");
		    return;
		}
		if ((rls.rls_flag&RLS_VALID) == RLS_VALID)
		    break;
	    }
	    if (b_offset > last_valid_entry)
		break;
	    b_offset += sizeof(struct rls);

	    for (t_offset = top_offset + sizeof(struct rls);
	        t_offset <= last_valid_entry; t_offset += sizeof(struct rls)) {
		lseek(rl_fd, t_offset, SEEK_SET);
		if (read(rl_fd, (char *)&rls, sizeof(struct rls)) != 
		    sizeof(struct rls)) {
		    printf("rl_down\n");
		    return;
		}
		if ((rls.rls_flag&RLS_VALID) == RLS_VALID)
		    break;
	    }
	    if (t_offset > last_valid_entry)
		break;
	    top_offset = t_offset;
	    t_offset += sizeof(struct rls);
	}
	rl_disp();
	return;
    }

    if (cnt > MAXLINE-1 - highlight)
        cnt = MAXLINE-1 - highlight;

    while ((char)(ch = readchar(WIN_REQ, highlight+cnt, 0)) == 0x20)
	if (--cnt <= 0)
		return ;

    for (i = 0; i < MAXCOL; i++) {
	ch = readchar(WIN_REQ, highlight, i);

	setcurpos(WIN_REQ, highlight, i);
	writechar((char)ch, NORMAL_VID, WIN_REQ, 1);
    }
    for (i = 0; i < MAXCOL; i++) {
	ch = readchar(WIN_REQ, highlight+cnt, i);

	setcurpos(WIN_REQ, highlight+cnt, i);
	writechar((char)ch, INVERSE_VID, WIN_REQ, 1);
    }
    highlight += cnt;

}

void rl_deque() {

    int i, ret;
    char cbuf[VSN_SIZE+1];
    char *args[4];
    struct win *wp;
    long offset;

    if (highlight < 2 || highlight >= MAXLINE)
	return;

    for (i = 0; i < VSN_SIZE; i++) {
	cbuf[i] = (char)readchar(WIN_REQ, highlight, i);

	if (i == 0 && cbuf[i] == 0x20)
	    return;
    }
    cbuf[VSN_SIZE] = '\0';

    args[0] ="DELETE";
    args[1] = cbuf;
    args[2] = "  y/n";
    args[3] = NULL;

    if ((wp = win_alloc()) == NULL)
	printf("win_alloc failed\n");

    wp->win_row = POPUP_RLQ_R;
    wp->win_col = POPUP_RLQ_C;

    win_popup(wp, args);

    ret = query();
    win_free(wp);

    if (ret == QUERY_YES) {
	rl_del(cbuf, 1);

        if ((offset = top_offset - sizeof(struct rls)) >= 0) {
	    for ( ; offset >= 0; offset -= sizeof(struct rls)) {
		lseek(rl_fd, offset, SEEK_SET);
		if (read(rl_fd, (char *)&rls, sizeof(struct rls)) != 
		    sizeof(struct rls)) {
		    printf("rl_up\n");
		    return;
		}
		if ((rls.rls_flag&RLS_VALID) == RLS_VALID)
		    break;
	    }
	    if (offset >= 0)
	        top_offset = offset;
	}
    }
    rl_disp();
}

/*
 * Sort the transaction log
 */
void rl_sort()
{
    struct rls *ptr;
    void *buffer;
    int i, compare();
    size_t ssize = sizeof(struct rls);
        
    if (rl_que_length == 0)
        return;

    if ((buffer = (void *) calloc((size_t)rl_que_length, ssize)) == NULL) {
       printf("rl_sort:cmalloc failed\n");
       return;
    }

    ptr = (struct rls *)buffer;
    lseek(rl_fd, START, SEEK_SET);
    for (i = 0; i < rl_que_length; ) {
        if (read(rl_fd, (char *)ptr, ssize) != ssize) {
    	    printf("Bad read\n");
	    free(buffer);
    	    return;
        }
	if (ptr->rls_flag == RLS_VALID) {
	    ptr++;
	    i++;
	}
    }
    qsort((void *)buffer, (size_t)rl_que_length, ssize, compare);   

    ptr = (struct rls *)buffer;
    lseek(rl_fd, 0L, SEEK_SET);
    for (i = 0; i < rl_que_length; i++)
        write(rl_fd, (char *)ptr++, ssize);
    free(buffer);

    first_valid_entry = 0L;
    last_valid_entry = rl_que_length * ssize;
    first_invalid_entry = -1L;
    chsize(rl_fd, last_valid_entry + ssize);

}

int compare(p1, p2)
struct rls *p1, *p2;
{
   if (p1->rls_index > p2->rls_index)
        return(1);	
    if (p1->rls_index < p2->rls_index)        
        return(-1);	    
    return(0);
}
