/*
 * TAPELIB.C 
 * Version 1.1 Beta 7/16/93
 * J. Musch
 */

#include "tapelib.h"
#include "win.h"
#ifdef DOS
#include <telxon.h>
#else
#include "telxon.h"
#endif
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#define BCBUFSIZE 20

extern struct section *cur_section;
extern struct tapes tapes;

static int bktlight = TELX_OFF;

main()
{
    struct win *wp, *iwp;	/* For the popups */
    char *args[5];
    int  iflag, ret, count=0;
    unsigned char ch, scan_code;
    char buf[80], buf2[80];
    char bc_buf[BCBUFSIZE+1];
    struct bar_stat bc_stat;
    struct label_def barcodes[2];

    barcodes[0].label_type = CODE_3_9;
    barcodes[0].label_length = 8;
    barcodes[0].generic_options = 0;
    barcodes[0].label_options = 0;
    barcodes[0].drop_options = 0x11;
    barcodes[0].gsn = 0;

    barcodes[1].label_type = 0;

    resetkeybrd();
    dispreset();
    setcurtype(NOCUR);
    setcurpos(0, 7, 5);
    writestr(0, "INITIALIZING", 0, NORMAL_VID);

    pwrdwnswch(PWR_CLR);
    setvolume(DEFAULT_VOL);
    
    db_init();
    rl_init();
    vid_init();
    scanconfig(LASER, barcodes);
    scanner(INT_SCAN);
    setctrlstatus(DIS_REBOOT);
    iflag = 0;

    if (rl_que_length > 0L)
	 win_rl();

    setpage(WIN_INV);
    settimeout(TIMEOUT_INV);

    for(;;) {         /* loop forever */

	if (ckpower() == PWR_PWRUP) {
	    if (rl_que_length > 0L)
		 win_rl();

	    setpage(WIN_INV);
	    settimeout(TIMEOUT_INV);
	    db_reset();
	    db_redraw();
	}

	while (chkkeybrd(&ch, &scan_code) == KEY_READY) {

	    switch(ch = readkeybrd(&scan_code)) {

	    case 'f' :
		db_right(count <= 0 ? 1 : count);
		count = 0;
		break;

	    case 'e' :
		db_left(count <= 0 ? 1 : count);
		count = 0;
		break;

	    case 'a' :
		db_up(count <= 0 ? 1 : count);
		count = 0;
		break;

	    case 'b' :
		db_down(count <= 0 ? 1 : count);
		count = 0;
		break;

	    case 'd' :		/* Delete current entry */
		count = 0;
		if (cur_section == NULL)
		    break;
		if ((wp = win_alloc()) == NULL)
		    break;
		wp->win_row = POPUP_INVD_R;
		wp->win_col = POPUP_INVD_C;

		scanner(TELX_OFF);
		get_record();
		sprintf(buf, "%6.6s (y/n)?", tapes.tapeid);

		args[0] = "  Delete";
		args[1] = buf;
		args[2] = NULL;

		win_popup(wp, args);
		ret = query();
		win_free(wp);

		switch(ret) {
		case QUERY_BAD_POWER:
		    db_reset();
	            db_redraw();
		    break;

		case QUERY_YES:
		    if (db_write(NULL, 1) == -1) {
			beep(BEEP_ERROR);
			break;
		    }
		    beep(BEEP_DELTA);
		    db_right(1);
		    redraw++;
		    db_redraw();
		    break;

		case QUERY_NO:
		    db_right(1);
		default:
		    break;
		}
		scanner(INT_SCAN);
		break;

	    case 'i' :
		if (cur_section == NULL)
		    break;
		if (iflag > 0) {
		    iflag = 0;
		    win_free(iwp);
		}
		else {
		    if ((iwp = win_alloc()) == NULL)
		        break;
		    iwp->win_row = POPUP_IW_R;
		    iwp->win_col = POPUP_IW_C;
		    args[0] = "Index Mode:";
		    args[1] = "Shoot tape";
		    args[2] = "or section.";
		    args[3] = NULL;
		    win_popup(iwp, args);
		    iflag++;
		}
		count = 0;
		break;

	    case 'x' :
		count = 0;
		win_rl();
		setpage(WIN_INV);
		settimeout(TIMEOUT_INV);
		db_reset();
		db_redraw();
		break;

	    case 'h' :
		count = 0;
		scanner(TELX_OFF);
		inv_help("A:INV.HLP");
		setpage(WIN_INV);
		settimeout(TIMEOUT_INV);
		scanner(INT_SCAN);
		break;

	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    case '8':
	    case '9':
		count = 10 * count + (ch - '0');
		break;
           
	    default:
		count = 0;
		if (scan_code == 0x1c) {
		    db_return();
		    break;
		}
		key_common(ch);
		break;
	    }
	}

	/*
	 * If Barcode Scanner data is available -
	 *  - If we are not currently in a valid section, see if we
	 *	can get a section match.
	 *  - If we are in index mode - see if we we can find a location
	 *  - else either add or replace the existing information.
	 */
	switch ( scanread(bc_buf, BCBUFSIZE, &bc_stat)) {
	case SCAN_DATA_AVAIL:
	    bc_buf[bc_stat.length] = '\0';
	    beep(BEEP_MATCH);
	    scanner(TELX_OFF);
	    if (strncmp(bc_buf, SECTION_ID, SECTION_ID_LEN) == 0) {
		if (db_scan_section(bc_buf) == 0)
		    beep(BEEP_MATCH);
		else
		    beep(BEEP_ERROR);
		break;
	    }
	    if (cur_section == NULL) {
		beep(BEEP_ERROR);
		break;	    	
	    }
	    if (iflag > 0) {	/* we're in index mode */
		if (db_search(bc_buf) == 0)
		    beep(BEEP_MATCH);
		else
		    beep(BEEP_ERROR);
		iflag = 0;
		win_free(iwp);
		redraw++;
		db_redraw();
	    } else {
		get_record();
		if ((tapes.flag&VALID) != VALID) {
		    db_write(bc_buf, 1);
		    db_right(1);
		    redraw++;
		    db_redraw();
		    beep(BEEP_DELTA);
		}
		else {

		    if(strncmp(tapes.tapeid,bc_buf,sizeof(tapes.tapeid))==0) {
			db_right(1);
			beep(BEEP_MATCH);
			break;
		    }
		    if ((wp = win_alloc()) == NULL)
			break;
		    wp->win_row = POPUP_INVO_R;
		    wp->win_col = POPUP_INVO_C;

		    sprintf(buf, "Replace %6.6s", tapes.tapeid);
		    sprintf(buf2, "with %6.6s ?", bc_buf);

		    args[0] = buf;
		    args[1] = buf2;
		    args[2] = NULL;

  		    win_popup(wp, args);
  		    ret = query();
	   	    win_free(wp);	/* release the popup */
	   	    
		    switch(ret) {
		    case QUERY_BAD_POWER:
			db_reset();
			db_redraw();
			break;

		    case QUERY_YES:
		        if (db_write(bc_buf, 1) == -1) {
			    beep(BEEP_ERROR);
			    break;
			}
			beep(BEEP_DELTA);
		        db_right(1);
		        redraw++;
			db_redraw();
			break;

		    case QUERY_NO:
		    default:
			break;
		    }
		}
	    }
	    break;

	case NO_DATA_AVAIL:
	case SCAN_ERROR:
	case BUFF_OVRFLW:
	default:
	    break;
	}
	scanner(INT_SCAN);
    }
}


win_rl()
{
    unsigned char ch, scan_code;
    char bc_buf[BCBUFSIZE+1];
    struct bar_stat bc_stat;
    int count = 0;

    rl_reset();
    
    for(;;) {

        if (rl_que_length == 0L)
            return;
            
	if (ckpower() == PWR_PWRUP)
	    rl_reset();

	while (chkkeybrd(&ch, &scan_code) == KEY_READY) {

	    switch(ch = readkeybrd(&scan_code)) {

	    case 'a' :
		rl_up((count == 0) ? 1 : count);
		count = 0;
		break;

	    case 'b' :
		rl_down((count == 0) ? 1 : count);
		count = 0;
		break;

	    case 'd' :
		rl_deque();
		count = 0;
		break;

	    case 'x' :
		return;

	    case 'h' :
		count = 0;	        
	    	scanner(TELX_OFF);
		inv_help("A:REQ.HLP");
		setpage(WIN_REQ);
	        settimeout(TIMEOUT_REQ);
	        scanner(INT_SCAN);
		break;

	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    case '8':
	    case '9':
		count = 10 * count + (ch - '0');
		break;

	    default:
		count = 0;
    		if (scan_code == 0x1c) {
		    rl_down(13);
		    break;
		}
		key_common(ch);
		break;
	    }
	}
	/*
	 * Check if barcode scanner data is available -
	 */
	switch ( scanread(bc_buf, BCBUFSIZE, &bc_stat)) {
	case SCAN_DATA_AVAIL:
	    bc_buf[bc_stat.length] = '\0';
	    beep(BEEP_MATCH);
	    scanner(TELX_OFF);
	    if (rl_del(bc_buf, 0) == 0) {
		beep(BEEP_DELTA);
		rl_disp();
	    }
	    else
		beep(BEEP_ERROR);
	    break;

	case NO_DATA_AVAIL:
	case SCAN_ERROR:
	case BUFF_OVRFLW:
	default:
	    break;
	}
	scanner(INT_SCAN);
    }
}


key_common(ch)
char ch;
{
    struct win *wp;
    char *args[3];
    int ret;

    switch(ch) {
    case 's':
        scanner(TELX_OFF);
	tl_status();
	purge_kbd();
	scanner(INT_SCAN);
	break;

    case 't':
	lcdadj(TELX_DN, 1);
	break;

    case 'u':
	if (bktlight == TELX_OFF)
 	    bktlight = TELX_ON;
	else
	    bktlight = TELX_OFF;
	backlite(bktlight);
	break;

    case 'v':
	if (++volume > 3)
	    volume = 1;
	beep(BEEP_MATCH);
        break;

    case 'z':
	scanner(TELX_OFF);
	zkey();
	scanner(INT_SCAN);
	break;

    case 0x03:

        if ((wp = win_alloc()) == NULL)
	    break;
	wp->win_row = POPUP_EXIT_R;
	wp->win_col = POPUP_EXIT_C;

	args[0] = "Exit to DOS ?";			
	args[1] = NULL;

	win_popup(wp, args);
	ret = query();
	win_free(wp);

	if (ret == QUERY_YES) {
	    dispreset();
	    setvolume(NORMAL_VOL);
            settimeout(60);
            setctrlstatus(0);            
            scanner(TELX_OFF);
            exit(0);
	}
	break;

    default:

	if ((wp = win_alloc()) == NULL)
	    break;
	wp->win_row = POPUP_ILC_R;
	wp->win_col = POPUP_ILC_C;

	args[0] = "Illegal Character";			
	args[1] = "Press H for Help";
	args[2] = NULL;
		    
	win_popup(wp, args);
	beep(BEEP_MATCH);
	delay(POPUP_ILC_W);		/* 2 sec delay */
	win_free(wp);
	purge_kbd();
	break;
    }
    return(0);
}

zkey() {

    struct win *wp;
    char *sec, *args[5];
    int ret, i = 0;
    unsigned char ch, scan_code, cbuf[VSN_SIZE+1];
    unsigned int row, col;
    unsigned long index;

    cbuf[0] = '\0';
    if ((wp = win_alloc()) == NULL)
        return;
    wp->win_row = POPUP_LOC_R;
    wp->win_col = POPUP_LOC_C;

    if (translog_size >= MAX_TRANSLOG) {
        args[0] = "Transaction";
	args[1] = "log too";
	args[2] = "large !";
	args[3] = NULL;
	win_popup(wp, args);
        beep(BEEP_ERROR);
        delay(POPUP_NH_W);		/* 5 sec delay */
	win_free(wp);
        return;   
    }

    args[0] = "Enter VSN";			
    args[1] = "> ";
    args[2] = NULL;

    win_popup(wp, args);
    beep(BEEP_QUERY);
    for (;;) {
      if (ckpower() == PWR_PWRUP) {
          win_free(wp);      	
          return;
      }
      while (chkkeybrd(&ch, &scan_code) == KEY_READY) {
        ch = readkeybrd(&scan_code);

        if (scan_code == 0x01) {   /* esc */
          win_free(wp);      	
          return;
        }
        if (scan_code == 0x0e) {   /* back-space */
            i -= 1;
	    if (i < 0)
	        i = 0;
            cbuf[i] = ' ';
        }
        else {
            cbuf[i++] = toupper(ch);
            cbuf[i] = '\0';
        }
	ttystr(cbuf, 0, POPUP_LOC_R+1, POPUP_LOC_C+strlen(args[1]));
      }
      if (i == VSN_SIZE)
        break;
    }
    win_free(wp);

    if ((wp = win_alloc()) == NULL)
        return;
    wp->win_row = POPUP_LOC_R;
    wp->win_col = POPUP_LOC_C;

    args[0] = "Confirm";			
    args[1] = "  VSN";			
    args[2] = cbuf;
    args[3] = NULL;

    win_popup(wp, args);
    ret = query();
    win_free(wp);

    if (ret != QUERY_YES)
        return;

    if ((wp = win_alloc()) == NULL)
        return;

    wp->win_row = POPUP_LOC_R;
    wp->win_col = POPUP_LOC_C;

    args[0] = "WORKING ";			
    args[1] = "        ";
    args[2] = NULL;    

    win_popup(wp, args);

    ret = db_full_search(cbuf, &sec, &row, &col, &index);
    if (ret == -1) {
        win_free(wp);    
        return;
    }
    if (ret == 0) {
        int mode, cols, page;

        getvidmode(&mode, &cols, &page);

	ttystr("SUCCESS", 0, POPUP_LOC_R+1, POPUP_LOC_C);
	beep(BEEP_MATCH);
	rl_add(cbuf, sec, row, col, index);
        delay(POPUP_LOC_W);
        win_free(wp);        
	if (page == WIN_REQ)
	    rl_disp();
	return;
    }

    ttystr("FAILURE", 0, POPUP_LOC_R+1, POPUP_LOC_C);
    beep(BEEP_ERROR);
    delay(POPUP_LOC_W);
    win_free(wp);        
}

void purge_kbd()
{
    unsigned char ch, scan_code;

    while (chkkeybrd(&ch, &scan_code) == KEY_READY)
	ch = readkeybrd(&scan_code);
}

inv_help(help_file)
char *help_file;
{

    struct win *wp;
    char *args[3];
    unsigned char ch, scan_code;
    FILE *stream;
    char line[MAXCOL+1], buf[MAXCOL+1];
    char *ptr, *cp, *strchr();
    int i, page = 0;

    cleardisp(WIN_HELP);
    setpage(WIN_HELP);
    settimeout(TIMEOUT_HLP);


    if ((stream = fopen(help_file, "r")) == NULL) {
        if ((wp = win_alloc()) != NULL) {
	    wp->win_row = POPUP_NH_R;
	    wp->win_col = POPUP_NH_C;
	    args[0] = help_file;
	    args[1] = "NOT Found";
	    args[2] = NULL;
	    win_popup(wp, args);
            delay(POPUP_NH_W);		/* 5 sec delay */
	    win_free(wp);
        }
	return;
    }

    for(;;) {

	for (i = 0; i <= MAXLINE-2; i++) {
	    if ((ptr = fgets(line, MAXCOL+1, stream)) == NULL)
		break;
	    if ((cp = strchr(line, '\n')) != NULL)
		*cp = '\0';
	    sprintf(buf, "%-21.21s", line);
	    if (page == 0)
		setcurpos(WIN_HELP, i, 0);
	    else {
		scroll(TELX_UP, 1, NORMAL_VID, 0, 0, MAXLINE-2, MAXCOL-1);
		setcurpos(WIN_HELP, MAXLINE-2, 0);
	    }
	    writestr(WIN_HELP, buf, 0, NORMAL_VID);
	}
	page++;
	setcurpos(WIN_HELP, MAXLINE-1, 0);

	sprintf(buf, "%-21.21s", (ptr == NULL) ? "     ENT = EXIT" : 
	    "  ESC=EXIT ENT=MORE");
	writestr(WIN_HELP, buf, 0, INVERSE_VID);

	for (;;) {
	    while (chkkeybrd(&ch, &scan_code) != KEY_READY)
		if(ckpower() != PWR_NODELTA) {
		    fclose(stream);
		    return;
		}

	    switch(ch = readkeybrd(&scan_code)) {

	    default:
	        if (scan_code == 0x01) {
		    if (ptr == NULL)
		        continue;
		    fclose(stream);
		    return;
		}
		if (scan_code == 0x1c)
		    break;
		continue;
	    }

	    if (ptr == NULL) {
		fclose(stream);
		return;
	    }
	    else
		break;
	}
    }
}



ckpower() {

    if (lowbat() == TELX_ON) {

	/*
	 * We are in a low battery condition
	 */
	struct win *wp;
        char *args[4];

        if ((wp = win_alloc()) != NULL) {
	    wp->win_row = POPUP_LB_R;
	    wp->win_col = POPUP_LB_C;
	    args[0] = "LOW BATTERY-";
	    args[1] = "PLEASE RETURN";
	    args[2] = "GUN TO CHARGER";
	    args[3] = NULL;
	    win_popup(wp, args);
            delay(POPUP_LB_W);
	    win_free(wp);
        }
	return(PWR_LOBAT);
    }

    while (pwrdwnswch(PWR_EXAM) == 1) {
        switch ( pwrupstatus()) {

	case PWRUP_NOR:
	    bktlight = TELX_OFF;				
	    pwrdwnswch(PWR_CLR);
	    return(PWR_PWRUP);

        case PWRUP_RI:
            scanner(TELX_OFF);
	    comm();		/* Falls Thru */
	    rl_sort();
	    trx_log_pack();
	    if (rl_que_length > 0)
	        beep(BEEP_QUERY);
	    scanner(INT_SCAN);
	    
        case PWRUP_ALARM:
	    poweroff();
            break;
        }
    }
    return(PWR_NODELTA);
}


/* This function returns -1 if we've powered up, 0 if y typed, and
 * 1 if n typed.
 */

query() {

    int loop_cnt, ret = 0;
    int poped = 0;
    unsigned char ch, scan_code;
    struct win *wp;
    char *args[2];

    beep(BEEP_QUERY);
    for (loop_cnt = 0; ;loop_cnt++) {
	while (chkkeybrd(&ch, &scan_code) != KEY_READY) {
	    if (ckpower() == PWR_PWRUP) {
		ret = QUERY_BAD_POWER;
		break;
	    }
	}
	if (ret == QUERY_BAD_POWER)
	    break;

	switch(ch = readkeybrd(&scan_code)) {

	case 'y':
	case 'Y':
	    ret = QUERY_YES;
	    break;

	case 'n':
        case 'N':
	    ret = QUERY_NO;
	    break;

	default:
	    if (loop_cnt == 0) {
		beep(BEEP_QUERY);
	    }
	    else {

		beep(BEEP_ERROR);
		if (poped > 0)
		    continue;
		poped++;
		
		if ((wp = win_alloc()) == NULL)
		    break;
		wp->win_row = POPUP_QURY_R;
		wp->win_col = POPUP_QURY_C;

		args[0] = "Press y/n";
		args[1] = NULL;

		win_popup(wp, args);
	    }
	    continue;
	}
	break;
    }

    if (poped > 0)
	win_free(wp);
    return(ret);
}


tl_status()
{
    struct win *wp;	/* For the popups */
    char *args[4], buf0[40], buf1[40], buf2[40];
    struct dt curr_date;

    if ((wp = win_alloc()) == NULL)
        return;
    wp->win_row = POPUP_INVS_R;
    wp->win_col = POPUP_INVS_C;

    /*
     * Get Time.
     */
    datetime(&curr_date);
    sprintf(buf0, "%02d/%02d/%02d %02d:%02d:%02d", curr_date.dt_month,
	 curr_date.dt_day, curr_date.dt_year - 1900, curr_date.dt_hour,
	 curr_date.dt_min, curr_date.dt_sec); 
    args[0] = buf0;

    /*
     * Get the info for the request log.
     */
    sprintf(buf1, "RL:%ld/%d entries", rl_que_length, MAX_REQLOG);
    args[1] = buf1;

    /*
     * Get the info for the transaction log.
     */
    sprintf(buf2, "TL:%ld/%d entries", translog_size, MAX_TRANSLOG);
    args[2] = buf2;
    args[3] = NULL;

    win_popup(wp, args);
    beep(BEEP_MATCH);
    delay(POPUP_INVS_W);
    win_free(wp);
}
