/*
 * TRXLOG.C 
 * Version 1.1 Beta 7/16/93
 * J. Musch
 */

#include "tapelib.h"
#include "win.h"
#ifdef DOS
#include <telxon.h>
#else
#include "telxon.h"
#endif
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef DOS
#include <io.h>
#else
#define O_BINARY 0
#endif

static char trx_file[] = "A:transact.log";
static int trx_fd;
long translog_size = 0;

extern struct tapes tapes;
extern struct section section[];
extern long total_tapes;

/* 
 * The following routines all deal with the transaction log
 */
struct translog {
    unsigned flag;
    char vsn[VSN_SIZE];
    unsigned long index;
    unsigned long date;
};

trx_log_init() {
    struct stat sbuf;    

    if ((trx_fd = open(trx_file, O_BINARY|O_CREAT|O_RDWR, 
        S_IREAD|S_IWRITE)) == -1) {
	perror("trx_log: open error");
	return(-1);
    }
    if ( fstat(trx_fd, &sbuf) == -1) {
        perror("trx_log: stat failed");
        return(-1);
    }
    translog_size = sbuf.st_size / sizeof(struct translog);
    return(0);
}

trx_add (vsn, index)
char *vsn;
unsigned long index;
{
    return(trx_log(TRX_ADD, vsn, index));
}

trx_del(vsn, index)
char *vsn;
unsigned long index;
{
    return(trx_log(TRX_DEL, vsn, index));
}

trx_log(flag, vsn, index)
int flag;
char *vsn;
unsigned long index;
{
    struct translog translog;
    unsigned long rec;
    translog.flag = flag;
    strncpy(translog.vsn, vsn, VSN_SIZE);
    translog.index = index;
    time(&translog.date);

    if ((rec = lseek(trx_fd, 0L, SEEK_END)) == -1L) {
	printf("seek error\n");
	return(-1);
    }
    translog_size = (rec / sizeof(struct translog));

    if (write(trx_fd, (char *)&translog, sizeof(translog)) != 
        sizeof(translog)) {
	printf("write error\n");
	return(-1);
    }
    translog_size++;
    close(trx_fd);
    if ((trx_fd = open(trx_file, O_BINARY|O_RDWR)) == -1) {
	perror("trx_log: re-open error");
	exit(-1);
    }
    return(0);
}

static unsigned long first_valid_entry = 0L;
static unsigned long next_read_entry = 0L;

void trx_rewind()
{
    next_read_entry = first_valid_entry;
}

char vsn_str[7];

unsigned long trx_report(cmd, vsn, sec, l_row, l_col, date)
unsigned int *cmd;
char **vsn;
char **sec;
unsigned int *l_row;
unsigned int *l_col;
unsigned long *date;
{
    struct translog translog;
    struct section *sp;
    unsigned long offset;

    if (lseek(trx_fd, next_read_entry*sizeof(translog), SEEK_SET) == -1L) {
	printf("seek error\n");
	return(-1);
    }

    while ( read(trx_fd, (char *)&translog, sizeof(translog)) ==
        sizeof(translog)) {
	next_read_entry++;

	if (translog.flag == TRX_ADD)
	    *cmd = TRX_ADD;
	else if (translog.flag == TRX_DEL)
	    *cmd = TRX_DEL;
	else 
	    continue;
	    
	strncpy(vsn_str, translog.vsn, VSN_SIZE);
	*vsn = vsn_str;
	/*
         * Get the section name, l_row, and l_column.
         */
	for (sp = section; *sp->section_name != NULL; sp++)
	    if (translog.index < sp->start + sp->num_rows * sp->num_cols)
		break;

	*sec = sp->section_name;

	if ((sp->start >= 0 && sp->start <= total_tapes) &&
	    (sp->num_cols > 0 )) {
	    offset = translog.index - sp->start;
	    *l_row = offset/sp->num_cols;
	    *l_col = offset%sp->num_cols;
	}
	else {
	    printf("invalid data\n");
	    *l_row = 0;
	    *l_col = 0;
	}
	*date = translog.date;
	return(next_read_entry-1);
    }
    return(-1L);
}

/*
 * clear the entry in the transaction log.
 */
trx_clear(index)
unsigned long index;
{
    struct translog translog;
    struct stat Sbuf;
    int ret;
    unsigned long offset;

    if ((ret = fstat(trx_fd, &Sbuf)) == -1) {
	printf("trx_clear: stat error\n");
	return(-1);
    }
    offset = index*sizeof(translog);
    if (offset > Sbuf.st_size - sizeof(translog)) {
	printf("trx_clear: index error\n");
	return(-1);
    }
    if (lseek(trx_fd, offset, SEEK_SET) == -1L) {
	printf("trx_clear: seek error\n");
	return(-1);
    }
    if (read(trx_fd, (char *)&translog, sizeof(translog)) != 
        sizeof(translog)) {
	printf("trx_clear: read error\n");
	return(-1);
    }
    if (lseek(trx_fd, offset, SEEK_SET) == -1L) {
	printf("trx_clear: seek error\n");
	return(-1);
    }
    /*
     * Remove the Updated flag from the DB
     */
    db_index(translog.index);
    get_record();
    if ((translog.flag&TRX_ADD) == TRX_ADD) {
	if ((tapes.flag&VALID) == VALID && 
	    strncmp(tapes.tapeid, translog.vsn, sizeof(tapes.tapeid)) == 0) {
	    tapes.flag = VALID;
	    put_record();
	}
    }
    else if ((translog.flag&TRX_DEL) == TRX_DEL) {
	if ((tapes.flag&VALID) != VALID &&     	
	    strncmp(tapes.tapeid, translog.vsn, sizeof(tapes.tapeid)) == 0) {
	    tapes.flag = INVALID;
	    put_record();
	}
    }
    /*
     * Clear the record from the transaction log
     */
    translog.flag = TRX_IGN;

    if (write(trx_fd, (char *)&translog, sizeof(translog)) != 
        sizeof(translog)) {
	printf("trx_clear: write error\n");
	return(-1);
    }
    if (index == first_valid_entry) {
	first_valid_entry++;

	while ((ret = read(trx_fd, (char *)&translog, sizeof(translog))) ==
	    sizeof(translog)) {
	    if (translog.flag = TRX_IGN)
		first_valid_entry++;
	    else
		break;
	}
	switch(ret) {
	case 0 :
	    /*
             * EOF on read.
             * Every record in the file from first_valid_entry to EOF
	     * is TRX_IGN. Truncate the file.
             */
	    lseek(trx_fd, 0L, SEEK_SET);
	    if (chsize(trx_fd, 0L) == 0) {
		first_valid_entry = 0L;
		next_read_entry = 0L;
	    }
	    if ((ret = fstat(trx_fd, &Sbuf)) == -1) {
		printf("trx_clear: stat error\n");
		return(-1);
	    }
	    translog_size = 0;
	    break;

	case -1:
	    printf("trx_clear");
	    return(-1);

	case sizeof(translog) :
	    break;

	default:
	    printf("trx_clear: ret = %d\n", ret);
	    return(-1);
	}
    }
    return(0);
}

/* 
 * Pack the transaction log
 */
void trx_log_pack()
{
    struct translog translog;
    struct section *sp;
    unsigned long read_offset, write_offset;

    write_offset = 0L;

    for (read_offset = next_read_entry*sizeof(struct translog); ;
      read_offset += sizeof(struct translog)) {

        if (lseek(trx_fd, read_offset, SEEK_SET) == -1L) {
  	    printf("seek error\n");
	    break;
        }

        if (read (trx_fd, (char *)&translog, sizeof(struct translog)) !=
          sizeof(translog))
	    break;

	if (translog.flag == TRX_IGN)
	    continue;

	if (read_offset == write_offset) {
            write_offset += sizeof(struct translog);
	    continue;
	}
	
        if (lseek(trx_fd, write_offset, SEEK_SET) == -1L) {
  	    printf("seek error\n");
	    break;
        }

        if (write (trx_fd, (char *)&translog, sizeof(struct translog)) ==
          sizeof(translog))
	    write_offset += sizeof(struct translog);
	else
	    break;
    }
    first_valid_entry = next_read_entry = 0L;
    chsize(trx_fd, write_offset);
}

