/*
 * WIN.C 
 * Version 1.1 Beta 7/16/93
 * J. Musch
 */

#include "tapelib.h"
#include "win.h"
#ifdef DOS
#include <telxon.h>
#else
#include "telxon.h"
#endif
#include <stdio.h>
#include <malloc.h>

struct win
*win_alloc() {

    struct win *wptr;
    int mode, cols, page;
    
    if ((wptr = (struct win *)malloc(sizeof(struct win))) == NULL)
    	return(NULL);
    	
    getvidmode(&mode, &cols, &page);	/* get the current page no. */
    
    wptr->win_height = -1;	/* -1 means automatically compute this */
    wptr->win_width = -1;	/* -1 means automatically compute this */
    wptr->win_vid = NORMAL_VID;
    wptr->win_screen = page;
    return(wptr);	
}

void win_size(win, args)
struct win *win;
char *args[];
{
    int len, height, width = 0;

    for (height = 0; height < WIN_MAXHEIGHT; height++) {
    	if (args[height] == NULL)
    	    break;
    	if ((len = strlen(args[height])) > width)
    	    width = len;
    }
    if (width > WIN_MAXWIDTH)
        width = WIN_MAXWIDTH;

    win->win_height = height;
    win->win_width = width;
}

void win_popup(win, args)
struct win *win;
char *args[];
{
    int i, j;
    char buf[25], map[25];
    
    if (win->win_height == -1 || win->win_width == -1)
        win_size(win, args);

    if (win->win_row < 1)  win->win_row = 1;	
        
    if (win->win_col < 1) win->win_col = 1;

    if (win->win_row + win->win_height > 15)
        win->win_height = (15 - win->win_row);

    if (win->win_col + win->win_width > 20)
        win->win_width = (20 - win->win_col);

    sprintf(map, "%%-%d.%ds", win->win_width, win->win_width);

    /*
     * Save the area we are going to overwrite.
     */
    for (i = 0; i < win->win_height+2; i++)
	for (j = 0; j < win->win_width+2; j++)
	    win->win_sbuf[i][j] = readchar(win->win_screen,
	      win->win_row+i-1, win->win_col+j-1);

    /* upper left corner */
    setcurpos(win->win_screen, win->win_row-1, win->win_col-1);
    writechar(0x10, win->win_vid, win->win_screen, 1);

    /* upper frame */
    setcurpos(win->win_screen, win->win_row-1, win->win_col);
    writechar(0x11, win->win_vid, win->win_screen, win->win_width);	
		

    /* upper right corner */
    setcurpos(win->win_screen, win->win_row-1, win->win_col+win->win_width);
    writechar(0x12, win->win_vid, win->win_screen, 1);

    for(i = 0; i < win->win_height; i++) {

	/* right frame */
	setcurpos(win->win_screen, win->win_row+i, win->win_col-1);
	writechar(0x13, win->win_vid, win->win_screen, 1);

	/* A line of data */
	setcurpos(win->win_screen, win->win_row+i, win->win_col);
	sprintf(buf, map, args[i]);
	writestr(win->win_screen, buf, win->win_width, win->win_vid);

	/* left frame */
	setcurpos(win->win_screen, win->win_row+i, 
		  win->win_col+win->win_width);
	writechar(0x13, win->win_vid, win->win_screen, 1);	
    }

    /* lower left corner */
    setcurpos(win->win_screen, win->win_row+win->win_height, win->win_col-1);
    writechar(0x15, win->win_vid, win->win_screen, 1);

    /* lower frame */
    setcurpos(win->win_screen, win->win_row+win->win_height, win->win_col);
    writechar(0x16, win->win_vid, win->win_screen, win->win_width);

    /* lower right corner */
    setcurpos(win->win_screen, win->win_row+win->win_height,
	      win->win_col+win->win_width);
    writechar(0x17, win->win_vid, win->win_screen, 1);
}

void win_free(win)
struct win *win;
{

    int i, j;

    /*
     * Restore the area we overwrote
     */
    for (i = 0; i < win->win_height+2; i++) {
	for (j = 0; j < win->win_width+2; j++) {
	
	    setcurpos(win->win_screen, win->win_row+i-1, win->win_col+j-1);
	    writechar((char)win->win_sbuf[i][j], (win->win_sbuf[i][j])>>8,
	       win->win_screen, 1);
	}
    }
    free(win);
}

