#include	<stdio.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<sys/time.h>
#include	<sys/param.h>

#include	"commline.h"
#include	"vmerror.h"
#include	"vltlib.h"
#include	"vconfig.h"

int		scannerfd;
int		db_vsn_fd;
struct		termios		term;
int		curr;
char		sendbuf[MAXCOMM - ZERO_PACKET];

#define		MAXREAD		255

 void
HangupLine()
{
	char	*id = "HangupLine";
	int	off = 0;
	int	modem;

	/*
	 * turn off dtr
	 */
	modem = TIOCM_DTR;
	IOCTL(scannerfd, TIOCMBIC, &modem);
	IOCTL(scannerfd, TCFLSH, TCIOFLUSH);
	/*
	 * turn off software carrier
	 */
	IOCTL(scannerfd, TIOCSSOFTCAR, &off);

	close(scannerfd);
}

 void
PrepLine()
{
	char	*id = "PrepLine";
	int	modem;

	if (( scannerfd = open(port, O_RDWR|O_NDELAY)) == -1)
		vLogMsg(EL_SYS|EL_FATAL, id, "open %s", port);
		
	/*
	 * turn on dtr and rts
	 */
	modem = TIOCM_RTS|TIOCM_DTR;
	IOCTL(scannerfd, TIOCMBIS, &modem);
	
	/*
	 * close line to wait for real open
	 */
	close(scannerfd);
	
	vLogMsg(EL_DEBUG1, id, "awaiting open");
	scannerfd = open(port, O_RDWR);
	vLogMsg(EL_DEBUG1, id, "open returned");
	
	/*
	 * set-up terminal line
	 */
	IOCTL(scannerfd, TCGETS, &term);
	term.c_iflag = IGNBRK;
	term.c_oflag = 0;
	term.c_cflag = B9600|CS8|CREAD;
	term.c_lflag = 0;
	term.c_cc[VMIN] = MAXREAD;	/* get this number of characters */
	term.c_cc[VTIME] = 1;		/* wait in 1/10 second units */
	IOCTL(scannerfd, TCSETS, &term);
	
	/*
	 * turn off rts
	 */
	modem = TIOCM_RTS;
	IOCTL(scannerfd, TIOCMBIC, &modem);
}


/*
 * Build a packet - return the size of the packet built
 */
 int
BuildPacket(out, mess, mlen)
 char	*out;		/* create packet here */
 char	*mess;		/* this is the message */
 int	mlen;		/* this is the length of the message */
{
	register ushort chksm;	/* working copy of checksum */
	register unsigned char c;
	unsigned int i, plen;
	char *p;
	
	chksm = 0;
	plen = mlen+HDR_SIZE+LEN_SIZE+CRC_SIZE;
	
	c = out[0] = SYN;
	c &= 0xff;
	chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
	
	c = out[1] = ((plen >>8) & 0xff);
	c &= 0xff;
	chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
	
	c = out[2] = (plen & 0xff);
	c &= 0xff;
	chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
	
	p = mess;
	for (i = 0; i < mlen; i++) {
		c = out[(MSG_OFFSET+i)] = *p++;
		c &= 0xff;
		chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
	}
	chksm &= 0xffff;
	
	out[(MSG_OFFSET+mlen)] = ((chksm >> 8) & 0xff);	/* MSB */
	out[(MSG_OFFSET+mlen+1)] = (chksm & 0xff);   	/* LSB */
	
	return(plen);
}

/*
**	SendPacket
**	Return a 1 on error, 0 otherwise.
*/
 int
SendPacket(mess, len)
 char	*mess;
 int	len;
{
	char	*id = "SendPacket";
	char	packet[MAXCOMM];
	int	i;
	int	ret = 1;

	if (len > 1)
		len = BuildPacket(packet, mess, len);
	else
		packet[0] = mess[0];

	IOCTL(scannerfd, TCFLSH, TCIOFLUSH);
	if (( i = write(scannerfd, packet, len)) == -1)
		vLogMsg(EL_SYS|EL_ERROR1, id, "scanner write");
	else if (i < len)
		vLogMsg(EL_ERROR1, id, "short write %d", i);
	else
		ret = 0;

	return ret;
}

 int
ReadComm(buf, size)
 char	*buf;
 int	size;
{
	char	*id = "ReadComm";
	int	got, extra;

	got = read(scannerfd, buf, size);
	switch (got) {
	case -1:
		vLogMsg(EL_SYS|EL_ERROR1, id, "read error");
		break;
	case 0:
		vLogMsg(EL_ERROR1, id, "zero length read");
		break;

	default:
		term.c_cc[VMIN] = 0;
		IOCTL(scannerfd, TCSETS, &term);
		for (;;) {
			if (size == got)
				break;
			extra = read(scannerfd, &buf[got], size-got);
			vLogMsg(EL_DEBUG3, id, "extra read %d", extra);
			if (extra == 0)
				break;
			else if (extra < 0) {
				vLogMsg(EL_SYS|EL_ERROR1, id, "read error");
				break;
			}
			else
				got += extra;
		}
		term.c_cc[VMIN] = MAXREAD;;
		IOCTL(scannerfd, TCSETS, &term);

		vLogMsg(EL_DEBUG2, id, "read %d chars", got);
		break;
	}
	return(got);
}

 int
GoodPacket(bufp, len)
 char	**bufp;
 int	*len;
{
	char	*id = "GoodPacket";
	char	*packet;
	u_char	c;
	int	ret = 0;
	int	i, hold, start = 0;
	ushort	plength, chksm;
	union	{
		u_short	i;
		u_char	c[2];
	} ctos;

	packet = *bufp;
	*bufp = NULL;

	for (start=0;;start=hold+1) {
		for (i=start; i < *len; i++)	/* look for start char */
			if (packet[i] == SYN)
				break;
		if ((*len - i) < ZERO_PACKET) {	/* not enough for a real pkt */
			vLogMsg(EL_ERROR1, id,
				"no SYN found or short pkt");
			break;
		}
		hold = i++;
		ctos.c[1] = packet[i++];
		ctos.c[0] = packet[i];
		plength = ctos.i;

		vLogMsg(EL_DEBUG5, id, "packet len %d", plength);
		if ((plength < ZERO_PACKET) ||
		    ((*len - start) < plength)) {
			vLogMsg(EL_ERROR1, id,
				"bad packet length %d data read %d",
			       plength, *len - start);
			continue;
		}

                /*
                 * compute CRC on local packet
                 */
                chksm = 0;
                for (i=hold; i < plength - CRC_SIZE; i++) {
			c = packet[i] & 0xff;
			chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];
                }
                chksm &= 0xffff;
		ctos.c[1] = packet[i++];
		ctos.c[0] = packet[i];
		if (chksm != ctos.i) {
			vLogMsg(EL_ERROR1, id, "CRC bad %04.4x %04.4x",
				chksm, ctos.i);
			continue;
		}

		/*
		**  We made it!  A good packet is here.
		*/
		ret = 1;
		*len = plength - ZERO_PACKET;
		*bufp = *len ? &packet[hold+MSG_OFFSET] : NULL;
		vLogMsg(EL_DEBUG2, id, "ok: (%d)", *len);
		break;
	}
	return ret;
}		

#define	RETRY	3

/*
**	GetPacket
**	Read comm line and crack a packet.  Return 1 on error, 0 if ok.
**	The first argument is a pointer char pointer which will be set
**	to the packet read
*/
 int
GetPacket(bufp, len)
 char	**bufp;
 int	*len;
{
	char	*id = "GetPacket";
	int	i, retval = 1;
	static	char	inbuf[MAXCOMM];
	static	int	got;

	vLogMsg(EL_DEBUG3, id, "entry");

	for (i=0; i<RETRY; i++) {
		*bufp = NULL;
		*len = 0;
		
		vLogMsg(EL_DEBUG3, id, "try %d", i);
		got = ReadComm(inbuf, MAXCOMM);
		switch (got) {
			
		case -1:
		case 0:
			break;
			
		case 1:				/* single char read */
			switch (inbuf[0]) {	/* see what it is */
			case ACK:
			case ETX:
			case NAK:
			case ENQ:
			case EOT:
			case RVI:
				vLogMsg(EL_DEBUG2, id,
					"got single char 0x%x", inbuf[0]);
				retval = 0;
				*bufp = inbuf;
				*len = got;
				break;
			default:
				vLogMsg(EL_ERROR1,
					"bad single char %d", inbuf[0]);
				break;
			}
			break;
			
		default:
			*len = got;
			*bufp = inbuf;
			if (GoodPacket(bufp, len))
				retval = 0;
			break;
		}
		
		if (retval) {
			vLogMsg(EL_STATUS1, id, "send NAK");
			inbuf[0] = NAK;
			SendPacket(inbuf, 1);
		}
		else
			break;
	}

	return(retval);
}

/*
**	Check a comm character to see if it show a good send.
**	Return 1 if it does, 0 otherwise.
*/
 int
CheckChar(ch)
 char	ch;
{
	switch (ch) {	/* see what it is */
	case ACK:
		return(1);
	case ETX:
		vLogMsg(EL_STATUS1, NULL,
			"battery low indication");
		HangupLine();
		sleep(600);
		PrepLine();
	default:
		return(0);
	}
}

/*
 *  toInt() - check & translate char -> int
 */

 static int
toInt(num, str)
 int	*num;
 char	*str;
{
	char	*id = "db to integer";
	int     i;

	if (str[INTLEN] != ' ') {
		vLogMsg(EL_ERROR1, id,
			"int field discrepancy (%.*s)",
			INTLEN+1, str);
		return(1);
	}
	for (i=0; i<INTLEN; i++)
		if (str[i] != ' '  &&  !isdigit(str[i])  &&  str[i] != '-') {
			vLogMsg(EL_ERROR1, id, 
				"int field (%.*s) has non-numeral",
				INTLEN, str);
			return(1);
		}
	*num = atoi(str);
	return(0);
}

CloseDB()
{
	/*
	 *  release lock
	 */

	if (unlink(DB_LOCK) == -1)
		vLogMsg(EL_ERROR1, NULL,
			"unlink lock file (%s)", DB_LOCK);

	vLogMsg(EL_STATUS1, NULL, "Database shut down");
}

die()
{
	CloseDB();
	HangupLine();
	exit(1);
}
	

 void
TruncStr(str, len)
 char	*str;
 int	len;
{
	int	i;

	for (i=len-1; i>=0; i--)
		if (str[i] != ' ')
			break;
	i++;
	if ((0 <= i) && (i < len) &&		/* in range */
	    str[i] == ' ')			/* and is last blank */
		str[i] = '\0';
}

 int
Xfer()
{
	char	*id = "Xfer";
	char	*packet;
	int	ret, i;

	for (i=0; i<RETRY; i++) {
		vLogMsg(EL_DEBUG2, id, "try %d", i);

		if (SendPacket(sendbuf, curr))
			return 1;
		if (GetPacket(&packet, &ret))
			return 1;
		vLogMsg(EL_DEBUG1, id, "got %d", ret);
		if (ret == 1) {
			if (CheckChar(packet[0]))
				break;
		}
		else {
			vLogMsg(EL_ERROR1, id,
				"error return %.*s",
				ret, packet);
			break;
		}
	}
	return 0;
}

 int
PutMess(mess, len)
 char	*mess;
 int	len;
{
	char	*id = "PutMess";

	if ((curr + len) > (MAXCOMM - ZERO_PACKET)) {
		vLogMsg(EL_DEBUG1, id,
			"message len %d", curr);
		if (Xfer())
			die();
		curr = 0;
	}
	strncpy(&sendbuf[curr], mess, len);
	curr += len;
	return 0;
}

main(argc, argv)
 int	argc;
 char	*argv[];
{
	struct	stat	sbuf;
	extern	int	vLogLevel;
	vsnrec_t	vsnbuf;
	locrec_t	*lrp;
	vaultrec_t	*vrp;
	int		i, lastloc, vrow, vslot, len;
	char		mess[100];

	vLogInit(argv[0], die, EL_TOSTDERR, "gundb.log", NULL);
	vLogLevel = EL_DEBUG4;
	if (stat(DB_LOCK, &sbuf) == 0) {
		vLogMsg(EL_ERROR1, NULL, "db in use");
		exit(1);
	}

	signal(SIGINT, die);
	signal(SIGQUIT, die);
	signal(SIGTERM, die);

	/*
	 *  db about to be set up, so lock
	 */
	if (open(DB_LOCK, O_CREAT|O_EXCL, 0600) == -1)
		vLogMsg(EL_FATAL|EL_SYS, NULL, "create lock failed");

	if ((db_vsn_fd = open(DB_VSN_TBL, O_RDONLY))== -1) {
		vLogMsg(EL_FATAL|EL_SYS, NULL,
			"vsn table (%s) open failed: %s", DB_VSN_TBL);
	}

	PrepLine();
	while (read(db_vsn_fd, &vsnbuf, sizeof(vsnrec_t)) ==
	       sizeof(vsnrec_t)) {

		/*
		 *  basic record sanity check
		 */
		if (vsnbuf.end != '\n') {
			vLogMsg(EL_ERROR1, NULL, 
				"error on (%.6s): bad record terminator", 
				vsnbuf.v_vsn);
			continue;
		}
		
		/*
		 *  see if record is really there
		 */
		if (vsnbuf.v_status == S_DELETED)
			continue;
		
		
		if (vsnbuf.v_vsn[VSNLEN] != ' ') {
			vLogMsg(EL_ERROR1, NULL,
				"%.7s: no space seperator after vsn", 
				vsnbuf.v_vsn);
			continue;
		}
		for (i=0; i<VSNLEN; i++) {
			if (vsnbuf.v_vsn[i] == ' ') {
				vLogMsg(EL_ERROR1, NULL,
					"vsn %.6s: too short", 
					vsnbuf.v_vsn);
				continue;
			}
			if (!isalnum(vsnbuf.v_vsn[i])) {
				vLogMsg(EL_ERROR1, NULL,
					"vsn %.6s: Illegal character (%c)",
					vsnbuf.v_vsn, vsnbuf.v_vsn[i]);
				continue;
			}
		}

		/*
		 *  check to make sure last known location is ok
		 */
		
		lastloc = vsnbuf.v_lastloc - '0';
		if (lastloc < 0 || HISTORY <= lastloc) {
			vLogMsg(EL_ERROR1, NULL, 
				"%.6s: bad location '%c' in history array",
				vsnbuf.v_vsn, vsnbuf.v_lastloc);
			continue;
		}

		lrp = &vsnbuf.v_loc[lastloc];
		if (lrp->l_type != VAULT)
			continue;

		vrp = &lrp->l_field.vu_vault;

		TruncStr(vrp->vr_section, SECSIZE);
		if (toInt(&vrow, vrp->vr_row) ||
		    toInt(&vslot, vrp->vr_slot)) {
			vLogMsg(EL_ERROR1, NULL,
				"%.6s: dbase translation error",
				vsnbuf.v_vsn);
			continue;
		}

		vLogMsg(EL_DEBUG1, NULL,
			"xtrasmit %.*s %s %d %d", VSNLEN, vsnbuf.v_vsn,
			vrp->vr_section, vrow, vslot);
		sprintf(mess, "%d %.*s %s %d %d\n", SCANNER_UPDATE,
			VSNLEN, vsnbuf.v_vsn,
			vrp->vr_section,
			vrow, vslot);

		if (PutMess(mess, strlen(mess)))
			break;
	}
	Xfer();			/* flush buffer */
	CloseDB();
	HangupLine();
	exit(0);
}
