#include	<stdio.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/time.h>
#include	<sys/param.h>

#include	"vltlib.h"
#include	"vconfig.h"

usage(name)
 char	*name;
{
	fprintf(stderr, "usage: %s [-v][-u][-g]\n", name);
	exit(2);
}

#define	PRINT_VAULT	0x01
#define	PRINT_UNKNOWN	0x02
#define	PRINT_GENERAL	0x04

main(argc, argv)
 int	argc;
 char	*argv[];
{
	struct	stat	sbuf;
	extern	int	vLogLevel;
	vsnrec_t	vsnbuf;
	locrec_t	*lrp;
	vaultrec_t	*vrp;
	char		*arg;
	int		i, lastloc;
	int		db_vsn_fd;
	int		flg = 0;
	int		type;

	for (i=1; i<argc; i++) {
		char	*arg = argv[i];

		if (arg[0] == '-') {
			if (strlen(arg) != 2)
				usage(argv[0]);

			switch (arg[1]) {
			case 'v':
				flg |= PRINT_VAULT;
				break;
			case 'u':
				flg |= PRINT_UNKNOWN;
				break;
			case 'g':
				flg |= PRINT_GENERAL;
				break;
			default:
				usage(argv[0]);
				break;
			}
		}
	}
	if (flg == 0)
		flg = PRINT_VAULT;
	
	if ((db_vsn_fd = open(DB_VSN_TBL, O_RDONLY))== -1) {
		fprintf(stderr,
			"vsn table (%s) open failed: %s\n", DB_VSN_TBL);
	}

	while (read(db_vsn_fd, &vsnbuf, sizeof(vsnrec_t)) ==
	       sizeof(vsnrec_t)) {

		/*
		 *  basic record sanity check
		 */
		if (vsnbuf.end != '\n') {
			fprintf(stderr, 
				"error on (%.6s): bad record terminator\n", 
				vsnbuf.v_vsn);
			continue;
		}
		
		/*
		 *  see if record is really there
		 */
		if (vsnbuf.v_status == S_DELETED)
			continue;

		if (vsnbuf.v_vsn[VSNLEN] != ' ') {
			fprintf(stderr,
				"%.7s: no space seperator after vsn\n", 
				vsnbuf.v_vsn);
			continue;
		}
		for (i=0; i<VSNLEN; i++) {
			if (vsnbuf.v_vsn[i] == ' ') {
				fprintf(stderr,
					"vsn %.6s: too short\n",
					vsnbuf.v_vsn);
				continue;
			}
			if (!isalnum(vsnbuf.v_vsn[i])) {
				fprintf(stderr,
					"vsn %.6s: Illegal character (%c)\n",
					vsnbuf.v_vsn, vsnbuf.v_vsn[i]);
				continue;
			}
		}

		/*
		 *  check to make sure last known location is ok
		 */
		
		lastloc = vsnbuf.v_lastloc - '0';
		if (lastloc < 0 || HISTORY <= lastloc) {
			fprintf(stderr, 
				"%.6s: bad location '%c' in history array\n",
				vsnbuf.v_vsn, vsnbuf.v_lastloc);
			continue;
		}

		lrp = &vsnbuf.v_loc[lastloc];

		switch (lrp->l_type) {
		case VAULT:
			type = PRINT_VAULT;
			break;
		case UNKNOWN:
			type = PRINT_UNKNOWN;
			break;
		case DISPLAY:
			type = PRINT_GENERAL;
			break;
		default:
			fprintf(stderr,
				"%.6s: bad location type %c\n",
				vsnbuf.v_vsn, lrp->l_type);
			break;
		}
		if (type & flg) {
			printf("%.*s\t", VSNLEN, vsnbuf.v_vsn);

			switch (lrp->l_type) {
			case VAULT:
				vrp = &lrp->l_field.vu_vault;
				printf("%.*s %.*s %.*s\n",
				       SECSIZE, vrp->vr_section,
				       INTLEN, vrp->vr_row,
				       INTLEN, vrp->vr_slot);
				break;
			case UNKNOWN:
				printf("unknown\n");
				break;
			case DISPLAY:
				printf("%.*s\n", LOCSIZE,
				       lrp->l_field.vu_general);
				break;
			}
		}
	}
	exit(0);
}
