#include	<stdio.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<string.h>
#include	<signal.h>
#include	<sys/types.h>
#include	<sys/time.h>
#include	<sys/param.h>

#include	<rpc/rpc.h>
#include	"vmerror.h"
#include	"vltlib.h"
#include	"vconfig.h"

/*
**	Scanner states
*/
#define		OUT		0
#define		GET_STATUS	1
#define		SEND_QUEUE	2
#define		POLL_LOOP	3

struct	messageq {
	struct	messageq	*m_forw;
	struct	messageq	*m_back;
	int			m_command;	/* command to scanner */
	vsn_t			m_vsn;		/* if any */
	spacetime_t		*m_stp;		/* for delete */
};

struct	auth {
	char		*a_mach;
	uid_t		a_uid;
	struct	auth	*a_link;
};

int	badtime;
int	NumUsers;
int	scannerfd = -1;
int	VMError;
int	ScannerState = OUT;
spacetime_t		*PrevLoc;		/* set when by UpdateVsn */
struct	displaylist	*displaylist = NULL;
struct	auth		*AuthUser = NULL;
struct	timeval		*tmhold, *tmout = NULL;
struct	messageq	messagehead = {
	&messagehead,
	&messagehead,
	NULL,
	0
};

 char	*
GetMem(len)
 int	len;
{
	char	*ret;

	if ((ret=(char *)malloc(len)) == NULL)
		vLogMsg(EL_ERROR1, NULL, "malloc failed");
	return ret;
}


/*
**	Come here if a fatal error occurs.
*/
 void
die()
{
	extern	pid_t	scanner_child;

	signal(SIGHUP, SIG_IGN);	/* forget about other signals */
	signal(SIGCHLD, SIG_IGN);
	signal(SIGPIPE, SIG_IGN);
	signal(SIGINT, SIG_IGN);
	signal(SIGQUIT, SIG_IGN);

	CloseDb();
	HangupLine();
	if (scanner_child != 0)		/* kill our kid */
		kill(scanner_child, SIGKILL);
	DestroyDisplay();
	exit(1);
}

/*
**	Come here if a signal is received
*/
 void
GotSignal(sig, code, scp, addr)
 int	sig, code;
 struct	sigcontext	*scp;
 char	*addr;
{
	vLogMsg(EL_ERROR1, NULL, "entered sig %d code 0x%x", sig, code);
	die();
}

/*
**	GetAuth
**	Routine used to read in a list of user we want to use this.
*/
 void
GetAuth()
{
	char	*id = "GetAuth";
	char	line[100], machine[80];
	int	number;
	struct	auth	*ap;
	int	uid;
	FILE	*fp;

	if ((fp=fopen(AUTHLIST, "r")) == NULL) {
		vLogMsg(EL_FATAL|EL_SYS, id,
			"error on open: %s", AUTHLIST);
	}
	for (number=1;;number++) {
		if (fgets(line, sizeof(line), fp) == NULL)
			break;
		if (line[0] == '!' ||
		    line[0] == '#' ||		/* comment */
		    line[0] == '\n')		/* blank */
			continue;
		if (sscanf(line, "%s %d", machine, &uid) != 2) {
			vLogMsg(EL_ERROR2, id,
				"line %d garbled: %s", number, line);
			continue;
		}
		vLogMsg(EL_DEBUG2, id, "got %s %d", machine, uid);
		if ((ap=(struct auth *)GetMem(sizeof(struct auth))) == NULL)
			break;
		if ((ap->a_mach=GetMem(strlen(machine))) == NULL)
			break;
		strcpy(ap->a_mach, machine);
		ap->a_uid = (uid_t)uid;
		ap->a_link = AuthUser;
		AuthUser = ap;
	}
	fclose(fp);
	if (AuthUser == NULL) {
		vLogMsg(EL_ERROR1|EL_FATAL, id,
			"no user's authorized");
	}
}

/*
**	CheckAuth
**	Check the ability of a user to call any routine.
**	return pointer to auth struct if ok, return NULL otherwise
*/
 static	struct	auth*
CheckAuth(rqstp)
 struct	svc_req	*rqstp;
{
	char	*id = "CheckAuth";
	struct	authunix_parms	*uc;
	struct	auth	*ap;

	switch (rqstp->rq_cred.oa_flavor) {
	case AUTH_UNIX:
		uc = (struct authunix_parms *)rqstp->rq_clntcred;
		vLogMsg(EL_DEBUG3, id,
			"got message from %d on %s",
			uc->aup_uid, uc->aup_machname);
		for (ap=AuthUser; ap; ap=ap->a_link) {
			if (strcmp(uc->aup_machname, ap->a_mach))
				continue;
			if (uc->aup_uid == ap->a_uid)
				return(ap);
		}
		break;
	case AUTH_NULL:
		vLogMsg(EL_ERROR2, id, "AUTH_NULL");
		break;
	default:
		break;
	}
	return(NULL);
}

/*
**	Return a pointer to a string which for the location lp.
*/
 char	*
LocationString(lp)
 location_t	*lp;
{
	static	char	locstr[100];

	switch (lp->tag) {

	case    unknown:
		sprintf(locstr, "UNKNOWN");
		break;

	case    vault:
		sprintf(locstr, "section %.*s row %d slot %d",
			SECSIZE, lp->section,
			lp->row, lp->slot);
		break;

	case    display:
		strcpy(locstr, lp->general);
		break;

	default:
		sprintf(locstr, "BAD LOCATION");
		break;
	}
	return(locstr);
}

/*
**	Add a request to the list being displayed.
*/
 int
AddDisplay(vsnreq, machine)
 vsnreq_t	*vsnreq;
 char		*machine;
{
	struct	displaylist	*dlp;
	char	*id = "AddDisplay";

	vLogMsg(EL_DEBUG2, id, "entered %.6s %s",
		&vsnreq->vsn, machine);

	for (dlp=displaylist; dlp; dlp=dlp->dl_next) {
		if (memcmp(&vsnreq->vsn, &dlp->dl_vsnreq.vsn, VSNLEN) == 0)
			break;
	}
	if (dlp)
		return(E_USER);

	dlp=(struct displaylist *)GetMem(sizeof(struct displaylist));
	if (dlp == NULL)
		return(E_NOMEM);

	memcpy((char *)&dlp->dl_vsnreq, vsnreq, sizeof(vsnreq_t));
	dlp->dl_time = time((time_t *)0);
	strncpy(dlp->dl_mach, machine, sizeof(dlp->dl_mach));
	dlp->dl_next = displaylist;
	displaylist = dlp;

	DisplayList();
	return(E_NOERROR);
}

/*
**	Build packet and save it in a queue to send when the gun
**	is available.
*/
 void
PutMessage(command, vsn, stp)
 int		command;
 vsn_t		*vsn;
 spacetime_t	*stp;
{
	char	*id = "PutMessage";
	struct	messageq	*me;

	vLogMsg(EL_DEBUG2, id, "entered cmd %d", command);
				/* malloc list element and save mess */
	if ((me=(struct messageq *)GetMem(sizeof(struct messageq))) == NULL)
		return;

	me->m_command = command;
	if (vsn)
		memcpy(&me->m_vsn, vsn, VSNLEN);
	else
		memset(&me->m_vsn, '\0', VSNLEN);
	if (stp) {
		if ((me->m_stp=
		     (spacetime_t *)GetMem(sizeof(spacetime_t))) == NULL) {
			free(me);
			return;
		}
		*(me->m_stp) = *stp;
	}
	else
		me->m_stp = NULL;
	me->m_forw = messagehead.m_forw;
	me->m_back = &messagehead;
	messagehead.m_forw = me;
	me->m_forw->m_back = me;
}

/*
**  Handle bad times from the scanner
*/
 void
TimeCheck(t)
 time_t	*t;
{
	static	time_t	last = 0;
	time_t		now;

	if (badtime) {			/* ignore times sent */
		*t = last;
		return;
	}

	now = time((time_t *) 0);
	if ((*t < last) || (now < *t)) {
		vLogMsg(EL_ERROR2, NULL, "bad time sent %d", *t);
		badtime = 1;
		*t = last = now;
		PutMessage(SCANNER_RESET, NULL, NULL);
		return;
	}
	last = *t;
}

/*
**	Add or update the database.  Tell the scanner too.
*/
 int	*
vault_update_1(vsnloc, rqstp)
 vsnloc_t	*vsnloc;
 struct	svc_req	*rqstp;
{
	static	int	retval;
	char	*id = "vault_update_1";

	vLogMsg(EL_DEBUG2, id, "entered %.6s", &vsnloc->vsn);

	if (CheckAuth(rqstp) == NULL) {
		retval = E_PERM;
		return(&retval);
	}

	retval = E_NOERROR;
	if (vsnloc == NULL) {
		vLogMsg(EL_DEBUG1, id, "NULL");
		retval = E_USER;
		return(&retval);
	}

	if (UpdateVsn(vsnloc))
		retval = VMError;
	else if (PrevLoc != &vsnloc->location) {	/* not the same loc */
		if (PrevLoc && PrevLoc->where.tag == vault)
			PutMessage(SCANNER_DELETE, vsnloc->vsn, PrevLoc);
		if (vsnloc->location.where.tag == vault)
			PutMessage(SCANNER_UPDATE, vsnloc->vsn, NULL);
	}
	return(&retval);
}

/*
**	Delete a vsn from the database.  Tell the scanner too.
*/
 int	*
vault_delete_1(vsn, rqstp)
 vsn_t		*vsn;
 struct	svc_req	*rqstp;
{
	static	int	retval;
	char	*id = "vault_delete_1";
	spacetime_t	*stp;

	vLogMsg(EL_DEBUG2, id, "entered %.6s", vsn);

	if (CheckAuth(rqstp) == NULL) {
		retval = E_PERM;
		return(&retval);
	}

	retval = E_NOERROR;
	if (vsn == NULL) {
		vLogMsg(EL_DEBUG1, id, "NULL");
		retval = E_USER;
		return(&retval);
	}

	if ((stp = DeleteVsn(vsn)) == NULL)
		retval = VMError;
	else if (stp->where.tag == vault)
		PutMessage(SCANNER_DELETE, vsn, stp);

	return(&retval);
}

/*
**	Request information about a vsn.
*/
 spacetime_t	*
vault_query_1(vsn, rqstp)
 vsn_t	*vsn;
 struct	svc_req	*rqstp;
{
	static	spacetime_t	retval;
	spacetime_t		*stp;
	char	*id = "vault_query_1";

	vLogMsg(EL_DEBUG2, id, "entered %.6s", vsn);

	memset(&retval, '\0', sizeof(retval));

	(void)CheckAuth(rqstp);

	if (vsn == NULL)
		vLogMsg(EL_DEBUG1, id, "NULL");
	else {
		if ((stp = GetVsn(vsn)) != NULL)
			memcpy(&retval, stp, sizeof(retval));
	}
	return(&retval);
}

/*
**	Make a request for a vsn.  The message string will be displayed
**	until canceled.
*/
 int	*
vault_request_1(vsnreq, rqstp)
 vsnreq_t	*vsnreq;
 struct	svc_req	*rqstp;
{
	static	int	retval;
	char	*id = "vault_request_1";
	spacetime_t	*stp;
	struct	auth	*ap;

	vLogMsg(EL_DEBUG2, id, "entered %.6s", &vsnreq->vsn);

	if ((ap = CheckAuth(rqstp)) == NULL) {
		retval = E_PERM;
		return(&retval);
	}

	if (vsnreq == NULL) {
		vLogMsg(EL_DEBUG1, id, "NULL");
		retval = E_USER;
		return(&retval);
	}

	if ((stp = GetVsn(&vsnreq->vsn)) == NULL) {
		retval = VMError;
		return(&retval);
	}

	if ((retval = AddDisplay(vsnreq, ap->a_mach)) == E_NOERROR)
		PutMessage(SCANNER_REQUEST, &vsnreq->vsn, NULL);

	return(&retval);
}

/*
**	Cancel any request for a vsn.
*/
 int	*
vault_cancel_1(vsn, rqstp)
 vsn_t	*vsn;
 struct	svc_req	*rqstp;
{
	static	retval;
	struct	displaylist	*dlp, *prev;
	char	*id = "vault_cancel_1";

	vLogMsg(EL_DEBUG2, id, "entered %.6s", vsn);

	if (CheckAuth(rqstp) == NULL) {
		retval = E_PERM;
		return(&retval);
	}

	if (vsn == NULL) {
		vLogMsg(EL_ERROR2, id, "NULL");
		retval = E_USER;
		return(&retval);
	}

	for (prev=NULL, dlp=displaylist; dlp; prev=dlp, dlp=dlp->dl_next) {
		if (memcmp(vsn, &dlp->dl_vsnreq.vsn, VSNLEN) == 0)
			break;
	}
	if (dlp) {
		if (prev == NULL)
			displaylist = dlp->dl_next;
		else
			prev->dl_next = dlp->dl_next;
		free(dlp);

		PutMessage(SCANNER_CANCEL, vsn, NULL);
		retval = E_NOERROR;
		DisplayList();
	}
	else
		retval = E_NOTFOUND;

	return(&retval);
}

 void
PollLoop()
{
	char	*id = "PollLoop";
	static	struct	timeval	timeout = {
		(long)(60*2),	/* seconds */
		0L,		/* microseconds */
	};

	vLogMsg(EL_DEBUG1, id, "entered");
	ScannerState = POLL_LOOP;
	tmhold = tmout = &timeout;
	HangupLine();
}

 void
SendQueue()
{
	char	*id = "SendQueue";
	int	len, command;
	char	mess[80];
	struct	messageq	*me;
	spacetime_t		*stp;
	vsn_t			*vsn;

	vLogMsg(EL_DEBUG3, id, "entered");

	if (scannerfd == -1) {
		vLogMsg(EL_STATUS2, id, "scanner not open");
		return;
	}

	for (;;) {
		me=messagehead.m_back;
		if (me == &messagehead) {
			vLogMsg(EL_DEBUG3, id, "done");
			PollLoop();
			return;
		}
		
		mess[0] = '\0';
		command = me->m_command;
		vsn = &me->m_vsn;
		stp = me->m_stp;
		ScannerState = SEND_QUEUE;
		
		/* send first entry on queue */
		vLogMsg(EL_DEBUG2, id, "doing cmd: %d %.*s",
			command, VSNLEN, vsn);
		
		switch (command) {
			
		case SCANNER_RESET:
			sprintf(mess, "%d %d\n", command,
				time((time_t *) 0));
			break;

		case SCANNER_UPDATE:
		case SCANNER_REQUEST:
			stp = GetVsn(vsn);
			
		case SCANNER_DELETE:
			if (stp == NULL)
				break;
			if (stp->where.tag != vault)
				break;
			sprintf(mess, "%d %.*s %s %d %d\n", command,
				VSNLEN, vsn,
				stp->where.section,
				stp->where.row,
				stp->where.slot);
			break;

		case SCANNER_CANCEL:
			sprintf(mess, "%d %.*s\n", command,
				VSNLEN, vsn);
			break;
			
		default:
			vLogMsg(EL_ERROR1, id,
				"bad command queued %d", command);
			break;
		}
		if ((len = strlen(mess)) == 0) {	/* error */
			vLogMsg(EL_STATUS1, id,
				"couldn't send command %d %.*s",
				command, VSNLEN, vsn);
			me = messagehead.m_back;
			me->m_back->m_forw = &messagehead;
			messagehead.m_back = me->m_back;
			if (me->m_stp)
				free(me->m_stp);
			free(me);
		}
		else {
			SendPacket(mess, len);
			return;
		}
	}
}

 void
GetResponce(mess, len)
 char	*mess;
 int	len;
{
	char		*id = "GetResponce";
	struct	messageq	*me;

	vLogMsg(EL_DEBUG1, id, "entered");

	if (len == 1) {
		if (CheckChar(mess[0])) {	/* bad return */
			SendQueue();
			return;
		}
	}
	else {					/* error packet */
		vLogMsg(EL_ERROR1, id,
			"error return from scanner: %.*s",
			len, mess);
	}
			/* delete queue entry */
	me = messagehead.m_back;
	me->m_back->m_forw = &messagehead;
	messagehead.m_back = me->m_back;
	if (me->m_stp)
		free(me->m_stp);
	free(me);
	SendQueue();
}

 void
RequestStatus()
{
	char	*id = "RequestStatus";
	char	mess[20];

	vLogMsg(EL_DEBUG2, id, "entered");

	ScannerState = GET_STATUS;
	badtime = 0;
	sprintf(mess, "%d\n", SCANNER_STATUS);
	SendPacket(mess, strlen(mess));
}

 void
GetStatus(mess, len)
 char	*mess;
 int	len;
{
	char		*id = "GetStatus";
	static		vsnloc_t	scanvl;
	spacetime_t	*stp;
	int		i, j;

	vLogMsg(EL_DEBUG1, id, "entered");

	if (len == 0) {		/* zero len message => EOT */
		SendQueue();
		return;
	}
	else if (len == 1) {
		if (CheckChar(mess[0]))		/* bad return */
			RequestStatus();
		else		/* good return, done with status */
			SendQueue();
		return;
	}

	for (i=0; i<len; i=j+1) {
		int	fields;

		/* limit sscanf to one "record" */
		for (j=i; j<len; j++)
			if (mess[j] == '\n')
				break;
		if (j < len)
			mess[j] = '\0';

		/* break out vsn and spacetime from mess */
		fields = sscanf(&mess[i], "%s %s %d %d %ld",
		       &scanvl.vsn,
		       scanvl.location.where.section,
		       &scanvl.location.where.row,
		       &scanvl.location.where.slot,
		       &scanvl.location.when);
		
		if (fields != 5) {
			vLogMsg(EL_ERROR1, id,
				"conversion of %d status fields: %s",
				fields, &mess[i]);
			continue;
		}

		if (CheckVsn((char *)&scanvl.vsn))
			continue;

		if (strcmp(scanvl.location.where.section, "?") == 0) {
			scanvl.location.where.tag = unknown;
			vLogMsg(EL_DEBUG2, id,
				"unknown location for vsn %s", &scanvl.vsn);
		}
		else {
			scanvl.location.where.tag = vault;
			vLogMsg(EL_DEBUG2, id, "vsn %s in %s",
				&scanvl.vsn,
				LocationString(&scanvl.location.where));
		}
		TimeCheck(&scanvl.location.when);
		UpdateVsn(&scanvl);

		/*
		 *  if the both the old and new locations are in
		 *  the vault, send the scanner a delete for the old loc
		 */
		if (PrevLoc != &scanvl.location &&
		    PrevLoc && PrevLoc->where.tag == vault &&
		    scanvl.location.where.tag == vault)
			PutMessage(SCANNER_DELETE, &scanvl.vsn, PrevLoc);
	}
	SendAck();
}

/*
**	Read information from the scanner.
*/
 void
ScannerRead()
{
	char		*id = "ScannerRead";
	char		*packet;
	int		len;

	vLogMsg(EL_DEBUG2, id, "entered");

	tmout = tmhold;
	if (GetPacket(&packet, &len))
		return;
	vLogMsg(EL_DEBUG1, id, "got %d", len);

	switch (ScannerState) {
	case OUT:
		vLogMsg(EL_ERROR1, id, "read %d when scanner is out", len);
		break;
	case GET_STATUS:
		GetStatus(packet, len);
		break;
	case SEND_QUEUE:
		GetResponce(packet, len);
		break;
	default:
		vLogMsg(EL_ERROR1, id, "scanner in strange state %d",
			ScannerState);
		break;
	}
	return;
}

/*
**	getkid
**	return 1 if ok, 0 if not
*/
 int
getkid()
{
	int	ret;
	char	*id = "getkid";

	if (wait(&ret) == -1) {
		vLogMsg(EL_ERROR1|EL_SYS, id, "wait error");
		return(0);
	}
	if (ret) {
		vLogMsg(EL_ERROR1, id, "child returned error 0x%x", ret);
		return(0);
	}
	else
		return(1);

}

 void
DeadScanner()
{
	switch (ScannerState) {
	case OUT:
	case POLL_LOOP:
		break;
	default:
		ScannerState = OUT;
		tmout = NULL;
		HangupLine();
		PrepLine();
		if (scannerfd == -1)
			PollLoop();
		break;
	}
}

 void
LiveScanner()
{
	static	struct	timeval	timeout = {
		20L,	/* seconds */
		0L,	/* microseconds */
	};

	if (getkid()) {		/* reap child */
		tmhold = tmout = &timeout;
		SetupComm();
		RequestStatus();
	}
	else			/* bad return */
		PollLoop();
}

 void
TimeOut()
{
	if (ScannerState == POLL_LOOP) {
		ScannerState = OUT;
		tmout = NULL;
		PrepLine();
		if (scannerfd == -1)
			PollLoop();
	}
	else {
		tmout = tmhold;
		SendEnquire();
	}
}
		
/*
**	This is the "main" routine for the server.  It is called
**	from the real main created by rpcgen.
*/
 void
svc_run()
{
	char	*id = "svc_run";
	struct	sigvec	sv;

	extern	int	vLogLevel;
	fd_set	readfds;
	time_t	before;
	struct	timeval		temptime;
	int	num, maxfd;
	int	time_expired;

        setsid();
	vLogInit("vaultdaemon", die, EL_TOSTDERR|EL_TOFILE|EL_RENAMELOG,
		 NULL, NULL);
	vLogLevel = EL_DEBUG2;

	GetAuth();		/* figure out who can use the db */
	InitDb();		/* open database */
	DisplayInit();

	signal(SIGINT, GotSignal);
	signal(SIGQUIT, GotSignal);
	signal(SIGTERM, GotSignal);
	signal(SIGPIPE, GotSignal);

	signal(SIGHUP, DeadScanner);
	sigvec(SIGHUP, NULL, &sv);
	sv.sv_flags = SV_INTERRUPT;
	sigvec(SIGHUP, &sv, NULL);

	signal(SIGCHLD, LiveScanner);
	sigvec(SIGCHLD, NULL, &sv);
	sv.sv_flags = SV_INTERRUPT;
	sigvec(SIGCHLD, &sv, NULL);

	PrepLine();		/* prepare serial line */
	if (scannerfd == -1)
		PollLoop();
	PutMessage(SCANNER_RESET, NULL, NULL);

	maxfd = MAX(scannerfd + 1, getdtablesize());

	for (;;) {
		readfds = svc_fdset;
		if (scannerfd != -1)
			FD_SET(scannerfd, &readfds);

		if (tmout) {
			vLogMsg(EL_DEBUG3, id,
				"timeout set to %d", tmout->tv_sec);
		}
		before = time((time_t *)0);
		num = select(maxfd, &readfds, NULL, NULL, tmout);

		switch (num) {

		case -1:	/* signal arrived? */
			vLogMsg(EL_ERROR1|EL_SYS, id, "select");
			if (errno != EBADF)
				continue;
			die();

		case 0:		/* timeout */
			TimeOut();
			continue;

		default:	/* X or RPC */
			time_expired = 0;
			if (tmout) {
				time_t  elapse = time((time_t *)0) - before;
				
				vLogMsg(EL_DEBUG3, id,
					"requested %d, elapsed %d",
					tmout->tv_sec, elapse);
				
				if (tmout->tv_sec > elapse) {
					temptime = *tmout;
					temptime.tv_sec -= elapse;
					tmout = &temptime;
				}
				else
					time_expired = 1;
			}
			
			if (scannerfd != -1 &&
			    FD_ISSET(scannerfd, &readfds)) {
				ScannerRead();
				if (num-- == 1)
					break;
				FD_CLR(scannerfd, &readfds);
			}
			else if (time_expired)
				TimeOut();

			svc_getreqset(&readfds);
			break;
		}
	}
}
