#include	<stdio.h>
#include	<string.h>
#include	<strings.h>
#include	<rpc/rpc.h>
#include	"vault.h"
#include	"vmerror.h"

char		server[] = "localhost";
CLIENT		*cl = NULL;

typedef		int	(*vfunc)();

int		argnum;
char		**arglst;

usage(err)
 char	*err;
{
	if (arglst[argnum])
		fprintf(stderr, "command line error @ %s: ", arglst[argnum]);
	fprintf(stderr, "%s\n\n", err);
	fprintf(stderr, "usage: %s [-u][-d][-p][-r][-c] [vsn(s)]\n",
		arglst[0]);
	fprintf(stderr,
		"-u is update/add. Each vsn is followed by a location:\n");
	fprintf(stderr, "\t{v|V|vault|VAULT} section row slot\n");
	fprintf(stderr, "\t{g|G|general|GENERAL} description\n");
	fprintf(stderr, "\t{u|U|unknown|UNKNOWN}\n");
	fprintf(stderr, "-d is delete.\n");
	fprintf(stderr, "-p is print.\n");
	fprintf(stderr, "-r is request. Each vsn is followed by a string.\n");
	fprintf(stderr, "-c is cancel request.\n");

	auth_destroy(cl->cl_auth);
	exit(2);
}

 char	*
nextarg()
{
	char	*arg;

	argnum++;
	if ((arg = arglst[argnum]) == NULL)
		usage("not enough arguments");
	return(arg);
}

 int
update(volser)
 char	*volser;
{
	vsnloc_t	vsnloc;
	int		*req, scan;
	char		*c;

	memset(&vsnloc, '\0', sizeof(vsnloc));
	strncpy(&vsnloc.vsn, volser, VSNLEN);
	vsnloc.location.when = time((time_t *)NULL);
	
	memset(vsnloc.location.where.general, '\0', LOCSIZE);
	c = nextarg();

	if (strcmp(c, "u") == 0 ||
	    strcmp(c, "U") == 0 ||
	    strcmp(c, "unknown") == 0 ||
	    strcmp(c, "UNKNOWN") == 0) {
		vsnloc.location.where.tag = unknown;
	}
	else if (strcmp(c, "v") == 0 ||
		 strcmp(c, "V") == 0 ||
		 strcmp(c, "vault") == 0 ||
		 strcmp(c, "VAULT") == 0) {
		vsnloc.location.where.tag = vault;

		strcpy(vsnloc.location.where.section, nextarg());
		scan = sscanf(nextarg(), "%d",
			      &vsnloc.location.where.row);
		if (scan != 1 || vsnloc.location.where.row < 0)
			usage("bad row number");
		scan = sscanf(nextarg(), "%d",
			      &vsnloc.location.where.slot);
		if (scan != 1 || vsnloc.location.where.slot < 0)
			usage("bad slot number");

		printf("%.*s in section %s row %d slot %d\n",
		       VSNLEN, vsnloc.vsn,
		       vsnloc.location.where.section,
		       vsnloc.location.where.row,
		       vsnloc.location.where.slot);
	}
	else if (strcmp(c, "g") == 0 ||
		 strcmp(c, "G") == 0 ||
		 strcmp(c, "general") == 0 ||
		 strcmp(c, "GENERAL") == 0) {
		vsnloc.location.where.tag = display;

		strcpy(vsnloc.location.where.general, nextarg(), LOCSIZE);
		if ((c=index(vsnloc.location.where.general, '\n')) != NULL)
			*c = '\0';
		printf("%.*s in %*s\n", VSNLEN, vsnloc.vsn,
			LOCSIZE, vsnloc.location.where.general);
	}
	else
		usage("location type not recognized");

	req = vault_update_1(&vsnloc, cl);
	if (req == NULL) {
		clnt_perror(cl, server);
		return 1;
	}
	if (*req != E_NOERROR) {
		printf("error return %d from vault_update_1\n", *req);
	}
	return 0;
}


 int
delete(volser)
 char	*volser;
{
	vsn_t		vsn;
	int		*req;

	memset(&vsn, '\0', VSNLEN);
	strncpy(&vsn, volser, VSNLEN);
	
	req = vault_delete_1(&vsn, cl);
	if (req == NULL) {
		clnt_perror(cl, server);
		return 1;
	}
	if (*req != E_NOERROR) {
		fprintf(stderr,
			"error return %d from vault_delete_1\n", *req);
	}
	return 0;
}

 int
request(volser)
 char	*volser;
{
	vsnreq_t	vsnreq;
	int		*req;
	char		*ch;

	memset(&vsnreq.vsn, '\0', VSNLEN);
	strncpy(&vsnreq.vsn, volser, VSNLEN);
	strcpy(vsnreq.message, nextarg(), MESSAGESIZE);

	if ((ch = index(vsnreq.message, '\n')) != NULL)
		*ch = '\0';

	req = vault_request_1(&vsnreq, cl);
	if (req == NULL) {
		clnt_perror(cl, server);
		return 1;
	}
	if (*req != E_NOERROR) {
		fprintf(stderr,
			"error return %d from vault_request_1\n", *req);
	}
	return 0;
}

 int
cancel(volser)
 char	*volser;
{
	vsn_t		vsn;
	int		*req;

	memset(&vsn, '\0', VSNLEN);
	strncpy(&vsn, volser, VSNLEN);
	
	req = vault_cancel_1(&vsn, cl);
	if (req == NULL) {
		clnt_perror(cl, server);
		return 1;
	}
	if (*req != E_NOERROR) {
		fprintf(stderr,
			"error return %d from vault_cancel_1\n", *req);
	}
	return 0;
}

 int
print(volser)
 char	*volser;
{
	vsn_t		vsn;
	spacetime_t	*result;

	memset(&vsn, '\0', VSNLEN);
	strncpy(&vsn, volser, VSNLEN);

	result = vault_query_1(&vsn, cl);
	if (result == NULL) {
		clnt_perror(cl, server);
		return 1;
	}

	printf("vsn %.*s ", VSNLEN, &vsn);
	if (result->when == 0) {
		printf("not found\n");
		return;
	}
	else
		printf("%s",  ctime(&result->when));

	switch (result->where.tag) {
	case	unknown:
		printf("unknown\n");
		break;
	case	vault:
		printf("in section %.*s row %d slot %d\n",
		       SECSIZE, result->where.section,
		       result->where.row, result->where.slot);
		break;
	case	display:
		printf("in %*s\n", LOCSIZE, result->where.general);
		break;
	default:
		printf("has a bad location\n");
		break;
	}
	return 0;
}

main(argc, argv)
 int	argc;
 char	*argv[];
{
	vfunc		operation = print;

	cl = clnt_create(server, VAULTPROG, VAULTVERS, "tcp");
	if (cl == NULL) {
		clnt_pcreateerror(server);
		exit(1);
	}
	cl->cl_auth = authunix_create_default();

	arglst = argv;

	for (argnum=1; argnum<argc; argnum++) {
		char	*arg = argv[argnum];

		if (arg[0] == '-') {
			if (strlen(arg) != 2)
				usage("illegal flag");

			switch (arg[1]) {
			case 'u':
				operation = update;
				break;
			case 'd':
				operation = delete;
				break;
			case 'p':
				operation = print;
				break;
			case 'r':
				operation = request;
				break;
			case 'c':
				operation = cancel;
				break;
			default:
				usage("flag not recognized");
				break;
			}
		}
		else {
			if (strlen(arg) != VSNLEN)
				usage("bad vsn");
			if (operation(arg))
				break;
		}
	}
	auth_destroy(cl->cl_auth);
}
