#include	"vault.h"

/***************************************
        table layout
 ***************************************/
/*
 *  Records are rounded up in size to a divider of disk block size to
 *  ensure that a record write will never cross a block boundary, for
 *  all-or-nothing success for a write interrupted by a machine crash.
 */

/***************************************
        vsn table layout and buffer
 ***************************************/

#define DATELEN 24
#define INTLEN  6

#define UNKNOWN	'U'
#define VAULT	'V'
#define DISPLAY 'D'

typedef struct vaultrec {
	char		vr_section[SECSIZE];	char	a00;
	char		vr_row[INTLEN];		char	a01;
	char		vr_slot[INTLEN];
} vaultrec_t;

typedef union vaultunion {
	vaultrec_t	vu_vault;
	char		vu_general[LOCSIZE];
} vaultunion_t;

typedef struct locrec {
	char		l_date[DATELEN];	char	a00;
	char		l_type;			char	a01;
	vaultunion_t	l_field;		char	a02;
} locrec_t;

typedef struct vsnrec {
        char    	v_vsn[VSNLEN];		char    a00;
	char		v_status;		char	a01;
	char		v_lastloc;		char	a02;
	locrec_t	v_loc[HISTORY];		/* fill to 256 */
        char		fill[1];		char    end;
} vsnrec_t;

/*
**	Scanner commands
*/
#define		SCANNER_RESET	1
#define		SCANNER_STATUS	2
#define		SCANNER_UPDATE	3
#define		SCANNER_DELETE	4
#define		SCANNER_REQUEST	5
#define		SCANNER_CANCEL	6

/*
**  Define the functions visable to the world in vltlib.c
*/

void		InitDb(
#ifdef	__STDC__
void
#endif
);

void		CloseDb(
#ifdef	__STDC__
void
#endif
);

spacetime_t	*GetVsn(
#ifdef	__STDC__
vsn_t
#endif
);

int		AddVsn(
#ifdef	__STDC__
vsnloc_t *
#endif
);

spacetime_t	*DeleteVsn(
#ifdef	__STDC__
vsn_t
#endif
);

int		UpdateVsn(
#ifdef	__STDC__
vsnloc_t *
#endif
);

void		DisplayList(
#ifdef	__STDC__
void
#endif
);

char		*LocationString(
#ifdef	__STDC__
location_t *			/* location to display */
#endif
);

int		CheckVsn(
#ifdef	__STDC__
char	*
#endif
);

int		LocEqual(
#ifdef	__STDC__
location_t *,
location_t *			/* two locations to compair */
#endif
);


struct	displaylist {
	vsnreq_t		dl_vsnreq;
	time_t			dl_time;
	char			dl_mach[20];
	struct	displaylist	*dl_next;
};
