/*
 * Volume Manager
 *
 * vmerror.h: Volume Manager errors
 *
 * $Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/rc/vault/server/vmerror.h,v 1.1.1.1 2000-02-28 02:18:34 wrstuden Exp $
 */

/*
 * Error reporting philosophy:
 *	All functions use LogMsg() to log errors.  LogMsg() supports
 *	both global variable-based (for the AC) and file-based (for
 *	RP/RC/VDM) error reporting.
 *
 * Global variable errors:
 *	VMError contains the error code for the last error.
 *	All functions can set VMError, some set VMErrStr to a comment
 *	about what happened.
 *	VMErrno is set to errno when a system error (E_SYS) occurs.
 *	VMErrorToString converts a VMError to a string (not the same
 *	as VMErrStr).
 *
 *	Functions return 0 for success, or the value of VMError if an
 *	error, except functions which return an address which return
 *	the address if success, or NULL (and set VMError) if error.
 */

#ifndef _VMERROR_
#define _VMERROR_

#define		vClrError()		VMError = E_NOERROR, \
					VMErrStr = NULL, \
					VMErrno = 0, \
					VMErrHdr = NULL

/*
 * Error codes for VMError
 *	These are the (only) error codes the user sees,
 *	so they should be simple and high-level in nature.
 *	They can be supplemented with a descriptive string in VMErrStr.
 *	Any changes to these errors should be reflected in vmp.c.
 */
#define	E_NOERROR	0x0000		/* indicates no error;must be 0 */
#define	E_NOMEM		0x0001		/* couldn't allocate memory	*/
#define	E_SYS		0x0002		/* system error (check errno)	*/
#define	E_DOWN		0x0003		/* RP, RC not processing requests*/
#define	E_INTERNAL	0x0004		/* internal error		*/
#define	E_BADPACKET	0x0005		/* packet check failed		*/
#define	E_IO		0x0006		/* I/O error			*/
#define E_USER		0x0007		/* user gave bad parameter	*/
#define E_DBASE		0x0008		/* Informix error		*/
#define E_ADMIN		0x0009		/* data base inconsistency	*/
#define E_PERM		0x000a		/* permission denied		*/
#define	E_NOTFOUND	0x000b		/* volume can't be located	*/
#define	E_UNAVAIL	0x000c		/* resource unavailable/in use	*/
#define E_OPS		0x000d		/* ops problem - not logged in	*/
#define E_CANCELED	0x000e		/* request was canceled		*/


/* flags for event logger: */
#define	EL_LEVEL		0x000000ff	/* mask for debug level	*/
#define		EL_FATALCORE	0x00000030	/* generate core dump */
#define		EL_FATAL_	0x00000040
#define		EL_URGENT	0x00000048
#define		EL_ERROR1	0x00000060
#define		EL_ERROR2	0x00000068
#define		EL_STATUS1	0x00000070
#define		EL_STATUS2	0x00000078
#define		EL_DEBUG1	0x00000080
#define		EL_DEBUG2	0x00000082
#define		EL_DEBUG3	0x00000084
#define		EL_DEBUG4	0x00000086
#define		EL_DEBUG5	0x00000088
#define	EL_SYS_			0x00000100	/* print sys err string	*/
#define	EL_NOTIME		0x00000200	/* don't print time info*/
#define	EL_RPCON		0x00000400	/* send msg to RP console */
#define	EL_LCLCON		0x00000800	/* send msg to local console */
#define	EL_ID_			0x00001000	/* print ID message	*/
#define	EL_INTERNAL_		0x00002000	/* internal error (message) */
#define	EL_CODE			0x00004000	/* contains error code (E_) */
#define	EL_TOSTDERR		0x00008000	/* log error to stderr	*/
#define	EL_TOFILE		0x00010000	/* log error to error log */
#define	EL_RENAMELOG		0x00020000	/* rename old log if exists */
#define	ERRNOMASK		0xff000000	/* see ERRNOSHFT	*/


#define	ERRNOSHFT	24

/* super macros: */
#define	EL_SYS		(EL_SYS_|((errno << ERRNOSHFT) & ERRNOMASK))
#define EL_ID		(EL_ID_ | EL_DEBUG1)	/* debug level of ID msgs */
#define	EL_INTERNAL	(EL_INTERNAL_ | EL_ERROR1 | EL_RPCON)
						/* level of internal errors */
#define	EL_FATAL	(EL_FATAL_ | EL_RPCON)	/* fatal msg also goes to con */

/* Console stuff: */
#define	CON_MAXCON	8		/* max number of consoles tracked */
#ifdef DEBUG
#define	CON_SYSCONNAM	vCat(vLibDir, "console")
#else
#define	CON_SYSCONNAM	"/dev/console"
#endif
#define	CON_SYSCON	(1 << CON_MAXCON) /* controls writes to CON_SYSCONNAM */
#define	CON_THRSLD	2		/* max terms to write before fork() */
#define	CON_MAXTERM	20		/* max terms per message */
#define	CON_MAXTERMNAM	40		/* max length of a terminal name */
#define	CON_ALARM	4		/* alarm time to wait on term writes */
#define	CON_SALARM	2		/* short alarm when we're in a hurry */

#endif	/* _VMERROR_ */
