#ifndef lint
static char rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/rc/vault/server/vmlog.c,v 1.1.1.1 2000-02-28 02:18:35 wrstuden Exp $";
#endif

/*
 * Volume Manager
 *
 * vmlog.c: Volume Manager Event Log Manager
 */

#include <stdio.h>
#include <errno.h>
#include <time.h>			/* for localtime()		*/
#include <fcntl.h>
#include "vmerror.h"

#ifdef UTS
#include <sys/timeb.h>
#endif

#ifdef EXTERN
#undef EXTERN
#define EXTERN		/* forces declarations in vmspace.h to allocate */
#endif	/* EXTERN */

#include "vconfig.h"
#include <sys/param.h>
#include <ctype.h>

/* 
 *  syslog stuff - syslog_fac and syslog_pri are for EL_RPCON messages.
 * 	LOG_LOCAL0 points to /etc/syslog.conf local0. Syslog stuff is
 *	described in syslog(3) and syslogd(1M) man pages
 */

#include <syslog.h>
static int	syslog_lclfac=0;	/* provided by vLogInit caller */
static int	syslog_flg=0, 
		syslog_fac = LOG_LOCAL0, 
		syslog_pri = LOG_NOTICE;


char	*vSpoolDir = SPOOLDIR;

int		VMErrno;		/* saved errno when E_SYS	*/
int		VMError;		/* last volume manager error	*/
char		*VMErrStr;		/* string describing last error	*/
int		vLogLevel = EL_STATUS1;	/* event log logging level	*/
int		child_ended;		/* child gave ok to continue	*/

static int	EventLog = -1;		/* file desc for event log	*/
static int	DefaultDest;		/* for error msgs (stderr or file) */
static int	(*OnFatal)();		/* called on fatal errors	*/
static char	*Caller;		/* program ID for log messages	*/

struct code {
        char    *c_name;
        int     c_val;
};


#define index	strchr
#define rindex	strrchr
char	*index(), *rindex();

/*
 * vCat(): concatenate two paths, insert a '/' in between
 */

  char *
vCat(str1, str2)
  char	*str1, *str2;
{
	static char	catbuf[LINELEN * 2];

	catbuf[sizeof (catbuf) - 1] = '\0';
	strcpy(catbuf, str1);
	strcat(catbuf, "/");
	strcat(catbuf, str2);
	if (catbuf[sizeof (catbuf) - 1] != '\0') {
		vLogMsg(EL_URGENT, "vCat",
			"overran buffer: %s/%s", str1, str2);
	}
	return (catbuf);
}


  char *
vSysErr(n)
{
	static char	buf[80];
	extern char	*sys_errlist[];
	extern int	sys_nerr;

	if (n < sys_nerr && n > 0)
		return (sys_errlist[n]);

	sprintf(buf, "%d: unknown system error", n);
	return (buf);
}

/*
 * vLogInit(): initialize error logger:
 *	caller is name of calling program (RP, VDM, etc.)
 *	onfatal() is a function to be called on fatal errors (EL_FATAL)
 *	defaultdest sets the default destination for error messages
 *		indicated by one or both of EL_TOSTDERR and EL_TOFILE
 *	filename is the name of the log file to be opened; if it
 *		is NULL, ERRORLOG is opened
 *	localsyslog is the LOG_LOCALx number of the syslogd facility 
 *		that will get the EL_LCLCON messages.
 *		These facilities are defined in /etc/syslog.conf.
 * Note: vLogInit() can be called multiple times and is (re)initialized
 *	each time, except the syslog.
 */

vLogInit(caller, onfatal, flags, filename, localsyslog)
  char	*caller, *filename;
  int	localsyslog;	/* local console syslog facility, if any */
  int	(*onfatal)();
{
	char	*id = "vLogInit";

	Caller = caller;
	OnFatal = onfatal;
	DefaultDest = flags & (EL_TOSTDERR|EL_TOFILE);
	if (EventLog >= 0)		/* if log was open, close it */
		close (EventLog);
	if (DefaultDest & EL_TOFILE) {
		if (filename == NULL) {
			vInitDirNames();	/* init vSpoolDir */
			filename = vCat(vSpoolDir, ERRORLOG);
		}
		if ((flags & EL_RENAMELOG) &&
		  (EventLog = open(filename, O_RDONLY)) >= 0) {
			time_t		timeval;
			char		newlogname[LINELEN*2],
					*logpath, savch;
			struct tm	*tp;

			close(EventLog);
			time(&timeval);
			tp = localtime(&timeval);
			/*
			 * see if the logfile has a path component;
			 *	if so, it must be prepended to newlogname
			 */
			if (logpath = rindex(filename, '/')) {
				logpath++;	/* point one past the '/' */
				savch = *logpath;
				*logpath = '\0'; /* temporarily null-term */
			}
			sprintf(newlogname, "%slog%02d-%02d.%02d%02d",
			  logpath ? filename : "",
			  tp->tm_mon+1, tp->tm_mday, tp->tm_hour, tp->tm_min);
			if (logpath) /* restore nulled-out char in filename */
				*logpath = savch;
			if (link(filename, newlogname) < 0) {
				fprintf(stderr, "link(%s, %s) failed\n",
				  filename, newlogname);
				perror("link error");
			}
		else
				unlink(filename);
		}
		if ((EventLog =
		  open(filename, O_WRONLY|O_APPEND|O_CREAT, 0644)) < 0) {
			fprintf(stderr, "%s: cannot open/create log file: %s\n",
			  filename, vSysErr(errno));
			if (OnFatal)
				(*OnFatal)(id, NULL);
		}
#ifdef DEBUG
		
#endif
	}
	if(!syslog_flg){
		openlog("", LOG_NDELAY | LOG_CONS, localsyslog);
		syslog_flg++;
		vLogMsg(EL_DEBUG2, id, "syslog opened facility:%s priority:%s",
			SYSLOG_FAC, SYSLOG_PRI);
	}
	if (localsyslog) {
		if (!syslog_lclfac)
			syslog_lclfac = localsyslog;
		vLogMsg(EL_STATUS2, id, "syslogd lclcon facility: (%X)",
			syslog_lclfac);
	}
	vLogMsg(EL_STATUS2, id, "Event Log Initialized (%s)",
	  (DefaultDest & EL_TOFILE) ? filename : "no file");

}


/*
 * vLogMsg(): write a message to log file or stderr
 */

vLogMsg(flag, id, fmt, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
  char	*id, *fmt,
  	*arg1, *arg2, *arg3, *arg4, *arg5, *arg6, *arg7, *arg8, *arg9;
{
	char		buf[LINELEN * 20],
			tbuf[LINELEN * 20],
			*errstrptr = "";		/* for VMErrStr */
	static int	broken = 0;	/* if true don't attempt to write */
	int		wcnt, level = (flag & EL_LEVEL),
			write2file, write2stderr,
			fatalerror = 0;
	unsigned	bcnt;

	if (broken)
		return;
	if (!(flag & EL_CODE) && (level > vLogLevel) && (level > EL_URGENT))
		return;

	buf[0] = '\0';
	if (!(flag & EL_NOTIME)) {
		long		secs;
		struct tm	*tp;
#ifdef UTS
		long		msecs;		/* miliseconds */
		struct timeb	tb;
		
		if (ftime(&tb) < 0) {
			secs = 0;
			msecs = 0;
		} else {
			secs = tb.time;
			msecs = tb.millitm;
		}
#else	/* UTS */
		if (time(&secs) < 0)
			secs = 0;
#endif	/* UTS */
		tp = localtime(&secs);
#ifdef DEBUG
#ifdef UTS
		/* give hr:min:sec.msec in debug mode */
		sprintf(buf, "%d:%02d:%02d.%003d",
		  tp->tm_hour, tp->tm_min, tp->tm_sec, msecs);
#else	/* UTS */
		/* give day hr:min:sec in debug mode */
		sprintf(buf, "%d %d:%02d:%02d", tp->tm_mday,
		  tp->tm_hour, tp->tm_min, tp->tm_sec);
#endif	/* UTS */
#else	/* DEBUG */
		sprintf(buf, "%d/%d %d:%02d:%02d", (tp->tm_mon + 1),
			tp->tm_mday, tp->tm_hour, tp->tm_min, tp->tm_sec);
#endif	/* DEBUG */
	}
	
	if (Caller) {
		strcat(buf, ": ");
		strcat(buf, Caller);
	}
	if (id) {
		strcat(buf, ": ");
		strcat(buf, id);
		if (flag & EL_ID_)
			strcat(buf, " here");
	}

	if (flag & EL_INTERNAL_) {
		strcat(buf, ": INTERNAL ERROR");
		VMError = E_INTERNAL;
	}

	if (!(flag & EL_ID_)) {
		sprintf(tbuf, fmt,
		  arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
		strcat(buf, ": ");
		if (flag & EL_CODE)	/* start VMErrStr from here */
			errstrptr = buf + strlen(buf);
		strcat(buf, tbuf);
		
		if (flag & EL_SYS) {
			int	erno = (flag & ERRNOMASK) >> ERRNOSHFT;

			strcat(buf, ": ");
			strcat(buf, vSysErr(erno));
		}
	}

	strcat(buf, "\n");				/* terminate message */
	buf[sizeof (buf) - 1] = '\0';			/* safety null */
	
	/*
	 * determinte destination: if either "TO" bit on in flag word,
	 *	then flag word determines dest; else use DefaultDest
	 */
	write2file = write2stderr = 0;
	if (flag & (EL_TOSTDERR|EL_TOFILE)) {
		if (flag & EL_TOSTDERR)
			write2stderr = 1;
		if (flag & EL_TOFILE)
			write2file = 1;
	} else {
		if (DefaultDest & EL_TOSTDERR)
			write2stderr = 1;
		if (DefaultDest & EL_TOFILE)
			write2file = 1;
	}
	bcnt = strlen(buf);
	if (write2file && (EventLog >= 0)) {
		wcnt = write(EventLog, buf, bcnt);	/* write out message */
		if (wcnt != bcnt) {
			broken = 1;
			sprintf(tbuf,
			  "vLogMsg (%s): can't write [%s] to event log: %s\n",
				Caller, buf, vSysErr(errno));
			fputs(tbuf, stderr);
			fatalerror = 1;
		}
	} else
		write2stderr = 1;		/* try to get the msg out */

	if (write2stderr) {
		register char	*msgptr = NULL;

		/*
		 * don't print time info to stderr; use index to skip it
		 */
		if (!(flag & EL_NOTIME) && (msgptr = index(buf, ':')))
			if (msgptr = index(msgptr+1, ':'))
				if (msgptr = index(msgptr+1, ':'))
					msgptr += 2;
		if (msgptr)
			fputs(msgptr, stderr);
		else
			fputs(buf, stderr);
		fflush(stderr);
	}

	/*
	 * perform writes to console(s) if requested
	 */
	if (flag & ( EL_RPCON)) 
		vConWrite(buf, (syslog_fac | syslog_pri), ' ');
	if ( (flag & (EL_LCLCON) )  && syslog_lclfac){
		/*
		 * The syslogd has a bug in that if the same message is
		 * sent to a different facility, the message is written 
		 * to the first facility twice. Therefore, if the message was
		 * sent to the RPCON, make the message different so syslogd
		 * can write it to the second facility
		 */
		if(flag & (EL_RPCON))
			vConWrite(buf, syslog_lclfac | syslog_pri, '^');
		else
			vConWrite(buf, syslog_lclfac | syslog_pri, ' ');
	}

	/*
	 * copy error info into globals (VMError and VMErrStr) if EL_CODE
	 */
	if (flag & EL_CODE) {	/* VMError should already have been assigned */
		static char	esbuf[LINELEN + 1];	/* for VMErrStr */
		char		*nlp;

		strncpy(esbuf, errstrptr, sizeof (esbuf));
		esbuf[sizeof (esbuf) - 1] = '\0';
		if (nlp = index(esbuf, '\n'))		/* get rid of \n */
			*nlp = '\0';
		VMErrStr = esbuf;
		if (flag & EL_SYS_)
			VMErrno = (flag & ERRNOMASK) >> ERRNOSHFT;
	}
	if (level == EL_FATAL_ || fatalerror)
		if (OnFatal)
			(*OnFatal)(id, errstrptr);
}


#include	<utmp.h>
#include 	<signal.h>

/*
 * vConWrite(): broadcast a message to the syslog
 */

vConWrite(msg,pri,c)
  char	*msg;	
  int	pri;
  char  c;
{
	char	*id = "vConWrite";
	register char   *msgptr = NULL;

	/*
	 * don't print time info to syslog: use index to skip it
	 */
	if (msgptr  = index(msg, ':'))
		if (msgptr = index(msgptr+1, ':'))
			if (msgptr = index(msgptr+1, ':'))
				msgptr += 2;
	if (msgptr)
		syslog(pri,"%c%s", c, msgptr);

	else
		syslog(pri,"%c%s", c, msg);
}


/*
 * vMvEnv(): move the shell environment variables to buy space for
 *	command line arguments installed by program
 */

vMvEnv()
{
	extern char	**environ;
	char		**envp, *newenv, *malloc();
	unsigned	bytes = 0;
	int		oenvp0;

	oenvp0 = (int) environ[0];
	for (envp = environ; *envp; envp++)	/* get size of environ block */
		bytes += strlen(*envp) + 1;
	if ((newenv = malloc(bytes)) == 0)
		return (0);
	memcpy(newenv, environ[0], (int) bytes);
	for (envp = environ; *envp; envp++)	/* get size of environ block */
		*envp = (char *) ((int)*envp - oenvp0) + (int)newenv;
	return (1);
}


vInitDirNames()
{
#ifdef DEBUG
	static int	called;
	char		*env, *getenv();

	if (!called) {
		if ((env = getenv(ALTSPOOLDIR)))
			vSpoolDir = env;
		if ((env = getenv(ALTLIBDIR)))
			vLibDir = env;
		if ((env = getenv(ALTNODEDIR)))
			vNodeDir = env;
		called = 1;
	}
#endif	/* DEBUG */
}

decode(name, codetab)
        char *name;
        struct code *codetab;
{
        register struct code *c;
        register char *p;
        char buf[40];
 
        if (isdigit(*name))
                return (atoi(name));
 
        (void) strcpy(buf, name);
        for (p = buf; *p; p++)
                if (isupper(*p))
                        *p = tolower(*p);
        for (c = codetab; c->c_name; c++)
                if (!strcmp(buf, c->c_name))
                        return (c->c_val);
 
        return (-1);
}
