#ifndef lint
static char rcsid[] = "$Header: /Volumes/TempStorage/repos/netbsd/othersrc/nastore/volman/srvr/server/cd_1.c,v 1.1.1.1 2000-02-28 02:18:36 wrstuden Exp $";
#endif
/*
 *  cd_1.c: Volume Manager client mgr daemon main routine.
 *
 *  	Client mgr = front-end for the 'main' vold server. This
 *	provides secure client connections (via keys controlled
 *	by file permissions). It is a separate process from the
 *	main server so that instances can be run on each of multiple 
 *	hosts.
 *
 *	It establishes a connection with the vold, then sets up 'local 
 *	ipc' connections w/ clients, verifying their identity and
 *	notifying the vold of their existence. It passes client requests 
 *	& vold responses transparently back and forth. 
 *
 */

#ifdef EXTERN
# undef EXTERN
#endif
#define EXTERN
#include "srv_space.h"

#include <pwd.h>
#include <setjmp.h>
#include <sys/resource.h>

static char	*startup_lock = "/tmp/volcd_lock";

extern jmp_buf	jmpbuf_i;
extern int	immortal;

char		*arg0 = NULL;
int		restartlog = 0;
char		myhost[MAXHOSTNAMELEN];
int		myhostlen;
char		myhostaddr[HOST_ADDRLEN];

usage()
{
	fprintf(stderr, "usage: %s \n", arg0);
	exit(1);
	return(1);	/* for compiler warnings */
}

main(int argc, char *argv[])
{
	char		*id = "main";
	extern int	optind;
	extern char	*optarg;
	int		cc, dflg = 0;

	arg0 = argv[0];
	CheckUID(arg0);

	/*
	 *  parse args
	 */
	while ((cc = getopt(argc, argv, "di")) != EOF) {
		switch (cc) {
		    case 'd':
			dflg = 1;
			break;
		    case 'i':
			immortal = 1;
			break;
		    default:
			fprintf(stderr, "unexpected option (%c)\n", cc);
			usage();
		}
	}
	if (optind < argc) {
		fprintf(stderr, "argc %d, last [%s]\n", argc, argv[argc-1]);
		usage();
	}

	/*
	 *  go into the background
	 */
#ifndef PURIFY
	if (!dflg) {
		switch (fork()) {
		  case -1:				/* fork failed */
			perror("fork");
			exit(1);
		  case 0:				/* child */
			if (setpgid(0,0))
				perror("setpgid");
			break;
		  default:				/* parent */
			exit(0);	/* detach */
		}
	}
#endif
	/*
	 *  start / read & process all msgs using configured IPC
	 */
	if (setjmp(jmpbuf_i)) {
		nLogMsg(EL_STATUS1, 0, "setjmp to beginning ok");
		IPCStopClientListen();
		UnreserveFD("setjmp");
		CDShutdownClients();
		sleep(30);
	}

	IPCServer0(CDInit());
}

int
CDShutdown()
{
	nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, "shutting down");
	exit(0);
	return(0);	/* for compiler warnings */
}

CDInit()
{
	char		*id = "CDInit";
	char		coredir[80];
	int		ret, uid, flag;
	char		buf[MAXPATHLEN];
	struct hostent	*htp;
	char		**cpp;
	struct rlimit	limits;

	vInitDirNames();

	/*
	 *  start log file as volcd
	 */
	nLogInit("CD", CDShutdown);
	nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
	nLogInitConsole(rp_syslog, EL_CONSOLE0, EL_LEVEL);
	nLogMsg(EL_STATUS1, id, "[pid %d] -----------------------------------",
								getpid());

	/*
	 *  get exclusive lock
	 */
	sprintf(buf, "%s.%s", startup_lock, vVoldHost);
	nGetLock(buf, "volcd");

	/*
	 *  make sure vSpoolDir/volcd/ exists and go to it
	 */
	sprintf(coredir, "%s/%s", vSpoolDir, "volcd");
	nCreatePath(coredir, 0755);

	if (chdir(coredir) < 0)
		nLogMsg(EL_ERROR1|EL_SYS|EL_FATAL|EL_TOSTDERR|EL_CONSOLE0, id,
						"chdir(%s)", coredir);
#ifndef PURIFY
	/*
	 *  max out the file descriptors
	 */
	if (getrlimit(RLIMIT_NOFILE, &limits) == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "getrlimit(RLIMIT_NOFILE)");
	nLogMsg(EL_STATUS1, id, "initial fd limits %d %d", limits.rlim_cur, 
						   limits.rlim_max);
	limits.rlim_cur = FD_SETSIZE;
	limits.rlim_max = FD_SETSIZE;
	if (setrlimit(RLIMIT_NOFILE, &limits) == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "setrlimit(RLIMIT_NOFILE..) %d",
						FD_SETSIZE);
	if (getrlimit(RLIMIT_NOFILE, &limits) == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "getrlimit(RLIMIT_NOFILE)");
	nLogMsg(EL_STATUS1, id, "reset fd limits %d %d", limits.rlim_cur, 
						   limits.rlim_max);
#endif
	/*
	 *  Reserve an 'ace-in-the-hole' file descriptor for
	 *	getting out of tight situations. 
	 */
	ReserveFD("init");

	/*
	 *  see if running on vold host
	 */
	if (gethostname(myhost, MAXHOSTNAMELEN) == -1)
		nLogMsg(EL_ERROR1|EL_SYS|EL_FATAL|EL_TOSTDERR|EL_CONSOLE0, id,
							"gethostname");
	htp = gethostbyname(myhost);
	if (htp == NULL)
		nLogMsg(EL_ERROR1|EL_SYS|EL_FATAL|EL_TOSTDERR|EL_CONSOLE0, id,
					"gethostbyname(own host = %s)", myhost);

	memcpy(myhostaddr, htp->h_addr_list[0], HOST_ADDRLEN);
	htp = gethostbyname(vVoldHost);
	if (htp == NULL)
		nLogMsg(EL_ERROR1|EL_SYS|EL_FATAL|EL_TOSTDERR|EL_CONSOLE0, id,
				"gethostbyname(vold host = %s", vVoldHost);
		

	restartlog = 1;
	for (cpp=htp->h_addr_list; *cpp; cpp++) {
		if (!memcmp(*cpp, myhostaddr, HOST_ADDRLEN)) {
			restartlog = 0;
			break;
		}
	}

	if (restartlog)
		nLogMsg(EL_STATUS2, id, "running on %s, vold on %s",
						myhost, vVoldHost);
	else
		nLogMsg(EL_STATUS2, id, "running on %s, same as vold", myhost);

	/*
	 *  release some file descriptors (for client slots)
	 */
	fclose(stdin);			/* don't need standard input */
	fclose(stdout);			/* don't need standard output */

	/*
	 *  connect/register w/ vold
	 */
	flag = 0;
	ret = RPCLInit(M_VOLCD, &flag);

	/*
	 *  catch all signals
	 */
	SigInit();

	/*
	 *  init client-tracking array, clear client connection dir 
	 *	and start client IPC
	 */
	CreateEmptyDir(vClientDir);
	IPCInitClients();
	IPCStartClientListen();

	nLogMsg(EL_CONSOLE0|EL_STATUS1, id, "volcd initialized (pid %d)", 
								getpid());
	return(ret);
}


