
/*
 *  cd_clnt.c - Volume Manager client daemon (volcd) client mgt
 *
 *	Client routines for managing the client-server connection.
 *
 *  	Each client is a process for which a file descriptor is being used
 *	in the client server. Clients are kept as an array which is indexed 
 *	by the file descriptor; some 'clients' in the array are never used 
 *	because the file descriptors are used for other things.
 *
 *	Some client manipulation takes place in cd_{ipc}.c
 */

#include "srv_space.h"
#include "rp_clnt.h"

Client_t	Clients[FD_SETSIZE];
int		totalClients;

/*
 *  CDNewClient(): init client struct, increment count of total clients
 *
 *	Leave IPC part of client for calling IPC routine to fill in.
 */
 void
CDNewClient(int i)
{
	char		*id = "CDNewClient";
	Client_t	*cp = &Clients[i];

	cp->c_flags = CL_EXIST;
	cp->c_pid = -1;
	cp->c_uid = -1;
	time(&cp->c_time);

	totalClients++;
}

 void
CDShutdownClient(Client_t *cp)
{
	CloseIPC(cp->c_ipcp);

	totalClients--;
	cp->c_flags = 0;
}

 void
CDShutdownClients()
{
	int		i;
	Client_t	*cp;

	for (i=0,cp=Clients; i<FD_SETSIZE; i++,cp++) {
		if (cp->c_flags)
			CDShutdownClient(cp);
	}
}


/*
 *  vLogClients() - print summary to log
 */
 void
vLogClients(Client_t *cp)
{
	int		i;

	nLogMsg(EL_ALWAYS|EL_NOTIME, 0, 
			"=============== Client list =================");
	for (i=0; i<FD_SETSIZE; i++, cp++) {
		if (cp->c_flags & CL_EXIST) {
			nLogMsg(EL_ALWAYS|EL_NOTIME, 0,
			  "\tslot=%d, addr=%x, pid=%d, uid=%d, flags=%x",
			  i, cp, cp->c_pid, cp->c_uid, cp->c_flags);
		}
	}
}

