
/*
 *  Volume Manager
 *
 *  cd_sock.c - volcd (volman client daemon) socket routines
 */

#include "srv_space.h"
#include "rp_clnt.h"

#include <time.h>
#include <setjmp.h>

extern int 		errno;

extern int		restartlog;

extern char		myhost[];
extern int		myhostlen;
extern char		myhostaddr[];
extern Client_t		Clients[];
extern int		totalClients;
extern IPC_t		SockArray[];

extern jmp_buf		jmpbuf_i;
extern int		immortal;
extern int		idleshutdown;
extern char		shutdownmsg[];

time_t			Now;	/* approximate time for non-critical use */

static fd_set		fds;
static int		client_port_sd;		/* for receiving local conns */

static int		clientSlots, unresvSlots;
static int		socksize = sizeof( struct sockaddr_in );

static int		foreign = 0;		/* conns from other hosts */
static int		hellogoodbye = 0;	/* incomplete conns */

 void
IPCInitClients()
{
	Client_t	*cp = Clients;
	IPC_t		*ip = SockArray;
	int		i;

	for (i=0; i<FD_SETSIZE; i++, cp++, ip++) {
		cp->c_id = i;
		cp->c_flags = 0;
        	cp->c_ipcp = (void *) ip;
		ip->sock_proc = (void *) cp;
		ip->sock_fd = -1;
        	ip->sock_id = -1;
        	ip->sock_state = SS_NOSOCK;
	}

}

 void
IPCStartClientListen()
{
	client_port_sd = IPCStartListening(volcd_port);
}

 void
IPCStopClientListen()
{
	shutdown(client_port_sd, 2);
	close(client_port_sd);
	client_port_sd = -1;
}

/*
 *  FUNCTION:	IPCAcceptClientConn()
 *		Accept connection on server's client master socket.
 */
 static void
IPCAcceptClientConn()
{
	char			*id = "AcceptClientConn";
	struct sockaddr_in	new_conn;
	struct hostent		*htp;
	register int 		slot;
	int			new_len, new_soc, hok = 0;
	IPC_t			*ip;

	if (totalClients >= clientSlots) {
		nLogMsg(EL_STATUS1, id, "all slots allocated");

		/*
		 *  try to free a slot
		 */
		if (CheckClients()) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
				"can't connect - out of client slots");
			return;
		}
	}

	/*
	 *  NOTE:  Must check for priority status of process later in
	 *  InitClient().  At that time, the reserved socket will be
	 *  freed if the client does not have authority to use it.
	 */
	if (totalClients > unresvSlots) {
		nLogMsg(EL_STATUS1, id, "into priority socket range");
	}

	/*
	 *  Create new socket with accept()
	 *	Note that resulting fd indexes Client array
	 */
	new_soc = accept(client_port_sd, (struct sockaddr *)&new_conn, 
							&socksize);
	if (new_soc == -1) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1|EL_SYS, id, "accept failed");
		return;
	}
	htp = gethostbyaddr((char *)&new_conn.sin_addr, 
					sizeof(new_conn.sin_addr), AF_INET);
	if (htp == NULL) {
		foreign++;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
			"attempted connection from untranslatable host");
		IPCClose(new_soc);
		return;
	}
	if (!strcmp(htp->h_name, "localhost"))
		hok = 1;
	else {
		char	**cpp;

		for (cpp=htp->h_addr_list; *cpp; cpp++) {
			if (!memcmp(*cpp, myhostaddr, HOST_ADDRLEN)) {
				hok = 1;
				break;
			}
		}
	}
	if (!hok) {
		foreign++;
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
			"attempted connection from different host %s",
			htp->h_name);
		IPCClose(new_soc);
		return;
	}

	nLogMsg(EL_DEBUG2, id, "new socket fd %d\n", new_soc);

	CDNewClient(new_soc);
	ip = &SockArray[new_soc];
	ip->sock_state = SS_CONNECTED;
	ip->sock_flags = IPC_NDELAY;
	ip->sock_fd = new_soc;
	ip->sock_id = -2;
}

/*
 *  IPCMHello(): process M_HELLO packets received by the volcd
 */
 static void
IPCMHello(vmpHdr_t *hp, Client_t *cp, IPC_t *ip)
{
  	char		*id = "IPCMHello";
	vmpHello_t	*hellop;
	char		*security;
	int		fd, ret, fdunreserved = 0;

	hellop = MSGPTR(hp, vmpHello_t);
	if (hellop->vhe_version != VM_VERSION) {
		nLogMsg(EL_ERROR1, id, "wrong version %d expected %d",
				hellop->vhe_version, VM_VERSION);
		CDShutdownClient(cp);
		return;
	}

	nLogMsg(EL_DEBUG1, id, "uid %d  pid %d ", hellop->vhe_uid, 
						  hellop->vhe_pid);
	switch (ip->sock_state) {
	    case SS_CONNECTED:
		/*
		 *  normal case - client needs to get key
		 *	first save client key for sending
		 */
		ip->sock_sndkey = hp->vh_key;
		/*
		 *  make up client key
		 */
		ip->sock_rcvkey = vGetID();	
		/*
		 *  note claimed uid, pid & flags (e.g. priority 
		 *	slot req'd) 
		 */
		ip->sock_id = hellop->vhe_pid;
		cp->c_pid = hellop->vhe_pid;
		cp->c_uid = hellop->vhe_uid;
		if (hellop->vhe_flag & VO_PRIORITY) {
			/*
			 *  note prio claim; check it later
			 */
			cp->c_flags |= CL_PRIO;
		}

		/*
		 *  tell client transfer key
		 */
		cp->c_fileid = vGetID();
		hellop->vhe_id = cp->c_fileid;

		/*
		 *  create a file w/ transfer key to write
		 *	client key into
		 */
		security = vIDFile(cp->c_fileid);
		fd = open(security, O_RDWR|O_CREAT, 0400);
		if (fd == -1) {
			UnreserveFD("security");
			fdunreserved = 1;
			fd = open(security, O_RDWR|O_CREAT, 0400);
		}
		if (fd == -1) {
			VMError = E_INTERNAL;
			nLogMsg(EL_SYS|EL_CONSOLE0|EL_CODE, id,
				    "security file (%s) open failed", security);
			CDErr2Client(hp->vh_seq, cp);
			if (fdunreserved)
				ReserveFD("fdunreserved 1");
			return;
		}

		/*
		 *  write client key to file
		 */
		ret = write(fd, &ip->sock_rcvkey, sizeof(ip->sock_rcvkey));
		if (ret != sizeof(ip->sock_rcvkey)) {
			nLogMsg(EL_CONSOLE0|EL_CODE|(ret==-1 ? EL_SYS : 0),
					"security file (%s) write failed",
								security);
			CDErr2Client(hp->vh_seq, cp);
			unlink(security);
			close(fd);
			if (fdunreserved)
				ReserveFD("fdunreserved 2");
			return;
		}
		close(fd);
		if (fdunreserved)
			ReserveFD("fdunreserved 3");

		/*
		 *  make file accessible only to the claimed id of
		 *	client 
		 */
		if ((ret=chown(security, hellop->vhe_uid, 0)) == -1) {
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1|
					(ret==-1 ? EL_SYS : 0), id,
					"security file (%s) chown failed",
								security);
			CDErr2Client(hp->vh_seq, cp);
			unlink(security);
			return;
		}
		IPCSnd2Client(hp, cp);
		ip->sock_state = SS_HELLOD;
		break;
	    case SS_HELLOD:
		nLogMsg(EL_ALWAYS, id,
				"Ignoring duplicate M_HELLO pkt for pid %d",
				ip->sock_id);
		break;
	    case SS_TALKING:
	    case SS_GOODBYE:
		nLogMsg(EL_ALWAYS, id,
				"Client %d protocol err: HELLO after connected",
				ip->sock_id);
		break;
	    case SS_NOSOCK:
	    default:
		nLogMsg(EL_FATAL, id, "connect w/ nosock");
	}
}

/*
 *  IPCClientMsg() - local handling for M_HELLO, 1st msg after M_HELLO,
 *	M_GOODBYE.  
 */
 static void
IPCClientMsg(int i)
{
	char		*id = "IPCClientMsg/sock";
	Client_t	*cp = &Clients[i];
	IPC_t		*ip = &SockArray[i];
	vmpHdr_t	*hp;
	int		shutdownclient = 0, firstreq = 0;

	nLogMsg(EL_DEBUG2, id, "clnt slot %d", i);
	if (ip->sock_state == SS_NOSOCK)
		nLogMsg(EL_STATUS1, id, "msg from former pid %d", cp->c_pid);

	hp = vRcvPkt(ip, IPC_NDELAY);
nLogMsg(EL_STATUS1, id, "got %s", vHdrStr(hp));
	if (hp == NULL) {
		char	*msg = "shutting down client";

		switch (ip->sock_state) {
		    case SS_CONNECTED:
			nLogMsg(EL_STATUS1, id, 
				"%s after connect, slot %d", msg, i);
			break;
		    case SS_HELLOD:
			nLogMsg(EL_STATUS1, id, 
				"%s after hello, pid %d slot %d", 
				msg, ip->sock_id, i);
			break;
		    case SS_TALKING:
			nLogMsg(EL_STATUS1, id, 
				"%s while talking, pid %d slot %d", 
				msg, ip->sock_id, i);
			break;
		    case SS_GOODBYE:
			nLogMsg(EL_STATUS2, id, 
				"%s after goodbye, pid %d slot %d", 
				msg, ip->sock_id, i);
			break;
		    default:
			nLogMsg(EL_FATAL, id, "unexpected sock_state %d",
				ip->sock_state);
		}
		if (ip->sock_state == SS_HELLOD) {
			hellogoodbye++;
			if (unlink(vIDFile(cp->c_fileid)) == -1)
				nLogMsg(EL_SYS|EL_CONSOLE0|EL_ERROR1, id, 
						"unlink %s failed",
						vIDFile(cp->c_fileid));
		}
		if (ip->sock_state == SS_TALKING) {
			/*
			 *  vold needs to be informed
			 */
			char	buf[PKTSIZ(M_GOODBYE)];

			hp = vBuildHdr(M_GOODBYE, NOSEQ, (vmpHdr_t *)buf);
			hp->vh_slot = cp->c_id;
			RPCLSend(hp, 1);
		}
		CDShutdownClient(cp);
		return;
	}

	if (hp->vh_type == M_HELLO) {
		IPCMHello(hp, cp, ip);
		return;
	}

	if (ip->sock_state == SS_HELLOD) {
		/*
		 *  this is the 1st msg after a HELLO 
		 */
		ip->sock_state = SS_TALKING;
		/*
		 *  remove the security file
		 */
		if (unlink(vIDFile(cp->c_fileid)) == -1) {
			nLogMsg(EL_SYS|EL_CONSOLE0|EL_ERROR1, id, 
					"unlink %s failed",
					vIDFile(cp->c_fileid));
		}
		/*
		 *  reject anyone wanting a priority connect
		 *	who isn't entitled
		 */
		if (cp->c_flags & CL_PRIO  && cp->c_uid != 0 &&
						cp->c_uid != VOLMANUID) {
			VMError = E_USER;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
				"Client unauthorized for priority connection");
			CDErr2Client(hp->vh_seq, cp);
			/*
			 *  HACK - sleep to give client chance to rcv
			 *	before shutdown, otherwise it will
			 *	think the vold died
			 */
			sleep(1);
			CDShutdownClient(cp);
			return;
		}

		/*
		 *  check slot count / priority status / req type
		 */
		if (totalClients > unresvSlots && !(cp->c_flags & CL_PRIO)) {

			VMError = E_COMM;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
				  "Out of non-priority client connections");
			CDErr2Client(hp->vh_seq, cp);
			CDShutdownClient(cp);
			/*
			 *  HACK - sleep to give client chance to rcv
			 *	before shutdown, otherwise it will
			 *	think the volcd died
			 */
			sleep(2);
			return;
		} 

		firstreq = 1;
	}

	if (firstreq) {
		/*
		 *  Notify vold that there's a new client
		 */
		char		buf[PKTSIZ(vmpHello_t)];
		vmpHdr_t	*hp2;
		vmpHello_t	*hellop;

		/*
		 *  -if client is shutting down immediately,
		 *	no need to inform vold
		 */
		if (hp->vh_type == M_GOODBYE) {
			CDShutdownClient(cp);
			return;
		}

		/* key is added in RPCLSend */
		hp2 = vBuildHdr(M_NEWCLIENT, NOSEQ, (vmpHdr_t *) buf);
		hp2->vh_slot = -1;
		hellop = MSGPTR(hp2, vmpHello_t);
		hellop->vhe_flag = cp->c_id;
		hellop->vhe_id = cp->c_uid;
		hellop->vhe_pid = cp->c_pid;
		hellop->vhe_version = VM_VERSION;
		RPCLSend(hp2, 1);	/* flush after sending client pkt */
	}

	/*
	 *  forward the request to vold
	 */
	hp->vh_slot = cp->c_id;
	RPCLSend(hp, 1);
	
	if (hp->vh_type == M_GOODBYE)
		CDShutdownClient(cp);

	nFree(hp);
}

/*
 *  FUNCTION:	IPCServer0()
 *
 *		Main select loop for new connections and input from clients.
 */
IPCServer0(int msg)
{
	char		*id = "IPCServer0/sockets";
	int		i, nsoc, socseen, procQ;
	struct stat	st;
	int		vold_sd = RPCLfd();

	/*
	 *  determine how many file descriptors are 
	 *	available for communications
	 */
	clientSlots = 0;
	for (i=0; i<FD_SETSIZE; i++) {
		if (fstat(i, &st) == -1) {
			if (errno == EBADF) 
				clientSlots++;
			else
				nLogMsg(EL_SYS, id, "fstat");
		}
	}
	clientSlots -= 2;	/* for good luck, host lookup, etc */
	unresvSlots = clientSlots - 5;

	nLogMsg(EL_STATUS1, id, "%d slots, %d unreserved", 
				clientSlots, unresvSlots);

	/*
	 *  make sure no extra msg from vold got buffered
	 *	when reading pkts at startup
	 */
	while (msg)
		msg = IPCVoldMsg();

	/*
	 *  go into select() loop
	 */
	while (1) {

		/*
		 *  set up array for select()
		 */
		FD_ZERO(&fds);
		FD_SET(vold_sd, &fds);
		FD_SET(client_port_sd, &fds);

		for (i=0; i < FD_SETSIZE; i++) {
			if (SockArray[i].sock_state != SS_NOSOCK) {
				FD_SET(SockArray[i].sock_fd, &fds);
			}
		}

		nLogMsg(EL_DEBUG4, id, "WAIT on select/rcv");
		while ((nsoc = select(FD_SETSIZE, &fds, (fd_set *) NULL, 
					(fd_set *) NULL, NULL)) == -1 
					&& errno == EINTR)
		time(&Now);
		if (nsoc == 0) {
			nLogMsg(EL_DEBUG2, id, "0 read.. -?");
			continue;
		}
		if (nsoc < 0) {
			nLogMsg(EL_SYS|EL_FATAL, id, "select failed");
			continue;
		}

		/*
		 *  handle 'perennial' sockets: vold msgs & client connects
		 */
		socseen = 0;
		if (FD_ISSET(vold_sd, &fds)) {
			FD_CLR(vold_sd, &fds);
			
			while (IPCVoldMsg());
			socseen++;
		}
		if (FD_ISSET(client_port_sd, &fds)) {
			FD_CLR(client_port_sd, &fds);
			IPCAcceptClientConn();
			socseen++;
		}

 		/*
		 *  process client msgs
		 */
		for (i=0; i<FD_SETSIZE && socseen<nsoc; i++) {
			if (FD_ISSET(i, &fds)) {
				socseen++;
				IPCClientMsg(i);
			}
		}
	}
}


IPCVoldMsg()
{
	char		*id = "IPCVoldMsg/sock";
	vmpHdr_t	*hp;
	Client_t	*cp;
	extern void	CDShutdown();

	hp = RPCLRecv(id);

	if (hp->vh_slot == -1) {
		/*
		 *  msg for volcd itself
		 */
		switch (hp->vh_type) {
		    case M_SHUTDOWN:
			nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
						"shutdown rcvd from vold");
			if (immortal)
				nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
				    "started with -i so waiting for new vold");
			if (hellogoodbye)
				nLogMsg(EL_CONSOLE0|EL_STATUS1, 0,
					"%d incomplete connections",
					hellogoodbye);
			if (foreign)
				nLogMsg(EL_CONSOLE0|EL_STATUS1, 0,
					"%d connections from foreign hosts!",
					foreign);
			if (immortal) {
				longjmp(jmpbuf_i, 1);
				nLogMsg(EL_FATAL|EL_SYS, 0, "longjmp");
			}
			exit(0);
		    case M_DEBUG:
			MMDebug(hp); break;
		    case M_NEWLOG:
			nLogMsg(EL_STATUS1, "MNewlog", 
				"============ Restarting log ================");
			nLogInitFile(vLogFile, EL_LEVEL, restartlog, NULL);
			nLogMsg(EL_STATUS1, "MNewLog", 
				"--------------------------------------------");
			nLogMsg(EL_STATUS1, "MNewLog", 
				"---------- RESTARTED LOG -------------------");
			break;
		    case M_NOP:
			break;
		    default:
			nLogMsg(EL_FATAL, id, "unexpected msg %s", vHdrStr(hp));
		}
		goto done;
	}

	/*
	 *  consistency checks
	 */
	if (hp->vh_slot < 0  ||  hp->vh_slot >= FD_SETSIZE) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "bad client slot (%d) %s", 
						hp->vh_slot, vHdrStr(hp));
		goto done;
	}
	nLogMsg(EL_DEBUG2, id, "for slot %d", hp->vh_slot);
	cp = &Clients[hp->vh_slot];
	if (cp->c_ipcp->sock_state != SS_TALKING) {
		/*
		 *  client bailed while waiting for response
		 */
		nLogMsg(EL_DEBUG2, id, "client not 'talking' %s", 
					vHdrStr(hp));
		goto done;
	}
	if (hp->vh_type == M_SIGNAL) {
		vmpVsn_t	*vp = MSGPTR(hp, vmpVsn_t);

		if (kill(cp->c_pid, vp->v_rc) < 0)
			nLogMsg(EL_STATUS2|EL_SYS, id, "kill (%d, %d)", 
						cp->c_pid, vp->v_rc);
		goto done;
	}
	
	/*
	 *  forward to client
	 */
	vSndPkt(hp, cp->c_ipcp);
done:
	nFree(hp);
	return(RPCLNextRecord());
}


/*
 *  IPCSnd2Client() - send msg, clean up if necc
 */
IPCSnd2Client(vmpHdr_t *hp, Client_t *cp)
{
	char	*id = "IPCSnd2Client";
	int	err;			/* retry count */

	vClearError();
	if (cp->c_flags & CL_SHUTDOWN) {
		nLogMsg(EL_DEBUG1, id, "skipped write to dead client (%s)",
		  vHdrStr(hp));
		return(E_CANCELED);
	}

	err = vSndPkt(hp, cp->c_ipcp);

	if (err != E_NOERROR) {
		if (VMError != E_BADPACKET) {
			nLogMsg(EL_STATUS1, id,
			  "Shutting down dead client (%d)", cp->c_pid);
			cp->c_flags |= CL_SHUTDOWN;
		}
	}
	return (VMError);
}






CheckClients()
{
	char		*id = "CheckClients";
	Client_t	*cp = Clients;
	IPC_t		*ip = SockArray;
	int		i, clnts = 0;
	time_t		timeout = time((time_t *) 0) - VM_SOCKTIMEOUT;

	for (i=0; i<FD_SETSIZE; i++, cp++, ip++) {
		if ( (ip->sock_state == SS_CONNECTED ||
		      ip->sock_state == SS_HELLOD) &&
		     (cp->c_time < timeout) ) {
			/*
			 *  client hasn't come back w/ key
			 */
			if (!IPCClose(ip->sock_fd)) 
				clnts++;
		}
	}
	if (clnts) {
		nLogMsg(EL_STATUS1, id, "freed %d slots", clnts);
		return(0);
	}
	return(1);
}

