
#define	MAXMOUNTERRS	3	/* before drive is varied off */

/* #define FILEDRIVES 		for hack vvm testing */

/*
 *  NAStore Volume Manager: Node Daemon
 *
 *  nd_1.c: volnd - volman node daemon - server prog to check labels & 
 *	set up /dev/vol/xxxxxx nodes. Rcvs msg from RCs via vold (with
 *	additional untested protocol to have vold originate requests), 
 *	forks process to do request, times out / waits for process, 
 *	informs vold of result if necc..
 *
 *	TODO: consider forking 1 proc per drive at startup to do
 *	all work on that drive.. unless it hangs...
 *
 *  As long as RC is at least checking label, might as well have
 *	the cmd create user node to avoid an extra vold<->volnd 
 *	exchange.  This is in spite of the fact that it would be 
 *	nice to have the vold create the node itself (implemented,
 *	not tested) since it is theoretically the user interface
 *	process, showing the mount at the other point of comm (ipc).
 *
 *  nd_2.c: child routines
 *
 *  Bill Ross, NASA Ames, July 1995
 */


#ifdef EXTERN
# undef EXTERN
#endif
#define EXTERN
#include "srv_space.h"
#include "volnd.h"

#include <stdio.h>
#include <dirent.h>
#include <signal.h>
#include <sys/wait.h>
#include <pwd.h>
#include <setjmp.h>

extern DevEnt_t		*DriveArray;
extern int		DriveTotal;

extern int		immortal;
extern jmp_buf		jmpbuf_i;

Node_t			NodeArray[MAXDEVS];
time_t			Now, LastTime = 0;
struct passwd		*pw;

char			**args;		/* argv */

int			host_type, host_id;

static DIR		*dirp = NULL;
static int		IdleSize = 0;
static vmpHdr_t		*vIdlePkt = NULL;

 void
usage()
{
	fprintf(stderr, "usage: %s [-i]\n", args[0]);
	exit(1);
}

 int
bomb()
{
	RPCLGoodbye();
	sleep(3);		/* try to let vold rcv goodbye */
	kill(0, SIGTERM);	/* try to take out children, too */
	exit(1);
	return(1);		/* for compiler warning */
}

 void
DriveError(int drive)
{
	char		buf[80];

	NodeArray[drive].d_err++;
	if (NodeArray[drive].d_err < MAXMOUNTERRS)
		return;

	/*
	 *  too many errors - vary off
	 */
	sprintf(buf, "volnd: %s - %d failures",
		DriveArray[drive].de_name, NodeArray[drive].d_err);
	RPCLVaryOff(drive, buf);
}

/*
 *  WaitStatus() - handle exit status returned by child via
 *	wait() or waitpid()
 */
  static void
WaitStatus(int drive, int status)
{
	char	*id = "WaitStatus";

	switch (status) {
	    case EXIT_OK:
		if (NodeArray[drive].d_cmd == M_CREATENODE  ||
		    NodeArray[drive].d_cmd == M_SCRATCH)
			NodeArray[drive].d_err = 0;
		break;
	    case EXIT_FAILVOL:
		/* mount error relating to this tape or any dismount err */
		break;
	    case EXIT_FAILDEV:
		/* mount error relating to drive */
		DriveError(drive);
		break;
	    default:
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
		    "unexpected response 0x%x pid %d drive %s",
		    status, NodeArray[drive].d_pid, DriveArray[drive].de_name);
	}
}

/*
 *  SigChild() - handle status & reset signal-catcher
 */
 void
SigChild(int sig)
{
	char		*id = "SigChild";
	int		status, pid, csig, drive;

	/*
	 *  handle all dead children
	 */
	while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
		if (status == 0)
			nLogMsg(EL_DEBUG1, id, "pid %d - status 0x%x", 
							pid, status);
		else
			nLogMsg(EL_STATUS1, id, "pid %d - status 0x%x", 
							pid, status);
		status &= 0177777;
		csig = status & 0177;
		status >>= 8;
		if (csig) 
			nLogMsg(EL_STATUS1, id, 
					"child %d terminated on signal %d",
					pid, csig);
		/*
		 *  figure out which drive was involved
		 */
		for (drive=0; drive<DriveTotal; drive++)
			if (pid == NodeArray[drive].d_pid)
				break;
		if (drive == DriveTotal) {
			nLogMsg(EL_STATUS2, id, 
					"pid %d forgotten - status 0x%x", 
					pid, status);
			continue;
		}
		if (NodeArray[drive].d_cmd == 0) {
			nLogMsg(EL_STATUS2, id, 
				"drive %s pid %d cmd forgotten - status 0x%x",
				DriveArray[drive].de_name, pid, status);
			continue;
		}

		/*
		 *  handle drive, reset it
		 */
		WaitStatus(drive, status);
		NodeArray[drive].d_pid = 0;
		NodeArray[drive].d_cmd = 0;
	}

	if (pid == -1  &&  errno != ECHILD)	/*  ECHILD == no child */
		nLogMsg(EL_SYS, id, "waitpid (status 0x%x)", status);

	/*
	 *  reset catcher
	 */
	if (SIGNAL(sig, SigChild))
		nLogMsg(EL_FATAL|EL_SYS, id, "signal SIGCHLD");
}

/*
 *  CheckChild() - to be sure a sigcld didn't get lost (may
 *	be overkill) - see if info can be gotten
 *	return=1 means child status handled.
 */
CheckChild(int drive)
{
	char	*id = "CheckChild";
	Node_t	*np = &NodeArray[drive];
	int	ret, status, csig;

	/*
	 *  waitpid() w/ 'instant return'
	 */
	ret = waitpid(np->d_pid, &status, WNOHANG);
	if (ret == -1) {
		nLogMsg(EL_SYS, id, "waitpid");
		if (errno == ECHILD) {
			/*
			 *  child death wasn't detected (?)
			 *	assume all went well
			 */
			WaitStatus(drive, EXIT_OK);
			NodeArray[drive].d_pid = 0;
			NodeArray[drive].d_cmd = 0;
			return(1);
		}
		return(0);
	} 
	if (ret == np->d_pid) {
		/*
		 *  child's SIGCLD was lost, handle
		 *	as if receiving now -translate
		 */
		status &= 0177777;
		csig = status & 0177;
		status >>= 8;
		if (csig) 
			nLogMsg(EL_STATUS1, id, 
					    "child %d terminated on signal %d",
					    np->d_pid, csig);
		WaitStatus(drive, status);
		NodeArray[drive].d_pid = 0;
		NodeArray[drive].d_cmd = 0;
		return(1);
	}
	if (ret == 0)
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
			"pid %d hung on open or read - problem w/ drive",
			np->d_pid);
	return(0);
}

void
ResetDrives()
{
	int	i;

	for (i=0; i<DriveTotal; i++) {
		Node_t	*np = &NodeArray[i];
		if (np->d_pid  &&  np->d_pid != -1  &&  np->d_cmd)
			kill(np->d_pid, SIGTERM);
		np->d_pid = 0;
		np->d_cmd = 0;
	}
}


/*
 *  VNDriveConfig() - make node names for local host type
 */
 static void
VNDriveConfig()
{
	STAT_T	sbuf;
	int	i, ct = 0;

	RPCLDriveConfig();
	memset(NodeArray, 0, sizeof(NodeArray));
	for (i=0; i<DriveTotal; i++) {
		if (DriveArray[i].de_host != host_id)
			continue;
		/* let's get a little wild & do a blind copy.. */
		strcpy(NodeArray[i].d_devname, 
			vDriveName(host_type, DriveArray[i].de_controller, 
						DriveArray[i].de_devno,
						DriveArray[i].de_medium));
		if (strlen(NodeArray[i].d_devname) >= 
						sizeof(NodeArray[i].d_devname))
			nLogMsg(EL_FATAL, 0, "device len too long");
		NodeArray[i].d_rcflags = DriveArray[i].de_rcflags;
		if (NodeArray[i].d_devname[0] == '\0')
			nLogMsg(EL_FATAL, 0, 
				"%s: medium not supported on this host",
				DriveArray[i].de_name);
#ifndef FILEDRIVES
		if (STAT(NodeArray[i].d_devname, &sbuf) == -1)
			nLogMsg(EL_FATAL|EL_SYS, 0, "%s (%s): can't stat",
						NodeArray[i].d_devname, 
						DriveArray[i].de_name);
#endif
		ct++;
	}
	if (ct == 0)
		nLogMsg(EL_STATUS1|EL_TOSTDERR, "volman config", 
		  			"(no drives configged on this host)");
}

 void
NDErr2RP(vsn_t *vsnp, Seq_t seq, int req, int rc, int retry)
{
	if (seq == NOSEQ)
		return;
	RPCLErr2RP(vsnp, seq, req, rc, (char) retry);
}

/*
 *  CheckIdleness() - make sure nothing happening w/ this drive
 */
 static int
CheckIdleness(int drive, vmpNode_t *nop, Seq_t seq, char *id)
{

	if (NodeArray[drive].d_cmd != 0) {
		nLogMsg(EL_ERROR1, id, "unexpected status: %s %s",
					DriveArray[drive].de_name, 
					vMsgType(NodeArray[drive].d_cmd));
		/*
		 *  see if that child is still alive
		 */
		(void)CheckChild(drive);
		if (NodeArray[drive].d_cmd != 0) {
			/*
			 *  TODO - maybe just go ahead anyway
			 */
			NDErr2RP(&nop->vs_vsn, seq, 
					nop->vs_req, nop->vs_rc, 1);
			return(1);
		}
	}
	return(0);
}


/*
 *  MNode() - create/delete request from RC (via vold) or vold 
 *	(create from vold not tested)
 */
 static void
MNode(vmpHdr_t *hp)
{
	char		*id = "MNode";
	vmpNode_t	*nop = MSGPTR(hp, vmpNode_t);
	int		i, drive = nop->vs_drive;
	static sigset_t	block_sig;

	pw = NULL;

	/*
	 *  check pkt
	 */
	if (drive < 0  ||  drive >= MAXDEVS) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "bad drive: %d", drive);
		NDErr2RP(&nop->vs_vsn, hp->vh_seq, nop->vs_req, nop->vs_rc, 0);
		return;
	}
	if (NodeArray[drive].d_devname[0] == '\0') {
		VMError = E_INTERNAL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "host doesn't have drive: %d:%s",
					drive, DriveArray[drive].de_name);
		NDErr2RP(&nop->vs_vsn, hp->vh_seq, nop->vs_req, nop->vs_rc, 0);
		return;
	}
	if (hp->vh_type == 0) {
		nLogMsg(EL_ERROR1, id, "%.*s_%d on %s: cmd is 0", 
			VSNLEN, &nop->vs_vsn, hp->vh_seq,
			DriveArray[drive].de_name);
		return;
	}

	if (hp->vh_seq == NodeArray[drive].d_seq  &&
	    hp->vh_type == NodeArray[drive].d_cmd  &&
	    !memcmp(&NodeArray[drive].d_req, nop, sizeof(*nop))) {
		/*
		 *  possibly dup reqs can occur via
		 *	existence of forked children
		 *	on socket?
		 */
		nLogMsg(EL_STATUS1, id, "%.*s_%d: discarding dup %s",
			VSNLEN, &nop->vs_vsn, hp->vh_seq,
			(hp->vh_type == M_CREATENODE ? "create" : "delete"));
		return;
	}

	if (hp->vh_type == M_CREATENODE) {

		/*
		 *  if CANCEL, deal w/ original req
		 */
nLogMsg(EL_STATUS1, id, (nop->vs_flag & VO_WRITE ? "write" : "readonly"));
		if (nop->vs_flag & VO_CANCEL) {
			/*
			 *  VO_CANCEL is handled by killing the
			 *	child, like a timeout; no ack for now.
			 */
			if (NodeArray[drive].d_cmd != M_CREATENODE) {
				nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"cancel but no create");
				return;
			}
	    		NodeArray[drive].d_cmd = 0;
			nLogMsg(EL_STATUS1, id, "cancel/kill %d",
						NodeArray[drive].d_pid);
			if (kill(NodeArray[drive].d_pid, SIGTERM) == -1)
				nLogMsg(EL_SYS|EL_ERROR1, id, "cancel/kill %d", 
							NodeArray[drive].d_pid);
			return;
		}
		/*
		 *  not cancel, so should be idle
		 */
		if (CheckIdleness(drive, nop, hp->vh_seq, "createnode"))
			return;
		/*
		 *  make sure uid ok 
		 *	(important for CREATENODE, not DELETENODE)
		 */
		if ((pw = getpwuid(nop->vs_uid)) == NULL) {
			VMError = E_INTERNAL;
			nLogMsg(EL_CODE|EL_SYS, id, 
				"getpwuid(%d) failed", nop->vs_uid);
			NDErr2RP(&nop->vs_vsn, hp->vh_seq, nop->vs_req, 
					nop->vs_rc, 0);
			return;
		}
	} else if (hp->vh_type == M_SCRATCH) {
		if (CheckIdleness(drive, nop, hp->vh_seq, "scratch"))
			return;
	}

	/*
	 *  track request
	 */
	NodeArray[drive].d_cmd = hp->vh_type;
	NodeArray[drive].d_seq = hp->vh_seq;
	NodeArray[drive].d_req = *nop;
	NodeArray[drive].d_time = Now;

	/*
	 *  child does the work, single parent waits on all results
	 */

	for (i=1; i<5; i++) {
		/*
		 *  Block notice of any children dying while forking, since
		 *	we have observed the child to complete and the
		 *	caught signal to cause reset of d_pid before the
		 *	test of it happens.
		 */
		sigemptyset(&block_sig);
		sigaddset(&block_sig, SIGCHLD);
		if (sigprocmask(SIG_BLOCK, &block_sig, NULL) == -1)
			nLogMsg(EL_FATAL|EL_SYS, id, "sig_block");

		/*
		 *  try the fork
		 */
		if ((NodeArray[drive].d_pid = fork()) != -1)
			break;

		/*
		 *  fork failed; reinstate SIGCHLD, checking for
		 *	any dead children, before waiting to retry
		 */
		nLogMsg(EL_ERROR1|EL_SYS, id, "fork");

		sigemptyset(&block_sig);
		sigaddset(&block_sig, SIGCHLD);
		if (sigprocmask(SIG_UNBLOCK, &block_sig, NULL) == -1)
			nLogMsg(EL_FATAL|EL_SYS, id, "sig_unblock");
		sleep(i*i*i);
	}
	if (NodeArray[drive].d_pid == -1)
		nLogMsg(EL_FATAL|EL_SYS|EL_CONSOLE0, id, "can't fork");

	/*
	 *  child does its thing
	 */
	if (NodeArray[drive].d_pid == 0)
		exit(NodeChild(drive));

	/*
	 *  parent notes the event & turns on SIGCHLD again
	 */
	nLogMsg(EL_STATUS1, 
			(hp->vh_type == M_CREATENODE ? "CREATE" : "DELETE"),
			"%.*s_%d on %s: forked %d", 
			VSNLEN, &nop->vs_vsn, hp->vh_seq,
			DriveArray[drive].de_name,
			NodeArray[drive].d_pid);
	sigemptyset(&block_sig);
	sigaddset(&block_sig, SIGCHLD);
	if (sigprocmask(SIG_UNBLOCK, &block_sig, NULL) == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "sig_unblock");
}

/*
 *  MTimer() - heartbeat from vold 
 */
 static void
MTimer(vmpHdr_t	*hp)
{
	char		*id = "MTimer";
	vmpTimer_t	*tp;
	int		i, nvsns;
	vmpNodeTime_t	*ip;
	struct dirent	*dp;
	STAT_T		sbuf;

	/*
	 *  since we are pkt-driven, make sure they have not
	 *	piled up somehow - no point in thrashing the
	 *	log file
	 */
	if (Now - LastTime < 5)
		return;
	LastTime = Now;

	/*
	 *  consider msg time-of-flight from vold
	 */
	tp = MSGPTR(hp, vmpTimer_t);
	if (Now - (time_t) tp->vt_time > 5) 
		nLogMsg(EL_STATUS1, id, "(msg time from vold %d)", 
					Now - (time_t) tp->vt_time);

	/*
	**      send pkt back for vold accounting
	*/
	RPCLSend(hp, 1);

	/*
	 *  check requests
	 */
	for (i=0; i<DriveTotal; i++) {
		Node_t	*np = &NodeArray[i];

		if (!np->d_cmd)
			continue;
		if (np->d_cmd == M_CREATENODE  &&  np->d_req.vs_flag & VO_WAIT)
			continue;		/* no timeout */
		if (Now - np->d_time > TIMEOUT) {
			/*
			 *  see if a SIGCLD was missed
			 */
			if (CheckChild(i))
				continue;

			VMError = E_UNAVAIL;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"%.*s_%d: pid %d hung on %s on %s",
                        	VSNLEN, &np->d_req.vs_vsn, np->d_seq, np->d_pid,
                        	(np->d_cmd == M_CREATENODE ? 
						"open or read" : "delete"),
                        	np->d_devname);
			NDErr2RP(&np->d_req.vs_vsn, np->d_seq, 
				 np->d_req.vs_req, np->d_req.vs_rc, 1);

			/*
			 *  assume it's a drive not a volume problem
			 */
			if (np->d_cmd == M_CREATENODE)
				DriveError(i);

			/*
			 *  plan to ignore child death (cmd=0)
			 *	and try to kill it
			 */
	    		np->d_cmd = 0;
			if (!kill(np->d_pid, SIGTERM))
				nLogMsg(EL_SYS, id, "kill %d", np->d_pid);
		}
	}

	/*
	 *  check node idle times
	 */
	nvsns = 0;
	rewinddir(dirp);
	for (i=0; dp = readdir(dirp); i++) {
		/*
		 *  make sure it's a vsn & get info
		 */
		if (strlen(dp->d_name) != VSNLEN)
			continue;
		if (STAT(dp->d_name, &sbuf) == -1) {
			nLogMsg(EL_SYS, id, "stat(%s)", dp->d_name);
			continue;
		}
		if (!S_ISCHR(sbuf.st_mode))
			continue;

		/*
		 *  make sure there's space in pkt
		 */
		if (nvsns == IdleSize) {
			IdleSize += 10;
			vIdlePkt = (vmpHdr_t *) nReAlloc(vIdlePkt, 
					PadSize(sizeof(vmpHdr_t)) +
					IdleSize * sizeof(vmpNodeTime_t));
		}
		ip = MSGPTR(vIdlePkt, vmpNodeTime_t);
		ip += nvsns;

		/*
		 *  stuff pkt
		 */
		memcpy(&ip->t_vsn, dp->d_name, VSNLEN);
		ip->t_idle = Now - MAX(sbuf.st_atime, sbuf.st_ctime);
		if (ip->t_idle < 0)
			ip->t_idle = 0;
		nvsns++;
	}
	if (!nvsns)
		return;
	vIdlePkt->vh_count = nvsns;
	vIdlePkt->vh_size = nvsns * sizeof(vmpNodeTime_t);
	RPCLSend(vIdlePkt, 1);
}



main(int argc, char *argv[])
{
	char		*id = argv[0];
	int		cc, i, err, dflg=0;
	extern int	optind;
	extern char	*optarg;

	args = argv;
	CheckUID(argv[0]);
	host_type = vHostCode(vHostType);
	if (host_type == -1) {
		fprintf(stderr, "%s: host type %s not configured - exiting\n", 
							argv[0], vHostType);
		exit(1);
	}
	/*
	 *  parse args
	 */
	while ((cc = getopt(argc, argv, "i")) != EOF) {
		switch (cc) {
		    case 'i':
			immortal = 1;
			break;
		    default:
			fprintf(stderr, "unexpected option (%c)\n", cc);
			usage();
		}
	}
	if (optind < argc) {
		fprintf(stderr, "argc %d, last [%s]\n", argc, argv[argc-1]);
		usage();
	}

	/*
	 *  go into the background
	 */
	if (!dflg) {
		switch (fork()) {
		    case -1:				/* fork failed */
			perror("fork");
			exit(1);
		    case 0:				/* child */
			if (setpgid(0,0))
				perror("setpgid");
			break;
		    default:				/* parent */
			exit(0);	/* detach */
		}
	}

	/*
	 *  get to an appropriate place in case core is dumped
	 */
	vInitDirNames();
	nLogInit("ND", bomb);
	nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
	nLogInitConsole(rp_syslog, EL_CONSOLE0, EL_LEVEL);
	nCreatePath(vNodeDir, 0755);
	chdir(vNodeDir);

	/*
	 *  open NODEDIR for scans of idle times
	 *	and set up pkt hdr
	 */
	if ((dirp = opendir(vNodeDir)) == NULL)
		nLogMsg(EL_FATAL|EL_SYS, id, "opendir(%s)", vNodeDir);
	IdleSize = 10;
	vIdlePkt = (vmpHdr_t *) nAlloc(PadSize(sizeof(vmpHdr_t)) +
					IdleSize * sizeof(vmpNodeTime_t));
	vIdlePkt->vh_type = M_IDLETIME;
	vIdlePkt->vh_magic = VMP_MAGIC;
	vIdlePkt->vh_seq = NOSEQ;
	vIdlePkt->vh_slot = -1;

	if (SIGNAL(SIGINT, bomb))
		nLogMsg(EL_FATAL|EL_SYS, id, "signal SIGINT");
	if (SIGNAL(SIGCHLD, SigChild))
		nLogMsg(EL_FATAL|EL_SYS, id, "signal SIGCHLD");

	/*
	 *  prepare for immortality
	 */
	if (setjmp(jmpbuf_i)) {
		nLogMsg(EL_STATUS1, 0, "setjmp to beginning ok");
		ResetDrives();
		sleep(30);
	}

	/*
	 *  connect/register w/ vold & get config; goes to setjmp
	 *	above if failure and -i arg used
	 */
	host_id = host_type;
	RPCLInit(M_NODED, &host_id);	/* sets host id */

	VNDriveConfig();

#ifndef FILEDRIVES
	CreateEmptyDir(vNodeDir);
#endif

	nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, "volnd initialized (pid %d)", 
								getpid());
	/*
	 *  service node setup/delete requests from/via vold
	 */
	while (1) {
		vmpHdr_t	*hp;
		int		err = 0;

		/*
		 *  since only RPCLRecv is used to drive the
		 *	main loop - i.e. no select())  - no need to use
		 *	the result of RPCLNextRecord() to see if anything 
		 *	is buffered.
		 */
		hp = RPCLRecv("main loop");
		RPCLNextRecord();

		nLogMsg(EL_DEBUG1, id, vHdrStr(hp));

		time(&Now);	/* a unique second of eternity */

		switch (hp->vh_type) {
		    case M_CREATENODE:
		    case M_DELETENODE:
		    case M_SCRATCH:
			MNode(hp); break;
		    case M_TIMER:
			MTimer(hp); break;
		    case M_SHUTDOWN:
			if (immortal) {
				nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, "shutdown");
				longjmp(jmpbuf_i, 1);
				nLogMsg(EL_SYS, 0, "longjmp on shutdown");
			}
			nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
						"shutdown rcvd - exiting");
			RPCLGoodbye();
			kill(0, SIGTERM); /* try to take out children, too */
		    case M_NEWLOG:
			/*
			 *  sleep a while so volcd has plenty of
			 *	time to create the new log file
			 */
			sleep(30);

			/*
			 *  last msg to old file
			 */
			nLogMsg(EL_STATUS1, "MNewlog", 
				"============ Restarting log ================");

			/*
			 *  start new file
			 */
			nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
			nLogMsg(EL_STATUS1, "MNewLog", 
				"--------------------------------------------");
			nLogMsg(EL_STATUS1, "MNewLog", 
				"---------- RESTARTED LOG -------------------");
			break;
		    case M_DEBUG:
			MMDebug(hp); break;
		    case M_NOP:
			break;
		    default:
			nLogMsg(EL_ERROR1|EL_CONSOLE0, 0, 
						"unexpected msg type %s",
						vHdrStr(hp));
			break;
		}
		nFree(hp);
	}
}





 void
PrintNodeArray(char *str)
{
	int	i;

	nLogMsg(EL_STATUS1, str, "-------------- ");

	for (i=0; i<DriveTotal; i++) {
		Node_t	*np = &NodeArray[i];

		nLogMsg(EL_STATUS1, 0, "%d  %s  cmd %d  time %d",
				i, np->d_devname, np->d_cmd, np->d_time);
	}
	nLogMsg(EL_STATUS1, 0, "-------------- %s", str);
}
