/*
 *  nd_2.c: volman node daemon child routines for doing the
 *	drive access / node creation/deletion.
 */

/* #define NO_GETSTATUS */
#define NO_ERASE

/* #define FILEDRIVES		hack for vvm testing */

#include <pwd.h>
#include <signal.h>

#include "srv_space.h"
#include "nas_tape.h"
#include "volnd.h"


extern Node_t		NodeArray[];
extern int		DriveTotal;
extern struct passwd	*pw;
extern DevEnt_t		*DriveArray;

extern int		bomb();
extern char		**args;		/* argv */

static int		HangTime = 0;

/*
 *  since this process handles only 1 request,
 *	make error pkt info global, locally.
 */
static Seq_t	seq;
static int	req, rc;

/*
 *  Keep a global pointer to the request to help construct
 *	an err msg (if necc to err out from the signal catcher).
 */
static Node_t	*Np = NULL;

static int	sigged = 0;
static int	mountmsg = 0, msgfd;

static void	SendOK(int drive);
static void	CheckDev(int drive, Node_t *np, int fd, char *devnam);
static void	MakeNode(int drive, Node_t *np, char *devnam);

/*
**	HACK for different styles of tape drive interface for mount:
**		open-as-poll (netbsd) vs. open-and-poll (others..?)
**		Later will package-ize this?
*/
#ifdef __NetBSD__
# include "nd_2a.c"
#else
# include "nd_2b.c"
#endif

/*
 *  Children catch signals from parent sent for timeout or shutdown.
 *	This should work on cancel of WaitCreateNode, but probably 
 *	won't help with driver problems since hang in driver is
 *	unkillable (at least w/ Convex). Because of this, the parent
 *	sends the error msg to vold on timeout. Always exit w/ no
 *	problem, since parent is handling status.
 */

 static int
sigterm()
{
	char	*id = "sigterm";

	if (mountmsg) {
		/*
		 *  cancel msg
		 */
		nLogMsg(EL_STATUS2, id, "cancelling mount msg");
		if (LED34X0(msgfd, 0, NULL, NULL) == -1)
			nLogMsg(EL_SYS|EL_ERROR1, id, "tapled");
	}
	if (sigged) {
		nLogMsg(EL_STATUS1, 0, "%.*s_%d: 2 sigterm cancel/timeout", 
				VSNLEN, &Np->d_req.vs_vsn, Np->d_seq);
		exit(EXIT_OK);
	}
	nLogMsg(EL_STATUS1, 0, "%.*s_%d: sigterm: cancel/timeout/shutdown", 
				VSNLEN, &Np->d_req.vs_vsn, Np->d_seq);
	sigged = 1;
/* try exit on 1st */
	exit(EXIT_OK);
	return(0);		/* for compiler warning */
}

static void
SendOK(int drive)
{
	char		buf[PKTSIZ(vmpNode_t)];
	vmpHdr_t	*hp;
	vmpNode_t	*nop;

	hp = vBuildHdr(NodeArray[drive].d_cmd, seq, (vmpHdr_t *)buf);
	nop = MSGPTR(hp, vmpNode_t);
	*nop = NodeArray[drive].d_req;
	RPCLSend(hp, 1);
}

 static void
CheckDev(int drive, Node_t *np, int fd, char *devnam)
{
	char	*id = "CheckDev";
	int	flag = np->d_req.vs_flag, i, saverr;
	char	*vsnp = (char *) &np->d_req.vs_vsn;

	/*
	**	get device status for write protect checks
	*/
#ifndef NO_GETSTATUS
	if (ntp_getstatus(fd, vsnp, 0) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, id, 
				"get status failed on %s", devnam);
		SaveErrs();

		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}

	/*
	**	check physical protection
	*/
	if ((flag & VO_WRITE) && ntp_write_protected(vsnp)) {

		/*
		**	mounted for write && no write ring
		*/
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		VMError = E_USER;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR2, 0,
				"%s: %s %s", vsnstr(vsnp),
				"Volume not physically write enabled ",
				"- contact operator");
		NDErr2RP(vsnp, seq, req, rc, 0);
		exit(EXIT_FAILVOL);
	}

	if (!(flag & VO_WRITE)  &&  !ntp_write_protected(vsnp)
				&& !(flag & VO_IGNORE_RING)) {

		/*
		**	mounted for read && physically writeable & 
		**		no ignore-ring
		*/
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		VMError = E_USER;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR2, 0, "%s %s",
				"Volume physically write enabled ",
				"but mounted for read - contact operator");
		NDErr2RP(vsnp, seq, req, rc, 0);
		exit(EXIT_FAILVOL);
	}
#endif /* ifndef NO_GETSTATUS */
	/*
	**	check label if bypass not req'd
	*/
	if (!(flag & VO_BYPASS)) {
		nLogMsg(EL_DEBUG3, id, "checking label: %.*s", 
						VSNLEN, &np->d_req.vs_label);
		if (LabelProblem(drive, fd, &np->d_req.vs_label, vsnp, 
							np, devnam)) {
			SaveErrs();
			close(fd);
			(void)ClearDev(drive, np, devnam, 1, 0);
			RestoreErrs();
			if (VMError == E_SYS) {
				NDErr2RP(vsnp, seq, req, rc, 1);
				exit(EXIT_FAILDEV);
			}
			NDErr2RP(vsnp, seq, req, rc, 0);
			exit(EXIT_FAILVOL);
		}
		nLogMsg(EL_DEBUG3, 0, "label ok");
	}

	/*
	**	note write-ring status for return pkt to vold
	*/
#ifndef NO_GETSTATUS
	if (ntp_write_protected(vsnp))
		np->d_req.vs_flag = VV_RING;
	else
		np->d_req.vs_flag = 0;
#else
	np->d_req.vs_flag = 0;
#endif
	nLogMsg(EL_DEBUG2, 0, "%s (%s) ok",
			vsnstr(vsnp), devnam);
}

/*
 *  MakeNode(): create node in NODEDIR for user access
 */
  static void
MakeNode(int drive, Node_t *np, char *devnam)
{
	char		*id = "MakeNode";
	char		*vsnp = (char *) &np->d_req.vs_vsn;
	char		nodnam[MAXPATHLEN];
	char		buf[VOL1LEN+1];
	DevEnt_t	*dep;
	struct stat	statb;
	dev_t		devno;
	int		fd, mode;

	/*
	 *  set up the /dev/vol/xxxxxx node
	 */
	sprintf(nodnam, "%s/%.*s", vNodeDir, VSNLEN, vsnp);

#ifdef FILEDRIVES
	if ((fd=open(nodnam, O_CREAT|O_RDWR, 0600)) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_SYS, id, "open %s", nodnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILVOL);
	}
	sprintf(buf, "VOL1%.6s", vsnp);
	memset(buf+10, ' ', VOL1LEN-10);
	if (lseek(fd, 0, SEEK_SET)!= 0) {
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_SYS, id, "lseek");
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILVOL);
	}
	if (write(fd, buf, VOL1LEN) != VOL1LEN) {
		nLogMsg(EL_CONSOLE0|EL_CODE, id, "writing lbl");
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILVOL);
	}
#else
	/*
	 *  normal setup of /dev/tape/ node that copies tape device
	 *
	 *  	make sure /dev/vol/ doesn't already exist
	 */
	if (stat(nodnam, &statb) != -1) {		/* test node name */
		/*
		 * there is no way that the node should exist here;
		 *	but if it does, unlink it;
		 */
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
		  "%s: node already exists; removing node", nodnam);
		if (unlink(nodnam)) {
			VMError = E_SYS;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_SYS, id, 
				"unlinking %s", nodnam);
			SaveErrs();
			(void)ClearDev(drive, np, devnam, 1, 0);
			RestoreErrs();
			NDErr2RP(vsnp, seq, req, rc, 1);
			exit(EXIT_FAILVOL);
		}
	}

	/*
	 *  get tape device info
	 */
	if (stat(devnam, &statb) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_CODE|EL_SYS, id, "can't stat drive %s", devnam);
		SaveErrs();
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}

	/*
	 *  make /dev/vol/ node
	 */
	mode = statb.st_mode;
	devno = statb.st_rdev;
	mode &= S_IFMT;
	mode &= ~0777;
	if (np->d_req.vs_flag & VO_WRITE)
		mode |= 0600;
	else
		mode |= 0400;
	nLogMsg(EL_STATUS1, id, "mknod(%s,%o,%x)", nodnam, mode, devno);
	if (mknod(nodnam, mode, (int) devno) < 0) {
		VMError = E_SYS;
		nLogMsg(EL_CODE|EL_SYS, id, "can't mknod %s", nodnam);
		SaveErrs();
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILVOL);
	}
#endif
	/*
	 *  set node ownership
	 */
	if (chown(nodnam, pw->pw_uid, pw->pw_gid) < 0) {
		VMError = E_SYS;
		nLogMsg(EL_CODE|EL_SYS, id, "can't chown %s", nodnam);
		SaveErrs();
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILVOL);
	}
}

/*
 *  NodeChild() - forked to do the job; return status is exit code
 *	caught by parent: if ok (0), let parent inform vold; if
 *	err (1), send msg directly to vold.
 */
  int
NodeChild(int drive)
{
	char	devnam[MAXDEVLEN+10];
	char	buf[VSNLEN+50];
nLogSetLevel(EL_DEBUG3);
	/*
	 *  set the global request pointer and prepare
	 *	to catch timeout signals
	 */
	Np = &NodeArray[drive];
if((Np->d_rcflags & (RCF_REWIND|RCF_REWUNLOAD)) == 0)
nLogMsg(EL_FATAL, 0, "no rcflags");
	seq = Np->d_seq;
	req = Np->d_req.vs_req;
	rc = Np->d_req.vs_rc;
	if (SIGNAL(SIGTERM, sigterm))
		nLogMsg(EL_FATAL|EL_SYS, 0, "signal SIGTERM");

	/*
	 *  change process name so it will be apparent
	 *	which drive/vsn involved if proc hangs
	 * 
	 * Apparently impossible to do on all systems - pity..
	 *
	sprintf(buf, "volnd %s %.*s %s", 
			(Np->d_cmd == M_CREATENODE ? "" : "del"),
			VSNLEN, &Np->d_req.vs_vsn, Np->d_devname);
	vChangeName(args, buf);
	 */
	
	/*
	 *  handle the request
	 */
	switch (Np->d_cmd) {
	    case M_CREATENODE:
		/*
		 *  set up the global device name 
		 */
		strcpy(devnam, Np->d_devname);

		if (Np->d_req.vs_flag & VO_WAIT)	/* test for mount */
			WaitCreateNode(drive, Np, devnam);	/* exits */
		CreateNode(drive, Np, devnam);			/* exits */
		break;
	    case M_DELETENODE:
		DeleteNode(drive, Np);				/* exits */
		break;
	    case M_SCRATCH:
		strcpy(devnam, Np->d_devname);
		Scratch(drive, Np, devnam);			/* exits */
		break;
	    default:
		nLogMsg(EL_CONSOLE0|EL_ERROR1, 0, "bad cmd %d drive %s",
			Np->d_cmd, DriveArray[drive].de_name);
		exit(-1);					/* exits */
	}
	return(-1);	/* for compiler warning */
}





/*
 *  LabelProblem() - check ANSI VOL1 label or absence thereof
 */
  int
LabelProblem(int drive, int fd, char *ilabel, 
			char *vsnp, Node_t *np, char *devnam)
{
	char	*id = "LabelProblem";
	char 	label[VOL1LEN];
	int	len;
	time_t	start;

#ifdef FILEDRIVES
	return(0);
#endif
	/*
	 *  ilabel[0] == NULL => tape never written (virgin)
	 *  ilabel[0] == ' '  => non-label data on tape
	 *
	 *	Check that there is no ANSI label if either of above.
	 *	Otherwise, the label must be ANSI and the volser field
	 *	equal to ilabel. ANSI labels for this purpose consist of
	 *
	 *	Field		Contents
	 *
	 *	0-3		"VOL1"
	 *	4-9		alphanumeric volume serial number
	 *	79		"4", " ", ...
	 *
	 */
	
	nLogMsg(EL_DEBUG2, 0, "reading label");
	time(&start);

	if (ntp_taprew(fd, vsnp, EL_CODE) == -1) {
		if (sigged)
			return(1);
		VMError = E_SYS;
		SaveErrs();
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
                NDErr2RP(vsnp, seq, req, rc, 0);
                exit(EXIT_FAILDEV);
	}
	if ((len = read(fd, label, VOL1LEN)) == -1) {
		if (sigged)
			return(1);

	 	/* note - don't close fd; caller uses it again */
		if (errno == EIO) {
			/*
			 *  EIO's can take minutes in some drivers
			 *	so note the fact in to avoid
			 *	timing out other reqs; TODO - time open
			 */
			start = time((time_t *)0) - start;
			if (start > 10)
				nLogMsg(EL_ERROR2, id, 
					"(lost %d seconds on read I/O error)",
					start);
			HangTime += start;

			/*
			 *  assume EIO means no 1st record
			 */

			if (!ilabel[0])		/* virgin */
				return(0);

			if (ilabel[0] == ' ') {	/* unlabeled w/ data */
				VMError = E_USER;
				nLogMsg(EL_CODE|EL_STATUS1, 0,
					"%.*s: I/O error on read %s %s",
					VSNLEN, vsnp,
					"possibly no 1st record when",
					"data expected");
				return(1);
			}
			/* labeled */
			VMError = E_USER;
			nLogMsg(EL_CODE|EL_STATUS1, 0,
					"%.*s: I/O error on read %s %.*s %s",
					VSNLEN, vsnp,
					"possibly no 1st record when label",
					VSNLEN, ilabel, "expected");
			return(1);
		}
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_SYS, 0, 
				"%.*s: can't read label from tape",
				VSNLEN, vsnp);
		return(1);
	}
	nLogMsg(EL_DEBUG3, 0, "label from tape:\n%.80s",label);

	if (!ilabel[0]) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0,
				"%.*s: read %d but supposed to be virgin",
				vsnstr(vsnp));
		return(1);
	}

	/*
	 *  check the ANSI details: length at least 80; VOL1.........{4/" "}
	 */

	if (len < VOL1LEN) {
		if (ilabel[0] == ' ')
			return(0);
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0, 
					"%.*s: label %d chars, expected %d",
					VSNLEN, vsnp, len, VOL1LEN);
		return(1);
	}
	if (strncmp(label, "VOL1", 4) != 0) {    /* not a legal label */
		if (ilabel[0] == ' ')
			return(0);
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0, 
				"%.*s: label does not begin with 'VOL1'",
				VSNLEN, vsnp);
		return(1);
	}
	/* if (label[79] != '4'  &&  label[79] != ' ') {
		if (!ilabel[0]  ||  ilabel[0] == ' ')
			return(0);
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0,
			"%.*s: last char in label (%c) not = '4' or ' '",
			VSNLEN, vsnp, label[79]);
		return(1);
	} */
	if (ilabel[0] == ' ') {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0, 
			"%.*s: ANSI label ('VOL1...') not expected",
				VSNLEN, vsnp);
		return(1);
	}
	if (strncmp(&label[4], ilabel, VSNLEN) != 0) {
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0,
				"%.*s: wrong volser in internal label (%.*s)",
				VSNLEN, vsnp, VSNLEN, &label[4]);
		return(1);
	}
	return(0);
}





