/*
**	nd_2a.c - this is for tape drivers that use open()
**		to poll when tape drive is mounted & ready.
**
**	To be included into nd_2.c
*/

extern void
ClearDev(int drive, Node_t *np, char *devnam, int varyoff, int senderr);

static int
open_mounted(int drive, Node_t *np, char *devnam, char *vsnp, int write, 
		int waitops)
{
	char	*id = "open_mounted";
	int	fd, i, retries;

	for (i=0; i<20; i++) {
		/*
		**	open device - should mean it's ready..
		*/
		for (retries=1; ; retries++) {
			fd = ntp_open(devnam, write, 0, 0, EL_CONSOLE0);
			if (fd != -1) 
				break;
			if (errno == ENOENT) {
				RPCLVaryOff(drive, "Node does not exist");
				return(-1);
			}
			nLogMsg(EL_CONSOLE0|EL_SYS, 0,
					"%.*s_%d: ntp_open(%s) failed",
					VSNLEN, vsnp, seq, devnam);
			if (!waitops  &&  retries >= OPENRETRIES)
				return(-1);
			sleep(SLEEPTIME);
		}
#ifdef NO_GETSTATUS
		nLogMsg(EL_FATAL|EL_ERROR1, id, 
				"HOST DOES NOT SUPPORT GETSTATUS");
#endif
		if (ntp_getstatus(fd, vsnp, 0) == -1  &&  errno != ENXIO) {
			VMError = E_SYS;
			nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, "Mount Error",
					"%s_%d: tapsense on drive %s",
					vsnstr(vsnp), seq, devnam);
			SaveErrs();

			close(fd);
			ClearDev(drive, np, devnam, 1, 0);

			RestoreErrs();
			NDErr2RP(vsnp, seq, req, rc, 1);
			exit(EXIT_FAILDEV);
		} 

		if (ntp_ismounted(vsnp))
			return(fd);

		close(fd);
		nLogMsg(EL_ERROR1, id, "open ok but not mounted?? - retrying");
		sleep(10);
	}

	/*
	**	doesn't make sense - tell the eager public
	*/
	return(-1);
}

void
ClearDev(int drive, Node_t *np, char *devnam, int varyoff, int senderr)
{
	char	*id = "ClearDev";
	char	*vsnp = (char *) &np->d_req.vs_vsn;
	int	fd, serrno, i;

	if (!(np->d_rcflags & (RCF_REWIND|RCF_REWUNLOAD)))
		return;

	fd = open_mounted(drive, np, devnam, vsnp, 0, 0);
	if (fd == -1) {
		serrno = errno;
		RPCLVaryOff(drive, "clear device - open failed"); 
		if (senderr) {
			VMError = E_SYS;
			errno = serrno;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_SYS, id, 
						"open(%s)", devnam);
			NDErr2RP(vsnp, seq, req, rc, 1);
		}
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	nLogMsg(EL_DEBUG2, id, "Open %s sucessfully, fd=%d", devnam, fd);
 
	for (i=0; i<5; i++) {
		if (np->d_rcflags & RCF_REWIND) {
nLogMsg(EL_STATUS1, id, "calling ntp_taprew");
			if (ntp_taprew(fd, vsnp, 0) == 0)
				break;
		} else {
nLogMsg(EL_STATUS1, id, "calling ntp_rewunload");
			if (ntp_rewunload(fd, vsnp, 0) == 0) {
int sl = 30;
nLogMsg(EL_STATUS1, id, "ntp_rewunload succeeded- sleeping 30 seconds");
do { sl = sleep(sl); } while (sl > 0);

				break;
			}
		}
		serrno = errno;
		nLogMsg(EL_SYS|EL_CODE|EL_CONSOLE0, id, "rewunload err %s", 
									devnam);
		if (i<4) {
			sleep(1 + i*i);
			continue;
		}
		if (varyoff)
			RPCLVaryOff(drive, 
					"clear device - rewind/unload failed"); 		if (senderr) {
			VMError = E_SYS;
			errno = serrno;
			NDErr2RP(vsnp, seq, req, rc, 1);
		}
		exit(EXIT_FAILDEV);
	}
	close(fd);
nLogMsg(EL_STATUS1, id, "done");
	nLogMsg(EL_DEBUG2, id, "for %s completed", devnam);
}

/*
 *  WaitCreateNode() - exits
 */
 static void
WaitCreateNode(int drive, Node_t *np, char *devnam)
{
	char	*id = "WaitCreateNode";
	char	*vsnp = (char *)&np->d_req.vs_vsn;
	int	flag = np->d_req.vs_flag;
	int	retries, fd;
	char	buf[20];

nLogMsg(EL_FATAL, id, 
"manual mounts not tested on netbsd - delete this msg & good luck");
	/*
	 *  track pid in log & change name for ps
	 */
	sprintf(buf, "ND_%d", getpid());
	nLogInit(buf, bomb);
	nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
	nLogMsg(EL_STATUS1, id, "mount %.*s_%d on %s", 
			VSNLEN, vsnp, seq, DriveArray[drive].de_name);
#ifndef FILEDRIVES

	/*
	 *  TODO: vary on the drive on this host (no 'vary' on Convex)
	 */

		/*
	if (flag & VO_CLEARFIRST) {
		**	if something mounted, unmount. (used
		**		by manual RC to allow 'cacheing' of
		**		mounts, since it can just pop off
		**		drive.. w/ our 3490's & D3's.)
		**	cacheing mounts maybe not as good an idea
		**		as 1st appeared, since idle mounts
		**		seem to wander into the weeds owing
		**		to tape slacking, or wear out tape
		**		& drive owing to helical scan always
		**		spinning heads.
#ifdef NO_GETSTATUS
		nLogMsg(EL_FATAL|EL_ERROR1, id, 
				"HOST DOES NOT SUPPORT GETSTATUS");
#endif
		fd = open_mounted(drive, np, devnam, vsnp, 0, 0);
		if (fd != -1) {
			close(fd);
			ClearDev(drive, np, devnam, 1, 1);
		}
	}
		*/

	/*
	 *  set buf for LED display
	 */


	/*
	 *  assume msg on goes on LED since 'wait' mount
	 *	is manual.. TODO - have led attrib 
	 *	in drive characteristics
	 *
	 *	For no good reason, after a ClearDev, the LED msg
	 *	doesn't display - even tho no err is reported. So
	 *	this is in the loop since later tries work..
can't do this w/ STK scsi interface,
and if it could be done, netbsd would need to open a control node
before the poll-for-mount open occurs
	sprintf(buf, "  %.*s  ", VSNLEN, vsnp);
	if (LED34X0(fd, TAPLED_ALTERNATE, " MOUNT  ", buf) == -1) {
			nLogMsg(EL_SYS|EL_CONSOLE0, id, "displaying 'MOUNT'");
	} else {
		mountmsg = 1;
		msgfd = fd;
	}
	 */

	fd = open_mounted(drive, np, devnam, vsnp, 0, 1);
	if (fd == -1) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE, "Mount Error",
				"%s_%d: problem mounting on drive %s",
				vsnstr(vsnp), seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	mountmsg = 0;
	CheckDev(drive, np, fd, devnam);
/*
	if (LED34X0(fd, TAPLED_REMOVE|TAPLED_FIRST, buf, buf) == -1)
		nLogMsg(EL_SYS|EL_ERROR1, id, "tapled 2");
*/
	close(fd);
#endif /* FILEDRIVES */

	MakeNode(drive, np, devnam);
	SendOK(drive);
	nLogMsg(EL_DEBUG1, 0, "exit ok");
	exit(EXIT_OK);			/* informs parent all went well */
}

/*
**	CreateNode() - exits
*/
 static void
CreateNode(int drive, Node_t *np, char *devnam)
{
	char	*id = "CreateNode";
	char	*vsnp = (char *)&np->d_req.vs_vsn;
	int	retries, fd;
	char	buf[20];

#ifdef FILEDRIVES
	MakeNode(drive, np, devnam);
	SendOK(drive);
	exit(EXIT_OK);
#endif
	/*
	**	for netbsd we use the open to poll whether the
	**		tape is ready
	*/
	fd = open_mounted(drive, np, devnam, vsnp, 0, 0);
	if (fd == -1) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE, "Mount Error",
				"%s_%d: problem mounting on drive %s",
				vsnstr(vsnp), seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	/*
	 *  TODO: vary on the drive (no 'vary' on Convex)
	 */

	/*
	 *  make sure volume is mounted
	 */
	CheckDev(drive, np, fd, devnam);
	close(fd);
	MakeNode(drive, np, devnam);
	SendOK(drive);
	nLogMsg(EL_DEBUG1, id, "pid %d exit ok", getpid());
	exit(EXIT_OK);
}

 static void
DeleteNode(int drive, Node_t *np)
{
	char	*id = "DeleteNode";
	char	*vsnp = (char *)&np->d_req.vs_vsn;
	char	*nodep = vVsnNode(vsnp);
	int	fd;

#ifdef FILEDRIVES
	SendOK(drive);
	exit(EXIT_OK);
#endif
	/*
	 *  remove the /dev/vol/xxxxxx node
	 */
	if (unlink(nodep)  &&  errno != ENOENT) {
		nLogMsg(EL_SYS|EL_CODE, id, "unlink %s", nodep);
		RPCLVaryOff(drive, nErrStr); 
	}

	/*
	 *  handle the device itself
	 */
	if (np->d_req.vs_flag & VO_CACHEMOUNT) {
/*
not tested since manual mounts no longer used locally
*/
		fd = ioctlopen(np->d_devname);
		if (fd == -1) {
			nLogMsg(EL_ERROR1|EL_SYS, id, 
					"ioctlopen for cache msg (unmount %s)",
					np->d_devname);
		} else {
			char	buf[10];
			sprintf(buf, "* %.*s *", VSNLEN, vsnp);
			if (LED34X0(fd, TAPLED_REMOVE|TAPLED_FIRST, buf, buf) 
									== -1) {
				nLogMsg(EL_ERROR1|EL_SYS, id, 
					"cache LED on %s vsn %.*s",
					np->d_devname, VSNLEN, vsnp);
				close(fd);
				ClearDev(drive, np, np->d_devname, 1, 1);
			} else
				nLogMsg(EL_STATUS2, id, 
					"(marked LED: %.*s cached on %s)",
					VSNLEN, vsnp, 
					DriveArray[drive].de_name);
			close(fd);
		}
	} else
		ClearDev(drive, np, np->d_devname, 0, 1);
	
	if (req)
		SendOK(drive);
	exit(EXIT_OK);
}

 static void
Scratch(int drive, Node_t *np, char *devnam)
{
	char	*id = "Scratch";
	char	*vsnp = (char *)&np->d_req.vs_vsn;
	int	flag = np->d_req.vs_flag;
	int	retries, fd, i;
	char	vol1buf[VOL1LEN];

#ifdef FILEDRIVES
	SendOK(drive);
	exit(EXIT_OK);
#endif
	fd = open_mounted(drive, np, devnam, vsnp, 1, 0);
	if (fd == -1) {
		VMError = E_SYS;
		nLogMsg(EL_ERROR1|EL_CODE, id, "%s_%d: open_mounted(%s) failed",
					vsnstr(vsnp), seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}

	/*
	 *  TODO: vary on the drive (no 'vary' on Convex)
	 */

	/*
	**	check physical protection
	*/
	if (ntp_write_protected(vsnp)) {
		/*
		**	mounted for write && no write ring
		*/
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR2, id,
				"%s: %s %s", vsnstr(vsnp),
				"Volume not physically write enabled ",
				"- contact operator");
		NDErr2RP(vsnp, seq, req, rc, 0);
		exit(EXIT_FAILVOL);
	}
	/*
	**	check label if bypass not req'd
	*/
	if (!(flag & VO_BYPASS)) {
		nLogMsg(EL_DEBUG3, id, "checking label: %s", 
					vsnstr((char *)&np->d_req.vs_label));
		if (LabelProblem(drive, fd, (char *)&np->d_req.vs_label, vsnp, 
							np, devnam)) {
			SaveErrs();

			close(fd);
			(void)ClearDev(drive, np, devnam, 1, 0);

			RestoreErrs();
			if (VMError == E_SYS) {
				NDErr2RP(vsnp, seq, req, rc, 1);
				exit(EXIT_FAILDEV);
			}
			NDErr2RP(vsnp, seq, req, rc, 0);
			exit(EXIT_FAILVOL);
		}
		nLogMsg(EL_DEBUG3, 0, "label ok");
	}

	/*
	**	rewind and erase
	*/
	if (ntp_taprew(fd, vsnp, EL_CODE) == -1) {
		VMError = E_SYS;
		SaveErrs();

		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
#ifndef NO_ERASE
nLogMsg(EL_STATUS1, id, "start dse");
	/*
	**	erase NOTE: tapdse(fd, vsnp) is used when tape
	**		driver's dse works, tapdse2(fd, mbytes, vsnp)
	**		when driver dse not implemented.
	*/
	if (ntp_tapdse(fd, vsnp, EL_CODE)) {
		VMError = E_SYS;
		SaveErrs();

		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
nLogMsg(EL_STATUS1, id, "end dse");
#endif

	/*
	**	relabel
	*/
	if (ntp_taprew(fd, vsnp, EL_CODE) == -1) {
		VMError = E_SYS;
		SaveErrs();

		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	sprintf(vol1buf, "VOL1%-75.6s4", vsnp);
	if ((i = write(fd, vol1buf, VOL1LEN)) != VOL1LEN) {
		VMError = E_SYS;
		nLogMsg(EL_SYS|EL_CODE, id, "writing label (%d)", fd);
		SaveErrs();

		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	/*
	**	Add 2 tape marks for "end of data"
	*/
	if (ntp_tapwtm(fd, vsnp, EL_CODE) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_SYS|EL_CODE, id, "writing tapemark1");
		SaveErrs();

		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	if (ntp_tapwtm(fd, vsnp, EL_CODE) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_SYS|EL_CODE, id, "writing tapemark1");
		SaveErrs();

		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);

		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}

	/*
	**	unmount
	*/
	close(fd);
	ClearDev(drive, np, devnam, 1, 1);

	/*
	**	report success
	*/
	SendOK(drive);
	exit(EXIT_OK);
}




