/*
**  nd_2b.c - these routines work w/ OS's that allow
**	open-and-poll of tape drive to tell when it's
**	ready. Last used with a modified enrst node
**	definition on netbsd w/ 3490 & D3 drives;
**	something like it was used on Amdahl & Convexen
*/



/*
 *  WaitCreateNode() - exits
 */
 static void
WaitCreateNode(int drive, Node_t *np, char *devnam)
{
	char	*id = "WaitCreateNode";
	char	*vsnp = (char *)&np->d_req.vs_vsn;
	int	flag = np->d_req.vs_flag;
	int	retries, fd;
	char	buf[20];

	/*
	 *  track pid in log & change name for ps
	 */
	sprintf(buf, "ND_%d", getpid());
	nLogInit(buf, bomb);
	nLogInitFile(vLogFile, EL_LEVEL, 0, NULL);
	nLogMsg(EL_STATUS1, id, "mount %.*s_%d on %s", 
			VSNLEN, vsnp, seq, DriveArray[drive].de_name);
#ifndef FILEDRIVES
	/*
	 *  open for driver-level queries of status & 
	 *	setting of state
	 */
	for (retries=1; retries<=OPENRETRIES; retries++) {
		if (sigged)
			exit(EXIT_OK);
		fd = ntp_open(devnam, 0, 0, 0, EL_CONSOLE0);
		if (fd != -1) 
			break;
		if (errno == ENOENT) {
			RPCLVaryOff(drive, "Node does not exist");
			break;
		}
		nLogMsg(EL_CONSOLE0|EL_ERROR1, 0,"%.*s_%d: ntp_open(%s) failed",
					VSNLEN, vsnp, seq, devnam);
		sleep(retries * retries);
	}
	if (fd == -1) {
		VMError = E_SYS;
		nLogMsg(EL_ERROR1|EL_CODE, 0, "%.*s_%d: ioctlopen %s failed",
					VSNLEN, vsnp, seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}

	/*
	 *  TODO: vary on the drive on this host (no 'vary' on Convex)
	 */

	if (flag & VO_CLEARFIRST) {
		/*
		 *  if something mounted, unmount. (used
		 *	by manual RC to allow 'cacheing' of
		 *	mounts, since it can just pop off
		 *	drive.. w/ our 3490's & D3's.)

		**	cacheing mounts maybe not as good an idea
		**		as 1st appeared, since idle mounts
		**		seem to wander into the weeds owing
		**		to tape slacking, or wear out tape
		**		& drive owing to helical scan always
		**		spinning heads.
		 */
#ifdef NO_GETSTATUS
		nLogMsg(EL_FATAL|EL_ERROR1, id, 
				"HOST DOES NOT SUPPORT GETSTATUS");
#endif
		if (ntp_getstatus(fd, vsnp, 0) == -1) {
			VMError = E_SYS;
			nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE|EL_ERROR1, 0,
                                         "%s_%d: getstatus drive %s failed", 
					vsnstr(vsnp), seq, devnam);
			NDErr2RP(vsnp, seq, req, rc, 1);
			exit(EXIT_FAILDEV);
		}
		if (ntp_ismounted(vsnp)) {
			nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
				"%s_%d: clearing vsn on %s for mount", 
				vsnstr(vsnp), seq, devnam);
			close(fd);
			fd = ClearDev(drive, np, devnam, 1, 1);
		}
	}
	if (sigged)
		exit(EXIT_OK);

	/*
	 *  set buf for LED display
	 */
	sprintf(buf, "  %.*s  ", VSNLEN, vsnp);

	/*
	 *  keep testing drive to see if mount has occurred.
	 *	arbitrary granularity of 30 sec
	 *
	 *	the RC takes care of the 'mount me' msgs to 
	 *	the operator, presumably w/ a 'backoff' timing
	 */
	while (1) {
		if (sigged)
			exit(EXIT_OK);
		/*
		 *  assume msg on goes on LED since 'wait' mount
		 *	is manual.. TODO - have led attrib 
		 *	in drive characteristics
		 *
		 *	For no good reason, after a ClearDev, the LED msg
		 *	doesn't display - even tho no err is reported. So
		 *	this is in the loop since later tries work..
		 */
		if (LED34X0(fd, TAPLED_ALTERNATE, " MOUNT  ", buf) == -1) {
			nLogMsg(EL_SYS|EL_CONSOLE0, id, "displaying 'MOUNT'");
		} else {
			mountmsg = 1;
			msgfd = fd;
		}

		nLogMsg(EL_DEBUG3, 0, "getting tapsense");
#ifdef NO_GETSTATUS
		nLogMsg(EL_FATAL|EL_ERROR1, id, 
				"HOST DOES NOT SUPPORT GETSTATUS");
#endif
		if (ntp_getstatus(fd, vsnp, 0) == -1  &&  errno != ENXIO) {
			VMError = E_SYS;
			nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, "Mount Error",
					"%s_%d: tapsense on drive %s",
					vsnstr(vsnp), seq, devnam);
			SaveErrs();
			close(fd);
			(void)ClearDev(drive, np, devnam, 1, 0);
			RestoreErrs();
			NDErr2RP(vsnp, seq, req, rc, 1);
			exit(EXIT_FAILDEV);
		} 

		if (ntp_ismounted(vsnp))
			break;
		sleep(10);
	}
	mountmsg = 0;
	CheckDev(drive, np, fd, devnam);
	if (LED34X0(fd, TAPLED_REMOVE|TAPLED_FIRST, buf, buf) == -1)
		nLogMsg(EL_SYS|EL_ERROR1, id, "tapled 2");
	close(fd);
#endif /* FILEDRIVES */

	MakeNode(drive, np, devnam);
	SendOK(drive);
	nLogMsg(EL_DEBUG1, 0, "exit ok");
	exit(EXIT_OK);			/* informs parent all went well */
}

/*
 *  CreateNode() - exits
 */
 static void
CreateNode(int drive, Node_t *np, char *devnam)
{
	char	*id = "CreateNode";
	char	*vsnp = (char *)&np->d_req.vs_vsn;
	int	retries, fd;
	char	buf[20];

#ifdef FILEDRIVES
	MakeNode(drive, np, devnam);
	SendOK(drive);
	exit(EXIT_OK);
#endif
	for (retries=1; retries<=OPENRETRIES; retries++) {
		if (sigged)
			exit(EXIT_OK);
		fd = ntp_open(devnam, 0, 0, 0, EL_CODE);
		if (fd != -1) 
			break;
		sleep(retries * retries);
	}
	if (fd == -1) {
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	/*
	 *  TODO: vary on the drive (no 'vary' on Convex)
	 */

	/*
	 *  make sure volume is mounted
	 */
#ifndef NO_GETSTATUS
	if (ntp_getstatus(fd, vsnp, 0) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, 0,
                                         "%s_%d: getstatus drive %s failed", 
					vsnstr(vsnp), seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	if (!ntp_ismounted(vsnp)) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, 0, 
					"%s_%d: drive %s not mounted", 
					vsnstr(vsnp), seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILVOL);
	}
#endif
	CheckDev(drive, np, fd, devnam);
	sprintf(buf, "  %.*s  ", VSNLEN, vsnp);
	if (LED34X0(fd, TAPLED_REMOVE|TAPLED_FIRST, buf, buf) == -1)
#ifdef _CONVEX_SOURCE
		if (errno != ETPNOSUPPORT)
#endif
			nLogMsg(EL_SYS|EL_ERROR1, id, "tapled");
	close(fd);
	MakeNode(drive, np, devnam);
	SendOK(drive);
	nLogMsg(EL_DEBUG1, id, "pid %d exit ok", getpid());
	exit(EXIT_OK);
}

 static void
DeleteNode(int drive, Node_t *np)
{
	char	*id = "DeleteNode";
	char	*vsnp = (char *)&np->d_req.vs_vsn;
	char	*nodep = vVsnNode(vsnp);
	int	fd;

#ifdef FILEDRIVES
	SendOK(drive);
	exit(EXIT_OK);
#endif
	/*
	 *  remove the /dev/vol/xxxxxx node
	 */
	if (unlink(nodep)  &&  errno != ENOENT) {
		nLogMsg(EL_SYS|EL_CODE, id, "unlink %s", nodep);
		RPCLVaryOff(drive, nErrStr); 
	}

	/*
	 *  handle the device itself
	 */
	if (np->d_req.vs_flag & VO_CLEARFIRST)
		close(ClearDev(drive, np, np->d_devname, 0, 1));
	else if (np->d_req.vs_flag & VO_CACHEMOUNT) {
		fd = ioctlopen(np->d_devname);
		if (fd == -1) {
			nLogMsg(EL_ERROR1|EL_SYS, id, 
					"ioctlopen for cache msg (unmount %s)",
					np->d_devname);
		} else {
			char	buf[10];
			sprintf(buf, "* %.*s *", VSNLEN, vsnp);
			if (LED34X0(fd, TAPLED_REMOVE|TAPLED_FIRST, buf, buf) 
									== -1) {
				nLogMsg(EL_ERROR1|EL_SYS, id, 
					"cache LED on %s vsn %.*s",
					np->d_devname, VSNLEN, vsnp);
				close(fd);
				fd = ClearDev(drive, np, np->d_devname, 1, 1);
			} else
				nLogMsg(EL_STATUS2, id, 
					"(marked LED: %.*s cached on %s)",
					VSNLEN, vsnp, 
					DriveArray[drive].de_name);
			close(fd);
		}
	}
	
	if (req)
		SendOK(drive);
	exit(EXIT_OK);
}

 static void
Scratch(int drive, Node_t *np, char *devnam)
{
	char	*id = "Scratch";
	char	*vsnp = (char *)&np->d_req.vs_vsn;
	int	flag = np->d_req.vs_flag;
	int	retries, fd, i;
	char	vol1buf[VOL1LEN];

#ifdef FILEDRIVES
	SendOK(drive);
	exit(EXIT_OK);
#endif
	for (retries=1; retries<=OPENRETRIES; retries++) {
		if (sigged)
			exit(EXIT_OK);
		fd = ntp_open(devnam, 1, 0, 1, EL_CONSOLE0);
		if (fd != -1) 
			break;
		sleep(retries * retries);
	}
	if (fd == -1) {
		VMError = E_SYS;
		nLogMsg(EL_ERROR1|EL_CODE, id, "%s_%d: ntp_open(%s) failed",
					vsnstr(vsnp), seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}

	/*
	 *  TODO: vary on the drive (no 'vary' on Convex)
	 */

	/*
	 *  make sure volume is mounted
	 */
#ifndef NO_GETSTATUS
	if (ntp_getstatus(fd, vsnp, 0) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, id,
                                         "%s_%d: getstatus drive %s failed", 
					vsnstr(vsnp), seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	if (!ntp_ismounted(vsnp)) {
		VMError = E_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
					"%s_%d: drive %s not mounted", 
					vsnstr(vsnp), seq, devnam);
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILVOL);
	}
	/*
	 *  check physical protection
	 */
	if (ntp_write_protected(vsnp)) {
		/*
		 *  mounted for write && no write ring
		 */
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		VMError = E_USER;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR2, id,
				"%s: %s %s", vsnstr(vsnp),
				"Volume not physically write enabled ",
				"- contact operator");
		NDErr2RP(vsnp, seq, req, rc, 0);
		exit(EXIT_FAILVOL);
	}
#endif
	/*
	 *  check label if bypass not req'd
	 */
	if (!(flag & VO_BYPASS)) {
		nLogMsg(EL_DEBUG3, id, "checking label: %s", 
					vsnstr((char *)&np->d_req.vs_label));
		if (LabelProblem(drive, fd, (char *)&np->d_req.vs_label, vsnp, 
							np, devnam)) {
			SaveErrs();
			close(fd);
			(void)ClearDev(drive, np, devnam, 1, 0);
			RestoreErrs();
			if (VMError == E_SYS) {
				NDErr2RP(vsnp, seq, req, rc, 1);
				exit(EXIT_FAILDEV);
			}
			NDErr2RP(vsnp, seq, req, rc, 0);
			exit(EXIT_FAILVOL);
		}
		nLogMsg(EL_DEBUG3, 0, "label ok");
	}

	/*
	 *  rewind and erase
	 */
	if (ntp_taprew(fd, vsnp, EL_CODE) == -1) {
		VMError = E_SYS;
		SaveErrs();
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
#ifndef NO_ERASE
nLogMsg(EL_STATUS1, id, "start dse");
	/*
	 *  erase NOTE: tapdse(fd, vsnp) is used when tape
	 *	driver's dse works, tapdse2(fd, mbytes, vsnp)
	 *	when driver dse not implemented.
	 */
	if (ntp_tapdse(fd, vsnp, EL_CODE)) {
		VMError = E_SYS;
		SaveErrs();
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
nLogMsg(EL_STATUS1, id, "end dse");
#endif

	/*
	 *  relabel
	 */
	if (ntp_taprew(fd, vsnp, EL_CODE) == -1) {
		VMError = E_SYS;
		SaveErrs();
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	sprintf(vol1buf, "VOL1%-75.6s4", vsnp);
	if ((i = write(fd, vol1buf, VOL1LEN)) != VOL1LEN) {
		VMError = E_SYS;
		nLogMsg(EL_SYS|EL_CODE, id, "writing label");
		SaveErrs();
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	/*
	 *  Add 2 tape marks for "end of data"
	 */
	if (ntp_tapwtm(fd, vsnp, EL_CODE) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_SYS|EL_CODE, id, "writing tapemark1");
		SaveErrs();
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}
	if (ntp_tapwtm(fd, vsnp, EL_CODE) == -1) {
		VMError = E_SYS;
		nLogMsg(EL_SYS|EL_CODE, id, "writing tapemark1");
		SaveErrs();
		close(fd);
		(void)ClearDev(drive, np, devnam, 1, 0);
		RestoreErrs();
		NDErr2RP(vsnp, seq, req, rc, 1);
		exit(EXIT_FAILDEV);
	}

	/*
	 *  unmount
	 */
	close(fd);
	(void)ClearDev(drive, np, devnam, 1, 1);

	/*
	 *  report success
	 */
	SendOK(drive);
	exit(EXIT_OK);
}

int
ClearDev(int drive, Node_t *np, char *devnam, int varyoff, int senderr)
{
	char	*id = "ClearDev";
	char	*vsnp = (char *) &np->d_req.vs_vsn;
	int	fd, serrno, i;

	/*
	**	the open can fail if some part of the client
	**		still has drive open when the main 
	**		part of client tht mounted the tape
	**		has gone away, so be patient with the
	**		open()
	*/
	for (i=0; i<12; i++) {
		fd = open(devnam, 0);
 
		if (fd != -1)
			break;
		if (errno != EBUSY)
			break;
		nLogMsg(EL_STATUS1, id, "busy - sleep/retry open(%s)", devnam);
		sleep(5 + i*i);
	}
	if (fd == -1) {
		serrno = errno;
		RPCLVaryOff(drive, "clear device - open failed"); 
		if (senderr) {
			VMError = E_SYS;
			errno = serrno;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_SYS, id, 
						"open(%s)", devnam);
			NDErr2RP(vsnp, seq, req, rc, 1);
		}
		exit(EXIT_FAILDEV);
	}
	nLogMsg(EL_DEBUG2, id, "Open %s sucessfully, fd=%d", devnam, fd);
 
	for (i=0; i<5; i++) {
nLogMsg(EL_STATUS1, id, "calling ntp_rewunload");
		if (ntp_rewunload(fd, vsnp, 0) == 0)
			break;
		serrno = errno;
		nLogMsg(EL_SYS|EL_CODE|EL_CONSOLE0, id, "rewunload err %s", 
									devnam);
		if (i<4) {
			sleep(1 + i*i);
			continue;
		}
		if (varyoff)
			RPCLVaryOff(drive, 
					"clear device - rewind/unload failed"); 		if (senderr) {
			VMError = E_SYS;
			errno = serrno;
			NDErr2RP(vsnp, seq, req, rc, 1);
		}
		exit(EXIT_FAILDEV);
	}
nLogMsg(EL_STATUS1, id, "done");
	nLogMsg(EL_DEBUG2, id, "for %s completed", devnam);
	return(fd);
}
