/*
 *  vvcl - NAStore Virtual Volume Manager console vv tester
 *
 *	Uses cl_lib.c interface except uses 'fatal-on-comm-error'
 *	connection. Built from vvconl.
 *
 *	Bill Ross, NASA Ames, October 1997
 */

#include "vv_cl.h"

char	*arg0 = NULL;

  static void
usage()
{
	fprintf(stderr, "usage: %s -i id -c class -s size\n", arg0);
	fprintf(stderr, "usage: %s vvol\n", arg0);
	fprintf(stderr, "usage: %s empty [percent]  NOT IMPLEMENTED\n", arg0);
	ListClasses();
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	int		cc, i, write = 0;
	OFF_T		size = -1;
	char		*id = NULL, *class = NULL;
	int		serial;
	extern  int     optind;
	extern  char    *optarg;
	vv_handle_t	*vhp;
	VVInfo_t	*vip;
	VV_t		*vvp;
	char		*bp;
	OFF_T		vdata, maxdata;
	int		free = 0;

	arg0 = argv[0];


	nLogInitStderr(EL_ERROR1|EL_ERROR2);
	nLogInitStderr(EL_LEVEL);

	vv_init(vvHost, 1);	/* 1 = exit if err */

	if ((argc == 2  ||  argc == 3)  &&  !strcmp(argv[1], "free")) {
		free = 50;
		if (argc == 3)
			free = atoi(argv[2]);
		if (free < 0  ||  free > 100) {
			fprintf(stderr, "free [percent]\n");
			usage();
		}
	} else {
		while ((cc = getopt(argc, argv, "s:i:c:")) != EOF) {
			switch (cc) {
			    case 's':
				if (nSymSize(optarg, &size)) {
					fprintf(stderr, 
						"%s: -s: expected size\n", 
						arg0);
					usage();
				}
				if (size < 0) {
					fprintf(stderr, 
						"%s: -s: expected size\n", 
						arg0);
					usage();
				}
				break;
		 	   case 'i':
				if (strlen(optarg) > VV_IDLEN) {
					fprintf(stderr, 
						"%s: -i: id too long\n", arg0);
					usage();
				}
				id = optarg;
				break;
			    case 'c':
				class = optarg;
				if (ClassInfo(class) == NULL) {
					fprintf(stderr, 
					"%s: -c: expected class prefix\n", 
					arg0);
					usage();
				}
				break;
			    default:
				usage();
			}
		}
	}

	/*
	 *  deal with user req
	 */
	if (class == NULL  &&  id == NULL  &&  size == -1  &&  free == 0) {
		if (optind == argc) {
			fprintf(stderr, 
			    "%s: need either '-' options or explicit vvol\n", 
			    arg0);
			usage();
		}
		if (argc > optind+1) {
			fprintf(stderr, 
				"%s: sorry, only 1 vvol at a time\n", arg0);
			usage();
		}
		vhp = vv_openread(argv[optind]);
	} else if (class  &&  id != NULL  &&  size != -1) {
		write = 1;
		vhp = vv_openwrite(id, class, size, 0);
	} else if (free) {
		fprintf(stderr, "no spacefree\n");
		exit(1);
	} else {
		fprintf(stderr, 
			"%s: need 'free', all '-' options, or explicit vvol\n", 
			arg0);
		usage();
	}
	if (vhp == NULL) {
		fprintf(stderr, "VVM error: %s\n", vv_errstr);
		VVCLGoodbye(1);
		exit(1);
	}
	vip = &vhp->v_info;

	HexOFF(&vdata, vip->v_data, sizeof(vip->v_data));
	HexOFF(&maxdata, vip->v_maxdata, sizeof(vip->v_maxdata));
	printf("%s:\n", VV2Name(&vip->v_vv));
	printf("\tid:\t\t%s\n", IdString(vip->v_id));
	printf("\tstatus:\t\t%s\n", vvStatusStr(vip->v_status));
	printf("\tuse:\t\t%s\n", vvUseStr(vip->v_use));
	printf("\tdata:\t\t%lld\n\tmaxdata:\t%lld\n",
				(long long)vdata, (long long)maxdata);
	if (vip->v_vsns[0].v_vsn[0] != ' ')
		printf("\tvsn1:\t\t%.*s\tnrec:\t%d\n",
					VSNLEN, vip->v_vsns[0].v_vsn,
					vip->v_vsns[0].v_nrec);
	else
		printf("\tvsn:\t\tnone\n");
	printf("\tnrestore:\t%d\n", vip->v_nrestore);
	printf("\tmntfail:\t%d\n", vip->v_mntfail);
	printf("\tcreat:\t\t%s", bsd_ctime((time_t *)&vip->v_creat));
	printf("\twrite:\t\t%s", bsd_ctime((time_t *)&vip->v_write));
	if (vip->v_finish != -1)
		printf("\tfinish:\t\t%s", bsd_ctime((time_t *)&vip->v_finish));
	if (vip->v_read != -1)
		printf("\tread:\t\t%s", bsd_ctime((time_t *)&vip->v_read));

	if (write) {
		bp = nAlloc(size);
		for (i=0; i<size; i++)
			bp[i] = 'A' + i%20;
		bp[size-1] = '\n';

/*		for (i=0; i<10; i++) { */
			if (vv_write(vhp, bp, size) != size)
				perror("vv_write");
	}
	vv_close(vhp, 0);
	VVCLGoodbye(1);
}
