/*
 *  vvconl - NAStore Virtual Volume Manager console vv lister
 *
 *	Bill Ross, NASA Ames, October 1997
 */

#include "vv_space.h"

char	*arg0 = NULL;

  static void
usage()
{
	fprintf(stderr, "usage: %s VV\n", arg0);
	fprintf(stderr, "usage: %s [-h] [-i id] [-c class] [-v vsn]\n", arg0);
	fprintf(stderr, "usage: %s [-f] [-i id] [-c class] [-v vsn]\n", arg0);
	fprintf(stderr, "\t(must give VV or at least one of: -h -f -i -v)\n");
	ListClasses();
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	char		buf[VV_PKTSIZ(vvpVolqry_t)];
	vvpHdr_t	*hp;
	vvpVolqry_t	*vqp;
	VV_t		*vvp;
	int		i, ct, cc, len;
	extern  int     optind;
	extern  char    *optarg;
	OFF_T		vdata, vmaxdata;

	arg0 = argv[0];

	hp = vvBuildHdr(VVM_VOLQRY, VVNOSEQ, (vvpHdr_t *)buf);
	vqp = VV_MSGPTR(hp, vvpVolqry_t);

	vqp->vq_flags = 0;

	while ((cc = getopt(argc, argv, "hfi:c:v:")) != EOF) {
		switch (cc) {
		    case 'h':
			vqp->vq_flags |= VQF_HOT;
			break;
		    case 'f':
			vqp->vq_flags |= VQF_FIN;
			break;
		    case 'i':
			vqp->vq_flags |= VQF_ID;
			len = strlen(optarg);
			if (len > VV_IDLEN) {
				fprintf(stderr, "%s: -i: id too long\n", arg0);
				usage();
			}
			if (len == 0) {
				fprintf(stderr, "%s: -i: id 0 length\n", arg0);
				usage();
			}
			memset(vqp->vq_id, ' ', VV_IDLEN);
			memcpy(vqp->vq_id, optarg, len);
			break;
		    case 'c':
			vqp->vq_flags |= VQF_CLASS;
			len = strlen(optarg);
			if (len > VV_CLASSLEN) {
				fprintf(stderr, 
					"%s: -c: class too long\n", 
					arg0);
				usage();
			}
			memset(vqp->vq_vv.class, ' ', VV_CLASSLEN);
			memcpy(vqp->vq_vv.class, optarg, len);
			if (ClassInfo(optarg) == NULL) {
				fprintf(stderr, 
					"%s: -c %s: expected class name\n", 
					arg0, optarg);
				usage();
			}
			break;
		    case 'v':
			if (strlen(optarg) != VSNLEN) {
				fprintf(stderr, "%s: -v: expected vsn\n", arg0);
				usage();
			}
			vqp->vq_flags |= VQF_VSN;
			memcpy(vqp->vq_vsn, optarg, VSNLEN);
			break;
		    default:
			usage();
		}
	}
	if (vqp->vq_flags  &&  optind != argc) {
		fprintf(stderr, 
			"%s: can't use '-' options with explicit vvol\n", arg0);
		usage();
	}
	if (vqp->vq_flags & (VQF_HOT|VQF_FIN) == (VQF_HOT|VQF_FIN)) {
		fprintf(stderr,
			"%s: can't use -h and -f together\n", arg0);
		usage();
	}
	if (!vqp->vq_flags) {
		if (optind == argc) {
			fprintf(stderr, 
			    "%s: need either '-' options or explicit vvol\n", 
			    arg0);
			usage();
		}
		if (argc > optind+1) {
			fprintf(stderr, 
				"%s: sorry, only 1 vvol at a time\n", arg0);
			usage();
		}
		vvp = Name2VV(argv[optind], 1);
		if (vvp == NULL) {
			fprintf(stderr, "%s: bad VV (%s)\n", 
							arg0, argv[optind]);
			usage();
		}
		vqp->vq_vv = *vvp;
		vqp->vq_flags = VQF_VOL;
	} 
	if (!(vqp->vq_flags & (VQF_VOL|VQF_HOT|VQF_FIN|VQF_VSN|VQF_ID)))
		usage();

	/*
	**	msg is built - send it
	*/
	nLogInit(arg0, bomb);
	nLogInitStderr(EL_ERROR1);
	VVCLInit(vvHost, VVM_CLIENT, RETRY_FATAL);

	(void)VVCLSend(hp, 1);		/* dies if problem */

	/*
	**	get response(s)
	*/
	while (1) {
		vvpHdr_t	*rhp;
		VVInfo_t	*vip;
		vvpError_t	*vep;

		/*
		 *  since only VVCLRecv is used to drive the
		 *	main loop - i.e. no select())  - no need to use
		 *	the result of VVCLNextRecord() to see if any pkt
		 *	is buffered.
		 */
		rhp = VVCLRecv("response");	/* dies if problem */
		(void)VVCLNextRecord();

		switch (rhp->vh_type) {
		    case VVM_VOLINFO:
			/*
			 *  success
			 */
			vip = VV_MSGPTR(rhp, VVInfo_t);
			printf("%s:\n", VV2Name(&vip->v_vv));
			printf("\tid:\t\t%s\n", IdString(vip->v_id));
			printf("\tstatus:\t\t%s\n", vvStatusStr(vip->v_status));
			C2BIN(HexOFF, &vdata, vip->v_data);
			C2BIN(HexOFF, &vmaxdata, vip->v_maxdata);
			printf("\tuse:\t\t%s\n", vvUseStr(vip->v_use));
			printf("\tdata:\t\t%lld\n\tmaxdata:\t%lld\n",
					(long long)vdata, (long long)vmaxdata);

			/*
			 *  vsns
			 */
			ct = 0;
			for (i=0; i<vip->v_nvsns; i++) {
				if (vip->v_vsns[i].v_vsn[0] != ' ') {
					printf("\tvsn%d:\t\t%.*s\tnrec:\t%d\n",
						i, VSNLEN, vip->v_vsns[i].v_vsn,
						vip->v_vsns[i].v_nrec);
					ct++;
				}
			}
			if (!ct)
				printf("\tvsns:\t\tnone\n");

			printf("\tcreat:\t\t%s", 
					bsd_ctime((time_t *)&vip->v_creat));
			printf("\twrite:\t\t%s", 
					bsd_ctime((time_t *)&vip->v_write));
			if ((time_t)vip->v_finish != -1)
				printf("\tfinish:\t\t%s", 
					bsd_ctime((time_t *)&vip->v_finish));
			if ((time_t)vip->v_read != -1)
				printf("\tread:\t\t%s", 
					bsd_ctime((time_t *)&vip->v_read));
			if ((time_t)vip->v_freed != -1)
				printf("\tfreed:\t\t%s", 
					bsd_ctime((time_t *)&vip->v_freed));
			printf("\tnrestore:\t%d\n", vip->v_nrestore);
			if ((time_t)vip->v_restore != -1)
				printf("\trestore:\t%s", 
					bsd_ctime((time_t *)&vip->v_restore));
			printf("\tmntfail:\t%d\n", vip->v_mntfail);

			nFree(rhp);
			break;
		    case VVM_ACK:
			VVCLGoodbye(0);
			exit(0);
		    case VVM_VVERROR:
			vep = VV_MSGPTR(rhp, vvpError_t);
			VVError = vep->vn_error;
			if (vqp->vq_flags == VQF_VOL)
				nLogMsg(EL_ERROR1, 0, 
					"%s: error from vvmd: %s", 
					argv[optind], vep->vn_msg);
			else
				nLogMsg(EL_ERROR1, 0, 
					"error from vvmd: %s", vep->vn_msg);
				
			VVCLGoodbye(0);
			exit(1);
		    case VVM_SHUTDOWN:
			nLogMsg(EL_ERROR1, 0, "vvmd shutdown received");
			exit(1);
		    default:
			fprintf(stderr, "unexpected pkt: %s\n", vvHdrStr(rhp));
			VVCLGoodbye(1);
			exit(1);
		}
	}
}
