/*
 *  vvdiskck - NAStore Virtual Volume Manager: check dbase against disk cache
 *
 *	Bill Ross, NASA Ames, August 1998
 */

#include "vv_space.h"

char	*arg0 = NULL;

  static void
usage()
{
	fprintf(stderr, "usage: %s\n", arg0);
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	vvpHdr_t	hpkt;
	int		err = 0;

	arg0 = argv[0];
#ifndef PURIFY
	if (getuid() != 0) {
		fprintf(stderr, "%s: must be root\n", arg0);
		exit(1);
	}
#endif
	if (argc != 1)
		usage();

	(void)vvBuildHdr(VVM_DISKCK, VVNOSEQ, &hpkt);

	nLogInit(arg0, bomb);
	nLogInitStderr(EL_ERROR1);
	VVCLInit(vvHost, VVM_CLIENT, RETRY_FATAL);

	(void)VVCLSend(&hpkt, 1);		/* dies if problem */

	/*
	 *  get response(s)
	 */
	while (1) {
		vvpHdr_t	*rhp;
		vvpError_t	*vep;

		/*
		 *  since only VVCLRecv is used to drive the
		 *	main loop - i.e. no select())  - no need to use
		 *	the result of VVCLNextRecord() to see if any pkt
		 *	is buffered.
		 */
		rhp = VVCLRecv("response");	/* dies if problem */
		(void)VVCLNextRecord();

		switch (rhp->vh_type) {
		    case VVM_ACK:
			VVCLGoodbye(1);
			exit(err);
		    case VVM_VVERROR:
			vep = VV_MSGPTR(rhp, vvpError_t);
			VVError = vep->vn_error;
			nLogMsg(EL_ERROR1, 0, 
					"error from vvmd: %s", vep->vn_msg);
			err++;
			continue;
		    case VVM_SHUTDOWN:
			nLogMsg(EL_ERROR1, 0, "vvmd shutdown received");
			exit(1);
		    default:
			fprintf(stderr, "unexpected pkt: %s\n", vvHdrStr(rhp));
			VVCLGoodbye(1);
			exit(1);
		}
	}
}
