/*
 *  vvfinish - NAStore Virtual Volume Manager: push VV's to tape.
 *
 *	Bill Ross, NASA Ames, March 1998
 */

#include "vv_space.h"

char	*arg0 = NULL;

  static void
usage(char *str)
{
	if (str)
		fprintf(stderr, "%s: %s\n", arg0, str);
	fprintf(stderr, "usage: %s -v VV\n", arg0);
	fprintf(stderr, "       %s -C class_prefix\n", arg0);
	fprintf(stderr, "       %s -t vsn\n", arg0);
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	kill(0, 5);
	exit(1);
	return(1);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	char		buf[VV_PKTSIZ(vvpFinish_t)];
	extern int	optind;
	extern char	*optarg;
	int		cc, cmd;
	vvpHdr_t	*hp = NULL;
	vvpVol_t	*vp;
        VV_t		*vvp;
	vvpFinish_t	*fip;
	Class_t		*cip;
	vvpVsn_t	*vsp;

	arg0 = argv[0];
#ifndef PURIFY
	if (getuid() != 0) {
		fprintf(stderr, "%s: must be root\n", arg0);
		exit(1);
	}
#endif
	while ((cc = getopt(argc, argv, "v:C:t:")) != EOF) {
		if (hp)
			usage("only one option allowed");
		switch (cc) {
		    case 'v':
			vvp = Name2VV(optarg, 1);
			if (vvp == NULL)
				usage("bad VV");
			cmd = VVM_FINISH;
			hp = vvBuildHdr(cmd, VVNOSEQ, NULL);
			vp = VV_MSGPTR(hp, vvpVol_t);
			vp->vo_vv = *vvp;
			break;
		    case 'C':
			if (!isatty(fileno(stdin)) || !isatty(fileno(stdout))) {
				fprintf(stderr, "-C: must run interactively\n");
				exit(1);
			}
			if (strlen(optarg) > VV_CLASSLEN)
				usage("class_prefix too long");
			cip = ClassInfo(optarg);
			if (cip == NULL)
				usage("unknown class");
			printf(
			  "This command will close all VV's in this class.\n");
			printf("Are you sure? (y/n) ");
			if (getYorN() != 'y')
				exit(0);
			cmd = VVM_FINCLASS;
			hp = vvBuildHdr(cmd, VVNOSEQ, NULL);
			fip = VV_MSGPTR(hp, vvpFinish_t);
			memset(fip->f_class, 0, sizeof(fip->f_class));
			strcpy(fip->f_class, optarg);
			break;
		    case 't':
			if (strlen(optarg) != VSNLEN)
				usage("bad vsn length");
			cmd = VVM_FINISHVSN;
			hp = vvBuildHdr(cmd, VVNOSEQ, NULL);
			vsp = VV_MSGPTR(hp, vvpVsn_t);
			memcpy(vsp->vq_vsn, optarg, VSNLEN);
			break;
		    default:
			usage("unknown option");
			break;
		}
	}
	if (hp == NULL)
		usage("specify option");

	nLogInit(arg0, bomb);
	nLogInitStderr(EL_ERROR1|EL_ERROR2);
	VVCLInit(vvHost, VVM_CLIENT, RETRY_FATAL);

	(void)VVCLSend(hp, 1);		/* dies if problem */

	/*
	 *  get response(s)
	 */
	while (1) {
		vvpHdr_t	*rhp;
		vvpError_t	*vep;
		char		*cp;

		/*
		 *  since only VVCLRecv is used to drive the
		 *	main loop - i.e. no select())  - no need to use
		 *	the result of VVCLNextRecord() to see if any pkt
		 *	is buffered.
		 */
		rhp = VVCLRecv("response");	/* dies if problem */
		(void)VVCLNextRecord();

		switch (rhp->vh_type) {
		    case VVM_BYTES:
			cp = VV_MSGPTR(rhp, char);
			printf("%.*s\n", rhp->vh_size, cp);
			nFree(rhp);
			continue;
		    case VVM_VOLINFO:
			if (cmd != VVM_FINISH) {
				fprintf(stderr, "unexpected VVM_VOLINFO\n");
				exit(1);
			}
			exit(0);
		    case VVM_ACK:
			VVCLGoodbye(1);
			exit(0);
		    case VVM_VVERROR:
			vep = VV_MSGPTR(rhp, vvpError_t);
			VVError = vep->vn_error;
			nLogMsg(EL_ERROR1, 0, 
					"error from vvmd: %s", vep->vn_msg);
			VVCLGoodbye(1);
			exit(1);
		    case VVM_SHUTDOWN:
			nLogMsg(EL_ERROR1, 0, "vvmd shutdown received");
			exit(1);
		    default:
			fprintf(stderr, "unexpected pkt: %s\n", vvHdrStr(rhp));
			VVCLGoodbye(1);
			exit(1);
		}
	}
}
