/*
 *  vvfinish - NAStore Virtual Volume Manager: push VV's to tape.
 *
 *	Bill Ross, NASA Ames, March 1998
 */

#include "vv_cl.h"

char	*arg0 = NULL;

  static void
usage(char *str)
{
	if (str)
		fprintf(stderr, "%s: %s\n", arg0, str);
	fprintf(stderr, "usage: %s  VV1 VV2 ..\n", arg0);
	fprintf(stderr, "usage: %s  < vvlist\n", arg0);
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

 static char *
getvv()
{
	static char	vvname[MAXVVNAMELEN];
	int		i, cc;


next:
	/*
	**	skip whitespace
	*/
	while (isspace(cc = getc(stdin)));
	if (cc == EOF)
		return(NULL);

	/*
	**	read vv
	*/
	for (i=0;;i++) {
		if (isspace(cc)) {
			vvname[i] = '\0';
			break;
		}
		if (!isalnum(cc)  &&  cc != '.') {
			fprintf(stderr, "Illegal VV character (%d)\n", cc);
			vvname[i] = '_';
			vvname[i+1] = '\0';
			fprintf(stderr, "skipping %s\n", vvname);
			goto next;
		}
		if (i > MAXVVNAMELEN-1) {
			fprintf(stderr, "VV too long\n");
			vvname[i] = cc;
			vvname[i+1] = '\0';
			fprintf(stderr, "skipping %s\n", vvname);
			while (!isspace(cc = getc(stdin)));
			goto next;
		}
		vvname[i] = cc;
		cc = getc(stdin);
	}
	return(vvname);
}

main(int argc, char *argv[])
{
	int	i, fromstdin = 0, err = 0;

	arg0 = argv[0];
#ifndef PURIFY
	if (getuid() != 0) {
		fprintf(stderr, "%s: must be root\n", arg0);
		exit(1);
	}
#endif
	if (argc == 1) 
		fromstdin = 1;

	if (vv_init(vvHost, RETRY_NOT)) {
		fprintf(stderr, "connect: %s\n", vv_errstr);
		exit(1);
	}
	if (fromstdin) {
		char	*vvname;

		while ((vvname = getvv()) != NULL) {
			if (vv_recycle(vvname)) {
				fprintf(stderr, "%s\n", vv_errstr);
				err++;
			}
		}
	} else {
		for (i=1; i<argc; i++) {
			if (vv_recycle(argv[i])) {
				fprintf(stderr, "%s\n", vv_errstr);
				err++;
			}
		}
	}
	vv_term();
	exit(err);
}
