/*
 *  vvrecyctp - NAStore Virtual Volume Manager: tell vvmd to recycle a tape:
 *	restore any nonresident VV's on tape to disk, 
 *	schedule all VV's on tape to write a copy to another tape,
 *	recycle tape to volman and delete from dbase.
 *
 *	Bill Ross, NASA Ames, February 1999
 */

#include "vv_space.h"

char	*arg0 = NULL;

  static void
usage()
{
	fprintf(stderr, "usage: %s vsn\n", arg0);
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	char		buf[VV_PKTSIZ(vvpVsn_t)];
	vvpHdr_t	*hp;
	vvpVsn_t	*vp;

	arg0 = argv[0];
#ifndef PURIFY
	if (getuid() != 0) {
		fprintf(stderr, "%s: must be root\n", arg0);
		exit(1);
	}
#endif
	if (argc != 2)
		usage();

	hp = vvBuildHdr(VVM_RECYCVSN, VVNOSEQ, (vvpHdr_t *)buf);
	vp = VV_MSGPTR(hp, vvpVsn_t);

	memcpy(vp->vq_vsn, argv[1], VSNLEN);

	nLogInit(arg0, bomb);
	nLogInitStderr(EL_ERROR1);
	VVCLInit(vvHost, VVM_CLIENT, RETRY_FATAL);

	(void)VVCLSend(hp, 1);		/* dies if problem */

	/*
	 *  get response(s)
	 */
	while (1) {
		vvpHdr_t	*rhp;
		vvpError_t	*vep;

		/*
		 *  since only VVCLRecv is used to drive the
		 *	main loop - i.e. no select())  - no need to use
		 *	the result of VVCLNextRecord() to see if any pkt
		 *	is buffered.
		 */
		rhp = VVCLRecv("response");	/* dies if problem */
		(void)VVCLNextRecord();

		switch (rhp->vh_type) {
		    case VVM_ACK:
			VVCLGoodbye(1);
			exit(0);
		    case VVM_VVERROR:
			vep = VV_MSGPTR(rhp, vvpError_t);
			VVError = vep->vn_error;
			nLogMsg(EL_ERROR1, 0, 
					"error from vvmd: %s", vep->vn_msg);
			VVCLGoodbye(1);
			exit(1);
		    case VVM_SHUTDOWN:
			nLogMsg(EL_ERROR1, 0, "vvmd shutdown received");
			exit(1);
		    default:
			fprintf(stderr, "unexpected pkt: %s\n", vvHdrStr(rhp));
			VVCLGoodbye(1);
			exit(1);
		}
	}
}
