/*
 *  vvstat - NAStore Virtual Volume Manager status displayer
 *
 *	Bill Ross, NASA Ames, October 1997
 */

#include "vv_space.h"

char	*arg0 = NULL;

  static void
usage()
{
	fprintf(stderr, "usage: %s [-a]\n", arg0);
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	vvpHdr_t	hdr;
	int		cc, stderrflags = EL_ERROR1|EL_ERROR2;
	extern int	optind;
	extern char	*optarg;

	OFF_T		vdata, vmaxdata;

	arg0 = argv[0];

	(void)vvBuildHdr(VVM_STATUS, VVNOSEQ, &hdr);
	hdr.vh_count = 0;

	while ((cc = getopt(argc, argv, "ad")) != EOF) {
		switch (cc) {
		    case 'a':
			hdr.vh_count = 1;
			break;
		    case 'd':
			stderrflags |= EL_STATUS1|EL_STATUS2;
			break;
		    default:
			usage();
		}
	}


	/*
	**	msg is built - send it
	*/
	nLogInit(arg0, bomb);
	nLogInitStderr(stderrflags);
	VVCLInit(vvHost, VVM_CLIENT, RETRY_FATAL);

	(void)VVCLSend(&hdr, 1);		/* dies if problem */

	/*
	**	get response(s)
	*/
	while (1) {
		vvpHdr_t	*rhp;
		char		*cp;
		vvpError_t	*vep;

		/*
		 *  since only VVCLRecv is used to drive the
		 *	main loop - i.e. no select())  - no need to use
		 *	the result of VVCLNextRecord() to see if any pkt
		 *	is buffered.
		 */
		rhp = VVCLRecv("response");	/* dies if problem */
		(void)VVCLNextRecord();

		switch (rhp->vh_type) {
		    case VVM_BYTES:
			/*
			 *  success
			 */
			cp = VV_MSGPTR(rhp, char);
			printf("%s\n", cp);
			nFree(rhp);
			break;
		    case VVM_ACK:
			/* done */
			VVCLGoodbye(0);
			exit(0);
		    case VVM_VVERROR:
			vep = VV_MSGPTR(rhp, vvpError_t);
			VVError = vep->vn_error;
			nLogMsg(EL_ERROR1, 0, 
					"error from vvmd: %s", vep->vn_msg);
			VVCLGoodbye(0);
			exit(1);
		    case VVM_SHUTDOWN:
			nLogMsg(EL_ERROR1, 0, "vvmd shutdown received");
			exit(1);
		    default:
			fprintf(stderr, "unexpected pkt: %s\n", vvHdrStr(rhp));
			VVCLGoodbye(1);
			exit(1);
		}
	}
}
