/*
 *  vvtpdbck - NAStore Virtual Volume Manager VV tape checker
 *	(tells vvmd to compare tape dump file w/ dbase)
 *
 *	Bill Ross, NASA Ames, January 1998
 */

#include "vv_space.h"

char	*arg0 = NULL;

  static void
usage()
{
	fprintf(stderr, "usage: [-u] %s vsn\n", arg0); 
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	char		buf[VV_PKTSIZ(vvpVsn_t)];
	char		*vsnp;
	vvpHdr_t	*hp;
	vvpVsn_t	*vp;

	arg0 = argv[0];
#ifndef PURIFY
	if (getuid() != 0) {
		fprintf(stderr, "%s: must be root\n", arg0);
		exit(1);
	}
#endif
	if (argc < 2  || argc > 3)
		usage();

	hp = vvBuildHdr(VVM_CHECKVSN, VVNOSEQ, (vvpHdr_t *)buf);
	vp = VV_MSGPTR(hp, vvpVsn_t);

	if (argc == 2) {
		vsnp = argv[1];
		vp->vq_flags = 0;
	} else {
		if (strcmp(argv[1], "-u"))
			usage();
		vp->vq_flags = VQF_ADD;
		vsnp = argv[2];
	}

	if (checkvsn(vsnp))
		usage();

	memcpy(vp->vq_vsn, vsnp, VSNLEN);

	nLogInit(arg0, bomb);
	nLogInitStderr(EL_ERROR1);
	VVCLInit(vvHost, VVM_CLIENT, RETRY_FATAL);

	/*
	 *  msg is built - send it
	 */
	(void)VVCLSend(hp, 1);		/* dies if problem */

	/*
	 *  get response(s)
	 */
	while (1) {
		vvpHdr_t	*rhp;
		vvpError_t	*vep;

		/*
		 *  since only VVCLRecv is used to drive the
		 *	main loop - i.e. no select())  - no need to use
		 *	the result of VVCLNextRecord() to see if any pkt
		 *	is buffered.
		 */
		rhp = VVCLRecv("response");	/* dies if problem */
		(void)VVCLNextRecord();

		switch (rhp->vh_type) {
		    case VVM_ACK:
			/*
			 *  success
			 */
			printf("%s: done\n", vsnp);
			VVCLGoodbye(1);
			exit(0);
		    case VVM_VVERROR:
			vep = VV_MSGPTR(rhp, vvpError_t);
			VVError = vep->vn_error;
			nLogMsg(EL_ERROR1, 0, 
					"error from vvmd: %s", vep->vn_msg);
				
			VVCLGoodbye(1);
			exit(1);
		    case VVM_SHUTDOWN:
			nLogMsg(EL_ERROR1, 0, "vvmd shutdown received");
			exit(1);
		    default:
			nLogMsg(EL_ERROR1, 0, "unexpected pkt: %s",
							vvHdrStr(rhp));
			VVCLGoodbye(1);
			exit(1);
		}
	}
}
