/*
 *  vvtpdmp - NAStore Virtual Volume Manager VV tape reader
 *
 *	Bill Ross, NASA Ames, January 1998
 */

#include "vv_space.h"
#include "cl_space.h"

char	*arg0 = NULL;

  static void
usage()
{
	fprintf(stderr, "usage: %s vsn\n", arg0);
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

GetLabel(int vfd, TapeLabel_t *lp, char *type, char *vsnp, int file)
{
	int	len;

	len = read(vfd, lp, sizeof(*lp));
	if (len == -1) {
		fprintf(stderr, "%s: reading %s file %d: %s\n", 
					vsnp, type, file, strerror(errno));
		return(-2);
	}
	if (len == 0)
		return(0);

	if (len != sizeof(TapeLabel_t)) {
		fprintf(stderr, "%s file %d: reading %s: %d instead of %d\n",
			vsnp, file, type, len, sizeof(TapeLabel_t));
		return(-1);
	}
	if (memcmp(lp->lb_type, type, LBLTYPELEN)) {
		fprintf(stderr, "%s file %d: expecting %s: got %.*s\n",
			vsnp, file, type, LBLTYPELEN, lp->lb_type);
		return(-1);
	}
	if (lp->b1  != ' '  ||  lp->b2  != ' '  ||  lp->b3  != ' '  ||
	    lp->b4  != ' '  ||  lp->b5  != ' '  ||  lp->b6  != ' '  ||
	    lp->b7  != ' '  ||  lp->b8  != ' '  ||  lp->b9  != ' '  ||
	    lp->b10 != ' '  ||  lp->b11 != ' '  ||  lp->b12 != ' '  ||
	    lp->b13 != ' ') {
		fprintf(stderr, "%s file %d: bad record separators\n",
			vsnp, file, type, LBLTYPELEN, lp->lb_type);
		return(-1);
	}
	return(1);
}

  int 
CompareLabels(TapeLabel_t *lb1, TapeLabel_t *lb2)
{
	int	err = 0;

	if (memcmp(lb1->lb_version, lb2->lb_version, sizeof(lb1->lb_version))) {
		fprintf(stderr, "version diff\n");
		err++;
	}
	if (memcmp(&lb1->lb_vv, &lb2->lb_vv, sizeof(lb1->lb_vv))) {
		fprintf(stderr, "id diff\n");
		err++;
	}
	if (lb1->lb_slot != lb2->lb_slot) {
		fprintf(stderr, "slot diff\n");
		err++;
	}
	if (lb1->lb_nvsns != lb2->lb_nvsns) {
		fprintf(stderr, "nvsns diff\n");
		err++;
	}
	if (memcmp(lb1->lb_nrec, lb2->lb_nrec, sizeof(lb1->lb_nrec))) {
		fprintf(stderr, "nrec diff\n");
		err++;
	}
	if (memcmp(lb1->lb_data, lb2->lb_data, sizeof(lb1->lb_data))) {
		fprintf(stderr, "data diff\n");
		err++;
	}
	if (memcmp(lb1->lb_creat, lb2->lb_creat, sizeof(lb1->lb_creat))) {
		fprintf(stderr, "creat diff\n");
		err++;
	}
	if (memcmp(lb1->lb_nwrite, lb2->lb_nwrite, sizeof(lb1->lb_nwrite))) {
		fprintf(stderr, "nwrite diff\n");
		err++;
	}
	if (memcmp(lb1->lb_lastwrite, lb2->lb_lastwrite, 
					sizeof(lb1->lb_lastwrite))) {
		fprintf(stderr, "lastwrite diff\n");
		err++;
	}
	if (memcmp(lb1->lb_finish, lb2->lb_finish, sizeof(lb1->lb_finish))) {
		fprintf(stderr, "finish diff\n");
		err++;
	}
	return(err);
}


main(int argc, char *argv[])
{
	char		*vsnp, *vname, buf[2 * sizeof(TapeLabel_t)];
	char		fname1[MAXPATHLEN], fname2[MAXPATHLEN];
	int		fd, vfd, len, file, warn = 0, tpmrk = 0;
	TapeLabel_t	*lp = (TapeLabel_t *) buf;
	uint32_t	block;

	arg0 = argv[0];
#ifndef PURIFY
	if (getuid() != 0) {
		fprintf(stderr, "%s: must be root\n", arg0);
		exit(1);
	}
#endif
	if (argc != 2)
		usage();
	vsnp = argv[1];

	if (checkvsn(vsnp))
		usage();

	/*
	 *  open working copy of dump file
	 */
	strcpy(fname1, vvVsnFile(vsnp, "op"));
	strcpy(fname2, vvVsnFile(vsnp, NULL));
	fd = open(fname1, O_RDWR|O_CREAT|O_TRUNC);
	if (fd == -1) {
		perror(fname1);
		exit(1);
	}

	/*
	 *  mount tape on current host
	 */
	if (vm_mount((vsn_t *)vsnp, VO_WAIT|VO_MYHOST, NULL, 
					nas_basename(arg0), 1, 1)) {
		vm_prerr(stderr);
		unlink(fname1);
		exit(1);
	}

	/*
	 *  open tape node
	 */
	vname = vVsnNode(vsnp);
	vfd = open(vname, O_RDWR);
	if (vfd == -1) {
		perror(vname);
		unlink(fname1);
		exit(1);
	}

	/*
	 *  check vol1
	 */
	if (ntp_taprew(vfd, vsnp, 0) == -1) {
		unlink(fname1);
		exit(1);
	}
	len = read(vfd, buf, sizeof(buf));
	if (len == -1) {
		fprintf(stderr, "%s: reading vol1: %s\n", 
						vsnp, strerror(errno));
		unlink(fname1);
		exit(1);
	}
	if (len != LABELLEN) {
		/*
		 *  in case lbl was overwritten, get ready to look around
		 */
		fprintf(stderr, "%s: reading vol1: %s %d instead of %d\n",
			vsnp, (len == sizeof(buf) ? ">=" : ""), len, LABELLEN);
		warn++;
		if (ntp_taprew(vfd, vsnp, 0) == -1) {
			fprintf(stderr, "%s: rewinding: %s\n", vsnp,
							strerror(errno));
			unlink(fname1);
			exit(1);
		}
	} else {
		if (memcmp(buf, "VOL1", 4) != 0) {
			fprintf(stderr, "%s: bad VOL1\n", vsnp);
			warn++;
		}
		if (memcmp(buf+4, vsnp, VSNLEN) != 0) {
			/*
			 *  could be a bad mount; anything
			 *	from this could corrupt dbase
			 */
			fprintf(stderr, "%s: VOL1 says %.*s\n", 
							vsnp, VSNLEN, buf+4);
			unlink(fname1);
			exit(1);
		}
		if (memcmp(buf+37, "VVM", 3)) {
			fprintf(stderr, "%s: VOL1 not VVM\n", vsnp);
			warn++;
		}
	}

	/*
	 *  page thru file hdrs
	 */
	file = 0;
	while (1) {
		TapeRec_t	tprec;
		TapeLabel_t	lbl;
		int		ret;

		/*
		 *  get block id
		 */
		if (ntp_getblk(vfd, &block, vsnp, EL_TOSTDERR)) {
			exit(1);
		}
		C2ASC(SetDecInt, tprec.lb_block, block);

		/*
		 *  get/check HDR label
		 */
		switch (GetLabel(vfd, &tprec.rec, HDRLABEL, vsnp, file))
									{
		    case -1:
			continue;
		    case -2:
			goto done;
		    case 0:
			tpmrk++;
			continue;
		    default:
			break;
		}

		/*
		 *  seems to be a real file.. save lbl, skip data
		 */
		if (ntp_tapfsf(vfd, vsnp, EL_TOSTDERR) == -1) {
			fprintf(stderr, "%s: fsf over data (file %d): %s\n", 
					vsnp, file, strerror(errno));
			exit(1);
		}

		/*
		 *  get/check eof lbl
		 */
		switch (GetLabel(vfd, &tprec.rec, EOFLABEL, vsnp, file))
									{
		    case -1:
			continue;
		    case -2:
			goto done;
		    case 0:
			tpmrk++;
			continue;
		    default:
			break;
		}

		/*
		 *  compare to HDR
		 */
		if (CompareLabels(&lbl, &tprec.rec)) {
			fprintf(stderr, "%s file %d: labels !=\n", 
							fname1, file);
			exit(1);
		}

		/*
		 *  write label to file
		 */
		lbl.b13 = '\n';
		ret = write(fd, &lbl, sizeof(lbl));
		if (ret == -1) {
			perror(fname1);
			exit(1);
		}
		if (ret != sizeof(lbl)) {
			fprintf(stderr, "%s file %d: wrote %d not %d\n", fname1,
						file, ret, sizeof(lbl));
			exit(1);
		}
	}
done:
	rename(fname1, fname2);
	exit(0);
}
