/*
 *  vvtpl - NAStore Virtual Volume Manager VV tape lister
 *
 *	Bill Ross, NASA Ames, January 1998
 */

#include "vv_space.h"

char	*arg0 = NULL;

  static void
usage()
{
	fprintf(stderr, "usage: %s vsn\n", arg0);
	fprintf(stderr, "usage: %s -h\n", arg0);
	fprintf(stderr, "usage: %s -r\n", arg0);
	exit(1);
}

bomb()
{
	fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

main(int argc, char *argv[])
{
	char		buf[VV_PKTSIZ(vvpVsn_t)];
	vvpHdr_t	*hp;
	vvpVsn_t	*vp;

	arg0 = argv[0];
#ifndef PURIFY
	if (getuid() != 0) {
		fprintf(stderr, "%s: must be root\n", arg0);
		exit(1);
	}
#endif
	if (argc != 2)
		usage();

	hp = vvBuildHdr(VVM_VSNQRY, VVNOSEQ, (vvpHdr_t *)buf);
	vp = VV_MSGPTR(hp, vvpVsn_t);

	if (!strcmp(argv[1], "-h"))
		vp->vq_flags = VQF_HOT;
	else if (!strcmp(argv[1], "-r"))
		vp->vq_flags = VQF_RECYCLING;
	else if (checkvsn(argv[1]))
		usage();
	else {
		vp->vq_flags = VQF_VSN;
		memcpy(vp->vq_vsn, argv[1], VSNLEN);
	}

	nLogInit(arg0, bomb);
	nLogInitStderr(EL_ERROR1);
	VVCLInit(vvHost, VVM_CLIENT, RETRY_FATAL);

	(void)VVCLSend(hp, 1);		/* dies if problem */

	/*
	**	get response(s)
	*/
	while (1) {
		vvpHdr_t	*rhp;
		vvpError_t	*vep;
		Vsn_t		*vsp;

		/*
		**	since only VVCLRecv is used to drive the
		**	main loop - i.e. no select())  - no need to use
		**	the result of VVCLNextRecord() to see if any pkt
		**	is buffered.
		*/
		rhp = VVCLRecv("response");	/* dies if problem */
		(void)VVCLNextRecord();

		switch (rhp->vh_type) {
		    case VVM_VSNINFO:
			vsp = VV_MSGPTR(rhp, Vsn_t);
			printf("%.*s: \n", VSNLEN, vsp->vs_vsn);
			printf("\tstatus:\t%s\n", vvStatusStr(vsp->vs_status));
			printf("\tclass:\t%.*s\n", VV_CLASSLEN, vsp->vs_class);
			printf("\tslot:\t%d\n", vsp->vs_classI);
			printf("\tlastvv:\t%s\n", VV2Name(&vsp->vs_lastvv));
			printf("\t  record:\t%d\n", vsp->vs_lastrec);
			printf("\t  block: \t%d\n", vsp->vs_lastblk);
			printf("\t  id:    \t%.*s\n", VV_IDLEN, vsp->vs_lastid);
			printf("\tconsec errs:\t%d\n", vsp->vs_errsinarow);
			printf("\ttot errs:\t%d\n", vsp->vs_toterrs);
			printf("\tlive vv's:\t%d\n", vsp->vs_livevv);
			printf("\trecyc vv's:\t%d\n", vsp->vs_delvv);
			break;
		    case VVM_ACK:
			VVCLGoodbye(1);
			exit(0);
		    case VVM_VVERROR:
			vep = VV_MSGPTR(rhp, vvpError_t);
			VVError = vep->vn_error;
			nLogMsg(EL_ERROR1, 0, 
					"error from vvmd: %s", vep->vn_msg);
			VVCLGoodbye(1);
			exit(1);
		    case VVM_SHUTDOWN:
			nLogMsg(EL_ERROR1, 0, "vvmd shutdown received");
			exit(1);
		    default:
			fprintf(stderr, "unexpected pkt: %s\n", vvHdrStr(rhp));
			VVCLGoodbye(1);
			exit(1);
		}
	}
}
