#include "vv_space.h"

typedef struct {
	char	dh_filetype[2];		char	b1;
	char	dh_recsize[INTLEN];	char	b2;
	char	dh_version[INTLEN];	char	b3;
	char	dh_class[VV_CLASSLEN];	char	b4;
	char	dh_serial[VV_SERIALLEN];char	b5;
	char	dh_id[VV_IDLEN];	char	b6;
	char	dh_creat[DATELEN];	char	end;
} disk_hdr_t;

  int
DiskHdrSize()
{
	return(sizeof(disk_hdr_t));
}

  int
WriteDiskHdr(int fd, char *class, char *serial, char *cl_id, time_t now)
{
	char		*id = "WriteDiskHdr";
	ssize_t		ret;
	disk_hdr_t	dh;

	memset(&dh, ' ', sizeof(dh));
	dh.end = '\n';

	dh.dh_filetype[0] = 'V';
	dh.dh_filetype[1] = 'V';
	SetDecInt(dh.dh_recsize, sizeof(dh.dh_recsize), (int)sizeof(dh));
	SetDecInt(dh.dh_version, sizeof(dh.dh_version), VV_VERSION);
	memcpy(dh.dh_class, class, sizeof(dh.dh_class));
	memcpy(dh.dh_serial, serial, sizeof(dh.dh_serial));
	memcpy(dh.dh_id, cl_id, sizeof(dh.dh_id));
	SetTime(dh.dh_creat, now);

	if (dh.b1 != ' '  ||  dh.b2 != ' '  ||  dh.b3 != ' '  ||
	    dh.b4 != ' '  ||  dh.b5 != ' '  ||  dh.b6 != ' '  ||
	    dh.end != '\n') {
		nLogMsg(EL_ERROR1, id, "bad record separator(s) in header");
	}
	ret = write(fd, &dh, sizeof(dh));
	if (ret == (ssize_t)sizeof(dh))
		return(0);
	if (ret == -1) {
		VVError = VVE_SYS;
		nLogMsg(EL_ERROR1|EL_SYS|EL_CODE, id, "writing hdr");
		return(1);
	}
	if (ret != (ssize_t)sizeof(dh)) {
		VVError = VVE_FILE;
		nLogMsg(EL_ERROR1|EL_CODE, id, "writing hdr: %d/%d",
						ret, sizeof(dh));
		return(1);
	}
	return(0);
}

  int
CheckDiskHdr(int fd, char *fname, char *class, char *serial, char *cl_id)
{
	char		*id = "CheckDiskHdr";
	int		size, version, err = 0;
	ssize_t		ret;
	disk_hdr_t	dh;

	VVError = VVE_NOERROR;
	ret = read(fd, &dh, sizeof(dh));
	if (ret == -1) {
		VVError = VVE_SYS;
		nLogMsg(EL_ERROR1|EL_SYS|EL_CODE, id, "%s: reading hdr", fname);
		return(-1);
	}
	if (ret != (ssize_t)sizeof(dh)) {
		VVError = VVE_FILE;
		nLogMsg(EL_ERROR1|EL_CODE, id, "%s: reading hdr: %d/%d",
						fname, ret, sizeof(dh));
		return(-1);
	}
	if (memcmp(dh.dh_filetype, "VV", (size_t)2)) {
		VVError = VVE_FILE;
		nLogMsg(EL_ERROR1|EL_CODE, id, "%s: no 'VV' in hdr:\n%*.*s",
					fname, sizeof(dh), sizeof(dh), &dh);
		return(-1);
	}
	if (dh.b1 != ' '  ||  dh.b2 != ' '  ||  dh.b3 != ' '  ||
	    dh.b4 != ' '  ||  dh.b5 != ' '  ||  dh.b6 != ' '  ||
	    dh.end != '\n') {
		nLogMsg(EL_ERROR1, id, "%s: bad record separator(s) in header",
							fname);
		err++;
	}
	if (DecInt(&size, dh.dh_recsize, sizeof(dh.dh_recsize))  ||
	    DecInt(&version, dh.dh_version, sizeof(dh.dh_version))) {
		nLogMsg(EL_ERROR1, id, "%s: scanning numbers from header",
							fname);
		err++;
	} else {
		if (size != sizeof(dh)) {
			nLogMsg(EL_ERROR1, id, "%s: rec size not consistent",
							fname);
			err++;
		}
		if (version != VV_VERSION) {
			nLogMsg(EL_ERROR1, id, "%s: unexpected version %d", 
							fname, version);
			err++;
		}
	}

	if (memcmp(dh.dh_class, class, sizeof(dh.dh_class))) {
		nLogMsg(EL_ERROR1, id, 
			"%s: class doesn't match [%s/%s]",
			fname, ClassString(dh.dh_class), ClassString(class));
		err++;
	}
	if (memcmp(dh.dh_serial, serial, sizeof(dh.dh_serial))) {
		nLogMsg(EL_ERROR1, id, "%s: serial doesn't match", fname);
		err++;
	}
	if (memcmp(dh.dh_id, cl_id, sizeof(dh.dh_id))) {
		nLogMsg(EL_ERROR1, id, "%s: id doesn't match [%*.*s/%*.*s]",
				fname, 16, 16, dh.dh_id, 16, 16, cl_id);
		err++;
	}
	if (err) {
		VVError = VVE_FILE;
		nLogMsg(EL_ERROR1|EL_CODE, id, "%s: VV header mismatch", fname);
		return(-1);
	}
	return(size);
}
