#ifndef	_VV_PKT_H
#define	_VV_PKT_H
%
%/* 
% *  rpcgen source: vv_pkt.x - .c & .h files containing this
% *	message should never be checked in.
% */
%
%
%#include "vv_format.h"

/*
 *  Virtual Volume Manager
 *
 *  vv_pkt.x: Virtual Volume Manager Packet Protocol
 *
 *	rpcgen is used to generate vv_pkt.h and vv_pkt_xdr.c from 
 *	this file. 
 *
 *	All structures that are included in packets must be defined in
 *	this file, in order to generate the necessary XDR routines to
 *	send them between hosts. 
 *
 *	Contents:
 *			volume definition
 *			packet structures
 *
 *	Note: since 64-bit numbers are not uniformly handled by XDR
 *	implementatons, they are defined as 16-char hex here.
 *
 *  Bill Ross, NASA Ames, September 1997. 
 */
/*
 *  All lines beginning with '%' are passed through rpcgen unchanged.
 *	Otherwise, comments are stripped and structures modified.
 *
 *	vv_pkt.h is used for all communications, vv_pkt_xdr.c is
 *	used for inter-host-compatible, XDR-encoded packets. Note: 
 *	in vv_pkt.h, for each "struct xx", rpcgen adds "typedef xx").
 *
 *	The XDR protocol guarantees that the data types in the packet 
 *	structures aren't affected by architectural differences
 *	between machines.
 */

/*
 *  HACK - time_t defined as an XDR-known type
 */
#define time_t int


typedef unsigned int    vvId_t;           /* message key type             */
typedef unsigned int    vvSeq_t;          /* sequence number type         */

#ifdef RPC_HDR
/*
 *  nasutil.h for LINELEN & VSNLEN
 *  VV_MAXPATH - max name for paths in util cmds
 */
%#include <nasutil.h>
%#define VV_MAXPATH	256

#endif

/*************************************************************************
			VOLUME DEFINITION
**************************************************************************/

/*
 *  Virtual Volume name definition
 */
struct VV_t {
	char	class[VV_CLASSLEN];
	char	serial[VV_SERIALLEN];
};

/*
 *  Virtual Volume Info record, used for dbase & packets (VVM_VOLINFO)
 */
struct VVsn_t {
	char		v_vsn[VSNLEN];	/* physical volume vv is on     */
	int		v_nrec;		/* number of vv on vsn		*/
	unsigned int	v_block;	/* block to seek to		*/
	time_t		v_date;		/* last access			*/
};

struct	VVInfo_t {
	int	v_version;		/* VVM version			*/
	VV_t	v_vv;			/* VV name info			*/
	char	v_cluster[VV_CLUSTERLEN]; /* for grouping VV's		*/
	char	v_id[VV_IDLEN];		/* client's id (uid for rash)	*/
	char	v_use;			/* volume use status		*/
	char	v_status;		/* volume status		*/
	char	v_data[HEXLEN];		/* data in volume		*/ 
	char	v_maxdata[HEXLEN];	/* max data in volume		*/ 
	time_t	v_creat;		/* creation date		*/
	time_t	v_write;		/* last write authorization	*/
	time_t	v_finish;		/* date	finished		*/
	time_t	v_save;			/* date written to tape		*/
	time_t	v_read;			/* date	open/read		*/
	time_t	v_freed;		/* date	last freed		*/
	time_t	v_restore;		/* date	last restored		*/
	int	v_nwrite;		/* times opened for write	*/
	int	v_nread;		/* times opened for read	*/
	int	v_nrestore;		/* times restored		*/
	int	v_mntfail;		/* times in a row restore failed*/
	int	v_nvsns;		/* from class defn		*/
	VVsn_t	v_vsns[4];		/* vsns written to;		*/
					/* max 4 convenient for dbase	*/
};

#ifdef RPC_HDR

%/* v_status: 	WARNING: these are the values used in the vvm dbase:
% *		changing them here could lead to serious inconsistencies
% *		unless the dbase were updated (IN THE PROPER ORDER) at
% *		the same time.
% */
%#define VVS_HOT	'h'	/* still being appended to on disk	*/
%#define VVS_FINISHED	'f'	/* no longer appendable			*/
%#define VVS_MIGRATED	'm'	/* saved to tape			*/
%#define VVS_RECOVER	'r'	/* restore to re-tape a slot		*/
%#define VVS_RECYCLING	'R'	/* 'live' VV's being pulled from tape	*/
%#define VVS_RECYCLED	'D'	/* 'live' VV's all pulled from tape	*/
%#define VVS_ADDED	'a'	/* added as a result of tape scan	*/
%#define VVS_LOST	'l'	/* all tape copies lost			*/
%#define VVS_DELETED	'd'	/* recycled (vsn returned to volman)	*/

%/* note - reading isn't counted as a use  */
%#define VVU_IDLE	'i'	/* no client action			*/
%#define VVU_WRITING	'w'	/* a client is writing			*/
%#define VVU_WTAPE	't'	/* server write to tape 		*/
%#define VVU_RTAPE	'f'	/* server read from tape		*/

#endif /* RPC_HDR */



/*************************************************************************
				PACKETS
**************************************************************************/

/* 
 *  VVP message types - vh_type = int in vvpHdr_t
 *	don't change types or numbers without also changing VvpTab[]!
 */

#ifdef RPC_HDR

%#define	VVM_MINTYPE	1	/* for range checking 		*/

%#define	VVM_MOVER	1	/* register as mover		*/
%#define	VVM_CLIENT	2	/* register as client		*/
%#define	VVM_TIMERD	3	/* register as timer		*/
%#define	VVM_GOODBYE	4	/* shutdown a connection	*/
%#define	VVM_ACK		5	/* acknowledge			*/
%#define	VVM_DEBUG	6	/* for debug messages		*/
%#define	VVM_VVERROR	7	/* error code			*/
%#define	VVM_SHUTDOWN	8	/* shut down daemon/client	*/
%#define	VVM_TIMER	9	/* timer packet			*/
%#define	VVM_NEWLOG	10	/* restart log file(s)		*/
%#define	VVM_OWRITE	11	/* client wants to write more	*/
%#define	VVM_CWRITE	12	/* client done writing for now	*/
%#define	VVM_VOLQRY	13	/* query volume info		*/
%#define	VVM_VOLINFO	14	/* volume info response		*/
%#define	VVM_FINISH	15	/* client won't write any more	*/
%#define	VVM_OREAD	16	/* open to read (maybe restore)	*/
%#define	VVM_VRECYCLE	17	/* discard a volume		*/
%#define	VVM_WTAPE	18	/* write vvol info to tape	*/
%#define	VVM_RTAPE	19	/* read vvol info from tape	*/
%#define	VVM_VSNQRY	20	/* get vsn info			*/
%#define	VVM_VSNINFO	21	/* vsn info			*/
%#define	VVM_CHECKVSN	22	/* compare tbl of VVs on vsn	*/
%#define	VVM_BADVSN	23	/* recover from lost vsn	*/
%#define	VVM_RECYCVSN	24	/* recycle vsn			*/
%#define	VVM_COPYIX	25	/* spin off copy of index file	*/
%#define	VVM_STATUS	26	/* get status report		*/
%#define	VVM_BYTES	27	/* for status report		*/
%#define	VVM_FINCLASS	28	/* push VV's in class 2 tape	*/
%#define	VVM_FINISHVSN	29	/* don't write any more to vsn	*/
%#define	VVM_DISKCK	30	/* check db vs. disk cache	*/
%#define	VVM_NOP		31	/* check comm link		*/

%#define VVM_MAXTYPE	VVM_NOP		/* for range checking		*/

#endif  /* RPC_HDR */

/*
 *  the VVP message header
 */

struct vvpHdr_t {
	char		vh_magic;	/* magic number (must be first)	*/
	vvId_t		vh_key;		/* identifies sender		*/
	vvSeq_t		vh_seq;		/* sequence number		*/
	int		vh_type;	/* message type			*/
	unsigned int	vh_size;	/* msg size (excl. of header)	*/
	unsigned int	vh_count;	/* count of subpkts if needed	*/
};

#ifdef RPC_HDR
%#define	VVP_MAGIC	'+'	/* (unusual) magic for header	*/
#endif /* RPC_HDR */

/*
 *	Each message type may have an associated data struct.
 *	The following message types have no struct associated
 *	with them (i.e. vh_size = 0):
 *
 *		VVM_ACK, VVM_GOODBYE, VVM_NOP, VVM_NEWLOG
 *
 */

struct vvpHello_t {		/* for VVM_MOVER & VVM_CLIENT packets	*/
	int		vhe_version;	/* program/protocol version	*/
	vvId_t		vhe_id;		/* clnt->srvr key		*/
	int		vhe_pid;	/* process id			*/
	int		vhe_flag;	/* another int to play w/	*/
};


struct vvpTimer_t {		/* for VVM_TIMER		*/
	time_t		vt_time;	/* time vold begins sending	*/
};


struct vvpVol_t {
	VV_t	vo_vv;
	int	vo_flag;
};

struct vvpError_t {		/* for VVM_ERROR, VVM_SHUTDOWN	*/
	VV_t		vn_vv;		/* vol this is in response to	*/
	char		vn_vsn[VSNLEN];	/* tape involved		*/
	int		vn_req;		/* state of volume request	*/
	int		vn_error;	/* error code			*/
	char		vn_msg[LINELEN]; /* error string		*/
};

#ifdef RPC_HDR
/* flags for shutdown used in vn_state */
%#define	SH_WAIT		0x001
%#define	SH_RESTART	0x002
#endif

struct vvpDebug_t {		/* for VVM_DEBUG			*/
	int		vd_cmd;
	int		vd_level;
};

struct vvpOpenwrite_t {
	char		ow_id[VV_IDLEN];
	char		ow_class[VV_CLASSLEN];
	char		ow_size[HEXLEN];
};

struct vvpMvreq_t {		/* for VVM_WTAPE, VVM_RTAPE		*/
	char		m_vsn[VSNLEN];
	VV_t		m_vv;
	char		m_id[VV_IDLEN];
	int		m_checkvol1;
	unsigned int	m_block;
	int		m_rec;
	time_t		m_creat;
	int		m_nwrite;
	time_t		m_lastwrite;
	VV_t		m_lastvv;
	int		m_slot;
	int		m_nvsns;
	time_t		m_finish;
	int		m_bufsize;
	char		m_fsize[HEXLEN];
};

struct vvpVolqry_t {		/* for VVM_VOLQRY 		*/
	int	vq_flags;
	VV_t	vq_vv;
	char	vq_vsn[VSNLEN];
	char	vq_id[VV_IDLEN];
};

#ifdef RPC_HDR

%#define VQF_VOL	0x01
%#define VQF_ID		0x02
%#define VQF_CLASS	0x04
%#define VQF_HOT	0x08
%#define VQF_VSN	0x10
%#define VQF_FIN	0x20
%#define VQF_ADD	0x40	/* checkvsn - add missing recs to db */
%#define VQF_RECYCLING	0x80

#endif

struct vvpVsn_t {		/* for VVM_CHECKVSN and VVM_FINISHVSN 	*/
	int	vq_flags;
	char	vq_vsn[VSNLEN];
};

/*
 *  vvm_vsn.h - info on vsn kept in vvmd's vsn tbl
 */
struct Vsn_t {
	char		vs_vsn[VSNLEN];	
	char		vs_class[VV_CLASSLEN];
	char		vs_status;
	int		vs_classI;	/* slot # in class */
	VV_t		vs_lastvv;	/* name of last VV */
	int		vs_lastrec;	/* # of last VV on tape */
	unsigned int	vs_lastblk;	/* block# of last VV's eof label */
	int		vs_blksize;	/* size used to write blocks */
	char		vs_lastid[VV_IDLEN];
	int		vs_errsinarow;	/* mount-related */
	int		vs_toterrs;	/* mount-related */
	int		vs_livevv;	/* total live vv's */
	int		vs_delvv;	/* total 'dead' vv's */
};

struct vvpIx_t {
	int		i_pid;
};

struct vvpFinish_t {
	char		f_class[VV_CLASSLEN];
};

#endif	/* _VV_PKT_H */
