/*
 *  vv_space.h - NAStore Virtual Volume Manager definitions
 */

/*
 *  length of VV_VERSION in ascii decimal cannot exceed INTLEN
 *	for integrity of all existing VV's.
 */
#define VV_VERSION	3

/*
 *  VV_HDRLENLEN = length of ascii decimal field describing total length
 *	of VV header - i.e. header can never exceed length of 9999 if
 *	this is set to 4. Don't change this unless you want to break
 *	access to all existing VV's.
 */
#define VV_HDRLENLEN	4


#include <rpc/rpc.h>	/* for XDR */
#include <errno.h>

#include "nasutil.h"

#include "vv_format.h"
#include "vv_error.h"
#include "vv_pkt.h"
#include "vv_pkt2.h"
#include "vv_sock.h"
#include "vv_class.h"
#include "vv_tape.h"

#define VVM_IORETRY	9

#define VV_NAMELEN	(sizeof(VV_t) + 2)

/* vv_vv.c */

extern char	*VV2Name(VV_t *vv);
extern char	*VV2Path(VV_t *vvp);
extern VV_t	*Name2VV(char *str, int complain);

/* vv_p.c */

extern int	VVError;
extern char	*vvHost;
extern char	*vvm_service;
extern short	vvm_port;
extern int	vv_syslog;
extern char	*vvSpoolDir;
extern char	*vvVVDir;
extern int	vvVsnGroupId;
extern int	vvm_class_policy_class_len;
extern int	vvm_class_policy_serial_len;

extern char	*vvLogFile();
extern char	*vvVsnFile(const char *vsn, const char *extra);

extern char	*vvMsgType(int msgno);
extern ProTab_t	*vvMsgInfo(int msgno);
extern char	*vvErrStr(int err);
extern char	*vvPortTypeStr(int type);
extern char	*vvStatusStr(char status);
extern char	*vvUseStr(char use);
extern char	*vvHdrStr(vvpHdr_t *hp);

extern vvpHdr_t	*vvBuildHdr(int msg, vvSeq_t seq, vvpHdr_t *chp);
extern int	vvCheckPacket(vvpHdr_t *hp);
extern vvId_t	vvGetID();

extern int	ForkExec(char *path, char *name, char *arg);
extern void	ReserveFD(char *str);
extern void	UnreserveFD(char *str);
extern int	GetVVFromName(char *classp, int *serialp, char *name);

/* vv_xdr.c */
extern void	XDRCloseSock(Sock_t *sockp);
extern int	XDRSend(vvpHdr_t *hp, XDR *xdrp, int now);
extern int	XDRNextRecord(XDR *xdrp);
extern vvpHdr_t	*XDRRecv(XDR *xdrp);
extern void	XDRSetup(Sock_t *sockp, XDR *xdrp);
extern void	XDRFlush(XDR *xdrp);

/* vv_xdr2.c */
extern int	VVCLSend(vvpHdr_t *hp, int now);
extern vvpHdr_t	*VVCLRecv(char *str);
extern int	VVCLNextRecord();
extern int	VVCLInit(char *host, int vv_type, int retry);
extern void	VVCLGoodbye(int sendmsg);
extern void	VVCLErr2VV(char *volp, char *vsn, vvSeq_t seq, int req);
extern void	MMDebug(vvpHdr_t *hp);

#define RETRY_NOT	0
#define RETRY_FOREVER	1
#define RETRY_FATAL	2

/*  from vv_class.c */

extern int	CheckClasses();
extern int	IncClassSerial(char *serial);
extern int	CmpClassSerial(const char *s1, const char *s2);
extern void	ListClasses();
extern Class_t	*ClassInfo(char *prefix);
extern Class_t	*ClassIndex(int i);
extern char	*ClassString(char *class);
extern void	ResetClasses();
extern Class_t	*NextClass();
extern void	PrintClass(Class_t *cip);

/*  from vv_format.c */

extern char	*IdString(char *id);

/*  from vv_tape.c */

extern void	LabelBuf(TapeLabel_t *lblbuf, const char *lbl, VV_t *vvp);
extern int	CheckLabel(TapeLabel_t *lbl1, TapeLabel_t *lbl2);
extern void	LabelBuf2(TapeLabel_t *lblbuf, char *id,
                        int slot, int nvsns,
                        int nrec, OFF_T data,
                        time_t creat,
                        int nwrite, time_t lastwrite,
                        time_t finish,
                        time_t ontape);
extern int	TapeRecExtract(TapeRec_t *tbuf, unsigned long *blkp, 
			int *version,
                        VV_t *vvp, char *id,
                        int *slot, int *nvsns,
                        int *nrec, char *data,
                        time_t *creat,
                        int *nwrite, time_t *lastwrite,
                        time_t *finish,
                        time_t *ontape);

/*  from vv_disk.c */

extern int	DiskHdrSize();

extern int	WriteDiskHdr(int fd, char *class, char *serial, 
			char *cl_id, time_t now);

extern int	CheckDiskHdr(int fd, char *fname, char *class, char *serial, 
			char *cl_id);
