
/*
 *  vv_tape.c - tape label stuff
 */

#include "vv_space.h"

/*
 *  LabelBuf() - write basic part of label
 */
  void
LabelBuf(TapeLabel_t *lblbuf, const char *lbl, VV_t *vvp)
{
	memset(lblbuf, ' ', sizeof(*lblbuf));
	SetDecInt(lblbuf->lb_version, sizeof(lblbuf->lb_version), VV_VERSION);
	memcpy(lblbuf->lb_type, lbl, sizeof(lblbuf->lb_type));
	memcpy(&lblbuf->lb_vv, vvp, sizeof(lblbuf->lb_vv));
}

/*
 *  CheckLabel() check basic identifier parts of label
 */
  int
CheckLabel(TapeLabel_t *lbl1, TapeLabel_t *lbl2)
{
	char	*id = "CheckLabel";

	/*
	 *  TODO - allow for old versions as new ones are added
	 */
	if (memcmp(lbl1->lb_version, lbl2->lb_version, 
						sizeof(lbl1->lb_version))) {
		nLogMsg(EL_STATUS2, id, "versions differ: [%.*s] [%.*s]",
			INTLEN, lbl1->lb_version, INTLEN, lbl2->lb_version);
		return(1);
	}
	if (memcmp(lbl1->lb_type, lbl2->lb_type, sizeof(lbl1->lb_type))) {
		nLogMsg(EL_STATUS2, id, "types differ: [%.*s] [%.*s]",
			LBLTYPELEN, lbl1->lb_type, LBLTYPELEN, lbl2->lb_type);
		return(1);
	}
	if (memcmp(&lbl1->lb_vv, &lbl2->lb_vv, sizeof(lbl1->lb_vv))) {
		nLogMsg(EL_STATUS1, id, "\n[%.*s].[%.*s] \n[%.*s].[%.*s]",
					VV_CLASSLEN, lbl1->lb_vv.class, 
					VV_SERIALLEN, lbl1->lb_vv.serial,
					VV_CLASSLEN, lbl2->lb_vv.class, 
					VV_SERIALLEN, lbl2->lb_vv.serial);
		return(1);
	}
	return(0);
}

  void
LabelBuf2(TapeLabel_t *lblbuf, char *id, 
			int slot, int nvsns, 
			int nrec, OFF_T data, 
			time_t creat, 
			int nwrite, time_t lastwrite, 
			time_t finish, 
			time_t ontape)
{
	memcpy(lblbuf->lb_id, id, sizeof(lblbuf->lb_id));
	lblbuf->lb_slot = INT2CHAR(slot);
	lblbuf->lb_nvsns = INT2CHAR(nvsns);
	SetDecInt(lblbuf->lb_nrec, sizeof(lblbuf->lb_nrec), nrec);
	SetHexOFF(lblbuf->lb_data, sizeof(lblbuf->lb_data), data);
	SetTime(lblbuf->lb_creat, creat);
	SetDecInt(lblbuf->lb_nwrite, sizeof(lblbuf->lb_nwrite), nwrite);
	SetTime(lblbuf->lb_lastwrite, lastwrite);
	SetTime(lblbuf->lb_finish, finish);
	SetTime(lblbuf->lb_ontape, ontape);
}

  int
TapeRecExtract(TapeRec_t *tbuf, unsigned long *blkp, int *version,
			VV_t *vvp, char *id, 
			int *slot, int *nvsns,
			int *nrec, char *data, 
			time_t *creat, 
			int *nwrite, time_t *lastwrite, 
			time_t *finish, 
			time_t *ontape)
{
	if (C2BIN(DecUlong, blkp, tbuf->lb_block))
		return(1);

	if (DecInt(version, tbuf->rec.lb_version, sizeof(tbuf->rec.lb_version)))
		return(1);

	memcpy(vvp, &tbuf->rec.lb_vv, sizeof(*vvp));
	memcpy(id, tbuf->rec.lb_id, sizeof(tbuf->rec.lb_id));
	*slot = CHAR2INT(tbuf->rec.lb_slot);
	*nvsns = CHAR2INT(tbuf->rec.lb_nvsns);
	if (DecInt(nrec, tbuf->rec.lb_nrec, sizeof(tbuf->rec.lb_nrec)))
		return(1);
	memcpy(data, tbuf->rec.lb_data, sizeof(tbuf->rec.lb_data));
	*creat = GetTime(tbuf->rec.lb_creat);
	if (DecInt(nwrite, tbuf->rec.lb_nwrite, sizeof(tbuf->rec.lb_nwrite)))
		return(1);
	*lastwrite = GetTime(tbuf->rec.lb_lastwrite);
	*finish = GetTime(tbuf->rec.lb_finish);
	*ontape = GetTime(tbuf->rec.lb_ontape);
	return(0);
}
