
#include <ctype.h>
#include "vv_space.h"

/*
 *  VV2Name() - make a null-terminated string out of a VV_t.
 */
  char *
VV2Name(VV_t *vv)
{
	char		*id = "VV2Name";
	static char	buf[VV_NAMELEN];
	int		i, j;

	/*
	 *  class is a left-justified alphanum thing
	 */
	for (i=0; i<sizeof(vv->class); i++) {
		if (!isalnum(vv->class[i]))
			break;
		buf[i] = vv->class[i];
	}
	if (i == 0) {
		VVError = VVE_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, "null class");
		return(NULL);
	}

	/*
	 *  use a dot to separate
	 */
	buf[i++] = '.';

	/*
	 *  serial is a right-justified alphanum number
	 */
	for (j=0; j<sizeof(vv->serial); j++)
		if (isalnum(vv->serial[j]))
			break;

	if (j == sizeof(vv->serial)) {
		VVError = VVE_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, "no serial defined");
		return(NULL);
	}

	while (j<sizeof(vv->serial))
		buf[i++] = vv->serial[j++];

	buf[i] = '\0';
	return(buf);
}

static char	*pathbuf = NULL, *pathvv;

  char *
VV2Path(VV_t *vvp)
{
	char		*cp;

	if (pathbuf == NULL) {
		pathbuf = (char *)nAlloc(strlen(vvVVDir) + sizeof(VV_t) +1);
		sprintf(pathbuf, "%s/", vvVVDir);
		pathvv = pathbuf + strlen(pathbuf);
	}
	cp = VV2Name(vvp);
	if (cp == NULL)
		return(NULL);
	strcpy(pathvv, cp);
	return(pathbuf);
}

/*
 *  Name2VV() - convert a null-terminated string to a VV_t;
 *	for speed, only the size of VV_t components is checked
 */
  VV_t *
Name2VV(char *str, int complain)
{
	char		*id = "Name2VV";
	static VV_t	vv;
	size_t		len;
	char		*cp;

	cp = strchr(str, '.');
	if (cp == NULL) {
		if (complain) {
			VVError = VVE_USER;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
					"no '.' delimiter [%s]", str);
		}
		return(NULL);
	}

	len = cp - str;
	if (!len  ||  len > sizeof(vv.class)) {
		if (complain) {
			VVError = VVE_USER;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
						"bad class defn [%s]", str);
		}
		return(NULL);
	}
	
	memcpy(vv.class, str, len);
	memset(vv.class+len, ' ', sizeof(vv.class)-len);

	cp++;
	len = strlen(cp);
	if (len == 0  ||  len > sizeof(vv.serial)) {
		if (complain) {
			VVError = VVE_USER;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
						"bad serial defn [%s]", str);
		}
		return(NULL);
	}
	memset(vv.serial, ' ', sizeof(vv.serial)-len);
	memcpy(vv.serial+sizeof(vv.serial)-len, cp, len);
	return(&vv);
}

