
/*
 *  Volume Manager
 *
 *  vv_xdr2.c - comm for vvmd clients & movers to talk to vvmd
 *
 *	Routines are named VVCL tho movers are servers..
 */

#include "vv_space.h"

#include <time.h>
#include <setjmp.h>
#include <rpc/xdr.h>

#define RETRYSLEEP	30

extern int		errno;

/*
 *  immortality in case of vvmd termination is available
 *	to users of this communications lib
 */
int			immortal = 0;
jmp_buf			jmpbuf_i;

static int		vvmd_sd = -1;
static XDR		VVxdr;
static Sock_t		VVsock;	

static vvId_t		SendKey, ReceiveKey;
static int		Fatal = 1;	/* whether to die if vvmd does */

static socklen_t	socklen = sizeof( struct sockaddr_in );
static char		vvmd_host[MAXHOSTNAMELEN+1];

  int
VVCLSockId()
{
	return(vvmd_sd);
}

  static int 
VVCLConnect(char *host, int retry)
{
	char			*id = "VVCLConnect";
	char			myhost[MAXHOSTNAMELEN];
	struct hostent 		*htp;
	struct sockaddr_in	ip_addr;
	struct servent		*servent;
	int			i;
	time_t			time0, time1;

	if (host == NULL) {
		/*
		**	default host = current
		*/
		nLogMsg(EL_STATUS1, id, 
				"(using localhost as default for vvmd)");
		if (gethostname(myhost, sizeof(myhost)) == -1)
			nLogMsg(EL_FATAL, id, "gethostname");
		host = myhost;
	}
	strncpy(vvmd_host, host, sizeof(vvmd_host));

	if ((servent = getservbyname(vvm_service, "tcp")) == NULL)
		nLogMsg(EL_FATAL|EL_SYS, id, "getservbyname");
	vvm_port = ntohs(servent->s_port);

	VVsock.sock_fd = -1;
	VVsock.sock_id = -1;
	VVsock.sock_host = NULL;
	VVsock.sock_proc = NULL;
	VVsock.sock_ptype = S_NOSOCK;

        for (i=0; i<20 ; i++) {
		(void) sleep((unsigned) (i ? RETRYSLEEP : 1));

		htp = nGetHostInfo(host);

		vvmd_sd = socket(AF_INET, SOCK_STREAM, 0);

		if (vvmd_sd == -1)
			nLogMsg(EL_FATAL|EL_SYS, id, "socket");

		memcpy(&ip_addr.sin_addr, htp->h_addr, (size_t)htp->h_length);
	
		ip_addr.sin_family = htp->h_addrtype;
		ip_addr.sin_port = htons(vvm_port);

		if (connect(vvmd_sd, (struct sockaddr *)&ip_addr, socklen) == 0)
			break;

		/*  connect failed */
		switch (errno) {
		    case ECONNREFUSED:
		    case ETIMEDOUT:
		    case EADDRINUSE:
			break;
		    default:
			if (Fatal)
				exit(1);
			break;
		}
		close(vvmd_sd);
		vvmd_sd = -1;
		if (i == 0) {
			nLogMsg(EL_STATUS1, id, "can't connect, retrying...");
			time(&time0);
		} else {
			if (retry == RETRY_FOREVER)
				i = 1;
			time(&time1);
			if (time1 - time0 > 7200) {
				nLogMsg(EL_STATUS1, id, "still no vvmd on %s",
									host);
				time0 = time1;
			}
		}
	}
	if (vvmd_sd == -1) {
		nLogMsg(EL_SYS|EL_CODE, 0, 
				"vvmd on %s probably down: connect system err",
									host);
		return(1);
	}
	nLogMsg(EL_STATUS1, id, "connected");
	VVsock.sock_fd = vvmd_sd;
	XDRSetup(&VVsock, &VVxdr);
	return(0);
}

  int
VVCLSend(vvpHdr_t *hp, int now)
{
	char	*id = "VVCLSend";

	hp->vh_key = SendKey;
	if (XDRSend(hp, &VVxdr, now)) {
		VVCLGoodbye(0);
		if (immortal) {
			nLogMsg(EL_STATUS1, id, "longjmp");
			longjmp(jmpbuf_i, 1);
			nLogMsg(EL_FATAL|EL_SYS, id, "longjmp");
		}
		if (!Fatal)
			return(1);
		nLogMsg(EL_FATAL, id, "(%s)", 
					vvMsgType(hp->vh_type));
	}
	return(0);
}

/*
 *  VVCLRecv() - receive a pkt 
 */
  static vvpHdr_t *
CLRecv(char *str, int badok)
{
	char		*id = "CLRecv";
	vvpHdr_t	*hp;

retry:

	hp = XDRRecv(&VVxdr);
	if (hp == NULL) {
		if (errno == EINTR) {	/* mover gets sigcld */
			goto retry;
		}
		if (badok) {
			badok--;
			goto retry;
		}
		VVCLGoodbye(0);
		if (immortal) {
			nLogMsg(EL_STATUS1, id, "longjmp on vvmd closing conn");
			longjmp(jmpbuf_i, 1);
			nLogMsg(EL_FATAL|EL_SYS, id, "longjmp");
		}
		if (!Fatal) {
			nLogMsg(EL_SYS|EL_CODE, id, 
				"%s: vvmd died or closed connection", str);
			return(NULL);
		}
		nLogMsg(EL_SYS|EL_FATAL, id, 
			"%s: vvmd died or closed connection", str);
	}

	if (hp->vh_key != ReceiveKey) {
		if (badok) {
			nLogMsg(EL_STATUS1, id, "%s: skipping bad key 0x%x %s",
				str, hp->vh_key, vvHdrStr(hp));
			badok--;
			goto retry;
		}
		if (!Fatal)
			return(NULL);
		nLogMsg(EL_FATAL|EL_ERROR1, id,
			"%s: rcvd vvmd packet w/ bad key: 0x%x != 0x%x\n\t(%s)",
			str, hp->vh_key, ReceiveKey, vvHdrStr(hp));
	}
	return(hp);
}

  vvpHdr_t *
VVCLRecv(char *str)
{
	return(CLRecv(str, 0));
}

  int
VVCLNextRecord()
{
	return(XDRNextRecord(&VVxdr));
}

/*
 *  FUNCTION:	VVCLInit()
 *		Initialize a client connection to the vvmd server.
 */
  int
VVCLInit(char *host, int vv_type, int retry)
{
	char		*id = "VVCLInit";
	char		hbuf[VV_PKTSIZ(vvpHello_t)];
	vvpHdr_t	*hp, *rhp;
	vvpHello_t	*hellop;

	switch (retry) {
	    case RETRY_NOT:
	    case RETRY_FOREVER:
		Fatal = 0;
		break;
	    case RETRY_FATAL:
		Fatal = 1;
		break;
	    default:
		nLogMsg(EL_FATAL, id, "illegal 'retry' value (%d)", retry);
		break;
	}

	if (VVCLConnect(host, retry))
		return(1);

	/*
	**	build and send VVM_{type} message
	*/
	SendKey = ReceiveKey = vvGetID();

	hp = vvBuildHdr(vv_type, VVNOSEQ, (vvpHdr_t *)hbuf);
	hellop = VV_MSGPTR(hp, vvpHello_t);
	hellop->vhe_version = VV_VERSION;
	hellop->vhe_pid = getpid();

	/*
	**	VVCLSend() sets vh_key to SendKey (= ReceiveKey)
	*/
	if (VVCLSend(hp, 1))
		return(1);	/* !Fatal */

	nLogMsg(EL_DEBUG2, id, "sent %s", vvMsgType(vv_type));

	/*
	**	get reply, should be same packet
	**	CLRecv() checks that vh_key == ReceiveKey
	**	and allows skipping pkts w/ bad keys that
	**	may be left over in the sender's XDR send
	**	buf after prev clnt on the slot bailed.
	*/
	rhp = CLRecv(vvMsgType(vv_type), 2);	
	if (rhp == NULL)
		return(1);	/* !Fatal */
	if (rhp->vh_type != vv_type) {
if (rhp == NULL) nLogMsg(EL_FATAL, id, "NULL?????");
		nLogMsg(EL_FATAL, id, "got wrong response %s", vvHdrStr(rhp));
	}
	if (rhp->vh_key != ReceiveKey) {
		nLogMsg(EL_FATAL, id, "rcvd key 0x%x expected 0x%x", 
					rhp->vh_key, ReceiveKey);
	}

	hellop = VV_MSGPTR(rhp, vvpHello_t);
	SendKey = hellop->vhe_id;
	nLogMsg(EL_STATUS2, id, "vvmd connection completed");
	VVCLNextRecord();
	return(0);
}

  void
VVCLGoodbye(int sendmsg)
{
	char		*id = "VVCLGoodbye";
	vvpHdr_t	hdr;

	if (sendmsg) {
		(void) vvBuildHdr(VVM_GOODBYE, VVNOSEQ, &hdr);
		VVCLSend(&hdr, 1);
	}
	shutdown(vvmd_sd, 2);
	if (close(vvmd_sd) == -1)
		nLogMsg(EL_ERROR2|EL_SYS, id, "socket close failed");
	VVsock.sock_fd = vvmd_sd = -1;
	xdr_destroy(&VVxdr);
}



  void
MMDebug(vvpHdr_t *hp)
{
	char		*id = "MMDebug";
	vvpDebug_t	*dp;

	dp = VV_MSGPTR(hp, vvpDebug_t);

	if (dp->vd_cmd != 0) {
		nLogMsg(EL_ERROR1, id, "0x%x: unknown debug cmd", dp->vd_cmd);
		return;
	}
	if (dp->vd_level < 0  ||  dp->vd_level > EL_DEBUG4) {
		nLogMsg(EL_ERROR1, id, "0x%x: unknown log level", dp->vd_level);
		return;
	}
	if (dp->vd_level & ~EL_LEVEL) {
		nLogMsg(EL_ERROR1, id, "0x%x: non-loglevel bit(s)", 
								dp->vd_level);
		return;
	}
	nLogSetLevel(dp->vd_level);
	nLogMsg(EL_CONSOLE1|EL_CONSOLE0|EL_STATUS1, id, 
				"setting log level to 0x%x", dp->vd_level);
}

  void
VVCLGetIxCpy()
{
	char		*id = "VVCLGetIxCpy";
	char		buf[VV_PKTSIZ(vvpIx_t)];
	vvpHdr_t	*hp;
	vvpIx_t		*ip;

	hp = vvBuildHdr(VVM_COPYIX, VVNOSEQ, (vvpHdr_t *)buf);
	ip = VV_MSGPTR(hp, vvpIx_t);

	ip->i_pid = getpid();

	(void)VVCLSend(hp, 1);		/* dies if problem */ 

	/*
	 *  get response(s)
	 */
	while (1) {
		vvpHdr_t	*rhp;
		vvpError_t	*vep;

		/*
		 *  since only VVCLRecv is used to drive the
		 *	main loop - i.e. no select())  - no need to use
		 *	the result of VVCLNextRecord() to see if any pkt
		 *	is buffered.
		 */
		rhp = VVCLRecv("response");	/* dies if problem */
		(void)VVCLNextRecord();

		switch (rhp->vh_type) {
		    case VVM_ACK:
			/*
			**	success
			*/
			return;
		    case VVM_VVERROR:
			vep = VV_MSGPTR(rhp, vvpError_t);
			VVError = vep->vn_error;
			nLogMsg(EL_FATAL,id,"error from vvmd: %s", vep->vn_msg);
			break;
		    case VVM_SHUTDOWN:
			nLogMsg(EL_FATAL,id, "vvmd shutdown received");
			break;
		    default:
			nLogMsg(EL_FATAL,id,"unexpected pkt: %s",vvHdrStr(rhp));
			break;
		}
	}
}
