/*
 *  vvm_1.c: Volume Manager Request Processor - main() routine
 *
 *	vvm_1.c	main, packet proc switch to routines, housekeeping routines
 *	vvm_2.c client virtual volume requests
 *
 */

#include "vvmd.h"

extern Sock_t	SockArray[];

char	*arg0;

 static void
usage()
{
	(void)fprintf(stderr, "usage: %s\n", arg0);
	exit(1);
}

/*  debugging options:
 *
 *	-d: don't auto-detach (for debug, undocumented)
 */

extern int	timerpid;
extern int	totalHosts;
extern time_t	Now;

time_t		Start;

char		shutdownmsg[LINELEN];
char		**argvIn;
char		StartDir[MAXPATHLEN+1];


main(int argc, char *argv[])
{
	int		cc, dflg = 0;
	extern int	optind;
	/* extern char	*optarg; */

	(void)time(&Start);

	arg0 = argv[0];

	if (strstr(argv[0], "vvmd.pure")) {
		(void)fprintf(stderr, "running no-fork for vvmd.pure\n");
		dflg = 1;
	}
#ifndef PURIFY
	if (getuid() != 0) {
		(void)fprintf(stderr, "%s: must be run as root\n", arg0);
		exit(1);
	}
#endif
	/*
	 *  save args for vvshutdown -r(estart)
	 */
	argvIn = argv;
	if (getcwd(StartDir, sizeof(StartDir)) == 0)
		(void)fprintf(stderr, "getwd() error: %s\n", StartDir);

	/*
	 *  parse args
	 */
	while ((cc = getopt(argc, argv, "d")) != EOF) {
		switch (cc) {
		    case 'd':			/* don't fork */
			dflg = 1;
			break;
		    default:
			usage();
		}
	}
#ifdef PURIFY
	dflg = 1;
#endif
	if (optind < argc) 
		usage();

	/*
	 *  go into the background
	 */

	if (!dflg) {
		switch (fork()) {
		    case -1:				/* fork failed */
			(void)fprintf(stderr, "%s: can't fork... exiting\n", 
									arg0);
			exit(1);
			break;
		    case 0:				/* child */
			if (setpgid(0,0))
				perror("setpgid");
			break;
		    default:				/* parent */
			exit(0);	/* detach */
		}
	}

	/*
	 *  start up
	 */

	VVInit();

	/*
	 *  read & process all msgs using configured IPC
	 */

	IPCServer();
}

/*
 *  MNewLog() - restart log file (gets unwieldy if vvmd runs a long time)
 */
  static void
MNewLog(vvpHdr_t *hp, int slot)
{
	char		*id = "MNewLog";

	/*
	**	restart log and tell movers
	*/
	SendAck(hp->vh_seq, slot);
	nLogMsg(EL_STATUS1, id, "=========== restarting log ==============");
	nLogInitFile(vvLogFile(), EL_LEVEL, 1, NULL);
	nLogMsg(EL_STATUS1, id, "---------- RESTARTED LOG -------------------");
	nLogMsg(EL_STATUS1, id, "process started %s", bsd_ctime(&Start));

	/*
	**	tell mover on each host
	*/
	Send2Movers(hp, id);
}

/*
 *  MShutdown() - process VVM_SHUTDOWN packets received by the VVMD
 */
  static void
MShutdown(vvpHdr_t *hp, int slot)
{
	char		*id = "MShutdown";
	vvpError_t	*ep;

	/*
	**	Permission ok, ack client
	*/
	(void)XDRSendI(hp, slot, 1);
	ep = VV_MSGPTR(hp, vvpError_t);
	(void)VVShutdown(id, ep->vn_msg[0] ? ep->vn_msg :
	  		"Shutdown command from console");
	memcpy(shutdownmsg, ep->vn_msg, sizeof(shutdownmsg));
}

  static void
LogConnections()
{
	int	i;
	Host_t	*hop;

	nLogMsg(EL_STATUS1, 0, "------------------ connections -------------");
	for (i=0; i<FD_SETSIZE; i++) {
		hop = (Host_t *)SockArray[i].sock_host;

		nLogMsg(EL_STATUS1, 0, "%s pid %d on host %s",
					vvPortTypeStr(SockArray[i].sock_ptype),
					SockArray[i].sock_id,
					hop->h_name);
	}
	nLogMsg(EL_STATUS1, 0, "-------------------------------------------");
}

/*
 *  MDebug() - process VVM_DEBUG packets received by the VV
 *	print status, change logging level
 */
 static void
MDebug(vvpHdr_t *hp)
{
	char		*id = "MDebug";
	vvpDebug_t	*dp;
  	vvpHdr_t	*rhp;
	vvpDebug_t	*rdp;
	char		buf[VV_PKTSIZ(vvpDebug_t)];

	dp = VV_MSGPTR(hp, vvpDebug_t);

	switch (dp->vd_cmd) {
	    case 0:
		if (dp->vd_level & ~EL_LEVEL) {
			nLogMsg(EL_ERROR1, id, "0x%x: unknown debug level",
			  				dp->vd_level);
			break;
		}

		nLogSetLevel(dp->vd_level);
		nLogMsg(EL_STATUS1, id, "debug level set to 0x%x",
			  				dp->vd_level);
		rhp = vvBuildHdr(VVM_DEBUG, VVNOSEQ, (vvpHdr_t *) buf);
		rdp = VV_MSGPTR(rhp, vvpDebug_t);
		rdp->vd_cmd = dp->vd_cmd;
		rdp->vd_level = dp->vd_level;
		Send2Movers(rhp, id);
		break;
	    case 1:
		LogConnections();
		break;
	    case 2:
		LogClasses();
		break;
	    case 3:
		LogDisk();
		break;
	    default:
		nLogMsg(EL_ERROR1, id, "%d: unknown debug cmd", dp->vd_cmd);
		break;
	}
}

/*
 *  *MsgSwitch() - called from IPC - Pass incoming packets with client to
 *	appropriate functions for processing. Note if there is a need to
 *	process the request queue (procQ) as a result.. this is as much
 *	to keep the debug Q logging restricted to state changes as to 
 *	save CPU.
 */

/*
 *  ClientMsgSwitch() - msgs from clients (vv accessors & console)
 */
  int
ClientMsgSwitch(vvpHdr_t *hp, int slot)
{
	char		*id = "ClientMsgSwitch";
	Host_t		*hop = (Host_t *)SockArray[slot].sock_host;
	int		procQ = 0;

	nLogMsg(EL_DEBUG1, id, "client(%s): %s", hop->h_name, vvHdrStr(hp));

	switch (hp->vh_type) {
	    case VVM_VOLQRY:			/* look up vv(s)	*/
		MVVlook(hp, slot); break;
	    case VVM_OWRITE:
		MVVOpenWrite(hp, slot); break;
	    case VVM_CWRITE:
		MVVCloseWrite(hp, slot); break;
	    case VVM_FINISH:
		MVVFinish(hp, slot); break;
	    case VVM_OREAD:
		MVVOpenRead(hp, slot); break;
	    case VVM_STATUS:
		MLogClasses(slot, hp->vh_count); 
		MLogHosts(slot); 
		SendAck(0, slot); 
		break;
	    case VVM_GOODBYE:
		(void)ShutdownServer(slot, 0); break;
	    case VVM_VSNQRY:
		MVsnQry(hp, slot); break;
	    case VVM_VRECYCLE:
		MVVRecycle(hp, slot); break;
	    case VVM_SHUTDOWN:
		MShutdown(hp, slot); break;
	    case VVM_NEWLOG:
		MNewLog(hp, slot); break;
	    case VVM_CHECKVSN:
		MCheckVsn(hp, slot); break;
	    case VVM_RECYCVSN:
		MRecycleVsn(hp, slot); break;
	    case VVM_BADVSN:
		MBadVsn(hp, slot); break;
	    case VVM_COPYIX:
		MCopyIx(hp, slot); break;
	    case VVM_FINCLASS:
		MFinishClass(hp, slot); break;
	    case VVM_FINISHVSN:
		MFinishVsn(hp, slot); break;
	    case VVM_DISKCK:
		MDiskCk(hp, slot); break;
	    case VVM_DEBUG:
		MDebug(hp); break;
	    default:
		nLogMsg(EL_FATAL, id, "unexpected msg %s", vvHdrStr(hp));
	}
	return(procQ);
}


/*
**	MoverMsgSwitch() - msgs from vvmvd on some host
*/
  int
MoverMsgSwitch(vvpHdr_t *hp, int slot)
{
	char		*id = "MoverMsgSwitch";
	Host_t		*hop = (Host_t *)SockArray[slot].sock_host;

	nLogMsg(EL_STATUS1, id, "mover(%s): %s", hop->h_name, vvHdrStr(hp));

	switch (hp->vh_type) {
	    case VVM_WTAPE:
	    case VVM_RTAPE:
		return(MMoverOk(hp, slot));
	    case VVM_VVERROR:
		return(MMoverError(hp, slot));
	    case VVM_GOODBYE:
		return(ShutdownServer(slot, 1));
	    default:
		VVError = VVE_INTERNAL;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_STATUS1, id, 
				"unexpected type %s", vvHdrStr(hp));
		return(0);
	}
}

/*
 *  CleanupMover() - remount all requests using mover host;
 *	this solution won't be enough if vold keeps mounting
 *	on that host, but is fine if the host is down so that
 *	vold won't mount on it.. to handle volnd up & vvmvd 
 *	down, maybe having vvmd vary off the host's drives is
 *	the way to go, then varying on - but if don't want to
 *	overrride an ops vary, need to do a 2-level vary in
 *	vold.
 */
  int
CleanupMover(int slot)
{
	char		*id = "CleanupMover";
	Host_t		*hop;
	VsnReq_t	*vrp, *vrp2;
	int		count = 0;

	hop = (Host_t *)SockArray[slot].sock_host;

	nLogMsg(EL_STATUS1, id, 
		"Remounting all mover requests on host %s:", hop->h_name);

	for (vrp=VOLQHEAD(); vrp; vrp=vrp2) {
		vrp2 = VOLQNEXT(vrp);

		if (vrp->vr_hostp != hop)
			continue;
		if (vrp->vr_status != VSQ_MOVER)
			continue;
		count++;
		VolmanRemount(vrp);
	}

	return(count);
}

/*
 *  CleanupClient() - only to be called from ShutdownServer()
 */
  int
CleanupClient(int slot)
{
	char		*id = "CleanupClient";
	Host_t		*hop;

	hop = (Host_t *)SockArray[slot].sock_host;

	nLogMsg(EL_STATUS1, id, "client pid %d on host %s",
						SockArray[slot].sock_id,
						hop->h_name);

	DBRecoverClient(SockArray[slot].sock_id, hop->h_name);
	return(1);
}

/*
**	MTimer() - received VVM_TIMER message from vvtimer - forward to
**		any Movers.  Use the occasion to check for overdue requests.
*/
static int	count = 0;

  void
MTimer(vvpHdr_t *hp)
{
	char		*id = "MTimer";
	vvpTimer_t	*tp;

	if (hp->vh_type != VVM_TIMER) {
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
				"got unexpected msg type %s", vvHdrStr(hp));
		return;
	}

	/*
	**	see how long packet took and rejuvenate it
	**		for retransmission
	*/
	tp = VV_MSGPTR(hp, vvpTimer_t);
	if (Now - tp->vt_time > 2)
		nLogMsg(EL_STATUS2, id, "time from vvtimer to vvmd %d",
						Now - tp->vt_time);
	tp->vt_time = Now;
	Send2Movers(hp, id);

	/*
	**	check for overdue volman/mover requests,
	**		check disk cache
	*/
	if (count++ > 10) {
		count = 0;
		DiskStat();
		VolmanCheckQ();
		CWCheckQs(0);
	}

}



static CodeString_t vvVsnTab[] = {
	VSQ_NONE,	"none",
	VSQ_IDLE,	"idle",
	VSQ_VOLALLOC,	"alloc",
	VSQ_RETRYMOUNT,	"rtry_mnt",
	VSQ_VOLMOUNT,	"mounting",
	VSQ_MOUNTED,	"mounted",
	VSQ_MOVER,	"mover",
	VSQ_WAIT,	"wait",
	0,
};

  char *
vvVsnStatus(int state)
{
	return(nCodeToStr(state, vvVsnTab));
}
