/*
 *  vvm_2.h - basic volume provision for clients
 */

#include "vvmd.h"

extern Sock_t	SockArray[];

  void
MVVOpenWrite(vvpHdr_t *hp, int slot)
{
	char		*id = "MVVOpenWrite";
	vvpOpenwrite_t	*op = VV_MSGPTR(hp, vvpOpenwrite_t);
	char		buf[VV_PKTSIZ(VVInfo_t)];
	Host_t		*hop;
	VVMClass_t	*cip;
	VVInfo_t	*vip, *vip2;
	vvpHdr_t	*rhp = NULL;
	OFF_T		msize;

nLogMsg(EL_STATUS1, id, "entered");
	hop = (Host_t *)SockArray[slot].sock_host;
	/*
	 *  look up class
	 */
	cip = VVMClassInfo(op->ow_class);
	if (cip == NULL) {
nLogMsg(EL_STATUS1, id, "cip null");
		SendErr(hp->vh_seq, slot);
		return;
	}
	C2BIN(HexOFF, &msize, op->ow_size);
	/*
	**	make sure disk cache is available
	*/
	if (DiskReserve(cip)) {
		SendErr(hp->vh_seq, slot);
		return;
	}

	vip = NULL;
	if (msize < cip->vc_class.cs_vvsize) {
		/*
		**	try for an existing hot vv
		*/
		vip = DBFindHotVV(op->ow_id, msize, cip, 
					SockArray[slot].sock_id, hop->h_name);

		if (vip != NULL) {
nLogMsg(EL_STATUS1, id, "got hot");
			/* TODO  %%%enq */
			rhp = vvBuildHdr(VVM_VOLINFO, hp->vh_seq, 
							(vvpHdr_t *)buf);
			vip2 = VV_MSGPTR(rhp, VVInfo_t);
			*vip2 = *vip;
			(void)XDRSendI(rhp, slot, 1);
			return;
		}
nLogMsg(EL_STATUS1, id, "no hot");
	}

	/*
	**	see if class is still 'live'
	*/
	if (cip->vc_class.cs_flags & CS_NONEW) {
		/*
		**	class has been limited
		*/
		VVError = VVE_UNAVAIL;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
			"class %s has CS_NONEW set: can't create needed VV",
			cip->vc_class.cs_prefix);
		SendErr(hp->vh_seq, slot);
		return;
	}
	/*
	**	make a new hot vv
	*/
	vip = DBCreateHotVV(op->ow_id, msize, cip,
					SockArray[slot].sock_id, hop->h_name);
	if (vip != NULL) {
nLogMsg(EL_STATUS1, id, "made hot");
		/* TODO  %%%enq */
		if (!DiskCreate(vip, cip)) {

nLogMsg(EL_STATUS1, id, "diskcreate ok");
			rhp = vvBuildHdr(VVM_VOLINFO, hp->vh_seq, 
							(vvpHdr_t *)buf);
			vip2 = VV_MSGPTR(rhp, VVInfo_t);
			*vip2 = *vip;
			(void)XDRSendI(rhp, slot, 1);
			return;
		}
	}

	VVError = VVE_UNAVAIL;
	nLogMsg(EL_ERROR1|EL_CODE, id, "can't do it");
	SendErr(hp->vh_seq, slot);
}


  void
MVVCloseWrite(vvpHdr_t *hp, int slot)
{
	char		*id = "MVVCloseWrite";
	vvpVol_t	*vop = VV_MSGPTR(hp, vvpVol_t);
	OFF_T		fsize;
	VVMClass_t	*cip;
	Host_t		*hop;

	hop = (Host_t *)SockArray[slot].sock_host;

	if (DiskResidentSize(&vop->vo_vv, &fsize)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"HOT %s not on disk!",
					VV2Name(&vop->vo_vv));
		fsize = -1;
	}
nLogMsg(EL_STATUS1, id, "fsize %lld", fsize);

	if (DBCloseWriteVV(&vop->vo_vv, fsize, 
					SockArray[slot].sock_id, hop->h_name)) {
		SendErr(hp->vh_seq, slot);
		return;
	}
	SendAck(hp->vh_seq, slot);

	/*
	 *  see if VV is full enough to mark finished
	 */
	cip = VVMClassInfo(vop->vo_vv.class);
	if (cip == NULL) {
		nLogMsg(EL_CONSOLE0, id, "Unknown class %s", 
					ClassString(vop->vo_vv.class));
		return;
	}
nLogMsg(EL_STATUS1, id, "cs_finishsize %lld", cip->vc_class.cs_finishsize);

	DiskUnreserve(cip);

	if (fsize > cip->vc_class.cs_finishsize  ||  vop->vo_flag) {
		VVInfo_t	*vip;
nLogMsg(EL_STATUS1, id, "fsize %lld > %lld", 
	(long long) fsize, (long long) cip->vc_class.cs_finishsize);
		vip = DBFinishVV(&vop->vo_vv);
		if (vip == NULL)
			return;
		if (DiskFinish(&vop->vo_vv))
			return;
		/*
		 *  start save-to-tape
		 */
		CWriteVV2Vsns(vip);
	}
}


  void
MVVFinish(vvpHdr_t *hp, int slot)
{
	char		*id = "MVVFinish";
	char		buf[VV_PKTSIZ(VVInfo_t)];
	vvpVol_t	*vop = VV_MSGPTR(hp, vvpVol_t);
	VVInfo_t	*vip, *vip2;
	vvpHdr_t	*rhp = NULL;

	nLogMsg(EL_STATUS2, id, VV2Name(&vop->vo_vv));

	/* %%% TODO: check queue to see if active; for now use dbase */
	vip = DBFinishVV(&vop->vo_vv);
	if (vip == NULL) {
		SendErr(hp->vh_seq, slot);
		return;
	}
	if (DiskFinish(&vop->vo_vv)) {
		SendErr(hp->vh_seq, slot);
		return;
	}
	rhp = vvBuildHdr(VVM_VOLINFO, hp->vh_seq, (vvpHdr_t *)buf);
	vip2 = VV_MSGPTR(rhp, VVInfo_t);
	*vip2 = *vip;
	(void)XDRSendI(rhp, slot, 1);

	/*
	 *  start save-to-tape
	 */
	CWriteVV2Vsns(vip);
}

  void
MVVOpenRead(vvpHdr_t *hp, int slot)
{
	char		*id = "MVVOpenRead";
	vvpVol_t	*vop = VV_MSGPTR(hp, vvpVol_t);
	VVInfo_t	*vip;
	OFF_T		fsize;

	nLogMsg(EL_DEBUG1, id, VV2Name(&vop->vo_vv));

	vip = DBOpenReadVV(&vop->vo_vv);
	if (vip == NULL) {
		SendErr(hp->vh_seq, slot);
		return;
	}
	if (!DiskResidentSize(&vip->v_vv, &fsize)) {
		char		buf[VV_PKTSIZ(VVInfo_t)];
		vvpHdr_t	*rhp;
		
		/*
		**	it's on-disk
		*/
		rhp = vvBuildHdr(VVM_VOLINFO, hp->vh_seq, (vvpHdr_t *)buf);
		*VV_MSGPTR(rhp, VVInfo_t) = *vip;
		(void)XDRSendI(rhp, slot, 1);
		return;
	}

	/*
	**	VV is non-resident - need to restore it (or queue on restore)
	*/
	CRestore(hp->vh_seq, slot, vip);
}

  void
MVVRecycle(vvpHdr_t *hp, int slot)
{
	char		*id = "MVVRecycle";
	vvpVol_t	*vop = VV_MSGPTR(hp, vvpVol_t);

	nLogMsg(EL_STATUS2, id, VV2Name(&vop->vo_vv));

	/* %%% TODO: check queue to see if active; for now use dbase */
	if (DBRecycleVV(&vop->vo_vv)) {
		SendErr(hp->vh_seq, slot);
		return;
	}
	if (DiskRecycle(&vop->vo_vv)) {
		SendErr(hp->vh_seq, slot);
		return;
	}
	SendAck(hp->vh_seq, slot);
}

  int
MMoverOk(vvpHdr_t *hp, int slot)
{
	char		*id = "MMoverOk";
	VsnReq_t	*vrp;
	vvpMvreq_t	*mp;
	VVMClass_t	*cip;

	mp = VV_MSGPTR(hp, vvpMvreq_t);

	vrp = GetVsnMoverSeq(hp->vh_seq);
	if (vrp == NULL) {
		nLogMsg(EL_STATUS1, id, "no request for %.*s (slot %d)",
						VSNLEN, mp->m_vsn, slot);
		return(0);
	}

	/*
	**	reset mover sloterrs to 0 and 
	**	hand off to writer or reader
	*/
	cip = (VVMClass_t *)vrp->vr_cp;
	switch (vrp->vr_type) {
	    case VQT_WRITE:
		cip->vc_msloterrs[vrp->vr_writeslot] = 0;
		CWMoverOk(vrp, mp->m_rec, mp->m_block);
		break;
	    case VQT_READ:
		cip->vc_msloterrs[vrp->vr_readslot] = 0;
		CRMoverOk(vrp);
		break;
	    case VQT_RECYCLE:
		cip->vc_msloterrs[vrp->vr_readslot] = 0;
		CRecycleMoverOk(vrp);
		break;
	    default:
		nLogMsg(EL_ERROR1|EL_FATAL, id, "bad type %d", vrp->vr_type);
	}
	return(0);
}

  int
MMoverError(vvpHdr_t *hp, int slot)
{
	char		*id = "MMoverError";
	vvpError_t	*ep = VV_MSGPTR(hp, vvpError_t);
	VsnReq_t	*vrp;

	nLogMsg(EL_ERROR1, id, "%.*s/%s: %s (slot %d)",
					VSNLEN, ep->vn_vsn,
					VV2Name(&ep->vn_vv),
					ep->vn_msg, slot);
	vrp = GetVsnMoverSeq(hp->vh_seq);
	if (vrp == NULL) {
		nLogMsg(EL_STATUS1, id, "no request for %.*s_%d", 
						VSNLEN, ep->vn_vsn,
						hp->vh_seq);
		return(0);
	}

	/*
	**	recover according to tape use
	*/
	switch (vrp->vr_type) {
	    case VQT_WRITE:
		CWMoverErr(vrp, ep->vn_error, ep->vn_msg);
		break;
	    case VQT_READ:
		CRMoverErr(vrp);
		break;
	    case VQT_RECYCLE:
		CRecycleMoverErr(vrp);
		break;
	    default:
		nLogMsg(EL_ERROR1|EL_FATAL, id, "bad type %d", vrp->vr_type);
	}
LogClasses();

	return(0);
}

 void
MFinishClass(vvpHdr_t *hp, int slot)
{
	char		*id = "MFinishClass";
	vvpFinish_t	*fip;
	VVMClass_t	*cip;
	int		nStillHot, nFinishedVV, nWaitVV, nTapes;
	char		*cp, buf[200];
	vvpHdr_t	*rhp;

	fip = VV_MSGPTR(hp, vvpFinish_t);

	nLogMsg(EL_STATUS1, id, ClassString(fip->f_class));

	cip = VVMClassInfo(fip->f_class);
	if (cip == NULL) {
		SendErr(hp->vh_seq, slot);
		return;
	}
	if (DBFinishClass(fip->f_class, &nStillHot, &nFinishedVV, &nWaitVV, 
								&nTapes)) {
		SendErr(hp->vh_seq, slot);
		return;
	}
	rhp = vvBuildHdr(VVM_BYTES, hp->vh_seq, (vvpHdr_t *)buf);
	cp = VV_MSGPTR(rhp, char);
	(void)sprintf(cp, 
			"Finished %d VV's; Total q'd to tape: %d; %d hot tapes",
			nFinishedVV, nWaitVV, nTapes);
	rhp->vh_size = (unsigned int)strlen(cp) + 1;
	(void)XDRSendI(rhp, slot, 1);
	if (nStillHot) {
		(void)sprintf(cp, "\tSTILL HOT: %d", nStillHot);
		rhp->vh_size = (unsigned int)strlen(cp);
		if (XDRSendI(rhp, slot, 1))
			return;
	}
	SendAck(hp->vh_seq, slot);
}
