
/*
 *  vvm_3.c - Virtual Volume Mgr database-only operations
 */

#include "vvmd.h"

extern Sock_t	SockArray[];

/*
 *  VVVsnQuery() - look up VV's by vsn
 */
  static void
VVVsnQuery(vvpHdr_t *hp, int slot, vvpHdr_t *rhp)
{
	char		*id = "VVVsnQuery";
	vvpVolqry_t	*vqp = VV_MSGPTR(hp, vvpVolqry_t);
	VVInfo_t	*vip, *vip2;
	int		i, pid, ret;

	vip = VV_MSGPTR(rhp, VVInfo_t);

	/*
	 *  look up vsn - 1st 20 done by this process
	 */
	VVError = VVE_NOERROR;
	for (i=0; ; i++) {
		vip2 = DBLookupVVVsn(i, vqp->vq_vsn);
		if (!vip2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			return;
		}
		if (vqp->vq_flags & 
			    (VQF_FIN|VQF_HOT|VQF_RECYCLING|VQF_CLASS|VQF_ID)) {
			/*
			**	check individual flags
			*/
			if (vqp->vq_flags & VQF_FIN  &&  
						vip->v_status != VVS_FINISHED)
				continue;
			if (vqp->vq_flags & VQF_HOT  &&  
						vip->v_status != VVS_HOT)
				continue;
			if (vqp->vq_flags & VQF_RECYCLING  &&  
						vip->v_status != VVS_RECYCLING)
				continue;
			if (vqp->vq_flags & VQF_CLASS  &&  
					memcmp(vqp->vq_vv.class, 
					       vip->v_vv.class, 
					       sizeof(vqp->vq_vv.class)))
				continue;
			if (vqp->vq_flags & VQF_ID  &&  
					memcmp(vqp->vq_id, vip->v_id, 
						sizeof(vqp->vq_id)))
				continue;
		}
		*vip = *vip2;
		if (i<19) {
			/*
			**	buffered send
			*/
			if (XDRSendI(rhp, slot, 0))
				return;
		} else {
			/*
			**	on 20th, flush &fork to continue
			*/
			if (XDRSendI(rhp, slot, 1))
				return;
			break;
		}
	}

	/*
	 *  more than 20 hot - copy vsn index and continue as
	 *	another process to avoid tying up server. The
	 *	index is copied because the server's bplus-based
	 *	paging could change its structure while this process
	 *	is using it
	 */
	nLogMsg(EL_DEBUG1, id, "more than 20 hot: forking");
	if (DBCloseForVVVsnQuery(SockArray[slot].sock_id)) {
		SendErr(hp->vh_seq, slot);
		if (DBReopenForVVVsnQuery(1))
			nLogMsg(EL_FATAL, id, "DBReopenForVVVsnQuery()");
		return;
	}

	if ( pid = fork() ) {

		/*
		 *  'real' vvmd (pid = child's)
		 */
		if (DBReopenForVVVsnQuery(1))
			nLogMsg(EL_FATAL, id, "DBReopenForVVVsnQuery()");
		nLogMsg(EL_STATUS2, id,
			"Child %d forked to complete the look up", pid);
		return;
	}

	/*
	 *  Child: every way out of this routine from here on
	 *	should be an exit - no returns; sleep before
	 *	exit so that msgs have a chance to flush thru
	 */

	/*
	 *  set up to not shut down on error,
	 *	and not log every message
	 */
	IsChild++;
	nLogSetLevel(EL_STATUS2);

	/*
	 *  reset dbase
	 */
	ret = DBReopenForVVVsnQuery(0);
	if (ret > 0) {
		SendErr(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(2);
	} 
	if (ret < 0) {
		SendAck(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(0);
	}

	/*
	 *  send the rest of the id to client
	 */
	for (;;i++) {
		vip2 = DBLookupVVVsn(i, vqp->vq_vsn);
		if (!vip2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			break;
		}
		/*
		 *  screen out if necc
		 */
		if (vqp->vq_flags & 
			    (VQF_FIN|VQF_HOT|VQF_RECYCLING|VQF_CLASS|VQF_ID)) {
			/*
			**	check individual flags
			*/
			if (vqp->vq_flags & VQF_FIN  &&  
						vip->v_status != VVS_FINISHED)
				continue;
			if (vqp->vq_flags & VQF_HOT  &&  
						vip->v_status != VVS_HOT)
				continue;
			if (vqp->vq_flags & VQF_RECYCLING  &&  
						vip->v_status != VVS_RECYCLING)
				continue;
			if (vqp->vq_flags & VQF_CLASS  &&  
					memcmp(vqp->vq_vv.class,
						vip->v_vv.class, 
						sizeof(vqp->vq_vv.class)))
				continue;
			if (vqp->vq_flags & VQF_ID  &&  
					memcmp(vqp->vq_id, vip->v_id, 
						sizeof(vqp->vq_id)))
				continue;
		}
		*vip = *vip2;
		if (XDRSendI(rhp, slot, 0))
			break;
	}
	DBCleanupChild();
	(void)sleep(5);
	exit(0);
}

  static void
FinishedQuery(vvpHdr_t *hp, int slot, vvpHdr_t *rhp)
{
	char		*id = "FinishedQuery";
	vvpVolqry_t	*vqp = VV_MSGPTR(hp, vvpVolqry_t);
	VVInfo_t	*vip, *vip2;
	int		i, pid, ret;

	vip = VV_MSGPTR(rhp, VVInfo_t);

	/*
	 *  look up finished - 1st 20 done by this process
	 */
	VVError = VVE_NOERROR;
	for (i=0; ; i++) {
		vip2 = DBQueryFinishedVV(i);
		if (!vip2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			return;
		}
		/*
		 *  screen out if necc
		 */
		if (vqp->vq_flags & VQF_ID  &&  
				memcmp(vqp->vq_id, vip->v_id, 
						sizeof(vqp->vq_id)))
			continue;
		if (vqp->vq_flags & VQF_CLASS  &&  
		   	    	memcmp(vqp->vq_vv.class, vip->v_vv.class, 
						sizeof(vqp->vq_vv.class)))
			continue;
		*vip = *vip2;
		if (i<19) {
			/*
			**	buffered send
			*/
			if (XDRSendI(rhp, slot, 0))
				return;
		} else {
			/*
			**	on 20th, flush &fork to continue
			*/
			if (XDRSendI(rhp, slot, 1))
				return;
			break;
		}
	}

	/*
	 *  more than 20 'finished' - copy finished index and continue 
	 *	as another process to avoid tying up server. The
	 *	index is copied because the server's bplus-based
	 *	paging could change its structure while this process
	 *	is using it
	 */
	nLogMsg(EL_DEBUG1, id, "more than 20 hot: forking");
	if (DBCloseForFinishedQuery(SockArray[slot].sock_id)) {
		SendErr(hp->vh_seq, slot);
		if (DBReopenForFinishedQuery(1))
			nLogMsg(EL_FATAL, id, "DBReopenForFinishedQuery()");
		return;
	}

	if ( pid = fork() ) {

		/*
		 *  'real' vvmd (pid = child's)
		 */
		if (DBReopenForFinishedQuery(1))
			nLogMsg(EL_FATAL, id, "DBReopenForFinishedQuery()");
		nLogMsg(EL_STATUS2, id,
			"Child %d forked to complete the look up", pid);
		return;
	}

	/*
	 *  Child: every way out of this routine from here on
	 *	should be an exit - no returns; sleep before
	 *	exit so that msgs have a chance to flush thru
	 */

	/*
	 *  set up to not shut down on error,
	 *	and not log every message
	 */
	IsChild++;
	nLogSetLevel(EL_STATUS2);

	/*
	 *  reset dbase
	 */
	ret = DBReopenForFinishedQuery(0);
	if (ret > 0) {
		SendErr(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(2);
	} 
	if (ret < 0) {
		SendAck(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(0);
	}

	/*
	 *  send the rest of the 'finished' to client
	 */
	for (;;i++) {
		vip2 = DBQueryFinishedVV(i);
		if (!vip2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			break;
		}
		/*
		 *  screen out if necc
		 */
		if (vqp->vq_flags & VQF_ID  &&  
				memcmp(vqp->vq_id, vip->v_id, 
						sizeof(vqp->vq_id)))
			continue;
		if (vqp->vq_flags & VQF_CLASS  &&  
		   	    	memcmp(vqp->vq_vv.class, vip->v_vv.class, 
						sizeof(vqp->vq_vv.class)))
			continue;
		*vip = *vip2;
		if (XDRSendI(rhp, slot, 0))
			break;
	}
	DBCleanupChild();
	(void)sleep(5);
	exit(0);
}

  static void
HotQuery(vvpHdr_t *hp, int slot, vvpHdr_t *rhp)
{
	char		*id = "HotQuery";
	vvpVolqry_t	*vqp = VV_MSGPTR(hp, vvpVolqry_t);
	VVInfo_t	*vip, *vip2;
	int		i, pid, ret, exact = 0;

	vip = VV_MSGPTR(rhp, VVInfo_t);

	if (vqp->vq_flags & VQF_ID  &&  vqp->vq_flags & VQF_CLASS)
		exact = 1;

	/*
	 *  look up hot - 1st 20 done by this process
	 */
	VVError = VVE_NOERROR;
	for (i=0; ; i++) {
		if (exact)
			vip2 = DBQueryHotExactVV(i, vqp->vq_id,
						vqp->vq_vv.class);
		else
			vip2 = DBQueryHotVV(i);
		if (!vip2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			return;
		}
		if (!exact) {
			/*
			 *  screen out if necc
			 */
			if (vqp->vq_flags & VQF_ID  &&  
					memcmp(vqp->vq_id, vip->v_id, 
						sizeof(vqp->vq_id)))
				continue;
			if (vqp->vq_flags & VQF_CLASS  &&  
		    	    memcmp(vqp->vq_vv.class, vip->v_vv.class, 
						sizeof(vqp->vq_vv.class)))
				continue;
		}
		*vip = *vip2;
		if (i<19) {
			/*
			**	buffered send
			*/
			if (XDRSendI(rhp, slot, 0))
				return;
		} else {
			/*
			**	on 20th, flush &fork to continue
			*/
			if (XDRSendI(rhp, slot, 1))
				return;
			break;
		}
	}

	/*
	 *  more than 20 hot - copy hot index and continue as
	 *	another process to avoid tying up server. The
	 *	index is copied because the server's (originally) bplus-based
	 *	paging could change its structure while this process
	 *	is using it
	 */
	nLogMsg(EL_DEBUG1, id, "more than 20 hot: forking");
	if (DBCloseForHotQuery(SockArray[slot].sock_id)) {
		SendErr(hp->vh_seq, slot);
		if (DBReopenForHotQuery(1, 0))
			nLogMsg(EL_FATAL, id, "DBReopenForHotQuery()");
		return;
	}

	if ( pid = fork() ) {

		/*
		 *  'real' vvmd (pid = child's)
		 */
		if (DBReopenForHotQuery(1, 0))
			nLogMsg(EL_FATAL, id, "DBReopenForHotQuery()");
		nLogMsg(EL_STATUS2, id,
			"Child %d forked to complete the look up", pid);
		return;
	}

	/*
	 *  Child: every way out of this routine from here on
	 *	should be an exit - no returns; sleep before
	 *	exit so that msgs have a chance to flush thru
	 */

	/*
	 *  set up to not shut down on error,
	 *	and not log every message
	 */
	IsChild++;
	nLogSetLevel(EL_STATUS2);

	/*
	 *  reset dbase
	 */
	ret = DBReopenForHotQuery(0, exact);
	if (ret > 0) {
		SendErr(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(2);
	} 
	if (ret < 0) {
		SendAck(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(0);
	}

	/*
	 *  send the rest of the hot to client
	 */
	for (;;i++) {
		if (exact)
			vip2 = DBQueryHotExactVV(i, vqp->vq_id, 
							vqp->vq_vv.class);
		else
			vip2 = DBQueryHotVV(i);
		if (!vip2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			break;
		}
		if (!exact) {
			/*
			 *  screen out if necc
			 */
			if (vqp->vq_flags & VQF_ID  &&  
					memcmp(vqp->vq_id, vip->v_id, 
							sizeof(vqp->vq_id)))
				continue;
			if (vqp->vq_flags & VQF_CLASS  &&  
		    	    memcmp(vqp->vq_vv.class, vip->v_vv.class, 
						sizeof(vqp->vq_vv.class)))
				continue;
		}
		*vip = *vip2;
		if (XDRSendI(rhp, slot, 0))
			break;
	}
	DBCleanupChild();
	(void)sleep(5);
	exit(0);
}

  static void
IdQuery(vvpHdr_t *hp, int slot, vvpHdr_t *rhp)
{
	char		*id = "IdQuery";
	vvpVolqry_t	*vqp = VV_MSGPTR(hp, vvpVolqry_t);
	VVInfo_t	*vip, *vip2;
	int		i, pid, ret;

	vip = VV_MSGPTR(rhp, VVInfo_t);
nLogMsg(EL_STATUS1, id, "id [%*.*s]", VV_IDLEN, VV_IDLEN, vqp->vq_id);
	/*
	**	look up hot - 1st 20 done by this process
	*/
	VVError = VVE_NOERROR;
	for (i=0; ; i++) {
		vip2 = DBLookupIdVV(i, vqp->vq_id);
		if (!vip2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			return;
		}
		if (vqp->vq_flags & VQF_CLASS  &&  
		    memcmp(vqp->vq_vv.class, vip->v_vv.class, 
						sizeof(vqp->vq_vv.class)))
			continue;
		*vip = *vip2;
		if (i<19) {
			/*
			**	buffered send
			*/
			if (XDRSendI(rhp, slot, 0))
				return;
		} else {
			/*
			**	on 20th, flush &fork to continue
			*/
			if (XDRSendI(rhp, slot, 1))
				return;
			break;
		}
	}

	/*
	 *  more than 20 w/ id - copy id index and continue as
	 *	another process to avoid tying up server. The
	 *	index is copied because the server's bplus-based
	 *	paging could change its structure while this process
	 *	is using it
	 */
	nLogMsg(EL_DEBUG1, id, "more than 20 hot: forking");
	if (DBCloseForIdQuery(SockArray[slot].sock_id)) {
		SendErr(hp->vh_seq, slot);
		if (DBReopenForIdQuery(1))
			nLogMsg(EL_FATAL, id, "DBReopenForIdQuery()");

		return;
	}

	if ( pid = fork() ) {

		/*
		 *  'real' vvmd (pid = child's)
		 */
		if (DBReopenForIdQuery(1))
			nLogMsg(EL_FATAL, id, "DBReopenForIdQuery()");
		nLogMsg(EL_STATUS2, id,
			"Child %d forked to complete the look up", pid);
		return;
	}

	/*
	 *  Child: every way out of this routine from here on
	 *	should be an exit - no returns; sleep before
	 *	exit so that msgs have a chance to flush thru
	 */

	/*
	 *  set up to not shut down on error,
	 *	and not log every message
	 */
	IsChild++;
	nLogSetLevel(EL_STATUS2);

	/*
	 *  reset dbase
	 */
	ret = DBReopenForIdQuery(0);
	if (ret > 0) {
		SendErr(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(2);
	} 
	if (ret < 0) {
		SendAck(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(0);
	}

	/*
	 *  send the rest of the id to client
	 */
	for (;;i++) {
		vip2 = DBLookupIdVV(i, vqp->vq_id);
		if (!vip2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			break;
		}
		/*
		 *  screen out if necc
		 */
		if (vqp->vq_flags & VQF_CLASS  &&  
		    memcmp(vqp->vq_vv.class, vip->v_vv.class, 
						sizeof(vqp->vq_vv.class)))
			continue;
		*vip = *vip2;
		if (XDRSendI(rhp, slot, 0))
			break;
	}
	DBCleanupChild();
	(void)sleep(5);
	exit(0);
}


  void
MVVlook(vvpHdr_t *hp, int slot)
{
	char		*id = "MVVlook";
	vvpVolqry_t	*vqp = VV_MSGPTR(hp, vvpVolqry_t);
	VVInfo_t	*vip, *vip2;
	vvpHdr_t	*rhp;
	char		buf[VV_PKTSIZ(VVInfo_t)];

	/*
	 *  set up reply pkt
	 */
	rhp = vvBuildHdr(VVM_VOLINFO, hp->vh_seq, (vvpHdr_t *)buf);
	vip = VV_MSGPTR(rhp, VVInfo_t);
	if (vqp->vq_flags & VQF_VOL) {
		/*
		**  query on single vol
		*/
		if (vip2 = DBLookupVV(&vqp->vq_vv)) {
			*vip = *vip2;
			/*
			**	client can set hp->vh_count if ack not
			**		wanted (in this single-response
			**		case)
			*/
			if (hp->vh_count == 0) {
				if (XDRSendI(rhp, slot, 0))
					return;
				SendAck(hp->vh_seq, slot);
			} else
				(void)XDRSendI(rhp, slot, 1);
		} else
			SendErr(hp->vh_seq, slot);
		return;
	}

	/*
	 *  if 'vsn' flag, use vsn index for primary search - beats
	 *	checking each vsn slot for vsn if another index were
	 *	used for primary. (But, have to check all others in loop.)
	 */
	if (vqp->vq_flags & VQF_VSN) {
		VVVsnQuery(hp, slot, rhp);
		return;
	}

	/*
	 *  if 'finished' flag, use finished index for primary search
	 *	(Don't have to check vsn or hot in loop)
	 */
	if (vqp->vq_flags & VQF_FIN) {
		FinishedQuery(hp, slot, rhp);
		return;
	}

	/*
	 *  if hot flag, use hot index for primary search
	 *	(Don't have to check vsn or finished in loop)
	 */
	if (vqp->vq_flags & VQF_HOT) {
		HotQuery(hp, slot, rhp);
		return;
	}

	/*
	**	use id index for primary search
	**		(only have to check class in loop)
	*/
	if (vqp->vq_flags & VQF_ID) {
		IdQuery(hp, slot, rhp);
		return;
	}

	VVError = VVE_USER;
	nLogMsg(EL_CODE|EL_ERROR1, id, 
				"must give vol, hot/finished, id or vsn");
	SendErr(hp->vh_seq, slot);
}


OpenWrite(char *path)
{
	char	*id = "OpenWrite";
	STAT_T	sbuf;
	int	fd;

	if (STAT(path, &sbuf) == -1) {
		if (errno != ENOENT) {
			VVError = VVE_USER;
			nLogMsg(EL_SYS|EL_CODE, id, "output file %s error",
							path);
			return(-1);
		}
	} else {
		VVError = VVE_USER;
		nLogMsg(EL_ERROR1|EL_CODE, id, "output file %s already exists",
							path);
		return(-1);
	}
	fd = open(path, O_RDWR);
	if (fd == -1) {
		VVError = VVE_USER;
		nLogMsg(EL_SYS|EL_CODE, id, "opening output file %s", path);
		return(-1);
	}
	return(fd);
}

  void
MVsnQry(vvpHdr_t *hp, int slot)
{
	char		*id = "MVsnQry";
	char		buf[VV_PKTSIZ(Vsn_t)];
	vvpVsn_t	*vp = VV_MSGPTR(hp, vvpVsn_t);
	char		*vsnp = vp->vq_vsn; /* not used */
	vvpHdr_t	*rhp;
	Vsn_t		*vsp, *vsp2;
	int		i, pid, ret;

	/*
	**	set up reply pkt
	*/
	rhp = vvBuildHdr(VVM_VSNINFO, hp->vh_seq, (vvpHdr_t *)buf);
	vsp = VV_MSGPTR(rhp, Vsn_t);

	if (vp->vq_flags == VQF_VSN) {
		/*
		**	handle query-by-name
		*/
		vsp2 = DBLookupVsn(vp->vq_vsn);
		if (vsp2 == NULL) {
			SendErr(hp->vh_seq, slot);
		} else {
			*vsp = *vsp2;

			/*
			**	client can set hp->vh_count if ack not
			**		wanted (in this single-response
			**		case)
			*/
			if (hp->vh_count == 0) {
				if (XDRSendI(rhp, slot, 0))
					return;
				SendAck(hp->vh_seq, slot);
			} else
				(void)XDRSendI(rhp, slot, 1);
		}
		return;
	}
	if (vp->vq_flags != VQF_HOT) {
		VVError = VVE_USER;
		nLogMsg(EL_CODE|EL_ERROR1, id, "bad option 0x%x", vp->vq_flags);
		SendErr(hp->vh_seq, slot);
		return;
	}
	VVError = VVE_NOERROR;
	for (i=0; ; i++) {
		vsp2 = DBQueryHotVsn(i);
		if (!vsp2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			return;
		}
		*vsp = *vsp2;
		if (i<19) {
			/*
			**	buffered send
			*/
			if (XDRSendI(rhp, slot, 0))
				return;
		} else {
			/*
			**	on 20th, flush &fork to continue
			*/
			if (XDRSendI(rhp, slot, 1))
				return;
			break;
		}
	}

	/*
	 *  more than 20 'finished' - copy finished index and continue 
	 *	as another process to avoid tying up server. The
	 *	index is copied because the server's bplus-based
	 *	paging could change its structure while this process
	 *	is using it
	 */
	nLogMsg(EL_DEBUG1, id, "more than 20 hot: forking");
	if (DBCloseForHotVsnQuery(SockArray[slot].sock_id)) {
		SendErr(hp->vh_seq, slot);
		if (DBReopenForHotVsnQuery(1))
			nLogMsg(EL_FATAL, id, "DBReopenForHotVsnQuery()");
		return;
	}

	if ( pid = fork() ) {

		/*
		 *  'real' vvmd (pid = child's)
		 */
		if (DBReopenForHotVsnQuery(1))
			nLogMsg(EL_FATAL, id, "DBReopenForHotVsnQuery()");
		nLogMsg(EL_STATUS2, id,
			"Child %d forked to complete the look up", pid);
		return;
	}

	/*
	 *  Child: every way out of this routine from here on
	 *	should be an exit - no returns; sleep before
	 *	exit so that msgs have a chance to flush thru
	 */

	/*
	 *  set up to not shut down on error,
	 *	and not log every message
	 */
	IsChild++;
	nLogSetLevel(EL_STATUS2);

	/*
	 *  reset dbase
	 */
	ret = DBReopenForHotVsnQuery(0);
	if (ret > 0) {
		SendErr(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(2);
	} 
	if (ret < 0) {
		SendAck(hp->vh_seq, slot);
		DBCleanupChild();
		(void)sleep(5);
		exit(0);
	}

	/*
	 *  send the rest of the 'finished' to client
	 */
	for (;;i++) {
		vsp2 = DBQueryHotVsn(i);
		if (!vsp2) {
			if (VVError == VVE_NOERROR)
				SendAck(hp->vh_seq, slot);
			else
				SendErr(hp->vh_seq, slot);
			break;
		}
		*vsp = *vsp2;
		if (XDRSendI(rhp, slot, 0))
			break;
	}
	DBCleanupChild();
	(void)sleep(5);
	exit(0);
}


  void
MCheckVsn(vvpHdr_t *hp, int slot)
{
	char		*id = "MVsnCheck";
	vvpVsn_t	*vp = VV_MSGPTR(hp, vvpVsn_t);
	char		*fname = vvVsnFile(vp->vq_vsn, NULL);
	int		fd = open(fname, O_RDWR);

	if (fd == -1) {
		VVError = VVE_USER;
		nLogMsg(EL_SYS|EL_CODE, id, "opening vsn file %s", fname);
		SendErr(hp->vh_seq, slot);
	}

	if (DBCheckVsn(vp->vq_vsn, fd, vp->vq_flags))
		SendErr(hp->vh_seq, slot);
	else
		SendAck(hp->vh_seq, slot);
	close(fd);
}

/*
 *  MBadVsn() - admin has decided that the tape is lost or broken;
 *	if it's mounted, unmount, and in any case schedule writing
 *	any VV's on it to a new tape.
 */
  void
MBadVsn(vvpHdr_t *hp, int slot)
{
	char		*id = "MBadVsn";
	vvpVsn_t	*vp = VV_MSGPTR(hp, vvpVsn_t);

	if (VolmanCancelBusy(vp->vq_vsn)) {
		SendErr(hp->vh_seq, slot);
	} else if (DBBadVsn(vp->vq_vsn)) {
		VVError = VVE_DBASE;	/* catch-all */
		nLogMsg(EL_ERROR1|EL_CODE, id, "Error(s) - see log file");
		SendErr(hp->vh_seq, slot);
	} else
		SendAck(hp->vh_seq, slot);
}


/*
 *  MFinishVsn() - mark vsn 'finished' (no more VV's written)
 */
 void
MFinishVsn(vvpHdr_t *hp, int slot)
{
	char		*id = "MFinishVsn";
	vvpVsn_t	*vp;
	Vsn_t		*Vsnp;
	char		buf[200];
	vvpHdr_t	*rhp;

	vp = VV_MSGPTR(hp, vvpVsn_t);
	if (VolmanCancelBusy(vp->vq_vsn)) {
		SendErr(hp->vh_seq, slot);
	} else if ((Vsnp=DBFinishVsn(vp->vq_vsn)) == NULL) {
		SendErr(hp->vh_seq, slot);
	} else {
		(void)sprintf(buf, 
			"Finished vsn class %s slot %d contains %d VV's", 
			ClassString(Vsnp->vs_class), Vsnp->vs_classI,
			Vsnp->vs_livevv);
		if (!SendBuf(slot, buf))
			SendAck(hp->vh_seq, slot);
	}
}


  void
MCopyIx(vvpHdr_t *hp, int slot)
{
	char		*id = "MCopyIx";
	vvpIx_t		*ip = VV_MSGPTR(hp, vvpIx_t);

	if (DBCopyIx(ip->i_pid)) {
		VVError = VVE_DBASE;	/* catch-all */
		nLogMsg(EL_ERROR1|EL_CODE, id, "Error(s) - see log file");
		SendErr(hp->vh_seq, slot);
	} else
		SendAck(hp->vh_seq, slot);
}
