
/*
 *  vvm_class.h - VVMClass_t includes vv_class.h's Class_t, plus
 *	info from dbase on last vv created in this class, plus
 *	list of current 'hot' vsns for writing this class to,
 *	and where on each vsn to write after. And a queue of
 *	vv write requests, a queue of vv restore requests, and
 *	a queue of vsn recycle requests.
 */

/*
 *  define a read-vv-from-vsn request to hang on VVMClass_t's vc_readq;
 *	its goal is to mount one of the vsns that a VV has been saved
 *	to and do the read.
 */
typedef struct ReadReq {
	void		*r_cp;			/* parent class struct	    */
	QLink_t		r_q;			/* queue from parent	    */
	QLink_t		r_waitq;		/* wait on other read of VV */
	char		r_panel[MAXVSN * VSNLEN];
	char		r_panel2[MAXVSN * VSNLEN];
	int		r_nvsns;		/* vsns left to try	    */
	int		r_clnt;			/* socket of client	    */
	vvSeq_t		r_seq;			/* client seq		    */
	VVInfo_t	r_vv;
	VsnReq_t	*r_vrp;		/* the mount aspect	    */
} ReadReq_t;

/*
 *  define a read-all-nonresident-vv's-from-vsn request to hang on
 *	VVMClass_t's vc_recycq; its goal is to mount the vsn and
 *	restore all vv's on it so they can be written to other
 *	vsns, in order to recycle the vsn to volman.
 */
typedef struct RecycReq {
	void		*ry_cp;			/* parent class struct	    */
	QLink_t		ry_q;			/* queue from parent	    */
	char		ry_slot;		/* slot of vsn in panel	    */
	char		ry_panel[MAXVSN*VSNLEN];/* vsn to mount/restore     */
	int		ry_totvv;		/* approx total to do	    */
	int		ry_donevv;		/* vv's restored	    */
	VVInfo_t	ry_vv;			/* current vv		    */
	VsnReq_t	*ry_vrp;		/* the mount aspect	    */
} RecycReq_t;

/*
 *  define a write-vv-to-vsns request to hang on VVMClass_t's vc_writeq;
 *	its goal is to write a copy of the vv to one 'hot' vsn 
 *	from each of the slots (rc,voltype) defined for the 
 *	class. It can get the vsn for a slot from any of the
 *	'panels' of hotvsns.
 *	These requests are started when a vv is closed,
 *	and are queued off of the class struct, below.
 */
typedef struct WriteReq {
	void		*wr_cp;			/* parent class struct	    */
	QLink_t		wr_q;			/* queue from parent	    */
	QLink_t		wr_slotq[MAXVSN];	/* queue for slots	    */
	VVInfo_t	wr_vv;			/* name of vv to write	    */
	int		wr_statpanel[MAXVSN];	/* each slot's panel status */
	int		wr_sloterrs[MAXVSN];	/* # of tapes tried	    */
	time_t		wr_start;		/* time launched	    */
} WriteReq_t;

/*
 *  wr_statpanel[] values; greater than -1 means a panel slot is
 *	allocated and the request is in progress (queued for mount,
 *	mounting, or writing, according to the VsnReq_t that the
 *	panel[wr_statpanel[slot]].hp_vsns[slot].hv_vrp is pointing to)
 */
#define WR_STWAIT	-1
#define WR_STDONE	-2
#define WR_STFAILED	-3


/*
 *  define a panel of hot vsns, i.e. 1 for each slot defined for the
 *	class; as each individual vsn is filled with vv's, it will be 
 *	replaced with a freshly allocated scratch vsn from volman that 
 *	matches the criteria configured in lib/vv_class.c for that slot 
 *	in the class. The HotVsn_t maps the dbase vsn info to any current
 *	write request & to any current request to volman or mover.
 */
typedef struct HotVsn {
	int		hv_panel;		/* panel # of this vsn	    */
	int		hv_slot;		/* slot # of this vsn	    */
	int		hv_status;		/* vsn defined or not	    */
	int		hv_error;		/* for noting blocked alloc */
	Vsn_t		hv_vsnrec;		/* dbase record if HV_ALLOC */
	WriteReq_t	*hv_writerp;		/* VV request		    */
	VsnReq_t	*hv_vrp;		/* the volman/mover aspect  */
	time_t		hv_wait;		/* for mount retries	    */
} HotVsn_t;

/* hv_status */
#define HV_NOVSN	1
#define HV_VOLALLOC	2
#define HV_ALLOC	3

typedef struct HotPanel {
	HotVsn_t	hp_vsns[MAXVSN];
	time_t		hp_lastuse;
} HotPanel_t;


/*
 *  define the master class struct, which has the config info
 *	defining the name and numbering of the vv's in the class,
 *	the type of vsn in each slot, the panels of current vsns
 *	available to write to, and the queues of writes in progress.
 */
typedef struct {
	Class_t		vc_class;		/* from lib/vv_class.h	    */
	int		vc_newwrite;		/* marker to check queues   */
	int		vc_npanels;		/* from VVCONF config file  */
	char		vc_lastnum[VV_SERIALLEN];/* from dbase at startup   */
	char		vc_tmpnum[VV_SERIALLEN];/* for checking dbase	    */
	HotPanel_t	*vc_hotpanel;		/* cs_maxpanel panels	    */
	QHead_t		vc_writeq;		/* WriteReq_t's		    */
	QHead_t		vc_readq;		/* ReadReq_t's		    */
	QHead_t		vc_recycq;		/* RecycReq_t's		    */
	QHead_t		vc_slotq[MAXVSN];	/* WriteReq_t's	waitng/slot */
	int		vc_waitct[MAXVSN];	/* vc_slotq counts	    */
	int		vc_activect[MAXVSN];	/* panels active per slot   */
	int		vc_idlemount[MAXVSN];	/* max idle b4 dismount	    */
	int		vc_vsloterrs[MAXVSN];	/* volman errors in a row   */
	int		vc_msloterrs[MAXVSN];	/* mover errors in a row    */
	time_t		vc_allocwait[MAXVSN];	/* wait b4 retry alloc	    */
} VVMClass_t;

/*
 *  some macros for queueing WriteReq_t's on the VVMClass_t
 */
#define CWRITEENQ(_wrp, _cip) \
		ENQUEUE(_wrp, &(_cip)->vc_writeq, wr_q, WriteReq_t)
#define CWRITEDEQ(_wrp, _cip) \
		DEQUEUE(_wrp, &(_cip)->vc_writeq, wr_q, WriteReq_t)

#define CWRITEHEAD(_cip)	((WriteReq_t *) (_cip)->vc_writeq.q_head)
#define CWRITENEXT(_wrp)	((WriteReq_t *) (_wrp)->wr_q.q_next)

#define CWRSLOTENQ(_wrp, _slot, _cip) { \
	ENQUEUE(_wrp, &(_cip)->vc_slotq[_slot], wr_slotq[_slot], WriteReq_t); \
	(_cip)->vc_waitct[_slot]++;   }
#define CWRSLOTDEQ(_wrp, _slot, _cip) { \
	DEQUEUE(_wrp, &(_cip)->vc_slotq[_slot], wr_slotq[_slot], WriteReq_t); \
	(_cip)->vc_waitct[_slot]--;   }

#define CWRSLOTHEAD(_cip, _slot) \
	((WriteReq_t *) (_cip)->vc_slotq[_slot].q_head)
#define CWRSLOTNEXT(_wrp, _slot) \
	((WriteReq_t *) (_wrp)->wr_slotq[_slot].q_next)

#define CREADENQ(_rp, _cip) ENQUEUE(_rp, &(_cip)->vc_readq, r_q, ReadReq_t)
#define CREADDEQ(_rp, _cip) DEQUEUE(_rp, &(_cip)->vc_readq, r_q, ReadReq_t)
#define CREADHEAD(_cip)	((ReadReq_t *) (_cip)->vc_readq.q_head)
#define CREADNEXT(_rp)	((ReadReq_t *) (_rp)->r_q.q_next)

#define CWAITNEXT(_rp)	((ReadReq_t *) (_rp)->r_waitq.q_next)
#define CWAITPREV(_rp)  ((ReadReq_t *) (_rp)->r_waitq.q_prev)

#define CRECYCENQ(_ryp, _cip) \
		ENQUEUE(_ryp, &(_cip)->vc_recycq, ry_q, RecycReq_t)
#define CRECYCDEQ(_ryp, _cip) \
		DEQUEUE(_ryp, &(_cip)->vc_recycq, ry_q, RecycReq_t)

#define CRECYCHEAD(_cip) ((RecycReq_t *) (_cip)->vc_recycq.q_head)
#define CRECYCNEXT(_ryp)  ((RecycReq_t *) (_cip)->ry_q.q_next)
