
/*
**	vvm_clwrite.c - managing the writing of VV's to tape
**
**		uses queue & hot panels on VVMClass_t
*/

#include "vvmd.h"

extern int		ClassMax;
extern VVMClass_t	*ClassArray;
extern int		volman_sock;

#define MAX_WR_SLOT_ERRS	3

#define WR_SLOT_BATCH		2
#define WR_MAX_WAIT		120

int		hot_mounted = 0;

/*
**	DelVsnWriteReq() - req needs to be dq'd from vsn waitq's;
**		removes req from hot panel that originated it.
*/
  static void
DelVsnWriteReq(VsnReq_t *vrp)
{
	char		*id = "DelVsnWriteReq";
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;

	if (vrp->vr_waitq.q_prev)
		nLogMsg(EL_FATAL, id, "prev != NULL");
	if (vrp->vr_waitq.q_next)
		nLogMsg(EL_FATAL, id, "next != NULL");
	DelVsnReq(vrp);
	if (hvp) {
		hvp->hv_vrp = NULL;
		hvp->hv_wait = 0;
	}
}


/************************************************************************
	routines for keeping panels filled w/ 'hot' vsns available
	for mount
 ************************************************************************/


/*
**	AssignHotVsn() - try to line up a hot vsn available to mount
*/
  static int
AssignHotVsn(VVMClass_t *cip, int slot, HotVsn_t *hvp)
{
	char		*id = "AssignHotVsn";
	Vsn_t		*vp;

	if (hvp->hv_vrp != NULL)
		nLogMsg(EL_FATAL, id, "hv_vrp != NULL");

	if (hvp->hv_status != HV_NOVSN) {
		/*
		**	hot vsn already ready or requested
		*/
		return(0);
	}

	/*
	**	'white out' any prev vsn
	*/
	memset(hvp->hv_vsnrec.vs_vsn, ' ', sizeof(hvp->hv_vsnrec.vs_vsn));

	/*
	**	no hot vsn assigned; try to get one from dbase
	*/
	vp = DBGetHotVsn(cip->vc_class.cs_prefix, slot);
	if (vp != NULL) {
		/*
		**	hot one in dbase - drop it in slot
		*/
nLogMsg(EL_STATUS1, id, "   slot %d got db hot", slot);
		hvp->hv_vsnrec = *vp;
		hvp->hv_status = HV_ALLOC;
		return(1);
	}

	/*
	**	skip if alloc requests on slot pausing b4 retry
	*/
	if (cip->vc_allocwait[slot]) {
		if (cip->vc_allocwait[slot] < Now) {
			nLogMsg(EL_STATUS2, id, 
			    "  (wait to retry remaining on slot %d: %d sec)",
				slot, Now - cip->vc_allocwait[slot]);
			return(0);
		}
nLogMsg(EL_STATUS1, id, "Now %d  wait %d", Now, cip->vc_allocwait[slot]);
		/*
		**	done waiting
		*/
		cip->vc_allocwait[slot] = 0;
		nLogMsg(EL_STATUS2, id, "  (done waiting on slot %d - retry)",
								slot);
	}

	/*
	**	no hot vsn in dbase; start volman scratch alloc if possible
	*/
nLogMsg(EL_STATUS1, id, "  slot %d start alloc", slot);
	if (volman_sock != -1) {
		hvp->hv_vrp = VolmanAlloc(cip, slot, hvp);
		if (hvp->hv_vrp != NULL)
			hvp->hv_status = HV_VOLALLOC;
	}
	return(0);
}



/*
**	CWInitHotPanels() - using default or rc.vvm-set panel counts 
**		for each class, set up 'hot' vsn tracking. Fill in as 
**		many hot vsns as possible from the dbase and start volman 
**		scratch allocations for positions that remain empty, 
**		if volman up.
*/
  void
CWInitHotPanels()
{
	char		*id = "CWInitHotPanels";
	VVMClass_t	*cip = ClassArray;
	int		i, panel, slot;

	for (i=0; i<ClassMax; i++, cip++) {
		int		npanels = cip->vc_npanels;
		int		nvsns = cip->vc_class.cs_nvsns;
		HotPanel_t	*hp;

		/*
		**	allocate panels for this class
		*/
		hp = (HotPanel_t *)nAlloc(npanels * sizeof(HotPanel_t));
		cip->vc_hotpanel = hp;

		/*
		**	fill in panels
		*/
		for (panel=0; panel<npanels; panel++, hp++) {
			HotVsn_t	*hvp;

			/*
			**	set all possible slots
			**	(defined & undefined ones)
			*/
			hvp = hp->hp_vsns;
			for (slot=0; slot<MAXVSN; slot++, hvp++) {
				/*
				**	set permanent, easy-lookup fields
				*/
				hvp->hv_panel = panel;
				hvp->hv_slot = slot;

				/*
				**	set status = nothing happening yet
				*/
				hvp->hv_status = HV_NOVSN;
				hvp->hv_writerp = NULL;
				hvp->hv_wait = 0;
				hvp->hv_vsnrec.vs_vsn[0] = '\0';
				hvp->hv_vrp = NULL;

				/*
				**	try to fill slot w/ hot vsn
				**	ready-to-mount
				*/
				(void)AssignHotVsn(cip, slot, hvp);
			}

		}
		nLogMsg(EL_STATUS1|EL_NOTIME, id, 
			"class %s: %d panel%s %d vsn%s",
			cip->vc_class.cs_prefix, 
			npanels, (npanels != 1 ? "s" : ""),
			nvsns, (nvsns != 1 ? "s" : ""));
	}
}


/*
**	CWAllocDone() - got new vsn allocation back from volman, 
**		so add to dbase and update HotVsn_t w/ dbase record 
**		info
*/
  void
CWAllocDone(VsnReq_t *vrp)
{
	char		*id = "CWAllocDone";
	VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;
	char		*vsnp = vrp->vr_vsns + vrp->vr_writeslot * VSNLEN;
	Vsn_t		*vp;

	/*
	**	add vsn to dbase
	*/
	vp = DBAddHotVsn(vsnp, cip->vc_class.cs_prefix, vrp->vr_writeslot);
	if (vp == NULL)
		nLogMsg(EL_FATAL, id, 
			"vsn %.*s allocated but can't add to dbase",
			VSNLEN, vsnp);

	/*
	**	plug info into hot panel
	*/
	hvp->hv_status = HV_ALLOC;
	hvp->hv_vsnrec = *vp;

	/*
	**	delete vrp and ref to it
	*/
	DelVsnReq(vrp);
	hvp->hv_vrp = NULL;

}

  void
CWAllocFailed(VsnReq_t *vrp, char *msg, int level)
{
	char		*id = "CWAllocFailed";
	VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;
	int		slot = vrp->vr_writeslot;
	char		*prefix = cip->vc_class.cs_prefix;
	char		*rc = cip->vc_class.cs_vsns[slot].vs_rc;
	char		*voltype = cip->vc_class.cs_vsns[slot].vs_voltype;

	/*
	**	log problem
	*/
	switch (level) {
	    case 0:
		/*
		**	generic problem (volman down)
		*/
		nLogMsg(EL_STATUS2, id, "volman alloc failed (%s)", msg);
		break;
	    case 1:
		/*
		**	class/slot-specific; ops can handle
		*/
		nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
			"volman alloc err %s - saves blocked (class %s: %s/%s)",
			msg, prefix, rc, voltype);
		break;
	    case 2:
		/*
		**	class/slot-specific; admin must handle
		*/
		nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
		    "volman alloc err %s - saves blocked %s (class %s: %s/%s)",
			msg, "ADMIN NEEDED", prefix, rc, voltype);
		break;
	    default:
		nLogMsg(EL_FATAL, id, "bad level %d", level);
	}

	/*
	**	give ops some time to respond, rather than
	**	pushing out another alloc right away
	**	TODO - maybe some volman errs are permanent,
	**	e.g. media not in RC, so will need to mark slot 
	**	as broken for alloc
	*/
	cip->vc_allocwait[vrp->vr_writeslot] = Now + 600;
nLogMsg(EL_STATUS1, id, "Now %d wait %d  ", Now, cip->vc_allocwait[vrp->vr_writeslot]);

	/*
	**	note that vsn not defined
	*/
	hvp->hv_status = HV_NOVSN;

	/*
	**	delete vrp and ref to it
	*/
	DelVsnReq(vrp);
	hvp->hv_vrp = NULL;

}


/************************************************************************
	routines for deciding to start mounts & dismount idle mounts
 ************************************************************************/

/*
**	MountHotVsn() - start a mount in a given slot
**		of any panel in the class for writing vv's to.  
*/
  static void
MountHotVsn(VVMClass_t *cip, int slot)
{
	char		*id = "MountHotVsn";
	HotPanel_t	*hp;
	int		panel, okpanel = -1, okstatus = VSQ_NONE;

	if (slot < 0  ||  slot >= cip->vc_class.cs_nvsns)
		nLogMsg(EL_FATAL, id, "bad slot %d for class %s", 
					slot, cip->vc_class.cs_prefix);

	if (volman_sock == -1)
		return;

	/*
	**	look at this slot on each hot panel;
	**	use first available slot.
	*/
	hp = cip->vc_hotpanel;
	for (panel=0; panel<cip->vc_npanels; panel++, hp++) {
		HotVsn_t	*hvp = &hp->hp_vsns[slot];

		/*
		**	hotvsn must be defined and not
		**	already mounting/mounted
		*/
		if (hvp->hv_status == HV_VOLALLOC) {
nLogMsg(EL_STATUS1, id, "skip volalloc");
			continue;
}
		if (hvp->hv_status == HV_NOVSN) {
			/*
			**	try to alloc; if one
			**	comes from dbase, ok
			*/
nLogMsg(EL_STATUS1, id, "get hot vsn slot %d panel %d", slot, panel);
			if (!AssignHotVsn(cip, slot, hvp))
				continue;
		}
		if (hvp->hv_status != HV_ALLOC)
			nLogMsg(EL_FATAL, id, "unexpected status %d", 
							hvp->hv_status);

		if (hvp->hv_vrp != NULL) {
nLogMsg(EL_STATUS1, id, "vrp!=NULL");
			continue;
		}

		/*
		**	see if wait time set after 
		**	prev mount failure
		*/
		if (hvp->hv_wait  &&  hvp->hv_wait < Now) {
			nLogMsg(EL_STATUS2, id, "waiting on retry");
			continue;
		}
nLogMsg(EL_STATUS1, id, "mount slot %d", slot);
		/*
		**	all criteria met; reset wait time & start mount
		*/
		hvp->hv_wait = 0;
		hvp->hv_vrp = VolmanMount1(hvp->hv_vsnrec.vs_vsn,
						(void *)hvp, cip, slot);
		return;
	}

	/*
	**	request will have to wait
	*/
	nLogMsg(EL_STATUS2, id, "no panel/slot avail");
}

/*
**	UnmountHotVsns() - unmount hot vsns that have been idle
*/
  static void
UnmountHotVsns(VVMClass_t *cip, int slot)
{
	char		*id = "UnmountHotVsns";
	HotPanel_t	*hp;
	int		panel, okpanel = -1, okstatus = VSQ_NONE;

	if (slot < 0  ||  slot >= cip->vc_class.cs_nvsns)
		nLogMsg(EL_FATAL, id, "bad slot %d for class %s", 
					slot, cip->vc_class.cs_prefix);

	if (volman_sock == -1)
		return;

	/*
	**	look at this slot on each hot panel
	*/
	hp = cip->vc_hotpanel;
	for (panel=0; panel<cip->vc_npanels; panel++, hp++) {
		HotVsn_t	*hvp = &hp->hp_vsns[slot];

		/*
		**	hotvsn must be defined and mounted
		*/
		if (hvp->hv_status != HV_ALLOC)
			continue;
		if (hvp->hv_vrp == NULL)
			continue;

		/*
		**	leave alone if read is using it
		*/
		if (hvp->hv_vrp->vr_waitq.q_next != NULL)
			continue;

		/*
		**	consider timeout
		*/
		if (hvp->hv_vrp->vr_time > Now)
			continue;

		/*
		**	dismount
		*/
		VolmanUnmount(hvp->hv_vrp, id);
		DelVsnWriteReq(hvp->hv_vrp);
		hvp->hv_vrp = NULL;
	}
}


/*
**	CheckSlotMount() - start a mount or dismount 
**		if situation warrants
*/
  static void
CheckSlotMount(VVMClass_t *cip, int slot, int urgent)
{
	char		*id = "CheckSlotMount";
	WriteReq_t	*wrp;
	int		panel;

	/*
	**	if nothing waiting, done
	*/
	if (cip->vc_waitct[slot] == 0) {
		/*
		**	if nothing active & something mounted, 
		**		consider dismounting
		*/
		if (cip->vc_activect[slot] == 0  &&  hot_mounted)
			UnmountHotVsns(cip, slot);
		return;
	}

	/*
	**	skip if no panel is idle in this slot
	*/
	if (cip->vc_activect[slot] == cip->vc_npanels) {
nLogMsg(EL_STATUS1, id, "%d - all panels busy", slot);
		return;
	}

	/*
	**	nervous check that 1st req is waiting idly
	*/
	wrp = CWRSLOTHEAD(cip, slot);
	if (wrp == NULL)
		nLogMsg(EL_FATAL, id, "no wrp");
	if (wrp->wr_statpanel[slot] != WR_STWAIT) {
	/*%%%
		nLogMsg(EL_FATAL, id, "req not waiting");
	*/
		nLogMsg(EL_ERROR1, id, "%s slot %d req not waiting (%d)" ,
			VV2Name(&wrp->wr_vv.v_vv),
			slot, wrp->wr_statpanel[slot] );
		if (wrp->wr_statpanel[slot] > 1)
			nLogMsg(EL_FATAL, id, "statpanel overflow");
		return;
	}
/* could see if mounted & apply waiting req; shouldn't
be necc %%% */

	/*
	**	skip if not enough VV's waiting long enough
	*/
	if (cip->vc_waitct[slot] < WR_SLOT_BATCH  &&  !urgent) {
		if (Now - wrp->wr_start < WR_MAX_WAIT) {
			return;
		}
	}

	/*
	**	if panel(s) are already active on
	**	this slot, see if backlog is enough
	**	to start another panel
	*/
	if (cip->vc_activect[slot]  &&  !urgent) {
		int	waitpanel = cip->vc_waitct[slot] / 
					cip->vc_activect[slot];

		/*
		**	criteria are a bit high
		**	for starting a new panel
		*/
		if (waitpanel < WR_SLOT_BATCH * cip->vc_npanels)
			return;
	}
nLogMsg(EL_STATUS1, id, 
"slot %d  cip->vc_waitct  %d / %d sec;  vc_activect  %d  hot_mounted %d", 
slot, cip->vc_waitct[slot], Now - wrp->wr_start, cip->vc_activect[slot],
hot_mounted);

	/*
	**	start mount
	*/
	MountHotVsn(cip, slot);
}

/*
**	CWCheckQs() - this is a general checker routine
**		for starting mounts - which shouldn't be 
**		needed in the end.
*/
  void
CWCheckQs(int urgent)
{
	char		*id = "CWCheckQs";
	VVMClass_t	*cip = ClassArray;
	int		i, panel, slot;

	if (volman_sock == -1)
		return;

	for (i=0; i<ClassMax; i++, cip++) {
		int		nvsns = cip->vc_class.cs_nvsns;

		for (slot=0; slot<nvsns; slot++)
			CheckSlotMount(cip, slot, urgent);
	}
}

CWIdleQs()
{
	int		class, panel, slot, idle = 0;
	VVMClass_t	*cip = ClassArray;

	for (class=0; class<ClassMax; class++, cip++) {
		int	nvsns = cip->vc_class.cs_nvsns;

		for (slot=0; slot<nvsns; slot++)
			if (cip->vc_waitct[slot] < 5)
				idle++;
	}
	return(idle);
}

/************************************************************************
	routines for hooking hot mounts to waiting write (or read) reqs 
 ************************************************************************/

/*
**	HandoffVrpRead() - give hot mount to waiting read, if any
*/
  static int
HandoffVrpRead(VsnReq_t *vrp)
{
	char		*id = "HandoffVrpRead";
	VsnReq_t	*vrp2 = (VsnReq_t *)vrp->vr_waitq.q_next;

	if (vrp2 == NULL)
		return(0);

	nLogMsg(EL_STATUS2, id, "read waiting");
	vrp->vr_status = VSQ_WAIT;
	vrp2->vr_status = VSQ_MOUNTED;
	vrp2->vr_hostp = vrp->vr_hostp;
	vrp2->vr_bufsize = vrp->vr_bufsize;
	CRMoverRead(vrp2);
	return(1);
}

  static void
AssignSlotPanel(VVMClass_t *cip, int slot, int panel, WriteReq_t *wrp)
{
	HotVsn_t	*hvp = &cip->vc_hotpanel[panel].hp_vsns[slot];

	/*
	**	point HotVsn_t at WriteReq_t
	*/
	hvp->hv_writerp = wrp;

	/*
	**	point WriteReq_t at HotVsn_t
	*/
	wrp->wr_statpanel[slot] = panel;

	/*
	**	note that the panel has been used recently
	*/
	cip->vc_hotpanel[panel].hp_lastuse = Now;
}

/*
**	MoverWrite() - send request to mover
*/
  static void
MoverWrite(WriteReq_t *wrp, int slot)
{
	char		*id = "MoverWrite";
	VVMClass_t	*cip = (VVMClass_t *) wrp->wr_cp;
	int		panel = wrp->wr_statpanel[slot];
	HotVsn_t	*hvp = &cip->vc_hotpanel[panel].hp_vsns[slot];
	VsnReq_t	*vrp = hvp->hv_vrp;
	char		*vsnp = vrp->vr_vsns+VSNLEN*vrp->vr_writeslot;
	char		buf[VV_PKTSIZ(vvpMvreq_t)];
	vvSeq_t		seq = vvGetSeq();
	vvpHdr_t	*hp;
	vvpMvreq_t	*mvp;
	int		comslot;

	nLogMsg(EL_STATUS1, id, " %s slot%d %.*s to mover %s, rec %d, blk %ld",
			VV2Name(&wrp->wr_vv.v_vv), slot, VSNLEN, vsnp,
			vrp->vr_hostp->h_name,
			hvp->hv_vsnrec.vs_lastrec, hvp->hv_vsnrec.vs_lastblk);


	comslot = HostMover(vrp->vr_hostp);
	if (comslot == -1) {
		nLogMsg(EL_STATUS1, id, "NO MOVER (mover or its host down)");
		return;
	}

	/*
	**	build pkt for mover
	*/
	hp = vvBuildHdr(VVM_WTAPE, seq, (vvpHdr_t *)buf);
	mvp = VV_MSGPTR(hp, vvpMvreq_t);

	memcpy(mvp->m_vsn, vsnp, sizeof(mvp->m_vsn));
	mvp->m_checkvol1 = 0;	/* TODO - think! */

	/* previous record on vsn to check for write */
	mvp->m_block = hvp->hv_vsnrec.vs_lastblk;
	mvp->m_rec = hvp->hv_vsnrec.vs_lastrec;
	mvp->m_lastvv = hvp->hv_vsnrec.vs_lastvv;

	/* hdr info for this vv */
	mvp->m_vv = wrp->wr_vv.v_vv;
	memcpy(mvp->m_id, wrp->wr_vv.v_id, sizeof(mvp->m_id));
	mvp->m_creat = wrp->wr_vv.v_creat;
	mvp->m_lastwrite = wrp->wr_vv.v_write;
	mvp->m_nwrite = wrp->wr_vv.v_nwrite;
	mvp->m_finish = wrp->wr_vv.v_finish;
	mvp->m_slot = vrp->vr_writeslot;
	mvp->m_nvsns = wrp->wr_vv.v_nvsns;
	memcpy(mvp->m_fsize, wrp->wr_vv.v_data, sizeof(mvp->m_fsize));

	/* useful tape info */
	mvp->m_bufsize = vrp->vr_bufsize;
if (vrp->vr_bufsize < 1)
nLogMsg(EL_FATAL, id, "bufsize %d", vrp->vr_bufsize);

	/*
	**	send pkt
	*/
	if (XDRSendI(hp, comslot, 1))
		return;

	/*
	**	mark status
	*/
	vrp->vr_mvseq = seq;
	vrp->vr_status = VSQ_MOVER;
	vrp->vr_time = Now;
	cip->vc_activect[slot]++;
}

/*
**	CWMoverWrite() - vestigial code, shouldn't happen if
**		WriteReq_t's are only assigned to vsn's with
**		movers.
*/
  void
CWMoverWrite(VsnReq_t *vrp)
{
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;
	int		slot = hvp->hv_slot;
	WriteReq_t	*wrp = hvp->hv_writerp;

	nLogMsg(EL_STATUS1, "CWMoverWrite", "HUH???? how about that?");
	if (wrp == NULL)
		return;

	MoverWrite(wrp, slot);
}

/*
**	CWHandoffHotMount() - hot vsn mount available, so hand off to
**		any waiting read or write req
*/
  void
CWHandoffHotMount(VsnReq_t *vrp)
{
	char		*id = "CWHandoffHotMount";
	VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;
	int		slot = hvp->hv_slot;
	int		panel = hvp->hv_panel;
	WriteReq_t	*wrp = hvp->hv_writerp;
	VsnReq_t	*readreq = (VsnReq_t *) vrp->vr_waitq.q_next;

	if (slot != vrp->vr_writeslot)
		nLogMsg(EL_FATAL, id, "slot mismatch");
	if (wrp != NULL) {
		nLogMsg(EL_STATUS1, id, 
				"%.*s: wrp exists, VV %s, maybe remounted vsn", 
				VSNLEN, vrp->vr_vsns,
				VV2Name(&wrp->wr_vv.v_vv));
		if (vrp->vr_status != VSQ_MOVER  &&  
		    vrp->vr_status != VSQ_MOUNTED)
			nLogMsg(EL_FATAL, id, "status not mover/mounted: %s",
						vvVsnStatus(vrp->vr_status));
		/*
		**	presumably req is waiting on mover
		**		that died or hung? Found this
		**		happening when mvr err pkt somehow
		**		got seq=-1 so vrp couldn't be found.
		**		When remount happened later & this
		**		routine called, wrp!=NULL was handled as fatal.
		*/
		MoverWrite(wrp, slot);
		return;
	}

	/*
	**	make sure mover exists
	*/
	if (vrp->vr_hostp == NULL)
		nLogMsg(EL_FATAL, id, "%.*s: mounted but no host defined",
					VSNLEN, vrp->vr_vsns);
	if (HostMover(vrp->vr_hostp) == -1) {
		/*
		**	let any proc that needs a vsn initiate mount
		*/
		VolmanUnmount(vrp, id);
		if (readreq) {
			vrp->vr_waitq.q_next = NULL;
			readreq->vr_waitq.q_prev = NULL;
			CRFailQdReads(readreq);
		}
		DelVsnWriteReq(vrp);
		hvp->hv_vrp = NULL;
		return;
	}

	/*
	**	read takes precedence (probably a rare case,
	**	especially right after initial mount)
	*/
	if (HandoffVrpRead(vrp)) 
		return;

	/*
	**	if nothing waiting, leave vsn waiting
	*/
	wrp = CWRSLOTHEAD(cip, slot);
	if (wrp == NULL) {
		/*
		**	leave vsn request in 'mounted' state to time
		**	out & dismount or be used, as the case may be
		*/
		return;
	}

	/*
	**	take waiting WriteReq_t off slot q, 
	**	connect to HotVsn_t, and send req to mover
	*/
	CWRSLOTDEQ(wrp, slot, cip);
	AssignSlotPanel(cip, slot, panel, wrp);
	MoverWrite(wrp, slot);
}

/*
**	CWMountFailed() - decide what to do w/ vsn & whether
**		to start another mount.
*/
  void
CWMountFailed(VsnReq_t *vrp, int level)
{
	char		*id = "CWMountFailed";
	VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;
	int		slot = vrp->vr_writeslot;
	char		*vsnp = vrp->vr_vsns + vrp->vr_writeslot * VSNLEN;
	int		retry = 1;

	if (hvp->hv_writerp != NULL)
		nLogMsg(EL_FATAL, id, "hv_writerp not NULL");

	/*
	**	take care of any q'd read reqs
	*/
	if (vrp->vr_waitq.q_next) {
		VsnReq_t	*readreq = (VsnReq_t *) vrp->vr_waitq.q_next;

		vrp->vr_waitq.q_next = NULL;
		readreq->vr_waitq.q_prev = NULL;
		CRFailQdReads(readreq);
	}

	/*
	**	now that all info is copied out of the VsnReq_t,
	**	delete it
	*/
	DelVsnWriteReq(vrp);

	/*
	**	if the current vsn is no good, get a new one in
	**	the same place
	*/
	if (level) {
		/*
		**	mark problem in dbase and forget this vsn
		*/
		DBVsnErr(&hvp->hv_vsnrec);
		hvp->hv_status = HV_NOVSN;

		/*
		**	get a replacement hot vsn
		*/
		retry = AssignHotVsn(cip, slot, hvp);
	}
	if (retry) {
		/*
		**	mount another if necc
		*/
		CheckSlotMount(cip, slot, 0);
	}
}

/*
**	CWMountCanceled() - outside agency apparently blew away the mount
*/
void
CWMountCanceled(VsnReq_t *vrp)
{
	char		*id = "CWMountCanceled";
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;
	VsnReq_t	*readreq = (VsnReq_t *) vrp->vr_waitq.q_next;

	/*
	**	take care of any q'd read reqs
	*/
	if (readreq) {
		vrp->vr_waitq.q_next = NULL;
		readreq->vr_waitq.q_prev = NULL;
		CRFailQdReads(readreq);
	}

	if (hvp->hv_writerp != NULL) {
		/*
		**	let current writer clean up - maybe it can
		**		finish its job..
		*/
		return;
	}

	/*
	**	see what happens with treating 
	**		it as a simple mount failure..
	*/
	nLogMsg(EL_STATUS1, id, "no writer, cleaning up..");
	CWMountFailed(vrp, 0);
}

/************************************************************************
	routines for handling VV write reqs: 
		setup
		grab idle hot mount or wait
		mover write (send/receive)
		vsn handoff
 ************************************************************************/

/*
**	GetHotMountedVsn() - get idle mounted vsn in given slot
**		of any panel in the class for writing a vv to.  
*/
  static int
GetHotMountedVsn(VVMClass_t *cip, int slot)
{
	char		*id = "GetHotMountedVsn";
	HotPanel_t	*hp;
	int		panel, okpanel = -1, okstatus = VSQ_NONE;

	if (slot < 0  ||  slot >= cip->vc_class.cs_nvsns)
		nLogMsg(EL_FATAL, id, "bad slot %d for class %s", 
					slot, cip->vc_class.cs_prefix);

	nLogMsg(EL_DEBUG2, id, "slot %d", slot);
	/*
	**	look at this slot on each hot panel;
	**	use first available slot.
	*/
	hp = cip->vc_hotpanel;
	for (panel=0; panel<cip->vc_npanels; panel++, hp++) {
		HotVsn_t	*hvp = &hp->hp_vsns[slot];

		/*
		**	needs to be allocated (superfluous
		**	test, but follows alloc/mount logic
		**	flow to make all look consistent)
		*/
		if (hvp->hv_status != HV_ALLOC)
			continue;

		/*
		**	needs to be mounted
		*/
		if (hvp->hv_vrp == NULL) {
			nLogMsg(EL_DEBUG2, id, "  no vrp");
			continue;
		}
		if (hvp->hv_vrp->vr_status != VSQ_MOUNTED) {
			nLogMsg(EL_DEBUG2, id, "  [status %s]", 
					vvVsnStatus(hvp->hv_vrp->vr_status));
			continue;
		}

		/*
		**	mount needs to be idle -
		**	skip write-in-progress
		*/
		if (hvp->hv_writerp != NULL) {
			nLogMsg(EL_DEBUG2, id, "  skip write-in-progress %s",
					VV2Name(&hvp->hv_writerp->wr_vv.v_vv));
			continue;
		}

		/*
		**	mount needs a mover on drive host
		*/
		if (hvp->hv_vrp->vr_hostp == NULL)
			nLogMsg(EL_FATAL, id, 
					"%.*s: mounted but no host defined",
					VSNLEN, hvp->hv_vrp->vr_vsns);
		if (HostMover(hvp->hv_vrp->vr_hostp) == -1) {
			nLogMsg(EL_STATUS1, id, "  no mover - COMPLAIN??");
			continue;
		}

		/*
		**	all criteria met
		*/
		return(panel);
	}

	/*
	**	request will have to wait
	*/
	nLogMsg(EL_DEBUG2, id, "no panel/slot avail");
	return(-1);
}





/*
**  	CWriteVV2Vsns() - a vv has been closed to future writing,
**		so start the process of writing it to a vsn in each
**		slot of the class. This is the entry point for the
**		process of writing VV's.
*/
#define _MSGSZ 100
  void
CWriteVV2Vsns(VVInfo_t *vvp)
{
	char		*id = "CWriteVV2Vsns";
	VVMClass_t	*cip;
	WriteReq_t	*wrp;
	int		slot, vsnsdone = 0;
	char		*cp, msg[_MSGSZ];

	/*
	**	get class info
	*/
	cip = VVMClassInfo(vvp->v_vv.class);
	if (cip == NULL)
		nLogMsg(EL_CONSOLE0|EL_FATAL, id, "can't look up class %s", 
						ClassString(vvp->v_vv.class));

	/*
	**	build request & enqueue it in WriteReq_t master q
	*/
	wrp = (WriteReq_t *) nAlloc(sizeof(WriteReq_t));
	wrp->wr_cp = (void *) cip;
	wrp->wr_start = Now;
	CWRITEENQ(wrp, cip);

	/*
	**	copy db vv info
	*/
	wrp->wr_vv = *vvp;

if (wrp->wr_vv.v_nvsns > 2)
nLogMsg(EL_FATAL, id, "1 v_nvsns > 2");

	/*
	**	init slot statuses: waiting to launch save
	*/
	for (slot=0; slot<MAXVSN; slot++) {
		wrp->wr_statpanel[slot] = WR_STWAIT;
		wrp->wr_sloterrs[slot] = 0;
		wrp->wr_slotq[slot].q_next = NULL;
		wrp->wr_slotq[slot].q_prev = NULL;
	}

	/*
	**	for each slot that hasn't been written to vsn per dbase,
	**	put in class slot q and start if possible
	*/
if (wrp->wr_vv.v_nvsns > 2)
nLogMsg(EL_FATAL, id, "v_nvsns > 2");

	msg[0] = '\0';
	cp = msg;
	for (slot=0; slot<wrp->wr_vv.v_nvsns; slot++) {
		int		panel;
		HotVsn_t	*hvp;

		if (isalnum(vvp->v_vsns[slot].v_vsn[0])) {
			/*
			**	this slot already saved
			*/
			(void)sprintf(cp, "(slot %d: %.*s) ", slot, VSNLEN, 
						vvp->v_vsns[slot].v_vsn);
			cp += strlen(cp);
if (cp > msg + _MSGSZ)
nLogMsg(EL_FATAL, id, "msg too long 1");
			vsnsdone++;
			wrp->wr_statpanel[slot] = WR_STDONE;
			continue;
		}
		(void)sprintf(cp, "Q slot %d ", slot);
		cp += strlen(cp);

if (cp > msg + _MSGSZ)
nLogMsg(EL_FATAL, id, "msg too long 2");
		/*
		**	see if a hot vsn is available
		*/
		panel = GetHotMountedVsn(cip, slot);
		if (panel < 0) {
			/*
			**	queue this slot 
			*/
			CWRSLOTENQ(wrp, slot, cip);
			continue;
		}

		/*
		**	start write to hot vsn
		*/
		AssignSlotPanel(cip, slot, panel, wrp);
		MoverWrite(wrp, slot);
	}
	nLogMsg(EL_STATUS1, id, "vv %s: %s", VV2Name(&vvp->v_vv), msg);
	if (vsnsdone == wrp->wr_vv.v_nvsns) {
		/* maybe ok & just need to update dbase, but let's see.. */
		nLogMsg(EL_FATAL, id, "trying to start done req %d==%d",
						vsnsdone, wrp->wr_vv.v_nvsns);
	}
}




/*
**	CWMoverOk() - mover did its job
*/
  void
CWMoverOk(VsnReq_t *vrp, int rec, unsigned int blk)
{
	char		*id = "CWMoverOk";
	int		slot = vrp->vr_writeslot;
	VVMClass_t	*cip = (VVMClass_t *)vrp->vr_cp;
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;
	WriteReq_t	*wrp2, *wrp = hvp->hv_writerp;
	char		*vsnp = vrp->vr_vsns + slot * VSNLEN;
	VsnReq_t	*vrp2 = (VsnReq_t *)vrp->vr_waitq.q_next;
	int		i, dbdone, failed = 0;
	char		*vvname = VV2Name(&wrp->wr_vv.v_vv);

	if (vrp->vr_status == VSQ_UNMOUNTED) {
		/*
		**	vsn was taken away.. should be able to delete;
		**	maybe will need to count errs & decide to get
		**	a new hotvsn
		*/
		DelVsnWriteReq(vrp);
		hvp->hv_vrp = NULL;
		hvp->hv_status = HV_ALLOC;
		vrp = NULL;
	} else {

		/*
		**	update time for idle unmount
		*/
		vrp->vr_time = Now;
		vrp->vr_status = VSQ_MOUNTED;
	}

	cip->vc_activect[slot]--;

	/*
	**	detach this VV request from hot panel/slot
	*/
	hvp->hv_writerp = NULL;
	wrp->wr_statpanel[slot] = WR_STDONE;

	/*
	**	update VV & vsn records, save to dbase
	*/
	hvp->hv_vsnrec.vs_lastvv = wrp->wr_vv.v_vv;
	memcpy(hvp->hv_vsnrec.vs_lastid, wrp->wr_vv.v_id, 
					sizeof(hvp->hv_vsnrec.vs_lastid));
	hvp->hv_vsnrec.vs_lastrec = rec;
	hvp->hv_vsnrec.vs_lastblk = blk;
	hvp->hv_vsnrec.vs_livevv++;

	memcpy(wrp->wr_vv.v_vsns[slot].v_vsn, vsnp, (size_t)VSNLEN);
	wrp->wr_vv.v_vsns[slot].v_nrec = rec;
	wrp->wr_vv.v_vsns[slot].v_block = blk;
	wrp->wr_vv.v_vsns[slot].v_date = Now;

	dbdone = DBSavedVV(&hvp->hv_vsnrec, &wrp->wr_vv, slot, cip);

	/*
	**	see if all slots finished/failed
	*/
	for (i=0; i<wrp->wr_vv.v_nvsns; i++) {
nLogMsg(EL_STATUS1, id, " statpan %d: %d", i, wrp->wr_statpanel[i]);
		if (wrp->wr_statpanel[i] > -1) /* in progress */
			break;
		if (wrp->wr_statpanel[i] == WR_STWAIT)
			break;
		if (wrp->wr_statpanel[i] == WR_STFAILED)
			failed++;
	}
	if (i == wrp->wr_vv.v_nvsns) {
nLogMsg(EL_STATUS1, id, "all done");
		if (!dbdone) {
			if (failed)
				nLogMsg(EL_ERROR1, id, 
					"leaving %s to next vvmd due to err(s)",
					vvname);
			else
				nLogMsg(EL_ERROR1, id, 
					"%s: all done, but not in db!",
					vvname);
		} else if (cip->vc_class.cs_flags & CS_FREEDISK) {
			nLogMsg(EL_STATUS1, id, "%s saved/freedisk", vvname);
			(void)DiskRecycle(&wrp->wr_vv.v_vv);
		} else
			nLogMsg(EL_STATUS1, id, "%s saved", vvname);

		/*
		**	done - clean up request
		**	(if part failed, let next vvmd worry
		**	about completing it via 'tp' index)
		*/
		CWRITEDEQ(wrp, (VVMClass_t *)wrp->wr_cp);
		nFree(wrp);
	}

	if (vrp) {
		/*
		**	give mount away (or leave idle to time out)
		*/
		CWHandoffHotMount(vrp);
	}
}


/*
**	CWMoverErr() - problem with tape, tape drive, or driver
*/
  void
CWMoverErr(VsnReq_t *vrp, int vverr, char *msg)
{
	char		*id = "CWMoverErr";
	int		slot = vrp->vr_writeslot;
	VVMClass_t	*cip = (VVMClass_t *)vrp->vr_cp;
	HotVsn_t	*hvp = (HotVsn_t *) vrp->vr_caller;
	WriteReq_t	*wrp = hvp->hv_writerp;
	VsnReq_t	*vrp2 = (VsnReq_t *) vrp->vr_waitq.q_next;
	int		retryvv = 0, delreq = 0, hotok = 0;

	if (vrp->vr_status == VSQ_UNMOUNTED) {
		if (vrp2 != NULL) 
			nLogMsg(EL_FATAL, id, "q'd read on unmounted vsn");

		/*
		**	vsn was taken away.. should be able to delete;
		**	maybe will need to count errs & decide to get
		**	a new hotvsn
		*/
		DelVsnWriteReq(vrp);
		hvp->hv_vrp = NULL;
		hvp->hv_status = HV_ALLOC;
		vrp = NULL;
	} else {
		vrp->vr_status = VSQ_MOUNTED;
		vrp->vr_time = Now;
	}

	if (slot != hvp->hv_slot)
		nLogMsg(EL_FATAL, id, "slot mismatch");

	if (wrp == NULL)
		nLogMsg(EL_FATAL, id, "wrp null");

	/*
	**	there are two aspects to an err in writing the tape: 
	**
	**	1. What about the tape? should we mark it 'full'?
	**		- because it doesn't have space 
	**		- because it can't be trusted with more data
	**		  (or maybe it should even be replaced)
	**
	**	2. What about the request? Can we write it to another tape?
	**		- how many tapes to try before concluding
	**		  there's a chronic system bug?
	**		- how often to retry?
	**
	**	The basic thing to keep in mind, based on many observations,
	**	is that operators will not notice a problem for timescales 
	**	of at least 12-24 hours.
	*/

	/*
	**      remove WriteReq_t from HotVsn_t and update 
	**		class/slot active count
	*/
	hvp->hv_writerp = NULL;
	cip->vc_activect[slot]--;

	/*
	**	remove HotVsn_t from WriteReq_t by
	**	marking slot/panel 'failed' for now
	*/
	wrp->wr_statpanel[slot] = WR_STFAILED;

	switch (vverr) {
	    case VVE_FILE:
	    case VVE_INTERNAL:
		/*
		**	problem with the VV itself (so tape can
		**	remain hot-in-good-standing)
		*/
nLogMsg(EL_STATUS1, id, "marking vv slot failed");
		hotok = 1;

		/*
		**	handle the write request
		*/
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"VV %s MAY BE BAD!",
						VV2Name(&wrp->wr_vv.v_vv));

		/*
		**	see if all slots for VV are dead yet
		*/
		for (slot=0; slot<wrp->wr_vv.v_nvsns; slot++)
			if (wrp->wr_statpanel[slot] != WR_STFAILED)
				break;
		if (slot == wrp->wr_vv.v_nvsns) {
			/*
			**  forget VV for the life of this vvmd
			*/
			CWRITEDEQ(wrp, (VVMClass_t *)wrp->wr_cp);
			nFree(wrp);
		}
		break;

	    case VVE_TAPE:
nLogMsg(EL_STATUS1, id, "VVE_TAPE - setting not hot");
		/*
		**	vv can be retried
		*/
		retryvv = 1;
		/*
		**	never write tape again & flag it as a problem tape
		*/
		if (vrp)
			(void)DBSetVsnNotHot(&hvp->hv_vsnrec, msg);
		/*
		 *  TODO - judge whether it's relatively 'empty'
		 *	and log the judgement; maybe log err in a
		 *	special file.
		 */
		break;

	    case VVE_SYS:
	    case VVE_HARDWARE:
		nLogMsg(EL_STATUS1, id, "(system err)");
		/*
		**	vv can be retried
		*/
		retryvv = 1;
		/*
		**	update/check class err count
		*/
		if (++cip->vc_msloterrs[slot] >= MAX_SLOT_ERRS) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
				  "Class %s slot %d: %d sys errs in a row",
				  cip->vc_class.cs_name, slot,
				  cip->vc_msloterrs[slot]);
		}
		break;

	    case VVE_NOTFOUND:
		/*
		**	if volman's volnd was restarted for
		**	some reason, the /dev/vol/xxxxxx
		**	node will have been deleted;
		**	simple approach is to remount or
		**	push this slot queue.
		*/
		nLogMsg(EL_STATUS1, id, "vsn node not found - remounting");
		if (vrp)
			VolmanRemount(vrp);
		else
			CheckSlotMount(cip, slot, 0);
		return;

	    default:
		nLogMsg(EL_FATAL, id, "add err case for err code %d: %s", 
					vverr, vvErrStr(vverr));
	}

	/*
	**	no matter what the problem, existing VV
	**	on the tape may be readable, so hand tape 
	**	off to waiting read if any
	*/
	if (vrp2) {
		if (vrp == NULL)
			nLogMsg(EL_FATAL, id, "unmounted vsn w/ read q'd..");

		/*
		**	handle hot request queueing
		*/
		if (hotok) {
			/*
			**	tape will be returned to hot
			**	panel when reader is done,
			**	so mark write vrp as waiting
			*/
			vrp->vr_status = VSQ_WAIT;
		} else {
			/*
			**	tape no longer wanted for write,
			**	so detach read vrp from it
			*/
			vrp2->vr_waitq.q_prev = NULL;
			vrp->vr_waitq.q_next = NULL;
			delreq = 1;
		}

		/*
		**	start off the read
		*/
		(void)HandoffVrpRead(vrp);

	} else if (!hotok  &&  vrp != NULL) {
		/*
		**	tape presumed to be useless as mounted
		**	(i.e. for now if not forever)
		*/
		VolmanUnmount(vrp, id);
		DelVsnWriteReq(hvp->hv_vrp);
		hvp->hv_vrp = NULL;
		/*
		**	get another tape for the slot
		*/
		hvp->hv_status = HV_NOVSN;
		AssignHotVsn(cip, slot, hvp);
	}

	/*
	**	push VV.slot write to other tape if appropriate
	*/
	if (retryvv) {
		/*
		**	see if a hot vsn is available
		*/
		int	panel = GetHotMountedVsn(cip, slot);

		wrp->wr_statpanel[slot] = WR_STWAIT;
		if (panel < 0) {
			/*
			**	queue this slot 
			*/
			CWRSLOTENQ(wrp, slot, cip);
		} else {

			/*
			**	start write to (another?) hot vsn
			*/
			AssignSlotPanel(cip, slot, panel, wrp);
			MoverWrite(wrp, slot);
		}
	}
	if (delreq)
		DelVsnWriteReq(vrp);
}



/************************************************************************/
/************************************************************************/


/*
**	CWVolmanRestart() - vold is back up, so see if allocs or
**		mounts are called for.
*/
  void
CWVolmanRestart()
{
	char		*id = "CWVolmanRestart";
	VVMClass_t	*cip = ClassArray;
	int		i, panel, slot;

	/*
	**	check for allocs
	*/
	for (i=0; i<ClassMax; i++, cip++) {
		int		npanels = cip->vc_npanels;
		int		nvsns = cip->vc_class.cs_nvsns;
		HotPanel_t	*hp = cip->vc_hotpanel;

		/*
		**	check panels
		*/
		for (panel=0; panel<npanels; panel++, hp++) {
			HotVsn_t	*hvp;

			hvp = hp->hp_vsns;
			for (slot=0; slot<nvsns; slot++, hvp++) {

				if (hvp->hv_status == HV_ALLOC)
					continue;

				(void)AssignHotVsn(cip, slot, hvp);
			}

		}
	}

	/*
	**	start any mounts needed
	*/
	CWCheckQs(0);
}


/*
**	CWMountHotRead() - if a hot vsn is needed for read,
**		provide it in mounting form if possible. 
**		Caller should check actual mounts 1st.
*/
  VsnReq_t *
CWMountHotRead(int nvsns, char *vsnp)
{
	char		*id = "CWMountHotRead";
	VVMClass_t	*cip = ClassArray;
	int		i, panel, slot;

	/*
	**	check for unmounted allocs
	*/
	for (i=0; i<ClassMax; i++, cip++) {
		int		npanels = cip->vc_npanels;
		int		c_nvsns = cip->vc_class.cs_nvsns;
		HotPanel_t	*hp = cip->vc_hotpanel;

		/*
		**	check panels
		*/
		for (panel=0; panel<npanels; panel++, hp++) {
			HotVsn_t	*hvp;

			hvp = hp->hp_vsns;
			for (slot=0; slot<c_nvsns; slot++, hvp++) {
				char	*hvsnp = hvp->hv_vsnrec.vs_vsn;
				int	i;

				if (hvp->hv_status != HV_ALLOC)
					continue;
				if (hvp->hv_vrp != NULL)
					continue;

				for (i=0; i<nvsns; i++) {
					if (!memcmp(vsnp+i*VSNLEN, hvsnp, 
							(size_t)VSNLEN))
						break;
				}
				if (i == nvsns)
					continue;

				/*
				**	got a match; start mount
				*/
				hvp->hv_wait = 0;
				hvp->hv_vrp = VolmanMount1(hvsnp,
							(void *)hvp, cip, slot);
				return(hvp->hv_vrp);
			}

		}
	}
	return(NULL);
}
