
/*
 *  vvm_db0.c - NAStore Virtual Volume Manager database routines
 *
 *	vvm_db0 is for routines that are used by vvm db utilities
 *	as well as vvmd - routines which do not reference other
 *	vvmd stuff. All such programs should be read-only w/
 *	respect to the dbase.
 *	
 *	Bill Ross, NASA Ames, 1997-9
 */

#include "vv_space.h"
#include "vvm_db.h"

/*
**	basic table file names, file descriptors, 
**		offsets, read/write buffers, binary equiv structs
*/

char		dbf_vv_tbl[MAXPATHLEN];
int		db_vv_fd = -1;
OFF_T		vv_addr;
vbuf_t		vbuf;
ClassSlot_t	*cslots[4];		/* points into vbuf */
VVInfo_t	VInfo;

char		dbf_vsn_tbl[MAXPATHLEN];
int		db_vsn_fd = -1;
OFF_T		vsn_addr;
vsbuf_t		vsnbuf;
Vsn_t		VsnInfo;


/*************************************************************************
			TABLE MANIPULATION
 *************************************************************************/

  int
Vbuf2VInfo(char *id)
{
	int	i;

	/*
	 *  check the record
	 */
	if (vbuf.end != '\n') {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
			"index error - bad record terminator");
		return(TBL_ERR);
	}
	if (vbuf.b01 != ' '  ||  vbuf.b02 != ' '  ||  vbuf.b03 != ' '  ||
	    vbuf.b04 != ' '  ||  vbuf.b05 != ' '  ||  vbuf.b06 != ' '  ||
	    vbuf.b07 != ' '  ||  vbuf.b08 != ' '  ||  vbuf.b09 != ' '  ||
	    vbuf.b10 != ' '  ||  vbuf.b11 != ' '  ||  vbuf.b12 != ' '  ||
	    vbuf.b13 != ' '  ||  vbuf.b14 != ' '  ||  vbuf.b15 != ' '  ||
	    vbuf.b16 != ' '  ||  vbuf.b17 != ' '  ||  vbuf.b18 != ' '  ||
	    vbuf.b19 != ' '  ||  vbuf.b20 != ' '  ||  vbuf.b21 != ' '  ||
	    vbuf.b22 != ' '  ||  vbuf.b23 != ' '  ||  vbuf.b24 != ' '  ||
	    vbuf.b25 != ' '  ||  vbuf.b26 != ' '  ||  vbuf.b27 != ' '  ||
	    vbuf.b28 != ' '  ||  vbuf.b29 != ' '  ||  vbuf.b30 != ' '  ||
	    vbuf.b31 != ' '  ||  vbuf.b32 != ' '  ||  vbuf.b33 != ' '  ||
	    vbuf.b34 != ' '  ||  vbuf.b35 != ' ') {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_FATAL, id,
			"format error - bad record separator(s)");
		return(TBL_ERR);
	}

	if (vbuf.vb_status == VVS_MIGRATED  &&  !isalnum(vbuf.vb_vsn0[0])) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_FATAL, id,
			"bad record - migrated but no vsn");
		return(TBL_ERR);
	}

	/*
	**	translate the record
	*/
	memcpy(VInfo.v_vv.class, vbuf.vb_class, sizeof(VInfo.v_vv.class));
	memcpy(VInfo.v_vv.serial, vbuf.vb_serial, sizeof(VInfo.v_vv.serial));
	memcpy(VInfo.v_cluster, vbuf.vb_cluster, sizeof(VInfo.v_cluster));
	memcpy(VInfo.v_id, vbuf.vb_id, sizeof(VInfo.v_id));
	VInfo.v_status = vbuf.vb_status;
	VInfo.v_use = vbuf.vb_use;
	if (C2BIN(DecInt, &VInfo.v_version, vbuf.vb_version)		||
	    C2BIN(DecInt, &VInfo.v_nwrite, vbuf.vb_nwrite)		||
	    C2BIN(DecInt, &VInfo.v_nread, vbuf.vb_nread)		||
	    C2BIN(DecInt, &VInfo.v_nrestore, vbuf.vb_nrestore)		||
	    C2BIN(DecInt, &VInfo.v_mntfail, vbuf.vb_mntfail)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
			"table error - int(s)");
		return(TBL_ERR);
	}
	/*
	**	OFF_T's - big for 64-bit fs's
	*/
	memcpy(VInfo.v_data, vbuf.vb_data, sizeof(VInfo.v_data));
	memcpy(VInfo.v_maxdata, vbuf.vb_maxdata, sizeof(VInfo.v_maxdata));

	VInfo.v_creat = GetTime(vbuf.vb_creat);
	VInfo.v_write = GetTime(vbuf.vb_write);
	VInfo.v_finish = GetTime(vbuf.vb_finish);
	VInfo.v_read = GetTime(vbuf.vb_read);
	VInfo.v_save = GetTime(vbuf.vb_save);
	VInfo.v_freed = GetTime(vbuf.vb_freed);
	VInfo.v_restore = GetTime(vbuf.vb_restore);

	/*
	 *  vsn fields
	 */
	VInfo.v_nvsns = CHAR2INT(vbuf.vb_nvsns);
	if (VInfo.v_nvsns > MAXVSN)
		nLogMsg(EL_FATAL, id, "%s: nvsns %d, max %d", 
					VV2Name(&VInfo.v_vv),
					VInfo.v_nvsns, MAXVSN);
	for (i=0; i<VInfo.v_nvsns; i++) {
		if (cslots[i]->vb_vflags[1] != VSN_OK) {
			memset(VInfo.v_vsns[i].v_vsn, ' ', (size_t)VSNLEN);
			continue;
		}
		if (C2BIN(DecInt, &VInfo.v_vsns[i].v_nrec,cslots[i]->vb_nrec)) {
			VVError = VVE_DBASE;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
				"%s: table error - nrec vsn %d",
				VV2Name(&VInfo.v_vv), i);
			return(TBL_ERR);
		}

		if (C2BIN(DecUint, &VInfo.v_vsns[i].v_block, 
						cslots[i]->vb_nblk)) {
			VVError = VVE_DBASE;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
				"%s: table error - block id %d",
				VV2Name(&VInfo.v_vv), i);
			return(TBL_ERR);
		}
		memcpy(VInfo.v_vsns[i].v_vsn, cslots[i]->vb_vsn,(size_t)VSNLEN);
		VInfo.v_vsns[i].v_date = GetTime(cslots[i]->vb_vdate);
	}
	for (; i<MAXVSN; i++)
		memset(VInfo.v_vsns[i].v_vsn, ' ', (size_t)VSNLEN);

	return(0);
}

  int
LoadVV(char *id, int translate)
{
	int	ret;

	/*
	 *  load the record
	 */
	if (LSEEK(db_vv_fd, vv_addr, SEEK_SET) != vv_addr) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, id, "table LSEEK");
		return(TBL_ERR);
	}
	ret = read(db_vv_fd, &vbuf, sizeof(vbuf_t));
	if (ret == 0) {
		/*
		 *  this should only happen when reading whole
		 *	table during an index rebuild
		 */
		nLogMsg(EL_STATUS2, id, "end of file");
		return(TBL_EOF);
	}
	if (ret != sizeof(vbuf_t)) {
		VVError = VVE_DBASE;
		if (ret == -1)
			nLogMsg(EL_SYS|EL_CODE, id, "table read");
		else
			nLogMsg(EL_ERROR1|EL_CODE, id, 
					"table read: %d bytes, got %d",
					ret, sizeof(vbuf_t));
		return(TBL_ERR);
	}
	if (translate)
		return(Vbuf2VInfo(id));
	else
		return(0);
}


/*
 *  Vbuf2VV() - a short-cut for LoadVV(__, 0)
 */
static VV_t	buf_vv;
  VV_t *
Vbuf2VV()
{
	memcpy(buf_vv.class, vbuf.vb_class, sizeof(buf_vv.class));
	memcpy(buf_vv.serial, vbuf.vb_serial, sizeof(buf_vv.serial));
	return(&buf_vv);
}

  int
Vsnbuf2VsnInfo(char *caller)
{
	char		*id = "Vsnbuf2VsnInfo";
	char		*cp;
	Class_t		*cip;

	/*
	**	check the record
	*/
	if (vsnbuf.end != '\n') {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
			"(%s) vsn tbl/index error - bad record terminator",
									caller);
		return(TBL_ERR);
	}
	if (vsnbuf.b01 != ' '  ||  vsnbuf.b02 != ' '  ||  vsnbuf.b03 != ' '  ||
	    vsnbuf.b04 != ' '  ||  vsnbuf.b05 != ' '  ||  vsnbuf.b06 != ' '  ||
	    vsnbuf.b07 != ' '  ||  vsnbuf.b08 != ' '  ||  vsnbuf.b09 != ' '  ||
	    vsnbuf.b10 != ' ') {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_FATAL, id,
			"(%s) vsn tbl error - bad record separator(s)", caller);
		return(TBL_ERR);
	}


	/*
	**	translate the record & check some more
	*/
	memcpy(VsnInfo.vs_class, vsnbuf.vb_class, sizeof(VsnInfo.vs_class));
	cp = ClassString(VsnInfo.vs_class);
	if ((cip = ClassInfo(cp)) == NULL) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_FATAL, id,
			"(%s) vsn tbl error - unknown class (%s)", caller, cp);
		return(TBL_ERR);
	}
	VsnInfo.vs_classI = CHAR2INT(vsnbuf.vb_classI);
	if (VsnInfo.vs_classI >= cip->cs_nvsns) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_FATAL, id,
			"(%s) vsn tbl error - slot %d ('%c') > max %d", 
			caller, VsnInfo.vs_classI, vsnbuf.vb_classI,
			cip->cs_nvsns-1);
		return(TBL_ERR);
	}
	VsnInfo.vs_status = vsnbuf.vb_status;
	memcpy(VsnInfo.vs_lastvv.class, vsnbuf.vb_lastclass, 
					sizeof(VsnInfo.vs_lastvv.class));
	memcpy(VsnInfo.vs_lastvv.serial, vsnbuf.vb_lastserial, 
					sizeof(VsnInfo.vs_lastvv.serial));
/*
if (VsnInfo.vs_lastvv.class[0] == ' ')
nLogMsg(EL_STATUS1, id, " -- %.6s: status %s/%c lastvv [none]", 
vsnbuf.vb_vsn, 
vvStatusStr((int)vsnbuf.vb_status), vsnbuf.vb_status);
else
nLogMsg(EL_STATUS1, id, " -- %.6s: status %s/%c lastvv %s", vsnbuf.vb_vsn,
vvStatusStr((int)vsnbuf.vb_status), vsnbuf.vb_status, VV2Name(&VsnInfo.vs_lastvv));
*/
	if (C2BIN(DecInt, &VsnInfo.vs_lastrec, vsnbuf.vb_lastrec)	 ||
	    C2BIN(DecInt, &VsnInfo.vs_errsinarow, vsnbuf.vb_errsinarow)  ||
	    C2BIN(DecInt, &VsnInfo.vs_toterrs, vsnbuf.vb_toterrs)	 ||
	    C2BIN(DecInt, &VsnInfo.vs_livevv, vsnbuf.vb_livevv)		 ||
	    C2BIN(DecInt, &VsnInfo.vs_delvv, vsnbuf.vb_delvv)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
				"(%s) vsn table error - int(s)", caller);
		return(TBL_ERR);
	}
/*
nLogMsg(EL_STATUS1, id, " vs_livevv %d  vs_delvv %d",
VsnInfo.vs_livevv, VsnInfo.vs_delvv);
*/
	if (C2BIN(DecUint, &VsnInfo.vs_lastblk, vsnbuf.vb_lastblk)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id,
			"(%s) vsn table error - ulong(s)", caller);
		return(TBL_ERR);
	}
	memcpy(VsnInfo.vs_vsn, vsnbuf.vb_vsn, sizeof(VsnInfo.vs_vsn));

	/* hack for now -add to table */
	VsnInfo.vs_blksize = 4096;
	return(0);
}

  int
LoadVsn(char *caller)
{
	char	*id = "LoadVsn";
	int	ret;

	/*
	 *  load the record
	 */
	if (LSEEK(db_vsn_fd, vsn_addr, SEEK_SET) != vsn_addr) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, id, "(%s) vsn table LSEEK",
									caller);
		return(TBL_ERR);
	}
	ret = read(db_vsn_fd, &vsnbuf, sizeof(vsbuf_t));
	if (ret == 0) {
		/*
		 *  this should only happen when reading whole
		 *	table during an index rebuild
		 */
		nLogMsg(EL_STATUS2, id, "(%s) end of vsn file", caller);
		return(TBL_EOF);
	}
	if (ret != sizeof(vsbuf_t)) {
		VVError = VVE_DBASE;
		if (ret == -1)
			nLogMsg(EL_SYS|EL_CODE, id, "(%s) vsn table read",
									caller);
		else
			nLogMsg(EL_ERROR1|EL_CODE, id, 
					"(%s) vsn table read: %d bytes, got %d",
					caller, ret, sizeof(vsbuf_t));
		return(TBL_ERR);
	}
	return(Vsnbuf2VsnInfo(caller));
}

