
/*
 *  vvm_db1.c - NAStore Virtual Volume Manager database routines
 *
 *	Bill Ross, NASA Ames, 1997-9
 */

#include "vvmd.h"
#include "avltree.h"
#include "vvm_db.h"

extern char		*vvDBDir;

/*
**	table stuff from vvm_db0.c
*/
extern char		dbf_vv_tbl[];
extern int		db_vv_fd;
extern OFF_T		vv_addr;
extern vbuf_t		vbuf;
extern ClassSlot_t	*cslots[];
extern VVInfo_t		VInfo;

extern char		dbf_vsn_tbl[MAXPATHLEN];
extern int		db_vsn_fd;
extern OFF_T		vsn_addr;
extern vsbuf_t		vsnbuf;
extern Vsn_t		VsnInfo;

/*
**	basic index stuff
*/
DBT		key, data;
char		vvkey[256], vsnkey[256];
int		dberr;

DB		*db_vv_hot_idx = NULL;
DB		*db_vv_vsn_idx = NULL;
DB		*db_vv_id_idx = NULL;
DB		*db_vv_vol_idx = NULL;
DB		*db_vv_wr_idx = NULL;
DB		*db_vv_tp_idx = NULL;
DB		*db_vv_tp2_idx = NULL;

DB		*db_vsn_hot_idx = NULL;
DB		*db_vsn_vsn_idx = NULL;
DB		*db_vsn_recyc_idx = NULL;

char		dbf_vv_hot_idx[MAXPATHLEN];
char		dbf_vv_id_idx[MAXPATHLEN];
char		dbf_vv_vsn_idx[MAXPATHLEN];
char		dbf_vv_vol_idx[MAXPATHLEN];
char		dbf_vv_tp_idx[MAXPATHLEN];
char		dbf_vv_tp2_idx[MAXPATHLEN];
char		dbf_vv_wr_idx[MAXPATHLEN];

char		dbf_vsn_vsn_idx[MAXPATHLEN];
char		dbf_vsn_hot_idx[MAXPATHLEN];
char		dbf_vsn_recyc_idx[MAXPATHLEN];

/*
**	local stuff
*/
static char		dbf_lock[MAXPATHLEN];
static char		dbf_classes[MAXPATHLEN];

static int		dbopen = 0;
static STAT_T		sbuf;

static AVL_IX_DESC	avl_blob_idx;
static AVL_IX_REC	*avl_entry = NULL;


/*************************************************************************
			TABLE MANIPULATION
 *************************************************************************/

/*
 *  Unlink() - ignore ENOENT
 */
  static void
Unlink(char *name)
{
	char	*id = "Unlink";

	if (unlink(name) == -1) {
		if (errno != ENOENT)
			nLogMsg(EL_SYS|EL_FATAL, id, "%s: unlink failed", name);
	}
	nLogMsg(EL_DEBUG1, id, "unlinked %s", name);
}

  static void
UnlinkVVIndexes()
{
	Unlink(dbf_vv_vol_idx);
	Unlink(dbf_vv_hot_idx);
	Unlink(dbf_vv_id_idx);
	Unlink(dbf_vv_vsn_idx);
	Unlink(dbf_vv_tp_idx);
	Unlink(dbf_vv_tp2_idx);
	Unlink(dbf_vv_wr_idx);
}

  static void
UnlinkVsnIndexes()
{
	Unlink(dbf_vsn_vsn_idx);
	Unlink(dbf_vsn_hot_idx);
	Unlink(dbf_vsn_recyc_idx);
}

  static int
StartVVIndexes()
{
	int	doindexes;

	doindexes =  StartIndex(dbf_vv_hot_idx, &db_vv_hot_idx, 0);
	doindexes += StartIndex(dbf_vv_vsn_idx, &db_vv_vsn_idx, 0);
	doindexes += StartIndex(dbf_vv_id_idx,  &db_vv_id_idx,  0);
	doindexes += StartIndex(dbf_vv_tp_idx,  &db_vv_tp_idx,  0);
	doindexes += StartIndex(dbf_vv_tp2_idx, &db_vv_tp2_idx, 0);
	doindexes += StartIndex(dbf_vv_wr_idx,  &db_vv_wr_idx,  0);
	doindexes += StartIndex(dbf_vv_vol_idx, &db_vv_vol_idx, 0);

	return(doindexes);
}

  static int
StartVsnIndexes()
{
	int	doindexes= 0;

	doindexes += StartIndex(dbf_vsn_hot_idx, &db_vsn_hot_idx, 0);
	doindexes += StartIndex(dbf_vsn_recyc_idx, &db_vsn_recyc_idx, 0);
	doindexes += StartIndex(dbf_vsn_vsn_idx, &db_vsn_vsn_idx, 0);

	return(doindexes);
}

  int
WriteVV(char *id)
{
	int	ret;

	/*
	**	go to place in file for the record
	*/
	if (LSEEK(db_vv_fd, vv_addr, SEEK_SET) != vv_addr) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, id, "VV table LSEEK");
		return(TBL_ERR);
	}
	ret = write(db_vv_fd, &vbuf, sizeof(vbuf_t));

	/*
	**	a write err is serious - table could be corrupted now.
	*/
	if (ret != sizeof(vbuf_t)) {
		if (ret == -1)
			nLogMsg(EL_SYS|EL_FATAL, id, "VV table write fd %d", 
							db_vv_fd);
		else
			nLogMsg(EL_ERROR1|EL_FATAL, id, 
					"VV table write: %d bytes, got %d",
					ret, sizeof(vbuf_t));
	}
	return(0);
}


 void
VsnMsg(char *msg, int date)
{
	char	buf1[sizeof(vsnbuf.vb_errmsg)+1];
	char	*cp;
	int	i;
	size_t	len1, len2, len3;

	/*
	**	set up new msg
	*/
	if (date) {
		(void)sprintf(buf1, "|%s %s", msg, bsd_ctime(&Now));
		cp = strchr(buf1, '\n');
		*cp = '\0';
	} else
		(void)strncpy(buf1, msg, sizeof(vsnbuf.vb_errmsg));

	len1 = strlen(buf1);

	/*
	**	see how much room is left in record
	*/
	cp = &vsnbuf.vb_errmsg[sizeof(vsnbuf.vb_errmsg)-1];
	for (len2=0; *cp == ' ' && len2<sizeof(vsnbuf.vb_errmsg); cp--, len2++);

	if (len1 <= len2+2) {
		/*
		 *  msg fits
		 */
		cp += 2;
		memcpy(cp, buf1, len1);
		return;
	}

	/*
	**	need to back up msg to make room
	*/
	cp = vsnbuf.vb_errmsg + 1;
	for (len3=0; *cp != '|'; cp++, len3++);
	if (len1 < len2+len3) {
		char	buf2[sizeof(vsnbuf.vb_errmsg)+1];

		/*
		**	move the msg back
		*/
		memset(buf2, ' ', sizeof(buf2));
		memcpy(buf2, cp, sizeof(buf2)-len3);
		memset(vsnbuf.vb_errmsg, ' ', sizeof(vsnbuf.vb_errmsg));
		memcpy(vsnbuf.vb_errmsg, buf2, sizeof(buf2)-len3);
		cp++;
		memcpy(cp, buf1, len1);
		return;
	}

	/*
	**	problems - overwrite everything w/ new msg
	*/
	memset(vsnbuf.vb_errmsg, ' ', sizeof(vsnbuf.vb_errmsg));
	memcpy(vsnbuf.vb_errmsg, buf1, len1);
}

  int
WriteVsn(char *id)
{
	int	ret;

	/*
	**	go to place in file for the record
	*/
	if (LSEEK(db_vsn_fd, vsn_addr, SEEK_SET) != vsn_addr) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_CODE, id, "vsn table LSEEK");
		return(TBL_ERR);
	}
	ret = write(db_vsn_fd, &vsnbuf, sizeof(vsbuf_t));

	/*
	**	a write err is serious - table could be corrupted now.
	*/
	if (ret != sizeof(vsbuf_t)) {
		if (ret == -1)
			nLogMsg(EL_SYS|EL_FATAL, id, "vsn table write fd %d", 
							db_vsn_fd);
		else
			nLogMsg(EL_ERROR1|EL_FATAL, id, 
					"vsn table write: %d bytes, got %d",
					ret, sizeof(vsbuf_t));
	}
	return(0);
}

/*
**	StartIndex() - open it if it exists, otherwise make it
*/
 int
StartIndex(char *name, DB **ix, int flags)
{
	char	*id = "StartIndex";
	int	ret;

	if (STAT(name, &sbuf) == -1) {
		if (errno != ENOENT)
			nLogMsg(EL_FATAL|EL_SYS, id,
					"index (%s) stat failed:", name);
		nLogMsg(EL_STATUS2, id, "creating %s index (%s)",
				(flags & DB_DUP ? "" : "exclusive ", name));
		ret = 1;
	} else
		ret = 0;
 
	if (dberr = db_create(ix, NULL, 0))
		nLogMsg(EL_FATAL, id, "db_create(): %s", db_strerror(dberr));
	if (flags & DB_DUP) {
		if (dberr = (*ix)->set_flags(*ix, DB_DUP))
			nLogMsg(EL_FATAL, id, "db->set_flags(%s): %s", name,
							db_strerror(dberr));
	}
nLogMsg(EL_STATUS1, id, "open %s ..", name);
	if (dberr = (*ix)->open(*ix, name, NULL, DB_BTREE,
					DB_CREATE|(flags & DB_RDONLY), 0600))
		nLogMsg(EL_FATAL, id, "db->open(%s): %s", name,
							db_strerror(dberr));
	return(ret);
}

  int
IndexVVix(char *id, char *vvname, char *idix, DB *ixp)
{
	/*
	**	add to vv to index (e.g. tp for 
	**		remembering vv's to write to tape,
	**		wr for tracking vv's open when clnt
	**		goes away)
	*/
	FILL_DBT(data, vv_addr);
	STR_DBT(key, vvname);
nLogMsg(EL_STATUS1, id, " index %s - %s", idix, vvname);
	if (dberr = ixp->put(ixp, NULL,  &key, &data, DB_NOOVERWRITE)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
			"%s put(vv %s) failed", idix, key.data);
		return(1);
	}
	return(0);
}

  static int
UnIndexVVix(char *id, char *vvname, char *idix, DB *ixp)
{
	/*
	**	delete from vv tp or wr index
	*/
	STR_DBT(key, vvname);
nLogMsg(EL_STATUS1, id, " unindex %s - %s", idix, key.data);
	if (dberr = ixp->del(ixp, NULL, &key, 0)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
			"%s del(vv %s) failed", idix, key.data);
		return(1);
	}
	return(0);
}


  static int
IndexVVVsn(char *id, char *vsnp)
{
	/*
	**	add to vv vsn index
	*/
	FILL_DBT(data, vv_addr);
	(void)sprintf(vvkey, "%.*s_%llx", VSNLEN, vsnp, (long long) vv_addr);
	STR_DBT(key, vvkey);
	FILL_DBT(data, vv_addr);
	if (dberr = db_vv_vsn_idx->put(db_vv_vsn_idx, NULL, &key, &data,
							DB_NOOVERWRITE)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
			"db put(vsn %s) failed", vvkey);
		return(1);
	}
	return(0);
}

  static int
UnIndexVVVsn(char *id, char *vsnp)
{
	/*
	**	delete from vv vsn index
	*/
	(void)sprintf(vvkey, "%.*s_%llx", VSNLEN, vsnp, (long long) vv_addr);
	STR_DBT(key, vvkey);
	if (dberr = db_vv_vsn_idx->del(db_vv_vsn_idx, NULL, &key, 0)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
				"db del(vsn %s): %s", vvkey, 
							db_strerror(dberr));
		return(1);
	}
	return(0);
}

/*
**	IndexVV() - put VV in all relevant indexes;
**	used for newly-created entries & index rebuilds.
**	Assumes VInfo has correct id, class, serial
*/
  static int
IndexVV(char *id)
{
	char	vvname[VV_NAMELEN];
	VV_t	*vvp;
	int	i, nvsns;
	OFF_T	size;

	vvp = Vbuf2VV();
	(void)strcpy(vvname, VV2Name(vvp));
	nvsns = CHAR2INT(vbuf.vb_nvsns);

	/*
	**	status-related indexing
	*/
	switch (vbuf.vb_status) {

	    case VVS_HOT:
		/*
		**	add to hot VV index
		*/
		(void)sprintf(vvkey, "%s_%s_%llx",
				ClassString(VInfo.v_vv.class),
				IdString(VInfo.v_id), 
				(long long) vv_addr);
		STR_DBT(key, vvkey);
		FILL_DBT(data, vv_addr);
		if (dberr = db_vv_hot_idx->put(db_vv_hot_idx, NULL, &key, &data,
							DB_NOOVERWRITE)) {
			VVError = VVE_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
				"db put(class_id %s) failed", vvkey);
			return(1);
		}
		break;

	    case VVS_FINISHED:
		/*
		**	add to list to write to tape
		*/
		if (IndexVVix(id, vvname, "tp", db_vv_tp_idx))
			return(1);
		break;

	    case VVS_RECYCLING:
		/*
		**	a tape is being recycled
		*/
		if (DiskResidentSize(vvp, &size) == 0)
			break;

		/*
		**	not on disk - add to list to restore from tape
		*/
		for (i=0; i<nvsns; i++) {
			if (cslots[i]->vb_vflags[1] != VSN_RECYCLING)
				continue;
			(void)sprintf(vvkey, "%.*s_%s", VSNLEN, 
						cslots[i]->vb_vsn, vvname);
			STR_DBT(key, vvkey);
			ZERO_OBJ(data);
			if (dberr = db_vv_tp2_idx->get(db_vv_tp2_idx, NULL,
						&key, &data, 0)) {
				if (dberr != DB_NOTFOUND)
					nLogMsg(EL_FATAL, id, "db get(%s): %s",
						vvkey, db_strerror(dberr));
				STR_DBT(key, vvkey);
				FILL_DBT(data, vv_addr);
				if (dberr = db_vv_tp2_idx->put(db_vv_tp2_idx,
							NULL, &key, &data,
							DB_NOOVERWRITE)) {
					nLogMsg(EL_ERROR1|EL_FATAL, id, 
						"db put(%s): %s",
						vvkey, db_strerror(dberr));
				}
			}
		}
		break;

	    default:
		break;
	}

	/*
	**	use-related indexing
	*/
	if (vbuf.vb_use == VVU_WRITING) {
		/*
		 *  add to list to finish if not properly closed
		 */
		if (IndexVVix(id, vvname, "wr", db_vv_wr_idx))
			return(1);
	}

	/*
	**	vsn-related indexing
	*/
	for (i=0; i<nvsns; i++) {
		char	flag = cslots[i]->vb_vflags[1];

		if (flag == VSN_OK  ||  flag == VSN_RECYCLING) {
			if (IndexVVVsn(id, cslots[i]->vb_vsn))
				return(1);
		}
	}

	/*
	**	add to id index
	*/
	(void)sprintf(vvkey, "%s_%llx", IdString(VInfo.v_id), 
					(long long) vv_addr);
	STR_DBT(key, vvkey);
	FILL_DBT(data, vv_addr);
	if (dberr = db_vv_id_idx->put(db_vv_id_idx, NULL, &key, &data, 
							DB_NOOVERWRITE)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, "db put(id %s): %s",
						vvkey, db_strerror(dberr));
		return(1);
	}

	/*
	**	add to vv index
	*/
	STR_DBT(key, vvname);
	FILL_DBT(data, vv_addr);
	if (dberr = db_vv_vol_idx->put(db_vv_vol_idx, NULL, &key, &data,
							DB_NOOVERWRITE)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
				"db put(vv %s) failed", vvkey);
		return(1);
	}
	return(0);
}

/*
**	UnIndexVV() - remove recycled VV from indexes, EXCEPT
**		never delete from vv index - the only way
**		to delete from this index is if the record 
**		is purged from the VV table & the index 
**		rebuilt. So if the record still exists, it 
**		can be used if a tape is being checked.
*/
  static int
UnIndexVV(char *id)
{
	char	vvname[VV_NAMELEN];
	int	i, nvsns;

	(void)strcpy(vvname, VV2Name(Vbuf2VV()));
	nvsns = CHAR2INT(vbuf.vb_nvsns);

	/*
	**	delete from VV hot index if there
	*/
	switch (vbuf.vb_status) {
	    case VVS_HOT:
		nLogMsg(EL_STATUS1, id, "DELETING HOT VV %s id %s",
				vvname, IdString(VInfo.v_id));
		(void)sprintf(vvkey, "%s_%s_%llx", 
				ClassString(VInfo.v_vv.class),
				IdString(VInfo.v_id), (long long)vv_addr);
		STR_DBT(key, vvkey);
		if (dberr = db_vv_hot_idx->del(db_vv_hot_idx, NULL, &key, 0)) {
			VVError = VVE_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
					"db del(class_id %s): %s", 
					vvkey, db_strerror(dberr));
			return(1);
		}
		break;
	    case VVS_FINISHED:
		(void)UnIndexVVix(id, vvname, "tp", db_vv_tp_idx);
		break;
	    case VVS_RECYCLING:
		for (i=0; i<nvsns; i++) {
			if (cslots[i]->vb_vflags[1] != VSN_RECYCLING)
				continue;
			(void)sprintf(vvkey, "%.*s_%s", VSNLEN, 
							cslots[i]->vb_vsn, 
							vvname);
			STR_DBT(key, vvkey);
			if (dberr = db_vv_tp2_idx->del(db_vv_tp2_idx, NULL,
							&key, 0)) {
				nLogMsg(EL_ERROR1, id, "db del(%s, tp2): %s",
						vvkey, db_strerror(dberr));
			}
		}
		break;
	}

	if (vbuf.vb_use == VVU_WRITING) 
		(void)UnIndexVVix(id, vvname, "wr", db_vv_wr_idx);

	for (i=0; i<nvsns; i++) {
		if (cslots[i]->vb_vflags[1] == VSN_OK)
			(void)UnIndexVVVsn(id, cslots[i]->vb_vsn);
	}

	/*
	**	id index
	*/
	(void)sprintf(vvkey, "%s_%llx", IdString(VInfo.v_id), 
						(long long)vv_addr);
	STR_DBT(key, vvkey);
	if (dberr = db_vv_id_idx->del(db_vv_id_idx, NULL, &key, 0)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
				"db del(id %s): %s", vvkey, db_strerror(dberr));
		return(1);
	}

	return(0);
}

/*
**	IndexVsn() - add to hot (normal for new vsn) and vsn indexes
*/
  static int
IndexVsn(char *id)
{
	/*
	**	status-based indexing
	*/
	switch (vsnbuf.vb_status) {

	    case VVS_HOT:
		/*
		**	available for appending VV's
		*/
		(void)sprintf(vsnkey, "%s_%c_%llx", 
					ClassString(vsnbuf.vb_class),
					vsnbuf.vb_classI, (long long) vsn_addr);
		STR_DBT(key, vsnkey);
		FILL_DBT(data, vsn_addr);
		if (dberr = db_vsn_hot_idx->put(db_vsn_hot_idx, NULL,
						 &key, &data, DB_NOOVERWRITE)) {
			VVError = VVE_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
						"db put(class_i %s): %s", 
						vsnkey, db_strerror(dberr));
			return(1);
		}
		break;

	    case VVS_RECYCLING:
		/*
		**	VV's being pulled off to recycle, so may need
		**	to be mounted; add to list for now.
		*/
		(void)sprintf(vsnkey, "REST_%.*s", VSNLEN, vsnbuf.vb_vsn);
		STR_DBT(key, vsnkey);
		FILL_DBT(data, vsn_addr);
		if (dberr = db_vsn_recyc_idx->put(db_vsn_recyc_idx, NULL,
						&key, &data, DB_NOOVERWRITE)) {
			VVError = VVE_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
						"db put(recyc vsn %s): %s", 
						vsnkey, db_strerror(dberr));
			return(1);
		}
		break;

	    case VVS_RECYCLED:
		/*
		**	waiting to be recycled to volman
		*/
		(void)sprintf(vsnkey, "DEL_%.*s", VSNLEN, vsnbuf.vb_vsn);
		STR_DBT(key, vsnkey);
		FILL_DBT(data, vsn_addr);
		if (dberr = db_vsn_recyc_idx->put(db_vsn_recyc_idx, NULL,
						&key, &data, DB_NOOVERWRITE)) {
			VVError = VVE_DBASE;
			nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
					"db put(recyc vsn %s): %s", 
					vsnkey, db_strerror(dberr));
			return(1);
		}
		break;

	    default:
		break;

	}

	/*
	**	add to vsn_vsn index
	*/
	memcpy(vsnkey, vsnbuf.vb_vsn, (size_t)VSNLEN);
	vsnkey[VSNLEN] = '\0';
	STR_DBT(key, vsnkey);
	FILL_DBT(data, vsn_addr);
	if (dberr = db_vsn_vsn_idx->put(db_vsn_vsn_idx, NULL, &key, &data, 
							DB_NOOVERWRITE)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, "db put(vsn %s): %s",
						vsnkey, db_strerror(dberr));
		return(1);
	}
	return(0);
}

/*
**	ResetHotVsns() - mark non-busy at startup so that they
**		can be reused in class panels
*/
  static void
ResetHotVsns()
{
	char	*id = "ResetHotVsns";
	int	i, inuse = 0;
	DBC	*cursor;

	if (dberr = db_vsn_hot_idx->cursor(db_vsn_hot_idx, NULL, &cursor, 0)) {
		nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
						db_strerror(dberr));
	}

	for (i=0; ; i++) {
		ZERO_OBJ(data);
		if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
			if (dberr == DB_NOTFOUND)
				break;
			nLogMsg(EL_FATAL, id, "cursor->c_get(): %s", 
							db_strerror(dberr));
		}
		COPY_DBT(vsn_addr, data);

		if (LoadVsn(id))
			nLogMsg(EL_FATAL, id, "can't load vsn rec");

		if (vsnbuf.vb_busy == 'b') {
			inuse++;
			vsnbuf.vb_busy = ' ';
			if (WriteVsn(id))
				nLogMsg(EL_FATAL, id, "can't write vsn rec");
		}
		if (vsnbuf.vb_status != VVS_HOT) {
			nLogMsg(EL_STATUS1, id, "%s: not hot -? /unindexing", 
						vsnstr(vsnbuf.vb_vsn));
			(void)UnIndexVsnHot(id);
		}
		
	}
	if (i)
		nLogMsg(EL_STATUS2, id, "%d hot, %d in use by prev vvmd",
								i, inuse);
	cursor->c_close(cursor);
}

/*
**	CloseIndexes() - close indexes to flush bplus buffers
*/
  int
CloseIndexes()
{
	char	*id = "CloseIndexes";
	int	err = 0;

	/*
	**	vv table indexes
	*/
	if (dberr = db_vv_hot_idx->close(db_vv_hot_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vv_hot_idx, db_strerror(dberr));
		err++;
	}
	if (dberr = db_vv_vsn_idx->close(db_vv_vsn_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vv_vsn_idx, db_strerror(dberr));
		err++;
	}
	if (dberr = db_vv_id_idx->close(db_vv_id_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vv_id_idx, db_strerror(dberr));
		err++;
	}
	if (dberr = db_vv_vol_idx->close(db_vv_vol_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vv_vol_idx, db_strerror(dberr));
		err++;
	}
	if (dberr = db_vv_tp_idx->close(db_vv_tp_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					db_vv_tp_idx, db_strerror(dberr));
		err++;
	}
	if (dberr = db_vv_tp2_idx->close(db_vv_tp2_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vv_tp2_idx, db_strerror(dberr));
		err++;
	}
	if (dberr = db_vv_wr_idx->close(db_vv_wr_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vv_wr_idx, db_strerror(dberr));
		err++;
	}

	/*
	 *  vsn table indexes
	 */
	if (dberr = db_vsn_hot_idx->close(db_vsn_hot_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vsn_hot_idx, db_strerror(dberr));
		err++;
	}
	if (dberr = db_vsn_vsn_idx->close(db_vsn_vsn_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vsn_vsn_idx, db_strerror(dberr));
		err++;
	}
	if (dberr = db_vsn_recyc_idx->close(db_vsn_recyc_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, " db close(%s): %s",
					dbf_vsn_recyc_idx, db_strerror(dberr));
		err++;
	}
	return(err);
}


/*
**	RebuildVsnIndexes() - TODO: fork & build indexes separately for speed
*/
  static void
RebuildVsnIndexes()
{
	char		*id = "RebuildVsnIndexes";
	int		i, ret, deleted = 0, lost = 0, hot = 0;
	int		unknown_class = 0, total = 0;
	VVMClass_t	*cip;

	/*
	 *  open vsn tbl /if forked
	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDONLY, 0)) == -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, "vsn table (%s) open failed",
								dbf_vsn_tbl);
		exit(2);
	}
	 */
	if (!StartIndex(dbf_vsn_hot_idx,   &db_vsn_hot_idx, 0)      ||
	    !StartIndex(dbf_vsn_recyc_idx, &db_vsn_recyc_idx, 0)  ||
	    !StartIndex(dbf_vsn_vsn_idx,   &db_vsn_vsn_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "what th");
		exit(2);
	}

	/*
	**	read the whole table
	*/
	vsn_addr = 0;
	if (LSEEK(db_vsn_fd, vsn_addr, SEEK_SET) != vsn_addr) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, "table LSEEK");
		exit(2);
	}
	for (i=1;;i++) {
		ret = read(db_vsn_fd, &vsnbuf, sizeof(vsnbuf));
		if (ret != sizeof(vsnbuf)) {
			if (ret == 0)
				break;
			if (ret == 1)
				nLogMsg(EL_CONSOLE0|EL_SYS, id, 
						"%s: failure at record %d",
						dbf_vsn_tbl, i);
			else
				nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
						"%s: incomplete record %d",
						dbf_vsn_tbl, i);
			exit(2);
		}

		/*
		**	see if record is really there
		*/
		if (vsnbuf.vb_status == VVS_DELETED) {
			deleted++;
			vsn_addr += sizeof(vsnbuf);
			continue;
		}
		if (vsnbuf.vb_status == VVS_LOST) {
			lost++;
			vsn_addr += sizeof(vsnbuf);
			continue;
		}

		/*
		**	check class code
		*/
		cip = VVMClassInfo(vsnbuf.vb_class);
		if (cip == NULL) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
				"%s: unknown class [%s], record %d vsn %.*s",
				dbf_vsn_tbl, ClassString(vsnbuf.vb_class), 
				i, VSNLEN, vsnbuf.vb_vsn);
			unknown_class++;
			vsn_addr += sizeof(vsnbuf);
			continue;
		}

		if (vsnbuf.vb_status == VVS_HOT)
			hot++;
		total++;

		/*
		**	translate record to guarantee validity
		**		TODO - just do partial check to save time
		*/
		if (Vsnbuf2VsnInfo(id)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: bad record %d", dbf_vsn_tbl, i);
			exit(2);
		}

		/*
		**	add to indexes
		*/
		if (IndexVsn(id)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: indexing record %d", 
					dbf_vsn_tbl, i);
			exit(2);
		}

		/*
		**	increment file pointer
		*/
		vsn_addr += sizeof(vsnbuf);
		if (!(i%25000))
			nLogMsg(EL_DEBUG1, id, "record %d..", i);
	}

	nLogMsg(EL_STATUS1, id, "vsn indexes built");
	nLogMsg(EL_STATUS1, id, "total: %d  hot: %d  deleted: %d  lost %d",
					total, hot, deleted, lost);
}

/*
**	RebuildVVIndexes() - TODO: fork & build indexes separately for speed
*/
  static void
RebuildVVIndexes()
{
	char		*id = "RebuildVVIndexes";
	int		i, j, ret, deleted = 0, hot = 0; 
	int		onvsn = 0, partvsn = 0, total = 0;
	int		notorder = 0, errs = 0;
	VVMClass_t	*cip;

	/*
	**	open vol tbl /if forked
	if ((db_vv_fd = open(dbf_vv_tbl, O_RDONLY, 0)) == -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, "vv table (%s) open failed",
								dbf_vv_tbl);
		exit(2);
	}
	*/
	if (!StartIndex(dbf_vv_hot_idx, &db_vv_hot_idx, 0)  ||
	    !StartIndex(dbf_vv_vsn_idx, &db_vv_vsn_idx, 0)  ||
	    !StartIndex(dbf_vv_id_idx,  &db_vv_id_idx,  0)  ||
	    !StartIndex(dbf_vv_tp_idx,  &db_vv_tp_idx,  0)  ||
	    !StartIndex(dbf_vv_tp2_idx, &db_vv_tp2_idx, 0)  ||
	    !StartIndex(dbf_vv_wr_idx,  &db_vv_wr_idx,  0)  ||
	    !StartIndex(dbf_vv_vol_idx, &db_vv_vol_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "what th");
		exit(2);
	}

	/*
	**	initialize per-class serial check
	*/
	for (i=0; cip=VVMClassIndex(i); i++)
		memset(cip->vc_tmpnum, ' ', sizeof(cip->vc_tmpnum));

	/*
	**	read the whole table
	*/
	vv_addr = 0;
	if (LSEEK(db_vv_fd, vv_addr, SEEK_SET) != vv_addr) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, "table LSEEK");
		exit(2);
	}
	for (i=1;;i++) {
		int	nvsns, ct;
		ret = read(db_vv_fd, &vbuf, sizeof(vbuf));
		if (ret != sizeof(vbuf)) {
			if (ret == 0)
				break;
			if (ret == 1)
				nLogMsg(EL_CONSOLE0|EL_SYS, id, 
						"%s: failure at record %d",
						dbf_vv_tbl, i);
			else
				nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
						"%s: incomplete record %d",
						dbf_vv_tbl, i);
			exit(2);
		}

		/*
		**	see if record is really there
		*/
		if (vbuf.vb_status == VVS_DELETED) {
			deleted++;
			vv_addr += sizeof(vbuf);
			continue;
		}

		/*
		**	check class code and update class's lastnum 
		**		(used for generating new vv names)
		*/
		cip = VVMClassInfo(vbuf.vb_class);
		if (cip == NULL) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
				"%s: unknown class, record %d vv %s_%.*s",
				dbf_vv_tbl, i, 
				ClassString(vbuf.vb_class), VV_SERIALLEN,
				vbuf.vb_serial);
			exit(2);
		}
/*
nLogMsg(EL_STATUS1, id, "-- %s  %s", 
VV2Name(Vbuf2VV()), vvStatusStr(vbuf.vb_status));
nLogMsg(EL_STATUS1|EL_NOTIME, id, "  tmp [%.*s] rec [%.*s]",
					VV_SERIALLEN, cip->vc_tmpnum,
					VV_SERIALLEN, vbuf.vb_serial);
*/
		switch (CmpClassSerial(cip->vc_tmpnum, vbuf.vb_serial)) {
		    case -1:
			memcpy(cip->vc_tmpnum, vbuf.vb_serial, 
						sizeof(cip->vc_tmpnum));
			break;
		    case 1:
			notorder++;
			break;
		    case 0:
			nLogMsg(EL_FATAL, id,  "duplicate VV %s",
						VV2Name(Vbuf2VV()));
		    default:
			nLogMsg(EL_FATAL, id,  
				"unexpected CmpClassSerial %d [%.*s] [%.*s]",
				CmpClassSerial(cip->vc_tmpnum, vbuf.vb_serial),
				VV_SERIALLEN, cip->vc_tmpnum,
				VV_SERIALLEN, vbuf.vb_serial);
 		}

		if (vbuf.vb_status == VVS_HOT)
			hot++;

		/*
		**	check the vsn status
		*/
		ct = 0;
		nvsns = CHAR2INT(vbuf.vb_nvsns);
		if (nvsns > MAXVSN)
			nLogMsg(EL_FATAL, id, "%s: nvsns %d > max %d",
					VV2Name(Vbuf2VV()), nvsns, MAXVSN);
		for (j=0; j<nvsns; j++) {
			if (cslots[j]->vb_vflags[1] == VSN_OK)
				ct++;
		}
		if (ct == 0) {
			if (vbuf.vb_status == VVS_MIGRATED) {
				nLogMsg(EL_ERROR1|EL_CONSOLE0, id,
						"%s: 'migrated', no tapes",
						VV2Name(Vbuf2VV()));
				errs++;
			}
		} else if (ct == nvsns) {
			onvsn++;
			/* TODO - maybe just fix status & write back record */
			if (vbuf.vb_status != VVS_MIGRATED) {
				nLogMsg(EL_ERROR1|EL_CONSOLE0, id,
						"%s: on tape, but status %s",
						VV2Name(Vbuf2VV()),
						vvStatusStr(vbuf.vb_status));
				vbuf.vb_status = VVS_MIGRATED;
				vbuf.vb_use = VVU_IDLE;
				WriteVV(id);
			}
		} else {
			OFF_T	disksize;
			int	offdisk;

			partvsn++;

			offdisk = DiskResidentSize(Vbuf2VV(), &disksize);

nLogMsg(EL_STATUS1, id, "%s: INCOMPLETE/%s %s", VV2Name(Vbuf2VV()),
vvStatusStr(vbuf.vb_status), (offdisk ? "offdisk" : "on disk"));
			/*
			**	fix status if necessary
			*/
			if (vbuf.vb_status == VVS_MIGRATED) {
				if (offdisk)
					vbuf.vb_status = VVS_RECOVER;
				else
					vbuf.vb_status = VVS_FINISHED;
				WriteVV(id);
			} else if (vbuf.vb_status == VVS_ADDED)
				nLogMsg(EL_CONSOLE0, id, 
						"%s: need to handle ADDED!!",
						VV2Name(Vbuf2VV()));
		}
		total++;

		/*
		**	translate record to guarantee validity
		**		TODO - just do partial check to save time
		**		(note tho that IndexVV() requires some VInfo
		**		stuff to be correct)
		*/
		if (Vbuf2VInfo(id)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: bad record %d", dbf_vv_tbl, i);
			exit(2);
		}

		/*
		**	add to indexes
		*/
		if (IndexVV(id)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: indexing record %d", 
					dbf_vv_tbl, i);
			exit(2);
		}

		/*
		**	increment file pointer
		*/
		vv_addr += sizeof(vbuf);
		if (!(i%25000))
			nLogMsg(EL_DEBUG1, id, "record %d..", i);
	}
	for (i=0; cip=VVMClassIndex(i); i++) {
		VV_t	vv;

		memset(&vv, ' ', sizeof(vv));
		memcpy(vv.class, cip->vc_class.cs_prefix, 
					strlen(cip->vc_class.cs_prefix));
		memcpy(vv.serial, cip->vc_lastnum, sizeof(vv.serial));
		nLogMsg(EL_STATUS1|EL_NOTIME, id, "class %s last VV: %s", 
					cip->vc_class.cs_name, VV2Name(&vv));

                switch (CmpClassSerial(cip->vc_tmpnum, cip->vc_lastnum)) {
		    case 0:
			break;
		    case 1:
			nLogMsg(EL_STATUS1|EL_NOTIME, id, 
					"  updating class max: %.*s -> %.*s",
					VV_SERIALLEN, cip->vc_lastnum,
					VV_SERIALLEN, cip->vc_tmpnum);
			memcpy(cip->vc_lastnum, cip->vc_tmpnum, 
						sizeof(cip->vc_lastnum));
			break;
		    case -1:
			/* presumably due to deleted VV's */

			nLogMsg(EL_STATUS1|EL_NOTIME, id, 
					"  class max was %.*s but using %.*s",
					VV_SERIALLEN, cip->vc_tmpnum,
					VV_SERIALLEN, cip->vc_lastnum);
			break;
		    default:
			nLogMsg(EL_FATAL, id, "CmpClassSerial");
		}

	}
	nLogMsg(EL_STATUS1, id, "vv indexes built");
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "total: %d  hot: %d", total, hot);
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "       fully on vsn:   %d", onvsn);
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "       partly on vsn:  %d", partvsn);
	if (notorder)
		nLogMsg(EL_STATUS1|EL_NOTIME, id, 
					  "   not in order: %d", notorder);
	if (errs)
		nLogMsg(EL_STATUS1|EL_NOTIME, id,
					  "APPARENTLY LOST: %d", errs);
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "   deleted: %d", deleted);
}

/*
**	Init_ReadClass() - read class info from file. Only at startup,
**		so just open/read/close the file.
*/
  static int
Init_ReadClass(VVMClass_t *cip)
{
	char	*id = "Init_ReadClass";
	int	ret, fd;

	/*
	**	initialize the db side of the vvmd class struct
	*/
	memset(cip->vc_lastnum, ' ', sizeof(cip->vc_lastnum));
	cip->vc_lastnum[sizeof(cip->vc_lastnum)-1] = '0';

	/*
	**	replace it with whatever is in the file,
	**		if it exists
	*/
	(void)sprintf(dbf_classes, "%s/DB_CLASS_%s", vvDBDir,
						cip->vc_class.cs_prefix);
	if ((fd = open(dbf_classes, O_RDONLY)) == -1) {
		if (errno != ENOENT)
			nLogMsg(EL_FATAL|EL_SYS, id, "opening %s", dbf_classes);
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id,
				"No class file %s (for [%s]): %s",
				dbf_classes, cip->vc_class.cs_prefix,
				"rebuilding indexes to count class");
		return(1);
	}
	ret = read(fd, cip->vc_lastnum, sizeof(cip->vc_lastnum));
	if (ret == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "reading %s", dbf_classes);
	if (ret != sizeof(cip->vc_lastnum))
		nLogMsg(EL_FATAL, id, "reading %s: ", dbf_classes);
	close(fd);

	return(0);
}


/*
**	WriteClass() - write class info to file. Not a constant event,
**		so just open/write/close the file.
*/
  static void
WriteClass(VVMClass_t *cip)
{
	char	*id = "WriteClass";
	int	i, fd, ret, flag = O_RDWR;

	/*
	**	figure if this is 1st time; serial is a right-justified alphanum
	*/
	for (i=0; i<sizeof(cip->vc_lastnum); i++)
		if (cip->vc_lastnum[i] != ' ')
			break;
	if (i != sizeof(cip->vc_lastnum)  &&  !isalnum(cip->vc_lastnum[i]))
		nLogMsg(EL_FATAL, id, "class %s: bad serial [%*.*s]",
				cip->vc_class.cs_prefix,
				VV_SERIALLEN, VV_SERIALLEN, cip->vc_lastnum);
	if (i == sizeof(cip->vc_lastnum)-1  &&  cip->vc_lastnum[i] == '0')
		nLogMsg(EL_STATUS1, id, "%s: serial 0",
					cip->vc_class.cs_prefix);
	
	(void)sprintf(dbf_classes, "%s/DB_CLASS_%s", vvDBDir, 
						cip->vc_class.cs_prefix);

	if (STAT(dbf_classes, &sbuf) == -1) {
		if (errno != ENOENT)
			nLogMsg(EL_ERROR1|EL_SYS, id, "stat %s", dbf_classes);
		else {
			nLogMsg(EL_STATUS1, id, 
					"creating %s to write [%*.*s]", 
					dbf_classes, VV_SERIALLEN, VV_SERIALLEN,
					cip->vc_lastnum);
			flag |= O_CREAT;
		}
	}

	if ((fd = open(dbf_classes, flag, 0600)) == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "open %s", dbf_classes);
	ret = write(fd, cip->vc_lastnum, VV_SERIALLEN);
	if (ret == -1)
		nLogMsg(EL_FATAL|EL_SYS, id, "writing %s", dbf_classes);
	if (ret != VV_SERIALLEN)
		nLogMsg(EL_FATAL, id, "writing %s - only %d/%d bytes", 
						dbf_classes, ret, VV_SERIALLEN);
	close(fd);
}

  static int
IncClass(VVMClass_t *cip)
{
	char	*id = "IncClass";
	int	fd;

	if (IncClassSerial(cip->vc_lastnum))
		return(1);
	WriteClass(cip);
	return(0);
}

  static void
MakeBlobIx()
{
	char	*id = "MakeBlobIx";

	avl_create_index(&avl_blob_idx, AVL_DUP_KEYS_OK, 0);
	avl_entry = (AVL_IX_REC *) nAlloc(sizeof(AVL_IX_REC) + 100);
}

  static int
IndexClient(char *id, int pid, char *hostname)
{
	avl_entry->recptr = (void *)nAlloc(sizeof(vv_addr));
	*((OFF_T *)avl_entry->recptr) = vv_addr;
	(void)sprintf(avl_entry->key, "%d_%s", pid, hostname);
	if (avl_add_key(avl_entry, &avl_blob_idx) == AVL_IX_FAIL) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
				"avl_add_key(%s) failed", avl_entry->key);
		return(1);
	}
	return(0);
}

/*
**	CleanUpIncompleteWrite() - AFTER an index lookup that
**		sets vv_addr - but WHAT index call? Well, look at
**		the id for a clue. If cleaning up on restart after 
**		shutdown or crash, there is no pid_host blob index
**		entry to delete, so when cleaning up after a client
**		who just closed connection, must delete blob_idx
**		entry b4 coming here. ??
*/
  static void
CleanUpIncompleteWrite(char *id)
{
        char	vvname[VV_NAMELEN];

	/*
	**	yank up the poor thing from its miserable state
	*/
	if (LoadVV(id, 1))
		nLogMsg(EL_FATAL, id, "can't load vv rec, key %s", vvkey);

	(void)strcpy(vvname, VV2Name(Vbuf2VV()));

	/*
	**	delete 'writing' index record 
	*/
	if (UnIndexVVix(id, vvname, "wr", db_vv_wr_idx))
		nLogMsg(EL_FATAL, id, "db del(writing %s)", vvname);

	if (VInfo.v_use != VVU_WRITING) {
		nLogMsg(EL_CONSOLE0, id, "%s: not writing (%s)",
						vvname, 
						vvUseStr(VInfo.v_use));
		return;
	}

	/*
	**	see if it was ever properly closed
	*/
	if (VInfo.v_nwrite == 1) {
		/*
		**	freshly created - delete
		*/
		vbuf.vb_use = VVU_IDLE;
		VInfo.v_use = VVU_IDLE;
		(void)UnIndexVV(id);
		vbuf.vb_status = VVS_DELETED;
		SetTime(vbuf.vb_delete, Now);
		WriteVV(id);
		(void)DiskRecycle(&VInfo.v_vv);
		return;
	}
	/*
	**	it has at least one valid session - 
	**	mark finished in table
	*/
	vbuf.vb_status = VVS_FINISHED;
	VInfo.v_status = VVS_FINISHED;
	SetTime(vbuf.vb_finish, Now);
	VInfo.v_finish = Now;
	(void) WriteVV(id);

	/*
	**	add to index of VV's to be written to tape
	*/
	(void)IndexVVix(id, vvname, "tp", db_vv_tp_idx);

	nLogMsg(EL_STATUS1, id, "FINISHED %s id %s", vvname,
					IdString(VInfo.v_id));
}

/*
**	FinishIncompletes() - on startup, mark all openwrite vv's
**		that were not closed finished
*/
  static void
FinishIncompletes()
{
	char	*id = "FinishIncompletes";	
	DBC	*cursor;

	if (dberr = db_vv_wr_idx->cursor(db_vv_wr_idx, NULL, &cursor, 0)) {
		nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
						db_strerror(dberr));
	}
	while (1) {
		ZERO_OBJ(data);
		if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
			if (dberr == DB_NOTFOUND)
				break;
			nLogMsg(EL_FATAL, id, "cursor->c_get(): %s", 
							db_strerror(dberr));
		}
		COPY_DBT(vv_addr, data);
		memcpy(vvkey, key.data, (size_t)key.size);
		vvkey[key.size] = '\0';
		CleanUpIncompleteWrite(id);
	}
	cursor->c_close(cursor);
}


/*************************************************************************
			EXTERNALLY CALLABLE ROUTINES
 *************************************************************************/

  void
DBInit()
{
	char		*id = "DBInit";
	int		i, ret, doindexes;
	int		cflag = O_RDWR|O_SYNC|O_CREAT;
	OFF_T		addr;
	VVMClass_t	*cip;
	FILE		*fp;

	/*
	**	make sure compiled to use records that won't overlap
	**	page boundary
	*/
	if (sizeof(vbuf) != 512)
		nLogMsg(EL_FATAL, id, 
				"db design err: vv record size (%d) != 512", 
				sizeof(vbuf));
	if (sizeof(vsnbuf) != 256)
		nLogMsg(EL_FATAL, id, 
				"db design err: vsn record size (%d) != 128", 
				sizeof(vsnbuf));
	memset(&vsnbuf, ' ', sizeof(vsnbuf));

	/*
	**	set up slot ptrs in the raw VV record
	*/
	cslots[0] = (ClassSlot_t *) vbuf.vb_vsn0;
	cslots[1] = (ClassSlot_t *) vbuf.vb_vsn1;
	cslots[2] = (ClassSlot_t *) vbuf.vb_vsn2;
	cslots[3] = (ClassSlot_t *) vbuf.vb_vsn3;

	/*
	**	check dbase files & create if necc
	*/
	nLogMsg(EL_STATUS1, id, "DB dir is %s", vvDBDir);

	(void)sprintf(dbf_lock, "%s/DB_LOCK", vvDBDir);

	(void)sprintf(dbf_vv_tbl, "%s/DB_VV_TBL", vvDBDir);
	(void)sprintf(dbf_vv_hot_idx, "%s/DB_VV_HOT_IDX", vvDBDir);
	(void)sprintf(dbf_vv_vsn_idx, "%s/DB_VV_VSN_IDX", vvDBDir);
	(void)sprintf(dbf_vv_id_idx, "%s/DB_VV_ID_IDX", vvDBDir);
	(void)sprintf(dbf_vv_tp_idx, "%s/DB_VV_TP_IDX", vvDBDir);
	(void)sprintf(dbf_vv_tp2_idx, "%s/DB_VV_TP2_IDX", vvDBDir);
	(void)sprintf(dbf_vv_wr_idx, "%s/DB_VV_WR_IDX", vvDBDir);
	(void)sprintf(dbf_vv_vol_idx, "%s/DB_VV_VOL_IDX", vvDBDir);

	if (STAT(dbf_lock, &sbuf) != -1) {

		/*
		**	dbase wasn't closed properly - 
		**	prepare to rebuild indexes
		*/
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
				"DBASE NOT CLOSED - REBUILDING INDEXES");
		UnlinkVVIndexes();
		UnlinkVsnIndexes();
	} else {

		/*
		**	db about to be set up, so lock
		*/
		if ((ret=open(dbf_lock, O_CREAT)) == -1)
			nLogMsg(EL_FATAL|EL_SYS, id, "create lock %s failed",
					(errno == ENOENT ? dbf_lock : ""));
		close(ret);
	}

	nLogMsg(EL_DEBUG3, id, "open tables & indexes:");

	/*
	**	open vv table and associated indexes
	*/
	if (STAT(dbf_vv_tbl, &sbuf) != -1) {

		/*
		**	vv table exists - open, 
		**	check size & check 1st record
		*/

		if ((db_vv_fd = open(dbf_vv_tbl, O_RDWR|O_SYNC, 0))== -1)
			nLogMsg(EL_FATAL|EL_SYS, id, 
				"vv table (%s) open failed: %s", dbf_vv_tbl);
		if ((addr = LSEEK(db_vv_fd, 0, SEEK_END)) == -1) 
			nLogMsg(EL_FATAL|EL_SYS, id, 
					"table LSEEK (%s) failed: %s", 
					dbf_vv_tbl);
		if (addr % sizeof(vbuf))	/* consistency check */
		  nLogMsg(EL_FATAL, id, 
		 "table (%s) size %lld inconsistent with recsize %d: diff %d",
					dbf_vv_tbl, (long long)addr,
					sizeof(vbuf), addr % sizeof(vbuf));
		vv_addr = 0;
		ret = LoadVV(id, 1);
		if (ret  &&  ret != TBL_EOF)
			nLogMsg(EL_FATAL, id, 
				    "vv table %s:\n\n\t\t%s\n\n", dbf_vv_tbl,
				    "1st record read problem");
	} else {

		/*
		**	vv table doesn't exist
		*/

		if (errno != ENOENT)
			nLogMsg(EL_FATAL|EL_SYS, id, "table stat problem: %s:",
							dbf_vv_tbl);
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
					"CREATING VV TABLE %s AND INDEXES", 
					dbf_vv_tbl);
		if ((db_vv_fd = open(dbf_vv_tbl, cflag, 0644))== -1)
			nLogMsg(EL_FATAL|EL_SYS, id, 
				"vv table (%s) open/create failed:", 
								dbf_vv_tbl);
		UnlinkVVIndexes();
	}
	doindexes =  StartVVIndexes();

	/*
	**	open vsn table and associated indexes
	*/
	(void)sprintf(dbf_vsn_tbl, "%s/DB_VSN_TBL", vvDBDir);
	(void)sprintf(dbf_vsn_hot_idx, "%s/DB_VSN_HOT_IDX", vvDBDir);
	(void)sprintf(dbf_vsn_recyc_idx, "%s/DB_VSN_RECYC_IDX", vvDBDir);
	(void)sprintf(dbf_vsn_vsn_idx, "%s/DB_VSN_VSN_IDX", vvDBDir);

	if (STAT(dbf_vsn_tbl, &sbuf) != -1) {

		/*
		**	vsn table exists - open, check size 
		**	& check 1st record
		*/

		if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDWR|O_SYNC, 0))== -1)
			nLogMsg(EL_FATAL|EL_SYS, id, 
				"vsn table (%s) open failed: %s", dbf_vsn_tbl);
		if ((addr = LSEEK(db_vsn_fd, 0, SEEK_END)) == -1) 
			nLogMsg(EL_FATAL|EL_SYS, id, 
					"vsn table LSEEK (%s) failed: %s", 
					dbf_vsn_tbl);
		if (addr % sizeof(vsnbuf))	/* consistency check */
		  nLogMsg(EL_FATAL, id, 
		 "table (%s) size %lld inconsistent with recsize %d: diff %d",
					dbf_vsn_tbl, (long long)addr,
					sizeof(vsnbuf), addr % sizeof(vsnbuf));
		vsn_addr = 0;
		ret = LoadVsn(id);
		if (ret  &&  ret != TBL_EOF)
			nLogMsg(EL_FATAL, id, 
				    "vsn table %s:\n\n\t\t%s\n\n", dbf_vsn_tbl,
				    "1st record read problem");
	} else {

		/*
		**	vsn table doesn't exist
		*/

		if (errno != ENOENT)
			nLogMsg(EL_FATAL|EL_SYS, id, "table stat problem: %s:",
							dbf_vsn_tbl);
		nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
					"CREATING VSN TABLE %s AND INDEXES", 
					dbf_vsn_tbl);
		if ((db_vsn_fd = open(dbf_vsn_tbl, cflag, 0644))== -1)
			nLogMsg(EL_FATAL|EL_SYS, id, 
				"vsn table (%s) open/create failed:", 
								dbf_vsn_tbl);
		UnlinkVsnIndexes();
	}

	doindexes += StartVsnIndexes();

	/*
	**	check class info files, updating each class's lastnum
	*/
	for (i=0; cip=VVMClassIndex(i); i++)
		doindexes += Init_ReadClass(cip);

	if (doindexes) {
		/*
		**	clear all away for simple, 1-process build
		*/
		UnlinkVVIndexes();
		for (i=0; cip=VVMClassIndex(i); i++) {
			(void)sprintf(dbf_classes, "%s/DB_CLASS_%s", vvDBDir, 
						cip->vc_class.cs_prefix);
			Unlink(dbf_classes);
			memset(cip->vc_lastnum, ' ', sizeof(cip->vc_lastnum));
			cip->vc_lastnum[sizeof(cip->vc_lastnum)-1] = '0';
		}
		RebuildVVIndexes();
		for (i=0; cip=VVMClassIndex(i); i++) {
			nLogMsg(EL_STATUS1, id, 
					"writing class %s serial: [%*.*s]",
						cip->vc_class.cs_prefix, 
						VV_SERIALLEN, VV_SERIALLEN,
						cip->vc_lastnum);
			WriteClass(cip);
		}

		UnlinkVsnIndexes();
		RebuildVsnIndexes();
	}

	/*
	**	take care of client writes-in-progress at time
	**	of last termination
	*/
	FinishIncompletes();

	/*
	**	mark all hot vsns non-busy
	*/
	ResetHotVsns();

	/*
	**	make in-memory 'blob' index for request tracking
	*/
	MakeBlobIx();

	dbopen++;
	nLogMsg(EL_STATUS2, "DBASE", "database started");
}


  void
DBClose()
{
	char	*id = "DBClose";

	if (!dbopen)
		return;

	close(db_vv_fd);

	/*
	**	if index close fails, leave lock file so
	**	indexes will be rebuilt
	*/
	if (!CloseIndexes())
		if (unlink(dbf_lock) == -1)
			nLogMsg(EL_CONSOLE0|EL_SYS, id, "unlink lock file (%s)",
								dbf_lock);
	dbopen = 0;
	nLogMsg(EL_STATUS1, id, "Database shut down");
}

  VVInfo_t *
DBLookupVV(VV_t *vvp)
{
	char	*id = "DBLookupVV";
	char	*vvname = VV2Name(vvp);

	if (vvname == NULL) {
		VVError = VVE_USER;
		return(NULL);
	}

	STR_DBT(key, vvname);
	ZERO_OBJ(data);
	if (dberr = db_vv_vol_idx->get(db_vv_vol_idx, NULL, &key, &data, 0)) {
		if (dberr == DB_NOTFOUND) {
			VVError = VVE_USER;
			nLogMsg(EL_CODE|EL_ERROR1, 0, "%s: not found", vvname);
		} else {
			VVError = VVE_DBASE;
			nLogMsg(EL_CODE|EL_ERROR1, 0, " db get(%s): %s", 
						vvname, db_strerror(dberr));
		}
		return(NULL);
	}
	COPY_DBT(vv_addr, data);
	if (LoadVV(id, 1))
		return(NULL);
	return(&VInfo);
}


/*
**	FixVVStatus() - a currently-loaded vv is wrongly indexed;
**		check/fix status - 
**		note: this is not comprehensive - only for known cases
**			and obvious extrapolations, so call with care
*/
static void
FixVVStatus(char *vvname)
{
	char		*id = "FixHotStatus";
	int		i, tapeq, writevv = 0, incomplete = 0, problemvsn = 0;

	/*
	**	see if VV is completely saved
	*/
	for (i=0; i<VInfo.v_nvsns; i++) {
		if (VInfo.v_vsns[i].v_vsn[0] == ' ') {
			incomplete++;
			continue;
		}
		if (cslots[i]->vb_vflags[1] != VSN_OK)
			problemvsn++;
	}

	/*
	**	see if in q-to-tape ix
	*/
	STR_DBT(key, vvname);
	ZERO_OBJ(data);
	dberr = db_vv_tp_idx->get(db_vv_tp_idx, NULL, &key, &data, 0);
	if (dberr == 0)
		tapeq = 1;
	else if (dberr == DB_NOTFOUND)
		tapeq = 0;
	else
		nLogMsg(EL_FATAL, id, "db tp get(%s): %s", vvname,
						db_strerror(dberr));

	nLogMsg(EL_STATUS1, id, "%s: %s  novsn %d  notokvsn %d  %s",
			vvname, vvStatusStr(VInfo.v_status),
			incomplete, problemvsn, 
			(tapeq ? "in tp Q" : "not in tp Q"));

	/*
	**	fix vv
	*/
	switch (VInfo.v_status) {
	    case VVS_FINISHED:
		if (!incomplete  &&  !problemvsn) {
			nLogMsg(EL_STATUS1, id, 
				"finished but completely saved %s %s",
				"-marking migrated",
				(tapeq ? "and deleting from tp Q" : ""));
			if (tapeq) 
				(void)UnIndexVVix(id, vvname, "tp", 
								db_vv_tp_idx);
			vbuf.vb_status = VVS_MIGRATED;
			writevv = 1;
		} else if (!problemvsn  &&  !tapeq) {
			/*
			**	Q to write tape; worry about vsnproblem later
			*/
			nLogMsg(EL_STATUS1, id, " adding %s to tp Q ix",
								vvname);
			(void)IndexVVix(id, vvname, "tp", db_vv_tp_idx);
		}
		break;
	    case VVS_MIGRATED:
		if (incomplete  ||  problemvsn  ||  tapeq)
			nLogMsg(EL_ERROR1, id, "need to add for VVS_MIGRATED");
		break;
	    default:
		nLogMsg(EL_ERROR1, id, "need to add case?");
	}

	/*
	**	update table entry
	*/
	if (writevv)
		WriteVV(id);
}

  VVInfo_t *
DBFindHotVV(char *xid, OFF_T size, VVMClass_t *cip, int clntpid, char *hostname)
{
	char		*id = "DBFindHotVV";
	char		vvname[VV_NAMELEN];
	int		i, ret, not_enough = 0;
	size_t		keylen;
	VVInfo_t	*vip = NULL;
	DBC		*cursor;

	/*
	**	open cursor
	*/
	if (dberr = db_vv_hot_idx->cursor(db_vv_hot_idx, NULL, &cursor, 0)) {
		nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
						db_strerror(dberr));
	}

	/*
	**	prepare id/class search key
	*/
	(void)sprintf(vvkey, "%s_%s", cip->vc_class.cs_prefix,
					  IdString(xid));
	STR_DBT(key, vvkey);
	keylen = (size_t)key.size;

	/*
	**	look at all matching hot vv's
	*/
	for (i=0;;i++) {
		OFF_T	vvdata, maxdata, fsize;

		/*
		**	get next hot rec
		*/
		ZERO_OBJ(data);
		if (!i) 
			dberr =cursor->c_get(cursor, &key, &data, DB_SET_RANGE);
		else
			dberr = cursor->c_get(cursor, &key, &data, DB_NEXT);
		if (dberr == DB_NOTFOUND)
			break;
		if (dberr)
			nLogMsg(EL_FATAL, id, "db cursor c_get(%s): %s",
						vvkey, db_strerror(dberr));
		if (memcmp(vvkey, key.data, keylen))
			break;

		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 1))
			return(NULL);
		(void)strcpy(vvname, VV2Name(Vbuf2VV()));

		/*
		**	make double-sure vv can still be written
		*/
		if (VInfo.v_status != VVS_HOT) {
			nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
						"%s: closed for writing (%s)",
						vvname, 
						vvStatusStr(VInfo.v_status));

			if (dberr = cursor->c_del(cursor, 0))
				nLogMsg(EL_FATAL, id, "c_del((%.*s)",
							key.size, key.data);

			/*
			**	make sure vv is taken care of & continue
			*/
			FixVVStatus(vvname);
			continue;
		}

		/*
		**	make sure vv is inactive
		*/
		nLogMsg(EL_DEBUG1, id, "%s use %s", vvname,
							vvUseStr(vbuf.vb_use));
		if (VInfo.v_use != VVU_IDLE)
			continue;

		/*
		**	make sure intended amount to add is ok
		*/
		C2BIN(HexOFF, &vvdata, VInfo.v_data);
		C2BIN(HexOFF, &maxdata, VInfo.v_maxdata);
		if (vvdata + size > maxdata) {
			not_enough++;
			continue;
		}

		/*
		**	looks good - check disk to be sure
		*/
		if (DiskResidentSize(&VInfo.v_vv, &fsize)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"HOT %s not on disk!", vvname);
			/*
			**	leave it hot to keep generating
			**		these msgs until someone
			**		notices & solves the underlying
			**		problem
			*/
			continue;
		}
		if (vvdata != fsize) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
				"HOT %s disk/db sizes %lld/%lld disagree!",
				vvname, (long long)fsize, (long long)vvdata);
			/*
			**	update record
			*/
			C2ASC(SetHexOFF, VInfo.v_data, fsize);
			memcpy(vbuf.vb_data, VInfo.v_data, HEXLEN);
			WriteVV(id);

			/*
			**	probably should finish it now, but
			**		easy way out = just skip it
			*/
			continue;
		}

		/*
		**	it works - mark as being written
		*/
		if (IndexVVix(id, vvname, "wr", db_vv_wr_idx))
			break;	/* give up */

		/*
		**	keep track of client relation to request
		*/
		if (IndexClient(id, clntpid, hostname))
			break;	/* give up */

		/*
		**	update/write record
		*/
		vbuf.vb_use = VVU_WRITING;
		VInfo.v_use = VVU_WRITING;
		SetTime(vbuf.vb_write, Now);
		VInfo.v_write = Now;
		if (IncDecInt(vbuf.vb_nwrite, sizeof(vbuf.vb_nwrite)))
			nLogMsg(EL_FATAL, id, "IncDecInt()");
		VInfo.v_nwrite++;
		if (WriteVV(id))
			break;	/* give up */
		nLogMsg(EL_STATUS2, id, "OPENWRITE %s id %s size %lld", 
					vvname,
					IdString(VInfo.v_id), (long long)size);

		/*
		**	set vip to register success
		*/
		vip = &VInfo;
		break;
	}
	cursor->c_close(cursor);

	/*
	**	nothing
	*/
	if (not_enough)
		nLogMsg(EL_STATUS2, id, 
				"id %d class %s:  %d w/ not enough space",
				IdString(xid), cip->vc_class.cs_name, 
				not_enough);
	return(vip);
}

  VVInfo_t *
DBCreateHotVV(char *xid, OFF_T size, VVMClass_t *cip, 
			int clntpid, char *hostname)
{
	char		*id = "DBCreateHotVV";
	int		i, ret;
	char		num[2*INTLEN];

	/*
	**	create new vv id
	*/
	if (IncClass(cip))
		return(NULL);

	/*
	**	build db record
	*/
	memset(&vbuf, ' ', sizeof(vbuf));
	vbuf.end = '\n';
	C2ASC(SetDecInt, vbuf.vb_version, VV_VERSION);
	memcpy(vbuf.vb_class, cip->vc_class.cs_prefix, 
				strlen(cip->vc_class.cs_prefix));
	memcpy(vbuf.vb_serial, cip->vc_lastnum, VV_SERIALLEN);
	memcpy(vbuf.vb_id, xid, VV_IDLEN);
nLogMsg(EL_STATUS1, id, "creating %s id [%*.*s]", VV2Name(Vbuf2VV()),
VV_IDLEN, VV_IDLEN, vbuf.vb_id);
	vbuf.vb_status = VVS_HOT;
	vbuf.vb_use = VVU_WRITING;
	C2ASC(SetDecInt, vbuf.vb_nwrite, 1);
	SetTime(vbuf.vb_creat, Now);
	SetTime(vbuf.vb_write, Now);
	C2ASC(SetDecInt, vbuf.vb_nread, 0);
	C2ASC(SetDecInt, vbuf.vb_nrestore, 0);
	C2ASC(SetDecInt, vbuf.vb_mntfail, 0);
	C2ASC(SetHexOFF, vbuf.vb_data, 0);
	C2ASC(SetHexOFF, vbuf.vb_maxdata, cip->vc_class.cs_vvsize);
	SetTime(vbuf.vb_save, -1);
	SetTime(vbuf.vb_read, -1);
	SetTime(vbuf.vb_finish, -1);
	SetTime(vbuf.vb_restore, -1);
	SetTime(vbuf.vb_delete, -1);

	/* in case class redefined later.. */
	vbuf.vb_nvsns = INT2CHAR(cip->vc_class.cs_nvsns);

	for (i=0; i<cip->vc_class.cs_nvsns; i++)
		C2ASC(SetDecInt, cslots[i]->vb_nrec, -1);

	if (Vbuf2VInfo(id))
		return(NULL);

	/*
	**	get pos of db record at end of file
	*/
	vv_addr = LSEEK(db_vv_fd, 0, SEEK_END);
	if (vv_addr == -1) {
		VVError = VVE_DBASE;
		nLogMsg(EL_SYS|EL_CODE|EL_CONSOLE0, id, "LSEEK");
		return(NULL);
	}
	if (vv_addr % sizeof(vbuf) != 0) {
		VVError = VVE_DBASE;
		nLogMsg(EL_SYS|EL_CONSOLE0, id, "table size not consistent");
		return(NULL);
	}

	/*
	**	add to indexes
	*/
	if (IndexVV(id))
		return(NULL);
	if (IndexClient(id, clntpid, hostname))
		return(NULL);

	/*
	**	no index conflicts - write vv record via AddOndisk()
	*/
	AddOndisk(id, Now);

	nLogMsg(EL_STATUS1, id, "CREATED %s id %s to write %lld",
						VV2Name(&VInfo.v_vv),
						IdString(xid), (long long)size);

	return(&VInfo);
}


  int
DBCloseWriteVV(VV_t *vvp, OFF_T fsize, int clntpid, char *hostname)
{
	char		*id = "DBCloseWrite";
	char		vvname[VV_NAMELEN];
	OFF_T		vdata;
	VVInfo_t	*vip;

	if (DBLookupVV(vvp) == NULL)
		return(1);

	(void)strcpy(vvname, VV2Name(Vbuf2VV()));

	if (VInfo.v_use != VVU_WRITING) {
		VVError = VVE_USER;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%s: not openwrite (%s)",
					vvname, vvUseStr(VInfo.v_use));
		return(1);
	}

	vbuf.vb_use = VVU_IDLE;
	VInfo.v_use = VVU_IDLE;
	(void)UnIndexVVix(id, vvname, "wr", db_vv_wr_idx);
	avl_entry->recptr = (void *)nAlloc(sizeof(vv_addr));
	*((OFF_T *)avl_entry->recptr) = vv_addr;
	(void)sprintf(avl_entry->key, "%d_%s", clntpid, hostname);
	if (avl_find_exact(avl_entry, &avl_blob_idx) == AVL_IX_FAIL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
				"%s: avl_find_exact(%s) failed", 
				vvname, avl_entry->key);
	} else {
		nFree(avl_entry->recptr);
		if (avl_delete_key(avl_entry, &avl_blob_idx) == AVL_IX_FAIL) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
				"%s: avl_delete_key(%s) failed", 
				vvname, avl_entry->key);
		}
	}

	C2BIN(HexOFF, &vdata, VInfo.v_data);
	if (fsize != -1  &&  fsize < vdata) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
				"HOT %s diminished in size (%lld -> %lld)",
				vvname, (long long)vdata, (long long)fsize);
	} else {
		nLogMsg(EL_STATUS2, id,
				"HOT %s size (%lld -> %lld)",
				vvname, (long long)vdata, (long long)fsize);
		C2ASC(SetHexOFF, vbuf.vb_data, fsize);
	}
	SetTime(vbuf.vb_write, Now);

	if (WriteVV(id))
		return(1);
	fsize = fsize - vdata;
	nLogMsg(EL_STATUS2, id, "CLOSEWRITE %s id %s added %lld", 
					vvname,
					IdString(VInfo.v_id), (long long)fsize);
	return(0);
}


  void
DBRecoverClient(int clntpid, char *hostname)
{
	char	*id = "DBRecoverClient";

	if (!dbopen)
		return;

	/*
	**	CleanUpIncompleteWrite() needs a global key
	*/
	(void)sprintf(vvkey, "%d_%s", clntpid, hostname);
	(void)strcpy(avl_entry->key, vvkey);
	while (avl_find_key(avl_entry, &avl_blob_idx) != AVL_IX_FAIL) { 
		vv_addr = *((OFF_T *) avl_entry->recptr);
		nFree(avl_entry->recptr);
		if (avl_delete_key(avl_entry, &avl_blob_idx) == AVL_IX_FAIL)
			nLogMsg(EL_FATAL, id, "avl_delete_key(%s)", 
						avl_entry->key);
		CleanUpIncompleteWrite(id);
	}
}

  VVInfo_t *
DBOpenReadVV(VV_t *vvp)
{
	char	*id = "DBOpenRead";

	if (DBLookupVV(vvp) == NULL)
		return(NULL);

	SetTime(vbuf.vb_read, Now);
	VInfo.v_nread++;
	C2ASC(SetDecInt, vbuf.vb_nread, VInfo.v_nread);
	if (WriteVV(id))
		return(NULL);
	nLogMsg(EL_STATUS2, id, "OPENREAD %s id %s", 
				VV2Name(&VInfo.v_vv),
				IdString(VInfo.v_id));
	return(&VInfo);
}

/*
**	DBRestored() - VV restored to disk: update stats
*/
  void
DBRestored(VV_t *vvp)
{
	char	*id = "DBRestored";

	if (DBLookupVV(vvp) == NULL)
		return;

	SetTime(vbuf.vb_restore, Now);
	VInfo.v_nrestore++;
	C2ASC(SetDecInt, vbuf.vb_nrestore, VInfo.v_nrestore);

	/*
	**	if this restore was to recreate a lost tape copy, 
	**		mark entry as normal 'still saving to tape'
	**		now that it's on disk
	*/
	if (vbuf.vb_status == VVS_RECOVER)
		vbuf.vb_status = VVS_FINISHED;

	/*
	**	add to on-disk list; use Now as approx atime
	**		AddOndisk() writes record to table
	*/
	AddOndisk(id, Now);
}

  VVInfo_t *
DBFinishVV(VV_t *vvp)
{
	char		*id = "DBFinishVV";
	char		vvname[VV_NAMELEN];
	VVInfo_t	*vip;

	if (DBLookupVV(vvp) == NULL)
		return(NULL);

	(void)strcpy(vvname, VV2Name(Vbuf2VV()));

	if (VInfo.v_use != VVU_IDLE) {
		VVError = VVE_UNAVAIL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%s: in use (%s)",
				vvname, vvUseStr(VInfo.v_use));
		return(NULL);
	}

	if (VInfo.v_status != VVS_HOT) {
		VVError = VVE_USER;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"%s: not in appending state (%c)",
				vvname, vvStatusStr(VInfo.v_status));
		return(NULL);
	}

	/*
	**	delete from hot index
	*/
	(void)sprintf(vvkey, "%s_%s_%llx", ClassString(VInfo.v_vv.class),
				IdString(VInfo.v_id), (long long)vv_addr);
	STR_DBT(key, vvkey);
	if (dberr = db_vv_hot_idx->del(db_vv_hot_idx, NULL, &key, 0)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id,
					"%s: del(class_id %s): %s", 
					vvname, vvkey, db_strerror(dberr));
		return(NULL);
	}

	/*
	**	mark in table
	*/
	vbuf.vb_status = VVS_FINISHED;
	VInfo.v_status = VVS_FINISHED;
	SetTime(vbuf.vb_finish, Now);
	VInfo.v_finish = Now;

	if (WriteVV(id))
		return(NULL);

	/*
	**	add to index of VV's to be written to tape
	*/
	(void)IndexVVix(id, vvname, "tp", db_vv_tp_idx);

	nLogMsg(EL_STATUS1, id, "FINISHED %s id %s", 
				vvname, IdString(VInfo.v_id));

	return(&VInfo);
}

 int 
DBSavedVV(Vsn_t *vp, VVInfo_t *vip, int slot, VVMClass_t *cip)
{
	char		*id = "DBSavedVV";
	char		*vsnp = vp->vs_vsn;
	char		vvname[VV_NAMELEN];
	int		i, dbdone = 0;

	(void)strcpy(vvname, VV2Name(&vip->v_vv));
	nLogMsg(EL_STATUS1, id, "%s slot %d: vsn %.*s",
				vvname, slot, VSNLEN, vsnp);
	/*
	**	update vsn 1st
	*/
	if (DBLookupVsn(vsnp) == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: Can't update/find vsn record %.*s",
					vvname, VSNLEN, vsnp);
	} else {
nLogMsg(EL_STATUS1, id, "set vs_livevv %d vs_delvv %d",
vp->vs_livevv, vp->vs_delvv);
		memcpy(vsnbuf.vb_lastclass, vp->vs_lastvv.class, VV_CLASSLEN);
		memcpy(vsnbuf.vb_lastserial, vp->vs_lastvv.serial,VV_SERIALLEN);
		if (C2ASC(SetDecInt, vsnbuf.vb_lastrec, vp->vs_lastrec)	     ||
	    	    C2ASC(SetDecInt, vsnbuf.vb_livevv, vp->vs_livevv)	     ||
	    	    C2ASC(SetDecInt, vsnbuf.vb_delvv, vp->vs_delvv)	     ||
	    	    C2ASC(SetDecInt, vsnbuf.vb_errsinarow, 0)		     ||
		    C2ASC(SetDecUlong, vsnbuf.vb_lastblk, vp->vs_lastblk)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: Can't update/set vsn record %.*s",
					vvname, VSNLEN, vsnp);
		} else if (WriteVsn(id)) {
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
				"%s: Can't update/write vsn record %.*s",
				vvname, VSNLEN, vsnp);
		} else
nLogMsg(EL_STATUS1, id, "%.*s: updated lastvv %s", 
VSNLEN, vsnp, VV2Name(&vp->vs_lastvv));
	}

	/*
	**	update VV
	*/
	if (DBLookupVV(&vip->v_vv) == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: Can't update/find VV record",
					vvname);
		return(0);
	}
	if (VInfo.v_vsns[slot].v_vsn[0] != ' ') {
		nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
				"%s: vsn %.*s slot %d already has %.*s",
				vvname, VSNLEN, vsnp, 
				VSNLEN, VInfo.v_vsns[slot].v_vsn);
		return(0);
	}

	/*
	**	add vsn to 'exportable' struct just for indexed check
	**		for completion below
	*/
	memcpy(VInfo.v_vsns[slot].v_vsn, vip->v_vsns[slot].v_vsn, VSNLEN);

	/*
	 *  add all info on this slot to db record for writing
	 */
	memcpy(cslots[slot]->vb_vsn, vip->v_vsns[slot].v_vsn, VSNLEN);
	if (C2ASC(SetDecInt, cslots[slot]->vb_nrec, vip->v_vsns[slot].v_nrec) ||
	    C2ASC(SetDecUlong, cslots[slot]->vb_nblk, 
					vip->v_vsns[slot].v_block)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: can't set record", vvname);
		return(0);
	}
	SetTime(cslots[slot]->vb_vdate,  vip->v_vsns[slot].v_date);
	cslots[slot]->vb_vflags[1] = VSN_OK;

	/*
	**	add vv to vv/vsn index
	*/
	if (IndexVVVsn(id, vsnp))
		return(0);

	/*
	**	see if VV is completely saved
	*/
	for (i=0; i<cip->vc_class.cs_nvsns; i++) {
		if (VInfo.v_vsns[i].v_vsn[0] == ' ')
			break;
	}
	if (i == cip->vc_class.cs_nvsns) {
		/*
		**	no need to keep trying to write this VV
		*/
		nLogMsg(EL_STATUS1, id, "%s: MIGRATED", vvname);
		dbdone = 1;
		vbuf.vb_status = VVS_MIGRATED;
		vip->v_status = VVS_MIGRATED;
		(void)UnIndexVVix(id, vvname, "tp", db_vv_tp_idx);
		/*
		**	add to on-disk list; use Now as approx atime
		**		AddOndisk() does WriteVV()
		*/
		AddOndisk(id, Now);
	} else
		WriteVV(id);

	return(dbdone);
}

#ifdef HACK_NOVSN
 int 
HackDBSavedVV(VVInfo_t *vip, int slot, VVMClass_t *cip)
{
	char		*id = "HackDBSavedVV";
	char		vvname[VV_NAMELEN];
	int		i, dbdone = 0;

	(void)strcpy(vvname, VV2Name(&vip->v_vv));
	nLogMsg(EL_STATUS1, id, "%s slot %d: hackvsn", vvname, slot);

	/*
	**	update VV
	*/
	if (DBLookupVV(&vip->v_vv) == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: Can't update/find VV record",
					vvname);
		return(0);
	}
	if (VInfo.v_vsns[slot].v_vsn[0] != ' ') {
		nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
				"%s: slot %d already has %.*s",
				vvname, VSNLEN, VInfo.v_vsns[slot].v_vsn);
		return(0);
	}

	/*
	 *  add vsn to 'exportable' struct just for indexed check
	 *	for completion below
	 */
	memcpy(VInfo.v_vsns[slot].v_vsn, vip->v_vsns[slot].v_vsn, VSNLEN);

	/*
	 *  add all info on this slot to db record for writing
	 */
	memcpy(cslots[slot]->vb_vsn, vip->v_vsns[slot].v_vsn, VSNLEN);
	if (C2ASC(SetDecInt, cslots[slot]->vb_nrec, vip->v_vsns[slot].v_nrec) ||
	    C2ASC(SetDecUlong, cslots[slot]->vb_nblk, 
					vip->v_vsns[slot].v_block)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"%s: can't set record", vvname);
		return(0);
	}
	SetTime(cslots[slot]->vb_vdate,  vip->v_vsns[slot].v_date);
	cslots[slot]->vb_vflags[1] = VSN_OK;

	/*
	 *  add vv to vv/vsn index
	 */
	if (IndexVVVsn(id, "%%%%%%"))
		return(0);

	/*
	 *  see if VV is completely saved
	 */
	for (i=0; i<cip->vc_class.cs_nvsns; i++) {
		if (VInfo.v_vsns[i].v_vsn[0] == ' ')
			break;
	}
	if (i == cip->vc_class.cs_nvsns) {
		/*
		 *  no need to keep trying to write this VV
		 */
		nLogMsg(EL_STATUS1, id, "%s: MIGRATED", vvname);
		dbdone = 1;
		vbuf.vb_status = VVS_MIGRATED;
		vip->v_status = VVS_MIGRATED;
		(void)UnIndexVVix(id, vvname, "tp", db_vv_tp_idx);
		/*
		**	add to on-disk list; use Now as approx atime
		**		AddOndisk() does WriteVV()
		*/
		AddOndisk(id, Now);
	} else
		WriteVV(id);

	return(dbdone);
}
#endif /* HACK_NOVSN */

  void
DBFreedVV(VVInfo_t *vip)
{
	char		*id = "DBFreedVV";

	if (DBLookupVV(&vip->v_vv) == NULL)
		return;
	SetTime(vbuf.vb_freed, Now);
	WriteVV(id);
}

  int
DBRecycleVV(VV_t *vvp)
{
	char		*id = "DBRecycleVV";
	VVInfo_t	*vip;
	int		i, nvsns;

	if (DBLookupVV(vvp) == NULL)
		return(1);

	if (VInfo.v_status == VVS_DELETED) {
		VVError = VVE_UNAVAIL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%s: already deleted", 
						VV2Name(&VInfo.v_vv));
		return(1);
	}

	if (VInfo.v_use != VVU_IDLE) {
		VVError = VVE_UNAVAIL;
		nLogMsg(EL_CODE|EL_ERROR1, id, "%s: in use (%s)",
				VV2Name(&VInfo.v_vv),
				vvUseStr(VInfo.v_use));
		return(1);
	}

	if (VInfo.v_status != VVS_MIGRATED)
		nLogMsg(EL_STATUS1|EL_CONSOLE0, id, 
					"%s: deleting w/ no tape copy",
					VV2Name(&VInfo.v_vv));

	/*
	**	do the delete
	*/
	if (UnIndexVV(id))
		return(1);
	vbuf.vb_status = VVS_DELETED;
	SetTime(vbuf.vb_delete, Now);
	if (WriteVV(id))
		return(1);

	nLogMsg(EL_STATUS1, id, "RECYCLED %s id %s", 
				VV2Name(&VInfo.v_vv),
				IdString(VInfo.v_id));

	/*
	**	update vv count on vsn(s)
	*/
	nvsns = CHAR2INT(vbuf.vb_nvsns);
	for (i=0; i<nvsns; i++) {
		Vsn_t	*vp;
		if (cslots[i]->vb_vflags[1] != VSN_OK)
			continue;
		if (DBLookupVsn(cslots[i]->vb_vsn) == NULL) {
			nLogMsg(EL_ERROR1, id, "%.*s: no vsn table entry",
						VSNLEN, cslots[i]->vb_vsn);
			continue;
		}
		VsnInfo.vs_livevv--;
		VsnInfo.vs_delvv++;
nLogMsg(EL_STATUS1, id, "set vs_livevv %d vs_delvv %d",
VsnInfo.vs_livevv, VsnInfo.vs_delvv);
		if (C2ASC(SetDecInt, vsnbuf.vb_livevv, VsnInfo.vs_livevv)  ||
		    C2ASC(SetDecInt, vsnbuf.vb_delvv, VsnInfo.vs_delvv)) {
			nLogMsg(EL_ERROR1, id, "%.*s: setting live/delvv",
						VSNLEN, cslots[i]->vb_vsn);
			continue;
		}
		(void)WriteVsn(id);
	}
	return(0);
}

/*
**	DBLookupVsn() - look up in vsn tbl
*/
  Vsn_t *
DBLookupVsn(char *vsnp)
{
	char	*id = "DBLookupVsn";

	memcpy(vsnkey, vsnp, VSNLEN);
	vsnkey[VSNLEN] = '\0';
nLogMsg(EL_STATUS1, id, "memcpy ok [%s]", vsnkey);
	STR_DBT(key, vsnkey);
nLogMsg(EL_STATUS1, id, "STR_DBT ok [%.*s]", key.size, key.data);
	ZERO_OBJ(data);
	if (dberr = db_vsn_vsn_idx->get(db_vsn_vsn_idx, NULL, &key, &data, 0)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "db get(%s): %s", vsnkey,
							db_strerror(dberr));
		VVError = VVE_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0, "%s: vsn not found", vsnkey);
		return(NULL);
	}
	COPY_DBT(vsn_addr, data);
nLogMsg(EL_STATUS1, id, "COPY_DBT ok");
	if (LoadVsn(id))
		return(NULL);
	return(&VsnInfo);
}

/*
**	DBAddHotVsn() - add to dbase
*/
  Vsn_t *
DBAddHotVsn(char *vsnp, char *class, int slot)
{
	char	*id = "DBAddHotVsn";

	if (slot >= MAXVSN) {
		VVError = VVE_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0, "slot %d too big", slot);
		return(NULL);
	}
	/*
	**	build db record - unset ints are blank == 0
	*/
	memset(&vsnbuf, ' ', sizeof(vsnbuf));
	vsnbuf.end = '\n';
	memcpy(vsnbuf.vb_vsn, vsnp, VSNLEN);
	memcpy(vsnbuf.vb_class, class, strlen(class));
	vsnbuf.vb_classI = INT2CHAR(slot);
	vsnbuf.vb_status = VVS_HOT;
	vsnbuf.vb_busy = 'b';
	C2ASC(SetDecInt, vsnbuf.vb_lastrec, -1);

	/*
	**	build struct to return
	*/
	memcpy(VsnInfo.vs_vsn, vsnp, VSNLEN);
	memcpy(VsnInfo.vs_class, vsnbuf.vb_class, sizeof(VsnInfo.vs_class));
	VsnInfo.vs_classI = slot;
	VsnInfo.vs_status = VVS_HOT;
	VsnInfo.vs_lastrec = -1;
	VsnInfo.vs_lastblk = 0;
	memset(VsnInfo.vs_lastid, ' ', sizeof(VsnInfo.vs_lastid));
	VsnInfo.vs_errsinarow = 0;
	VsnInfo.vs_toterrs = 0;
	VsnInfo.vs_livevv = 0;

	/*
	**	write to tbl
	*/
	vsn_addr = LSEEK(db_vsn_fd, 0, SEEK_END);
	if (vsn_addr == -1) {
		VVError = VVE_DBASE;
		nLogMsg(EL_SYS|EL_CODE|EL_CONSOLE0, id, "LSEEK vsn tbl");
		return(NULL);
	}
	if (vsn_addr % sizeof(vsnbuf) != 0) {
		VVError = VVE_DBASE;
		nLogMsg(EL_SYS|EL_CONSOLE0, id, "vsn tbl size not consistent");
		return(NULL);
	}
	if (WriteVsn(id))
		return(NULL);

	/*
	**	add to indexes
	*/
	if (IndexVsn(id))
		return(NULL);
	nLogMsg(EL_STATUS1, id, "ADDED hot vsn %.*s", VSNLEN, vsnp);

	return(&VsnInfo);
}


/*
**	DBGetHotVsn() - find an idle hot vsn matching class criteria
**		TODO: maybe find the one w/ the highest lastrec
**		to try to finish sooner.
*/
  Vsn_t *
DBGetHotVsn(char *class, int slot)
{
	char	*id = "DBGetHotVsn";
	int	i;
	size_t	keylen;
	DBC	*cursor;
	Vsn_t	*hotvsn = NULL;

	if (slot >= MAXVSN) {
		VVError = VVE_USER;
		nLogMsg(EL_CODE|EL_ERROR1, 0, "slot %d too big", slot);
		return(NULL);
	}

	/*
	**	set up cursor
	*/
	if (dberr = db_vsn_hot_idx->cursor(db_vsn_hot_idx, NULL, &cursor, 0)) {
		nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
						db_strerror(dberr));
	}
	(void)sprintf(vsnkey, "%s_%d", class, slot);
	STR_DBT(key, vsnkey);
	keylen = (size_t)key.size;
	for (i=0; ; i++) {
		ZERO_OBJ(data);
		if (!i) 
			dberr =cursor->c_get(cursor, &key, &data, DB_SET_RANGE);
		else
			dberr = cursor->c_get(cursor, &key, &data, DB_NEXT);
		if (dberr == DB_NOTFOUND)
			break;
		if (memcmp(key.data, vsnkey, keylen))
			break;
		COPY_DBT(vsn_addr, data);
		if (LoadVsn(id))
			break;

		/*
		**	make sure it's not busy
		*/
		if (vsnbuf.vb_busy == 'b')
			continue;

		/*
		**	mark it as busy
		*/
		vsnbuf.vb_busy = 'b';
		if (WriteVsn(id))
			break;
		hotvsn = &VsnInfo;
		break;
	}
	cursor->c_close(cursor);
	return(hotvsn);
}

/*
**	DBVsnErr() - increment error count for lightweight problems
*/
  void
DBVsnErr(Vsn_t *vp)
{
	char	*id = "DBVsnErr";

	if (DBLookupVsn(vp->vs_vsn) == NULL)
		return;

	/*
	**	increment total - negative if something major happened once
	*/
	if (VsnInfo.vs_toterrs > 0)
		VsnInfo.vs_toterrs++;
	else
		VsnInfo.vs_toterrs--;

	if (C2ASC(SetDecInt, vsnbuf.vb_toterrs, VsnInfo.vs_toterrs))
		return;

	/*
	**	increment the errs-in-a-row count too
	*/
	VsnInfo.vs_errsinarow++;
	if (C2ASC(SetDecInt, vsnbuf.vb_errsinarow, VsnInfo.vs_errsinarow))
		return;

	if (WriteVsn(id))
		return;
	*vp = VsnInfo;
}

  int
DBSetVsnNotHot(Vsn_t *vp, char *errstr)
{
	char	*id = "DBSetVsnNotHot";

nLogMsg(EL_STATUS1, id, "vsn [%.*s]", vp->vs_vsn);
	if (DBLookupVsn(vp->vs_vsn) == NULL)
		return(1);

	vsnbuf.vb_busy = ' ';
	vsnbuf.vb_status = VVS_FINISHED;
	VsnInfo.vs_status = VVS_FINISHED;
	if (errstr != NULL) {
		/*
		**	save err info
		*/
		VsnMsg(errstr, 0);

		/*
		**	start the life-of-vsn err count into the negative
		**		range to signify something serious
		*/
		if (VsnInfo.vs_toterrs > 0)
			VsnInfo.vs_toterrs = -VsnInfo.vs_toterrs - 1;
		else
			VsnInfo.vs_toterrs--;
		if (C2ASC(SetDecInt, vsnbuf.vb_toterrs, VsnInfo.vs_toterrs))
			return(1);

		/*
		**	increment the errs-in-a-row count too
		*/
		VsnInfo.vs_errsinarow++;
		if (C2ASC(SetDecInt, vsnbuf.vb_errsinarow, 
						VsnInfo.vs_errsinarow))
			return(1);
	}
	if (WriteVsn(id))
		return(1);
	*vp = VsnInfo;

	return(UnIndexVsnHot(id));
}

/*
**	UnIndexVsnHot() - assumes vsn currently loaded
*/
  int
UnIndexVsnHot(char *id)
{
	/*
	**	delete from vsn hot index
	*/
	(void)sprintf(vsnkey, "%s_%c_%llx", ClassString(VsnInfo.vs_class), 
					VsnInfo.vs_classI, (long long)vsn_addr);
	STR_DBT(key, vsnkey);
	if (dberr = db_vsn_hot_idx->del(db_vsn_hot_idx, NULL, &key, 0)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, "db del(vsn %s): %s",
						vsnkey, db_strerror(dberr));
		return(1);
	}
	return(0);
}


/*
**	DBGetIncompleteSave() - go thru index of finished VV's that
**		haven't been written to tape.
*/
  VVInfo_t *
DBGetIncompleteSave(int ii)
{
	char		*id = "DBGetIncompleteSave";
	int		i, writevv = 0, incomplete = 0, problemvsn = 0;
	static DBC	*cursor;

	if (ii==0) {
		if (dberr = db_vv_tp_idx->cursor(db_vv_tp_idx, NULL, 
								&cursor, 0))
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
							db_strerror(dberr));
	}

retry:
	ZERO_OBJ(data);
	ZERO_OBJ(key);
	if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
		if (dberr == DB_NOTFOUND) {
			cursor->c_close(cursor);
			return(NULL);
		}
		nLogMsg(EL_FATAL, id, "cursor->c_get(): %s",db_strerror(dberr));
	}
	COPY_DBT(vv_addr, data);
	if (LoadVV(id, 1))
		nLogMsg(EL_FATAL, id, "can't load vv rec");

	/*
	**	check that status agrees w/ ix
	*/
	if (VInfo.v_status != VVS_FINISHED  &&  VInfo.v_status != VVS_RECOVER
	    &&  VInfo.v_status != VVS_RECYCLING)
		nLogMsg(EL_FATAL, id, "not finished/recover: %.*s status %s", 
						key.size, key.data,
						vvStatusStr(VInfo.v_status));

	/*
	**	check if VV is completely saved i.e. status correct
	*/
	for (i=0; i<VInfo.v_nvsns; i++) {
		if (VInfo.v_vsns[i].v_vsn[0] == ' ') {
			incomplete++;
			continue;
		}
		if (cslots[i]->vb_vflags[1] != VSN_OK)
			problemvsn++;
	}
	if (!incomplete  &&  !problemvsn) {
		nLogMsg(EL_ERROR1, id, 
				"%s: %s  but should be 'migrated'?",
				VV2Name(Vbuf2VV()),
				vvStatusStr(VInfo.v_status));
		goto retry;
	}
	return(&VInfo);
}

 int
DBMigrating(VV_t *vvp)
{
	(void)strcpy(vvkey, VV2Name(vvp));
	STR_DBT(key, vvkey);
	ZERO_OBJ(data);
	if (dberr = db_vv_tp_idx->get(db_vv_tp_idx, NULL, &key, &data, 0)) {
		if (dberr == DB_NOTFOUND)
			return(0);
		nLogMsg(EL_FATAL, "DBMigrating", "db get(%s): %s",
						vvkey, db_strerror(dberr));
	}
	return(1);
}
