
/*
 *  vvm_db2.c - dbase queries that cause vvmd to fork
 */

#include "vvmd.h"
#include "vvm_db.h"

#define FILECOPYBUFSIZE	4096

extern int		db_vv_fd;
extern OFF_T		vv_addr;
extern vbuf_t		vbuf;
extern ClassSlot_t	*cslots[];
extern VVInfo_t		VInfo;

extern DBT		key, data;
extern char		vvkey[], vsnkey[];
extern int		dberr;

extern DB		*db_vv_hot_idx, *db_vv_vsn_idx;
extern DB		*db_vv_id_idx, *db_vv_vol_idx;
extern DB		*db_vv_wr_idx, *db_vv_tp_idx;
extern DB		*db_vv_tp2_idx;

extern DB		*db_vsn_hot_idx, *db_vsn_vsn_idx, *db_vsn_recyc_idx;

extern char		dbf_vv_tbl[];
extern char		dbf_vv_hot_idx[];
extern char		dbf_vv_id_idx[];
extern char		dbf_vv_vsn_idx[];
extern char		dbf_vv_vol_idx[];
extern char		dbf_vv_tp_idx[];
extern char		dbf_vv_tp2_idx[];
extern char		dbf_vv_wr_idx[];


extern int		db_vsn_fd;
extern OFF_T		vsn_addr;
extern vsbuf_t		vsnbuf;
extern Vsn_t		VsnInfo;

extern char		dbf_vsn_tbl[];

extern char		dbf_vsn_hot_idx[];

static char		vsn_hot_ix_cpy_name[MAXPATHLEN];
static char		hot_ix_cpy_name[MAXPATHLEN];
static char		vsn_ix_cpy_name[MAXPATHLEN];
static char		id_ix_cpy_name[MAXPATHLEN];
static char		finished_ix_cpy_name[MAXPATHLEN];


static DBC		*cursor = NULL;
static size_t		keylen;
/*************************************************************************
			UTILITY ROUTINES
 *************************************************************************/


/*
 *  CopyIx() - copy the file. Caller is responsible for
 *	flushing index first via close_index()
 */
  static int
CopyIx(char *id, char *in_ixname, char *out_ixname, char *which_ix, 
		int clientpid)
{
	char	buf[FILECOPYBUFSIZE];
	int	ifd, ofd;
	OFF_T	rd, wr;

	/*
	 *  name the tmp file after the client pid
	 */
	(void)sprintf(out_ixname, "/tmp/vvmd_%s.%d", which_ix, clientpid);
	
	if ((ofd = open(out_ixname, O_RDWR|O_CREAT)) == -1) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_SYS, id, "opening index copy %s",out_ixname);
		return(1);
	}

	if ((ifd = open(in_ixname, O_RDONLY)) == -1) {
		VVError = VVE_DBASE;
		nLogMsg(EL_CODE|EL_SYS, id, "opening index file %s", in_ixname);
		close(ofd);
		return(1);
	}
	
	nLogMsg(EL_DEBUG2, id, "copying %s", in_ixname);
	while (1) {
		rd = read(ifd, buf, FILECOPYBUFSIZE);
		if (rd < 1) {
			if (!rd)
				break;
			VVError = VVE_DBASE;
			nLogMsg(EL_CODE|EL_SYS, id, "reading index %s", 
							in_ixname);
			close(ifd);
			close(ofd);
			return(1);
		}
		if ((wr = write(ofd, buf, rd)) < rd) {
			int flg = EL_CODE|EL_ERROR1;
			if (wr == -1)
				flg |= EL_SYS;
			VVError = VVE_DBASE;
			nLogMsg(flg, id, "writing index copy %s", out_ixname);
			close(ifd);
			close(ofd);
			return(1);
		}
	}
	close(ifd);
	close(ofd);
	return(0);
}

  static int
CloseForQuery(int clientpid, char *ixf)
{
	char	*id = "CloseForQuery";

	/*
	**	close the requested index to flush it and copy 
	**		it for the child to query without fear 
	**		of rearrangement by parent
	*/
	if (ixf == dbf_vv_hot_idx) {
		if (dberr = db_vv_hot_idx->close(db_vv_hot_idx, 0))
			nLogMsg(EL_FATAL, id, "db close(%s): %s",
						ixf, db_strerror(dberr));

		return(CopyIx(id, dbf_vv_hot_idx, hot_ix_cpy_name, 
						"hot", clientpid));
	}
	if (ixf == dbf_vv_tp_idx) {
		if (dberr = db_vv_tp_idx->close(db_vv_tp_idx, 0))
			nLogMsg(EL_FATAL, id, "db close(%s): %s",
						ixf, db_strerror(dberr));

		return(CopyIx(id, dbf_vv_tp_idx, finished_ix_cpy_name, 
						"finished", clientpid));
	}
	if (ixf == dbf_vv_vsn_idx) {
		if (dberr = db_vv_vsn_idx->close(db_vv_vsn_idx, 0))
			nLogMsg(EL_FATAL, id, "db close(%s): %s",
						ixf, db_strerror(dberr));

		return(CopyIx(id, dbf_vv_vsn_idx, vsn_ix_cpy_name, 
						"vsn", clientpid));
	}
	if (ixf == dbf_vv_id_idx) {
		if (dberr = db_vv_id_idx->close(db_vv_id_idx, 0))
			nLogMsg(EL_FATAL, id, "db close(%s): %s",
						ixf, db_strerror(dberr));

		return(CopyIx(id, dbf_vv_id_idx, id_ix_cpy_name, 
						"id", clientpid));
	}
	nLogMsg(EL_FATAL, id, "prog err");
	return(1);	/* for compiler warnings */
}


static int
nextcursor(int i, char *id)
{
	ZERO_OBJ(data);
	if (i) {
		ZERO_OBJ(key);
		dberr = cursor->c_get(cursor, &key, &data, DB_NEXT);
	} else
		dberr =cursor->c_get(cursor, &key, &data, DB_SET_RANGE);
	if (dberr == DB_NOTFOUND)
		return(1);
	if (dberr)
		nLogMsg(EL_FATAL, id, "cursor->c_get(): %s",db_strerror(dberr));	return(0);
}

/*
**	renewcursor() - a forked child is recreating/repositioning cursor.
**		Uses exit() instead of EL_FATAL since the latter
**		tries to shut down the whole vvmd.
*/
static void
renewcursor(DB *ixp, char *id)
{
	OFF_T	addr;

	if (dberr = ixp->cursor(ixp, NULL, &cursor, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "db cursor(): %s",
							db_strerror(dberr));
		exit(2);
	}

	/*
	**	skip over index entries till last previous vv
	**		- depends on key remaining the same
	*/
	ZERO_OBJ(data);
	dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE);
	if (dberr) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "c_get(): %s", 
							db_strerror(dberr)); 
		exit(2);
	}

	COPY_DBT(addr, data);
	if (addr != vv_addr) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "addr %lld expected %lld",
					(long long) addr, (long long)vv_addr);
		exit(2);
	}
}

/*************************************************************************
			EXTERNALLY CALLABLE ROUTINES
 *************************************************************************/

/*************************************
 *  routines for lookups on hot index
 */

  VVInfo_t *
DBQueryHotExactVV(int started, char *xid, char *class)
{
	char		*id = "DBQueryHotExact";
	VVInfo_t	*vip = NULL;

	if (!started) {
		/*
		**	main vvmd starting a query
		*/
		hot_ix_cpy_name[0] = 0;

		/*
		**	make cursor
		*/
		if (dberr = db_vv_hot_idx->cursor(db_vv_hot_idx, NULL, 
								&cursor, 0)) {
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
							db_strerror(dberr));
        	}
		/*
		**	prepare for search of hot index by class_id
		*/
		(void)sprintf(vvkey, "%s_%s", ClassString(class), 
						IdString(xid));
		STR_DBT(key, vvkey);
		keylen = (size_t)key.size;
	}
	for (;; started++) {
		/*
		**	get next entry
		*/
		if (nextcursor(started, id))
			break;

		/*
		**	see if key still matches
		*/
		if (memcmp(vvkey, key.data, keylen))
			break;

		/*
		**	got vv address - look up
		*/
		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0))
			break;

		/*
		**	entry could be changed after index copied..
		*/
		if (vbuf.vb_status != VVS_HOT) {
			if (!IsChild) 
				nLogMsg(EL_CONSOLE0, id, 
					"FOUND DELETED ENTRY IN HOT IX");
			continue;
		}

		/*
		**	id should never change!
		*/
		if (memcmp(vbuf.vb_id, xid, sizeof(vbuf.vb_id))) {
			nLogMsg(EL_CONSOLE0|EL_FATAL, id, 
					"FOUND CHANGED ID IN HOT IX");
			continue;
		}

		/*
		**	entry is valid - translate record
		*/
		(void)Vbuf2VInfo(id);
		vip = &VInfo;
		break;
	}
	if (vip == NULL) {
		if (hot_ix_cpy_name[0] != 0) {
			/*
			**	forked for a big list: throw away 
			**		index copy
			*/
			(void)unlink(hot_ix_cpy_name);
		}
		cursor->c_close(cursor);
	}
	return(vip);
}

  VVInfo_t *
DBQueryHotVV(int started)
{
	char		*id = "DBQueryHotVV";
	VVInfo_t	*vip = NULL;

	if (!started) {
		/*
		**	main vvmd starting a query
		*/
		hot_ix_cpy_name[0] = 0;

		/*
		**	create cursor
		*/
		if (dberr = db_vv_hot_idx->cursor(db_vv_hot_idx, NULL, 
								&cursor, 0)) {
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
							db_strerror(dberr));
        	}
	}
	for (;; started++) {
		ZERO_OBJ(key);
		ZERO_OBJ(data);
		dberr = cursor->c_get(cursor, &key, &data, DB_NEXT);
		if (dberr == DB_NOTFOUND)
			break;
		if (dberr)
			nLogMsg(EL_FATAL, id, "cursor->c_get(): %s", 
							db_strerror(dberr));
		/*
		**	got vv address - look up
		*/
		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0))
			break;

		/*
		**	entry could be changed after index copied..
		*/
		if (vbuf.vb_status != VVS_HOT) {
			if (!IsChild) 
				nLogMsg(EL_CONSOLE0, id, 
					"FOUND DELETED ENTRY IN HOT IX");
			continue;
		}

		/*
		**	entry is valid - translate record
		*/
		(void)Vbuf2VInfo(id);
		vip = &VInfo;
		break;
	}
	if (vip == NULL) {
		if (hot_ix_cpy_name[0] != 0) {
			/*
			**	forked for a big list: 
			**		throw away index copy
			*/
			(void)unlink(hot_ix_cpy_name);
		}
		(void)cursor->c_close(cursor);
	}
	return(vip);
}

  int
DBCloseForHotQuery(int clientpid)
{
	return(CloseForQuery(clientpid, dbf_vv_hot_idx));
}

  int
DBReopenForHotQuery(int parent, int exact)
{
	char	*id = "DBReopenForHotQuery";
	int	i;

	if (parent) {
		hot_ix_cpy_name[0] = 0;
		return(0);
	}


	/*
	**	child - reopen table readonly
	*/
	close(db_vv_fd);
	if ((db_vv_fd = open(dbf_vv_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, 
				"vv table (%s) reopen failed", dbf_vv_tbl);
		exit(2);
	}
	if (StartIndex(hot_ix_cpy_name, &db_vv_hot_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "restarting index");
		exit(2);
	}

	/*
	**	create cursor again & position on last-seen entry
	*/
	renewcursor(db_vv_hot_idx, id);
	return(0);
}

/************************************
 *  routines for lookups on vv vsn index
 */

  VVInfo_t *
DBLookupVVVsn(int started, char *vsnp)
{
	char		*id = "DBLookupVsn";
	int		nvsns, i;
	VVInfo_t	*vip = NULL;

	if (!started) {
		/*
		**	main vvmd starting a query
		*/
		id_ix_cpy_name[0] = 0;

		/*
		**	create cursor
		*/
		if (dberr = db_vv_vsn_idx->cursor(db_vv_vsn_idx, NULL, 
								&cursor, 0)) {
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
							db_strerror(dberr));
        	}
		(void)sprintf(vvkey, "%.*s", VSNLEN, vsnp);
		STR_DBT(key, vvkey);
		keylen = (size_t)key.size;
	}
	for (;; started++) {
		/*
		**	get next entry
		*/
		if (nextcursor(started, id))
			break;

		/*
		**	make sure still in range
		*/
		if (memcmp(vvkey, key.data, keylen))
			break;

		/*
		**	got vv address - look up
		*/
		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0))
			return(NULL);

		/*
		**	entry could be deleted after index copied..
		*/
		if (vbuf.vb_status == VVS_DELETED) {
			if (!IsChild) 
				nLogMsg(EL_CONSOLE0, id, 
					"FOUND DELETED ENTRY IN ID IX");
			continue;
		}

		/*
		**	vsn should never change!
		*/
		nvsns = CHAR2INT(vbuf.vb_nvsns);
		for (i=0; i<nvsns; i++) {
			if (!memcmp(cslots[i]->vb_vsn, vsnp, (size_t)VSNLEN))
				break;
		}
		if (i == nvsns) {
			nLogMsg(EL_CONSOLE0|EL_FATAL, id, 
					"FOUND CHANGED VSN IN VSN IX");
			continue;
		}

		/*
		**	entry is valid - translate record
		*/
		(void)Vbuf2VInfo(id);
		vip = &VInfo;
		break;
	}
	if (vip == NULL) {
		if (id_ix_cpy_name[0] != 0) {
			/*
			**	forked for a big list: throw 
			**		away index copy
			*/
			(void)unlink(vsn_ix_cpy_name);
		}
		(void)cursor->c_close(cursor);
	}
	return(vip);
}

  int
DBCloseForVVVsnQuery(int clientpid)
{
	return(CloseForQuery(clientpid, dbf_vv_vsn_idx));
}

  int
DBReopenForVVVsnQuery(int parent)
{
	char	*id = "DBReopenForVVVsnQuery";
	int	i;

	if (parent) {
		vsn_ix_cpy_name[0] = 0;
		return(0);
	}


	/*
	**	child - reopen table readonly
	*/
	close(db_vv_fd);
	if ((db_vv_fd = open(dbf_vv_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, 
				"vv table (%s) reopen failed", dbf_vv_tbl);
		exit(2);
	}
	if (StartIndex(vsn_ix_cpy_name, &db_vv_vsn_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "restarting index");
		return(1);
	}

	/*
	**	re-create cursor & position to last seen entry
	*/
	renewcursor(db_vv_vsn_idx, id);
	return(0);
}

/************************************
 *  routines for lookups on id index
 */

  VVInfo_t *
DBLookupIdVV(int started, char *xid)
{
	char		*id = "DBLookupIdVV";
	VVInfo_t	*vip = NULL;

	nLogMsg(EL_DEBUG1, id, "id %s", IdString(xid));
	if (!started) {
		/*
		**	main vvmd starting a query
		*/
		id_ix_cpy_name[0] = 0;

		/*
		**	create cursor
		*/
		if (dberr = db_vv_id_idx->cursor(db_vv_id_idx, NULL, 
								&cursor, 0)) {
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
							db_strerror(dberr));
        	}
		/*
		**	prepare for search of id index
		*/
		(void)strcpy(vvkey, IdString(xid));
		STR_DBT(key, vvkey);
		keylen = (size_t)key.size;
	}
	for (;; started++) {
		/*
		**	get next entry
		*/
		if (nextcursor(started, id))
			break;

		/*
		**	make sure cursor is still in range
		*/
		if (memcmp(vvkey, key.data, keylen))
			break;

		/*
		**	got vv address - look up
		*/
		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0))
			break;

		/*
		**	entry could be changed after index copied..
		*/
		if (vbuf.vb_status == VVS_DELETED) {
			if (!IsChild) 
				nLogMsg(EL_CONSOLE0, id, 
					"FOUND DELETED ENTRY IN ID IX");
			continue;
		}

		/*
		**	id should never change!
		*/
		if (memcmp(vbuf.vb_id, xid, sizeof(vbuf.vb_id))) {
			nLogMsg(EL_CONSOLE0|EL_FATAL, id, 
				"%s (%s) %s instead of %s",
				"FOUND CHANGED ID IN ID IX:",
				VV2Name(Vbuf2VV()),
				IdString(xid), IdString(vbuf.vb_id));
			continue;
		}

		/*
		**	entry is valid - translate record
		*/
		(void)Vbuf2VInfo(id);
		vip = &VInfo;
		break;
	}
	if (vip == NULL) {
		if (id_ix_cpy_name[0] != 0) {
			/*
			**	forked for a big list: throw away index copy
			*/
			(void)unlink(id_ix_cpy_name);
		}
		cursor->c_close(cursor);
	}
	return(vip);
}

  int
DBCloseForIdQuery(int clientpid)
{
	return(CloseForQuery(clientpid, dbf_vv_id_idx));
}

  int
DBReopenForIdQuery(int parent)
{
	char	*id = "DBReopenForIdQuery";
	int	i;

	if (parent) {
		id_ix_cpy_name[0] = 0;
		return(0);
	}


	/*
	**	child - reopen table readonly
	*/
	close(db_vv_fd);
	if ((db_vv_fd = open(dbf_vv_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, 
				"vv table (%s) reopen failed", dbf_vv_tbl);
		exit(2);
	}
	if (StartIndex(id_ix_cpy_name, &db_vv_id_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "restarting index");
		return(1);
	}

	/*
	**	re-create cursor & position to last seen entry
	*/
	renewcursor(db_vv_vsn_idx, id);
	return(0);
}

/*************************************
 *  routines for lookups on 'finished' index
 */

  VVInfo_t *
DBQueryFinishedVV(int started)
{
	char		*id = "DBQueryFinishedVV";
	VVInfo_t	*vip = NULL;

	if (!started) {
		/*
		**	main vvmd starting a query
		*/
		finished_ix_cpy_name[0] = 0;

		/*
		**	create cursor
		*/
		if (dberr = db_vv_tp_idx->cursor(db_vv_tp_idx, NULL, 
								&cursor, 0)) {
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
							db_strerror(dberr));
        	}
	}
	for (;; started++) {
		ZERO_OBJ(key);
		ZERO_OBJ(data);
		dberr = cursor->c_get(cursor, &key, &data, DB_NEXT);
		if (dberr == DB_NOTFOUND)
			break;
		if (dberr)
			nLogMsg(EL_FATAL, id, "cursor->c_get(): %s", 
							db_strerror(dberr));

		/*
		**	got vv address - look up
		*/
		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0))
			return(NULL);

		/*
		**	entry could be changed after index copied..
		*/
		if (vbuf.vb_status != VVS_FINISHED) {
			if (!IsChild) 
				nLogMsg(EL_CONSOLE0, id, 
					"FOUND DELETED ENTRY IN FINISHED IX");
			continue;
		}

		/*
		**	entry is valid - translate record
		*/
		(void)Vbuf2VInfo(id);
		vip = &VInfo;
		break;
	}
	if (vip == NULL) {
		if (finished_ix_cpy_name[0] != 0) {
			/*
			**	forked for a big list: throw 
			**		away index copy
			*/
			(void)unlink(finished_ix_cpy_name);
		}
		cursor->c_close(cursor);
	}
	return(vip);
}

  int
DBCloseForFinishedQuery(int clientpid)
{
	return(CloseForQuery(clientpid, dbf_vv_tp_idx));
}

  int
DBReopenForFinishedQuery(int parent)
{
	char	*id = "DBReopenForFinishedQuery";
	int	i;

	if (parent) {
		finished_ix_cpy_name[0] = 0;
		return(0);
	}


	/*
	**	child - reopen table readonly
	*/
	close(db_vv_fd);
	if ((db_vv_fd = open(dbf_vv_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, 
				"vv table (%s) reopen failed", dbf_vv_tbl);
		exit(2);
	}
	if (StartIndex(finished_ix_cpy_name, &db_vv_tp_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "restarting index");
		return(1);
	}

	/*
	**	re-create cursor & position to last seen entry
	*/
	renewcursor(db_vv_vsn_idx, id);
	return(0);
}


/*************************************
 *  routines for vsn lookups on hot vsn index
 */

  Vsn_t *
DBQueryHotVsn(int started)
{
	char	*id = "DBQueryHotVsn";
	Vsn_t	*vp = NULL;

	if (!started) {
		/*
		**	main vvmd starting a query
		*/
		vsn_hot_ix_cpy_name[0] = 0;

		/*
		**	create cursor
		*/
		if (dberr = db_vsn_hot_idx->cursor(db_vsn_hot_idx, NULL, 
								&cursor, 0)) {
			nLogMsg(EL_FATAL|EL_ERROR1, id, "db cursor(): %s",
							db_strerror(dberr));
        	}
	}
	for (;; started++) {
		ZERO_OBJ(key);
		ZERO_OBJ(data);
		dberr = cursor->c_get(cursor, &key, &data, DB_NEXT);
		if (dberr == DB_NOTFOUND)
			break;
		if (dberr)
			nLogMsg(EL_FATAL, id, "cursor->c_get(): %s", 
							db_strerror(dberr));

		/*
		**	got vsn address - look up
		*/
		COPY_DBT(vsn_addr, data);
		if (LoadVsn(id))
			return(NULL);

		/*
		**	check status
		*/
		if (vsnbuf.vb_status != VVS_HOT) {
			nLogMsg(EL_ERROR1, id, "vsn not hot");
			continue;
		}

		/*
		**	translate record
		*/
		if (Vsnbuf2VsnInfo(id))
			nLogMsg(EL_FATAL, id, "Vsnbuf2VsnInfo()");
		vp = &VsnInfo;
		break;
	}
	if (vp == NULL) {
		if (vsn_hot_ix_cpy_name[0] != 0) {
			/*
			**	forked for a big list: throw away index copy
			*/
			(void)unlink(vsn_hot_ix_cpy_name);
		}
		cursor->c_close(cursor);
	}

	return(vp);
}


  int
DBCloseForHotVsnQuery(int clientpid)
{
	return(CloseForQuery(clientpid, dbf_vsn_hot_idx));
}

  int
DBReopenForHotVsnQuery(int parent)
{
	char	*id = "DBReopenForHotVsnQuery";
	int	i;

	if (parent) {
		vsn_hot_ix_cpy_name[0] = 0;
		return(0);
	}


	/*
	**	child - reopen table readonly
	*/
	close(db_vsn_fd);
	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDONLY, 0))== -1) {
		nLogMsg(EL_CONSOLE0|EL_SYS, id, 
				"vsn table (%s) reopen failed", dbf_vsn_tbl);
		exit(2);
	}
	if (StartIndex(vsn_hot_ix_cpy_name, &db_vsn_hot_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "restarting index");
		return(1);
	}
	/*
	**	re-create cursor & position to last seen entry
	*/
	renewcursor(db_vv_vsn_idx, id);

	return(0);
}

  void
DBCleanupChild()
{
	if (hot_ix_cpy_name[0] != 0)
		(void)unlink(hot_ix_cpy_name);
	if (vsn_ix_cpy_name[0] != 0)
		(void)unlink(vsn_ix_cpy_name);
	if (id_ix_cpy_name[0] != 0)
		(void)unlink(id_ix_cpy_name);
	if (finished_ix_cpy_name[0] != 0)
		(void)unlink(id_ix_cpy_name);
}

  int
DBCopyIx(int clientpid)
{
	char	*id = "DBCopyIx";
	char	buf[MAXPATHLEN];
	int	err;

	/*
	**	flush index to file
	*/
	if (dberr = db_vv_vsn_idx->sync(db_vv_vsn_idx, 0)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "db sync(%s): %s",
					dbf_vv_vsn_idx, db_strerror(dberr));
		return(1);
	}

	/*
	**	copy index file
	*/
	err = CopyIx(id, dbf_vv_vsn_idx, buf, "vv_vsn", clientpid);

	return(err);
}
