
/*
 *  vvm_db3.c - dbase check/recovery/maintenance routines
 */

#include "vvmd.h"
#include "vvm_db.h"

extern int		db_vv_fd;
extern OFF_T		vv_addr;
extern vbuf_t		vbuf;
extern ClassSlot_t	*cslots[];
extern VVInfo_t		VInfo;

extern DB		*db_vv_hot_idx, *db_vv_vsn_idx;
extern DB		*db_vv_tp_idx, *db_vv_tp2_idx;;

extern char		dbf_vv_hot_idx[], dbf_vv_tp_idx[];

extern OFF_T		vsn_addr;
extern vsbuf_t		vsnbuf; 
extern Vsn_t		VsnInfo;

extern DB		*db_vsn_vsn_idx, *db_vsn_hot_idx, *db_vsn_recyc_idx;

extern DBT		key, data;
extern char		vvkey[], vsnkey[];
extern int		dberr;

extern char		dbf_vsn_recyc_idx[];

/*******************************************************************/


/*
**	RecoverVV() - if possible, schedule VV for write
**		(maybe read first) to recover a missing slot.
**		Assumes VV is current one loaded and that 
**		missing vsn(s) have been 'blanked' out.
*/
 static int
RecoverVV(char *vvname)
{
	char		*id = "RecoverVV";
	int		i, ondisk, saving, nslots, okvsns = 0;
	int		nvsns = CHAR2INT(vbuf.vb_nvsns);
	VV_t		*vvp = Vbuf2VV();
	OFF_T		size;


	if (vbuf.vb_status == VVS_HOT)
		nLogMsg(EL_FATAL, id, "recovering hot VV (%s) -??", vvname);

	/*
	**	size up the situation
	*/
	for (i=0; i<nvsns; i++) {
		if (cslots[i]->vb_vflags[1] == VSN_OK) {
			okvsns++;
			if (cslots[i]->vb_vsn[0] == ' ')
				nLogMsg(EL_FATAL, id, "ok vsn blank");
		}
	}
	if (okvsns == nvsns)
		nLogMsg(EL_FATAL, id, "called with ok record %s", vvname);

	if (DiskResidentSize(vvp, &size))
		ondisk = 0;
	else
		ondisk = 1;

	STR_DBT(key, vvname);
	ZERO_OBJ(data);
	dberr = db_vv_tp_idx->get(db_vv_tp_idx, NULL, &key, &data, 0);
	if (dberr) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "db get(): %s", 
							db_strerror(dberr));
		saving = 0;
	} else {
		OFF_T	addr;

		/*
		**	check things as long as the info is available
		*/
		COPY_DBT(addr, data);
		if (addr != vv_addr)
			nLogMsg(EL_FATAL, id, "addr 0x%llx != vv_addr 0x%llx",
				(long long) addr, (long long) vv_addr);
		saving = 1;
	}

nLogMsg(EL_STATUS1, id, "    %s ondisk %d saving %d okvsns %d", 
vvname, ondisk, saving, okvsns);

	if (okvsns == 0  &&  !ondisk) {
		nLogMsg(EL_ERROR1|EL_CONSOLE0, id,
				"%s: all vsns lost & VV deleted, marking LOST",
				vvname);
		vbuf.vb_status = VVS_LOST;
		WriteVV(id);
		if (saving) {
			STR_DBT(key, vvname);
			if (dberr = db_vv_tp_idx->del(db_vv_tp_idx, NULL, 
								&key, 0))
				nLogMsg(EL_FATAL, id, "db del(): %s",
							db_strerror(dberr));
		}
		return(1);
	}

	/*
	**	needs recovery work
	*/
	if (!saving)
		(void)IndexVVix(id, vvname, "tp", db_vv_tp_idx);

	nslots = nvsns - okvsns;

	if (ondisk) {
		if (vbuf.vb_status != VVS_FINISHED  &&  
		    vbuf.vb_status != VVS_RECYCLING)
			nLogMsg(EL_STATUS1|EL_CONSOLE0, id, 
				"%s - fill %d slot%s",
				vvname, nslots, (nslots == 1 ? "" : "s"));
		vbuf.vb_status = VVS_FINISHED;
	} else {
		nLogMsg(EL_STATUS1|EL_CONSOLE0, id, 
				"%s - recover/fill %d slot%s",
				vvname, nslots, (nslots == 1 ? "" : "s"));
		vbuf.vb_status = VVS_RECOVER;
	}
	WriteVV(id);

	/*
	**	now that dbase is taken care of, 
	**		make sure record is translated 
	**		& start off recovery
	*/
	(void)Vbuf2VInfo(id);
	if (ondisk)
		CWriteVV2Vsns(&VInfo);
	else
		CRestore(NOSEQ, -1, &VInfo);
	return(0);
}


/*
**	DBCheckVsn() - compare vsn dump to dbase
**		Note that this routine assumes that a single
**		process has exclusive write access to the
**		database for the whole time.
*/

static int	nscratch = 0;

  int
DBCheckVsn(char *vsnp, int fd, int flags)
{
	char		*id = "DBCheckVsn";
	int		i, live_vv, do_recs, nrecs, multi_slot, slots[MAXVSN];
	int		novsn, file_rec, err = 0, wait_del = 0, recover = 0;
	int		scratch_ct = 0, lost = 0;
	int		in_db = 0, not_in_db = 0, disagree = 0;
	int		tp_not_slot, dropslot = 0;
	char		vvname[VV_NAMELEN];
	ClassSlot_t	*cslot;
	DBC		*cursor;

	nLogMsg(EL_STATUS1, id, "--------------- %.*s", VSNLEN, vsnp);
	
	/*
	**	see if there are any live VV's indexed 
	**		as being on this vsn
	*/
	if (dberr = db_vv_vsn_idx->cursor(db_vv_vsn_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));

	(void)sprintf(vvkey, "%.*s", VSNLEN, vsnp);
	STR_DBT(key, vvkey);
	ZERO_OBJ(data);
	if (dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		live_vv = 0;
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "no live VV's indexed");
	} else if (memcmp(vsnp, key.data, (size_t)VSNLEN)) {
		live_vv = 0;
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "no live VV's indexed");
	} else
		live_vv = 1;

	/*
	**	get vsn record
	*/
	if (DBLookupVsn(vsnp) == NULL) {
		/* TODO - check it anyway.. */
		VVError = VVE_USER;
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"%.*s: vsn not in dbase (%s 'live' VV's too)",
				VSNLEN, vsnp, (live_vv ? "but" : "no"));
		return(1);
	}
	if (VsnInfo.vs_status == VVS_HOT)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "-- vsn is hot");

	/*
	**	mark live VV's for detection of missing ones
	*/
	multi_slot = 0;
	novsn = 0;
	nrecs = 0;
	do_recs = live_vv;
	memset(slots, 0, sizeof(slots));

	/*
	**	start cursor again to be on the safe side
	*/
	(void)sprintf(vvkey, "%.*s", VSNLEN, vsnp);
	STR_DBT(key, vvkey);
	ZERO_OBJ(data);
	if (dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE))
		nLogMsg(EL_FATAL, id, "c_get(): %s", db_strerror(dberr));
	if (memcmp(vsnp, key.data, (size_t)VSNLEN))
		nLogMsg(EL_FATAL, id, "2nd c_get(): nothing");

	while (1) {
		int	nvsns, nslots;
		VV_t	*vvp;

		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0)) {	/*  0 = don't translate record */
			err++;
			(void)cursor->c_close(cursor);
			return(err);
		}
		vvp = Vbuf2VV();

		if (vbuf.vb_status == VVS_HOT)
			nLogMsg(EL_FATAL, id, "hot VV (%s) on tape -??", 
						VV2Name(vvp));
		nvsns = CHAR2INT(vbuf.vb_nvsns);
		nslots = 0;
		nrecs++;

		/*
		**	note that this vsn is being checked
		*/
		for (i=0; i<nvsns; i++) {
			if (memcmp(vsnp, cslots[i]->vb_vsn, (size_t)VSNLEN))
				continue;
			nslots++;
			slots[i]++;
			cslots[i]->vb_vflags[0] = VSN_CHECK;
		}
		if (!nslots) {
			nLogMsg(EL_ERROR1, id, 
				"vv_vsn idx corrupted: no slot for %.*s in %s",
				VSNLEN, vsnp, VV2Name(vvp));
			novsn++;
			err++;
		} else {
			if (nslots > 1) {
				nLogMsg(EL_STATUS1, id, 
					"%.*s in %d slots in %s",
					VSNLEN, vsnp, VV2Name(vvp));
				multi_slot++;
			}
			/*
			**	write back marked record
			*/
			if (WriteVV(id)) {
				(void) cursor->c_close(cursor);
				return(1);
			}
		}
		/*
		**	get next record index
		*/
		if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
			if (dberr != DB_NOTFOUND)
				nLogMsg(EL_FATAL, id, "c_get(): %s",
							db_strerror(dberr));
			break;
		}
		if (memcmp(vsnp, key.data, (size_t)VSNLEN))
			break;
	}
	/*
	**	toss out the cursor in case it won't be needed again
	*/
	(void) cursor->c_close(cursor);

	/*
	**	report initial findings
	*/
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "initial counts of 'live' VV's:");
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "  Total: %d", nrecs);
	for (i=0; i<MAXVSN; i++) {
		if (slots[i])
			nLogMsg(EL_STATUS1|EL_NOTIME, id, 
							"   slot %d total: %d",
							i, slots[i]);
	}
	if (multi_slot)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, 
			"  total w/ multiple slots on vsn: %d", multi_slot);
	if (novsn)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0,
			"  total w/ vsn not in record: %d", novsn);

	/*
	**	read the vsn dump file
	*/
	for (file_rec=0; ; file_rec++) {
		TapeRec_t	tprec;
		int		recsize, version;
		unsigned long	block;
		VV_t		vv;
		char		vid[VV_IDLEN];
		int		slot, nvsns;
		int		nrec, nwrite;
		char		data[HEXLEN];
		time_t		creat, lastwrite, finish, ontape;
		int		writerec, nogood;

		/*
		**	get tape record
		*/
		recsize = read(fd, &tprec, sizeof(tprec));
		if (recsize == EOF)
			break;
		if (recsize == -1) {
			VVError = VVE_SYS;
			nLogMsg(EL_SYS|EL_CODE, id, "reading tape dump file");
			err++;
			return(err);
		}
		if (recsize != sizeof(tprec)) {
			VVError = VVE_INTERNAL;
			nLogMsg(EL_ERROR1|EL_CODE, id,
					"tape dump file: recsize %d not %d",
					recsize, sizeof(tprec));
			err++;
			return(err);
		}
		if (TapeRecExtract(&tprec, &block, &version, 
					&vv, vid, 
					&slot, &nvsns,
					&nrec, data, 
					&creat, 
					&nwrite, &lastwrite, 
					&finish, 
					&ontape)) {
			err++;
			return(err);
		}
		(void)strcpy(vvname, VV2Name(&vv));

		/*
		**	check/set slot from tape
		*/
		if (slot < 0  ||  slot > 3) {
			nLogMsg(EL_ERROR1, id, "%s: bad slot from tape [%d]",
							vvname, slot);
			err++;
			slot = -1;
			cslot = NULL;
		} else

		if (slot > nvsns) {
			nLogMsg(EL_ERROR1, id, 
					"%s: slot from tape [%d] > nvsns [%d]",
					vvname, slot, nvsns);
			err++;
		}

		/*
		**	get VV record from dbase
		*/
		if (DBLookupVV(&vv) == NULL) {
			err++;
			not_in_db++;
			nLogMsg(EL_STATUS1, id, "%s not found in dbase",
								vvname);
			if (!(flags & VQF_ADD))
				continue;

			if (cslot == NULL) {
				nLogMsg(EL_STATUS1, id, 
					"%s: not adding because of bad slot",
					vvname);
				continue;
			}

			/*
			**	add new rec..
			*/
			memset(&vbuf, ' ', sizeof(vbuf));
			vbuf.end = '\n';
			memcpy(vbuf.vb_version, tprec.rec.lb_version, 
						sizeof(vbuf.vb_version));
			memcpy(vbuf.vb_class, tprec.rec.lb_vv.class, 
						sizeof(vbuf.vb_class));
			memcpy(vbuf.vb_serial, tprec.rec.lb_vv.serial, 
						sizeof(vbuf.vb_serial));
			memcpy(vbuf.vb_id, tprec.rec.lb_id, 
						sizeof(vbuf.vb_id));
			vbuf.vb_status = VVS_ADDED;
			vbuf.vb_use = VVU_IDLE;
			C2ASC(SetDecInt, vbuf.vb_nwrite, nwrite);
			SetTime(vbuf.vb_creat, creat);
			SetTime(vbuf.vb_write, lastwrite);
			C2ASC(SetDecInt, vbuf.vb_nread, 0);/* just don't know */
			C2ASC(SetDecInt, vbuf.vb_nrestore, 0);/* don't know */
			C2ASC(SetDecInt, vbuf.vb_mntfail, 0);/* don't know */
			memcpy(vbuf.vb_data, data, sizeof(vbuf.vb_data));
			C2ASC(SetHexOFF, vbuf.vb_maxdata, 0);/*doesn't matter */
			SetTime(vbuf.vb_save, ontape);	/* best guess */
			SetTime(vbuf.vb_read, -1);	/* just don't know */
			SetTime(vbuf.vb_finish, finish);
			SetTime(vbuf.vb_restore, -1);	/* just don't know */
			SetTime(vbuf.vb_delete, -1);	/* just assuming.. */

			vbuf.vb_nvsns = INT2CHAR(nvsns);

			C2ASC(SetDecInt, vbuf.vb_nrec0, -1);
			C2ASC(SetDecInt, vbuf.vb_nrec1, -1);
			C2ASC(SetDecInt, vbuf.vb_nrec2, -1);
			C2ASC(SetDecInt, vbuf.vb_nrec3, -1);

			memcpy(cslot->vb_vsn, vsnp, sizeof(cslot->vb_vsn));
			C2ASC(SetDecInt, cslot->vb_nrec, nrec);
			C2ASC(SetDecUlong, cslot->vb_nblk, block);
			SetTime(cslot->vb_vdate, ontape);
			cslot->vb_vflags[1] = VSN_OK;

			/*
			**	write db record at end of file
			*/
			vv_addr = LSEEK(db_vv_fd, 0, SEEK_END);
			if (vv_addr == -1)
				nLogMsg(EL_SYS|EL_FATAL, id, "LSEEK");
			if (vv_addr % sizeof(vbuf) != 0)
				nLogMsg(EL_SYS|EL_FATAL, id, 
						"table size not consistent");
			if (WriteVV(id))
				 nLogMsg(EL_FATAL, id, "VV TBL PROBLEM");
			continue;
		}

		/*
		**	VV record is in dbase, so check it
		*/
		in_db++;
		writerec = 0;

		/*
		**	check VV info, seeing if the copy on
		**		this tape may be worth adding in
		**		an empty slot if it isn't there
		**		- for now, checking is strict;
		**		relax only for concrete cases
		**		of repairable corruption.
		*/
		nogood = 0;
		if (version != VInfo.v_version) {
			err++;
			nogood++;
			nLogMsg(EL_ERROR1, id, "%s/%.*s: versions %d/%d",
						vvname, VSNLEN, vsnp,
						VInfo.v_version, version);
		}
		if (memcmp(vid, VInfo.v_id, sizeof(vid))) {
			err++;
			nogood++;
			nLogMsg(EL_ERROR1, id, "%s/%.*s: id [%.*s][%.*s]",
						vvname, VSNLEN, vsnp,
						VV_IDLEN, VInfo.v_id,
						VV_IDLEN, vid);
		}
		if (memcmp(data, VInfo.v_data, sizeof(data))) {
			err++;
			nogood++;
			nLogMsg(EL_ERROR1, id, "%s/%.*s: data %.*s/%.*s",
						vvname, VSNLEN, vsnp,
						HEXLEN, VInfo.v_data, 
						HEXLEN, data);
		}
		if (nvsns != VInfo.v_nvsns) {
			err++;
			nogood++;
			nLogMsg(EL_ERROR1, id, "%s/%.*s: nvsns %d/%d",
						vvname, VSNLEN, vsnp,
						VInfo.v_nvsns, nvsns);
		}
		if (creat != (time_t)VInfo.v_creat) {
			err++;
			nogood++;
			nLogMsg(EL_ERROR1, id, "%s/%.*s: creat %s / %s",
					vvname, VSNLEN, vsnp,
					bsd_ctime((time_t *)&VInfo.v_creat),
					bsd_ctime(&creat));
		}
		if (nwrite != VInfo.v_nwrite) {
			err++;
			nogood++;
			nLogMsg(EL_ERROR1, id, "%s/%.*s: nwrite %d / %d",
					vvname, VSNLEN, vsnp,
					VInfo.v_nwrite, nwrite);
		}
		if (lastwrite != (time_t)VInfo.v_write){
			err++;
			nogood++;
			nLogMsg(EL_ERROR1, id, "%s/%.*s: lastwrite %s / %s",
					vvname, VSNLEN, vsnp,
					bsd_ctime((time_t *)&VInfo.v_write),
					bsd_ctime(&lastwrite));
		}
		if (finish != (time_t)VInfo.v_finish) {
			err++;
			nogood++;
			nLogMsg(EL_ERROR1, id, "%s/%.*s: finish %s / %s",
					vvname, VSNLEN, vsnp,
					bsd_ctime((time_t *)&VInfo.v_finish),
					bsd_ctime(&finish));
		}
		
		/*
		**	check info about tape itself; if it didn't
		**		claim a valid slot, see if it is in one
		**		TODO (hah) handle if in multiple slots
		*/
		if (cslot == NULL) {
			int	lastslot, inslot = 0;

			nLogMsg(EL_STATUS1, id, 
					"%s/%.*s: checking slots for match",
					vvname, VSNLEN, vsnp);
			/*
			**	see if some slot has tape
			*/
			for (i=0; i<4; i++) {
				if (!memcmp(cslots[i]->vb_vsn, vsnp, VSNLEN)) {
					inslot++;
					lastslot = i;
				}
			}
			switch (inslot) {
			    case 0:
				nLogMsg(EL_STATUS1, id, "\t-not in any slot");
				break;
			    case 1:
				nLogMsg(EL_STATUS1, id, 
						"\t-in slot %d - comparing",
						lastslot);
				slot = lastslot;
				cslot = cslots[slot];
				break;
			    default:
				nLogMsg(EL_ERROR1, id, 
						"%s/%.*s: vsn in %d slots",
						 vvname, VSNLEN, vsnp, inslot);
			}
		}

		if (cslot != NULL) {
			/*
			**	rec from tape claims a valid slot or
			**		it has just been assigned a match
			**		above
			*/
			if (memcmp(vsnp, cslot->vb_vsn, VSNLEN)) {
				/*
				**	bad vsn, unless slot is empty
				*/
				err++;
				tp_not_slot++;
				if (cslot->vb_vsn[0] == ' '  &&  !nogood) {
					/* TODO - BAD_VSN there */
					if (flags & VQF_ADD) {
						/*
						 *  drop tape into empty slot
						 *	where it apparently 
						 *	belongs
						 */
						nLogMsg(EL_STATUS1, id,
						  "%s: adding %.*s to slot %d",
						  vvname, VSNLEN, vsnp);
						memcpy(cslot->vb_vsn, vsnp, 
									VSNLEN);
						C2ASC(SetDecInt, 
							cslot->vb_nrec, nrec);
						C2ASC(SetDecUlong,
							cslot->vb_nblk, block);
						SetTime(cslot->vb_vdate,ontape);
						writerec++;
						dropslot++;
					}
				} else
					nLogMsg(EL_ERROR1, id, 
						"%s: %.*s in slot %d, not %.*s",
						vvname, VSNLEN, cslot->vb_vsn, 
						slot, VSNLEN, vsnp);
			} else {
				/*
				**	vsn in slot; check rest
				*/
				if (block != VInfo.v_vsns[slot].v_block) {
					nLogMsg(EL_ERROR1, id, 
						"%s/%.*s: bad blkid %d / %d",
						vvname, VSNLEN, vsnp,
						VInfo.v_vsns[slot].v_block,
						block);
					err++;
					nogood++;
				}
				if (nrec != VInfo.v_vsns[slot].v_nrec) {
					nLogMsg(EL_ERROR1, id, 
						"%s/%.*s: bad recid %d / %d",
						vvname, VSNLEN, vsnp,
						VInfo.v_vsns[slot].v_nrec,
						nrec);
					err++;
					nogood++;
				}
			}
		}

		/*
		**	extra check for safety
		*/
		if (nogood) {
			disagree++;
			writerec = 0;
		}

		/*
		**	save updated record
		*/
		if (writerec)
			if (WriteVV(id))
				nLogMsg(EL_FATAL, id, "writing mod rec");
	}
	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "Total records from tape file: %d", 
							file_rec);
	if (not_in_db)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "\tTotal not in db: %d", 
								not_in_db);
	if (in_db)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "\tTotal in db: %d", in_db);
	if (disagree)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, 
					"\tTotal disagreeing records: %d",
					disagree);
	if (tp_not_slot)
		 nLogMsg(EL_STATUS1|EL_NOTIME, 0, "\t\tTape != slot: %d",
								tp_not_slot);
	if (dropslot)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "\t\tPut tape in slot: %d",
								dropslot);
	if (!live_vv) {
		if (err)
			nLogMsg(EL_STATUS1|EL_NOTIME, 0, "Total errs: %d", err);
		nLogMsg(EL_STATUS1, id, "--------------- %.*s done", 
							VSNLEN, vsnp);
		return(err);
	}

	/*
	**	post-check the records marked in the 1st phase
	*/
	if (dberr = db_vv_vsn_idx->cursor(db_vv_vsn_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));
	(void)sprintf(vvkey, "%.*s", VSNLEN, vsnp);
	STR_DBT(key, vvkey);
	ZERO_OBJ(data);
	if (dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE))
		nLogMsg(EL_FATAL, id, "post c_get(): %s", db_strerror(dberr));
	if (memcmp(vsnp, key.data, VSNLEN))
		nLogMsg(EL_FATAL, id, "2nd pass no match");

	while (1) {
		int	problem, okvsn, nvsns;
		VV_t	*vvp;

		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0)) {	/*  0 = don't translate record */
			(void)cursor->c_close(cursor);
			err++;
			return(err);
		}
		vvp = Vbuf2VV();
		(void)strcpy(vvname, VV2Name(vvp));
		nvsns = CHAR2INT(vbuf.vb_nvsns);
		memset(slots, 0, sizeof(slots));
		problem = okvsn = 0;
		for (i=0; i<nvsns; i++) {
			if (memcmp(vsnp, cslots[i]->vb_vsn, VSNLEN))
				continue;
			if (cslots[i]->vb_vflags[0] == VSN_CHECK) {
				memset(cslots[i]->vb_vsn, ' ', VSNLEN);
				cslots[i]->vb_vflags[1] = VSN_NOTFOUND;
				problem++;
			} else
				okvsn++;
		}
		if (problem) {
			nLogMsg(EL_STATUS1|EL_CONSOLE0, id,
					"%s: %d slot%s: not on vsn %.*s",
					vvname,
					problem, (problem > 1 ? "s" : ""),
					VSNLEN, vsnp);
			err++;
			recover++;
			lost += RecoverVV(vvname);
			if (!okvsn) {
				/*
				**	delete vv_vsn index for this record
				*/
				if (dberr = cursor->c_del(cursor, 0))
					nLogMsg(EL_FATAL, id, "c_del(): %s",
							db_strerror(dberr));
			}
		}
		/*
		**	get next record index
		*/
		if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
			if (dberr != DB_NOTFOUND)
				nLogMsg(EL_FATAL, id, "c_get(): %s",
							db_strerror(dberr));
			break;
		}
		if (memcmp(vsnp, key.data, VSNLEN))
			break;
	}
	if (recover)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "Total VV's to recover: %d", 
								recover);
	if (lost)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "Total lost VV's: %d", lost);

	nLogMsg(EL_STATUS1, id, "--------------- %.*s done", VSNLEN, vsnp);
	return(err);
}

/*
 *  DBBadVsn() - mark the vsn bad & start recovery operations
 *	for any VV's on it
 */
  int
DBBadVsn(char *vsnp)
{
	char		*id = "DBBadVsn";
	int		live_vv = 0, nrecs = 0, novsn = 0, lost = 0, err = 0;
	int		vvnovsn = 0, multi_slot = 0, slots[MAXVSN];
	int		rec, slot, listsize;
	char		vvname[VV_NAMELEN];
	DBC		*cursor;

	/*
	**	see if there are any live VV's indexed 
	**		as being on this vsn
	*/
	if (dberr = db_vv_vsn_idx->cursor(db_vv_vsn_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));

	(void)sprintf(vvkey, "%.*s", VSNLEN, vsnp);
	STR_DBT(key, vvkey);
	ZERO_OBJ(data);
	if (dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		live_vv = 0;
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "no live VV's indexed");
	} else if (memcmp(vsnp, key.data, VSNLEN)) {
		live_vv = 0;
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "no live VV's indexed");
	} else
		live_vv = 1;

	/*
	**	get vsn record
	*/
	if (DBLookupVsn(vsnp) == NULL) {
		if (!live_vv) {
			VVError = VVE_USER;
			nLogMsg(EL_CODE|EL_ERROR1, id, 
				"%.*s: vsn not in dbase and no 'live' VV's",
				VSNLEN, vsnp);
			return(1);
		}
		novsn = 1;
		listsize = 5;
		err++;
	} else {
		if (VsnInfo.vs_status == VVS_HOT) {
			nLogMsg(EL_STATUS1, id, "%.*s: vsn is hot",
							VSNLEN, vsnp);
			(void)UnIndexVsnHot(id);
		}
		vsnbuf.vb_status = VVS_LOST;
		VsnMsg("marked bad", 1);
		(void)WriteVsn(id);
		listsize = VsnInfo.vs_livevv + 5;
	}
	if (live_vv == 0) {
		(void) cursor->c_close(cursor);
		return(novsn);
	}

	/*
	**  init slot counts
	*/
	memset(slots, 0, sizeof(slots));

	/*
	**	restart cursor
	*/
	(void)sprintf(vvkey, "%.*s", VSNLEN, vsnp);
	STR_DBT(key, vvkey);
	ZERO_OBJ(data);
	if (dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE))
		nLogMsg(EL_FATAL, id, "c_get(): %s", db_strerror(dberr));
	if (memcmp(vsnp, key.data, VSNLEN))
		nLogMsg(EL_FATAL, id, "nothing, should be something");

	/*
	**	mark vsn dead in live VV's
	*/
	for (nrecs=0; ; nrecs++) {
		int	nvsns, nslots;
		VV_t	*vvp;

nLogMsg(EL_STATUS1, id, "---  pass1 %d  0x%llx", nrecs, vv_addr);
		/*
		**	delete the vv index entry for this vsn
		*/
		if (dberr = cursor->c_del(cursor, 0))
			nLogMsg(EL_FATAL, id, "c_del() rec %d: %s",
						nrecs, db_strerror(dberr));

		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0)) {	/*  0 = don't translate record */
			err++;
			(void) cursor->c_close(cursor);
			return(err);
		}
		vvp = Vbuf2VV();
		(void)strcpy(vvname, VV2Name(vvp));
nLogMsg(EL_STATUS1, id, "--- pass2  %d/%d  %s  0x%llx", 
rec, nrecs, vvname, vv_addr);
		if (vbuf.vb_status == VVS_HOT) {
			/*
			**	hot VV's shouldn't be on tape
			*/
			nLogMsg(EL_FATAL, id, "hot VV (%s) on tape -??",vvname);
		}
		nvsns = CHAR2INT(vbuf.vb_nvsns);
		nslots = 0;
		for (slot=0; slot<nvsns; slot++) {
			if (memcmp(vsnp, cslots[slot]->vb_vsn, VSNLEN))
				continue;
			memset(cslots[slot]->vb_vsn, ' ', VSNLEN);
			nslots++;
			slots[slot]++;
			cslots[slot]->vb_vflags[1] = VSN_BAD;
		}
		if (!nslots) {
			nLogMsg(EL_ERROR1, id, 
				"vv_vsn idx corrupted: no slot for %.*s in %s",
				VSNLEN, vsnp, vvname);
			vvnovsn++;
			err++;
		} else {
			nLogMsg(EL_STATUS1, id, 
					"%s: DELETED vsn %.*s in %d slot%s",
					vvname, VSNLEN, vsnp,
					nslots, (nslots > 1 ? "s" : ""));
			if (nslots > 1)
				multi_slot++;

			/*
			**	try to recover / write back marked record
			**		(index entry deleted at end)
			**		(RecoverVV() writes back VV record)
			*/
			lost += RecoverVV(vvname);
		}

		/*
		**	get next record index
		*/
		ZERO_OBJ(data);
		if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
			if (dberr != DB_NOTFOUND)
				nLogMsg(EL_FATAL, id, "c_get(): %s",
							db_strerror(dberr));
			break;
		}
		if (memcmp(vsnp, key.data, VSNLEN))
			break;
	}
	(void) cursor->c_close(cursor);

	/*
	**	report findings
	*/
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "counts of 'live' VV's:");
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "  Total: %d", nrecs);
	for (slot=0; slot<MAXVSN; slot++) {
		if (slots[slot])
			nLogMsg(EL_STATUS1|EL_NOTIME, id, 
							"   slot %d total: %d",
							slot, slots[slot]);
	}
	if (multi_slot)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, 
			"  total w/ multiple slots on vsn: %d", multi_slot);
	if (vvnovsn)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0,
			"  total w/ vsn not in record: %d", vvnovsn);

	return(err);
}

/*
**	MarkVsnRecycledVVs() - update vsn status to reflect
**		that VV's have been pulled off it
*/
 static int
MarkVsnRecycledVVs(char *vsnp, int was_restoring)
{
	char		*id = "MarkVsnRecycledVVs";

	/*
	**	update status in recycle index
	*/
	if (was_restoring) {
		/*
		**	had been mounted to restore files - delete
		**	the index q entry used for restarting that
		**	process across shutdowns
		*/
		(void)sprintf(vsnkey, "REST_%.*s", VSNLEN, vsnp);
		STR_DBT(key, vsnkey);
		if (dberr = db_vsn_recyc_idx->del(db_vsn_recyc_idx, NULL, 
								&key, 0))
			nLogMsg(EL_FATAL, id, "db del(): %s", 
							db_strerror(dberr));
	}

	/*
	**	add to index q for recycling to volman
	*/
	(void)sprintf(vsnkey, "DEL_%.*s", VSNLEN, vsnp);
	STR_DBT(key, vsnkey);
	FILL_DBT(data, vsn_addr);
	if (dberr = db_vsn_recyc_idx->put(db_vsn_recyc_idx, NULL, &key, &data,
							DB_NOOVERWRITE)) {
		VVError = VVE_DBASE;
		nLogMsg(EL_ERROR1|EL_CODE, id, "db put(%s): %s", 
						vsnkey, db_strerror(dberr));
		return(1);
	}

	/*
	**	update vsn status in table
	*/
	vsnbuf.vb_status = VVS_RECYCLED;
	(void)WriteVsn(id);

	return(0);
}


/*
**	DBInitRecycleVsn() - start recycling process. Return number of
**		vv's req'd to get off tape before it is recycled to
**		volume mgr, -1 if error.
*/
  int
DBInitRecycleVsn(char *vsnp)
{
	char		*id = "DBInitRecycleVsn";
	int		live_vv = 0, nrecs = 0, restorevv = 0, ondisk = 0;
	int		vvnotmigrated = 0, vvnovsn = 0, multi_slot = 0;
	int		slots[MAXVSN];
	int		rec, slot, listsize;
	char		vvname[VV_NAMELEN];
	OFF_T		*VVlist = NULL;
	DBC		*cursor;

	/*
	**	vsn record should have been loaded by caller
	**	checking status
	*/
	if (memcmp(vsnp, vsnbuf.vb_vsn, VSNLEN))
		nLogMsg(EL_FATAL, id, "vsn not loaded");
	if (VsnInfo.vs_status != VVS_FINISHED)
		nLogMsg(EL_FATAL, id, "vsn not finished");

	/*
	**	see if there are any live VV's indexed 
	**		as being on this vsn
	*/
	if (dberr = db_vv_vsn_idx->cursor(db_vv_vsn_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));

	(void)sprintf(vvkey, "%.*s", VSNLEN, vsnp);
	STR_DBT(key, vvkey);
	ZERO_OBJ(data);
	if (dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		live_vv = 0;
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "no live VV's indexed");
	} else if (memcmp(vsnp, key.data, VSNLEN)) {
		live_vv = 0;
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, "no live VV's indexed");
	} else
		live_vv = 1;

nLogMsg(EL_STATUS1, id, "live_vv %s  VsnInfo.vs_livevv %d  VsnInfo.vs_delvv %d",
(live_vv ? "yes" : "no"), VsnInfo.vs_livevv, VsnInfo.vs_delvv);

	/*
	**	require some consistency between vsn & vv tables
	**	to be on the safe side
	*/
	if (live_vv == 0  &&  VsnInfo.vs_livevv) {
		VVError = VVE_DBASE;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
			"%.*s: no VV's in index but %d 'live' in vsn record",
			VSNLEN, vsnp, VsnInfo.vs_livevv);
		(void) cursor->c_close(cursor);
		return(-1);
	}
	if (VsnInfo.vs_livevv == 0  &&  live_vv) {
		VVError = VVE_DBASE;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
			"%.*s: VV's in index but none 'live' in vsn record",
			VSNLEN, vsnp);
		(void) cursor->c_close(cursor);
		return(-1);
	}

	/*
	**	for the record note how many VV's have been scratched
	*/
	nLogMsg(EL_STATUS1, id, "%.*s: %d recycled VV's", VSNLEN, vsnp, 
							VsnInfo.vs_delvv);

	if (live_vv == 0) {
		/*
		**	since no live VV's, proceed to next step
		**	of putting vsn in recycle-to-volman state
		*/
		nLogMsg(EL_STATUS1, id, "%.*s: no VV's to recover", 
							VSNLEN, vsnp);
		(void) cursor->c_close(cursor);
		if (MarkVsnRecycledVVs(vsnp, 0)) {
			return(1);
		}
		return(0);
	}
	/*
	**	init slot counts
	*/
	memset(slots, 0, sizeof(slots));

	/*
	**	cursor & data are still good
	*/
	for (nrecs=1; ; nrecs++) {
		int	nvsns, nslots, slotx = -1;
		VV_t	*vvp;

		/*
		**	resize list if necc
		*/
		if (nrecs == listsize) {
			nLogMsg(EL_STATUS1, id, "surprise - listsize 2small");
			listsize += 100;
			VVlist = (OFF_T *) 
				nReAlloc(VVlist, listsize * sizeof(OFF_T));
		}
		/*
		**	save current position for next pass
		*/
		COPY_DBT(VVlist[nrecs], data);

		/*
		**	load record
		*/
		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0)) {    /*  0 = don't translate record */
			(void) cursor->c_close(cursor);
			return(-1);
		}
		vvp = Vbuf2VV();
		(void)strcpy(vvname, VV2Name(vvp));
nLogMsg(EL_STATUS1, id, "--- %d  %s  0x%llx", nrecs, vvname, vv_addr);

		if (vbuf.vb_status == VVS_HOT) {
			/*
			**	hot VV's shouldn't be on tape
			*/
			nLogMsg(EL_FATAL, id, "hot VV (%s) on tape -??", 
						vvname);
		}
		if (vbuf.vb_status != VVS_FINISHED) {
			nLogMsg(EL_ERROR1, id, "%s: %s rather than finished",
						vvname, 
						vvStatusStr(vbuf.vb_status));
			vvnotmigrated++;
		}
		nvsns = CHAR2INT(vbuf.vb_nvsns);
		nslots = 0;
		for (slot=0; slot<nvsns; slot++) {
			if (memcmp(vsnp, cslots[slot]->vb_vsn, VSNLEN))
				continue;
			nslots++;
			slots[slot]++;
			slotx = slot;
		}
		if (!nslots) {
			nLogMsg(EL_ERROR1, id, 
				"vv_vsn idx corrupted: no slot for %.*s in %s",
				VSNLEN, vsnp, vvname);
			vvnovsn++;
		} else if (nslots > 1) {
			nLogMsg(EL_ERROR1, id, "%s in >1 slot on %.*s",
						vvname, VSNLEN, vsnp);
			multi_slot++;
		}

		/*
		**	get next record index
		*/
		ZERO_OBJ(data);
		if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
			if (dberr != DB_NOTFOUND)
				nLogMsg(EL_FATAL, id, "c_get(): %s",
							db_strerror(dberr));
			break;
		}
		if (memcmp(vsnp, key.data, VSNLEN))
			break;
	}
	(void) cursor->c_close(cursor);

	for (slot=0; slot<MAXVSN; slot++) {
		if (slots[slot])
			nLogMsg(EL_STATUS1|EL_NOTIME, id, 
							"   slot %d total: %d",
							slot, slots[slot]);
	}
	if (vvnotmigrated)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0,
			"  total VV's not 'migrated': %d", vvnotmigrated);

	if (multi_slot)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, 
			"  total w/ multiple slots on vsn: %d", multi_slot);
	if (vvnovsn)
		nLogMsg(EL_STATUS1|EL_NOTIME, 0,
			"  total w/ vsn not in record: %d", vvnovsn);

	/*
	**	don't go forward if anything looks fishy
	*/
	if (vvnotmigrated  ||  vvnovsn  ||  multi_slot) {
		VVError = VVE_DBASE;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
			"%.*s: unexpected VV status - see log", VSNLEN, vsnp);
		return(-1);
	}

	/*
	**	mark vv & vv.vsn recycling in live VV's
	*/
	for (rec=0; rec<nrecs; rec++) {
		int	nvsns;
		VV_t	*vvp;
		OFF_T	size;

		/*
		**	get VV again
		*/
		vv_addr = VVlist[rec];
		if (LoadVV(id, 0)) 	/*  0 = don't translate record */
			return(-1);

		vvp = Vbuf2VV();
		(void)strcpy(vvname, VV2Name(vvp));
		nvsns = CHAR2INT(vbuf.vb_nvsns);

		for (slot=0; slot<nvsns; slot++) {
			if (!memcmp(vsnp, cslots[slot]->vb_vsn, VSNLEN))
				break;
		}

		/*
		**	set VV recycling (prevents freeing from
		**		disk until another copy has been written
		**		to tape) & write back record
		*/
		vbuf.vb_status = VVS_RECYCLING;
		cslots[slot]->vb_vflags[1] = VSN_RECYCLING;
		if (WriteVV(id))
			nLogMsg(EL_FATAL, id, "quitting");
		nLogMsg(EL_STATUS1, id, "%s: marked RECYCLING vsn %.*s slot %d",
					vvname, VSNLEN, vsnp, slot);

		/*
		**	put VV on write or restore-then-write queue
		*/
		if (DiskResidentSize(vvp, &size)) {
			int	vno;

			/*
			**	not resident - add to restore q 
			**		for this vsn
			*/
			restorevv++;
			if (C2BIN(DecInt, &vno, cslots[slot]->vb_nrec))
				nLogMsg(EL_FATAL, id, "nrec");
			(void)sprintf(vvkey, "%.*s_%4d", VSNLEN, vsnp, vno);
			STR_DBT(key, vvkey);
			if (dberr = db_vv_tp2_idx->get(db_vv_tp2_idx, NULL,
							&key, &data, 0)) {
				if (dberr != DB_NOTFOUND)
					nLogMsg(EL_FATAL, id, "db get(%s): %s",
						vvkey, db_strerror(dberr));
				STR_DBT(key, vvkey);
				FILL_DBT(data, vv_addr);
				if (dberr = db_vv_tp2_idx->put(db_vv_tp2_idx,
					    NULL, &key, &data, DB_NOOVERWRITE))
					nLogMsg(EL_ERROR1|EL_FATAL, id, 
						"db put(%s): %s", 
						vvkey, db_strerror(dberr));
			}
		} else {
			/*
			**	already disk-resident, so add to write q
			*/
			ondisk++;
			if (IndexVVix(id, vvname, "tp", db_vv_tp_idx))
				nLogMsg(EL_ERROR1, id, "rats");
		}
	}

	/*
	**	report findings
	*/
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "counts of 'live' VV's:");
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "  Restoring: %d", restorevv);
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "  On disk:   %d", ondisk);
	nLogMsg(EL_STATUS1|EL_NOTIME, id, "  Total:     %d", nrecs);

	nFree(VVlist);

	if (restorevv) {
		/*
		**	will be mounting tape to restore VV's -
		**	note status in vsn record
		*/
		vsnbuf.vb_status = VVS_RECYCLING;
		VsnMsg("recycling", 1);
		(void)WriteVsn(id);

		/*
		**	add vsn to recyc index to prompt (re)mount if
		**	volman/vvmd down
		*/
		(void)sprintf(vsnkey, "REST_%.*s", VSNLEN, vsnp);
		STR_DBT(key, vsnkey);
		FILL_DBT(data, vsn_addr);
		if (dberr = db_vsn_recyc_idx->put(db_vsn_recyc_idx, NULL,
						&key, &data, DB_NOOVERWRITE)) {
			VVError = VVE_DBASE;
			nLogMsg(EL_ERROR1|EL_CODE, id, "db put(%s): %s",
						vsnkey, db_strerror(dberr));
			return(-1);
		}
	} else {
		/*
		**	no need to mount tape, so proceed to volman recycle
		*/
		if (MarkVsnRecycledVVs(vsnp, 0))
			return(-1);
	}

	return(restorevv);
}

/*
**	UnRecycleVV() - VV has somehow been restored, so undo
**		any recycle indexing
*/
  static void
UnRecycleVV()
{
	char	*id = "UnRecycleVV";
	int	i, nrecyc = 0;

	for (i=0; i<MAXVSN; i++) {
		if (cslots[i]->vb_vflags[1] != VSN_RECYCLING)
			continue;
		(void)sprintf(vvkey, "%.*s_%4d", VSNLEN, cslots[i]->vb_vsn, 
						VInfo.v_vsns[i].v_nrec);
		STR_DBT(key, vvkey);
		if (dberr = db_vv_tp2_idx->del(db_vv_tp2_idx, NULL, &key, 0))
			nLogMsg(EL_ERROR1, id, "db del(%s): %s", 
						vvkey, db_strerror(dberr));
		cslots[i]->vb_vflags[1] = VSN_RECYCLED;
		nrecyc++;
	}
	if (nrecyc) {
		nLogMsg(EL_STATUS1, id, "%s: %d slots", 
					VV2Name(&VInfo.v_vv), nrecyc);
		if (WriteVV(id))
			nLogMsg(EL_FATAL, id, "write");
	}
}

/*
**	DBRecycleGetNextRestoreVV() - fill in VV info
*/
 int
DBRecycleGetNextRestoreVV(char *vsnp, int slot, VVInfo_t *vp)
{
	char	*id = "DBRecycleGetNextRestoreVV";
	OFF_T	size;
	DBC	*cursor;
	int	keylen;

	if (dberr = db_vv_tp2_idx->cursor(db_vv_tp2_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));

	/*
	**	since VV's that were indexed earlier may have
	**	been restored independantly, may need to retry
	*/
nextvv:
	(void)sprintf(vvkey, "%.*s_", VSNLEN, vsnp);
	STR_DBT(key, vvkey);
	keylen = key.size;
	if (dberr = cursor->c_get(cursor, &key, &data, DB_SET_RANGE)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
						db_strerror(dberr));
		/*
		**	indicate nothing found
		*/
		(void) cursor->c_close(cursor);
		vp->v_vsns[slot].v_nrec = -1;
		return(0);
	}
	if (memcmp(vvkey, key.data, keylen)) {
		/*
		**	indicate nothing found
		*/
		(void) cursor->c_close(cursor);
		vp->v_vsns[slot].v_nrec = -1;
		return(0);
	}
	COPY_DBT(vv_addr, data);
	if (LoadVV(id, 1))
		nLogMsg(EL_FATAL, id, "loading");

	/*
	**	check record
	*/
	if (memcmp(vsnp, cslots[slot]->vb_vsn, VSNLEN)) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
						"%.*s: got %.*s for %s slot %d",
						VSNLEN, vsnp, 
						VSNLEN, cslots[slot]->vb_vsn,
						VV2Name(&VInfo.v_vv), slot);
		(void) cursor->c_close(cursor);
		return(1);
	}
	/* check 'hidden', ascii-record-only vsn flag */
	if (cslots[slot]->vb_vflags[1] != VSN_RECYCLING) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "%.*s: flags[1] = [%c]",
						VSNLEN, vsnp, 
						cslots[slot]->vb_vflags[1]);
		(void) cursor->c_close(cursor);
		return(1);
	}

	/*
	**	make sure nrec sequence is consistent
	*/
	if (VInfo.v_vsns[slot].v_nrec <= vp->v_vsns[slot].v_nrec) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "%.*s: nrec %d -> %d",
						VSNLEN, vsnp,
						vp->v_vsns[slot].v_nrec,
						VInfo.v_vsns[slot].v_nrec);
		(void) cursor->c_close(cursor);
		return(1);
	}

	/*
	**	make sure VV is still nonresident
	*/
	if (!DiskResidentSize(Vbuf2VV(), &size)) {
		UnRecycleVV();
		goto nextvv;
	}

	(void) cursor->c_close(cursor);

	/*
	**	nonresident.. a final consistency check
	*/
	if (VInfo.v_status != VVS_RECYCLING) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, "%.*s not recycling (%s)",
						VSNLEN, vsnp, 
						vvStatusStr(vsnbuf.vb_status));
		return(1);
	}

	/*
	**	set VV info & say 'ok'
	*/
	*vp = VInfo;
	return(0);
}

/*
**	DBRecycRestoredVV() - VV has been restored to disk
**		in a vsn recycle operation
*/
 void
DBRecycRestoredVV(VVInfo_t *vp)
{
	char	*id = "DBRecycRestoredVV";

	if (DBLookupVV(&vp->v_vv) == NULL)
		nLogMsg(EL_FATAL, id, "no vv");

	/*
	**	undo vsn recycle indexing .. TODO - maybe wait till resaved?
	*/
	UnRecycleVV();
}

/*
**	DBRecycleVsnRestoresDone() - vv's have been recovered, so
**		mark vsn record and add vsn to volman recycle index
*/
 int
DBRecycleVsnRestoresDone(char *vsnp)
{
	char	*id = "DBRecycleVsnRestoresDone";

	nLogMsg(EL_STATUS1, id, "%.*s", VSNLEN, vsnp);

	if (DBLookupVsn(vsnp) == NULL)
		return(1);

	if (vsnbuf.vb_status != VVS_RECYCLING) {
		VVError = VVE_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, "unexpected recycledone");
		return(1);
	}

	if (MarkVsnRecycledVVs(vsnp, 1))
		return(1);
	return(0);
}


/*
 *  DBDeleteVsn() - recycle to volman completed, so mark
 *	record deleted & un-index
 */
int
DBDeleteVsn(char *vsnp)
{
	char	*id = "DBDeleteVsn";

	if (DBLookupVsn(vsnp) == NULL)
		return(1);

	nLogMsg(EL_STATUS1, id, "DELETING %.*s status %s", VSNLEN, vsnp,
						vvStatusStr(vsnbuf.vb_status));

	if (vsnbuf.vb_status != VVS_RECYCLED) {
		VVError = VVE_INTERNAL;
		nLogMsg(EL_ERROR1|EL_CODE, id, "unexpected delete");
		return(1);
	}

	/*
	**	update vsn status
	*/
	vsnbuf.vb_status = VVS_DELETED;
	VsnMsg("deleted", 1);
	(void)WriteVsn(id);

	/*
	**	Delete volman-recycle status from recycle index
	*/
	(void)sprintf(vsnkey, "DEL_%.*s", VSNLEN, vsnp);
	STR_DBT(key, vsnkey);
	if (dberr = db_vsn_recyc_idx->del(db_vsn_recyc_idx, NULL, &key, 0)) {
		if (vsnbuf.vb_status == VVS_RECYCLED) {
			VVError = VVE_DBASE;
			nLogMsg(EL_ERROR1|EL_CODE, id, "db del(%s): %s", 
						vsnkey, db_strerror(dberr));
			return(1);
		}
	}

	/*
	**	Delete VV-recycle status from recycle index
	*/
	(void)sprintf(vsnkey, "REST_%.*s", VSNLEN, vsnp);
	STR_DBT(key, vsnkey);
	if (dberr = db_vsn_recyc_idx->del(db_vsn_recyc_idx, NULL, &key, 0))
		nLogMsg(EL_ERROR2, id, "db del(%s): %s",
						vsnkey, db_strerror(dberr));
		

	/*
	**	delete from vsn index
	*/
	(void)sprintf(vsnkey, "%.*s", VSNLEN, vsnp);
	STR_DBT(key, vsnkey);
	if (dberr = db_vsn_vsn_idx->del(db_vsn_vsn_idx, NULL, &key, 0))
		nLogMsg(EL_ERROR1, id, "vsn_vsn del(%s): %s",
						vsnkey, db_strerror(dberr));

	return(0);
}


 Vsn_t *
DBFinishVsn(char *vsnp)
{
	char	*id = "DBFinishVsn";

	/*
	**	get vsn record
	*/
	if (DBLookupVsn(vsnp) == NULL) {
		VVError = VVE_USER;
		nLogMsg(EL_ERROR1|EL_CODE, id, "vsn unknown [%.*s]", 
								VSNLEN, vsnp);
		return(NULL);
	}
	if (VsnInfo.vs_status != VVS_HOT) {
		VVError = VVE_USER;
		nLogMsg(EL_ERROR1|EL_CODE, id, "%.*s: vsn not hot",
								VSNLEN, vsnp);
		return(NULL);
	}
	(void)UnIndexVsnHot(id);
	vsnbuf.vb_status = VVS_FINISHED;
	VsnMsg("finished", 1);
	(void)WriteVsn(id);
	return(&VsnInfo);
}

 int
DBFinishClass(char *class, int *nStillHot, int *nFinishedVV, int *nWaitVV, 
		int *nTapes)
{
	char		*id = "DBFinishClass";
	char		searchkey[100];
	int		i, keylen, errs = 0;
	char		vvname[VV_NAMELEN];
	DBC		*cursor;

	*nStillHot = 0;
	*nFinishedVV = 0;
	*nWaitVV = 0;
	*nTapes = 0;

	if (dberr = db_vv_hot_idx->cursor(db_vv_hot_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));

	/*
	**	look at all hot vv's in class
	*/
	(void)sprintf(searchkey, "%s_", class);
	STR_DBT(key, searchkey);
	keylen = key.size;
	for (i=0; ; i++) {
		ZERO_OBJ(data);
		if (i)
			dberr =cursor->c_get(cursor, &key, &data, DB_NEXT);
		else
			dberr =cursor->c_get(cursor, &key, &data,DB_SET_RANGE);
		if (dberr == DB_NOTFOUND)
			break;
		if (dberr)
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		if (memcmp(key.data, searchkey, keylen))
			break;

		/*
		**	index checks out - load & inspect record
		*/
		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 1))
			nLogMsg(EL_FATAL, id, "loading");
		(void)strcpy(vvname, VV2Name(&VInfo.v_vv));

		/*
		**	make double-sure VV can still be written
		*/
		if (VInfo.v_status != VVS_HOT) {
			/*
			 *  TODO - delete from index once the reason
			 *	for this discrepancy is figured out
			 *	(if in fact it ever happens)
			 */
			errs++;
			nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
						"%s: already finished",
						vvname);
			continue;
		}

		/*
		**	make sure VV isn't busy
		*/
		if (VInfo.v_use != VVU_IDLE) {
			(*nStillHot)++;
			nLogMsg(EL_CONSOLE0, id, "%s: busy", vvname);
			continue;
		}
		/*
		**	delete from hot index
		*/
		if (dberr = cursor->c_del(cursor, 0)) {
			errs++;
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id,
					"c_del(class_id %s): %s", 
					searchkey, db_strerror(dberr));
		}

		/*
		**	mark in table
		*/
		vbuf.vb_status = VVS_FINISHED;
		VInfo.v_status = VVS_FINISHED;
		SetTime(vbuf.vb_finish, Now);
		VInfo.v_finish = Now;
		if (WriteVV(id))
			nLogMsg(EL_FATAL, id, "writing");
			
		/*
		**	add to index of VV's to be written to tape
		*/
		(void)IndexVVix(id, vvname, "tp", db_vv_tp_idx);

		nLogMsg(EL_STATUS1, id, "FINISHED %s id %s", vvname,
							IdString(VInfo.v_id));
		(*nFinishedVV)++;
	}

	(void) cursor->c_close(cursor);

	/*
	**	count all the finished VV's in class that are waiting
	**		to go to tape
	*/
	if (dberr = db_vv_tp_idx->cursor(db_vv_tp_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));

	(void)sprintf(searchkey, "%s.", class);
	STR_DBT(key, searchkey);
	keylen = key.size;
	for (i=0; ; i++) {

		ZERO_OBJ(data);
		if (i)
			dberr =cursor->c_get(cursor, &key, &data, DB_NEXT);
		else
			dberr =cursor->c_get(cursor, &key, &data,DB_SET_RANGE);
		if (dberr == DB_NOTFOUND)
			break;
		if (dberr)
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		if (memcmp(key.data, searchkey, keylen))
			break;

		(*nWaitVV)++;
	}
	(void) cursor->c_close(cursor);

	/*
	**	count the hot tapes assigned to class
	*/
	if (dberr = db_vsn_hot_idx->cursor(db_vsn_hot_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));
	(void)sprintf(searchkey, "%s_", class);
	STR_DBT(key, searchkey);
	keylen = key.size;
	for (i=0; ; i++) {

		ZERO_OBJ(data);
		if (i)
			dberr =cursor->c_get(cursor, &key, &data, DB_NEXT);
		else
			dberr =cursor->c_get(cursor, &key, &data,DB_SET_RANGE);
		if (dberr == DB_NOTFOUND)
			break;
		if (dberr)
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		if (memcmp(key.data, searchkey, keylen))
			break;

		(*nTapes)++;
	}
	(void) cursor->c_close(cursor);
	return(0);
}

  void
MDiskCk(vvpHdr_t *hp, int slot)
{
	char	*id = "MDiskCk";
	int	err1 = 0, err2 = 0;
	DBC	*cursor;

	nLogMsg(EL_STATUS1, id, "checking disk -------------------------");
	if (dberr = db_vv_hot_idx->cursor(db_vv_hot_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));

	/*
	**	all hot VV's should be on disk
	*/
	while (1) {
		VV_t	*vvp;
		OFF_T	dsize, vdata;

		ZERO_OBJ(key);
		ZERO_OBJ(data);
		if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
			if (dberr == DB_NOTFOUND)
				break;
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		}
		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0)) {
			err1++;
			continue;
		}
		vvp = Vbuf2VV();
		if (DiskResidentSize(vvp, &dsize)) {
			if (errno == ENOENT) {
				VVError = VVE_NOTFOUND;
				nLogMsg(EL_CODE|EL_ERROR1, id,
					"%s: HOT & GONE", VV2Name(vvp));
				SendErr(hp->vh_seq, slot);
			}
			err1++;
		}
	}
	(void) cursor->c_close(cursor);

	/*
	**	all finished (not yet taped) VV's should be on disk
	*/
	if (dberr = db_vv_tp_idx->cursor(db_vv_tp_idx, NULL, &cursor, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));

	while (1) {
		VV_t	*vvp;
		OFF_T	dsize, vdata;

		ZERO_OBJ(key);
		ZERO_OBJ(data);
		if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
			if (dberr == DB_NOTFOUND)
				break;
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		}

		COPY_DBT(vv_addr, data);
		if (LoadVV(id, 0)) {
			err2++;
			continue;
		}
		vvp = Vbuf2VV();
		if (DiskResidentSize(vvp, &dsize)) {
			if (errno == ENOENT) {
				VVError = VVE_NOTFOUND;
				nLogMsg(EL_CODE|EL_ERROR1, id,
					"%s: FINISHED & GONE", VV2Name(vvp));
				SendErr(hp->vh_seq, slot);
			}
			err2++;
		}
		C2BIN(HexOFF, &vdata, vbuf.vb_data);
		if (dsize != vdata) {
			VVError = VVE_INTERNAL;
			nLogMsg(EL_CODE|EL_ERROR1, id,
					"%s: FINISHED & SIZE DIFF", 
					VV2Name(vvp));
			SendErr(hp->vh_seq, slot);
			err2++;
		}
	}
	(void) cursor->c_close(cursor);
	if (err1 || err2) {
		nLogMsg(EL_ERROR1|EL_CONSOLE0|EL_CODE, id, 
				"%d errs with 'hot', %d errs with 'finished'", 
				err1, err2);
		SendErr(hp->vh_seq, slot);
	} else
		nLogMsg(EL_STATUS1, id, 
				"no errors -----------------------------");
	SendAck(hp->vh_seq, slot);
}

/*
**	DBGetIncompleteRecycle() - go thru index of finished VV's that
**		haven't been written to tape.
*/
 Vsn_t *
DBGetIncompleteRecycle(int i, int *recovp)
{
	char		*id = "DBGetIncompleteRecycle";
	static DBC	*cursor;
	DBC		*cursor2;

	if (i==0) {
		if (dberr = db_vsn_recyc_idx->cursor(db_vsn_recyc_idx, 
							NULL, &cursor, 0))
			nLogMsg(EL_FATAL, id, "cursor(): %s", 
							db_strerror(dberr));
	}
nextkey:
	ZERO_OBJ(data);
	if (dberr = cursor->c_get(cursor, &key, &data, DB_NEXT)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "c_get(): %s", 
							db_strerror(dberr));
		(void) cursor->c_close(cursor);
		return(NULL);
	}
	/* looking for restore in progress */
	if (memcmp(key.data, "REST_", 5))
		goto nextkey;

	/*
	**	get vsn record
	*/
	COPY_DBT(vsn_addr, data);
	if (LoadVsn(id))
		nLogMsg(EL_FATAL, id, "can't load vsn rec (%.*s)", 
						key.size, key.data);

	if (vsnbuf.vb_status != VVS_RECYCLING) {
		nLogMsg(EL_ERROR1, id, "skipping %.*s status %s",
						VSNLEN, VsnInfo.vs_vsn,
						vvStatusStr(vsnbuf.vb_status));
		goto nextkey;
	}

	/*
	**	see how many VV's need to be restored
	*/
	if (dberr = db_vv_tp2_idx->cursor(db_vv_tp2_idx, NULL, &cursor2, 0))
		nLogMsg(EL_FATAL, id, "cursor(): %s", db_strerror(dberr));
	*recovp = 0;
	(void)sprintf(vvkey, "%.*s_", VSNLEN, VsnInfo.vs_vsn);
	STR_DBT(key, vvkey);
	ZERO_OBJ(data);
	if (dberr = cursor2->c_get(cursor2, &key, &data, DB_SET_RANGE)) {
		if (dberr != DB_NOTFOUND)
			nLogMsg(EL_FATAL, id, "cursor2 c_get(): %s", 
							db_strerror(dberr));
		(void) cursor2->c_close(cursor2);
		return(&VsnInfo);
	}
	*recovp = 1;
	while (1) {
		*recovp += 1;
		ZERO_OBJ(data);
		if (dberr = cursor2->c_get(cursor2, &key, &data, DB_NEXT)) {
			if (dberr != DB_NOTFOUND)
				nLogMsg(EL_FATAL, id, "cursor2 c_get2: %s",
							db_strerror(dberr));
			break;
		}
	}
	(void) cursor2->c_close(cursor2);
	return(&VsnInfo);
}
