
/*
 *  vvm_db4.c - for the run-time-only 'freelist' index of on-disk VV's
 */

#include "vvmd.h"
#include "avltree.h"
#include "vvm_db.h"

extern VVInfo_t		VInfo;
extern vbuf_t		vbuf;
extern OFF_T		vv_addr;
extern int		db_vv_fd;

static AVL_IX_DESC	avl_ondisk_idx;
static AVL_IX_REC	*avl_entry = NULL, *avl_entry2 = NULL;

/*******************************************************************/

/*
**	InitOndiskIndex() - create throwaway index
*/
void
InitOndiskIndex()
{
	char	*id = "InitOndiskIndex";

	/*
	**	start index and allocate space for record
	*/
	avl_create_index(&avl_ondisk_idx, AVL_NO_DUP_KEYS, 0);
	avl_entry = (AVL_IX_REC *) nAlloc(sizeof(AVL_IX_REC) + VV_NAMELEN + 1);
	avl_entry2 = (AVL_IX_REC *) nAlloc(sizeof(AVL_IX_REC) + VV_NAMELEN + 1);
}

/*
**	OndiskKey() - make key based on atime+vv
*/
static void
OndiskKey()
{
	char	*cp = avl_entry->key + sizeof(vbuf.vb_atime);

	memcpy(avl_entry->key, vbuf.vb_atime, sizeof(vbuf.vb_atime));
	*cp = '_';
	(void)strcpy(cp+1, VV2Name(Vbuf2VV()));
}

/*
**	AddOndisk() - add current VV to ondisk index
*/
void
AddOndisk(char *id, time_t atime)
{
	SetTime(vbuf.vb_atime, atime);
	OndiskKey();
	avl_entry->recptr = (void *)nAlloc(sizeof(OFF_T));
	memcpy(avl_entry->recptr, &vv_addr, sizeof(OFF_T));
	if (avl_add_key(avl_entry, &avl_ondisk_idx) != AVL_IX_OK)
		nLogMsg(EL_FATAL, "AddOndisk", "(%s) avl_add_key(%s)", 
						id, avl_entry->key);
	WriteVV(id);
}


/*
**	DeleteOndisk() - delete current VV from ondisk index
*/
void
DeleteOndisk(VV_t *vvp)
{
	char	*id = "DeleteOndisk";

	if (DBLookupVV(vvp) == NULL)
		nLogMsg(EL_FATAL, id, "DBLookupVV()");

	OndiskKey();
	if (avl_find_key(avl_entry, &avl_ondisk_idx) != AVL_IX_OK)
		nLogMsg(EL_FATAL, id, "avl_find_key(%s)", avl_entry->key);
	nFree(avl_entry->recptr);
	if (avl_delete_key(avl_entry, &avl_ondisk_idx) != AVL_IX_OK)
		nLogMsg(EL_FATAL, id, "avl_delete_key(%s)", avl_entry->key);

nLogMsg(EL_STATUS1, id, "deleted [%s]", avl_entry->key);

	memset(vbuf.vb_atime, ' ', sizeof(vbuf.vb_atime));
	WriteVV(id);
}

/*
**	GetOndisk() - get next-VV-to-free loaded
*/
VVInfo_t *
GetOndisk(int which)
{
	char	*id = "GetOndisk";
	int	ret;

	if (which == 0)
		avl_first_key(&avl_ondisk_idx);

	ret = avl_next_key(avl_entry2, &avl_ondisk_idx);

	if (ret == AVL_EOIX)
		return(NULL);

	if (ret != AVL_IX_OK)
		nLogMsg(EL_FATAL, id, "avl_next_key()");

	vv_addr = *((OFF_T *)avl_entry2->recptr);
	if (LoadVV(id, 1))
		nLogMsg(EL_FATAL, id, "LoadVV()");
	return(&VInfo);
}


void
LogDisk()
{
	char	*id = "LogDisk";

	avl_first_key(&avl_ondisk_idx);

	while (avl_next_key(avl_entry, &avl_ondisk_idx) == AVL_IX_OK) {
		char	buf[80];
		time_t	tt;
/*
		vv_addr = *((OFF_T*)avl_entry->recptr);
		if (LoadVV(id, 1))
			nLogMsg(EL_FATAL, id, "LoadVV()");
*/
		char	*cp = strchr(avl_entry->key, '_');
		*cp = '\0';
		cp++;
		tt = GetTime(avl_entry->key);
		(void)sprintf(buf, "%s %s", cp, bsd_ctime(&tt)); 
		*strchr(buf, '\n') = '\0';
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, buf);
	}
}
