/*
**	vvm_disk.c - changes to virtual volume on disk and cache mgt
*/

#define FS_USE_SPACE_PERCENT	40

#define WRITEADVANTAGE		86400/* 1 day grace period for writable VV's */


#include "vvmd.h"

#include <dirent.h>

#ifdef __NetBSD__
# include <sys/mount.h>
#endif
#ifdef __sgi
# include <sys/statfs.h>
#endif

extern int		volman_failed;

static STAT_T		sbuf;

static struct statfs	fsbuf;
static long		reserved_blocks = 0;	/* for VV's coming in */
static long		blocksize = -1;

#define MB	(1024 * 1024)

static int
FreeSpace(OFF_T freesize)
{
	char		*id = "FreeSpace";
	int		i, end, first = 1;
	OFF_T		size_freed = 0;
	int		skiphot=0, skiphoterr=0;
	int		hotfree=0, migratedfree=0, finishednotfree=0, recover=0;
	OFF_T		hotfreesize=0, migratedfreesize=0;
	OFF_T		finishednotfreesize=0, recoversize=0, recyclingsize=0;
	int		migrating=0, recycling=0;
	OFF_T		migratingsize=0;
	int		dayold=0, weekold=0, monthold=0, oldold=0;
	time_t		tmptm;
	VVInfo_t	*vip;

nLogMsg(EL_STATUS1, id, " need %lld  sizeof(OFF_T) %d", (long long)freesize,
sizeof(OFF_T));
if (freesize < 0)
nLogMsg(EL_FATAL, id, "need < 0");
	/*
	**	plow thru the list, 
	*/
	for (i=0; vip = GetOndisk(i); i++) {
		char		*fname;
		VV_t		*vvp;

		/*
		**	if enough space freed, quit
		*/
		if (freesize < size_freed)
			break;

		/*
		**	make sure it's still on disk (should be..)
		*/
		vvp = &vip->v_vv;
		fname = VV2Path(vvp);
nLogMsg(EL_STATUS1, id, "---- %s", fname);
		if (STAT(fname, &sbuf) == -1) {
			nLogMsg(EL_SYS|EL_CONSOLE0|EL_FATAL, id, 
					"WHAT?? stat %s:", fname);
			continue;
		}

		if (first) {
			/*
			**	see how old oldest is
			*/
			if (Now - sbuf.st_atime < 86400)
					nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
						"Need to free space & oldest%s",
						" node is < 24 hours 'old'");
			first = 0;
		}

		switch (vip->v_status) {

		    case VVS_MIGRATED:
			/*
			**	Not hot, so can free it directly.
			*/
			nLogMsg(EL_STATUS1, id, "migrated/free: %s", fname);
			if (unlink(fname) == -1) {
				nLogMsg(EL_SYS|EL_CONSOLE0, id, 
							"unlink(%s)", fname);
				if (errno == ENOENT)
					nLogMsg(EL_FATAL, id, "ENOENT: %s", 
								fname);
			} else {
nLogMsg(EL_STATUS1, id, "unlink migrated: %s  size %d", 
fname, (int)sbuf.st_size);
				DBFreedVV(vip);
				migratedfree++;
				migratedfreesize += sbuf.st_size;
				size_freed += sbuf.st_size;
				DeleteOndisk(vvp);
			}
			break;

		    case VVS_HOT:
			/*
			**	if hot & idle, finish (starts write to tape)
			**		but don't count the space
			*/
			if (vip->v_use != VVU_IDLE) {
				skiphot++;
				continue;
			}
			nLogMsg(EL_STATUS1, id, "finishing off a hot one: %s", 
								fname);
			vip = DBFinishVV(vvp);
			if (vip == NULL) {
				skiphoterr++;
				continue;
			}
			if (DiskFinish(vvp)) {
				skiphoterr++;
				continue;
			}
			CWriteVV2Vsns(vip);
			hotfree++;
			hotfreesize += sbuf.st_size;
			break;

		    case VVS_FINISHED:
			/*
			**	presumably migrating or stuck?
			*/
			if (!DBMigrating(vvp)) {
				nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
						"%s: finished but idle(?)",
						fname);
				finishednotfree++;
				finishednotfreesize += sbuf.st_size;
				continue;
			}
			nLogMsg(EL_DEBUG2, id, "already migrating/queued: %s", 
								fname);
			migrating++;
			migratingsize += sbuf.st_size;
			break;

		    case VVS_RECYCLING:
			recycling++;
			recyclingsize += sbuf.st_size;
			break;

		    case VVS_RECOVER:
			recover++;
			recoversize += sbuf.st_size;
			break;

		    default:
			nLogMsg(EL_CONSOLE0|EL_FATAL, id, "missing status %s",
						vvStatusStr(vip->v_status));
		}

		/*
		**	statistics
		*/
		tmptm = Now - sbuf.st_atime;
		if (tmptm < 86400)
			dayold++;
		else if (tmptm < 604800)
			weekold++;
		else if (tmptm < 2419200)
			monthold++;
		else
			oldold++;
			
	}

	end = 0;

	if (hotfree  || migratedfree) {
		end++;
		nLogMsg(EL_STATUS1, id, 
				"---- Space free stats ----------------------");
		if (dayold  ||  weekold  ||  monthold  ||  oldold)
			nLogMsg(EL_STATUS1|EL_NOTIME, 0, 
			"AGES:  <1day: %d  <1week: %d  <1month: %d  older: %d",
			dayold, weekold, monthold, oldold);
	}
	if (hotfree) {
		double	megs = (double)hotfreesize / MB;

		end++;
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, 
			"hot finished: %d VV's, total size %.3f MB", 
			hotfree, megs);
	}
	if (migratedfree) {
		double	megs = (double)migratedfreesize / MB;

		end++;
		nLogMsg(EL_STATUS1|EL_NOTIME, 0, 
			"migrated: %d VV's total size %.3f MB", 
			migratedfree, megs);
	}
	if (migrating && volman_failed < 2) {
		double	megs = (double)migratingsize / MB;

		end++;
		nLogMsg(EL_STATUS1|EL_CONSOLE0, id,
		    "already migrating/scheduled: %d VV's total size %.3f MB",
			migrating, megs);
	}
	if (finishednotfree) {
		double	megs = (double)finishednotfreesize / MB;

		end++;
		nLogMsg(EL_STATUS1|EL_CONSOLE0, id,
			"finished not free: %d VV's, total size %.3f MB", 
			finishednotfree, megs);
	}
	if (recover) {
		double	megs = (double)recoversize / MB;

		end++;
		nLogMsg(EL_STATUS1|EL_CONSOLE0, id,
			"vsn recover processing: %d VV's, total size %.3f MB",
			recover, megs);
	}
	if (recycling) {
		double   megs = (double)recyclingsize / MB;

		end++;
		nLogMsg(EL_STATUS1|EL_CONSOLE0, id,
			"vsn recycle processing: %d VV's, total size %.3f MB",
			recycling, megs);
	}
	if (end)
		nLogMsg(EL_STATUS1, 0, 
		    "-------------------------------------------------------");
{
time_t now; (void)time(&now);
if (now - Now > 60)
nLogMsg(EL_CONSOLE0|EL_STATUS1, id, "TIME = %d", now - Now);
}
	if (size_freed < freesize) {
		VVError = VVE_UNAVAIL;
		nLogMsg(EL_CONSOLE0|EL_ERROR1|EL_CODE, id, 
				"Cache space unavailable!!");
		return(1);
	}
	return(0);
}

/*
**	DiskReserve() - for simplicity, always reserve a whole VV's size
*/
  int
DiskReserve(VVMClass_t *cip)
{
	char	*id = "DiskReserve";
	long	free_blocks, used_blocks, max_blocks, needed_blocks;
	double	tmp1;
	OFF_T	needed_bytes;

	/*
	**	check that there is space for the VV;
	**		for now at least, do a statfs
	**		rather than trying to keep track
	**		between DiskStat() calls
	*/
#ifdef __sgi
	if (statfs(vvVVDir, &fsbuf, sizeof(fsbuf), 0))
#else
	if (statfs(vvVVDir, &fsbuf))
#endif
		nLogMsg(EL_FATAL|EL_SYS|EL_CONSOLE0, id, 
				"statfs cache dir %s", vvVVDir);

	free_blocks = fsbuf.f_bfree;
	used_blocks = fsbuf.f_blocks - free_blocks;
	/* use floating-point to avoid overflow on multiplication */
	tmp1 = ((double)fsbuf.f_blocks * FS_USE_SPACE_PERCENT) / 100.0;
	max_blocks = (long) tmp1;
nLogMsg(EL_STATUS1, id, "free %ld  total %ld  used %ld max usable %ld",
fsbuf.f_bfree, fsbuf.f_blocks, used_blocks, max_blocks);

	free_blocks = max_blocks - used_blocks;
	free_blocks -= reserved_blocks;

nLogMsg(EL_STATUS1, id, "tot usable free %ld  reserved %ld tot %ld",
max_blocks - used_blocks, reserved_blocks, free_blocks);

	needed_blocks = cip->vc_class.cs_vvsize / fsbuf.f_bsize + 2;

	if (free_blocks < needed_blocks) {
		/*
		**	make some space
		*/
nLogMsg(EL_STATUS1, id, "need %ld blocks (size %d) free %ld (%ld)",
needed_blocks, fsbuf.f_bsize, free_blocks, labs(free_blocks));
		if (free_blocks < 0L) {
nLogMsg(EL_STATUS1, id, "bumping needed_blocks");
			needed_blocks += labs(free_blocks);
		}
		needed_bytes = (OFF_T)needed_blocks * (OFF_T)fsbuf.f_bsize;

		if (FreeSpace(needed_bytes)) {
			CWCheckQs(1);	/* start any waiting saves */
			return(1);
		}
	}

	/*
	**	add to reserved blocks
	*/
nLogMsg(EL_STATUS1, id, "reserved_blocks %d -> %d",
(int)reserved_blocks, (int)reserved_blocks + (int)needed_blocks);

	reserved_blocks += needed_blocks;

	return(0);
}

  void
DiskUnreserve(VVMClass_t *cip)
{
	/*
	**	refund reserved blocks
	*/
nLogMsg(EL_STATUS1, "DiskUnreserve", "reserved_blocks %d -> %d",
(int)reserved_blocks, (int)reserved_blocks - 
cip->vc_class.cs_vvsize / (int)fsbuf.f_bsize - 2);
	reserved_blocks -= cip->vc_class.cs_vvsize / (int)fsbuf.f_bsize + 2;
}


/*
**	DiskCreate() - start new vv
*/
  int
DiskCreate(VVInfo_t *vip, VVMClass_t *cip)
{
	char	*id = "DiskCreate";
	char	*fname = VV2Path(&vip->v_vv);
	int	fd, ret;

	/*
	**	open/create the file
	*/
	if ((fd = open(fname, O_RDWR|O_CREAT|O_EXCL, 0600)) == -1) {
		VVError = VVE_SYS;
		nLogMsg(EL_CODE|EL_SYS|EL_CONSOLE0, 0, "open/create %s",
								fname);
		return(1);
	}
nLogMsg(EL_STATUS1, id, "opened %s for id [%*.*s]", fname,
sizeof(vip->v_id), sizeof(vip->v_id), vip->v_id);
	/*
	**	write VVM hdr
	*/
	if (WriteDiskHdr(fd, vip->v_vv.class, vip->v_vv.serial, vip->v_id, 
								Now)) {
		close(fd);
		(void)unlink(fname);
		return(1);
	}
	close(fd);
	return(0);
}


/*
**	DiskFinish() - 'close' vv to future writing
*/
  int
DiskFinish(VV_t *vvp)
{
	char	*fname = VV2Path(vvp);


	/* TODO - check actual size, update dbase */

	if (chmod(fname, 0400) == -1) {
		VVError = VVE_SYS;
		nLogMsg(EL_CODE|EL_SYS|EL_CONSOLE0, 0, "chmod 0400 %s", fname);
		return(1);
	}
	return(0);
}

/*
**	DiskRecycle() - throw away disk-resident vv if it exists
*/
  int
DiskRecycle(VV_t *vvp)
{
	char	*fname = VV2Path(vvp);

	if (unlink(fname) == -1) {
		if (errno != ENOENT) {
			VVError = VVE_SYS;
			nLogMsg(EL_CODE|EL_SYS, 0, "unlink %s", fname);
			return(1);
		}
	}
	nLogMsg(EL_DEBUG1, 0, "unlinked %s", fname);
	DeleteOndisk(vvp);
	return(0);
}

/*
**	DiskResidentSize() - verify that vv is on disk; if err say 'no'
*/
  int
DiskResidentSize(VV_t *vvp, OFF_T *size)
{
	char	*fname = VV2Path(vvp);

	if (STAT(fname, &sbuf) == -1) {
		if (errno != ENOENT)
			nLogMsg(EL_SYS, "DiskResident", "stat(%s)", fname);
		return(1);
	}
nLogMsg(EL_STATUS1,0," %s size %lld", fname, (long long)sbuf.st_size);
	*size = sbuf.st_size;
	return(0);
}

/***************************************
 *  disk space mgt
 */



/*
**	DiskStat() - get fs info to check free space; 
**		if free space is low && enough time
**		has passed, build/update a sorted array of
**		VV's for freeing.
**		
*/
static int	mnt_ck = 1;

 void
DiskStat()
{
	char		*id = "DiskStat";
	double		pct_free;

	if (mnt_ck) {
		if (nCheckMount(vvVVDir))
			nLogMsg(EL_FATAL, id, "can't mount cache");
		mnt_ck = 0;
	}

#if __sgi
	if (statfs(vvVVDir, &fsbuf, sizeof(fsbuf), 0))
#else
	if (statfs(vvVVDir, &fsbuf))
#endif
		nLogMsg(EL_FATAL|EL_SYS|EL_CONSOLE0, id, 
				"statfs cache dir %s", vvVVDir);
	pct_free = 100.0 * (double)fsbuf.f_bfree / (double)fsbuf.f_blocks;
	if (pct_free < 20)
		nLogMsg(EL_STATUS1, id, "%2.1f%% free", pct_free);
	if (100 - pct_free > FS_USE_SPACE_PERCENT) {
		nLogMsg(EL_STATUS1, id, "need to free space");
	}
}


void
DiskInitList()
{	
	char	*id = "DiskInitList";

	DIR		*dirp;
	struct dirent	*dep;
	char		buf[MAXPATHLEN];
	int		nvv = 0, notvv = 0;
	OFF_T		notvvsize = 0;
	VV_t		*vvp;
	VVInfo_t	*vip;

	/*
	**	create index
	*/
	InitOndiskIndex();

	/*
	**	scan dir
	*/
	if ((dirp = opendir(vvVVDir)) == NULL)
		nLogMsg(EL_FATAL|EL_SYS, id, "opendir(%s)", vvVVDir);

	while (dep = readdir(dirp)) {

		/*
		**	get info on file
		*/
		(void)sprintf(buf, "%s/%s", vvVVDir, dep->d_name);
		if (STAT(buf, &sbuf) == -1) {
			nLogMsg(EL_SYS|EL_CONSOLE0, id, "stat %s:", buf);
			continue;
		}

		/*
		**	screen out non-VV's
		*/
		if (!S_ISREG(sbuf.st_mode)) {
			if (strcmp(".", dep->d_name)  &&
			    strcmp("..", dep->d_name) &&
			    strcmp("VSNCHECK", dep->d_name))
				nLogMsg(EL_CONSOLE0, id, 
							"non-regular file: %s", 
							dep->d_name);
			continue;
		}

		/*
		**	look up VV info
		*/
		vvp = Name2VV(dep->d_name, 0);
		if (vvp == NULL) {
			notvv++;
			notvvsize += sbuf.st_size;
			nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
					"not a VV? [%s] size %lld",
					dep->d_name, (long long)sbuf.st_size);
			continue;
		}
		vip = DBLookupVV(vvp);
		if (vip == NULL) {
			notvv++;
			notvvsize += sbuf.st_size;
			nLogMsg(EL_CONSOLE0|EL_STATUS1, id, 
					"not a VV? [%s] size %lld",
					dep->d_name, (long long)sbuf.st_size);
			continue;
		}

		/*
		**	got a VV - add to index
		*/
		nvv++;
		AddOndisk(id, sbuf.st_atime);
	}
}
