
#include "vvmd.h"

static Host_t		*HostArray = NULL;
int			totalHosts = 0;


/*
 *  InitSrv() - initialize vvmd server process tracking
 */
  static void
InitSrv(SrvProc_t *sp)
{
	sp->s_pid = -1;
	sp->s_sd = -1;
}

/*
 *  NewHost() - config a host qualified to run vvm servers/clients
 */
  int
NewHost(char *name)
{
	char		*id = "NewHost";
	struct hostent	*htp;
	int		i;
	Host_t		*hop;

	/*
	 *  it better be an interpretable host name
	 */
	if ((htp = nGetHostInfo(name)) == NULL) {
		nLogMsg(EL_ERROR1|EL_SYS, id, "CAN'T GET INFO ON HOST %s",name);
		return(1);
	}
	if (htp->h_length != HOST_ADDRLEN)
		nLogMsg(EL_FATAL, id, "%s h_length=%d != HOST_ADDRLEN=%d: %s",
				"struct hostent", htp->h_length, HOST_ADDRLEN,
				"change vvm_host.h");

	if (HostArray != NULL) {
		/*
		 *  check for duplicate entry
		 */
		for (i=0; i<totalHosts; i++) {
			char	*cp = (char *)HostArray[i].h_addr0;
			char	**cpp;

			for (cpp=htp->h_addr_list; *cpp; cpp++) {
				if (!memcmp(cp, *cpp, (size_t)HOST_ADDRLEN)) {
					nLogMsg(EL_ERROR1, id, 
						"%s: host already added as %s",
						name, HostArray[i].h_name);
					return(1);
				}
			}
		}
		/*
		 *  prepare space
		 */
		HostArray = (Host_t *) nReAlloc((void *)HostArray,
					(totalHosts+1) * sizeof(Host_t));
	} else
		HostArray = (Host_t *) nAlloc(sizeof(Host_t));

	if (HostArray == NULL)
		nLogMsg(EL_FATAL|EL_SYS, id, "malloc host %d", totalHosts);

	hop = &HostArray[totalHosts];
	hop->h_id = totalHosts;
	(void)strcpy(hop->h_name, name);
	(void)strcpy(hop->h_ipname, htp->h_name);
	(void)memcpy(hop->h_addr0, htp->h_addr_list[0], (size_t)HOST_ADDRLEN);
	InitSrv(&hop->h_mover);
	nLogMsg(EL_STATUS1|EL_NOTIME, 0, "\tadded %s = %s %d.%d.%d.%d",
					hop->h_name, hop->h_ipname,
					hop->h_addr0[0], 
					hop->h_addr0[1], 
					hop->h_addr0[2],
					hop->h_addr0[3]);
	totalHosts++;
	return(0);
}

  Host_t *
LookupHost(struct hostent *htp)
{
	int	i;

	for (i=0; i<totalHosts; i++) {
		unsigned char	*cp = HostArray[i].h_addr0;
		char		**cpp;

		nLogMsg(EL_DEBUG1, 0, 
				"compare %s %d.%d.%d.%d  (%s %d.%d.%d.%d)",
				HostArray[i].h_name, cp[0], cp[1], cp[2], cp[3],
				htp->h_name, 
				(unsigned char)htp->h_addr[0], 
				(unsigned char)htp->h_addr[1], 
				(unsigned char)htp->h_addr[2],
				(unsigned char)htp->h_addr[3]);

		for (cpp=htp->h_addr_list; *cpp; cpp++) {
			if (!memcmp(cp, *cpp, (size_t)HOST_ADDRLEN))
				return(&HostArray[i]);
		}
	}
	return(NULL);
}

/*
 *  HostByName() - look up under name in VVCONF
 */
  Host_t *
HostByName(char *str)
{
	int	i;

	for (i=0; i<totalHosts; i++) {
		if (!strcmp(HostArray[i].h_name, str))
			return(&HostArray[i]);
	}
	return(NULL);
}

  Host_t *
HostIndex(int i)
{
	if (i < 0)
		nLogMsg(EL_FATAL, "HostIndex", "index %d", i);
	if (i >= totalHosts)
		return(NULL);

	return(&HostArray[i]);
}

  void
MLogHosts(int cl_slot)
{
	int	i, notup = 0;
	char	*line, *cp, buf[1000];

	(void)strcpy(buf, "\nno mover on:\n");
	cp = buf + strlen(buf);
	line = cp;
	(void)strcpy(cp, "            ");
	cp += strlen(cp);

	if (SendBuf(cl_slot, "---------- Movers -----------------------\n"))
		return;
	for (i=0; i<totalHosts; i++) {

		if (HostArray[i].h_mover.s_sd != -1) {
			char	buf2[100];
			(void)sprintf(buf2, "    %s: pid %d", 
						HostArray[i].h_name,
						HostArray[i].h_mover.s_pid);
			if (SendBuf(cl_slot, buf2))
				return;
			continue;
		}

		/*
		 *  no mover
		 */
		notup++;
		(void)sprintf(cp, " %s", HostArray[i].h_name);
		cp += strlen(cp);
		if (cp - line > 60) {
			(void)strcpy(cp, "\n");
			cp++;
			line = cp;
			(void)strcpy(cp, "            ");
			cp += strlen(cp);
		}
	}
	if (notup)
		if (SendBuf(cl_slot, buf))
			return;
	if (SendBuf(cl_slot, "----------------------------------------"))
		return;
}
