/*
 *  vvm_mover.c
 */

#include "vvmd.h"

/*
 *  GetVsnMoverSeq() - look up request by mover seq
 */
  VsnReq_t *
GetVsnMoverSeq(vvSeq_t seq)
{
	char		*id = "GetVsnMoverSeq";
	VsnReq_t	*vrp;

	for (vrp=VOLQHEAD(); vrp; vrp=VOLQNEXT(vrp)) {
		if (vrp->vr_mvseq == seq)
			break;
	}
	return(vrp);
}

/*
 *  MoverStart() - start any requests waiting for mover on
 *	host in question
 */
  void
MoverStart(Host_t *hop)
{
	char		*id = "MoverStart";
	int		started = 0;
	VsnReq_t	*vrp;

	for (vrp=VOLQHEAD(); vrp; vrp=VOLQNEXT(vrp)) {

		if (vrp->vr_hostp != hop)
			continue;
		if (vrp->vr_status != VSQ_MOUNTED)
			continue;
		switch (vrp->vr_type) {
		    case VQT_WRITE:
			/* not sure how this would happen */
			CWMoverWrite(vrp);
			break;
		    case VQT_READ:
			CRMoverRead(vrp);
			break;
		    case VQT_RECYCLE:
			CRecycleMounted(vrp);
			break;
		    default:
			nLogMsg(EL_FATAL, id, "unexpected type %d", 
							vrp->vr_type);
		}

		if (vrp->vr_status == VSQ_MOVER) {
			/*
			 *  something got done
			 */
			started++;
			switch (vrp->vr_type) {
			    case VQT_WRITE:
				nLogMsg(EL_STATUS1, id, "%.*s write", 
					VSNLEN, 
					vrp->vr_vsns+vrp->vr_writeslot*VSNLEN);
				break;
			    case VQT_READ:
				nLogMsg(EL_STATUS1, id, "%.*s read", 
					VSNLEN, 
					vrp->vr_vsns+vrp->vr_readslot*VSNLEN);
				break;
			    case VQT_RECYCLE:
				nLogMsg(EL_STATUS1, id, "%.*s recycle", 
					VSNLEN, 
					vrp->vr_vsns+vrp->vr_readslot*VSNLEN);
				break;
			}
		}
	}
	if (started)
		nLogMsg(EL_STATUS1, id, "started %d request%s", 
					started, (started == 1 ? "" : "s"));
}
