
/*
 *  vvm_msg.c - Virtual Volume Manager msg protocol support
 *
 *	Where convenient, stack is used for packet memory
 *	to avoid cost of malloc().
 */

#include "vvmd.h"

extern int	totalHosts;
extern Sock_t	SockArray[];

static vvpHdr_t	*HostInfoPkt = NULL;
static int	HostInfoSize;

static vvSeq_t	seq = -1;

  vvSeq_t
vvGetSeq()
{
	if (seq == -1)
		seq = (vvSeq_t)vvGetID();
	return(seq++);
}

/*
**	Send2Movers() - send pkt to all movers (vvmvd)
*/

  void
Send2Movers(vvpHdr_t *hp, char *id)
{
	int	i;
	Host_t	*hop;

	/*
	 *  send to movers
	 */
	for (i=0; i<totalHosts; i++) {
		hop = HostIndex(i);
		if (hop->h_mover.s_sd != -1) {
			nLogMsg(EL_DEBUG2, id, "to %s", hop->h_name);
			(void)XDRSendI(hp, hop->h_mover.s_sd, 1);
		}
	}
}


/*
**	SendAck() - send a simple ack
*/
  void
SendAck(vvSeq_t seq, int slot)
{
	vvpHdr_t	hdr;

	(void) vvBuildHdr(VVM_ACK, seq, &hdr);
        (void) XDRSendI(&hdr, slot, 1);
}

/*
**	SendNOP() - send simple no-op to test comm
*/
  void
SendNOP(int slot)
{
	vvpHdr_t	hdr;

	(void) vvBuildHdr(VVM_NOP, 0, &hdr);
	(void) XDRSendI(&hdr, slot, 1);
}


  void
SendErr(vvSeq_t seq, int slot)
{
	vvpHdr_t	*hp;
	vvpError_t	*ep;
	char		ebuf[VV_PKTSIZ(vvpError_t)];

	hp = vvBuildHdr(VVM_VVERROR, seq, (vvpHdr_t *)ebuf);
	ep = VV_MSGPTR(hp, vvpError_t);
	ep->vn_error = VVError;
	emsgcpy(ep->vn_msg, nErrStr);
	(void)XDRSendI(hp, slot, 1);
}

/*
 *  SendBuf() - buffered send of char arrays (for printing) -
 *	a synchronous msg from caller is req'd to flush the
 *	XDR buffer (most obviously, a VVM_ACK).
 */
#define MAXMSG	500

  int
SendBuf(int slot, char *msgbuf)
{
	char		buf[MAXMSG];
	vvpHdr_t	*hp;
	char		*cp;
	size_t		len = strlen(msgbuf) + 1;

	if (len + sizeof(vvpHdr_t) > MAXMSG) {
		nLogMsg(EL_ERROR1, 0, "MAXMSG %d, need %d", 
					MAXMSG, len + sizeof(vvpHdr_t));
		(void)kill(0,5);
	}
	hp = (vvpHdr_t *) buf;
	hp->vh_type = VVM_BYTES;
	hp->vh_magic = VVP_MAGIC;
	hp->vh_seq = 0;
	hp->vh_size = len;
	cp = VV_MSGPTR(hp, char);

	memcpy(cp, msgbuf, len);
	return(XDRSendI(hp, slot, 0));
}
