
/*
 *  Virtual Volume Manager
 *
 *  vvm_sock.c - vvmd connection mgt
 */

#include "vvmd.h"

extern int 		errno;

extern int		volman_sock;

extern int		timerpid;
extern char		shutdownmsg[];

Sock_t			SockArray[FD_SETSIZE];

static int		timerfd;
static fd_set		fds;
static int		Master_sd;		/* for receiving conns */

static int		socksize = sizeof( struct sockaddr_in );

  static char *
PortDesc(int i)
{
	static char		portdesc[100];

	if (SockArray[i].sock_ptype == S_MOVER) {
		Host_t	*hop = (Host_t *)SockArray[i].sock_host;

		(void)sprintf(portdesc, "vvmvd pid %d on %s", 
					SockArray[i].sock_id, hop->h_name);
		return(portdesc);
	}
	return(vvPortTypeStr(SockArray[i].sock_ptype));
}

/*
 *  IPCStartListening() - Open a 'well-known' socket to accept connections
 */
  static int
IPCStartListening(u_long port)
{
	char			*id = "IPCStartListening";
	struct sockaddr_in 	ip_addr;
	int 			listen_sd, i, nsoc, len, on;
	Sock_t			*csp;

	/*
	 *  create master socket
	 */
	listen_sd = socket(AF_INET, SOCK_STREAM, 0);
	if (listen_sd == -1)
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "socket() failed");
	nLogMsg(EL_DEBUG1, id, "listen_sd %d", listen_sd);

	/*
	 *  set socket to recycle quickly .. SO_LINGER too?
	 */
	on = 1;
	if (setsockopt(listen_sd, SOL_SOCKET, SO_REUSEADDR, 
					(char *)&on, sizeof(int)) == -1) {
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "setsockopt() failed");
	}

	memset(&ip_addr, 0, sizeof(ip_addr));	/* for Purify */

	ip_addr.sin_addr.s_addr = htonl( INADDR_ANY );
	ip_addr.sin_family = AF_INET;
	ip_addr.sin_port = htons(port);

	for (i=1; i<16; i++) {
	    if ((bind(listen_sd, (struct sockaddr *)&ip_addr, socksize)) ==-1) {
		if (errno == EADDRINUSE) {
		    nLogMsg(EL_TOSTDERR|EL_STATUS2, id,
			"Main port address already in use - retrying");
		    (void)sleep(5*i);
		    continue;
		}
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "bind() failed");
	    }
	    break;
	}
	if (i == 16)
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, 
					"giving up - wait & retry later");
	if (i != 1)
		nLogMsg(EL_TOSTDERR|EL_STATUS2, id, "Main port address OK");

	if (listen(listen_sd, 5) == -1)
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "listen() failed");
	return(listen_sd);
}

  static void
IPCClose(fd)
{
	char	*id = "IPCClose";

	if (shutdown(fd, 2) == -1) 
		nLogMsg(EL_SYS|EL_CONSOLE0, id, "socket shutdown");
	if (close(fd) == -1) {
		nLogMsg(EL_SYS|EL_CONSOLE0, id, "socket close");
	}
}

  void
IPCInit()
{
	Sock_t		*sp;
	int		i;

	/*
	 *  initialize sock structs
	 */
	for (sp=SockArray,i=0; i<FD_SETSIZE; i++, sp++) {
		sp->sock_fd = -1;
        	sp->sock_id = -1;
		sp->sock_ptype = S_NOSOCK;
		sp->sock_proc = NULL;
		sp->sock_host = NULL;
	}
}

  void
IPCStartup()
{
	static	char	id[] = "IPCStartup";
	struct	servent	*servent;

	if ((servent = getservbyname(vvm_service, "tcp")) == NULL)
		nLogMsg(EL_FATAL|EL_SYS|EL_TOSTDERR, id, "getservbyname");
	vvm_port = ntohs(servent->s_port);

	/*
	 *  tie XDR structs to sock structs
	 *	TODO - only on connect to save mem
	 */
	XDRInit();
	Master_sd = IPCStartListening(vvm_port);
}

  void
IPCShutdown()
{
	shutdown(Master_sd, 2);
	close(Master_sd);
}

 void
IPCAcceptConn()
{
	char			*id = "AcceptConn";
	int			new_soc;
	struct sockaddr_in	new_conn;
	int			new_len;
	struct hostent		*htp;
	Host_t			*hop;
	Sock_t			*sp;

	/*
	**	Create new socket with accept()
	**	Note that resulting fd indexes Connections array
	*/
	socksize = sizeof(struct sockaddr_in);
	new_soc = accept(Master_sd, (struct sockaddr *)&new_conn, 
							&socksize);
	if (new_soc == -1) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1|EL_SYS, id, "accept failed");
		return;
	}
	nLogMsg(EL_DEBUG2, id, "new socket fd %d\n", new_soc);

	/*
	**	See if authorized host
	*/
	htp = gethostbyaddr((char *)&new_conn.sin_addr, 
				sizeof(new_conn.sin_addr), AF_INET);
	if (htp == NULL) {
		nLogMsg(EL_CONSOLE0|EL_SYS|EL_ERROR1, id, 
			"attempted connection from untranslatable host");
		IPCClose(new_soc);
		return;
	}
	hop = LookupHost(htp);
	if (hop == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
			"attempted connection from unauthorized host (%s)",
								htp->h_name);
		IPCClose(new_soc);
		return;
	}
	nLogMsg(EL_STATUS2, id, "accepted slot %d from %s",
						new_soc, htp->h_name);

	/*
	**	note the connection & wait for identifying 1st msg
	*/
	sp = &SockArray[new_soc];
	sp->sock_host = (void *) hop;
	sp->sock_addr = new_conn.sin_addr;
	sp->sock_ptype = S_CONNECTED;
	sp->sock_flags = 0;
	sp->sock_fd = new_soc;
	sp->sock_id = -2;
}

/*
 *  FUNCTION:	IPCServer()
 *
 *		Main select loop for new connections and input from clients.
 */
  void
IPCServer()
{
	char		*id = "IPCServer";
	int		i, nsoc, socseen, procQ;
	STAT_T		st;
	vvpHdr_t	*hp;

	while(1) {

		/*
		**	try to start volman connection if down
		*/
		if (volman_sock == -1) {
			if (VolmanInit())
				CWVolmanRestart();
		}

		/*
		**	set up array for select() & do it
		*/
		FD_ZERO(&fds);
		FD_SET(Master_sd, &fds);
		if (volman_sock != -1)
			FD_SET(volman_sock, &fds);

		for (i=0; i < FD_SETSIZE; i++) {
			if (SockArray[i].sock_ptype != S_NOSOCK) {
				FD_SET(i, &fds);
			}
		}

		nLogMsg(EL_DEBUG4, id, "WAIT on select/rcv");
		while ((nsoc = select(FD_SETSIZE, &fds, (fd_set *) NULL, 
					(fd_set *) NULL, NULL)) == -1 
					&& errno == EINTR);
		if (nsoc == 0) {
			nLogMsg(EL_DEBUG2, id, "0 read.. -?");
			continue;
		}
		if (nsoc < 0)
			nLogMsg(EL_SYS|EL_FATAL, id, "select failed");

		(void)time(&Now);
		socseen = 0;

		/*
		**	handle connect request if any
		*/
		if (FD_ISSET(Master_sd, &fds)) {
			socseen++;
			FD_CLR(Master_sd, &fds);
			IPCAcceptConn();
		}

		/*
		**	handle volman msgs if any
		*/
		if (volman_sock != -1) {
			if (FD_ISSET(volman_sock, &fds)) {
				socseen++;
				FD_CLR(volman_sock, &fds);
				VolmanRecv();
			}
		}

 		/*
		**	handle others (clients & movers)
		*/
		procQ = 0;
		for (i=0; i<FD_SETSIZE && socseen<nsoc; i++) {
			if (!FD_ISSET(i, &fds)) 
				continue;

			socseen++;

			if (SockArray[i].sock_ptype == S_NOSOCK)
				nLogMsg(EL_STATUS1, id, 
					"msg from former conn slot %d", i);

			if (i != timerfd)
				nLogMsg(EL_DEBUG3, id, "rcvd fd %d", i);

			/*
			**	receive all (possibly buffered) packets
			*/
			do {
			    hp = XDRRecvI(i);
			    if (hp == NULL) {
				nLogMsg(EL_STATUS1, id, 
				    "null msg from slot %d %s - %s", i,
				    PortDesc(i), "shutting down connection");
				procQ += ShutdownServer(i, 1);
				break;
			    }
if (hp->vh_type != VVM_TIMER)
nLogMsg(EL_STATUS1, id, "got %s", vvHdrStr(hp));

			    /*
			    **	use the information
			    */
			    procQ += IPCProcPkt(hp, i);

			    nFree(hp);
			} while ( XDRNextRecordI(i));
			nLogMsg(EL_DEBUG2, id, "no more from %d", i);
		}

/*
		if (procQ)
			ProcReqQ();
*/
	}
}


 void
CloseSock(int slot, char *id)
{
	Sock_t		*sp = &SockArray[slot];
	SrvProc_t	*srvp = (SrvProc_t *)sp->sock_proc;

nLogMsg(EL_STATUS1, "CloseSock", "%s sock %d", id, slot);
	if (srvp != NULL) {
nLogMsg(EL_STATUS1, "CloseSock", "marking server off too");
		srvp->s_pid = -1;
		srvp->s_sd = -1;
	}

	if (sp->sock_fd == -1)
		return;

	XDRCloseSock(sp);

	/*
	**	flush any xdr buffer stuff
	*/
	XDRFlushI(slot);
}

 void
ShutdownServers()
{
	int	i;

	for (i=0; i<FD_SETSIZE; i++)
		if (SockArray[i].sock_ptype != S_NOSOCK)
			(void)ShutdownServer(i, 0);
}

  int
ShutdownServer(int slot, int recover)
{
	char	*id = "ShutdownServer";
	Sock_t	*sp = &SockArray[slot];
	Host_t	*hop = (Host_t *) sp->sock_host;
	int	procQ = 0;

	switch (sp->sock_ptype) {
	    case S_MOVER:
		if (recover)
			procQ = CleanupMover(slot);
		hop->h_mover.s_sd = -1;
		break;
	    case S_CLIENT:
		/* not a server but.. */
		procQ = CleanupClient(slot);
		break;
	    case S_TIMER:
		/* assume timer handled by catching child death */
	    case S_CONNECTED:
		/* type not assigned yet */
	    case S_NOSOCK:
		/* nothing to disconnect */
		break;
	    default:
		nLogMsg(EL_FATAL, id, "unexpected sock_ptype %d", 
						sp->sock_ptype);
	}
	CloseSock(slot, id);
	sp->sock_ptype = S_NOSOCK;

	return(procQ);
}

/*
 *  IPCNewProcSwitch() - check/setup new connection. 
 *
 *	Since the vvmd port is in the root-only range (<5000) and
 *	only trusted-host connections are accepted per the vvmd
 *	config file, the request is taken at face value.
 */
 static void
IPCNewProcSwitch(vvpHdr_t *hp, int slot, Sock_t *sp, Host_t *hop)
{
	char		*id = "IPCNewProcSwitch";
	vvpHello_t	*hep = VV_MSGPTR(hp, vvpHello_t);
	int		i;

	/*
	**	set pkt for response
	*/
	hep->vhe_id = vvGetID();		/* rcvkey for future msgs */

	/*
	**	simple protocol check on this, the 1st pkt from process
	*/
	if (hep->vhe_version != VV_VERSION) {
		VVError = VVE_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
			"%s version %d wrong on %s - refusing (version=%d)",
			vvHdrStr(hp), hep->vhe_version, hop->h_name, 
			VV_VERSION);
		SendErr(hp->vh_seq, slot);
		CloseSock(slot, id);
		return;
	}

	/*
	**	set up the appropriate type of server
	*/
	switch (hp->vh_type) {
	    case VVM_CLIENT:
		sp->sock_id = hep->vhe_pid;
		sp->sock_rcvkey = hep->vhe_id;	/* set above */
		sp->sock_sndkey = hp->vh_key;
nLogMsg(EL_STATUS1, id, "VVM_CLIENT %d  rcv 0x%x snd 0x%x", slot,
sp->sock_rcvkey, sp->sock_sndkey);
		if (XDRSendI(hp, slot, 1))
			return;

nLogMsg(EL_STATUS1, id, "VVM_CLIENT ok");
		sp->sock_ptype = S_CLIENT;
		sp->sock_proc = NULL;
		return;

	    case VVM_MOVER:
		if (hop->h_mover.s_sd != -1) {
			nLogMsg(EL_STATUS1, id, "sending no-op to test mover");
			SendNOP(hop->h_mover.s_sd);
		}
		if (hop->h_mover.s_sd != -1) {
			VVError = VVE_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"duplicate mover (slot %d) on %s - refusing",
				hop->h_mover.s_sd, hop->h_name);
			SendErr(hp->vh_seq, slot);
			CloseSock(slot, id);
			return;
		}
		sp->sock_id = hep->vhe_pid;
		sp->sock_rcvkey = hep->vhe_id;	/* set above */
		sp->sock_sndkey = hp->vh_key;

nLogMsg(EL_STATUS1, id, "VVM_MOVER %d  rcv 0x%x snd 0x%x", slot,
sp->sock_rcvkey, sp->sock_sndkey);
		if (XDRSendI(hp, slot, 1))
			return;

		sp->sock_ptype = S_MOVER;
		sp->sock_proc = (void *) &hop->h_mover;
		hop->h_mover.s_sd = slot;
		hop->h_mover.s_pid = hep->vhe_pid;
		nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
				"==== registered mover pid %d on %s (slot %d)",
				hop->h_mover.s_pid, hop->h_name, slot);
		/*
		**	kick off any requests waiting for this mover
		*/
		MoverStart(hop);
		return;

	    case VVM_TIMERD:		/* timer			*/
		if (hep->vhe_pid != timerpid) {
			VVError = VVE_INTERNAL;
			nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
					"vvtimer pid %d != %d - refusing",
					hep->vhe_pid, timerpid);
			SendErr(hp->vh_seq, slot);
			CloseSock(slot, id);
			return;
		}
		sp->sock_id = hep->vhe_pid;
		sp->sock_rcvkey = hep->vhe_id;	/* set above */
		sp->sock_sndkey = hp->vh_key;

		if (XDRSendI(hp, slot, 1))
			return;

		sp->sock_ptype = S_TIMER;
		sp->sock_proc = NULL;
		nLogMsg(EL_CONSOLE0|EL_STATUS1, 0, 
					"==== registered timer pid %d on %s",
					hep->vhe_pid, hop->h_name);
		return;
	    default:
		VVError = VVE_INTERNAL;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_ERROR1, id, 
				"unexpected msg type from new conn on %s: %s",
				hop->h_name, vvHdrStr(hp));
		SendErr(hp->vh_seq, slot);
		CloseSock(slot, id);
		return;
	}
}
 int
IPCProcPkt(vvpHdr_t *hp, int slot)
{
	char	*id = "IPCProcPkt";
	Sock_t	*sp = &SockArray[slot];
	Host_t	*hop = (Host_t *) sp->sock_host;

	/*
	 *  special case: handle 1st msg after connection
	 */
	if (sp->sock_ptype == S_CONNECTED) {
		IPCNewProcSwitch(hp, slot, sp, hop);
		return(0);
	}

	switch (sp->sock_ptype) {
	    case S_TIMER:
		MTimer(hp);
		return(0);

	    case S_CLIENT:
		return(ClientMsgSwitch(hp, slot));

	    case S_MOVER:
		return(MoverMsgSwitch(hp, slot));

	    case S_NOSOCK:
		nLogMsg(EL_STATUS2, id, "slot %d exited or closed connection",
							slot);
		return(0);

	    default:
		nLogMsg(EL_FATAL, id, "unexpected sock_ptype %d", 
						sp->sock_ptype);
	}
	return(0);	/* for compiler warnings */
}



/*
 *  HostMover() - return fd slot of mover on given host
 */
  int
HostMover(Host_t *hop)
{
	char	*id = "HostMover";
	int	i;

	if (hop == NULL)
		nLogMsg(EL_FATAL, id, "host null");

	for (i=0; i<FD_SETSIZE; i++) {
		if (SockArray[i].sock_ptype == S_MOVER) {
			if (SockArray[i].sock_host == (void *)hop)
				return(i);
		}
	}
	VVError = VVE_UNAVAIL;
	nLogMsg(EL_CODE|EL_CONSOLE0|EL_ERROR1, id, 
				"No vvmvd on host %s", hop->h_name);
	return(-1);
}
