
/*
 *  vvm_volman.c - Virtual Volume Manager tape alloc/mount/dismount
 *
 *	Note that in this file, where volman & vvm things
 *	are mixed, the volman naming conventions
 *	like vmpHdr_t may become confused with the vvm ones
 *	like vvpHdr_t which were derived from volman's..
 */

#include "vvmd.h"

#define READ_IDLE_WAIT	180	/* timeout cacheing of finished read-mount */

#define MAX_MOUNT_RETRIES	10	/* for working around downed srvrs */
#define MOUNT_RETRY_WAIT	30

int			volman_sock = -1;

extern int		read_idle;
extern int		hot_mounted;

static int		mount_retry = 0;

static vmpHdr_t		*hostpkt = NULL;

CodeString_t vsnReqTypes[] = {
	VQT_WRITE,	"save",
	VQT_READ,	"restore",
	VQT_ALLOC,	"alloc",
	VQT_RECYCLE,	"recycle",
	0,		0,
};

  static char *
vvMountType(int code)
{
	return(nCodeToStr(code, vsnReqTypes));
}

int		volman_failed = 0;
static char	savederrmsg[LINELEN*2];

  int
VolmanInit()
{
	char		*id = "VolmanInit";

	if (volman_sock != -1) {
		nLogMsg(EL_FATAL, id, "already initted");
		return(0);
	}

	/*
	 *  make connection
	 */
	vInitDirNames();
	(void)ClientInitIPC(0, EL_CODEONLY|EL_NOTIME);
	if (VMError != E_NOERROR) {
		char	savederrmsg2[LINELEN*2];
		/*
		 *  consider logging; will try later
		 */
		if (nErrStr == NULL) {
			nLogMsg(EL_STATUS1, id, "(nErrStr is null)");
			savederrmsg2[0] = '\0';
		} else
			(void)strcpy(savederrmsg2, nErrStr);
		volman_failed++;
		if (volman_failed == 1) {
			nLogMsg(EL_STATUS1, id, "%s", savederrmsg2);
		} else {
			if (strcmp(savederrmsg, savederrmsg2)) {
				nLogMsg(EL_STATUS1, id, "new err: %s", 
							savederrmsg2);
				volman_failed = 1;
			} else if (volman_failed % 50 == 0)
				nLogMsg(EL_STATUS1, id, "%d times: %s", 
						volman_failed, savederrmsg2);
		}
		(void)strcpy(savederrmsg, savederrmsg2);
		return(0);
	}
	volman_failed = 0;

	/*
	 *  get volman host table
	 */
	if (vm_hostinfo()) {
		(void)vm_term();
		return(0);
	}
	/*
	 *  get RC info for looking up class slot RCs
	 */
	if (vm_configRC()) {
		(void)vm_term();
		return(0);
	}

	volman_sock = ClientIPCRcvfd();
	nLogMsg(EL_STATUS1, id, "connected to vold");
	return(1);
}

/************************************************
 * local for volman request/response
 */

  static void
VolmanDown()
{
	char		*id = "VolmanDown";
	VsnReq_t	*vrp, *vrp2;

	nLogMsg(EL_STATUS1, id, "------ vold down - cleaning up");

	mount_retry = 0;
	hot_mounted = 0;
nLogMsg(EL_STATUS1, id, "hot_mounted -> 0");
	for (vrp=VOLQHEAD(); vrp; vrp=vrp2) {
		VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;

		vrp2 = VOLQNEXT(vrp);	/* since vrp may be DQ'd */

		/*
		**	handle cases where answer from volman is expected
		*/
		if (vrp->vr_status == VSQ_VOLALLOC) {
			/*
			**	deletes request, so vr_retries
			**	doesn't need to be reset
			*/
			CWAllocFailed(vrp, "volman down", 0);
			continue;
		} 
		if (vrp->vr_status == VSQ_VOLMOUNT ||
			   vrp->vr_status == VSQ_RETRYMOUNT) {
			/*
			**	reset retry counter since new volman will
			**	be a clean slate
			*/
			vrp->vr_retries = 0;

			switch (vrp->vr_type) {
			    case VQT_WRITE:
				nLogMsg(EL_STATUS2, 0, "(mount/write %.*s)",
						cip->vc_class.cs_nvsns *VSNLEN,
						vrp->vr_vsns);
				CWMountFailed(vrp, 0);
				break;
			    case VQT_READ:
				nLogMsg(EL_STATUS2, 0, "(mount/read %.*s)",
						cip->vc_class.cs_nvsns *VSNLEN,
						vrp->vr_vsns);
				CRVolmanDown(vrp);
				break;
			    case VQT_RECYCLE:
				nLogMsg(EL_STATUS2, 0, "(mount/recycle %.*s)",
						VSNLEN, 
						vrp->vr_vsns + 
						vrp->vr_readslot*VSNLEN);
				CRecycleVolmanDown(vrp);
				break;
			    default:
				nLogMsg(EL_ERROR1|EL_FATAL, id, 
						"bad type %d", vrp->vr_type);
			}
		}
	}
	nLogMsg(EL_STATUS1, id, "------ cleanup done ");
}


/*
 *  NewVsnReq() - make a relatively 'blank slate' request that only
 *	points back to its originator
 */
  static VsnReq_t *
NewVsnReq(void *caller, int type, VVMClass_t *cip)
{
	VsnReq_t	*vrp = (VsnReq_t *)nAlloc(sizeof(VsnReq_t));

	vrp->vr_writeslot = -1;
	vrp->vr_readslot = -1;
	vrp->vr_nvsns = 0;
	memset(vrp->vr_vsns, ' ', sizeof(vrp->vr_vsns));
	memset(vrp->vr_cancel, 0, sizeof(vrp->vr_cancel));
	vrp->vr_status = VSQ_NONE;
	vrp->vr_bufsize = -1;
	vrp->vr_volseq = NOSEQ;
	vrp->vr_mvseq = VVNOSEQ;
	vrp->vr_retries = 0;
	vrp->vr_hostp = NULL;

	vrp->vr_caller = caller;
	vrp->vr_type = type;

	vrp->vr_waitq.q_next = NULL;
	vrp->vr_waitq.q_prev = NULL;

	vrp->vr_cp = (void *)cip;

	VOLENQ(vrp);

	return(vrp);
}

/*
**	GetVsnVolmanSeq() - look up request by volman client seq
*/
  static VsnReq_t *
GetVsnVolmanSeq(Seq_t seq)
{
	char		*id = "GetVsnVolmanSeq";
	VsnReq_t	*vrp;

	for (vrp=VOLQHEAD(); vrp; vrp=VOLQNEXT(vrp)) {
		if (vrp->vr_volseq == seq)
			break;
	}
	return(vrp);
}

/*
**	UseMountedVsn() - want to read; vsn could be mount for read or write
*/
  VsnReq_t *
UseMountedVsn(int nvsns, char *vsnp, void *caller, VVMClass_t *cip)
{
	char		*id = "UseMountedVsn";
	VsnReq_t	*vrp, *vrp2, *lastvrp, *bestvrp = NULL;
	int		i, bestslot, qct;

nLogMsg(EL_STATUS1, id, "select from [%.*s]", nvsns*VSNLEN, vsnp);

	/*
	**	check for idle or active mounted-for-read
	*/
	for (vrp=VOLQHEAD(); vrp; vrp=VOLQNEXT(vrp)) {
		char		*vsnp2;

		if (vrp->vr_type != VQT_READ)
			continue;
		if (vrp->vr_status != VSQ_MOVER  &&  vrp->vr_status != VSQ_IDLE)
			continue;

		vsnp2 = vrp->vr_vsns + (vrp->vr_readslot * VSNLEN);
		if (memcmp(vsnp, vsnp2, (size_t)VSNLEN))
			continue;

		/*
		**	an idle vsn is ideal - 
		**	can use the vrp itself, too
		*/
		if (vrp->vr_status == VSQ_IDLE) {
			if (vrp->vr_caller != NULL)
				nLogMsg(EL_FATAL, id, "caller exists");
			vrp->vr_caller = caller;
			read_idle--;
			return(vrp);
		}

		/*
		**	accept the first match for now
		*/
		if (bestvrp == NULL)
			bestvrp = vrp;
	}

	if (bestvrp != NULL) {
		/*
		**	found an active vrp to q on - make a vrp/VSQ_WAIT
		*/
		vrp2 = NewVsnReq(caller, VQT_READ, cip);
		vrp2->vr_readslot = bestvrp->vr_readslot;
		vrp2->vr_nvsns = nvsns;
		memcpy(vrp2->vr_vsns, vsnp, (size_t)(nvsns * VSNLEN));
		vrp2->vr_status = VSQ_WAIT;

		/*
		**	append vrp to active one's read q
		*/
		qct = 1;
		for (lastvrp=bestvrp; lastvrp->vr_waitq.q_next; 
			lastvrp=(VsnReq_t *)lastvrp->vr_waitq.q_next,qct++);
		lastvrp->vr_waitq.q_next = (void *)vrp2;

nLogMsg(EL_STATUS1, id, "assigned wait on %s ",
vvVsnStatus(lastvrp->vr_status));

		vrp2->vr_waitq.q_prev = (void *)lastvrp;
		vrp2->vr_status = VSQ_WAIT;
		vrp2->vr_hostp = lastvrp->vr_hostp;
		vrp2->vr_bufsize = lastvrp->vr_bufsize;
nLogMsg(EL_STATUS1, id, "ok.. readq %d", qct);
		return(vrp2);
	}

	/*
	**	check the mounted-for-writes
	*/
	for (vrp=VOLQHEAD(); vrp; vrp=VOLQNEXT(vrp)) {
		char	*vsnp2;

		if (vrp->vr_type != VQT_WRITE)
			continue;
		if (vrp->vr_status == VSQ_NONE)
			continue;

		vsnp2 = vrp->vr_vsns + (vrp->vr_writeslot * VSNLEN);

		/*
		**	check all slots in read req, in case 
		**	classes have been redefined
		*/
		for (i=0; i<nvsns; i++) 
			if (!memcmp(vsnp + (i*VSNLEN), vsnp2, (size_t)VSNLEN))
				break;
		if (i == nvsns)
			continue;

		/*
		**	write-mount vsn matches one of the 
		**	sought-for read ones
		*/

nLogMsg(EL_STATUS1, id, "match %d/%s status %s", i,
vsnstr(vsnp2), vvVsnStatus(vrp->vr_status));
		if (vrp->vr_status == VSQ_IDLE) {
nLogMsg(EL_STATUS1, id, "perfect");
			/*
			**	vsn ready to go; search no further
			*/
			bestvrp = vrp;
			bestslot = i;
			break;
		}

		/*
		**	it's in some sort of unready/busy state
		*/
		if (bestvrp == NULL) {
			/*
			**	1st found
			*/
nLogMsg(EL_STATUS1, id, "1st");
			bestvrp = vrp;
			bestslot = i;
			continue;
		}
		/*
		**	got a prev unready/busy candidate -
		**	compare the 2 requests..
		*/
		if (bestvrp->vr_status == vrp->vr_status) {
			/*
			**	other things being equal,
			**	prefer the lesser-numbered slot
			*/
			if (i < bestslot) {
nLogMsg(EL_STATUS1, id, "better slot");
				bestvrp = vrp;
				bestslot = i;
			}
			continue;
		}

		/*
		**	waiting on a mover req is the best of 2nd best
		**	mover _writing_ for this req: VSQ_MOVER
		**	mover reading for another req on this vsn: VSQ_WAIT
		*/
		if (bestvrp->vr_status == VSQ_MOVER  ||
		    bestvrp->vr_status == VSQ_WAIT)
			continue;

		/*
		**	a simple rule for comparing.. maybe not the best
		*/
		if (bestvrp->vr_status < vrp->vr_status) {
nLogMsg(EL_STATUS1, id, "status greater");
			bestvrp = vrp;
			bestslot = i;
		}
	}
	if (bestvrp == NULL)
		return(NULL);

	/*
	**	got one; find end of its read q
	*/
	qct = 0;
	for (lastvrp=bestvrp; lastvrp->vr_waitq.q_next; 
			lastvrp=(VsnReq_t *)lastvrp->vr_waitq.q_next,qct++);

nLogMsg(EL_STATUS1, id, "bestvrp/write %.*s %s ct %d", 
VSNLEN, vsnp+VSNLEN*bestslot, vvVsnStatus(bestvrp->vr_status), qct);

	if (bestvrp->vr_waitq.q_next != NULL  &&  
	    bestvrp->vr_status == VSQ_IDLE)
		nLogMsg(EL_FATAL, id, "idle req w/ one waiting");

	/*
	**	got a hot mount vsn matching 
	**	one of the ones in this one:
	**	make a request & enqueue
	*/
	vrp = NewVsnReq(caller, VQT_READ, cip);
	vrp->vr_readslot = bestslot;
	vrp->vr_nvsns = nvsns;
	memcpy(vrp->vr_vsns, vsnp, (size_t)(nvsns * VSNLEN));

	lastvrp->vr_waitq.q_next = (void *)vrp;
nLogMsg(EL_STATUS1, id, "assigned wait on %s ",
vvVsnStatus(lastvrp->vr_status));
	vrp->vr_waitq.q_prev = (void *)lastvrp;

	vrp->vr_hostp = bestvrp->vr_hostp;
	vrp->vr_bufsize = bestvrp->vr_bufsize;

	switch (lastvrp->vr_status) {
	    case VSQ_MOUNTED:
		/*
		**	just waiting to be used: queue up
		*/
nLogMsg(EL_STATUS1, id, "got idle %.*s", VSNLEN, vsnp+VSNLEN*bestslot);

		/*
		**	bestvrp==lastvrp: tell write to wait on read
		*/
		bestvrp->vr_status = VSQ_WAIT;
		vrp->vr_status = VSQ_IDLE;
		break;

	    case VSQ_VOLMOUNT:
	    case VSQ_MOVER:
	    case VSQ_WAIT:
		/*
		**	not ready or in use - this request will wait
		*/
		vrp->vr_status = VSQ_WAIT;

		break;
	    default:
		nLogMsg(EL_FATAL, id, "%s: unexpected status %s",
					vsnstr(vsnp+VSNLEN*bestslot),
					vvVsnStatus(bestvrp->vr_status));
	}
nLogMsg(EL_STATUS1, id, "ok..");
	return(vrp);
}

  VsnReq_t *
VolmanMountHotVsn(int nvsns, char *vsnp, void *caller, VVMClass_t *cip)
{
	char		*id = "VolmanMountHotVsn";
	VsnReq_t	*hot_vrp, *vrp;

	hot_vrp = CWMountHotRead(nvsns, vsnp);
	if (hot_vrp == NULL)
		return(NULL);

	/*
	**	 a hot mount has been started; make req & queue
	*/
	vrp = NewVsnReq(caller, VQT_READ, cip);
	vrp->vr_status = VSQ_WAIT;
	vrp->vr_readslot = hot_vrp->vr_writeslot;
	vrp->vr_nvsns = nvsns;
	memcpy(vrp->vr_vsns, vsnp, (size_t)(nvsns * VSNLEN));

	hot_vrp->vr_waitq.q_next = (void *)vrp;
	vrp->vr_waitq.q_prev = (void *)hot_vrp;

	return(vrp);
}

/*
 *  SendMount() - make & send request. For generality, a
 *	slotwise array of vsns is passed which may only have 
 *	1 vsn in it, or more if 'select-one' mount.
 *
 *	return vsns or -1
 */
  static int
SendMount(int nvsns, char *vsnp, int type, Seq_t seq)
{
	char		*id = "SendMount";
	char		*cp1, *cp2, vbuf[MAXVSN * VSNLEN];
	char		pbuf[PKTSIZ2(vmpUsrMount_t, vmpUsrMountN_t) 
						+ MAXVSN * VSNLEN];
	char		tag[TAGLEN];
	vmpHdr_t	*hp;
	int		i, ct, flags;
	char		ch = '?';

	if (volman_sock == -1)
		return(-1);

	memset(pbuf, 0, sizeof(pbuf));	/* for purify */

	nLogMsg(EL_STATUS1, id, "n=%d, [%.*s]", nvsns, nvsns*VSNLEN, vsnp);
	/*
	 *  pack array of vsns for pkt
	 */
	ct = 0;
	cp2 = vbuf;
	cp1 = vsnp;
	for (i=0; i<nvsns; i++, cp1+=VSNLEN) {
#ifdef HACK_NOVSN
		if (*cp1 != ' '  &&  *cp1 != '%') {
#else
		if (*cp1 != ' ') {
#endif
			memcpy(cp2, cp1, (size_t)VSNLEN);
			cp2 += VSNLEN;
			ct++;
		}
	}

	if (!ct)
		nLogMsg(EL_FATAL, id, "no vsns");

	/*
	 *  build pkt 
	 */

	switch (type) {
	    case VQT_READ:	ch = 'r'; break;
	    case VQT_WRITE:	ch = 'w'; break;
	    case VQT_RECYCLE:	ch = 'R'; break;
	}
	(void)sprintf(tag, "VVM.%c", ch);
	flags = VO_WAIT;
	if (type == VQT_WRITE)
		flags |= VO_WRITE;
	else
		flags |= VO_IGNORE_RING;

	if (ct == 1) {
		vmpUsrMount_t	*mntp;
		/*
		 *  use regular mount to enable per-vsn error to come back
		 *	note: packet protocol is volman
		 */
		hp = vBuildHdr(M_USRMOUNT, seq, (vmpHdr_t *)pbuf);
		mntp = MSGPTR(hp, vmpUsrMount_t);
		memcpy(&mntp->vu_vsn, vbuf, sizeof(mntp->vu_vsn));
nLogMsg(EL_STATUS1, id, "[%.6s]", &mntp->vu_vsn);
		mntp->vu_flags = flags;
		memset(mntp->vu_rid, '\0', sizeof(mntp->vu_rid));
		copytag(mntp->vu_tag, tag);

	} else {
		vmpUsrMountN_t	*mntp;
		/*
		 *  use M_USRMOUNT_SEL to get 1 vsn of a group
		 *	note: packet protocol is volman
		 */
		hp = (vmpHdr_t *) pbuf;
        	hp->vh_magic = VMP_MAGIC;
		hp->vh_seq = seq;
        	hp->vh_type = M_USRMOUNT_SEL;
		hp->vh_size = sizeof(vmpUsrMountN_t) + ct * VSNLEN; 

		hp->vh_count = ct;

		mntp = MSGPTR(hp, vmpUsrMountN_t);
		mntp->vx_flags = flags;
		copytag(mntp->vx_tag, tag);

		cp1 = (char *)(mntp+1);
		memcpy(cp1, vbuf, (size_t)(ct * VSNLEN));
	}

	/*
	 *  send pkt
	 */
	if (lSnd2RP(hp))
		return(-1);

	return(ct);
}

/***********************************************
 *  volman request routines - all async
 */

/*
 *  VolmanAlloc() - get a scratch vsn from volman. Since volman may
 *	be sending responses to other requests (mount, unmount), this
 *	routine can't use the synchronous vm_alloc() & instead just
 *	uses the msg sending part of that code from volman's cl_lib.c,
 *	creating a VsnReq_t to wait on the response.
 */
  VsnReq_t *
VolmanAlloc(VVMClass_t *cip, int writeslot, HotVsn_t *hvp)
{
	char		*id = "VolmanAlloc";
	char		pbuf[PKTSIZ(vmpVAlloc_t)];
	VsnT_t		*vp = &cip->vc_class.cs_vsns[writeslot];
	Seq_t		seq = vGetSeq();
	vmpHdr_t	*hp;
	vmpVAlloc_t	*ap;
	VsnReq_t	*vrp;
	VolType_t	*vtp;
	RC_t		*rcp;

	if (volman_sock == -1)
		return(NULL);

	nLogMsg(EL_STATUS1, id, " class %s slot %d rc %s type %s", 
			cip->vc_class.cs_prefix,
			writeslot, vp->vs_rc, vp->vs_voltype);

	/*
	 *  if recent request failed, back off 
	 *	to allow admin to fix 
	 */
	if (cip->vc_allocwait[writeslot] > Now)
		return(NULL);

	vtp = vVolTypeByID(vp->vs_voltype);
	if (vtp == NULL) {
		hvp->hv_error = 1;
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
			"class %s slot %d: voltype %s unknown - SAVES BLOCKED", 
			cip->vc_class.cs_prefix, writeslot, vp->vs_voltype);
		return(NULL);
	}

	rcp = cGetRcp(vp->vs_rc);
	if (rcp == NULL) {
		hvp->hv_error = 1;
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
			"class %s slot %d: RC %s unknown - SAVES BLOCKED",
			cip->vc_class.cs_prefix, writeslot, vp->vs_rc);
		return(NULL);
	}

	/*
	 *  build request
	 */
	hp = vBuildHdr(M_VALLOC, seq, (vmpHdr_t *) pbuf);
	ap = MSGPTR(hp, vmpVAlloc_t);
	ap->va_rc = rcp->rcd_number;
	ap->va_poolgid = 0;		/* TODO - could add pool to VsnT_t */
	ap->va_type = vtp->vt_map;
	ap->va_mode = 0600;
	ap->va_gid = vvVsnGroupId;
	(void) strncpy(ap->va_desc, "VVM", DESCLEN);

	/*
	 *  send request
	 */
	if (lSnd2RP(hp))
		return(NULL);

	/*
	 *  make/enqueue VsnReq_t to track response
	 */
	vrp = NewVsnReq((void *)hvp, VQT_ALLOC, cip);
	vrp->vr_writeslot = writeslot;
	vrp->vr_status = VSQ_VOLALLOC;
	vrp->vr_volseq = seq;

	return(vrp);
}

/*
**	VolmanMount() - send request & put VsnReq_t in queue to track response
*/
  VsnReq_t *
VolmanMount(int nvsns, char *vsnp, int type, void *caller, VVMClass_t *cip)
{
	char		*id = "VolmanMount";
	Seq_t		seq = vGetSeq();
	VsnReq_t	*vrp;
	int		ct;

	if (volman_sock == -1)
		return(NULL);

	/*
	**	build & send pkt
	*/
	if ((ct = SendMount(nvsns, vsnp, type, seq)) == -1)
		return(NULL);

	/*
	**	make/enqueue VsnReq_t to track response
	*/
	vrp = NewVsnReq(caller, type, cip);
	
	memcpy(vrp->vr_vsns, vsnp, (size_t)(nvsns * VSNLEN));
	vrp->vr_nvsns = ct;	/* # of slots filled in nvsns */
	vrp->vr_status = VSQ_VOLMOUNT;
	vrp->vr_volseq = seq;
	vrp->vr_time = Now;

	return(vrp);
}

/*
 *  VolmanMount1() - mount single vsn for write
 */
  VsnReq_t *
VolmanMount1(char *vsnp, void *caller, VVMClass_t *cip, int slot)
{
	char		buf[MAXVSN * VSNLEN];
	VsnReq_t	*vrp;

	memset(buf, ' ', sizeof(buf));

	memcpy(buf+slot*VSNLEN, vsnp, (size_t)VSNLEN);

	vrp = VolmanMount(cip->vc_class.cs_nvsns, buf, VQT_WRITE, caller, cip);
	if (vrp != NULL)
		vrp->vr_writeslot = slot;
	return(vrp);
}


/*
 *  VolmanUnmount() - vvmd is done w/ vsn; send async unmount.
 *	Let caller clean up other aspects of VsnReq_t.
 */
  void
VolmanUnmount(VsnReq_t *vrp, char *clnt)
{
	char		*id = "VolmanUnmount";
	char		pbuf[PKTSIZ(vmpVsn_t)];
	Seq_t		seq = vGetSeq();
	int		slot;
	vmpHdr_t	*hp;
	vmpVsn_t	*ump;

	memset(pbuf, 0, sizeof(pbuf));	/* for purify */

	if (volman_sock == -1) {
		/*
		 *  volman down - consider it unmounted
		 */
		return;
	}
	if (vrp->vr_type == VQT_WRITE) {
		hot_mounted--;
nLogMsg(EL_STATUS1, id, "hot_mounted-- -> %d",  hot_mounted);
}
	/*
	**	build pkt
	*/
	if (vrp->vr_type == VQT_WRITE)
		slot = vrp->vr_writeslot;
	else
		slot = vrp->vr_readslot;
	if (slot < 0) {
		nLogMsg(EL_ERROR1, id, "slot < 0");
		(void)kill(0,5);
	}
	hp = vBuildHdr(M_UNMOUNT, seq, (vmpHdr_t *)pbuf);
	ump = MSGPTR(hp, vmpVsn_t);
	memcpy(&ump->v_vsn, vrp->vr_vsns + VSNLEN * slot, (size_t)VSNLEN);
	ump->v_flag = VO_ASYNC;
	nLogMsg(EL_STATUS2, id, "%.*s for %s", VSNLEN, &ump->v_vsn, clnt);

	/*
	**	send pkt; if volman down - consider it unmounted
	*/
	(void) lSnd2RP(hp);

	vrp->vr_time = Now;
}


  static void
Remount(VsnReq_t *vrp)
{
	char		*id = "Remount";
	VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;
	int		ct;

	/*
	 *  reset status
	 */
	vrp->vr_status = VSQ_VOLMOUNT;
	vrp->vr_volseq = vGetSeq();
	vrp->vr_mvseq = VVNOSEQ;
	vrp->vr_hostp = NULL;
	vrp->vr_bufsize = -1;
	memset(vrp->vr_cancel, 0, sizeof(vrp->vr_cancel));

        ct = SendMount(cip->vc_class.cs_nvsns, vrp->vr_vsns, vrp->vr_type,
						vrp->vr_volseq);
	if (ct == -1) {
		switch (vrp->vr_type) {
		    case VQT_WRITE:
			CWMountFailed(vrp, 0);
			break;
		    case VQT_READ:
			CRVolmanDown(vrp);
			break;
		    case VQT_RECYCLE:
			CRecycleVolmanDown(vrp);
			break;
		    default:
			nLogMsg(EL_FATAL, id, "missing case %d", vrp->vr_type);
		}
		return;
	} 
	vrp->vr_time = Now;
}

  void
VolmanRemount(VsnReq_t *vrp)
{
	char		*id = "VolmanRemount";

	if (vrp->vr_status != VSQ_MOUNTED  &&  vrp->vr_status != VSQ_MOVER)
		nLogMsg(EL_FATAL, id, "unexpected status %s",
						vvVsnStatus(vrp->vr_status));

	/*
	 *  kick it off the drive (unmount is async)
	 */
	VolmanUnmount(vrp, id);

	Remount(vrp);
}

/***********************************************
 *  volman-response handling routines 
 */

/*
 *  MVsn() - handle M_VSN from volman: scratch allocation succeeded
 */
  static void
MVsn(VsnReq_t *vrp, vmpHdr_t *hp)
{	
	char		*id = "MVsn";
	vmpVsn_t	*vsp = MSGPTR(hp, vmpVsn_t);
	VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;
	Seq_t		seq = vGetSeq();
	int		ct;

	if (vrp->vr_type != VQT_ALLOC)
		nLogMsg(EL_FATAL, id, "not alloc: %s", 
					vvMountType(vrp->vr_type));
		
	if (isalnum(vrp->vr_vsns[vrp->vr_writeslot*VSNLEN]))
		nLogMsg(EL_FATAL, id, 
			"vsn %.*s allocated but one (%.*s) already exists",
			VSNLEN, &vsp->v_vsn, 
			VSNLEN, vrp->vr_vsns+vrp->vr_writeslot*VSNLEN);

	/*
	**	fill in vrp
	*/
	memcpy(vrp->vr_vsns+vrp->vr_writeslot*VSNLEN, &vsp->v_vsn, 
							(size_t)VSNLEN);

	/*
	**	let class mgr update info & delete vrp
	*/
	CWAllocDone(vrp);
}


/*
**	MVolstat() - handle M_VOLSTAT from volman: mount is done
*/
  static void
MVolstat(VsnReq_t *vrp, vmpHdr_t *hp)
{
	char		*id = "MVolstat";
	VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;
	vmpVolStat_t	*vsp = MSGPTR(hp, vmpVolStat_t);
	HostInfo_t	*hop;	/* volman struct */
	VolType_t	*vtp;	/* volman struct */
	char		*vrpvsn;
	int		slot, i;

nLogMsg(EL_STATUS1, id, "(%s)  retries %d", vvMountType(vrp->vr_type),
vrp->vr_retries);

	/*
	**	consistency checks
	*/
	if (vrp->vr_status != VSQ_VOLMOUNT)
		nLogMsg(EL_FATAL, id, "status %s not mount", 
					vvVsnStatus(vrp->vr_status));
	if (vsp->vv_use != U_MOUNT)
		nLogMsg(EL_FATAL, id, "unexpected use");

	vrp->vr_time = Now;

	/*
	**	figure out which slot
	*/
	vrpvsn = vrp->vr_vsns;
	for (slot=0; slot<cip->vc_class.cs_nvsns; slot++, vrpvsn+=VSNLEN) {
		if (!memcmp(vrpvsn, &vsp->vv_vsn, (size_t)VSNLEN))
			break;
	}

	/*
	**	handle/check slot
	*/
	if (slot == cip->vc_class.cs_nvsns) /* FATAL for now.. */
		nLogMsg(EL_FATAL, id, "vsns don't match [%.*s] [%.*s]",
				VSNLEN * cip->vc_class.cs_nvsns, vrp->vr_vsns, 
				VSNLEN, &vsp->vv_vsn);

	switch (vrp->vr_type) {
	    case VQT_READ:
		/*
		**	note which vsn was mounted
		*/
		vrp->vr_readslot = slot;
		break;
	    case VQT_WRITE:
		/*
		**	consistency check
		*/
		if (slot != vrp->vr_writeslot)
			nLogMsg(EL_FATAL, id, "!= writeslot");
		break;
	    case VQT_RECYCLE:
		/*
		**	consistency check
		*/
		if (slot != vrp->vr_readslot)
			nLogMsg(EL_FATAL, id, "recyc != readslot");
		break;
	    default:
		nLogMsg(EL_FATAL, id, "unknown vr_type %d", vrp->vr_type);
	}
	if (vrp->vr_retries) {
		/*
		**	success, so reset vr_retries
		*/
		vrp->vr_retries = 0;
	}
		
	/*
	**	handle if req was canceled
	*/
	if (vrp->vr_cancel[slot]) {
		/*
		**	something came up during mount - throw it away
		*/
		VolmanUnmount(vrp, id);

		/*
		**	all calls delete vrp
		*/
		switch (vrp->vr_type) {
		    case VQT_WRITE:
			CWMountFailed(vrp, 1);
			break;
		    case VQT_READ:
			CRChooseAlternateMount(vrp);
			break;
		    case VQT_RECYCLE:
			CRecycleCancel(vrp);
			break;
		}
		return;
	}

	/*
	**	vvmd needs to know host that tape drive is
	**	on in order to send mover request to
	**	mover on that host. Maybe the mover
	**	isn't up now, but for now make sure
	**	the host is at least configged properly.
	*/
	hop = HostInfo(vsp->vv_host);
	if (hop == NULL)
		nLogMsg(EL_FATAL, id, "nonconfig volman host %d", vsp->vv_host);
	vrp->vr_hostp = HostByName(hop->h_name);
	if (vrp->vr_hostp == NULL)
		nLogMsg(EL_FATAL, id, "nonconfig vvm host %s", hop->h_name);

	/*
	**	reset errs-in-a-row for this slot
	*/
	cip->vc_vsloterrs[slot] = 0;

	/*
	**	note vsn status
	*/
	vrp->vr_status = VSQ_MOUNTED;
	vrp->vr_time = Now;

	/*
	**	retain seq for write requests,since they
	**	stay mounted, and an operator may
	**	need to force a dismount
	*/
	if (vrp->vr_type != VQT_WRITE)
		vrp->vr_volseq = NOSEQ;

	/*
	**	note optimal buffer size for this medium
	*/
	vtp = vVolType(vsp->vv_voltype);
	if (vtp == NULL) {
		nLogMsg(EL_CONSOLE0|EL_ERROR2, id, 
				"unknown vsntype %d - guessing bufsize", 
				vsp->vv_voltype);
		vrp->vr_bufsize = 4096;
	} else {
nLogMsg(EL_STATUS1, id, "bufsize %d", vtp->vt_bufsize);
		vrp->vr_bufsize = vtp->vt_bufsize;
	}

	/*
	**	pass off to requestor: 
	**		writer/recycle = vvmd class
	**		reader = client request
	*/
	switch (vrp->vr_type) {
	    case VQT_READ:
		CRMoverRead(vrp);
		break;
	    case VQT_WRITE:
		hot_mounted++;
nLogMsg(EL_STATUS1, id, "hot_mounted++ -> %d",  hot_mounted);
		CWHandoffHotMount(vrp);
		break;
	    case VQT_RECYCLE:
		CRecycleMounted(vrp);
		break;
	}
}

/*
**	MountError() - error on waiting-for-mount
*/
static void
MountError(VsnReq_t *vrp, vmpHdr_t *hp)
{
	char		*id = "MountError";
	vmpError_t	*ep = MSGPTR(hp, vmpError_t);
	VVMClass_t	*cip = (VVMClass_t *)vrp->vr_cp;
	int		i, level;
	char		*cp;

	/*
	**	note problem according 
	**	to type of request
	*/
	switch (vrp->vr_type) {
	    case VQT_WRITE:
		nLogMsg(EL_ERROR1, id, "volman mount/write err (%.*s): %s",
							VSNLEN,
							vrp->vr_vsns + 
							  vrp->vr_writeslot * 
							  VSNLEN, ep->vn_msg);
		break;
	    case VQT_READ:
		nLogMsg(EL_ERROR1, id, "volman mount/read err on %.*s: %s",
					VSNLEN * cip->vc_class.cs_nvsns,
					vrp->vr_vsns, ep->vn_msg);
		break;
	    case VQT_RECYCLE:
		nLogMsg(EL_ERROR1, id, "volman mount/recyc err on %.*s: %s",
					VSNLEN, &ep->vn_vsn, ep->vn_msg);
		break;
	    default:
		nLogMsg(EL_FATAL, id, "unknown vr_type %d, err %s", 
					vrp->vr_type, ep->vn_msg);
	}

	/*
	**	increment errors-in-a-row count for slot(s)
	*/
	cp = vrp->vr_vsns;
	for (i=0; i<cip->vc_class.cs_nvsns; i++, cp+=VSNLEN) {
		if (*cp != ' ') {
			if (cip->vc_vsloterrs[i]++ > MAX_SLOT_ERRS) {
				nLogMsg(EL_CONSOLE0, id, 
				  "Class %s slot %d: %d volman errs in a row",
				  cip->vc_class.cs_name, i,
				  cip->vc_vsloterrs[i]);
			}
		}
	}

	/*
	**	wait/retry for possibly transient errs
	*/
	if (ep->vn_error == E_UNAVAIL) {
		mount_retry++;
nLogMsg(EL_STATUS1, id, "  mount_retry up to %d", mount_retry);
		vrp->vr_retries++;
		if (vrp->vr_retries < MAX_MOUNT_RETRIES) {
nLogMsg(EL_STATUS1, id, "  setting retry");
			vrp->vr_status = VSQ_RETRYMOUNT;
			return;
		}
	}

	/*
	**	bad enough err or retries exhausted;
	**	interpret error level
	*/
	switch (ep->vn_error) {
	    case E_UNAVAIL:
nLogMsg(EL_STATUS1, id, "unavail");
		level = 0;
		break;
	    case E_STUCK:
nLogMsg(EL_STATUS1, id, "stuck");
		level = 1;
		break;
	    case E_ADMIN:
		level = 1;
		nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
					"UNRECOVERABLE volman ADMIN err: %s",
					ep->vn_msg);
		break;
	    default:
		level = 2;
		nLogMsg(EL_ERROR1|EL_CONSOLE0, id, 
					"UNRECOVERABLE volman err: %s",
					ep->vn_msg);
	}

	if (vrp->vr_retries) {
		vrp->vr_retries = 0;
		mount_retry--;
nLogMsg(EL_STATUS1, id, "  mount_retry down to %d on new err or retries",
mount_retry);
	}
nLogMsg(EL_STATUS1, id, "  level %d", level);

	/*
	**	handle according to type of mount
	*/
	switch (vrp->vr_type) {
	    case VQT_WRITE:
		/*
		**	writing to tape: try another eligible tape if
		**	possible
		*/
		CWMountFailed(vrp, level);
		break;
	    case VQT_READ:
		/*
		**	reading: simple failure of all vsns
		*/
		CRMountFailed(vrp);
		break;
	    case VQT_RECYCLE:
		/*
		**	recycling: simple failure of only vsn
		*/
		CRecycleMountFailed(vrp);
		break;
	}
}


  static void
MError(VsnReq_t *vrp, vmpHdr_t *hp)
{
	char		*id = "MError";
	vmpError_t	*ep = MSGPTR(hp, vmpError_t);

	vrp->vr_time = Now;

	switch (vrp->vr_status) {

	    case VSQ_VOLMOUNT:
		MountError(vrp, hp);
		break;

	    case VSQ_VOLALLOC:
		CWAllocFailed(vrp, ep->vn_msg, 1);
		break;

	    case VSQ_MOUNTED:
	    case VSQ_WAIT:
		/*
		**	vsn was dismounted from under us? 
		*/
		nLogMsg(EL_ERROR1, id, "volman err on mounted/%s",
					vvMountType(vrp->vr_type));

		vrp->vr_status = VSQ_UNMOUNTED;

		switch (vrp->vr_type) {
		    case VQT_WRITE:
			CWMountCanceled(vrp);
			break;
		    case VQT_READ:
			CRMountCanceled(vrp);
			break;
		    case VQT_RECYCLE:
			/* TODO - maybe need to count errs & give up? */
			CRRecycleRemount(vrp);
			break;
		}
		break;

	    case VSQ_MOVER:

		/*
		**	vsn was mounted & something happened on
		**		the volman side (e.g. RC went down),
		**		but as long as the mover can work 
		**		w/ the mount, it's not an immediate 
		**		problem. Big question is whether to
		**		keep sending mover reqs after the
		**		current one - let's try ignoring the
		**		error for now.
		*/
		nLogMsg(EL_STATUS1, id, 
			"Mover working on %.*s ignoring volman err for now",
					VSNLEN, &ep->vn_vsn);
		break;

	    default:
		nLogMsg(EL_FATAL, id, "unexpected req status %s", 
						vvVsnStatus(vrp->vr_status));
	}
}



  static void
HandlePkt(vmpHdr_t *hp)
{
	char		*id = "HandlePkt";
	VsnReq_t	*vrp;
	vmpVolStat_t	*vsp;

	if (hp->vh_seq != NOSEQ)
		vrp = GetVsnVolmanSeq(hp->vh_seq);
	else
		vrp = NULL;

	switch (hp->vh_type) {

	    case M_VOLSTAT:
		vsp = MSGPTR(hp, vmpVolStat_t);
		nLogMsg(EL_STATUS1, id, "mounted %.*s", VSNLEN, &vsp->vv_vsn);

		if (vrp != NULL) {
			MVolstat(vrp, hp);
		} else {
			nLogMsg(EL_ERROR1, id, "volstat seq %d w/ no req on %.*s", 
						hp->vh_seq, VSNLEN, &vsp->vv_vsn);
		}
		break;

	    case M_VSN:
		if (vrp != NULL) {
			MVsn(vrp, hp);
		} else {
			vmpVsn_t	*vsp = MSGPTR(hp, vmpVsn_t);
			nLogMsg(EL_FATAL, id, "vsn w/ no req: %.*s", 
						VSNLEN, &vsp->v_vsn);
			/* TODO - recycle? */
		}
		break;

	    case M_ACK:
		/* ignore for now */
		break;

	    case M_VERROR:
		if (vrp != NULL) {
			MError(vrp, hp);
		} else {
			vmpError_t	*ep = MSGPTR(hp, vmpError_t);
			nLogMsg(EL_ERROR1, id,
					"volman err w/ no req on %.*s: %s",
					VSNLEN, &ep->vn_vsn, ep->vn_msg);
		}
		break;
		
	    case M_SHUTDOWN:
		volman_sock = -1;
		VolmanDown();
		break;

	    case M_NOP:
		break;

	    default:
		nLogMsg(EL_FATAL, id, "unexpected pkt %s", vHdrStr(hp));
	}
}

/*
 *  VolmanRecv() - match volman responses to requests and call handler;
 *	discard pkts when done
 */
  void
VolmanRecv()
{
	char		*id = "VolmanRecv";
	vmpHdr_t	*hp;

	while (hp = lRcvFromRP(IPC_NDELAY)) {
		HandlePkt(hp);
		nFree(hp);
	}

	if (VMError != E_NOERROR) {
		nLogMsg(EL_STATUS1, id, "vold down, cleaning up");
		volman_sock = -1;
		VolmanDown();
	}
}



  void
DelVsnReq(VsnReq_t *vrp)
{
	VsnReq_t	*vrp2;

	VOLDEQ(vrp);

	if ((vrp2=(VsnReq_t*)vrp->vr_waitq.q_prev) != NULL)
		vrp2->vr_waitq.q_next = vrp->vr_waitq.q_next;
	if ((vrp2=(VsnReq_t*)vrp->vr_waitq.q_next) != NULL)
		vrp2->vr_waitq.q_prev = vrp->vr_waitq.q_prev;
		
	nFree(vrp);
}


  int
VolmanBusy(char *vsnp)
{
	char		*id = "VolmanBusy";
	VsnReq_t	*vrp;

	for (vrp=VOLQHEAD(); vrp; vrp=VOLQNEXT(vrp)) {
		char	*vsnp2;

		if (vrp->vr_type == VQT_WRITE) {
			vsnp2 = vrp->vr_vsns + (vrp->vr_writeslot * VSNLEN);
			if (!memcmp(vsnp, vsnp2, (size_t)VSNLEN))
				return(1);
		} else {
			int	i, nvsns = vrp->vr_nvsns;

			vsnp2 = vrp->vr_vsns;
			for (i=0; i<nvsns; i++) 
				if (!memcmp(vsnp2 + (i*VSNLEN), vsnp, 
								(size_t)VSNLEN))
					return(1);
		}
	}
	return(0);
}



  int
VolmanCancelBusy(char *vsnp)
{
	char		*id = "VolmanCancelBusy";
	VsnReq_t	*vrp;

	for (vrp=VOLQHEAD(); vrp; vrp=VOLQNEXT(vrp)) {
		char	*vsnp2;

		if (vrp->vr_type == VQT_WRITE) {
			vsnp2 = vrp->vr_vsns + (vrp->vr_writeslot * VSNLEN);
			if (memcmp(vsnp, vsnp2, (size_t)VSNLEN))
				continue;

			nLogMsg(EL_STATUS1, id, "%s status %s", vsnstr(vsnp),
						vvVsnStatus(vrp->vr_status));
			switch (vrp->vr_status) {
			    case VSQ_IDLE:
			    case VSQ_MOUNTED:
				/*
				 *  no problem - unmount
				 */
				VolmanUnmount(vrp, id);
				CWMountFailed(vrp, 1);
				return(0);
			    default:
				/*
				 *  TODO - figure out what to do
				 */
				nLogMsg(EL_ERROR1|EL_CODE, id,
						"writable/busy: %s",
						vvVsnStatus(vrp->vr_status));
				return(1);
			}
		} else {
			int	i, nvsns = vrp->vr_nvsns;

			vsnp2 = vrp->vr_vsns;
			for (i=0; i<nvsns; i++) 
				if (!memcmp(vsnp2 + (i*VSNLEN), vsnp, 
								(size_t)VSNLEN))
					break;
			if (i == nvsns)
				continue;
			switch (vrp->vr_status) {
			    case VSQ_VOLMOUNT:
				/*
				 *  mark this vsn as a no-go in case
				 *	it gets mounted
				 */
				vrp->vr_cancel[i] = 1;
				break;
			    case VSQ_MOUNTED:
			    case VSQ_MOVER:
				VolmanUnmount(vrp, id);
				CRChooseAlternateMount(vrp);
				break;
			    default:
				nLogMsg(EL_ERROR1, id,
						"read/??-think..: %s",
						vvVsnStatus(vrp->vr_status));
			}
		}
	}
	return(0);
}

/*
**	VolmanRecycle() - release tape to volman
*/
int
VolmanRecycle(char *vsnp)
{
	char		*id = "VolmanRecycle";
	char		pbuf[PKTSIZ(vmpVsn_t)];
	vmpHdr_t	*shp, *rhp;
	vmpVsn_t	*vp;
	vmpError_t	*ep;

	/*
	**	make pkt & send it 
	*/
	shp = vBuildHdr(M_VRECYCLE, vGetSeq(), (vmpHdr_t *) pbuf);
	vp = MSGPTR(shp, vmpVsn_t);

	vp->v_flag = 0;
	memcpy(&vp->v_vsn, vsnp, sizeof(vp->v_vsn));
	if (lSnd2RP(shp))
		return(-1);

	/*
	**	get response, return status
	*/
	while (1) {
		if ((rhp = lRcvFromRP(IPC_DELAY)) == NULL)
			return(-1);

		if (rhp->vh_seq != shp->vh_seq) {
			HandlePkt(rhp);
			nFree(rhp);
			continue;
		}
		switch (rhp->vh_type) {
		    case M_VRECYCLE:
			vp = MSGPTR(rhp, vmpVsn_t);
			if (strncmp((char *)&vp->v_vsn, vsnp, VSNLEN)) {
				nLogMsg(EL_ERROR1|EL_CODE, id,
					"wrong vsn returned (%.*s)",
							VSNLEN, &vp->v_vsn);
				nFree(rhp);
				return(-1);
			}
			/*
			**	success
			*/
			nFree(rhp);
			(void)DBDeleteVsn(vsnp);
			return(0);
		    case M_VERROR: 
			ep = MSGPTR(rhp, vmpError_t);
			nLogMsg(EL_ERROR1, id,
					"volman err on %.*s: %s",
					VSNLEN, &ep->vn_vsn, ep->vn_msg);
			nFree(rhp);
			return(-1);
		    case M_SHUTDOWN:
			volman_sock = -1;
			VolmanDown();
			nFree(rhp);
			return(-1);
		    case M_NOP:
			nFree(rhp);
			continue;
		    default:
			nLogMsg(EL_ERROR1|EL_CODE, id, "unexpected pkt: %s", 
							vHdrStr(rhp));
			nFree(rhp);
			return(-1);
		}
	}
}

 VsnReq_t *
VolmanMountRecycle(char *vsnp, void *caller, VVMClass_t *cip)
{
	char		*id = "VolmanMountRecycle";
	char		pbuf[PKTSIZ(vmpUsrMount_t)];
	char		tag[TAGLEN];
	vmpHdr_t	*hp;
	vmpUsrMount_t	*mntp;
	Seq_t		seq = vGetSeq();
	VsnReq_t	*vrp;

	if (volman_sock == -1)
		return(NULL);

	/*
	**	build pkt (note: packet protocol is volman)
	*/
	hp = vBuildHdr(M_USRMOUNT, seq, (vmpHdr_t *)pbuf);
	mntp = MSGPTR(hp, vmpUsrMount_t);
	memcpy(&mntp->vu_vsn, vsnp, sizeof(mntp->vu_vsn));
nLogMsg(EL_STATUS1, id, "[%.6s]", &mntp->vu_vsn);
	mntp->vu_flags = VO_WAIT|VO_IGNORE_RING;
	memset(mntp->vu_rid, '\0', sizeof(mntp->vu_rid));
	(void)sprintf(tag, "VVM.recyc");
	copytag(mntp->vu_tag, tag);

	/*
	**	send pkt
	*/
	if (lSnd2RP(hp))
		return(NULL);

	/*
	**	make/enqueue VsnReq_t to track response
	*/
	vrp = NewVsnReq(caller, VQT_RECYCLE, cip);
	memcpy(vrp->vr_vsns, vsnp, (size_t)VSNLEN);
	vrp->vr_nvsns = 1;	/* # of slots filled in nvsns */
	vrp->vr_status = VSQ_VOLMOUNT;
	vrp->vr_volseq = seq;

	vrp->vr_time = Now;

	return(vrp);
}


/*
**	VolmanCheckQ() - see if any mount waiting for
**		retries on E_UNAVAIL are ready to launch again,
**		and whether any mount/reads have had their grace 
**		period on the drive expire.
*/
 void
VolmanCheckQ()
{
	char		*id = "VolmanCheckQ";
	VsnReq_t	*vrp, *vrp2;

	if (!read_idle  &&  !mount_retry)
		return;

	for (vrp=VOLQHEAD(); vrp; vrp=vrp2) {
		vrp2 = VOLQNEXT(vrp);	/* since vrp may be DQ'd */

		/*
		**	handle mount waiting for retry
		*/
		if (vrp->vr_status == VSQ_RETRYMOUNT) {
			VVMClass_t	*cip = (VVMClass_t *) vrp->vr_cp;

			if (Now - vrp->vr_time < MOUNT_RETRY_WAIT)
				continue;
			/*
			**	start it off again
			*/
			nLogMsg(EL_STATUS2, id, "retry mount: %.*s",
						cip->vc_class.cs_nvsns *VSNLEN,
						vrp->vr_vsns);
			Remount(vrp);
			mount_retry--;
nLogMsg(EL_STATUS1, id, "  mount_retry down to %d on remount", mount_retry);
			continue;
		}

		/*
		**	check for read/idle
		*/
		if (vrp->vr_type != VQT_READ)
			continue;
		if (vrp->vr_status != VSQ_IDLE)
			continue;

		if (Now - vrp->vr_time < READ_IDLE_WAIT)
			continue;

		if (vrp->vr_caller != NULL)
			nLogMsg(EL_FATAL, id, "caller exists");

		VolmanUnmount(vrp, id);
		DelVsnReq(vrp);
		read_idle--;
	}
}
