/*
 *  vvm_tpq.h - volman request monitoring (tape=vsn queue)
 */

#include "cl_space.h"	/* volman for Seq_t */

typedef struct	VsnReq {
	QLink_t		vr_vrq;		/* queue of all vsn reqs	    */
	QLink_t		vr_waitq;	/* waiting vsn reqs		    */
	int		vr_type;	/* write/read/recycle		    */
	void		*vr_caller;	/* e.g. WriteReq_t if write	    */
	Host_t		*vr_hostp;	/* host that tape drive is on	    */
	char		vr_vsns[MAXVSN*VSNLEN];
	char		vr_cancel[MAXVSN];
	int		vr_readslot;	/* class slot			    */
	int		vr_writeslot;	/* class slot			    */
	int		vr_nvsns;	/* # populated slots		    */
	int		vr_status;	/* volman/mover/idle		    */
	int		vr_bufsize;	/* from volman 'mounted' msg	    */
	Seq_t		vr_volseq;	/* seq for volman reqs		    */
	vvSeq_t		vr_mvseq;	/* seq for mover reqs		    */
	time_t		vr_time;	/* last volman-related action	    */
	int		vr_retries;	/* for working around downed srvrs  */
	void		*vr_cp;		/* VVMClass_t			    */
} VsnReq_t;

/*
 *  vr_type values
 */
#define VQT_ALLOC	1
#define VQT_WRITE	2
#define VQT_READ	3
#define VQT_RECYCLE	4

/*
 *  vr_status values
 */
#define VSQ_NONE	0	/* just allocated struct		    */
#define VSQ_VOLALLOC	1	/* volman M_VALLOC			    */
#define VSQ_RETRYMOUNT	2	/* waiting for volman E_UNAVAIL to clear    */
#define VSQ_VOLMOUNT	3	/* volman M_USRMOUNT			    */
#define VSQ_MOUNTED	4	/* mounted & waiting for mover to exist	    */
#define VSQ_IDLE	5	/* mounted waiting for write (or read)	    */
#define VSQ_MOVER	6	/* mounted & being read/written		    */
#define VSQ_WAIT	7	/* waiting on another req		    */
#define VSQ_UNMOUNTED	8	/* bumped by operator - not normal state    */

#define VOLENQ(_vrp)	ENQUEUE(_vrp, &VsnQ, vr_vrq, VsnReq_t)
#define VOLDEQ(_vrp)	DEQUEUE(_vrp, &VsnQ, vr_vrq, VsnReq_t)
#define VOLQHEAD()	((VsnReq_t *) VsnQ.q_head)
#define VOLQNEXT(_vrp)	((VsnReq_t *)_vrp->vr_vrq.q_next)

#define WAITQNEXT(_vrp)	((VsnReq_t *)_vrp->vr_waitq.q_next)
