
/*
 *  vvm_xdr.c: Virtual Volume Manager vvmd XDR routines for communicating 
 *	with its clients & servers
 *
 *	All VVM communication uses XDR over Internet sockets; 
 *	inet sockets are used so that connections can be made
 *	from remote hosts, and XDR is used to translate packets
 *	to and from a universal format for making sure that packet
 *	data structures are the same on different host architectures.
 *
 *	An array of XDR structures parallels the array of Sock_t
 *	socket management structures.
 *
 */

#include "vvmd.h"
#include <rpc/xdr.h>

extern Sock_t		SockArray[];
static XDR		XDRArray[FD_SETSIZE];

  void
XDRInit()
{
	Sock_t	*ip = SockArray;
	XDR	*xp = XDRArray;
	int	i;

	/*
	 *  associate socket array w/ XDR array
	 */
	for (i=0; i<FD_SETSIZE; i++, ip++, xp++)
		XDRSetup(ip, xp);
}

  vvpHdr_t *
XDRRecvI(int i)
{
	char		*id = "XDRRecvI";
	vvpHdr_t	*hp;

	if (i < 0  ||  i >= FD_SETSIZE)
		nLogMsg(EL_FATAL, id, "index i out of range: %d", i);

	hp = XDRRecv(&XDRArray[i]);
	if (hp == NULL)
		return(NULL);

	if (SockArray[i].sock_ptype != S_NOSOCK     &&
	    SockArray[i].sock_ptype != S_CONNECTED  &&
	    hp->vh_key != SockArray[i].sock_rcvkey) {
		nLogMsg(EL_CONSOLE0|EL_ERROR1, id, 
				"sock %d bad key 0x%x expected 0x%x, type %s",
				i, hp->vh_key, SockArray[i].sock_rcvkey,
				vvPortTypeStr(SockArray[i].sock_ptype));
		(void)ShutdownServer(i, 1);
		return(NULL);
	}
	return(hp);
}

  int
XDRNextRecordI(int i)
{
	return(XDRNextRecord(&XDRArray[i]));
}

  int
XDRSendI(vvpHdr_t *hp, int i, int now)
{
	int	ret;

	if (i < 0  ||  i >= FD_SETSIZE)
		nLogMsg(EL_FATAL, "XDRSendI", "index i out of range: %d", i);

	if (SockArray[i].sock_ptype == S_NOSOCK)
		return(-1);

	hp->vh_key = SockArray[i].sock_sndkey;
	if (ret = XDRSend(hp, &XDRArray[i], now))
		(void)ShutdownServer(i, 1);

	return(ret);
}

 void 
XDRFlushI(int slot)
{
	XDRFlush(&XDRArray[slot]);
}
