
#include "vv_space.h"
#include "vvm_host.h"
#include "vvm_vsnq.h"
#include "vvm_class.h"

#define MAX_SLOT_ERRS	5	/* errs-in-a-row for console complaints */


#define VVTIMER		"vvtimer"
#define VVM_TIMERSECS	"30"

extern char		*vvLibDir;

/*
 *  the most common vars, initialized in vv_init()
 */
#ifndef EXTERN
#define EXTERN extern
#endif 

EXTERN int	IsChild;
EXTERN QHead_t	VsnQ;
EXTERN time_t	Now;

/*
 *  prototypes
 */

/*  from vvm_init.c */

void		VVInit();
int		VVShutdown(char *caller, char *msg);

/*  from vvm_db1.c */

void		DBInit();
void		DBClose();
VVInfo_t	*DBGetIncompleteSave(int i);

void		DBNewClient(int clntpid, char *hostname);
void		DBRecoverClient(int clntpid, char *hostname);

VVInfo_t	*DBFindHotVV(char *xid, OFF_T size, VVMClass_t *ccip, 
			int pid, char *hostname);
VVInfo_t	*DBCreateHotVV(char *xid, OFF_T size, VVMClass_t *ccip,
			int pid, char *hostname);
int		DBCloseWriteVV(VV_t *vvp, OFF_T written, 
			int clntpid, char *hostname);
VVInfo_t	*DBOpenReadVV(VV_t *vvp);
void		DBFreedVV(VVInfo_t *vip);
void		DBRestored(VV_t *vvp);

VVInfo_t	*DBFinishVV(VV_t *vvp);
int		DBSavedVV(Vsn_t *vp, VVInfo_t *vip, int slot, VVMClass_t *cip);
int		DBRecycleVV(VV_t *vvp);

VVInfo_t	*DBLookupVV(VV_t *vvp);

Vsn_t		*DBAddHotVsn(char *vsnp, char *class, int slot);
Vsn_t		*DBGetHotVsn(char *class, int slot);
void		DBVsnErr(Vsn_t *vp);
int		DBUpdateVsnLast(Vsn_t *vp);
int		DBSetVsnNotHot(Vsn_t *vp, char *errstr);
Vsn_t		*DBLookupVsn(char *vsnp);
int		DBCheckVsn(char *vsnp, int fd, int flag);

int		DBMigrating(VV_t *vvp);

/*  from vvm_db2.c */

VVInfo_t	*DBQueryHotExactVV(int started, char *xid, char *class);
VVInfo_t	*DBQueryHotVV(int started);
int		DBCloseForHotQuery(int clientpid);
int		DBReopenForHotQuery(int parent, int exact);

VVInfo_t	*DBQueryFinishedVV(int started);
int		DBCloseForFinishedQuery(int clientpid);
int		DBReopenForFinishedQuery(int parent);

VVInfo_t	*DBLookupVVVsn(int started, char *vsnp);
int		DBCloseForVVVsnQuery(int clientpid);
int		DBReopenForVVVsnQuery(int parent);

VVInfo_t	*DBLookupIdVV(int started, char *xid);
int		DBCloseForIdQuery(int clientpid);
int		DBReopenForIdQuery(int parent);

Vsn_t		*DBQueryHotVsn(int started);
int		DBCloseForHotVsnQuery(int clientpid);
int		DBReopenForHotVsnQuery(int parent);

void		DBCleanupChild();

/*  from vvm_db3.c */
int		DBCheckVsn(char *vsnp, int fd, int flags);
int		DBBadVsn(char *vsnp);
Vsn_t		*DBFinishVsn(char *vsnp);
int		DBFinishClass(char *class, int *nStillHot, 
			int *nFinishedVV, int *nWaitVV, int *nTapes);
void		MDiskCk(vvpHdr_t *hp, int slot);
int		DBRecycleVsn(char *vsnp);
int		DBRecycleVsnDone(char *vsnp);
int		DBDeleteVsn(char *vsnp);
Vsn_t		*DBGetIncompleteRecycle(int i, int *recovp);
void		DBRecycRestoredVV(VVInfo_t *vp);

/*  from vvm_db4.c */
void		InitOndiskIndex();
void		AddOndisk(char *id, time_t atime);
void		DeleteOndisk(VV_t *vvp);
VVInfo_t	*GetOndisk(int which);

/*  from vvm_class.c */

void		InitClasses();
void		LogClasses();
void		MLogClasses(int cl_slot, unsigned int writes);
VVMClass_t	*VVMClassInfo(char *prefix);
VVMClass_t	*VVMClassInfoByName(char *name);
VVMClass_t	*VVMClassIndex(int i);

/*  from vvm_clwrite.c */

void		CWInitCatchups();
void		CWInitHotPanels();
void		CWAllocDone(VsnReq_t *vrp);
void		CWAllocFailed(VsnReq_t *vrp, char *msg, int level);
void		CWCheckQs(int urgent);
int		CWIdleQs();
void		CWHandoffHotMount(VsnReq_t *vrp);
void		CWMountFailed(VsnReq_t *vrp, int level);
void		CWMountCanceled(VsnReq_t *vrp);
void		CWriteVV2Vsns(VVInfo_t *vvp);
void		CWMoverOk(VsnReq_t *vrp, int rec, unsigned int blk);
void		CWMoverErr(VsnReq_t *vrp, int vverr, char *msg);
void		CWVolmanRestart();
VsnReq_t	*CWMountHotRead(int nvsns, char *vsnp);

/*  from vvm_clread.c */

void		CRestore(vvSeq_t seq, int clnt_sock, VVInfo_t *vip);
void		CRChooseAlternateMount(VsnReq_t *vrp);
void		CRMountFailed(VsnReq_t *vrp);
void		CRMountCanceled(VsnReq_t *vrp);
void		CRVolmanDown(VsnReq_t *vrp);
void		CRMoverRead(VsnReq_t *vrp);
void		CRMoverErr(VsnReq_t *vrp);

/*  from vvm_clrecycle.c */

void		MRecycleVsn(vvpHdr_t *hp, int slot);
void		CRRecycleRemount(VsnReq_t *vrp);
void		CRecycleMounted(VsnReq_t *vrp);
void		CRecycleMountFailed(VsnReq_t *vrp);
void		CRecycleMoverOk(VsnReq_t *vrp);
void		CRecycleMoverErr(VsnReq_t *vrp);
void		CRecycleVolmanDown(VsnReq_t *vrp);
void		CRecycleCancel(VsnReq_t *vrp);
int		CRecycleRestarts();

/*  from vvm_sock.c */

void		IPCInit();
void		IPCShutdown();
void		IPCServer();
int		ShutdownServer(int slot, int recover);
void		ShutdownServers();

/*  from vvm_xdr.c */

void		XDRInit();
vvpHdr_t	*XDRRecvI(int i);
int		XDRNextRecordI(int i);
int		XDRSendI(vvpHdr_t *hp, int i, int now);
void		XDRFlushI(int slot);

/*  from vvm_host.c */

int		NewHost(char *name);
Host_t 		*LookupHost(struct hostent *htp);
Host_t		*HostByName(char *str);
Host_t		*HostIndex(int i);

/*  from vvm_msg.c */

vvSeq_t		vvGetSeq();
void		Send2Movers(vvpHdr_t *hp, char *id);
void		SendAck(vvSeq_t seq, int slot);
void		SendNOP(int slot);
void		SendErr(vvSeq_t seq, int slot);
int		SendBuf(int slot, char *msgbuf);

/*  from vvm_disk.c */

void		DiskInitList();
void		DiskStat();
int		DiskReserve(VVMClass_t *cip);
void		DiskUnreserve(VVMClass_t *cip);
int		DiskCreate(VVInfo_t *vip, VVMClass_t *cip);
int		DiskFinish(VV_t *vvp);
int		DiskRecycle(VV_t *vvp);
int		DiskResidentSize(VV_t *vvp, OFF_T *size);

/*  from vvm_volman.c */

int		VolmanInit();

VsnReq_t	*VolmanAlloc(VVMClass_t *cip, int slot, HotVsn_t *hvp);

VsnReq_t	*VolmanMount(int nvsns, char *vsnp, int type, void *caller, 
				VVMClass_t *cip);
VsnReq_t	*VolmanMount1(char *vsnp, void *caller, 
				VVMClass_t *cip, int slot);
VsnReq_t	*VolmanMountRecycle(char *vsnp, void *caller, VVMClass_t *cip);
VsnReq_t	*UseMountedVsn(int nvsns, char *vsnp, void *caller, 
				VVMClass_t *cip);
VsnReq_t	*VolmanMountHotVsn(int nvsns, char *vsnp, void *caller, 
				VVMClass_t *cip);
void		VolmanUnmount(VsnReq_t *vrp, char *clnt);
void		VolmanRemount(VsnReq_t *vrp);

int		VolmanBusy(char *vsnp);
int		VolmanCancelBusy(char *vsnp);

void		VolmanCheckQ();

/*  from vvm_mover.c */

VsnReq_t	*GetVsnMoverSeq(vvSeq_t seq);
void	MoverStart(Host_t *hop);

/*  from vvm_1.c */

char	*vvVsnStatus(int state);

/*  from vvm_2.c */

void		MVVOpenWrite(vvpHdr_t *hp, int slot);
void		MVVCloseWrite(vvpHdr_t *hp, int slot) ;
void		MVVFinish(vvpHdr_t *hp, int slot);
void		MVVOpenRead(vvpHdr_t *hp, int slot);
void		MVVCloseRead(vvpHdr_t *hp, int slot);
void		MVVRecycle(vvpHdr_t *hp, int slot);

int		MMoverOk(vvpHdr_t *hp, int slot);
int		MMoverError(vvpHdr_t *hp, int slot);

void		MFinishClass(vvpHdr_t *hp, int slot);

/*  from vvm_3.c */

void		MVVlook(vvpHdr_t *hp, int slot);

