/*
 *  vvrecyclelist - generate list of favored VV's to
 *	recycle that will free relatively empty tapes.
 *
 *	Note: must run on vvmd's host since it directly
 *	accesses the dbase (readonly).
 */

#include "vv_space.h"
#include "vvm_db.h"
#include "avltree.h"

char	*arg0 = NULL;

extern char		*vvDBDir;

/*  vv, vsn tables  */
extern char		dbf_vv_tbl[];
extern int		db_vv_fd;
extern OFF_T		vv_addr;
extern vbuf_t		vbuf;
extern ClassSlot_t	*cslots[];

extern char		dbf_vsn_tbl[];
extern int		db_vsn_fd;
extern OFF_T		vsn_addr;
extern vsbuf_t		vsnbuf;

/*  vsn->vv index  */
DB		*db_vv_vsn_idx;
DBC		*cursor;
char		dbf_vv_vsn_idx[100];
char		vvkey[100];
int		dberr;
DBT		key, data;

  static void
usage(char *str)
{
	(void)fprintf(stderr, "%s\n", str);
	(void)fprintf(stderr, 
	    "usage: %s -t tapes [-c class [-s slot]] [-v nvv] [-n]\n", arg0);
	exit(1);
}

bomb()
{
	(void)fprintf(stderr, "exiting\n");
	exit(1);
	return(1);	/* for compiler warning */
}

typedef struct {
	char	vsn[VSNLEN];
	int	live;
	int	dead;
} vsnsort_t;


  int
comp(vsnsort_t *vsnp1, vsnsort_t *vsnp2)
{
	return(vsnp1->live > vsnp2->live);
}

main(int argc, char *argv[])
{
	char		class[VV_CLASSLEN];
	int		ntapes = 0, nvv = 0, gotclass = 0, slot = -1;
	int		i, cc, ret, printvv = 1, output_vv = 0;
	extern int	optind;
	extern char	*optarg;
	vsnsort_t	*vsp, *vsnarray = NULL;
	Class_t		*cip = NULL;
	int		count, maxvsns;

	arg0 = argv[0];
#ifndef PURIFY
	if (getuid() != 0) {
		(void)fprintf(stderr, "%s: must be root\n", arg0);
		exit(1);
	}
#endif
	arg0 = argv[0];

	/*
	 *  parse args
	 */
	while ((cc = getopt(argc, argv, "t:c:s:v:n")) != EOF) {
		switch (cc) {
		    case 't':
			if (!isnum(optarg))
				usage("-t: expected ntapes");
			ntapes = atoi(optarg);
			if (ntapes < 1)
				usage("-t: ntapes must be > 0");
			break;
		    case 'c':
			if (strlen(optarg) > VV_CLASSLEN)
				usage("-c: class prefix too long");
			if (vvm_class_policy_class_len  &&
			    strlen(optarg) != vvm_class_policy_class_len)
				usage("-c: class name not policy length");
			(void)strncpy(class, optarg, VV_CLASSLEN);
			gotclass = 1;
			break;
		    case 'n':
			printvv = 0;
			break;
		    case 's':
			if (!isnum(optarg))
				usage("-s: expected slot");
			slot = atoi(optarg);
			if (slot < 0)
				usage("-s: slot must be >= 0");
			break;
		    case 'v':
			if (!isnum(optarg))
				usage("-v: expected number of vv's");
			nvv = atoi(optarg);
			if (nvv < 1)
				usage("-v: vv's must be > 0");
			break;
		    default:
			break;
		}
	}

	/*
	 *  check args
	 */
	if (ntapes == 0)
		usage("-t must be given");

	if (gotclass) {
		cip = ClassInfo(class);
		if (cip == NULL)
			usage("-c: unknown class");
	}
	if (slot != -1) {
		if (!gotclass)
			usage("-s requires -c");
		if (slot >= cip->cs_nvsns)
			usage("-s: slot too high");
		slot = INT2CHAR(slot);
	}

	/*
	 *  open vsn & vv tables
	 */
	(void)sprintf(dbf_vsn_tbl, "%s/DB_VSN_TBL", vvDBDir);
	if ((db_vsn_fd = open(dbf_vsn_tbl, O_RDONLY)) == -1) {
		(void)fprintf(stderr, 
				"%s: %s\n\t(note: must run on vvmd host)\n",
				dbf_vsn_tbl, strerror(errno));
		exit(1);
	}
	(void)sprintf(dbf_vv_tbl, "%s/DB_VV_TBL", vvDBDir);
	if ((db_vv_fd = open(dbf_vv_tbl, O_RDONLY)) == -1) {
		perror(dbf_vv_tbl);
		exit(1);
	}
             
	/*
	 *  set up slot ptrs in the raw VV record
	 */
	cslots[0] = (ClassSlot_t *) vbuf.vb_vsn0;
	cslots[1] = (ClassSlot_t *) vbuf.vb_vsn1;
	cslots[2] = (ClassSlot_t *) vbuf.vb_vsn2;
	cslots[3] = (ClassSlot_t *) vbuf.vb_vsn3;

	/*
	 *  get a copy of the vsn->vv index from vvmd
	 */
	nLogInit(arg0, bomb);
	nLogInitStderr(EL_ERROR1|EL_ERROR2);
	VVCLInit(vvHost, VVM_CLIENT, RETRY_FATAL);
	VVCLGetIxCpy();
	VVCLGoodbye(1);

	/*
	**	open index copy & make cursor
	*/
	if (dberr = db_create(&db_vv_vsn_idx, NULL, 0)) {
		(void)fprintf(stderr, "db_create(): %s\n", db_strerror(dberr));
		exit(1);
	}
	(void)sprintf(dbf_vv_vsn_idx, "/tmp/vvmd_vv_vsn.%d", getpid());
	if (dberr = db_vv_vsn_idx->open(db_vv_vsn_idx, dbf_vv_vsn_idx, NULL, 
						DB_BTREE, DB_RDONLY, 0600)) {
		(void)fprintf(stderr, "db open(%s): %s\n", dbf_vv_vsn_idx,
							db_strerror(dberr));
		exit(1);
	}
	if (dberr = db_vv_vsn_idx->cursor(db_vv_vsn_idx, NULL, &cursor, 0)) {
		(void)fprintf(stderr, "cursor(): %s\n", db_strerror(dberr));
		exit(1);
	}

	/*
	**	unlink index now in case this proc dies
	*/
	(void)unlink(dbf_vv_vsn_idx);

	/*
	**	read vsn table, screening & making array of things to sort
	*/
	count = 0;
	maxvsns = 500;
	vsnarray = (vsnsort_t *) nAlloc(maxvsns * sizeof(vsnsort_t));
	vsp = vsnarray;
	for (i=1;;i++) {
		ret = read(db_vsn_fd, &vsnbuf, sizeof(vsnbuf));
		if (ret != sizeof(vsnbuf)) {
			if (ret == 0)
				break;
			if (ret == 1)
				nLogMsg(EL_FATAL, 0, 
						"%s: failure at record %d",
						dbf_vsn_tbl, i);
			else
				nLogMsg(EL_FATAL, 0,
						"%s: incomplete record %d",
						dbf_vsn_tbl, i);
		}

		/*
		**	see if record is eligible
		*/
		if (vsnbuf.vb_status != VVS_FINISHED)
			continue;

		/*
		**	apply user's selection criteria
		*/
		if (slot != -1  &&  vsnbuf.vb_classI != slot)
			continue;
		if (gotclass  &&  strncmp(class, vsnbuf.vb_class, VV_CLASSLEN))
			continue;

		if (count == maxvsns) {
			maxvsns += 500;
			vsnarray = (vsnsort_t *) 
				nReAlloc(vsnarray, maxvsns * sizeof(vsnsort_t));
			vsp = vsnarray + count;
		}
		memcpy(vsp->vsn, vsnbuf.vb_vsn, sizeof(vsp->vsn));
		if (C2BIN(DecInt, &vsp->live, vsnbuf.vb_livevv)  ||
		    C2BIN(DecInt, &vsp->dead, vsnbuf.vb_delvv)) {
			(void)fprintf(stderr, "converting fields\n");
			exit(1);
		}
		count++;
		vsp++;
	}

	/*
	**	sort vsns
	*/
	(void)qsort((void *)vsnarray, (size_t)count,
			(size_t)sizeof(vsnsort_t),
			(int (*)(const void *, const void *))comp); 

	/*
	**	dump vv's on the number of vsns requested
	*/
	if (ntapes > count)
		ntapes = count;
	vsp = vsnarray;
	for (i=0; i<ntapes; i++, vsp++) {
		/*
		**	search vsn->vv index for vv's
		*/
		int	j, vvct = 0;
		(void)sprintf(vvkey, "%.*s", VSNLEN, vsp->vsn);
		printf("==== %s\n", vvkey);
		STR_DBT(key, vvkey);
		for (j=0;;j++) {
			char	*vvname;

			if (j)
				dberr = cursor->c_get(cursor, &key, &data,
								DB_NEXT);
			else
				dberr = cursor->c_get(cursor, &key, &data,
								DB_SET_RANGE);
			if (dberr) {
				if (dberr != DB_NOTFOUND) {
					(void)fprintf(stderr, "c_get(): %s\n",
							db_strerror(dberr));
					exit(1);
				}
				break;
			}
			if (memcmp(vvkey, key.data, (size_t)(VSNLEN+1)))
				break;

			if (data.size != sizeof(vv_addr)) {
				(void)fprintf(stderr, "bad data.size\n");
				exit(1);
			}
			memcpy(&vv_addr, data.data, sizeof(vv_addr));
			if (LoadVV(0, 0))
				exit(1);

			vvname = VV2Name(Vbuf2VV());

			/*
			**	check that vsn is in vv record
			*/
			if (slot == -1) {
				int	k, nvsns = CHAR2INT(vbuf.vb_nvsns);
				/*
				**	check each slot
				*/
				for (k=0; k<nvsns; k++) {
					if (!memcmp(cslots[k]->vb_vsn, vsp->vsn,
							    sizeof(vsp->vsn)))
						break;
				}
				if (k == nvsns)
					continue;
			} else {
				if (memcmp(cslots[slot]->vb_vsn, vsp->vsn, 
							sizeof(vsp->vsn))) {
					continue;
				}
			}
			if (printvv) {
				/*
				**	output vv
				*/
				printf("%s\n", vvname);
			}
			output_vv++;
		}
		if (vvct != vsp->live)
			(void)fprintf(stderr, "%s:  live %d  counted %d\n",
					vvkey, vsp->live, vvct);
		printf("==== %s  live %d  recycled %d\n", vvkey,
							vvct, vsp->dead);
		/*
		**	now that vsn is finished, if limit was
		**		placed on vv's, consider it
		*/
		if (nvv  &&  output_vv >= nvv)
			break;
	}
}

