/*
 *  mv_2.c: NAStore Virtual Volume Mgr's Mover daemon's children
 *	for doing drive access
 */

/* #define FILEDRIVES */

#define TAPEOPENRETRIES	5


#include <pwd.h>
#include <signal.h>

#include "mv.h"
#include "nas_tape.h"

#ifdef FILEDRIVES
# include "filedrives.c"
#endif


extern char		*vNodeDir;	/* from libvol.a */


extern Req_t		*ReqArray;
extern int		MaxReq;

extern int		bomb();
extern char		**args;		/* argv */
extern time_t		Now;

static STAT_T		sbuf;

/*
 *  since this process handles only 1 request,
 *	make error pkt info global, locally.
 */
static VV_t	*vvp;
static char	VVName[VV_NAMELEN];	
static char	*vvPath;
static char	*vsnp;
static vvSeq_t	seq;
static int	cmd;

/*
 *  Keep a global pointer to the request to help construct
 *	an err msg (if necc to err out from the signal catcher).
 */
static Req_t	*Rp = NULL;

static char		tpnode[MAXPATHLEN];
static TapeLabel_t	lblbuf;

/*
 *  Children catch signals from parent sent for timeout or shutdown.
 *	This probably won't help with driver problems (i.e. timeout),
 *	since hang in driver is unkillable (at least w/ Convex). 
 *	Because of this, the parent sends the error msg to vvmd on 
 *	timeout. Always exit w/ no problem when catching signal, 
 *	since parent is handling reporting of status in this case.
 */
static int	sigged = 0;

static
sigterm()
{
	char	*id = "sigterm";

	if (sigged) {
		nLogMsg(EL_STATUS1, 0, " %s_%d %.*s: 2 sigterm/timeouts", 
				VVName, seq, VSNLEN, vsnp);
		exit(EXIT_OK);
	}
	nLogMsg(EL_STATUS1, 0, " %s_%d %.*s: sigterm", 
				VVName, seq, VSNLEN, vsnp);
	sigged = 1;
/* try exit on 1st */
	exit(EXIT_OK);
	return(1);		/* for compiler warning */
}

  static void
ChildErr(int exit_status)
{
	MVErr2VV(vvp, vsnp, seq, cmd);
	exit(exit_status);
}

  static void
SendOK(req)
{
	char		buf[VV_PKTSIZ(vvpMvreq_t)];
	vvpHdr_t	*hp;
	vvpMvreq_t	*rp;

	hp = vvBuildHdr(cmd, seq, (vvpHdr_t *)buf);
	rp = VV_MSGPTR(hp, vvpMvreq_t);
	*rp = ReqArray[req].r_req;
	VVCLSend(hp, 1);
}





/*
 *  TapeOpenRetry() - in case some sort of device busy status
 *	prevents immediate opening of the tape. Assumes global
 *	tpnode, flags are set.
 */
  static int
TapeOpenRetry(int flags)
{
	int	i, fd;

	for (i=1; i<=TAPEOPENRETRIES; i++) {
		if (sigged)
			exit(EXIT_OK);
		fd = open(tpnode, flags);
		if (fd != -1) 
			break;
		nLogMsg(EL_CONSOLE0|EL_SYS, 0, "%s_%d: open %s failed",
					VVName, seq, tpnode);
		if (errno == ENOENT)
			break;
		sleep(i * i);
	}
	if (fd == -1) {
		if (errno == ENOENT)
			VVError = VVE_NOTFOUND;
		else
			VVError = VVE_SYS;
		nLogMsg(EL_SYS|EL_CODE, 0, "%s_%d: open %s failed",
					VVName, seq, tpnode);
		ChildErr(EXIT_FAILDEV);
	}
	return(fd);
}


/*
 *  CheckVol1() - check ANSI VOL1 label - should be same as external label
 *	and have the VVM signature in it if any files are on tape
 */
  static void
CheckVol1(int fd, char *buf)
{
	char	*id = "CheckVol1";
	int	len;
	time_t	start;

	/*
	 *  The label must be ANSI and the volser field equal to 
	 *	external. ANSI labels for this purpose consist of
	 *
	 *	Field		Contents
	 *
	 *	0-3		"VOL1"
	 *	4-9		alphanumeric volume serial number
	 *	79		"4", " ", ...
	 *
	 */
	
	nLogMsg(EL_STATUS1, id, "reading label");
	time(&start);
#ifdef FILEDRIVES
	if (lseek(fd, 0, SEEK_SET) != 0)
		nLogMsg(EL_FATAL|EL_SYS, id, "seek 0");
#else
	if (ntp_taprew(fd, vsnp, EL_CODE) == -1) {
		if (sigged)
			exit(EXIT_OK);
		VVError = VVE_SYS;
                ChildErr(EXIT_FAILDEV);
	}
#endif
#ifdef FILEDRIVES
	if ((len = read(fd, buf, LABELLEN)) == -1) {
#else
	if ((len = read(fd, buf, Rp->r_req.m_bufsize)) == -1) {
#endif
		if (sigged)
			exit(EXIT_OK);

	 	/* note - don't close fd; caller uses it again */
		if (errno == EIO) {
			/*
			 *  EIO's can take minutes in some drivers
			 *	so note the fact 
			 *	timing out other reqs; TODO - time open
			 */
			start = time((time_t *)0) - start;
			if (start > 10)
				nLogMsg(EL_ERROR2, id, 
					"(lost %d seconds on read I/O error)",
					start);

			/*
			 *  assume EIO means no 1st record
			 */

			VVError = VVE_TAPE;
			nLogMsg(EL_CODE|EL_STATUS1, 0,
					"%c%d: %s: I/O error on read %s %s",
					VVName, tpnode,
					"possibly no 1st record when label",
					"expected");
			ChildErr(EXIT_FAILVOL);
		}
		VVError = VVE_SYS;
		nLogMsg(EL_CONSOLE0|EL_CODE|EL_SYS, 0, 
				"%.*s: can't read label from tape",
				VSNLEN, vsnp);
		ChildErr(EXIT_FAILDEV);
	}
	nLogMsg(EL_DEBUG3, 0, "label from tape:\n%.80s", buf);

	/*
	 *  check the ANSI details: length at least 80; VOL1.........{4/" "}
	 */
	if (len != LABELLEN) {
		VVError = VVE_TAPE;
		nLogMsg(EL_CODE|EL_ERROR1, 0, 
					"%.*s: label %d chars, expected %d",
					VSNLEN, vsnp, len, LABELLEN);
		ChildErr(EXIT_FAILVOL);
	}
	if (memcmp(buf, "VOL1", 4) != 0) {    /* not a legal label */
		VVError = VVE_TAPE;
		nLogMsg(EL_CODE|EL_ERROR1, 0, 
				"%.*s: label does not begin with 'VOL1'",
				VSNLEN, vsnp);
		ChildErr(EXIT_FAILVOL);
	}
	if (memcmp(&buf[4], vsnp, VSNLEN) != 0) {
		VVError = VVE_TAPE;
		nLogMsg(EL_CODE|EL_ERROR1, 0,
				"%.*s: wrong volser in internal label (%.*s)",
				VSNLEN, vsnp, VSNLEN, &buf[4]);
		ChildErr(EXIT_FAILVOL);
	}
	if (memcmp(&buf[37], "VVM", 3) == 0) {
nLogMsg(EL_STATUS1, id, "TESTING CONFIG - allowing write on already-vvm tape");
		/*
		 *  VOL1 already written by VVM
		if (Rp->r_req.m_rec == -1) {
			VVError = VVE_TAPE;
			nLogMsg(EL_CODE|EL_ERROR1, 0,
				"%.*s: 'VVM' already in internal label\n%.*s",
				VSNLEN, vsnp, LABELLEN, buf);
			ChildErr(EXIT_FAILVOL);
		}
		 */
	} else {
		/*
		 *  VOL1 not written by VVM
		 */
		if (Rp->r_req.m_rec != -1) {
			VVError = VVE_TAPE;
			nLogMsg(EL_CODE|EL_ERROR1, 0,
				"%.*s: no 'VVM' in internal label (%.*s)",
				VSNLEN, vsnp, 3, &buf[14]);
			ChildErr(EXIT_FAILVOL);
		}
	}
}

/*
 *  WriteVol1() - write ANSI VOL1 label with VVM tag
 */
  static void
WriteVol1(int fd, char *buf)
{
	char	*id = "WriteVol1";
	int	len;
	time_t	start;

	/*
	 *  The label must be ANSI and the volser field equal to 
	 *	external. ANSI labels for this purpose consist of
	 *
	 *	Field		Contents
	 *
	 *	0-3		"VOL1"
	 *	4-9		alphanumeric volume serial number
	 *	79		"4", " ", ...
	 *
	 */
	
	nLogMsg(EL_STATUS1, 0, "writing VOL1 label");
	time(&start);

	/*
	 *  position tape
	 */
#ifdef FILEDRIVES
	if (lseek(fd, 0, SEEK_SET) != 0)
		nLogMsg(EL_FATAL|EL_SYS, id, "seek 0");
#else
	if (ntp_taprew(fd, vsnp, EL_CODE) == -1) {
		if (sigged)
			exit(EXIT_OK);
		VVError = VVE_SYS;
                ChildErr(EXIT_FAILDEV);
	}
#endif
	/*
	 *  prepare label
	 */
	memset(buf, ' ', LABELLEN);
	memcpy(buf, "VOL1", 4);
	memcpy(&buf[4], vsnp, VSNLEN);
	memcpy(&buf[37], "VVM", 3);
	buf[79] = '4';

	/*
	 *  write label
	 */
	if ((len = write(fd, buf, LABELLEN)) != LABELLEN) {
		if (sigged)
			exit(EXIT_OK);

		if (errno == -1) {
			nLogMsg(EL_CODE|EL_SYS, 0,
					"%s: %s: write error",
					VVName, tpnode);
			/*
			 *  EIO's can take minutes in some drivers
			 *	so note the fact 
			 *	timing out other reqs; TODO - time open
			 */
			start = time((time_t *)0) - start;
			if (start > 10)
				nLogMsg(EL_ERROR2, id, 
					"(lost %d seconds on write I/O error)",
					start);
		} else {
			nLogMsg(EL_CODE|EL_ERROR1, 0,
					"%s: %s: wrote %d of %d",
					VVName, tpnode, len, LABELLEN);
		}

		/*
		 *  fail tape to be on safe side..
		 */
		VVError = VVE_TAPE;
		ChildErr(EXIT_FAILVOL);
	}
	nLogMsg(EL_STATUS1, 0, "VOL1 label written");
}

/*
 *  CheckFileLbl() - check HDR/EOF
 */
  static int
CheckFileLbl(int tpfd, char *lbl, VV_t *lblvvp, char *buf)
{
	char	*id = "CheckFileLbl";
	int	size, fno;

	/*
	 *  make comparison label
	 */
	LabelBuf(&lblbuf, lbl, lblvvp);

	/*
	 *  read label from tape
	 */
#ifdef FILEDRIVES
	nLogMsg(EL_STATUS1, id, "at %d", lseek(tpfd, 0, SEEK_CUR));
	size = read(tpfd, buf, sizeof(lblbuf));
#else
	size = read(tpfd, buf, Rp->r_req.m_bufsize);
#endif
	if (size == -1) {
		VVError = VVE_SYS;
		nLogMsg(EL_SYS|EL_CODE, id, "%s label", lbl);
		return(1);
	}
	if (size != sizeof(lblbuf)) {
		VVError = VVE_TAPE;
		nLogMsg(EL_ERROR1|EL_CODE, id, 
				"expected %s size %d, tried %d, got size %d", 
				lbl, sizeof(lblbuf), Rp->r_req.m_bufsize, size);
		return(1);
	}
	if (CheckLabel((TapeLabel_t *)buf, &lblbuf)) {
		VVError = VVE_TAPE;
		nLogMsg(EL_ERROR1|EL_CODE, id, "tape label mismatch: see log");
		return(1);
	}
	return(0);
}


/*
 *  WriteTape() - check tape & write vv to it; exits
 */
 void
WriteTape(int req, Req_t *Rp)
{
	char	*id = "WriteTape";
	int	i, fd, tpfd, written, rsize, wsize;
	char	*tpbuf, *cp;
/*
VVError = VVE_SYS;
nLogMsg(EL_STATUS1|EL_CODE, id, "ARTIFICIAL sys ERROR");
ChildErr(1);
*/
	/*
	 *  open the file
	 */
	fd = open(vvPath, O_RDONLY);
	if (fd == -1) {
		VVError = VVE_INTERNAL;
		nLogMsg(EL_SYS|EL_CODE, id, "%s_%d: opening %s", 
					VVName, seq, vvPath);
		ChildErr(EXIT_FAILFILE);
	}
	if (FSTAT(fd, &sbuf) == -1) {
		VVError = VVE_INTERNAL;
		nLogMsg(EL_SYS|EL_CODE, id, "%s_%d: fstat %s", 
					VVName, seq, vvPath);
		ChildErr(EXIT_FAILFILE);
	}

	/*
	 *  open the tape
	 */
	tpfd = TapeOpenRetry(O_RDWR);

	/*
	 *  get a buffer to read into
	 */
	tpbuf = (char *) nAlloc(Rp->r_req.m_bufsize);

	/*
	 *  check tape internal label (VOL1) if req'd or in
	 *	any case if this will be the 1st file on tape.
	 */
	if (Rp->r_req.m_checkvol1  ||  Rp->r_req.m_rec == -1)
		CheckVol1(tpfd, tpbuf);

	/*
	 *  if this is 1st VV on tape, write VOL1 w/ VVM tag
	 */
	if (Rp->r_req.m_rec == -1) {
		/*
		 *  write volume hdr, leaving tape
		 *	positioned to write file
		 */
		WriteVol1(tpfd, tpbuf);
	} else {
		/*
		 *  this won't be the 1st vv on tape, so check the
		 *	previous one
		 */
		nLogMsg(EL_STATUS1, id, "check rec %d blk %d", 
					Rp->r_req.m_rec, Rp->r_req.m_block);
		/*
		 *  position tape
		 */
#ifdef FILEDRIVES
nLogMsg(EL_STATUS1, id, "locblk/seek to %d", Rp->r_req.m_block);
		if (lseek(tpfd, Rp->r_req.m_block, SEEK_SET) 
							!= Rp->r_req.m_block)
			nLogMsg(EL_FATAL|EL_SYS, id, "seek 'blk'");
#else
		if (ntp_locblk(tpfd, Rp->r_req.m_block, vsnp, EL_CODE) == -1) {
			VVError = VVE_SYS;
			ChildErr(EXIT_FAILVOL);
		}
#endif
		/*
		 *  check HDR
		 */
		if (CheckFileLbl(tpfd, HDRLABEL, &Rp->r_req.m_lastvv, tpbuf))
			ChildErr(EXIT_FAILVOL);

		/*
		 *  skip data
		 */
#ifdef FILEDRIVES
		nextmark(tpfd);
#else
		if (ntp_tapfsf(tpfd, 1, vsnp, EL_CODE) == -1) {
			VVError = VVE_SYS;
			ChildErr(EXIT_FAILVOL);
		}
#endif

		/*
		 *  check EOF
		 */
		if (CheckFileLbl(tpfd, EOFLABEL, &Rp->r_req.m_lastvv, tpbuf))
			ChildErr(EXIT_FAILVOL);

		/*
		 *  should be 2 tapemarks at end of file (after labels) to
		 *	signify eot
		 */
#ifdef FILEDRIVES
		get2tpmk(tpfd);
#else
		for (i=0; i<2; i++) {
			int	rc;

			rc = read(tpfd, tpbuf, Rp->r_req.m_bufsize);
			if (rc == -1) {
				VVError = VVE_SYS;
				nLogMsg(EL_SYS|EL_CODE, id, 
						"reading tapemark %d", i+1);
				ChildErr(EXIT_FAILVOL);
			}
			if (rc != 0) {
				VVError = VVE_TAPE;
				nLogMsg(EL_ERROR1|EL_CODE, id, 
				  "got block size %d, expected tapemark #%d",
							rc, i);
				ChildErr(EXIT_FAILVOL);
			}
		}

		/*
		 *  back up over the 2 tapemarks
		 */
		if (ntp_tapbsf(tpfd, 2, vsnp, EL_CODE) == -1) {
			VVError = VVE_SYS;
			ChildErr(EXIT_FAILVOL);
		}
#endif
	}
nLogMsg(EL_STATUS1, id, "get blk");

	/*
	 *  get block #
	 */
#ifdef FILEDRIVES
	getblk(tpfd, &Rp->r_req.m_block);
#else
	if (ntp_getblk(tpfd, &Rp->r_req.m_block, vsnp, EL_ERROR1)) {
		VVError = VVE_SYS;
		ChildErr(EXIT_FAILDEV);
	}
#endif

	/*
	 *  write the headers for the new file
	 */
	LabelBuf(&lblbuf, "HDR", vvp);
	LabelBuf2(&lblbuf, Rp->r_req.m_id, Rp->r_req.m_slot, Rp->r_req.m_nvsns,
			Rp->r_req.m_rec, sbuf.st_size, Rp->r_req.m_creat,
			Rp->r_req.m_nwrite, Rp->r_req.m_lastwrite,
			Rp->r_req.m_finish, Now);

nLogMsg(EL_STATUS1, id, "write");
	i = write(tpfd, &lblbuf, sizeof(lblbuf));
	if (i == -1) {
		nLogMsg(EL_SYS, id, "writing hdr label");
		VVError = VVE_TAPE;
		goto fixtape;
	}
	if (i != sizeof(lblbuf)) {
		nLogMsg(EL_ERROR1, id, "hdr wrote %d instead of %d",
						i, sizeof(lblbuf));
		VVError = VVE_TAPE;
		goto fixtape;
	}
nLogMsg(EL_STATUS1, id, "label ok");

	/*
	 *  write the data
	 */
	for (i=0,written=0; written<sbuf.st_size; i++) {
		/*
		 *  read disk file into buffer
		 */
		rsize = read(fd, tpbuf, Rp->r_req.m_bufsize);
		if (rsize == -1) {
			VVError = VVE_FILE;
			nLogMsg(EL_SYS, id, "reading file block %d size %d",
						i, Rp->r_req.m_bufsize);
			goto fixtape;
		}
		if (rsize != Rp->r_req.m_bufsize  &&  
		    rsize+written < sbuf.st_size) {
			VVError = VVE_FILE;
			nLogMsg(EL_ERROR1, id, 
				"reading file block %d size %d, read %d",
				i, Rp->r_req.m_bufsize, rsize);
			nLogMsg(EL_STATUS1, id, "written %d fsize %d", 
						written, sbuf.st_size);
			goto fixtape;
		}

		/*
		 *  write buffer to tape
		 */
		wsize = write(tpfd, tpbuf, rsize);
		if (wsize == -1) {
			VVError = VVE_TAPE;
			nLogMsg(EL_SYS, id, "writing tape block %d size %d",
						i, Rp->r_req.m_bufsize);
			goto fixtape;
		}
		if (wsize != rsize) {
			VVError = VVE_TAPE;
			nLogMsg(EL_ERROR1, id, 
				"writing tape block %d size %d, wrote %d",
				i, Rp->r_req.m_bufsize, rsize);
			goto fixtape;
		}
		written += wsize;
	}
nLogMsg(EL_STATUS1, id, "data ok");

	/*
	 *  write tapemark before EOF label -
	 *	so that forward-space-file can be used
	 *	to skip data when label checking
	 */
#ifdef FILEDRIVES
	wtm(tpfd);
#else
	if (ntp_tapwtm(tpfd, vsnp, EL_CODE) == -1) {
		nLogMsg(EL_ERROR1, id, 
				"writing tape mark before file EOF label");
		VVError = VVE_TAPE;
		goto fixtape;
	}
#endif

	/*
	 *  write EOF label
	 */
	memcpy(lblbuf.lb_type, EOFLABEL, LBLTYPELEN);
	i = write(tpfd, &lblbuf, sizeof(lblbuf));
	if (i == -1) {
		nLogMsg(EL_SYS, id, "writing eof");
		VVError = VVE_TAPE;
		goto fixtape;
	}
	if (i != sizeof(lblbuf)) {
		nLogMsg(EL_ERROR1, id, "eof wrote %d instead of %d",
						i, sizeof(lblbuf));
		VVError = VVE_TAPE;
		goto fixtape;
	}
nLogMsg(EL_STATUS1, id, "eof ok");

#ifdef FILEDRIVES
	wtm(tpfd);
	wtm(tpfd);
#else
	/*
	 *  write final 2 tapemarks
	 */
	for (i=0; i<2; i++) {
		if (ntp_tapwtm(tpfd, vsnp, EL_CODE) == -1) {
			nLogMsg(EL_ERROR1, id, 
				"writing eotape mark %d", i);
			VVError = VVE_TAPE;
			goto fixtape;
		}
	}
#endif
nLogMsg(EL_STATUS1, id, "tpmk2 ok");

	close(tpfd);
	Rp->r_req.m_rec++;
	SendOK(req);
	nLogMsg(EL_DEBUG1, id, "pid %d exit ok", getpid());
	exit(EXIT_OK);

fixtape:
	/*
	 *  got an error writing vv to tape, presumably
	 *	because there wasn't room left (or some
	 *	problem reading file). Try to back up and 
	 *	write 2 tapemarks at the end of the prev file. 
	 *	Leave VVError alone since it indicates to vvmd
	 *	whether the problem was with file or tape.
	 */
/* TODO */
	/*
	 *  send err on original problem to vvmd
	 */
	if (VVError == VVE_TAPE)
		nLogMsg(EL_CODE|EL_ERROR1, id, 
				"tape problem - probably full (vv %d)",
				Rp->r_req.m_rec);
	else
		nLogMsg(EL_CODE|EL_ERROR1, id, "problem with file");
	ChildErr(EXIT_FAILVOL);
}

/*
 *  ReadTape() - check label(s) & read vv to disk
 */
 void
ReadTape(int req, Req_t *Rp)
{
	char	*id = "ReadTape";
	int	i, fd, tpfd, written, rsize, wsize;
	char	*tpbuf, *cp, vvTmpPath[MAXPATHLEN];
	OFF_T	fsize;

	C2BIN(HexOFF, &fsize, Rp->r_req.m_fsize);

	nLogMsg(EL_STATUS1, id, "fsize %lld", (long long)fsize);
	/*
	 *  open the tape
	 */
	tpfd = TapeOpenRetry(O_RDONLY);

	/*
	 *  position tape
	 */
#ifdef FILEDRIVES
	nLogMsg(EL_STATUS1, id, "hdr locblk/seek to %d", Rp->r_req.m_block);
	if (lseek(tpfd, Rp->r_req.m_block, SEEK_SET) != Rp->r_req.m_block)
		nLogMsg(EL_FATAL|EL_SYS, id, "seek 'blk'");
#else
	if (ntp_locblk(tpfd, Rp->r_req.m_block, vsnp, EL_CODE) == -1) {
		VVError = VVE_SYS;
		ChildErr(EXIT_FAILVOL);
	}
#endif

	/*
	 *  get a buffer to read into
	 */
	tpbuf = (char *) nAlloc(Rp->r_req.m_bufsize);

	/*
	 *  check tape internal label (VOL1) if req'd
	 */
	if (Rp->r_req.m_checkvol1) {
nLogMsg(EL_STATUS1, id, "check vol1");
		CheckVol1(tpfd, tpbuf);
	}

	/*
	 *  check file's HDR
	 */
nLogMsg(EL_STATUS1, id, "check hdr1");
	if (CheckFileLbl(tpfd, HDRLABEL, vvp, tpbuf))
		ChildErr(EXIT_FAILVOL);

	/*
	 *  open the tmp copy of the vv file - note
	 *	that it's open for writing, but 
	 *	created read-only
	 */
	sprintf(vvTmpPath, "%s_tmp", vvPath);
	fd = open(vvTmpPath, O_RDWR|O_CREAT, 0400);
	if (fd == -1) {
		VVError = VVE_INTERNAL;
		nLogMsg(EL_SYS|EL_CODE, id, "%s_%d: opening %s", 
					VVName, seq, vvTmpPath);
		ChildErr(EXIT_FAILFILE);
	}

	/* TODO - stat file? LSEEK? */

	/*
	 *  read the data from tape into disk file
	 */
	for (i=0,written=0; written<fsize; i++) {
		/*
		 *  read into buffer
		 */
#ifdef FILEDRIVES
		rsize = read(tpfd, tpbuf, 
				MIN(fsize-written, Rp->r_req.m_bufsize));
#else
		rsize = read(tpfd, tpbuf, Rp->r_req.m_bufsize);
#endif
		if (rsize == -1) {
			VVError = VVE_TAPE;
			nLogMsg(EL_SYS, id, "reading tape block %d size %d",
						i, Rp->r_req.m_bufsize);
			goto fixfile;
		}
		if (rsize != Rp->r_req.m_bufsize  &&  rsize+written < fsize) {
			VVError = VVE_TAPE;
			nLogMsg(EL_ERROR1, id, 
				"reading tape block %d size %d, got %d",
				i, Rp->r_req.m_bufsize, rsize);
nLogMsg(EL_STATUS1, id, "written %d fsize %d  rsize+written %d",
written, fsize, rsize+written);
			goto fixfile;
		}

		/*
		 *  write buffer to file
		 */
		wsize = write(fd, tpbuf, rsize);
		if (wsize == -1) {
			VVError = VVE_FILE;
			nLogMsg(EL_SYS, id, "writing file %s block %d size %d",
					vvTmpPath, i, Rp->r_req.m_bufsize);
			goto fixfile;
		}
		if (wsize != rsize) {
			VVError = VVE_FILE;
			nLogMsg(EL_ERROR1, id, 
				"writing file %s block %d size %d, wrote %d",
				vvTmpPath, i, Rp->r_req.m_bufsize, rsize);
			goto fixfile;
		}
		written += wsize;
	}

	/*
	 *  skip tapemark
	 */
#ifdef FILEDRIVES
	gettpmk(tpfd, 0);
#else
	rsize = read(tpfd, tpbuf, Rp->r_req.m_bufsize);
	if (rsize == -1) {
		VVError = VVE_TAPE;
		nLogMsg(EL_SYS|EL_CODE, id, "expected tapemark");
		goto fixfile;
	} else if (rsize != 0) {
		VVError = VVE_TAPE;
		nLogMsg(EL_ERROR1|EL_CODE, id, "expected tapemark, read %d",
								rsize);
		goto fixfile;
	}
#endif

	/*
	 *  check EOF
	 */
nLogMsg(EL_STATUS1, id, "check eof");
	if (CheckFileLbl(tpfd, EOFLABEL, vvp, tpbuf)) {
		goto fixfile;
	}

	/*
	 *  move the file into place
	 */
	close(tpfd);
	if (rename(vvTmpPath, vvPath) == -1) {
		VVError = VVE_FILE;
		nLogMsg(EL_SYS, id, "rename(%s, %s)", vvTmpPath, vvPath);
		goto fixfile;
	}

	/*
	 *  report success
	 */
	SendOK(req);
	nLogMsg(EL_DEBUG1, id, "pid %d exit ok", getpid());
	exit(EXIT_OK);

fixfile:
	/*
	 *  got an error writing vv to file, recover somehow?
	 *	Leave VVError alone since it indicates to vvmd
	 *	whether the problem was with file or tape.
	 */
/* TODO */
	unlink(vvPath);
	unlink(vvTmpPath);
	/*
	 *  send err on original problem to vvmd
	 */
	if (VVError == VVE_TAPE)
		nLogMsg(EL_CODE|EL_ERROR1, id, "problem with tape");
	else
		nLogMsg(EL_CODE|EL_ERROR1, id, "problem with file");
	ChildErr(EXIT_FAILVOL);
}


/*
 *  MoverChild() - forked to do the job; return status is exit code
 *	caught by parent: if ok (0), let parent inform vvmd; if
 *	err (1), send msg directly to vvmd.
 */
 void
MoverChild(int req)
{
	char	buf[100];

	/*
	 *  set the global request pointer and prepare
	 *	to catch timeout signals
	 */
	Rp = &ReqArray[req];
	seq = Rp->r_seq;
	cmd = Rp->r_cmd;
	vvp = &Rp->r_req.m_vv;
	strcpy(VVName, VV2Name(vvp));
	vvPath = VV2Path(vvp);
	vsnp = Rp->r_req.m_vsn;

if (Rp->r_req.m_bufsize < 1)
nLogMsg(EL_FATAL, 0, "req %d bufsize %d", req, Rp->r_req.m_bufsize);
nLogMsg(EL_STATUS1, 0, "req %d bufsize %d", req, Rp->r_req.m_bufsize);

	if (SIGNAL(SIGTERM, sigterm))
		nLogMsg(EL_FATAL|EL_SYS, 0, "signal SIGTERM");

	/*
	 *  change process name so it will be apparent
	 *	which vol/vsn involved if proc hangs
	 */
	sprintf(buf, "vvmvd %s %s %.*s", 
			(cmd == VVM_WTAPE ? "WTAPE" : "RTAPE"),
			VVName, VSNLEN, vsnp);
/*
	nChangeName(args, buf);
*/

	/*
	 *  create vsn path
	 */
	sprintf(tpnode, "%s/%.*s", vNodeDir, VSNLEN, vsnp);

	/*
	 *  handle the request
	 */
	switch (cmd) {
	    case VVM_WTAPE:
		WriteTape(req, Rp);			/* exits */
		break;
	    case VVM_RTAPE:
		ReadTape(req, Rp);			/* exits */
		break;
	    default:
		nLogMsg(EL_CONSOLE0|EL_ERROR1, 0, "bad cmd %d", cmd);
		exit(-1);				/* exits */
	}
}

