/*	$NetBSD$	*/

/*-
 * Copyright (c) 1998-1999 Brett Lymn
 *                         (blymn@baea.com.au, brett_lymn@yahoo.com.au)
 * All rights reserved.
 *
 * This code has been donated to The NetBSD Foundation by the Author.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 */

#include <stdio.h>
#include <menu.h>
#include "edit.h"
#include "help.h"
#include "screen.h"
#include "utils.h"

ITEM *help_items[3];
MENU *help_menu;

/* the following are defined in edit.c */
extern WINDOW *edit_win;

/* menu item callbacks */
void
help_item_callback(MENU *parent);
void
help_about_item_callback(void);
	
void
help_item_init(void)
{
	if ((help_items[0] = new_item("Keys", "")) == NULL) {
                error_exit("Help keys item failed");
        }

	if ((help_items[1] = new_item("About Edit...", "")) == NULL) {
                error_exit("Help about item failed");
        }

	help_items[2] = NULL;

	set_item_userptr(help_items[0], (char *) help_keys_item_callback);
	set_item_userptr(help_items[1], (char *) help_about_item_callback);
	
	if ((help_menu = new_menu(help_items)) == NULL) {
		error_exit("Failed to create help menu");
	}

	set_menu_format(help_menu, 2, 1);
	menu_opts_on(help_menu, O_SHOWDESC);
}

/*
 * Popup the help menu.
 */
void
help_item_callback(MENU *parent)
{
	WINDOW *help_popup;

	if ((help_popup = create_menu_popup(parent, help_menu, 1, -1, -1, -1))
	    == NULL) {
		fprintf(stderr, "Help menu popup failed.\n");
		return;
	}

	handle_menu_popup(help_menu, help_popup, "help");
}

	

/*
 * Present a short help screen about the key bindings and how to use.
 */
void
help_keys_item_callback(void)
{
	int c;
	
	wclear(edit_win);
	wmove(edit_win, 1, 0);
	wprintw(edit_win, "Welcome to Edit, the simple editor");
	wmove(edit_win, 3, 0);
	wprintw(edit_win,
		"To enter the menus press the escape key.  The menu navigation");
	wmove(edit_win, 4, 0);
	wprintw(edit_win, "keys are as follows:");
	wmove(edit_win, 6, 8);
	wprintw(edit_win, "Top line menu:");
	wmove(edit_win, 7, 16);
	wprintw(edit_win,
		"   Key Pressed                     Action");
	wmove(edit_win, 8, 16);
	wprintw(edit_win,
		"Right Arrow, l, TAB             Next menu item");
	wmove(edit_win, 9, 16);
	wprintw(edit_win,
		"Left Arrow, h                   Previous menu item");
	wmove(edit_win, 10, 16);
	wprintw(edit_win,
		"Down Arrow, Return, j           Select menu item");

	wmove(edit_win, 12, 8);
	wprintw(edit_win, "Pull down menus:");
	wmove(edit_win, 13, 16);
	wprintw(edit_win,
		"   Key Pressed                     Action");
	wmove(edit_win, 15, 16);
	wprintw(edit_win,
		"Down Arrow, TAB, j              Next menu item");
	wmove(edit_win, 16, 16);
	wprintw(edit_win,
		"Up Arrow, k                     Previous menu item");
	wmove(edit_win, 17, 16);
	wprintw(edit_win,
		"Return                          Select menu item");


	wmove(edit_win, 18, 8);
	wprintw(edit_win, "Pop up dialogue boxes with horizontal menus:");
	wmove(edit_win, 19, 16);
	wprintw(edit_win, "Left Arrow, TAB, l    Next menu item");
	wmove(edit_win, 20, 16);
	wprintw(edit_win, "Right Arrow, h        Previous menu item");
	wmove(edit_win, 21, 16);
	wprintw(edit_win, "Return                Select menu item");
	wmove(edit_win, 23, 0);
	wprintw(edit_win, "Press any key for next help page...");
	wrefresh(edit_win);
	c = wgetch(edit_win);

	  /* next page */
	wclear(edit_win);
	wmove(edit_win, 1, 8);
	wprintw(edit_win,
		"Search and Replace dialogue boxes are modal.  You need to");
	wmove(edit_win, 2, 8);
	wprintw(edit_win,
		"enter the search string and hit return (do the same for the");
	wmove(edit_win, 3, 8);
	wprintw(edit_win,
		"replace string if this is a replace), you will then be able");
	wmove(edit_win, 4, 8);
	wprintw(edit_win,
		"to access the menu.  Pressing escape when in the menu will");
	wmove(edit_win, 5, 8);
	wprintw(edit_win,
		"return you to the search/replace prompts.");
	wmove(edit_win, 6, 0);
	wprintw(edit_win,
		"Any menu, apart from the search and replace popups,can be");
	wmove(edit_win, 7, 0);
	wprintw(edit_win,
		"closed without making a selection by pressing the escape key.");
	wmove(edit_win, 7, 0);
	wprintw(edit_win,
		"In any menu you can skip to an item by typing the start");
	wmove(edit_win, 8, 0);
	wprintw(edit_win,
		"of the item name, the cursor will skip to the item name");
	wmove(edit_win, 9, 0);
	wprintw(edit_win,
		"that matches what you have typed.");

	wmove(edit_win, 11, 0);
	wprintw(edit_win,
		"In the editing window the following control keys are ");
	wmove(edit_win, 12, 0);
	wprintw(edit_win, "defined:");
	wmove(edit_win, 14, 8);
	wprintw(edit_win,
		"control-space              Set the mark for a cut or copy");
	wmove(edit_win, 15, 8);
	wprintw(edit_win,
		"control-b                  Scroll back one page");
	wmove(edit_win, 16, 8);
	wprintw(edit_win,
		"control-c                  Copy the lines from the mark set");
	wmove(edit_win, 17, 8);
	wprintw(edit_win,
		"                           to the current line to the cut");
	wmove(edit_win, 18, 8);
	wprintw(edit_win,
		"                           buffer.  Whole lines are copied.");
	wmove(edit_win, 19, 8);
	wprintw(edit_win,
		"control-d, DEL             Delete the current character.");
	wmove(edit_win, 20, 8);
	wprintw(edit_win,
		"control-f                  Scroll one page forward");
	wmove(edit_win, 21, 8);
	wprintw(edit_win,
		"control-h, BackSpace       Delete character to the left of");
	wmove(edit_win, 22, 8);
	wprintw(edit_win,
		"                           the current character.");
	wmove(edit_win, 23, 0);
	wprintw(edit_win,
		"Press any key to go to next help page...");
	wrefresh(edit_win);
	c = wgetch(edit_win);

	wclear(edit_win);
	wmove(edit_win, 1, 8);
	wprintw(edit_win,
		"control-l                   Redraw the screen");
	wmove(edit_win, 2, 8);
	wprintw(edit_win,
		"control-o                   Open a file");
	wmove(edit_win, 3, 8);
	wprintw(edit_win,
		"control-p                   Paste contents of cut buffer");
	wmove(edit_win, 4, 8);
	wprintw(edit_win,
		"                            after current line.");
	wmove(edit_win, 5, 8);
	wprintw(edit_win,
		"control-r                   Search backwards");
	wmove(edit_win, 6, 8);
	wprintw(edit_win,
		"control-s                   Search forwards");
	wmove(edit_win, 7, 8);
	wprintw(edit_win,
		"control-v                   Copy the region between the mark");
	wmove(edit_win, 8, 8);
	wprintw(edit_win,
		"                            and the current line to the cut");
	wmove(edit_win, 9, 8);
	wprintw(edit_win,
		"                            buffer and the delete the lines.");
	wmove(edit_win, 10, 8);
	wprintw(edit_win,
		"control-w                   Write the file to disk");
	wmove(edit_win, 11, 8);
	wprintw(edit_win,
		"control-x                   Quit edit.");
	wmove(edit_win, 12, 8);
	wprintw(edit_win,
		"F1 function key             These help screens");
	wmove(edit_win, 13, 8);
	wprintw(edit_win,
		"Arrow keys                  Move the cursor as expected.");
	wmove(edit_win, 16, 0);
	wprintw(edit_win,
		"Press any key to return to the edit window....");
	wrefresh(edit_win);
	c = wgetch(edit_win);
	
	redraw_screen();
}

/*
 * Popup version information about edit.
 */
void
help_about_item_callback(void)
{
	int x, y, c;
	WINDOW *about;

	x = (COLS / 2) - 15;
	y = (LINES / 2) - 3;

	if (x < 0)
		x = 0;
	if (y < 0)
		y = 0;

	if ((about = newwin(6, 30, y, x)) == NULL) {
		fprintf(stderr, "Cannot create about window\n");
		return;
	}

	box(about, '*', '*');
	wmove(about, 2, 7);
	wprintw(about, "Edit version 0.3");
	wmove(about, 3, 3);
	wprintw(about, "Press any key to dismiss");
	wrefresh(about);
	c = wgetch(about);
}
