/*	$NetBSD$	*/

/*-
 * Copyright (c) 1998-1999 Brett Lymn
 *                         (blymn@baea.com.au, brett_lymn@yahoo.com.au)
 * All rights reserved.
 *
 * This code has been donated to The NetBSD Foundation by the Author.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 */

#include <menu.h>
#include "edit.h"
#include "screen.h"
#include "utils.h"

/* these are defined in edit.c */
extern EDIT_LINE *first_line, *top_line, *current_line;
extern WINDOW *edit_win;
extern int cursor_pos, screen_pos, cursor_line, edit_maxy;

/*
 * Put the given line at the top of the screen and position the cursor
 * at the position in the string pointed to by mp.  Note that mp is
 * assumed to point at a character position in the given line.
 */
void
show_match(EDIT_LINE *this, char *mp)
{
	cursor_pos = (mp - this->line) / sizeof(char);
	if (cursor_pos < 0) {
		fprintf(stderr, "Huh? mp < this->line\n");
		cursor_pos = 0;
		screen_pos = 0;
	} else {
		screen_pos = tab_expanded_length(cursor_pos);
	}
	
	top_line = this;
	redraw_screen();
	wmove(edit_win, 1, screen_pos);
	wrefresh(edit_win);
}

/*
 * Redraw all the lines on the screen.
 */
void
redraw_screen(void)
{
	int i, dist;
	EDIT_LINE *cl;

	wclear(edit_win);

	cl = top_line;
	if (cl == NULL)
		return;

	  /* count the number of lines between current line and top line */
	dist = 0;
	
	while ((cl != NULL) && (cl != current_line)) {
		dist++;
		cl = cl->next;
	}

	cl = top_line;
	  /* adjust top line if current_line is off screen */
	if (dist > edit_maxy - 1) {
		do {
			cl = cl->next;
			dist--;
		} while ((cl != NULL) && (dist > (edit_maxy - 1)));
	}

	if (cl == NULL)
		cl = top_line; /* not right but better than NULL */
	else
		top_line = cl;
	
	for (i = 0; i < edit_maxy; i++) {
		wmove(edit_win, i, 0);
		wprintw(edit_win, "%s", cl->line);
		if (cl == current_line)
			cursor_line = i; /* note the screen line for current_line */
		
		cl = cl->next;
		if (cl == NULL)
			break;
	}

	wmove(edit_win, cursor_line, screen_pos);
	return;
}

/*
 * Redraw the current line when it is updated.
 */
void
redraw_current_line()
{
	wmove(edit_win, cursor_line, 0);
	wclrtoeol(edit_win);
	wprintw(edit_win, "%s", current_line->line);
	wmove(edit_win, cursor_line, screen_pos);
}

/*
 * Scroll the window up by the number of lines given.
 */
void
scroll_up(int n)
{
	int i;
	EDIT_LINE *bottom;

	  /* if scroll amount changes all of the screen then just redraw */
	if (n >= edit_maxy) {
		i = n;
		while ((current_line->next != NULL) && (i > 0)) {
			top_line = top_line->next;
			current_line = current_line->next;
			i--;
		}
		redraw_screen();
	} else {
		  /* otherwise, note where the bottom line used to be and
		     redraw from there down.... */
		bottom = top_line;
		for (i = 0; i < edit_maxy; i++) {
			if (bottom->next != NULL)
				bottom = bottom->next;
			else
				break;
		}

		  /* scroll the screen */
		i = n;
		while ((current_line->next != NULL) && (i > 0)) {
			top_line = top_line->next;
			current_line = current_line->next;
			i--;
			scroll(edit_win);
		}

		  /* update the region left by the scroll */
		wmove(edit_win, edit_maxy - n, 0);
		  /*	wclrtobot(edit_win);*/
		for (i = edit_maxy - n; i < edit_maxy; i++) {
			if (bottom != NULL) {
				wmove(edit_win, i, 0);
				wprintw(edit_win, "%s", bottom->line);
				bottom = bottom->next;
			} else
				break;
		}
	}

	wrefresh(edit_win);
}

/*
 * Scroll the screen downwards.
 */
void
scroll_down(int n)
{
	int i;

	if (top_line->prev == NULL)
		return;

	  /* we could be more clever about this if we had wscrl but we
	     do not at the moment :-( */
	i = n;
	while ((top_line->prev != NULL) && (i > 0)) {
		top_line = top_line->prev;
		current_line = current_line->prev;
		i--;
	}
	redraw_screen();
}
